@@ -9,33 +9,30 @@ import android.content.Context;
 import android.graphics.Color;
 import android.graphics.PixelFormat;
 import android.graphics.Rect;
+import android.os.Build;
+import android.view.Display;
 import android.view.MotionEvent;
 import android.view.Surface;
 import android.view.SurfaceHolder;
 import android.view.SurfaceView;
 import android.view.View;
+import android.view.WindowManager;
 
-import org.chromium.base.ApiCompatibilityUtils;
 import org.chromium.base.CommandLine;
 import org.chromium.base.Log;
 import org.chromium.base.TraceEvent;
 import org.chromium.base.VisibleForTesting;
 import org.chromium.base.annotations.CalledByNative;
 import org.chromium.base.annotations.JNINamespace;
-import org.chromium.chrome.R;
 import org.chromium.chrome.browser.ChromeSwitches;
 import org.chromium.chrome.browser.compositor.layouts.Layout;
-import org.chromium.chrome.browser.compositor.layouts.Layout.SizingFlags;
 import org.chromium.chrome.browser.compositor.layouts.LayoutProvider;
 import org.chromium.chrome.browser.compositor.layouts.LayoutRenderHost;
 import org.chromium.chrome.browser.compositor.layouts.components.LayoutTab;
-import org.chromium.chrome.browser.compositor.layouts.content.ContentOffsetProvider;
 import org.chromium.chrome.browser.compositor.layouts.content.TabContentManager;
 import org.chromium.chrome.browser.compositor.resources.StaticResourcePreloads;
 import org.chromium.chrome.browser.compositor.scene_layer.SceneLayer;
-import org.chromium.chrome.browser.device.DeviceClassManager;
 import org.chromium.chrome.browser.externalnav.IntentWithGesturesHandler;
-import org.chromium.chrome.browser.fullscreen.ChromeFullscreenManager;
 import org.chromium.chrome.browser.multiwindow.MultiWindowUtils;
 import org.chromium.chrome.browser.tabmodel.TabModelImpl;
 import org.chromium.chrome.browser.widget.ClipDrawableProgressBar.DrawingInfo;
@@ -47,15 +44,15 @@ import org.chromium.ui.resources.ResourceManager;
 /**
  * The is the {@link View} displaying the ui compositor results; including webpages and tabswitcher.
  */
-@JNINamespace("chrome::android")
+@JNINamespace("android")
 public class CompositorView
-        extends SurfaceView implements ContentOffsetProvider, SurfaceHolder.Callback2 {
+        extends SurfaceView implements SurfaceHolder.Callback {
     private static final String TAG = "CompositorView";
+    private static final long NANOSECONDS_PER_MILLISECOND = 1000000;
 
     // Cache objects that should not be created every frame
     private final Rect mCacheViewport = new Rect();
     private final Rect mCacheAppRect = new Rect();
-    private final Rect mCacheVisibleViewport = new Rect();
     private final int[] mCacheViewPosition = new int[2];
 
     private long mNativeCompositorView;
@@ -65,6 +62,9 @@ public class CompositorView
 
     private int mLastLayerCount;
 
+    // A conservative estimate of when a frame is guaranteed to be presented after being submitted.
+    private long mFramePresentationDelay;
+
     // Resource Management
     private ResourceManager mResourceManager;
 
@@ -129,7 +129,8 @@ public class CompositorView
             mPreviousWindowTop = windowTop;
 
             Activity activity = mWindowAndroid != null ? mWindowAndroid.getActivity().get() : null;
-            boolean isMultiWindow = MultiWindowUtils.getInstance().isMultiWindow(activity);
+            boolean isMultiWindow = MultiWindowUtils.getInstance().isLegacyMultiWindow(activity)
+                    || MultiWindowUtils.getInstance().isInMultiWindowMode(activity);
 
             // If the measured width is the same as the allowed width (i.e. the orientation has
             // not changed) and multi-window mode is off, use the largest measured height seen thus
@@ -144,12 +145,6 @@ public class CompositorView
         super.onMeasure(widthMeasureSpec, heightMeasureSpec);
     }
 
-    @Override
-    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
-        super.onLayout(changed, left, top, right, bottom);
-        mRenderHost.onOverdrawBottomHeightChanged(getOverdrawBottomHeight());
-    }
-
     @Override
     protected void onDetachedFromWindow() {
         super.onDetachedFromWindow();
@@ -163,21 +158,6 @@ public class CompositorView
         return mResourceManager;
     }
 
-    /**
-     * @return The amount the surface view is overdrawing the window bounds.
-     */
-    public int getOverdrawBottomHeight() {
-        if (mRootActivityView == null) {
-            mRootActivityView = mRootView.findViewById(android.R.id.content);
-        }
-        if (mRootActivityView != null) {
-            int compositorHeight = getHeight();
-            int rootViewHeight = mRootActivityView.getHeight();
-            return Math.max(0, compositorHeight - rootViewHeight);
-        }
-        return 0;
-    }
-
     /**
      * Should be called for cleanup when the CompositorView instance is no longer used.
      */
@@ -201,7 +181,6 @@ public class CompositorView
         mTabContentManager = tabContentManager;
 
         mNativeCompositorView = nativeInit(lowMemDevice,
-                ApiCompatibilityUtils.getColor(getResources(), R.color.tab_switcher_background),
                 windowAndroid.getNativePointer(), layerTitleCache, tabContentManager);
 
         assert !getHolder().getSurface().isValid()
@@ -212,6 +191,18 @@ public class CompositorView
         setBackgroundColor(Color.WHITE);
         setVisibility(View.VISIBLE);
 
+        mFramePresentationDelay = 0;
+        if (Build.VERSION.SDK_INT > Build.VERSION_CODES.M) {
+            Display display =
+                    ((WindowManager) getContext().getSystemService(Context.WINDOW_SERVICE))
+                    .getDefaultDisplay();
+            long presentationDeadline = display.getPresentationDeadlineNanos()
+                    / NANOSECONDS_PER_MILLISECOND;
+            long vsyncPeriod = mWindowAndroid.getVsyncPeriodInMillis();
+            mFramePresentationDelay = Math.min(3 * vsyncPeriod,
+                    ((presentationDeadline + vsyncPeriod - 1) / vsyncPeriod) * vsyncPeriod);
+        }
+
         // Grab the Resource Manager
         mResourceManager = nativeGetResourceManager(mNativeCompositorView);
     }
@@ -253,9 +244,6 @@ public class CompositorView
         nativeSurfaceDestroyed(mNativeCompositorView);
     }
 
-    @Override
-    public void surfaceRedrawNeeded(SurfaceHolder holder) {}
-
     @Override
     public void onWindowVisibilityChanged(int visibility) {
         super.onWindowVisibilityChanged(visibility);
@@ -317,69 +305,17 @@ public class CompositorView
     private void onSwapBuffersCompleted(int pendingSwapBuffersCount) {
         // Clear the color used to cover the uninitialized surface.
         if (getBackground() != null) {
-            post(new Runnable() {
+            postDelayed(new Runnable() {
                 @Override
                 public void run() {
                     setBackgroundResource(0);
                 }
-            });
+            }, mFramePresentationDelay);
         }
 
         mRenderHost.onSwapBuffersCompleted(pendingSwapBuffersCount);
     }
 
-    private void updateToolbarLayer(LayoutProvider provider, boolean forRotation,
-            final DrawingInfo progressBarDrawingInfo) {
-        if (forRotation || !DeviceClassManager.enableFullscreen()) return;
-
-        ChromeFullscreenManager fullscreenManager = provider.getFullscreenManager();
-        if (fullscreenManager == null) return;
-
-        float offset = fullscreenManager.getControlOffset();
-        boolean forceHideTopControlsAndroidView =
-                provider.getActiveLayout().forceHideTopControlsAndroidView();
-        boolean useTexture = fullscreenManager.drawControlsAsTexture() || offset == 0
-                || forceHideTopControlsAndroidView;
-
-        float dpToPx = getContext().getResources().getDisplayMetrics().density;
-        float layoutOffsetDp = provider.getActiveLayout().getTopControlsOffset(offset / dpToPx);
-        boolean validLayoutOffset = !Float.isNaN(layoutOffsetDp);
-
-        if (validLayoutOffset) {
-            offset = layoutOffsetDp * dpToPx;
-            useTexture = true;
-        }
-
-        fullscreenManager.setHideTopControlsAndroidView(forceHideTopControlsAndroidView
-                || (validLayoutOffset && layoutOffsetDp != 0.f));
-
-        int flags = provider.getActiveLayout().getSizingFlags();
-        if ((flags & SizingFlags.REQUIRE_FULLSCREEN_SIZE) != 0
-                && (flags & SizingFlags.ALLOW_TOOLBAR_HIDE) == 0
-                && (flags & SizingFlags.ALLOW_TOOLBAR_ANIMATE) == 0) {
-            useTexture = false;
-        }
-
-        nativeUpdateToolbarLayer(mNativeCompositorView, R.id.control_container,
-                mRenderHost.getTopControlsBackgroundColor(), R.drawable.textbox,
-                mRenderHost.getTopControlsUrlBarAlpha(), offset,
-                provider.getActiveLayout().getToolbarBrightness(),
-                useTexture, forceHideTopControlsAndroidView);
-
-        if (progressBarDrawingInfo == null) return;
-        nativeUpdateProgressBar(mNativeCompositorView,
-                progressBarDrawingInfo.progressBarRect.left,
-                progressBarDrawingInfo.progressBarRect.top,
-                progressBarDrawingInfo.progressBarRect.width(),
-                progressBarDrawingInfo.progressBarRect.height(),
-                progressBarDrawingInfo.progressBarColor,
-                progressBarDrawingInfo.progressBarBackgroundRect.left,
-                progressBarDrawingInfo.progressBarBackgroundRect.top,
-                progressBarDrawingInfo.progressBarBackgroundRect.width(),
-                progressBarDrawingInfo.progressBarBackgroundRect.height(),
-                progressBarDrawingInfo.progressBarBackgroundColor);
-    }
-
     /**
      * Converts the layout into compositor layers. This is to be called on every frame the layout
      * is changing.
@@ -407,24 +343,13 @@ public class CompositorView
         // If you do, you could inadvertently trigger follow up renders.  For further information
         // see dtrainor@, tedchoc@, or klobag@.
 
-        // TODO(jscholler): change 1.0f to dpToPx once the native part is fully supporting dp.
-        mRenderHost.getVisibleViewport(mCacheVisibleViewport);
         provider.getViewportPixel(mCacheViewport);
-        nativeSetLayoutViewport(mNativeCompositorView, mCacheViewport.left, mCacheViewport.top,
-                mCacheViewport.width(), mCacheViewport.height(), mCacheVisibleViewport.left,
-                mCacheVisibleViewport.top, mRenderHost.getCurrentOverdrawBottomHeight(), 1.0f);
 
-        mCacheVisibleViewport.right = mCacheVisibleViewport.left + mSurfaceWidth;
-        mCacheVisibleViewport.bottom = mCacheVisibleViewport.top
-                + Math.max(mSurfaceHeight - mRenderHost.getCurrentOverdrawBottomHeight(), 0);
-
-        // TODO(changwan): move to treeprovider.
-        updateToolbarLayer(provider, forRotation, progressBarDrawingInfo);
+        nativeSetLayoutBounds(mNativeCompositorView);
 
         SceneLayer sceneLayer =
-                provider.getUpdatedActiveSceneLayer(mCacheViewport, mCacheVisibleViewport,
-                        mLayerTitleCache, mTabContentManager, mResourceManager,
-                        provider.getFullscreenManager());
+                provider.getUpdatedActiveSceneLayer(mCacheViewport, mLayerTitleCache,
+                        mTabContentManager, mResourceManager, provider.getFullscreenManager());
 
         nativeSetSceneLayer(mNativeCompositorView, sceneLayer);
 
@@ -444,15 +369,8 @@ public class CompositorView
         return mLastLayerCount;
     }
 
-    @Override
-    public int getOverlayTranslateY() {
-        return mRenderHost.areTopControlsPermanentlyHidden()
-                ? mRenderHost.getTopControlsHeightPixels()
-                : mRenderHost.getVisibleViewport(mCacheVisibleViewport).top;
-    }
-
     // Implemented in native
-    private native long nativeInit(boolean lowMemDevice, int emptyColor, long nativeWindowAndroid,
+    private native long nativeInit(boolean lowMemDevice, long nativeWindowAndroid,
             LayerTitleCache layerTitleCache, TabContentManager tabContentManager);
     private native void nativeDestroy(long nativeCompositorView);
     private native ResourceManager nativeGetResourceManager(long nativeCompositorView);
@@ -462,24 +380,7 @@ public class CompositorView
             long nativeCompositorView, int format, int width, int height, Surface surface);
     private native void nativeFinalizeLayers(long nativeCompositorView);
     private native void nativeSetNeedsComposite(long nativeCompositorView);
-    private native void nativeSetLayoutViewport(long nativeCompositorView, float x, float y,
-            float width, float height, float visibleXOffset, float visibleYOffset,
-            float overdrawBottomHeight, float dpToPixel);
-    private native void nativeUpdateToolbarLayer(long nativeCompositorView, int resourceId,
-            int toolbarBackgroundColor, int urlBarResourceId, float urlBarAlpha, float topOffset,
-            float brightness, boolean visible, boolean showShadow);
-    private native void nativeUpdateProgressBar(
-            long nativeCompositorView,
-            int progressBarX,
-            int progressBarY,
-            int progressBarWidth,
-            int progressBarHeight,
-            int progressBarColor,
-            int progressBarBackgroundX,
-            int progressBarBackgroundY,
-            int progressBarBackgroundWidth,
-            int progressBarBackgroundHeight,
-            int progressBarBackgroundColor);
+    private native void nativeSetLayoutBounds(long nativeCompositorView);
     private native void nativeSetOverlayVideoMode(long nativeCompositorView, boolean enabled);
     private native void nativeSetSceneLayer(long nativeCompositorView, SceneLayer sceneLayer);
 }
