@@ -12,19 +12,19 @@ import android.graphics.Rect;
 import android.graphics.RectF;
 import android.os.Bundle;
 import android.os.Handler;
+import android.support.design.widget.CoordinatorLayout;
 import android.support.v4.view.ViewCompat;
 import android.support.v4.view.accessibility.AccessibilityEventCompat;
 import android.support.v4.view.accessibility.AccessibilityNodeInfoCompat;
 import android.support.v4.widget.ExploreByTouchHelper;
 import android.util.AttributeSet;
 import android.util.Pair;
+import android.view.DragEvent;
 import android.view.MotionEvent;
-import android.view.SurfaceHolder;
 import android.view.SurfaceView;
 import android.view.View;
 import android.view.ViewGroup;
 import android.view.accessibility.AccessibilityEvent;
-import android.view.inputmethod.InputMethodManager;
 import android.widget.FrameLayout;
 
 import org.chromium.base.SysUtils;
@@ -45,6 +45,7 @@ import org.chromium.chrome.browser.fullscreen.ChromeFullscreenManager;
 import org.chromium.chrome.browser.fullscreen.ChromeFullscreenManager.FullscreenListener;
 import org.chromium.chrome.browser.tab.EmptyTabObserver;
 import org.chromium.chrome.browser.tab.Tab;
+import org.chromium.chrome.browser.tab.TabContentViewParent;
 import org.chromium.chrome.browser.tab.TabObserver;
 import org.chromium.chrome.browser.tabmodel.EmptyTabModelSelectorObserver;
 import org.chromium.chrome.browser.tabmodel.TabCreatorManager;
@@ -71,10 +72,9 @@ import java.util.List;
  * This class also holds the {@link LayoutManager} responsible to describe the items to be
  * drawn by the UI compositor on the native side.
  */
-public class CompositorViewHolder extends FrameLayout
-        implements LayoutManagerHost, LayoutRenderHost, Invalidator.Host, FullscreenListener {
-    private static List<View> sCachedViewList = new ArrayList<View>();
-    private static List<ContentViewCore> sCachedCVCList = new ArrayList<ContentViewCore>();
+public class CompositorViewHolder extends CoordinatorLayout
+        implements ContentOffsetProvider, LayoutManagerHost, LayoutRenderHost, Invalidator.Host,
+                FullscreenListener {
 
     private boolean mIsKeyboardShowing = false;
 
@@ -91,8 +91,6 @@ public class CompositorViewHolder extends FrameLayout
             new ArrayList<Invalidator.Client>();
     private boolean mSkipInvalidation = false;
 
-    private boolean mSkipNextToolbarTextureUpdate = false;
-
     /**
      * A task to be performed after a resize event.
      */
@@ -113,7 +111,7 @@ public class CompositorViewHolder extends FrameLayout
     private Tab mTabVisible;
 
     /** The currently attached View. */
-    private View mView;
+    private TabContentViewParent mView;
 
     private TabObserver mTabObserver;
     private boolean mEnableCompositorTabStrip;
@@ -198,12 +196,6 @@ public class CompositorViewHolder extends FrameLayout
             public void onContentChanged(Tab tab) {
                 CompositorViewHolder.this.onContentChanged();
             }
-
-            @Override
-            public void onOverlayContentViewCoreAdded(Tab tab, ContentViewCore content) {
-                initializeContentViewCore(content);
-                setSizeOfUnattachedView(content.getContainerView());
-            }
         };
 
         mEnableCompositorTabStrip = DeviceFormFactor.isTablet(getContext());
@@ -227,7 +219,8 @@ public class CompositorViewHolder extends FrameLayout
         });
 
         mCompositorView = new CompositorView(getContext(), this);
-        addView(mCompositorView,
+        // mCompositorView should always be the first child.
+        addView(mCompositorView, 0,
                 new FrameLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.WRAP_CONTENT));
     }
 
@@ -264,13 +257,6 @@ public class CompositorViewHolder extends FrameLayout
         }
     }
 
-    /**
-     * @return The CompositorView.
-     */
-    public SurfaceHolder.Callback2 getSurfaceHolderCallback2() {
-        return mCompositorView;
-    }
-
     /**
      * Reset command line flags. This gets called after the native library finishes
      * loading.
@@ -312,15 +298,22 @@ public class CompositorViewHolder extends FrameLayout
         }
     }
 
+    /**
+     * Perform any initialization necessary for showing a reparented tab.
+     */
+    public void prepareForTabReparenting() {
+        if (mHasDrawnOnce) return;
+
+        // Set the background to white while we wait for the first swap of buffers. This gets
+        // corrected inside the view.
+        mCompositorView.setBackgroundColor(Color.WHITE);
+    }
+
     @Override
     public ResourceManager getResourceManager() {
         return mCompositorView.getResourceManager();
     }
 
-    public ContentOffsetProvider getContentOffsetProvider() {
-        return mCompositorView;
-    }
-
     /**
      * @return The {@link DynamicResourceLoader} for registering resources.
      */
@@ -337,7 +330,8 @@ public class CompositorViewHolder extends FrameLayout
 
     @Override
     public boolean onInterceptTouchEvent(MotionEvent e) {
-        super.onInterceptTouchEvent(e);
+        boolean consumedBySuper = super.onInterceptTouchEvent(e);
+        if (consumedBySuper) return true;
 
         if (mLayoutManager == null) return false;
 
@@ -355,6 +349,9 @@ public class CompositorViewHolder extends FrameLayout
 
     @Override
     public boolean onTouchEvent(MotionEvent e) {
+        boolean consumedBySuper = super.onTouchEvent(e);
+        if (consumedBySuper) return true;
+
         if (mFullscreenManager != null) mFullscreenManager.onMotionEvent(e);
         if (mFullscreenTouchEvent) return true;
         boolean consumed = mLayoutManager != null && mLayoutManager.onTouchEvent(e);
@@ -378,6 +375,23 @@ public class CompositorViewHolder extends FrameLayout
         return super.dispatchHoverEvent(e);
     }
 
+    @Override
+    public boolean dispatchDragEvent(DragEvent e) {
+        ContentViewCore contentViewCore = mTabVisible.getContentViewCore();
+        if (contentViewCore == null) return false;
+
+        if (mLayoutManager != null) mLayoutManager.getViewportPixel(mCacheViewport);
+        contentViewCore.setCurrentTouchEventOffsets(-mCacheViewport.left, -mCacheViewport.top);
+        boolean ret = super.dispatchDragEvent(e);
+
+        int action = e.getAction();
+        if (action == DragEvent.ACTION_DRAG_EXITED || action == DragEvent.ACTION_DRAG_ENDED
+                || action == DragEvent.ACTION_DROP) {
+            contentViewCore.setCurrentTouchEventOffsets(0.f, 0.f);
+        }
+        return ret;
+    }
+
     /**
      * @return The {@link LayoutManager} associated with this view.
      */
@@ -392,51 +406,29 @@ public class CompositorViewHolder extends FrameLayout
         return mCompositorView;
     }
 
+    private View getActiveView() {
+        if (mLayoutManager == null || mTabModelSelector == null) return null;
+        Tab tab = mTabModelSelector.getCurrentTab();
+        return tab != null ? tab.getContentView() : null;
+    }
+
+    private ContentViewCore getActiveContent() {
+        if (mLayoutManager == null || mTabModelSelector == null) return null;
+        Tab tab = mTabModelSelector.getCurrentTab();
+        return tab != null ? tab.getActiveContentViewCore() : null;
+    }
+
     @Override
     protected void onSizeChanged(int w, int h, int oldw, int oldh) {
         super.onSizeChanged(w, h, oldw, oldh);
-        if (mLayoutManager == null) return;
-
-        sCachedViewList.clear();
-        mLayoutManager.getActiveLayout().getAllViews(sCachedViewList);
-
-        boolean resized = false;
-        for (int i = 0; i < sCachedViewList.size(); i++) {
-            resized |= setSizeOfUnattachedView(sCachedViewList.get(i));
-        }
-        sCachedViewList.clear();
-
-        if (resized) requestRender();
+        View view = getActiveView();
+        if (view != null && setSizeOfUnattachedView(view)) requestRender();
     }
 
     @Override
     public void onPhysicalBackingSizeChanged(int width, int height) {
-        if (mLayoutManager == null) return;
-
-        sCachedCVCList.clear();
-        mLayoutManager.getActiveLayout().getAllContentViewCores(sCachedCVCList);
-
-        for (int i = 0; i < sCachedCVCList.size(); i++) {
-            adjustPhysicalBackingSize(sCachedCVCList.get(i), width, height);
-        }
-        sCachedCVCList.clear();
-    }
-
-    @Override
-    public void onOverdrawBottomHeightChanged(int overdrawHeight) {
-        mSkipNextToolbarTextureUpdate = true;
-        requestRender();
-    }
-
-    @Override
-    public int getCurrentOverdrawBottomHeight() {
-        if (mTabVisible != null) {
-            float overdrawBottomHeight = mTabVisible.getFullscreenOverdrawBottomHeightPix();
-            if (!Float.isNaN(overdrawBottomHeight)) {
-                return (int) overdrawBottomHeight;
-            }
-        }
-        return mCompositorView.getOverdrawBottomHeight();
+        ContentViewCore content = getActiveContent();
+        if (content != null) adjustPhysicalBackingSize(content, width, height);
     }
 
     /**
@@ -445,7 +437,7 @@ public class CompositorViewHolder extends FrameLayout
     public void onStart() {
         if (mFullscreenManager != null) {
             mLastContentOffset = mFullscreenManager.getContentOffset();
-            mLastVisibleContentOffset = mFullscreenManager.getVisibleContentOffset();
+            mLastVisibleContentOffset = mFullscreenManager.getTopVisibleContentOffset();
             mFullscreenManager.addListener(this);
         }
         requestRender();
@@ -465,10 +457,10 @@ public class CompositorViewHolder extends FrameLayout
     }
 
     @Override
-    public void onVisibleContentOffsetChanged(float offset) {
+    public void onVisibleContentOffsetChanged(float offset, boolean needsAnimate) {
         mLastVisibleContentOffset = offset;
         propagateViewportToLayouts(getWidth(), getHeight());
-        requestRender();
+        if (needsAnimate) requestRender();
     }
 
     @Override
@@ -494,18 +486,26 @@ public class CompositorViewHolder extends FrameLayout
         if (actionMasked == MotionEvent.ACTION_DOWN
                 || actionMasked == MotionEvent.ACTION_HOVER_ENTER) {
             if (mLayoutManager != null) mLayoutManager.getViewportPixel(mCacheViewport);
-            contentViewCore.setCurrentMotionEventOffsets(-mCacheViewport.left, -mCacheViewport.top);
+            contentViewCore.setCurrentTouchEventOffsets(-mCacheViewport.left, -mCacheViewport.top);
         } else if (canClear && (actionMasked == MotionEvent.ACTION_UP
                                        || actionMasked == MotionEvent.ACTION_CANCEL
                                        || actionMasked == MotionEvent.ACTION_HOVER_EXIT)) {
-            contentViewCore.setCurrentMotionEventOffsets(0.f, 0.f);
+            contentViewCore.setCurrentTouchEventOffsets(0.f, 0.f);
         }
     }
 
     private void propagateViewportToLayouts(int contentWidth, int contentHeight) {
-        int heightMinusTopControls = contentHeight - getTopControlsHeightPixels();
-        mCacheViewport.set(0, (int) mLastContentOffset, contentWidth, contentHeight);
-        mCacheVisibleViewport.set(0, (int) mLastVisibleContentOffset, contentWidth, contentHeight);
+        int heightMinusBrowserControls = contentHeight
+                - (getTopControlsHeightPixels() + getBottomControlsHeightPixels());
+        int bottomControlOffset = mFullscreenManager != null
+                ? (int) mFullscreenManager.getBottomControlOffset() : 0;
+        int viewportBottom =
+                contentHeight - (getBottomControlsHeightPixels() - bottomControlOffset);
+
+        // The only time that mCacheViewport and mCacheVisibleViewport are different is when the
+        // browser has manipulated the browser controls offset.
+        mCacheViewport.set(0, (int) mLastContentOffset, contentWidth, viewportBottom);
+        mCacheVisibleViewport.set(0, (int) mLastVisibleContentOffset, contentWidth, viewportBottom);
         // TODO(changwan): check if this can be merged with setContentMotionEventOffsets.
         if (mTabVisible != null && mTabVisible.getContentViewCore() != null) {
             mTabVisible.getContentViewCore().setSmartClipOffsets(
@@ -513,7 +513,7 @@ public class CompositorViewHolder extends FrameLayout
         }
         if (mLayoutManager != null) {
             mLayoutManager.pushNewViewport(
-                    mCacheViewport, mCacheVisibleViewport, heightMinusTopControls);
+                    mCacheViewport, mCacheVisibleViewport, heightMinusBrowserControls);
         }
     }
 
@@ -533,19 +533,11 @@ public class CompositorViewHolder extends FrameLayout
                 assert mProgressBarDrawingInfo == null;
             }
 
-            mCompositorView.finalizeLayers(mLayoutManager, mSkipNextToolbarTextureUpdate,
+            mCompositorView.finalizeLayers(mLayoutManager, false,
                     mProgressBarDrawingInfo);
-
-            // TODO(changwan): Check if this hack can be removed.
-            // This is a hack to draw one more frame if the screen just rotated for Nexus 10 + L.
-            // See http://crbug/440469 for more.
-            if (mSkipNextToolbarTextureUpdate) {
-                requestRender();
-            }
         }
 
         TraceEvent.end("CompositorViewHolder:layout");
-        mSkipNextToolbarTextureUpdate = false;
     }
 
     @Override
@@ -649,37 +641,28 @@ public class CompositorViewHolder extends FrameLayout
         mFullscreenManager = fullscreen;
         if (mFullscreenManager != null) {
             mLastContentOffset = mFullscreenManager.getContentOffset();
-            mLastVisibleContentOffset = mFullscreenManager.getVisibleContentOffset();
+            mLastVisibleContentOffset = mFullscreenManager.getTopVisibleContentOffset();
             mFullscreenManager.addListener(this);
         }
         propagateViewportToLayouts(getWidth(), getHeight());
     }
 
-    /**
-     * Note that the returned rect is reused for other calls.
-     */
     @Override
-    public Rect getVisibleViewport(Rect rect) {
-        if (rect == null) rect = new Rect();
-        rect.set(0, (int) mLastVisibleContentOffset, getWidth(), getHeight());
-        return rect;
-    }
-
-    @Override
-    public int getTopControlsBackgroundColor() {
+    public int getBrowserControlsBackgroundColor() {
         return mTabVisible == null ? Color.WHITE : mTabVisible.getThemeColor();
     }
 
     @Override
-    public float getTopControlsUrlBarAlpha() {
+    public float getBrowserControlsUrlBarAlpha() {
         return mTabVisible == null
                 ? 1.f
                 : ColorUtils.getTextBoxAlphaForToolbarBackground(mTabVisible);
     }
 
     @Override
-    public boolean areTopControlsPermanentlyHidden() {
-        return mFullscreenManager != null && mFullscreenManager.areTopControlsPermanentlyHidden();
+    public boolean areBrowserControlsPermanentlyHidden() {
+        return mFullscreenManager != null
+                && mFullscreenManager.areBrowserControlsPermanentlyHidden();
     }
 
     @Override
@@ -687,6 +670,20 @@ public class CompositorViewHolder extends FrameLayout
         return mFullscreenManager != null ? mFullscreenManager.getTopControlsHeight() : 0;
     }
 
+    /**
+     * @return The height of the bottom conrols in pixels.
+     */
+    public int getBottomControlsHeightPixels() {
+        return mFullscreenManager != null ? mFullscreenManager.getBottomControlsHeight() : 0;
+    }
+
+    @Override
+    public int getOverlayTranslateY() {
+        return areBrowserControlsPermanentlyHidden()
+                ? getTopControlsHeightPixels()
+                : mCacheVisibleViewport.top;
+    }
+
     /**
      * Sets the URL bar. This is needed so that the ContentViewHolder can find out
      * whether it can claim focus.
@@ -696,13 +693,13 @@ public class CompositorViewHolder extends FrameLayout
     }
 
     @Override
-    protected void onAttachedToWindow() {
+    public void onAttachedToWindow() {
         mInvalidator.set(this);
         super.onAttachedToWindow();
     }
 
     @Override
-    protected void onDetachedFromWindow() {
+    public void onDetachedFromWindow() {
         if (mLayoutManager != null) mLayoutManager.destroy();
         flushInvalidation();
         mInvalidator.set(null);
@@ -778,25 +775,22 @@ public class CompositorViewHolder extends FrameLayout
             }
         });
 
+        mLayerTitleCache.setTabModelSelector(mTabModelSelector);
+
         onContentChanged();
     }
 
     private void updateContentOverlayVisibility(boolean show) {
         if (mView == null) return;
-
-        sCachedCVCList.clear();
-        if (mLayoutManager != null) {
-            mLayoutManager.getActiveLayout().getAllContentViewCores(sCachedCVCList);
-        }
+        ContentViewCore content = getActiveContent();
         if (show) {
             if (mView.getParent() != this) {
-                // Make sure the view isn't a child of something else before we attempt to add it.
-                if (mView.getParent() instanceof ViewGroup) {
-                    ((ViewGroup) mView.getParent()).removeView(mView);
-                }
+                // During tab creation, we temporarily add the new tab's view to a FrameLayout to
+                // measure and lay it out. This way we could show the animation in the stack view.
+                // Therefore we should remove the view from that temporary FrameLayout here.
+                UiUtils.removeViewFromParent(mView);
 
-                for (int i = 0; i < sCachedCVCList.size(); i++) {
-                    ContentViewCore content = sCachedCVCList.get(i);
+                if (content != null) {
                     assert content.isAlive();
                     content.getContainerView().setVisibility(View.VISIBLE);
                     if (mFullscreenManager != null) {
@@ -804,17 +798,20 @@ public class CompositorViewHolder extends FrameLayout
                     }
                 }
 
-                FrameLayout.LayoutParams layoutParams = new FrameLayout.LayoutParams(
-                        LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT);
-                if (mView.getLayoutParams() instanceof MarginLayoutParams) {
-                    MarginLayoutParams existingLayoutParams =
-                            (MarginLayoutParams) mView.getLayoutParams();
-                    layoutParams.leftMargin = existingLayoutParams.leftMargin;
-                    layoutParams.rightMargin = existingLayoutParams.rightMargin;
-                    layoutParams.topMargin = existingLayoutParams.topMargin;
-                    layoutParams.bottomMargin = existingLayoutParams.bottomMargin;
+                CoordinatorLayout.LayoutParams layoutParams;
+                if (mView.getLayoutParams() instanceof CoordinatorLayout.LayoutParams) {
+                    layoutParams = (CoordinatorLayout.LayoutParams) mView.getLayoutParams();
+                } else {
+                    layoutParams = new CoordinatorLayout.LayoutParams(
+                            LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT);
                 }
-                addView(mView, layoutParams);
+                layoutParams.setBehavior(mView.getBehavior());
+                // CompositorView has index of 0; TabContentViewParent has index of 1; omnibox
+                // result container (the scrim) has index of 2, Snackbar (if any) has index of 3.
+                // Setting index here explicitly to avoid TabContentViewParent hiding the scrim.
+                // TODO(ianwen): Use more advanced technologies to ensure z-order of the children of
+                // this class, instead of hard-coding.
+                addView(mView, 1, layoutParams);
 
                 setFocusable(false);
                 setFocusableInTouchMode(false);
@@ -827,22 +824,12 @@ public class CompositorViewHolder extends FrameLayout
                 setFocusable(true);
                 setFocusableInTouchMode(true);
 
-                for (int i = 0; i < sCachedCVCList.size(); i++) {
-                    ContentViewCore content = sCachedCVCList.get(i);
+                if (content != null) {
                     if (content.isAlive()) content.getContainerView().setVisibility(View.INVISIBLE);
                 }
-
-                if (hasFocus()) {
-                    InputMethodManager manager = (InputMethodManager) getContext().getSystemService(
-                            Context.INPUT_METHOD_SERVICE);
-                    if (manager.isActive(this)) {
-                        manager.hideSoftInputFromWindow(getWindowToken(), 0, null);
-                    }
-                }
                 removeView(mView);
             }
         }
-        sCachedCVCList.clear();
     }
 
     @Override
@@ -866,7 +853,7 @@ public class CompositorViewHolder extends FrameLayout
     private void setTab(Tab tab) {
         if (tab != null) tab.loadIfNeeded();
 
-        View newView = tab != null ? tab.getView() : null;
+        TabContentViewParent newView = tab != null ? tab.getView() : null;
         if (mView == newView) return;
 
         // TODO(dtrainor): Look into changing this only if the views differ, but still parse the
@@ -887,34 +874,16 @@ public class CompositorViewHolder extends FrameLayout
     }
 
     /**
-     * Sets the correct size for all {@link View}s on {@code tab} and sets the correct rendering
-     * parameters on all {@link ContentViewCore}s on {@code tab}.
+     * Sets the correct size for {@link View} on {@code tab} and sets the correct rendering
+     * parameters on {@link ContentViewCore} on {@code tab}.
      * @param tab The {@link Tab} to initialize.
      */
     private void initializeTab(Tab tab) {
-        sCachedCVCList.clear();
-        if (mLayoutManager != null) {
-            mLayoutManager.getActiveLayout().getAllContentViewCores(sCachedCVCList);
-        }
+        ContentViewCore content = tab.getActiveContentViewCore();
+        if (content != null) initializeContentViewCore(content);
 
-        for (int i = 0; i < sCachedCVCList.size(); i++) {
-            initializeContentViewCore(sCachedCVCList.get(i));
-        }
-        sCachedCVCList.clear();
-
-        sCachedViewList.clear();
-        tab.getAllContentViews(sCachedViewList);
-
-        for (int i = 0; i < sCachedViewList.size(); i++) {
-            View view = sCachedViewList.get(i);
-            // Calling View#measure() and View#layout() on a View before adding it to the view
-            // hierarchy seems to cause issues with compound drawables on some versions of Android.
-            // We don't need to proactively size the NTP as we don't need the Android view to render
-            // if it's not actually attached to the view hierarchy (http://crbug.com/462114).
-            if (view == tab.getView() && tab.isNativePage()) continue;
-            setSizeOfUnattachedView(view);
-        }
-        sCachedViewList.clear();
+        View view = tab.getContentView();
+        if (view != tab.getView() || !tab.isNativePage()) setSizeOfUnattachedView(view);
     }
 
     /**
@@ -923,9 +892,10 @@ public class CompositorViewHolder extends FrameLayout
      * @param contentViewCore The {@link ContentViewCore} to initialize.
      */
     private void initializeContentViewCore(ContentViewCore contentViewCore) {
-        contentViewCore.setCurrentMotionEventOffsets(0.f, 0.f);
-        contentViewCore.setTopControlsHeight(
-                getTopControlsHeightPixels(), contentViewCore.doTopControlsShrinkBlinkSize());
+        contentViewCore.setCurrentTouchEventOffsets(0.f, 0.f);
+        contentViewCore.setTopControlsHeight(getTopControlsHeightPixels(),
+                contentViewCore.doBrowserControlsShrinkBlinkSize());
+        contentViewCore.setBottomControlsHeight(getBottomControlsHeightPixels());
 
         adjustPhysicalBackingSize(contentViewCore,
                 mCompositorView.getWidth(), mCompositorView.getHeight());
