@@ -340,6 +340,9 @@ public class ToolbarSwipeLayout extends Layout implements Animatable<ToolbarSwip
         }
     }
 
+    @Override
+    public void onPropertyAnimationFinished(Property prop) {}
+
     @Override
     protected SceneLayer getSceneLayer() {
         return mSceneLayer;
@@ -353,6 +356,6 @@ public class ToolbarSwipeLayout extends Layout implements Animatable<ToolbarSwip
                 resourceManager, fullscreenManager);
         assert mSceneLayer != null;
         mSceneLayer.pushLayers(getContext(), viewport, contentViewport, this, layerTitleCache,
-                tabContentManager, resourceManager);
+                tabContentManager, resourceManager, fullscreenManager);
     }
 }
