@@ -22,13 +22,20 @@ public interface SceneOverlay {
      *
      * @param layerTitleCache A layer title cache.
      * @param resourceManager A resource manager.
-     * @param yOffset Current top controls offset in dp.
+     * @param yOffset Current browser controls offset in dp.
      * @return A {@link SceneOverlayLayer} that represents an scene overlay.
      * Or {@code null} if this {@link SceneOverlay} doesn't have a tree.
      */
     SceneOverlayLayer getUpdatedSceneOverlayTree(LayerTitleCache layerTitleCache,
             ResourceManager resourceManager, float yOffset);
 
+    /**
+     * Notify the layout that a SceneOverlay is visible. If not visible, the content tree will not
+     * be modified.
+     * @return True if the SceneOverlay tree is showing.
+     */
+    boolean isSceneOverlayTreeShowing();
+
     /**
      * @return The {@link EventFilter} that processes events for this {@link SceneOverlay}.
      */
@@ -48,6 +55,11 @@ public interface SceneOverlay {
      */
     void getVirtualViews(List<VirtualView> views);
 
+    /**
+     * @return True if the overlay requires the Android browser controls view to be hidden.
+     */
+    boolean shouldHideAndroidBrowserControls();
+
     /**
      * Helper-specific updates. Cascades the values updated by the animations and flings.
      * @param time The current time of the app in ms.
@@ -56,6 +68,22 @@ public interface SceneOverlay {
      */
     boolean updateOverlay(long time, long dt);
 
+    /**
+     * Notification that the system back button was pressed.
+     * @return True if system back button press was consumed by this overlay.
+     */
+    boolean onBackPressed();
+
+    /**
+     * A notification to the overlay that the containing layout is being hidden.
+     */
+    void onHideLayout();
+
+    /**
+     * @return True if this overlay handles tab creation.
+     */
+    boolean handlesTabCreating();
+
     /**
      * Notify the a title has changed.
      *
