@@ -15,14 +15,16 @@ import org.chromium.chrome.browser.compositor.layouts.LayoutRenderHost;
 import org.chromium.chrome.browser.compositor.layouts.LayoutUpdateHost;
 import org.chromium.chrome.browser.compositor.layouts.components.CompositorButton;
 import org.chromium.chrome.browser.compositor.layouts.components.VirtualView;
-import org.chromium.chrome.browser.compositor.layouts.content.TabContentManager;
 import org.chromium.chrome.browser.compositor.layouts.eventfilter.AreaGestureEventFilter;
 import org.chromium.chrome.browser.compositor.layouts.eventfilter.EventFilter;
 import org.chromium.chrome.browser.compositor.overlays.SceneOverlay;
 import org.chromium.chrome.browser.compositor.scene_layer.SceneOverlayLayer;
 import org.chromium.chrome.browser.compositor.scene_layer.TabStripSceneLayer;
+import org.chromium.chrome.browser.tab.Tab;
 import org.chromium.chrome.browser.tabmodel.TabCreatorManager;
+import org.chromium.chrome.browser.tabmodel.TabModel;
 import org.chromium.chrome.browser.tabmodel.TabModelSelector;
+import org.chromium.ui.base.LocalizationUtils;
 import org.chromium.ui.resources.ResourceManager;
 
 import java.util.List;
@@ -32,9 +34,6 @@ import java.util.List;
  * all input and model events to the proper destination.
  */
 public class StripLayoutHelperManager implements SceneOverlay {
-    // Visibility Constants
-    private static final float TAB_STRIP_HEIGHT_DP = 40.f;
-
     // Caching Variables
     private final RectF mStripFilterArea = new RectF();
 
@@ -44,15 +43,14 @@ public class StripLayoutHelperManager implements SceneOverlay {
 
     // Model selector buttons constants.
     private static final float MODEL_SELECTOR_BUTTON_Y_OFFSET_DP = 10.f;
-    private static final float MODEL_SELECTOR_BUTTON_RIGHT_PADDING_DP = 6.f;
-    private static final float MODEL_SELECTOR_BUTTON_LEFT_PADDING_DP = 3.f;
+    private static final float MODEL_SELECTOR_BUTTON_END_PADDING_DP = 6.f;
+    private static final float MODEL_SELECTOR_BUTTON_START_PADDING_DP = 3.f;
     private static final float MODEL_SELECTOR_BUTTON_WIDTH_DP = 24.f;
     private static final float MODEL_SELECTOR_BUTTON_HEIGHT_DP = 24.f;
 
     // External influences
     private TabModelSelector mTabModelSelector;
     private final LayoutUpdateHost mUpdateHost;
-    private final LayoutRenderHost mRenderHost;
 
     // Event Filters
     private final AreaGestureEventFilter mEventFilter;
@@ -63,8 +61,8 @@ public class StripLayoutHelperManager implements SceneOverlay {
     private final StripLayoutHelper mIncognitoHelper;
 
     // UI State
-    private float mWidth;
-    private final float mHeight;
+    private float mWidth;  // in dp units
+    private final float mHeight;  // in dp units
     private int mOrientation;
     private final CompositorButton mModelSelectorButton;
 
@@ -78,10 +76,7 @@ public class StripLayoutHelperManager implements SceneOverlay {
      */
     public StripLayoutHelperManager(Context context, LayoutUpdateHost updateHost,
             LayoutRenderHost renderHost, AreaGestureEventFilter eventFilter) {
-        mHeight = TAB_STRIP_HEIGHT_DP;
-
         mUpdateHost = updateHost;
-        mRenderHost = renderHost;
         mTabStripTreeProvider = new TabStripSceneLayer(context);
 
         mEventFilter = eventFilter;
@@ -100,6 +95,7 @@ public class StripLayoutHelperManager implements SceneOverlay {
         mModelSelectorButton.setY(MODEL_SELECTOR_BUTTON_Y_OFFSET_DP);
 
         Resources res = context.getResources();
+        mHeight = res.getDimension(R.dimen.tab_strip_height) / res.getDisplayMetrics().density;
         mModelSelectorButton.setAccessibilityDescription(
                 res.getString(R.string.accessibility_tabstrip_btn_incognito_toggle_standard),
                 res.getString(R.string.accessibility_tabstrip_btn_incognito_toggle_incognito));
@@ -113,6 +109,8 @@ public class StripLayoutHelperManager implements SceneOverlay {
     public void destroy() {
         mTabStripTreeProvider.destroy();
         mTabStripTreeProvider = null;
+        mIncognitoHelper.destroy();
+        mNormalHelper.destroy();
     }
 
     @Override
@@ -120,11 +118,22 @@ public class StripLayoutHelperManager implements SceneOverlay {
             ResourceManager resourceManager, float yOffset) {
         assert mTabStripTreeProvider != null;
 
+        Tab selectedTab = mTabModelSelector.getCurrentModel().getTabAt(
+                mTabModelSelector.getCurrentModel().index());
+        int selectedTabId = selectedTab == null ? TabModel.INVALID_TAB_INDEX : selectedTab.getId();
         mTabStripTreeProvider.pushAndUpdateStrip(this, layerTitleCache, resourceManager,
-                getActiveStripLayoutHelper().getStripLayoutTabsToRender(), yOffset);
+                getActiveStripLayoutHelper().getStripLayoutTabsToRender(), yOffset,
+                selectedTabId);
         return mTabStripTreeProvider;
     }
 
+    @Override
+    public boolean isSceneOverlayTreeShowing() {
+        // TODO(mdjones): This matches existing behavior but can be improved to return false if
+        // the browser controls offset is equal to the browser controls height.
+        return true;
+    }
+
     @Override
     public EventFilter getEventFilter() {
         return mEventFilter;
@@ -135,8 +144,13 @@ public class StripLayoutHelperManager implements SceneOverlay {
             float width, float height, float visibleViewportOffsetY, int orientation) {
         mWidth = width;
         mOrientation = orientation;
-        mModelSelectorButton.setX(
-                mWidth - MODEL_SELECTOR_BUTTON_WIDTH_DP - MODEL_SELECTOR_BUTTON_RIGHT_PADDING_DP);
+        if (!LocalizationUtils.isLayoutRtl()) {
+            mModelSelectorButton.setX(
+                    mWidth - MODEL_SELECTOR_BUTTON_WIDTH_DP - MODEL_SELECTOR_BUTTON_END_PADDING_DP);
+        } else {
+            mModelSelectorButton.setX(MODEL_SELECTOR_BUTTON_END_PADDING_DP);
+        }
+
         mNormalHelper.onSizeChanged(mWidth, mHeight);
         mIncognitoHelper.onSizeChanged(mWidth, mHeight);
 
@@ -158,6 +172,25 @@ public class StripLayoutHelperManager implements SceneOverlay {
         getActiveStripLayoutHelper().getVirtualViews(views);
     }
 
+    @Override
+    public boolean shouldHideAndroidBrowserControls() {
+        return false;
+    }
+
+    /**
+     * @return The opacity to use for the fade on the left side of the tab strip.
+     */
+    public float getLeftFadeOpacity() {
+        return getActiveStripLayoutHelper().getLeftFadeOpacity();
+    }
+
+    /**
+     * @return The opacity to use for the fade on the right side of the tab strip.
+     */
+    public float getRightFadeOpacity() {
+        return getActiveStripLayoutHelper().getRightFadeOpacity();
+    }
+
     /**
      * @return The brightness of background tabs in the tabstrip.
      */
@@ -165,13 +198,6 @@ public class StripLayoutHelperManager implements SceneOverlay {
         return getActiveStripLayoutHelper().getBackgroundTabBrightness();
     }
 
-    /**
-     * @param brightness Sets the brightness for the entire tabstrip.
-     */
-    public void setBrightness(float brightness) {
-        getActiveStripLayoutHelper().setBrightness(brightness);
-    }
-
     /**
      * @return The brightness of the entire tabstrip.
      */
@@ -184,16 +210,15 @@ public class StripLayoutHelperManager implements SceneOverlay {
      * represent, and various objects associated with it.
      * @param modelSelector The {@link TabModelSelector} to visually represent.
      * @param tabCreatorManager The {@link TabCreatorManager}, used to create new tabs.
-     * @param tabContentManager The {@link TabContentManager}, used to provide display content for
-     *                          tabs.
      */
     public void setTabModelSelector(TabModelSelector modelSelector,
-            TabCreatorManager tabCreatorManager, TabContentManager tabContentManager) {
+            TabCreatorManager tabCreatorManager) {
         if (mTabModelSelector == modelSelector) return;
+
         mTabModelSelector = modelSelector;
-        mNormalHelper.setTabModel(mTabModelSelector.getModel(false), tabContentManager,
+        mNormalHelper.setTabModel(mTabModelSelector.getModel(false),
                 tabCreatorManager.getTabCreator(false));
-        mIncognitoHelper.setTabModel(mTabModelSelector.getModel(true), tabContentManager,
+        mIncognitoHelper.setTabModel(mTabModelSelector.getModel(true),
                 tabCreatorManager.getTabCreator(true));
         tabModelSwitched(mTabModelSelector.isIncognitoSelected());
     }
@@ -234,6 +259,19 @@ public class StripLayoutHelperManager implements SceneOverlay {
         return getActiveStripLayoutHelper().updateLayout(time, dt);
     }
 
+    @Override
+    public boolean onBackPressed() {
+        return false;
+    }
+
+    @Override
+    public void onHideLayout() {}
+
+    @Override
+    public boolean handlesTabCreating() {
+        return false;
+    }
+
     @Override
     public void tabStateInitialized() {
         updateModelSwitcherButton();
@@ -244,6 +282,12 @@ public class StripLayoutHelperManager implements SceneOverlay {
         if (incognito == mIsIncognito) return;
         mIsIncognito = incognito;
 
+        if (mIsIncognito) {
+            mIncognitoHelper.tabModelSelected();
+        } else {
+            mNormalHelper.tabModelSelected();
+        }
+
         updateModelSwitcherButton();
 
         mUpdateHost.requestUpdate();
@@ -255,12 +299,13 @@ public class StripLayoutHelperManager implements SceneOverlay {
             boolean isVisible = mTabModelSelector.getModel(true).getCount() != 0;
             mModelSelectorButton.setVisible(isVisible);
 
-            float rightMargin = isVisible
-                    ? MODEL_SELECTOR_BUTTON_WIDTH_DP + MODEL_SELECTOR_BUTTON_RIGHT_PADDING_DP
-                            + MODEL_SELECTOR_BUTTON_LEFT_PADDING_DP
+            float endMargin = isVisible
+                    ? MODEL_SELECTOR_BUTTON_WIDTH_DP + MODEL_SELECTOR_BUTTON_END_PADDING_DP
+                            + MODEL_SELECTOR_BUTTON_START_PADDING_DP
                     : 0.0f;
-            mNormalHelper.setRightMargin(rightMargin);
-            mIncognitoHelper.setRightMargin(rightMargin);
+
+            mNormalHelper.setEndMargin(endMargin);
+            mIncognitoHelper.setEndMargin(endMargin);
         }
     }
 
