@@ -5,6 +5,7 @@
 package org.chromium.chrome.browser.contextualsearch;
 
 import android.app.Activity;
+import android.view.ContextMenu;
 
 import org.chromium.base.annotations.CalledByNative;
 import org.chromium.chrome.browser.ChromeActivity;
@@ -17,11 +18,13 @@ import org.chromium.chrome.browser.tab.EmptyTabObserver;
 import org.chromium.chrome.browser.tab.Tab;
 import org.chromium.content.browser.ContentViewCore;
 import org.chromium.content_public.browser.GestureStateListener;
+import org.chromium.net.NetworkChangeNotifier;
 
 /**
  * Manages the activation and gesture listeners for ContextualSearch on a given tab.
  */
-public class ContextualSearchTabHelper extends EmptyTabObserver {
+public class ContextualSearchTabHelper
+        extends EmptyTabObserver implements NetworkChangeNotifier.ConnectionTypeObserver {
     /**
      * Notification handler for Contextual Search events.
      */
@@ -52,8 +55,16 @@ public class ContextualSearchTabHelper extends EmptyTabObserver {
     private ContextualSearchTabHelper(Tab tab) {
         mTab = tab;
         tab.addObserver(this);
+        // Connect to a network, unless under test.
+        if (NetworkChangeNotifier.isInitialized()) {
+            NetworkChangeNotifier.addConnectionTypeObserver(this);
+        }
     }
 
+    // ============================================================================================
+    // EmptyTabObserver overrides.
+    // ============================================================================================
+
     @Override
     public void onPageLoadStarted(Tab tab, String url) {
         if (tab.getContentViewCore() == null) {
@@ -64,7 +75,7 @@ public class ContextualSearchTabHelper extends EmptyTabObserver {
         mBaseContentViewCore = tab.getContentViewCore();
         // Add Contextual Search here in case it couldn't get added in onContentChanged() due to
         // being too early in initialization of Chrome (ContextualSearchManager being null).
-        setContextualSearchHooks(mBaseContentViewCore);
+        updateContextualSearchHooks(mBaseContentViewCore);
 
         ContextualSearchManager manager = getContextualSearchManager();
         if (manager != null) {
@@ -84,7 +95,7 @@ public class ContextualSearchTabHelper extends EmptyTabObserver {
                     new TemplateUrlServiceObserver() {
                         @Override
                         public void onTemplateURLServiceChanged() {
-                            onContextualSearchPrefChanged();
+                            updateContextualSearchHooks(mBaseContentViewCore);
                         }
                     };
             TemplateUrlService.getInstance().addObserver(mTemplateUrlObserver);
@@ -118,6 +129,32 @@ public class ContextualSearchTabHelper extends EmptyTabObserver {
         }
     }
 
+    @Override
+    public void onReparentingFinished(Tab tab) {
+        updateHooksForNewContentViewCore(tab);
+    }
+
+    @Override
+    public void onContextMenuShown(Tab tab, ContextMenu menu) {
+        ContextualSearchManager manager = getContextualSearchManager();
+        if (manager != null) {
+            manager.onContextMenuShown();
+        }
+    }
+
+    // ============================================================================================
+    // NetworkChangeNotifier.ConnectionTypeObserver overrides.
+    // ============================================================================================
+
+    @Override
+    public void onConnectionTypeChanged(int connectionType) {
+        updateContextualSearchHooks(mBaseContentViewCore);
+    }
+
+    // ============================================================================================
+    // Private helpers.
+    // ============================================================================================
+
     /**
      * Should be called whenever the Tab's ContentViewCore changes. Removes hooks from the
      * existing ContentViewCore, if necessary and then adds hooks for the new ContentViewCore.
@@ -126,15 +163,15 @@ public class ContextualSearchTabHelper extends EmptyTabObserver {
     private void updateHooksForNewContentViewCore(Tab tab) {
         removeContextualSearchHooks(mBaseContentViewCore);
         mBaseContentViewCore = tab.getContentViewCore();
-        setContextualSearchHooks(mBaseContentViewCore);
+        updateContextualSearchHooks(mBaseContentViewCore);
     }
 
     /**
-     * Sets up the Contextual Search hooks, adding or removing them depending on whether it is
+     * Updates the Contextual Search hooks, adding or removing them depending on whether it is
      * currently active.
      * @param cvc The content view core to attach the gesture state listener to.
      */
-    private void setContextualSearchHooks(ContentViewCore cvc) {
+    private void updateContextualSearchHooks(ContentViewCore cvc) {
         if (cvc == null) return;
 
         if (isContextualSearchActive(cvc)) {
@@ -149,10 +186,11 @@ public class ContextualSearchTabHelper extends EmptyTabObserver {
      * @param cvc The content view core to attach the gesture state listener to.
      */
     private void addContextualSearchHooks(ContentViewCore cvc) {
-        if (mGestureStateListener == null) {
-            mGestureStateListener = getContextualSearchManager().getGestureStateListener();
+        ContextualSearchManager manager = getContextualSearchManager();
+        if (mGestureStateListener == null && manager != null) {
+            mGestureStateListener = manager.getGestureStateListener();
             cvc.addGestureStateListener(mGestureStateListener);
-            cvc.setContextualSearchClient(getContextualSearchManager());
+            cvc.setContextualSearchClient(manager);
         }
     }
 
@@ -178,13 +216,23 @@ public class ContextualSearchTabHelper extends EmptyTabObserver {
         if (manager == null) return false;
 
         return !cvc.getWebContents().isIncognito()
-            && !PrefServiceBridge.getInstance().isContextualSearchDisabled()
-            && TemplateUrlService.getInstance().isDefaultSearchEngineGoogle()
-            // Svelte and Accessibility devices are incompatible with the first-run flow and
-            // Talkback has poor interaction with tap to search (see http://crbug.com/399708 and
-            // http://crbug.com/396934).
-            // TODO(jeremycho): Handle these cases.
-            && !manager.isRunningInCompatibilityMode();
+                && !PrefServiceBridge.getInstance().isContextualSearchDisabled()
+                && TemplateUrlService.getInstance().isDefaultSearchEngineGoogle()
+                // Svelte and Accessibility devices are incompatible with the first-run flow and
+                // Talkback has poor interaction with tap to search (see http://crbug.com/399708 and
+                // http://crbug.com/396934).
+                && !manager.isRunningInCompatibilityMode()
+                && !(mTab.isShowingErrorPage() || mTab.isShowingInterstitialPage())
+                && isDeviceOnline(manager);
+    }
+
+    /**
+     * @return Whether the device is online, or we have disabled online-detection.
+     */
+    private boolean isDeviceOnline(ContextualSearchManager manager) {
+        if (ContextualSearchFieldTrial.isOnlineDetectionDisabled()) return true;
+
+        return manager.isDeviceOnline();
     }
 
     /**
@@ -198,9 +246,20 @@ public class ContextualSearchTabHelper extends EmptyTabObserver {
         return null;
     }
 
+    // ============================================================================================
+    // Native support.
+    // ============================================================================================
+
     @CalledByNative
     private void onContextualSearchPrefChanged() {
-        setContextualSearchHooks(mBaseContentViewCore);
+        updateContextualSearchHooks(mBaseContentViewCore);
+
+        ContextualSearchManager manager = getContextualSearchManager();
+        if (manager != null) {
+            boolean isEnabled = !PrefServiceBridge.getInstance().isContextualSearchDisabled()
+                    && !PrefServiceBridge.getInstance().isContextualSearchUninitialized();
+            manager.onContextualSearchPrefChanged(isEnabled);
+        }
     }
 
     private native long nativeInit(Profile profile);
