@@ -6,64 +6,88 @@ package org.chromium.chrome.browser.externalnav;
 
 import android.Manifest.permission;
 import android.app.Activity;
-import android.content.ComponentName;
 import android.content.Context;
 import android.content.DialogInterface;
 import android.content.DialogInterface.OnCancelListener;
 import android.content.DialogInterface.OnClickListener;
 import android.content.Intent;
 import android.content.IntentFilter;
-import android.content.pm.ActivityInfo;
 import android.content.pm.PackageManager;
 import android.content.pm.ResolveInfo;
 import android.net.Uri;
 import android.os.Build;
-import android.os.TransactionTooLargeException;
+import android.os.StrictMode;
 import android.provider.Browser;
 import android.provider.Telephony;
 import android.support.v7.app.AlertDialog;
 import android.text.TextUtils;
-import android.util.Log;
+import android.webkit.MimeTypeMap;
 
 import org.chromium.base.ApplicationState;
 import org.chromium.base.ApplicationStatus;
+import org.chromium.base.ContextUtils;
 import org.chromium.base.PathUtils;
 import org.chromium.base.ThreadUtils;
+import org.chromium.base.VisibleForTesting;
+import org.chromium.base.metrics.RecordUserAction;
 import org.chromium.chrome.R;
 import org.chromium.chrome.browser.ChromeActivity;
+import org.chromium.chrome.browser.ChromeTabbedActivity2;
 import org.chromium.chrome.browser.IntentHandler;
 import org.chromium.chrome.browser.document.ChromeLauncherActivity;
 import org.chromium.chrome.browser.externalnav.ExternalNavigationHandler.OverrideUrlLoadingResult;
+import org.chromium.chrome.browser.instantapps.AuthenticatedProxyActivity;
+import org.chromium.chrome.browser.instantapps.InstantAppsHandler;
 import org.chromium.chrome.browser.tab.Tab;
-import org.chromium.chrome.browser.util.FeatureUtilities;
 import org.chromium.chrome.browser.util.IntentUtils;
 import org.chromium.chrome.browser.util.UrlUtilities;
+import org.chromium.chrome.browser.webapps.WebappActivity;
 import org.chromium.content_public.browser.LoadUrlParams;
+import org.chromium.content_public.browser.NavigationController;
+import org.chromium.content_public.browser.NavigationEntry;
 import org.chromium.content_public.common.Referrer;
 import org.chromium.ui.base.PageTransition;
+import org.chromium.ui.base.WindowAndroid;
 import org.chromium.ui.base.WindowAndroid.PermissionCallback;
+import org.chromium.webapk.lib.client.WebApkValidator;
 
+import java.util.ArrayList;
 import java.util.List;
 
 /**
  * The main implementation of the {@link ExternalNavigationDelegate}.
  */
 public class ExternalNavigationDelegateImpl implements ExternalNavigationDelegate {
-    private static final String TAG = "ExternalNavigationDelegateImpl";
+    // Instant Apps system resolver activity on N-MR1+.
+    @VisibleForTesting
+    static final String EPHEMERAL_INSTALLER_CLASS =
+            "com.google.android.gms.instantapps.routing.EphemeralInstallerActivity";
     private static final String PDF_VIEWER = "com.google.android.apps.docs";
     private static final String PDF_MIME = "application/pdf";
     private static final String PDF_SUFFIX = ".pdf";
-    private final ChromeActivity mActivity;
+    private static final String PDF_EXTENSION = "pdf";
 
-    public ExternalNavigationDelegateImpl(ChromeActivity activity) {
-        mActivity = activity;
+    protected final Context mApplicationContext;
+    private final Tab mTab;
+
+    public ExternalNavigationDelegateImpl(Tab tab) {
+        mTab = tab;
+        mApplicationContext = tab.getWindowAndroid().getApplicationContext();
     }
 
     /**
-     * @return The activity that this delegate is associated with.
+     * Get a {@link Context} linked to this delegate with preference to {@link Activity}.
+     * The tab this delegate associates with can swap the {@link Activity} it is hosted in and
+     * during the swap, there might not be an available {@link Activity}.
+     * @return The activity {@link Context} if it can be reached.
+     *         Application {@link Context} if not.
      */
-    protected final Activity getActivity() {
-        return mActivity;
+    protected final Context getAvailableContext() {
+        if (mTab.getWindowAndroid() == null) return mApplicationContext;
+        Context activityContext = WindowAndroid.activityFromContext(
+                mTab.getWindowAndroid().getContext().get());
+        if (activityContext == null) return mApplicationContext;
+        return activityContext;
     }
 
     /**
@@ -117,6 +141,10 @@ public class ExternalNavigationDelegateImpl implements ExternalNavigationDelegat
                             } else if (PDF_VIEWER.equals(pName)) {
                                 if (isPdfIntent(intent)) {
                                     intent.setClassName(pName, resolveInfo.activityInfo.name);
+                                    Uri referrer = new Uri.Builder().scheme(
+                                            IntentHandler.ANDROID_APP_REFERRER_SCHEME).authority(
+                                                    packageName).build();
+                                    intent.putExtra(Intent.EXTRA_REFERRER, referrer);
                                     hasPdfViewer = true;
                                     break;
                                 }
@@ -130,7 +158,7 @@ public class ExternalNavigationDelegateImpl implements ExternalNavigationDelegat
             }
             return activityResolved;
         } catch (RuntimeException e) {
-            logTransactionTooLargeOrRethrow(e, intent);
+            IntentUtils.logTransactionTooLargeOrRethrow(e, intent);
         }
         return false;
     }
@@ -155,11 +183,11 @@ public class ExternalNavigationDelegateImpl implements ExternalNavigationDelegat
         assert Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP
                 || !ThreadUtils.runningOnUiThread();
         try {
-            Context context = ApplicationStatus.getApplicationContext();
+            Context context = ContextUtils.getApplicationContext();
             PackageManager pm = context.getPackageManager();
             return pm.resolveActivity(intent, 0);
         } catch (RuntimeException e) {
-            logTransactionTooLargeOrRethrow(e, intent);
+            IntentUtils.logTransactionTooLargeOrRethrow(e, intent);
         }
         return null;
     }
@@ -177,29 +205,101 @@ public class ExternalNavigationDelegateImpl implements ExternalNavigationDelegat
     public static boolean willChromeHandleIntent(
             Context context, Intent intent, boolean matchDefaultOnly) {
         try {
+            // Early-out if the intent targets Chrome.
+            if (intent.getComponent() != null
+                    && context.getPackageName().equals(intent.getComponent().getPackageName())) {
+                return true;
+            }
+
+            // Fall back to the more expensive querying of Android when the intent doesn't target
+            // Chrome.
             ResolveInfo info = context.getPackageManager().resolveActivity(
                     intent, matchDefaultOnly ? PackageManager.MATCH_DEFAULT_ONLY : 0);
             return info != null
                     && info.activityInfo.packageName.equals(context.getPackageName());
         } catch (RuntimeException e) {
-            logTransactionTooLargeOrRethrow(e, intent);
+            IntentUtils.logTransactionTooLargeOrRethrow(e, intent);
             return false;
         }
     }
 
     @Override
-    public List<ComponentName> queryIntentActivities(Intent intent) {
-        return IntentUtils.getIntentHandlers(mActivity, intent);
+    public List<ResolveInfo> queryIntentActivities(Intent intent) {
+        // White-list for Samsung. See http://crbug.com/613977 for more context.
+        StrictMode.ThreadPolicy oldPolicy = StrictMode.allowThreadDiskReads();
+        try {
+            return mApplicationContext.getPackageManager().queryIntentActivities(intent,
+                    PackageManager.GET_RESOLVED_FILTER);
+        } catch (RuntimeException e) {
+            IntentUtils.logTransactionTooLargeOrRethrow(e, intent);
+            return null;
+        } finally {
+            StrictMode.setThreadPolicy(oldPolicy);
+        }
     }
 
     @Override
     public boolean willChromeHandleIntent(Intent intent) {
-        return willChromeHandleIntent(mActivity, intent, false);
+        return willChromeHandleIntent(mApplicationContext, intent, false);
     }
 
     @Override
-    public boolean isSpecializedHandlerAvailable(Intent intent) {
-        return isPackageSpecializedHandler(mActivity, null, intent);
+    public boolean isSpecializedHandlerAvailable(List<ResolveInfo> infos) {
+        return countSpecializedHandlers(infos) > 0;
+    }
+
+    @Override
+    public boolean isWithinCurrentWebappScope(String url) {
+        Context context = getAvailableContext();
+        if (context instanceof WebappActivity) {
+            String scope = ((WebappActivity) context).getWebappScope();
+            return url.startsWith(scope);
+        }
+        return false;
+    }
+
+    @Override
+    public int countSpecializedHandlers(List<ResolveInfo> infos) {
+        return getSpecializedHandlersWithFilter(infos, null).size();
+    }
+
+    @VisibleForTesting
+    static ArrayList<String> getSpecializedHandlersWithFilter(
+            List<ResolveInfo> infos, String filterPackageName) {
+        ArrayList<String> result = new ArrayList<>();
+        if (infos == null) {
+            return result;
+        }
+
+        for (ResolveInfo info : infos) {
+            IntentFilter filter = info.filter;
+            if (filter == null) {
+                // Error on the side of classifying ResolveInfo as generic.
+                continue;
+            }
+            if (filter.countDataAuthorities() == 0 && filter.countDataPaths() == 0) {
+                // Don't count generic handlers.
+                continue;
+            }
+
+            if (!TextUtils.isEmpty(filterPackageName)
+                    && (info.activityInfo == null
+                               || !info.activityInfo.packageName.equals(filterPackageName))) {
+                continue;
+            }
+
+            if (info.activityInfo != null) {
+                if (EPHEMERAL_INSTALLER_CLASS.equals(info.activityInfo.name)) {
+                    // Don't consider the Instant Apps resolver a specialized application.
+                    continue;
+                }
+
+                result.add(info.activityInfo.packageName);
+            } else {
+                result.add("");
+            }
+        }
+        return result;
     }
 
     /**
@@ -216,66 +316,92 @@ public class ExternalNavigationDelegateImpl implements ExternalNavigationDelegat
         try {
             List<ResolveInfo> handlers = context.getPackageManager().queryIntentActivities(
                     intent, PackageManager.GET_RESOLVED_FILTER);
-            if (handlers == null || handlers.size() == 0) return false;
-            for (ResolveInfo resolveInfo : handlers) {
-                IntentFilter filter = resolveInfo.filter;
-                if (filter == null) {
-                    // No intent filter matches this intent?
-                    // Error on the side of staying in the browser, ignore
-                    continue;
-                }
-                if (filter.countDataAuthorities() == 0 || filter.countDataPaths() == 0) {
-                    // Generic handler, skip
-                    continue;
-                }
-                if (TextUtils.isEmpty(packageName)) return true;
-                ActivityInfo activityInfo = resolveInfo.activityInfo;
-                if (activityInfo == null) continue;
-                if (!activityInfo.packageName.equals(packageName)) continue;
-                return true;
-            }
+            return getSpecializedHandlersWithFilter(handlers, packageName).size() > 0;
         } catch (RuntimeException e) {
-            logTransactionTooLargeOrRethrow(e, intent);
+            IntentUtils.logTransactionTooLargeOrRethrow(e, intent);
         }
         return false;
     }
 
     @Override
-    public String getPackageName() {
-        return mActivity.getPackageName();
+    public String findWebApkPackageName(List<ResolveInfo> infos) {
+        return WebApkValidator.findWebApkPackage(mApplicationContext, infos);
     }
 
     @Override
-    public void startActivity(Intent intent) {
+    public String getPackageName() {
+        return mApplicationContext.getPackageName();
+    }
+
+    @Override
+    public void startActivity(Intent intent, boolean proxy) {
         try {
-            forcePdfViewerAsIntentHandlerIfNeeded(mActivity, intent);
-            mActivity.startActivity(intent);
+            forcePdfViewerAsIntentHandlerIfNeeded(mApplicationContext, intent);
+            if (proxy) {
+                dispatchAuthenticatedIntent(intent);
+            } else {
+                Context context = getAvailableContext();
+                if (!(context instanceof Activity)) intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
+                context.startActivity(intent);
+            }
+            recordExternalNavigationDispatched(intent);
         } catch (RuntimeException e) {
-            logTransactionTooLargeOrRethrow(e, intent);
+            IntentUtils.logTransactionTooLargeOrRethrow(e, intent);
         }
     }
 
     @Override
-    public boolean startActivityIfNeeded(Intent intent) {
+    public boolean startActivityIfNeeded(Intent intent, boolean proxy) {
+        boolean activityWasLaunched;
+        // Only touches disk on Kitkat. See http://crbug.com/617725 for more context.
+        StrictMode.ThreadPolicy oldPolicy = StrictMode.allowThreadDiskWrites();
+        StrictMode.allowThreadDiskReads();
         try {
-            forcePdfViewerAsIntentHandlerIfNeeded(mActivity, intent);
-            return mActivity.startActivityIfNeeded(intent, -1);
+            forcePdfViewerAsIntentHandlerIfNeeded(mApplicationContext, intent);
+            if (proxy) {
+                dispatchAuthenticatedIntent(intent);
+                activityWasLaunched = true;
+            } else {
+                Context context = getAvailableContext();
+                if (context instanceof Activity) {
+                    activityWasLaunched = ((Activity) context).startActivityIfNeeded(intent, -1);
+                } else {
+                    activityWasLaunched = false;
+                }
+            }
+            if (activityWasLaunched) recordExternalNavigationDispatched(intent);
+            return activityWasLaunched;
         } catch (RuntimeException e) {
-            logTransactionTooLargeOrRethrow(e, intent);
+            IntentUtils.logTransactionTooLargeOrRethrow(e, intent);
             return false;
+        } finally {
+            StrictMode.setThreadPolicy(oldPolicy);
+        }
+    }
+
+    private void recordExternalNavigationDispatched(Intent intent) {
+        ArrayList<String> specializedHandlers = intent.getStringArrayListExtra(
+                IntentHandler.EXTRA_EXTERNAL_NAV_PACKAGES);
+        if (specializedHandlers != null && specializedHandlers.size() > 0) {
+            RecordUserAction.record("MobileExternalNavigationDispatched");
         }
     }
 
     @Override
     public void startIncognitoIntent(final Intent intent, final String referrerUrl,
-            final String fallbackUrl, final Tab tab, final boolean needsToCloseTab) {
-        new AlertDialog.Builder(mActivity, R.style.AlertDialogTheme)
+            final String fallbackUrl, final Tab tab, final boolean needsToCloseTab,
+            final boolean proxy) {
+        Context context = tab.getWindowAndroid().getContext().get();
+        if (!(context instanceof Activity)) return;
+
+        Activity activity = (Activity) context;
+        new AlertDialog.Builder(activity, R.style.AlertDialogTheme)
             .setTitle(R.string.external_app_leave_incognito_warning_title)
             .setMessage(R.string.external_app_leave_incognito_warning)
             .setPositiveButton(R.string.ok, new OnClickListener() {
                     @Override
                     public void onClick(DialogInterface dialog, int which) {
-                        startActivity(intent);
+                        startActivity(intent, proxy);
                         if (tab != null && !tab.isClosing() && tab.isInitialized()
                                 && needsToCloseTab) {
                             closeTab(tab);
@@ -304,8 +430,7 @@ public class ExternalNavigationDelegateImpl implements ExternalNavigationDelegat
 
         // If the url points inside of Chromium's data directory, no permissions are necessary.
         // This is required to prevent permission prompt when uses wants to access offline pages.
-        if (url.startsWith("file://" + PathUtils.getDataDirectory(
-                mActivity.getApplicationContext()))) {
+        if (url.startsWith("file://" + PathUtils.getDataDirectory())) {
             return false;
         }
 
@@ -359,8 +484,8 @@ public class ExternalNavigationDelegateImpl implements ExternalNavigationDelegat
             intent.addCategory(Intent.CATEGORY_BROWSABLE);
             intent.setClassName(getPackageName(), ChromeLauncherActivity.class.getName());
             intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
-            IntentHandler.addTrustedIntentExtras(intent, mActivity);
-            startActivity(intent);
+            IntentHandler.addTrustedIntentExtras(intent, mApplicationContext);
+            startActivity(intent, false);
 
             if (needsToCloseTab) closeTab(tab);
             return;
@@ -393,7 +518,7 @@ public class ExternalNavigationDelegateImpl implements ExternalNavigationDelegat
             intent.putExtra(Browser.EXTRA_APPLICATION_ID, getPackageName());
             intent.addCategory(Intent.CATEGORY_BROWSABLE);
             intent.setPackage(getPackageName());
-            startActivity(intent);
+            startActivity(intent, false);
             return OverrideUrlLoadingResult.OVERRIDE_WITH_EXTERNAL_INTENT;
         }
     }
@@ -405,26 +530,92 @@ public class ExternalNavigationDelegateImpl implements ExternalNavigationDelegat
     }
 
     @Override
-    public boolean isDocumentMode() {
-        return FeatureUtilities.isDocumentMode(mActivity);
+    public void maybeSetWindowId(Intent intent) {
+        Context context = getAvailableContext();
+        if (!(context instanceof ChromeTabbedActivity2)) return;
+        intent.putExtra(IntentHandler.EXTRA_WINDOW_ID, 2);
     }
 
     @Override
     public String getDefaultSmsPackageName() {
         if (Build.VERSION.SDK_INT < Build.VERSION_CODES.KITKAT) return null;
-        return Telephony.Sms.getDefaultSmsPackage(mActivity);
-    }
-
-    private static void logTransactionTooLargeOrRethrow(RuntimeException e, Intent intent) {
-        // See http://crbug.com/369574.
-        if (e.getCause() instanceof TransactionTooLargeException) {
-            Log.e(TAG, "Could not resolve Activity for intent " + intent.toString(), e);
-        } else {
-            throw e;
-        }
+        return Telephony.Sms.getDefaultSmsPackage(mApplicationContext);
     }
 
     private void closeTab(Tab tab) {
-        mActivity.getTabModelSelector().closeTab(tab);
+        Context context = tab.getWindowAndroid().getContext().get();
+        if (context instanceof ChromeActivity) {
+            ((ChromeActivity) context).getTabModelSelector().closeTab(tab);
+        }
+    }
+
+    @Override
+    public boolean isPdfDownload(String url) {
+        String fileExtension = MimeTypeMap.getFileExtensionFromUrl(url);
+        if (TextUtils.isEmpty(fileExtension)) return false;
+
+        return PDF_EXTENSION.equals(fileExtension);
+    }
+
+    @Override
+    public void maybeRecordAppHandlersInIntent(Intent intent, List<ResolveInfo> infos) {
+        intent.putExtra(IntentHandler.EXTRA_EXTERNAL_NAV_PACKAGES,
+                getSpecializedHandlersWithFilter(infos, null));
+    }
+
+    @Override
+    public boolean isSerpReferrer(String referrerUrl, Tab tab) {
+        if (tab == null || tab.getWebContents() == null) {
+            return false;
+        }
+
+        NavigationController nController = tab.getWebContents().getNavigationController();
+        int index = nController.getLastCommittedEntryIndex();
+        if (index == -1) return false;
+
+        NavigationEntry entry = nController.getEntryAtIndex(index);
+        if (entry == null) return false;
+
+        return UrlUtilities.nativeIsGoogleSearchUrl(entry.getUrl());
+    }
+
+    @Override
+    public boolean maybeLaunchInstantApp(Tab tab, String url, String referrerUrl,
+            boolean isIncomingRedirect) {
+        if (tab == null || tab.getWebContents() == null) return false;
+
+        InstantAppsHandler handler = InstantAppsHandler.getInstance();
+        Intent intent = tab.getTabRedirectHandler() != null
+                ? tab.getTabRedirectHandler().getInitialIntent() : null;
+        // TODO(mariakhomenko): consider also handling NDEF_DISCOVER action redirects.
+        if (isIncomingRedirect && intent != null && intent.getAction() == Intent.ACTION_VIEW) {
+            // Set the URL the redirect was resolved to for checking the existence of the
+            // instant app inside handleIncomingIntent().
+            Intent resolvedIntent = new Intent(intent);
+            resolvedIntent.setData(Uri.parse(url));
+            return handler.handleIncomingIntent(getAvailableContext(), resolvedIntent,
+                    ChromeLauncherActivity.isCustomTabIntent(resolvedIntent));
+        } else if (!isIncomingRedirect) {
+            // Check if the navigation is coming from SERP and skip instant app handling.
+            if (isSerpReferrer(referrerUrl, tab)) return false;
+            return handler.handleNavigation(
+                    getAvailableContext(), url,
+                    TextUtils.isEmpty(referrerUrl) ? null : Uri.parse(referrerUrl),
+                    tab.getWebContents());
+        }
+        return false;
+    }
+
+    /**
+     * Dispatches the intent through a proxy activity, so that startActivityForResult can be used
+     * and the intent recipient can verify the caller.
+     * @param intent The bare intent we were going to send.
+     */
+    protected void dispatchAuthenticatedIntent(Intent intent) {
+        Intent proxyIntent = new Intent(Intent.ACTION_MAIN);
+        proxyIntent.setClass(getAvailableContext(), AuthenticatedProxyActivity.class);
+        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
+        proxyIntent.putExtra(AuthenticatedProxyActivity.AUTHENTICATED_INTENT_EXTRA, intent);
+        getAvailableContext().startActivity(proxyIntent);
     }
 }
