@@ -7,6 +7,7 @@ package org.chromium.chrome.browser.externalnav;
 import android.content.ActivityNotFoundException;
 import android.content.ComponentName;
 import android.content.Intent;
+import android.content.pm.ResolveInfo;
 import android.net.Uri;
 import android.os.SystemClock;
 import android.provider.Browser;
@@ -17,15 +18,19 @@ import org.chromium.base.CommandLine;
 import org.chromium.base.Log;
 import org.chromium.base.VisibleForTesting;
 import org.chromium.base.metrics.RecordHistogram;
-import org.chromium.chrome.browser.ChromeActivity;
 import org.chromium.chrome.browser.ChromeSwitches;
 import org.chromium.chrome.browser.IntentHandler;
 import org.chromium.chrome.browser.UrlConstants;
+import org.chromium.chrome.browser.instantapps.InstantAppsHandler;
+import org.chromium.chrome.browser.tab.Tab;
+import org.chromium.chrome.browser.tab.TabRedirectHandler;
 import org.chromium.chrome.browser.util.IntentUtils;
 import org.chromium.chrome.browser.util.UrlUtilities;
+import org.chromium.chrome.browser.webapps.ChromeWebApkHost;
 import org.chromium.ui.base.PageTransition;
 
 import java.net.URI;
+import java.util.HashSet;
 import java.util.List;
 import java.util.concurrent.TimeUnit;
 
@@ -38,17 +43,29 @@ public class ExternalNavigationHandler {
     private static final String TAG = "UrlHandler";
     private static final String SCHEME_WTAI = "wtai://wp/";
     private static final String SCHEME_WTAI_MC = "wtai://wp/mc;";
-    private static final String SCHEME_SMS = "sms:";
+    private static final String SCHEME_SMS = "sms";
 
     @VisibleForTesting
-    public static final String EXTRA_BROWSER_FALLBACK_URL = "browser_fallback_url";
+    static final String EXTRA_BROWSER_FALLBACK_URL = "browser_fallback_url";
+
     @VisibleForTesting
-    static boolean sReportingDisabledForTests = false;
+    static final String SUPERVISOR_PKG = "com.google.android.instantapps.supervisor";
+    @VisibleForTesting
+    static final String[] SUPERVISOR_START_ACTIONS = {
+            "com.google.android.instantapps.START",
+            "com.google.android.instantapps.nmr1.INSTALL",
+            "com.google.android.instantapps.nmr1.VIEW" };
+
+    // An extra that may be specified on an intent:// URL that contains an encoded value for the
+    // referrer field passed to the market:// URL in the case where the app is not present.
+    @VisibleForTesting
+    static final String EXTRA_MARKET_REFERRER = "market_referrer";
 
     private final ExternalNavigationDelegate mDelegate;
 
     /**
      * Result types for checking if we should override URL loading.
+     * NOTE: this enum is used in UMA, do not reorder values. Changes should be append only.
      */
     public enum OverrideUrlLoadingResult {
         /* We should override the URL loading and launch an intent. */
@@ -65,10 +82,10 @@ public class ExternalNavigationHandler {
     /**
      * A constructor for UrlHandler.
      *
-     * @param activity The activity to launch an external intent from.
+     * @param tab The tab that initiated the external intent.
      */
-    public ExternalNavigationHandler(ChromeActivity activity) {
-        this(new ExternalNavigationDelegateImpl(activity));
+    public ExternalNavigationHandler(Tab tab) {
+        this(new ExternalNavigationDelegateImpl(tab));
     }
 
     /**
@@ -108,7 +125,8 @@ public class ExternalNavigationHandler {
         long time = SystemClock.elapsedRealtime();
         OverrideUrlLoadingResult result = shouldOverrideUrlLoadingInternal(
                 params, intent, hasBrowserFallbackUrl, browserFallbackUrl);
-        maybeLogExecutionTime(time);
+        RecordHistogram.recordTimesHistogram("Android.StrictMode.OverrideUrlLoadingTime",
+                SystemClock.elapsedRealtime() - time, TimeUnit.MILLISECONDS);
 
         if (result == OverrideUrlLoadingResult.NO_OVERRIDE && hasBrowserFallbackUrl
                 && (params.getRedirectHandler() == null
@@ -119,11 +137,20 @@ public class ExternalNavigationHandler {
         return result;
     }
 
-    private void maybeLogExecutionTime(long time) {
-        if (!sReportingDisabledForTests) {
-            RecordHistogram.recordTimesHistogram("Android.StrictMode.OverrideUrlLoadingTime",
-                    SystemClock.elapsedRealtime() - time, TimeUnit.MILLISECONDS);
+    private boolean resolversSubsetOf(List<ResolveInfo> infos, List<ResolveInfo> container) {
+        if (container == null) return false;
+        HashSet<ComponentName> containerSet = new HashSet<>();
+        for (ResolveInfo info : container) {
+            containerSet.add(
+                    new ComponentName(info.activityInfo.packageName, info.activityInfo.name));
         }
+        for (ResolveInfo info : infos) {
+            if (!containerSet.contains(new ComponentName(
+                    info.activityInfo.packageName, info.activityInfo.name))) {
+                return false;
+            }
+        }
+        return true;
     }
 
     private OverrideUrlLoadingResult shouldOverrideUrlLoadingInternal(
@@ -169,6 +196,11 @@ public class ExternalNavigationHandler {
             return OverrideUrlLoadingResult.NO_OVERRIDE;
         }
 
+        // http://crbug.com/605302 : Allow Chrome to handle all pdf file downloads.
+        if (!isExternalProtocol && mDelegate.isPdfDownload(params.getUrl())) {
+            return OverrideUrlLoadingResult.NO_OVERRIDE;
+        }
+
         // If accessing a file URL, ensure that the user has granted the necessary file access
         // to Chrome.  This check should happen for reloads, navigations, etc..., which is why
         // it occurs before the subsequent blocks.
@@ -180,15 +212,6 @@ public class ExternalNavigationHandler {
             return OverrideUrlLoadingResult.OVERRIDE_WITH_ASYNC_ACTION;
         }
 
-        // http://crbug/331571 : Do not override a navigation started from user typing.
-        // http://crbug/424029 : Need to stay in Chrome for an intent heading explicitly to Chrome.
-        if (params.getRedirectHandler() != null) {
-            if (params.getRedirectHandler().shouldStayInChrome(isExternalProtocol)
-                    || params.getRedirectHandler().shouldNotOverrideUrlLoading()) {
-                return OverrideUrlLoadingResult.NO_OVERRIDE;
-            }
-        }
-
         // http://crbug.com/149218: We want to show the intent picker for ordinary links, providing
         // the link is not an incoming intent from another application, unless it's a redirect (see
         // below).
@@ -202,6 +225,31 @@ public class ExternalNavigationHandler {
         boolean incomingIntentRedirect = (isLink && isFromIntent && params.isRedirect())
                 || isOnEffectiveIntentRedirect;
 
+
+        // http://crbug/331571 : Do not override a navigation started from user typing.
+        // http://crbug/424029 : Need to stay in Chrome for an intent heading explicitly to Chrome.
+        if (params.getRedirectHandler() != null) {
+            TabRedirectHandler handler = params.getRedirectHandler();
+            if (handler.shouldStayInChrome(isExternalProtocol)
+                    || handler.shouldNotOverrideUrlLoading()) {
+                // http://crbug.com/659301: Handle redirects to Instant Apps out of Custom Tabs.
+                if (handler.isFromCustomTabIntent()
+                        && !isExternalProtocol
+                        && incomingIntentRedirect
+                        && !handler.shouldNavigationTypeStayInChrome()
+                        && mDelegate.maybeLaunchInstantApp(params.getTab(), params.getUrl(),
+                                params.getReferrerUrl(), true)) {
+                    return OverrideUrlLoadingResult.OVERRIDE_WITH_EXTERNAL_INTENT;
+                }
+                return OverrideUrlLoadingResult.NO_OVERRIDE;
+            }
+        }
+
+        // http://crbug.com/647569 : Stay in a PWA window for a URL within the same scope.
+        if (mDelegate.isWithinCurrentWebappScope(params.getUrl())) {
+            return OverrideUrlLoadingResult.NO_OVERRIDE;
+        }
+
         // http://crbug.com/181186: We need to show the intent picker when we receive a redirect
         // following a form submit.
         boolean isRedirectFromFormSubmit = isFormSubmit && params.isRedirect();
@@ -230,9 +278,11 @@ public class ExternalNavigationHandler {
             // number=string(phone-number)
             mDelegate.startActivity(new Intent(Intent.ACTION_VIEW,
                     Uri.parse(WebView.SCHEME_TEL
-                            + params.getUrl().substring(SCHEME_WTAI_MC.length()))));
+                            + params.getUrl().substring(SCHEME_WTAI_MC.length()))), false);
             return OverrideUrlLoadingResult.OVERRIDE_WITH_EXTERNAL_INTENT;
-        } else if (params.getUrl().startsWith(SCHEME_WTAI)) {
+        }
+
+        if (params.getUrl().startsWith(SCHEME_WTAI)) {
             // TODO: handle other WTAI schemes.
             return OverrideUrlLoadingResult.NO_OVERRIDE;
         }
@@ -265,40 +315,7 @@ public class ExternalNavigationHandler {
             return OverrideUrlLoadingResult.NO_OVERRIDE;
         }
 
-        List<ComponentName> resolvingComponentNames = mDelegate.queryIntentActivities(intent);
-        boolean canResolveActivity = resolvingComponentNames.size() > 0;
-        // check whether the intent can be resolved. If not, we will see
-        // whether we can download it from the Market.
-        if (!canResolveActivity) {
-            if (hasBrowserFallbackUrl) {
-                return clobberCurrentTabWithFallbackUrl(browserFallbackUrl, params);
-            }
-            String packagename = intent.getPackage();
-            if (packagename != null) {
-                try {
-                    intent = new Intent(Intent.ACTION_VIEW, Uri.parse(
-                            "market://details?id=" + packagename
-                            + "&referrer=" + mDelegate.getPackageName()));
-                    intent.addCategory(Intent.CATEGORY_BROWSABLE);
-                    intent.setPackage("com.android.vending");
-                    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
-                    mDelegate.startActivity(intent);
-                    return OverrideUrlLoadingResult.OVERRIDE_WITH_EXTERNAL_INTENT;
-                } catch (ActivityNotFoundException ex) {
-                    // ignore the error on devices that does not have
-                    // play market installed.
-                    return OverrideUrlLoadingResult.NO_OVERRIDE;
-                }
-            } else {
-                return OverrideUrlLoadingResult.NO_OVERRIDE;
-            }
-        }
-
-        if (hasBrowserFallbackUrl) {
-            intent.removeExtra(EXTRA_BROWSER_FALLBACK_URL);
-        }
-
-        // sanitize the Intent, ensuring web pages can not bypass browser
+        // Sanitize the Intent, ensuring web pages can not bypass browser
         // security (only access to BROWSABLE activities).
         intent.addCategory(Intent.CATEGORY_BROWSABLE);
         intent.setComponent(null);
@@ -308,8 +325,59 @@ public class ExternalNavigationHandler {
             selector.setComponent(null);
         }
 
-        if (intent.getPackage() == null && params.getUrl().startsWith(SCHEME_SMS)) {
-            intent.setPackage(getDefaultSmsPackageName(resolvingComponentNames));
+        List<ResolveInfo> resolvingInfos = mDelegate.queryIntentActivities(intent);
+        if (resolvingInfos == null) return OverrideUrlLoadingResult.NO_OVERRIDE;
+
+        boolean canResolveActivity = resolvingInfos.size() > 0;
+        // check whether the intent can be resolved. If not, we will see
+        // whether we can download it from the Market.
+        if (!canResolveActivity) {
+            if (hasBrowserFallbackUrl) {
+                return clobberCurrentTabWithFallbackUrl(browserFallbackUrl, params);
+            }
+
+            String packagename = intent.getPackage();
+            if (packagename != null) {
+                String marketReferrer =
+                        IntentUtils.safeGetStringExtra(intent, EXTRA_MARKET_REFERRER);
+                if (TextUtils.isEmpty(marketReferrer)) {
+                    marketReferrer = mDelegate.getPackageName();
+                }
+                try {
+                    Uri marketUri = new Uri.Builder()
+                            .scheme("market")
+                            .authority("details")
+                            .appendQueryParameter("id", packagename)
+                            .appendQueryParameter("referrer", Uri.decode(marketReferrer))
+                            .build();
+                    intent = new Intent(Intent.ACTION_VIEW, marketUri);
+                    intent.addCategory(Intent.CATEGORY_BROWSABLE);
+                    intent.setPackage("com.android.vending");
+                    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
+                    if (params.getReferrerUrl() != null) {
+                        intent.putExtra(Intent.EXTRA_REFERRER, Uri.parse(params.getReferrerUrl()));
+                    }
+                    mDelegate.startActivity(intent, false);
+                    return OverrideUrlLoadingResult.OVERRIDE_WITH_EXTERNAL_INTENT;
+                } catch (ActivityNotFoundException ex) {
+                    // ignore the error on devices that does not have
+                    // play market installed.
+                    return OverrideUrlLoadingResult.NO_OVERRIDE;
+                }
+            }
+
+            return OverrideUrlLoadingResult.NO_OVERRIDE;
+        }
+
+        if (hasBrowserFallbackUrl) {
+            intent.removeExtra(EXTRA_BROWSER_FALLBACK_URL);
+        }
+
+        if (intent.getPackage() == null) {
+            final Uri uri = intent.getData();
+            if (uri != null && SCHEME_SMS.equals(uri.getScheme())) {
+                intent.setPackage(getDefaultSmsPackageName(resolvingInfos));
+            }
         }
 
         // Set the Browser application ID to us in case the user chooses Chrome
@@ -318,18 +386,47 @@ public class ExternalNavigationHandler {
         intent.putExtra(Browser.EXTRA_APPLICATION_ID, mDelegate.getPackageName());
         if (params.isOpenInNewTab()) intent.putExtra(Browser.EXTRA_CREATE_NEW_TAB, true);
         intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
+        mDelegate.maybeSetWindowId(intent);
+        mDelegate.maybeRecordAppHandlersInIntent(intent, resolvingInfos);
 
         if (params.getReferrerUrl() != null) {
             IntentHandler.setPendingReferrer(intent, params.getReferrerUrl());
         }
 
+        if (params.isIncognito()) {
+            // In incognito mode, links that can be handled within the browser should just do so,
+            // without asking the user.
+            if (!isExternalProtocol) {
+                return OverrideUrlLoadingResult.NO_OVERRIDE;
+            }
+
+            IntentHandler.setPendingIncognitoUrl(intent);
+        }
+
         // Make sure webkit can handle it internally before checking for specialized
         // handlers. If webkit can't handle it internally, we need to call
         // startActivityIfNeeded or startActivity.
         if (!isExternalProtocol) {
-            if (!mDelegate.isSpecializedHandlerAvailable(intent)) {
+            if (!mDelegate.isSpecializedHandlerAvailable(resolvingInfos)) {
+                if (params.webApkPackageName() != null) {
+                    intent.setPackage(mDelegate.getPackageName());
+                    mDelegate.startActivity(intent, false);
+                    return OverrideUrlLoadingResult.OVERRIDE_WITH_EXTERNAL_INTENT;
+                }
+
+                if (incomingIntentRedirect && mDelegate.maybeLaunchInstantApp(
+                        params.getTab(), params.getUrl(), params.getReferrerUrl(), true)) {
+                    return OverrideUrlLoadingResult.OVERRIDE_WITH_EXTERNAL_INTENT;
+                } else if (linkNotFromIntent && !params.isIncognito()
+                        && mDelegate.maybeLaunchInstantApp(params.getTab(), params.getUrl(),
+                                params.getReferrerUrl(), false)) {
+                    return OverrideUrlLoadingResult.OVERRIDE_WITH_EXTERNAL_INTENT;
+                }
+
                 return OverrideUrlLoadingResult.NO_OVERRIDE;
-            } else if (params.getReferrerUrl() != null && (isLink || isFormSubmit)) {
+            }
+
+            if (params.getReferrerUrl() != null && (isLink || isFormSubmit)) {
                 // Current URL has at least one specialized handler available. For navigations
                 // within the same host, keep the navigation inside the browser unless the set of
                 // available apps to handle the new navigation is different. http://crbug.com/463138
@@ -353,45 +450,86 @@ public class ExternalNavigationHandler {
                         previousIntent = null;
                     }
 
-                    if (previousIntent != null)  {
-                        List<ComponentName> currentHandlers = mDelegate.queryIntentActivities(
-                                intent);
-                        List<ComponentName> previousHandlers = mDelegate.queryIntentActivities(
-                                previousIntent);
-
-                        if (previousHandlers.containsAll(currentHandlers)) {
-                            return OverrideUrlLoadingResult.NO_OVERRIDE;
-                        }
+                    if (previousIntent != null
+                            && resolversSubsetOf(resolvingInfos,
+                                    mDelegate.queryIntentActivities(previousIntent))) {
+                        return OverrideUrlLoadingResult.NO_OVERRIDE;
                     }
                 }
             }
         }
 
+        boolean isDirectInstantAppsIntent = isExternalProtocol && isIntentToInstantApp(intent);
+        boolean shouldProxyForInstantApps = isDirectInstantAppsIntent
+                && mDelegate.isSerpReferrer(params.getReferrerUrl(), params.getTab());
+        if (shouldProxyForInstantApps) {
+            intent.putExtra(InstantAppsHandler.IS_GOOGLE_SEARCH_REFERRER, true);
+        } else if (isDirectInstantAppsIntent) {
+            // For security reasons, we disable all intent:// URLs to Instant Apps that are
+            // not coming from SERP.
+            return OverrideUrlLoadingResult.NO_OVERRIDE;
+        } else {
+            // Make sure this extra is not sent unless we've done the verification.
+            intent.removeExtra(InstantAppsHandler.IS_GOOGLE_SEARCH_REFERRER);
+        }
+
         try {
             if (params.isIncognito() && !mDelegate.willChromeHandleIntent(intent)) {
                 // This intent may leave Chrome.  Warn the user that incognito does not carry over
                 // to apps out side of Chrome.
                 mDelegate.startIncognitoIntent(intent, params.getReferrerUrl(),
                         hasBrowserFallbackUrl ? browserFallbackUrl : null, params.getTab(),
-                        params.shouldCloseContentsOnOverrideUrlLoadingAndLaunchIntent());
+                        params.shouldCloseContentsOnOverrideUrlLoadingAndLaunchIntent(),
+                        shouldProxyForInstantApps);
                 return OverrideUrlLoadingResult.OVERRIDE_WITH_ASYNC_ACTION;
-            } else {
-                if (params.getRedirectHandler() != null && incomingIntentRedirect) {
-                    if (!params.getRedirectHandler().hasNewResolver(intent)) {
-                        return OverrideUrlLoadingResult.NO_OVERRIDE;
-                    }
-                }
-                // The intent can be used to launch Chrome itself, record the user
-                // gesture here so that it can be used later.
-                if (params.hasUserGesture()) {
-                    IntentWithGesturesHandler.getInstance().onNewIntentWithGesture(intent);
-                }
-                if (mDelegate.startActivityIfNeeded(intent)) {
-                    return OverrideUrlLoadingResult.OVERRIDE_WITH_EXTERNAL_INTENT;
-                } else {
+            }
+
+            // Some third-party app launched Chrome with an intent, and the URL got redirected. The
+            // user has explicitly chosen Chrome over other intent handlers, so stay in Chrome
+            // unless there was a new intent handler after redirection or Chrome cannot handle it
+            // any more.
+            // Custom tabs are an exception to this rule, since at no point, the user sees an intent
+            // picker and "picking Chrome" is handled inside the support library.
+            if (params.getRedirectHandler() != null && incomingIntentRedirect) {
+                if (!isExternalProtocol && !params.getRedirectHandler().isFromCustomTabIntent()
+                        && !params.getRedirectHandler().hasNewResolver(intent)) {
                     return OverrideUrlLoadingResult.NO_OVERRIDE;
                 }
             }
+
+            // The intent can be used to launch Chrome itself, record the user
+            // gesture here so that it can be used later.
+            if (params.hasUserGesture()) {
+                IntentWithGesturesHandler.getInstance().onNewIntentWithGesture(intent);
+            }
+
+            if (ChromeWebApkHost.isEnabled()) {
+                // If the only specialized intent handler is a WebAPK, set the intent's package to
+                // launch the WebAPK without showing the intent picker.
+                String targetWebApkPackageName = mDelegate.findWebApkPackageName(resolvingInfos);
+
+                // We can't rely on this falling through to startActivityIfNeeded and behaving
+                // correctly for WebAPKs. This is because the target of the intent is the WebApk's
+                // main activity but that's just a bouncer which will redirect to WebApkActivity in
+                // chrome. To avoid bouncing indefinitely, don't override the navigation if we are
+                // currently showing the WebApk |params.webApkPackageName()| that we will redirect
+                // to.
+                if (targetWebApkPackageName != null
+                        && targetWebApkPackageName.equals(params.webApkPackageName())) {
+                    return OverrideUrlLoadingResult.NO_OVERRIDE;
+                }
+
+                if (targetWebApkPackageName != null
+                        && mDelegate.countSpecializedHandlers(resolvingInfos) == 1) {
+                    intent.setPackage(targetWebApkPackageName);
+                }
+            }
+
+            if (mDelegate.startActivityIfNeeded(intent, shouldProxyForInstantApps)) {
+                return OverrideUrlLoadingResult.OVERRIDE_WITH_EXTERNAL_INTENT;
+            }
+
+            return OverrideUrlLoadingResult.NO_OVERRIDE;
         } catch (ActivityNotFoundException ex) {
             // Ignore the error. If no application can handle the URL,
             // assume the browser can handle it.
@@ -400,6 +538,23 @@ public class ExternalNavigationHandler {
         return OverrideUrlLoadingResult.NO_OVERRIDE;
     }
 
+    /**
+     * Checks whether {@param intent} is for an Instant App. Considers both package and actions that
+     * would resolve to Supervisor.
+     * @return Whether the given intent is going to open an Instant App.
+     */
+    private boolean isIntentToInstantApp(Intent intent) {
+        if (SUPERVISOR_PKG.equals(intent.getPackage())) return true;
+
+        String intentAction = intent.getAction();
+        for (String action: SUPERVISOR_START_ACTIONS) {
+            if (action.equals(intentAction)) {
+                return true;
+            }
+        }
+        return false;
+    }
+
     /**
      * Clobber the current tab with fallback URL.
      *
@@ -433,8 +588,10 @@ public class ExternalNavigationHandler {
         if (url.startsWith(SCHEME_WTAI_MC)) return true;
         try {
             Intent intent = Intent.parseUri(url, Intent.URI_INTENT_SCHEME);
-            return intent.getPackage() != null
-                    || mDelegate.queryIntentActivities(intent).size() > 0;
+            if (intent.getPackage() != null) return true;
+
+            List<ResolveInfo> resolvingInfos = mDelegate.queryIntentActivities(intent);
+            if (resolvingInfos != null && resolvingInfos.size() > 0) return true;
         } catch (Exception ex) {
             // Ignore the error.
             Log.w(TAG, "Bad URI %s", url, ex);
@@ -448,12 +605,12 @@ public class ExternalNavigationHandler {
      *
      * @param resolvingComponentNames The list of ComponentName that resolves the current intent.
      */
-    private String getDefaultSmsPackageName(List<ComponentName> resolvingComponentNames) {
+    private String getDefaultSmsPackageName(List<ResolveInfo> resolvingComponentNames) {
         String defaultSmsPackageName = mDelegate.getDefaultSmsPackageName();
         if (defaultSmsPackageName == null) return null;
         // Makes sure that the default SMS app actually resolves the intent.
-        for (ComponentName componentName : resolvingComponentNames) {
-            if (defaultSmsPackageName.equals(componentName.getPackageName())) {
+        for (ResolveInfo resolveInfo : resolvingComponentNames) {
+            if (defaultSmsPackageName.equals(resolveInfo.activityInfo.packageName)) {
                 return defaultSmsPackageName;
             }
         }
