@@ -16,11 +16,11 @@ import android.support.v4.app.FragmentManager;
 import android.support.v7.app.MediaRouteChooserDialogFragment;
 import android.support.v7.app.MediaRouteControllerDialogFragment;
 import android.support.v7.app.MediaRouteDialogFactory;
-import android.util.Log;
 
 import com.google.android.gms.cast.CastMediaControlIntent;
 
 import org.chromium.base.ApplicationStatus;
+import org.chromium.base.Log;
 import org.chromium.base.ThreadUtils;
 import org.chromium.base.VisibleForTesting;
 import org.chromium.chrome.R;
@@ -39,12 +39,11 @@ public class RemoteMediaPlayerController implements MediaRouteController.UiListe
     // Singleton instance of the class. May only be accessed from UI thread.
     private static RemoteMediaPlayerController sInstance;
 
-    private static final String TAG = "VideoFling";
+    private static final String TAG = "MediaFling";
 
     private static final String DEFAULT_CASTING_MESSAGE = "Casting to Chromecast";
 
-    private TransportControl mNotificationControl;
-    private TransportControl mLockScreenControl;
+    private CastNotificationControl mNotificationControl;
 
     private Context mCastContextApplicationContext;
     // The Activity that was in the foreground when the video was cast.
@@ -55,8 +54,6 @@ public class RemoteMediaPlayerController implements MediaRouteController.UiListe
     // points to mDefaultRouteSelector, mYouTubeRouteSelector or null
     private MediaRouteController mCurrentRouteController;
 
-    private boolean mFirstConnection = true;
-
     // This is a key for meta-data in the package manifest.
     private static final String REMOTE_MEDIA_PLAYERS_KEY =
             "org.chromium.content.browser.REMOTE_MEDIA_PLAYERS";
@@ -143,7 +140,7 @@ public class RemoteMediaPlayerController implements MediaRouteController.UiListe
             if (classNameString != null) {
                 String[] classNames = classNameString.split(",");
                 for (String className : classNames) {
-                    Log.i(TAG, "Adding remote media route controller " + className.trim());
+                    Log.d(TAG, "Adding remote media route controller %s", className.trim());
                     Class<?> mediaRouteControllerClass = Class.forName(className.trim());
                     Object mediaRouteController = mediaRouteControllerClass.newInstance();
                     assert mediaRouteController instanceof MediaRouteController;
@@ -162,17 +159,9 @@ public class RemoteMediaPlayerController implements MediaRouteController.UiListe
 
         if (!controller.initialize()) return;
 
-        if (mFirstConnection) {
-            controller.reconnectAnyExistingRoute();
-            mFirstConnection = false;
-        }
-
-        if (mNotificationControl != null) {
-            mNotificationControl.setRouteController(controller);
-        }
-        if (mLockScreenControl != null) {
-            mLockScreenControl.setRouteController(controller);
-        }
+        mNotificationControl = CastNotificationControl.getOrCreate(
+                mChromeVideoActivity.get(), controller);
+        mNotificationControl.setPosterBitmap(getPoster());
         controller.prepareMediaRoute();
 
         controller.addUiListener(this);
@@ -194,9 +183,7 @@ public class RemoteMediaPlayerController implements MediaRouteController.UiListe
         }
 
         onStateReset(controller);
-        if (controller.shouldResetState(player)) {
-            showMediaRouteDialog(player, controller, currentActivity);
-        }
+        showMediaRouteDialog(player, controller, currentActivity);
 
     }
 
@@ -213,7 +200,18 @@ public class RemoteMediaPlayerController implements MediaRouteController.UiListe
         if (mCurrentRouteController == null) return;
         if (mCurrentRouteController.getMediaStateListener() != player) return;
 
-        showMediaRouteControlDialog(ApplicationStatus.getLastTrackedFocusedActivity());
+        showMediaRouteControlDialog(player, ApplicationStatus.getLastTrackedFocusedActivity());
+    }
+
+    /**
+     * Called when a lower layer requests to stop casting the video.
+     * @param player The player to stop remote playback for.
+     */
+    public void requestRemotePlaybackStop(MediaRouteController.MediaStateListener player) {
+        if (mCurrentRouteController == null) return;
+        if (mCurrentRouteController.getMediaStateListener() != player) return;
+
+        mCurrentRouteController.release();
     }
 
     private void showMediaRouteDialog(MediaStateListener player, MediaRouteController controller,
@@ -238,13 +236,12 @@ public class RemoteMediaPlayerController implements MediaRouteController.UiListe
         f.show(fm, "android.support.v7.mediarouter:MediaRouteChooserDialogFragment");
     }
 
-    private void showMediaRouteControlDialog(Activity activity) {
-
+    private void showMediaRouteControlDialog(MediaStateListener player, Activity activity) {
         FragmentManager fm = ((FragmentActivity) activity).getSupportFragmentManager();
         if (fm == null) {
             throw new IllegalStateException("The activity must be a subclass of FragmentActivity");
         }
-        MediaRouteDialogFactory factory = new MediaRouteControllerDialogFactory();
+        MediaRouteDialogFactory factory = new MediaRouteControllerDialogFactory(player);
 
         if (fm.findFragmentByTag(
                 "android.support.v7.mediarouter:MediaRouteControllerDialogFragment") != null) {
@@ -256,22 +253,6 @@ public class RemoteMediaPlayerController implements MediaRouteController.UiListe
         f.show(fm, "android.support.v7.mediarouter:MediaRouteControllerDialogFragment");
     }
 
-     /**
-     * Starts up the notification and lock screen with the given playback state.
-     *
-     * @param initialState the initial state of the notification
-     * @param mediaRouteController the mediaRouteController for which these are needed
-     */
-    public void startNotificationAndLockScreen(PlayerState initialState,
-            MediaRouteController mediaRouteController) {
-        mCurrentRouteController = mediaRouteController;
-        createNotificationControl();
-        getNotification().show(initialState);
-        createLockScreen();
-        TransportControl lockScreen = getLockScreen();
-        if (lockScreen != null) lockScreen.show(initialState);
-    }
-
     /**
      * @return the currently playing MediaRouteController
      */
@@ -287,49 +268,16 @@ public class RemoteMediaPlayerController implements MediaRouteController.UiListe
         mCurrentRouteController = controller;
     }
 
-    private TransportControl getNotification() {
+    private CastNotificationControl getNotificationControl() {
         return mNotificationControl;
     }
 
-    /**
-     *
-     */
-    private void createNotificationControl() {
-        mNotificationControl = NotificationTransportControl.getOrCreate(
-                mChromeVideoActivity.get(), mCurrentRouteController);
-        mNotificationControl.setError(null);
-        mNotificationControl.setScreenName(mCurrentRouteController.getRouteName());
-        mNotificationControl.addListener(mCurrentRouteController);
-    }
-
-    private TransportControl getLockScreen() {
-        return mLockScreenControl;
-    }
-
-    private void createLockScreen() {
-        mLockScreenControl = LockScreenTransportControl.getOrCreate(
-                mChromeVideoActivity.get(), mCurrentRouteController);
-        mLockScreenControl.setError(null);
-        mLockScreenControl.setScreenName(mCurrentRouteController.getRouteName());
-        mLockScreenControl.addListener(mCurrentRouteController);
-        mLockScreenControl.setPosterBitmap(getPoster());
-    }
-
     @Override
     public void onPrepared(MediaRouteController mediaRouteController) {
-
-        startNotificationAndLockScreen(PlayerState.PLAYING, mediaRouteController);
     }
 
     @Override
-    public void onPlaybackStateChanged(PlayerState oldState, PlayerState newState) {
-        if (newState == PlayerState.PLAYING || newState == PlayerState.LOADING
-                || newState == PlayerState.PAUSED) {
-            TransportControl notificationControl = getNotification();
-            if (notificationControl != null) notificationControl.show(newState);
-            TransportControl lockScreen = getLockScreen();
-            if (lockScreen != null) lockScreen.show(newState);
-        }
+    public void onPlaybackStateChanged(PlayerState newState) {
     }
 
     @Override
@@ -389,9 +337,6 @@ public class RemoteMediaPlayerController implements MediaRouteController.UiListe
         if (mNotificationControl != null) {
             mNotificationControl.setRouteController(mCurrentRouteController);
         }
-        if (mLockScreenControl != null) {
-            mLockScreenControl.setRouteController(mCurrentRouteController);
-        }
     }
 
     @VisibleForTesting
