@@ -10,6 +10,9 @@ import android.animation.AnimatorSet;
 import android.animation.ObjectAnimator;
 import android.app.Activity;
 import android.app.Dialog;
+import android.content.ActivityNotFoundException;
+import android.content.ClipData;
+import android.content.ClipboardManager;
 import android.content.Context;
 import android.content.DialogInterface;
 import android.content.Intent;
@@ -18,13 +21,14 @@ import android.graphics.Color;
 import android.graphics.drawable.ColorDrawable;
 import android.net.Uri;
 import android.os.Bundle;
-import android.os.Process;
 import android.provider.Settings;
+import android.support.annotation.IntDef;
 import android.support.v7.widget.AppCompatTextView;
 import android.text.Layout;
 import android.text.Spannable;
 import android.text.SpannableString;
 import android.text.SpannableStringBuilder;
+import android.text.TextUtils;
 import android.text.style.ForegroundColorSpan;
 import android.text.style.StyleSpan;
 import android.util.AttributeSet;
@@ -32,6 +36,7 @@ import android.view.Gravity;
 import android.view.LayoutInflater;
 import android.view.View;
 import android.view.View.OnClickListener;
+import android.view.View.OnLongClickListener;
 import android.view.ViewGroup;
 import android.view.Window;
 import android.widget.Button;
@@ -42,8 +47,13 @@ import android.widget.TextView;
 
 import org.chromium.base.ApiCompatibilityUtils;
 import org.chromium.base.annotations.CalledByNative;
+import org.chromium.base.metrics.RecordHistogram;
+import org.chromium.base.metrics.RecordUserAction;
 import org.chromium.chrome.R;
 import org.chromium.chrome.browser.ContentSettingsType;
+import org.chromium.chrome.browser.instantapps.InstantAppsHandler;
+import org.chromium.chrome.browser.offlinepages.OfflinePageItem;
+import org.chromium.chrome.browser.offlinepages.OfflinePageUtils;
 import org.chromium.chrome.browser.omnibox.OmniboxUrlEmphasizer;
 import org.chromium.chrome.browser.preferences.PrefServiceBridge;
 import org.chromium.chrome.browser.preferences.Preferences;
@@ -53,8 +63,9 @@ import org.chromium.chrome.browser.preferences.website.ContentSettingsResources;
 import org.chromium.chrome.browser.preferences.website.SingleWebsitePreferences;
 import org.chromium.chrome.browser.profiles.Profile;
 import org.chromium.chrome.browser.ssl.SecurityStateModel;
+import org.chromium.chrome.browser.tab.Tab;
 import org.chromium.chrome.browser.util.UrlUtilities;
-import org.chromium.components.security_state.ConnectionSecurityLevel;
+import org.chromium.components.location.LocationUtils;
 import org.chromium.content.browser.ContentViewCore;
 import org.chromium.content_public.browser.WebContents;
 import org.chromium.content_public.browser.WebContentsObserver;
@@ -62,10 +73,15 @@ import org.chromium.ui.base.DeviceFormFactor;
 import org.chromium.ui.base.WindowAndroid;
 import org.chromium.ui.base.WindowAndroid.PermissionCallback;
 import org.chromium.ui.interpolators.BakedBezierInterpolator;
+import org.chromium.ui.widget.Toast;
 
+import java.lang.annotation.Retention;
+import java.lang.annotation.RetentionPolicy;
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.text.DateFormat;
 import java.util.ArrayList;
+import java.util.Date;
 import java.util.List;
 
 /**
@@ -74,6 +90,13 @@ import java.util.List;
  *               WebsiteSettings* and website_settings_*. Do this on the C++ side as well.
  */
 public class WebsiteSettingsPopup implements OnClickListener {
+    @Retention(RetentionPolicy.SOURCE)
+    @IntDef({OPENED_FROM_MENU, OPENED_FROM_TOOLBAR})
+    private @interface OpenedFromSource {}
+
+    public static final int OPENED_FROM_MENU = 1;
+    public static final int OPENED_FROM_TOOLBAR = 2;
+
     /**
      * An entry in the settings dropdown for a given permission. There are two options for each
      * permission: Allow and Block.
@@ -222,18 +245,22 @@ public class WebsiteSettingsPopup implements OnClickListener {
     private final Profile mProfile;
     private final WebContents mWebContents;
     private final WindowAndroid mWindowAndroid;
+    private final Tab mTab;
 
     // A pointer to the C++ object for this UI.
-    private final long mNativeWebsiteSettingsPopup;
+    private long mNativeWebsiteSettingsPopup;
 
     // The outer container, filled with the layout from website_settings.xml.
     private final LinearLayout mContainer;
 
     // UI elements in the dialog.
     private final ElidedUrlTextView mUrlTitle;
-    private final TextView mUrlConnectionMessage;
+    private final TextView mConnectionSummary;
+    private final TextView mConnectionMessage;
     private final LinearLayout mPermissionsList;
+    private final Button mInstantAppButton;
     private final Button mSiteSettingsButton;
+    private final Button mOpenOnlineButton;
 
     // The dialog the container is placed in.
     private final Dialog mDialog;
@@ -258,28 +285,37 @@ public class WebsiteSettingsPopup implements OnClickListener {
     // The security level of the page (a valid ConnectionSecurityLevel).
     private int mSecurityLevel;
 
-    // Whether the security level of the page was downgraded due to SHA-1.
-    private boolean mDeprecatedSHA1Present;
-
-    // Whether the security level of the page was downgraded due to passive mixed content.
-    private boolean mPassiveMixedContentPresent;
-
     // Permissions available to be displayed in mPermissionsList.
     private List<PageInfoPermissionEntry> mDisplayedPermissions;
 
+    // Creation date of an offline copy, if web contents contains an offline page.
+    private String mOfflinePageCreationDate;
+
+    // The name of the content publisher, if any.
+    private String mContentPublisher;
+
+    // The intent associated with the instant app for this URL (or null if one does not exist).
+    private Intent mInstantAppIntent;
+
     /**
      * Creates the WebsiteSettingsPopup, but does not display it. Also initializes the corresponding
      * C++ object and saves a pointer to it.
-     *
-     * @param context Context which is used for launching a dialog.
-     * @param webContents The WebContents for which to show Website information. This information is
-     *                    retrieved for the visible entry.
+     * @param activity                 Activity which is used for showing a popup.
+     * @param tab                      Tab for which the pop up is shown.
+     * @param offlinePageCreationDate  Date when the offline page was created.
+     * @param publisher                The name of the content publisher, if any.
      */
-    private WebsiteSettingsPopup(Activity activity, Profile profile, WebContents webContents) {
+    private WebsiteSettingsPopup(Activity activity, Tab tab, String offlinePageCreationDate,
+            String publisher) {
         mContext = activity;
-        mProfile = profile;
-        mWebContents = webContents;
+        mProfile = tab.getProfile();
+        mWebContents = tab.getWebContents();
+        mTab = tab;
+        if (offlinePageCreationDate != null) {
+            mOfflinePageCreationDate = offlinePageCreationDate;
+        }
         mWindowAndroid = ContentViewCore.fromWebContents(mWebContents).getWindowAndroid();
+        mContentPublisher = publisher;
 
         // Find the container and all it's important subviews.
         mContainer = (LinearLayout) LayoutInflater.from(mContext).inflate(
@@ -300,21 +336,82 @@ public class WebsiteSettingsPopup implements OnClickListener {
         mUrlTitle = (ElidedUrlTextView) mContainer.findViewById(R.id.website_settings_url);
         mUrlTitle.setProfile(mProfile);
         mUrlTitle.setOnClickListener(this);
+        // Long press the url text to copy it to the clipboard.
+        mUrlTitle.setOnLongClickListener(new OnLongClickListener() {
+            @Override
+            public boolean onLongClick(View v) {
+                ClipboardManager clipboard = (ClipboardManager) mContext
+                        .getSystemService(Context.CLIPBOARD_SERVICE);
+                ClipData clip = ClipData.newPlainText("url", mFullUrl);
+                clipboard.setPrimaryClip(clip);
+                Toast.makeText(mContext, R.string.url_copied, Toast.LENGTH_SHORT).show();
+                return true;
+            }
+        });
 
-        mUrlConnectionMessage = (TextView) mContainer
+        mConnectionSummary = (TextView) mContainer
+                .findViewById(R.id.website_settings_connection_summary);
+        mConnectionMessage = (TextView) mContainer
                 .findViewById(R.id.website_settings_connection_message);
         mPermissionsList = (LinearLayout) mContainer
                 .findViewById(R.id.website_settings_permissions_list);
 
+        mInstantAppButton =
+                (Button) mContainer.findViewById(R.id.website_settings_instant_app_button);
+        mInstantAppButton.setOnClickListener(this);
+
         mSiteSettingsButton =
                 (Button) mContainer.findViewById(R.id.website_settings_site_settings_button);
         mSiteSettingsButton.setOnClickListener(this);
 
+        mOpenOnlineButton =
+                (Button) mContainer.findViewById(R.id.website_settings_open_online_button);
+        mOpenOnlineButton.setOnClickListener(this);
+
         mDisplayedPermissions = new ArrayList<PageInfoPermissionEntry>();
 
         // Hide the permissions list for sites with no permissions.
         setVisibilityOfPermissionsList(false);
 
+        // Work out the URL and connection message and status visibility.
+        mFullUrl = mWebContents.getVisibleUrl();
+        if (isShowingOfflinePage()) {
+            mFullUrl = OfflinePageUtils.stripSchemeFromOnlineUrl(mFullUrl);
+        }
+
+        try {
+            mParsedUrl = new URI(mFullUrl);
+            mIsInternalPage = UrlUtilities.isInternalScheme(mParsedUrl);
+        } catch (URISyntaxException e) {
+            mParsedUrl = null;
+            mIsInternalPage = false;
+        }
+        mSecurityLevel = SecurityStateModel.getSecurityLevelForWebContents(mWebContents);
+
+        SpannableStringBuilder urlBuilder = new SpannableStringBuilder(mFullUrl);
+        OmniboxUrlEmphasizer.emphasizeUrl(urlBuilder, mContext.getResources(), mProfile,
+                mSecurityLevel, mIsInternalPage, true, true);
+        mUrlTitle.setText(urlBuilder);
+
+        if (mParsedUrl == null || mParsedUrl.getScheme() == null
+                || !(mParsedUrl.getScheme().equals("http")
+                           || mParsedUrl.getScheme().equals("https"))) {
+            mSiteSettingsButton.setVisibility(View.GONE);
+        }
+
+        if (isShowingOfflinePage()) {
+            boolean isConnected = OfflinePageUtils.isConnected();
+            RecordHistogram.recordBooleanHistogram(
+                    "OfflinePages.WebsiteSettings.OpenOnlineButtonVisible", isConnected);
+            if (!isConnected) mOpenOnlineButton.setVisibility(View.GONE);
+        } else {
+            mOpenOnlineButton.setVisibility(View.GONE);
+        }
+
+        mInstantAppIntent = (mIsInternalPage || isShowingOfflinePage()) ? null
+                : InstantAppsHandler.getInstance().getInstantAppIntentForUrl(mFullUrl);
+        if (mInstantAppIntent == null) mInstantAppButton.setVisibility(View.GONE);
+
         // Create the dialog.
         mDialog = new Dialog(mContext) {
             private void superDismiss() {
@@ -357,7 +454,7 @@ public class WebsiteSettingsPopup implements OnClickListener {
         }
 
         // This needs to come after other member initialization.
-        mNativeWebsiteSettingsPopup = nativeInit(this, webContents);
+        mNativeWebsiteSettingsPopup = nativeInit(this, mWebContents);
         final WebContentsObserver webContentsObserver = new WebContentsObserver(mWebContents) {
             @Override
             public void navigationEntryCommitted() {
@@ -366,6 +463,13 @@ public class WebsiteSettingsPopup implements OnClickListener {
                 mDialog.dismiss();
             }
 
+            @Override
+            public void wasHidden() {
+                // The web contents were hidden (potentially by loading another URL via an intent),
+                // so dismiss the dialog).
+                mDialog.dismiss();
+            }
+
             @Override
             public void destroy() {
                 super.destroy();
@@ -381,37 +485,11 @@ public class WebsiteSettingsPopup implements OnClickListener {
                 assert mNativeWebsiteSettingsPopup != 0;
                 webContentsObserver.destroy();
                 nativeDestroy(mNativeWebsiteSettingsPopup);
+                mNativeWebsiteSettingsPopup = 0;
             }
         });
 
-        // Work out the URL and connection message.
-        mFullUrl = mWebContents.getVisibleUrl();
-        try {
-            mParsedUrl = new URI(mFullUrl);
-            mIsInternalPage = UrlUtilities.isInternalScheme(mParsedUrl);
-        } catch (URISyntaxException e) {
-            mParsedUrl = null;
-            mIsInternalPage = false;
-        }
-        mSecurityLevel = SecurityStateModel.getSecurityLevelForWebContents(mWebContents);
-        mDeprecatedSHA1Present = SecurityStateModel.isDeprecatedSHA1Present(mWebContents);
-        mPassiveMixedContentPresent = SecurityStateModel.isPassiveMixedContentPresent(mWebContents);
-
-        SpannableStringBuilder urlBuilder = new SpannableStringBuilder(mFullUrl);
-        OmniboxUrlEmphasizer.emphasizeUrl(urlBuilder, mContext.getResources(), mProfile,
-                mSecurityLevel, mIsInternalPage, true, true);
-        mUrlTitle.setText(urlBuilder);
-
-        // Set the URL connection message now, and the URL after layout (so it
-        // can calculate its ideal height).
-        mUrlConnectionMessage.setText(getUrlConnectionMessage());
-        if (isConnectionDetailsLinkVisible()) mUrlConnectionMessage.setOnClickListener(this);
-
-        if (mParsedUrl == null || mParsedUrl.getScheme() == null
-                || !(mParsedUrl.getScheme().equals("http")
-                           || mParsedUrl.getScheme().equals("https"))) {
-            mSiteSettingsButton.setVisibility(View.GONE);
-        }
+        showDialog();
     }
 
     /**
@@ -438,103 +516,19 @@ public class WebsiteSettingsPopup implements OnClickListener {
         return icon;
     }
 
-    /**
-     * Gets the message to display in the connection message box for the given security level. Does
-     * not apply to SECURITY_ERROR pages, since these have their own coloured/formatted message.
-     *
-     * @param securityLevel A valid ConnectionSecurityLevel, which is the security
-     *                      level of the page.
-     * @param isInternalPage Whether or not this page is an internal chrome page (e.g. the
-     *                       chrome://settings page).
-     * @return The ID of the message to display in the connection message box.
-     */
-    private int getConnectionMessageId(int securityLevel, boolean isInternalPage) {
-        if (isInternalPage) return R.string.page_info_connection_internal_page;
-
-        switch (securityLevel) {
-            case ConnectionSecurityLevel.NONE:
-                return R.string.page_info_connection_http;
-            case ConnectionSecurityLevel.SECURE:
-            case ConnectionSecurityLevel.EV_SECURE:
-                return R.string.page_info_connection_https;
-            default:
-                assert false : "Invalid security level specified: " + securityLevel;
-                return R.string.page_info_connection_http;
-        }
-    }
-
     /**
      * Whether to show a 'Details' link to the connection info popup. The link is only shown for
      * HTTPS connections.
      */
     private boolean isConnectionDetailsLinkVisible() {
-        // TODO(tsergeant): If this logic gets any more complicated from additional deprecations,
-        // change it to use something like |SchemeIsCryptographic|.
-        return !mIsInternalPage && (mSecurityLevel != ConnectionSecurityLevel.NONE
-                || mPassiveMixedContentPresent || mDeprecatedSHA1Present);
-    }
-
-    /**
-     * Gets the styled connection message to display below the URL.
-     */
-    private Spannable getUrlConnectionMessage() {
-        // Display the appropriate connection message.
-        SpannableStringBuilder messageBuilder = new SpannableStringBuilder();
-        if (mDeprecatedSHA1Present) {
-            messageBuilder.append(
-                    mContext.getResources().getString(R.string.page_info_connection_sha1));
-        } else if (mPassiveMixedContentPresent) {
-            messageBuilder.append(
-                    mContext.getResources().getString(R.string.page_info_connection_mixed));
-        } else if (mSecurityLevel != ConnectionSecurityLevel.SECURITY_ERROR
-                && mSecurityLevel != ConnectionSecurityLevel.SECURITY_WARNING
-                && mSecurityLevel != ConnectionSecurityLevel.SECURITY_POLICY_WARNING) {
-            messageBuilder.append(mContext.getResources().getString(
-                    getConnectionMessageId(mSecurityLevel, mIsInternalPage)));
-        } else {
-            String originToDisplay;
-            try {
-                URI parsedUrl = new URI(mFullUrl);
-                originToDisplay = UrlUtilities.formatUrlForSecurityDisplay(parsedUrl, false);
-            } catch (URISyntaxException e) {
-                // The URL is invalid - just display the full URL.
-                originToDisplay = mFullUrl;
-            }
-
-            messageBuilder.append(mContext.getResources().getString(
-                    R.string.page_info_connection_broken, originToDisplay));
-        }
-
-        if (isConnectionDetailsLinkVisible()) {
-            messageBuilder.append(" ");
-            SpannableString detailsText = new SpannableString(
-                    mContext.getResources().getString(R.string.page_info_details_link));
-            final ForegroundColorSpan blueSpan = new ForegroundColorSpan(
-                    ApiCompatibilityUtils.getColor(mContext.getResources(),
-                            R.color.website_settings_popup_text_link));
-            detailsText.setSpan(
-                    blueSpan, 0, detailsText.length(), Spannable.SPAN_INCLUSIVE_EXCLUSIVE);
-            messageBuilder.append(detailsText);
-        }
-
-        return messageBuilder;
+        return mContentPublisher == null && !isShowingOfflinePage() && mParsedUrl != null
+                && mParsedUrl.getScheme() != null && mParsedUrl.getScheme().equals("https");
     }
 
     private boolean hasAndroidPermission(int contentSettingType) {
         String androidPermission = PrefServiceBridge.getAndroidPermissionForContentSetting(
                 contentSettingType);
-        return androidPermission == null
-                || (mContext.checkPermission(androidPermission, Process.myPid(), Process.myUid())
-                           == PackageManager.PERMISSION_GRANTED);
-    }
-
-    private boolean isAndroidLocationDisabled() {
-        try {
-            return Settings.Secure.getInt(mContext.getContentResolver(),
-                    Settings.Secure.LOCATION_MODE) == Settings.Secure.LOCATION_MODE_OFF;
-        } catch (Settings.SettingNotFoundException e) {
-            return false;
-        }
+        return androidPermission == null || mWindowAndroid.hasPermission(androidPermission);
     }
 
     /**
@@ -576,11 +570,12 @@ public class WebsiteSettingsPopup implements OnClickListener {
 
             // If warningTextResource is non-zero, then the view must be tagged with either
             // permission_intent_override or permission_type.
+            LocationUtils locationUtils = LocationUtils.getInstance();
             if (permission.type == ContentSettingsType.CONTENT_SETTINGS_TYPE_GEOLOCATION
-                    && isAndroidLocationDisabled()) {
+                    && !locationUtils.isSystemLocationSettingEnabled()) {
                 warningTextResource = R.string.page_info_android_location_blocked;
                 permissionRow.setTag(R.id.permission_intent_override,
-                        new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS));
+                        locationUtils.getSystemLocationSettingsIntent());
             } else if (!hasAndroidPermission(permission.type)) {
                 warningTextResource = R.string.page_info_android_permission_blocked;
                 permissionRow.setTag(R.id.permission_type,
@@ -613,12 +608,10 @@ public class WebsiteSettingsPopup implements OnClickListener {
         String status_text = "";
         switch (permission.setting) {
             case ALLOW:
-                status_text =
-                        mContext.getResources().getString(R.string.page_info_permission_allowed);
+                status_text = mContext.getString(R.string.page_info_permission_allowed);
                 break;
             case BLOCK:
-                status_text =
-                        mContext.getResources().getString(R.string.page_info_permission_blocked);
+                status_text = mContext.getString(R.string.page_info_permission_blocked);
                 break;
             default:
                 assert false : "Invalid setting " + permission.setting + " for permission "
@@ -630,9 +623,46 @@ public class WebsiteSettingsPopup implements OnClickListener {
     }
 
     /**
-     * Displays the WebsiteSettingsPopup.
+     * Sets the connection security summary and detailed description strings. These strings may be
+     * overridden based on the state of the Android UI.
      */
     @CalledByNative
+    private void setSecurityDescription(String summary, String details) {
+        // Display the appropriate connection message.
+        SpannableStringBuilder messageBuilder = new SpannableStringBuilder();
+        if (mContentPublisher != null) {
+            messageBuilder.append(
+                    mContext.getString(R.string.page_info_domain_hidden, mContentPublisher));
+        } else if (isShowingOfflinePage()) {
+            messageBuilder.append(String.format(
+                    mContext.getString(R.string.page_info_connection_offline),
+                    mOfflinePageCreationDate));
+        } else {
+            if (!TextUtils.equals(summary, details)) {
+                mConnectionSummary.setVisibility(View.VISIBLE);
+                mConnectionSummary.setText(summary);
+            }
+            messageBuilder.append(details);
+        }
+
+        if (isConnectionDetailsLinkVisible()) {
+            messageBuilder.append(" ");
+            SpannableString detailsText = new SpannableString(
+                    mContext.getString(R.string.page_info_details_link));
+            final ForegroundColorSpan blueSpan = new ForegroundColorSpan(
+                    ApiCompatibilityUtils.getColor(mContext.getResources(),
+                            R.color.website_settings_popup_text_link));
+            detailsText.setSpan(
+                    blueSpan, 0, detailsText.length(), Spannable.SPAN_INCLUSIVE_EXCLUSIVE);
+            messageBuilder.append(detailsText);
+        }
+        mConnectionMessage.setText(messageBuilder);
+        if (isConnectionDetailsLinkVisible()) mConnectionMessage.setOnClickListener(this);
+    }
+
+    /**
+     * Displays the WebsiteSettingsPopup.
+     */
     private void showDialog() {
         if (!DeviceFormFactor.isTablet(mContext)) {
             // On smaller screens, make the dialog fill the width of the screen.
@@ -689,6 +719,7 @@ public class WebsiteSettingsPopup implements OnClickListener {
             runAfterDismiss(new Runnable() {
                 @Override
                 public void run() {
+                    recordAction(WebsiteSettingsAction.WEBSITE_SETTINGS_SITE_SETTINGS_OPENED);
                     Bundle fragmentArguments =
                             SingleWebsitePreferences.createFragmentArgsForSite(mFullUrl);
                     fragmentArguments.putParcelable(SingleWebsitePreferences.EXTRA_WEB_CONTENTS,
@@ -700,14 +731,23 @@ public class WebsiteSettingsPopup implements OnClickListener {
                     mContext.startActivity(preferencesIntent);
                 }
             });
+        } else if (view == mInstantAppButton) {
+            try {
+                mContext.startActivity(mInstantAppIntent);
+                RecordUserAction.record("Android.InstantApps.LaunchedFromWebsiteSettingsPopup");
+            } catch (ActivityNotFoundException e) {
+                mInstantAppButton.setEnabled(false);
+            }
         } else if (view == mUrlTitle) {
             // Expand/collapse the displayed URL title.
             mUrlTitle.toggleTruncation();
-        } else if (view == mUrlConnectionMessage) {
+        } else if (view == mConnectionMessage) {
             runAfterDismiss(new Runnable() {
                 @Override
                 public void run() {
                     if (!mWebContents.isDestroyed()) {
+                        recordAction(
+                                WebsiteSettingsAction.WEBSITE_SETTINGS_SECURITY_DETAILS_OPENED);
                         ConnectionInfoPopup.show(mContext, mWebContents);
                     }
                 }
@@ -748,6 +788,19 @@ public class WebsiteSettingsPopup implements OnClickListener {
                     mContext.startActivity(settingsIntent);
                 }
             });
+        } else if (view == mOpenOnlineButton) {
+            runAfterDismiss(new Runnable() {
+                @Override
+                public void run() {
+                    // Attempt to reload to an online version of the viewed offline web page. This
+                    // attempt might fail if the user is offline, in which case an offline copy will
+                    // be reloaded.
+                    RecordHistogram.recordBooleanHistogram(
+                            "OfflinePages.WebsiteSettings.ConnectedWhenOpenOnlineButtonClicked",
+                            OfflinePageUtils.isConnected());
+                    OfflinePageUtils.reload(mTab);
+                }
+            });
         }
     }
 
@@ -757,7 +810,11 @@ public class WebsiteSettingsPopup implements OnClickListener {
     private List<View> collectAnimatableViews() {
         List<View> animatableViews = new ArrayList<View>();
         animatableViews.add(mUrlTitle);
-        animatableViews.add(mUrlConnectionMessage);
+        if (mConnectionSummary.getVisibility() == View.VISIBLE) {
+            animatableViews.add(mConnectionSummary);
+        }
+        animatableViews.add(mConnectionMessage);
+        animatableViews.add(mInstantAppButton);
         for (int i = 0; i < mPermissionsList.getChildCount(); i++) {
             animatableViews.add(mPermissionsList.getChildAt(i));
         }
@@ -842,23 +899,56 @@ public class WebsiteSettingsPopup implements OnClickListener {
         return animation;
     }
 
+    private void recordAction(int action) {
+        if (mNativeWebsiteSettingsPopup != 0) {
+            nativeRecordWebsiteSettingsAction(mNativeWebsiteSettingsPopup, action);
+        }
+    }
+
     /**
-     * Shows a WebsiteSettings dialog for the provided WebContents. The popup adds itself to the
-     * view hierarchy which owns the reference while it's visible.
+     * Whether website dialog is displayed for an offline page.
+     */
+    private boolean isShowingOfflinePage() {
+        return mOfflinePageCreationDate != null;
+    }
+
+    /**
+     * Shows a WebsiteSettings dialog for the provided Tab. The popup adds itself to the view
+     * hierarchy which owns the reference while it's visible.
      *
      * @param activity Activity which is used for launching a dialog.
-     * @param webContents The WebContents for which to show Website information. This information is
-     *                    retrieved for the visible entry.
+     * @param tab The tab hosting the web contents for which to show Website information. This
+     *            information is retrieved for the visible entry.
+     * @param contentPublisher The name of the publisher of the content.
+     * @param source Determines the source that triggered the popup.
      */
-    @SuppressWarnings("unused")
-    public static void show(Activity activity, Profile profile, WebContents webContents) {
-        new WebsiteSettingsPopup(activity, profile, webContents);
+    public static void show(final Activity activity, final Tab tab, final String contentPublisher,
+            @OpenedFromSource int source) {
+        if (source == OPENED_FROM_MENU) {
+            RecordUserAction.record("MobileWebsiteSettingsOpenedFromMenu");
+        } else if (source == OPENED_FROM_TOOLBAR) {
+            RecordUserAction.record("MobileWebsiteSettingsOpenedFromToolbar");
+        } else {
+            assert false : "Invalid source passed";
+        }
+
+        String offlinePageCreationDate = null;
+
+        OfflinePageItem offlinePage = tab.getOfflinePage();
+        if (offlinePage != null) {
+            // Get formatted creation date of the offline page.
+            Date creationDate = new Date(offlinePage.getCreationTimeMs());
+            DateFormat df = DateFormat.getDateInstance(DateFormat.MEDIUM);
+            offlinePageCreationDate = df.format(creationDate);
+        }
+
+        new WebsiteSettingsPopup(activity, tab, offlinePageCreationDate, contentPublisher);
     }
 
     private static native long nativeInit(WebsiteSettingsPopup popup, WebContents webContents);
 
     private native void nativeDestroy(long nativeWebsiteSettingsPopupAndroid);
 
-    private native void nativeOnPermissionSettingChanged(long nativeWebsiteSettingsPopupAndroid,
-            int type, int setting);
+    private native void nativeRecordWebsiteSettingsAction(
+            long nativeWebsiteSettingsPopupAndroid, int action);
 }
