@@ -11,6 +11,8 @@ import android.content.BroadcastReceiver;
 import android.content.ClipData;
 import android.content.ComponentName;
 import android.content.Context;
+import android.content.DialogInterface;
+import android.content.DialogInterface.OnDismissListener;
 import android.content.Intent;
 import android.content.IntentFilter;
 import android.content.SharedPreferences;
@@ -24,8 +26,9 @@ import android.graphics.drawable.Drawable;
 import android.net.Uri;
 import android.os.AsyncTask;
 import android.os.Build;
-import android.preference.PreferenceManager;
+import android.support.annotation.Nullable;
 import android.support.v7.app.AlertDialog;
+import android.text.TextUtils;
 import android.util.Pair;
 import android.view.MenuItem;
 import android.view.View;
@@ -35,7 +38,10 @@ import android.widget.AdapterView.OnItemClickListener;
 import org.chromium.base.ApiCompatibilityUtils;
 import org.chromium.base.ApplicationState;
 import org.chromium.base.ApplicationStatus;
+import org.chromium.base.Callback;
+import org.chromium.base.ContextUtils;
 import org.chromium.base.Log;
+import org.chromium.base.StreamUtil;
 import org.chromium.base.VisibleForTesting;
 import org.chromium.base.annotations.SuppressFBWarnings;
 import org.chromium.base.metrics.RecordHistogram;
@@ -57,8 +63,28 @@ import java.util.concurrent.TimeoutException;
  */
 public class ShareHelper {
 
+    /** Interface that receives intents for testing (to fake out actually sending them). */
+    public interface FakeIntentReceiver {
+        /** Sets the intent to send back in the broadcast. */
+        public void setIntentToSendBack(Intent intent);
+
+        /** Called when a custom chooser dialog is shown. */
+        public void onCustomChooserShown(AlertDialog dialog);
+
+        /**
+         * Simulates firing the given intent, without actually doing so.
+         *
+         * @param context The context that will receive broadcasts from the simulated activity.
+         * @param intent The intent to send to the system.
+         */
+        public void fireIntent(Context context, Intent intent);
+    }
+
     private static final String TAG = "share";
 
+    /** The task ID of the activity that triggered the share action. */
+    public static final String EXTRA_TASK_ID = "org.chromium.chrome.extra.TASK_ID";
+
     private static final String JPEG_EXTENSION = ".jpg";
     private static final String PACKAGE_NAME_KEY = "last_shared_package_name";
     private static final String CLASS_NAME_KEY = "last_shared_class_name";
@@ -72,8 +98,23 @@ public class ShareHelper {
      */
     private static final String SHARE_IMAGES_DIRECTORY_NAME = "screenshot";
 
+    /** Force the use of a Chrome-specific intent chooser, not the system chooser. */
+    private static boolean sForceCustomChooserForTesting = false;
+
+    /** If non-null, will be used instead of the real activity. */
+    private static FakeIntentReceiver sFakeIntentReceiverForTesting;
+
     private ShareHelper() {}
 
+    private static void fireIntent(Activity activity, Intent intent) {
+        if (sFakeIntentReceiverForTesting != null) {
+            Context context = activity.getApplicationContext();
+            sFakeIntentReceiverForTesting.fireIntent(context, intent);
+        } else {
+            activity.startActivity(intent);
+        }
+    }
+
     @SuppressFBWarnings("NP_NULL_ON_SOME_PATH_FROM_RETURN_VALUE")
     private static void deleteShareImageFiles(File file) {
         if (!file.exists()) return;
@@ -85,6 +126,45 @@ public class ShareHelper {
         }
     }
 
+    /**
+     * Force the use of a Chrome-specific intent chooser, not the system chooser.
+     *
+     * This emulates the behavior on pre Lollipop-MR1 systems, where the system chooser is not
+     * available.
+     */
+    public static void setForceCustomChooserForTesting(boolean enabled) {
+        sForceCustomChooserForTesting = enabled;
+    }
+
+    /**
+     * Uses a FakeIntentReceiver instead of actually sending intents to the system.
+     *
+     * @param receiver The object to send intents to. If null, resets back to the default behavior
+     *                 (really send intents).
+     */
+    public static void setFakeIntentReceiverForTesting(FakeIntentReceiver receiver) {
+        sFakeIntentReceiverForTesting = receiver;
+    }
+
+    /**
+     * Callback interface for when a target is chosen.
+     */
+    public static interface TargetChosenCallback {
+        /**
+         * Called when the user chooses a target in the share dialog.
+         *
+         * Note that if the user cancels the share dialog, this callback is never called.
+         */
+        public void onTargetChosen(ComponentName chosenComponent);
+
+        /**
+         * Called when the user cancels the share dialog.
+         *
+         * Guaranteed that either this, or onTargetChosen (but not both) will be called, eventually.
+         */
+        public void onCancel();
+    }
+
     /**
      * Receiver to record the chosen component when sharing an Intent.
      */
@@ -95,12 +175,24 @@ public class ShareHelper {
         private static String sTargetChosenReceiveAction;
         private static TargetChosenReceiver sLastRegisteredReceiver;
 
+        private final boolean mSaveLastUsed;
+        @Nullable private final TargetChosenCallback mCallback;
+
+        private TargetChosenReceiver(boolean saveLastUsed,
+                                     @Nullable TargetChosenCallback callback) {
+            mSaveLastUsed = saveLastUsed;
+            mCallback = callback;
+        }
+
         static boolean isSupported() {
-            return Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP_MR1;
+            return !sForceCustomChooserForTesting
+                && Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP_MR1;
         }
 
         @TargetApi(Build.VERSION_CODES.LOLLIPOP_MR1)
-        static void sendChooserIntent(Activity activity, Intent sharingIntent) {
+        static void sendChooserIntent(boolean saveLastUsed, Activity activity,
+                                      Intent sharingIntent,
+                                      @Nullable TargetChosenCallback callback) {
             synchronized (LOCK) {
                 if (sTargetChosenReceiveAction == null) {
                     sTargetChosenReceiveAction = activity.getPackageName() + "/"
@@ -109,8 +201,13 @@ public class ShareHelper {
                 Context context = activity.getApplicationContext();
                 if (sLastRegisteredReceiver != null) {
                     context.unregisterReceiver(sLastRegisteredReceiver);
+                    // Must cancel the callback (to satisfy guarantee that exactly one method of
+                    // TargetChosenCallback is called).
+                    // TODO(mgiuca): This should be called immediately upon cancelling the chooser,
+                    // not just when the next share takes place (https://crbug.com/636274).
+                    sLastRegisteredReceiver.cancel();
                 }
-                sLastRegisteredReceiver = new TargetChosenReceiver();
+                sLastRegisteredReceiver = new TargetChosenReceiver(saveLastUsed, callback);
                 context.registerReceiver(
                         sLastRegisteredReceiver, new IntentFilter(sTargetChosenReceiveAction));
             }
@@ -118,12 +215,15 @@ public class ShareHelper {
             Intent intent = new Intent(sTargetChosenReceiveAction);
             intent.setPackage(activity.getPackageName());
             intent.putExtra(EXTRA_RECEIVER_TOKEN, sLastRegisteredReceiver.hashCode());
-            final PendingIntent callback = PendingIntent.getBroadcast(activity, 0, intent,
+            final PendingIntent pendingIntent = PendingIntent.getBroadcast(activity, 0, intent,
                     PendingIntent.FLAG_CANCEL_CURRENT | PendingIntent.FLAG_ONE_SHOT);
             Intent chooserIntent = Intent.createChooser(sharingIntent,
                     activity.getString(R.string.share_link_chooser_title),
-                    callback.getIntentSender());
-            activity.startActivity(chooserIntent);
+                    pendingIntent.getIntentSender());
+            if (sFakeIntentReceiverForTesting != null) {
+                sFakeIntentReceiverForTesting.setIntentToSendBack(intent);
+            }
+            fireIntent(activity, chooserIntent);
         }
 
         @Override
@@ -139,8 +239,17 @@ public class ShareHelper {
             }
 
             ComponentName target = intent.getParcelableExtra(Intent.EXTRA_CHOSEN_COMPONENT);
-            if (target != null) {
-                setLastShareComponentName(context, target);
+            if (mCallback != null) {
+                mCallback.onTargetChosen(target);
+            }
+            if (mSaveLastUsed && target != null) {
+                setLastShareComponentName(target);
+            }
+        }
+
+        private void cancel() {
+            if (mCallback != null) {
+                mCallback.onCancel();
             }
         }
     }
@@ -148,12 +257,13 @@ public class ShareHelper {
     /**
      * Clears all shared image files.
      */
-    public static void clearSharedImages(final Context context) {
+    public static void clearSharedImages() {
         new AsyncTask<Void, Void, Void>() {
             @Override
             protected Void doInBackground(Void... params) {
                 try {
-                    File imagePath = UiUtils.getDirectoryForImageCapture(context);
+                    File imagePath = UiUtils.getDirectoryForImageCapture(
+                            ContextUtils.getApplicationContext());
                     deleteShareImageFiles(new File(imagePath, SHARE_IMAGES_DIRECTORY_NAME));
                 } catch (IOException ie) {
                     // Ignore exception.
@@ -174,19 +284,29 @@ public class ShareHelper {
      *
      * @param shareDirectly Whether it should share directly with the activity that was most
      *                      recently used to share.
+     * @param saveLastUsed Whether to save the chosen activity for future direct sharing.
      * @param activity Activity that is used to access package manager.
      * @param title Title of the page to be shared.
+     * @param text Text to be shared. If both |text| and |url| are supplied, they are concatenated
+     *             with a space.
      * @param url URL of the page to be shared.
-     * @param screenshot Screenshot of the page to be shared.
+     * @param offlineUri URI to the offline MHTML file to be shared.
+     * @param screenshotUri Uri of the screenshot of the page to be shared.
+     * @param callback Optional callback to be called when user makes a choice. Will not be called
+     *                 if receiving a response when the user makes a choice is not supported (on
+     *                 older Android versions).
      */
-    public static void share(boolean shareDirectly, Activity activity, String title, String url,
-            Bitmap screenshot) {
+    public static void share(boolean shareDirectly, boolean saveLastUsed, Activity activity,
+            String title, String text, String url, @Nullable Uri offlineUri, Uri screenshotUri,
+            @Nullable TargetChosenCallback callback) {
         if (shareDirectly) {
-            shareWithLastUsed(activity, title, url, screenshot);
+            shareWithLastUsed(activity, title, text, url, offlineUri, screenshotUri);
         } else if (TargetChosenReceiver.isSupported()) {
-            makeIntentAndShare(activity, title, url, screenshot, null);
+            makeIntentAndShare(saveLastUsed, activity, title, text, url, offlineUri, screenshotUri,
+                    null, callback);
         } else {
-            showShareDialog(activity, title, url, screenshot);
+            showShareDialog(
+                    saveLastUsed, activity, title, text, url, offlineUri, screenshotUri, callback);
         }
     }
 
@@ -243,23 +363,83 @@ public class ShareHelper {
 
                     Intent chooserIntent = Intent.createChooser(getShareImageIntent(imageUri),
                             activity.getString(R.string.share_link_chooser_title));
-                    activity.startActivity(chooserIntent);
+                    fireIntent(activity, chooserIntent);
                 }
             }
         }.execute();
     }
 
+    /**
+     * Persists the screenshot file and notifies the file provider that the file is ready to be
+     * accessed by the client.
+     *
+     * The bitmap is compressed to JPEG before being written to the file.
+     *
+     * @param screenshot  The screenshot bitmap to be written to file.
+     * @param callback    The callback that will be called once the bitmap is saved.
+     */
+    public static void saveScreenshotToDisk(final Bitmap screenshot, final Context context,
+            final Callback<Uri> callback) {
+        if (screenshot == null) {
+            callback.onResult(null);
+            return;
+        }
+
+        new AsyncTask<Void, Void, File>() {
+            @Override
+            protected File doInBackground(Void... params) {
+                FileOutputStream fOut = null;
+                try {
+                    File path = new File(UiUtils.getDirectoryForImageCapture(context) + "/"
+                            + SHARE_IMAGES_DIRECTORY_NAME);
+                    if (path.exists() || path.mkdir()) {
+                        String fileName = String.valueOf(System.currentTimeMillis());
+                        File saveFile = File.createTempFile(fileName, JPEG_EXTENSION, path);
+                        fOut = new FileOutputStream(saveFile);
+                        screenshot.compress(Bitmap.CompressFormat.JPEG, 85, fOut);
+                        return saveFile;
+                    }
+                } catch (IOException ie) {
+                    Log.w(TAG, "Ignoring IOException when saving screenshot.", ie);
+                } finally {
+                    StreamUtil.closeQuietly(fOut);
+                }
+
+                return null;
+            }
+
+            @Override
+            protected void onPostExecute(File savedFile) {
+                Uri fileUri = null;
+                if (ApplicationStatus.getStateForApplication()
+                        != ApplicationState.HAS_DESTROYED_ACTIVITIES
+                        && savedFile != null) {
+                    fileUri = UiUtils.getUriForImageCaptureFile(context, savedFile);
+                }
+                callback.onResult(fileUri);
+            }
+        }.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
+    }
+
     /**
      * Creates and shows a share intent picker dialog.
      *
+     * @param saveLastUsed Whether to save the chosen activity for future direct sharing.
      * @param activity Activity that is used to access package manager.
      * @param title Title of the page to be shared.
+     * @param text Text to be shared. If both |text| and |url| are supplied, they are concatenated
+     *             with a space.
      * @param url URL of the page to be shared.
-     * @param screenshot Screenshot of the page to be shared.
+     * @oaram offlineUri URI of the offline page to be shared.
+     * @param screenshotUri Uri of the screenshot of the page to be shared.
+     * @param callback Optional callback to be called when user makes a choice. Will not be called
+     *                 if receiving a response when the user makes a choice is not supported (on
+     *                 older Android versions).
      */
-    private static void showShareDialog(final Activity activity, final String title,
-            final String url, final Bitmap screenshot) {
-        Intent intent = getShareIntent(title, url, null);
+    private static void showShareDialog(final boolean saveLastUsed, final Activity activity,
+            final String title, final String text, final String url, final Uri offlineUri,
+            final Uri screenshotUri, @Nullable final TargetChosenCallback callback) {
+        Intent intent = getShareIntent(activity, title, text, url, null, null);
         PackageManager manager = activity.getPackageManager();
         List<ResolveInfo> resolveInfoList = manager.queryIntentActivities(intent, 0);
         assert resolveInfoList.size() > 0;
@@ -272,6 +452,9 @@ public class ShareHelper {
         builder.setTitle(activity.getString(R.string.share_link_chooser_title));
         builder.setAdapter(adapter, null);
 
+        // Need a mutable object to record whether the callback has been fired.
+        final boolean[] callbackCalled = new boolean[1];
+
         final AlertDialog dialog = builder.create();
         dialog.show();
         dialog.getListView().setOnItemClickListener(new OnItemClickListener() {
@@ -281,11 +464,32 @@ public class ShareHelper {
                 ActivityInfo ai = info.activityInfo;
                 ComponentName component =
                         new ComponentName(ai.applicationInfo.packageName, ai.name);
-                setLastShareComponentName(activity, component);
-                makeIntentAndShare(activity, title, url, screenshot, component);
+                if (callback != null && !callbackCalled[0]) {
+                    callback.onTargetChosen(component);
+                    callbackCalled[0] = true;
+                }
+                if (saveLastUsed) setLastShareComponentName(component);
+                makeIntentAndShare(false, activity, title, text, url, offlineUri, screenshotUri,
+                        component, null);
                 dialog.dismiss();
             }
         });
+
+        if (callback != null) {
+            dialog.setOnDismissListener(new OnDismissListener() {
+                @Override
+                public void onDismiss(DialogInterface dialog) {
+                    if (!callbackCalled[0]) {
+                        callback.onCancel();
+                        callbackCalled[0] = true;
+                    }
+                }
+            });
+        }
+
+        if (sFakeIntentReceiverForTesting != null) {
+            sFakeIntentReceiverForTesting.onCustomChooserShown(dialog);
+        }
     }
 
     /**
@@ -293,73 +497,39 @@ public class ShareHelper {
      * If there is no most recently used activity, it does nothing.
      * @param activity Activity that is used to start the share intent.
      * @param title Title of the page to be shared.
+     * @param text Text to be shared. If both |text| and |url| are supplied, they are concatenated
+     *             with a space.
      * @param url URL of the page to be shared.
-     * @param screenshot Screenshot of the page to be shared.
+     * @oaram offlineUri URI of the offline page to be shared.
+     * @param screenshotUri Uri of the screenshot of the page to be shared.
      */
-    private static void shareWithLastUsed(
-            Activity activity, String title, String url, Bitmap screenshot) {
-        ComponentName component = getLastShareComponentName(activity);
+    private static void shareWithLastUsed(Activity activity, String title, String text, String url,
+            Uri offlineUri, Uri screenshotUri) {
+        ComponentName component = getLastShareComponentName();
         if (component == null) return;
-        makeIntentAndShare(activity, title, url, screenshot, component);
+        makeIntentAndShare(
+                false, activity, title, text, url, offlineUri, screenshotUri, component, null);
     }
 
-    private static void shareIntent(Activity activity, Intent sharingIntent) {
+    private static void shareIntent(boolean saveLastUsed, Activity activity, Intent sharingIntent,
+                                    @Nullable TargetChosenCallback callback) {
         if (sharingIntent.getComponent() != null) {
-            activity.startActivity(sharingIntent);
+            // If a component was specified, there should not also be a callback.
+            assert callback == null;
+            fireIntent(activity, sharingIntent);
         } else {
             assert TargetChosenReceiver.isSupported();
-            TargetChosenReceiver.sendChooserIntent(activity, sharingIntent);
+            TargetChosenReceiver.sendChooserIntent(saveLastUsed, activity, sharingIntent, callback);
         }
     }
 
-    private static void makeIntentAndShare(final Activity activity, final String title,
-            final String url, final Bitmap screenshot, final ComponentName component) {
-        if (screenshot == null) {
-            shareIntent(activity, getDirectShareIntentForComponent(title, url, null, component));
-        } else {
-            new AsyncTask<Void, Void, File>() {
-                @Override
-                protected File doInBackground(Void... params) {
-                    FileOutputStream fOut = null;
-                    try {
-                        File path = new File(UiUtils.getDirectoryForImageCapture(activity),
-                                SHARE_IMAGES_DIRECTORY_NAME);
-                        if (path.exists() || path.mkdir()) {
-                            File saveFile = File.createTempFile(
-                                    String.valueOf(System.currentTimeMillis()),
-                                    JPEG_EXTENSION, path);
-                            fOut = new FileOutputStream(saveFile);
-                            screenshot.compress(Bitmap.CompressFormat.JPEG, 85, fOut);
-                            fOut.flush();
-                            fOut.close();
-
-                            return saveFile;
-                        }
-                    } catch (IOException ie) {
-                        if (fOut != null) {
-                            try {
-                                fOut.close();
-                            } catch (IOException e) {
-                                // Ignore exception.
-                            }
-                        }
-                    }
-
-                    return null;
-                }
-
-                @Override
-                protected void onPostExecute(File saveFile) {
-                    if (ApplicationStatus.getStateForApplication()
-                            != ApplicationState.HAS_DESTROYED_ACTIVITIES) {
-                        Uri screenshotUri = saveFile == null
-                                ? null : UiUtils.getUriForImageCaptureFile(activity, saveFile);
-                        shareIntent(activity, getDirectShareIntentForComponent(
-                                title, url, screenshotUri, component));
-                    }
-                }
-            }.execute();
-        }
+    private static void makeIntentAndShare(final boolean saveLastUsed, final Activity activity,
+            final String title, final String text, final String url, final Uri offlineUri,
+            final Uri screenshotUri, final ComponentName component,
+            @Nullable final TargetChosenCallback callback) {
+        Intent intent = getDirectShareIntentForComponent(
+                activity, title, text, url, offlineUri, screenshotUri, component);
+        shareIntent(saveLastUsed, activity, intent, callback);
     }
 
     /**
@@ -372,10 +542,10 @@ public class ShareHelper {
         Drawable directShareIcon = null;
         CharSequence directShareTitle = null;
 
-        final ComponentName component = getLastShareComponentName(activity);
+        final ComponentName component = getLastShareComponentName();
         boolean isComponentValid = false;
         if (component != null) {
-            Intent intent = getShareIntent("", "", null);
+            Intent intent = getShareIntent(activity, "", "", "", null, null);
             intent.setPackage(component.getPackageName());
             PackageManager manager = activity.getPackageManager();
             List<ResolveInfo> resolveInfoList = manager.queryIntentActivities(intent, 0);
@@ -433,22 +603,55 @@ public class ShareHelper {
         }
     }
 
+    /*
+     * Stores the component selected for sharing last time share was called.
+     *
+     * This method is public since it is used in tests to avoid creating share dialog.
+     */
     @VisibleForTesting
-    public static Intent getShareIntent(String title, String url, Uri screenshotUri) {
-        url = DomDistillerUrlUtils.getOriginalUrlFromDistillerUrl(url);
+    public static void setLastShareComponentName(ComponentName component) {
+        SharedPreferences preferences = ContextUtils.getAppSharedPreferences();
+        SharedPreferences.Editor editor = preferences.edit();
+        editor.putString(PACKAGE_NAME_KEY, component.getPackageName());
+        editor.putString(CLASS_NAME_KEY, component.getClassName());
+        editor.apply();
+    }
+
+    @VisibleForTesting
+    public static Intent getShareIntent(Activity activity, String title, String text, String url,
+            Uri offlineUri, Uri screenshotUri) {
+        if (!TextUtils.isEmpty(url)) {
+            url = DomDistillerUrlUtils.getOriginalUrlFromDistillerUrl(url);
+            if (!TextUtils.isEmpty(text)) {
+                // Concatenate text and URL with a space.
+                text = text + " " + url;
+            } else {
+                text = url;
+            }
+        }
+
         Intent intent = new Intent(Intent.ACTION_SEND);
         intent.addFlags(ApiCompatibilityUtils.getActivityNewDocumentFlag());
-        intent.setType("text/plain");
         intent.putExtra(Intent.EXTRA_SUBJECT, title);
-        intent.putExtra(Intent.EXTRA_TEXT, url);
+        intent.putExtra(Intent.EXTRA_TEXT, text);
+        intent.putExtra(EXTRA_TASK_ID, activity.getTaskId());
+
         if (screenshotUri != null) {
             intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
+        }
+        if (screenshotUri != null) {
             // To give read access to an Intent target, we need to put |screenshotUri| in clipData
             // because adding Intent.FLAG_GRANT_READ_URI_PERMISSION doesn't work for
             // EXTRA_SHARE_SCREENSHOT_AS_STREAM.
             intent.setClipData(ClipData.newRawUri("", screenshotUri));
             intent.putExtra(EXTRA_SHARE_SCREENSHOT_AS_STREAM, screenshotUri);
         }
+        if (offlineUri == null) {
+            intent.setType("text/plain");
+        } else {
+            intent.setType("multipart/related");
+            intent.putExtra(Intent.EXTRA_STREAM, offlineUri);
+        }
         return intent;
     }
 
@@ -461,28 +664,20 @@ public class ShareHelper {
         return intent;
     }
 
-    private static Intent getDirectShareIntentForComponent(
-            String title, String url, Uri screenshotUri, ComponentName component) {
-        Intent intent = getShareIntent(title, url, screenshotUri);
+    private static Intent getDirectShareIntentForComponent(Activity activity, String title,
+            String text, String url, Uri offlineUri, Uri screenshotUri, ComponentName component) {
+        Intent intent = getShareIntent(activity, title, text, url, offlineUri, screenshotUri);
         intent.addFlags(Intent.FLAG_ACTIVITY_FORWARD_RESULT
                 | Intent.FLAG_ACTIVITY_PREVIOUS_IS_TOP);
         intent.setComponent(component);
         return intent;
     }
 
-    private static ComponentName getLastShareComponentName(Context context) {
-        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(context);
+    private static ComponentName getLastShareComponentName() {
+        SharedPreferences preferences = ContextUtils.getAppSharedPreferences();
         String packageName = preferences.getString(PACKAGE_NAME_KEY, null);
         String className = preferences.getString(CLASS_NAME_KEY, null);
         if (packageName == null || className == null) return null;
         return new ComponentName(packageName, className);
     }
-
-    private static void setLastShareComponentName(Context context, ComponentName component) {
-        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(context);
-        SharedPreferences.Editor editor = preferences.edit();
-        editor.putString(PACKAGE_NAME_KEY, component.getPackageName());
-        editor.putString(CLASS_NAME_KEY, component.getClassName());
-        editor.apply();
-    }
 }
