@@ -11,17 +11,15 @@ import org.chromium.base.SysUtils;
 import org.chromium.base.TraceEvent;
 import org.chromium.chrome.browser.ChromeActivity;
 import org.chromium.chrome.browser.IntentHandler;
+import org.chromium.chrome.browser.ServiceTabLauncher;
 import org.chromium.chrome.browser.TabState;
-import org.chromium.chrome.browser.WarmupManager;
 import org.chromium.chrome.browser.compositor.layouts.content.TabContentManager;
+import org.chromium.chrome.browser.net.spdyproxy.DataReductionProxySettings;
 import org.chromium.chrome.browser.tab.Tab;
 import org.chromium.chrome.browser.tab.TabDelegateFactory;
 import org.chromium.chrome.browser.tabmodel.TabModel.TabLaunchType;
-import org.chromium.chrome.browser.tabmodel.document.AsyncTabCreationParams;
-import org.chromium.chrome.browser.tabmodel.document.AsyncTabCreationParamsManager;
 import org.chromium.chrome.browser.util.IntentUtils;
-import org.chromium.chrome.browser.util.UrlUtilities;
-import org.chromium.components.service_tab_launcher.ServiceTabLauncher;
+import org.chromium.components.url_formatter.UrlFormatter;
 import org.chromium.content_public.browser.LoadUrlParams;
 import org.chromium.content_public.browser.WebContents;
 import org.chromium.content_public.common.Referrer;
@@ -35,20 +33,16 @@ public class ChromeTabCreator extends TabCreatorManager.TabCreator {
 
     private final ChromeActivity mActivity;
     private final WindowAndroid mNativeWindow;
-    private final TabModelOrderController mOrderController;
-    private final TabPersistentStore mTabSaver;
     private final boolean mIncognito;
 
     private TabModel mTabModel;
     private TabContentManager mTabContentManager;
+    private TabModelOrderController mOrderController;
 
-    public ChromeTabCreator(ChromeActivity activity, WindowAndroid nativeWindow,
-            TabModelOrderController orderController, TabPersistentStore tabSaver,
-            boolean incognito) {
+    public ChromeTabCreator(
+            ChromeActivity activity, WindowAndroid nativeWindow, boolean incognito) {
         mActivity = activity;
         mNativeWindow = nativeWindow;
-        mOrderController = orderController;
-        mTabSaver = tabSaver;
         mIncognito = incognito;
     }
 
@@ -78,7 +72,7 @@ public class ChromeTabCreator extends TabCreatorManager.TabCreator {
      * @param intent the source of the url if it isn't null.
      * @return The new tab.
      */
-    private Tab createNewTab(LoadUrlParams loadUrlParams, TabModel.TabLaunchType type,
+    public Tab createNewTab(LoadUrlParams loadUrlParams, TabModel.TabLaunchType type,
             Tab parent, Intent intent) {
         // If parent is in the same tab model, place the new tab next to it.
         int position = TabModel.INVALID_TAB_INDEX;
@@ -104,22 +98,30 @@ public class ChromeTabCreator extends TabCreatorManager.TabCreator {
             int parentId = parent != null ? parent.getId() : Tab.INVALID_TAB_ID;
 
             // Sanitize the url.
-            loadUrlParams.setUrl(UrlUtilities.fixupUrl(loadUrlParams.getUrl()));
-            loadUrlParams.setTransitionType(getTransitionType(type));
+            loadUrlParams.setUrl(UrlFormatter.fixupUrl(loadUrlParams.getUrl()));
+            loadUrlParams.setTransitionType(getTransitionType(type, intent));
 
             // Check if the tab is being created asynchronously.
             int assignedTabId = intent == null ? Tab.INVALID_TAB_ID : IntentUtils.safeGetIntExtra(
                     intent, IntentHandler.EXTRA_TAB_ID, Tab.INVALID_TAB_ID);
-            AsyncTabCreationParams asyncParams =
-                    AsyncTabCreationParamsManager.remove(assignedTabId);
-            WebContents webContents = asyncParams == null ? null : asyncParams.getWebContents();
+            AsyncTabParams asyncParams =
+                    AsyncTabParamsManager.remove(assignedTabId);
 
-            boolean openInForeground = mOrderController.willOpenInForeground(type, mIncognito)
-                    || webContents != null;
-            TabDelegateFactory delegateFactory = parent == null ? new TabDelegateFactory()
+            boolean openInForeground = mOrderController.willOpenInForeground(type, mIncognito);
+            TabDelegateFactory delegateFactory = parent == null ? createDefaultTabDelegateFactory()
                     : parent.getDelegateFactory();
             Tab tab;
-            if (webContents != null) {
+            if (asyncParams != null && asyncParams.getTabToReparent() != null) {
+                type = TabLaunchType.FROM_REPARENTING;
+                openInForeground = true;
+
+                TabReparentingParams params = (TabReparentingParams) asyncParams;
+                tab = params.getTabToReparent();
+                tab.attachAndFinishReparenting(
+                        mActivity, createDefaultTabDelegateFactory(), params);
+            } else if (asyncParams != null && asyncParams.getWebContents() != null) {
+                openInForeground = true;
+                WebContents webContents = asyncParams.getWebContents();
                 // A WebContents was passed through the Intent.  Create a new Tab to hold it.
                 Intent parentIntent = IntentUtils.safeGetParcelableExtra(
                         intent, IntentHandler.EXTRA_PARENT_INTENT);
@@ -138,22 +140,13 @@ public class ChromeTabCreator extends TabCreatorManager.TabCreator {
                 // On low memory devices the tabs opened in background are not loaded automatically
                 // to preserve resources (cpu, memory, strong renderer binding) for the foreground
                 // tab.
-                // TODO(dfalcantara): Fallback Tabs created when the TabState couldn't be restored
-                //                    on startup should go through this path, as well, but there's
-                //                    currently no way to pipe that information to this function.
                 tab = Tab.createTabForLazyLoad(mActivity, mIncognito, mNativeWindow, type,
                         parentId, loadUrlParams);
                 tab.initialize(null, mTabContentManager, delegateFactory, !openInForeground, false);
-                mTabSaver.addTabToSaveQueue(tab);
             } else {
-                webContents =
-                        WarmupManager.getInstance().hasPrerenderedUrl(loadUrlParams.getUrl())
-                        ? WarmupManager.getInstance().takePrerenderedWebContents() : null;
-
                 tab = Tab.createLiveTab(Tab.INVALID_TAB_ID, mActivity, mIncognito,
                         mNativeWindow, type, parentId, !openInForeground);
-                tab.initialize(
-                        webContents, mTabContentManager, delegateFactory, !openInForeground, false);
+                tab.initialize(null, mTabContentManager, delegateFactory, !openInForeground, false);
                 tab.loadUrl(loadUrlParams);
             }
             tab.getTabRedirectHandler().updateIntent(intent);
@@ -183,7 +176,7 @@ public class ChromeTabCreator extends TabCreatorManager.TabCreator {
         if (index != TabModel.INVALID_TAB_INDEX) position = index + 1;
 
         boolean openInForeground = mOrderController.willOpenInForeground(type, mIncognito);
-        TabDelegateFactory delegateFactory = parent == null ? new TabDelegateFactory()
+        TabDelegateFactory delegateFactory = parent == null ? createDefaultTabDelegateFactory()
                 : parent.getDelegateFactory();
         Tab tab = Tab.createLiveTab(Tab.INVALID_TAB_ID, mActivity, mIncognito,
                 mNativeWindow, type, parentId, !openInForeground);
@@ -233,6 +226,13 @@ public class ChromeTabCreator extends TabCreatorManager.TabCreator {
             String appId, boolean forceNewTab, Intent intent, long intentTimestamp) {
         assert !mIncognito;
         boolean isLaunchedFromChrome = TextUtils.equals(appId, mActivity.getPackageName());
+
+        // If an external app sends an intent for a Weblite URL and the Data Reduction Proxy is
+        // using Weblite mode, then use the URL in the lite_url parameter if its scheme is HTTP.
+        // This is used by ChromeTabbedActvity intents so that the user does not receive Weblite
+        // pages when they could be served a Data Reduction Proxy preview page.
+        if (url != null) url = DataReductionProxySettings.getInstance().maybeRewriteWebliteUrl(url);
+
         if (forceNewTab && !isLaunchedFromChrome) {
             // We don't associate the tab with that app ID, as it is assumed that if the
             // application wanted to open this tab as a new tab, it probably does not want it
@@ -280,7 +280,8 @@ public class ChromeTabCreator extends TabCreatorManager.TabCreator {
                 id, mActivity, state.isIncognito(), mNativeWindow, state.parentId, state);
         boolean selectTab = mOrderController.willOpenInForeground(TabLaunchType.FROM_RESTORE,
                 state.isIncognito());
-        tab.initialize(null, mTabContentManager, new TabDelegateFactory(), !selectTab, false);
+        tab.initialize(
+                null, mTabContentManager, createDefaultTabDelegateFactory(), !selectTab, false);
         assert state.isIncognito() == mIncognito;
         mTabModel.addTab(tab, index, TabLaunchType.FROM_RESTORE);
         return tab;
@@ -288,32 +289,49 @@ public class ChromeTabCreator extends TabCreatorManager.TabCreator {
 
     /**
      * @param type Type of the tab launch.
+     * @param intent The intent causing the tab launch.
      * @return The page transition type constant.
      */
-    private static int getTransitionType(TabLaunchType type) {
+    private int getTransitionType(TabLaunchType type, Intent intent) {
+        int transition = PageTransition.LINK;
         switch (type) {
+            case FROM_RESTORE:
             case FROM_LINK:
             case FROM_EXTERNAL_APP:
-                return PageTransition.LINK | PageTransition.FROM_API;
-            case FROM_MENU_OR_OVERVIEW:
+                transition = PageTransition.LINK | PageTransition.FROM_API;
+                break;
+            case FROM_CHROME_UI:
             case FROM_LONGPRESS_FOREGROUND:
             case FROM_LONGPRESS_BACKGROUND:
-            case FROM_KEYBOARD:
-                return PageTransition.AUTO_TOPLEVEL;
+                transition = PageTransition.AUTO_TOPLEVEL;
+                break;
             default:
                 assert false;
-                return PageTransition.LINK;
+                break;
         }
+
+        return IntentHandler.getTransitionTypeFromIntent(mActivity.getApplicationContext(),
+                intent, transition);
     }
 
     /**
      * Sets the tab model and tab content manager to use.
-     * @param model   The new {@link TabModel} to use.
-     * @param manager The new {@link TabContentManager} to use.
+     * @param model           The new {@link TabModel} to use.
+     * @param orderController The controller for determining the order of tabs.
+     * @param manager         The new {@link TabContentManager} to use.
      */
-    public void setTabModel(TabModel model, TabContentManager manager) {
+    public void setTabModel(
+            TabModel model, TabModelOrderController orderController, TabContentManager manager) {
         mTabModel = model;
+        mOrderController = orderController;
         mTabContentManager = manager;
     }
 
+    /**
+     * @return The default tab delegate factory to be used if creating new tabs w/o parents.
+     */
+    public TabDelegateFactory createDefaultTabDelegateFactory() {
+        return new TabDelegateFactory();
+    }
+
 }
