@@ -8,15 +8,17 @@ import android.content.Context;
 import android.text.TextUtils;
 
 import org.chromium.base.ApiCompatibilityUtils;
-import org.chromium.base.ApplicationStatus;
+import org.chromium.base.ContextUtils;
 import org.chromium.chrome.R;
 import org.chromium.chrome.browser.dom_distiller.DomDistillerServiceFactory;
 import org.chromium.chrome.browser.dom_distiller.DomDistillerTabUtils;
 import org.chromium.chrome.browser.ntp.NewTabPage;
+import org.chromium.chrome.browser.offlinepages.OfflinePageUtils;
 import org.chromium.chrome.browser.tab.Tab;
 import org.chromium.chrome.browser.toolbar.ToolbarModel.ToolbarModelDelegate;
 import org.chromium.components.dom_distiller.core.DomDistillerService;
 import org.chromium.components.dom_distiller.core.DomDistillerUrlUtils;
+import org.chromium.components.security_state.ConnectionSecurityLevel;
 import org.chromium.content_public.browser.WebContents;
 
 /**
@@ -29,6 +31,16 @@ class ToolbarModelImpl extends ToolbarModel implements ToolbarDataProvider, Tool
     private int mPrimaryColor;
     private boolean mIsUsingBrandColor;
 
+    /**
+     * Default constructor for this class.
+     */
+    public ToolbarModelImpl() {
+        super();
+        mPrimaryColor = ApiCompatibilityUtils.getColor(
+                ContextUtils.getApplicationContext().getResources(),
+                R.color.default_primary_color);
+    }
+
     /**
      * Handle any initialization that must occur after native has been initialized.
      */
@@ -76,12 +88,14 @@ class ToolbarModelImpl extends ToolbarModel implements ToolbarDataProvider, Tool
 
     @Override
     public String getText() {
+        if (mTab != null && mTab.isBlimpTab()) return mTab.getUrl().trim();
+
         String displayText = super.getText();
 
-        if (mTab == null) return displayText;
+        if (mTab == null || mTab.isFrozen()) return displayText;
 
         String url = mTab.getUrl().trim();
-        if (!mTab.isFrozen() && DomDistillerUrlUtils.isDistilledPage(url)) {
+        if (DomDistillerUrlUtils.isDistilledPage(url)) {
             if (isStoredArticle(url)) {
                 DomDistillerService domDistillerService =
                         DomDistillerServiceFactory.getForProfile(mTab.getProfile());
@@ -94,6 +108,11 @@ class ToolbarModelImpl extends ToolbarModel implements ToolbarDataProvider, Tool
                 displayText =
                         DomDistillerTabUtils.getFormattedUrlFromOriginalDistillerUrl(originalUrl);
             }
+        } else if (mTab.isOfflinePage()
+                && mTab.getSecurityLevel() == ConnectionSecurityLevel.NONE) {
+            String originalUrl = mTab.getOriginalUrl();
+            displayText = OfflinePageUtils.stripSchemeFromOnlineUrl(
+                  DomDistillerTabUtils.getFormattedUrlFromOriginalDistillerUrl(originalUrl));
         }
 
         return displayText;
@@ -118,7 +137,7 @@ class ToolbarModelImpl extends ToolbarModel implements ToolbarDataProvider, Tool
      */
     public void setPrimaryColor(int color) {
         mPrimaryColor = color;
-        Context context = ApplicationStatus.getApplicationContext();
+        Context context = ContextUtils.getApplicationContext();
         mIsUsingBrandColor = !isIncognito()
                 && mPrimaryColor != ApiCompatibilityUtils.getColor(context.getResources(),
                         R.color.default_primary_color)
