@@ -7,6 +7,7 @@ package org.chromium.chrome.browser.webapps;
 import android.content.Context;
 import android.graphics.Color;
 import android.util.AttributeSet;
+import android.view.View;
 
 import org.chromium.chrome.browser.compositor.layouts.eventfilter.EdgeSwipeHandler;
 import org.chromium.chrome.browser.widget.ClipDrawableProgressBar.DrawingInfo;
@@ -25,6 +26,10 @@ public class WebappControlContainer extends ViewResourceFrameLayout
         super(context, attrs);
     }
 
+    @Override
+    public void initWithToolbar(int toolbarLayoutId) {
+    }
+
     @Override
     public ViewResourceAdapter getToolbarResourceAdapter() {
         return getResourceAdapter();
@@ -42,4 +47,9 @@ public class WebappControlContainer extends ViewResourceFrameLayout
     public int getToolbarBackgroundColor() {
         return Color.WHITE;
     }
+
+    @Override
+    public View getView() {
+        return this;
+    }
 }
