@@ -7,8 +7,10 @@ package org.chromium.chrome.browser.webapps;
 import android.content.Intent;
 import android.graphics.Bitmap;
 import android.net.Uri;
-import android.util.Log;
+import android.text.TextUtils;
 
+import org.chromium.base.Log;
+import org.chromium.blink_public.platform.WebDisplayMode;
 import org.chromium.chrome.browser.ShortcutHelper;
 import org.chromium.chrome.browser.ShortcutSource;
 import org.chromium.chrome.browser.util.IntentUtils;
@@ -18,13 +20,17 @@ import org.chromium.content_public.common.ScreenOrientationValues;
  * Stores info about a web app.
  */
 public class WebappInfo {
+    private static final String TAG = "WebappInfo";
+
     private boolean mIsInitialized;
     private String mId;
     private String mEncodedIcon;
     private Bitmap mDecodedIcon;
     private Uri mUri;
+    private Uri mScopeUri;
     private String mName;
     private String mShortName;
+    private int mDisplayMode;
     private int mOrientation;
     private int mSource;
     private long mThemeColor;
@@ -35,6 +41,15 @@ public class WebappInfo {
         return new WebappInfo();
     }
 
+    protected static String urlFromIntent(Intent intent) {
+        return IntentUtils.safeGetStringExtra(intent, ShortcutHelper.EXTRA_URL);
+    }
+
+    protected static int sourceFromIntent(Intent intent) {
+        return IntentUtils.safeGetIntExtra(
+                intent, ShortcutHelper.EXTRA_SOURCE, ShortcutSource.UNKNOWN);
+    }
+
     private static String titleFromIntent(Intent intent) {
         // The reference to title has been kept for reasons of backward compatibility. For intents
         // and shortcuts which were created before we utilized the concept of name and shortName,
@@ -43,12 +58,12 @@ public class WebappInfo {
         return title == null ? "" : title;
     }
 
-    public static String nameFromIntent(Intent intent) {
+    private static String nameFromIntent(Intent intent) {
         String name = IntentUtils.safeGetStringExtra(intent, ShortcutHelper.EXTRA_NAME);
         return name == null ? titleFromIntent(intent) : name;
     }
 
-    public static String shortNameFromIntent(Intent intent) {
+    private static String shortNameFromIntent(Intent intent) {
         String shortName = IntentUtils.safeGetStringExtra(intent, ShortcutHelper.EXTRA_SHORT_NAME);
         return shortName == null ? titleFromIntent(intent) : shortName;
     }
@@ -60,11 +75,13 @@ public class WebappInfo {
     public static WebappInfo create(Intent intent) {
         String id = IntentUtils.safeGetStringExtra(intent, ShortcutHelper.EXTRA_ID);
         String icon = IntentUtils.safeGetStringExtra(intent, ShortcutHelper.EXTRA_ICON);
-        String url = IntentUtils.safeGetStringExtra(intent, ShortcutHelper.EXTRA_URL);
-        int orientation = IntentUtils.safeGetIntExtra(intent,
-                ShortcutHelper.EXTRA_ORIENTATION, ScreenOrientationValues.DEFAULT);
-        int source = IntentUtils.safeGetIntExtra(intent,
-                ShortcutHelper.EXTRA_SOURCE, ShortcutSource.UNKNOWN);
+        String url = urlFromIntent(intent);
+        String scope = IntentUtils.safeGetStringExtra(intent, ShortcutHelper.EXTRA_SCOPE);
+        int displayMode = IntentUtils.safeGetIntExtra(
+                intent, ShortcutHelper.EXTRA_DISPLAY_MODE, WebDisplayMode.Standalone);
+        int orientation = IntentUtils.safeGetIntExtra(
+                intent, ShortcutHelper.EXTRA_ORIENTATION, ScreenOrientationValues.DEFAULT);
+        int source = sourceFromIntent(intent);
         long themeColor = IntentUtils.safeGetLongExtra(intent,
                 ShortcutHelper.EXTRA_THEME_COLOR,
                 ShortcutHelper.MANIFEST_COLOR_INVALID_OR_MISSING);
@@ -77,43 +94,53 @@ public class WebappInfo {
         String name = nameFromIntent(intent);
         String shortName = shortNameFromIntent(intent);
 
-        return create(id, url, icon, name, shortName, orientation, source,
+        return create(id, url, scope, icon, name, shortName, displayMode, orientation, source,
                 themeColor, backgroundColor, isIconGenerated);
     }
 
     /**
      * Construct a WebappInfo.
-     * @param id ID for the webapp.
-     * @param url URL for the webapp.
-     * @param icon Icon to show for the webapp.
-     * @param name Name of the webapp.
-     * @param shortName The short name of the webapp.
-     * @param orientation Orientation of the webapp.
-     * @param source Source where the webapp was added from.
-     * @param themeColor The theme color of the webapp.
+     * @param id              ID for the webapp.
+     * @param url             URL for the webapp.
+     * @param scope           Scope for the webapp.
+     * @param icon            Icon to show for the webapp.
+     * @param name            Name of the webapp.
+     * @param shortName       The short name of the webapp.
+     * @param displayMode     Display mode of the webapp.
+     * @param orientation     Orientation of the webapp.
+     * @param source          Source where the webapp was added from.
+     * @param themeColor      The theme color of the webapp.
+     * @param backgroundColor The background color of the webapp.
      * @param isIconGenerated Whether the |icon| was generated by Chromium.
      */
-    public static WebappInfo create(String id, String url, String icon, String name,
-            String shortName, int orientation, int source, long themeColor,
+    public static WebappInfo create(String id, String url, String scope, String icon, String name,
+            String shortName, int displayMode, int orientation, int source, long themeColor,
             long backgroundColor, boolean isIconGenerated) {
         if (id == null || url == null) {
-            Log.e("WebappInfo", "Data passed in was incomplete: " + id + ", " + url);
+            Log.e(TAG, "Incomplete data provided: " + id + ", " + url);
             return null;
         }
 
-        Uri uri = Uri.parse(url);
-        return new WebappInfo(id, uri, icon, name, shortName, orientation, source,
-                themeColor, backgroundColor, isIconGenerated);
+        return new WebappInfo(id, url, scope, icon, name, shortName, displayMode, orientation,
+                source, themeColor, backgroundColor, isIconGenerated);
     }
 
-    private WebappInfo(String id, Uri uri, String encodedIcon, String name,
-            String shortName, int orientation, int source, long themeColor,
+    protected WebappInfo(String id, String url, String scope, String encodedIcon, String name,
+            String shortName, int displayMode, int orientation, int source, long themeColor,
             long backgroundColor, boolean isIconGenerated) {
+        Uri uri = Uri.parse(url);
+        if (TextUtils.isEmpty(scope)) {
+            scope = ShortcutHelper.getScopeFromUrl(url);
+        }
+        Uri scopeUri = Uri.parse(scope);
+
         mEncodedIcon = encodedIcon;
         mId = id;
         mName = name;
         mShortName = shortName;
         mUri = uri;
+        mScopeUri = scopeUri;
+        mDisplayMode = displayMode;
         mOrientation = orientation;
         mSource = source;
         mThemeColor = themeColor;
@@ -122,26 +149,7 @@ public class WebappInfo {
         mIsInitialized = mUri != null;
     }
 
-    private WebappInfo() {
-    }
-
-    /**
-     * Copies all the fields from the given WebappInfo into this instance.
-     * @param newInfo Information about the new webapp.
-     */
-    void copy(WebappInfo newInfo) {
-        mIsInitialized = newInfo.mIsInitialized;
-        mEncodedIcon = newInfo.mEncodedIcon;
-        mDecodedIcon = newInfo.mDecodedIcon;
-        mId = newInfo.mId;
-        mUri = newInfo.mUri;
-        mName = newInfo.mName;
-        mShortName = newInfo.mShortName;
-        mOrientation = newInfo.mOrientation;
-        mSource = newInfo.mSource;
-        mThemeColor = newInfo.mThemeColor;
-        mBackgroundColor = newInfo.mBackgroundColor;
-        mIsIconGenerated = newInfo.mIsIconGenerated;
+    protected WebappInfo() {
     }
 
     public boolean isInitialized() {
@@ -156,6 +164,10 @@ public class WebappInfo {
         return mUri;
     }
 
+    public Uri scopeUri() {
+        return mScopeUri;
+    }
+
     public String name() {
         return mName;
     }
@@ -164,6 +176,14 @@ public class WebappInfo {
         return mShortName;
     }
 
+    public int displayMode() {
+        return mDisplayMode;
+    }
+
+    public String webApkPackageName() {
+        return null;
+    }
+
     public int orientation() {
         return mOrientation;
     }
@@ -243,13 +263,27 @@ public class WebappInfo {
     public void setWebappIntentExtras(Intent intent) {
         intent.putExtra(ShortcutHelper.EXTRA_ID, id());
         intent.putExtra(ShortcutHelper.EXTRA_URL, uri().toString());
+        intent.putExtra(ShortcutHelper.EXTRA_SCOPE, scopeUri().toString());
         intent.putExtra(ShortcutHelper.EXTRA_ICON, encodedIcon());
+        intent.putExtra(ShortcutHelper.EXTRA_VERSION, ShortcutHelper.WEBAPP_SHORTCUT_VERSION);
         intent.putExtra(ShortcutHelper.EXTRA_NAME, name());
         intent.putExtra(ShortcutHelper.EXTRA_SHORT_NAME, shortName());
+        intent.putExtra(ShortcutHelper.EXTRA_DISPLAY_MODE, displayMode());
         intent.putExtra(ShortcutHelper.EXTRA_ORIENTATION, orientation());
         intent.putExtra(ShortcutHelper.EXTRA_SOURCE, source());
         intent.putExtra(ShortcutHelper.EXTRA_THEME_COLOR, themeColor());
         intent.putExtra(ShortcutHelper.EXTRA_BACKGROUND_COLOR, backgroundColor());
         intent.putExtra(ShortcutHelper.EXTRA_IS_ICON_GENERATED, isIconGenerated());
+        if (webApkPackageName() != null) {
+            intent.putExtra(ShortcutHelper.EXTRA_WEBAPK_PACKAGE_NAME, webApkPackageName());
+        }
+    }
+
+    /**
+     * Returns true if the WebappInfo was created for an Intent fired from a launcher shortcut (as
+     * opposed to an intent from a push notification or other internal source).
+     */
+    public boolean isLaunchedFromHomescreen() {
+        return source() != ShortcutSource.NOTIFICATION;
     }
 }
