@@ -22,6 +22,7 @@ import android.widget.TextView;
 
 import org.chromium.base.ThreadUtils;
 import org.chromium.base.test.util.Feature;
+import org.chromium.base.test.util.RetryOnFailure;
 import org.chromium.chrome.R;
 import org.chromium.chrome.browser.ShortcutHelper;
 import org.chromium.chrome.browser.infobar.AppBannerInfoBarAndroid;
@@ -31,24 +32,26 @@ import org.chromium.chrome.browser.infobar.InfoBarContainer;
 import org.chromium.chrome.browser.infobar.InfoBarContainer.InfoBarAnimationListener;
 import org.chromium.chrome.browser.webapps.WebappDataStorage;
 import org.chromium.chrome.test.ChromeTabbedActivityTestBase;
-import org.chromium.chrome.test.util.TestHttpServerClient;
 import org.chromium.chrome.test.util.browser.TabLoadObserver;
+import org.chromium.chrome.test.util.browser.WebappTestPage;
 import org.chromium.content.browser.test.util.Criteria;
 import org.chromium.content.browser.test.util.CriteriaHelper;
 import org.chromium.content.browser.test.util.TouchCommon;
+import org.chromium.net.test.EmbeddedTestServer;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.concurrent.Callable;
 
 /**
  * Tests the app banners.
  */
 public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
-    private static final String NATIVE_APP_URL =
-            TestHttpServerClient.getUrl("chrome/test/data/banners/play_app_test_page.html");
+    private static final String NATIVE_APP_PATH =
+            "/chrome/test/data/banners/play_app_test_page.html";
 
-    private static final String NATIVE_ICON_URL =
-            TestHttpServerClient.getUrl("chrome/test/data/banners/launcher-icon-4x.png");
+    private static final String NATIVE_ICON_PATH =
+            "/chrome/test/data/banners/launcher-icon-4x.png";
 
     private static final String NATIVE_APP_TITLE = "Mock app title";
 
@@ -60,17 +63,22 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
 
     private static final String NATIVE_APP_BLANK_REFERRER = "playinline=chrome_inline";
 
-    private static final String NATIVE_APP_URL_WITH_MANIFEST_URL =
-            TestHttpServerClient.getUrl("chrome/test/data/banners/play_app_url_test_page.html");
+    private static final String NATIVE_APP_URL_WITH_MANIFEST_PATH =
+            "/chrome/test/data/banners/play_app_url_test_page.html";
 
-    private static final String WEB_APP_URL =
-            TestHttpServerClient.getUrl("chrome/test/data/banners/manifest_test_page.html");
+    private static final String WEB_APP_PATH =
+            "/chrome/test/data/banners/manifest_test_page.html";
+
+    private static final String WEB_APP_SHORT_TITLE_MANIFEST =
+            "/chrome/test/data/banners/manifest_short_name_only.json";
 
     private static final String WEB_APP_TITLE = "Manifest test app";
 
+    private static final String WEB_APP_SHORT_TITLE = "Manifest";
+
     private static final String INSTALL_ACTION = "INSTALL_ACTION";
 
-    private static class MockAppDetailsDelegate extends AppDetailsDelegate {
+    private class MockAppDetailsDelegate extends AppDetailsDelegate {
         private Observer mObserver;
         private AppData mAppData;
         private int mNumRetrieved;
@@ -86,7 +94,7 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
             mInstallIntent = new Intent(INSTALL_ACTION);
 
             mAppData = new AppData(url, packageName);
-            mAppData.setPackageInfo(NATIVE_APP_TITLE, NATIVE_ICON_URL, 4.5f,
+            mAppData.setPackageInfo(NATIVE_APP_TITLE, mTestServer.getURL(NATIVE_ICON_PATH), 4.5f,
                     NATIVE_APP_INSTALL_TEXT, null, mInstallIntent);
             ThreadUtils.runOnUiThread(new Runnable() {
                 @Override
@@ -119,21 +127,21 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
     }
 
     private static class TestDataStorageFactory extends WebappDataStorage.Factory {
-        public Bitmap mSplashImage;
+        public String mSplashImage;
 
         @Override
-        public WebappDataStorage create(final Context context, final String webappId) {
-            return new WebappDataStorageWrapper(context, webappId);
+        public WebappDataStorage create(final String webappId) {
+            return new WebappDataStorageWrapper(webappId);
         }
 
         private class WebappDataStorageWrapper extends WebappDataStorage {
 
-            public WebappDataStorageWrapper(Context context, String webappId) {
-                super(context, webappId);
+            public WebappDataStorageWrapper(String webappId) {
+                super(webappId);
             }
 
             @Override
-            public void updateSplashScreenImage(Bitmap splashScreenImage) {
+            public void updateSplashScreenImage(String splashScreenImage) {
                 assertNull(mSplashImage);
                 mSplashImage = splashScreenImage;
             }
@@ -152,7 +160,10 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
     }
 
     private MockAppDetailsDelegate mDetailsDelegate;
+    private String mNativeAppUrl;
     private TestPackageManager mPackageManager;
+    private EmbeddedTestServer mTestServer;
+    private String mWebAppUrl;
 
     @Override
     public void startMainActivity() throws InterruptedException {
@@ -162,7 +173,7 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
     @Override
     protected void setUp() throws Exception {
         mPackageManager = new TestPackageManager();
-        AppBannerManager.setIsEnabledForTesting(true);
+        AppBannerManager.setIsSupported(true);
         AppBannerInfoBarDelegateAndroid.setPackageManagerForTesting(mPackageManager);
         ShortcutHelper.setDelegateForTests(new ShortcutHelper.Delegate() {
             @Override
@@ -182,15 +193,23 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
             }
         });
 
-        AppBannerManager.disableSecureSchemeCheckForTesting();
-
         // Navigations in this test are all of type ui::PAGE_TRANSITION_LINK, i.e. indirect.
         // Force indirect navigations to be worth the same as direct for testing.
         AppBannerManager.setEngagementWeights(1, 1);
+
+        mTestServer = EmbeddedTestServer.createAndStartServer(getInstrumentation().getContext());
+        mNativeAppUrl = mTestServer.getURL(NATIVE_APP_PATH);
+        mWebAppUrl = mTestServer.getURL(WEB_APP_PATH);
+    }
+
+    @Override
+    protected void tearDown() throws Exception {
+        mTestServer.stopAndDestroyServer();
+        super.tearDown();
     }
 
     private void waitUntilNoInfoBarsExist() throws Exception {
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 return getInfoBars().isEmpty();
@@ -199,19 +218,18 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
     }
 
     private void waitUntilAppDetailsRetrieved(final int numExpected) throws Exception {
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
-                AppBannerManager manager =
-                        getActivity().getActivityTab().getAppBannerManagerForTesting();
+                AppBannerManager manager = getActivity().getActivityTab().getAppBannerManager();
                 return mDetailsDelegate.mNumRetrieved == numExpected
-                        && !manager.isFetcherActiveForTesting();
+                        && !manager.isActiveForTesting();
             }
         });
     }
 
     private void waitUntilAppBannerInfoBarAppears(final String title) throws Exception {
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 List<InfoBar> infobars = getInfoBars();
@@ -241,7 +259,7 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
         new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(url);
         waitUntilAppDetailsRetrieved(2);
         waitUntilAppBannerInfoBarAppears(NATIVE_APP_TITLE);
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 return listener.mDoneAnimating;
@@ -264,7 +282,7 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
         // Wait for the infobar to register that the app is installing.
         final String installingText =
                 getInstrumentation().getTargetContext().getString(R.string.app_banner_installing);
-        CriteriaHelper.pollForCriteria(new Criteria() {
+        CriteriaHelper.pollInstrumentationThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 return getInstrumentation().checkMonitorHit(activityMonitor, 1)
@@ -276,7 +294,7 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
         mPackageManager.isInstalled = true;
         final String openText =
                 getInstrumentation().getTargetContext().getString(R.string.app_banner_open);
-        CriteriaHelper.pollForCriteria(new Criteria() {
+        CriteriaHelper.pollInstrumentationThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 return TextUtils.equals(button.getText(), openText);
@@ -284,57 +302,87 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
         });
     }
 
+    public void triggerWebAppBanner(String url, String expectedTitle) throws Exception {
+        // Visit the site in a new tab.
+        loadUrlInNewTab("about:blank");
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(url);
+
+        CriteriaHelper.pollUiThread(new Criteria() {
+            @Override
+            public boolean isSatisfied() {
+                AppBannerManager manager = getActivity().getActivityTab().getAppBannerManager();
+                return !manager.isActiveForTesting();
+            }
+        });
+        waitUntilNoInfoBarsExist();
+
+        // Indicate a day has passed, then revisit the page to show the banner.
+        AppBannerManager.setTimeDeltaForTesting(1);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(url);
+        CriteriaHelper.pollUiThread(new Criteria() {
+            @Override
+            public boolean isSatisfied() {
+                AppBannerManager manager = getActivity().getActivityTab().getAppBannerManager();
+                return !manager.isActiveForTesting();
+            }
+        });
+        waitUntilAppBannerInfoBarAppears(expectedTitle);
+    }
+
     @SmallTest
     @Feature({"AppBanners"})
+    @RetryOnFailure
     public void testFullNativeInstallPathwayFromId() throws Exception {
-        runFullNativeInstallPathway(NATIVE_APP_URL, NATIVE_APP_BLANK_REFERRER);
+        runFullNativeInstallPathway(mNativeAppUrl, NATIVE_APP_BLANK_REFERRER);
     }
 
     @SmallTest
     @Feature({"AppBanners"})
     public void testFullNativeInstallPathwayFromUrl() throws Exception {
-        runFullNativeInstallPathway(NATIVE_APP_URL_WITH_MANIFEST_URL, NATIVE_APP_REFERRER);
+        runFullNativeInstallPathway(
+                mTestServer.getURL(NATIVE_APP_URL_WITH_MANIFEST_PATH), NATIVE_APP_REFERRER);
     }
 
     @MediumTest
     @Feature({"AppBanners"})
-    public void testBannerAppearsThenDoesNotAppearAgainForMonths() throws Exception {
+    @RetryOnFailure
+    public void testBannerAppearsThenDoesNotAppearAgainForWeeks() throws Exception {
         // Visit a site that requests a banner.
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(1);
         waitUntilNoInfoBarsExist();
 
         // Indicate a day has passed, then revisit the page.
         AppBannerManager.setTimeDeltaForTesting(1);
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(2);
         waitUntilAppBannerInfoBarAppears(NATIVE_APP_TITLE);
 
         // Revisit the page to make the banner go away, but don't explicitly dismiss it.
-        // This hides the banner for a few months.
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        // This hides the banner for two weeks.
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(3);
         waitUntilNoInfoBarsExist();
 
-        // Wait a month until revisiting the page.
-        AppBannerManager.setTimeDeltaForTesting(31);
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        // Wait a week until revisiting the page.
+        AppBannerManager.setTimeDeltaForTesting(7);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(4);
         waitUntilNoInfoBarsExist();
 
-        AppBannerManager.setTimeDeltaForTesting(32);
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        AppBannerManager.setTimeDeltaForTesting(8);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(5);
         waitUntilNoInfoBarsExist();
 
-        // Wait two months until revisiting the page, which should pop up the banner.
-        AppBannerManager.setTimeDeltaForTesting(61);
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        // Wait two weeks until revisiting the page, which should pop up the banner.
+        AppBannerManager.setTimeDeltaForTesting(14);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(6);
         waitUntilNoInfoBarsExist();
 
-        AppBannerManager.setTimeDeltaForTesting(62);
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        AppBannerManager.setTimeDeltaForTesting(15);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(7);
         waitUntilAppBannerInfoBarAppears(NATIVE_APP_TITLE);
     }
@@ -343,7 +391,7 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
     @Feature({"AppBanners"})
     public void testBlockedBannerDoesNotAppearAgainForMonths() throws Exception {
         // Visit a site that requests a banner.
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(1);
         waitUntilNoInfoBarsExist();
 
@@ -352,12 +400,12 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
         final InfobarListener listener = new InfobarListener();
         container.setAnimationListener(listener);
         AppBannerManager.setTimeDeltaForTesting(1);
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(2);
         waitUntilAppBannerInfoBarAppears(NATIVE_APP_TITLE);
 
         // Explicitly dismiss the banner.
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 return listener.mDoneAnimating;
@@ -370,77 +418,65 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
 
         // Waiting two months shouldn't be long enough.
         AppBannerManager.setTimeDeltaForTesting(61);
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(3);
         waitUntilNoInfoBarsExist();
 
         AppBannerManager.setTimeDeltaForTesting(62);
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(4);
         waitUntilNoInfoBarsExist();
 
         // Waiting three months should allow banners to reappear.
         AppBannerManager.setTimeDeltaForTesting(91);
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(5);
         waitUntilNoInfoBarsExist();
 
         AppBannerManager.setTimeDeltaForTesting(92);
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
         waitUntilAppDetailsRetrieved(6);
         waitUntilAppBannerInfoBarAppears(NATIVE_APP_TITLE);
     }
 
     @MediumTest
     @Feature({"AppBanners"})
+    @RetryOnFailure
     public void testBitmapFetchersCanOverlapWithoutCrashing() throws Exception {
         // Visit a site that requests a banner rapidly and repeatedly.
         for (int i = 1; i <= 10; i++) {
-            new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(NATIVE_APP_URL);
+            new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mNativeAppUrl);
 
             final Integer iteration = Integer.valueOf(i);
-            CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
-                @Override
-                public boolean isSatisfied() {
-                    return mDetailsDelegate.mNumRetrieved == iteration;
-                }
-            });
+            CriteriaHelper.pollUiThread(
+                    Criteria.equals(iteration, new Callable<Integer>() {
+                        @Override
+                        public Integer call() {
+                            return mDetailsDelegate.mNumRetrieved;
+                        }
+                    }));
         }
     }
 
     @SmallTest
     @Feature({"AppBanners"})
+    @RetryOnFailure
     public void testWebAppBannerAppears() throws Exception {
-        // Visit the site in a new tab.
-        loadUrlInNewTab("about:blank");
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(WEB_APP_URL);
-
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
-            @Override
-            public boolean isSatisfied() {
-                AppBannerManager manager =
-                        getActivity().getActivityTab().getAppBannerManagerForTesting();
-                return !manager.isFetcherActiveForTesting();
-            }
-        });
-        waitUntilNoInfoBarsExist();
-
-        // Indicate a day has passed, then revisit the page to show the banner.
-        AppBannerManager.setTimeDeltaForTesting(1);
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(WEB_APP_URL);
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
-            @Override
-            public boolean isSatisfied() {
-                AppBannerManager manager =
-                        getActivity().getActivityTab().getAppBannerManagerForTesting();
-                return !manager.isFetcherActiveForTesting();
-            }
-        });
-        waitUntilAppBannerInfoBarAppears(WEB_APP_TITLE);
+        triggerWebAppBanner(mWebAppUrl, WEB_APP_TITLE);
     }
 
     @SmallTest
     @Feature({"AppBanners"})
+    @RetryOnFailure
+    public void testBannerFallsBackToShortName() throws Exception {
+        triggerWebAppBanner(WebappTestPage.urlOfPageWithServiceWorkerAndManifest(
+                                    mTestServer, WEB_APP_SHORT_TITLE_MANIFEST),
+                WEB_APP_SHORT_TITLE);
+    }
+
+    @SmallTest
+    @Feature({"AppBanners"})
+    @RetryOnFailure
     public void testWebAppSplashscreenIsDownloaded() throws Exception {
         // Sets the overriden factory to observer splash screen update.
         final TestDataStorageFactory dataStorageFactory = new TestDataStorageFactory();
@@ -448,14 +484,13 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
 
         // Visit the site in a new tab.
         loadUrlInNewTab("about:blank");
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(WEB_APP_URL);
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mWebAppUrl);
 
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
-                AppBannerManager manager =
-                        getActivity().getActivityTab().getAppBannerManagerForTesting();
-                return !manager.isFetcherActiveForTesting();
+                AppBannerManager manager = getActivity().getActivityTab().getAppBannerManager();
+                return !manager.isActiveForTesting();
             }
         });
         waitUntilNoInfoBarsExist();
@@ -467,17 +502,16 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
 
         // Indicate a day has passed, then revisit the page to show the banner.
         AppBannerManager.setTimeDeltaForTesting(1);
-        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(WEB_APP_URL);
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        new TabLoadObserver(getActivity().getActivityTab()).fullyLoadUrl(mWebAppUrl);
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
-                AppBannerManager manager =
-                        getActivity().getActivityTab().getAppBannerManagerForTesting();
-                return !manager.isFetcherActiveForTesting();
+                AppBannerManager manager = getActivity().getActivityTab().getAppBannerManager();
+                return !manager.isActiveForTesting();
             }
         });
         waitUntilAppBannerInfoBarAppears(WEB_APP_TITLE);
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 return listener.mDoneAnimating;
@@ -491,7 +525,7 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
         TouchCommon.singleClickView(button);
 
         // Make sure that the splash screen icon was downloaded.
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 return dataStorageFactory.mSplashImage != null;
@@ -501,7 +535,8 @@ public class AppBannerManagerTest extends ChromeTabbedActivityTestBase {
         // Test that bitmap sizes match expectations.
         int idealSize = getActivity().getResources().getDimensionPixelSize(
                 R.dimen.webapp_splash_image_size_ideal);
-        assertEquals(idealSize, dataStorageFactory.mSplashImage.getWidth());
-        assertEquals(idealSize, dataStorageFactory.mSplashImage.getHeight());
+        Bitmap splashImage = ShortcutHelper.decodeBitmapFromString(dataStorageFactory.mSplashImage);
+        assertEquals(idealSize, splashImage.getWidth());
+        assertEquals(idealSize, splashImage.getHeight());
     }
 }
