@@ -63,14 +63,6 @@ class MockLayoutHost implements LayoutManagerHost, LayoutRenderHost {
     @Override
     public void onPhysicalBackingSizeChanged(int width, int height) { }
 
-    @Override
-    public void onOverdrawBottomHeightChanged(int overdrawHeight) {}
-
-    @Override
-    public int getCurrentOverdrawBottomHeight() {
-        return 0;
-    }
-
     @Override
     public Context getContext() {
         return mContext;
@@ -119,20 +111,13 @@ class MockLayoutHost implements LayoutManagerHost, LayoutRenderHost {
         return null;
     }
 
-    @Override
-    public Rect getVisibleViewport(Rect rect) {
-        if (rect == null) rect = new Rect();
-        rect.set(0, 0, getWidth(), getHeight());
-        return rect;
-    }
-
     @Override
     public int getTopControlsHeightPixels() {
         return 0;
     }
 
     @Override
-    public boolean areTopControlsPermanentlyHidden() {
+    public boolean areBrowserControlsPermanentlyHidden() {
         return false;
     }
 
@@ -151,12 +136,12 @@ class MockLayoutHost implements LayoutManagerHost, LayoutRenderHost {
     public void onContentChanged() { }
 
     @Override
-    public int getTopControlsBackgroundColor() {
+    public int getBrowserControlsBackgroundColor() {
         return 0;
     }
 
     @Override
-    public float getTopControlsUrlBarAlpha() {
+    public float getBrowserControlsUrlBarAlpha() {
         return 1f;
     }
 
