@@ -53,6 +53,7 @@ class ContextualSearchFakeServer
 
     private String mSearchTermRequested;
     private boolean mShouldUseHttps;
+    private boolean mIsOnline = true;
 
     private boolean mDidEverCallContentViewCoreOnShow;
 
@@ -139,11 +140,15 @@ class ContextualSearchFakeServer
         protected final String mSearchTerm;
         private final String mDisplayText;
         private final String mAlternateTerm;
+        private final String mMid;
         private final boolean mDoPreventPreload;
         private final int mStartAdjust;
         private final int mEndAdjust;
         private final String mContextLanguage;
-
+        private final String mThumbnailUrl;
+        private final String mCaption;
+        private final String mQuickActionUri;
+        private final int mQuickActionCategory;
 
         boolean mDidStartResolution;
         boolean mDidFinishResolution;
@@ -155,15 +160,21 @@ class ContextualSearchFakeServer
          * @param searchTerm            The resolved search term.
          * @param displayText           The display text.
          * @param alternateTerm         The alternate text.
+         * @param mid                   The MID to specify a KP, or an empty string.
          * @param doPreventPreload      Whether search preload should be prevented.
          * @param startAdjust           The start adjustment of the selection.
          * @param endAdjust             The end adjustment of the selection.
          * @param contextLanguage       The language of the context determined by the server.
+         * @param thumbnailUrl          The URL of a thumbnail to display.
+         * @param caption               The caption to display.
+         * @param quickActionUri        The URI for the intent associated with the quick action.
+         * @param quickActionCategory   The category for the quick action.
          */
         FakeTapSearch(String nodeId, boolean isNetworkUnavailable, int responseCode,
-                      String searchTerm, String displayText, String alternateTerm,
-                      boolean doPreventPreload, int startAdjust, int endAdjust,
-                      String contextLanguage) {
+                String searchTerm, String displayText, String alternateTerm, String mid,
+                boolean doPreventPreload, int startAdjust, int endAdjust, String contextLanguage,
+                String thumbnailUrl, String caption, String quickActionUri,
+                int quickActionCategory) {
             super(nodeId);
 
             mIsNetworkUnavailable = isNetworkUnavailable;
@@ -171,10 +182,15 @@ class ContextualSearchFakeServer
             mSearchTerm = searchTerm;
             mDisplayText = displayText;
             mAlternateTerm = alternateTerm;
+            mMid = mid;
             mDoPreventPreload = doPreventPreload;
             mStartAdjust = startAdjust;
             mEndAdjust = endAdjust;
             mContextLanguage = contextLanguage;
+            mThumbnailUrl = thumbnailUrl;
+            mCaption = caption;
+            mQuickActionUri = quickActionUri;
+            mQuickActionCategory = quickActionCategory;
         }
 
         @Override
@@ -189,7 +205,7 @@ class ContextualSearchFakeServer
             mManagerTest.clickNode(getNodeId());
             mManagerTest.waitForSelectionToBe(mSearchTerm);
 
-            if (mPolicy.shouldPreviousTapResolve(getBasePageUrl())) {
+            if (mPolicy.shouldPreviousTapResolve()) {
                 // Now wait for the Search Term Resolution to start.
                 mManagerTest.waitForSearchTermResolutionToStart(this);
 
@@ -244,10 +260,10 @@ class ContextualSearchFakeServer
                 @Override
                 public void run() {
                     if (!mDidFinishResolution) {
-                        handleSearchTermResolutionResponse(
-                                mIsNetworkUnavailable, mResponseCode, mSearchTerm, mDisplayText,
-                                mAlternateTerm, mDoPreventPreload, mStartAdjust, mEndAdjust,
-                                mContextLanguage);
+                        handleSearchTermResolutionResponse(mIsNetworkUnavailable, mResponseCode,
+                                mSearchTerm, mDisplayText, mAlternateTerm, mMid, mDoPreventPreload,
+                                mStartAdjust, mEndAdjust, mContextLanguage, mThumbnailUrl,
+                                mCaption, mQuickActionUri, mQuickActionCategory);
 
                         mActiveFakeTapSearch = null;
                         mDidFinishResolution = true;
@@ -272,16 +288,24 @@ class ContextualSearchFakeServer
          * @param searchTerm
          * @param displayText
          * @param alternateTerm
+         * @param mid
          * @param doPreventPreload
          * @param startAdjust
          * @param endAdjust
          * @param contextLanguage
+         * @param thumbnailUrl
+         * @param caption
+         * @param quickActionUri
+         * @param quickActionCategory
          */
         FakeSlowResolveSearch(String nodeId, boolean isNetworkUnavailable, int responseCode,
-                String searchTerm, String displayText, String alternateTerm,
-                boolean doPreventPreload, int startAdjust, int endAdjust, String contextLanguage) {
+                String searchTerm, String displayText, String alternateTerm, String mid,
+                boolean doPreventPreload, int startAdjust, int endAdjust, String contextLanguage,
+                String thumbnailUrl, String caption, String quickActionUri,
+                int quickActionCategory) {
             super(nodeId, isNetworkUnavailable, responseCode, searchTerm, displayText,
-                    alternateTerm, doPreventPreload, startAdjust, endAdjust, contextLanguage);
+                    alternateTerm, mid, doPreventPreload, startAdjust, endAdjust, contextLanguage,
+                    thumbnailUrl, caption, quickActionUri, quickActionCategory);
         }
 
         @Override
@@ -296,7 +320,7 @@ class ContextualSearchFakeServer
             mManagerTest.clickNode(getNodeId());
             mManagerTest.waitForSelectionToBe(mSearchTerm);
 
-            if (mPolicy.shouldPreviousTapResolve(getBasePageUrl())) {
+            if (mPolicy.shouldPreviousTapResolve()) {
                 // Now wait for the Search Term Resolution to start.
                 mManagerTest.waitForSearchTermResolutionToStart(this);
             } else {
@@ -362,7 +386,7 @@ class ContextualSearchFakeServer
         private boolean mIsVisible;
 
         ContentViewCoreWrapper(ChromeActivity activity) {
-            super(activity);
+            super(activity, "");
         }
 
         @Override
@@ -462,6 +486,14 @@ class ContextualSearchFakeServer
         return mDidEverCallContentViewCoreOnShow;
     }
 
+    /**
+     * Sets whether the device is currently online or not.
+     */
+    @VisibleForTesting
+    void setIsOnline(boolean isOnline) {
+        mIsOnline = isOnline;
+    }
+
     /**
      * Resets the fake server's member data.
      */
@@ -470,6 +502,7 @@ class ContextualSearchFakeServer
         mLoadedUrl = null;
         mSearchTermRequested = null;
         mShouldUseHttps = false;
+        mIsOnline = true;
         mLoadedUrlCount = 0;
     }
 
@@ -501,15 +534,24 @@ class ContextualSearchFakeServer
 
     @Override
     public void handleSearchTermResolutionResponse(boolean isNetworkUnavailable, int responseCode,
-            String searchTerm, String displayText, String alternateTerm, boolean doPreventPreload,
-            int selectionStartAdjust, int selectionEndAdjust, String contextLanguage) {
+            String searchTerm, String displayText, String alternateTerm, String mid,
+            boolean doPreventPreload, int selectionStartAdjust, int selectionEndAdjust,
+            String contextLanguage, String thumbnailUrl, String caption, String quickActionUri,
+            int quickActionCategory) {
         mBaseManager.handleSearchTermResolutionResponse(isNetworkUnavailable, responseCode,
-                searchTerm, displayText, alternateTerm, doPreventPreload, selectionStartAdjust,
-                selectionEndAdjust, contextLanguage);
+                searchTerm, displayText, alternateTerm, mid, doPreventPreload, selectionStartAdjust,
+                selectionEndAdjust, contextLanguage, thumbnailUrl, caption, quickActionUri,
+                quickActionCategory);
     }
 
     @Override
-    @Nullable public URL getBasePageUrl() {
+    public boolean isOnline() {
+        return mIsOnline;
+    }
+
+    @Override
+    @Nullable
+    public URL getBasePageUrl() {
         URL baseUrl = mBaseManager.getBasePageUrl();
         if (mShouldUseHttps && baseUrl != null) {
             try {
@@ -537,17 +579,21 @@ class ContextualSearchFakeServer
         registerFakeLongPressSearch(new FakeLongPressSearch("term", "Term"));
         registerFakeLongPressSearch(new FakeLongPressSearch("resolution", "Resolution"));
 
-        registerFakeTapSearch(new FakeTapSearch("search", false, 200,
-                "Search", "Search", "alternate-term", false, 0, 0, ""));
-        registerFakeTapSearch(new FakeTapSearch("term", false, 200,
-                "Term", "Term", "alternate-term", false, 0, 0, ""));
-        registerFakeTapSearch(new FakeTapSearch("resolution", false, 200,
-                "Resolution", "Resolution", "alternate-term", false, 0, 0, ""));
-        registerFakeTapSearch(new FakeTapSearch("german", false, 200,
-                "Deutsche", "Deutsche", "alternate-term", false, 0, 0, "de"));
+        registerFakeTapSearch(new FakeTapSearch(
+                "search", false, 200, "Search", "Search", "alternate-term", "", false, 0, 0, "",
+                "", "", "", QuickActionCategory.NONE));
+        registerFakeTapSearch(new FakeTapSearch(
+                "term", false, 200, "Term", "Term", "alternate-term", "", false, 0, 0, "",
+                "", "", "", QuickActionCategory.NONE));
+        registerFakeTapSearch(new FakeTapSearch("resolution", false, 200, "Resolution",
+                "Resolution", "alternate-term", "", false, 0, 0, "", "", "", "",
+                QuickActionCategory.NONE));
+        registerFakeTapSearch(new FakeTapSearch("german", false, 200, "Deutsche", "Deutsche",
+                "alternate-term", "", false, 0, 0, "de", "", "", "", QuickActionCategory.NONE));
 
         registerFakeSlowResolveSearch(new FakeSlowResolveSearch(
-                "search", false, 200, "Search", "Search", "alternate-term", false, 0, 0, ""));
+                "search", false, 200, "Search", "Search", "alternate-term", "", false, 0, 0, "",
+                "", "", "", QuickActionCategory.NONE));
     }
 
     /**
