@@ -12,19 +12,26 @@ import android.content.Intent;
 import android.net.Uri;
 import android.os.Build;
 import android.os.Bundle;
-import android.os.IBinder;
 import android.os.Process;
 import android.support.customtabs.CustomTabsService;
-import android.support.customtabs.ICustomTabsCallback;
+import android.support.customtabs.CustomTabsSessionToken;
 import android.test.InstrumentationTestCase;
 import android.test.suitebuilder.annotation.SmallTest;
 
+import org.chromium.base.PathUtils;
 import org.chromium.base.ThreadUtils;
+import org.chromium.base.library_loader.LibraryLoader;
+import org.chromium.base.library_loader.LibraryProcessType;
 import org.chromium.base.test.util.Restriction;
+import org.chromium.base.test.util.RetryOnFailure;
+import org.chromium.chrome.browser.WarmupManager;
+import org.chromium.chrome.browser.preferences.PrefServiceBridge;
 import org.chromium.content_public.browser.WebContents;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.concurrent.Callable;
+import java.util.concurrent.FutureTask;
 
 /** Tests for CustomTabsConnection. */
 public class CustomTabsConnectionTest extends InstrumentationTestCase {
@@ -32,20 +39,30 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
     private static final String URL = "http://www.google.com";
     private static final String URL2 = "https://www.android.com";
     private static final String INVALID_SCHEME_URL = "intent://www.google.com";
+    private static final String PRIVATE_DATA_DIRECTORY_SUFFIX = "chrome";
 
-    private Context mContext;
+    private Context mAppContext;
 
     @Override
     protected void setUp() throws Exception {
         super.setUp();
-        mContext = getInstrumentation().getTargetContext().getApplicationContext();
-        mCustomTabsConnection = CustomTabsTestUtils.setUpConnection((Application) mContext);
+        mAppContext = getInstrumentation().getTargetContext().getApplicationContext();
+        PathUtils.setPrivateDataDirectorySuffix(PRIVATE_DATA_DIRECTORY_SUFFIX);
+        LibraryLoader.get(LibraryProcessType.PROCESS_BROWSER).ensureInitialized();
+        mCustomTabsConnection = CustomTabsTestUtils.setUpConnection((Application) mAppContext);
+        mCustomTabsConnection.resetThrottling(mAppContext, Process.myUid());
     }
 
     @Override
     protected void tearDown() throws Exception {
         super.tearDown();
         CustomTabsTestUtils.cleanupSessions(mCustomTabsConnection);
+        ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+            @Override
+            public void run() {
+                WarmupManager.getInstance().destroySpareWebContents();
+            }
+        });
     }
 
     /**
@@ -55,9 +72,9 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
     @SmallTest
     public void testNewSession() {
         assertEquals(false, mCustomTabsConnection.newSession(null));
-        ICustomTabsCallback cb = new CustomTabsTestUtils.DummyCallback();
-        assertEquals(true, mCustomTabsConnection.newSession(cb));
-        assertEquals(false, mCustomTabsConnection.newSession(cb));
+        CustomTabsSessionToken token = CustomTabsSessionToken.createDummySessionTokenForTesting();
+        assertEquals(true, mCustomTabsConnection.newSession(token));
+        assertEquals(false, mCustomTabsConnection.newSession(token));
     }
 
     /**
@@ -65,10 +82,10 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
      */
     @SmallTest
     public void testSeveralSessions() {
-        ICustomTabsCallback cb = new CustomTabsTestUtils.DummyCallback();
-        assertEquals(true, mCustomTabsConnection.newSession(cb));
-        ICustomTabsCallback cb2 = new CustomTabsTestUtils.DummyCallback();
-        assertEquals(true, mCustomTabsConnection.newSession(cb2));
+        CustomTabsSessionToken token = CustomTabsSessionToken.createDummySessionTokenForTesting();
+        assertEquals(true, mCustomTabsConnection.newSession(token));
+        CustomTabsSessionToken token2 = CustomTabsSessionToken.createDummySessionTokenForTesting();
+        assertEquals(true, mCustomTabsConnection.newSession(token2));
     }
 
     /**
@@ -91,8 +108,12 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
-                assertNotNull(mCustomTabsConnection.takeSpareWebContents());
-                assertNull(mCustomTabsConnection.takeSpareWebContents());
+                WarmupManager warmupManager = WarmupManager.getInstance();
+                assertTrue(warmupManager.hasSpareWebContents());
+                WebContents webContents = warmupManager.takeSpareWebContents(false, false);
+                assertNotNull(webContents);
+                assertFalse(warmupManager.hasSpareWebContents());
+                webContents.destroy();
             }
         });
     }
@@ -105,7 +126,7 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
             @Override
             public void run() {
                 assertSpareWebContentsNotNullAndDestroy();
-                assertNull(mCustomTabsConnection.takeSpareWebContents());
+                assertFalse(WarmupManager.getInstance().hasSpareWebContents());
             }
         });
         assertTrue(mCustomTabsConnection.warmup(0));
@@ -120,31 +141,57 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
     @SmallTest
     @Restriction(RESTRICTION_TYPE_NON_LOW_END_DEVICE)
     public void testPrerenderDestroysSpareRenderer() {
-        final ICustomTabsCallback cb = assertWarmupAndMayLaunchUrl(null, URL, true);
+        final CustomTabsSessionToken token = assertWarmupAndMayLaunchUrl(null, URL, true);
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
-                assertNull(mCustomTabsConnection.takeSpareWebContents());
+                assertFalse(WarmupManager.getInstance().hasSpareWebContents());
                 String referrer =
-                        mCustomTabsConnection.getReferrerForSession(cb.asBinder()).getUrl();
+                        mCustomTabsConnection.getReferrerForSession(token).getUrl();
                 WebContents webContents =
-                        mCustomTabsConnection.takePrerenderedUrl(cb.asBinder(), URL, referrer);
+                        mCustomTabsConnection.takePrerenderedUrl(token, URL, referrer);
                 assertNotNull(webContents);
                 webContents.destroy();
             }
         });
     }
 
+    /*
+     * Tests that when the disconnection notification comes from a non-UI thread, Chrome doesn't
+     * crash. Non-regression test for crbug.com/623128.
+     */
+    @SmallTest
+    @Restriction(RESTRICTION_TYPE_NON_LOW_END_DEVICE)
+    public void testPrerenderAndDisconnectOnOtherThread() {
+        final CustomTabsSessionToken token = assertWarmupAndMayLaunchUrl(null, URL, true);
+        final Thread otherThread = new Thread(new Runnable() {
+            @Override
+            public void run() {
+                mCustomTabsConnection.cleanUpSession(token);
+            }
+        });
+
+        ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+            @Override
+            public void run() {
+                otherThread.start();
+            }
+        });
+        // Should not crash, hence no assertions below.
+    }
+
     @SmallTest
     @Restriction(RESTRICTION_TYPE_NON_LOW_END_DEVICE)
     public void testMayLaunchUrlKeepsSpareRendererWithoutPrerendering() {
         assertTrue(mCustomTabsConnection.warmup(0));
-        final ICustomTabsCallback cb = new CustomTabsTestUtils.DummyCallback();
-        assertTrue(mCustomTabsConnection.newSession(cb));
+        final CustomTabsSessionToken token =
+                CustomTabsSessionToken.createDummySessionTokenForTesting();
+        assertTrue(mCustomTabsConnection.newSession(token));
 
         Bundle extras = new Bundle();
-        extras.putBoolean(CustomTabsConnection.NO_PRERENDERING_KEY, true);
-        assertTrue(mCustomTabsConnection.mayLaunchUrl(cb, Uri.parse(URL), extras, null));
+        extras.putInt(
+                CustomTabsConnection.DEBUG_OVERRIDE_KEY, CustomTabsConnection.NO_PRERENDERING);
+        assertTrue(mCustomTabsConnection.mayLaunchUrl(token, Uri.parse(URL), extras, null));
 
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
@@ -164,29 +211,30 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
     @SmallTest
     @Restriction(RESTRICTION_TYPE_NON_LOW_END_DEVICE)
     public void testUnderstandsLowConfidenceMayLaunchUrl() {
-        final ICustomTabsCallback cb = new CustomTabsTestUtils.DummyCallback();
-        assertTrue(mCustomTabsConnection.newSession(cb));
+        final CustomTabsSessionToken token =
+                CustomTabsSessionToken.createDummySessionTokenForTesting();
+        assertTrue(mCustomTabsConnection.newSession(token));
         List<Bundle> urls = new ArrayList<>();
         Bundle urlBundle = new Bundle();
         urlBundle.putParcelable(CustomTabsService.KEY_URL, Uri.parse(URL));
         urls.add(urlBundle);
-        mCustomTabsConnection.mayLaunchUrl(cb, null, null, urls);
+        mCustomTabsConnection.mayLaunchUrl(token, null, null, urls);
 
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
                 assertSpareWebContentsNotNullAndDestroy();
-                IBinder session = cb.asBinder();
-                String referrer = mCustomTabsConnection.getReferrerForSession(session).getUrl();
-                assertNull(mCustomTabsConnection.takePrerenderedUrl(session, URL, referrer));
+                String referrer = mCustomTabsConnection.getReferrerForSession(token).getUrl();
+                assertNull(mCustomTabsConnection.takePrerenderedUrl(token, URL, referrer));
             }
         });
     }
 
     @SmallTest
     public void testLowConfidenceMayLaunchUrlOnlyAcceptUris() {
-        final ICustomTabsCallback cb = new CustomTabsTestUtils.DummyCallback();
-        assertTrue(mCustomTabsConnection.newSession(cb));
+        final CustomTabsSessionToken token =
+                CustomTabsSessionToken.createDummySessionTokenForTesting();
+        assertTrue(mCustomTabsConnection.newSession(token));
         assertTrue(mCustomTabsConnection.warmup(0));
 
         final List<Bundle> urlsAsString = new ArrayList<>();
@@ -210,8 +258,9 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
 
     @SmallTest
     public void testLowConfidenceMayLaunchUrlDoesntCrash() {
-        final ICustomTabsCallback cb = new CustomTabsTestUtils.DummyCallback();
-        assertTrue(mCustomTabsConnection.newSession(cb));
+        final CustomTabsSessionToken token =
+                CustomTabsSessionToken.createDummySessionTokenForTesting();
+        assertTrue(mCustomTabsConnection.newSession(token));
         assertTrue(mCustomTabsConnection.warmup(0));
 
         final List<Bundle> invalidBundles = new ArrayList<>();
@@ -234,49 +283,69 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
     @SmallTest
     @Restriction(RESTRICTION_TYPE_NON_LOW_END_DEVICE)
     public void testStillHighConfidenceMayLaunchUrlWithSeveralUrls() {
-        final ICustomTabsCallback cb = new CustomTabsTestUtils.DummyCallback();
-        assertTrue(mCustomTabsConnection.newSession(cb));
+        final CustomTabsSessionToken token =
+                CustomTabsSessionToken.createDummySessionTokenForTesting();
+        assertTrue(mCustomTabsConnection.newSession(token));
         List<Bundle> urls = new ArrayList<>();
         Bundle urlBundle = new Bundle();
         urlBundle.putParcelable(CustomTabsService.KEY_URL, Uri.parse(URL));
         urls.add(urlBundle);
 
-        mCustomTabsConnection.mayLaunchUrl(cb, Uri.parse(URL), null, urls);
+        mCustomTabsConnection.mayLaunchUrl(token, Uri.parse(URL), null, urls);
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
-                assertNull(mCustomTabsConnection.takeSpareWebContents());
-                IBinder session = cb.asBinder();
-                String referrer = mCustomTabsConnection.getReferrerForSession(session).getUrl();
-                assertNotNull(mCustomTabsConnection.takePrerenderedUrl(session, URL, referrer));
+                assertNull(WarmupManager.getInstance().takeSpareWebContents(false, false));
+                String referrer = mCustomTabsConnection.getReferrerForSession(token).getUrl();
+                assertNotNull(mCustomTabsConnection.takePrerenderedUrl(token, URL, referrer));
             }
         });
     }
 
     @SmallTest
     @Restriction(RESTRICTION_TYPE_NON_LOW_END_DEVICE)
-    public void testCanCancelPrerender() {
-        final ICustomTabsCallback cb = assertWarmupAndMayLaunchUrl(null, URL, true);
+    public void testPrefetchOnlyNoPrerenderHasSpareWebContents() {
+        assertTrue(mCustomTabsConnection.warmup(0));
+        final CustomTabsSessionToken token =
+                CustomTabsSessionToken.createDummySessionTokenForTesting();
+        assertTrue(mCustomTabsConnection.newSession(token));
+
+        Bundle extras = new Bundle();
+        extras.putInt(CustomTabsConnection.DEBUG_OVERRIDE_KEY, CustomTabsConnection.PREFETCH_ONLY);
+        assertTrue(mCustomTabsConnection.mayLaunchUrl(token, Uri.parse(URL), extras, null));
+
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
-                assertNull(mCustomTabsConnection.takeSpareWebContents());
+                assertSpareWebContentsNotNullAndDestroy();
             }
         });
-        assertTrue(mCustomTabsConnection.mayLaunchUrl(cb, null, null, null));
+    }
+
+    @SmallTest
+    @Restriction(RESTRICTION_TYPE_NON_LOW_END_DEVICE)
+    @RetryOnFailure
+    public void testCanCancelPrerender() {
+        final CustomTabsSessionToken token = assertWarmupAndMayLaunchUrl(null, URL, true);
+        ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+            @Override
+            public void run() {
+                assertNull(WarmupManager.getInstance().takeSpareWebContents(false, false));
+            }
+        });
+        assertTrue(mCustomTabsConnection.mayLaunchUrl(token, null, null, null));
         // mayLaunchUrl() posts a task, the following has to run after it.
         ThreadUtils.runOnUiThreadBlocking(new Runnable() {
             @Override
             public void run() {
-                IBinder session = cb.asBinder();
-                String referrer = mCustomTabsConnection.getReferrerForSession(session).getUrl();
-                assertNull(mCustomTabsConnection.takePrerenderedUrl(session, URL, referrer));
+                String referrer = mCustomTabsConnection.getReferrerForSession(token).getUrl();
+                assertNull(mCustomTabsConnection.takePrerenderedUrl(token, URL, referrer));
             }
         });
     }
 
     private void assertSpareWebContentsNotNullAndDestroy() {
-        WebContents webContents = mCustomTabsConnection.takeSpareWebContents();
+        WebContents webContents = WarmupManager.getInstance().takeSpareWebContents(false, false);
         assertNotNull(webContents);
         webContents.destroy();
     }
@@ -285,33 +354,34 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
      * Calls warmup() and mayLaunchUrl(), checks for the expected result
      * (success or failure) and returns the result code.
      */
-    private ICustomTabsCallback assertWarmupAndMayLaunchUrl(
-            ICustomTabsCallback cb, String url, boolean shouldSucceed) {
+    private CustomTabsSessionToken assertWarmupAndMayLaunchUrl(
+            CustomTabsSessionToken token, String url, boolean shouldSucceed) {
         mCustomTabsConnection.warmup(0);
-        if (cb == null) {
-            cb = new CustomTabsTestUtils.DummyCallback();
-            mCustomTabsConnection.newSession(cb);
+        if (token == null) {
+            token = CustomTabsSessionToken.createDummySessionTokenForTesting();
+            mCustomTabsConnection.newSession(token);
         }
         Uri uri = url == null ? null : Uri.parse(url);
-        boolean succeeded = mCustomTabsConnection.mayLaunchUrl(cb, uri, null, null);
+        boolean succeeded = mCustomTabsConnection.mayLaunchUrl(token, uri, null, null);
         assertEquals(shouldSucceed, succeeded);
-        return shouldSucceed ? cb : null;
+        return shouldSucceed ? token : null;
     }
 
     /**
      * Tests that
      * {@link CustomTabsConnection#mayLaunchUrl(
-     * ICustomTabsCallback, Uri, android.os.Bundle, java.util.List)}
+     * CustomTabsSessionToken, Uri, android.os.Bundle, java.util.List)}
      * returns an error when called with an invalid session ID.
      */
     @SmallTest
     public void testNoMayLaunchUrlWithInvalidSessionId() {
-        assertWarmupAndMayLaunchUrl(new CustomTabsTestUtils.DummyCallback(), URL, false);
+        assertWarmupAndMayLaunchUrl(
+                CustomTabsSessionToken.createDummySessionTokenForTesting(), URL, false);
     }
 
     /**
      * Tests that
-     * {@link CustomTabsConnection#mayLaunchUrl(long, String, Bundle, List<Bundle>)}
+     * {@link CustomTabsConnection#mayLaunchUrl(CustomTabsSessionToken, Uri, Bundle, List)}
      * rejects invalid URL schemes.
      */
     @SmallTest
@@ -321,7 +391,7 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
 
     /**
      * Tests that
-     * {@link CustomTabsConnection#mayLaunchUrl(long, String, Bundle, List<Bundle>)}
+     * {@link CustomTabsConnection#mayLaunchUrl(CustomTabsSessionToken, Uri, Bundle, List)}
      * succeeds.
      */
     @SmallTest
@@ -331,16 +401,16 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
 
     /**
      * Tests that
-     * {@link CustomTabsConnection#mayLaunchUrl(long, String, Bundle, List<Bundle>)}
+     * {@link CustomTabsConnection#mayLaunchUrl(CustomTabsSessionToken, Uri, Bundle, List)}
      * can be called several times with the same, and different URLs.
      */
     @SmallTest
     public void testMultipleMayLaunchUrl() {
-        ICustomTabsCallback cb = assertWarmupAndMayLaunchUrl(null, URL, true);
-        mCustomTabsConnection.resetThrottling(mContext, Process.myUid());
-        assertWarmupAndMayLaunchUrl(cb, URL, true);
-        mCustomTabsConnection.resetThrottling(mContext, Process.myUid());
-        assertWarmupAndMayLaunchUrl(cb, URL2, true);
+        CustomTabsSessionToken token = assertWarmupAndMayLaunchUrl(null, URL, true);
+        mCustomTabsConnection.resetThrottling(mAppContext, Process.myUid());
+        assertWarmupAndMayLaunchUrl(token, URL, true);
+        mCustomTabsConnection.resetThrottling(mAppContext, Process.myUid());
+        assertWarmupAndMayLaunchUrl(token, URL2, true);
     }
 
     /**
@@ -348,9 +418,9 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
      */
     @SmallTest
     public void testForgetsSession() {
-        ICustomTabsCallback cb = assertWarmupAndMayLaunchUrl(null, URL, true);
+        CustomTabsSessionToken token = assertWarmupAndMayLaunchUrl(null, URL, true);
         CustomTabsTestUtils.cleanupSessions(mCustomTabsConnection);
-        assertWarmupAndMayLaunchUrl(cb, URL, false);
+        assertWarmupAndMayLaunchUrl(token, URL, false);
     }
 
     /**
@@ -395,11 +465,11 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
      */
     @SmallTest
     public void testThrottleMayLaunchUrl() {
-        ICustomTabsCallback cb = assertWarmupAndMayLaunchUrl(null, URL, true);
+        CustomTabsSessionToken token = assertWarmupAndMayLaunchUrl(null, URL, true);
         int successfulRequests = 0;
         // Send a burst of requests instead of checking for precise delays to avoid flakiness.
         while (successfulRequests < 10) {
-            if (!mCustomTabsConnection.mayLaunchUrl(cb, Uri.parse(URL), null, null)) break;
+            if (!mCustomTabsConnection.mayLaunchUrl(token, Uri.parse(URL), null, null)) break;
             successfulRequests++;
         }
         assertTrue("10 requests in a row should not all succeed.", successfulRequests < 10);
@@ -410,10 +480,10 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
      */
     @SmallTest
     public void testThrottlingIsReset() {
-        ICustomTabsCallback cb = assertWarmupAndMayLaunchUrl(null, URL, true);
-        mCustomTabsConnection.mayLaunchUrl(cb, Uri.parse(URL), null, null);
+        CustomTabsSessionToken token = assertWarmupAndMayLaunchUrl(null, URL, true);
+        mCustomTabsConnection.mayLaunchUrl(token, Uri.parse(URL), null, null);
         // Depending on the timing, the delay should be 100 or 200ms here.
-        assertWarmupAndMayLaunchUrl(cb, URL, false);
+        assertWarmupAndMayLaunchUrl(token, URL, false);
         // Wait for more than 2 * MAX_POSSIBLE_DELAY to clear the delay
         try {
             Thread.sleep(450); // 2 * MAX_POSSIBLE_DELAY + 50ms
@@ -421,7 +491,7 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
             fail();
             return;
         }
-        assertWarmupAndMayLaunchUrl(cb, URL, true);
+        assertWarmupAndMayLaunchUrl(token, URL, true);
         // Check that the delay has been reset, by waiting for 100ms.
         try {
             Thread.sleep(150); // MIN_DELAY + 50ms margin
@@ -429,7 +499,7 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
             fail();
             return;
         }
-        assertWarmupAndMayLaunchUrl(cb, URL, true);
+        assertWarmupAndMayLaunchUrl(token, URL, true);
     }
 
     /**
@@ -437,13 +507,92 @@ public class CustomTabsConnectionTest extends InstrumentationTestCase {
      */
     @SmallTest
     public void testThrottlingAcrossSessions() {
-        ICustomTabsCallback cb = assertWarmupAndMayLaunchUrl(null, URL, true);
-        mCustomTabsConnection.resetThrottling(mContext, Process.myUid());
-        ICustomTabsCallback cb2 = assertWarmupAndMayLaunchUrl(null, URL, true);
-        mCustomTabsConnection.resetThrottling(mContext, Process.myUid());
+        CustomTabsSessionToken token = assertWarmupAndMayLaunchUrl(null, URL, true);
+        mCustomTabsConnection.resetThrottling(mAppContext, Process.myUid());
+        CustomTabsSessionToken token2 = assertWarmupAndMayLaunchUrl(null, URL, true);
+        mCustomTabsConnection.resetThrottling(mAppContext, Process.myUid());
         for (int i = 0; i < 10; i++) {
-            mCustomTabsConnection.mayLaunchUrl(cb, Uri.parse(URL), null, null);
+            mCustomTabsConnection.mayLaunchUrl(token, Uri.parse(URL), null, null);
+        }
+        assertWarmupAndMayLaunchUrl(token2, URL, false);
+    }
+
+    @SmallTest
+    @Restriction(RESTRICTION_TYPE_NON_LOW_END_DEVICE)
+    public void testBanningWorks() {
+        mCustomTabsConnection.ban(mAppContext, Process.myUid());
+        final CustomTabsSessionToken token =
+                CustomTabsSessionToken.createDummySessionTokenForTesting();
+        assertTrue(mCustomTabsConnection.newSession(token));
+
+        assertTrue(mCustomTabsConnection.mayLaunchUrl(token, Uri.parse(URL), null, null));
+        ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+            @Override
+            public void run() {
+                assertSpareWebContentsNotNullAndDestroy();
+                String referrer = mCustomTabsConnection.getReferrerForSession(token).getUrl();
+                assertNull(mCustomTabsConnection.takePrerenderedUrl(token, URL, referrer));
+            }
+        });
+    }
+
+    @SmallTest
+    @Restriction(RESTRICTION_TYPE_NON_LOW_END_DEVICE)
+    public void testBanningDisabledForCellular() {
+        mCustomTabsConnection.ban(mAppContext, Process.myUid());
+        final CustomTabsSessionToken token =
+                CustomTabsSessionToken.createDummySessionTokenForTesting();
+        assertTrue(mCustomTabsConnection.newSession(token));
+        mCustomTabsConnection.setShouldPrerenderOnCellularForSession(token, true);
+
+        assertTrue(mCustomTabsConnection.mayLaunchUrl(token, Uri.parse(URL), null, null));
+        ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+            @Override
+            public void run() {
+                assertNull(WarmupManager.getInstance().takeSpareWebContents(false, false));
+                String referrer = mCustomTabsConnection.getReferrerForSession(token).getUrl();
+                WebContents prerender = mCustomTabsConnection.takePrerenderedUrl(
+                        token, URL, referrer);
+                assertNotNull(prerender);
+                prerender.destroy();
+            }
+        });
+    }
+
+    @SmallTest
+    @Restriction(RESTRICTION_TYPE_NON_LOW_END_DEVICE)
+    public void testCellularPrerenderingDoesntOverrideSettings() throws Exception {
+        CustomTabsSessionToken token = CustomTabsSessionToken.createDummySessionTokenForTesting();
+        assertTrue(mCustomTabsConnection.newSession(token));
+        mCustomTabsConnection.setShouldPrerenderOnCellularForSession(token, true);
+        mCustomTabsConnection.warmup(0);
+
+        // Needs the browser process to be initialized.
+        FutureTask<Boolean> result = ThreadUtils.runOnUiThread(new Callable<Boolean>() {
+            @Override
+            public Boolean call() {
+                PrefServiceBridge prefs = PrefServiceBridge.getInstance();
+                boolean result = prefs.getNetworkPredictionEnabled();
+                prefs.setNetworkPredictionEnabled(false);
+                return result;
+            }});
+        final boolean enabled = result.get();
+
+        try {
+            assertTrue(mCustomTabsConnection.mayLaunchUrl(token, Uri.parse(URL), null, null));
+            ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+                @Override
+                public void run() {
+                    assertSpareWebContentsNotNullAndDestroy();
+                }
+            });
+        } finally {
+            ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+                @Override
+                public void run() {
+                    PrefServiceBridge.getInstance().setNetworkPredictionEnabled(enabled);
+                }
+            });
         }
-        assertWarmupAndMayLaunchUrl(cb2, URL, false);
     }
 }
