@@ -4,6 +4,8 @@
 
 package org.chromium.chrome.browser.ntp;
 
+import android.graphics.Canvas;
+import android.test.UiThreadTest;
 import android.test.suitebuilder.annotation.LargeTest;
 import android.test.suitebuilder.annotation.MediumTest;
 import android.test.suitebuilder.annotation.SmallTest;
@@ -12,10 +14,15 @@ import android.view.View;
 import android.view.ViewGroup;
 
 import org.chromium.base.ThreadUtils;
+import org.chromium.base.test.util.CallbackHelper;
+import org.chromium.base.test.util.CommandLineFlags;
+import org.chromium.base.test.util.DisableIf;
 import org.chromium.base.test.util.DisabledTest;
 import org.chromium.base.test.util.Feature;
+import org.chromium.base.test.util.RetryOnFailure;
 import org.chromium.chrome.R;
 import org.chromium.chrome.browser.UrlConstants;
+import org.chromium.chrome.browser.ntp.cards.NewTabPageRecyclerView;
 import org.chromium.chrome.browser.omnibox.LocationBarLayout;
 import org.chromium.chrome.browser.omnibox.UrlBar;
 import org.chromium.chrome.browser.tab.EmptyTabObserver;
@@ -24,16 +31,17 @@ import org.chromium.chrome.test.ChromeTabbedActivityTestBase;
 import org.chromium.chrome.test.util.ChromeTabUtils;
 import org.chromium.chrome.test.util.NewTabPageTestUtils;
 import org.chromium.chrome.test.util.OmniboxTestUtils;
-import org.chromium.chrome.test.util.TestHttpServerClient;
-import org.chromium.content.browser.test.util.CallbackHelper;
+import org.chromium.chrome.test.util.RenderUtils.ViewRenderer;
 import org.chromium.content.browser.test.util.Criteria;
 import org.chromium.content.browser.test.util.CriteriaHelper;
 import org.chromium.content.browser.test.util.KeyUtils;
 import org.chromium.content.browser.test.util.TestTouchUtils;
 import org.chromium.content_public.browser.LoadUrlParams;
+import org.chromium.net.test.EmbeddedTestServer;
 import org.chromium.net.test.util.TestWebServer;
 import org.chromium.ui.base.PageTransition;
 
+import java.io.IOException;
 import java.util.ArrayList;
 import java.util.concurrent.Callable;
 import java.util.concurrent.Semaphore;
@@ -42,19 +50,35 @@ import java.util.concurrent.TimeUnit;
 /**
  * Tests for the native android New Tab Page.
  */
+@RetryOnFailure
 public class NewTabPageTest extends ChromeTabbedActivityTestBase {
 
-    private static final String TEST_PAGE =
-            TestHttpServerClient.getUrl("chrome/test/data/android/navigate/simple.html");
+    private static final String TEST_PAGE = "/chrome/test/data/android/navigate/simple.html";
 
     private static final String[] FAKE_MOST_VISITED_TITLES = new String[] { "Simple" };
-    private static final String[] FAKE_MOST_VISITED_URLS = new String[] { TEST_PAGE };
+    private static final String[] FAKE_MOST_VISITED_WHITELIST_ICON_PATHS = new String[] { "" };
+    private static final int[] FAKE_MOST_VISITED_SOURCES = new int[] {NTPTileSource.TOP_SITES};
 
     private Tab mTab;
     private NewTabPage mNtp;
     private View mFakebox;
     private ViewGroup mMostVisitedLayout;
+    private String[] mFakeMostVisitedUrls;
     private FakeMostVisitedSites mFakeMostVisitedSites;
+    private EmbeddedTestServer mTestServer;
+
+    @Override
+    protected void setUp() throws Exception {
+        mTestServer = EmbeddedTestServer.createAndStartServer(getInstrumentation().getContext());
+        mFakeMostVisitedUrls = new String[] {mTestServer.getURL(TEST_PAGE)};
+        super.setUp();
+    }
+
+    @Override
+    protected void tearDown() throws Exception {
+        mTestServer.stopAndDestroyServer();
+        super.tearDown();
+    }
 
     @Override
     public void startMainActivity() throws InterruptedException {
@@ -68,7 +92,8 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
                     // Create FakeMostVisitedSites after starting the activity, since it depends on
                     // native code.
                     mFakeMostVisitedSites = new FakeMostVisitedSites(mTab.getProfile(),
-                            FAKE_MOST_VISITED_TITLES, FAKE_MOST_VISITED_URLS);
+                            FAKE_MOST_VISITED_TITLES, mFakeMostVisitedUrls,
+                            FAKE_MOST_VISITED_WHITELIST_ICON_PATHS, FAKE_MOST_VISITED_SOURCES);
                 }
             });
         } catch (Throwable t) {
@@ -83,7 +108,75 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
         mNtp = (NewTabPage) mTab.getNativePage();
         mFakebox = mNtp.getView().findViewById(R.id.search_box);
         mMostVisitedLayout = (ViewGroup) mNtp.getView().findViewById(R.id.most_visited_layout);
-        assertEquals(FAKE_MOST_VISITED_URLS.length, mMostVisitedLayout.getChildCount());
+        assertEquals(mFakeMostVisitedUrls.length, mMostVisitedLayout.getChildCount());
+    }
+
+    @MediumTest
+    @Feature({"NewTabPage", "RenderTest"})
+    @CommandLineFlags.Add("enable-features=NTPSnippets")
+    public void testRender() throws IOException, InterruptedException {
+        ViewRenderer viewRenderer = new ViewRenderer(getActivity(),
+                "chrome/test/data/android/render_tests", "NewTabPageTest");
+        viewRenderer.renderAndCompare(mMostVisitedLayout, "most_visited");
+        viewRenderer.renderAndCompare(mFakebox, "fakebox");
+        viewRenderer.renderAndCompare(mNtp.getView().getRootView(), "new_tab_page");
+
+        // Scroll to search bar
+        final NewTabPageRecyclerView recyclerView =
+                (NewTabPageRecyclerView) mNtp.getNewTabPageView().getWrapperView();
+
+        ThreadUtils.runOnUiThreadBlocking(new Runnable() {
+            @Override
+            public void run() {
+                recyclerView.smoothScrollBy(0, mFakebox.getTop());
+            }
+        });
+
+        CriteriaHelper.pollUiThread(new Criteria(){
+            @Override
+            public boolean isSatisfied() {
+                return recyclerView.computeVerticalScrollOffset() == mFakebox.getTop();
+            }
+        });
+
+        viewRenderer.renderAndCompare(mNtp.getView().getRootView(), "new_tab_page_scrolled");
+    }
+
+    @MediumTest
+    @Feature({"NewTabPage"})
+    @CommandLineFlags.Add("enable-features=NTPSnippets")
+    @UiThreadTest
+    public void testThumbnailInvalidations() {
+        captureThumbnail();
+        assertFalse(mNtp.shouldCaptureThumbnail());
+
+        // Check that we invalidate the thumbnail when the Recycler View is updated.
+        NewTabPageRecyclerView recyclerView =
+                (NewTabPageRecyclerView) mNtp.getNewTabPageView().getWrapperView();
+
+        recyclerView.getAdapter().notifyDataSetChanged();
+        assertThumbnailInvalidAndRecapture();
+
+        recyclerView.getAdapter().notifyItemChanged(0);
+        assertThumbnailInvalidAndRecapture();
+
+        recyclerView.getAdapter().notifyItemInserted(0);
+        assertThumbnailInvalidAndRecapture();
+
+        recyclerView.getAdapter().notifyItemMoved(0, 1);
+        assertThumbnailInvalidAndRecapture();
+
+        recyclerView.getAdapter().notifyItemRangeChanged(0, 1);
+        assertThumbnailInvalidAndRecapture();
+
+        recyclerView.getAdapter().notifyItemRangeInserted(0, 1);
+        assertThumbnailInvalidAndRecapture();
+
+        recyclerView.getAdapter().notifyItemRangeRemoved(0, 1);
+        assertThumbnailInvalidAndRecapture();
+
+        recyclerView.getAdapter().notifyItemRemoved(0);
+        assertThumbnailInvalidAndRecapture();
     }
 
     /**
@@ -113,6 +206,7 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
      */
     @SmallTest
     @Feature({"NewTabPage"})
+    @DisableIf.Build(sdk_is_greater_than = 22, message = "crbug.com/593007")
     public void testSearchFromFakebox() throws InterruptedException {
         singleClickView(mFakebox);
         waitForFakeboxFocusAnimationComplete(mNtp);
@@ -145,7 +239,7 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
                 singleClickView(mostVisitedItem);
             }
         });
-        assertEquals(FAKE_MOST_VISITED_URLS[0], mTab.getUrl());
+        assertEquals(mFakeMostVisitedUrls[0], mTab.getUrl());
     }
 
     /**
@@ -156,7 +250,7 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
     @Feature({"NewTabPage"})
     public void testOpenMostVisitedItemInNewTab() throws InterruptedException {
         invokeContextMenuAndOpenInANewTab(mMostVisitedLayout.getChildAt(0),
-                NewTabPage.ID_OPEN_IN_NEW_TAB, false, FAKE_MOST_VISITED_URLS[0]);
+                ContextMenuManager.ID_OPEN_IN_NEW_TAB, false, mFakeMostVisitedUrls[0]);
     }
 
     /**
@@ -166,7 +260,7 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
     @Feature({"NewTabPage"})
     public void testOpenMostVisitedItemInIncognitoTab() throws InterruptedException {
         invokeContextMenuAndOpenInANewTab(mMostVisitedLayout.getChildAt(0),
-                NewTabPage.ID_OPEN_IN_INCOGNITO_TAB, true, FAKE_MOST_VISITED_URLS[0]);
+                ContextMenuManager.ID_OPEN_IN_INCOGNITO_TAB, true, mFakeMostVisitedUrls[0]);
     }
 
     /**
@@ -176,16 +270,16 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
     @Feature({"NewTabPage"})
     public void testRemoveMostVisitedItem() {
         View mostVisitedItem = mMostVisitedLayout.getChildAt(0);
-        ArrayList<View> views = new ArrayList<View>();
+        ArrayList<View> views = new ArrayList<>();
         mMostVisitedLayout.findViewsWithText(views, FAKE_MOST_VISITED_TITLES[0],
                 View.FIND_VIEWS_WITH_TEXT);
         assertEquals(1, views.size());
 
         TestTouchUtils.longClickView(getInstrumentation(), mostVisitedItem);
-        assertTrue(getInstrumentation().invokeContextMenuAction(getActivity(),
-                NewTabPage.ID_REMOVE, 0));
+        assertTrue(getInstrumentation().invokeContextMenuAction(
+                getActivity(), ContextMenuManager.ID_REMOVE, 0));
 
-        assertTrue(mFakeMostVisitedSites.isUrlBlacklisted(FAKE_MOST_VISITED_URLS[0]));
+        assertTrue(mFakeMostVisitedSites.isUrlBlacklisted(mFakeMostVisitedUrls[0]));
     }
 
     @MediumTest
@@ -200,7 +294,8 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
                     public void run() {
                         int pageTransition =
                                 PageTransition.TYPED | PageTransition.FROM_ADDRESS_BAR;
-                        mTab.loadUrl(new LoadUrlParams(TEST_PAGE, pageTransition));
+                        mTab.loadUrl(new LoadUrlParams(mTestServer.getURL(TEST_PAGE),
+                                pageTransition));
                         // It should be disabled as soon as a load URL is triggered.
                         assertTrue(getUrlFocusAnimatonsDisabled());
                     }
@@ -212,8 +307,9 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
     }
 
     @LargeTest
-    @Feature({"NewTagPage"})
+    @Feature({"NewTabPage"})
     public void testUrlFocusAnimationsEnabledOnFailedLoad() throws Exception {
+        // TODO(jbudorick): switch this to EmbeddedTestServer.
         TestWebServer webServer = TestWebServer.start();
         try {
             final Semaphore delaySemaphore = new Semaphore(0);
@@ -271,6 +367,34 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
         }
     }
 
+    /**
+     * Tests setting whether the search provider has a logo.
+     */
+    @SmallTest
+    @Feature({"NewTabPage"})
+    @CommandLineFlags.Add("disable-features=NTPSnippets") // Exercise the old ui.
+    @UiThreadTest
+    public void testSetSearchProviderHasLogo() {
+        NewTabPageView ntpView = mNtp.getNewTabPageView();
+        View logoView = ntpView.findViewById(R.id.search_provider_logo);
+        assertEquals(View.VISIBLE, logoView.getVisibility());
+        ntpView.setSearchProviderHasLogo(false);
+        assertEquals(View.GONE, logoView.getVisibility());
+        ntpView.setSearchProviderHasLogo(true);
+        assertEquals(View.VISIBLE, logoView.getVisibility());
+    }
+
+    private void assertThumbnailInvalidAndRecapture() {
+        assertTrue(mNtp.shouldCaptureThumbnail());
+        captureThumbnail();
+        assertFalse(mNtp.shouldCaptureThumbnail());
+    }
+
+    private void captureThumbnail() {
+        Canvas canvas = new Canvas();
+        mNtp.captureThumbnail(canvas);
+    }
+
     private boolean getUrlFocusAnimatonsDisabled() {
         return ThreadUtils.runOnUiThreadBlockingNoException(new Callable<Boolean>() {
             @Override
@@ -280,18 +404,18 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
         });
     }
 
-    private void waitForUrlFocusAnimationsDisabledState(final boolean disabled)
+    private void waitForUrlFocusAnimationsDisabledState(boolean disabled)
             throws InterruptedException {
-        CriteriaHelper.pollForCriteria(new Criteria() {
+        CriteriaHelper.pollInstrumentationThread(Criteria.equals(disabled, new Callable<Boolean>() {
             @Override
-            public boolean isSatisfied() {
-                return getUrlFocusAnimatonsDisabled() == disabled;
+            public Boolean call() {
+                return getUrlFocusAnimatonsDisabled();
             }
-        });
+        }));
     }
 
     private void waitForTabLoading() throws InterruptedException {
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 return mTab.isLoading();
@@ -303,14 +427,14 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
         waitForUrlFocusPercent(ntp, 1f);
     }
 
-    private void waitForUrlFocusPercent(final NewTabPage ntp, final float percent)
+    private void waitForUrlFocusPercent(final NewTabPage ntp, float percent)
             throws InterruptedException {
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(Criteria.equals(percent, new Callable<Float>() {
             @Override
-            public boolean isSatisfied() {
-                return ntp.getNewTabPageView().getUrlFocusChangeAnimationPercent() == percent;
+            public Float call() {
+                return ntp.getNewTabPageView().getUrlFocusChangeAnimationPercent();
             }
-        });
+        }));
     }
 
     private void clickFakebox() {
@@ -336,13 +460,13 @@ public class NewTabPageTest extends ChromeTabbedActivityTestBase {
     /**
      * Waits until the top of the fakebox reaches the given position.
      */
-    private void waitForFakeboxTopPosition(final NewTabPage ntp, final int position)
+    private void waitForFakeboxTopPosition(final NewTabPage ntp, int position)
             throws InterruptedException {
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(Criteria.equals(position, new Callable<Integer>() {
             @Override
-            public boolean isSatisfied() {
-                return getFakeboxTop(ntp) == position;
+            public Integer call() {
+                return getFakeboxTop(ntp);
             }
-        });
+        }));
     }
 }
