@@ -17,10 +17,9 @@ import android.test.suitebuilder.annotation.SmallTest;
 
 import org.chromium.base.ThreadUtils;
 import org.chromium.base.test.util.AdvancedMockContext;
-import org.chromium.base.test.util.DisabledTest;
 import org.chromium.base.test.util.Feature;
-import org.chromium.sync.signin.AccountManagerHelper;
-import org.chromium.sync.test.util.MockAccountManager;
+import org.chromium.components.signin.AccountManagerHelper;
+import org.chromium.components.signin.test.util.MockAccountManager;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -203,7 +202,11 @@ public class FeatureUtilitiesTest extends InstrumentationTestCase {
         assertFalse(RecognizesSpeechUncached);
     }
 
-    @DisabledTest // Flaked on the try bot: http://crbug.com/543160
+    // This test previously flaked on the try bot: http://crbug.com/543160.
+    // Re-enabling this test since there has been related cleanup/refactoring
+    // during the time the test was disabled. If the test starts flaking again,
+    // re-open the bug.
+    // TODO(nyquist): Remove this if the test is not flaky anymore.
     @SmallTest
     @Feature({"FeatureUtilities", "GoogleAccounts"})
     public void testHasGoogleAccountCorrectlyDetected() {
@@ -223,7 +226,11 @@ public class FeatureUtilitiesTest extends InstrumentationTestCase {
         assertTrue(hasAuthenticator);
     }
 
-    @DisabledTest // Flaked on the try bot: http://crbug.com/543160
+    // This test previously flaked on the try bot: http://crbug.com/543160.
+    // Re-enabling this test since there has been related cleanup/refactoring
+    // during the time the test was disabled. If the test starts flaking again,
+    // re-open the bug.
+    // TODO(nyquist): Remove this if the test is not flaky anymore.
     @SmallTest
     @Feature({"FeatureUtilities", "GoogleAccounts"})
     public void testHasNoGoogleAccountCorrectlyDetected() {
