@@ -8,26 +8,25 @@ import android.app.Activity;
 import android.content.Context;
 import android.content.Intent;
 import android.content.SharedPreferences;
-import android.preference.PreferenceManager;
 import android.test.suitebuilder.annotation.MediumTest;
 import android.view.View;
 
 import org.chromium.base.ApplicationStatus;
+import org.chromium.base.ContextUtils;
 import org.chromium.base.ThreadUtils;
-import org.chromium.base.test.util.CommandLineFlags;
+import org.chromium.base.test.util.Feature;
+import org.chromium.base.test.util.RetryOnFailure;
 import org.chromium.base.test.util.UrlUtils;
+import org.chromium.blink_public.platform.WebDisplayMode;
 import org.chromium.chrome.browser.ChromeActivity;
-import org.chromium.chrome.browser.ChromeSwitches;
 import org.chromium.chrome.browser.ChromeTabbedActivity;
 import org.chromium.chrome.browser.ShortcutHelper;
 import org.chromium.chrome.browser.ShortcutSource;
-import org.chromium.chrome.browser.document.DocumentActivity;
 import org.chromium.chrome.browser.tab.Tab;
 import org.chromium.chrome.browser.tab.TabIdManager;
 import org.chromium.chrome.test.MultiActivityTestBase;
 import org.chromium.chrome.test.util.ActivityUtils;
 import org.chromium.chrome.test.util.ApplicationTestUtils;
-import org.chromium.chrome.test.util.DisableInTabbedMode;
 import org.chromium.chrome.test.util.browser.TabLoadObserver;
 import org.chromium.content.browser.test.util.Criteria;
 import org.chromium.content.browser.test.util.CriteriaHelper;
@@ -44,6 +43,7 @@ import org.chromium.content_public.common.ScreenOrientationValues;
  * FLAG_ACTIVITY_NEW_DOCUMENT mechanism.  Moreover, we don't have access to the task list pre-L so
  * we have to assume that any non-running WebappActivities are not listed in Android's Overview.
  */
+@RetryOnFailure
 public class WebappModeTest extends MultiActivityTestBase {
     private static final String WEBAPP_1_ID = "webapp_id_1";
     private static final String WEBAPP_1_URL = UrlUtils.encodeHtmlDataUri(
@@ -61,8 +61,7 @@ public class WebappModeTest extends MultiActivityTestBase {
             + "IWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH3wQIFB4cxOfiSQAAABl0RVh0Q29tbWVudABDcmVhdGVkIHdpdG"
             + "ggR0lNUFeBDhcAAAAMSURBVAjXY2AUawEAALcAnI/TkI8AAAAASUVORK5CYII=";
 
-    private void fireWebappIntent(String id, String url, String title, String icon,
-            boolean addMac) throws Exception {
+    private Intent createIntent(String id, String url, String title, String icon, boolean addMac) {
         Intent intent = new Intent();
         intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
         intent.setPackage(getInstrumentation().getTargetContext().getPackageName());
@@ -74,12 +73,19 @@ public class WebappModeTest extends MultiActivityTestBase {
             intent.putExtra(ShortcutHelper.EXTRA_MAC, mac);
         }
 
-        WebappInfo webappInfo = WebappInfo.create(id, url, icon, title, null,
-                ScreenOrientationValues.PORTRAIT, ShortcutSource.UNKNOWN,
+        WebappInfo webappInfo = WebappInfo.create(id, url, null, icon, title, null,
+                WebDisplayMode.Standalone, ScreenOrientationValues.PORTRAIT, ShortcutSource.UNKNOWN,
                 ShortcutHelper.MANIFEST_COLOR_INVALID_OR_MISSING,
                 ShortcutHelper.MANIFEST_COLOR_INVALID_OR_MISSING, false);
         webappInfo.setWebappIntentExtras(intent);
 
+        return intent;
+    }
+
+    private void fireWebappIntent(String id, String url, String title, String icon,
+            boolean addMac) throws Exception {
+        Intent intent = createIntent(id, url, title, icon, addMac);
+
         getInstrumentation().getTargetContext().startActivity(intent);
         getInstrumentation().waitForIdleSync();
         ApplicationTestUtils.waitUntilChromeInForeground();
@@ -88,18 +94,34 @@ public class WebappModeTest extends MultiActivityTestBase {
     @Override
     public void setUp() throws Exception {
         super.setUp();
+        WebappRegistry.refreshSharedPrefsForTesting();
 
         // Register the webapps so when the data storage is opened, the test doesn't crash. There is
-        // no race condition with the retrival as AsyncTasks are run sequentially on the background
+        // no race condition with the retrieval as AsyncTasks are run sequentially on the background
         // thread.
-        WebappRegistry.registerWebapp(getInstrumentation().getTargetContext(), WEBAPP_1_ID);
-        WebappRegistry.registerWebapp(getInstrumentation().getTargetContext(), WEBAPP_2_ID);
+        WebappRegistry.getInstance().register(
+                WEBAPP_1_ID, new WebappRegistry.FetchWebappDataStorageCallback() {
+                    @Override
+                    public void onWebappDataStorageRetrieved(WebappDataStorage storage) {
+                        storage.updateFromShortcutIntent(createIntent(
+                                WEBAPP_1_ID, WEBAPP_1_URL, WEBAPP_1_TITLE, WEBAPP_ICON, true));
+                    }
+                });
+        WebappRegistry.getInstance().register(
+                WEBAPP_2_ID, new WebappRegistry.FetchWebappDataStorageCallback() {
+                    @Override
+                    public void onWebappDataStorageRetrieved(WebappDataStorage storage) {
+                        storage.updateFromShortcutIntent(createIntent(
+                                WEBAPP_1_ID, WEBAPP_1_URL, WEBAPP_1_TITLE, WEBAPP_ICON, true));
+                    }
+                });
     }
 
     /**
      * Tests that WebappActivities are started properly.
      */
     @MediumTest
+    @Feature({"Webapps"})
     public void testWebappLaunches() throws Exception {
         final WebappActivity firstActivity =
                 startWebappActivity(WEBAPP_1_ID, WEBAPP_1_URL, WEBAPP_1_TITLE, WEBAPP_ICON);
@@ -107,7 +129,7 @@ public class WebappModeTest extends MultiActivityTestBase {
 
         // Firing a different Intent should start a new WebappActivity instance.
         fireWebappIntent(WEBAPP_2_ID, WEBAPP_2_URL, WEBAPP_2_TITLE, WEBAPP_ICON, true);
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 Activity lastActivity = ApplicationStatus.getLastTrackedFocusedActivity();
@@ -121,7 +143,7 @@ public class WebappModeTest extends MultiActivityTestBase {
         // Firing the first Intent should bring back the first WebappActivity instance, or at least
         // a WebappActivity with the same tab if the other one was killed by Android mid-test.
         fireWebappIntent(WEBAPP_1_ID, WEBAPP_1_URL, WEBAPP_1_TITLE, WEBAPP_ICON, true);
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 Activity lastActivity = ApplicationStatus.getLastTrackedFocusedActivity();
@@ -137,9 +159,9 @@ public class WebappModeTest extends MultiActivityTestBase {
      * Tests that the WebappActivity gets the next available Tab ID instead of 0.
      */
     @MediumTest
+    @Feature({"Webapps"})
     public void testWebappTabIdsProperlyAssigned() throws Exception {
-        Context context = getInstrumentation().getTargetContext();
-        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(context);
+        SharedPreferences prefs = ContextUtils.getAppSharedPreferences();
         SharedPreferences.Editor editor = prefs.edit();
         editor.putInt(TabIdManager.PREF_NEXT_ID, 11684);
         editor.apply();
@@ -154,6 +176,7 @@ public class WebappModeTest extends MultiActivityTestBase {
      * TabOpenType.BRING_TAB_TO_FRONT.
      */
     @MediumTest
+    @Feature({"Webapps"})
     public void testBringTabToFront() throws Exception {
         // Start the WebappActivity.
         final WebappActivity firstActivity =
@@ -175,7 +198,7 @@ public class WebappModeTest extends MultiActivityTestBase {
         // it should have the same Tab ID.
         getInstrumentation().waitForIdleSync();
         ApplicationTestUtils.waitUntilChromeInForeground();
-        CriteriaHelper.pollForCriteria(new Criteria() {
+        CriteriaHelper.pollInstrumentationThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 Activity lastActivity = ApplicationStatus.getLastTrackedFocusedActivity();
@@ -191,15 +214,15 @@ public class WebappModeTest extends MultiActivityTestBase {
      * Ensure WebappActivities can't be launched without proper security checks.
      */
     @MediumTest
+    @Feature({"Webapps"})
     public void testWebappRequiresValidMac() throws Exception {
         // Try to start a WebappActivity.  Fail because the Intent is insecure.
         fireWebappIntent(WEBAPP_1_ID, WEBAPP_1_URL, WEBAPP_1_TITLE, WEBAPP_ICON, false);
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 Activity lastActivity = ApplicationStatus.getLastTrackedFocusedActivity();
-                return lastActivity instanceof ChromeTabbedActivity
-                        || lastActivity instanceof DocumentActivity;
+                return lastActivity instanceof ChromeTabbedActivity;
             }
         });
         ChromeActivity chromeActivity =
@@ -208,7 +231,7 @@ public class WebappModeTest extends MultiActivityTestBase {
 
         // Firing a correct Intent should start a WebappActivity instance instead of the browser.
         fireWebappIntent(WEBAPP_2_ID, WEBAPP_2_URL, WEBAPP_2_TITLE, WEBAPP_ICON, true);
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 return isWebappActivityReady(ApplicationStatus.getLastTrackedFocusedActivity());
@@ -216,38 +239,20 @@ public class WebappModeTest extends MultiActivityTestBase {
         });
     }
 
-    /**
-     * Tests that WebappActivities handle window.open() properly in document mode.
-     */
-    @DisableInTabbedMode
-    @MediumTest
-    public void testWebappHandlesWindowOpenInDocumentMode() throws Exception {
-        triggerWindowOpenAndWaitForLoad(DocumentActivity.class, ONCLICK_LINK, true);
-    }
-
     /**
      * Tests that WebappActivities handle window.open() properly in tabbed mode.
      */
-    @CommandLineFlags.Add(ChromeSwitches.DISABLE_DOCUMENT_MODE)
     @MediumTest
+    @Feature({"Webapps"})
     public void testWebappHandlesWindowOpenInTabbedMode() throws Exception {
         triggerWindowOpenAndWaitForLoad(ChromeTabbedActivity.class, ONCLICK_LINK, true);
     }
 
-    /**
-     * Tests that WebappActivities handle suppressed window.open() properly in document mode.
-     */
-    @DisableInTabbedMode
-    @MediumTest
-    public void testWebappHandlesSuppressedWindowOpenInDocumentMode() throws Exception {
-        triggerWindowOpenAndWaitForLoad(DocumentActivity.class, HREF_NO_REFERRER_LINK, false);
-    }
-
     /**
      * Tests that WebappActivities handle suppressed window.open() properly in tabbed mode.
      */
-    @CommandLineFlags.Add(ChromeSwitches.DISABLE_DOCUMENT_MODE)
     @MediumTest
+    @Feature({"Webapps"})
     public void testWebappHandlesSuppressedWindowOpenInTabbedMode() throws Exception {
         triggerWindowOpenAndWaitForLoad(ChromeTabbedActivity.class, HREF_NO_REFERRER_LINK, false);
     }
@@ -289,7 +294,7 @@ public class WebappModeTest extends MultiActivityTestBase {
         // Close the child window to kick the user back to the WebappActivity.
         JavaScriptUtils.executeJavaScript(
                 secondActivity.getActivityTab().getWebContents(), "window.close()");
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 Activity lastActivity = ApplicationStatus.getLastTrackedFocusedActivity();
@@ -310,7 +315,7 @@ public class WebappModeTest extends MultiActivityTestBase {
     private WebappActivity startWebappActivity(String id, String url, String title, String icon)
             throws Exception {
         fireWebappIntent(id, url, title, icon, true);
-        CriteriaHelper.pollForUIThreadCriteria(new Criteria() {
+        CriteriaHelper.pollUiThread(new Criteria() {
             @Override
             public boolean isSatisfied() {
                 Activity lastActivity = ApplicationStatus.getLastTrackedFocusedActivity();
