@@ -9,30 +9,33 @@ import android.util.Pair;
 
 import org.chromium.base.ThreadUtils;
 import org.chromium.base.test.util.Feature;
+import org.chromium.base.test.util.RetryOnFailure;
 import org.chromium.chrome.test.util.browser.sync.SyncTestUtil;
+import org.chromium.components.sync.ModelType;
+import org.chromium.components.sync.protocol.EntitySpecifics;
+import org.chromium.components.sync.protocol.SyncEnums;
+import org.chromium.components.sync.protocol.TypedUrlSpecifics;
 import org.chromium.content.browser.test.util.Criteria;
 import org.chromium.content.browser.test.util.CriteriaHelper;
 import org.chromium.content_public.browser.LoadUrlParams;
-import org.chromium.sync.ModelType;
-import org.chromium.sync.protocol.EntitySpecifics;
-import org.chromium.sync.protocol.SyncEnums;
-import org.chromium.sync.protocol.TypedUrlSpecifics;
 import org.chromium.ui.base.PageTransition;
 import org.json.JSONException;
 import org.json.JSONObject;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.concurrent.Callable;
 
 /**
  * Test suite for the typed URLs sync data type.
  */
+@RetryOnFailure  // crbug.com/637448
 public class TypedUrlsTest extends SyncTestBase {
     private static final String TAG = "TypedUrlsTest";
 
     private static final String TYPED_URLS_TYPE = "Typed URLs";
 
-    // TestHttpServerClient is preferred here but it can't be used. The test server
+    // EmbeddedTestServer is preferred here but it can't be used. The test server
     // serves pages on localhost and Chrome doesn't sync localhost URLs as typed URLs.
     // This type of URL requires no external data connection or resources.
     private static final String URL = "data:text,testTypedUrl";
@@ -51,7 +54,7 @@ public class TypedUrlsTest extends SyncTestBase {
     @Override
     protected void setUp() throws Exception {
         super.setUp();
-        setUpTestAccountAndSignInToSync();
+        setUpTestAccountAndSignIn();
         // Make sure the initial state is clean.
         assertClientTypedUrlCount(0);
         assertServerTypedUrlCountWithName(0, URL);
@@ -140,23 +143,18 @@ public class TypedUrlsTest extends SyncTestBase {
                         count, ModelType.TYPED_URLS, name));
     }
 
-    private void waitForClientTypedUrlCount(final int count) throws InterruptedException {
-        CriteriaHelper.pollForCriteria(new Criteria(
-                "Expected " + count + " local typed URL entities.") {
+    private void waitForClientTypedUrlCount(int count) throws InterruptedException {
+        CriteriaHelper.pollInstrumentationThread(Criteria.equals(count, new Callable<Integer>() {
             @Override
-            public boolean isSatisfied() {
-                try {
-                    return SyncTestUtil.getLocalData(mContext, TYPED_URLS_TYPE).size() == count;
-                } catch (Exception e) {
-                    throw new RuntimeException(e);
-                }
+            public Integer call() throws Exception {
+                return SyncTestUtil.getLocalData(mContext, TYPED_URLS_TYPE).size();
             }
-        }, SyncTestUtil.TIMEOUT_MS, SyncTestUtil.INTERVAL_MS);
+        }), SyncTestUtil.TIMEOUT_MS, SyncTestUtil.INTERVAL_MS);
     }
 
     private void waitForServerTypedUrlCountWithName(final int count, final String name)
             throws InterruptedException {
-        CriteriaHelper.pollForCriteria(new Criteria(
+        CriteriaHelper.pollInstrumentationThread(new Criteria(
                 "Expected " + count + " server typed URLs with name " + name + ".") {
             @Override
             public boolean isSatisfied() {
