@@ -7,10 +7,12 @@
 
 #include <stddef.h>
 
+#include <memory>
+#include <string>
+
 #include "base/callback.h"
 #include "base/gtest_prod_util.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/values.h"
 #include "chrome/browser/android/contextualsearch/contextual_search_context.h"
@@ -18,12 +20,17 @@
 #include "content/public/browser/android/content_view_core.h"
 #include "net/url_request/url_fetcher_delegate.h"
 
+namespace content {
+class WebContents;
+}
+
 namespace net {
 class URLRequestContextGetter;
 }  // namespace net
 
 class Profile;
 class TemplateURLService;
+class ContextualSearchFieldTrial;
 
 // Handles tasks for the ContextualSearchManager in a separable and testable
 // way, without the complication of being connected to a Java object.
@@ -31,12 +38,17 @@ class ContextualSearchDelegate
     : public net::URLFetcherDelegate,
       public base::SupportsWeakPtr<ContextualSearchDelegate> {
  public:
+  // Callback that provides the surrounding text past the selection for the UX.
   typedef base::Callback<void(const std::string&)> SurroundingTextCallback;
+  // Provides the Resolved Search Term, called when the Resolve Request returns.
   typedef base::Callback<void(const ResolvedSearchTerm&)>
       SearchTermResolutionCallback;
+  // Provides the surrounding text and selection start/end when Blink gathers
+  // surrounding text for the Context.
   typedef base::Callback<
       void(const base::string16&, int, int)>
       HandleSurroundingsCallback;
+  // Provides limited surrounding text for icing.
   typedef base::Callback<
       void(const std::string&, const base::string16&, size_t, size_t)>
       IcingCallback;
@@ -61,13 +73,13 @@ class ContextualSearchDelegate
   void StartSearchTermResolutionRequest(
       const std::string& selection,
       bool use_resolved_search_term,
-      content::ContentViewCore* content_view_core,
+      content::WebContents* web_contents,
       bool may_send_base_page_url);
 
   // Gathers surrounding text and saves it locally for a future query.
   void GatherAndSaveSurroundingText(const std::string& selection,
                                     bool use_resolved_search_term,
-                                    content::ContentViewCore* content_view_core,
+                                    content::WebContents* web_contents,
                                     bool may_send_base_page_url);
 
   // Continues making a Search Term Resolution request, once the surrounding
@@ -102,12 +114,21 @@ class ContextualSearchDelegate
                            SurroundingTextForIcingNegativeLimit);
   FRIEND_TEST_ALL_PREFIXES(ContextualSearchDelegateTest,
                            DecodeSearchTermFromJsonResponse);
+  FRIEND_TEST_ALL_PREFIXES(ContextualSearchDelegateTest, DecodeStringMentions);
 
   // net::URLFetcherDelegate:
   void OnURLFetchComplete(const net::URLFetcher* source) override;
 
-  // Builds the search term resolution request URL from the current context.
-  GURL BuildRequestUrl();
+  // Builds the ContextualSearchContext in the current context from
+  // the given parameters.
+  void BuildContext(const std::string& selection,
+                    bool use_resolved_search_term,
+                    content::WebContents* web_contents,
+                    bool may_send_base_page_url);
+
+  // Builds and returns the search term resolution request URL.
+  // |selection| is used as the default query.
+  std::string BuildRequestUrl(std::string selection);
 
   // Uses the TemplateURL service to construct a search term resolution URL from
   // the given parameters.
@@ -121,7 +142,7 @@ class ContextualSearchDelegate
   void GatherSurroundingTextWithCallback(
       const std::string& selection,
       bool use_resolved_search_term,
-      content::ContentViewCore* content_view_core,
+      content::WebContents* web_contents,
       bool may_send_base_page_url,
       HandleSurroundingsCallback callback);
 
@@ -147,6 +168,9 @@ class ContextualSearchDelegate
   void SetDiscourseContextAndAddToHeader(
       const ContextualSearchContext& context);
 
+  // Populates and returns the discourse context.
+  std::string GetDiscourseContext(const ContextualSearchContext& context);
+
   // Checks if we can send the URL for this user. Several conditions are checked
   // to make sure it's OK to send the URL.  These fall into two categories:
   // 1) check if it's allowed by our policy, and 2) ensure that the user is
@@ -155,27 +179,33 @@ class ContextualSearchDelegate
                       Profile* profile,
                       TemplateURLService* template_url_service);
 
-  // Decodes the given json response string and extracts parameters.
-  void DecodeSearchTermFromJsonResponse(const std::string& response,
-                                        std::string* search_term,
-                                        std::string* display_text,
-                                        std::string* alternate_term,
-                                        std::string* prevent_preload,
-                                        int* mention_start,
-                                        int* mention_end,
-                                        std::string* context_language);
+  // Builds a Resolved Search Term by decoding the given JSON string.
+  std::unique_ptr<ResolvedSearchTerm> GetResolvedSearchTermFromJson(
+      int response_code,
+      const std::string& json_string);
 
+  // Decodes the given json response string and extracts parameters.
+  void DecodeSearchTermFromJsonResponse(
+      const std::string& response,
+      std::string* search_term,
+      std::string* display_text,
+      std::string* alternate_term,
+      std::string* mid,
+      std::string* prevent_preload,
+      int* mention_start,
+      int* mention_end,
+      std::string* context_language,
+      std::string* thumbnail_url,
+      std::string* caption,
+      std::string* quick_action_uri,
+      QuickActionCategory* quick_action_category);
+
+  // Extracts the start and end location from a mentions list, and sets the
+  // integers referenced by |startResult| and |endResult|.
   void ExtractMentionsStartEnd(const base::ListValue& mentions_list,
                                int* startResult,
                                int* endResult);
 
-  // Returns the surrounding size to use for the search term resolution
-  // request.
-  int GetSearchTermSurroundingSize();
-
-  // Returns the size of the surroundings to be sent to Icing.
-  int GetIcingSurroundingSize();
-
   // Generates a subset of the given surrounding_text string, for Icing
   // integration.
   // |surrounding_text| the entire text context that contains the selection.
@@ -194,7 +224,7 @@ class ContextualSearchDelegate
                                          size_t* end);
 
   // The current request in progress, or NULL.
-  scoped_ptr<net::URLFetcher> search_term_fetcher_;
+  std::unique_ptr<net::URLFetcher> search_term_fetcher_;
 
   // Holds the URL request context. Not owned.
   net::URLRequestContextGetter* url_request_context_;
@@ -202,6 +232,9 @@ class ContextualSearchDelegate
   // Holds the TemplateURLService. Not owned.
   TemplateURLService* template_url_service_;
 
+  // The field trial helper instance, always set up by the constructor.
+  std::unique_ptr<ContextualSearchFieldTrial> field_trial_;
+
   // The callback for notifications of completed URL fetches.
   SearchTermResolutionCallback search_term_callback_;
 
@@ -212,7 +245,7 @@ class ContextualSearchDelegate
   IcingCallback icing_callback_;
 
   // Used to hold the context until an upcoming search term request is started.
-  scoped_ptr<ContextualSearchContext> context_;
+  std::unique_ptr<ContextualSearchContext> context_;
 
   DISALLOW_COPY_AND_ASSIGN(ContextualSearchDelegate);
 };
