@@ -4,12 +4,25 @@
 
 #include "chrome/browser/android/contextualsearch/contextual_search_delegate.h"
 
-#include "base/memory/scoped_ptr.h"
+#include <stddef.h>
+
+#include <algorithm>
+#include <memory>
+#include <string>
+
+#include "base/base64.h"
+#include "base/command_line.h"
+#include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/browser/android/contextualsearch/contextual_search_context.h"
+#include "chrome/browser/android/contextualsearch/resolved_search_term.h"
+#include "chrome/browser/android/proto/client_discourse_context.pb.h"
+#include "chrome/common/chrome_switches.h"
 #include "components/search_engines/template_url_service.h"
+#include "net/base/escape.h"
 #include "net/url_request/test_url_fetcher_factory.h"
 #include "net/url_request/url_request_test_util.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -18,7 +31,8 @@ using base::ListValue;
 
 namespace {
 
-const char kSomeSpecificBasePage[] = "http://some.specific.host.name.com";
+const char kSomeSpecificBasePage[] = "http://some.specific.host.name.com/";
+const char kDiscourseContextHeaderName[] = "X-Additional-Discourse-Context";
 
 }  // namespace
 
@@ -29,6 +43,8 @@ class ContextualSearchDelegateTest : public testing::Test {
 
  protected:
   void SetUp() override {
+    base::CommandLine::ForCurrentProcess()->AppendSwitch(
+        switches::kEnableContextualSearchContextualCardsBarIntegration);
     request_context_ =
         new net::TestURLRequestContextGetter(io_message_loop_.task_runner());
     template_url_service_.reset(CreateTemplateURLService());
@@ -48,7 +64,9 @@ class ContextualSearchDelegateTest : public testing::Test {
     is_invalid_ = true;
     response_code_ = -1;
     search_term_ = "invalid";
+    mid_ = "";
     display_text_ = "unknown";
+    context_language_ = "";
   }
 
   TemplateURLService* CreateTemplateURLService() {
@@ -57,10 +75,9 @@ class ContextualSearchDelegateTest : public testing::Test {
     data.SetURL("https://foobar.com/url?bar={searchTerms}");
     data.contextual_search_url = "https://foobar.com/_/contextualsearch?"
         "{google:contextualSearchVersion}{google:contextualSearchContextData}";
-    TemplateURL* template_url = new TemplateURL(data);
-    // Takes ownership of |template_url|.
     TemplateURLService* template_url_service = new TemplateURLService(NULL, 0);
-    template_url_service->Add(template_url);
+    TemplateURL* template_url =
+        template_url_service->Add(base::MakeUnique<TemplateURL>(data));
     template_url_service->SetUserSelectedDefaultSearchProvider(template_url);
     return template_url_service;
   }
@@ -90,6 +107,28 @@ class ContextualSearchDelegateTest : public testing::Test {
     ASSERT_TRUE(fetcher());
   }
 
+  // Allows using the vertical bar "|" as a quote character, which makes
+  // test cases more readable versus the escaped double quote that is otherwise
+  // needed for JSON literals.
+  std::string escapeBarQuoted(std::string bar_quoted) {
+    std::replace(bar_quoted.begin(), bar_quoted.end(), '|', '\"');
+    return bar_quoted;
+  }
+
+  void CreateDefaultSearchWithAdditionalJsonData(
+      const std::string additional_json_data) {
+    CreateDefaultSearchContextAndRequestSearchTerm();
+    fetcher()->set_response_code(200);
+    std::string response =
+        escapeBarQuoted("{|search_term|:|obama|" + additional_json_data + "}");
+    fetcher()->SetResponseString(response);
+    fetcher()->delegate()->OnURLFetchComplete(fetcher());
+
+    EXPECT_FALSE(is_invalid());
+    EXPECT_EQ(200, response_code());
+    EXPECT_EQ("obama", search_term());
+  }
+
   void SetResponseStringAndFetch(const std::string& selected_text,
                                  const std::string& mentions_start,
                                  const std::string& mentions_end) {
@@ -117,13 +156,41 @@ class ContextualSearchDelegateTest : public testing::Test {
     delegate_->set_context_for_testing(test_context_);
   }
 
-  bool DoesRequestContainOurSpecificBasePage() {
-    return fetcher()->GetOriginalURL().spec().find(
-        specific_base_page_URL_escaped()) != std::string::npos;
+  // Gets the Client Discourse Context proto from the request header.
+  discourse_context::ClientDiscourseContext GetDiscourseContextFromRequest() {
+    discourse_context::ClientDiscourseContext cdc;
+    // Make sure we can get the actual raw headers from the fake fetcher.
+    net::HttpRequestHeaders fetch_headers;
+    fetcher()->GetExtraRequestHeaders(&fetch_headers);
+    if (fetch_headers.HasHeader(kDiscourseContextHeaderName)) {
+      std::string actual_header_value;
+      fetch_headers.GetHeader(kDiscourseContextHeaderName,
+                              &actual_header_value);
+
+      // Unescape, since the server memoizer expects a web-safe encoding.
+      std::string unescaped_header = actual_header_value;
+      std::replace(unescaped_header.begin(), unescaped_header.end(), '-', '+');
+      std::replace(unescaped_header.begin(), unescaped_header.end(), '_', '/');
+
+      // Base64 decode the header.
+      std::string decoded_header;
+      if (base::Base64Decode(unescaped_header, &decoded_header)) {
+        cdc.ParseFromString(decoded_header);
+      }
+    }
+    return cdc;
   }
 
-  std::string specific_base_page_URL_escaped() {
-    return net::EscapeQueryParamValue(kSomeSpecificBasePage, true);
+  // Gets the base-page URL from the request, or an empty string if not present.
+  std::string getBasePageUrlFromRequest() {
+    std::string result;
+    discourse_context::ClientDiscourseContext cdc =
+        GetDiscourseContextFromRequest();
+    if (cdc.display_size() > 0) {
+      const discourse_context::Display& first_display = cdc.display(0);
+      result = first_display.uri();
+    }
+    return result;
   }
 
   net::TestURLFetcher* fetcher() { return fetcher_; }
@@ -132,37 +199,36 @@ class ContextualSearchDelegateTest : public testing::Test {
   std::string search_term() { return search_term_; }
   std::string display_text() { return display_text_; }
   std::string alternate_term() { return alternate_term_; }
+  std::string mid() { return mid_; }
+  std::string caption() { return caption_; }
+  std::string thumbnail_url() { return thumbnail_url_; }
   bool do_prevent_preload() { return prevent_preload_; }
-  std::string before_text() { return before_text_; }
   std::string after_text() { return after_text_; }
   int start_adjust() { return start_adjust_; }
   int end_adjust() { return end_adjust_; }
+  std::string context_language() { return context_language_; }
 
   // The delegate under test.
-  scoped_ptr<ContextualSearchDelegate> delegate_;
+  std::unique_ptr<ContextualSearchDelegate> delegate_;
 
  private:
-  void recordSearchTermResolutionResponse(bool is_invalid,
-                                          int response_code,
-                                          const std::string& search_term,
-                                          const std::string& display_text,
-                                          const std::string& alternate_term,
-                                          bool prevent_preload,
-                                          int start_adjust,
-                                          int end_adjust) {
-    is_invalid_ = is_invalid;
-    response_code_ = response_code;
-    search_term_ = search_term;
-    display_text_ = display_text;
-    alternate_term_ = alternate_term;
-    prevent_preload_ = prevent_preload;
-    start_adjust_ = start_adjust;
-    end_adjust_ = end_adjust;
+  void recordSearchTermResolutionResponse(
+      const ResolvedSearchTerm& resolved_search_term) {
+    is_invalid_ = resolved_search_term.is_invalid;
+    response_code_ = resolved_search_term.response_code;
+    search_term_ = resolved_search_term.search_term;
+    display_text_ = resolved_search_term.display_text;
+    alternate_term_ = resolved_search_term.alternate_term;
+    mid_ = resolved_search_term.mid;
+    thumbnail_url_ = resolved_search_term.thumbnail_url;
+    caption_ = resolved_search_term.caption;
+    prevent_preload_ = resolved_search_term.prevent_preload;
+    start_adjust_ = resolved_search_term.selection_start_adjust;
+    end_adjust_ = resolved_search_term.selection_end_adjust;
+    context_language_ = resolved_search_term.context_language;
   }
 
-  void recordSurroundingText(const std::string& before_text,
-                             const std::string& after_text) {
-    before_text_ = before_text;
+  void recordSurroundingText(const std::string& after_text) {
     after_text_ = after_text;
   }
 
@@ -178,16 +244,19 @@ class ContextualSearchDelegateTest : public testing::Test {
   std::string search_term_;
   std::string display_text_;
   std::string alternate_term_;
+  std::string mid_;
+  std::string thumbnail_url_;
+  std::string caption_;
   bool prevent_preload_;
   int start_adjust_;
   int end_adjust_;
-  std::string before_text_;
   std::string after_text_;
+  std::string context_language_;
 
   base::MessageLoopForIO io_message_loop_;
   net::TestURLFetcherFactory test_factory_;
   net::TestURLFetcher* fetcher_;
-  scoped_ptr<TemplateURLService> template_url_service_;
+  std::unique_ptr<TemplateURLService> template_url_service_;
   scoped_refptr<net::TestURLRequestContextGetter> request_context_;
 
   // Will be owned by the delegate.
@@ -211,8 +280,8 @@ TEST_F(ContextualSearchDelegateTest, NormalFetchWithXssiEscape) {
   EXPECT_EQ(200, response_code());
   EXPECT_EQ("obama", search_term());
   EXPECT_EQ("Barack Obama", display_text());
+  EXPECT_EQ("/m/02mjmr", mid());
   EXPECT_FALSE(do_prevent_preload());
-  EXPECT_TRUE(DoesRequestContainOurSpecificBasePage());
 }
 
 TEST_F(ContextualSearchDelegateTest, NormalFetchWithoutXssiEscape) {
@@ -229,8 +298,8 @@ TEST_F(ContextualSearchDelegateTest, NormalFetchWithoutXssiEscape) {
   EXPECT_EQ(200, response_code());
   EXPECT_EQ("obama", search_term());
   EXPECT_EQ("Barack Obama", display_text());
+  EXPECT_EQ("/m/02mjmr", mid());
   EXPECT_FALSE(do_prevent_preload());
-  EXPECT_TRUE(DoesRequestContainOurSpecificBasePage());
 }
 
 TEST_F(ContextualSearchDelegateTest, ResponseWithNoDisplayText) {
@@ -245,8 +314,8 @@ TEST_F(ContextualSearchDelegateTest, ResponseWithNoDisplayText) {
   EXPECT_EQ(200, response_code());
   EXPECT_EQ("obama", search_term());
   EXPECT_EQ("obama", display_text());
+  EXPECT_EQ("/m/02mjmr", mid());
   EXPECT_FALSE(do_prevent_preload());
-  EXPECT_TRUE(DoesRequestContainOurSpecificBasePage());
 }
 
 TEST_F(ContextualSearchDelegateTest, ResponseWithPreventPreload) {
@@ -261,8 +330,8 @@ TEST_F(ContextualSearchDelegateTest, ResponseWithPreventPreload) {
   EXPECT_EQ(200, response_code());
   EXPECT_EQ("obama", search_term());
   EXPECT_EQ("obama", display_text());
+  EXPECT_EQ("/m/02mjmr", mid());
   EXPECT_TRUE(do_prevent_preload());
-  EXPECT_TRUE(DoesRequestContainOurSpecificBasePage());
 }
 
 TEST_F(ContextualSearchDelegateTest, NonJsonResponse) {
@@ -275,8 +344,8 @@ TEST_F(ContextualSearchDelegateTest, NonJsonResponse) {
   EXPECT_EQ(200, response_code());
   EXPECT_EQ("", search_term());
   EXPECT_EQ("", display_text());
+  EXPECT_EQ("", mid());
   EXPECT_FALSE(do_prevent_preload());
-  EXPECT_TRUE(DoesRequestContainOurSpecificBasePage());
 }
 
 TEST_F(ContextualSearchDelegateTest, InvalidResponse) {
@@ -395,7 +464,6 @@ TEST_F(ContextualSearchDelegateTest, SurroundingTextHighMaximum) {
   base::string16 surrounding = base::ASCIIToUTF16("aa bb Bogus dd ee");
   SetSurroundingContext(surrounding, 6, 11);
   delegate_->SendSurroundingText(30);  // High maximum # of surrounding chars.
-  EXPECT_EQ("aa bb", before_text());
   EXPECT_EQ("dd ee", after_text());
 }
 
@@ -404,7 +472,6 @@ TEST_F(ContextualSearchDelegateTest, SurroundingTextLowMaximum) {
   SetSurroundingContext(surrounding, 6, 11);
   delegate_->SendSurroundingText(3);  // Low maximum # of surrounding chars.
   // Whitespaces are trimmed.
-  EXPECT_EQ("bb", before_text());
   EXPECT_EQ("dd", after_text());
 }
 
@@ -412,10 +479,16 @@ TEST_F(ContextualSearchDelegateTest, SurroundingTextNoBeforeText) {
   base::string16 surrounding = base::ASCIIToUTF16("Bogus ee ff gg");
   SetSurroundingContext(surrounding, 0, 5);
   delegate_->SendSurroundingText(5);
-  EXPECT_EQ("", before_text());
   EXPECT_EQ("ee f", after_text());
 }
 
+TEST_F(ContextualSearchDelegateTest, SurroundingTextNoAfterText) {
+  base::string16 surrounding = base::ASCIIToUTF16("aa bb Bogus");
+  SetSurroundingContext(surrounding, 6, 11);
+  delegate_->SendSurroundingText(5);
+  EXPECT_EQ("", after_text());
+}
+
 TEST_F(ContextualSearchDelegateTest, ExtractMentionsStartEnd) {
   ListValue mentions_list;
   mentions_list.AppendInteger(1);
@@ -451,7 +524,7 @@ TEST_F(ContextualSearchDelegateTest, SurroundingTextForIcingNegativeLimit) {
   EXPECT_EQ(base::ASCIIToUTF16("Barack Obama"), result);
 }
 
-TEST_F(ContextualSearchDelegateTest, DecodeSearchTermsFromJsonResponse) {
+TEST_F(ContextualSearchDelegateTest, DecodeSearchTermFromJsonResponse) {
   std::string json_with_escape =
       ")]}'\n"
       "{\"mid\":\"/m/02mjmr\", \"search_term\":\"obama\","
@@ -461,15 +534,74 @@ TEST_F(ContextualSearchDelegateTest, DecodeSearchTermsFromJsonResponse) {
   std::string search_term;
   std::string display_text;
   std::string alternate_term;
+  std::string mid;
   std::string prevent_preload;
   int mention_start;
   int mention_end;
-  delegate_->DecodeSearchTermsFromJsonResponse(json_with_escape, &search_term,
-                                               &display_text, &alternate_term,
-                                               &prevent_preload, &mention_start,
-                                               &mention_end);
+  std::string context_language;
+  std::string thumbnail_url;
+  std::string caption;
+  std::string quick_action_uri;
+  QuickActionCategory quick_action_category = QUICK_ACTION_CATEGORY_NONE;
+
+  delegate_->DecodeSearchTermFromJsonResponse(
+      json_with_escape, &search_term, &display_text, &alternate_term,
+      &mid, &prevent_preload, &mention_start, &mention_end, &context_language,
+      &thumbnail_url, &caption, &quick_action_uri, &quick_action_category);
+
   EXPECT_EQ("obama", search_term);
   EXPECT_EQ("Barack Obama", display_text);
   EXPECT_EQ("barack obama", alternate_term);
+  EXPECT_EQ("/m/02mjmr", mid);
   EXPECT_EQ("", prevent_preload);
+  EXPECT_EQ("", context_language);
+  EXPECT_EQ("", thumbnail_url);
+  EXPECT_EQ("", caption);
+  EXPECT_EQ("", quick_action_uri);
+  EXPECT_EQ(QUICK_ACTION_CATEGORY_NONE, quick_action_category);
+}
+
+TEST_F(ContextualSearchDelegateTest, ResponseWithLanguage) {
+  CreateDefaultSearchContextAndRequestSearchTerm();
+  fetcher()->set_response_code(200);
+  fetcher()->SetResponseString(
+      "{\"mid\":\"/m/02mjmr\",\"search_term\":\"obama\","
+      "\"mentions\":[0,15],\"prevent_preload\":\"1\", "
+      "\"lang\":\"de\"}");
+  fetcher()->delegate()->OnURLFetchComplete(fetcher());
+
+  EXPECT_FALSE(is_invalid());
+  EXPECT_EQ(200, response_code());
+  EXPECT_EQ("obama", search_term());
+  EXPECT_EQ("obama", display_text());
+  EXPECT_EQ("/m/02mjmr", mid());
+  EXPECT_TRUE(do_prevent_preload());
+  EXPECT_EQ("de", context_language());
+}
+
+TEST_F(ContextualSearchDelegateTest, HeaderContainsBasePageUrl) {
+  CreateDefaultSearchContextAndRequestSearchTerm();
+  EXPECT_EQ(kSomeSpecificBasePage, getBasePageUrlFromRequest());
+}
+
+// Missing all Contextual Cards data.
+TEST_F(ContextualSearchDelegateTest, ContextualCardsResponseWithNoData) {
+  CreateDefaultSearchWithAdditionalJsonData("");
+  EXPECT_EQ("", caption());
+  EXPECT_EQ("", thumbnail_url());
+}
+
+// Test just the root level caption.
+TEST_F(ContextualSearchDelegateTest, ContextualCardsResponseWithCaption) {
+  CreateDefaultSearchWithAdditionalJsonData(", |caption|:|aCaption|");
+  EXPECT_EQ("aCaption", caption());
+  EXPECT_EQ("", thumbnail_url());
+}
+
+// Test just the root level thumbnail.
+TEST_F(ContextualSearchDelegateTest, ContextualCardsResponseWithThumbnail) {
+  CreateDefaultSearchWithAdditionalJsonData(
+      ", |thumbnail|:|https://t0.gstatic.com/images?q=tbn:ANd9|");
+  EXPECT_EQ("", caption());
+  EXPECT_EQ("https://t0.gstatic.com/images?q=tbn:ANd9", thumbnail_url());
 }
