@@ -7,9 +7,11 @@
 #include <stddef.h>
 
 #include "base/android/jni_android.h"
+#include "base/android/jni_array.h"
 #include "base/android/jni_string.h"
-#include "base/prefs/pref_service.h"
+#include "base/memory/ptr_util.h"
 #include "base/strings/string16.h"
+#include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/time/time.h"
 #include "base/timer/timer.h"
@@ -28,8 +30,8 @@
 #include "chrome/browser/search_engines/template_url_service_factory.h"
 #include "chrome/browser/sessions/session_tab_helper.h"
 #include "chrome/browser/ui/search/instant_search_prerenderer.h"
-#include "chrome/common/instant_types.h"
 #include "chrome/common/pref_names.h"
+#include "chrome/common/search/instant_types.h"
 #include "chrome/common/url_constants.h"
 #include "components/bookmarks/browser/bookmark_model.h"
 #include "components/keyed_service/content/browser_context_dependency_manager.h"
@@ -43,6 +45,7 @@
 #include "components/omnibox/browser/omnibox_field_trial.h"
 #include "components/omnibox/browser/omnibox_log.h"
 #include "components/omnibox/browser/search_provider.h"
+#include "components/prefs/pref_service.h"
 #include "components/search/search.h"
 #include "components/search_engines/template_url_service.h"
 #include "components/toolbar/toolbar_model.h"
@@ -57,7 +60,10 @@ using base::android::AttachCurrentThread;
 using base::android::ConvertJavaStringToUTF16;
 using base::android::ConvertUTF8ToJavaString;
 using base::android::ConvertUTF16ToJavaString;
+using base::android::JavaParamRef;
 using base::android::JavaRef;
+using base::android::ScopedJavaLocalRef;
+using base::android::ToJavaIntArray;
 using bookmarks::BookmarkModel;
 using metrics::OmniboxEventProto;
 
@@ -82,13 +88,13 @@ class ZeroSuggestPrefetcher : public AutocompleteControllerDelegate {
   // AutocompleteControllerDelegate:
   void OnResultChanged(bool default_match_changed) override;
 
-  scoped_ptr<AutocompleteController> controller_;
+  std::unique_ptr<AutocompleteController> controller_;
   base::OneShotTimer expire_timer_;
 };
 
 ZeroSuggestPrefetcher::ZeroSuggestPrefetcher(Profile* profile)
     : controller_(new AutocompleteController(
-          make_scoped_ptr(new ChromeAutocompleteProviderClient(profile)),
+          base::MakeUnique<ChromeAutocompleteProviderClient>(profile),
           this,
           AutocompleteProvider::TYPE_ZERO_SUGGEST)) {
   // Creating an arbitrary fake_request_source to avoid passing in an invalid
@@ -123,12 +129,11 @@ void ZeroSuggestPrefetcher::OnResultChanged(bool default_match_changed) {
 
 AutocompleteControllerAndroid::AutocompleteControllerAndroid(Profile* profile)
     : autocomplete_controller_(new AutocompleteController(
-          make_scoped_ptr(new ChromeAutocompleteProviderClient(profile)),
+          base::WrapUnique(new ChromeAutocompleteProviderClient(profile)),
           this,
           kAndroidAutocompleteProviders)),
       inside_synchronous_start_(false),
-      profile_(profile) {
-}
+      profile_(profile) {}
 
 void AutocompleteControllerAndroid::Start(
     JNIEnv* env,
@@ -174,7 +179,6 @@ void AutocompleteControllerAndroid::OnOmniboxFocused(
     const JavaParamRef<jobject>& obj,
     const JavaParamRef<jstring>& j_omnibox_text,
     const JavaParamRef<jstring>& j_current_url,
-    jboolean is_query_in_omnibox,
     jboolean focused_from_fakebox) {
   if (!autocomplete_controller_)
     return;
@@ -190,7 +194,7 @@ void AutocompleteControllerAndroid::OnOmniboxFocused(
 
   input_ = AutocompleteInput(
       omnibox_text, base::string16::npos, std::string(), current_url,
-      ClassifyPage(current_url, is_query_in_omnibox, focused_from_fakebox),
+      ClassifyPage(current_url, focused_from_fakebox),
       false, false, true, true, true,
       ChromeAutocompleteSchemeClassifier(profile_));
   autocomplete_controller_->Start(input_);
@@ -199,14 +203,14 @@ void AutocompleteControllerAndroid::OnOmniboxFocused(
 void AutocompleteControllerAndroid::Stop(JNIEnv* env,
                                          const JavaParamRef<jobject>& obj,
                                          bool clear_results) {
-  if (autocomplete_controller_ != NULL)
+  if (autocomplete_controller_ != nullptr)
     autocomplete_controller_->Stop(clear_results);
 }
 
 void AutocompleteControllerAndroid::ResetSession(
     JNIEnv* env,
     const JavaParamRef<jobject>& obj) {
-  if (autocomplete_controller_ != NULL)
+  if (autocomplete_controller_ != nullptr)
     autocomplete_controller_->ResetSession();
 }
 
@@ -215,14 +219,14 @@ void AutocompleteControllerAndroid::OnSuggestionSelected(
     const JavaParamRef<jobject>& obj,
     jint selected_index,
     const JavaParamRef<jstring>& j_current_url,
-    jboolean is_query_in_omnibox,
     jboolean focused_from_fakebox,
     jlong elapsed_time_since_first_modified,
+    jint completed_length,
     const JavaParamRef<jobject>& j_web_contents) {
   base::string16 url = ConvertJavaStringToUTF16(env, j_current_url);
   const GURL current_url = GURL(url);
   OmniboxEventProto::PageClassification current_page_classification =
-      ClassifyPage(current_url, is_query_in_omnibox, focused_from_fakebox);
+      ClassifyPage(current_url, focused_from_fakebox);
   const base::TimeTicks& now(base::TimeTicks::Now());
   content::WebContents* web_contents =
       content::WebContents::FromJavaWebContents(j_web_contents);
@@ -239,7 +243,7 @@ void AutocompleteControllerAndroid::OnSuggestionSelected(
       SessionTabHelper::IdForTab(web_contents),
       current_page_classification,
       base::TimeDelta::FromMilliseconds(elapsed_time_since_first_modified),
-      base::string16::npos,
+      completed_length,
       now - autocomplete_controller_->last_time_default_match_changed(),
       autocomplete_controller_->result());
   autocomplete_controller_->AddProvidersInfo(&log.providers_info);
@@ -286,7 +290,7 @@ void AutocompleteControllerAndroid::Shutdown() {
   ScopedJavaLocalRef<jobject> java_bridge =
       weak_java_autocomplete_controller_android_.get(env);
   if (java_bridge.obj())
-    Java_AutocompleteController_notifyNativeDestroyed(env, java_bridge.obj());
+    Java_AutocompleteController_notifyNativeDestroyed(env, java_bridge);
 
   weak_java_autocomplete_controller_android_.reset();
 }
@@ -381,7 +385,7 @@ void AutocompleteControllerAndroid::NotifySuggestionsReceived(
     ScopedJavaLocalRef<jobject> j_omnibox_suggestion =
         BuildOmniboxSuggestion(env, autocomplete_result.match_at(i));
     Java_AutocompleteController_addOmniboxSuggestionToList(
-        env, suggestion_list_obj.obj(), j_omnibox_suggestion.obj());
+        env, suggestion_list_obj, j_omnibox_suggestion);
   }
 
   // Get the inline-autocomplete text.
@@ -395,16 +399,13 @@ void AutocompleteControllerAndroid::NotifySuggestionsReceived(
       ConvertUTF16ToJavaString(env, inline_autocomplete_text);
   jlong j_autocomplete_result =
       reinterpret_cast<intptr_t>(&(autocomplete_result));
-  Java_AutocompleteController_onSuggestionsReceived(env,
-                                                    java_bridge.obj(),
-                                                    suggestion_list_obj.obj(),
-                                                    inline_text.obj(),
-                                                    j_autocomplete_result);
+  Java_AutocompleteController_onSuggestionsReceived(
+      env, java_bridge, suggestion_list_obj, inline_text,
+      j_autocomplete_result);
 }
 
 OmniboxEventProto::PageClassification
 AutocompleteControllerAndroid::ClassifyPage(const GURL& gurl,
-                                            bool is_query_in_omnibox,
                                             bool focused_from_fakebox) const {
   if (!gurl.is_valid())
     return OmniboxEventProto::INVALID_SPEC;
@@ -428,9 +429,6 @@ AutocompleteControllerAndroid::ClassifyPage(const GURL& gurl,
   if (url == profile_->GetPrefs()->GetString(prefs::kHomePage))
     return OmniboxEventProto::HOME_PAGE;
 
-  if (is_query_in_omnibox)
-    return OmniboxEventProto::SEARCH_RESULT_PAGE_DOING_SEARCH_TERM_REPLACEMENT;
-
   bool is_search_url = TemplateURLServiceFactory::GetForProfile(profile_)->
       IsSearchResultsPageFromDefaultSearchProvider(gurl);
   if (is_search_url)
@@ -439,14 +437,90 @@ AutocompleteControllerAndroid::ClassifyPage(const GURL& gurl,
   return OmniboxEventProto::OTHER;
 }
 
+namespace {
+
+// Updates the formatting of Android omnibox suggestions where we intentionally
+// deviate from the desktop logic.
+//
+// For URL suggestions, the leading https:// and www. will be omitted if the
+// user query did not explicitly contain them.  The http:// portion is already
+// omitted for all ports.
+//
+// If the match is not for a URL, it will leave |out_content| and
+// |out_classifications| untouched.
+void FormatMatchContentsForDisplay(
+    const AutocompleteMatch& match,
+    base::string16* out_content,
+    ACMatchClassifications* out_classifications) {
+  if (AutocompleteMatch::IsSearchType(match.type))
+    return;
+
+  int match_offset = -1;
+  for (auto contents_class : match.contents_class) {
+    if (contents_class.style & ACMatchClassification::MATCH) {
+      match_offset = contents_class.offset;
+      break;
+    }
+  }
+  int original_match_offset(match_offset);
+  const base::string16 https(base::ASCIIToUTF16("https://"));
+  if (base::StartsWith(*out_content, https, base::CompareCase::SENSITIVE)) {
+    if (match_offset >= static_cast<int>(https.length())) {
+      *out_content = out_content->substr(https.length());
+      match_offset -= https.length();
+    }
+  }
+  const base::string16 www(base::ASCIIToUTF16("www."));
+  if (base::StartsWith(*out_content, www, base::CompareCase::SENSITIVE)) {
+    if (match_offset >= static_cast<int>(www.length())) {
+      *out_content = out_content->substr(www.length());
+      match_offset -= www.length();
+    }
+  }
+  int match_offset_delta = original_match_offset - match_offset;
+  if (match_offset_delta > 0) {
+    out_classifications->clear();
+    for (size_t i = match.contents_class.size(); i > 0; --i) {
+      ACMatchClassification classification(match.contents_class[i - 1]);
+      int updated_offset = std::max(
+          0 , static_cast<int>(classification.offset) - match_offset_delta);
+      out_classifications->insert(
+          out_classifications->begin(),
+          ACMatchClassification(updated_offset, classification.style));
+      if (updated_offset == 0)
+        break;
+    }
+  }
+}
+
+}  // namespace
+
 ScopedJavaLocalRef<jobject>
 AutocompleteControllerAndroid::BuildOmniboxSuggestion(
     JNIEnv* env,
     const AutocompleteMatch& match) {
-  ScopedJavaLocalRef<jstring> contents =
-      ConvertUTF16ToJavaString(env, match.contents);
+  base::string16 contents(match.contents);
+  ACMatchClassifications contents_classifications(match.contents_class);
+  FormatMatchContentsForDisplay(match, &contents, &contents_classifications);
+
+  ScopedJavaLocalRef<jstring> jcontents =
+      ConvertUTF16ToJavaString(env, contents);
+  std::vector<int> contents_class_offsets;
+  std::vector<int> contents_class_styles;
+  for (auto contents_class : contents_classifications) {
+    contents_class_offsets.push_back(contents_class.offset);
+    contents_class_styles.push_back(contents_class.style);
+  }
+
   ScopedJavaLocalRef<jstring> description =
       ConvertUTF16ToJavaString(env, match.description);
+  std::vector<int> description_class_offsets;
+  std::vector<int> description_class_styles;
+  for (auto description_class : match.description_class) {
+    description_class_offsets.push_back(description_class.offset);
+    description_class_styles.push_back(description_class.style);
+  }
+
   ScopedJavaLocalRef<jstring> answer_contents =
       ConvertUTF16ToJavaString(env, match.answer_contents);
   ScopedJavaLocalRef<jstring> answer_type =
@@ -455,40 +529,20 @@ AutocompleteControllerAndroid::BuildOmniboxSuggestion(
       ConvertUTF16ToJavaString(env, match.fill_into_edit);
   ScopedJavaLocalRef<jstring> destination_url =
       ConvertUTF8ToJavaString(env, match.destination_url.spec());
-  // Note that we are also removing 'www' host from formatted url.
-  ScopedJavaLocalRef<jstring> formatted_url = ConvertUTF16ToJavaString(env,
-      FormatURLUsingAcceptLanguages(match.stripped_destination_url));
-  BookmarkModel* bookmark_model = BookmarkModelFactory::GetForProfile(profile_);
+  BookmarkModel* bookmark_model =
+      BookmarkModelFactory::GetForBrowserContext(profile_);
   return Java_AutocompleteController_buildOmniboxSuggestion(
-      env,
-      match.type,
-      AutocompleteMatch::IsSearchType(match.type),
-      match.relevance,
-      match.transition,
-      contents.obj(),
-      description.obj(),
-      answer_contents.obj(),
-      answer_type.obj(),
-      fill_into_edit.obj(),
-      destination_url.obj(),
-      formatted_url.obj(),
+      env, match.type, AutocompleteMatch::IsSearchType(match.type),
+      match.relevance, match.transition, jcontents,
+      ToJavaIntArray(env, contents_class_offsets),
+      ToJavaIntArray(env, contents_class_styles), description,
+      ToJavaIntArray(env, description_class_offsets),
+      ToJavaIntArray(env, description_class_styles), answer_contents,
+      answer_type, fill_into_edit, destination_url,
       bookmark_model && bookmark_model->IsBookmarked(match.destination_url),
       match.SupportsDeletion());
 }
 
-base::string16 AutocompleteControllerAndroid::FormatURLUsingAcceptLanguages(
-    GURL url) {
-  if (profile_ == NULL)
-    return base::string16();
-
-  std::string languages(
-      profile_->GetPrefs()->GetString(prefs::kAcceptLanguages));
-
-  return url_formatter::FormatUrl(
-      url, languages, url_formatter::kFormatUrlOmitAll,
-      net::UnescapeRule::SPACES, nullptr, nullptr, nullptr);
-}
-
 ScopedJavaLocalRef<jobject>
 AutocompleteControllerAndroid::GetTopSynchronousResult(
     JNIEnv* env,
@@ -503,8 +557,8 @@ AutocompleteControllerAndroid::GetTopSynchronousResult(
         obj,
         j_text,
         -1,
-        NULL,
-        NULL,
+        nullptr,
+        nullptr,
         prevent_inline_autocomplete,
         false,
         false,
@@ -545,7 +599,7 @@ static ScopedJavaLocalRef<jstring> QualifyPartialURLQuery(
       false,
       OmniboxEventProto::INVALID_SPEC,
       &match,
-      NULL);
+      nullptr);
   if (!match.destination_url.is_valid())
     return ScopedJavaLocalRef<jstring>();
 
