@@ -8,14 +8,15 @@
 #if defined(__OBJC__)
 
 #import <Cocoa/Cocoa.h>
+
+#include <memory>
 #include <vector>
 
 #include "base/files/file_path.h"
 #include "base/mac/scoped_nsobject.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/observer_list.h"
-#include "base/prefs/pref_change_registrar.h"
 #include "base/time/time.h"
+#include "components/prefs/pref_change_registrar.h"
 #include "ui/base/work_area_watcher_observer.h"
 
 class AppControllerProfileObserver;
@@ -29,6 +30,7 @@ class HistoryMenuBridge;
 class Profile;
 @class ProfileMenuController;
 class QuitWithAppsController;
+class ScopedKeepAlive;
 
 namespace ui {
 class WorkAreaWatcherObserver;
@@ -42,15 +44,16 @@ class WorkAreaWatcherObserver;
                                     NSApplicationDelegate> {
  @private
   // Manages the state of the command menu items.
-  scoped_ptr<CommandUpdater> menuState_;
+  std::unique_ptr<CommandUpdater> menuState_;
 
   // The profile last used by a Browser. It is this profile that was used to
   // build the user-data specific main menu items.
   Profile* lastProfile_;
 
-  // The ProfileObserver observes the ProfileInfoCache and gets notified
+  // The ProfileObserver observes the ProfileAttrbutesStorage and gets notified
   // when a profile has been deleted.
-  scoped_ptr<AppControllerProfileObserver> profileInfoCacheObserver_;
+  std::unique_ptr<AppControllerProfileObserver>
+      profileAttributesStorageObserver_;
 
   // Management of the bookmark menu which spans across all windows
   // (and Browser*s). |profileBookmarkMenuBridgeMap_| is a cache that owns one
@@ -58,9 +61,10 @@ class WorkAreaWatcherObserver;
   // a weak pointer that is updated to match the corresponding cache entry
   // during a profile switch.
   BookmarkMenuBridge* bookmarkMenuBridge_;
-  std::map<base::FilePath, BookmarkMenuBridge*> profileBookmarkMenuBridgeMap_;
+  std::map<base::FilePath, std::unique_ptr<BookmarkMenuBridge>>
+      profileBookmarkMenuBridgeMap_;
 
-  scoped_ptr<HistoryMenuBridge> historyMenuBridge_;
+  std::unique_ptr<HistoryMenuBridge> historyMenuBridge_;
 
   // Controller that manages main menu items for packaged apps.
   base::scoped_nsobject<AppShimMenuController> appShimMenuController_;
@@ -92,7 +96,7 @@ class WorkAreaWatcherObserver;
   // Observers that listen to the work area changes.
   base::ObserverList<ui::WorkAreaWatcherObserver> workAreaChangeObservers_;
 
-  scoped_ptr<PrefChangeRegistrar> profilePrefRegistrar_;
+  std::unique_ptr<PrefChangeRegistrar> profilePrefRegistrar_;
   PrefChangeRegistrar localPrefRegistrar_;
 
   // Displays a notification when quitting while apps are running.
@@ -102,22 +106,19 @@ class WorkAreaWatcherObserver;
   base::scoped_nsobject<HandoffManager> handoffManager_;
 
   // Observes changes to the active URL.
-  scoped_ptr<HandoffActiveURLObserverBridge>
+  std::unique_ptr<HandoffActiveURLObserverBridge>
       handoff_active_url_observer_bridge_;
 
   // This will be true after receiving a NSWorkspaceWillPowerOffNotification.
   BOOL isPoweringOff_;
+
+  // Request to keep the browser alive during that object's lifetime.
+  std::unique_ptr<ScopedKeepAlive> keep_alive_;
 }
 
 @property(readonly, nonatomic) BOOL startupComplete;
 @property(readonly, nonatomic) Profile* lastProfile;
 
-// Helper method used to update the "Signin" menu item in the main menu and the
-// wrench menu to reflect the current signed in state.
-+ (void)updateSigninItem:(id)signinItem
-              shouldShow:(BOOL)showSigninMenuItem
-          currentProfile:(Profile*)profile;
-
 - (void)didEndMainMessageLoop;
 
 // Try to close all browser windows, and if that succeeds then quit.
@@ -136,6 +137,13 @@ class WorkAreaWatcherObserver;
 // dialogs that are window modal will block the browser.
 - (BOOL)keyWindowIsModal;
 
+// Called when the user picks a menu item when there are no key windows, or when
+// there is no foreground browser window. Calls through to the browser object to
+// execute the command. This assumes that the command is supported and doesn't
+// check, otherwise it should have been disabled in the UI in
+// |-validateUserInterfaceItem:|.
+- (void)commandDispatch:(id)sender;
+
 // Show the preferences window, or bring it to the front if it's already
 // visible.
 - (IBAction)showPreferences:(id)sender;
