@@ -9,7 +9,7 @@
 #include "base/single_thread_task_runner.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "build/build_config.h"
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/browser/apps/app_browsertest_util.h"
@@ -26,12 +26,14 @@
 #include "components/guest_view/browser/guest_view_manager_factory.h"
 #include "components/guest_view/browser/test_guest_view_manager.h"
 #include "content/public/browser/notification_service.h"
+#include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/browser/render_view_host.h"
 #include "content/public/browser/render_widget_host.h"
 #include "content/public/browser/render_widget_host_iterator.h"
 #include "content/public/browser/render_widget_host_view.h"
 #include "content/public/browser/web_contents.h"
+#include "content/public/common/content_features.h"
 #include "content/public/common/content_switches.h"
 #include "content/public/test/browser_test_utils.h"
 #include "extensions/browser/api/extensions_api_client.h"
@@ -51,10 +53,9 @@ using guest_view::GuestViewManager;
 using guest_view::TestGuestViewManager;
 using guest_view::TestGuestViewManagerFactory;
 
-class WebViewInteractiveTest
-    : public extensions::PlatformAppBrowserTest {
+class WebViewInteractiveTestBase : public extensions::PlatformAppBrowserTest {
  public:
-  WebViewInteractiveTest()
+  WebViewInteractiveTestBase()
       : guest_web_contents_(NULL),
         embedder_web_contents_(NULL),
         corner_(gfx::Point()),
@@ -183,7 +184,7 @@ class WebViewInteractiveTest
     NO_TEST_SERVER
   };
 
-  scoped_ptr<ExtensionTestMessageListener> RunAppHelper(
+  std::unique_ptr<ExtensionTestMessageListener> RunAppHelper(
       const std::string& test_name,
       const std::string& app_location,
       TestServer test_server,
@@ -191,13 +192,13 @@ class WebViewInteractiveTest
     // For serving guest pages.
     if ((test_server == NEEDS_TEST_SERVER) && !StartEmbeddedTestServer()) {
       LOG(ERROR) << "FAILED TO START TEST SERVER.";
-      return scoped_ptr<ExtensionTestMessageListener>();
+      return std::unique_ptr<ExtensionTestMessageListener>();
     }
 
     LoadAndLaunchPlatformApp(app_location.c_str(), "Launched");
     if (!ui_test_utils::ShowAndFocusNativeWindow(GetPlatformAppWindow())) {
       LOG(ERROR) << "UNABLE TO FOCUS TEST WINDOW.";
-      return scoped_ptr<ExtensionTestMessageListener>();
+      return std::unique_ptr<ExtensionTestMessageListener>();
     }
 
     // Flush any pending events to make sure we start with a clean slate.
@@ -205,14 +206,14 @@ class WebViewInteractiveTest
 
     *embedder_web_contents = GetFirstAppWindowWebContents();
 
-    scoped_ptr<ExtensionTestMessageListener> done_listener(
+    std::unique_ptr<ExtensionTestMessageListener> done_listener(
         new ExtensionTestMessageListener("TEST_PASSED", false));
     done_listener->set_failure_message("TEST_FAILED");
     if (!content::ExecuteScript(
             *embedder_web_contents,
             base::StringPrintf("runTest('%s')", test_name.c_str()))) {
       LOG(ERROR) << "UNABLE TO START TEST";
-      return scoped_ptr<ExtensionTestMessageListener>();
+      return std::unique_ptr<ExtensionTestMessageListener>();
     }
 
     return done_listener;
@@ -222,13 +223,13 @@ class WebViewInteractiveTest
                   const std::string& app_location,
                   TestServer test_server) {
     content::WebContents* embedder_web_contents = NULL;
-    scoped_ptr<ExtensionTestMessageListener> done_listener(
-        RunAppHelper(
-            test_name, app_location, test_server, &embedder_web_contents));
+    std::unique_ptr<ExtensionTestMessageListener> done_listener(RunAppHelper(
+        test_name, app_location, test_server, &embedder_web_contents));
 
     ASSERT_TRUE(done_listener);
     ASSERT_TRUE(done_listener->WaitUntilSatisfied());
 
+    embedder_web_contents_ = embedder_web_contents;
     guest_web_contents_ = GetGuestViewManager()->WaitForSingleGuestCreated();
   }
 
@@ -354,7 +355,7 @@ class WebViewInteractiveTest
     }
 
     size_t CountWidgets() {
-      scoped_ptr<content::RenderWidgetHostIterator> widgets(
+      std::unique_ptr<content::RenderWidgetHostIterator> widgets(
           content::RenderWidgetHost::GetRenderWidgetHosts());
       size_t num_widgets = 0;
       while (content::RenderWidgetHost* widget = widgets->GetNextHost()) {
@@ -369,7 +370,6 @@ class WebViewInteractiveTest
     size_t initial_widget_count_;
     content::RenderWidgetHost* last_render_widget_host_;
     scoped_refptr<content::MessageLoopRunner> message_loop_;
-    bool seen_new_widget_;
 
     DISALLOW_COPY_AND_ASSIGN(PopupCreatedObserver);
   };
@@ -379,8 +379,9 @@ class WebViewInteractiveTest
     popup_observer.Init();
     // Press alt+DOWN to open popup.
     bool alt = true;
-    content::SimulateKeyPress(
-        guest_web_contents(), ui::VKEY_DOWN, false, false, alt, false);
+    content::SimulateKeyPress(guest_web_contents(), ui::DomKey::ARROW_DOWN,
+                              ui::DomCode::ARROW_DOWN, ui::VKEY_DOWN, false,
+                              false, alt, false);
     popup_observer.Wait();
 
     content::RenderWidgetHost* popup_rwh =
@@ -395,8 +396,8 @@ class WebViewInteractiveTest
     LOG(INFO) << "DIFF: x = " << diff.x() << ", y = " << diff.y();
 
     const int left_spacing = 40 + padding.x();  // div.style.paddingLeft = 40px.
-    // div.style.paddingTop = 50px + (input box height = 26px).
-    const int top_spacing = 50 + 26 + padding.y();
+    // div.style.paddingTop = 60px + (input box height = 26px).
+    const int top_spacing = 60 + 26 + padding.y();
 
     // If the popup is placed within |threshold_px| of the expected position,
     // then we consider the test as a pass.
@@ -406,8 +407,9 @@ class WebViewInteractiveTest
     EXPECT_LE(std::abs(diff.y() - top_spacing), threshold_px);
 
     // Close the popup.
-    content::SimulateKeyPress(
-        guest_web_contents(), ui::VKEY_ESCAPE, false, false, false, false);
+    content::SimulateKeyPress(guest_web_contents(), ui::DomKey::ESCAPE,
+                              ui::DomCode::ESCAPE, ui::VKEY_ESCAPE, false,
+                              false, false, false);
   }
 
   void DragTestStep1() {
@@ -421,7 +423,7 @@ class WebViewInteractiveTest
 
     // Now wait a bit before moving mouse to initiate drag/drop.
     base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
-        FROM_HERE, base::Bind(&WebViewInteractiveTest::DragTestStep2,
+        FROM_HERE, base::Bind(&WebViewInteractiveTestBase::DragTestStep2,
                               base::Unretained(this)),
         base::TimeDelta::FromMilliseconds(200));
   }
@@ -474,7 +476,7 @@ class WebViewInteractiveTest
     passed_listener.set_failure_message("TEST_FAILED");
     content::SimulateMouseClickAt(guest_web_contents(),
                                   0,
-                                  blink::WebMouseEvent::ButtonLeft,
+                                  blink::WebMouseEvent::Button::Left,
                                   gfx::Point(20, 20));
     ASSERT_TRUE(passed_listener.WaitUntilSatisfied());
   }
@@ -491,6 +493,47 @@ class WebViewInteractiveTest
   std::string last_drop_data_;
 };
 
+// The following class of tests work for both OOPIF and non-OOPIF <webview>.
+class WebViewInteractiveTest : public WebViewInteractiveTestBase,
+                               public testing::WithParamInterface<bool> {
+ public:
+  void SetUpCommandLine(base::CommandLine* command_line) override {
+    WebViewInteractiveTestBase::SetUpCommandLine(command_line);
+
+    bool use_cross_process_frames_for_guests = GetParam();
+    if (use_cross_process_frames_for_guests) {
+      command_line->AppendSwitchASCII(
+          switches::kEnableFeatures,
+          ::features::kGuestViewCrossProcessFrames.name);
+    } else {
+      command_line->AppendSwitchASCII(
+          switches::kDisableFeatures,
+          ::features::kGuestViewCrossProcessFrames.name);
+    }
+  }
+};
+
+class WebViewNewWindowInteractiveTest : public WebViewInteractiveTest {};
+
+// The tests below aren't needed in --use-cross-process-frames-for-guests.
+class WebViewContextMenuInteractiveTest : public WebViewInteractiveTestBase {};
+
+// The following class of tests do not work for OOPIF <webview>.
+// TODO(ekaramad): Make this tests work with OOPIF and replace the test classes
+// with WebViewInteractiveTest (see crbug.com/582562).
+class WebViewFocusInteractiveTest : public WebViewInteractiveTestBase {};
+class WebViewPopupInteractiveTest : public WebViewInteractiveTestBase {};
+class WebViewPointerLockInteractiveTest : public WebViewInteractiveTestBase {};
+class WebViewDragDropInteractiveTest : public WebViewInteractiveTestBase {};
+
+INSTANTIATE_TEST_CASE_P(WebViewInteractiveTests,
+                        WebViewInteractiveTest,
+                        testing::Bool());
+
+INSTANTIATE_TEST_CASE_P(WebViewInteractiveTests,
+                        WebViewNewWindowInteractiveTest,
+                        testing::Bool());
+
 // ui_test_utils::SendMouseMoveSync doesn't seem to work on OS_MACOSX, and
 // likely won't work on many other platforms as well, so for now this test
 // is for Windows and Linux only. As of Sept 17th, 2013 this test is disabled
@@ -501,7 +544,8 @@ class WebViewInteractiveTest
 
 #if defined(OS_LINUX)
 // flaky http://crbug.com/412086
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, DISABLED_PointerLock) {
+IN_PROC_BROWSER_TEST_F(WebViewPointerLockInteractiveTest,
+                       DISABLED_PointerLock) {
   SetupTest("web_view/pointer_lock",
             "/extensions/platform_apps/web_view/pointer_lock/guest.html");
 
@@ -574,7 +618,8 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, DISABLED_PointerLock) {
 }
 
 // flaky http://crbug.com/412086
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, DISABLED_PointerLockFocus) {
+IN_PROC_BROWSER_TEST_F(WebViewPointerLockInteractiveTest,
+                       DISABLED_PointerLockFocus) {
   SetupTest("web_view/pointer_lock_focus",
             "/extensions/platform_apps/web_view/pointer_lock_focus/guest.html");
 
@@ -599,19 +644,20 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, DISABLED_PointerLockFocus) {
 
 // Tests that if a <webview> is focused before navigation then the guest starts
 // off focused.
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_FocusBeforeNavigation) {
+IN_PROC_BROWSER_TEST_F(WebViewFocusInteractiveTest,
+                       Focus_FocusBeforeNavigation) {
   TestHelper("testFocusBeforeNavigation", "web_view/focus", NO_TEST_SERVER);
 }
 
 // Tests that setting focus on the <webview> sets focus on the guest.
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_FocusEvent) {
+IN_PROC_BROWSER_TEST_F(WebViewFocusInteractiveTest, Focus_FocusEvent) {
   TestHelper("testFocusEvent", "web_view/focus", NO_TEST_SERVER);
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_FocusTracksEmbedder) {
+IN_PROC_BROWSER_TEST_F(WebViewFocusInteractiveTest, Focus_FocusTracksEmbedder) {
   content::WebContents* embedder_web_contents = NULL;
 
-  scoped_ptr<ExtensionTestMessageListener> done_listener(
+  std::unique_ptr<ExtensionTestMessageListener> done_listener(
       RunAppHelper("testFocusTracksEmbedder", "web_view/focus", NO_TEST_SERVER,
                    &embedder_web_contents));
   done_listener->WaitUntilSatisfied();
@@ -628,11 +674,11 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_FocusTracksEmbedder) {
   ASSERT_TRUE(next_step_listener.WaitUntilSatisfied());
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_AdvanceFocus) {
+IN_PROC_BROWSER_TEST_F(WebViewFocusInteractiveTest, Focus_AdvanceFocus) {
   content::WebContents* embedder_web_contents = NULL;
 
   {
-    scoped_ptr<ExtensionTestMessageListener> done_listener(
+    std::unique_ptr<ExtensionTestMessageListener> done_listener(
         RunAppHelper("testAdvanceFocus", "web_view/focus", NO_TEST_SERVER,
                      &embedder_web_contents));
     done_listener->WaitUntilSatisfied();
@@ -643,9 +689,10 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_AdvanceFocus) {
     listener.set_failure_message("TEST_FAILED");
     SimulateRWHMouseClick(
         embedder_web_contents->GetRenderViewHost()->GetWidget(),
-        blink::WebMouseEvent::ButtonLeft, 200, 20);
-    content::SimulateKeyPress(embedder_web_contents, ui::VKEY_TAB,
-        false, false, false, false);
+        blink::WebMouseEvent::Button::Left, 200, 20);
+    content::SimulateKeyPress(embedder_web_contents, ui::DomKey::TAB,
+                              ui::DomCode::TAB, ui::VKEY_TAB, false, false,
+                              false, false);
     ASSERT_TRUE(listener.WaitUntilSatisfied());
   }
 
@@ -659,23 +706,31 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_AdvanceFocus) {
     // element. The initial element is dependent upon tab direction which blink
     // does not propagate to the plugin.
     // See http://crbug.com/147644.
-    content::SimulateKeyPress(embedder_web_contents, ui::VKEY_TAB,
-        false, false, false, false);
-    content::SimulateKeyPress(embedder_web_contents, ui::VKEY_TAB,
-        false, false, false, false);
-    content::SimulateKeyPress(embedder_web_contents, ui::VKEY_TAB,
-        false, false, false, false);
+    content::SimulateKeyPress(embedder_web_contents, ui::DomKey::TAB,
+                              ui::DomCode::TAB, ui::VKEY_TAB, false, false,
+                              false, false);
+    content::SimulateKeyPress(embedder_web_contents, ui::DomKey::TAB,
+                              ui::DomCode::TAB, ui::VKEY_TAB, false, false,
+                              false, false);
+    content::SimulateKeyPress(embedder_web_contents, ui::DomKey::TAB,
+                              ui::DomCode::TAB, ui::VKEY_TAB, false, false,
+                              false, false);
     ASSERT_TRUE(listener.WaitUntilSatisfied());
   }
 }
 
 // Tests that blurring <webview> also blurs the guest.
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_BlurEvent) {
+IN_PROC_BROWSER_TEST_F(WebViewFocusInteractiveTest, Focus_BlurEvent) {
   TestHelper("testBlurEvent", "web_view/focus", NO_TEST_SERVER);
 }
 
 // Tests that guests receive edit commands and respond appropriately.
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, EditCommands) {
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest, EditCommands) {
+#if defined(OS_MACOSX)
+  // TODO(ekaramad): This test is failing under OOPIF in MAC.
+  if (GetParam())
+    return;
+#endif
   LoadAndLaunchPlatformApp("web_view/edit_commands", "connected");
 
   ASSERT_TRUE(ui_test_utils::ShowAndFocusNativeWindow(
@@ -693,7 +748,7 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, EditCommands) {
 
 // Tests that guests receive edit commands and respond appropriately.
 // http://crbug.com/417892
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, DISABLED_EditCommandsNoMenu) {
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest, DISABLED_EditCommandsNoMenu) {
   SetupTest("web_view/edit_commands_no_menu",
       "/extensions/platform_apps/web_view/edit_commands_no_menu/"
       "guest.html");
@@ -710,28 +765,27 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, DISABLED_EditCommandsNoMenu) {
   ASSERT_TRUE(start_of_line_listener.WaitUntilSatisfied());
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest,
                        NewWindow_AttachAfterOpenerDestroyed) {
   TestHelper("testNewWindowAttachAfterOpenerDestroyed",
              "web_view/newwindow",
              NEEDS_TEST_SERVER);
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest,
                        NewWindow_NewWindowNameTakesPrecedence) {
   TestHelper("testNewWindowNameTakesPrecedence",
              "web_view/newwindow",
              NEEDS_TEST_SERVER);
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest,
                        NewWindow_WebViewNameTakesPrecedence) {
-  TestHelper("testNewWindowWebViewNameTakesPrecedence",
-             "web_view/newwindow",
+  TestHelper("testNewWindowWebViewNameTakesPrecedence", "web_view/newwindow",
              NEEDS_TEST_SERVER);
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, NewWindow_NoName) {
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest, NewWindow_NoName) {
   TestHelper("testNewWindowNoName",
              "web_view/newwindow",
              NEEDS_TEST_SERVER);
@@ -743,38 +797,41 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, NewWindow_NoName) {
 #else
 #define MAYBE_NewWindow_Redirect NewWindow_Redirect
 #endif
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, MAYBE_NewWindow_Redirect) {
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest,
+                       MAYBE_NewWindow_Redirect) {
   TestHelper("testNewWindowRedirect",
              "web_view/newwindow",
              NEEDS_TEST_SERVER);
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, NewWindow_Close) {
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest, NewWindow_Close) {
   TestHelper("testNewWindowClose",
              "web_view/newwindow",
              NEEDS_TEST_SERVER);
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, NewWindow_DeferredAttachment) {
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest,
+                       NewWindow_DeferredAttachment) {
   TestHelper("testNewWindowDeferredAttachment",
              "web_view/newwindow",
              NEEDS_TEST_SERVER);
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, NewWindow_ExecuteScript) {
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest,
+                       NewWindow_ExecuteScript) {
   TestHelper("testNewWindowExecuteScript",
              "web_view/newwindow",
              NEEDS_TEST_SERVER);
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest,
                        NewWindow_DeclarativeWebRequest) {
   TestHelper("testNewWindowDeclarativeWebRequest",
              "web_view/newwindow",
              NEEDS_TEST_SERVER);
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest,
                        NewWindow_DiscardAfterOpenerDestroyed) {
   TestHelper("testNewWindowDiscardAfterOpenerDestroyed",
              "web_view/newwindow",
@@ -787,7 +844,8 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
 #else
 #define MAYBE_NewWindow_WebRequest NewWindow_WebRequest
 #endif
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, MAYBE_NewWindow_WebRequest) {
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest,
+                       MAYBE_NewWindow_WebRequest) {
   TestHelper("testNewWindowWebRequest",
              "web_view/newwindow",
              NEEDS_TEST_SERVER);
@@ -795,14 +853,14 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, MAYBE_NewWindow_WebRequest) {
 
 // A custom elements bug needs to be addressed to enable this test:
 // See http://crbug.com/282477 for more information.
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest,
                        DISABLED_NewWindow_WebRequestCloseWindow) {
   TestHelper("testNewWindowWebRequestCloseWindow",
              "web_view/newwindow",
              NEEDS_TEST_SERVER);
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest,
                        NewWindow_WebRequestRemoveElement) {
   TestHelper("testNewWindowWebRequestRemoveElement",
              "web_view/newwindow",
@@ -815,15 +873,13 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
 // See http://crbug.com/425859 for more information.
 #if !defined(OS_MACOSX)
 // Tests that Ctrl+Click/Cmd+Click on a link fires up the newwindow API.
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, NewWindow_OpenInNewTab) {
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest, NewWindow_OpenInNewTab) {
   content::WebContents* embedder_web_contents = NULL;
 
   ExtensionTestMessageListener loaded_listener("Loaded", false);
-  scoped_ptr<ExtensionTestMessageListener> done_listener(
-    RunAppHelper("testNewWindowOpenInNewTab",
-                 "web_view/newwindow",
-                 NEEDS_TEST_SERVER,
-                 &embedder_web_contents));
+  std::unique_ptr<ExtensionTestMessageListener> done_listener(
+      RunAppHelper("testNewWindowOpenInNewTab", "web_view/newwindow",
+                   NEEDS_TEST_SERVER, &embedder_web_contents));
 
   loaded_listener.WaitUntilSatisfied();
 #if defined(OS_MACOSX)
@@ -841,7 +897,7 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, NewWindow_OpenInNewTab) {
 }
 #endif
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_P(WebViewNewWindowInteractiveTest,
                        NewWindow_OpenerDestroyedWhileUnattached) {
   TestHelper("testNewWindowOpenerDestroyedWhileUnattached",
              "web_view/newwindow",
@@ -858,14 +914,21 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
 // Tests whether <webview> context menu sees <webview> local coordinates
 // in its RenderViewContextMenu params.
 // Local coordinates are required for plugin actions to work properly.
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, ContextMenuParamCoordinates) {
+//
+// This test is not needed in --use-cross-process-frames-for-guests,
+// since it tests that the <webview> sees local coordinates, which is not
+// true with the out-of-process iframes architecture. In that case, the
+// <webview> sees transformed coordinates, the point is transformed in
+// CrossProcessFrameConnector::TransformPointToRootCoordSpace.
+IN_PROC_BROWSER_TEST_F(WebViewContextMenuInteractiveTest,
+                       ContextMenuParamCoordinates) {
   TestHelper("testCoordinates", "web_view/context_menus/coordinates",
              NO_TEST_SERVER);
   ASSERT_TRUE(guest_web_contents());
 
   ContextMenuWaiter menu_observer(content::NotificationService::AllSources());
   SimulateRWHMouseClick(guest_web_contents()->GetRenderViewHost()->GetWidget(),
-                        blink::WebMouseEvent::ButtonRight, 10, 20);
+                        blink::WebMouseEvent::Button::Right, 10, 20);
   // Wait until the context menu is opened and closed.
   menu_observer.WaitForMenuOpenAndClose();
   ASSERT_EQ(10, menu_observer.params().x);
@@ -874,8 +937,15 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, ContextMenuParamCoordinates) {
 
 // Tests whether <webview> context menu sees <webview> local coordinates in its
 // RenderViewContextMenu params, when it is subject to CSS transforms.
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
-                       ContextMenuParamsAfterCSSTransforms) {
+//
+// This test doesn't makes sense in --use-cross-process-frames-for-guests, since
+// it tests that events forwarded from the embedder are properly transformed,
+// and in oopif-mode the events are sent directly to the child process without
+// the forwarding code path (relying on surface-based hittesting).
+
+// Flaky.  http://crbug.com/613258
+IN_PROC_BROWSER_TEST_F(WebViewContextMenuInteractiveTest,
+                       DISABLED_ContextMenuParamsAfterCSSTransforms) {
   LoadAndLaunchPlatformApp("web_view/context_menus/coordinates_with_transforms",
                            "Launched");
 
@@ -912,7 +982,7 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
     embedder_window_point -= embedder_view_bounds.OffsetFromOrigin();
     SimulateRWHMouseClick(
         embedder_web_contents()->GetRenderViewHost()->GetWidget(),
-        blink::WebMouseEvent::ButtonRight,
+        blink::WebMouseEvent::Button::Right,
         /* Using window coordinates for the embedder */
         embedder_window_point.x(), embedder_window_point.y());
 
@@ -922,7 +992,7 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
   }
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, ExecuteCode) {
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest, ExecuteCode) {
   ASSERT_TRUE(RunPlatformAppTestWithArg(
       "platform_apps/web_view/common", "execute_code")) << message_;
 }
@@ -933,7 +1003,8 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, ExecuteCode) {
 #else
 #define MAYBE_PopupPositioningBasic PopupPositioningBasic
 #endif
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, MAYBE_PopupPositioningBasic) {
+IN_PROC_BROWSER_TEST_F(WebViewPopupInteractiveTest,
+                       MAYBE_PopupPositioningBasic) {
   TestHelper("testBasic", "web_view/popup_positioning", NO_TEST_SERVER);
   ASSERT_TRUE(guest_web_contents());
   PopupTestHelper(gfx::Point());
@@ -942,16 +1013,17 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, MAYBE_PopupPositioningBasic) {
   // make sure we keep rendering popups correct in webview.
 }
 
-// Flaky on ChromeOS: http://crbug.com/526886
-// Causes problems on windows: http://crbug.com/544037
-#if defined(OS_CHROMEOS) || defined(OS_WIN)
+// Flaky on ChromeOS and Linux: http://crbug.com/526886
+// Causes problems on windows: http://crbug.com/544998
+#if defined(OS_CHROMEOS) || defined(OS_WIN) || defined(OS_LINUX)
 #define MAYBE_PopupPositioningMoved DISABLED_PopupPositioningMoved
 #else
 #define MAYBE_PopupPositioningMoved PopupPositioningMoved
 #endif
 // Tests that moving browser plugin (without resize/UpdateRects) correctly
 // repositions popup.
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, MAYBE_PopupPositioningMoved) {
+IN_PROC_BROWSER_TEST_F(WebViewPopupInteractiveTest,
+                       MAYBE_PopupPositioningMoved) {
   TestHelper("testMoved", "web_view/popup_positioning", NO_TEST_SERVER);
   ASSERT_TRUE(guest_web_contents());
   PopupTestHelper(gfx::Point(20, 0));
@@ -961,7 +1033,7 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, MAYBE_PopupPositioningMoved) {
 // but the tests don't work on anything except chromeos for now. This is because
 // of simulating mouse drag code's dependency on platforms.
 #if defined(OS_CHROMEOS) && !defined(USE_OZONE)
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, DragDropWithinWebView) {
+IN_PROC_BROWSER_TEST_F(WebViewDragDropInteractiveTest, DragDropWithinWebView) {
   LoadAndLaunchPlatformApp("web_view/dnd_within_webview", "connected");
   ASSERT_TRUE(ui_test_utils::ShowAndFocusNativeWindow(GetPlatformAppWindow()));
 
@@ -979,10 +1051,9 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, DragDropWithinWebView) {
   for (;;) {
     base::RunLoop run_loop;
     quit_closure_ = run_loop.QuitClosure();
-    base::MessageLoop::current()->PostTask(
-        FROM_HERE,
-        base::Bind(&WebViewInteractiveTest::DragTestStep1,
-                   base::Unretained(this)));
+    base::ThreadTaskRunnerHandle::Get()->PostTask(
+        FROM_HERE, base::Bind(&WebViewInteractiveTestBase::DragTestStep1,
+                              base::Unretained(this)));
     run_loop.Run();
 
     if (last_drop_data_ == "Drop me")
@@ -1000,11 +1071,11 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, DragDropWithinWebView) {
 }
 #endif  // (defined(OS_CHROMEOS))
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Navigation) {
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest, Navigation) {
   TestHelper("testNavigation", "web_view/navigation", NO_TEST_SERVER);
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Navigation_BackForwardKeys) {
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest, Navigation_BackForwardKeys) {
   LoadAndLaunchPlatformApp("web_view/navigation", "Launched");
   ASSERT_TRUE(ui_test_utils::ShowAndFocusNativeWindow(
       GetPlatformAppWindow()));
@@ -1035,7 +1106,7 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Navigation_BackForwardKeys) {
   ASSERT_TRUE(done_listener.WaitUntilSatisfied());
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_F(WebViewPointerLockInteractiveTest,
                        PointerLock_PointerLockLostWithFocus) {
   TestHelper("testPointerLockLostWithFocus",
              "web_view/pointerlock",
@@ -1052,25 +1123,25 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
 //
 // In addition to the above, these tests are flaky on many platforms:
 // http://crbug.com/468660
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest,
                        DISABLED_FullscreenAllow_EmbedderHasPermission) {
   FullscreenTestHelper("testFullscreenAllow",
                        "web_view/fullscreen/embedder_has_permission");
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest,
                        DISABLED_FullscreenDeny_EmbedderHasPermission) {
   FullscreenTestHelper("testFullscreenDeny",
                        "web_view/fullscreen/embedder_has_permission");
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest,
                        DISABLED_FullscreenAllow_EmbedderHasNoPermission) {
   FullscreenTestHelper("testFullscreenAllow",
                        "web_view/fullscreen/embedder_has_no_permission");
 }
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest,
                        DISABLED_FullscreenDeny_EmbedderHasNoPermission) {
   FullscreenTestHelper("testFullscreenDeny",
                        "web_view/fullscreen/embedder_has_no_permission");
@@ -1084,7 +1155,7 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest,
 // Now we need to make sure TextInputTypeChanged fires properly for the guest's
 // view upon step #3. We simply read the input type's state after #3 to
 // make sure it's not TEXT_INPUT_TYPE_NONE.
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_FocusRestored) {
+IN_PROC_BROWSER_TEST_F(WebViewFocusInteractiveTest, Focus_FocusRestored) {
   TestHelper("testFocusRestored", "web_view/focus", NO_TEST_SERVER);
   content::WebContents* embedder_web_contents = GetFirstAppWindowWebContents();
   ASSERT_TRUE(embedder_web_contents);
@@ -1096,7 +1167,7 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_FocusRestored) {
   {
     content::SimulateMouseClickAt(guest_web_contents(),
                                   0,
-                                  blink::WebMouseEvent::ButtonLeft,
+                                  blink::WebMouseEvent::Button::Left,
                                   gfx::Point(10, 10));
     EXPECT_TRUE(content::ExecuteScript(
                     embedder_web_contents,
@@ -1111,7 +1182,7 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_FocusRestored) {
   {
     content::SimulateMouseClickAt(embedder_web_contents,
                                   0,
-                                  blink::WebMouseEvent::ButtonLeft,
+                                  blink::WebMouseEvent::Button::Left,
                                   gfx::Point(200, 20));
     EXPECT_TRUE(content::ExecuteScript(
                     embedder_web_contents,
@@ -1126,7 +1197,7 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_FocusRestored) {
   {
     content::SimulateMouseClickAt(guest_web_contents(),
                                   0,
-                                  blink::WebMouseEvent::ButtonLeft,
+                                  blink::WebMouseEvent::Button::Left,
                                   gfx::Point(10, 10));
     EXPECT_TRUE(content::ExecuteScript(
                     embedder_web_contents,
@@ -1150,9 +1221,9 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, Focus_FocusRestored) {
 
 // ui::TextInputClient is NULL for mac and android.
 #if !defined(OS_MACOSX) && !defined(OS_ANDROID)
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, DISABLED_Focus_InputMethod) {
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest, DISABLED_Focus_InputMethod) {
   content::WebContents* embedder_web_contents = NULL;
-  scoped_ptr<ExtensionTestMessageListener> done_listener(
+  std::unique_ptr<ExtensionTestMessageListener> done_listener(
       RunAppHelper("testInputMethod", "web_view/focus", NO_TEST_SERVER,
                    &embedder_web_contents));
   ASSERT_TRUE(done_listener->WaitUntilSatisfied());
@@ -1225,7 +1296,12 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, DISABLED_Focus_InputMethod) {
 #endif
 
 #if defined(OS_MACOSX)
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, TextSelection) {
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest, TextSelection) {
+#if defined(OS_MACOSX)
+  // TODO(ekaramad): This test is failing under OOPIF for MAC.
+  if (GetParam())
+    return;
+#endif
   SetupTest("web_view/text_selection",
             "/extensions/platform_apps/web_view/text_selection/guest.html");
   ASSERT_TRUE(guest_web_contents());
@@ -1236,7 +1312,7 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, TextSelection) {
   ExtensionTestMessageListener ctx_listener("MSG_CONTEXTMENU", false);
   ContextMenuNotificationObserver menu_observer(IDC_CONTENT_CONTEXT_COPY);
   SimulateRWHMouseClick(guest_web_contents()->GetRenderViewHost()->GetWidget(),
-                        blink::WebMouseEvent::ButtonRight, 20, 20);
+                        blink::WebMouseEvent::Button::Right, 20, 20);
   ASSERT_TRUE(ctx_listener.WaitUntilSatisfied());
 
   // Now verify that the selection text propagates properly to RWHV.
@@ -1249,7 +1325,7 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, TextSelection) {
 }
 #endif
 
-IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, FocusAndVisibility) {
+IN_PROC_BROWSER_TEST_F(WebViewFocusInteractiveTest, FocusAndVisibility) {
   ASSERT_TRUE(StartEmbeddedTestServer());
   LoadAndLaunchPlatformApp("web_view/focus_visibility",
                            "WebViewInteractiveTest.LOADED");
@@ -1287,3 +1363,104 @@ IN_PROC_BROWSER_TEST_F(WebViewInteractiveTest, FocusAndVisibility) {
   SendMessageToEmbedder("verify");
   EXPECT_TRUE(webview_button_not_focused_listener.WaitUntilSatisfied());
 }
+
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest, KeyboardFocusSimple) {
+  TestHelper("testKeyboardFocusSimple", "web_view/focus", NO_TEST_SERVER);
+
+  EXPECT_EQ(embedder_web_contents()->GetFocusedFrame(),
+            embedder_web_contents()->GetMainFrame());
+  ExtensionTestMessageListener next_step_listener("TEST_STEP_PASSED", false);
+  next_step_listener.set_failure_message("TEST_STEP_FAILED");
+  {
+    gfx::Rect offset = embedder_web_contents()->GetContainerBounds();
+    // Click the <input> element inside the <webview>.
+    // If we wanted, we could ask the embedder to compute an appropriate point.
+    MoveMouseInsideWindow(gfx::Point(offset.x() + 40, offset.y() + 40));
+    SendMouseClick(ui_controls::LEFT);
+  }
+
+  // Waits for the renderer to know the input has focus.
+  ASSERT_TRUE(next_step_listener.WaitUntilSatisfied());
+
+  ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+      GetPlatformAppWindow(), ui::VKEY_A, false, false, false, false));
+  ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+      GetPlatformAppWindow(), ui::VKEY_B, false, true, false, false));
+  ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+      GetPlatformAppWindow(), ui::VKEY_C, false, false, false, false));
+
+  next_step_listener.Reset();
+  EXPECT_TRUE(content::ExecuteScript(
+      embedder_web_contents(),
+      "window.runCommand('testKeyboardFocusRunNextStep', 'aBc');"));
+
+  ASSERT_TRUE(next_step_listener.WaitUntilSatisfied());
+}
+
+// Ensures that input is routed to the webview after the containing window loses
+// and regains focus. Additionally, the webview does not process keypresses sent
+// while another window is focused.
+// http://crbug.com/660044.
+IN_PROC_BROWSER_TEST_P(WebViewInteractiveTest, KeyboardFocusWindowCycle) {
+  TestHelper("testKeyboardFocusWindowFocusCycle", "web_view/focus",
+             NO_TEST_SERVER);
+
+  EXPECT_EQ(embedder_web_contents()->GetFocusedFrame(),
+            embedder_web_contents()->GetMainFrame());
+  ExtensionTestMessageListener next_step_listener("TEST_STEP_PASSED", false);
+  next_step_listener.set_failure_message("TEST_STEP_FAILED");
+  {
+    gfx::Rect offset = embedder_web_contents()->GetContainerBounds();
+    // Click the <input> element inside the <webview>.
+    // If we wanted, we could ask the embedder to compute an appropriate point.
+    MoveMouseInsideWindow(gfx::Point(offset.x() + 40, offset.y() + 40));
+    SendMouseClick(ui_controls::LEFT);
+  }
+
+  // Waits for the renderer to know the input has focus.
+  ASSERT_TRUE(next_step_listener.WaitUntilSatisfied());
+
+  ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+      GetPlatformAppWindow(), ui::VKEY_A, false, false, false, false));
+  ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+      GetPlatformAppWindow(), ui::VKEY_B, false, true, false, false));
+  ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+      GetPlatformAppWindow(), ui::VKEY_C, false, false, false, false));
+
+  const extensions::Extension* extension =
+      LoadAndLaunchPlatformApp("minimal", "Launched");
+  extensions::AppWindow* window = GetFirstAppWindowForApp(extension->id());
+  EXPECT_TRUE(content::ExecuteScript(
+      embedder_web_contents(),
+      "window.runCommand('monitorGuestEvent', 'focus');"));
+
+  ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+      GetPlatformAppWindow(), ui::VKEY_F, false, false, false, false));
+  ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+      GetPlatformAppWindow(), ui::VKEY_O, false, true, false, false));
+  ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+      GetPlatformAppWindow(), ui::VKEY_O, false, true, false, false));
+
+  // Close the other window and wait for the webview to regain focus.
+  CloseAppWindow(window);
+  ASSERT_TRUE(ui_test_utils::ShowAndFocusNativeWindow(GetPlatformAppWindow()));
+  next_step_listener.Reset();
+  EXPECT_TRUE(
+      content::ExecuteScript(embedder_web_contents(),
+                             "window.runCommand('waitGuestEvent', 'focus');"));
+  ASSERT_TRUE(next_step_listener.WaitUntilSatisfied());
+
+  ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+      GetPlatformAppWindow(), ui::VKEY_X, false, false, false, false));
+  ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+      GetPlatformAppWindow(), ui::VKEY_Y, false, true, false, false));
+  ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+      GetPlatformAppWindow(), ui::VKEY_Z, false, false, false, false));
+
+  next_step_listener.Reset();
+  EXPECT_TRUE(content::ExecuteScript(
+      embedder_web_contents(),
+      "window.runCommand('testKeyboardFocusRunNextStep', 'aBcxYz');"));
+
+  ASSERT_TRUE(next_step_listener.WaitUntilSatisfied());
+}
