@@ -4,20 +4,19 @@
 
 #include "chrome/browser/background/background_contents_service.h"
 
+#include <utility>
+
 #include "apps/app_load_service.h"
 #include "base/bind.h"
 #include "base/command_line.h"
 #include "base/compiler_specific.h"
 #include "base/location.h"
 #include "base/macros.h"
-#include "base/message_loop/message_loop.h"
 #include "base/metrics/histogram_macros.h"
-#include "base/prefs/pref_service.h"
-#include "base/prefs/scoped_user_pref_update.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/time/time.h"
 #include "base/values.h"
 #include "chrome/browser/background/background_contents_service_factory.h"
@@ -32,10 +31,11 @@
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_finder.h"
 #include "chrome/browser/ui/browser_tabstrip.h"
-#include "chrome/browser/ui/host_desktop.h"
 #include "chrome/common/extensions/extension_constants.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/grit/generated_resources.h"
+#include "components/prefs/pref_service.h"
+#include "components/prefs/scoped_user_pref_update.h"
 #include "content/public/browser/notification_service.h"
 #include "content/public/browser/site_instance.h"
 #include "content/public/browser/web_contents.h"
@@ -50,6 +50,7 @@
 #include "extensions/common/extension_set.h"
 #include "extensions/common/manifest_handlers/background_info.h"
 #include "extensions/common/manifest_handlers/icons_handler.h"
+#include "extensions/common/one_shot_event.h"
 #include "extensions/grit/extensions_browser_resources.h"
 #include "ipc/ipc_message.h"
 #include "ui/base/l10n/l10n_util.h"
@@ -164,6 +165,9 @@ void NotificationImageReady(
     scoped_refptr<CrashNotificationDelegate> delegate,
     Profile* profile,
     const gfx::Image& icon) {
+  if (g_browser_process->IsShuttingDown())
+    return;
+
   gfx::Image notification_icon(icon);
   if (notification_icon.IsEmpty()) {
     ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
@@ -260,12 +264,26 @@ void ReloadExtension(const std::string& extension_id, Profile* profile) {
 const char kUrlKey[] = "url";
 const char kFrameNameKey[] = "name";
 
+// Defines the backoff policy used for attempting to reload extensions.
+const net::BackoffEntry::Policy kExtensionReloadBackoffPolicy = {
+  0,                // Initial errors to ignore before applying backoff.
+  3000,             // Initial delay: 3 seconds.
+  2,                // Multiply factor.
+  0.1,              // Fuzzing percentage.
+  -1,               // Maximum backoff time: -1 for no maximum.
+  -1,               // Entry lifetime: -1 to never discard.
+  false,            // Whether to always use initial delay. No-op as there are
+                    // no initial errors to ignore.
+};
+
 int BackgroundContentsService::restart_delay_in_ms_ = 3000;  // 3 seconds.
 
 BackgroundContentsService::BackgroundContentsService(
     Profile* profile,
     const base::CommandLine* command_line)
-    : prefs_(NULL), extension_registry_observer_(this) {
+    : prefs_(nullptr),
+      extension_registry_observer_(this),
+      weak_ptr_factory_(this) {
   // Don't load/store preferences if the parent profile is incognito.
   if (!profile->IsOffTheRecord())
     prefs_ = profile->GetPrefs();
@@ -320,9 +338,9 @@ BackgroundContentsService::GetBackgroundContents() const
 
 void BackgroundContentsService::StartObserving(Profile* profile) {
   // On startup, load our background pages after extension-apps have loaded.
-  registrar_.Add(this,
-                 extensions::NOTIFICATION_EXTENSIONS_READY_DEPRECATED,
-                 content::Source<Profile>(profile));
+  extensions::ExtensionSystem::Get(profile)->ready().Post(
+      FROM_HERE, base::Bind(&BackgroundContentsService::OnExtensionSystemReady,
+                            weak_ptr_factory_.GetWeakPtr(), profile));
 
   // Track the lifecycle of all BackgroundContents in the system to allow us
   // to store an up-to-date list of the urls. Start tracking contents when they
@@ -353,21 +371,19 @@ void BackgroundContentsService::StartObserving(Profile* profile) {
   extension_registry_observer_.Add(extensions::ExtensionRegistry::Get(profile));
 }
 
+void BackgroundContentsService::OnExtensionSystemReady(Profile* profile) {
+  SCOPED_UMA_HISTOGRAM_TIMER("Extensions.BackgroundContentsServiceStartupTime");
+  LoadBackgroundContentsFromManifests(profile);
+  LoadBackgroundContentsFromPrefs(profile);
+  SendChangeNotification(profile);
+}
+
 void BackgroundContentsService::Observe(
     int type,
     const content::NotificationSource& source,
     const content::NotificationDetails& details) {
   TRACE_EVENT0("browser,startup", "BackgroundContentsService::Observe");
   switch (type) {
-    case extensions::NOTIFICATION_EXTENSIONS_READY_DEPRECATED: {
-      SCOPED_UMA_HISTOGRAM_TIMER(
-          "Extensions.BackgroundContentsServiceStartupTime");
-      Profile* profile = content::Source<Profile>(source).ptr();
-      LoadBackgroundContentsFromManifests(profile);
-      LoadBackgroundContentsFromPrefs(profile);
-      SendChangeNotification(profile);
-      break;
-    }
     case chrome::NOTIFICATION_BACKGROUND_CONTENTS_DELETED:
       BackgroundContentsShutdown(
           content::Details<BackgroundContents>(details).ptr());
@@ -463,6 +479,23 @@ void BackgroundContentsService::OnExtensionLoaded(
     }
   }
 
+  // If there is an existing BackoffEntry for the extension, clear it if
+  // the component extension stays loaded for 60 seconds. This avoids the
+  // situation of effectively disabling an extension for the entire browser
+  // session if there was a periodic crash (sometimes caused by another source).
+  if (extensions::Manifest::IsComponentLocation(extension->location())) {
+    ComponentExtensionBackoffEntryMap::const_iterator it =
+        component_backoff_map_.find(extension->id());
+    if (it != component_backoff_map_.end()) {
+      net::BackoffEntry* entry = component_backoff_map_[extension->id()].get();
+      base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(FROM_HERE,
+          base::Bind(&BackgroundContentsService::MaybeClearBackoffEntry,
+              weak_ptr_factory_.GetWeakPtr(), extension->id(),
+              entry->failure_count()),
+          base::TimeDelta::FromSeconds(60));
+    }
+  }
+
   // Close the crash notification balloon for the app/extension, if any.
   ScheduleCloseBalloon(extension->id(), profile);
   SendChangeNotification(profile);
@@ -520,9 +553,33 @@ void BackgroundContentsService::OnExtensionUninstalled(
 void BackgroundContentsService::RestartForceInstalledExtensionOnCrash(
     const Extension* extension,
     Profile* profile) {
-  base::MessageLoop::current()->PostDelayedTask(FROM_HERE,
-      base::Bind(&ReloadExtension, extension->id(), profile),
-      base::TimeDelta::FromMilliseconds(restart_delay_in_ms_));
+  int restart_delay = restart_delay_in_ms_;
+
+  // If the extension was a component extension, use exponential backoff when
+  // attempting to reload.
+  if (extensions::Manifest::IsComponentLocation(extension->location())) {
+    ComponentExtensionBackoffEntryMap::const_iterator it =
+        component_backoff_map_.find(extension->id());
+
+    // Create a BackoffEntry if this is the first time we try to reload this
+    // particular extension.
+    if (it == component_backoff_map_.end()) {
+      std::unique_ptr<net::BackoffEntry> backoff_entry(
+          new net::BackoffEntry(&kExtensionReloadBackoffPolicy));
+      component_backoff_map_.insert(
+          std::pair<extensions::ExtensionId,
+                    std::unique_ptr<net::BackoffEntry>>(
+              extension->id(), std::move(backoff_entry)));
+    }
+
+    net::BackoffEntry* entry = component_backoff_map_[extension->id()].get();
+    entry->InformOfRequest(false);
+    restart_delay = entry->GetTimeUntilRelease().InMilliseconds();
+  }
+
+  base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+      FROM_HERE, base::Bind(&ReloadExtension, extension->id(), profile),
+      base::TimeDelta::FromMilliseconds(restart_delay));
 }
 
 // Loads all background contents whose urls have been stored in prefs.
@@ -565,6 +622,22 @@ void BackgroundContentsService::SendChangeNotification(Profile* profile) {
       content::Details<BackgroundContentsService>(this));
 }
 
+void BackgroundContentsService::MaybeClearBackoffEntry(
+    const std::string extension_id,
+    int expected_failure_count) {
+  ComponentExtensionBackoffEntryMap::const_iterator it =
+      component_backoff_map_.find(extension_id);
+  if (it == component_backoff_map_.end())
+    return;
+
+  net::BackoffEntry* entry = component_backoff_map_[extension_id].get();
+
+  // Only remove the BackoffEntry if there has has been no failure for
+  // |extension_id| since loading.
+  if (entry->failure_count() == expected_failure_count)
+    component_backoff_map_.erase(it);
+}
+
 void BackgroundContentsService::LoadBackgroundContentsForExtension(
     Profile* profile,
     const std::string& extension_id) {
@@ -649,7 +722,7 @@ void BackgroundContentsService::LoadBackgroundContents(
 }
 
 BackgroundContents* BackgroundContentsService::CreateBackgroundContents(
-    SiteInstance* site,
+    scoped_refptr<SiteInstance> site,
     int32_t routing_id,
     int32_t main_frame_route_id,
     int32_t main_frame_widget_route_id,
@@ -658,9 +731,10 @@ BackgroundContents* BackgroundContentsService::CreateBackgroundContents(
     const base::string16& application_id,
     const std::string& partition_id,
     content::SessionStorageNamespace* session_storage_namespace) {
-  BackgroundContents* contents = new BackgroundContents(
-      site, routing_id, main_frame_route_id, main_frame_widget_route_id, this,
-      partition_id, session_storage_namespace);
+  BackgroundContents* contents =
+      new BackgroundContents(std::move(site), routing_id, main_frame_route_id,
+                             main_frame_widget_route_id, this, partition_id,
+                             session_storage_namespace);
 
   // Register the BackgroundContents internally, then send out a notification
   // to external listeners.
@@ -784,8 +858,7 @@ void BackgroundContentsService::AddWebContents(
     bool user_gesture,
     bool* was_blocked) {
   Browser* browser = chrome::FindLastActiveWithProfile(
-      Profile::FromBrowserContext(new_contents->GetBrowserContext()),
-      chrome::GetActiveDesktop());
+      Profile::FromBrowserContext(new_contents->GetBrowserContext()));
   if (browser) {
     chrome::AddWebContents(browser, NULL, new_contents, disposition,
                            initial_rect, user_gesture, was_blocked);
