@@ -4,7 +4,6 @@
 
 #include "chrome/browser/browser_process_platform_part_chromeos.h"
 
-#include "base/command_line.h"
 #include "base/logging.h"
 #include "base/time/default_tick_clock.h"
 #include "base/time/tick_clock.h"
@@ -19,8 +18,10 @@
 #include "chrome/browser/chromeos/system/device_disabling_manager.h"
 #include "chrome/browser/chromeos/system/device_disabling_manager_default_delegate.h"
 #include "chrome/browser/chromeos/system/system_clock.h"
+#include "chrome/browser/chromeos/system/timezone_resolver_manager.h"
 #include "chrome/browser/chromeos/system/timezone_util.h"
-#include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/lifetime/keep_alive_types.h"
+#include "chrome/browser/lifetime/scoped_keep_alive.h"
 #include "chrome/common/chrome_switches.h"
 #include "chromeos/geolocation/simple_geolocation_provider.h"
 #include "chromeos/timezone/timezone_resolver.h"
@@ -28,17 +29,15 @@
 #include "components/user_manager/user_manager.h"
 
 BrowserProcessPlatformPart::BrowserProcessPlatformPart()
-    : created_profile_helper_(false) {
-}
+    : created_profile_helper_(false) {}
 
-BrowserProcessPlatformPart::~BrowserProcessPlatformPart() {
-}
+BrowserProcessPlatformPart::~BrowserProcessPlatformPart() {}
 
 void BrowserProcessPlatformPart::InitializeAutomaticRebootManager() {
   DCHECK(!automatic_reboot_manager_);
 
   automatic_reboot_manager_.reset(new chromeos::system::AutomaticRebootManager(
-      scoped_ptr<base::TickClock>(new base::DefaultTickClock)));
+      std::unique_ptr<base::TickClock>(new base::DefaultTickClock)));
 }
 
 void BrowserProcessPlatformPart::ShutdownAutomaticRebootManager() {
@@ -73,22 +72,24 @@ void BrowserProcessPlatformPart::ShutdownDeviceDisablingManager() {
   device_disabling_manager_delegate_.reset();
 }
 
-void BrowserProcessPlatformPart::InitializeSessionManager(
-    const base::CommandLine& parsed_command_line,
-    Profile* profile,
-    bool is_running_test) {
+void BrowserProcessPlatformPart::InitializeSessionManager() {
   DCHECK(!session_manager_);
-  session_manager_ = chromeos::ChromeSessionManager::CreateSessionManager(
-      parsed_command_line, profile, is_running_test);
+  session_manager_ = base::MakeUnique<chromeos::ChromeSessionManager>();
 }
 
 void BrowserProcessPlatformPart::ShutdownSessionManager() {
   session_manager_.reset();
 }
 
-session_manager::SessionManager* BrowserProcessPlatformPart::SessionManager() {
-  DCHECK(CalledOnValidThread());
-  return session_manager_.get();
+void BrowserProcessPlatformPart::RegisterKeepAlive() {
+  DCHECK(!keep_alive_);
+  keep_alive_.reset(
+      new ScopedKeepAlive(KeepAliveOrigin::BROWSER_PROCESS_CHROMEOS,
+                          KeepAliveRestartOption::DISABLED));
+}
+
+void BrowserProcessPlatformPart::UnregisterKeepAlive() {
+  keep_alive_.reset();
 }
 
 chromeos::ProfileHelper* BrowserProcessPlatformPart::profile_helper() {
@@ -104,9 +105,19 @@ BrowserProcessPlatformPart::browser_policy_connector_chromeos() {
       g_browser_process->browser_policy_connector());
 }
 
+chromeos::system::TimeZoneResolverManager*
+BrowserProcessPlatformPart::GetTimezoneResolverManager() {
+  if (!timezone_resolver_manager_.get()) {
+    timezone_resolver_manager_.reset(
+        new chromeos::system::TimeZoneResolverManager());
+  }
+  return timezone_resolver_manager_.get();
+}
+
 chromeos::TimeZoneResolver* BrowserProcessPlatformPart::GetTimezoneResolver() {
   if (!timezone_resolver_.get()) {
     timezone_resolver_.reset(new chromeos::TimeZoneResolver(
+        GetTimezoneResolverManager(),
         g_browser_process->system_request_context(),
         chromeos::SimpleGeolocationProvider::DefaultGeolocationProviderURL(),
         base::Bind(&chromeos::system::ApplyTimeZone),
@@ -118,12 +129,6 @@ chromeos::TimeZoneResolver* BrowserProcessPlatformPart::GetTimezoneResolver() {
   return timezone_resolver_.get();
 }
 
-chromeos::system::SystemClock* BrowserProcessPlatformPart::GetSystemClock() {
-  if (!system_clock_.get())
-    system_clock_.reset(new chromeos::system::SystemClock());
-
-  return system_clock_.get();
-}
 void BrowserProcessPlatformPart::StartTearDown() {
   // interactive_ui_tests check for memory leaks before this object is
   // destroyed.  So we need to destroy |timezone_resolver_| here.
@@ -131,14 +136,24 @@ void BrowserProcessPlatformPart::StartTearDown() {
   profile_helper_.reset();
 }
 
-scoped_ptr<policy::BrowserPolicyConnector>
+std::unique_ptr<policy::BrowserPolicyConnector>
 BrowserProcessPlatformPart::CreateBrowserPolicyConnector() {
-  return scoped_ptr<policy::BrowserPolicyConnector>(
+  return std::unique_ptr<policy::BrowserPolicyConnector>(
       new policy::BrowserPolicyConnectorChromeOS());
 }
 
+chromeos::system::SystemClock* BrowserProcessPlatformPart::GetSystemClock() {
+  if (!system_clock_.get())
+    system_clock_.reset(new chromeos::system::SystemClock());
+  return system_clock_.get();
+}
+
+void BrowserProcessPlatformPart::DestroySystemClock() {
+  system_clock_.reset();
+}
+
 void BrowserProcessPlatformPart::CreateProfileHelper() {
-  DCHECK(!created_profile_helper_ && profile_helper_.get() == NULL);
+  DCHECK(!created_profile_helper_ && !profile_helper_);
   created_profile_helper_ = true;
   profile_helper_.reset(new chromeos::ProfileHelper());
 }
