@@ -9,6 +9,7 @@
 // inline code (which shouldn't be inline).
 
 #include <list>
+#include <memory>
 #include <string>
 #include <vector>
 
@@ -28,6 +29,7 @@
 #include "chrome/browser/browsing_data/browsing_data_service_worker_helper.h"
 #include "chrome/browser/browsing_data/local_data_container.h"
 #include "components/content_settings/core/common/content_settings.h"
+#include "extensions/features/features.h"
 #include "net/ssl/channel_id_store.h"
 #include "ui/base/models/tree_node_model.h"
 
@@ -52,6 +54,8 @@ class CookieTreeIndexedDBNode;
 class CookieTreeIndexedDBsNode;
 class CookieTreeLocalStorageNode;
 class CookieTreeLocalStoragesNode;
+class CookieTreeMediaLicenseNode;
+class CookieTreeMediaLicensesNode;
 class CookieTreeQuotaNode;
 class CookieTreeServiceWorkerNode;
 class CookieTreeServiceWorkersNode;
@@ -108,9 +112,12 @@ class CookieTreeNode : public ui::TreeNode<CookieTreeNode> {
       TYPE_CACHE_STORAGES,    // This is used for CookieTreeCacheStoragesNode.
       TYPE_CACHE_STORAGE,     // This is used for CookieTreeCacheStorageNode.
       TYPE_FLASH_LSO,         // This is used for CookieTreeFlashLSONode.
+      TYPE_MEDIA_LICENSES,    // This is used for CookieTreeMediaLicensesNode.
+      TYPE_MEDIA_LICENSE,     // This is used for CookieTreeMediaLicenseNode.
     };
 
     DetailedInfo();
+    DetailedInfo(const DetailedInfo& other);
     ~DetailedInfo();
 
     DetailedInfo& Init(NodeType type);
@@ -139,6 +146,9 @@ class CookieTreeNode : public ui::TreeNode<CookieTreeNode> {
     DetailedInfo& InitCacheStorage(
         const content::CacheStorageUsageInfo* cache_storage_info);
     DetailedInfo& InitFlashLSO(const std::string& flash_lso_domain);
+    DetailedInfo& InitMediaLicense(
+        const BrowsingDataMediaLicenseHelper::MediaLicenseInfo*
+            media_license_info);
 
     NodeType node_type;
     GURL origin;
@@ -157,6 +167,8 @@ class CookieTreeNode : public ui::TreeNode<CookieTreeNode> {
     const content::ServiceWorkerUsageInfo* service_worker_info = nullptr;
     const content::CacheStorageUsageInfo* cache_storage_info = nullptr;
     std::string flash_lso_domain;
+    const BrowsingDataMediaLicenseHelper::MediaLicenseInfo* media_license_info =
+        nullptr;
   };
 
   CookieTreeNode() {}
@@ -176,7 +188,7 @@ class CookieTreeNode : public ui::TreeNode<CookieTreeNode> {
   virtual DetailedInfo GetDetailedInfo() const = 0;
 
  protected:
-  void AddChildSortedByTitle(CookieTreeNode* new_child);
+  void AddChildSortedByTitle(std::unique_ptr<CookieTreeNode> new_child);
 
  private:
   DISALLOW_COPY_AND_ASSIGN(CookieTreeNode);
@@ -227,6 +239,7 @@ class CookieTreeHostNode : public CookieTreeNode {
   CookieTreeQuotaNode* UpdateOrCreateQuotaNode(
       std::list<BrowsingDataQuotaHelper::QuotaInfo>::iterator quota_info);
   CookieTreeFlashLSONode* GetOrCreateFlashLSONode(const std::string& domain);
+  CookieTreeMediaLicensesNode* GetOrCreateMediaLicensesNode();
 
   std::string canonicalized_host() const { return canonicalized_host_; }
 
@@ -258,6 +271,7 @@ class CookieTreeHostNode : public CookieTreeNode {
   CookieTreeServiceWorkersNode* service_workers_child_ = nullptr;
   CookieTreeCacheStoragesNode* cache_storages_child_ = nullptr;
   CookieTreeFlashLSONode* flash_lso_child_ = nullptr;
+  CookieTreeMediaLicensesNode* media_licenses_child_ = nullptr;
 
   // The URL for which this node was initially created.
   GURL url_;
@@ -297,8 +311,8 @@ class CookieTreeCookiesNode : public CookieTreeNode {
 
   DetailedInfo GetDetailedInfo() const override;
 
-  void AddCookieNode(CookieTreeCookieNode* child) {
-    AddChildSortedByTitle(child);
+  void AddCookieNode(std::unique_ptr<CookieTreeCookieNode> child) {
+    AddChildSortedByTitle(std::move(child));
   }
 
  private:
@@ -333,8 +347,8 @@ class CookieTreeAppCachesNode : public CookieTreeNode {
 
   DetailedInfo GetDetailedInfo() const override;
 
-  void AddAppCacheNode(CookieTreeAppCacheNode* child) {
-    AddChildSortedByTitle(child);
+  void AddAppCacheNode(std::unique_ptr<CookieTreeAppCacheNode> child) {
+    AddChildSortedByTitle(std::move(child));
   }
 
  private:
@@ -372,8 +386,8 @@ class CookieTreeDatabasesNode : public CookieTreeNode {
 
   DetailedInfo GetDetailedInfo() const override;
 
-  void AddDatabaseNode(CookieTreeDatabaseNode* child) {
-    AddChildSortedByTitle(child);
+  void AddDatabaseNode(std::unique_ptr<CookieTreeDatabaseNode> child) {
+    AddChildSortedByTitle(std::move(child));
   }
 
  private:
@@ -411,8 +425,8 @@ class CookieTreeFileSystemsNode : public CookieTreeNode {
 
   DetailedInfo GetDetailedInfo() const override;
 
-  void AddFileSystemNode(CookieTreeFileSystemNode* child) {
-    AddChildSortedByTitle(child);
+  void AddFileSystemNode(std::unique_ptr<CookieTreeFileSystemNode> child) {
+    AddChildSortedByTitle(std::move(child));
   }
 
  private:
@@ -449,8 +463,8 @@ class CookieTreeLocalStoragesNode : public CookieTreeNode {
 
   DetailedInfo GetDetailedInfo() const override;
 
-  void AddLocalStorageNode(CookieTreeLocalStorageNode* child) {
-    AddChildSortedByTitle(child);
+  void AddLocalStorageNode(std::unique_ptr<CookieTreeLocalStorageNode> child) {
+    AddChildSortedByTitle(std::move(child));
   }
 
  private:
@@ -488,8 +502,9 @@ class CookieTreeSessionStoragesNode : public CookieTreeNode {
 
   DetailedInfo GetDetailedInfo() const override;
 
-  void AddSessionStorageNode(CookieTreeSessionStorageNode* child) {
-    AddChildSortedByTitle(child);
+  void AddSessionStorageNode(
+      std::unique_ptr<CookieTreeSessionStorageNode> child) {
+    AddChildSortedByTitle(std::move(child));
   }
 
  private:
@@ -526,8 +541,8 @@ class CookieTreeIndexedDBsNode : public CookieTreeNode {
 
   DetailedInfo GetDetailedInfo() const override;
 
-  void AddIndexedDBNode(CookieTreeIndexedDBNode* child) {
-    AddChildSortedByTitle(child);
+  void AddIndexedDBNode(std::unique_ptr<CookieTreeIndexedDBNode> child) {
+    AddChildSortedByTitle(std::move(child));
   }
 
  private:
@@ -584,8 +599,8 @@ class CookieTreeChannelIDsNode : public CookieTreeNode {
 
   DetailedInfo GetDetailedInfo() const override;
 
-  void AddChannelIDNode(CookieTreeChannelIDNode* child) {
-    AddChildSortedByTitle(child);
+  void AddChannelIDNode(std::unique_ptr<CookieTreeChannelIDNode> child) {
+    AddChildSortedByTitle(std::move(child));
   }
 
  private:
@@ -620,8 +635,9 @@ class CookieTreeServiceWorkersNode : public CookieTreeNode {
 
   DetailedInfo GetDetailedInfo() const override;
 
-  void AddServiceWorkerNode(CookieTreeServiceWorkerNode* child) {
-    AddChildSortedByTitle(child);
+  void AddServiceWorkerNode(
+      std::unique_ptr<CookieTreeServiceWorkerNode> child) {
+    AddChildSortedByTitle(std::move(child));
   }
 
  private:
@@ -656,8 +672,8 @@ class CookieTreeCacheStoragesNode : public CookieTreeNode {
 
   DetailedInfo GetDetailedInfo() const override;
 
-  void AddCacheStorageNode(CookieTreeCacheStorageNode* child) {
-    AddChildSortedByTitle(child);
+  void AddCacheStorageNode(std::unique_ptr<CookieTreeCacheStorageNode> child) {
+    AddChildSortedByTitle(std::move(child));
   }
 
  private:
@@ -680,12 +696,50 @@ class CookieTreeFlashLSONode : public CookieTreeNode {
   DISALLOW_COPY_AND_ASSIGN(CookieTreeFlashLSONode);
 };
 
+// CookieTreeMediaLicenseNode -----------------------------------------------
+class CookieTreeMediaLicenseNode : public CookieTreeNode {
+ public:
+  friend class CookieTreeMediaLicensesNode;
+
+  // |media_license_info| is expected to remain valid as long as the
+  // CookieTreeMediaLicenseNode is valid.
+  explicit CookieTreeMediaLicenseNode(
+      const std::list<BrowsingDataMediaLicenseHelper::MediaLicenseInfo>::
+          iterator media_license_info);
+  ~CookieTreeMediaLicenseNode() override;
+
+  void DeleteStoredObjects() override;
+  DetailedInfo GetDetailedInfo() const override;
+
+ private:
+  // |media_license_info_| is expected to remain valid as long as the
+  // CookieTreeMediaLicenseNode is valid.
+  std::list<BrowsingDataMediaLicenseHelper::MediaLicenseInfo>::iterator
+      media_license_info_;
+
+  DISALLOW_COPY_AND_ASSIGN(CookieTreeMediaLicenseNode);
+};
+
+class CookieTreeMediaLicensesNode : public CookieTreeNode {
+ public:
+  CookieTreeMediaLicensesNode();
+  ~CookieTreeMediaLicensesNode() override;
+
+  DetailedInfo GetDetailedInfo() const override;
+
+  void AddMediaLicenseNode(std::unique_ptr<CookieTreeMediaLicenseNode> child) {
+    AddChildSortedByTitle(std::move(child));
+  }
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(CookieTreeMediaLicensesNode);
+};
+
 // CookiesTreeModel -----------------------------------------------------------
 class CookiesTreeModel : public ui::TreeNodeModel<CookieTreeNode> {
  public:
   CookiesTreeModel(LocalDataContainer* data_container,
-                   ExtensionSpecialStoragePolicy* special_storage_policy,
-                   bool group_by_cookie_source);
+                   ExtensionSpecialStoragePolicy* special_storage_policy);
   ~CookiesTreeModel() override;
 
   // Given a CanonicalCookie, return the ID of the message which should be
@@ -740,7 +794,7 @@ class CookiesTreeModel : public ui::TreeNodeModel<CookieTreeNode> {
   // Filter the origins to only display matched results.
   void UpdateSearchResults(const base::string16& filter);
 
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
   // Returns the set of extensions which protect the data item represented by
   // this node from deletion.
   // Returns nullptr if the node doesn't represent a protected data item or the
@@ -770,6 +824,7 @@ class CookiesTreeModel : public ui::TreeNodeModel<CookieTreeNode> {
   void PopulateServiceWorkerUsageInfo(LocalDataContainer* container);
   void PopulateCacheStorageUsageInfo(LocalDataContainer* container);
   void PopulateFlashLSOInfo(LocalDataContainer* container);
+  void PopulateMediaLicenseInfo(LocalDataContainer* container);
 
   BrowsingDataCookieHelper* GetCookieHelper(const std::string& app_id);
   LocalDataContainer* data_container() {
@@ -846,24 +901,23 @@ class CookiesTreeModel : public ui::TreeNodeModel<CookieTreeNode> {
   void PopulateFlashLSOInfoWithFilter(LocalDataContainer* container,
                                       ScopedBatchUpdateNotifier* notifier,
                                       const base::string16& filter);
+  void PopulateMediaLicenseInfoWithFilter(LocalDataContainer* container,
+                                          ScopedBatchUpdateNotifier* notifier,
+                                          const base::string16& filter);
 
-  // Map of app ids to LocalDataContainer objects to use when retrieving
-  // locally stored data.
-  scoped_ptr<LocalDataContainer> data_container_;
-
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
   // The extension special storage policy; see ExtensionsProtectingNode() above.
   scoped_refptr<ExtensionSpecialStoragePolicy> special_storage_policy_;
 #endif
 
+  // Map of app ids to LocalDataContainer objects to use when retrieving
+  // locally stored data.
+  std::unique_ptr<LocalDataContainer> data_container_;
+
   // The CookiesTreeModel maintains a separate list of observers that are
   // specifically of the type CookiesTreeModel::Observer.
   base::ObserverList<Observer> cookies_observer_list_;
 
-  // If true, use the CanonicalCookie::Source attribute to group cookies.
-  // Otherwise, use the CanonicalCookie::Domain attribute.
-  bool group_by_cookie_source_;
-
   // Keeps track of how many batches the consumer of this class says it is going
   // to send.
   int batches_expected_ = 0;
