@@ -4,11 +4,15 @@
 
 #include <queue>
 
-#include "ash/accelerators/accelerator_controller.h"
-#include "ash/accelerators/accelerator_table.h"
+#include "ash/common/accelerators/accelerator_controller.h"
+#include "ash/common/accelerators/accelerator_table.h"
+#include "ash/common/accessibility_types.h"
+#include "ash/common/material_design/material_design_controller.h"
+#include "ash/common/system/tray/system_tray.h"
+#include "ash/common/wm_shell.h"
 #include "ash/shell.h"
-#include "ash/system/tray/system_tray.h"
 #include "base/command_line.h"
+#include "base/macros.h"
 #include "base/strings/pattern.h"
 #include "base/strings/string_util.h"
 #include "chrome/app/chrome_command_ids.h"
@@ -17,7 +21,6 @@
 #include "chrome/browser/chromeos/accessibility/speech_monitor.h"
 #include "chrome/browser/chromeos/login/login_manager_test.h"
 #include "chrome/browser/chromeos/login/ui/login_display_host.h"
-#include "chrome/browser/chromeos/login/ui/login_display_host_impl.h"
 #include "chrome/browser/chromeos/login/ui/webui_login_view.h"
 #include "chrome/browser/chromeos/profiles/profile_helper.h"
 #include "chrome/browser/extensions/api/braille_display_private/stub_braille_controller.h"
@@ -34,7 +37,9 @@
 #include "chrome/test/base/testing_profile.h"
 #include "chrome/test/base/ui_test_utils.h"
 #include "chromeos/chromeos_switches.h"
-#include "chromeos/login/user_names.h"
+#include "components/signin/core/account_id/account_id.h"
+#include "components/user_manager/user_names.h"
+#include "content/public/browser/browser_thread.h"
 #include "content/public/common/url_constants.h"
 #include "content/public/test/browser_test_utils.h"
 #include "content/public/test/test_utils.h"
@@ -55,7 +60,7 @@ namespace chromeos {
 //
 
 class LoggedInSpokenFeedbackTest : public InProcessBrowserTest {
- protected:
+ public:
   LoggedInSpokenFeedbackTest()
       : animation_mode_(ui::ScopedAnimationDurationScaleMode::ZERO_DURATION) {}
   ~LoggedInSpokenFeedbackTest() override {}
@@ -83,6 +88,11 @@ class LoggedInSpokenFeedbackTest : public InProcessBrowserTest {
         nullptr, key, false, true, false, true)));
   }
 
+  void SendKeyPressWithSearch(ui::KeyboardCode key) {
+    ASSERT_NO_FATAL_FAILURE(ASSERT_TRUE(ui_test_utils::SendKeyPressToWindowSync(
+        nullptr, key, false, false, false, true)));
+  }
+
   void RunJavaScriptInChromeVoxBackgroundPage(const std::string& script) {
     extensions::ExtensionHost* host =
         extensions::ProcessManager::Get(browser()->profile())
@@ -101,7 +111,7 @@ class LoggedInSpokenFeedbackTest : public InProcessBrowserTest {
 
   bool PerformAcceleratorAction(ash::AcceleratorAction action) {
     ash::AcceleratorController* controller =
-        ash::Shell::GetInstance()->accelerator_controller();
+        ash::WmShell::Get()->accelerator_controller();
     return controller->PerformActionIfEnabled(action);
   }
 
@@ -116,44 +126,15 @@ class LoggedInSpokenFeedbackTest : public InProcessBrowserTest {
 
   void EnableChromeVox() {
     // Test setup.
-    // Enable ChromeVox, skip welcome message, and disable earcons.
+    // Enable ChromeVox, skip welcome message/notification, and disable earcons.
     ASSERT_FALSE(AccessibilityManager::Get()->IsSpokenFeedbackEnabled());
 
     AccessibilityManager::Get()->EnableSpokenFeedback(
-        true, ui::A11Y_NOTIFICATION_NONE);
+        true, ash::A11Y_NOTIFICATION_NONE);
     EXPECT_TRUE(speech_monitor_.SkipChromeVoxEnabledMessage());
     DisableEarcons();
   }
 
-  void LoadChromeVoxAndThenNavigateToURL(const GURL& url) {
-    // The goal of this helper function is to avoid race conditions between
-    // the page loading and the ChromeVox extension loading and fully
-    // initializing. To do this, we first load a test url that repeatedly
-    // asks ChromeVox to speak 'ready', then we load ChromeVox and block
-    // until we get that 'ready' speech.
-
-    ui_test_utils::NavigateToURL(
-        browser(),
-        GURL("data:text/html;charset=utf-8,"
-             "<script>"
-             "window.setInterval(function() {"
-             "  try {"
-             "    cvox.Api.speak('ready');"
-             "  } catch (e) {}"
-             "}, 100);"
-             "</script>"));
-    EXPECT_FALSE(AccessibilityManager::Get()->IsSpokenFeedbackEnabled());
-    AccessibilityManager::Get()->EnableSpokenFeedback(
-        true, ui::A11Y_NOTIFICATION_NONE);
-
-    // Block until we get "ready".
-    while (speech_monitor_.GetNextUtterance() != "ready") {
-    }
-
-    // Now load the requested url.
-    ui_test_utils::NavigateToURL(browser(), url);
-  }
-
   void PressRepeatedlyUntilUtterance(ui::KeyboardCode key,
                                      const std::string& expected_utterance) {
     // This helper function is needed when you want to poll for something
@@ -224,6 +205,43 @@ IN_PROC_BROWSER_TEST_F(LoggedInSpokenFeedbackTest, DISABLED_AddBookmark) {
   EXPECT_EQ("button", speech_monitor_.GetNextUtterance());
 }
 
+IN_PROC_BROWSER_TEST_F(LoggedInSpokenFeedbackTest,
+                       DISABLED_NavigateNotificationCenter) {
+  EnableChromeVox();
+
+  EXPECT_TRUE(PerformAcceleratorAction(ash::SHOW_MESSAGE_CENTER_BUBBLE));
+
+  // Tab to request the initial focus.
+  SendKeyPress(ui::VKEY_TAB);
+
+  // Wait for it to say "Notification Center, window".
+  while ("Notification Center, window" != speech_monitor_.GetNextUtterance()) {
+  }
+
+  // Tab until we get to the Do Not Disturb button.
+  SendKeyPress(ui::VKEY_TAB);
+  do {
+    std::string ut = speech_monitor_.GetNextUtterance();
+
+    if (ut == "Do not disturb")
+      break;
+    else if (ut == "Button")
+      SendKeyPress(ui::VKEY_TAB);
+  } while (true);
+  EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
+  EXPECT_EQ("Not pressed", speech_monitor_.GetNextUtterance());
+
+  SendKeyPress(ui::VKEY_SPACE);
+  EXPECT_EQ("Do not disturb", speech_monitor_.GetNextUtterance());
+  EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
+  EXPECT_EQ("Pressed", speech_monitor_.GetNextUtterance());
+
+  SendKeyPress(ui::VKEY_SPACE);
+  EXPECT_EQ("Do not disturb", speech_monitor_.GetNextUtterance());
+  EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
+  EXPECT_EQ("Not pressed", speech_monitor_.GetNextUtterance());
+}
+
 //
 // Spoken feedback tests in both a logged in browser window and guest mode.
 //
@@ -246,8 +264,8 @@ class SpokenFeedbackTest
       command_line->AppendSwitch(::switches::kIncognito);
       command_line->AppendSwitchASCII(chromeos::switches::kLoginProfile,
                                       "user");
-      command_line->AppendSwitchASCII(chromeos::switches::kLoginUser,
-                                      chromeos::login::kGuestUserName);
+      command_line->AppendSwitchASCII(
+          switches::kLoginUser, user_manager::GuestAccountId().GetUserEmail());
     }
   }
 };
@@ -258,33 +276,23 @@ INSTANTIATE_TEST_CASE_P(
     ::testing::Values(kTestAsNormalUser,
                       kTestAsGuestUser));
 
-IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, EnableSpokenFeedback) {
+// TODO(tommi): Flakily hitting HasOneRef DCHECK in
+// AudioOutputResampler::Shutdown, see crbug.com/630031.
+IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, DISABLED_EnableSpokenFeedback) {
   EnableChromeVox();
 }
 
 IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, FocusToolbar) {
   EnableChromeVox();
   chrome::ExecuteCommand(browser(), IDC_FOCUS_TOOLBAR);
-  EXPECT_TRUE(
-      base::MatchPattern(speech_monitor_.GetNextUtterance(), "about:blank*"));
-  EXPECT_EQ("main", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("Tool bar", speech_monitor_.GetNextUtterance());
   EXPECT_EQ("Reload", speech_monitor_.GetNextUtterance());
   EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
 }
 
-IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, TypeInOmnibox) {
+IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, DISABLED_TypeInOmnibox) {
   EnableChromeVox();
 
-  chrome::ExecuteCommand(browser(), IDC_FOCUS_LOCATION);
-  EXPECT_TRUE(
-      base::MatchPattern(speech_monitor_.GetNextUtterance(), "*about:blank*"));
-  EXPECT_EQ("main", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("Tool bar", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("Address and search bar", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("about:blank", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("Edit text", speech_monitor_.GetNextUtterance());
-
+  // Location bar has focus by default so just start typing.
   SendKeyPress(ui::VKEY_X);
   EXPECT_EQ("x", speech_monitor_.GetNextUtterance());
 
@@ -302,13 +310,16 @@ IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, FocusShelf) {
   EnableChromeVox();
 
   EXPECT_TRUE(PerformAcceleratorAction(ash::FOCUS_SHELF));
+  while (true) {
+    std::string utterance = speech_monitor_.GetNextUtterance();
+    if (base::MatchPattern(utterance, "Launcher"))
+      break;
+  }
+  EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
+
   EXPECT_EQ("Shelf", speech_monitor_.GetNextUtterance());
   EXPECT_EQ("Tool bar", speech_monitor_.GetNextUtterance());
-  if (app_list::switches::IsExperimentalAppListEnabled())
-    EXPECT_EQ("Launcher", speech_monitor_.GetNextUtterance());
-  else
-    EXPECT_EQ("Apps", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
+  EXPECT_EQ(", window", speech_monitor_.GetNextUtterance());
 
   SendKeyPress(ui::VKEY_TAB);
   EXPECT_TRUE(base::MatchPattern(speech_monitor_.GetNextUtterance(), "*"));
@@ -318,35 +329,70 @@ IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, FocusShelf) {
 IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, NavigateAppLauncher) {
   EnableChromeVox();
 
+  EXPECT_TRUE(PerformAcceleratorAction(ash::FOCUS_SHELF));
+
+  // Wait for it to say "Launcher", "Button", "Shelf", "Tool bar".
+  while (true) {
+    std::string utterance = speech_monitor_.GetNextUtterance();
+    if (base::MatchPattern(utterance, "Launcher"))
+      break;
+  }
+  EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
+  EXPECT_EQ("Shelf", speech_monitor_.GetNextUtterance());
+  EXPECT_EQ("Tool bar", speech_monitor_.GetNextUtterance());
+
+  // Click on the launcher, it brings up the app list UI.
+  SendKeyPress(ui::VKEY_SPACE);
+  while ("Search or type URL" != speech_monitor_.GetNextUtterance()) {
+  }
+  while ("Edit text" != speech_monitor_.GetNextUtterance()) {
+  }
+
+  // Close it and open it again.
+  SendKeyPress(ui::VKEY_ESCAPE);
+  while (true) {
+    std::string utterance = speech_monitor_.GetNextUtterance();
+    if (base::MatchPattern(utterance, "*window*"))
+      break;
+  }
+
   EXPECT_TRUE(PerformAcceleratorAction(ash::FOCUS_SHELF));
   while (true) {
     std::string utterance = speech_monitor_.GetNextUtterance();
     if (base::MatchPattern(utterance, "Button"))
       break;
   }
+  SendKeyPress(ui::VKEY_SPACE);
 
-  SendKeyPress(ui::VKEY_RETURN);
-
-  EXPECT_EQ("Search or type U R L", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("Edit text", speech_monitor_.GetNextUtterance());
+  // Now type a space into the text field and wait until we hear "space".
+  // This makes the test more robust as it allows us to skip over other
+  // speech along the way.
+  SendKeyPress(ui::VKEY_SPACE);
+  while (true) {
+    if ("space" == speech_monitor_.GetNextUtterance())
+      break;
+  }
 
+  // Now press the down arrow and we should be focused on an app button
+  // in a dialog.
   SendKeyPress(ui::VKEY_DOWN);
-  EXPECT_EQ("Dialog", speech_monitor_.GetNextUtterance());
-  EXPECT_TRUE(base::MatchPattern(speech_monitor_.GetNextUtterance(), "*"));
-  EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
+  while ("Button" != speech_monitor_.GetNextUtterance()) {
+  }
 }
 
 IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, OpenStatusTray) {
   EnableChromeVox();
 
   EXPECT_TRUE(PerformAcceleratorAction(ash::SHOW_SYSTEM_TRAY_BUBBLE));
-  EXPECT_TRUE(
-      base::MatchPattern(speech_monitor_.GetNextUtterance(), "Status tray*"));
+  while (true) {
+    std::string utterance = speech_monitor_.GetNextUtterance();
+    if (base::MatchPattern(utterance, "Status tray*"))
+      break;
+  }
   EXPECT_TRUE(base::MatchPattern(speech_monitor_.GetNextUtterance(), "time *"));
   EXPECT_TRUE(base::MatchPattern(speech_monitor_.GetNextUtterance(),
-                                 "Battery is*full."));
-  EXPECT_TRUE(base::MatchPattern(speech_monitor_.GetNextUtterance(), "*"));
-  EXPECT_TRUE(base::MatchPattern(speech_monitor_.GetNextUtterance(), "Button"));
+                                 "Battery is*full.,"));
+  EXPECT_EQ("window", speech_monitor_.GetNextUtterance());
 }
 
 IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, NavigateSystemTray) {
@@ -355,22 +401,29 @@ IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, NavigateSystemTray) {
   EXPECT_TRUE(PerformAcceleratorAction(ash::SHOW_SYSTEM_TRAY_BUBBLE));
   while (true) {
     std::string utterance = speech_monitor_.GetNextUtterance();
-    if (base::MatchPattern(utterance, "Button"))
+    if (base::MatchPattern(utterance, "Status tray,"))
+      break;
+  }
+  while (true) {
+    std::string utterance = speech_monitor_.GetNextUtterance();
+    if (base::MatchPattern(utterance, "window"))
       break;
   }
 
   SendKeyPress(ui::VKEY_TAB);
+  while (true) {
+    std::string utterance = speech_monitor_.GetNextUtterance();
+    if (base::MatchPattern(utterance, "Button"))
+      break;
+  }
+
+  // Next element.
+  SendKeyPressWithSearch(ui::VKEY_RIGHT);
   EXPECT_TRUE(base::MatchPattern(speech_monitor_.GetNextUtterance(), "*"));
   EXPECT_TRUE(base::MatchPattern(speech_monitor_.GetNextUtterance(), "Button"));
 
-  // Compat next element.
-  SendKeyPressWithSearchAndShift(ui::VKEY_RIGHT);
-  EXPECT_TRUE(base::MatchPattern(speech_monitor_.GetNextUtterance(), "*"));
-  EXPECT_TRUE(base::MatchPattern(speech_monitor_.GetNextUtterance(), "Button"));
-
-  // Compat next button.
-  SendKeyPressWithSearchAndShift(ui::VKEY_N);
-  SendKeyPressWithSearchAndShift(ui::VKEY_B);
+  // Next button.
+  SendKeyPressWithSearch(ui::VKEY_B);
   EXPECT_TRUE(base::MatchPattern(speech_monitor_.GetNextUtterance(), "*"));
   EXPECT_TRUE(base::MatchPattern(speech_monitor_.GetNextUtterance(), "Button"));
 
@@ -385,19 +438,31 @@ IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, NavigateSystemTray) {
   }
   SendKeyPress(ui::VKEY_RETURN);
 
+  if (!ash::MaterialDesignController::IsSystemTrayMenuMaterial()) {
+    while (true) {
+      if (base::MatchPattern(speech_monitor_.GetNextUtterance(), "*Bluetooth"))
+        break;
+    }
+  }
+
   // Navigate to return to previous menu button and press it.
   while (true) {
-    SendKeyPress(ui::VKEY_TAB);
     std::string utterance = speech_monitor_.GetNextUtterance();
     if (base::MatchPattern(utterance, "Previous menu"))
       break;
+    SendKeyPress(ui::VKEY_TAB);
   }
   SendKeyPress(ui::VKEY_RETURN);
 
   while (true) {
     std::string utterance = speech_monitor_.GetNextUtterance();
-    if (base::MatchPattern(speech_monitor_.GetNextUtterance(), "*Bluetooth"))
-      break;
+    if (ash::MaterialDesignController::IsSystemTrayMenuMaterial()) {
+      if (base::MatchPattern(utterance, "Bluetooth*"))
+        break;
+    } else {
+      if (base::MatchPattern(utterance, "*Bluetooth"))
+        break;
+    }
   }
 }
 
@@ -417,23 +482,38 @@ IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, DISABLED_ScreenBrightness) {
 IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, VolumeSlider) {
   EnableChromeVox();
 
+  // Volume slider does not fire valueChanged event on first key press because
+  // it has no widget.
+  EXPECT_TRUE(PerformAcceleratorAction(ash::VOLUME_UP));
   EXPECT_TRUE(PerformAcceleratorAction(ash::VOLUME_UP));
   EXPECT_TRUE(
       base::MatchPattern(speech_monitor_.GetNextUtterance(), "* percent*"));
-  EXPECT_EQ("Volume,", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("slider", speech_monitor_.GetNextUtterance());
 }
 
 IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, OverviewMode) {
   EnableChromeVox();
 
   EXPECT_TRUE(PerformAcceleratorAction(ash::TOGGLE_OVERVIEW));
-  EXPECT_EQ("Edit text", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("Alert", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("Entered window overview mode", speech_monitor_.GetNextUtterance());
+  while (true) {
+    std::string utterance = speech_monitor_.GetNextUtterance();
+    if (base::MatchPattern(utterance, "Edit text"))
+      break;
+  }
+
+  while (true) {
+    std::string utterance = speech_monitor_.GetNextUtterance();
+    if (utterance == "Entered window overview mode")
+      break;
+  }
 
   SendKeyPress(ui::VKEY_TAB);
-  EXPECT_EQ("about:blank", speech_monitor_.GetNextUtterance());
+  // On Chrome OS accessibility title for tabbed browser windows contains app
+  // name ("Chrome" or "Chromium") in overview mode.
+  while (true) {
+    std::string utterance = speech_monitor_.GetNextUtterance();
+    if (base::MatchPattern(utterance, "Chromium - about:blank"))
+      break;
+  }
   EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
 }
 
@@ -444,7 +524,10 @@ IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, OverviewMode) {
 #define MAYBE_ChromeVoxShiftSearch ChromeVoxShiftSearch
 #endif
 IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, MAYBE_ChromeVoxShiftSearch) {
-  LoadChromeVoxAndThenNavigateToURL(
+  EnableChromeVox();
+
+  ui_test_utils::NavigateToURL(
+      browser(),
       GURL("data:text/html;charset=utf-8,<button autofocus>Click me</button>"));
   while (true) {
     std::string utterance = speech_monitor_.GetNextUtterance();
@@ -453,34 +536,14 @@ IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, MAYBE_ChromeVoxShiftSearch) {
   }
   EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
 
-  // Press Search+Shift+/ to enter ChromeVox's "find in page".
-  SendKeyPressWithSearchAndShift(ui::VKEY_OEM_2);
-  EXPECT_EQ("Find in page.", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("Enter a search query.", speech_monitor_.GetNextUtterance());
-}
+  // Press Search+/ to enter ChromeVox's "find in page".
+  SendKeyPressWithSearch(ui::VKEY_OEM_2);
 
-#if defined(MEMORY_SANITIZER)
-// Fails under MemorySanitizer: http://crbug.com/472125
-#define MAYBE_ChromeVoxPrefixKey DISABLED_ChromeVoxPrefixKey
-#else
-#define MAYBE_ChromeVoxPrefixKey ChromeVoxPrefixKey
-#endif
-IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, MAYBE_ChromeVoxPrefixKey) {
-  LoadChromeVoxAndThenNavigateToURL(
-      GURL("data:text/html;charset=utf-8,<button autofocus>Click me</button>"));
   while (true) {
     std::string utterance = speech_monitor_.GetNextUtterance();
-    if (utterance == "Click me")
+    if (utterance == "Find in page.")
       break;
   }
-  EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
-
-  // Press the prefix key Ctrl+';' followed by '/'
-  // to enter ChromeVox's "find in page".
-  SendKeyPressWithControl(ui::VKEY_OEM_1);
-  SendKeyPress(ui::VKEY_OEM_2);
-  EXPECT_EQ("Find in page.", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("Enter a search query.", speech_monitor_.GetNextUtterance());
 }
 
 #if defined(MEMORY_SANITIZER)
@@ -490,10 +553,12 @@ IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, MAYBE_ChromeVoxPrefixKey) {
 #define MAYBE_ChromeVoxNavigateAndSelect ChromeVoxNavigateAndSelect
 #endif
 IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, MAYBE_ChromeVoxNavigateAndSelect) {
-  LoadChromeVoxAndThenNavigateToURL(
-      GURL("data:text/html;charset=utf-8,"
-           "<h1>Title</h1>"
-           "<button autofocus>Click me</button>"));
+  EnableChromeVox();
+
+  ui_test_utils::NavigateToURL(browser(),
+                               GURL("data:text/html;charset=utf-8,"
+                                    "<h1>Title</h1>"
+                                    "<button autofocus>Click me</button>"));
   while (true) {
     std::string utterance = speech_monitor_.GetNextUtterance();
     if (utterance == "Click me")
@@ -501,64 +566,60 @@ IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, MAYBE_ChromeVoxNavigateAndSelect) {
   }
   EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
 
-  // Press Search+Shift+Up to navigate to the previous item.
-  SendKeyPressWithSearchAndShift(ui::VKEY_UP);
+  // Press Search+Left to navigate to the previous item.
+  SendKeyPressWithSearch(ui::VKEY_LEFT);
   EXPECT_EQ("Title", speech_monitor_.GetNextUtterance());
   EXPECT_EQ("Heading 1", speech_monitor_.GetNextUtterance());
 
-  // Press Search+Shift+S to select the text.
-  SendKeyPressWithSearchAndShift(ui::VKEY_S);
-  EXPECT_EQ("Start selection", speech_monitor_.GetNextUtterance());
+  // Press Search+S to select the text.
+  SendKeyPressWithSearch(ui::VKEY_S);
   EXPECT_EQ("Title", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ(", selected", speech_monitor_.GetNextUtterance());
+  EXPECT_EQ("selected", speech_monitor_.GetNextUtterance());
 
   // Press again to end the selection.
-  SendKeyPressWithSearchAndShift(ui::VKEY_S);
+  SendKeyPressWithSearch(ui::VKEY_S);
   EXPECT_EQ("End selection", speech_monitor_.GetNextUtterance());
   EXPECT_EQ("Title", speech_monitor_.GetNextUtterance());
 }
 
-// flaky: http://crbug.com/418572
-IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, DISABLED_ChromeVoxStickyMode) {
-  LoadChromeVoxAndThenNavigateToURL(
-      GURL("data:text/html;charset=utf-8,"
-           "<label>Enter your name <input autofocus></label>"
-           "<p>One</p>"
-           "<h2>Two</h2>"));
-  while (speech_monitor_.GetNextUtterance() != "Enter your name") {
+IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, ChromeVoxNextStickyMode) {
+  EnableChromeVox();
+
+  ui_test_utils::NavigateToURL(
+      browser(),
+      GURL("data:text/html;charset=utf-8,<button autofocus>Click me</button>"));
+  while ("Button" != speech_monitor_.GetNextUtterance()) {
   }
-  EXPECT_EQ("Edit text", speech_monitor_.GetNextUtterance());
 
   // Press the sticky-key sequence: Search Search.
   SendKeyPress(ui::VKEY_LWIN);
-  SendKeyPress(ui::VKEY_LWIN);
+
+  // Sticky key has a minimum 100 ms check to prevent key repeat from toggling
+  // it.
+  content::BrowserThread::PostDelayedTask(
+      content::BrowserThread::UI, FROM_HERE,
+      base::Bind(&LoggedInSpokenFeedbackTest::SendKeyPress,
+                 base::Unretained(this), ui::VKEY_LWIN),
+      base::TimeDelta::FromMilliseconds(200));
+
   EXPECT_EQ("Sticky mode enabled", speech_monitor_.GetNextUtterance());
 
-  // Even once we hear "sticky mode enabled" from the ChromeVox background
-  // page, there's a short window of time when the content script still
-  // hasn't switched to sticky mode. That's why we're focused on a text box.
-  // Keep pressing the '/' key. If sticky mode is off, it will echo the word
-  // "slash". If sticky mode is on, it will open "Find in page". Keep pressing
-  // '/' until we get "Find in page.".
-  PressRepeatedlyUntilUtterance(ui::VKEY_OEM_2, "Find in page.");
-  EXPECT_EQ("Enter a search query.", speech_monitor_.GetNextUtterance());
-
-  // Press Esc to exit Find in Page mode.
-  SendKeyPress(ui::VKEY_ESCAPE);
-  EXPECT_EQ("Exited", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("Find in page.", speech_monitor_.GetNextUtterance());
-
-  // Press N H to jump to the next heading. Skip over speech in-between
-  // but make sure we end up at the heading.
-  SendKeyPress(ui::VKEY_N);
   SendKeyPress(ui::VKEY_H);
-  while (speech_monitor_.GetNextUtterance() != "Two") {
+  while ("No next heading." != speech_monitor_.GetNextUtterance()) {
   }
-  EXPECT_EQ("Heading 2", speech_monitor_.GetNextUtterance());
 
-  // Press the up arrow to go to the previous element.
-  SendKeyPress(ui::VKEY_UP);
-  EXPECT_EQ("One", speech_monitor_.GetNextUtterance());
+  SendKeyPress(ui::VKEY_LWIN);
+
+  // Sticky key has a minimum 100 ms check to prevent key repeat from toggling
+  // it.
+  content::BrowserThread::PostDelayedTask(
+      content::BrowserThread::UI, FROM_HERE,
+      base::Bind(&LoggedInSpokenFeedbackTest::SendKeyPress,
+                 base::Unretained(this), ui::VKEY_LWIN),
+      base::TimeDelta::FromMilliseconds(200));
+
+  while ("Sticky mode disabled" != speech_monitor_.GetNextUtterance()) {
+  }
 }
 
 IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, TouchExploreStatusTray) {
@@ -577,6 +638,48 @@ IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, TouchExploreStatusTray) {
   EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
 }
 
+IN_PROC_BROWSER_TEST_P(SpokenFeedbackTest, ChromeVoxNextTabRecovery) {
+  EnableChromeVox();
+
+  ui_test_utils::NavigateToURL(
+      browser(), GURL("data:text/html;charset=utf-8,"
+                      "<button id='b1' autofocus>11</button>"
+                      "<button>22</button>"
+                      "<button>33</button>"
+                      "<h1>Middle</h1>"
+                      "<button>44</button>"
+                      "<button>55</button>"
+                      "<div id=console aria-live=polite></div>"
+                      "<script>"
+                      "var b1 = document.getElementById('b1');"
+                      "b1.addEventListener('blur', function() {"
+                      "  document.getElementById('console').innerText = "
+                      "'button lost focus';"
+                      "});"
+                      "</script>"));
+  while ("Button" != speech_monitor_.GetNextUtterance()) {
+  }
+
+  // Press Search+H to go to the next heading
+  SendKeyPressWithSearch(ui::VKEY_H);
+  while ("Middle" != speech_monitor_.GetNextUtterance()) {
+  }
+
+  // To ensure that the setSequentialFocusNavigationStartingPoint has
+  // executed before pressing Tab, the page has an event handler waiting
+  // for the 'blur' event on the button, and when it loses focus it
+  // triggers a live region announcement that we wait for, here.
+  while ("button lost focus" != speech_monitor_.GetNextUtterance()) {
+  }
+
+  // Now we know that focus has left the button, so the sequential focus
+  // navigation starting point must be on the heading. Press Tab and
+  // ensure that we land on the first link past the heading.
+  SendKeyPress(ui::VKEY_TAB);
+  while ("44" != speech_monitor_.GetNextUtterance()) {
+  }
+}
+
 //
 // Spoken feedback tests that run only in guest mode.
 //
@@ -590,8 +693,8 @@ class GuestSpokenFeedbackTest : public LoggedInSpokenFeedbackTest {
     command_line->AppendSwitch(chromeos::switches::kGuestSession);
     command_line->AppendSwitch(::switches::kIncognito);
     command_line->AppendSwitchASCII(chromeos::switches::kLoginProfile, "user");
-    command_line->AppendSwitchASCII(chromeos::switches::kLoginUser,
-                                    chromeos::login::kGuestUserName);
+    command_line->AppendSwitchASCII(
+        switches::kLoginUser, user_manager::GuestAccountId().GetUserEmail());
   }
 
  private:
@@ -603,10 +706,6 @@ IN_PROC_BROWSER_TEST_F(GuestSpokenFeedbackTest, FocusToolbar) {
 
   chrome::ExecuteCommand(browser(), IDC_FOCUS_TOOLBAR);
 
-  EXPECT_TRUE(
-      base::MatchPattern(speech_monitor_.GetNextUtterance(), "about:blank*"));
-  EXPECT_EQ("main", speech_monitor_.GetNextUtterance());
-  EXPECT_EQ("Tool bar", speech_monitor_.GetNextUtterance());
   EXPECT_EQ("Reload", speech_monitor_.GetNextUtterance());
   EXPECT_EQ("Button", speech_monitor_.GetNextUtterance());
 }
@@ -638,16 +737,16 @@ IN_PROC_BROWSER_TEST_F(OobeSpokenFeedbackTest, DISABLED_SpokenFeedbackInOobe) {
   ui_controls::EnableUIControls();
   ASSERT_FALSE(AccessibilityManager::Get()->IsSpokenFeedbackEnabled());
 
-  LoginDisplayHost* login_display_host = LoginDisplayHostImpl::default_host();
+  LoginDisplayHost* login_display_host = LoginDisplayHost::default_host();
   WebUILoginView* web_ui_login_view = login_display_host->GetWebUILoginView();
   views::Widget* widget = web_ui_login_view->GetWidget();
   gfx::NativeWindow window = widget->GetNativeWindow();
 
   // We expect to be in the language select dropdown for this test to work,
   // so make sure that's the case.
-  js_checker().Execute("$('language-select').focus()");
+  js_checker().ExecuteAsync("$('language-select').focus()");
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      true, ui::A11Y_NOTIFICATION_NONE);
+      true, ash::A11Y_NOTIFICATION_NONE);
   ASSERT_TRUE(speech_monitor_.SkipChromeVoxEnabledMessage());
   // There's no guarantee that ChromeVox speaks anything when injected after
   // the page loads, which is by design.  Tab forward and then backward
