@@ -4,13 +4,15 @@
 
 #include <stddef.h>
 
+#include "ash/common/system/tray/system_tray.h"
+#include "ash/common/test/wm_shell_test_api.h"
+#include "ash/common/wm_shell.h"
+#include "ash/public/interfaces/new_window.mojom.h"
 #include "ash/shell.h"
 #include "ash/sticky_keys/sticky_keys_controller.h"
 #include "ash/sticky_keys/sticky_keys_overlay.h"
-#include "ash/system/tray/system_tray.h"
 #include "base/command_line.h"
 #include "base/macros.h"
-#include "base/prefs/pref_service.h"
 #include "chrome/browser/chromeos/accessibility/accessibility_manager.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
@@ -22,6 +24,7 @@
 #include "chrome/test/base/in_process_browser_test.h"
 #include "chrome/test/base/interactive_test_utils.h"
 #include "components/omnibox/browser/omnibox_view.h"
+#include "components/prefs/pref_service.h"
 #include "ui/aura/window_event_dispatcher.h"
 #include "ui/events/keycodes/keyboard_codes.h"
 #include "ui/events/test/event_generator.h"
@@ -29,12 +32,40 @@
 
 namespace chromeos {
 
+class CountingNewWindowClient : public ash::mojom::NewWindowClient {
+ public:
+  CountingNewWindowClient() {}
+  ~CountingNewWindowClient() override {}
+
+  int new_tab_action_count() const { return new_tab_action_count_; }
+
+  // ash::mojom::NewWindowClient:
+  void NewTab() override { new_tab_action_count_++; }
+  void NewWindow(bool incognito) override {}
+  void OpenFileManager() override {}
+  void OpenCrosh() override {}
+  void OpenGetHelp() override {}
+  void RestoreTab() override {}
+  void ShowKeyboardOverlay() override {}
+  void ShowTaskManager() override {}
+  void OpenFeedbackPage() override {}
+
+ private:
+  int new_tab_action_count_ = 0;
+
+  DISALLOW_COPY_AND_ASSIGN(CountingNewWindowClient);
+};
+
 class StickyKeysBrowserTest : public InProcessBrowserTest {
  public:
   void SetUpOnMainThread() override {
     content::BrowserTestBase::SetUpOnMainThread();
     event_generator_.reset(
         new ui::test::EventGenerator(browser()->window()->GetNativeWindow()));
+
+    new_window_client_ = new CountingNewWindowClient;
+    ash::WmShellTestApi().SetNewWindowClient(
+        base::WrapUnique(new_window_client_));
   }
 
  protected:
@@ -59,7 +90,8 @@ class StickyKeysBrowserTest : public InProcessBrowserTest {
   }
 
   content::NotificationRegistrar registrar_;
-  scoped_ptr<ui::test::EventGenerator> event_generator_;
+  std::unique_ptr<ui::test::EventGenerator> event_generator_;
+  CountingNewWindowClient* new_window_client_;
 
   DISALLOW_COPY_AND_ASSIGN(StickyKeysBrowserTest);
 };
@@ -96,24 +128,23 @@ IN_PROC_BROWSER_TEST_F(StickyKeysBrowserTest, OpenNewTabs) {
   SendKeyPress(ui::VKEY_CONTROL);
 
   // In the locked state, pressing 't' should open a new tab each time.
-  TabStripModel* tab_strip_model = browser()->tab_strip_model();
-  int tab_count = 1;
-  for (; tab_count < 5; ++tab_count) {
-    EXPECT_EQ(tab_count, tab_strip_model->count());
+  int tab_count = 0;
+  for (; tab_count < 4; ++tab_count) {
+    EXPECT_EQ(tab_count, new_window_client_->new_tab_action_count());
     SendKeyPress(ui::VKEY_T);
   }
 
   // Unlock the modifier key and shortcut should no longer activate.
   SendKeyPress(ui::VKEY_CONTROL);
   SendKeyPress(ui::VKEY_T);
-  EXPECT_EQ(tab_count, tab_strip_model->count());
+  EXPECT_EQ(tab_count, new_window_client_->new_tab_action_count());
 
   // Shortcut should not work after disabling sticky keys.
   DisableStickyKeys();
   SendKeyPress(ui::VKEY_CONTROL);
   SendKeyPress(ui::VKEY_CONTROL);
   SendKeyPress(ui::VKEY_T);
-  EXPECT_EQ(tab_count, tab_strip_model->count());
+  EXPECT_EQ(tab_count, new_window_client_->new_tab_action_count());
 }
 
 IN_PROC_BROWSER_TEST_F(StickyKeysBrowserTest, CtrlClickHomeButton) {
@@ -156,11 +187,11 @@ IN_PROC_BROWSER_TEST_F(StickyKeysBrowserTest, SearchLeftOmnibox) {
       browser()->window()->GetLocationBar()->GetOmniboxView();
 
   // Give the omnibox focus.
-  omnibox->ShowURL();
+  omnibox->SetFocus();
 
-  // Make sure that the AppList is not erronously displayed and the omnibox
-  // doesn't lost focus
-  EXPECT_FALSE(ash::Shell::GetInstance()->GetAppListTargetVisibility());
+  // Make sure that the AppList is not erroneously displayed and the omnibox
+  // doesn't lose focus.
+  EXPECT_FALSE(ash::WmShell::Get()->GetAppListTargetVisibility());
   EXPECT_TRUE(omnibox->GetNativeView()->HasFocus());
 
   // Type 'foo'.
@@ -174,14 +205,14 @@ IN_PROC_BROWSER_TEST_F(StickyKeysBrowserTest, SearchLeftOmnibox) {
   ASSERT_EQ(3U, start);
   ASSERT_EQ(3U, end);
 
-  EXPECT_FALSE(ash::Shell::GetInstance()->GetAppListTargetVisibility());
+  EXPECT_FALSE(ash::WmShell::Get()->GetAppListTargetVisibility());
   EXPECT_TRUE(omnibox->GetNativeView()->HasFocus());
 
   // Hit Home by sequencing Search (left Windows) and Left (arrow).
   SendKeyPress(ui::VKEY_LWIN);
   SendKeyPress(ui::VKEY_LEFT);
 
-  EXPECT_FALSE(ash::Shell::GetInstance()->GetAppListTargetVisibility());
+  EXPECT_FALSE(ash::WmShell::Get()->GetAppListTargetVisibility());
   EXPECT_TRUE(omnibox->GetNativeView()->HasFocus());
 
   // Verify caret moved to the beginning.
