@@ -14,12 +14,12 @@
 #include "base/version.h"
 #include "chrome/browser/chromeos/app_mode/kiosk_app_manager.h"
 #include "chrome/browser/chromeos/ui/kiosk_external_update_notification.h"
+#include "chrome/grit/chromium_strings.h"
+#include "chrome/grit/generated_resources.h"
 #include "components/version_info/version_info.h"
 #include "content/public/browser/browser_thread.h"
 #include "extensions/browser/sandboxed_unpacker.h"
 #include "extensions/common/extension.h"
-#include "grit/chromium_strings.h"
-#include "grit/generated_resources.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 
@@ -95,6 +95,9 @@ bool ShouldUpdateForHigherVersion(const std::string& version_1,
 KioskExternalUpdater::ExternalUpdate::ExternalUpdate() {
 }
 
+KioskExternalUpdater::ExternalUpdate::ExternalUpdate(
+    const ExternalUpdate& other) = default;
+
 KioskExternalUpdater::ExternalUpdate::~ExternalUpdate() {
 }
 
