@@ -6,7 +6,7 @@
 
 #include "chrome/browser/chromeos/events/event_rewriter.h"
 #include "chrome/browser/chromeos/system/input_device_settings.h"
-#include "components/user_manager/user_manager.h"
+#include "components/session_manager/core/session_manager.h"
 #include "ui/events/event.h"
 #include "ui/events/event_utils.h"
 
@@ -16,11 +16,13 @@ namespace {
 
 const int kModifierMask = ui::EF_SHIFT_DOWN;
 
+KeyboardDrivenEventRewriter* instance = nullptr;
+
 // Returns true if and only if it is on login screen (i.e. user is not logged
 // in) and the keyboard driven flag in the OEM manifest is on.
 bool ShouldStripModifiersForArrowKeysAndEnter() {
-  if (user_manager::UserManager::IsInitialized() &&
-      !user_manager::UserManager::Get()->IsSessionStarted()) {
+  if (session_manager::SessionManager::Get() &&
+      !session_manager::SessionManager::Get()->IsSessionStarted()) {
     return system::InputDeviceSettings::Get()
         ->ForceKeyboardDrivenUINavigation();
   }
@@ -30,21 +32,31 @@ bool ShouldStripModifiersForArrowKeysAndEnter() {
 
 }  // namespace
 
+// static
+KeyboardDrivenEventRewriter* KeyboardDrivenEventRewriter::GetInstance() {
+  DCHECK(instance);
+  return instance;
+}
+
 KeyboardDrivenEventRewriter::KeyboardDrivenEventRewriter() {
+  DCHECK(!instance);
+  instance = this;
 }
 
 KeyboardDrivenEventRewriter::~KeyboardDrivenEventRewriter() {
+  DCHECK_EQ(instance, this);
+  instance = nullptr;
 }
 
 ui::EventRewriteStatus KeyboardDrivenEventRewriter::RewriteForTesting(
     const ui::Event& event,
-    scoped_ptr<ui::Event>* rewritten_event) {
+    std::unique_ptr<ui::Event>* rewritten_event) {
   return Rewrite(event, rewritten_event);
 }
 
 ui::EventRewriteStatus KeyboardDrivenEventRewriter::RewriteEvent(
     const ui::Event& event,
-    scoped_ptr<ui::Event>* rewritten_event) {
+    std::unique_ptr<ui::Event>* rewritten_event) {
   if (!ShouldStripModifiersForArrowKeysAndEnter())
     return ui::EVENT_REWRITE_CONTINUE;
 
@@ -53,14 +65,14 @@ ui::EventRewriteStatus KeyboardDrivenEventRewriter::RewriteEvent(
 
 ui::EventRewriteStatus KeyboardDrivenEventRewriter::NextDispatchEvent(
     const ui::Event& last_event,
-    scoped_ptr<ui::Event>* new_event) {
+    std::unique_ptr<ui::Event>* new_event) {
   NOTREACHED();
   return ui::EVENT_REWRITE_CONTINUE;
 }
 
 ui::EventRewriteStatus KeyboardDrivenEventRewriter::Rewrite(
     const ui::Event& event,
-    scoped_ptr<ui::Event>* rewritten_event) {
+    std::unique_ptr<ui::Event>* rewritten_event) {
   int flags = event.flags();
   if ((flags & kModifierMask) != kModifierMask)
     return ui::EVENT_REWRITE_CONTINUE;
@@ -83,6 +95,19 @@ ui::EventRewriteStatus KeyboardDrivenEventRewriter::Rewrite(
       key_event.GetDomKey(),
       key_event.key_code()};
 
+  if (rewritten_to_tab_) {
+    if (key_code == ui::VKEY_LEFT || key_code == ui::VKEY_RIGHT ||
+        key_code == ui::VKEY_UP || key_code == ui::VKEY_DOWN) {
+      const ui::KeyEvent tab_event(ui::ET_KEY_PRESSED, ui::VKEY_TAB,
+                                   ui::EF_NONE);
+      state.code = tab_event.code();
+      state.key = tab_event.GetDomKey();
+      state.key_code = tab_event.key_code();
+      if (key_code == ui::VKEY_LEFT || key_code == ui::VKEY_UP)
+        state.flags |= ui::EF_SHIFT_DOWN;
+    }
+  }
+
   chromeos::EventRewriter::BuildRewrittenKeyEvent(key_event, state,
                                                   rewritten_event);
   return ui::EVENT_REWRITE_REWRITTEN;
