@@ -8,19 +8,22 @@
 #include <utility>
 #include <vector>
 
-#include "ash/desktop_background/desktop_background_controller.h"
 #include "base/files/file_util.h"
 #include "base/lazy_instance.h"
-#include "base/prefs/pref_service.h"
+#include "base/logging.h"
+#include "base/memory/ref_counted_memory.h"
 #include "base/strings/stringprintf.h"
 #include "base/threading/worker_pool.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/chromeos/extensions/wallpaper_private_api.h"
+#include "chrome/browser/chromeos/file_manager/app_id.h"
 #include "chrome/browser/chromeos/login/users/wallpaper/wallpaper_manager.h"
+#include "chrome/browser/chromeos/profiles/profile_helper.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/extensions/extension_constants.h"
 #include "chrome/common/pref_names.h"
+#include "components/prefs/pref_service.h"
 #include "components/user_manager/user.h"
 #include "components/user_manager/user_manager.h"
 #include "components/wallpaper/wallpaper_layout.h"
@@ -85,13 +88,25 @@ class WallpaperFetcher : public net::URLFetcherDelegate {
     }
   }
 
-  scoped_ptr<net::URLFetcher> url_fetcher_;
+  std::unique_ptr<net::URLFetcher> url_fetcher_;
   FetchCallback callback_;
 };
 
 base::LazyInstance<WallpaperFetcher> g_wallpaper_fetcher =
     LAZY_INSTANCE_INITIALIZER;
 
+// Gets the |User| for a given |BrowserContext|. The function will only return
+// valid objects.
+const user_manager::User* GetUserFromBrowserContext(
+    content::BrowserContext* context) {
+  Profile* profile = Profile::FromBrowserContext(context);
+  DCHECK(profile);
+  const user_manager::User* user =
+      chromeos::ProfileHelper::Get()->GetUserByProfile(profile);
+  DCHECK(user);
+  return user;
+}
+
 }  // namespace
 
 WallpaperSetWallpaperFunction::WallpaperSetWallpaperFunction() {
@@ -105,15 +120,16 @@ bool WallpaperSetWallpaperFunction::RunAsync() {
   params_ = set_wallpaper::Params::Create(*args_);
   EXTENSION_FUNCTION_VALIDATE(params_);
 
-  // Gets account id and username hash while at UI thread.
-  account_id_ =
-      user_manager::UserManager::Get()->GetLoggedInUser()->GetAccountId();
-  user_id_hash_ =
-      user_manager::UserManager::Get()->GetLoggedInUser()->username_hash();
+  // Gets account id from the caller, ensuring multiprofile compatibility.
+  const user_manager::User* user = GetUserFromBrowserContext(browser_context());
+  account_id_ = user->GetAccountId();
+  chromeos::WallpaperManager* wallpaper_manager =
+      chromeos::WallpaperManager::Get();
+  wallpaper_files_id_ = wallpaper_manager->GetFilesId(account_id_);
 
   if (params_->details.data) {
     StartDecode(*params_->details.data);
-  } else {
+  } else if (params_->details.url) {
     GURL wallpaper_url(*params_->details.url);
     if (wallpaper_url.is_valid()) {
       g_wallpaper_fetcher.Get().FetchWallpaper(
@@ -123,6 +139,9 @@ bool WallpaperSetWallpaperFunction::RunAsync() {
       SetError("URL is invalid.");
       SendResponse(false);
     }
+  } else {
+    SetError("Either url or data field is required.");
+    SendResponse(false);
   }
   return true;
 }
@@ -132,14 +151,14 @@ void WallpaperSetWallpaperFunction::OnWallpaperDecoded(
   chromeos::WallpaperManager* wallpaper_manager =
       chromeos::WallpaperManager::Get();
   base::FilePath thumbnail_path = wallpaper_manager->GetCustomWallpaperPath(
-      wallpaper::kThumbnailWallpaperSubDir, user_id_hash_,
+      wallpaper::kThumbnailWallpaperSubDir, wallpaper_files_id_,
       params_->details.filename);
 
-  sequence_token_ = BrowserThread::GetBlockingPool()->GetNamedSequenceToken(
-      wallpaper::kWallpaperSequenceTokenName);
   scoped_refptr<base::SequencedTaskRunner> task_runner =
-      BrowserThread::GetBlockingPool()->
-          GetSequencedTaskRunnerWithShutdownBehavior(sequence_token_,
+      BrowserThread::GetBlockingPool()
+          ->GetSequencedTaskRunnerWithShutdownBehavior(
+              BrowserThread::GetBlockingPool()->GetNamedSequenceToken(
+                  wallpaper::kWallpaperSequenceTokenName),
               base::SequencedWorkerPool::BLOCK_SHUTDOWN);
   wallpaper::WallpaperLayout layout = wallpaper_api_util::GetLayoutEnum(
       extensions::api::wallpaper::ToString(params_->details.layout));
@@ -149,13 +168,15 @@ void WallpaperSetWallpaperFunction::OnWallpaperDecoded(
       account_id_ ==
       user_manager::UserManager::Get()->GetActiveUser()->GetAccountId();
   wallpaper_manager->SetCustomWallpaper(
-      account_id_, user_id_hash_, params_->details.filename, layout,
+      account_id_, wallpaper_files_id_, params_->details.filename, layout,
       user_manager::User::CUSTOMIZED, image, update_wallpaper);
   unsafe_wallpaper_decoder_ = NULL;
 
-  // Save current extenion name. It will be displayed in the component
+  // Save current extension name. It will be displayed in the component
   // wallpaper picker app. If current extension is the component wallpaper
   // picker, set an empty string.
+  // TODO(xdai): This preference is unused now. For compatiblity concern, we
+  // need to keep it until it's safe to clean it up.
   Profile* profile = Profile::FromBrowserContext(browser_context());
   if (extension()->id() == extension_misc::kWallpaperManagerId) {
     profile->GetPrefs()->SetString(prefs::kCurrentWallpaperAppName,
@@ -171,7 +192,7 @@ void WallpaperSetWallpaperFunction::OnWallpaperDecoded(
   // We need to generate thumbnail image anyway to make the current third party
   // wallpaper syncable through different devices.
   image.EnsureRepsForSupportedScales();
-  scoped_ptr<gfx::ImageSkia> deep_copy(image.DeepCopy());
+  std::unique_ptr<gfx::ImageSkia> deep_copy(image.DeepCopy());
   // Generates thumbnail before call api function callback. We can then
   // request thumbnail in the javascript callback.
   task_runner->PostTask(
@@ -181,9 +202,9 @@ void WallpaperSetWallpaperFunction::OnWallpaperDecoded(
 }
 
 void WallpaperSetWallpaperFunction::GenerateThumbnail(
-    const base::FilePath& thumbnail_path, scoped_ptr<gfx::ImageSkia> image) {
-  DCHECK(BrowserThread::GetBlockingPool()->IsRunningSequenceOnCurrentThread(
-      sequence_token_));
+    const base::FilePath& thumbnail_path,
+    std::unique_ptr<gfx::ImageSkia> image) {
+  wallpaper::AssertCalledOnWallpaperSequence();
   if (!base::PathExists(thumbnail_path.DirName()))
     base::CreateDirectory(thumbnail_path.DirName());
 
@@ -199,40 +220,57 @@ void WallpaperSetWallpaperFunction::GenerateThumbnail(
   BrowserThread::PostTask(
       BrowserThread::UI, FROM_HERE,
       base::Bind(&WallpaperSetWallpaperFunction::ThumbnailGenerated, this,
-                 original_data, thumbnail_data));
+                 base::RetainedRef(original_data),
+                 base::RetainedRef(thumbnail_data)));
 }
 
 void WallpaperSetWallpaperFunction::ThumbnailGenerated(
     base::RefCountedBytes* original_data,
     base::RefCountedBytes* thumbnail_data) {
-  BinaryValue* original_result = BinaryValue::CreateWithCopiedBuffer(
-      reinterpret_cast<const char*>(original_data->front()),
-      original_data->size());
-  BinaryValue* thumbnail_result = BinaryValue::CreateWithCopiedBuffer(
-      reinterpret_cast<const char*>(thumbnail_data->front()),
-      thumbnail_data->size());
+  std::unique_ptr<BinaryValue> original_result =
+      BinaryValue::CreateWithCopiedBuffer(
+          reinterpret_cast<const char*>(original_data->front()),
+          original_data->size());
+  std::unique_ptr<BinaryValue> thumbnail_result =
+      BinaryValue::CreateWithCopiedBuffer(
+          reinterpret_cast<const char*>(thumbnail_data->front()),
+          thumbnail_data->size());
 
   if (params_->details.thumbnail) {
-    SetResult(thumbnail_result);
+    SetResult(thumbnail_result->CreateDeepCopy());
     SendResponse(true);
   }
 
   // Inform the native Wallpaper Picker Application that the current wallpaper
   // has been modified by a third party application.
-  Profile* profile = Profile::FromBrowserContext(browser_context());
-  extensions::EventRouter* event_router = extensions::EventRouter::Get(profile);
-  scoped_ptr<base::ListValue> event_args(new base::ListValue());
-  event_args->Append(original_result);
-  event_args->Append(thumbnail_result);
-  event_args->Append(new base::StringValue(
-      extensions::api::wallpaper::ToString(params_->details.layout)));
-  scoped_ptr<extensions::Event> event(new extensions::Event(
-      extensions::events::WALLPAPER_PRIVATE_ON_WALLPAPER_CHANGED_BY_3RD_PARTY,
-      extensions::api::wallpaper_private::OnWallpaperChangedBy3rdParty::
-          kEventName,
-      std::move(event_args)));
-  event_router->DispatchEventToExtension(extension_misc::kWallpaperManagerId,
-                                         std::move(event));
+  if (extension()->id() != extension_misc::kWallpaperManagerId) {
+    Profile* profile = Profile::FromBrowserContext(browser_context());
+    extensions::EventRouter* event_router =
+        extensions::EventRouter::Get(profile);
+    std::unique_ptr<base::ListValue> event_args(new base::ListValue());
+    event_args->Append(original_result->CreateDeepCopy());
+    event_args->Append(thumbnail_result->CreateDeepCopy());
+    event_args->AppendString(
+        extensions::api::wallpaper::ToString(params_->details.layout));
+    // Setting wallpaper from right click menu in 'Files' app is a feature that
+    // was implemented in crbug.com/578935. Since 'Files' app is a built-in v1
+    // app in ChromeOS, we should treat it slightly differently with other third
+    // party apps: the wallpaper set by the 'Files' app should still be syncable
+    // and it should not appear in the wallpaper grid in the Wallpaper Picker.
+    // But we should not display the 'wallpaper-set-by-mesage' since it might
+    // introduce confusion as shown in crbug.com/599407.
+    event_args->AppendString(
+        (extension()->id() == file_manager::kFileManagerAppId)
+            ? std::string()
+            : extension()->name());
+    std::unique_ptr<extensions::Event> event(new extensions::Event(
+        extensions::events::WALLPAPER_PRIVATE_ON_WALLPAPER_CHANGED_BY_3RD_PARTY,
+        extensions::api::wallpaper_private::OnWallpaperChangedBy3rdParty::
+            kEventName,
+        std::move(event_args)));
+    event_router->DispatchEventToExtension(extension_misc::kWallpaperManagerId,
+                                           std::move(event));
+  }
 }
 
 void WallpaperSetWallpaperFunction::OnWallpaperFetched(
