@@ -6,6 +6,7 @@
 #define CHROME_BROWSER_CHROMEOS_FILE_MANAGER_VOLUME_MANAGER_H_
 
 #include <map>
+#include <memory>
 #include <string>
 #include <vector>
 
@@ -14,16 +15,15 @@
 #include "base/files/file_path.h"
 #include "base/macros.h"
 #include "base/memory/linked_ptr.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/observer_list.h"
-#include "base/prefs/pref_change_registrar.h"
 #include "chrome/browser/chromeos/drive/drive_integration_service.h"
 #include "chrome/browser/chromeos/file_system_provider/observer.h"
 #include "chrome/browser/chromeos/file_system_provider/service.h"
 #include "chromeos/dbus/cros_disks_client.h"
 #include "chromeos/disks/disk_mount_manager.h"
 #include "components/keyed_service/core/keyed_service.h"
+#include "components/prefs/pref_change_registrar.h"
 #include "components/storage_monitor/removable_storage_observer.h"
 #include "device/media_transfer_protocol/mtp_storage_info.pb.h"
 
@@ -116,7 +116,18 @@ class Volume : public base::SupportsWeakPtr<Volume> {
   }
   const std::string& volume_label() const { return volume_label_; }
   bool is_parent() const { return is_parent_; }
+  // Whether the applications can write to the volume. True if not writable.
+  // For example, when write access to external storage is restricted by the
+  // policy (ExternalStorageReadOnly), is_read_only() will be true even when
+  // is_read_only_removable_device() is false.
   bool is_read_only() const { return is_read_only_; }
+  // Whether the device is write-protected by hardware. This field is valid
+  // only when device_type is VOLUME_TYPE_REMOVABLE_DISK_PARTITION and
+  // source is SOURCE_DEVICE.
+  // When this value is true, is_read_only() is also true.
+  bool is_read_only_removable_device() const {
+    return is_read_only_removable_device_;
+  }
   bool has_media() const { return has_media_; }
   bool configurable() const { return configurable_; }
   bool watchable() const { return watchable_; }
@@ -175,9 +186,13 @@ class Volume : public base::SupportsWeakPtr<Volume> {
   // Is the device is a parent device (i.e. sdb rather than sdb1).
   bool is_parent_;
 
-  // True if the volume is read only.
+  // True if the volume is not writable by applications.
   bool is_read_only_;
 
+  // True if the volume is made read_only due to its hardware.
+  // This implies is_read_only_.
+  bool is_read_only_removable_device_;
+
   // True if the volume contains media.
   bool has_media_;
 
@@ -286,6 +301,9 @@ class VolumeManager : public KeyedService,
   // Called on change to kExternalStorageDisabled pref.
   void OnExternalStorageDisabledChanged();
 
+  // Called on change to kExternalStorageReadOnly pref.
+  void OnExternalStorageReadOnlyChanged();
+
   // RemovableStorageObserver overrides.
   void OnRemovableStorageAttached(
       const storage_monitor::StorageInfo& info) override;
@@ -301,6 +319,8 @@ class VolumeManager : public KeyedService,
                     const linked_ptr<Volume>& volume);
   void DoUnmountEvent(chromeos::MountError error_code,
                       const linked_ptr<Volume>& volume);
+  void OnExternalStorageDisabledChangedUnmountCallback(
+      chromeos::MountError error_code);
 
   Profile* profile_;
   drive::DriveIntegrationService* drive_integration_service_;  // Not owned.
@@ -311,7 +331,7 @@ class VolumeManager : public KeyedService,
       file_system_provider_service_;  // Not owned by this class.
   GetMtpStorageInfoCallback get_mtp_storage_info_callback_;
   std::map<std::string, linked_ptr<Volume>> mounted_volumes_;
-  scoped_ptr<SnapshotManager> snapshot_manager_;
+  std::unique_ptr<SnapshotManager> snapshot_manager_;
 
   // Note: This should remain the last member so it'll be destroyed and
   // invalidate its weak pointers before any other members are destroyed.
