@@ -4,18 +4,18 @@
 
 #include "chrome/browser/chromeos/file_system_provider/registry.h"
 
+#include <memory>
 #include <string>
 #include <vector>
 
 #include "base/files/file_path.h"
-#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/chromeos/file_system_provider/provided_file_system_info.h"
 #include "chrome/common/extensions/api/file_system_provider_capabilities/file_system_provider_capabilities_handler.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/test/base/testing_browser_process.h"
 #include "chrome/test/base/testing_profile.h"
 #include "chrome/test/base/testing_profile_manager.h"
-#include "components/syncable_prefs/testing_pref_service_syncable.h"
+#include "components/sync_preferences/testing_pref_service_syncable.h"
 #include "components/user_prefs/user_prefs.h"
 #include "content/public/test/test_browser_thread_bundle.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -50,7 +50,7 @@ void RememberFakeFileSystem(TestingProfile* profile,
                             const Watcher& watcher) {
   // Warning. Updating this code means that backward compatibility may be
   // broken, what is unexpected and should be avoided.
-  syncable_prefs::TestingPrefServiceSyncable* const pref_service =
+  sync_preferences::TestingPrefServiceSyncable* const pref_service =
       profile->GetTestingPrefService();
   ASSERT_TRUE(pref_service);
 
@@ -116,9 +116,9 @@ class FileSystemProviderRegistryTest : public testing::Test {
   }
 
   content::TestBrowserThreadBundle thread_bundle_;
-  scoped_ptr<TestingProfileManager> profile_manager_;
+  std::unique_ptr<TestingProfileManager> profile_manager_;
   TestingProfile* profile_;
-  scoped_ptr<RegistryInterface> registry_;
+  std::unique_ptr<RegistryInterface> registry_;
   Watcher fake_watcher_;
 };
 
@@ -128,8 +128,8 @@ TEST_F(FileSystemProviderRegistryTest, RestoreFileSystems) {
                          true /* writable */, true /* supports_notify_tag */,
                          kOpenedFilesLimit, fake_watcher_);
 
-  scoped_ptr<RegistryInterface::RestoredFileSystems> restored_file_systems =
-      registry_->RestoreFileSystems(kExtensionId);
+  std::unique_ptr<RegistryInterface::RestoredFileSystems>
+      restored_file_systems = registry_->RestoreFileSystems(kExtensionId);
 
   ASSERT_EQ(1u, restored_file_systems->size());
   const RegistryInterface::RestoredFileSystem& restored_file_system =
@@ -167,7 +167,7 @@ TEST_F(FileSystemProviderRegistryTest, RememberFileSystem) {
 
   registry_->RememberFileSystem(file_system_info, watchers);
 
-  syncable_prefs::TestingPrefServiceSyncable* const pref_service =
+  sync_preferences::TestingPrefServiceSyncable* const pref_service =
       profile_->GetTestingPrefService();
   ASSERT_TRUE(pref_service);
 
@@ -255,7 +255,7 @@ TEST_F(FileSystemProviderRegistryTest, ForgetFileSystem) {
 
   registry_->ForgetFileSystem(kExtensionId, kFileSystemId);
 
-  syncable_prefs::TestingPrefServiceSyncable* const pref_service =
+  sync_preferences::TestingPrefServiceSyncable* const pref_service =
       profile_->GetTestingPrefService();
   ASSERT_TRUE(pref_service);
 
@@ -286,7 +286,7 @@ TEST_F(FileSystemProviderRegistryTest, UpdateWatcherTag) {
   fake_watcher_.last_tag = "updated-tag";
   registry_->UpdateWatcherTag(file_system_info, fake_watcher_);
 
-  syncable_prefs::TestingPrefServiceSyncable* const pref_service =
+  sync_preferences::TestingPrefServiceSyncable* const pref_service =
       profile_->GetTestingPrefService();
   ASSERT_TRUE(pref_service);
 
