@@ -2,16 +2,18 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <memory>
+
 #include "base/bind.h"
 #include "base/files/file_path.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/message_loop/message_loop.h"
 #include "base/path_service.h"
 #include "base/strings/string_number_conversions.h"
 #include "chrome/browser/chromeos/first_run/drive_first_run_controller.h"
+#include "chrome/browser/extensions/chrome_extension_test_notification_observer.h"
 #include "chrome/browser/extensions/crx_installer.h"
 #include "chrome/browser/extensions/extension_service.h"
-#include "chrome/browser/extensions/extension_test_notification_observer.h"
+#include "chrome/browser/ui/browser.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/test/base/in_process_browser_test.h"
 #include "content/public/test/test_utils.h"
@@ -127,7 +129,7 @@ void DriveFirstRunTest::InstallApp() {
       extensions::CrxInstaller::CreateSilent(extension_service);
 
   installer->InstallCrx(test_data_dir_.AppendASCII(kTestAppCrxName));
-  ExtensionTestNotificationObserver observer(browser());
+  ChromeExtensionTestNotificationObserver observer(browser());
   observer.WaitForExtensionLoad();
 
   ASSERT_TRUE(extension_service->GetExtensionById(kTestAppId, false));
