@@ -4,6 +4,7 @@
 
 #include "chrome/browser/chromeos/login/existing_user_controller.h"
 
+#include <memory>
 #include <vector>
 
 #include "base/bind.h"
@@ -11,10 +12,8 @@
 #include "base/callback.h"
 #include "base/command_line.h"
 #include "base/logging.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/message_loop/message_loop.h"
-#include "base/metrics/histogram.h"
-#include "base/prefs/pref_service.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
@@ -46,6 +45,7 @@
 #include "chrome/browser/signin/easy_unlock_service.h"
 #include "chrome/browser/ui/aura/accessibility/automation_manager_aura.h"
 #include "chrome/browser/ui/webui/chromeos/login/l10n_util.h"
+#include "chrome/common/channel_info.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/grit/generated_resources.h"
@@ -53,7 +53,6 @@
 #include "chromeos/dbus/dbus_thread_manager.h"
 #include "chromeos/dbus/power_manager_client.h"
 #include "chromeos/dbus/session_manager_client.h"
-#include "chromeos/login/user_names.h"
 #include "chromeos/settings/cros_settings_names.h"
 #include "components/google/core/browser/google_util.h"
 #include "components/policy/core/common/cloud/cloud_policy_core.h"
@@ -61,14 +60,20 @@
 #include "components/policy/core/common/policy_map.h"
 #include "components/policy/core/common/policy_service.h"
 #include "components/policy/core/common/policy_types.h"
+#include "components/policy/policy_constants.h"
+#include "components/prefs/pref_service.h"
+#include "components/session_manager/core/session_manager.h"
 #include "components/signin/core/account_id/account_id.h"
 #include "components/signin/core/browser/signin_client.h"
 #include "components/user_manager/known_user.h"
 #include "components/user_manager/user_manager.h"
+#include "components/user_manager/user_names.h"
 #include "components/user_manager/user_type.h"
+#include "components/version_info/version_info.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/notification_service.h"
 #include "content/public/browser/notification_types.h"
+#include "content/public/browser/storage_partition.h"
 #include "content/public/browser/user_metrics.h"
 #include "google_apis/gaia/gaia_auth_util.h"
 #include "google_apis/gaia/google_service_auth_error.h"
@@ -77,7 +82,6 @@
 #include "net/http/http_transaction_factory.h"
 #include "net/url_request/url_request_context.h"
 #include "net/url_request/url_request_context_getter.h"
-#include "policy/policy_constants.h"
 #include "ui/accessibility/ax_enums.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/views/widget/widget.h"
@@ -86,6 +90,19 @@ namespace chromeos {
 
 namespace {
 
+// Enum types for Login.PasswordChangeFlow.
+// Don't change the existing values and update LoginPasswordChangeFlow in
+// histogram.xml when making changes here.
+enum LoginPasswordChangeFlow {
+  // User is sent to the password changed flow. This is the normal case.
+  LOGIN_PASSWORD_CHANGE_FLOW_PASSWORD_CHANGED = 0,
+  // User is sent to the unrecoverable cryptohome failure flow. This is the
+  // case when http://crbug.com/547857 happens.
+  LOGIN_PASSWORD_CHANGE_FLOW_CRYPTOHOME_FAILURE = 1,
+
+  LOGIN_PASSWORD_CHANGE_FLOW_COUNT,  // Must be the last entry.
+};
+
 // Delay for transferring the auth cache to the system profile.
 const long int kAuthCacheTransferDelayMs = 2000;
 
@@ -99,11 +116,12 @@ void RefreshPoliciesOnUIThread() {
     g_browser_process->policy_service()->RefreshPolicies(base::Closure());
 }
 
-// Copies any authentication details that were entered in the login profile in
-// the mail profile to make sure all subsystems of Chrome can access the network
+// Copies any authentication details that were entered in the login profile to
+// the main profile to make sure all subsystems of Chrome can access the network
 // with the provided authentication which are possibly for a proxy server.
 void TransferContextAuthenticationsOnIOThread(
     net::URLRequestContextGetter* default_profile_context_getter,
+    net::URLRequestContextGetter* webview_context_getter,
     net::URLRequestContextGetter* browser_process_context_getter) {
   net::HttpAuthCache* new_cache =
       browser_process_context_getter->GetURLRequestContext()->
@@ -112,6 +130,18 @@ void TransferContextAuthenticationsOnIOThread(
       default_profile_context_getter->GetURLRequestContext()->
       http_transaction_factory()->GetSession()->http_auth_cache();
   new_cache->UpdateAllFrom(*old_cache);
+
+  // Copy the auth cache from webview's context since the proxy authentication
+  // information is saved in webview's context.
+  if (webview_context_getter) {
+    net::HttpAuthCache* webview_cache =
+        webview_context_getter->GetURLRequestContext()
+            ->http_transaction_factory()
+            ->GetSession()
+            ->http_auth_cache();
+    new_cache->UpdateAllFrom(*webview_cache);
+  }
+
   VLOG(1) << "Main request context populated with authentication data.";
   // Last but not least tell the policy subsystem to refresh now as it might
   // have been stuck until now too.
@@ -136,30 +166,29 @@ bool CanShowDebuggingFeatures() {
              chromeos::switches::kSystemDevMode) &&
          base::CommandLine::ForCurrentProcess()->HasSwitch(
              chromeos::switches::kLoginManager) &&
-         !user_manager::UserManager::Get()->IsSessionStarted();
+         !session_manager::SessionManager::Get()->IsSessionStarted();
+}
+
+void RecordPasswordChangeFlow(LoginPasswordChangeFlow flow) {
+  UMA_HISTOGRAM_ENUMERATION("Login.PasswordChangeFlow", flow,
+                            LOGIN_PASSWORD_CHANGE_FLOW_COUNT);
 }
 
 }  // namespace
 
 // static
-ExistingUserController* ExistingUserController::current_controller_ = NULL;
+ExistingUserController* ExistingUserController::current_controller_ = nullptr;
 
 ////////////////////////////////////////////////////////////////////////////////
 // ExistingUserController, public:
 
 ExistingUserController::ExistingUserController(LoginDisplayHost* host)
-    : auth_status_consumer_(NULL),
-      host_(host),
+    : host_(host),
       login_display_(host_->CreateLoginDisplay(this)),
-      num_login_attempts_(0),
       cros_settings_(CrosSettings::Get()),
-      is_login_in_progress_(false),
-      password_changed_(false),
-      auth_mode_(LoginPerformer::AUTH_MODE_EXTENSION),
-      signin_screen_ready_(false),
       network_state_helper_(new login::NetworkStateHelper),
       weak_factory_(this) {
-  DCHECK(current_controller_ == NULL);
+  DCHECK(current_controller_ == nullptr);
   current_controller_ = this;
 
   registrar_.Add(this,
@@ -194,19 +223,19 @@ ExistingUserController::ExistingUserController(LoginDisplayHost* host)
   local_account_auto_login_id_subscription_ =
       cros_settings_->AddSettingsObserver(
           kAccountsPrefDeviceLocalAccountAutoLoginId,
-          base::Bind(&ExistingUserController::ConfigurePublicSessionAutoLogin,
+          base::Bind(&ExistingUserController::ConfigureAutoLogin,
                      base::Unretained(this)));
   local_account_auto_login_delay_subscription_ =
       cros_settings_->AddSettingsObserver(
           kAccountsPrefDeviceLocalAccountAutoLoginDelay,
-          base::Bind(&ExistingUserController::ConfigurePublicSessionAutoLogin,
+          base::Bind(&ExistingUserController::ConfigureAutoLogin,
                      base::Unretained(this)));
 }
 
 void ExistingUserController::Init(const user_manager::UserList& users) {
   time_init_ = base::Time::Now();
   UpdateLoginDisplay(users);
-  ConfigurePublicSessionAutoLogin();
+  ConfigureAutoLogin();
 }
 
 void ExistingUserController::UpdateLoginDisplay(
@@ -216,25 +245,32 @@ void ExistingUserController::UpdateLoginDisplay(
 
   cros_settings_->GetBoolean(kAccountsPrefShowUserNamesOnSignIn,
                              &show_users_on_signin);
-  for (user_manager::UserList::const_iterator it = users.begin();
-       it != users.end();
-       ++it) {
+  for (auto* user : users) {
+    // Skip kiosk apps for login screen user list. Kiosk apps as pods (aka new
+    // kiosk UI) is currently disabled and it gets the apps directly from
+    // KioskAppManager and ArcKioskAppManager.
+    if (user->GetType() == user_manager::USER_TYPE_KIOSK_APP ||
+        user->GetType() == user_manager::USER_TYPE_ARC_KIOSK_APP) {
+      continue;
+    }
+
     // TODO(xiyuan): Clean user profile whose email is not in whitelist.
-    bool meets_supervised_requirements =
-        (*it)->GetType() != user_manager::USER_TYPE_SUPERVISED ||
+    const bool meets_supervised_requirements =
+        user->GetType() != user_manager::USER_TYPE_SUPERVISED ||
         user_manager::UserManager::Get()->AreSupervisedUsersAllowed();
-    bool meets_whitelist_requirements =
-        CrosSettings::IsWhitelisted((*it)->email(), NULL) ||
-        !(*it)->HasGaiaAccount();
+    const bool meets_whitelist_requirements =
+        CrosSettings::IsWhitelisted(user->GetAccountId().GetUserEmail(),
+                                    nullptr) ||
+        !user->HasGaiaAccount();
 
     // Public session accounts are always shown on login screen.
-    bool meets_show_users_requirements =
+    const bool meets_show_users_requirements =
         show_users_on_signin ||
-        (*it)->GetType() == user_manager::USER_TYPE_PUBLIC_ACCOUNT;
+        user->GetType() == user_manager::USER_TYPE_PUBLIC_ACCOUNT;
     if (meets_supervised_requirements &&
         meets_whitelist_requirements &&
         meets_show_users_requirements) {
-      filtered_users.push_back(*it);
+      filtered_users.push_back(user);
     }
   }
 
@@ -244,10 +280,11 @@ void ExistingUserController::UpdateLoginDisplay(
   cros_settings_->GetBoolean(kAccountsPrefAllowGuest, &show_guest);
   show_users_on_signin |= !filtered_users.empty();
   show_guest &= !filtered_users.empty();
-  bool show_new_user = true;
+  bool allow_new_user = true;
+  cros_settings_->GetBoolean(kAccountsPrefAllowNewUser, &allow_new_user);
   login_display_->set_parent_window(GetNativeWindow());
-  login_display_->Init(
-      filtered_users, show_guest, show_users_on_signin, show_new_user);
+  login_display_->Init(filtered_users, show_guest, show_users_on_signin,
+                       allow_new_user);
   host_->OnPreferencesChanged();
 }
 
@@ -289,15 +326,32 @@ void ExistingUserController::Observe(
         signin_profile->GetRequestContext();
     DCHECK(browser_process_context_getter.get());
     DCHECK(signin_profile_context_getter.get());
+
+    content::StoragePartition* signin_partition = login::GetSigninPartition();
+    scoped_refptr<net::URLRequestContextGetter> webview_context_getter;
+    if (signin_partition) {
+      webview_context_getter = signin_partition->GetURLRequestContext();
+      DCHECK(webview_context_getter.get());
+    }
+
     content::BrowserThread::PostDelayedTask(
         content::BrowserThread::IO, FROM_HERE,
         base::Bind(&TransferContextAuthenticationsOnIOThread,
-                   signin_profile_context_getter,
-                   browser_process_context_getter),
+                   base::RetainedRef(signin_profile_context_getter),
+                   base::RetainedRef(webview_context_getter),
+                   base::RetainedRef(browser_process_context_getter)),
         base::TimeDelta::FromMilliseconds(kAuthCacheTransferDelayMs));
   }
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// ExistingUserController, ArcKioskAppManager::ArcKioskAppManagerObserver
+// implementation:
+//
+
+void ExistingUserController::OnArcKioskAppsChanged() {
+  ConfigureAutoLogin();
+}
 ////////////////////////////////////////////////////////////////////////////////
 // ExistingUserController, private:
 
@@ -305,7 +359,7 @@ ExistingUserController::~ExistingUserController() {
   UserSessionManager::GetInstance()->DelegateDeleted(this);
 
   if (current_controller_ == this) {
-    current_controller_ = NULL;
+    current_controller_ = nullptr;
   } else {
     NOTREACHED() << "More than one controller are alive.";
   }
@@ -317,17 +371,21 @@ ExistingUserController::~ExistingUserController() {
 //
 
 void ExistingUserController::CancelPasswordChangedFlow() {
-  login_performer_.reset(NULL);
-  PerformLoginFinishedActions(true /* start public session timer */);
+  login_performer_.reset(nullptr);
+  PerformLoginFinishedActions(true /* start auto login timer */);
 }
 
 void ExistingUserController::CompleteLogin(const UserContext& user_context) {
-  login_display_->set_signin_completed(true);
   if (!host_) {
     // Complete login event was generated already from UI. Ignore notification.
     return;
   }
 
+  if (is_login_in_progress_)
+    return;
+
+  is_login_in_progress_ = true;
+
   ContinueLoginIfDeviceNotDisabled(base::Bind(
       &ExistingUserController::DoCompleteLogin,
       weak_factory_.GetWeakPtr(),
@@ -344,6 +402,24 @@ bool ExistingUserController::IsSigninInProgress() const {
 
 void ExistingUserController::Login(const UserContext& user_context,
                                    const SigninSpecifics& specifics) {
+  if (is_login_in_progress_) {
+    // If there is another login in progress, bail out. Do not re-enable
+    // clicking on other windows and the status area. Do not start the
+    // auto-login timer.
+    return;
+  }
+
+  is_login_in_progress_ = true;
+
+  if (user_context.GetUserType() != user_manager::USER_TYPE_REGULAR &&
+      user_manager::UserManager::Get()->IsUserLoggedIn()) {
+    // Multi-login is only allowed for regular users. If we are attempting to
+    // do multi-login as another type of user somehow, bail out. Do not
+    // re-enable clicking on other windows and the status area. Do not start the
+    // auto-login timer.
+    return;
+  }
+
   ContinueLoginIfDeviceNotDisabled(base::Bind(
       &ExistingUserController::DoLogin,
       weak_factory_.GetWeakPtr(),
@@ -365,12 +441,12 @@ void ExistingUserController::PerformLogin(
   // such as Authenticator instance.
   if (!login_performer_.get() || num_login_attempts_ <= 1) {
     // Only one instance of LoginPerformer should exist at a time.
-    login_performer_.reset(NULL);
+    login_performer_.reset(nullptr);
     login_performer_.reset(new ChromeLoginPerformer(this));
   }
 
   if (gaia::ExtractDomainName(user_context.GetAccountId().GetUserEmail()) ==
-      chromeos::login::kSupervisedUserDomain) {
+      user_manager::kSupervisedUserDomain) {
     login_performer_->LoginAsSupervisedUser(user_context);
   } else {
     login_performer_->PerformLogin(user_context, auth_mode);
@@ -382,14 +458,20 @@ void ExistingUserController::PerformLogin(
 
 void ExistingUserController::MigrateUserData(const std::string& old_password) {
   // LoginPerformer instance has state of the user so it should exist.
-  if (login_performer_.get())
+  if (login_performer_.get()) {
+    VLOG(1) << "Migrate the existing cryptohome to new password.";
     login_performer_->RecoverEncryptedData(old_password);
+  }
 }
 
-
 void ExistingUserController::OnSigninScreenReady() {
-  signin_screen_ready_ = true;
-  StartPublicSessionAutoLoginTimer();
+  auto_launch_ready_ = true;
+  StartAutoLoginTimer();
+}
+
+void ExistingUserController::OnGaiaScreenReady() {
+  auto_launch_ready_ = true;
+  StartAutoLoginTimer();
 }
 
 void ExistingUserController::OnStartEnterpriseEnrollment() {
@@ -422,8 +504,10 @@ void ExistingUserController::OnStartKioskAutolaunchScreen() {
 
 void ExistingUserController::ResyncUserData() {
   // LoginPerformer instance has state of the user so it should exist.
-  if (login_performer_.get())
+  if (login_performer_.get()) {
+    VLOG(1) << "Create a new cryptohome and resync user data.";
     login_performer_->ResyncEncryptedData();
+  }
 }
 
 void ExistingUserController::SetDisplayEmail(const std::string& email) {
@@ -500,6 +584,25 @@ void ExistingUserController::ShowTPMError() {
   login_display_->ShowErrorScreen(LoginDisplay::TPM_ERROR);
 }
 
+void ExistingUserController::ShowPasswordChangedDialog() {
+  RecordPasswordChangeFlow(LOGIN_PASSWORD_CHANGE_FLOW_PASSWORD_CHANGED);
+
+  VLOG(1) << "Show password changed dialog"
+          << ", count=" << login_performer_->password_changed_callback_count();
+
+  // True if user has already made an attempt to enter old password and failed.
+  bool show_invalid_old_password_error =
+      login_performer_->password_changed_callback_count() > 1;
+
+  // Note: We allow owner using "full sync" mode which will recreate
+  // cryptohome and deal with owner private key being lost. This also allows
+  // us to recover from a lost owner password/homedir.
+  // TODO(gspencer): We shouldn't have to erase stateful data when
+  // doing this.  See http://crosbug.com/9115 http://crosbug.com/7792
+  login_display_->ShowPasswordChangedDialog(show_invalid_old_password_error,
+                                            display_email_);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // ExistingUserController, LoginPerformer::Delegate implementation:
 //
@@ -508,7 +611,7 @@ void ExistingUserController::OnAuthFailure(const AuthFailure& failure) {
   guest_mode_url_ = GURL::EmptyGURL();
   std::string error = failure.GetErrorString();
 
-  PerformLoginFinishedActions(false /* don't start public session timer */);
+  PerformLoginFinishedActions(false /* don't start auto login timer */);
 
   if (ChromeUserManager::Get()
           ->GetUserFlow(last_login_attempt_account_id_)
@@ -526,10 +629,10 @@ void ExistingUserController::OnAuthFailure(const AuthFailure& failure) {
         base::TimeDelta::FromMilliseconds(kSafeModeRestartUiDelayMs));
   } else if (failure.reason() == AuthFailure::TPM_ERROR) {
     ShowTPMError();
-  } else if (last_login_attempt_account_id_ == login::GuestAccountId()) {
+  } else if (last_login_attempt_account_id_ == user_manager::GuestAccountId()) {
     // Show no errors, just re-enable input.
     login_display_->ClearAndEnablePassword();
-    StartPublicSessionAutoLoginTimer();
+    StartAutoLoginTimer();
   } else {
     // Check networking after trying to login in case user is
     // cached locally or the local admin account.
@@ -542,19 +645,16 @@ void ExistingUserController::OnAuthFailure(const AuthFailure& failure) {
         ShowError(IDS_LOGIN_ERROR_OFFLINE_FAILED_NETWORK_NOT_CONNECTED, error);
     } else {
       // TODO(nkostylev): Cleanup rest of ClientLogin related code.
-      if (failure.reason() == AuthFailure::NETWORK_AUTH_FAILED &&
-          failure.error().state() ==
-              GoogleServiceAuthError::HOSTED_NOT_ALLOWED) {
-        ShowError(IDS_LOGIN_ERROR_AUTHENTICATING_HOSTED, error);
-      } else {
-        if (!is_known_user)
-          ShowError(IDS_LOGIN_ERROR_AUTHENTICATING_NEW, error);
-        else
-          ShowError(IDS_LOGIN_ERROR_AUTHENTICATING, error);
-      }
+      if (!is_known_user)
+        ShowError(IDS_LOGIN_ERROR_AUTHENTICATING_NEW, error);
+      else
+        ShowError(IDS_LOGIN_ERROR_AUTHENTICATING, error);
     }
+    if (auth_flow_offline_)
+      UMA_HISTOGRAM_BOOLEAN("Login.OfflineFailure.IsKnownUser", is_known_user);
+
     login_display_->ClearAndEnablePassword();
-    StartPublicSessionAutoLoginTimer();
+    StartAutoLoginTimer();
   }
 
   // Reset user flow to default, so that special flow will not affect next
@@ -589,7 +689,7 @@ void ExistingUserController::OnAuthSuccess(const UserContext& user_context) {
       ->GetUserFlow(user_context.GetAccountId())
       ->HandleLoginSuccess(user_context);
 
-  StopPublicSessionAutoLoginTimer();
+  StopAutoLoginTimer();
 
   // Truth table of |has_auth_cookies|:
   //                          Regular        SAML
@@ -603,9 +703,13 @@ void ExistingUserController::OnAuthSuccess(const UserContext& user_context) {
       user_context.GetAuthFlow() != UserContext::AUTH_FLOW_EASY_BOOTSTRAP;
 
   // LoginPerformer instance will delete itself in case of successful auth.
-  login_performer_->set_delegate(NULL);
+  login_performer_->set_delegate(nullptr);
   ignore_result(login_performer_.release());
 
+  if (user_context.GetAuthFlow() == UserContext::AUTH_FLOW_OFFLINE)
+    UMA_HISTOGRAM_COUNTS_100("Login.OfflineSuccess.Attempts",
+                             num_login_attempts_);
+
   UserSessionManager::StartSessionType start_session_type =
       UserAddingScreen::Get()->IsRunning()
           ? UserSessionManager::SECONDARY_USER_SESSION
@@ -629,7 +733,7 @@ void ExistingUserController::OnProfilePrepared(Profile* profile,
   login_display_->SetUIEnabled(true);
 
   if (browser_launched)
-    host_ = NULL;
+    host_ = nullptr;
 
   // Inform |auth_status_consumer_| about successful login.
   // TODO(nkostylev): Pass UserContext back crbug.com/424550
@@ -670,24 +774,29 @@ void ExistingUserController::OnPasswordChangeDetected() {
     return;
   }
 
-  // True if user has already made an attempt to enter old password and failed.
-  bool show_invalid_old_password_error =
-      login_performer_->password_changed_callback_count() > 1;
-
-  // Note: We allow owner using "full sync" mode which will recreate
-  // cryptohome and deal with owner private key being lost. This also allows
-  // us to recover from a lost owner password/homedir.
-  // TODO(gspencer): We shouldn't have to erase stateful data when
-  // doing this.  See http://crosbug.com/9115 http://crosbug.com/7792
-  login_display_->ShowPasswordChangedDialog(show_invalid_old_password_error,
-                                            display_email_);
-
   if (auth_status_consumer_)
     auth_status_consumer_->OnPasswordChangeDetected();
+
+  // If the password change happens after an online auth, do a TokenHandle check
+  // to find out whether the user password is really changed or not.
+  // TODO(xiyuan): Remove channel restriction. See http://crbug.com/585530
+  if (chrome::GetChannel() <= version_info::Channel::DEV &&
+      auth_mode() == LoginPerformer::AUTH_MODE_EXTENSION) {
+    token_handle_util_.reset(new TokenHandleUtil);
+    if (token_handle_util_->HasToken(last_login_attempt_account_id_)) {
+      token_handle_util_->CheckToken(
+          last_login_attempt_account_id_,
+          base::Bind(&ExistingUserController::OnTokenHandleChecked,
+                     weak_factory_.GetWeakPtr()));
+      return;
+    }
+  }
+
+  ShowPasswordChangedDialog();
 }
 
 void ExistingUserController::WhiteListCheckFailed(const std::string& email) {
-  PerformLoginFinishedActions(true /* start public session timer */);
+  PerformLoginFinishedActions(true /* start auto login timer */);
 
   login_display_->ShowWhitelistCheckFailedError();
 
@@ -702,20 +811,24 @@ void ExistingUserController::WhiteListCheckFailed(const std::string& email) {
 void ExistingUserController::PolicyLoadFailed() {
   ShowError(IDS_LOGIN_ERROR_OWNER_KEY_LOST, "");
 
-  PerformLoginFinishedActions(false /* don't start public session timer */);
+  PerformLoginFinishedActions(false /* don't start auto login timer */);
   display_email_.clear();
 }
 
+void ExistingUserController::SetAuthFlowOffline(bool offline) {
+  auth_flow_offline_ = offline;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // ExistingUserController, private:
 
 void ExistingUserController::DeviceSettingsChanged() {
   // If login was already completed, we should avoid any signin screen
   // transitions, see http://crbug.com/461604 for example.
-  if (host_ != NULL && !login_display_->is_signin_completed()) {
+  if (host_ != nullptr && !login_display_->is_signin_completed()) {
     // Signed settings or user list changed. Notify views and update them.
     UpdateLoginDisplay(user_manager::UserManager::Get()->GetUsers());
-    ConfigurePublicSessionAutoLogin();
+    ConfigureAutoLogin();
   }
 }
 
@@ -735,20 +848,20 @@ bool ExistingUserController::password_changed() const {
 
 void ExistingUserController::LoginAsGuest() {
   PerformPreLoginActions(UserContext(user_manager::USER_TYPE_GUEST,
-                                     login::GuestAccountId().GetUserEmail()));
+                                     user_manager::GuestAccountId()));
 
   bool allow_guest;
   cros_settings_->GetBoolean(kAccountsPrefAllowGuest, &allow_guest);
   if (!allow_guest) {
     // Disallowed. The UI should normally not show the guest session button.
     LOG(ERROR) << "Guest login attempt when guest mode is disallowed.";
-    PerformLoginFinishedActions(true /* start public session timer */);
+    PerformLoginFinishedActions(true /* start auto login timer */);
     display_email_.clear();
     return;
   }
 
   // Only one instance of LoginPerformer should exist at a time.
-  login_performer_.reset(NULL);
+  login_performer_.reset(nullptr);
   login_performer_.reset(new ChromeLoginPerformer(this));
   login_performer_->LoginOffTheRecord();
   SendAccessibilityAlert(
@@ -764,7 +877,7 @@ void ExistingUserController::LoginAsPublicSession(
   const user_manager::User* user =
       user_manager::UserManager::Get()->FindUser(user_context.GetAccountId());
   if (!user || user->GetType() != user_manager::USER_TYPE_PUBLIC_ACCOUNT) {
-    PerformLoginFinishedActions(true /* start public session timer */);
+    PerformLoginFinishedActions(true /* start auto login timer */);
     return;
   }
 
@@ -784,7 +897,7 @@ void ExistingUserController::LoginAsPublicSession(
             ->store()
             ->policy_map()
             .Get(policy::key::kSessionLocales);
-    base::ListValue const* list = NULL;
+    base::ListValue const* list = nullptr;
     if (entry &&
         entry->level == policy::POLICY_LEVEL_RECOMMENDED &&
         entry->value &&
@@ -831,78 +944,115 @@ void ExistingUserController::LoginAsKioskApp(const std::string& app_id,
   host_->StartAppLaunch(app_id, diagnostic_mode, auto_start);
 }
 
-void ExistingUserController::ConfigurePublicSessionAutoLogin() {
+void ExistingUserController::LoginAsArcKioskApp(const AccountId& account_id) {
+  login_performer_.reset(nullptr);
+  login_performer_.reset(new ChromeLoginPerformer(this));
+  login_performer_->LoginAsArcKioskAccount(account_id);
+}
+
+void ExistingUserController::ConfigureAutoLogin() {
   std::string auto_login_account_id;
   cros_settings_->GetString(kAccountsPrefDeviceLocalAccountAutoLoginId,
                             &auto_login_account_id);
   const std::vector<policy::DeviceLocalAccount> device_local_accounts =
       policy::GetDeviceLocalAccounts(cros_settings_);
 
-  public_session_auto_login_username_.clear();
+  public_session_auto_login_account_id_ = EmptyAccountId();
   for (std::vector<policy::DeviceLocalAccount>::const_iterator
            it = device_local_accounts.begin();
        it != device_local_accounts.end(); ++it) {
     if (it->account_id == auto_login_account_id) {
-      public_session_auto_login_username_ = it->user_id;
+      public_session_auto_login_account_id_ =
+          AccountId::FromUserEmail(it->user_id);
       break;
     }
   }
 
-  const user_manager::User* user = user_manager::UserManager::Get()->FindUser(
-      AccountId::FromUserEmail(public_session_auto_login_username_));
-  if (!user || user->GetType() != user_manager::USER_TYPE_PUBLIC_ACCOUNT)
-    public_session_auto_login_username_.clear();
-
-  if (!cros_settings_->GetInteger(
-          kAccountsPrefDeviceLocalAccountAutoLoginDelay,
-          &public_session_auto_login_delay_)) {
-    public_session_auto_login_delay_ = 0;
+  const user_manager::User* public_session_user =
+      user_manager::UserManager::Get()->FindUser(
+          public_session_auto_login_account_id_);
+  if (!public_session_user ||
+      public_session_user->GetType() !=
+          user_manager::USER_TYPE_PUBLIC_ACCOUNT) {
+    public_session_auto_login_account_id_ = EmptyAccountId();
   }
 
-  if (!public_session_auto_login_username_.empty())
-    StartPublicSessionAutoLoginTimer();
-  else
-    StopPublicSessionAutoLoginTimer();
+  arc_kiosk_auto_login_account_id_ =
+      ArcKioskAppManager::Get()->GetAutoLaunchAccountId();
+  const user_manager::User* arc_kiosk_user =
+      user_manager::UserManager::Get()->FindUser(
+          arc_kiosk_auto_login_account_id_);
+  if (!arc_kiosk_user ||
+      arc_kiosk_user->GetType() != user_manager::USER_TYPE_ARC_KIOSK_APP) {
+    arc_kiosk_auto_login_account_id_ = EmptyAccountId();
+  }
+
+  if (!cros_settings_->GetInteger(kAccountsPrefDeviceLocalAccountAutoLoginDelay,
+                                  &auto_login_delay_)) {
+    auto_login_delay_ = 0;
+  }
+
+  if (public_session_auto_login_account_id_.is_valid() ||
+      arc_kiosk_auto_login_account_id_.is_valid()) {
+    StartAutoLoginTimer();
+  } else {
+    StopAutoLoginTimer();
+  }
 }
 
-void ExistingUserController::ResetPublicSessionAutoLoginTimer() {
+void ExistingUserController::ResetAutoLoginTimer() {
   // Only restart the auto-login timer if it's already running.
   if (auto_login_timer_ && auto_login_timer_->IsRunning()) {
-    StopPublicSessionAutoLoginTimer();
-    StartPublicSessionAutoLoginTimer();
+    StopAutoLoginTimer();
+    StartAutoLoginTimer();
   }
 }
 
 void ExistingUserController::OnPublicSessionAutoLoginTimerFire() {
-  CHECK(signin_screen_ready_ && !public_session_auto_login_username_.empty());
+  CHECK(auto_launch_ready_ && public_session_auto_login_account_id_.is_valid());
   Login(UserContext(user_manager::USER_TYPE_PUBLIC_ACCOUNT,
-                    public_session_auto_login_username_),
+                    public_session_auto_login_account_id_),
         SigninSpecifics());
 }
 
-void ExistingUserController::StopPublicSessionAutoLoginTimer() {
+void ExistingUserController::OnArcKioskAutoLoginTimerFire() {
+  CHECK(auto_launch_ready_ && (arc_kiosk_auto_login_account_id_.is_valid()));
+  Login(UserContext(user_manager::USER_TYPE_ARC_KIOSK_APP,
+                    arc_kiosk_auto_login_account_id_),
+        SigninSpecifics());
+}
+
+void ExistingUserController::StopAutoLoginTimer() {
   if (auto_login_timer_)
     auto_login_timer_->Stop();
 }
 
-void ExistingUserController::StartPublicSessionAutoLoginTimer() {
-  if (!signin_screen_ready_ ||
-      is_login_in_progress_ ||
-      public_session_auto_login_username_.empty()) {
+void ExistingUserController::StartAutoLoginTimer() {
+  if (!auto_launch_ready_ || is_login_in_progress_ ||
+      (!public_session_auto_login_account_id_.is_valid() &&
+       !arc_kiosk_auto_login_account_id_.is_valid())) {
     return;
   }
 
+  if (auto_login_timer_ && auto_login_timer_->IsRunning()) {
+    StopAutoLoginTimer();
+  }
+
   // Start the auto-login timer.
   if (!auto_login_timer_)
     auto_login_timer_.reset(new base::OneShotTimer);
 
-  auto_login_timer_->Start(
-      FROM_HERE,
-      base::TimeDelta::FromMilliseconds(
-          public_session_auto_login_delay_),
-      base::Bind(
-          &ExistingUserController::OnPublicSessionAutoLoginTimerFire,
-          weak_factory_.GetWeakPtr()));
+  if (public_session_auto_login_account_id_.is_valid()) {
+    auto_login_timer_->Start(
+        FROM_HERE, base::TimeDelta::FromMilliseconds(auto_login_delay_),
+        base::Bind(&ExistingUserController::OnPublicSessionAutoLoginTimerFire,
+                   weak_factory_.GetWeakPtr()));
+  } else {
+    auto_login_timer_->Start(
+        FROM_HERE, base::TimeDelta::FromMilliseconds(auto_login_delay_),
+        base::Bind(&ExistingUserController::OnArcKioskAutoLoginTimerFire,
+                   weak_factory_.GetWeakPtr()));
+  }
 }
 
 gfx::NativeWindow ExistingUserController::GetNativeWindow() const {
@@ -918,9 +1068,6 @@ void ExistingUserController::ShowError(int error_id,
       case GoogleServiceAuthError::ACCOUNT_DISABLED:
         help_topic_id = HelpAppLauncher::HELP_ACCOUNT_DISABLED;
         break;
-      case GoogleServiceAuthError::HOSTED_NOT_ALLOWED:
-        help_topic_id = HelpAppLauncher::HELP_HOSTED_ACCOUNT;
-        break;
       default:
         help_topic_id = HelpAppLauncher::HELP_CANT_ACCESS_ACCOUNT;
         break;
@@ -952,11 +1099,11 @@ void ExistingUserController::SendAccessibilityAlert(
 
 void ExistingUserController::SetPublicSessionKeyboardLayoutAndLogin(
     const UserContext& user_context,
-    scoped_ptr<base::ListValue> keyboard_layouts) {
+    std::unique_ptr<base::ListValue> keyboard_layouts) {
   UserContext new_user_context = user_context;
   std::string keyboard_layout;
   for (size_t i = 0; i < keyboard_layouts->GetSize(); ++i) {
-    base::DictionaryValue* entry = NULL;
+    base::DictionaryValue* entry = nullptr;
     keyboard_layouts->GetDictionary(i, &entry);
     bool selected = false;
     entry->GetBoolean("selected", &selected);
@@ -974,7 +1121,7 @@ void ExistingUserController::SetPublicSessionKeyboardLayoutAndLogin(
 void ExistingUserController::LoginAsPublicSessionInternal(
     const UserContext& user_context) {
   // Only one instance of LoginPerformer should exist at a time.
-  login_performer_.reset(NULL);
+  login_performer_.reset(nullptr);
   login_performer_.reset(new ChromeLoginPerformer(this));
   login_performer_->LoginAsPublicSession(user_context);
   SendAccessibilityAlert(
@@ -990,28 +1137,21 @@ void ExistingUserController::PerformPreLoginActions(
     last_login_attempt_account_id_ = user_context.GetAccountId();
     num_login_attempts_ = 0;
   }
-
-  // Guard in cases when we're called twice but login process is still active.
-  // This might happen when login process is paused till signed settings status
-  // is verified which results in Login* method called again as a callback.
-  if (!is_login_in_progress_)
-    num_login_attempts_++;
-
-  is_login_in_progress_ = true;
+  num_login_attempts_++;
 
   // Stop the auto-login timer when attempting login.
-  StopPublicSessionAutoLoginTimer();
+  StopAutoLoginTimer();
 }
 
 void ExistingUserController::PerformLoginFinishedActions(
-    bool start_public_session_timer) {
+    bool start_auto_login_timer) {
   is_login_in_progress_ = false;
 
   // Reenable clicking on other windows and status area.
   login_display_->SetUIEnabled(true);
 
-  if (start_public_session_timer)
-    StartPublicSessionAutoLoginTimer();
+  if (start_auto_login_timer)
+    StartAutoLoginTimer();
 }
 
 void ExistingUserController::ContinueLoginIfDeviceNotDisabled(
@@ -1020,7 +1160,7 @@ void ExistingUserController::ContinueLoginIfDeviceNotDisabled(
   login_display_->SetUIEnabled(false);
 
   // Stop the auto-login timer.
-  StopPublicSessionAutoLoginTimer();
+  StopAutoLoginTimer();
 
   // Wait for the |cros_settings_| to become either trusted or permanently
   // untrusted.
@@ -1117,22 +1257,6 @@ void ExistingUserController::DoCompleteLogin(
 
 void ExistingUserController::DoLogin(const UserContext& user_context,
                                      const SigninSpecifics& specifics) {
-  if (is_login_in_progress_) {
-    // If there is another login in progress, bail out. Do not re-enable
-    // clicking on other windows and the status area. Do not start the
-    // auto-login timer.
-    return;
-  }
-
-  if (user_context.GetUserType() != user_manager::USER_TYPE_REGULAR &&
-      user_manager::UserManager::Get()->IsUserLoggedIn()) {
-    // Multi-login is only allowed for regular users. If we are attempting to
-    // do multi-login as another type of user somehow, bail out. Do not
-    // re-enable clicking on other windows and the status area. Do not start the
-    // auto-login timer.
-    return;
-  }
-
   if (user_context.GetUserType() == user_manager::USER_TYPE_GUEST) {
     if (!specifics.guest_mode_url.empty()) {
       guest_mode_url_ = GURL(specifics.guest_mode_url);
@@ -1155,6 +1279,11 @@ void ExistingUserController::DoLogin(const UserContext& user_context,
     return;
   }
 
+  if (user_context.GetUserType() == user_manager::USER_TYPE_ARC_KIOSK_APP) {
+    LoginAsArcKioskApp(user_context.GetAccountId());
+    return;
+  }
+
   // Regular user or supervised user login.
 
   if (!user_context.HasCredentials()) {
@@ -1163,7 +1292,7 @@ void ExistingUserController::DoLogin(const UserContext& user_context,
     // Reenable clicking on other windows and status area.
     login_display_->SetUIEnabled(true);
     // Restart the auto-login timer.
-    StartPublicSessionAutoLoginTimer();
+    StartAutoLoginTimer();
   }
 
   PerformPreLoginActions(user_context);
@@ -1202,4 +1331,22 @@ void ExistingUserController::OnOAuth2TokensFetched(
   PerformLogin(user_context, LoginPerformer::AUTH_MODE_EXTENSION);
 }
 
+void ExistingUserController::OnTokenHandleChecked(
+    const AccountId&,
+    TokenHandleUtil::TokenHandleStatus token_handle_status) {
+  // If TokenHandle is invalid or unknown, continue with regular password
+  // changed flow.
+  if (token_handle_status != TokenHandleUtil::VALID) {
+    VLOG(1) << "Checked TokenHandle status=" << token_handle_status;
+    ShowPasswordChangedDialog();
+    return;
+  }
+
+  // Otherwise, show the unrecoverable cryptohome error UI and ask user's
+  // permission to collect a feedback.
+  RecordPasswordChangeFlow(LOGIN_PASSWORD_CHANGE_FLOW_CRYPTOHOME_FAILURE);
+  VLOG(1) << "Show unrecoverable cryptohome error dialog.";
+  login_display_->ShowUnrecoverableCrypthomeErrorDialog();
+}
+
 }  // namespace chromeos
