@@ -7,25 +7,31 @@
 #include <string>
 #include <vector>
 
-#include "ash/ash_switches.h"
-#include "ash/audio/sounds.h"
-#include "ash/desktop_background/desktop_background_controller.h"
+#include "ash/common/wallpaper/wallpaper_controller.h"
+#include "ash/common/wm/window_state.h"
+#include "ash/common/wm/wm_event.h"
+#include "ash/common/wm_shell.h"
 #include "ash/shell.h"
 #include "ash/wm/lock_state_controller.h"
-#include "ash/wm/window_state.h"
+#include "ash/wm/window_state_aura.h"
 #include "ash/wm/window_util.h"
-#include "ash/wm/wm_event.h"
 #include "base/bind.h"
 #include "base/command_line.h"
 #include "base/lazy_instance.h"
+#include "base/location.h"
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
 #include "base/message_loop/message_loop.h"
-#include "base/metrics/histogram.h"
+#include "base/metrics/histogram_macros.h"
+#include "base/single_thread_task_runner.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "chrome/browser/chrome_notification_types.h"
+#include "chrome/browser/chromeos/accessibility/accessibility_manager.h"
 #include "chrome/browser/chromeos/login/lock/webui_screen_locker.h"
+#include "chrome/browser/chromeos/login/quick_unlock/pin_storage.h"
+#include "chrome/browser/chromeos/login/quick_unlock/pin_storage_factory.h"
 #include "chrome/browser/chromeos/login/session/user_session_manager.h"
 #include "chrome/browser/chromeos/login/supervised/supervised_user_authentication.h"
 #include "chrome/browser/chromeos/login/ui/user_adding_screen.h"
@@ -45,6 +51,7 @@
 #include "chromeos/login/auth/authenticator.h"
 #include "chromeos/login/auth/extended_authenticator.h"
 #include "chromeos/network/portal_detector/network_portal_detector.h"
+#include "components/session_manager/core/session_manager.h"
 #include "components/signin/core/browser/signin_manager.h"
 #include "components/user_manager/user_manager.h"
 #include "components/user_manager/user_type.h"
@@ -99,10 +106,19 @@ class ScreenLockObserver : public SessionManagerClient::StubDelegate,
   void Observe(int type,
                const content::NotificationSource& source,
                const content::NotificationDetails& details) override {
-    if (type == chrome::NOTIFICATION_SESSION_STARTED)
+    if (type == chrome::NOTIFICATION_SESSION_STARTED) {
       session_started_ = true;
-    else
+
+      // The user session has just started, so the user has logged in. Mark a
+      // strong authentication to allow them to use PIN to unlock the device.
+      user_manager::User* user =
+          content::Details<user_manager::User>(details).ptr();
+      PinStorage* pin_storage = PinStorageFactory::GetForUser(user);
+      if (pin_storage)
+        pin_storage->MarkStrongAuth();
+    } else {
       NOTREACHED() << "Unexpected notification " << type;
+    }
   }
 
   // UserAddingScreen::Observer overrides:
@@ -130,10 +146,6 @@ ScreenLocker* ScreenLocker::screen_locker_ = NULL;
 
 ScreenLocker::ScreenLocker(const user_manager::UserList& users)
     : users_(users),
-      locked_(false),
-      start_time_(base::Time::Now()),
-      auth_status_consumer_(NULL),
-      incorrect_passwords_count_(0),
       weak_factory_(this) {
   DCHECK(!screen_locker_);
   screen_locker_ = this;
@@ -145,11 +157,12 @@ ScreenLocker::ScreenLocker(const user_manager::UserList& users)
   manager->Initialize(SOUND_UNLOCK,
                       bundle.GetRawDataResource(IDR_SOUND_UNLOCK_WAV));
 
-  ash::Shell::GetInstance()->
-      lock_state_controller()->SetLockScreenDisplayedCallback(
-          base::Bind(base::IgnoreResult(&ash::PlaySystemSoundIfSpokenFeedback),
-                     static_cast<media::SoundsManager::SoundKey>(
-                         chromeos::SOUND_LOCK)));
+  ash::Shell::GetInstance()
+      ->lock_state_controller()
+      ->SetLockScreenDisplayedCallback(base::Bind(
+          base::IgnoreResult(&AccessibilityManager::PlayEarcon),
+          base::Unretained(AccessibilityManager::Get()), chromeos::SOUND_LOCK,
+          PlaySoundOption::SPOKEN_FEEDBACK_ENABLED));
 }
 
 void ScreenLocker::Init() {
@@ -160,16 +173,15 @@ void ScreenLocker::Init() {
 
   authenticator_ = UserSessionManager::GetInstance()->CreateAuthenticator(this);
   extended_authenticator_ = ExtendedAuthenticator::Create(this);
-  delegate_.reset(new WebUIScreenLocker(this));
-  delegate_->LockScreen();
+  web_ui_.reset(new WebUIScreenLocker(this));
+  web_ui()->LockScreen();
 
   // Ownership of |icon_image_source| is passed.
   screenlock_icon_provider_.reset(new ScreenlockIconProvider);
   ScreenlockIconSource* screenlock_icon_source =
       new ScreenlockIconSource(screenlock_icon_provider_->AsWeakPtr());
-  content::URLDataSource::Add(
-      GetAssociatedWebUI()->GetWebContents()->GetBrowserContext(),
-      screenlock_icon_source);
+  content::URLDataSource::Add(web_ui()->GetWebContents()->GetBrowserContext(),
+                              screenlock_icon_source);
 }
 
 void ScreenLocker::OnAuthFailure(const AuthFailure& error) {
@@ -182,14 +194,19 @@ void ScreenLocker::OnAuthFailure(const AuthFailure& error) {
     UMA_HISTOGRAM_TIMES("ScreenLocker.AuthenticationFailureTime", delta);
   }
 
+  UMA_HISTOGRAM_ENUMERATION(
+      "ScreenLocker.AuthenticationFailure",
+      is_pin_attempt_ ? UnlockType::AUTH_PIN : UnlockType::AUTH_PASSWORD,
+      UnlockType::AUTH_COUNT);
+
   EnableInput();
   // Don't enable signout button here as we're showing
   // MessageBubble.
 
-  delegate_->ShowErrorMessage(incorrect_passwords_count_++ ?
-                                  IDS_LOGIN_ERROR_AUTHENTICATING_2ND_TIME :
-                                  IDS_LOGIN_ERROR_AUTHENTICATING,
-                              HelpAppLauncher::HELP_CANT_ACCESS_ACCOUNT);
+  web_ui()->ShowErrorMessage(incorrect_passwords_count_++
+                                 ? IDS_LOGIN_ERROR_AUTHENTICATING_2ND_TIME
+                                 : IDS_LOGIN_ERROR_AUTHENTICATING,
+                             HelpAppLauncher::HELP_CANT_ACCESS_ACCOUNT);
 
   if (auth_status_consumer_)
     auth_status_consumer_->OnAuthFailure(error);
@@ -206,6 +223,11 @@ void ScreenLocker::OnAuthSuccess(const UserContext& user_context) {
     UMA_HISTOGRAM_TIMES("ScreenLocker.AuthenticationSuccessTime", delta);
   }
 
+  UMA_HISTOGRAM_ENUMERATION(
+      "ScreenLocker.AuthenticationSuccess",
+      is_pin_attempt_ ? UnlockType::AUTH_PIN : UnlockType::AUTH_PASSWORD,
+      UnlockType::AUTH_COUNT);
+
   const user_manager::User* user =
       user_manager::UserManager::Get()->FindUser(user_context.GetAccountId());
   if (user) {
@@ -214,6 +236,18 @@ void ScreenLocker::OnAuthSuccess(const UserContext& user_context) {
       user_manager::UserManager::Get()->SwitchActiveUser(
           user_context.GetAccountId());
     }
+
+    // Reset the number of PIN attempts available to the user. We always do this
+    // because:
+    // 1. If the user signed in with a PIN, that means they should be able to
+    //    continue signing in with a PIN.
+    // 2. If the user signed in with cryptohome keys, then the PIN timeout is
+    //    going to be reset as well, so it is safe to reset the unlock attempt
+    //    count.
+    PinStorage* pin_storage = PinStorageFactory::GetForUser(user);
+    if (pin_storage)
+      pin_storage->ResetUnlockAttemptCount();
+
     UserSessionManager::GetInstance()->UpdateEasyUnlockKeys(user_context);
   } else {
     NOTREACHED() << "Logged in user not found.";
@@ -224,12 +258,19 @@ void ScreenLocker::OnAuthSuccess(const UserContext& user_context) {
 
   // Add guard for case when something get broken in call chain to unlock
   // for sure.
-  base::MessageLoop::current()->PostDelayedTask(
-      FROM_HERE,
-      base::Bind(&ScreenLocker::UnlockOnLoginSuccess,
-          weak_factory_.GetWeakPtr()),
+  base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+      FROM_HERE, base::Bind(&ScreenLocker::UnlockOnLoginSuccess,
+                            weak_factory_.GetWeakPtr()),
       base::TimeDelta::FromMilliseconds(kUnlockGuardTimeoutMs));
-  delegate_->AnimateAuthenticationSuccess();
+  web_ui()->AnimateAuthenticationSuccess();
+}
+
+void ScreenLocker::OnPasswordAuthSuccess(const UserContext& user_context) {
+  // The user has signed in using their password, so reset the PIN timeout.
+  PinStorage* pin_storage =
+      PinStorageFactory::GetForAccountId(user_context.GetAccountId());
+  if (pin_storage)
+    pin_storage->MarkStrongAuth();
 }
 
 void ScreenLocker::UnlockOnLoginSuccess() {
@@ -251,30 +292,43 @@ void ScreenLocker::UnlockOnLoginSuccess() {
 }
 
 void ScreenLocker::Authenticate(const UserContext& user_context) {
-  LOG_ASSERT(IsUserLoggedIn(user_context.GetAccountId().GetUserEmail()))
+  LOG_ASSERT(IsUserLoggedIn(user_context.GetAccountId()))
       << "Invalid user trying to unlock.";
 
   authentication_start_time_ = base::Time::Now();
-  delegate_->SetInputEnabled(false);
-  delegate_->OnAuthenticate();
+  web_ui()->SetInputEnabled(false);
+  is_pin_attempt_ = user_context.IsUsingPin();
 
-  // Special case: supervised users. Use special authenticator.
-  if (const user_manager::User* user =
-          FindUnlockUser(user_context.GetAccountId().GetUserEmail())) {
+  const user_manager::User* user = FindUnlockUser(user_context.GetAccountId());
+  if (user) {
+    // Check to see if the user submitted a PIN and it is valid.
+    const std::string pin = user_context.GetKey()->GetSecret();
+
+    // We only want to try authenticating the pin if it is a number,
+    // otherwise we will timeout PIN if the user enters their account password
+    // incorrectly more than a few times.
+    int dummy_value;
+    if (is_pin_attempt_ && base::StringToInt(pin, &dummy_value)) {
+      chromeos::PinStorage* pin_storage =
+          chromeos::PinStorageFactory::GetForUser(user);
+      if (pin_storage && pin_storage->TryAuthenticatePin(pin)) {
+        OnAuthSuccess(user_context);
+        return;
+      }
+    }
+
+    // Special case: supervised users. Use special authenticator.
     if (user->GetType() == user_manager::USER_TYPE_SUPERVISED) {
       UserContext updated_context = ChromeUserManager::Get()
                                         ->GetSupervisedUserManager()
                                         ->GetAuthentication()
                                         ->TransformKey(user_context);
-      // TODO(antrim) : replace empty closure with explicit method.
-      // http://crbug.com/351268
       BrowserThread::PostTask(
-          BrowserThread::UI,
-          FROM_HERE,
+          BrowserThread::UI, FROM_HERE,
           base::Bind(&ExtendedAuthenticator::AuthenticateToCheck,
-                     extended_authenticator_.get(),
-                     updated_context,
-                     base::Closure()));
+                     extended_authenticator_.get(), updated_context,
+                     base::Bind(&ScreenLocker::OnPasswordAuthSuccess,
+                                weak_factory_.GetWeakPtr(), updated_context)));
       return;
     }
   }
@@ -282,31 +336,26 @@ void ScreenLocker::Authenticate(const UserContext& user_context) {
   BrowserThread::PostTask(
       BrowserThread::UI, FROM_HERE,
       base::Bind(&ExtendedAuthenticator::AuthenticateToCheck,
-                 extended_authenticator_.get(),
-                 user_context,
-                 base::Closure()));
+                 extended_authenticator_.get(), user_context,
+                 base::Bind(&ScreenLocker::OnPasswordAuthSuccess,
+                            weak_factory_.GetWeakPtr(), user_context)));
 }
 
 const user_manager::User* ScreenLocker::FindUnlockUser(
-    const std::string& user_id) {
-  const user_manager::User* unlock_user = NULL;
-  for (user_manager::UserList::const_iterator it = users_.begin();
-       it != users_.end();
-       ++it) {
-    if ((*it)->email() == user_id) {
-      unlock_user = *it;
-      break;
-    }
+    const AccountId& account_id) {
+  for (const user_manager::User* user : users_) {
+    if (user->GetAccountId() == account_id)
+      return user;
   }
-  return unlock_user;
+  return nullptr;
 }
 
 void ScreenLocker::ClearErrors() {
-  delegate_->ClearErrors();
+  web_ui()->ClearErrors();
 }
 
 void ScreenLocker::Signout() {
-  delegate_->ClearErrors();
+  web_ui()->ClearErrors();
   content::RecordAction(UserMetricsAction("ScreenLocker_Signout"));
   // We expect that this call will not wait for any user input.
   // If it changes at some point, we will need to force exit.
@@ -317,14 +366,14 @@ void ScreenLocker::Signout() {
 }
 
 void ScreenLocker::EnableInput() {
-  delegate_->SetInputEnabled(true);
+  web_ui()->SetInputEnabled(true);
 }
 
 void ScreenLocker::ShowErrorMessage(int error_msg_id,
                                     HelpAppLauncher::HelpTopic help_topic_id,
                                     bool sign_out_only) {
-  delegate_->SetInputEnabled(!sign_out_only);
-  delegate_->ShowErrorMessage(error_msg_id, help_topic_id);
+  web_ui()->SetInputEnabled(!sign_out_only);
+  web_ui()->ShowErrorMessage(error_msg_id, help_topic_id);
 }
 
 void ScreenLocker::SetLoginStatusConsumer(
@@ -389,8 +438,7 @@ void ScreenLocker::Show() {
   // visible while in fullscreen because the shelf makes it harder for a web
   // page or app to mimick the lock screen.
   ash::wm::WindowState* active_window_state = ash::wm::GetActiveWindowState();
-  if (active_window_state &&
-      active_window_state->IsFullscreen() &&
+  if (active_window_state && active_window_state->IsFullscreen() &&
       active_window_state->hide_shelf_when_fullscreen()) {
     const ash::wm::WMEvent event(ash::wm::WM_EVENT_TOGGLE_FULLSCREEN);
     active_window_state->OnWMEvent(&event);
@@ -431,7 +479,8 @@ void ScreenLocker::ScheduleDeletion() {
     return;
   VLOG(1) << "Deleting ScreenLocker " << screen_locker_;
 
-  ash::PlaySystemSoundIfSpokenFeedback(SOUND_UNLOCK);
+  AccessibilityManager::Get()->PlayEarcon(
+      SOUND_UNLOCK, PlaySoundOption::SPOKEN_FEEDBACK_ENABLED);
 
   delete screen_locker_;
   screen_locker_ = NULL;
@@ -448,9 +497,8 @@ ScreenLocker::~ScreenLocker() {
     authenticator_->SetConsumer(NULL);
   ClearErrors();
 
-  VLOG(1) << "Moving desktop background to unlocked container";
-  ash::Shell::GetInstance()->
-      desktop_background_controller()->MoveDesktopToUnlockedContainer();
+  VLOG(1) << "Moving wallpaper to unlocked container";
+  ash::WmShell::Get()->wallpaper_controller()->MoveToUnlockedContainer();
 
   screen_locker_ = NULL;
   bool state = false;
@@ -464,6 +512,9 @@ ScreenLocker::~ScreenLocker() {
   DBusThreadManager::Get()->GetSessionManagerClient()->
       NotifyLockScreenDismissed();
 
+  session_manager::SessionManager::Get()->SetSessionState(
+      session_manager::SessionState::ACTIVE);
+
   if (saved_ime_state_.get()) {
     input_method::InputMethodManager::Get()->SetState(saved_ime_state_);
   }
@@ -480,9 +531,8 @@ void ScreenLocker::ScreenLockReady() {
           << delta.InSecondsF() << " second(s)";
   UMA_HISTOGRAM_TIMES("ScreenLocker.ScreenLockTime", delta);
 
-  VLOG(1) << "Moving desktop background to locked container";
-  ash::Shell::GetInstance()->
-      desktop_background_controller()->MoveDesktopToLockedContainer();
+  VLOG(1) << "Moving wallpaper to locked container";
+  ash::WmShell::Get()->wallpaper_controller()->MoveToLockedContainer();
 
   bool state = true;
   VLOG(1) << "Emitting SCREEN_LOCK_STATE_CHANGED with state=" << state;
@@ -493,20 +543,17 @@ void ScreenLocker::ScreenLockReady() {
   VLOG(1) << "Calling session manager's HandleLockScreenShown D-Bus method";
   DBusThreadManager::Get()->GetSessionManagerClient()->NotifyLockScreenShown();
 
+  session_manager::SessionManager::Get()->SetSessionState(
+      session_manager::SessionState::LOCKED);
+
   input_method::InputMethodManager::Get()
       ->GetActiveIMEState()
       ->EnableLockScreenLayouts();
 }
 
-content::WebUI* ScreenLocker::GetAssociatedWebUI() {
-  return delegate_->GetAssociatedWebUI();
-}
-
-bool ScreenLocker::IsUserLoggedIn(const std::string& username) {
-  for (user_manager::UserList::const_iterator it = users_.begin();
-       it != users_.end();
-       ++it) {
-    if ((*it)->email() == username)
+bool ScreenLocker::IsUserLoggedIn(const AccountId& account_id) const {
+  for (user_manager::User* user : users_) {
+    if (user->GetAccountId() == account_id)
       return true;
   }
   return false;
