@@ -11,6 +11,7 @@
 #include "chrome/browser/chromeos/customization/customization_document.h"
 #include "chrome/browser/chromeos/login/screens/base_screen_delegate.h"
 #include "chrome/browser/chromeos/login/screens/eula_view.h"
+#include "chrome/browser/chromeos/policy/device_cloud_policy_manager_chromeos.h"
 #include "chromeos/dbus/cryptohome_client.h"
 #include "chromeos/dbus/dbus_method_call_status.h"
 #include "chromeos/dbus/dbus_thread_manager.h"
@@ -44,7 +45,10 @@ void EulaScreen::Show() {
   // Command to own the TPM.
   DBusThreadManager::Get()->GetCryptohomeClient()->TpmCanAttemptOwnership(
       EmptyVoidDBusMethodCallback());
-  if (view_)
+  if (policy::DeviceCloudPolicyManagerChromeOS::GetZeroTouchEnrollmentMode() ==
+      policy::ZeroTouchEnrollmentMode::HANDS_OFF)
+    OnUserAction(EulaModel::kUserActionAcceptButtonClicked);
+  else if (view_)
     view_->Show();
 }
 
