@@ -6,15 +6,19 @@
 
 #include <stddef.h>
 
+#include <utility>
+
 #include "base/location.h"
 #include "base/logging.h"
-#include "base/prefs/pref_service.h"
+#include "base/memory/ptr_util.h"
 #include "base/values.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/browser_process_platform_part.h"
 #include "chrome/browser/chromeos/login/lock/screen_locker.h"
+#include "chrome/browser/chromeos/login/quick_unlock/pin_storage.h"
+#include "chrome/browser/chromeos/login/quick_unlock/pin_storage_factory.h"
 #include "chrome/browser/chromeos/login/reauth_stats.h"
-#include "chrome/browser/chromeos/login/ui/login_display_host_impl.h"
+#include "chrome/browser/chromeos/login/ui/login_display_host.h"
 #include "chrome/browser/chromeos/login/ui/views/user_board_view.h"
 #include "chrome/browser/chromeos/login/users/chrome_user_manager.h"
 #include "chrome/browser/chromeos/login/users/multi_profile_user_controller.h"
@@ -23,6 +27,7 @@
 #include "chrome/browser/signin/easy_unlock_service.h"
 #include "chrome/browser/ui/webui/chromeos/login/l10n_util.h"
 #include "chrome/browser/ui/webui/chromeos/login/signin_screen_handler.h"
+#include "components/prefs/pref_service.h"
 #include "components/proximity_auth/screenlock_bridge.h"
 #include "components/signin/core/account_id/account_id.h"
 #include "components/user_manager/known_user.h"
@@ -44,6 +49,7 @@ const char kKeyPublicAccount[] = "publicAccount";
 const char kKeyLegacySupervisedUser[] = "legacySupervisedUser";
 const char kKeyChildUser[] = "childUser";
 const char kKeyDesktopUser[] = "isDesktopUser";
+const char kKeyShowPin[] = "showPin";
 const char kKeySignedIn[] = "signedIn";
 const char kKeyCanRemove[] = "canRemove";
 const char kKeyIsOwner[] = "isOwner";
@@ -80,7 +86,7 @@ void AddPublicSessionDetailsToUserDictionaryEntry(
 
   // Construct the list of available locales. This list consists of the
   // recommended locales, followed by all others.
-  scoped_ptr<base::ListValue> available_locales =
+  std::unique_ptr<base::ListValue> available_locales =
       GetUILanguageList(&recommended_locales, std::string());
 
   // Select the the first recommended locale that is actually available or the
@@ -111,6 +117,18 @@ void AddPublicSessionDetailsToUserDictionaryEntry(
                  GetCurrentKeyboardLayout().release());
 }
 
+// Returns true if the PIN keyboard should be displayed for the given |user|.
+bool CanShowPinForUser(user_manager::User* user) {
+  if (!user->is_logged_in())
+    return false;
+
+  PinStorage* pin_storage = PinStorageFactory::GetForUser(user);
+  if (!pin_storage)
+    return false;
+
+  return pin_storage->IsPinAuthenticationAvailable();
+}
+
 }  // namespace
 
 UserSelectionScreen::UserSelectionScreen(const std::string& display_type)
@@ -124,7 +142,7 @@ UserSelectionScreen::UserSelectionScreen(const std::string& display_type)
 UserSelectionScreen::~UserSelectionScreen() {
   proximity_auth::ScreenlockBridge::Get()->SetLockHandler(nullptr);
   ui::UserActivityDetector* activity_detector = ui::UserActivityDetector::Get();
-  if (activity_detector->HasObserver(this))
+  if (activity_detector && activity_detector->HasObserver(this))
     activity_detector->RemoveObserver(this);
 }
 
@@ -159,6 +177,7 @@ void UserSelectionScreen::FillUserDictionary(
   user_dict->SetBoolean(kKeyChildUser, is_child_user);
   user_dict->SetBoolean(kKeyDesktopUser, false);
   user_dict->SetInteger(kKeyInitialAuthType, auth_type);
+  user_dict->SetBoolean(kKeyShowPin, CanShowPinForUser(user));
   user_dict->SetBoolean(kKeySignedIn, user->is_logged_in());
   user_dict->SetBoolean(kKeyIsOwner, is_owner);
 
@@ -185,7 +204,7 @@ void UserSelectionScreen::FillMultiProfileUserPrefs(
     user_manager::User* user,
     base::DictionaryValue* user_dict,
     bool is_signin_to_add) {
-  const std::string& user_id = user->email();
+  const std::string& user_id = user->GetAccountId().GetUserEmail();
 
   if (is_signin_to_add) {
     MultiProfileUserController* multi_profile_user_controller =
@@ -213,13 +232,10 @@ void UserSelectionScreen::FillMultiProfileUserPrefs(
 bool UserSelectionScreen::ShouldForceOnlineSignIn(
     const user_manager::User* user) {
   // Public sessions are always allowed to log in offline.
-  // Supervised user are allowed to log in offline if their OAuth token status
-  // is unknown or valid.
+  // Supervised users are always allowed to log in offline.
   // For all other users, force online sign in if:
   // * The flag to force online sign-in is set for the user.
-  // * The user's OAuth token is invalid.
-  // * The user's OAuth token status is unknown (except supervised users,
-  //   see above).
+  // * The user's OAuth token is invalid or unknown.
   if (user->is_logged_in())
     return false;
 
@@ -230,10 +246,8 @@ bool UserSelectionScreen::ShouldForceOnlineSignIn(
   const bool is_public_session =
       user->GetType() == user_manager::USER_TYPE_PUBLIC_ACCOUNT;
 
-  if (is_supervised_user &&
-      token_status == user_manager::User::OAUTH_TOKEN_STATUS_UNKNOWN) {
+  if (is_supervised_user)
     return false;
-  }
 
   if (is_public_session)
     return false;
@@ -262,7 +276,7 @@ void UserSelectionScreen::Init(const user_manager::UserList& users,
   show_guest_ = show_guest;
 
   ui::UserActivityDetector* activity_detector = ui::UserActivityDetector::Get();
-  if (!activity_detector->HasObserver(this))
+  if (activity_detector && !activity_detector->HasObserver(this))
     activity_detector->AddObserver(this);
 }
 
@@ -346,7 +360,7 @@ void UserSelectionScreen::SendUserList() {
   // TODO(nkostylev): Move to a separate method in UserManager.
   // http://crbug.com/230852
   bool single_user = users_.size() == 1;
-  bool is_signin_to_add = LoginDisplayHostImpl::default_host() &&
+  bool is_signin_to_add = LoginDisplayHost::default_host() &&
                           user_manager::UserManager::Get()->IsUserLoggedIn();
   std::string owner_email;
   chromeos::CrosSettings::Get()->GetString(chromeos::kDeviceOwner,
@@ -377,18 +391,14 @@ void UserSelectionScreen::SendUserList() {
                                                           : OFFLINE_PASSWORD);
     user_auth_type_map_[account_id] = initial_auth_type;
 
-    base::DictionaryValue* user_dict = new base::DictionaryValue();
+    auto user_dict = base::MakeUnique<base::DictionaryValue>();
     const std::vector<std::string>* public_session_recommended_locales =
         public_session_recommended_locales_.find(account_id) ==
                 public_session_recommended_locales_.end()
             ? &kEmptyRecommendedLocales
             : &public_session_recommended_locales_[account_id];
-    FillUserDictionary(*it,
-                       is_owner,
-                       is_signin_to_add,
-                       initial_auth_type,
-                       public_session_recommended_locales,
-                       user_dict);
+    FillUserDictionary(*it, is_owner, is_signin_to_add, initial_auth_type,
+                       public_session_recommended_locales, user_dict.get());
     bool signed_in = (*it)->is_logged_in();
 
     // Single user check here is necessary because owner info might not be
@@ -398,7 +408,7 @@ void UserSelectionScreen::SendUserList() {
         ((!single_user || is_enterprise_managed) && account_id.is_valid() &&
          !is_owner && !is_public_account && !signed_in && !is_signin_to_add);
     user_dict->SetBoolean(kKeyCanRemove, can_remove_user);
-    users_list.Append(user_dict);
+    users_list.Append(std::move(user_dict));
   }
 
   handler_->LoadUsers(users_list, show_guest_);
@@ -473,7 +483,8 @@ void UserSelectionScreen::ShowUserPodCustomIcon(
     const AccountId& account_id,
     const proximity_auth::ScreenlockBridge::UserPodCustomIconOptions&
         icon_options) {
-  scoped_ptr<base::DictionaryValue> icon = icon_options.ToDictionaryValue();
+  std::unique_ptr<base::DictionaryValue> icon =
+      icon_options.ToDictionaryValue();
   if (!icon || icon->empty())
     return;
   view_->ShowUserPodCustomIcon(account_id, *icon);
