@@ -4,8 +4,12 @@
 
 #include "chrome/browser/chromeos/login/supervised/supervised_user_creation_screen.h"
 
-#include "ash/desktop_background/desktop_background_controller.h"
-#include "ash/shell.h"
+#include <utility>
+
+#include "ash/common/shelf/wm_shelf.h"
+#include "ash/common/wallpaper/wallpaper_controller.h"
+#include "ash/common/wm_shell.h"
+#include "base/memory/ptr_util.h"
 #include "base/rand_util.h"
 #include "base/values.h"
 #include "chrome/browser/chromeos/camera_detector.h"
@@ -29,6 +33,7 @@
 #include "chrome/browser/supervised_user/legacy/supervised_user_sync_service.h"
 #include "chrome/browser/supervised_user/legacy/supervised_user_sync_service_factory.h"
 #include "chrome/browser/supervised_user/supervised_user_constants.h"
+#include "chrome/browser/ui/webui/chromeos/login/oobe_ui.h"
 #include "chrome/grit/generated_resources.h"
 #include "chromeos/dbus/dbus_thread_manager.h"
 #include "chromeos/dbus/session_manager_client.h"
@@ -369,10 +374,11 @@ void SupervisedUserCreationScreen::OnManagerFullyAuthenticated(
       ->NotifySupervisedUserCreationStarted();
   manager_signin_in_progress_ = false;
   DCHECK(controller_.get());
-  // For manager user, move desktop to locked container so that windows created
-  // during the user image picker step are below it.
-  ash::Shell::GetInstance()->
-      desktop_background_controller()->MoveDesktopToLockedContainer();
+  // For manager user, move wallpaper to locked container so that windows
+  // created during the user image picker step are below it.
+  ash::WmShell::Get()->wallpaper_controller()->MoveToLockedContainer();
+  ash::WmShelf::ForWindow(ash::WmShell::Get()->GetPrimaryRootWindow())
+      ->SetAlignment(ash::ShelfAlignment::SHELF_ALIGNMENT_BOTTOM_LOCKED);
 
   controller_->SetManagerProfile(manager_profile);
   if (actor_)
@@ -522,7 +528,7 @@ void SupervisedUserCreationScreen::OnCameraPresenceCheckDone(
 void SupervisedUserCreationScreen::OnGetSupervisedUsers(
     const base::DictionaryValue* users) {
   // Copy for passing to WebUI, contains only id, name and avatar URL.
-  scoped_ptr<base::ListValue> ui_users(new base::ListValue());
+  std::unique_ptr<base::ListValue> ui_users(new base::ListValue());
   SupervisedUserManager* supervised_user_manager =
       ChromeUserManager::Get()->GetSupervisedUserManager();
 
@@ -535,8 +541,7 @@ void SupervisedUserCreationScreen::OnGetSupervisedUsers(
         static_cast<base::DictionaryValue*>(it.value().DeepCopy());
     // Copy that would be passed to WebUI. It has some extra values for
     // displaying, but does not contain sensitive data, such as master password.
-    base::DictionaryValue* ui_copy =
-        static_cast<base::DictionaryValue*>(new base::DictionaryValue());
+    auto ui_copy = base::MakeUnique<base::DictionaryValue>();
 
     int avatar_index = SupervisedUserCreationController::kDummyAvatarIndex;
     std::string chromeos_avatar;
@@ -587,7 +592,7 @@ void SupervisedUserCreationScreen::OnGetSupervisedUsers(
     ui_copy->SetString("id", it.key());
 
     existing_users_->Set(it.key(), local_copy);
-    ui_users->Append(ui_copy);
+    ui_users->Append(std::move(ui_copy));
   }
   actor_->ShowExistingSupervisedUsers(ui_users.get());
 }
