@@ -36,14 +36,14 @@
 #include "chromeos/cryptohome/mock_homedir_methods.h"
 #include "chromeos/login/auth/key.h"
 #include "chromeos/login/auth/user_context.h"
+#include "components/sync/model/attachments/attachment_service_proxy_for_test.h"
+#include "components/sync/model/fake_sync_change_processor.h"
+#include "components/sync/model/sync_change.h"
+#include "components/sync/model/sync_error_factory_mock.h"
+#include "components/sync/protocol/sync.pb.h"
 #include "content/public/browser/notification_service.h"
 #include "content/public/test/browser_test_utils.h"
 #include "content/public/test/test_utils.h"
-#include "sync/api/fake_sync_change_processor.h"
-#include "sync/api/sync_change.h"
-#include "sync/api/sync_error_factory_mock.h"
-#include "sync/internal_api/public/attachments/attachment_service_proxy_for_test.h"
-#include "sync/protocol/sync.pb.h"
 
 using testing::_;
 using base::StringPrintf;
@@ -63,15 +63,15 @@ SupervisedUsersSyncTestAdapter::SupervisedUsersSyncTestAdapter(Profile* profile)
   service_ = SupervisedUserSyncServiceFactory::GetForProfile(profile);
   processor_ = new syncer::FakeSyncChangeProcessor();
   service_->MergeDataAndStartSyncing(
-      syncer::SUPERVISED_USERS,
-      syncer::SyncDataList(),
-      scoped_ptr<syncer::SyncChangeProcessor>(processor_),
-      scoped_ptr<syncer::SyncErrorFactory>(new syncer::SyncErrorFactoryMock));
+      syncer::SUPERVISED_USERS, syncer::SyncDataList(),
+      std::unique_ptr<syncer::SyncChangeProcessor>(processor_),
+      std::unique_ptr<syncer::SyncErrorFactory>(
+          new syncer::SyncErrorFactoryMock));
 }
 
-scoped_ptr< ::sync_pb::ManagedUserSpecifics>
+std::unique_ptr<::sync_pb::ManagedUserSpecifics>
 SupervisedUsersSyncTestAdapter::GetFirstChange() {
-  scoped_ptr< ::sync_pb::ManagedUserSpecifics> result(
+  std::unique_ptr<::sync_pb::ManagedUserSpecifics> result(
       new ::sync_pb::ManagedUserSpecifics);
   CHECK(HasChanges())
       << "GetFirstChange() should only be callled if HasChanges() is true";
@@ -112,15 +112,15 @@ SupervisedUsersSharedSettingsSyncTestAdapter::
       SupervisedUserSharedSettingsServiceFactory::GetForBrowserContext(profile);
   processor_ = new syncer::FakeSyncChangeProcessor();
   service_->MergeDataAndStartSyncing(
-      syncer::SUPERVISED_USER_SHARED_SETTINGS,
-      syncer::SyncDataList(),
-      scoped_ptr<syncer::SyncChangeProcessor>(processor_),
-      scoped_ptr<syncer::SyncErrorFactory>(new syncer::SyncErrorFactoryMock));
+      syncer::SUPERVISED_USER_SHARED_SETTINGS, syncer::SyncDataList(),
+      std::unique_ptr<syncer::SyncChangeProcessor>(processor_),
+      std::unique_ptr<syncer::SyncErrorFactory>(
+          new syncer::SyncErrorFactoryMock));
 }
 
-scoped_ptr< ::sync_pb::ManagedUserSharedSettingSpecifics>
+std::unique_ptr<::sync_pb::ManagedUserSharedSettingSpecifics>
 SupervisedUsersSharedSettingsSyncTestAdapter::GetFirstChange() {
-  scoped_ptr< ::sync_pb::ManagedUserSharedSettingSpecifics> result(
+  std::unique_ptr<::sync_pb::ManagedUserSharedSettingSpecifics> result(
       new ::sync_pb::ManagedUserSharedSettingSpecifics);
   CHECK(HasChanges())
       << "GetFirstChange() should only be callled if HasChanges() is true";
@@ -296,7 +296,6 @@ void SupervisedUserTestBase::StartFlowLoginAsManager() {
   JSExpect("!$('supervised-user-creation-next-button').disabled");
   UserContext user_context(AccountId::FromUserEmailGaiaId(
       kTestManager, GetGaiaIDForUserID(kTestManager)));
-  user_context.SetGaiaID(GetGaiaIDForUserID(kTestManager));
   user_context.SetKey(Key(kTestManagerPassword));
   SetExpectedCredentials(user_context);
   content::WindowedNotificationObserver login_observer(
@@ -374,9 +373,9 @@ void SupervisedUserTestBase::SigninAsSupervisedUser(
   // Clean first run flag before logging in.
   static_cast<SupervisedUserManagerImpl*>(
       ChromeUserManager::Get()->GetSupervisedUserManager())
-      ->CheckForFirstRun(user->email());
+      ->CheckForFirstRun(user->GetAccountId().GetUserEmail());
 
-  LoginUser(user->email());
+  LoginUser(user->GetAccountId().GetUserEmail());
   if (check_homedir_calls)
     ::testing::Mock::VerifyAndClearExpectations(mock_homedir_methods_);
   Profile* profile = ProfileHelper::Get()->GetProfileByUserUnsafe(user);
@@ -395,7 +394,7 @@ void SupervisedUserTestBase::SigninAsManager(int user_index) {
   // Created supervised user have to be first in a list.
   const user_manager::User* user =
       user_manager::UserManager::Get()->GetUsers().at(user_index);
-  LoginUser(user->email());
+  LoginUser(user->GetAccountId().GetUserEmail());
   Profile* profile = ProfileHelper::Get()->GetProfileByUserUnsafe(user);
   shared_settings_adapter_.reset(
       new SupervisedUsersSharedSettingsSyncTestAdapter(profile));
