@@ -9,7 +9,7 @@
 #include "chrome/browser/chromeos/login/screens/chrome_user_selection_screen.h"
 #include "chrome/browser/chromeos/login/signin_screen_controller.h"
 #include "chrome/browser/chromeos/login/startup_utils.h"
-#include "chrome/browser/chromeos/login/ui/login_display_host_impl.h"
+#include "chrome/browser/chromeos/login/ui/login_display_host.h"
 #include "chrome/browser/chromeos/login/ui/user_adding_screen.h"
 #include "chrome/browser/chromeos/login/ui/webui_login_view.h"
 #include "chrome/browser/chromeos/login/users/wallpaper/wallpaper_manager.h"
@@ -17,10 +17,10 @@
 #include "chrome/browser/ui/browser_window.h"
 #include "chrome/grit/chromium_strings.h"
 #include "chrome/grit/generated_resources.h"
-#include "chromeos/login/user_names.h"
 #include "components/signin/core/account_id/account_id.h"
+#include "components/strings/grit/components_strings.h"
 #include "components/user_manager/user_manager.h"
-#include "grit/components_strings.h"
+#include "components/user_manager/user_names.h"
 #include "ui/base/ime/chromeos/ime_keyboard.h"
 #include "ui/base/ime/chromeos/input_method_manager.h"
 #include "ui/base/l10n/l10n_util.h"
@@ -35,7 +35,7 @@ WebUILoginDisplay::~WebUILoginDisplay() {
   if (webui_handler_)
     webui_handler_->ResetSigninScreenHandlerDelegate();
   ui::UserActivityDetector* activity_detector = ui::UserActivityDetector::Get();
-  if (activity_detector->HasObserver(this))
+  if (activity_detector && activity_detector->HasObserver(this))
     activity_detector->RemoveObserver(this);
 }
 
@@ -52,16 +52,18 @@ void WebUILoginDisplay::ClearAndEnablePassword() {
 void WebUILoginDisplay::Init(const user_manager::UserList& users,
                              bool show_guest,
                              bool show_users,
-                             bool show_new_user) {
+                             bool allow_new_user) {
   // Testing that the delegate has been set.
   DCHECK(delegate_);
   SignInScreenController::Get()->Init(users, show_guest);
   show_guest_ = show_guest;
+  show_users_changed_ = (show_users_ != show_users);
   show_users_ = show_users;
-  show_new_user_ = show_new_user;
+  allow_new_user_changed_ = (allow_new_user_ != allow_new_user);
+  allow_new_user_ = allow_new_user;
 
   ui::UserActivityDetector* activity_detector = ui::UserActivityDetector::Get();
-  if (!activity_detector->HasObserver(this))
+  if (activity_detector && !activity_detector->HasObserver(this))
     activity_detector->AddObserver(this);
 }
 
@@ -97,11 +99,9 @@ void WebUILoginDisplay::SetUIEnabled(bool is_enabled) {
     ClearAndEnablePassword();
   }
 
-  if (chromeos::LoginDisplayHost* host =
-          chromeos::LoginDisplayHostImpl::default_host()) {
-    if (chromeos::WebUILoginView* login_view = host->GetWebUILoginView())
-      login_view->SetUIEnabled(is_enabled);
-  }
+  LoginDisplayHost* host = LoginDisplayHost::default_host();
+  if (host && host->GetWebUILoginView())
+    host->GetWebUILoginView()->SetUIEnabled(is_enabled);
 }
 
 void WebUILoginDisplay::ShowError(int error_msg_id,
@@ -115,10 +115,6 @@ void WebUILoginDisplay::ShowError(int error_msg_id,
 
   std::string error_text;
   switch (error_msg_id) {
-    case IDS_LOGIN_ERROR_AUTHENTICATING_HOSTED:
-      error_text = l10n_util::GetStringFUTF8(
-          error_msg_id, l10n_util::GetStringUTF16(IDS_SHORT_PRODUCT_OS_NAME));
-      break;
     case IDS_LOGIN_ERROR_CAPTIVE_PORTAL:
       error_text = l10n_util::GetStringFUTF8(
           error_msg_id, delegate()->GetConnectedNetworkName());
@@ -152,15 +148,8 @@ void WebUILoginDisplay::ShowError(int error_msg_id,
   }
 
   std::string help_link;
-  switch (error_msg_id) {
-    case IDS_LOGIN_ERROR_AUTHENTICATING_HOSTED:
-      help_link = l10n_util::GetStringUTF8(IDS_LEARN_MORE);
-      break;
-    default:
-      if (login_attempts > 1)
-        help_link = l10n_util::GetStringUTF8(IDS_LEARN_MORE);
-      break;
-  }
+  if (login_attempts > 1)
+    help_link = l10n_util::GetStringUTF8(IDS_LEARN_MORE);
 
   webui_handler_->ShowError(login_attempts, error_text, help_link,
                             help_topic_id);
@@ -189,6 +178,11 @@ void WebUILoginDisplay::ShowWhitelistCheckFailedError() {
     webui_handler_->ShowWhitelistCheckFailedError();
 }
 
+void WebUILoginDisplay::ShowUnrecoverableCrypthomeErrorDialog() {
+  if (webui_handler_)
+    webui_handler_->ShowUnrecoverableCrypthomeErrorDialog();
+}
+
 // WebUILoginDisplay, NativeWindowDelegate implementation: ---------------------
 gfx::NativeWindow WebUILoginDisplay::GetNativeWindow() const {
   return parent_window();
@@ -233,7 +227,8 @@ void WebUILoginDisplay::LoadWallpaper(const AccountId& account_id) {
 }
 
 void WebUILoginDisplay::LoadSigninWallpaper() {
-  WallpaperManager::Get()->SetDefaultWallpaperDelayed(login::SignInAccountId());
+  WallpaperManager::Get()->SetDefaultWallpaperDelayed(
+      user_manager::SignInAccountId());
 }
 
 void WebUILoginDisplay::OnSigninScreenReady() {
@@ -243,6 +238,11 @@ void WebUILoginDisplay::OnSigninScreenReady() {
     delegate_->OnSigninScreenReady();
 }
 
+void WebUILoginDisplay::OnGaiaScreenReady() {
+  if (delegate_)
+    delegate_->OnGaiaScreenReady();
+}
+
 void WebUILoginDisplay::RemoveUser(const AccountId& account_id) {
   SignInScreenController::Get()->RemoveUser(account_id);
 }
@@ -299,6 +299,18 @@ bool WebUILoginDisplay::IsShowUsers() const {
   return show_users_;
 }
 
+bool WebUILoginDisplay::ShowUsersHasChanged() const {
+  return show_users_changed_;
+}
+
+bool WebUILoginDisplay::IsAllowNewUser() const {
+  return allow_new_user_;
+}
+
+bool WebUILoginDisplay::AllowNewUserChanged() const {
+  return allow_new_user_changed_;
+}
+
 bool WebUILoginDisplay::IsSigninInProgress() const {
   return delegate_->IsSigninInProgress();
 }
@@ -318,7 +330,7 @@ void WebUILoginDisplay::Signout() {
 
 void WebUILoginDisplay::OnUserActivity(const ui::Event* event) {
   if (delegate_)
-    delegate_->ResetPublicSessionAutoLoginTimer();
+    delegate_->ResetAutoLoginTimer();
 }
 
 bool WebUILoginDisplay::IsUserWhitelisted(const AccountId& account_id) {
