@@ -29,6 +29,7 @@ class UserFlow {
   // Indicates if screen locking should be enabled or disabled for a flow.
   virtual bool CanLockScreen() = 0;
   virtual bool ShouldShowSettings() = 0;
+  virtual bool ShouldShowNotificationTray() = 0;
   virtual bool ShouldLaunchBrowser() = 0;
   virtual bool ShouldSkipPostLoginScreens() = 0;
   virtual bool SupportsEarlyRestartToApplyFlags() = 0;
@@ -56,6 +57,7 @@ class DefaultUserFlow : public UserFlow {
   void AppendAdditionalCommandLineSwitches() override;
   bool CanLockScreen() override;
   bool ShouldShowSettings() override;
+  bool ShouldShowNotificationTray() override;
   bool ShouldLaunchBrowser() override;
   bool ShouldSkipPostLoginScreens() override;
   bool SupportsEarlyRestartToApplyFlags() override;
@@ -75,6 +77,7 @@ class ExtendedUserFlow : public UserFlow {
 
   void AppendAdditionalCommandLineSwitches() override;
   bool ShouldShowSettings() override;
+  bool ShouldShowNotificationTray() override;
   void HandleOAuthTokenStatusChange(
       user_manager::User::OAuthTokenStatus status) override;
 
