@@ -2,12 +2,16 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "chrome/browser/chromeos/login/users/avatar/user_image_manager.h"
+
 #include <stdint.h>
 
 #include <map>
+#include <memory>
 #include <string>
 #include <vector>
 
+#include "base/command_line.h"
 #include "base/compiler_specific.h"
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
@@ -16,20 +20,15 @@
 #include "base/memory/linked_ptr.h"
 #include "base/memory/ref_counted.h"
 #include "base/memory/ref_counted_memory.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/path_service.h"
-#include "base/prefs/pref_change_registrar.h"
-#include "base/prefs/pref_service.h"
-#include "base/prefs/scoped_user_pref_update.h"
 #include "base/run_loop.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/time/time.h"
 #include "base/values.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/chrome_notification_types.h"
 #include "chrome/browser/chromeos/login/login_manager_test.h"
 #include "chrome/browser/chromeos/login/startup_utils.h"
-#include "chrome/browser/chromeos/login/users/avatar/user_image_manager.h"
 #include "chrome/browser/chromeos/login/users/avatar/user_image_manager_impl.h"
 #include "chrome/browser/chromeos/login/users/avatar/user_image_manager_test_util.h"
 #include "chrome/browser/chromeos/login/users/chrome_user_manager.h"
@@ -47,6 +46,7 @@
 #include "chrome/test/base/in_process_browser_test.h"
 #include "chrome/test/base/testing_browser_process.h"
 #include "chromeos/chromeos_paths.h"
+#include "chromeos/chromeos_switches.h"
 #include "chromeos/dbus/cryptohome_client.h"
 #include "chromeos/dbus/dbus_thread_manager.h"
 #include "chromeos/dbus/fake_session_manager_client.h"
@@ -54,6 +54,10 @@
 #include "components/policy/core/common/cloud/cloud_policy_core.h"
 #include "components/policy/core/common/cloud/cloud_policy_store.h"
 #include "components/policy/core/common/cloud/policy_builder.h"
+#include "components/policy/proto/cloud_policy.pb.h"
+#include "components/prefs/pref_change_registrar.h"
+#include "components/prefs/pref_service.h"
+#include "components/prefs/scoped_user_pref_update.h"
 #include "components/user_manager/user.h"
 #include "components/user_manager/user_image/user_image.h"
 #include "components/user_manager/user_manager.h"
@@ -67,7 +71,6 @@
 #include "net/url_request/test_url_fetcher_factory.h"
 #include "net/url_request/url_fetcher_delegate.h"
 #include "net/url_request/url_request_status.h"
-#include "policy/proto/cloud_policy.pb.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/base/layout.h"
@@ -105,9 +108,7 @@ policy::CloudPolicyStore* GetStoreForUser(const user_manager::User* user) {
 class UserImageManagerTest : public LoginManagerTest,
                              public user_manager::UserManager::Observer {
  protected:
-  UserImageManagerTest()
-      : LoginManagerTest(true),
-        enterprise_account_id_(AccountId::FromUserEmail(kEnterpriseUser1)) {}
+  UserImageManagerTest() : LoginManagerTest(true) {}
 
   // LoginManagerTest overrides:
   void SetUpInProcessBrowserTestFixture() override {
@@ -117,6 +118,16 @@ class UserImageManagerTest : public LoginManagerTest,
     ASSERT_TRUE(PathService::Get(chrome::DIR_USER_DATA, &user_data_dir_));
   }
 
+  void SetUpCommandLine(base::CommandLine* command_line) override {
+    LoginManagerTest::SetUpCommandLine(command_line);
+    // These tests create new users and then inject policy after the fact,
+    // to avoid having to set up a mock policy server. UserCloudPolicyManager
+    // will shut down the profile if there's an error loading the initial
+    // policy, so disable this behavior so we can inject policy directly.
+    command_line->AppendSwitch(
+        chromeos::switches::kAllowFailedPolicyFetchForTest);
+  }
+
   void SetUpOnMainThread() override {
     LoginManagerTest::SetUpOnMainThread();
     local_state_ = g_browser_process->local_state();
@@ -140,25 +151,12 @@ class UserImageManagerTest : public LoginManagerTest,
         account_id, account_id.GetUserEmail(), false);
   }
 
-  // Stores old (pre-migration) user image info.
-  void SetOldUserImageInfo(const AccountId& account_id,
-                           int image_index,
-                           const base::FilePath& image_path) {
-    RegisterUser(account_id.GetUserEmail());
-    DictionaryPrefUpdate images_pref(local_state_, "UserImages");
-    base::DictionaryValue* image_properties = new base::DictionaryValue();
-    image_properties->Set("index", new base::FundamentalValue(image_index));
-    image_properties->Set(
-        "path" , new base::StringValue(image_path.value()));
-    images_pref->SetWithoutPathExpansion(account_id.GetUserEmail(),
-                                         image_properties);
-  }
-
-  // Verifies user image info in |images_pref| dictionary.
-  void ExpectUserImageInfo(const base::DictionaryValue* images_pref,
-                           const AccountId& account_id,
+  // Verifies user image info.
+  void ExpectUserImageInfo(const AccountId& account_id,
                            int image_index,
                            const base::FilePath& image_path) {
+    const base::DictionaryValue* images_pref =
+        local_state_->GetDictionary(UserImageManagerImpl::kUserImageProperties);
     ASSERT_TRUE(images_pref);
     const base::DictionaryValue* image_properties = NULL;
     images_pref->GetDictionaryWithoutPathExpansion(account_id.GetUserEmail(),
@@ -166,8 +164,11 @@ class UserImageManagerTest : public LoginManagerTest,
     ASSERT_TRUE(image_properties);
     int actual_image_index;
     std::string actual_image_path;
-    ASSERT_TRUE(image_properties->GetInteger("index", &actual_image_index) &&
-                image_properties->GetString("path", &actual_image_path));
+    ASSERT_TRUE(
+        image_properties->GetInteger(UserImageManagerImpl::kImageIndexNodeName,
+                                     &actual_image_index) &&
+        image_properties->GetString(UserImageManagerImpl::kImagePathNodeName,
+                                    &actual_image_path));
     EXPECT_EQ(image_index, actual_image_index);
     EXPECT_EQ(image_path.value(), actual_image_path);
   }
@@ -183,43 +184,6 @@ class UserImageManagerTest : public LoginManagerTest,
     ASSERT_FALSE(image_properties);
   }
 
-  // Verifies that old user image info matches |image_index| and |image_path|
-  // and that new user image info does not exist.
-  void ExpectOldUserImageInfo(const AccountId& account_id,
-                              int image_index,
-                              const base::FilePath& image_path) {
-    ExpectUserImageInfo(local_state_->GetDictionary("UserImages"), account_id,
-                        image_index, image_path);
-    ExpectNoUserImageInfo(local_state_->GetDictionary("user_image_info"),
-                          account_id);
-  }
-
-  // Verifies that new user image info matches |image_index| and |image_path|
-  // and that old user image info does not exist.
-  void ExpectNewUserImageInfo(const AccountId& account_id,
-                              int image_index,
-                              const base::FilePath& image_path) {
-    ExpectUserImageInfo(local_state_->GetDictionary("user_image_info"),
-                        account_id, image_index, image_path);
-    ExpectNoUserImageInfo(local_state_->GetDictionary("UserImages"),
-                          account_id);
-  }
-
-  // Sets bitmap |resource_id| as image for |account_id| and saves it to disk.
-  void SaveUserImagePNG(const AccountId& account_id, int resource_id) {
-    base::FilePath image_path = GetUserImagePath(account_id, "png");
-    scoped_refptr<base::RefCountedStaticMemory> image_data(
-        ResourceBundle::GetSharedInstance().LoadDataResourceBytesForScale(
-            resource_id, ui::SCALE_FACTOR_100P));
-    int written = base::WriteFile(
-        image_path,
-        reinterpret_cast<const char*>(image_data->front()),
-        image_data->size());
-    EXPECT_EQ(static_cast<int>(image_data->size()), written);
-    SetOldUserImageInfo(account_id, user_manager::User::USER_IMAGE_EXTERNAL,
-                        image_path);
-  }
-
   // Returns the image path for user |account_id| with specified |extension|.
   base::FilePath GetUserImagePath(const AccountId& account_id,
                                   const std::string& extension) {
@@ -294,7 +258,7 @@ class UserImageManagerTest : public LoginManagerTest,
     run_loop.Run();
 
     const user_manager::User* user =
-        user_manager::UserManager::Get()->GetLoggedInUser();
+        user_manager::UserManager::Get()->GetActiveUser();
     ASSERT_TRUE(user);
     UserImageManagerImpl* uim = reinterpret_cast<UserImageManagerImpl*>(
         ChromeUserManager::Get()->GetUserImageManager(user->GetAccountId()));
@@ -309,111 +273,37 @@ class UserImageManagerTest : public LoginManagerTest,
 
   PrefService* local_state_;
 
-  scoped_ptr<gfx::ImageSkia> decoded_image_;
+  std::unique_ptr<gfx::ImageSkia> decoded_image_;
 
-  scoped_ptr<base::RunLoop> run_loop_;
+  std::unique_ptr<base::RunLoop> run_loop_;
 
   const AccountId test_account_id1_ = AccountId::FromUserEmail(kTestUser1);
   const AccountId test_account_id2_ = AccountId::FromUserEmail(kTestUser2);
-  const AccountId enterprise_account_id_;
+  const AccountId enterprise_account_id_ =
+      AccountId::FromUserEmail(kEnterpriseUser1);
+  const cryptohome::Identification cryptohome_id_ =
+      cryptohome::Identification(enterprise_account_id_);
 
  private:
   DISALLOW_COPY_AND_ASSIGN(UserImageManagerTest);
 };
 
-IN_PROC_BROWSER_TEST_F(UserImageManagerTest, PRE_DefaultUserImagePreserved) {
-  // Setup an old default (stock) user image.
-  ScopedUserManagerEnabler(new MockUserManager);
-  SetOldUserImageInfo(test_account_id1_,
-                      default_user_image::kFirstDefaultImageIndex,
-                      base::FilePath());
-}
+IN_PROC_BROWSER_TEST_F(UserImageManagerTest, PRE_SaveAndLoadUserImage) {
+  RegisterUser(test_account_id1_.GetUserEmail());
 
-IN_PROC_BROWSER_TEST_F(UserImageManagerTest, DefaultUserImagePreserved) {
-  user_manager::UserManager::Get()->GetUsers();  // Load users.
-  // Old info preserved.
-  ExpectOldUserImageInfo(test_account_id1_,
-                         default_user_image::kFirstDefaultImageIndex,
-                         base::FilePath());
-  LogIn(test_account_id1_);
-  // Image info is migrated now.
-  ExpectNewUserImageInfo(test_account_id1_,
-                         default_user_image::kFirstDefaultImageIndex,
-                         base::FilePath());
-}
-
-IN_PROC_BROWSER_TEST_F(UserImageManagerTest, PRE_OtherUsersUnaffected) {
-  // Setup two users with stock images.
-  ScopedUserManagerEnabler(new MockUserManager);
-  SetOldUserImageInfo(test_account_id1_,
-                      default_user_image::kFirstDefaultImageIndex,
-                      base::FilePath());
-  SetOldUserImageInfo(test_account_id2_,
-                      default_user_image::kFirstDefaultImageIndex + 1,
-                      base::FilePath());
-}
-
-IN_PROC_BROWSER_TEST_F(UserImageManagerTest, OtherUsersUnaffected) {
-  user_manager::UserManager::Get()->GetUsers();  // Load users.
-  // Old info preserved.
-  ExpectOldUserImageInfo(test_account_id1_,
-                         default_user_image::kFirstDefaultImageIndex,
-                         base::FilePath());
-  ExpectOldUserImageInfo(test_account_id2_,
-                         default_user_image::kFirstDefaultImageIndex + 1,
-                         base::FilePath());
-  LogIn(test_account_id1_);
-  // Image info is migrated for the first user and unaffected for the rest.
-  ExpectNewUserImageInfo(test_account_id1_,
-                         default_user_image::kFirstDefaultImageIndex,
-                         base::FilePath());
-  ExpectOldUserImageInfo(test_account_id2_,
-                         default_user_image::kFirstDefaultImageIndex + 1,
-                         base::FilePath());
-}
-
-IN_PROC_BROWSER_TEST_F(UserImageManagerTest, PRE_PRE_NonJPEGImageFromFile) {
-  // Setup a user with non-JPEG image.
-  ScopedUserManagerEnabler(new MockUserManager);
-  SaveUserImagePNG(test_account_id1_,
-                   default_user_image::kDefaultImageResourceIDs
-                       [default_user_image::kFirstDefaultImageIndex]);
-}
-
-IN_PROC_BROWSER_TEST_F(UserImageManagerTest, PRE_NonJPEGImageFromFile) {
-  user_manager::UserManager::Get()->GetUsers();  // Load users.
-  // Old info preserved.
-  ExpectOldUserImageInfo(test_account_id1_,
-                         user_manager::User::USER_IMAGE_EXTERNAL,
-                         GetUserImagePath(test_account_id1_, "png"));
-  const user_manager::User* user =
-      user_manager::UserManager::Get()->FindUser(test_account_id1_);
-  EXPECT_TRUE(user->image_is_stub());
-
-  base::RunLoop run_loop;
-  PrefChangeRegistrar pref_change_registrar_;
-  pref_change_registrar_.Init(local_state_);
-  pref_change_registrar_.Add("UserImages", run_loop.QuitClosure());
-  LogIn(test_account_id1_);
-
-  // Wait for migration.
-  run_loop.Run();
-
-  // Image info is migrated and the image is converted to JPG.
-  ExpectNewUserImageInfo(test_account_id1_,
-                         user_manager::User::USER_IMAGE_EXTERNAL,
-                         GetUserImagePath(test_account_id1_, "jpg"));
-  user = user_manager::UserManager::Get()->GetLoggedInUser();
-  ASSERT_TRUE(user);
-  EXPECT_FALSE(user->image_is_safe_format());
-  // Check image dimensions.
-  const gfx::ImageSkia& saved_image = default_user_image::GetDefaultImage(
+  // Setup a user with JPEG image.
+  run_loop_.reset(new base::RunLoop);
+  const gfx::ImageSkia& image = default_user_image::GetDefaultImage(
       default_user_image::kFirstDefaultImageIndex);
-  EXPECT_EQ(saved_image.width(), user->GetImage().width());
-  EXPECT_EQ(saved_image.height(), user->GetImage().height());
+  UserImageManager* user_image_manager =
+      ChromeUserManager::Get()->GetUserImageManager(test_account_id1_);
+  user_image_manager->SaveUserImage(
+      user_manager::UserImage::CreateAndEncode(image));
+  run_loop_->Run();
 }
 
-IN_PROC_BROWSER_TEST_F(UserImageManagerTest, NonJPEGImageFromFile) {
+// Ensures that the user image in JPEG format is loaded correctly.
+IN_PROC_BROWSER_TEST_F(UserImageManagerTest, SaveAndLoadUserImage) {
   user_manager::UserManager::Get()->GetUsers();  // Load users.
   const user_manager::User* user =
       user_manager::UserManager::Get()->FindUser(test_account_id1_);
@@ -424,7 +314,7 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerTest, NonJPEGImageFromFile) {
         chrome::NOTIFICATION_LOGIN_USER_IMAGE_CHANGED,
         content::NotificationService::AllSources()).Wait();
   }
-  // Now the migrated image is used.
+  // The image should be in the safe format.
   EXPECT_TRUE(user->image_is_safe_format());
   // Check image dimensions. Images can't be compared since JPEG is lossy.
   const gfx::ImageSkia& saved_image = default_user_image::GetDefaultImage(
@@ -455,9 +345,9 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerTest, SaveUserDefaultImageIndex) {
   EXPECT_TRUE(user->HasDefaultImage());
   EXPECT_EQ(default_user_image::kFirstDefaultImageIndex, user->image_index());
   EXPECT_TRUE(test::AreImagesEqual(default_image, user->GetImage()));
-  ExpectNewUserImageInfo(test_account_id1_,
-                         default_user_image::kFirstDefaultImageIndex,
-                         base::FilePath());
+  ExpectUserImageInfo(test_account_id1_,
+                      default_user_image::kFirstDefaultImageIndex,
+                      base::FilePath());
 }
 
 IN_PROC_BROWSER_TEST_F(UserImageManagerTest, PRE_SaveUserImage) {
@@ -488,11 +378,11 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerTest, SaveUserImage) {
   EXPECT_FALSE(user->HasDefaultImage());
   EXPECT_EQ(user_manager::User::USER_IMAGE_EXTERNAL, user->image_index());
   EXPECT_TRUE(test::AreImagesEqual(custom_image, user->GetImage()));
-  ExpectNewUserImageInfo(test_account_id1_,
-                         user_manager::User::USER_IMAGE_EXTERNAL,
-                         GetUserImagePath(test_account_id1_, "jpg"));
+  ExpectUserImageInfo(test_account_id1_,
+                      user_manager::User::USER_IMAGE_EXTERNAL,
+                      GetUserImagePath(test_account_id1_, "jpg"));
 
-  const scoped_ptr<gfx::ImageSkia> saved_image =
+  const std::unique_ptr<gfx::ImageSkia> saved_image =
       test::ImageLoader(GetUserImagePath(test_account_id1_, "jpg")).Load();
   ASSERT_TRUE(saved_image);
 
@@ -514,7 +404,7 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerTest, SaveUserImageFromFile) {
 
   const base::FilePath custom_image_path =
       test_data_dir_.Append(test::kUserAvatarImage1RelativePath);
-  const scoped_ptr<gfx::ImageSkia> custom_image =
+  const std::unique_ptr<gfx::ImageSkia> custom_image =
       test::ImageLoader(custom_image_path).Load();
   ASSERT_TRUE(custom_image);
 
@@ -527,11 +417,11 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerTest, SaveUserImageFromFile) {
   EXPECT_FALSE(user->HasDefaultImage());
   EXPECT_EQ(user_manager::User::USER_IMAGE_EXTERNAL, user->image_index());
   EXPECT_TRUE(test::AreImagesEqual(*custom_image, user->GetImage()));
-  ExpectNewUserImageInfo(test_account_id1_,
-                         user_manager::User::USER_IMAGE_EXTERNAL,
-                         GetUserImagePath(test_account_id1_, "jpg"));
+  ExpectUserImageInfo(test_account_id1_,
+                      user_manager::User::USER_IMAGE_EXTERNAL,
+                      GetUserImagePath(test_account_id1_, "jpg"));
 
-  const scoped_ptr<gfx::ImageSkia> saved_image =
+  const std::unique_ptr<gfx::ImageSkia> saved_image =
       test::ImageLoader(GetUserImagePath(test_account_id1_, "jpg")).Load();
   ASSERT_TRUE(saved_image);
 
@@ -574,11 +464,10 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerTest, SaveUserImageFromProfileImage) {
   EXPECT_FALSE(user->HasDefaultImage());
   EXPECT_EQ(user_manager::User::USER_IMAGE_PROFILE, user->image_index());
   EXPECT_TRUE(test::AreImagesEqual(profile_image, user->GetImage()));
-  ExpectNewUserImageInfo(test_account_id1_,
-                         user_manager::User::USER_IMAGE_PROFILE,
-                         GetUserImagePath(test_account_id1_, "jpg"));
+  ExpectUserImageInfo(test_account_id1_, user_manager::User::USER_IMAGE_PROFILE,
+                      GetUserImagePath(test_account_id1_, "jpg"));
 
-  const scoped_ptr<gfx::ImageSkia> saved_image =
+  const std::unique_ptr<gfx::ImageSkia> saved_image =
       test::ImageLoader(GetUserImagePath(test_account_id1_, "jpg")).Load();
   ASSERT_TRUE(saved_image);
 
@@ -628,9 +517,9 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerTest,
   EXPECT_TRUE(user->HasDefaultImage());
   EXPECT_EQ(default_user_image::kFirstDefaultImageIndex, user->image_index());
   EXPECT_TRUE(test::AreImagesEqual(default_image, user->GetImage()));
-  ExpectNewUserImageInfo(test_account_id1_,
-                         default_user_image::kFirstDefaultImageIndex,
-                         base::FilePath());
+  ExpectUserImageInfo(test_account_id1_,
+                      default_user_image::kFirstDefaultImageIndex,
+                      base::FilePath());
 }
 
 class UserImageManagerPolicyTest : public UserImageManagerTest,
@@ -643,7 +532,7 @@ class UserImageManagerPolicyTest : public UserImageManagerTest,
   // UserImageManagerTest overrides:
   void SetUpInProcessBrowserTestFixture() override {
     DBusThreadManager::GetSetterForTesting()->SetSessionManagerClient(
-        scoped_ptr<SessionManagerClient>(fake_session_manager_client_));
+        std::unique_ptr<SessionManagerClient>(fake_session_manager_client_));
     UserImageManagerTest::SetUpInProcessBrowserTestFixture();
   }
 
@@ -654,8 +543,7 @@ class UserImageManagerPolicyTest : public UserImageManagerTest,
     ASSERT_TRUE(PathService::Get(chromeos::DIR_USER_POLICY_KEYS,
                                  &user_keys_dir));
     const std::string sanitized_username =
-        chromeos::CryptohomeClient::GetStubSanitizedUsername(
-            enterprise_account_id_.GetUserEmail());
+        chromeos::CryptohomeClient::GetStubSanitizedUsername(cryptohome_id_);
     const base::FilePath user_key_file =
         user_keys_dir.AppendASCII(sanitized_username)
                      .AppendASCII("policy.pub");
@@ -705,7 +593,7 @@ class UserImageManagerPolicyTest : public UserImageManagerTest,
   policy::UserPolicyBuilder user_policy_;
   FakeSessionManagerClient* fake_session_manager_client_;
 
-  scoped_ptr<gfx::ImageSkia> policy_image_;
+  std::unique_ptr<gfx::ImageSkia> policy_image_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(UserImageManagerPolicyTest);
@@ -736,8 +624,8 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerPolicyTest, DISABLED_SetAndClear) {
   user_policy_.payload().mutable_useravatarimage()->set_value(
       ConstructPolicy(test::kUserAvatarImage2RelativePath));
   user_policy_.Build();
-  fake_session_manager_client_->set_user_policy(
-      enterprise_account_id_.GetUserEmail(), user_policy_.GetBlob());
+  fake_session_manager_client_->set_user_policy(cryptohome_id_,
+                                                user_policy_.GetBlob());
   run_loop_.reset(new base::RunLoop);
   store->Load();
   run_loop_->Run();
@@ -745,11 +633,11 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerPolicyTest, DISABLED_SetAndClear) {
   EXPECT_FALSE(user->HasDefaultImage());
   EXPECT_EQ(user_manager::User::USER_IMAGE_EXTERNAL, user->image_index());
   EXPECT_TRUE(test::AreImagesEqual(*policy_image_, user->GetImage()));
-  ExpectNewUserImageInfo(enterprise_account_id_,
-                         user_manager::User::USER_IMAGE_EXTERNAL,
-                         GetUserImagePath(enterprise_account_id_, "jpg"));
+  ExpectUserImageInfo(enterprise_account_id_,
+                      user_manager::User::USER_IMAGE_EXTERNAL,
+                      GetUserImagePath(enterprise_account_id_, "jpg"));
 
-  scoped_ptr<gfx::ImageSkia> saved_image =
+  std::unique_ptr<gfx::ImageSkia> saved_image =
       test::ImageLoader(GetUserImagePath(enterprise_account_id_, "jpg")).Load();
   ASSERT_TRUE(saved_image);
 
@@ -761,8 +649,8 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerPolicyTest, DISABLED_SetAndClear) {
   // image.
   user_policy_.payload().Clear();
   user_policy_.Build();
-  fake_session_manager_client_->set_user_policy(
-      enterprise_account_id_.GetUserEmail(), user_policy_.GetBlob());
+  fake_session_manager_client_->set_user_policy(cryptohome_id_,
+                                                user_policy_.GetBlob());
   run_loop_.reset(new base::RunLoop);
   store->AddObserver(this);
   store->Load();
@@ -779,8 +667,8 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerPolicyTest, DISABLED_SetAndClear) {
   const gfx::ImageSkia& default_image =
       default_user_image::GetDefaultImage(default_image_index);
   EXPECT_TRUE(test::AreImagesEqual(default_image, user->GetImage()));
-  ExpectNewUserImageInfo(enterprise_account_id_, default_image_index,
-                         base::FilePath());
+  ExpectUserImageInfo(enterprise_account_id_, default_image_index,
+                      base::FilePath());
 
   // Choose a different user image. Verify that the chosen user image is set and
   // persisted.
@@ -798,8 +686,8 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerPolicyTest, DISABLED_SetAndClear) {
   EXPECT_TRUE(user->HasDefaultImage());
   EXPECT_EQ(user_image_index, user->image_index());
   EXPECT_TRUE(test::AreImagesEqual(user_image, user->GetImage()));
-  ExpectNewUserImageInfo(enterprise_account_id_, user_image_index,
-                         base::FilePath());
+  ExpectUserImageInfo(enterprise_account_id_, user_image_index,
+                      base::FilePath());
 }
 
 IN_PROC_BROWSER_TEST_F(UserImageManagerPolicyTest, PRE_PolicyOverridesUser) {
@@ -834,17 +722,17 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerPolicyTest, PolicyOverridesUser) {
   EXPECT_TRUE(user->HasDefaultImage());
   EXPECT_EQ(default_user_image::kFirstDefaultImageIndex, user->image_index());
   EXPECT_TRUE(test::AreImagesEqual(default_image, user->GetImage()));
-  ExpectNewUserImageInfo(enterprise_account_id_,
-                         default_user_image::kFirstDefaultImageIndex,
-                         base::FilePath());
+  ExpectUserImageInfo(enterprise_account_id_,
+                      default_user_image::kFirstDefaultImageIndex,
+                      base::FilePath());
 
   // Set policy. Verify that the policy-provided user image is downloaded, set
   // and persisted, overriding the previously set image.
   user_policy_.payload().mutable_useravatarimage()->set_value(
       ConstructPolicy(test::kUserAvatarImage2RelativePath));
   user_policy_.Build();
-  fake_session_manager_client_->set_user_policy(
-      enterprise_account_id_.GetUserEmail(), user_policy_.GetBlob());
+  fake_session_manager_client_->set_user_policy(cryptohome_id_,
+                                                user_policy_.GetBlob());
   run_loop_.reset(new base::RunLoop);
   store->Load();
   run_loop_->Run();
@@ -852,11 +740,11 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerPolicyTest, PolicyOverridesUser) {
   EXPECT_FALSE(user->HasDefaultImage());
   EXPECT_EQ(user_manager::User::USER_IMAGE_EXTERNAL, user->image_index());
   EXPECT_TRUE(test::AreImagesEqual(*policy_image_, user->GetImage()));
-  ExpectNewUserImageInfo(enterprise_account_id_,
-                         user_manager::User::USER_IMAGE_EXTERNAL,
-                         GetUserImagePath(enterprise_account_id_, "jpg"));
+  ExpectUserImageInfo(enterprise_account_id_,
+                      user_manager::User::USER_IMAGE_EXTERNAL,
+                      GetUserImagePath(enterprise_account_id_, "jpg"));
 
-  scoped_ptr<gfx::ImageSkia> saved_image =
+  std::unique_ptr<gfx::ImageSkia> saved_image =
       test::ImageLoader(GetUserImagePath(enterprise_account_id_, "jpg")).Load();
   ASSERT_TRUE(saved_image);
 
@@ -890,8 +778,8 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerPolicyTest, UserDoesNotOverridePolicy) {
   user_policy_.payload().mutable_useravatarimage()->set_value(
       ConstructPolicy(test::kUserAvatarImage2RelativePath));
   user_policy_.Build();
-  fake_session_manager_client_->set_user_policy(
-      enterprise_account_id_.GetUserEmail(), user_policy_.GetBlob());
+  fake_session_manager_client_->set_user_policy(cryptohome_id_,
+                                                user_policy_.GetBlob());
   run_loop_.reset(new base::RunLoop);
   store->Load();
   run_loop_->Run();
@@ -899,11 +787,11 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerPolicyTest, UserDoesNotOverridePolicy) {
   EXPECT_FALSE(user->HasDefaultImage());
   EXPECT_EQ(user_manager::User::USER_IMAGE_EXTERNAL, user->image_index());
   EXPECT_TRUE(test::AreImagesEqual(*policy_image_, user->GetImage()));
-  ExpectNewUserImageInfo(enterprise_account_id_,
-                         user_manager::User::USER_IMAGE_EXTERNAL,
-                         GetUserImagePath(enterprise_account_id_, "jpg"));
+  ExpectUserImageInfo(enterprise_account_id_,
+                      user_manager::User::USER_IMAGE_EXTERNAL,
+                      GetUserImagePath(enterprise_account_id_, "jpg"));
 
-  scoped_ptr<gfx::ImageSkia> saved_image =
+  std::unique_ptr<gfx::ImageSkia> saved_image =
       test::ImageLoader(GetUserImagePath(enterprise_account_id_, "jpg")).Load();
   ASSERT_TRUE(saved_image);
 
@@ -921,9 +809,9 @@ IN_PROC_BROWSER_TEST_F(UserImageManagerPolicyTest, UserDoesNotOverridePolicy) {
   EXPECT_FALSE(user->HasDefaultImage());
   EXPECT_EQ(user_manager::User::USER_IMAGE_EXTERNAL, user->image_index());
   EXPECT_TRUE(test::AreImagesEqual(*policy_image_, user->GetImage()));
-  ExpectNewUserImageInfo(enterprise_account_id_,
-                         user_manager::User::USER_IMAGE_EXTERNAL,
-                         GetUserImagePath(enterprise_account_id_, "jpg"));
+  ExpectUserImageInfo(enterprise_account_id_,
+                      user_manager::User::USER_IMAGE_EXTERNAL,
+                      GetUserImagePath(enterprise_account_id_, "jpg"));
 
   saved_image =
       test::ImageLoader(GetUserImagePath(enterprise_account_id_, "jpg")).Load();
