@@ -5,27 +5,28 @@
 #include "chrome/browser/chromeos/login/users/chrome_user_manager_impl.h"
 
 #include <stddef.h>
+
 #include <cstddef>
 #include <set>
 #include <utility>
 
-#include "ash/multi_profile_uma.h"
+#include "ash/common/multi_profile_uma.h"
 #include "base/bind.h"
 #include "base/bind_helpers.h"
 #include "base/command_line.h"
 #include "base/compiler_specific.h"
 #include "base/format_macros.h"
+#include "base/location.h"
 #include "base/logging.h"
-#include "base/metrics/histogram.h"
-#include "base/prefs/pref_registry_simple.h"
-#include "base/prefs/pref_service.h"
-#include "base/prefs/scoped_user_pref_update.h"
+#include "base/memory/ptr_util.h"
+#include "base/metrics/histogram_macros.h"
+#include "base/single_thread_task_runner.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/sys_info.h"
 #include "base/task_runner.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/values.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/chrome_notification_types.h"
@@ -41,11 +42,11 @@
 #include "chrome/browser/chromeos/login/users/supervised_user_manager_impl.h"
 #include "chrome/browser/chromeos/login/users/wallpaper/wallpaper_manager.h"
 #include "chrome/browser/chromeos/policy/browser_policy_connector_chromeos.h"
-#include "chrome/browser/chromeos/policy/device_local_account.h"
 #include "chrome/browser/chromeos/profiles/multiprofiles_session_aborted_dialog.h"
 #include "chrome/browser/chromeos/profiles/profile_helper.h"
 #include "chrome/browser/chromeos/session_length_limiter.h"
 #include "chrome/browser/chromeos/settings/cros_settings.h"
+#include "chrome/browser/chromeos/system/timezone_resolver_manager.h"
 #include "chrome/browser/chromeos/system/timezone_util.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/signin/easy_unlock_service.h"
@@ -59,18 +60,23 @@
 #include "chromeos/chromeos_switches.h"
 #include "chromeos/cryptohome/async_method_caller.h"
 #include "chromeos/login/login_state.h"
-#include "chromeos/login/user_names.h"
 #include "chromeos/settings/cros_settings_names.h"
 #include "chromeos/timezone/timezone_resolver.h"
+#include "components/policy/policy_constants.h"
+#include "components/prefs/pref_registry_simple.h"
+#include "components/prefs/pref_service.h"
+#include "components/prefs/scoped_user_pref_update.h"
 #include "components/session_manager/core/session_manager.h"
 #include "components/signin/core/account_id/account_id.h"
 #include "components/user_manager/known_user.h"
 #include "components/user_manager/remove_user_delegate.h"
+#include "components/user_manager/user.h"
 #include "components/user_manager/user_image/user_image.h"
+#include "components/user_manager/user_names.h"
 #include "components/user_manager/user_type.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/notification_service.h"
-#include "policy/policy_constants.h"
+#include "extensions/common/features/feature_session_type.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 #include "ui/chromeos/resources/grit/ui_chromeos_resources.h"
@@ -87,16 +93,16 @@ namespace {
 // order.
 const char kRegularUsers[] = "LoggedInUsers";
 
-// A vector pref of the public accounts defined on this device.
-const char kPublicAccounts[] = "PublicAccounts";
+// A vector pref of the device local accounts defined on this device.
+const char kDeviceLocalAccounts[] = "PublicAccounts";
 
 // Key for list of users that should be reported.
 const char kReportingUsers[] = "reporting_users";
 
-// A string pref that gets set when a public account is removed but a user is
-// currently logged into that account, requiring the account's data to be
-// removed after logout.
-const char kPublicAccountPendingDataRemoval[] =
+// A string pref that gets set when a device local account is removed but a
+// user is currently logged into that account, requiring the account's data to
+// be removed after logout.
+const char kDeviceLocalAccountPendingDataRemoval[] =
     "PublicAccountPendingDataRemoval";
 
 bool FakeOwnership() {
@@ -125,14 +131,45 @@ void ResolveLocale(const std::string& raw_locale,
   ignore_result(l10n_util::CheckAndResolveLocale(raw_locale, resolved_locale));
 }
 
+bool GetUserLockAttributes(const user_manager::User* user,
+                           bool* can_lock,
+                           std::string* multi_profile_behavior) {
+  Profile* const profile = ProfileHelper::Get()->GetProfileByUser(user);
+  if (!profile)
+    return false;
+  PrefService* const prefs = profile->GetPrefs();
+  if (can_lock)
+    *can_lock = user->can_lock() && prefs->GetBoolean(prefs::kAllowScreenLock);
+  if (multi_profile_behavior) {
+    *multi_profile_behavior =
+        prefs->GetString(prefs::kMultiProfileUserBehavior);
+  }
+  return true;
+}
+
+extensions::FeatureSessionType GetFeatureSessionTypeForUser(
+    const user_manager::User* user) {
+  switch (user->GetType()) {
+    case user_manager::USER_TYPE_REGULAR:
+    case user_manager::USER_TYPE_CHILD:
+      return extensions::FeatureSessionType::REGULAR;
+    case user_manager::USER_TYPE_KIOSK_APP:
+      return extensions::FeatureSessionType::KIOSK;
+    default:
+      // The rest of user types is not used by extensions features.
+      return extensions::FeatureSessionType::UNKNOWN;
+  }
+}
+
 }  // namespace
 
 // static
 void ChromeUserManagerImpl::RegisterPrefs(PrefRegistrySimple* registry) {
   ChromeUserManager::RegisterPrefs(registry);
 
-  registry->RegisterListPref(kPublicAccounts);
-  registry->RegisterStringPref(kPublicAccountPendingDataRemoval, std::string());
+  registry->RegisterListPref(kDeviceLocalAccounts);
+  registry->RegisterStringPref(kDeviceLocalAccountPendingDataRemoval,
+                               std::string());
   registry->RegisterListPref(kReportingUsers);
 
   SupervisedUserManager::RegisterPrefs(registry);
@@ -141,8 +178,9 @@ void ChromeUserManagerImpl::RegisterPrefs(PrefRegistrySimple* registry) {
 }
 
 // static
-scoped_ptr<ChromeUserManager> ChromeUserManagerImpl::CreateChromeUserManager() {
-  return scoped_ptr<ChromeUserManager>(new ChromeUserManagerImpl());
+std::unique_ptr<ChromeUserManager>
+ChromeUserManagerImpl::CreateChromeUserManager() {
+  return std::unique_ptr<ChromeUserManager>(new ChromeUserManagerImpl());
 }
 
 ChromeUserManagerImpl::ChromeUserManagerImpl()
@@ -167,8 +205,8 @@ ChromeUserManagerImpl::ChromeUserManagerImpl()
                  content::NotificationService::AllSources());
 
   // Since we're in ctor postpone any actions till this is fully created.
-  if (base::MessageLoop::current()) {
-    base::MessageLoop::current()->PostTask(
+  if (base::ThreadTaskRunnerHandle::IsSet()) {
+    base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
         base::Bind(&ChromeUserManagerImpl::RetrieveTrustedDevicePolicies,
                    weak_factory_.GetWeakPtr()));
@@ -265,8 +303,8 @@ user_manager::UserList ChromeUserManagerImpl::GetUsersAllowedForMultiProfile()
     if ((*it)->GetType() == user_manager::USER_TYPE_REGULAR &&
         !(*it)->is_logged_in()) {
       MultiProfileUserController::UserAllowedInSessionReason check;
-      multi_profile_user_controller_->IsUserAllowedInSession((*it)->email(),
-                                                             &check);
+      multi_profile_user_controller_->IsUserAllowedInSession(
+          (*it)->GetAccountId().GetUserEmail(), &check);
       if (check ==
           MultiProfileUserController::NOT_ALLOWED_PRIMARY_USER_POLICY_FORBIDS) {
         return user_manager::UserList();
@@ -301,29 +339,31 @@ user_manager::UserList ChromeUserManagerImpl::GetUnlockUsers() const {
   if (logged_in_users.empty())
     return user_manager::UserList();
 
+  bool can_primary_lock = false;
+  std::string primary_behavior;
+  if (!GetUserLockAttributes(GetPrimaryUser(), &can_primary_lock,
+                             &primary_behavior)) {
+    // Locking is not allowed until the primary user profile is created.
+    return user_manager::UserList();
+  }
+
   user_manager::UserList unlock_users;
-  Profile* profile =
-      ProfileHelper::Get()->GetProfileByUserUnsafe(GetPrimaryUser());
-  std::string primary_behavior =
-      profile->GetPrefs()->GetString(prefs::kMultiProfileUserBehavior);
 
   // Specific case: only one logged in user or
   // primary user has primary-only multi-profile policy.
   if (logged_in_users.size() == 1 ||
       primary_behavior == MultiProfileUserController::kBehaviorPrimaryOnly) {
-    if (GetPrimaryUser()->can_lock())
+    if (can_primary_lock)
       unlock_users.push_back(primary_user_);
   } else {
     // Fill list of potential unlock users based on multi-profile policy state.
-    for (user_manager::UserList::const_iterator it = logged_in_users.begin();
-         it != logged_in_users.end();
-         ++it) {
-      user_manager::User* user = (*it);
-      Profile* profile = ProfileHelper::Get()->GetProfileByUserUnsafe(user);
-      const std::string behavior =
-          profile->GetPrefs()->GetString(prefs::kMultiProfileUserBehavior);
+    for (user_manager::User* user : logged_in_users) {
+      bool can_lock = false;
+      std::string behavior;
+      if (!GetUserLockAttributes(user, &can_lock, &behavior))
+        continue;
       if (behavior == MultiProfileUserController::kBehaviorUnrestricted &&
-          user->can_lock()) {
+          can_lock) {
         unlock_users.push_back(user);
       } else if (behavior == MultiProfileUserController::kBehaviorPrimaryOnly) {
         NOTREACHED()
@@ -335,16 +375,6 @@ user_manager::UserList ChromeUserManagerImpl::GetUnlockUsers() const {
   return unlock_users;
 }
 
-void ChromeUserManagerImpl::SessionStarted() {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  ChromeUserManager::SessionStarted();
-
-  content::NotificationService::current()->Notify(
-      chrome::NOTIFICATION_SESSION_STARTED,
-      content::Source<UserManager>(this),
-      content::Details<const user_manager::User>(GetActiveUser()));
-}
-
 void ChromeUserManagerImpl::RemoveUserInternal(
     const AccountId& account_id,
     user_manager::RemoveUserDelegate* delegate) {
@@ -418,7 +448,8 @@ void ChromeUserManagerImpl::Observe(
       break;
     case chrome::NOTIFICATION_LOGIN_USER_PROFILE_PREPARED: {
       Profile* profile = content::Details<Profile>(details).ptr();
-      if (IsUserLoggedIn() && !IsLoggedInAsGuest() && !IsLoggedInAsKioskApp()) {
+      if (IsUserLoggedIn() && !IsLoggedInAsGuest() && !IsLoggedInAsKioskApp() &&
+          !IsLoggedInAsArcKioskApp()) {
         if (IsLoggedInAsSupervisedUser())
           SupervisedUserPasswordServiceFactory::GetForProfile(profile);
         if (IsLoggedInAsUserWithGaiaAccount())
@@ -451,11 +482,10 @@ void ChromeUserManagerImpl::Observe(
         // ProfileManager::GetProfile before the profile gets registered
         // in ProfileManager. It happens in case of sync profile load when
         // NOTIFICATION_PROFILE_CREATED is called synchronously.
-        base::MessageLoop::current()->PostTask(
+        base::ThreadTaskRunnerHandle::Get()->PostTask(
             FROM_HERE,
             base::Bind(&ChromeUserManagerImpl::SwitchActiveUser,
-                       weak_factory_.GetWeakPtr(),
-                       GetPendingUserSwitchID()));
+                       weak_factory_.GetWeakPtr(), GetPendingUserSwitchID()));
         SetPendingUserSwitchId(EmptyAccountId());
       }
       break;
@@ -492,7 +522,7 @@ void ChromeUserManagerImpl::OnExternalDataCleared(const std::string& policy,
 void ChromeUserManagerImpl::OnExternalDataFetched(
     const std::string& policy,
     const std::string& user_id,
-    scoped_ptr<std::string> data) {
+    std::unique_ptr<std::string> data) {
   const AccountId account_id =
       user_manager::known_user::GetAccountId(user_id, std::string());
   if (policy == policy::key::kUserAvatarImage)
@@ -520,17 +550,24 @@ void ChromeUserManagerImpl::OnDeviceLocalAccountsChanged() {
 }
 
 bool ChromeUserManagerImpl::CanCurrentUserLock() const {
-  return ChromeUserManager::CanCurrentUserLock() &&
-         GetCurrentUserFlow()->CanLockScreen();
+  if (!ChromeUserManager::CanCurrentUserLock() ||
+      !GetCurrentUserFlow()->CanLockScreen()) {
+    return false;
+  }
+  bool can_lock = false;
+  if (!GetUserLockAttributes(active_user_, &can_lock, nullptr))
+    return false;
+  return can_lock;
 }
 
 bool ChromeUserManagerImpl::IsUserNonCryptohomeDataEphemeral(
     const AccountId& account_id) const {
-  // Data belonging to the obsolete public accounts whose data has not been
-  // removed yet is not ephemeral.
-  bool is_obsolete_public_account = IsPublicAccountMarkedForRemoval(account_id);
+  // Data belonging to the obsolete device local accounts whose data has not
+  // been removed yet is not ephemeral.
+  const bool is_obsolete_device_local_account =
+      IsDeviceLocalAccountMarkedForRemoval(account_id);
 
-  return !is_obsolete_public_account &&
+  return !is_obsolete_device_local_account &&
          ChromeUserManager::IsUserNonCryptohomeDataEphemeral(account_id);
 }
 
@@ -565,16 +602,24 @@ bool ChromeUserManagerImpl::IsEnterpriseManaged() const {
   return connector->IsEnterpriseManaged();
 }
 
-void ChromeUserManagerImpl::LoadPublicAccounts(
-    std::set<AccountId>* public_sessions_set) {
-  const base::ListValue* prefs_public_sessions =
-      GetLocalState()->GetList(kPublicAccounts);
-  std::vector<AccountId> public_sessions;
-  ParseUserList(*prefs_public_sessions, std::set<AccountId>(), &public_sessions,
-                public_sessions_set);
-  for (const AccountId& account_id : public_sessions) {
-    users_.push_back(user_manager::User::CreatePublicAccountUser(account_id));
-    UpdatePublicAccountDisplayName(account_id.GetUserEmail());
+void ChromeUserManagerImpl::LoadDeviceLocalAccounts(
+    std::set<AccountId>* device_local_accounts_set) {
+  const base::ListValue* prefs_device_local_accounts =
+      GetLocalState()->GetList(kDeviceLocalAccounts);
+  std::vector<AccountId> device_local_accounts;
+  ParseUserList(*prefs_device_local_accounts, std::set<AccountId>(),
+                &device_local_accounts, device_local_accounts_set);
+  for (const AccountId& account_id : device_local_accounts) {
+    policy::DeviceLocalAccount::Type type;
+    if (!policy::IsDeviceLocalAccountUser(account_id.GetUserEmail(), &type)) {
+      NOTREACHED();
+      continue;
+    }
+
+    users_.push_back(
+        CreateUserFromDeviceLocalAccount(account_id, type).release());
+    if (type == policy::DeviceLocalAccount::TYPE_PUBLIC_SESSION)
+      UpdatePublicAccountDisplayName(account_id.GetUserEmail());
   }
 }
 
@@ -606,21 +651,13 @@ void ChromeUserManagerImpl::PerformPostUserLoggedInActions(
 }
 
 bool ChromeUserManagerImpl::IsDemoApp(const AccountId& account_id) const {
-  return DemoAppLauncher::IsDemoAppSession(account_id.GetUserEmail());
+  return DemoAppLauncher::IsDemoAppSession(account_id);
 }
 
-bool ChromeUserManagerImpl::IsKioskApp(const AccountId& account_id) const {
-  policy::DeviceLocalAccount::Type device_local_account_type;
-  return policy::IsDeviceLocalAccountUser(account_id.GetUserEmail(),
-                                          &device_local_account_type) &&
-         device_local_account_type ==
-             policy::DeviceLocalAccount::TYPE_KIOSK_APP;
-}
-
-bool ChromeUserManagerImpl::IsPublicAccountMarkedForRemoval(
+bool ChromeUserManagerImpl::IsDeviceLocalAccountMarkedForRemoval(
     const AccountId& account_id) const {
   return account_id == AccountId::FromUserEmail(GetLocalState()->GetString(
-                           kPublicAccountPendingDataRemoval));
+                           kDeviceLocalAccountPendingDataRemoval));
 }
 
 void ChromeUserManagerImpl::RetrieveTrustedDevicePolicies() {
@@ -650,7 +687,7 @@ void ChromeUserManagerImpl::RetrieveTrustedDevicePolicies() {
 
   EnsureUsersLoaded();
 
-  bool changed = UpdateAndCleanUpPublicAccounts(
+  bool changed = UpdateAndCleanUpDeviceLocalAccounts(
       policy::GetDeviceLocalAccounts(cros_settings_));
 
   // If ephemeral users are enabled and we are on the login screen, take this
@@ -668,8 +705,7 @@ void ChromeUserManagerImpl::RetrieveTrustedDevicePolicies() {
         changed = true;
       } else {
         if ((*it)->GetType() != user_manager::USER_TYPE_PUBLIC_ACCOUNT)
-          prefs_users_update->Append(
-              new base::StringValue(account_id.GetUserEmail()));
+          prefs_users_update->AppendString(account_id.GetUserEmail());
         ++it;
       }
     }
@@ -687,14 +723,13 @@ void ChromeUserManagerImpl::GuestUserLoggedIn() {
   // mount point. Legacy (--login-profile) value will be used for now.
   // http://crosbug.com/230859
   active_user_->SetStubImage(
-      user_manager::UserImage(
+      base::MakeUnique<user_manager::UserImage>(
           *ResourceBundle::GetSharedInstance().GetImageSkiaNamed(
               IDR_PROFILE_PICTURE_LOADING)),
-      user_manager::User::USER_IMAGE_INVALID,
-      false);
+      user_manager::User::USER_IMAGE_INVALID, false);
 
   // Initializes wallpaper after active_user_ is set.
-  WallpaperManager::Get()->SetUserWallpaperNow(login::GuestAccountId());
+  WallpaperManager::Get()->SetUserWallpaperNow(user_manager::GuestAccountId());
 }
 
 void ChromeUserManagerImpl::RegularUserLoggedIn(const AccountId& account_id) {
@@ -752,8 +787,8 @@ void ChromeUserManagerImpl::SupervisedUserLoggedIn(
 
   // Add the user to the front of the user list.
   ListPrefUpdate prefs_users_update(GetLocalState(), kRegularUsers);
-  prefs_users_update->Insert(0,
-                             new base::StringValue(account_id.GetUserEmail()));
+  prefs_users_update->Insert(
+      0, base::MakeUnique<base::StringValue>(account_id.GetUserEmail()));
   users_.insert(users_.begin(), active_user_);
 
   // Now that user is in the list, save display name.
@@ -786,23 +821,17 @@ void ChromeUserManagerImpl::PublicAccountUserLoggedIn(
   WallpaperManager::Get()->EnsureLoggedInUserWallpaperLoaded();
 }
 
-void ChromeUserManagerImpl::KioskAppLoggedIn(
-    const AccountId& kiosk_app_account_id) {
+void ChromeUserManagerImpl::KioskAppLoggedIn(user_manager::User* user) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  policy::DeviceLocalAccount::Type device_local_account_type;
-  DCHECK(policy::IsDeviceLocalAccountUser(kiosk_app_account_id.GetUserEmail(),
-                                          &device_local_account_type));
-  DCHECK_EQ(policy::DeviceLocalAccount::TYPE_KIOSK_APP,
-            device_local_account_type);
 
-  active_user_ = user_manager::User::CreateKioskAppUser(kiosk_app_account_id);
+  active_user_ = user;
   active_user_->SetStubImage(
-      user_manager::UserImage(
+      base::MakeUnique<user_manager::UserImage>(
           *ResourceBundle::GetSharedInstance().GetImageSkiaNamed(
               IDR_PROFILE_PICTURE_LOADING)),
-      user_manager::User::USER_IMAGE_INVALID,
-      false);
+      user_manager::User::USER_IMAGE_INVALID, false);
 
+  const AccountId& kiosk_app_account_id = user->GetAccountId();
   WallpaperManager::Get()->SetUserWallpaperNow(kiosk_app_account_id);
 
   // TODO(bartfab): Add KioskAppUsers to the users_ list and keep metadata like
@@ -820,9 +849,9 @@ void ChromeUserManagerImpl::KioskAppLoggedIn(
       break;
     }
   }
-  std::string kiosk_app_name;
+  std::string kiosk_app_id;
   if (account) {
-    kiosk_app_name = account->kiosk_app_id;
+    kiosk_app_id = account->kiosk_app_id;
   } else {
     LOG(ERROR) << "Logged into nonexistent kiosk-app account: "
                << kiosk_app_account_id.GetUserEmail();
@@ -831,7 +860,27 @@ void ChromeUserManagerImpl::KioskAppLoggedIn(
 
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
   command_line->AppendSwitch(::switches::kForceAppMode);
-  command_line->AppendSwitchASCII(::switches::kAppId, kiosk_app_name);
+  command_line->AppendSwitchASCII(::switches::kAppId, kiosk_app_id);
+
+  // Disable window animation since kiosk app runs in a single full screen
+  // window and window animation causes start-up janks.
+  command_line->AppendSwitch(wm::switches::kWindowAnimationsDisabled);
+}
+
+void ChromeUserManagerImpl::ArcKioskAppLoggedIn(user_manager::User* user) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+
+  active_user_ = user;
+  active_user_->SetStubImage(
+      base::MakeUnique<user_manager::UserImage>(
+          *ResourceBundle::GetSharedInstance().GetImageSkiaNamed(
+              IDR_PROFILE_PICTURE_LOADING)),
+      user_manager::User::USER_IMAGE_INVALID, false);
+
+  base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
+  command_line->AppendSwitch(chromeos::switches::kEnableArc);
+  command_line->AppendSwitch(::switches::kForceAndroidAppMode);
+  command_line->AppendSwitch(::switches::kSilentLaunch);
 
   // Disable window animation since kiosk app runs in a single full screen
   // window and window animation causes start-up janks.
@@ -840,14 +889,14 @@ void ChromeUserManagerImpl::KioskAppLoggedIn(
 
 void ChromeUserManagerImpl::DemoAccountLoggedIn() {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  active_user_ = user_manager::User::CreateKioskAppUser(login::DemoAccountId());
+  active_user_ =
+      user_manager::User::CreateKioskAppUser(user_manager::DemoAccountId());
   active_user_->SetStubImage(
-      user_manager::UserImage(
+      base::MakeUnique<user_manager::UserImage>(
           *ResourceBundle::GetSharedInstance().GetImageSkiaNamed(
               IDR_PROFILE_PICTURE_LOADING)),
-      user_manager::User::USER_IMAGE_INVALID,
-      false);
-  WallpaperManager::Get()->SetUserWallpaperNow(login::DemoAccountId());
+      user_manager::User::USER_IMAGE_INVALID, false);
+  WallpaperManager::Get()->SetUserWallpaperNow(user_manager::DemoAccountId());
 
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
   command_line->AppendSwitch(::switches::kForceAppMode);
@@ -900,83 +949,72 @@ void ChromeUserManagerImpl::RemoveNonCryptohomeData(
   EasyUnlockService::ResetLocalStateForUser(account_id);
 }
 
-void
-ChromeUserManagerImpl::CleanUpPublicAccountNonCryptohomeDataPendingRemoval() {
+void ChromeUserManagerImpl::
+    CleanUpDeviceLocalAccountNonCryptohomeDataPendingRemoval() {
   PrefService* local_state = GetLocalState();
-  const std::string public_account_pending_data_removal =
-      local_state->GetString(kPublicAccountPendingDataRemoval);
-  if (public_account_pending_data_removal.empty() ||
+  const std::string device_local_account_pending_data_removal =
+      local_state->GetString(kDeviceLocalAccountPendingDataRemoval);
+  if (device_local_account_pending_data_removal.empty() ||
       (IsUserLoggedIn() &&
-       public_account_pending_data_removal == GetActiveUser()->email())) {
+       device_local_account_pending_data_removal ==
+           GetActiveUser()->GetAccountId().GetUserEmail())) {
     return;
   }
 
   RemoveNonCryptohomeData(
-      AccountId::FromUserEmail(public_account_pending_data_removal));
-  local_state->ClearPref(kPublicAccountPendingDataRemoval);
+      AccountId::FromUserEmail(device_local_account_pending_data_removal));
+  local_state->ClearPref(kDeviceLocalAccountPendingDataRemoval);
 }
 
-void ChromeUserManagerImpl::CleanUpPublicAccountNonCryptohomeData(
-    const std::vector<std::string>& old_public_accounts) {
+void ChromeUserManagerImpl::CleanUpDeviceLocalAccountNonCryptohomeData(
+    const std::vector<std::string>& old_device_local_accounts) {
   std::set<std::string> users;
   for (user_manager::UserList::const_iterator it = users_.begin();
        it != users_.end();
        ++it)
-    users.insert((*it)->email());
+    users.insert((*it)->GetAccountId().GetUserEmail());
 
-  // If the user is logged into a public account that has been removed from the
-  // user list, mark the account's data as pending removal after logout.
-  if (IsLoggedInAsPublicAccount()) {
-    const std::string active_user_id = GetActiveUser()->email();
+  // If the user is logged into a device local account that has been removed
+  // from the user list, mark the account's data as pending removal after
+  // logout.
+  const user_manager::User* const active_user = GetActiveUser();
+  if (active_user && active_user->IsDeviceLocalAccount()) {
+    const std::string active_user_id =
+        active_user->GetAccountId().GetUserEmail();
     if (users.find(active_user_id) == users.end()) {
-      GetLocalState()->SetString(kPublicAccountPendingDataRemoval,
+      GetLocalState()->SetString(kDeviceLocalAccountPendingDataRemoval,
                                  active_user_id);
       users.insert(active_user_id);
     }
   }
 
-  // Remove the data belonging to any other public accounts that are no longer
-  // found on the user list.
+  // Remove the data belonging to any other device local accounts that are no
+  // longer found on the user list.
   for (std::vector<std::string>::const_iterator it =
-           old_public_accounts.begin();
-       it != old_public_accounts.end();
-       ++it) {
+           old_device_local_accounts.begin();
+       it != old_device_local_accounts.end(); ++it) {
     if (users.find(*it) == users.end())
       RemoveNonCryptohomeData(AccountId::FromUserEmail(*it));
   }
 }
 
-bool ChromeUserManagerImpl::UpdateAndCleanUpPublicAccounts(
+bool ChromeUserManagerImpl::UpdateAndCleanUpDeviceLocalAccounts(
     const std::vector<policy::DeviceLocalAccount>& device_local_accounts) {
-  // Try to remove any public account data marked as pending removal.
-  CleanUpPublicAccountNonCryptohomeDataPendingRemoval();
+  // Try to remove any device local account data marked as pending removal.
+  CleanUpDeviceLocalAccountNonCryptohomeDataPendingRemoval();
 
-  // Get the current list of public accounts.
-  std::vector<std::string> old_public_accounts;
-  for (user_manager::UserList::const_iterator it = users_.begin();
-       it != users_.end();
-       ++it) {
-    if ((*it)->GetType() == user_manager::USER_TYPE_PUBLIC_ACCOUNT)
-      old_public_accounts.push_back((*it)->email());
+  // Get the current list of device local accounts.
+  std::vector<std::string> old_accounts;
+  for (auto* user : users_) {
+    if (user->IsDeviceLocalAccount())
+      old_accounts.push_back(user->GetAccountId().GetUserEmail());
   }
 
-  // Get the new list of public accounts from policy.
-  std::vector<std::string> new_public_accounts;
-  for (std::vector<policy::DeviceLocalAccount>::const_iterator it =
-           device_local_accounts.begin();
-       it != device_local_accounts.end();
-       ++it) {
-    // TODO(mnissler, nkostylev, bartfab): Process Kiosk Apps within the
-    // standard login framework: http://crbug.com/234694
-    if (it->type == policy::DeviceLocalAccount::TYPE_PUBLIC_SESSION)
-      new_public_accounts.push_back(it->user_id);
-  }
-
-  // If the list of public accounts has not changed, return.
-  if (new_public_accounts.size() == old_public_accounts.size()) {
+  // If the list of device local accounts has not changed, return.
+  if (device_local_accounts.size() == old_accounts.size()) {
     bool changed = false;
-    for (size_t i = 0; i < new_public_accounts.size(); ++i) {
-      if (new_public_accounts[i] != old_public_accounts[i]) {
+    for (size_t i = 0; i < device_local_accounts.size(); ++i) {
+      if (device_local_accounts[i].user_id != old_accounts[i]) {
         changed = true;
         break;
       }
@@ -985,21 +1023,18 @@ bool ChromeUserManagerImpl::UpdateAndCleanUpPublicAccounts(
       return false;
   }
 
-  // Persist the new list of public accounts in a pref.
-  ListPrefUpdate prefs_public_accounts_update(GetLocalState(), kPublicAccounts);
-  prefs_public_accounts_update->Clear();
-  for (std::vector<std::string>::const_iterator it =
-           new_public_accounts.begin();
-       it != new_public_accounts.end();
-       ++it) {
-    prefs_public_accounts_update->AppendString(*it);
-  }
+  // Persist the new list of device local accounts in a pref.
+  ListPrefUpdate prefs_device_local_accounts_update(GetLocalState(),
+                                                    kDeviceLocalAccounts);
+  prefs_device_local_accounts_update->Clear();
+  for (const auto& account : device_local_accounts)
+    prefs_device_local_accounts_update->AppendString(account.user_id);
 
-  // Remove the old public accounts from the user list.
+  // Remove the old device local accounts from the user list.
   for (user_manager::UserList::iterator it = users_.begin();
        it != users_.end();) {
-    if ((*it)->GetType() == user_manager::USER_TYPE_PUBLIC_ACCOUNT) {
-      if (*it != GetLoggedInUser())
+    if ((*it)->IsDeviceLocalAccount()) {
+      if (*it != GetActiveUser())
         DeleteUser(*it);
       it = users_.erase(it);
     } else {
@@ -1007,30 +1042,36 @@ bool ChromeUserManagerImpl::UpdateAndCleanUpPublicAccounts(
     }
   }
 
-  // Add the new public accounts to the front of the user list.
-  for (std::vector<std::string>::const_reverse_iterator it =
-           new_public_accounts.rbegin();
-       it != new_public_accounts.rend();
-       ++it) {
-    if (IsLoggedInAsPublicAccount() && *it == GetActiveUser()->email())
-      users_.insert(users_.begin(), GetLoggedInUser());
-    else
-      users_.insert(users_.begin(), user_manager::User::CreatePublicAccountUser(
-                                        AccountId::FromUserEmail(*it)));
-    UpdatePublicAccountDisplayName(*it);
+  // Add the new device local accounts to the front of the user list.
+  user_manager::User* const active_user = GetActiveUser();
+  const bool is_device_local_account_session =
+      active_user && active_user->IsDeviceLocalAccount();
+  for (auto it = device_local_accounts.rbegin();
+       it != device_local_accounts.rend(); ++it) {
+    if (is_device_local_account_session &&
+        AccountId::FromUserEmail(it->user_id) == active_user->GetAccountId()) {
+      users_.insert(users_.begin(), active_user);
+    } else {
+      users_.insert(users_.begin(),
+                    CreateUserFromDeviceLocalAccount(
+                        AccountId::FromUserEmail(it->user_id), it->type)
+                        .release());
+    }
+    if (it->type == policy::DeviceLocalAccount::TYPE_PUBLIC_SESSION) {
+      UpdatePublicAccountDisplayName(it->user_id);
+    }
   }
 
   for (user_manager::UserList::iterator
            ui = users_.begin(),
-           ue = users_.begin() + new_public_accounts.size();
-       ui != ue;
-       ++ui) {
+           ue = users_.begin() + device_local_accounts.size();
+       ui != ue; ++ui) {
     GetUserImageManager((*ui)->GetAccountId())->LoadUserImage();
   }
 
-  // Remove data belonging to public accounts that are no longer found on the
-  // user list.
-  CleanUpPublicAccountNonCryptohomeData(old_public_accounts);
+  // Remove data belonging to device local accounts that are no longer found on
+  // the user list.
+  CleanUpDeviceLocalAccountNonCryptohomeData(old_accounts);
 
   return true;
 }
@@ -1055,7 +1096,7 @@ UserFlow* ChromeUserManagerImpl::GetCurrentUserFlow() const {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   if (!IsUserLoggedIn())
     return GetDefaultUserFlow();
-  return GetUserFlow(GetLoggedInUser()->GetAccountId());
+  return GetUserFlow(GetActiveUser()->GetAccountId());
 }
 
 UserFlow* ChromeUserManagerImpl::GetUserFlow(
@@ -1146,11 +1187,6 @@ void ChromeUserManagerImpl::UpdateNumberOfUsers() {
 }
 
 void ChromeUserManagerImpl::UpdateUserTimeZoneRefresher(Profile* profile) {
-  if (base::CommandLine::ForCurrentProcess()->HasSwitch(
-          chromeos::switches::kDisableTimeZoneTrackingOption)) {
-    return;
-  }
-
   const user_manager::User* user =
       ProfileHelper::Get()->GetUserByProfile(profile);
   if (user == NULL)
@@ -1170,13 +1206,9 @@ void ChromeUserManagerImpl::UpdateUserTimeZoneRefresher(Profile* profile) {
     g_browser_process->platform_part()->GetTimezoneResolver()->Stop();
     return;
   }
-
-  if (profile->GetPrefs()->GetBoolean(prefs::kResolveTimezoneByGeolocation) &&
-      !system::HasSystemTimezonePolicy()) {
-    g_browser_process->platform_part()->GetTimezoneResolver()->Start();
-  } else {
-    g_browser_process->platform_part()->GetTimezoneResolver()->Stop();
-  }
+  g_browser_process->platform_part()
+      ->GetTimezoneResolverManager()
+      ->UpdateTimezoneResolver();
 }
 
 void ChromeUserManagerImpl::SetUserAffiliation(
@@ -1191,7 +1223,7 @@ void ChromeUserManagerImpl::SetUserAffiliation(
         g_browser_process->platform_part()->browser_policy_connector_chromeos();
     const bool is_affiliated = chromeos::IsUserAffiliated(
         user_affiliation_ids, connector->GetDeviceAffiliationIDs(),
-        account_id.GetUserEmail(), connector->GetEnterpriseDomain());
+        account_id.GetUserEmail());
     user->SetAffiliation(is_affiliated);
 
     if (user->GetType() == user_manager::USER_TYPE_REGULAR) {
@@ -1214,7 +1246,7 @@ bool ChromeUserManagerImpl::ShouldReportUser(const std::string& user_id) const {
 void ChromeUserManagerImpl::AddReportingUser(const AccountId& account_id) {
   ListPrefUpdate users_update(GetLocalState(), kReportingUsers);
   users_update->AppendIfNotPresent(
-      new base::StringValue(account_id.GetUserEmail()));
+      base::MakeUnique<base::StringValue>(account_id.GetUserEmail()));
 }
 
 void ChromeUserManagerImpl::RemoveReportingUser(const AccountId& account_id) {
@@ -1229,6 +1261,11 @@ void ChromeUserManagerImpl::UpdateLoginState(
     bool is_current_user_owner) const {
   chrome_user_manager_util::UpdateLoginState(active_user, primary_user,
                                              is_current_user_owner);
+  // If a user is logged in, update session type as seen by extensions system.
+  if (active_user) {
+    extensions::SetCurrentFeatureSessionType(
+        GetFeatureSessionTypeForUser(active_user));
+  }
 }
 
 bool ChromeUserManagerImpl::GetPlatformKnownUserId(
@@ -1240,7 +1277,7 @@ bool ChromeUserManagerImpl::GetPlatformKnownUserId(
 }
 
 const AccountId& ChromeUserManagerImpl::GetGuestAccountId() const {
-  return login::GuestAccountId();
+  return user_manager::GuestAccountId();
 }
 
 bool ChromeUserManagerImpl::IsFirstExecAfterBoot() const {
@@ -1251,22 +1288,23 @@ bool ChromeUserManagerImpl::IsFirstExecAfterBoot() const {
 void ChromeUserManagerImpl::AsyncRemoveCryptohome(
     const AccountId& account_id) const {
   cryptohome::AsyncMethodCaller::GetInstance()->AsyncRemove(
-      account_id.GetUserEmail(), base::Bind(&OnRemoveUserComplete, account_id));
+      cryptohome::Identification(account_id),
+      base::Bind(&OnRemoveUserComplete, account_id));
 }
 
 bool ChromeUserManagerImpl::IsGuestAccountId(
     const AccountId& account_id) const {
-  return account_id == login::GuestAccountId();
+  return account_id == user_manager::GuestAccountId();
 }
 
 bool ChromeUserManagerImpl::IsStubAccountId(const AccountId& account_id) const {
-  return account_id == login::StubAccountId();
+  return account_id == user_manager::StubAccountId();
 }
 
 bool ChromeUserManagerImpl::IsSupervisedAccountId(
     const AccountId& account_id) const {
   return gaia::ExtractDomainName(account_id.GetUserEmail()) ==
-         chromeos::login::kSupervisedUserDomain;
+         user_manager::kSupervisedUserDomain;
 }
 
 bool ChromeUserManagerImpl::HasBrowserRestarted() const {
@@ -1300,4 +1338,27 @@ bool ChromeUserManagerImpl::IsValidDefaultUserImageId(int image_index) const {
          image_index < chromeos::default_user_image::kDefaultImagesCount;
 }
 
+std::unique_ptr<user_manager::User>
+ChromeUserManagerImpl::CreateUserFromDeviceLocalAccount(
+    const AccountId& account_id,
+    const policy::DeviceLocalAccount::Type type) const {
+  std::unique_ptr<user_manager::User> user;
+  switch (type) {
+    case policy::DeviceLocalAccount::TYPE_PUBLIC_SESSION:
+      user.reset(user_manager::User::CreatePublicAccountUser(account_id));
+      break;
+    case policy::DeviceLocalAccount::TYPE_KIOSK_APP:
+      user.reset(user_manager::User::CreateKioskAppUser(account_id));
+      break;
+    case policy::DeviceLocalAccount::TYPE_ARC_KIOSK_APP:
+      user.reset(user_manager::User::CreateArcKioskAppUser(account_id));
+      break;
+    default:
+      NOTREACHED();
+      break;
+  }
+
+  return user;
+}
+
 }  // namespace chromeos
