@@ -12,7 +12,7 @@
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "chrome/browser/chromeos/enrollment_dialog_view.h"
-#include "chrome/browser/chromeos/net/onc_utils.h"
+#include "chrome/browser/chromeos/net/shill_error.h"
 #include "chrome/browser/chromeos/options/passphrase_textfield.h"
 #include "chrome/browser/profiles/profile_manager.h"
 #include "chrome/grit/generated_resources.h"
@@ -20,17 +20,18 @@
 #include "chromeos/login/login_state.h"
 #include "chromeos/network/client_cert_util.h"
 #include "chromeos/network/network_configuration_handler.h"
+#include "chromeos/network/network_connect.h"
 #include "chromeos/network/network_event_log.h"
 #include "chromeos/network/network_handler.h"
 #include "chromeos/network/network_state.h"
 #include "chromeos/network/network_state_handler.h"
 #include "chromeos/network/network_ui_data.h"
+#include "chromeos/network/onc/onc_utils.h"
 #include "chromeos/network/shill_property_util.h"
 #include "components/onc/onc_constants.h"
 #include "third_party/cros_system_api/dbus/service_constants.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
-#include "ui/chromeos/network/network_connect.h"
 #include "ui/events/event.h"
 #include "ui/views/controls/button/checkbox.h"
 #include "ui/views/controls/button/image_button.h"
@@ -94,7 +95,7 @@ enum Phase2AuthComboboxIndex {
 
 void ShillError(const std::string& function,
                 const std::string& error_name,
-                scoped_ptr<base::DictionaryValue> error_data) {
+                std::unique_ptr<base::DictionaryValue> error_data) {
   NET_LOG_ERROR("Shill Error from WifiConfigView: " + error_name, function);
 }
 
@@ -592,8 +593,8 @@ void WifiConfigView::UpdateErrorLabel() {
   if (error_msg.empty() && !service_path_.empty()) {
     const NetworkState* network = GetNetworkState();
     if (network && network->connection_state() == shill::kStateFailure) {
-      error_msg = ui::NetworkConnect::Get()->GetShillErrorString(
-          network->last_error(), network->path());
+      error_msg = shill_error::GetShillErrorString(network->last_error(),
+                                                   network->guid());
     }
   }
   if (!error_msg.empty()) {
@@ -612,6 +613,7 @@ void WifiConfigView::ContentsChanged(views::Textfield* sender,
 bool WifiConfigView::HandleKeyEvent(views::Textfield* sender,
                                     const ui::KeyEvent& key_event) {
   if (sender == passphrase_textfield_ &&
+      key_event.type() == ui::ET_KEY_PRESSED &&
       key_event.key_code() == ui::VKEY_RETURN) {
     parent_->GetDialogClientView()->AcceptWindow();
   }
@@ -709,8 +711,8 @@ bool WifiConfigView::Login() {
         shill::kSecurityClassProperty, security_class);
 
     // Configure and connect to network.
-    ui::NetworkConnect::Get()->CreateConfigurationAndConnect(&properties,
-                                                             share_network);
+    NetworkConnect::Get()->CreateConfigurationAndConnect(&properties,
+                                                         share_network);
   } else {
     if (!network) {
       // Shill no longer knows about this network (edge case).
@@ -736,11 +738,10 @@ bool WifiConfigView::Login() {
       properties.SetStringWithoutPathExpansion(shill::kTypeProperty,
                                                shill::kTypeEthernetEap);
       share_network = false;
-      ui::NetworkConnect::Get()->CreateConfiguration(&properties,
-                                                     share_network);
+      NetworkConnect::Get()->CreateConfiguration(&properties, share_network);
     } else {
-      ui::NetworkConnect::Get()->ConfigureNetworkAndConnect(
-          service_path_, properties, share_network);
+      NetworkConnect::Get()->ConfigureNetworkIdAndConnect(
+          network->guid(), properties, share_network);
     }
   }
   return true;  // dialog will be closed
@@ -1109,7 +1110,8 @@ void WifiConfigView::Init(bool show_8021x) {
   } else {
     // Password visible button.
     passphrase_visible_button_ = new views::ToggleImageButton(this);
-    passphrase_visible_button_->SetFocusable(true);
+    passphrase_visible_button_->SetFocusForPlatform();
+    passphrase_visible_button_->set_request_focus_on_press(true);
     passphrase_visible_button_->SetTooltipText(
         l10n_util::GetStringUTF16(
             IDS_OPTIONS_SETTINGS_INTERNET_OPTIONS_PASSPHRASE_SHOW));
