@@ -12,32 +12,35 @@
 #include "base/files/file_path.h"
 #include "base/location.h"
 #include "base/logging.h"
-#include "base/message_loop/message_loop.h"
+#include "base/memory/ptr_util.h"
 #include "base/path_service.h"
-#include "base/prefs/pref_registry_simple.h"
 #include "base/sequenced_task_runner.h"
+#include "base/single_thread_task_runner.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/thread_task_runner_handle.h"
 #include "base/threading/sequenced_worker_pool.h"
+#include "base/threading/thread_task_runner_handle.h"
+#include "chrome/browser/chromeos/attestation/attestation_ca_client.h"
 #include "chrome/browser/chromeos/policy/affiliated_cloud_policy_invalidator.h"
 #include "chrome/browser/chromeos/policy/affiliated_invalidation_service_provider.h"
 #include "chrome/browser/chromeos/policy/affiliated_invalidation_service_provider_impl.h"
-#include "chrome/browser/chromeos/policy/consumer_management_service.h"
+#include "chrome/browser/chromeos/policy/bluetooth_policy_handler.h"
 #include "chrome/browser/chromeos/policy/device_cloud_policy_initializer.h"
 #include "chrome/browser/chromeos/policy/device_cloud_policy_store_chromeos.h"
 #include "chrome/browser/chromeos/policy/device_local_account.h"
 #include "chrome/browser/chromeos/policy/device_local_account_policy_service.h"
 #include "chrome/browser/chromeos/policy/device_network_configuration_updater.h"
 #include "chrome/browser/chromeos/policy/enrollment_config.h"
-#include "chrome/browser/chromeos/policy/enterprise_install_attributes.h"
 #include "chrome/browser/chromeos/policy/remote_commands/affiliated_remote_commands_invalidator.h"
 #include "chrome/browser/chromeos/policy/server_backed_state_keys_broker.h"
 #include "chrome/browser/chromeos/settings/cros_settings.h"
 #include "chrome/browser/chromeos/settings/device_settings_service.h"
+#include "chrome/browser/chromeos/settings/install_attributes.h"
 #include "chrome/browser/policy/device_management_service_configuration.h"
 #include "chrome/common/pref_names.h"
+#include "chromeos/attestation/attestation_flow.h"
 #include "chromeos/chromeos_paths.h"
 #include "chromeos/chromeos_switches.h"
+#include "chromeos/cryptohome/async_method_caller.h"
 #include "chromeos/cryptohome/system_salt_getter.h"
 #include "chromeos/dbus/cryptohome_client.h"
 #include "chromeos/dbus/dbus_thread_manager.h"
@@ -46,13 +49,15 @@
 #include "chromeos/settings/cros_settings_names.h"
 #include "chromeos/settings/cros_settings_provider.h"
 #include "chromeos/settings/timezone_settings.h"
+#include "chromeos/system/statistics_provider.h"
 #include "components/policy/core/common/cloud/cloud_policy_client.h"
 #include "components/policy/core/common/cloud/cloud_policy_refresh_scheduler.h"
 #include "components/policy/core/common/proxy_policy_provider.h"
+#include "components/policy/proto/device_management_backend.pb.h"
+#include "components/prefs/pref_registry_simple.h"
 #include "content/public/browser/browser_thread.h"
 #include "google_apis/gaia/gaia_auth_util.h"
 #include "net/url_request/url_request_context_getter.h"
-#include "policy/proto/device_management_backend.pb.h"
 
 using content::BrowserThread;
 
@@ -60,15 +65,8 @@ namespace policy {
 
 namespace {
 
-// TODO(davidyu): Update the URL to the real one once it is ready.
-// http://crbug.com/366491.
-//
-// The URL for the consumer device management server.
-const char kDefaultConsumerDeviceManagementServerUrl[] =
-    "https://m.google.com/devicemanagement/data/api";
-
 // Install attributes for tests.
-EnterpriseInstallAttributes* g_testing_install_attributes = NULL;
+chromeos::InstallAttributes* g_testing_install_attributes = nullptr;
 
 // Helper that returns a new SequencedTaskRunner backed by the blocking pool.
 // Each SequencedTaskRunner returned is independent from the others.
@@ -79,26 +77,15 @@ scoped_refptr<base::SequencedTaskRunner> GetBackgroundTaskRunner() {
       pool->GetSequenceToken(), base::SequencedWorkerPool::SKIP_ON_SHUTDOWN);
 }
 
-std::string GetDeviceManagementServerUrlForConsumer() {
-  const base::CommandLine* command_line =
-      base::CommandLine::ForCurrentProcess();
-  if (command_line->HasSwitch(
-          chromeos::switches::kConsumerDeviceManagementUrl)) {
-    return command_line->GetSwitchValueASCII(
-        chromeos::switches::kConsumerDeviceManagementUrl);
-  }
-  return kDefaultConsumerDeviceManagementServerUrl;
-}
-
 }  // namespace
 
 BrowserPolicyConnectorChromeOS::BrowserPolicyConnectorChromeOS()
-    : device_cloud_policy_manager_(NULL),
-      global_user_cloud_policy_provider_(NULL),
+    : device_cloud_policy_manager_(nullptr),
+      global_user_cloud_policy_provider_(nullptr),
       weak_ptr_factory_(this) {
   if (g_testing_install_attributes) {
     install_attributes_.reset(g_testing_install_attributes);
-    g_testing_install_attributes = NULL;
+    g_testing_install_attributes = nullptr;
   }
 
   // SystemSaltGetter or DBusThreadManager may be uninitialized on unit tests.
@@ -107,9 +94,9 @@ BrowserPolicyConnectorChromeOS::BrowserPolicyConnectorChromeOS()
   // (removing it now breaks tests). crbug.com/141016.
   if (chromeos::SystemSaltGetter::IsInitialized() &&
       chromeos::DBusThreadManager::IsInitialized()) {
-    state_keys_broker_.reset(new ServerBackedStateKeysBroker(
+    state_keys_broker_ = base::MakeUnique<ServerBackedStateKeysBroker>(
         chromeos::DBusThreadManager::Get()->GetSessionManagerClient(),
-        base::ThreadTaskRunnerHandle::Get()));
+        base::ThreadTaskRunnerHandle::Get());
 
     chromeos::CryptohomeClient* cryptohome_client =
         chromeos::DBusThreadManager::Get()->GetCryptohomeClient();
@@ -117,38 +104,27 @@ BrowserPolicyConnectorChromeOS::BrowserPolicyConnectorChromeOS()
     // Don't initialize install attributes if g_testing_install_attributes have
     // been injected.
     if (!install_attributes_) {
-      install_attributes_.reset(
-          new EnterpriseInstallAttributes(cryptohome_client));
+      install_attributes_ =
+          base::MakeUnique<chromeos::InstallAttributes>(cryptohome_client);
       base::FilePath install_attrs_file;
       CHECK(PathService::Get(chromeos::FILE_INSTALL_ATTRIBUTES,
                              &install_attrs_file));
       install_attributes_->Init(install_attrs_file);
     }
 
-    const base::CommandLine* command_line =
-        base::CommandLine::ForCurrentProcess();
-    if (command_line->HasSwitch(
-            chromeos::switches::kEnableConsumerManagement)) {
-      consumer_management_service_.reset(
-          new ConsumerManagementService(
-              cryptohome_client,
-              chromeos::DeviceSettingsService::Get()));
-    }
-
-    scoped_ptr<DeviceCloudPolicyStoreChromeOS> device_cloud_policy_store(
-        new DeviceCloudPolicyStoreChromeOS(
-            chromeos::DeviceSettingsService::Get(),
-            install_attributes_.get(),
-            GetBackgroundTaskRunner()));
+    std::unique_ptr<DeviceCloudPolicyStoreChromeOS> device_cloud_policy_store =
+        base::MakeUnique<DeviceCloudPolicyStoreChromeOS>(
+            chromeos::DeviceSettingsService::Get(), install_attributes_.get(),
+            GetBackgroundTaskRunner());
     device_cloud_policy_manager_ = new DeviceCloudPolicyManagerChromeOS(
         std::move(device_cloud_policy_store),
         base::ThreadTaskRunnerHandle::Get(), state_keys_broker_.get());
-    AddPolicyProvider(
-        scoped_ptr<ConfigurationPolicyProvider>(device_cloud_policy_manager_));
+    AddPolicyProvider(std::unique_ptr<ConfigurationPolicyProvider>(
+        device_cloud_policy_manager_));
   }
 
   global_user_cloud_policy_provider_ = new ProxyPolicyProvider();
-  AddPolicyProvider(scoped_ptr<ConfigurationPolicyProvider>(
+  AddPolicyProvider(std::unique_ptr<ConfigurationPolicyProvider>(
       global_user_cloud_policy_provider_));
 }
 
@@ -160,20 +136,8 @@ void BrowserPolicyConnectorChromeOS::Init(
   local_state_ = local_state;
   ChromeBrowserPolicyConnector::Init(local_state, request_context);
 
-  affiliated_invalidation_service_provider_.reset(
-      new AffiliatedInvalidationServiceProviderImpl);
-
-  const base::CommandLine* command_line =
-      base::CommandLine::ForCurrentProcess();
-  if (command_line->HasSwitch(chromeos::switches::kEnableConsumerManagement)) {
-    scoped_ptr<DeviceManagementService::Configuration> configuration(
-        new DeviceManagementServiceConfiguration(
-            GetDeviceManagementServerUrlForConsumer()));
-    consumer_device_management_service_.reset(
-        new DeviceManagementService(std::move(configuration)));
-    consumer_device_management_service_->ScheduleInitialization(
-        kServiceInitializationStartupDelay);
-  }
+  affiliated_invalidation_service_provider_ =
+      base::MakeUnique<AffiliatedInvalidationServiceProviderImpl>();
 
   if (device_cloud_policy_manager_) {
     // Note: for now the |device_cloud_policy_manager_| is using the global
@@ -186,28 +150,27 @@ void BrowserPolicyConnectorChromeOS::Init(
     RestartDeviceCloudPolicyInitializer();
   }
 
-  device_local_account_policy_service_.reset(
-      new DeviceLocalAccountPolicyService(
+  device_local_account_policy_service_ =
+      base::MakeUnique<DeviceLocalAccountPolicyService>(
           chromeos::DBusThreadManager::Get()->GetSessionManagerClient(),
-          chromeos::DeviceSettingsService::Get(),
-          chromeos::CrosSettings::Get(),
+          chromeos::DeviceSettingsService::Get(), chromeos::CrosSettings::Get(),
           affiliated_invalidation_service_provider_.get(),
+          GetBackgroundTaskRunner(), GetBackgroundTaskRunner(),
           GetBackgroundTaskRunner(),
-          GetBackgroundTaskRunner(),
-          GetBackgroundTaskRunner(),
-          content::BrowserThread::GetMessageLoopProxyForThread(
+          content::BrowserThread::GetTaskRunnerForThread(
               content::BrowserThread::IO),
-          request_context));
+          request_context);
   device_local_account_policy_service_->Connect(device_management_service());
   if (device_cloud_policy_manager_) {
-    device_cloud_policy_invalidator_.reset(new AffiliatedCloudPolicyInvalidator(
-        enterprise_management::DeviceRegisterRequest::DEVICE,
-        device_cloud_policy_manager_->core(),
-        affiliated_invalidation_service_provider_.get()));
-    device_remote_commands_invalidator_.reset(
-        new AffiliatedRemoteCommandsInvalidator(
+    device_cloud_policy_invalidator_ =
+        base::MakeUnique<AffiliatedCloudPolicyInvalidator>(
+            enterprise_management::DeviceRegisterRequest::DEVICE,
             device_cloud_policy_manager_->core(),
-            affiliated_invalidation_service_provider_.get()));
+            affiliated_invalidation_service_provider_.get());
+    device_remote_commands_invalidator_ =
+        base::MakeUnique<AffiliatedRemoteCommandsInvalidator>(
+            device_cloud_policy_manager_->core(),
+            affiliated_invalidation_service_provider_.get());
   }
 
   SetTimezoneIfPolicyAvailable();
@@ -219,6 +182,9 @@ void BrowserPolicyConnectorChromeOS::Init(
               ->managed_network_configuration_handler(),
           chromeos::NetworkHandler::Get()->network_device_handler(),
           chromeos::CrosSettings::Get());
+
+  bluetooth_policy_handler_ =
+      base::MakeUnique<BluetoothPolicyHandler>(chromeos::CrosSettings::Get());
 }
 
 void BrowserPolicyConnectorChromeOS::PreShutdown() {
@@ -245,7 +211,7 @@ void BrowserPolicyConnectorChromeOS::Shutdown() {
   ChromeBrowserPolicyConnector::Shutdown();
 }
 
-bool BrowserPolicyConnectorChromeOS::IsEnterpriseManaged() {
+bool BrowserPolicyConnectorChromeOS::IsEnterpriseManaged() const {
   return install_attributes_ && install_attributes_->IsEnterpriseDevice();
 }
 
@@ -253,7 +219,11 @@ std::string BrowserPolicyConnectorChromeOS::GetEnterpriseDomain() const {
   return install_attributes_ ? install_attributes_->GetDomain() : std::string();
 }
 
-std::string BrowserPolicyConnectorChromeOS::GetDeviceAssetID() {
+std::string BrowserPolicyConnectorChromeOS::GetRealm() const {
+  return install_attributes_ ? install_attributes_->GetRealm() : std::string();
+}
+
+std::string BrowserPolicyConnectorChromeOS::GetDeviceAssetID() const {
   if (device_cloud_policy_manager_) {
     const enterprise_management::PolicyData* policy =
         device_cloud_policy_manager_->device_store()->policy();
@@ -263,7 +233,7 @@ std::string BrowserPolicyConnectorChromeOS::GetDeviceAssetID() {
   return std::string();
 }
 
-std::string BrowserPolicyConnectorChromeOS::GetDirectoryApiID() {
+std::string BrowserPolicyConnectorChromeOS::GetDirectoryApiID() const {
   if (device_cloud_policy_manager_) {
     const enterprise_management::PolicyData* policy =
         device_cloud_policy_manager_->device_store()->policy();
@@ -273,7 +243,7 @@ std::string BrowserPolicyConnectorChromeOS::GetDirectoryApiID() {
   return std::string();
 }
 
-DeviceMode BrowserPolicyConnectorChromeOS::GetDeviceMode() {
+DeviceMode BrowserPolicyConnectorChromeOS::GetDeviceMode() const {
   return install_attributes_ ? install_attributes_->GetMode()
                              : DEVICE_MODE_NOT_SET;
 }
@@ -291,19 +261,14 @@ void BrowserPolicyConnectorChromeOS::SetUserPolicyDelegate(
   global_user_cloud_policy_provider_->SetDelegate(user_policy_provider);
 }
 
-void BrowserPolicyConnectorChromeOS::SetConsumerManagementServiceForTesting(
-    scoped_ptr<ConsumerManagementService> service) {
-  consumer_management_service_ = std::move(service);
-}
-
 void BrowserPolicyConnectorChromeOS::SetDeviceCloudPolicyInitializerForTesting(
-    scoped_ptr<DeviceCloudPolicyInitializer> initializer) {
+    std::unique_ptr<DeviceCloudPolicyInitializer> initializer) {
   device_cloud_policy_initializer_ = std::move(initializer);
 }
 
 // static
 void BrowserPolicyConnectorChromeOS::SetInstallAttributesForTesting(
-    EnterpriseInstallAttributes* attributes) {
+    chromeos::InstallAttributes* attributes) {
   DCHECK(!g_testing_install_attributes);
   g_testing_install_attributes = attributes;
 }
@@ -312,7 +277,7 @@ void BrowserPolicyConnectorChromeOS::SetInstallAttributesForTesting(
 void BrowserPolicyConnectorChromeOS::RemoveInstallAttributesForTesting() {
   if (g_testing_install_attributes) {
     delete g_testing_install_attributes;
-    g_testing_install_attributes = NULL;
+    g_testing_install_attributes = nullptr;
   }
 }
 
@@ -330,7 +295,7 @@ void BrowserPolicyConnectorChromeOS::OnDeviceCloudPolicyManagerConnected() {
   // DeviceCloudPolicyInitializer might still be on the call stack, so we
   // should release the initializer after this function returns.
   device_cloud_policy_initializer_->Shutdown();
-  base::MessageLoop::current()->DeleteSoon(
+  base::ThreadTaskRunnerHandle::Get()->DeleteSoon(
       FROM_HERE, device_cloud_policy_initializer_.release());
 }
 
@@ -360,19 +325,25 @@ void BrowserPolicyConnectorChromeOS::SetTimezoneIfPolicyAvailable() {
 }
 
 void BrowserPolicyConnectorChromeOS::RestartDeviceCloudPolicyInitializer() {
-  device_cloud_policy_initializer_.reset(
-      new DeviceCloudPolicyInitializer(
-          local_state_,
-          device_management_service(),
-          consumer_device_management_service_.get(),
-          GetBackgroundTaskRunner(),
-          install_attributes_.get(),
-          state_keys_broker_.get(),
+  device_cloud_policy_initializer_ =
+      base::MakeUnique<DeviceCloudPolicyInitializer>(
+          local_state_, device_management_service(), GetBackgroundTaskRunner(),
+          install_attributes_.get(), state_keys_broker_.get(),
           device_cloud_policy_manager_->device_store(),
-          device_cloud_policy_manager_));
+          device_cloud_policy_manager_,
+          cryptohome::AsyncMethodCaller::GetInstance(), CreateAttestationFlow(),
+          chromeos::system::StatisticsProvider::GetInstance());
   device_cloud_policy_initializer_->Init();
 }
 
+std::unique_ptr<chromeos::attestation::AttestationFlow>
+BrowserPolicyConnectorChromeOS::CreateAttestationFlow() {
+  return base::MakeUnique<chromeos::attestation::AttestationFlow>(
+      cryptohome::AsyncMethodCaller::GetInstance(),
+      chromeos::DBusThreadManager::Get()->GetCryptohomeClient(),
+      base::MakeUnique<chromeos::attestation::AttestationCAClient>());
+}
+
 chromeos::AffiliationIDSet
 BrowserPolicyConnectorChromeOS::GetDeviceAffiliationIDs() const {
   chromeos::AffiliationIDSet affiliation_ids;
