@@ -5,12 +5,12 @@
 #ifndef CHROME_BROWSER_CHROMEOS_POLICY_BROWSER_POLICY_CONNECTOR_CHROMEOS_H_
 #define CHROME_BROWSER_CHROMEOS_POLICY_BROWSER_POLICY_CONNECTOR_CHROMEOS_H_
 
+#include <memory>
 #include <set>
 #include <string>
 
 #include "base/macros.h"
 #include "base/memory/ref_counted.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "chrome/browser/chromeos/login/users/affiliation.h"
 #include "chrome/browser/chromeos/policy/device_cloud_policy_manager_chromeos.h"
@@ -20,6 +20,15 @@
 class PrefRegistrySimple;
 class PrefService;
 
+namespace chromeos {
+
+class InstallAttributes;
+
+namespace attestation {
+class AttestationFlow;
+}
+}
+
 namespace net {
 class URLRequestContextGetter;
 }
@@ -29,17 +38,16 @@ namespace policy {
 class AffiliatedCloudPolicyInvalidator;
 class AffiliatedInvalidationServiceProvider;
 class AffiliatedRemoteCommandsInvalidator;
-class ConsumerManagementService;
+class BluetoothPolicyHandler;
 class DeviceCloudPolicyInitializer;
 class DeviceLocalAccountPolicyService;
 class DeviceManagementService;
 struct EnrollmentConfig;
-class EnterpriseInstallAttributes;
 class NetworkConfigurationUpdater;
 class ProxyPolicyProvider;
 class ServerBackedStateKeysBroker;
 
-// Extends ChromeBrowserPolicyConnector with the setup specific to ChromeOS.
+// Extends ChromeBrowserPolicyConnector with the setup specific to Chrome OS.
 class BrowserPolicyConnectorChromeOS
     : public ChromeBrowserPolicyConnector,
       public DeviceCloudPolicyManagerChromeOS::Observer {
@@ -62,45 +70,47 @@ class BrowserPolicyConnectorChromeOS
 
   // Returns true if this device is managed by an enterprise (as opposed to
   // a local owner).
-  bool IsEnterpriseManaged();
+  bool IsEnterpriseManaged() const;
 
   // Returns the enterprise domain if device is managed.
   std::string GetEnterpriseDomain() const;
 
+  std::string GetRealm() const;
+
   // Returns the device asset ID if it is set.
-  std::string GetDeviceAssetID();
+  std::string GetDeviceAssetID() const;
 
   // Returns the cloud directory API ID or an empty string if it is not set.
-  std::string GetDirectoryApiID();
+  std::string GetDirectoryApiID() const;
 
-  // Returns the device mode. For ChromeOS this function will return the mode
+  // Returns the device mode. For Chrome OS this function will return the mode
   // stored in the lockbox, or DEVICE_MODE_CONSUMER if the lockbox has been
   // locked empty, or DEVICE_MODE_UNKNOWN if the device has not been owned yet.
   // For other OSes the function will always return DEVICE_MODE_CONSUMER.
-  DeviceMode GetDeviceMode();
+  DeviceMode GetDeviceMode() const;
 
   // Get the enrollment configuration for the device as decided by various
   // factors. See DeviceCloudPolicyInitializer::GetPrescribedEnrollmentConfig()
   // for details.
   EnrollmentConfig GetPrescribedEnrollmentConfig() const;
 
-  DeviceCloudPolicyManagerChromeOS* GetDeviceCloudPolicyManager() {
+  DeviceCloudPolicyManagerChromeOS* GetDeviceCloudPolicyManager() const {
     return device_cloud_policy_manager_;
   }
 
-  DeviceCloudPolicyInitializer* GetDeviceCloudPolicyInitializer() {
+  DeviceCloudPolicyInitializer* GetDeviceCloudPolicyInitializer() const {
     return device_cloud_policy_initializer_.get();
   }
 
-  DeviceLocalAccountPolicyService* GetDeviceLocalAccountPolicyService() {
+  DeviceLocalAccountPolicyService* GetDeviceLocalAccountPolicyService() const {
     return device_local_account_policy_service_.get();
   }
 
-  EnterpriseInstallAttributes* GetInstallAttributes() {
+  chromeos::InstallAttributes* GetInstallAttributes() const {
     return install_attributes_.get();
   }
 
-  ServerBackedStateKeysBroker* GetStateKeysBroker() {
+  ServerBackedStateKeysBroker* GetStateKeysBroker() const {
     return state_keys_broker_.get();
   }
 
@@ -115,27 +125,15 @@ class BrowserPolicyConnectorChromeOS
   // delegate, if there is one.
   void SetUserPolicyDelegate(ConfigurationPolicyProvider* user_policy_provider);
 
-  ConsumerManagementService* GetConsumerManagementService() const {
-    return consumer_management_service_.get();
-  }
-
-  DeviceManagementService* GetDeviceManagementServiceForConsumer() const {
-    return consumer_device_management_service_.get();
-  }
-
-  // Sets the consumer management service for testing.
-  void SetConsumerManagementServiceForTesting(
-      scoped_ptr<ConsumerManagementService> service);
-
   // Sets the device cloud policy initializer for testing.
   void SetDeviceCloudPolicyInitializerForTesting(
-      scoped_ptr<DeviceCloudPolicyInitializer> initializer);
+      std::unique_ptr<DeviceCloudPolicyInitializer> initializer);
 
   // Sets the install attributes for testing. Must be called before the browser
   // is created. RemoveInstallAttributesForTesting must be called after the test
   // to free the attributes.
   static void SetInstallAttributesForTesting(
-      EnterpriseInstallAttributes* attributes);
+      chromeos::InstallAttributes* attributes);
   static void RemoveInstallAttributesForTesting();
 
   // Registers device refresh rate pref.
@@ -155,22 +153,29 @@ class BrowserPolicyConnectorChromeOS
   // registration status changed from registered to unregistered.
   void RestartDeviceCloudPolicyInitializer();
 
+  // Creates an attestation flow using our async method handler and
+  // cryptohome client.
+  std::unique_ptr<chromeos::attestation::AttestationFlow>
+  CreateAttestationFlow();
+
   // Components of the device cloud policy implementation.
-  scoped_ptr<ServerBackedStateKeysBroker> state_keys_broker_;
-  scoped_ptr<EnterpriseInstallAttributes> install_attributes_;
-  scoped_ptr<AffiliatedInvalidationServiceProvider>
+  std::unique_ptr<ServerBackedStateKeysBroker> state_keys_broker_;
+  std::unique_ptr<chromeos::InstallAttributes> install_attributes_;
+  std::unique_ptr<AffiliatedInvalidationServiceProvider>
       affiliated_invalidation_service_provider_;
-  scoped_ptr<ConsumerManagementService> consumer_management_service_;
   DeviceCloudPolicyManagerChromeOS* device_cloud_policy_manager_;
   PrefService* local_state_;
-  scoped_ptr<DeviceManagementService> consumer_device_management_service_;
-  scoped_ptr<DeviceCloudPolicyInitializer> device_cloud_policy_initializer_;
-  scoped_ptr<DeviceLocalAccountPolicyService>
+  std::unique_ptr<DeviceCloudPolicyInitializer>
+      device_cloud_policy_initializer_;
+  std::unique_ptr<DeviceLocalAccountPolicyService>
       device_local_account_policy_service_;
-  scoped_ptr<AffiliatedCloudPolicyInvalidator> device_cloud_policy_invalidator_;
-  scoped_ptr<AffiliatedRemoteCommandsInvalidator>
+  std::unique_ptr<AffiliatedCloudPolicyInvalidator>
+      device_cloud_policy_invalidator_;
+  std::unique_ptr<AffiliatedRemoteCommandsInvalidator>
       device_remote_commands_invalidator_;
 
+  std::unique_ptr<BluetoothPolicyHandler> bluetooth_policy_handler_;
+
   // This policy provider is used on Chrome OS to feed user policy into the
   // global PolicyService instance. This works by installing the cloud policy
   // provider of the primary profile as the delegate of the ProxyPolicyProvider,
@@ -179,7 +184,7 @@ class BrowserPolicyConnectorChromeOS
   // pointer to get to the ProxyPolicyProvider at SetUserPolicyDelegate().
   ProxyPolicyProvider* global_user_cloud_policy_provider_;
 
-  scoped_ptr<NetworkConfigurationUpdater> network_configuration_updater_;
+  std::unique_ptr<NetworkConfigurationUpdater> network_configuration_updater_;
 
   base::WeakPtrFactory<BrowserPolicyConnectorChromeOS> weak_ptr_factory_;
 
