@@ -4,26 +4,26 @@
 
 #include "chrome/browser/chromeos/policy/device_cloud_policy_store_chromeos.h"
 
+#include <memory>
 #include <string>
 #include <vector>
 
 #include "base/compiler_specific.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/run_loop.h"
-#include "base/thread_task_runner_handle.h"
-#include "chrome/browser/chromeos/policy/enterprise_install_attributes.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "chrome/browser/chromeos/policy/proto/chrome_device_policy.pb.h"
 #include "chrome/browser/chromeos/settings/device_settings_test_helper.h"
+#include "chrome/browser/chromeos/settings/install_attributes.h"
 #include "chrome/test/base/scoped_testing_local_state.h"
 #include "chrome/test/base/testing_browser_process.h"
 #include "chromeos/cryptohome/cryptohome_util.h"
 #include "chromeos/dbus/dbus_thread_manager.h"
 #include "chromeos/dbus/fake_cryptohome_client.h"
 #include "components/policy/core/common/cloud/cloud_policy_constants.h"
+#include "components/policy/policy_constants.h"
+#include "components/policy/proto/device_management_backend.pb.h"
 #include "content/public/test/test_utils.h"
-#include "policy/policy_constants.h"
-#include "policy/proto/device_management_backend.pb.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace em = enterprise_management;
@@ -33,8 +33,8 @@ namespace policy {
 namespace {
 
 void CopyLockResult(base::RunLoop* loop,
-                    EnterpriseInstallAttributes::LockResult* out,
-                    EnterpriseInstallAttributes::LockResult result) {
+                    chromeos::InstallAttributes::LockResult* out,
+                    chromeos::InstallAttributes::LockResult result) {
   *out = result;
   loop->Quit();
 }
@@ -48,7 +48,7 @@ class DeviceCloudPolicyStoreChromeOSTest
       : local_state_(TestingBrowserProcess::GetGlobal()),
         fake_cryptohome_client_(new chromeos::FakeCryptohomeClient()),
         install_attributes_(
-            new EnterpriseInstallAttributes(fake_cryptohome_client_)),
+            new chromeos::InstallAttributes(fake_cryptohome_client_)),
         store_(new DeviceCloudPolicyStoreChromeOS(
             &device_settings_service_,
             install_attributes_.get(),
@@ -58,27 +58,18 @@ class DeviceCloudPolicyStoreChromeOSTest
     DeviceSettingsTestBase::SetUp();
 
     dbus_setter_->SetCryptohomeClient(
-        scoped_ptr<chromeos::CryptohomeClient>(fake_cryptohome_client_));
+        std::unique_ptr<chromeos::CryptohomeClient>(fake_cryptohome_client_));
 
     base::RunLoop loop;
-    EnterpriseInstallAttributes::LockResult result;
+    chromeos::InstallAttributes::LockResult result;
     install_attributes_->LockDevice(
-        PolicyBuilder::kFakeUsername,
         DEVICE_MODE_ENTERPRISE,
+        PolicyBuilder::kFakeDomain,
+        std::string(),  // realm
         PolicyBuilder::kFakeDeviceId,
         base::Bind(&CopyLockResult, &loop, &result));
     loop.Run();
-    ASSERT_EQ(EnterpriseInstallAttributes::LOCK_SUCCESS, result);
-  }
-
-  void SetManagementModeAndLoad(em::PolicyData::ManagementMode mode) {
-    device_policy_.policy_data().set_management_mode(mode);
-    device_policy_.Build();
-    device_settings_test_helper_.set_policy_blob(device_policy_.GetBlob());
-    device_settings_service_.Load();
-    FlushDeviceSettings();
-    EXPECT_EQ(chromeos::DeviceSettingsService::STORE_SUCCESS,
-              device_settings_service_.status());
+    ASSERT_EQ(chromeos::InstallAttributes::LOCK_SUCCESS, result);
   }
 
   void ExpectFailure(CloudPolicyStore::Status expected_status) {
@@ -88,17 +79,6 @@ class DeviceCloudPolicyStoreChromeOSTest
     EXPECT_FALSE(store_->is_managed());
   }
 
-  void ExpectSuccessWithManagementMode(ManagementMode mode) {
-    EXPECT_EQ(CloudPolicyStore::STATUS_OK, store_->status());
-    EXPECT_TRUE(store_->is_initialized());
-    EXPECT_TRUE(store_->has_policy());
-    EXPECT_TRUE(store_->is_managed());
-    EXPECT_TRUE(store_->policy());
-    EXPECT_TRUE(store_->policy_map().empty());
-    if (store_->policy())
-      EXPECT_EQ(mode, GetManagementMode(*store_->policy()));
-  }
-
   void ExpectSuccess() {
     EXPECT_EQ(CloudPolicyStore::STATUS_OK, store_->status());
     EXPECT_TRUE(store_->is_initialized());
@@ -133,7 +113,7 @@ class DeviceCloudPolicyStoreChromeOSTest
     chromeos::cryptohome_util::InstallAttributesSet("enterprise.owned",
                                                     std::string());
     install_attributes_.reset(
-        new EnterpriseInstallAttributes(fake_cryptohome_client_));
+        new chromeos::InstallAttributes(fake_cryptohome_client_));
     store_.reset(new DeviceCloudPolicyStoreChromeOS(
         &device_settings_service_, install_attributes_.get(),
         base::ThreadTaskRunnerHandle::Get()));
@@ -141,9 +121,9 @@ class DeviceCloudPolicyStoreChromeOSTest
 
   ScopedTestingLocalState local_state_;
   chromeos::FakeCryptohomeClient* fake_cryptohome_client_;
-  scoped_ptr<EnterpriseInstallAttributes> install_attributes_;
+  std::unique_ptr<chromeos::InstallAttributes> install_attributes_;
 
-  scoped_ptr<DeviceCloudPolicyStoreChromeOS> store_;
+  std::unique_ptr<DeviceCloudPolicyStoreChromeOS> store_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(DeviceCloudPolicyStoreChromeOSTest);
@@ -176,17 +156,6 @@ TEST_F(DeviceCloudPolicyStoreChromeOSTest, LoadSuccess) {
   ExpectSuccess();
 }
 
-TEST_F(DeviceCloudPolicyStoreChromeOSTest, UpdateFromServiceOnConsumerDevices) {
-  ResetToNonEnterprise();
-
-  SetManagementModeAndLoad(em::PolicyData::CONSUMER_MANAGED);
-  ExpectSuccessWithManagementMode(MANAGEMENT_MODE_CONSUMER_MANAGED);
-
-  // Unenroll from consumer management.
-  SetManagementModeAndLoad(em::PolicyData::LOCAL_OWNER);
-  ExpectSuccessWithManagementMode(MANAGEMENT_MODE_LOCAL_OWNER);
-}
-
 TEST_F(DeviceCloudPolicyStoreChromeOSTest, StoreSuccess) {
   PrepareExistingPolicy();
   store_->Store(device_policy_.policy());
@@ -232,8 +201,8 @@ TEST_F(DeviceCloudPolicyStoreChromeOSTest,
   PrepareExistingPolicy();
   device_policy_.SetDefaultNewSigningKey();
   device_policy_.Build();
-  *device_policy_.policy().mutable_new_public_key_verification_signature() =
-      "garbage";
+  *device_policy_.policy()
+       .mutable_new_public_key_verification_signature_deprecated() = "garbage";
   store_->Store(device_policy_.policy());
   FlushDeviceSettings();
   EXPECT_EQ(CloudPolicyStore::STATUS_VALIDATION_ERROR, store_->status());
@@ -246,7 +215,8 @@ TEST_F(DeviceCloudPolicyStoreChromeOSTest,
   PrepareExistingPolicy();
   device_policy_.SetDefaultNewSigningKey();
   device_policy_.Build();
-  device_policy_.policy().clear_new_public_key_verification_signature();
+  device_policy_.policy()
+      .clear_new_public_key_verification_signature_deprecated();
   store_->Store(device_policy_.policy());
   FlushDeviceSettings();
   EXPECT_EQ(CloudPolicyStore::STATUS_VALIDATION_ERROR, store_->status());
@@ -274,8 +244,8 @@ TEST_F(DeviceCloudPolicyStoreChromeOSTest, InstallInitialPolicyNoSignature) {
 TEST_F(DeviceCloudPolicyStoreChromeOSTest,
        InstallInitialPolicyVerificationFailure) {
   PrepareNewSigningKey();
-  *device_policy_.policy().mutable_new_public_key_verification_signature() =
-      "garbage";
+  *device_policy_.policy()
+       .mutable_new_public_key_verification_signature_deprecated() = "garbage";
   store_->InstallInitialPolicy(device_policy_.policy());
   FlushDeviceSettings();
   ExpectFailure(CloudPolicyStore::STATUS_VALIDATION_ERROR);
@@ -286,7 +256,8 @@ TEST_F(DeviceCloudPolicyStoreChromeOSTest,
 TEST_F(DeviceCloudPolicyStoreChromeOSTest,
        InstallInitialPolicyMissingSignatureFailure) {
   PrepareNewSigningKey();
-  device_policy_.policy().clear_new_public_key_verification_signature();
+  device_policy_.policy()
+      .clear_new_public_key_verification_signature_deprecated();
   store_->InstallInitialPolicy(device_policy_.policy());
   FlushDeviceSettings();
   ExpectFailure(CloudPolicyStore::STATUS_VALIDATION_ERROR);
