@@ -18,8 +18,7 @@
 #include "base/sequenced_task_runner.h"
 #include "base/stl_util.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/thread_task_runner_handle.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/chromeos/policy/affiliated_cloud_policy_invalidator.h"
 #include "chrome/browser/chromeos/policy/device_local_account.h"
@@ -37,13 +36,12 @@
 #include "components/policy/core/common/cloud/cloud_policy_refresh_scheduler.h"
 #include "components/policy/core/common/cloud/device_management_service.h"
 #include "components/policy/core/common/cloud/resource_cache.h"
-#include "components/policy/core/common/cloud/system_policy_request_context.h"
 #include "components/policy/core/common/policy_namespace.h"
 #include "components/policy/core/common/policy_switches.h"
+#include "components/policy/policy_constants.h"
+#include "components/policy/proto/device_management_backend.pb.h"
 #include "content/public/browser/browser_thread.h"
 #include "net/url_request/url_request_context_getter.h"
-#include "policy/policy_constants.h"
-#include "policy/proto/device_management_backend.pb.h"
 #include "url/gurl.h"
 
 namespace em = enterprise_management;
@@ -53,26 +51,24 @@ namespace policy {
 namespace {
 
 // Creates and initializes a cloud policy client. Returns nullptr if the device
-// is not enterprise-enrolled.
-scoped_ptr<CloudPolicyClient> CreateClient(
+// doesn't have credentials in device settings (i.e. is not
+// enterprise-enrolled).
+std::unique_ptr<CloudPolicyClient> CreateClient(
     chromeos::DeviceSettingsService* device_settings_service,
     DeviceManagementService* device_management_service,
     scoped_refptr<net::URLRequestContextGetter> system_request_context) {
   const em::PolicyData* policy_data = device_settings_service->policy_data();
   if (!policy_data ||
-      GetManagementMode(*policy_data) != MANAGEMENT_MODE_ENTERPRISE_MANAGED ||
+      !policy_data->has_request_token() ||
+      !policy_data->has_device_id() ||
       !device_management_service) {
-    return scoped_ptr<CloudPolicyClient>();
+    return std::unique_ptr<CloudPolicyClient>();
   }
 
-  scoped_refptr<net::URLRequestContextGetter> request_context =
-      new SystemPolicyRequestContext(
-          system_request_context, GetUserAgent());
-
-  scoped_ptr<CloudPolicyClient> client(
-      new CloudPolicyClient(std::string(), std::string(),
-                            kPolicyVerificationKeyHash,
-                            device_management_service, request_context));
+  std::unique_ptr<CloudPolicyClient> client(new CloudPolicyClient(
+      std::string(), std::string(), kPolicyVerificationKeyHash,
+      device_management_service, system_request_context,
+      nullptr /* signing_service */));
   client->SetupRegistration(policy_data->request_token(),
                             policy_data->device_id());
   return client;
@@ -96,7 +92,7 @@ void DeleteOrphanedCaches(
   for (base::FilePath path = enumerator.Next(); !path.empty();
        path = enumerator.Next()) {
     const std::string subdirectory(path.BaseName().MaybeAsASCII());
-    if (!ContainsKey(subdirectories_to_keep, subdirectory))
+    if (!base::ContainsKey(subdirectories_to_keep, subdirectory))
       base::DeleteFile(path, true);
   }
 }
@@ -119,7 +115,7 @@ void DeleteObsoleteExtensionCache(const std::string& account_id_to_delete) {
 DeviceLocalAccountPolicyBroker::DeviceLocalAccountPolicyBroker(
     const DeviceLocalAccount& account,
     const base::FilePath& component_policy_cache_path,
-    scoped_ptr<DeviceLocalAccountPolicyStore> store,
+    std::unique_ptr<DeviceLocalAccountPolicyStore> store,
     scoped_refptr<DeviceLocalAccountExternalDataManager> external_data_manager,
     const base::Closure& policy_update_callback,
     const scoped_refptr<base::SequencedTaskRunner>& task_runner,
@@ -129,13 +125,16 @@ DeviceLocalAccountPolicyBroker::DeviceLocalAccountPolicyBroker(
       user_id_(account.user_id),
       component_policy_cache_path_(component_policy_cache_path),
       store_(std::move(store)),
-      extension_tracker_(account, store_.get(), &schema_registry_),
       external_data_manager_(external_data_manager),
       core_(dm_protocol::kChromePublicAccountPolicyType,
             store_->account_id(),
             store_.get(),
             task_runner),
       policy_update_callback_(policy_update_callback) {
+  if (account.type != DeviceLocalAccount::TYPE_ARC_KIOSK_APP) {
+    extension_tracker_.reset(new DeviceLocalAccountExtensionTracker(
+        account, store_.get(), &schema_registry_));
+  }
   base::FilePath cache_root_dir;
   CHECK(PathService::Get(chromeos::DIR_DEVICE_LOCAL_ACCOUNT_EXTENSIONS,
                          &cache_root_dir));
@@ -150,8 +149,7 @@ DeviceLocalAccountPolicyBroker::DeviceLocalAccountPolicyBroker(
   schema_registry_.RegisterComponent(
       PolicyNamespace(POLICY_DOMAIN_CHROME, std::string()),
       g_browser_process->browser_policy_connector()->GetChromeSchema());
-  schema_registry_.SetReady(POLICY_DOMAIN_CHROME);
-  schema_registry_.SetReady(POLICY_DOMAIN_EXTENSIONS);
+  schema_registry_.SetAllDomainsReady();
 }
 
 DeviceLocalAccountPolicyBroker::~DeviceLocalAccountPolicyBroker() {
@@ -165,7 +163,7 @@ void DeviceLocalAccountPolicyBroker::Initialize() {
 }
 
 bool DeviceLocalAccountPolicyBroker::HasInvalidatorForTest() const {
-  return invalidator_;
+  return invalidator_ != nullptr;
 }
 
 void DeviceLocalAccountPolicyBroker::ConnectIfPossible(
@@ -175,9 +173,8 @@ void DeviceLocalAccountPolicyBroker::ConnectIfPossible(
   if (core_.client())
     return;
 
-  scoped_ptr<CloudPolicyClient> client(CreateClient(device_settings_service,
-                                                    device_management_service,
-                                                    request_context));
+  std::unique_ptr<CloudPolicyClient> client(CreateClient(
+      device_settings_service, device_management_service, request_context));
   if (!client)
     return;
 
@@ -198,7 +195,7 @@ void DeviceLocalAccountPolicyBroker::UpdateRefreshDelay() {
         store_->policy_map().GetValue(key::kPolicyRefreshRate);
     int delay = 0;
     if (policy_value && policy_value->GetAsInteger(&delay))
-      core_.refresh_scheduler()->SetRefreshDelay(delay);
+      core_.refresh_scheduler()->SetDesiredRefreshDelay(delay);
   }
 }
 
@@ -233,16 +230,17 @@ void DeviceLocalAccountPolicyBroker::CreateComponentCloudPolicyService(
     return;
   }
 
-  scoped_ptr<ResourceCache> resource_cache(
+  std::unique_ptr<ResourceCache> resource_cache(
       new ResourceCache(component_policy_cache_path_,
-                        content::BrowserThread::GetMessageLoopProxyForThread(
+                        content::BrowserThread::GetTaskRunnerForThread(
                             content::BrowserThread::FILE)));
 
   component_policy_service_.reset(new ComponentCloudPolicyService(
-      this, &schema_registry_, core(), client, std::move(resource_cache),
-      request_context, content::BrowserThread::GetMessageLoopProxyForThread(
-                           content::BrowserThread::FILE),
-      content::BrowserThread::GetMessageLoopProxyForThread(
+      dm_protocol::kChromeExtensionPolicyType, this, &schema_registry_, core(),
+      client, std::move(resource_cache), request_context,
+      content::BrowserThread::GetTaskRunnerForThread(
+          content::BrowserThread::FILE),
+      content::BrowserThread::GetTaskRunnerForThread(
           content::BrowserThread::IO)));
 }
 
@@ -428,7 +426,10 @@ void DeviceLocalAccountPolicyService::UpdateAccountList() {
       break;
     case chromeos::CrosSettingsProvider::TEMPORARILY_UNTRUSTED:
       waiting_for_cros_settings_ = true;
-      return;
+      // Purposely break to allow initialization with temporarily untrusted
+      // settings so that a crash-n-restart public session have its loader
+      // properly registered as ExtensionService's external provider.
+      break;
     case chromeos::CrosSettingsProvider::PERMANENTLY_UNTRUSTED:
       waiting_for_cros_settings_ = false;
       return;
@@ -445,7 +446,7 @@ void DeviceLocalAccountPolicyService::UpdateAccountList() {
        it != device_local_accounts.end(); ++it) {
     PolicyBrokerMap::iterator broker_it = old_policy_brokers.find(it->user_id);
 
-    scoped_ptr<DeviceLocalAccountPolicyBroker> broker;
+    std::unique_ptr<DeviceLocalAccountPolicyBroker> broker;
     bool broker_initialized = false;
     if (broker_it != old_policy_brokers.end()) {
       // Reuse the existing broker if present.
@@ -453,11 +454,10 @@ void DeviceLocalAccountPolicyService::UpdateAccountList() {
       old_policy_brokers.erase(broker_it);
       broker_initialized = true;
     } else {
-      scoped_ptr<DeviceLocalAccountPolicyStore> store(
-          new DeviceLocalAccountPolicyStore(it->account_id,
-                                            session_manager_client_,
-                                            device_settings_service_,
-                                            store_background_task_runner_));
+      std::unique_ptr<DeviceLocalAccountPolicyStore> store(
+          new DeviceLocalAccountPolicyStore(
+              it->account_id, session_manager_client_, device_settings_service_,
+              store_background_task_runner_));
       scoped_refptr<DeviceLocalAccountExternalDataManager>
           external_data_manager =
               external_data_service_->GetExternalDataManager(it->account_id,
@@ -535,7 +535,8 @@ void DeviceLocalAccountPolicyService::UpdateAccountList() {
                                               component_policy_cache_root_,
                                               subdirectories_to_keep));
 
-  FOR_EACH_OBSERVER(Observer, observers_, OnDeviceLocalAccountsChanged());
+  for (auto& observer : observers_)
+    observer.OnDeviceLocalAccountsChanged();
 }
 
 void DeviceLocalAccountPolicyService::DeleteBrokers(PolicyBrokerMap* map) {
@@ -569,7 +570,8 @@ DeviceLocalAccountPolicyBroker*
 
 void DeviceLocalAccountPolicyService::NotifyPolicyUpdated(
     const std::string& user_id) {
-  FOR_EACH_OBSERVER(Observer, observers_, OnPolicyUpdated(user_id));
+  for (auto& observer : observers_)
+    observer.OnPolicyUpdated(user_id);
 }
 
 }  // namespace policy
