@@ -8,16 +8,15 @@
 
 #include "base/bind.h"
 #include "base/callback.h"
-#include "chrome/browser/browser_process.h"
-#include "chrome/browser/chromeos/policy/browser_policy_connector_chromeos.h"
+#include "base/memory/ptr_util.h"
 #include "chromeos/dbus/session_manager_client.h"
 #include "components/ownership/owner_key_util.h"
 #include "components/policy/core/common/cloud/device_management_service.h"
 #include "components/policy/core/common/external_data_fetcher.h"
 #include "components/policy/core/common/policy_map.h"
 #include "components/policy/core/common/policy_types.h"
-#include "policy/proto/cloud_policy.pb.h"
-#include "policy/proto/device_management_backend.pb.h"
+#include "components/policy/proto/cloud_policy.pb.h"
+#include "components/policy/proto/device_management_backend.pb.h"
 
 namespace em = enterprise_management;
 
@@ -48,8 +47,7 @@ void DeviceLocalAccountPolicyStore::Store(
     const em::PolicyFetchResponse& policy) {
   weak_factory_.InvalidateWeakPtrs();
   CheckKeyAndValidate(
-      true,
-      make_scoped_ptr(new em::PolicyFetchResponse(policy)),
+      true, base::MakeUnique<em::PolicyFetchResponse>(policy),
       base::Bind(&DeviceLocalAccountPolicyStore::StoreValidatedPolicy,
                  weak_factory_.GetWeakPtr()));
 }
@@ -60,7 +58,8 @@ void DeviceLocalAccountPolicyStore::ValidateLoadedPolicyBlob(
     status_ = CloudPolicyStore::STATUS_LOAD_ERROR;
     NotifyStoreError();
   } else {
-    scoped_ptr<em::PolicyFetchResponse> policy(new em::PolicyFetchResponse());
+    std::unique_ptr<em::PolicyFetchResponse> policy(
+        new em::PolicyFetchResponse());
     if (policy->ParseFromString(policy_blob)) {
       CheckKeyAndValidate(
           false, std::move(policy),
@@ -122,7 +121,7 @@ void DeviceLocalAccountPolicyStore::HandleStoreResult(bool success) {
 
 void DeviceLocalAccountPolicyStore::CheckKeyAndValidate(
     bool valid_timestamp_required,
-    scoped_ptr<em::PolicyFetchResponse> policy,
+    std::unique_ptr<em::PolicyFetchResponse> policy,
     const UserCloudPolicyValidator::CompletionCallback& callback) {
   device_settings_service_->GetOwnershipStatusAsync(
       base::Bind(&DeviceLocalAccountPolicyStore::Validate,
@@ -134,7 +133,7 @@ void DeviceLocalAccountPolicyStore::CheckKeyAndValidate(
 
 void DeviceLocalAccountPolicyStore::Validate(
     bool valid_timestamp_required,
-    scoped_ptr<em::PolicyFetchResponse> policy_response,
+    std::unique_ptr<em::PolicyFetchResponse> policy_response,
     const UserCloudPolicyValidator::CompletionCallback& callback,
     chromeos::DeviceSettingsService::OwnershipStatus ownership_status) {
   DCHECK_NE(chromeos::DeviceSettingsService::OWNERSHIP_UNKNOWN,
@@ -149,7 +148,7 @@ void DeviceLocalAccountPolicyStore::Validate(
     return;
   }
 
-  scoped_ptr<UserCloudPolicyValidator> validator(
+  std::unique_ptr<UserCloudPolicyValidator> validator(
       UserCloudPolicyValidator::Create(std::move(policy_response),
                                        background_task_runner()));
   validator->ValidateUsername(account_id_, false);
@@ -160,21 +159,21 @@ void DeviceLocalAccountPolicyStore::Validate(
   validator->ValidateAgainstCurrentPolicy(
       policy(),
       valid_timestamp_required
-          ? CloudPolicyValidatorBase::TIMESTAMP_REQUIRED
-          : CloudPolicyValidatorBase::TIMESTAMP_NOT_REQUIRED,
-      CloudPolicyValidatorBase::DM_TOKEN_NOT_REQUIRED);
+          ? CloudPolicyValidatorBase::TIMESTAMP_FULLY_VALIDATED
+          : CloudPolicyValidatorBase::TIMESTAMP_NOT_VALIDATED,
+      CloudPolicyValidatorBase::DM_TOKEN_NOT_REQUIRED,
+      CloudPolicyValidatorBase::DEVICE_ID_NOT_REQUIRED);
 
   // Validate the DMToken to match what device policy has.
   validator->ValidateDMToken(device_policy_data->request_token(),
                              CloudPolicyValidatorBase::DM_TOKEN_REQUIRED);
 
+  // Validate the device id to match what device policy has.
+  validator->ValidateDeviceId(device_policy_data->device_id(),
+                              CloudPolicyValidatorBase::DEVICE_ID_REQUIRED);
+
   validator->ValidatePayload();
-  policy::BrowserPolicyConnectorChromeOS* connector =
-      g_browser_process->platform_part()->browser_policy_connector_chromeos();
-  validator->ValidateSignature(key->as_string(),
-                               GetPolicyVerificationKey(),
-                               connector->GetEnterpriseDomain(),
-                               false);
+  validator->ValidateSignature(key->as_string());
   validator.release()->StartValidation(callback);
 }
 
