@@ -6,7 +6,7 @@
 
 #include <utility>
 
-#include "base/prefs/testing_pref_service.h"
+#include "base/memory/ptr_util.h"
 #include "base/test/test_simple_task_runner.h"
 #include "base/time/time.h"
 #include "chrome/browser/chromeos/policy/device_local_account.h"
@@ -16,6 +16,7 @@
 #include "components/policy/core/common/cloud/cloud_policy_client.h"
 #include "components/policy/core/common/cloud/mock_cloud_policy_client.h"
 #include "components/policy/core/common/cloud/mock_device_management_service.h"
+#include "components/prefs/testing_pref_service.h"
 #include "content/public/test/test_browser_thread_bundle.h"
 #include "content/public/test/test_utils.h"
 #include "net/url_request/url_request_context_getter.h"
@@ -52,21 +53,21 @@ class MockDeviceStatusCollector : public policy::DeviceStatusCollector {
       : DeviceStatusCollector(
             local_state,
             nullptr,
-            policy::DeviceStatusCollector::LocationUpdateRequester(),
             policy::DeviceStatusCollector::VolumeInfoFetcher(),
             policy::DeviceStatusCollector::CPUStatisticsFetcher(),
-            policy::DeviceStatusCollector::CPUTempFetcher()) {}
+            policy::DeviceStatusCollector::CPUTempFetcher(),
+            policy::DeviceStatusCollector::AndroidStatusFetcher()) {}
 
-  MOCK_METHOD1(GetDeviceStatus, bool(em::DeviceStatusReportRequest*));
-  MOCK_METHOD1(GetDeviceSessionStatus, bool(em::SessionStatusReportRequest*));
+  MOCK_METHOD1(GetDeviceAndSessionStatusAsync,
+               void(const policy::DeviceStatusCollector::StatusCallback&));
 
   // Explicit mock implementation declared here, since gmock::Invoke can't
   // handle returning non-moveable types like scoped_ptr.
-  scoped_ptr<policy::DeviceLocalAccount> GetAutoLaunchedKioskSessionInfo()
+  std::unique_ptr<policy::DeviceLocalAccount> GetAutoLaunchedKioskSessionInfo()
       override {
-    return make_scoped_ptr(new policy::DeviceLocalAccount(
+    return base::MakeUnique<policy::DeviceLocalAccount>(
         policy::DeviceLocalAccount::TYPE_KIOSK_APP, "account_id", "app_id",
-        "update_url"));
+        "update_url");
   }
 };
 
@@ -86,6 +87,10 @@ class StatusUploaderTest : public testing::Test {
     client_.SetDMToken("dm_token");
     collector_.reset(new MockDeviceStatusCollector(&prefs_));
     settings_helper_.ReplaceProvider(chromeos::kReportUploadFrequency);
+
+    // Keep a pointer to the mock collector because collector_ gets cleared
+    // when it is passed to the StatusUploader constructor.
+    collector_ptr_ = collector_.get();
   }
 
   void TearDown() override {
@@ -95,23 +100,40 @@ class StatusUploaderTest : public testing::Test {
   // Given a pending task to upload status, mocks out a server response.
   void RunPendingUploadTaskAndCheckNext(const StatusUploader& uploader,
                                         base::TimeDelta expected_delay) {
-    EXPECT_FALSE(task_runner_->GetPendingTasks().empty());
+    // Running the task should pass two callbacks into GetDeviceStatusAsync
+    // and GetDeviceSessionStatusAsync. We'll grab these two callbacks.
+    EXPECT_TRUE(task_runner_->HasPendingTask());
+    DeviceStatusCollector::StatusCallback status_callback;
+    EXPECT_CALL(*collector_ptr_, GetDeviceAndSessionStatusAsync(_))
+        .WillOnce(SaveArg<0>(&status_callback));
+    task_runner_->RunPendingTasks();
+    testing::Mock::VerifyAndClearExpectations(&device_management_service_);
+
+    // Send some "valid" (read: non-nullptr) to the device/session callbacks
+    // in order to simulate valid status data.
+    std::unique_ptr<em::DeviceStatusReportRequest> device_status =
+        base::MakeUnique<em::DeviceStatusReportRequest>();
+    std::unique_ptr<em::SessionStatusReportRequest> session_status =
+        base::MakeUnique<em::SessionStatusReportRequest>();
+
+    // Running the session and device callbacks should trigger
+    // CloudPolicyClient::UploadDeviceStatus.
     CloudPolicyClient::StatusCallback callback;
     EXPECT_CALL(client_, UploadDeviceStatus(_, _, _))
         .WillOnce(SaveArg<2>(&callback));
-    task_runner_->RunPendingTasks();
+    status_callback.Run(std::move(device_status), std::move(session_status));
     testing::Mock::VerifyAndClearExpectations(&device_management_service_);
 
     // Make sure no status upload is queued up yet (since an upload is in
     // progress).
-    EXPECT_TRUE(task_runner_->GetPendingTasks().empty());
+    EXPECT_FALSE(task_runner_->HasPendingTask());
 
     // Now invoke the response.
     callback.Run(true);
 
     // Now that the previous request was satisfied, a task to do the next
     // upload should be queued.
-    EXPECT_EQ(1U, task_runner_->GetPendingTasks().size());
+    EXPECT_EQ(1U, task_runner_->NumPendingTasks());
 
     CheckPendingTaskDelay(uploader, expected_delay);
   }
@@ -130,7 +152,8 @@ class StatusUploaderTest : public testing::Test {
   content::TestBrowserThreadBundle thread_bundle_;
   scoped_refptr<base::TestSimpleTaskRunner> task_runner_;
   chromeos::ScopedCrosSettingsTestHelper settings_helper_;
-  scoped_ptr<MockDeviceStatusCollector> collector_;
+  std::unique_ptr<MockDeviceStatusCollector> collector_;
+  MockDeviceStatusCollector* collector_ptr_;
   ui::UserActivityDetector detector_;
   MockCloudPolicyClient client_;
   MockDeviceManagementService device_management_service_;
@@ -138,42 +161,32 @@ class StatusUploaderTest : public testing::Test {
 };
 
 TEST_F(StatusUploaderTest, BasicTest) {
-  EXPECT_TRUE(task_runner_->GetPendingTasks().empty());
+  EXPECT_FALSE(task_runner_->HasPendingTask());
   StatusUploader uploader(&client_, std::move(collector_), task_runner_);
-  EXPECT_EQ(1U, task_runner_->GetPendingTasks().size());
-  // On startup, first update should happen immediately.
-  EXPECT_EQ(base::TimeDelta(), task_runner_->NextPendingTaskDelay());
+  EXPECT_EQ(1U, task_runner_->NumPendingTasks());
+  // On startup, first update should happen in 1 minute.
+  EXPECT_EQ(base::TimeDelta::FromMinutes(1),
+            task_runner_->NextPendingTaskDelay());
 }
 
 TEST_F(StatusUploaderTest, DifferentFrequencyAtStart) {
-  // Keep a pointer to the mock collector because collector_ gets cleared
-  // when it is passed to the StatusUploader constructor below.
-  MockDeviceStatusCollector* const mock_collector = collector_.get();
   const int new_delay = StatusUploader::kDefaultUploadDelayMs * 2;
   settings_helper_.SetInteger(chromeos::kReportUploadFrequency, new_delay);
   const base::TimeDelta expected_delay = base::TimeDelta::FromMilliseconds(
       new_delay);
-  EXPECT_TRUE(task_runner_->GetPendingTasks().empty());
+  EXPECT_FALSE(task_runner_->HasPendingTask());
   StatusUploader uploader(&client_, std::move(collector_), task_runner_);
-  ASSERT_EQ(1U, task_runner_->GetPendingTasks().size());
-  // On startup, first update should happen immediately.
-  EXPECT_EQ(base::TimeDelta(), task_runner_->NextPendingTaskDelay());
+  ASSERT_EQ(1U, task_runner_->NumPendingTasks());
+  // On startup, first update should happen in 1 minute.
+  EXPECT_EQ(base::TimeDelta::FromMinutes(1),
+            task_runner_->NextPendingTaskDelay());
 
-  EXPECT_CALL(*mock_collector, GetDeviceStatus(_)).WillRepeatedly(Return(true));
-  EXPECT_CALL(*mock_collector, GetDeviceSessionStatus(_)).WillRepeatedly(
-      Return(true));
   // Second update should use the delay specified in settings.
   RunPendingUploadTaskAndCheckNext(uploader, expected_delay);
 }
 
 TEST_F(StatusUploaderTest, ResetTimerAfterStatusCollection) {
-  // Keep a pointer to the mock collector because collector_ gets cleared
-  // when it is passed to the StatusUploader constructor below.
-  MockDeviceStatusCollector* const mock_collector = collector_.get();
   StatusUploader uploader(&client_, std::move(collector_), task_runner_);
-  EXPECT_CALL(*mock_collector, GetDeviceStatus(_)).WillRepeatedly(Return(true));
-  EXPECT_CALL(*mock_collector, GetDeviceSessionStatus(_)).WillRepeatedly(
-      Return(true));
   const base::TimeDelta expected_delay = base::TimeDelta::FromMilliseconds(
       StatusUploader::kDefaultUploadDelayMs);
   RunPendingUploadTaskAndCheckNext(uploader, expected_delay);
@@ -183,34 +196,39 @@ TEST_F(StatusUploaderTest, ResetTimerAfterStatusCollection) {
 
   // Now that the previous request was satisfied, a task to do the next
   // upload should be queued again.
-  EXPECT_EQ(1U, task_runner_->GetPendingTasks().size());
+  EXPECT_EQ(1U, task_runner_->NumPendingTasks());
 }
 
 TEST_F(StatusUploaderTest, ResetTimerAfterFailedStatusCollection) {
-  // Keep a pointer to the mock collector because collector_ gets cleared
-  // when it is passed to the StatusUploader constructor below.
-  MockDeviceStatusCollector* mock_collector = collector_.get();
   StatusUploader uploader(&client_, std::move(collector_), task_runner_);
-  EXPECT_CALL(*mock_collector, GetDeviceStatus(_)).WillOnce(Return(false));
-  EXPECT_CALL(*mock_collector, GetDeviceSessionStatus(_)).WillOnce(
-      Return(false));
-  task_runner_->RunPendingTasks();
 
-  // Make sure the next status upload is queued up.
-  EXPECT_EQ(1U, task_runner_->GetPendingTasks().size());
+  // Running the queued task should pass two callbacks into GetDeviceStatusAsync
+  // and GetDeviceSessionStatusAsync. We'll grab these two callbacks and send
+  // nullptrs to them in order to simulate failure to get status.
+  EXPECT_EQ(1U, task_runner_->NumPendingTasks());
+  DeviceStatusCollector::StatusCallback status_callback;
+  EXPECT_CALL(*collector_ptr_, GetDeviceAndSessionStatusAsync(_))
+      .WillOnce(SaveArg<0>(&status_callback));
+  task_runner_->RunPendingTasks();
+  testing::Mock::VerifyAndClearExpectations(&device_management_service_);
+
+  // Running the session and device callbacks should trigger
+  // StatusUploader::OnStatusReceived, which in turn should recognize the
+  // failure to get status and queue another status upload.
+  std::unique_ptr<em::DeviceStatusReportRequest> invalid_device_status;
+  std::unique_ptr<em::SessionStatusReportRequest> invalid_session_status;
+  status_callback.Run(std::move(invalid_device_status),
+                      std::move(invalid_session_status));
+  EXPECT_EQ(1U, task_runner_->NumPendingTasks());
+
+  // Check the delay of the queued upload
   const base::TimeDelta expected_delay = base::TimeDelta::FromMilliseconds(
       StatusUploader::kDefaultUploadDelayMs);
   CheckPendingTaskDelay(uploader, expected_delay);
 }
 
 TEST_F(StatusUploaderTest, ChangeFrequency) {
-  // Keep a pointer to the mock collector because collector_ gets cleared
-  // when it is passed to the StatusUploader constructor below.
-  MockDeviceStatusCollector* const mock_collector = collector_.get();
   StatusUploader uploader(&client_, std::move(collector_), task_runner_);
-  EXPECT_CALL(*mock_collector, GetDeviceStatus(_)).WillRepeatedly(Return(true));
-  EXPECT_CALL(*mock_collector, GetDeviceSessionStatus(_)).WillRepeatedly(
-      Return(true));
   // Change the frequency. The new frequency should be reflected in the timing
   // used for the next callback.
   const int new_delay = StatusUploader::kDefaultUploadDelayMs * 2;
