@@ -4,23 +4,31 @@
 
 #include "chrome/browser/chromeos/policy/user_cloud_policy_store_chromeos.h"
 
+#include <stdint.h>
+
+#include <memory>
 #include <vector>
 
-#include "base/basictypes.h"
 #include "base/bind.h"
 #include "base/files/file_util.h"
 #include "base/files/scoped_temp_dir.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/location.h"
+#include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
+#include "base/run_loop.h"
+#include "base/single_thread_task_runner.h"
 #include "base/threading/sequenced_worker_pool.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "chromeos/dbus/mock_cryptohome_client.h"
 #include "chromeos/dbus/mock_session_manager_client.h"
 #include "components/policy/core/common/cloud/cloud_policy_constants.h"
 #include "components/policy/core/common/cloud/mock_cloud_policy_store.h"
 #include "components/policy/core/common/cloud/policy_builder.h"
-#include "policy/policy_constants.h"
-#include "policy/proto/cloud_policy.pb.h"
-#include "policy/proto/device_management_local.pb.h"
+#include "components/policy/core/common/policy_types.h"
+#include "components/policy/policy_constants.h"
+#include "components/policy/proto/cloud_policy.pb.h"
+#include "components/policy/proto/device_management_local.pb.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -46,7 +54,7 @@ const char kSanitizedUsername[] =
 const char kDefaultHomepage[] = "http://chromium.org";
 
 ACTION_P2(SendSanitizedUsername, call_status, sanitized_username) {
-  base::MessageLoop::current()->PostTask(
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE, base::Bind(arg1, call_status, sanitized_username));
 }
 
@@ -55,23 +63,20 @@ class UserCloudPolicyStoreChromeOSTest : public testing::Test {
   UserCloudPolicyStoreChromeOSTest() {}
 
   void SetUp() override {
-    EXPECT_CALL(cryptohome_client_,
-                GetSanitizedUsername(PolicyBuilder::kFakeUsername, _))
+    EXPECT_CALL(cryptohome_client_, GetSanitizedUsername(cryptohome_id_, _))
         .Times(AnyNumber())
-        .WillRepeatedly(
-            SendSanitizedUsername(chromeos::DBUS_METHOD_CALL_SUCCESS,
-                                  kSanitizedUsername));
+        .WillRepeatedly(SendSanitizedUsername(
+            chromeos::DBUS_METHOD_CALL_SUCCESS, kSanitizedUsername));
 
     ASSERT_TRUE(tmp_dir_.CreateUniqueTempDir());
     store_.reset(new UserCloudPolicyStoreChromeOS(
         &cryptohome_client_, &session_manager_client_, loop_.task_runner(),
-        PolicyBuilder::kFakeUsername, user_policy_dir(), token_file(),
-        policy_file()));
+        account_id_, user_policy_dir(), token_file(), policy_file()));
     store_->AddObserver(&observer_);
 
     // Install the initial public key, so that by default the validation of
     // the stored/loaded policy blob succeeds.
-    std::vector<uint8> public_key;
+    std::vector<uint8_t> public_key;
     ASSERT_TRUE(policy_.GetSigningKey()->ExportPublicKey(&public_key));
     StoreUserPolicyKey(public_key);
 
@@ -82,7 +87,7 @@ class UserCloudPolicyStoreChromeOSTest : public testing::Test {
   void TearDown() override {
     store_->RemoveObserver(&observer_);
     store_.reset();
-    RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   // Install an expectation on |observer_| for an error code.
@@ -99,16 +104,16 @@ class UserCloudPolicyStoreChromeOSTest : public testing::Test {
     // Issue a load command.
     chromeos::SessionManagerClient::RetrievePolicyCallback retrieve_callback;
     EXPECT_CALL(session_manager_client_,
-                RetrievePolicyForUser(PolicyBuilder::kFakeUsername, _))
+                RetrievePolicyForUser(cryptohome_id_, _))
         .WillOnce(SaveArg<1>(&retrieve_callback));
     store_->Load();
-    RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
     Mock::VerifyAndClearExpectations(&session_manager_client_);
     ASSERT_FALSE(retrieve_callback.is_null());
 
     // Run the callback.
     retrieve_callback.Run(response);
-    RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   // Verifies that store_->policy_map() has the HomepageLocation entry with
@@ -118,10 +123,10 @@ class UserCloudPolicyStoreChromeOSTest : public testing::Test {
     const PolicyMap::Entry* entry =
         store_->policy_map().Get(key::kHomepageLocation);
     ASSERT_TRUE(entry);
-    EXPECT_TRUE(base::StringValue(expected_value).Equals(entry->value));
+    EXPECT_TRUE(base::StringValue(expected_value).Equals(entry->value.get()));
   }
 
-  void StoreUserPolicyKey(const std::vector<uint8>& public_key) {
+  void StoreUserPolicyKey(const std::vector<uint8_t>& public_key) {
     ASSERT_TRUE(base::CreateDirectory(user_policy_key_file().DirName()));
     ASSERT_TRUE(
         base::WriteFile(user_policy_key_file(),
@@ -136,16 +141,15 @@ class UserCloudPolicyStoreChromeOSTest : public testing::Test {
   // value will be expected; otherwise no previous policy is expected.
   // If |new_value| is set then a new policy with that value is expected after
   // storing the |policy_| blob.
-  void PerformStorePolicy(const std::vector<uint8>* new_public_key,
+  void PerformStorePolicy(const std::vector<uint8_t>* new_public_key,
                           const char* previous_value,
                           const char* new_value) {
     chromeos::SessionManagerClient::StorePolicyCallback store_callback;
     EXPECT_CALL(session_manager_client_,
-                StorePolicyForUser(PolicyBuilder::kFakeUsername,
-                                   policy_.GetBlob(), _))
+                StorePolicyForUser(cryptohome_id_, policy_.GetBlob(), _))
         .WillOnce(SaveArg<2>(&store_callback));
     store_->Store(policy_.policy());
-    RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
     Mock::VerifyAndClearExpectations(&session_manager_client_);
     ASSERT_FALSE(store_callback.is_null());
 
@@ -153,10 +157,10 @@ class UserCloudPolicyStoreChromeOSTest : public testing::Test {
     PolicyMap previous_policy;
     EXPECT_EQ(previous_value != NULL, store_->policy() != NULL);
     if (previous_value) {
-      previous_policy.Set(key::kHomepageLocation,
-                          POLICY_LEVEL_MANDATORY,
-                          POLICY_SCOPE_USER,
-                          new base::StringValue(previous_value), NULL);
+      previous_policy.Set(key::kHomepageLocation, POLICY_LEVEL_MANDATORY,
+                          POLICY_SCOPE_USER, POLICY_SOURCE_CLOUD,
+                          base::MakeUnique<base::StringValue>(previous_value),
+                          nullptr);
     }
     EXPECT_TRUE(previous_policy.Equals(store_->policy_map()));
     EXPECT_EQ(CloudPolicyStore::STATUS_OK, store_->status());
@@ -169,10 +173,10 @@ class UserCloudPolicyStoreChromeOSTest : public testing::Test {
     // Let the store operation complete.
     chromeos::SessionManagerClient::RetrievePolicyCallback retrieve_callback;
     EXPECT_CALL(session_manager_client_,
-                RetrievePolicyForUser(PolicyBuilder::kFakeUsername, _))
+                RetrievePolicyForUser(cryptohome_id_, _))
         .WillOnce(SaveArg<1>(&retrieve_callback));
     store_callback.Run(true);
-    RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
     EXPECT_TRUE(previous_policy.Equals(store_->policy_map()));
     EXPECT_EQ(CloudPolicyStore::STATUS_OK, store_->status());
     Mock::VerifyAndClearExpectations(&session_manager_client_);
@@ -181,7 +185,7 @@ class UserCloudPolicyStoreChromeOSTest : public testing::Test {
     // Finish the retrieve callback.
     EXPECT_CALL(observer_, OnStoreLoaded(store_.get()));
     retrieve_callback.Run(policy_.GetBlob());
-    RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
     ASSERT_TRUE(store_->policy());
     EXPECT_EQ(policy_.policy_data().SerializeAsString(),
               store_->policy()->SerializeAsString());
@@ -195,13 +199,8 @@ class UserCloudPolicyStoreChromeOSTest : public testing::Test {
     EXPECT_EQ(CloudPolicyStore::STATUS_VALIDATION_ERROR, store_->status());
   }
 
-  void RunUntilIdle() {
-    loop_.RunUntilIdle();
-    loop_.RunUntilIdle();
-  }
-
   base::FilePath user_policy_dir() {
-    return tmp_dir_.path().AppendASCII("var_run_user_policy");
+    return tmp_dir_.GetPath().AppendASCII("var_run_user_policy");
   }
 
   base::FilePath user_policy_key_file() {
@@ -210,11 +209,11 @@ class UserCloudPolicyStoreChromeOSTest : public testing::Test {
   }
 
   base::FilePath token_file() {
-    return tmp_dir_.path().AppendASCII("token");
+    return tmp_dir_.GetPath().AppendASCII("token");
   }
 
   base::FilePath policy_file() {
-    return tmp_dir_.path().AppendASCII("policy");
+    return tmp_dir_.GetPath().AppendASCII("policy");
   }
 
   base::MessageLoopForUI loop_;
@@ -222,7 +221,11 @@ class UserCloudPolicyStoreChromeOSTest : public testing::Test {
   chromeos::MockSessionManagerClient session_manager_client_;
   UserPolicyBuilder policy_;
   MockCloudPolicyStoreObserver observer_;
-  scoped_ptr<UserCloudPolicyStoreChromeOS> store_;
+  std::unique_ptr<UserCloudPolicyStoreChromeOS> store_;
+  const AccountId account_id_ =
+      AccountId::FromUserEmail(PolicyBuilder::kFakeUsername);
+  const cryptohome::Identification cryptohome_id_ =
+      cryptohome::Identification(account_id_);
 
  private:
   base::ScopedTempDir tmp_dir_;
@@ -236,7 +239,7 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, InitialStore) {
   // Make the policy blob contain a new public key.
   policy_.SetDefaultNewSigningKey();
   policy_.Build();
-  std::vector<uint8> new_public_key;
+  std::vector<uint8_t> new_public_key;
   ASSERT_TRUE(policy_.GetNewSigningKey()->ExportPublicKey(&new_public_key));
   ASSERT_NO_FATAL_FAILURE(
       PerformStorePolicy(&new_public_key, NULL, kDefaultHomepage));
@@ -248,13 +251,14 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, InitialStoreValidationFail) {
   // Make the policy blob contain a new public key.
   policy_.SetDefaultSigningKey();
   policy_.Build();
-  *policy_.policy().mutable_new_public_key_verification_signature() = "garbage";
+  *policy_.policy().mutable_new_public_key_verification_signature_deprecated() =
+      "garbage";
 
   EXPECT_CALL(session_manager_client_,
-              StorePolicyForUser(
-                  PolicyBuilder::kFakeUsername, policy_.GetBlob(), _)).Times(0);
+              StorePolicyForUser(cryptohome_id_, policy_.GetBlob(), _))
+      .Times(0);
   store_->Store(policy_.policy());
-  RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClearExpectations(&session_manager_client_);
 }
 
@@ -264,13 +268,13 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, InitialStoreMissingSignatureFailure) {
   // Make the policy blob contain a new public key.
   policy_.SetDefaultSigningKey();
   policy_.Build();
-  policy_.policy().clear_new_public_key_verification_signature();
+  policy_.policy().clear_new_public_key_verification_signature_deprecated();
 
   EXPECT_CALL(session_manager_client_,
-              StorePolicyForUser(
-                  PolicyBuilder::kFakeUsername, policy_.GetBlob(), _)).Times(0);
+              StorePolicyForUser(cryptohome_id_, policy_.GetBlob(), _))
+      .Times(0);
   store_->Store(policy_.policy());
-  RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClearExpectations(&session_manager_client_);
 }
 
@@ -283,7 +287,7 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, StoreWithRotation) {
   // Make the policy blob contain a new public key.
   policy_.SetDefaultNewSigningKey();
   policy_.Build();
-  std::vector<uint8> new_public_key;
+  std::vector<uint8_t> new_public_key;
   ASSERT_TRUE(policy_.GetNewSigningKey()->ExportPublicKey(&new_public_key));
   ASSERT_NO_FATAL_FAILURE(
       PerformStorePolicy(&new_public_key, NULL, kDefaultHomepage));
@@ -294,13 +298,13 @@ TEST_F(UserCloudPolicyStoreChromeOSTest,
   // Make the policy blob contain a new public key.
   policy_.SetDefaultNewSigningKey();
   policy_.Build();
-  policy_.policy().clear_new_public_key_verification_signature();
+  policy_.policy().clear_new_public_key_verification_signature_deprecated();
 
   EXPECT_CALL(session_manager_client_,
-              StorePolicyForUser(
-                  PolicyBuilder::kFakeUsername, policy_.GetBlob(), _)).Times(0);
+              StorePolicyForUser(cryptohome_id_, policy_.GetBlob(), _))
+      .Times(0);
   store_->Store(policy_.policy());
-  RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClearExpectations(&session_manager_client_);
 }
 
@@ -308,13 +312,14 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, StoreWithRotationValidationError) {
   // Make the policy blob contain a new public key.
   policy_.SetDefaultNewSigningKey();
   policy_.Build();
-  *policy_.policy().mutable_new_public_key_verification_signature() = "garbage";
+  *policy_.policy().mutable_new_public_key_verification_signature_deprecated() =
+      "garbage";
 
   EXPECT_CALL(session_manager_client_,
-              StorePolicyForUser(
-                  PolicyBuilder::kFakeUsername, policy_.GetBlob(), _)).Times(0);
+              StorePolicyForUser(cryptohome_id_, policy_.GetBlob(), _))
+      .Times(0);
   store_->Store(policy_.policy());
-  RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClearExpectations(&session_manager_client_);
 }
 
@@ -322,18 +327,17 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, StoreFail) {
   // Store policy.
   chromeos::SessionManagerClient::StorePolicyCallback store_callback;
   EXPECT_CALL(session_manager_client_,
-              StorePolicyForUser(PolicyBuilder::kFakeUsername,
-                                 policy_.GetBlob(), _))
+              StorePolicyForUser(cryptohome_id_, policy_.GetBlob(), _))
       .WillOnce(SaveArg<2>(&store_callback));
   store_->Store(policy_.policy());
-  RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClearExpectations(&session_manager_client_);
   ASSERT_FALSE(store_callback.is_null());
 
   // Let the store operation complete.
   ExpectError(CloudPolicyStore::STATUS_STORE_ERROR);
   store_callback.Run(false);
-  RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_FALSE(store_->policy());
   EXPECT_TRUE(store_->policy_map().empty());
   EXPECT_EQ(CloudPolicyStore::STATUS_STORE_ERROR, store_->status());
@@ -347,19 +351,17 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, StoreValidationError) {
   chromeos::SessionManagerClient::StorePolicyCallback store_callback;
   ExpectError(CloudPolicyStore::STATUS_VALIDATION_ERROR);
   EXPECT_CALL(session_manager_client_,
-              StorePolicyForUser(PolicyBuilder::kFakeUsername,
-                                 policy_.GetBlob(), _))
+              StorePolicyForUser(cryptohome_id_, policy_.GetBlob(), _))
       .Times(0);
   store_->Store(policy_.policy());
-  RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClearExpectations(&session_manager_client_);
 }
 
 TEST_F(UserCloudPolicyStoreChromeOSTest, StoreWithoutPolicyKey) {
   // Make the dbus call to cryptohome fail.
   Mock::VerifyAndClearExpectations(&cryptohome_client_);
-  EXPECT_CALL(cryptohome_client_,
-              GetSanitizedUsername(PolicyBuilder::kFakeUsername, _))
+  EXPECT_CALL(cryptohome_client_, GetSanitizedUsername(cryptohome_id_, _))
       .Times(AnyNumber())
       .WillRepeatedly(SendSanitizedUsername(chromeos::DBUS_METHOD_CALL_FAILURE,
                                             std::string()));
@@ -368,11 +370,10 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, StoreWithoutPolicyKey) {
   chromeos::SessionManagerClient::StorePolicyCallback store_callback;
   ExpectError(CloudPolicyStore::STATUS_VALIDATION_ERROR);
   EXPECT_CALL(session_manager_client_,
-              StorePolicyForUser(PolicyBuilder::kFakeUsername,
-                                 policy_.GetBlob(), _))
+              StorePolicyForUser(cryptohome_id_, policy_.GetBlob(), _))
       .Times(0);
   store_->Store(policy_.policy());
-  RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClearExpectations(&session_manager_client_);
 }
 
@@ -384,11 +385,10 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, StoreWithInvalidSignature) {
   chromeos::SessionManagerClient::StorePolicyCallback store_callback;
   ExpectError(CloudPolicyStore::STATUS_VALIDATION_ERROR);
   EXPECT_CALL(session_manager_client_,
-              StorePolicyForUser(PolicyBuilder::kFakeUsername,
-                                 policy_.GetBlob(), _))
+              StorePolicyForUser(cryptohome_id_, policy_.GetBlob(), _))
       .Times(0);
   store_->Store(policy_.policy());
-  RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClearExpectations(&session_manager_client_);
 }
 
@@ -563,7 +563,7 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, MigrationAndStoreNew) {
   policy_.payload().mutable_homepagelocation()->set_value(kNewHomepage);
   policy_.SetDefaultNewSigningKey();
   policy_.Build();
-  std::vector<uint8> new_public_key;
+  std::vector<uint8_t> new_public_key;
   ASSERT_TRUE(policy_.GetNewSigningKey()->ExportPublicKey(&new_public_key));
   ASSERT_NO_FATAL_FAILURE(
       PerformStorePolicy(&new_public_key, kDefaultHomepage, kNewHomepage));
@@ -576,10 +576,9 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, MigrationAndStoreNew) {
 TEST_F(UserCloudPolicyStoreChromeOSTest, LoadImmediately) {
   EXPECT_CALL(observer_, OnStoreLoaded(store_.get()));
   EXPECT_CALL(session_manager_client_,
-              BlockingRetrievePolicyForUser(PolicyBuilder::kFakeUsername))
+              BlockingRetrievePolicyForUser(cryptohome_id_))
       .WillOnce(Return(policy_.GetBlob()));
-  EXPECT_CALL(cryptohome_client_,
-              BlockingGetSanitizedUsername(PolicyBuilder::kFakeUsername))
+  EXPECT_CALL(cryptohome_client_, BlockingGetSanitizedUsername(cryptohome_id_))
       .WillOnce(Return(kSanitizedUsername));
 
   EXPECT_FALSE(store_->policy());
@@ -602,7 +601,7 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, LoadImmediately) {
 TEST_F(UserCloudPolicyStoreChromeOSTest, LoadImmediatelyNoPolicy) {
   EXPECT_CALL(observer_, OnStoreLoaded(store_.get()));
   EXPECT_CALL(session_manager_client_,
-              BlockingRetrievePolicyForUser(PolicyBuilder::kFakeUsername))
+              BlockingRetrievePolicyForUser(cryptohome_id_))
       .WillOnce(Return(""));
 
   EXPECT_FALSE(store_->policy());
@@ -618,7 +617,7 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, LoadImmediatelyNoPolicy) {
 TEST_F(UserCloudPolicyStoreChromeOSTest, LoadImmediatelyInvalidBlob) {
   EXPECT_CALL(observer_, OnStoreError(store_.get()));
   EXPECT_CALL(session_manager_client_,
-              BlockingRetrievePolicyForUser(PolicyBuilder::kFakeUsername))
+              BlockingRetrievePolicyForUser(cryptohome_id_))
       .WillOnce(Return("le blob"));
 
   EXPECT_FALSE(store_->policy());
@@ -634,10 +633,9 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, LoadImmediatelyInvalidBlob) {
 TEST_F(UserCloudPolicyStoreChromeOSTest, LoadImmediatelyDBusFailure) {
   EXPECT_CALL(observer_, OnStoreError(store_.get()));
   EXPECT_CALL(session_manager_client_,
-              BlockingRetrievePolicyForUser(PolicyBuilder::kFakeUsername))
+              BlockingRetrievePolicyForUser(cryptohome_id_))
       .WillOnce(Return(policy_.GetBlob()));
-  EXPECT_CALL(cryptohome_client_,
-              BlockingGetSanitizedUsername(PolicyBuilder::kFakeUsername))
+  EXPECT_CALL(cryptohome_client_, BlockingGetSanitizedUsername(cryptohome_id_))
       .WillOnce(Return(""));
 
   EXPECT_FALSE(store_->policy());
@@ -654,10 +652,9 @@ TEST_F(UserCloudPolicyStoreChromeOSTest, LoadImmediatelyDBusFailure) {
 TEST_F(UserCloudPolicyStoreChromeOSTest, LoadImmediatelyNoUserPolicyKey) {
   EXPECT_CALL(observer_, OnStoreError(store_.get()));
   EXPECT_CALL(session_manager_client_,
-              BlockingRetrievePolicyForUser(PolicyBuilder::kFakeUsername))
+              BlockingRetrievePolicyForUser(cryptohome_id_))
       .WillOnce(Return(policy_.GetBlob()));
-  EXPECT_CALL(cryptohome_client_,
-              BlockingGetSanitizedUsername(PolicyBuilder::kFakeUsername))
+  EXPECT_CALL(cryptohome_client_, BlockingGetSanitizedUsername(cryptohome_id_))
       .WillOnce(Return("wrong@example.com"));
 
   EXPECT_FALSE(store_->policy());
