@@ -5,30 +5,34 @@
 #include "chrome/browser/chromeos/power/power_prefs.h"
 
 #include <string>
+#include <utility>
+#include <vector>
 
 #include "base/command_line.h"
 #include "base/files/file_path.h"
+#include "base/macros.h"
 #include "base/memory/ref_counted.h"
-#include "base/prefs/pref_service.h"
 #include "chrome/browser/chrome_notification_types.h"
 #include "chrome/browser/chromeos/login/users/fake_chrome_user_manager.h"
 #include "chrome/browser/chromeos/login/users/scoped_user_manager_enabler.h"
 #include "chrome/browser/chromeos/profiles/profile_helper.h"
 #include "chrome/browser/extensions/extension_special_storage_policy.h"
 #include "chrome/browser/prefs/browser_prefs.h"
-#include "chrome/browser/prefs/pref_service_syncable.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/profiles/profile_manager.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/test/base/testing_browser_process.h"
-#include "chrome/test/base/testing_pref_service_syncable.h"
 #include "chrome/test/base/testing_profile.h"
 #include "chrome/test/base/testing_profile_manager.h"
 #include "chromeos/dbus/fake_power_manager_client.h"
 #include "chromeos/dbus/power_manager/policy.pb.h"
 #include "chromeos/dbus/power_policy_controller.h"
 #include "components/pref_registry/pref_registry_syncable.h"
+#include "components/prefs/pref_service.h"
+#include "components/signin/core/account_id/account_id.h"
+#include "components/sync_preferences/pref_service_syncable.h"
+#include "components/sync_preferences/testing_pref_service_syncable.h"
 #include "content/public/browser/notification_details.h"
 #include "content/public/browser/notification_service.h"
 #include "content/public/browser/notification_source.h"
@@ -64,9 +68,9 @@ class PowerPrefsTest : public testing::Test {
   content::TestBrowserThreadBundle thread_bundle_;
   TestingProfileManager profile_manager_;
   PowerPolicyController* power_policy_controller_;     // Not owned.
-  scoped_ptr<FakePowerManagerClient> fake_power_manager_client_;
+  std::unique_ptr<FakePowerManagerClient> fake_power_manager_client_;
 
-  scoped_ptr<PowerPrefs> power_prefs_;
+  std::unique_ptr<PowerPrefs> power_prefs_;
 
   DISALLOW_COPY_AND_ASSIGN(PowerPrefsTest);
 };
@@ -172,13 +176,13 @@ bool PowerPrefsTest::GetExpectedAllowScreenWakeLocksForProfile(
 
 TEST_F(PowerPrefsTest, LoginScreen) {
   // Set up login profile.
-  scoped_ptr<TestingPrefServiceSyncable> login_profile_prefs(
-      new TestingPrefServiceSyncable);
+  std::unique_ptr<sync_preferences::TestingPrefServiceSyncable>
+      login_profile_prefs(new sync_preferences::TestingPrefServiceSyncable);
   chrome::RegisterLoginProfilePrefs(login_profile_prefs->registry());
   TestingProfile::Builder builder;
   builder.SetPath(
       profile_manager_.profiles_dir().AppendASCII(chrome::kInitialProfile));
-  builder.SetPrefService(login_profile_prefs.Pass());
+  builder.SetPrefService(std::move(login_profile_prefs));
   TestingProfile* login_profile = builder.BuildIncognito(
       profile_manager_.CreateTestingProfile(chrome::kInitialProfile));
 
@@ -235,46 +239,77 @@ TEST_F(PowerPrefsTest, LoginScreen) {
             GetCurrentPowerPolicy());
 }
 
-TEST_F(PowerPrefsTest, UserSession) {
-  FakeChromeUserManager* user_manager = new FakeChromeUserManager();
-  ScopedUserManagerEnabler user_manager_enabler(user_manager);
+class PowerPrefsUserSessionTest : public PowerPrefsTest {
+ public:
+  PowerPrefsUserSessionTest();
+
+ protected:
+  // PowerPrefsTest:
+  void SetUp() override;
+
+  std::vector<power_manager::PowerManagementPolicy_Action>
+  GetCurrentPowerPolicyActions() const {
+    return {fake_power_manager_client_->policy().ac_idle_action(),
+            fake_power_manager_client_->policy().battery_idle_action(),
+            fake_power_manager_client_->policy().lid_closed_action()};
+  }
+
+  TestingProfile* user_profile_ = nullptr;
+  TestingProfile* second_user_profile_ = nullptr;
+
+ private:
+  FakeChromeUserManager* user_manager_;
+  ScopedUserManagerEnabler user_manager_enabler_;
+
+  DISALLOW_COPY_AND_ASSIGN(PowerPrefsUserSessionTest);
+};
+
+PowerPrefsUserSessionTest::PowerPrefsUserSessionTest()
+    : user_manager_(new FakeChromeUserManager),
+      user_manager_enabler_(user_manager_) {}
+
+void PowerPrefsUserSessionTest::SetUp() {
+  PowerPrefsTest::SetUp();
 
-  // Set up user profile.
   const char test_user1[] = "test-user1@example.com";
-  user_manager->AddUser(test_user1);
-  user_manager->LoginUser(test_user1);
-  TestingProfile* user_profile =
-      profile_manager_.CreateTestingProfile(test_user1);
+  const AccountId test_account_id1(AccountId::FromUserEmail(test_user1));
+  user_manager_->AddUser(test_account_id1);
+  user_manager_->LoginUser(test_account_id1);
+  user_profile_ =
+      profile_manager_.CreateTestingProfile(test_account_id1.GetUserEmail());
+
+  const char test_user2[] = "test-user2@example.com";
+  const AccountId test_account_id2(AccountId::FromUserEmail(test_user2));
+  user_manager_->AddUser(test_account_id2);
+  user_manager_->LoginUser(test_account_id2);
+  second_user_profile_ =
+      profile_manager_.CreateTestingProfile(test_account_id2.GetUserEmail());
 
   profile_manager_.SetLoggedIn(true);
+}
 
+TEST_F(PowerPrefsUserSessionTest, Basic) {
   // Inform power_prefs_ that a session has started.
   power_prefs_->Observe(chrome::NOTIFICATION_SESSION_STARTED,
                         content::Source<PowerPrefsTest>(this),
                         content::NotificationService::NoDetails());
 
-  EXPECT_EQ(user_profile, GetProfile());
-  EXPECT_EQ(GetExpectedPowerPolicyForProfile(user_profile, UNLOCKED),
+  EXPECT_EQ(user_profile_, GetProfile());
+  EXPECT_EQ(GetExpectedPowerPolicyForProfile(user_profile_, UNLOCKED),
             GetCurrentPowerPolicy());
-  EXPECT_EQ(GetExpectedAllowScreenWakeLocksForProfile(user_profile),
+  EXPECT_EQ(GetExpectedAllowScreenWakeLocksForProfile(user_profile_),
             GetCurrentAllowScreenWakeLocks());
 
-  const char test_user2[] = "test-user2@example.com";
-  user_manager->AddUser(test_user2);
-  user_manager->LoginUser(test_user2);
-  TestingProfile* other_profile =
-      profile_manager_.CreateTestingProfile(test_user2);
-
   // Inform power_prefs_ that an unrelated profile has been destroyed.
   power_prefs_->Observe(chrome::NOTIFICATION_PROFILE_DESTROYED,
-                        content::Source<Profile>(other_profile),
+                        content::Source<Profile>(second_user_profile_),
                         content::NotificationService::NoDetails());
 
   // Verify that the user profile's power prefs are still being used.
-  EXPECT_EQ(user_profile, GetProfile());
-  EXPECT_EQ(GetExpectedPowerPolicyForProfile(user_profile, UNLOCKED),
+  EXPECT_EQ(user_profile_, GetProfile());
+  EXPECT_EQ(GetExpectedPowerPolicyForProfile(user_profile_, UNLOCKED),
             GetCurrentPowerPolicy());
-  EXPECT_EQ(GetExpectedAllowScreenWakeLocksForProfile(user_profile),
+  EXPECT_EQ(GetExpectedAllowScreenWakeLocksForProfile(user_profile_),
             GetCurrentAllowScreenWakeLocks());
 
   // Simulate the login screen coming up as part of screen locking.
@@ -283,22 +318,58 @@ TEST_F(PowerPrefsTest, UserSession) {
                         content::NotificationService::NoDetails());
 
   // Verify that power policy didn't revert to login screen settings.
-  EXPECT_EQ(user_profile, GetProfile());
-  EXPECT_EQ(GetExpectedPowerPolicyForProfile(user_profile, UNLOCKED),
+  EXPECT_EQ(user_profile_, GetProfile());
+  EXPECT_EQ(GetExpectedPowerPolicyForProfile(user_profile_, UNLOCKED),
             GetCurrentPowerPolicy());
-  EXPECT_EQ(GetExpectedAllowScreenWakeLocksForProfile(user_profile),
+  EXPECT_EQ(GetExpectedAllowScreenWakeLocksForProfile(user_profile_),
             GetCurrentAllowScreenWakeLocks());
 
   // Inform power_prefs_ that the session has ended and the user profile has
   // been destroyed.
   power_prefs_->Observe(chrome::NOTIFICATION_PROFILE_DESTROYED,
-                        content::Source<Profile>(user_profile),
+                        content::Source<Profile>(user_profile_),
                         content::NotificationService::NoDetails());
 
   // The user profile's prefs should still be used.
   EXPECT_FALSE(GetProfile());
-  EXPECT_EQ(GetExpectedPowerPolicyForProfile(user_profile, UNLOCKED),
+  EXPECT_EQ(GetExpectedPowerPolicyForProfile(user_profile_, UNLOCKED),
             GetCurrentPowerPolicy());
 }
 
+TEST_F(PowerPrefsUserSessionTest, DisabledLockScreen) {
+  // Inform power_prefs_ that a session has started.
+  power_prefs_->Observe(chrome::NOTIFICATION_SESSION_STARTED,
+                        content::Source<PowerPrefsTest>(this),
+                        content::NotificationService::NoDetails());
+
+  EXPECT_EQ(user_profile_, GetProfile());
+
+  // Verify that the power policy actions are set to default values initially.
+  EXPECT_EQ(std::vector<power_manager::PowerManagementPolicy_Action>(
+                3, power_manager::PowerManagementPolicy_Action_SUSPEND),
+            GetCurrentPowerPolicyActions());
+
+  // The automatic screen locking is enabled, but, as the lock screen is
+  // allowed, the power policy actions still have the default values.
+  user_profile_->GetPrefs()->SetBoolean(prefs::kEnableAutoScreenLock, true);
+  EXPECT_EQ(std::vector<power_manager::PowerManagementPolicy_Action>(
+                3, power_manager::PowerManagementPolicy_Action_SUSPEND),
+            GetCurrentPowerPolicyActions());
+
+  // The lock screen is disabled, but, as automatic screen locking is not
+  // enabled, the power policy actions still have the default values.
+  user_profile_->GetPrefs()->ClearPref(prefs::kEnableAutoScreenLock);
+  user_profile_->GetPrefs()->SetBoolean(prefs::kAllowScreenLock, false);
+  EXPECT_EQ(std::vector<power_manager::PowerManagementPolicy_Action>(
+                3, power_manager::PowerManagementPolicy_Action_SUSPEND),
+            GetCurrentPowerPolicyActions());
+
+  // The automatic screen locking is enabled and the lock screen is disabled, so
+  // the power policy actions are set now to stop the user session.
+  user_profile_->GetPrefs()->SetBoolean(prefs::kEnableAutoScreenLock, true);
+  EXPECT_EQ(std::vector<power_manager::PowerManagementPolicy_Action>(
+                3, power_manager::PowerManagementPolicy_Action_STOP_SESSION),
+            GetCurrentPowerPolicyActions());
+}
+
 }  // namespace chromeos
