@@ -6,10 +6,10 @@
 
 #include <stdint.h>
 
+#include <memory>
 #include <utility>
 
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/run_loop.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
@@ -23,7 +23,7 @@
 #include "chrome/test/base/testing_profile.h"
 #include "components/user_manager/fake_user_manager.h"
 #include "content/public/test/test_browser_thread_bundle.h"
-#include "device/core/device_client.h"
+#include "device/base/mock_device_client.h"
 #include "device/usb/mock_usb_device.h"
 #include "device/usb/mock_usb_service.h"
 #include "device/usb/usb_descriptors.h"
@@ -50,29 +50,9 @@ const char kPrinterAppExistsDelegateIDTemplate[] =
 const char kPrinterAppNotFoundDelegateIDTemplate[] =
     "system.printer.no_printer_provider_found/%s:%s";
 
-class FakeDeviceClient : public device::DeviceClient {
- public:
-  FakeDeviceClient() : usb_service_(nullptr) {}
-
-  ~FakeDeviceClient() override {}
-
-  // device::DeviceClient implementation:
-  device::UsbService* GetUsbService() override {
-    EXPECT_TRUE(usb_service_);
-    return usb_service_;
-  }
-
-  void set_usb_service(device::UsbService* service) { usb_service_ = service; }
-
- private:
-  device::UsbService* usb_service_;
-
-  DISALLOW_COPY_AND_ASSIGN(FakeDeviceClient);
-};
-
-scoped_ptr<KeyedService> CreatePrinterDetector(
+std::unique_ptr<KeyedService> CreatePrinterDetector(
     content::BrowserContext* context) {
-  return scoped_ptr<KeyedService>(
+  return std::unique_ptr<KeyedService>(
       new chromeos::PrinterDetector(Profile::FromBrowserContext(context)));
 }
 
@@ -88,7 +68,7 @@ class PrinterDetectorAppSearchEnabledTest : public testing::Test {
   ~PrinterDetectorAppSearchEnabledTest() override = default;
 
   void SetUp() override {
-    device_client_.set_usb_service(&usb_service_);
+    device_client_.GetUsbService();
     // Make sure the profile is created after adding the switch and setting up
     // device client.
     profile_.reset(new TestingProfile());
@@ -121,34 +101,35 @@ class PrinterDetectorAppSearchEnabledTest : public testing::Test {
   void InvokeUsbAdded(uint16_t vendor_id,
                       uint16_t product_id,
                       uint8_t interface_class) {
-    device::UsbInterfaceDescriptor interface;
-    interface.interface_number = 1;
-    interface.interface_class = interface_class;
-    device::UsbConfigDescriptor config;
-    config.interfaces.push_back(interface);
-    usb_service_.AddDevice(
+    device::UsbConfigDescriptor config(1, false, false, 0);
+    config.interfaces.emplace_back(1, 0, interface_class, 0, 0);
+    device_client_.usb_service()->AddDevice(
         new device::MockUsbDevice(vendor_id, product_id, config));
   }
 
   // Creates a test extension with the provided permissions.
   scoped_refptr<extensions::Extension> CreateTestExtension(
-      ListBuilder permissions_builder,
-      DictionaryBuilder usb_printers_builder) {
+      std::unique_ptr<base::ListValue> permissions_builder,
+      std::unique_ptr<base::DictionaryValue> usb_printers_builder) {
     return extensions::ExtensionBuilder()
         .SetID("fake_extension_id")
-        .SetManifest(std::move(
+        .SetManifest(
             DictionaryBuilder()
                 .Set("name", "Printer provider extension")
                 .Set("manifest_version", 2)
                 .Set("version", "1.0")
                 // Needed to enable usb API.
-                .Set("app", std::move(DictionaryBuilder().Set(
-                                "background",
-                                std::move(DictionaryBuilder().Set(
-                                    "scripts", std::move(ListBuilder().Append(
-                                                   "bg.js")))))))
+                .Set("app",
+                     DictionaryBuilder()
+                         .Set("background",
+                              DictionaryBuilder()
+                                  .Set("scripts",
+                                       ListBuilder().Append("bg.js").Build())
+                                  .Build())
+                         .Build())
                 .Set("permissions", std::move(permissions_builder))
-                .Set("usb_printers", std::move(usb_printers_builder))))
+                .Set("usb_printers", std::move(usb_printers_builder))
+                .Build())
         .Build();
   }
 
@@ -156,9 +137,8 @@ class PrinterDetectorAppSearchEnabledTest : public testing::Test {
   StubNotificationUIManager notification_ui_manager_;
   user_manager::FakeUserManager* user_manager_;
   chromeos::ScopedUserManagerEnabler user_manager_enabler_;
-  device::MockUsbService usb_service_;
-  scoped_ptr<TestingProfile> profile_;
-  FakeDeviceClient device_client_;
+  device::MockDeviceClient device_client_;
+  std::unique_ptr<TestingProfile> profile_;
 
   DISALLOW_COPY_AND_ASSIGN(PrinterDetectorAppSearchEnabledTest);
 };
@@ -177,15 +157,19 @@ TEST_F(PrinterDetectorAppSearchEnabledTest, ShowFindAppNotification) {
 
 TEST_F(PrinterDetectorAppSearchEnabledTest, ShowAppFoundNotification) {
   scoped_refptr<extensions::Extension> extension = CreateTestExtension(
-      std::move(ListBuilder()
-                    .Append("usb")
-                    .Append("printerProvider")
-                    .Append(std::move(DictionaryBuilder().Set(
-                        "usbDevices", std::move(ListBuilder().Append(std::move(
-                                          DictionaryBuilder()
-                                              .Set("vendorId", 123)
-                                              .Set("productId", 456)))))))),
-      std::move(DictionaryBuilder().Set("filters", ListBuilder())));
+      ListBuilder()
+          .Append("usb")
+          .Append("printerProvider")
+          .Append(DictionaryBuilder()
+                      .Set("usbDevices", ListBuilder()
+                                             .Append(DictionaryBuilder()
+                                                         .Set("vendorId", 123)
+                                                         .Set("productId", 456)
+                                                         .Build())
+                                             .Build())
+                      .Build())
+          .Build(),
+      DictionaryBuilder().Set("filters", ListBuilder().Build()).Build());
   ASSERT_TRUE(extensions::ExtensionRegistry::Get(profile_.get())
                   ->AddEnabled(extension));
 
@@ -203,13 +187,18 @@ TEST_F(PrinterDetectorAppSearchEnabledTest, ShowAppFoundNotification) {
 TEST_F(PrinterDetectorAppSearchEnabledTest,
        UsbHandlerExists_NotPrinterProvider) {
   scoped_refptr<extensions::Extension> extension = CreateTestExtension(
-      std::move(
-          ListBuilder().Append("usb").Append(std::move(DictionaryBuilder().Set(
-              "usbDevices", std::move(ListBuilder().Append(
-                                std::move(DictionaryBuilder()
-                                              .Set("vendorId", 123)
-                                              .Set("productId", 756)))))))),
-      std::move(DictionaryBuilder().Set("filters", ListBuilder())));
+      ListBuilder()
+          .Append("usb")
+          .Append(DictionaryBuilder()
+                      .Set("usbDevices", ListBuilder()
+                                             .Append(DictionaryBuilder()
+                                                         .Set("vendorId", 123)
+                                                         .Set("productId", 756)
+                                                         .Build())
+                                             .Build())
+                      .Build())
+          .Build(),
+      DictionaryBuilder().Set("filters", ListBuilder().Build()).Build());
   ASSERT_TRUE(extensions::ExtensionRegistry::Get(profile_.get())
                   ->AddEnabled(extension));
 
@@ -227,15 +216,19 @@ TEST_F(PrinterDetectorAppSearchEnabledTest,
 TEST_F(PrinterDetectorAppSearchEnabledTest,
        PrinterProvider_DifferentUsbProductId) {
   scoped_refptr<extensions::Extension> extension = CreateTestExtension(
-      std::move(ListBuilder()
-                    .Append("usb")
-                    .Append("printerProvider")
-                    .Append(std::move(DictionaryBuilder().Set(
-                        "usbDevices", std::move(ListBuilder().Append(std::move(
-                                          DictionaryBuilder()
-                                              .Set("vendorId", 123)
-                                              .Set("productId", 001)))))))),
-      std::move(DictionaryBuilder().Set("filters", ListBuilder())));
+      ListBuilder()
+          .Append("usb")
+          .Append("printerProvider")
+          .Append(DictionaryBuilder()
+                      .Set("usbDevices", ListBuilder()
+                                             .Append(DictionaryBuilder()
+                                                         .Set("vendorId", 123)
+                                                         .Set("productId", 001)
+                                                         .Build())
+                                             .Build())
+                      .Build())
+          .Build(),
+      DictionaryBuilder().Set("filters", ListBuilder().Build()).Build());
   ASSERT_TRUE(extensions::ExtensionRegistry::Get(profile_.get())
                   ->AddEnabled(extension));
 
@@ -253,12 +246,15 @@ TEST_F(PrinterDetectorAppSearchEnabledTest,
 TEST_F(PrinterDetectorAppSearchEnabledTest,
        PrinterProvider_UsbPrinters_NotFound) {
   scoped_refptr<extensions::Extension> extension = CreateTestExtension(
-      std::move(ListBuilder().Append("usb").Append("printerProvider")),
-      std::move(DictionaryBuilder().Set(
-          "filters", std::move(ListBuilder().Append(
-                         std::move(DictionaryBuilder()
-                                       .Set("vendorId", 123)
-                                       .Set("productId", 001)))))));
+      ListBuilder().Append("usb").Append("printerProvider").Build(),
+      DictionaryBuilder()
+          .Set("filters", ListBuilder()
+                              .Append(DictionaryBuilder()
+                                          .Set("vendorId", 123)
+                                          .Set("productId", 001)
+                                          .Build())
+                              .Build())
+          .Build());
   ASSERT_TRUE(extensions::ExtensionRegistry::Get(profile_.get())
                   ->AddEnabled(extension));
 
@@ -276,12 +272,15 @@ TEST_F(PrinterDetectorAppSearchEnabledTest,
 TEST_F(PrinterDetectorAppSearchEnabledTest,
        PrinterProvider_UsbPrinters_WithProductId) {
   scoped_refptr<extensions::Extension> extension = CreateTestExtension(
-      std::move(ListBuilder().Append("usb").Append("printerProvider")),
-      std::move(DictionaryBuilder().Set(
-          "filters", std::move(ListBuilder().Append(
-                         std::move(DictionaryBuilder()
-                                       .Set("vendorId", 123)
-                                       .Set("productId", 456)))))));
+      ListBuilder().Append("usb").Append("printerProvider").Build(),
+      DictionaryBuilder()
+          .Set("filters", ListBuilder()
+                              .Append(DictionaryBuilder()
+                                          .Set("vendorId", 123)
+                                          .Set("productId", 456)
+                                          .Build())
+                              .Build())
+          .Build());
   ASSERT_TRUE(extensions::ExtensionRegistry::Get(profile_.get())
                   ->AddEnabled(extension));
 
@@ -299,13 +298,16 @@ TEST_F(PrinterDetectorAppSearchEnabledTest,
 TEST_F(PrinterDetectorAppSearchEnabledTest,
        PrinterProvider_UsbPrinters_WithInterfaceClass) {
   scoped_refptr<extensions::Extension> extension = CreateTestExtension(
-      std::move(ListBuilder().Append("usb").Append("printerProvider")),
-      std::move(DictionaryBuilder().Set(
-          "filters",
-          std::move(ListBuilder().Append(std::move(
-              DictionaryBuilder()
-                  .Set("vendorId", 123)
-                  .Set("interfaceClass", kPrinterInterfaceClass)))))));
+      ListBuilder().Append("usb").Append("printerProvider").Build(),
+      DictionaryBuilder()
+          .Set("filters",
+               ListBuilder()
+                   .Append(DictionaryBuilder()
+                               .Set("vendorId", 123)
+                               .Set("interfaceClass", kPrinterInterfaceClass)
+                               .Build())
+                   .Build())
+          .Build());
   ASSERT_TRUE(extensions::ExtensionRegistry::Get(profile_.get())
                   ->AddEnabled(extension));
 
@@ -322,13 +324,16 @@ TEST_F(PrinterDetectorAppSearchEnabledTest,
 
 TEST_F(PrinterDetectorAppSearchEnabledTest, IgnoreNonPrinters) {
   scoped_refptr<extensions::Extension> extension = CreateTestExtension(
-      std::move(ListBuilder().Append("usb").Append("printerProvider")),
-      std::move(DictionaryBuilder().Set(
-          "filters",
-          std::move(ListBuilder().Append(std::move(
-              DictionaryBuilder()
-                  .Set("vendorId", 123)
-                  .Set("interfaceClass", kPrinterInterfaceClass)))))));
+      ListBuilder().Append("usb").Append("printerProvider").Build(),
+      DictionaryBuilder()
+          .Set("filters",
+               ListBuilder()
+                   .Append(DictionaryBuilder()
+                               .Set("vendorId", 123)
+                               .Set("interfaceClass", kPrinterInterfaceClass)
+                               .Build())
+                   .Build())
+          .Build());
   ASSERT_TRUE(extensions::ExtensionRegistry::Get(profile_.get())
                   ->AddEnabled(extension));
 
