@@ -17,10 +17,10 @@
 #include "chrome/browser/net/nss_context.h"
 #include "components/ownership/owner_key_util.h"
 #include "components/policy/core/common/cloud/cloud_policy_constants.h"
+#include "components/policy/proto/device_management_backend.pb.h"
 #include "content/public/browser/browser_thread.h"
 #include "crypto/rsa_private_key.h"
 #include "crypto/signature_creator.h"
-#include "policy/proto/device_management_backend.pb.h"
 
 using ownership::OwnerKeyUtil;
 using ownership::PublicKey;
@@ -134,7 +134,8 @@ void SessionManagerOperation::RetrieveDeviceSettings() {
 
 void SessionManagerOperation::ValidateDeviceSettings(
     const std::string& policy_blob) {
-  scoped_ptr<em::PolicyFetchResponse> policy(new em::PolicyFetchResponse());
+  std::unique_ptr<em::PolicyFetchResponse> policy(
+      new em::PolicyFetchResponse());
   if (policy_blob.empty()) {
     ReportResult(DeviceSettingsService::STORE_NO_POLICY);
     return;
@@ -167,21 +168,24 @@ void SessionManagerOperation::ValidateDeviceSettings(
   // is back online, but the network configuration may come from device ONC.
   //
   // To prevent all of these issues the timestamp is just not verified when
-  // loading the device policy from the cache. Note that the timestamp is still
-  // verified during enrollment and when a new policy is fetched from the
+  // loading the device policy from session manager. Note that the timestamp is
+  // still verified during enrollment and when a new policy is fetched from the
   // server.
+  //
+  // The two *_NOT_REQUIRED options are necessary because both the DM token and
+  // the device id are empty for a user logging in on an actual Chrome OS device
+  // that is not enterprise-managed. Note for devs: The strings are not empty
+  // when you test Chrome with target_os = "chromeos" on Linux!
   validator->ValidateAgainstCurrentPolicy(
       policy_data_.get(),
-      policy::CloudPolicyValidatorBase::TIMESTAMP_NOT_REQUIRED,
-      policy::CloudPolicyValidatorBase::DM_TOKEN_NOT_REQUIRED);
+      policy::CloudPolicyValidatorBase::TIMESTAMP_NOT_VALIDATED,
+      policy::CloudPolicyValidatorBase::DM_TOKEN_NOT_REQUIRED,
+      policy::CloudPolicyValidatorBase::DEVICE_ID_NOT_REQUIRED);
   validator->ValidatePolicyType(policy::dm_protocol::kChromeDevicePolicyType);
   validator->ValidatePayload();
   // We don't check the DMServer verification key below, because the signing
   // key is validated when it is installed.
-  validator->ValidateSignature(public_key_->as_string(),
-                               std::string(),  // No key validation check.
-                               std::string(),
-                               false);
+  validator->ValidateSignature(public_key_->as_string());
   validator->StartValidation(
       base::Bind(&SessionManagerOperation::ReportValidatorStatus,
                  weak_factory_.GetWeakPtr()));
@@ -219,7 +223,7 @@ void LoadSettingsOperation::Run() {
 
 StoreSettingsOperation::StoreSettingsOperation(
     const Callback& callback,
-    scoped_ptr<em::PolicyFetchResponse> policy)
+    std::unique_ptr<em::PolicyFetchResponse> policy)
     : SessionManagerOperation(callback),
       policy_(std::move(policy)),
       weak_factory_(this) {}
