@@ -2,16 +2,17 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "ash/common/accessibility_types.h"
+#include "ash/common/login_status.h"
+#include "ash/common/material_design/material_design_controller.h"
+#include "ash/common/system/tray/system_tray.h"
+#include "ash/common/system/tray_accessibility.h"
+#include "ash/common/test/test_session_state_delegate.h"
 #include "ash/magnifier/magnification_controller.h"
 #include "ash/shell.h"
-#include "ash/system/tray/system_tray.h"
-#include "ash/system/tray_accessibility.h"
-#include "ash/system/user/login_status.h"
 #include "ash/test/shell_test_api.h"
-#include "ash/test/test_session_state_delegate.h"
 #include "base/callback.h"
 #include "base/command_line.h"
-#include "base/prefs/pref_service.h"
 #include "base/strings/utf_string_conversions.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/chrome_notification_types.h"
@@ -32,10 +33,13 @@
 #include "components/policy/core/common/mock_configuration_policy_provider.h"
 #include "components/policy/core/common/policy_map.h"
 #include "components/policy/core/common/policy_types.h"
-#include "components/user_manager/user_manager.h"
+#include "components/policy/policy_constants.h"
+#include "components/prefs/pref_service.h"
+#include "components/session_manager/core/session_manager.h"
 #include "content/public/test/test_utils.h"
-#include "policy/policy_constants.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "ui/views/controls/button/button.h"
+#include "ui/views/controls/button/custom_button.h"
 #include "ui/views/controls/label.h"
 #include "ui/views/widget/widget.h"
 
@@ -102,11 +106,9 @@ class TrayAccessibilityTest
     } else if (GetParam() == POLICY) {
       policy::PolicyMap policy_map;
       policy_map.Set(policy::key::kShowAccessibilityOptionsInSystemTrayMenu,
-                     policy::POLICY_LEVEL_MANDATORY,
-                     policy::POLICY_SCOPE_USER,
+                     policy::POLICY_LEVEL_MANDATORY, policy::POLICY_SCOPE_USER,
                      policy::POLICY_SOURCE_CLOUD,
-                     new base::FundamentalValue(value),
-                     NULL);
+                     base::MakeUnique<base::FundamentalValue>(value), nullptr);
       provider_.UpdateChromePolicy(policy_map);
       base::RunLoop().RunUntilIdle();
     } else {
@@ -141,13 +143,11 @@ class TrayAccessibilityTest
     return menu_item_view != NULL;
   }
 
-  void SetLoginStatus(ash::user::LoginStatus status) {
+  void SetLoginStatus(ash::LoginStatus status) {
     tray()->UpdateAfterLoginStatusChange(status);
   }
 
-  ash::user::LoginStatus GetLoginStatus() {
-    return tray()->login_;
-  }
+  ash::LoginStatus GetLoginStatus() { return tray()->login_; }
 
   bool CreateDetailedMenu() {
     tray()->PopupDetailedView(0, false);
@@ -238,11 +238,23 @@ class TrayAccessibilityTest
     return tray()->detailed_menu_->virtual_keyboard_view_;
   }
 
-  bool IsHelpShownOnDetailMenu() const {
+  // In material design we show the help button but theme it as disabled if
+  // it is not possible to load the the help page.
+  bool IsHelpAvailableOnDetailMenu() const {
+    if (ash::MaterialDesignController::IsSystemTrayMenuMaterial()) {
+      return tray()->detailed_menu_->help_view_->state() ==
+             views::Button::STATE_NORMAL;
+    }
     return tray()->detailed_menu_->help_view_;
   }
 
-  bool IsSettingsShownOnDetailMenu() const {
+  // In material design we show the settings button but theme it as disabled if
+  // it is not possible to load the the settings page.
+  bool IsSettingsAvailableOnDetailMenu() const {
+    if (ash::MaterialDesignController::IsSystemTrayMenuMaterial()) {
+      return tray()->detailed_menu_->settings_view_->state() ==
+             views::Button::STATE_NORMAL;
+    }
     return tray()->detailed_menu_->settings_view_;
   }
 
@@ -269,36 +281,34 @@ class TrayAccessibilityTest
 };
 
 IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, LoginStatus) {
-  EXPECT_EQ(ash::user::LOGGED_IN_NONE, GetLoginStatus());
+  EXPECT_EQ(ash::LoginStatus::NOT_LOGGED_IN, GetLoginStatus());
 
-  user_manager::UserManager::Get()->UserLoggedIn(
-      AccountId::FromUserEmail("owner@invalid.domain"), "owner@invalid.domain",
-      true);
-  user_manager::UserManager::Get()->SessionStarted();
+  session_manager::SessionManager::Get()->CreateSession(
+      AccountId::FromUserEmail("owner@invalid.domain"), "owner@invalid.domain");
+  session_manager::SessionManager::Get()->SessionStarted();
 
-  EXPECT_EQ(ash::user::LOGGED_IN_USER, GetLoginStatus());
+  EXPECT_EQ(ash::LoginStatus::USER, GetLoginStatus());
 }
 
 IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowTrayIcon) {
-  SetLoginStatus(ash::user::LOGGED_IN_NONE);
+  SetLoginStatus(ash::LoginStatus::NOT_LOGGED_IN);
 
   // Confirms that the icon is invisible before login.
   EXPECT_FALSE(IsTrayIconVisible());
 
-  user_manager::UserManager::Get()->UserLoggedIn(
-      AccountId::FromUserEmail("owner@invalid.domain"), "owner@invalid.domain",
-      true);
-  user_manager::UserManager::Get()->SessionStarted();
+  session_manager::SessionManager::Get()->CreateSession(
+      AccountId::FromUserEmail("owner@invalid.domain"), "owner@invalid.domain");
+  session_manager::SessionManager::Get()->SessionStarted();
 
   // Confirms that the icon is invisible just after login.
   EXPECT_FALSE(IsTrayIconVisible());
 
   // Toggling spoken feedback changes the visibillity of the icon.
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      true, ui::A11Y_NOTIFICATION_NONE);
+      true, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(IsTrayIconVisible());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      false, ui::A11Y_NOTIFICATION_NONE);
+      false, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_FALSE(IsTrayIconVisible());
 
   // Toggling high contrast the visibillity of the icon.
@@ -326,12 +336,12 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowTrayIcon) {
   AccessibilityManager::Get()->EnableHighContrast(true);
   EXPECT_TRUE(IsTrayIconVisible());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      true, ui::A11Y_NOTIFICATION_NONE);
+      true, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(IsTrayIconVisible());
   AccessibilityManager::Get()->EnableVirtualKeyboard(true);
   EXPECT_TRUE(IsTrayIconVisible());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      false, ui::A11Y_NOTIFICATION_NONE);
+      false, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(IsTrayIconVisible());
   AccessibilityManager::Get()->EnableHighContrast(false);
   EXPECT_TRUE(IsTrayIconVisible());
@@ -351,10 +361,9 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowTrayIcon) {
 
 IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowMenu) {
   // Login
-  user_manager::UserManager::Get()->UserLoggedIn(
-      AccountId::FromUserEmail("owner@invalid.domain"), "owner@invalid.domain",
-      true);
-  user_manager::UserManager::Get()->SessionStarted();
+  session_manager::SessionManager::Get()->CreateSession(
+      AccountId::FromUserEmail("owner@invalid.domain"), "owner@invalid.domain");
+  session_manager::SessionManager::Get()->SessionStarted();
 
   SetShowAccessibilityOptionsInSystemTrayMenu(false);
 
@@ -363,10 +372,10 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowMenu) {
 
   // Toggling spoken feedback changes the visibillity of the menu.
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      true, ui::A11Y_NOTIFICATION_NONE);
+      true, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      false, ui::A11Y_NOTIFICATION_NONE);
+      false, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_FALSE(CanCreateMenuItem());
 
   // Toggling high contrast changes the visibillity of the menu.
@@ -399,7 +408,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowMenu) {
   AccessibilityManager::Get()->EnableHighContrast(true);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      true, ui::A11Y_NOTIFICATION_NONE);
+      true, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableAutoclick(true);
   EXPECT_TRUE(CanCreateMenuItem());
@@ -410,7 +419,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowMenu) {
   AccessibilityManager::Get()->EnableAutoclick(false);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      false, ui::A11Y_NOTIFICATION_NONE);
+      false, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableHighContrast(false);
   EXPECT_TRUE(CanCreateMenuItem());
@@ -420,10 +429,9 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowMenu) {
 
 IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowMenuWithShowMenuOption) {
   // Login
-  user_manager::UserManager::Get()->UserLoggedIn(
-      AccountId::FromUserEmail("owner@invalid.domain"), "owner@invalid.domain",
-      true);
-  user_manager::UserManager::Get()->SessionStarted();
+  session_manager::SessionManager::Get()->CreateSession(
+      AccountId::FromUserEmail("owner@invalid.domain"), "owner@invalid.domain");
+  session_manager::SessionManager::Get()->SessionStarted();
 
   SetShowAccessibilityOptionsInSystemTrayMenu(true);
 
@@ -432,10 +440,10 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowMenuWithShowMenuOption) {
 
   // The menu remains visible regardless of toggling spoken feedback.
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      true, ui::A11Y_NOTIFICATION_NONE);
+      true, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      false, ui::A11Y_NOTIFICATION_NONE);
+      false, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CanCreateMenuItem());
 
   // The menu remains visible regardless of toggling high contrast.
@@ -468,7 +476,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowMenuWithShowMenuOption) {
   AccessibilityManager::Get()->EnableHighContrast(true);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      true, ui::A11Y_NOTIFICATION_NONE);
+      true, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableAutoclick(true);
   EXPECT_TRUE(CanCreateMenuItem());
@@ -479,7 +487,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowMenuWithShowMenuOption) {
   AccessibilityManager::Get()->EnableAutoclick(false);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      false, ui::A11Y_NOTIFICATION_NONE);
+      false, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableHighContrast(false);
   EXPECT_TRUE(CanCreateMenuItem());
@@ -493,17 +501,17 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowMenuWithShowMenuOption) {
 }
 
 IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowMenuWithShowOnLoginScreen) {
-  SetLoginStatus(ash::user::LOGGED_IN_NONE);
+  SetLoginStatus(ash::LoginStatus::NOT_LOGGED_IN);
 
   // Confirms that the menu is visible.
   EXPECT_TRUE(CanCreateMenuItem());
 
   // The menu remains visible regardless of toggling spoken feedback.
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      true, ui::A11Y_NOTIFICATION_NONE);
+      true, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      false, ui::A11Y_NOTIFICATION_NONE);
+      false, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CanCreateMenuItem());
 
   // The menu remains visible regardless of toggling high contrast.
@@ -530,14 +538,14 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowMenuWithShowOnLoginScreen) {
   AccessibilityManager::Get()->EnableHighContrast(true);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      true, ui::A11Y_NOTIFICATION_NONE);
+      true, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableVirtualKeyboard(true);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableVirtualKeyboard(false);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      false, ui::A11Y_NOTIFICATION_NONE);
+      false, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CanCreateMenuItem());
   AccessibilityManager::Get()->EnableHighContrast(false);
   EXPECT_TRUE(CanCreateMenuItem());
@@ -567,7 +575,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowNotification) {
 
   // Enabling spoken feedback should show the notification.
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      true, ui::A11Y_NOTIFICATION_SHOW);
+      true, ash::A11Y_NOTIFICATION_SHOW);
   EXPECT_EQ(CHROMEVOX_ENABLED, GetNotificationText());
 
   // Connecting a braille display when spoken feedback is already enabled
@@ -581,7 +589,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ShowNotification) {
   EXPECT_TRUE(AccessibilityManager::Get()->IsSpokenFeedbackEnabled());
   EXPECT_FALSE(IsNotificationShown());
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      false, ui::A11Y_NOTIFICATION_SHOW);
+      false, ash::A11Y_NOTIFICATION_SHOW);
   EXPECT_FALSE(IsNotificationShown());
   EXPECT_FALSE(AccessibilityManager::Get()->IsSpokenFeedbackEnabled());
 
@@ -598,7 +606,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, KeepMenuVisibilityOnLockScreen) {
   EXPECT_TRUE(CanCreateMenuItem());
 
   // Locks the screen.
-  SetLoginStatus(ash::user::LOGGED_IN_LOCKED);
+  SetLoginStatus(ash::LoginStatus::LOCKED);
   EXPECT_TRUE(CanCreateMenuItem());
 
   // Disables high contrast mode.
@@ -609,7 +617,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, KeepMenuVisibilityOnLockScreen) {
 }
 
 IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ClickDetailMenu) {
-  SetLoginStatus(ash::user::LOGGED_IN_USER);
+  SetLoginStatus(ash::LoginStatus::USER);
 
   // Confirms that the check item toggles the spoken feedback.
   EXPECT_FALSE(AccessibilityManager::Get()->IsSpokenFeedbackEnabled());
@@ -669,7 +677,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, ClickDetailMenu) {
 }
 
 IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMarksOnDetailMenu) {
-  SetLoginStatus(ash::user::LOGGED_IN_NONE);
+  SetLoginStatus(ash::LoginStatus::NOT_LOGGED_IN);
 
   // At first, all of the check is unchecked.
   EXPECT_TRUE(CreateDetailedMenu());
@@ -683,7 +691,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMarksOnDetailMenu) {
 
   // Enabling spoken feedback.
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      true, ui::A11Y_NOTIFICATION_NONE);
+      true, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CreateDetailedMenu());
   EXPECT_TRUE(IsSpokenFeedbackEnabledOnDetailMenu());
   EXPECT_FALSE(IsHighContrastEnabledOnDetailMenu());
@@ -695,7 +703,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMarksOnDetailMenu) {
 
   // Disabling spoken feedback.
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      false, ui::A11Y_NOTIFICATION_NONE);
+      false, ash::A11Y_NOTIFICATION_NONE);
   EXPECT_TRUE(CreateDetailedMenu());
   EXPECT_FALSE(IsSpokenFeedbackEnabledOnDetailMenu());
   EXPECT_FALSE(IsHighContrastEnabledOnDetailMenu());
@@ -795,7 +803,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMarksOnDetailMenu) {
 
   // Enabling all of the a11y features.
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      true, ui::A11Y_NOTIFICATION_NONE);
+      true, ash::A11Y_NOTIFICATION_NONE);
   AccessibilityManager::Get()->EnableHighContrast(true);
   SetMagnifierEnabled(true);
   AccessibilityManager::Get()->EnableLargeCursor(true);
@@ -811,7 +819,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMarksOnDetailMenu) {
 
   // Disabling all of the a11y features.
   AccessibilityManager::Get()->EnableSpokenFeedback(
-      false, ui::A11Y_NOTIFICATION_NONE);
+      false, ash::A11Y_NOTIFICATION_NONE);
   AccessibilityManager::Get()->EnableHighContrast(false);
   SetMagnifierEnabled(false);
   AccessibilityManager::Get()->EnableLargeCursor(false);
@@ -826,7 +834,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMarksOnDetailMenu) {
   CloseDetailMenu();
 
   // Autoclick is disabled on login screen.
-  SetLoginStatus(ash::user::LOGGED_IN_USER);
+  SetLoginStatus(ash::LoginStatus::USER);
 
   // Enabling autoclick.
   AccessibilityManager::Get()->EnableAutoclick(true);
@@ -852,7 +860,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMarksOnDetailMenu) {
 }
 
 IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMenuVisibilityOnDetailMenu) {
-  SetLoginStatus(ash::user::LOGGED_IN_NONE);
+  SetLoginStatus(ash::LoginStatus::NOT_LOGGED_IN);
   EXPECT_TRUE(CreateDetailedMenu());
   EXPECT_TRUE(IsSpokenFeedbackMenuShownOnDetailMenu());
   EXPECT_TRUE(IsHighContrastMenuShownOnDetailMenu());
@@ -860,11 +868,11 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMenuVisibilityOnDetailMenu) {
   EXPECT_TRUE(IsLargeCursorMenuShownOnDetailMenu());
   EXPECT_FALSE(IsAutoclickMenuShownOnDetailMenu());
   EXPECT_TRUE(IsVirtualKeyboardMenuShownOnDetailMenu());
-  EXPECT_FALSE(IsHelpShownOnDetailMenu());
-  EXPECT_FALSE(IsSettingsShownOnDetailMenu());
+  EXPECT_FALSE(IsHelpAvailableOnDetailMenu());
+  EXPECT_FALSE(IsSettingsAvailableOnDetailMenu());
   CloseDetailMenu();
 
-  SetLoginStatus(ash::user::LOGGED_IN_USER);
+  SetLoginStatus(ash::LoginStatus::USER);
   EXPECT_TRUE(CreateDetailedMenu());
   EXPECT_TRUE(IsSpokenFeedbackMenuShownOnDetailMenu());
   EXPECT_TRUE(IsHighContrastMenuShownOnDetailMenu());
@@ -872,11 +880,11 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMenuVisibilityOnDetailMenu) {
   EXPECT_FALSE(IsLargeCursorMenuShownOnDetailMenu());
   EXPECT_TRUE(IsAutoclickMenuShownOnDetailMenu());
   EXPECT_TRUE(IsVirtualKeyboardMenuShownOnDetailMenu());
-  EXPECT_TRUE(IsHelpShownOnDetailMenu());
-  EXPECT_TRUE(IsSettingsShownOnDetailMenu());
+  EXPECT_TRUE(IsHelpAvailableOnDetailMenu());
+  EXPECT_TRUE(IsSettingsAvailableOnDetailMenu());
   CloseDetailMenu();
 
-  SetLoginStatus(ash::user::LOGGED_IN_LOCKED);
+  SetLoginStatus(ash::LoginStatus::LOCKED);
   EXPECT_TRUE(CreateDetailedMenu());
   EXPECT_TRUE(IsSpokenFeedbackMenuShownOnDetailMenu());
   EXPECT_TRUE(IsHighContrastMenuShownOnDetailMenu());
@@ -884,8 +892,8 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMenuVisibilityOnDetailMenu) {
   EXPECT_FALSE(IsLargeCursorMenuShownOnDetailMenu());
   EXPECT_TRUE(IsAutoclickMenuShownOnDetailMenu());
   EXPECT_TRUE(IsVirtualKeyboardMenuShownOnDetailMenu());
-  EXPECT_FALSE(IsHelpShownOnDetailMenu());
-  EXPECT_FALSE(IsSettingsShownOnDetailMenu());
+  EXPECT_FALSE(IsHelpAvailableOnDetailMenu());
+  EXPECT_FALSE(IsSettingsAvailableOnDetailMenu());
   CloseDetailMenu();
 
   ash::test::TestSessionStateDelegate* session_state_delegate =
@@ -893,7 +901,7 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMenuVisibilityOnDetailMenu) {
   ash::test::ShellTestApi test_api(ash::Shell::GetInstance());
   test_api.SetSessionStateDelegate(session_state_delegate);
   session_state_delegate->SetUserAddingScreenRunning(true);
-  SetLoginStatus(ash::user::LOGGED_IN_USER);
+  SetLoginStatus(ash::LoginStatus::USER);
   EXPECT_TRUE(CreateDetailedMenu());
   EXPECT_TRUE(IsSpokenFeedbackMenuShownOnDetailMenu());
   EXPECT_TRUE(IsHighContrastMenuShownOnDetailMenu());
@@ -901,8 +909,8 @@ IN_PROC_BROWSER_TEST_P(TrayAccessibilityTest, CheckMenuVisibilityOnDetailMenu) {
   EXPECT_FALSE(IsLargeCursorMenuShownOnDetailMenu());
   EXPECT_TRUE(IsAutoclickMenuShownOnDetailMenu());
   EXPECT_TRUE(IsVirtualKeyboardMenuShownOnDetailMenu());
-  EXPECT_FALSE(IsHelpShownOnDetailMenu());
-  EXPECT_FALSE(IsSettingsShownOnDetailMenu());
+  EXPECT_FALSE(IsHelpAvailableOnDetailMenu());
+  EXPECT_FALSE(IsSettingsAvailableOnDetailMenu());
   CloseDetailMenu();
 }
 
