@@ -5,21 +5,20 @@
 #include "chrome/browser/component_updater/recovery_component_installer.h"
 
 #include <stdint.h>
+
+#include <memory>
 #include <string>
 
 #include "base/base_paths.h"
 #include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/command_line.h"
-#include "base/compiler_specific.h"
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
 #include "base/json/json_file_value_serializer.h"
 #include "base/logging.h"
-#include "base/memory/scoped_ptr.h"
-#include "base/metrics/histogram.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/path_service.h"
-#include "base/prefs/pref_registry_simple.h"
-#include "base/prefs/pref_service.h"
 #include "base/process/kill.h"
 #include "base/process/launch.h"
 #include "base/process/process.h"
@@ -30,7 +29,10 @@
 #include "components/component_updater/component_updater_paths.h"
 #include "components/component_updater/component_updater_service.h"
 #include "components/component_updater/pref_names.h"
+#include "components/prefs/pref_registry_simple.h"
+#include "components/prefs/pref_service.h"
 #include "components/update_client/update_client.h"
+#include "components/update_client/utils.h"
 #include "content/public/browser/browser_thread.h"
 
 using content::BrowserThread;
@@ -80,20 +82,20 @@ void RecordRecoveryComponentUMAEvent(RecoveryComponentEvent event) {
   UMA_HISTOGRAM_ENUMERATION("RecoveryComponent.Event", event, RCE_COUNT);
 }
 
-#if !defined(OS_CHROMEOS)
+#if !defined(OS_CHROMEOS) && defined(GOOGLE_CHROME_BUILD)
 // Checks if elevated recovery simulation switch was present on the command
 // line. This is for testing purpose.
 bool SimulatingElevatedRecovery() {
   return base::CommandLine::ForCurrentProcess()->HasSwitch(
       switches::kSimulateElevatedRecovery);
 }
-#endif  // !defined(OS_CHROMEOS)
+#endif  // !defined(OS_CHROMEOS) && defined(GOOGLE_CHROME_BUILD)
 
 base::CommandLine GetRecoveryInstallCommandLine(
     const base::FilePath& command,
     const base::DictionaryValue& manifest,
     bool is_deferred_run,
-    const Version& version) {
+    const base::Version& version) {
   base::CommandLine command_line(command);
 
   // Add a flag to for re-attempted install with elevated privilege so that the
@@ -116,7 +118,8 @@ base::CommandLine GetRecoveryInstallCommandLine(
 }
 
 #if defined(OS_WIN)
-scoped_ptr<base::DictionaryValue> ReadManifest(const base::FilePath& manifest) {
+std::unique_ptr<base::DictionaryValue> ReadManifest(
+    const base::FilePath& manifest) {
   JSONFileValueDeserializer deserializer(manifest);
   std::string error;
   return base::DictionaryValue::From(deserializer.Deserialize(NULL, &error));
@@ -143,14 +146,14 @@ void DoElevatedInstallRecoveryComponent(const base::FilePath& path) {
   if (!base::PathExists(main_file) || !base::PathExists(manifest_file))
     return;
 
-  scoped_ptr<base::DictionaryValue> manifest(ReadManifest(manifest_file));
+  std::unique_ptr<base::DictionaryValue> manifest(ReadManifest(manifest_file));
   std::string name;
   manifest->GetStringASCII("name", &name);
   if (name != kRecoveryManifestName)
     return;
   std::string proposed_version;
   manifest->GetStringASCII("version", &proposed_version);
-  const Version version(proposed_version.c_str());
+  const base::Version version(proposed_version.c_str());
   if (!version.IsValid())
     return;
 
@@ -191,13 +194,14 @@ void ElevatedInstallRecoveryComponent(const base::FilePath& installer_path) {
 // See chrome/browser/recovery/recovery_install_global_error.cc for details.
 class RecoveryComponentInstaller : public update_client::CrxInstaller {
  public:
-  RecoveryComponentInstaller(const Version& version, PrefService* prefs);
+  RecoveryComponentInstaller(const base::Version& version, PrefService* prefs);
 
   // ComponentInstaller implementation:
   void OnUpdateError(int error) override;
 
-  bool Install(const base::DictionaryValue& manifest,
-               const base::FilePath& unpack_path) override;
+  update_client::CrxInstaller::Result Install(
+      const base::DictionaryValue& manifest,
+      const base::FilePath& unpack_path) override;
 
   bool GetInstalledFile(const std::string& file,
                         base::FilePath* installed_file) override;
@@ -207,10 +211,13 @@ class RecoveryComponentInstaller : public update_client::CrxInstaller {
  private:
   ~RecoveryComponentInstaller() override {}
 
+  bool DoInstall(const base::DictionaryValue& manifest,
+                 const base::FilePath& unpack_path);
+
   bool RunInstallCommand(const base::CommandLine& cmdline,
                          const base::FilePath& installer_folder) const;
 
-  Version current_version_;
+  base::Version current_version_;
   PrefService* prefs_;
 };
 
@@ -220,23 +227,25 @@ void SimulateElevatedRecoveryHelper(PrefService* prefs) {
 
 void RecoveryRegisterHelper(ComponentUpdateService* cus, PrefService* prefs) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  Version version(prefs->GetString(prefs::kRecoveryComponentVersion));
+  base::Version version(prefs->GetString(prefs::kRecoveryComponentVersion));
   if (!version.IsValid()) {
     NOTREACHED();
     return;
   }
-
   update_client::CrxComponent recovery;
   recovery.name = "recovery";
   recovery.installer = new RecoveryComponentInstaller(version, prefs);
   recovery.version = version;
   recovery.pk_hash.assign(kSha2Hash, &kSha2Hash[sizeof(kSha2Hash)]);
+  recovery.supports_group_policy_enable_component_updates = true;
+  recovery.requires_network_encryption = false;
   if (!cus->RegisterComponent(recovery)) {
     NOTREACHED() << "Recovery component registration failed.";
   }
 }
 
-void RecoveryUpdateVersionHelper(const Version& version, PrefService* prefs) {
+void RecoveryUpdateVersionHelper(
+    const base::Version& version, PrefService* prefs) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   prefs->SetString(prefs::kRecoveryComponentVersion, version.GetString());
 }
@@ -248,8 +257,8 @@ void SetPrefsForElevatedRecoveryInstall(const base::FilePath& unpack_path,
   prefs->SetBoolean(prefs::kRecoveryComponentNeedsElevation, true);
 }
 
-RecoveryComponentInstaller::RecoveryComponentInstaller(const Version& version,
-                                                       PrefService* prefs)
+RecoveryComponentInstaller::RecoveryComponentInstaller(
+    const base::Version& version, PrefService* prefs)
     : current_version_(version), prefs_(prefs) {
   DCHECK(version.IsValid());
 }
@@ -331,15 +340,24 @@ bool SetPosixExecutablePermission(const base::FilePath& path) {
 }
 #endif  // defined(OS_POSIX)
 
-bool RecoveryComponentInstaller::Install(const base::DictionaryValue& manifest,
-                                         const base::FilePath& unpack_path) {
+update_client::CrxInstaller::Result RecoveryComponentInstaller::Install(
+    const base::DictionaryValue& manifest,
+    const base::FilePath& unpack_path) {
+  return update_client::InstallFunctionWrapper(
+      base::Bind(&RecoveryComponentInstaller::DoInstall, base::Unretained(this),
+                 base::ConstRef(manifest), base::ConstRef(unpack_path)));
+}
+
+bool RecoveryComponentInstaller::DoInstall(
+    const base::DictionaryValue& manifest,
+    const base::FilePath& unpack_path) {
   std::string name;
   manifest.GetStringASCII("name", &name);
   if (name != kRecoveryManifestName)
     return false;
   std::string proposed_version;
   manifest.GetStringASCII("version", &proposed_version);
-  Version version(proposed_version.c_str());
+  base::Version version(proposed_version.c_str());
   if (!version.IsValid())
     return false;
   if (current_version_.CompareTo(version) >= 0)
@@ -405,7 +423,7 @@ bool RecoveryComponentInstaller::Uninstall() {
 
 void RegisterRecoveryComponent(ComponentUpdateService* cus,
                                PrefService* prefs) {
-#if !defined(OS_CHROMEOS)
+#if !defined(OS_CHROMEOS) && defined(GOOGLE_CHROME_BUILD)
   if (SimulatingElevatedRecovery()) {
     BrowserThread::PostTask(
         BrowserThread::UI,
@@ -420,7 +438,7 @@ void RegisterRecoveryComponent(ComponentUpdateService* cus,
       FROM_HERE,
       base::Bind(&RecoveryRegisterHelper, cus, prefs),
       base::TimeDelta::FromSeconds(6));
-#endif  // !defined(OS_CHROMEOS)
+#endif  // !defined(OS_CHROMEOS) && defined(GOOGLE_CHROME_BUILD)
 }
 
 void RegisterPrefsForRecoveryComponent(PrefRegistrySimple* registry) {
