@@ -10,7 +10,7 @@
 
 #include "base/base_paths.h"
 #include "base/bind.h"
-#include "base/compiler_specific.h"
+#include "base/bind_helpers.h"
 #include "base/files/file_enumerator.h"
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
@@ -18,13 +18,16 @@
 #include "base/path_service.h"
 #include "base/strings/string_util.h"
 #include "base/values.h"
+#include "build/build_config.h"
 #include "components/component_updater/component_updater_paths.h"
 #include "components/component_updater/component_updater_service.h"
 #include "components/update_client/update_client.h"
+#include "components/update_client/utils.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/gpu_data_manager.h"
 #include "content/public/browser/gpu_data_manager_observer.h"
 #include "gpu/config/gpu_feature_type.h"
+#include "ui/gl/gl_features.h"
 
 using content::BrowserThread;
 using content::GpuDataManager;
@@ -64,7 +67,7 @@ base::FilePath GetSwiftShaderBaseDirectory() {
 // On success it returns something like:
 // <profile>\AppData\Local\Google\Chrome\User Data\SwiftShader\10.3.44.555\.
 bool GetLatestSwiftShaderDirectory(base::FilePath* result,
-                                   Version* latest,
+                                   base::Version* latest,
                                    std::vector<base::FilePath>* older_dirs) {
   base::FilePath base_dir = GetSwiftShaderBaseDirectory();
   bool found = false;
@@ -72,7 +75,7 @@ bool GetLatestSwiftShaderDirectory(base::FilePath* result,
       base_dir, false, base::FileEnumerator::DIRECTORIES);
   for (base::FilePath path = file_enumerator.Next(); !path.value().empty();
        path = file_enumerator.Next()) {
-    Version version(path.BaseName().MaybeAsASCII());
+    base::Version version(path.BaseName().MaybeAsASCII());
     if (!version.IsValid())
       continue;
     if (version.CompareTo(*latest) > 0 &&
@@ -98,13 +101,14 @@ void RegisterSwiftShaderWithChrome(const base::FilePath& path) {
 
 class SwiftShaderComponentInstaller : public update_client::CrxInstaller {
  public:
-  explicit SwiftShaderComponentInstaller(const Version& version);
+  explicit SwiftShaderComponentInstaller(const base::Version& version);
 
   // ComponentInstaller implementation:
   void OnUpdateError(int error) override;
 
-  bool Install(const base::DictionaryValue& manifest,
-               const base::FilePath& unpack_path) override;
+  update_client::CrxInstaller::Result Install(
+      const base::DictionaryValue& manifest,
+      const base::FilePath& unpack_path) override;
 
   bool GetInstalledFile(const std::string& file,
                         base::FilePath* installed_file) override;
@@ -114,11 +118,14 @@ class SwiftShaderComponentInstaller : public update_client::CrxInstaller {
  private:
   ~SwiftShaderComponentInstaller() override {}
 
-  Version current_version_;
+  bool DoInstall(const base::DictionaryValue& manifest,
+                 const base::FilePath& unpack_path);
+
+  base::Version current_version_;
 };
 
 SwiftShaderComponentInstaller::SwiftShaderComponentInstaller(
-    const Version& version)
+    const base::Version& version)
     : current_version_(version) {
   DCHECK(version.IsValid());
 }
@@ -127,7 +134,15 @@ void SwiftShaderComponentInstaller::OnUpdateError(int error) {
   NOTREACHED() << "SwiftShader update error: " << error;
 }
 
-bool SwiftShaderComponentInstaller::Install(
+update_client::CrxInstaller::Result SwiftShaderComponentInstaller::Install(
+    const base::DictionaryValue& manifest,
+    const base::FilePath& unpack_path) {
+  return update_client::InstallFunctionWrapper(base::Bind(
+      &SwiftShaderComponentInstaller::DoInstall, base::Unretained(this),
+      base::ConstRef(manifest), base::ConstRef(unpack_path)));
+}
+
+bool SwiftShaderComponentInstaller::DoInstall(
     const base::DictionaryValue& manifest,
     const base::FilePath& unpack_path) {
   std::string name;
@@ -136,7 +151,7 @@ bool SwiftShaderComponentInstaller::Install(
     return false;
   std::string proposed_version;
   manifest.GetStringASCII("version", &proposed_version);
-  Version version(proposed_version.c_str());
+  base::Version version(proposed_version.c_str());
   if (!version.IsValid())
     return false;
   if (current_version_.CompareTo(version) >= 0)
@@ -170,7 +185,7 @@ bool SwiftShaderComponentInstaller::Uninstall() {
 }
 
 void FinishSwiftShaderUpdateRegistration(ComponentUpdateService* cus,
-                                         const Version& version) {
+                                         const base::Version& version) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
   update_client::CrxComponent swiftshader;
@@ -178,6 +193,8 @@ void FinishSwiftShaderUpdateRegistration(ComponentUpdateService* cus,
   swiftshader.installer = new SwiftShaderComponentInstaller(version);
   swiftshader.version = version;
   swiftshader.pk_hash.assign(kSha2Hash, &kSha2Hash[sizeof(kSha2Hash)]);
+  swiftshader.supports_group_policy_enable_component_updates = true;
+  swiftshader.requires_network_encryption = false;
   if (!cus->RegisterComponent(swiftshader)) {
     NOTREACHED() << "SwiftShader component registration fail";
   }
@@ -206,7 +223,7 @@ void UpdateChecker::OnGpuInfoUpdate() {
     DCHECK_CURRENTLY_ON(BrowserThread::FILE);
     base::FilePath path = GetSwiftShaderBaseDirectory();
 
-    Version version(kNullVersion);
+    base::Version version(kNullVersion);
     GetLatestSwiftShaderDirectory(&path, &version, NULL);
 
     BrowserThread::PostTask(
@@ -216,7 +233,7 @@ void UpdateChecker::OnGpuInfoUpdate() {
   }
 }
 
-#if defined(ENABLE_SWIFTSHADER)
+#if BUILDFLAG(ENABLE_SWIFTSHADER)
 
 // Check if there already is a version of swiftshader installed,
 // and if so register it.
@@ -230,7 +247,7 @@ void RegisterSwiftShaderPath(ComponentUpdateService* cus) {
     }
   }
 
-  Version version(kNullVersion);
+  base::Version version(kNullVersion);
   std::vector<base::FilePath> older_dirs;
   if (GetLatestSwiftShaderDirectory(&path, &version, &older_dirs))
     BrowserThread::PostTask(BrowserThread::UI,
@@ -256,7 +273,7 @@ void RegisterSwiftShaderPath(ComponentUpdateService* cus) {
 }  // namespace
 
 void RegisterSwiftShaderComponent(ComponentUpdateService* cus) {
-#if defined(ENABLE_SWIFTSHADER)
+#if BUILDFLAG(ENABLE_SWIFTSHADER) && defined(ARCH_CPU_X86)
   BrowserThread::PostTask(BrowserThread::FILE,
                           FROM_HERE,
                           base::Bind(&RegisterSwiftShaderPath, cus));
