@@ -7,21 +7,23 @@
 #include <stddef.h>
 #include <stdint.h>
 #include <string.h>
+#include <memory>
 #include <string>
 #include <utility>
 #include <vector>
 
 #include "base/base_paths.h"
 #include "base/bind.h"
-#include "base/compiler_specific.h"
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
 #include "base/logging.h"
 #include "base/macros.h"
+#include "base/native_library.h"
 #include "base/path_service.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
+#include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
 #include "build/build_config.h"
@@ -31,7 +33,9 @@
 #include "components/component_updater/default_component_installer.h"
 #include "components/version_info/version_info.h"
 #include "content/public/browser/browser_thread.h"
+#include "content/public/browser/cdm_service.h"
 #include "content/public/browser/plugin_service.h"
+#include "content/public/common/cdm_info.h"
 #include "content/public/common/pepper_plugin_info.h"
 #include "media/cdm/supported_cdm_versions.h"
 #include "third_party/widevine/cdm/widevine_cdm_common.h"
@@ -39,6 +43,7 @@
 #include "widevine_cdm_version.h"  // In SHARED_INTERMEDIATE_DIR. NOLINT
 
 using content::BrowserThread;
+using content::CdmService;
 using content::PluginService;
 
 namespace component_updater {
@@ -53,9 +58,6 @@ const uint8_t kSha2Hash[] = {0xe8, 0xce, 0xcf, 0x42, 0x06, 0xd0, 0x93, 0x49,
                              0x8f, 0xbd, 0x86, 0x12, 0xb9, 0x58, 0x9b, 0xfb,
                              0x4f, 0xbb, 0x1b, 0xa9, 0xd3, 0x85, 0x37, 0xef};
 
-// File name of the Widevine CDM component manifest on different platforms.
-const char kWidevineCdmManifestName[] = "WidevineCdm";
-
 // File name of the Widevine CDM adapter version file. The CDM adapter shares
 // the same version number with Chromium version.
 const char kCdmAdapterVersionName[] = "CdmAdapterVersion";
@@ -112,22 +114,22 @@ base::FilePath GetPlatformDirectory(const base::FilePath& base_path) {
   return base_path.AppendASCII("_platform_specific").AppendASCII(platform_arch);
 }
 
-bool MakeWidevineCdmPluginInfo(
-    const base::Version& version,
-    const base::FilePath& path,
-    const std::vector<base::string16>& additional_param_names,
-    const std::vector<base::string16>& additional_param_values,
-    content::PepperPluginInfo* plugin_info) {
+bool MakeWidevineCdmPluginInfo(const base::Version& version,
+                               const base::FilePath& cdm_install_dir,
+                               const std::string& codecs,
+                               content::PepperPluginInfo* plugin_info) {
   if (!version.IsValid() ||
       version.components().size() !=
           static_cast<size_t>(kWidevineCdmVersionNumComponents)) {
+    DVLOG(1) << "Invalid version.";
     return false;
   }
 
   plugin_info->is_internal = false;
   // Widevine CDM must run out of process.
   plugin_info->is_out_of_process = true;
-  plugin_info->path = path;
+  plugin_info->path = GetPlatformDirectory(cdm_install_dir)
+                          .AppendASCII(kWidevineCdmAdapterFileName);
   plugin_info->name = kWidevineCdmDisplayName;
   plugin_info->description = kWidevineCdmDescription +
                              std::string(" (version: ") + version.GetString() +
@@ -137,8 +139,11 @@ bool MakeWidevineCdmPluginInfo(
       kWidevineCdmPluginMimeType,
       kWidevineCdmPluginExtension,
       kWidevineCdmPluginMimeTypeDescription);
-  widevine_cdm_mime_type.additional_param_names = additional_param_names;
-  widevine_cdm_mime_type.additional_param_values = additional_param_values;
+  widevine_cdm_mime_type.additional_param_names.push_back(
+      base::ASCIIToUTF16(kCdmSupportedCodecsParamName));
+  widevine_cdm_mime_type.additional_param_values.push_back(
+      base::ASCIIToUTF16(codecs));
+
   plugin_info->mime_types.push_back(widevine_cdm_mime_type);
   plugin_info->permissions = kWidevineCdmPluginPermissions;
 
@@ -152,10 +157,10 @@ bool CheckForCompatibleVersion(const base::DictionaryValue& manifest,
                                VersionCheckFunc version_check_func) {
   std::string versions_string;
   if (!manifest.GetString(version_name, &versions_string)) {
-    DLOG(WARNING) << "Widevine CDM component manifest missing " << version_name;
+    DVLOG(1) << "Widevine CDM component manifest missing " << version_name;
     return false;
   }
-  DLOG_IF(WARNING, versions_string.empty())
+  DVLOG_IF(1, versions_string.empty())
       << "Widevine CDM component manifest has empty " << version_name;
 
   for (const base::StringPiece& ver_str : base::SplitStringPiece(
@@ -167,8 +172,8 @@ bool CheckForCompatibleVersion(const base::DictionaryValue& manifest,
         return true;
   }
 
-  DLOG(WARNING) << "Widevine CDM component manifest has no supported "
-                << version_name << " in '" << versions_string << "'";
+  DVLOG(1) << "Widevine CDM component manifest has no supported "
+           << version_name << " in '" << versions_string << "'";
   return false;
 }
 
@@ -189,38 +194,32 @@ bool IsCompatibleWithChrome(const base::DictionaryValue& manifest) {
                                    media::IsSupportedCdmHostVersion);
 }
 
-void GetAdditionalParams(const base::DictionaryValue& manifest,
-                         std::vector<base::string16>* additional_param_names,
-                         std::vector<base::string16>* additional_param_values) {
-  base::string16 codecs;
-  if (manifest.GetString(kCdmCodecsListName, &codecs)) {
-    DLOG_IF(WARNING, codecs.empty())
+std::string GetCodecs(const base::DictionaryValue& manifest) {
+  std::string codecs;
+  if (manifest.GetStringASCII(kCdmCodecsListName, &codecs)) {
+    DVLOG_IF(1, codecs.empty())
         << "Widevine CDM component manifest has empty codecs list";
-    additional_param_names->push_back(
-        base::ASCIIToUTF16(kCdmSupportedCodecsParamName));
-    additional_param_values->push_back(codecs);
   } else {
-    DLOG(WARNING) << "Widevine CDM component manifest is missing codecs";
+    DVLOG(1) << "Widevine CDM component manifest is missing codecs";
   }
+  return codecs;
 }
 
-void RegisterWidevineCdmWithChrome(const base::Version& cdm_version,
-                                   const base::FilePath& adapter_install_path,
-                                   scoped_ptr<base::DictionaryValue> manifest) {
+void RegisterWidevineCdmWithChrome(
+    const base::Version& cdm_version,
+    const base::FilePath& cdm_install_dir,
+    std::unique_ptr<base::DictionaryValue> manifest) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  std::vector<base::string16> additional_param_names;
-  std::vector<base::string16> additional_param_values;
-  GetAdditionalParams(
-      *manifest, &additional_param_names, &additional_param_values);
+  const std::string codecs = GetCodecs(*manifest);
+
   content::PepperPluginInfo plugin_info;
-  if (!MakeWidevineCdmPluginInfo(cdm_version,
-                                 adapter_install_path,
-                                 additional_param_names,
-                                 additional_param_values,
+  if (!MakeWidevineCdmPluginInfo(cdm_version, cdm_install_dir, codecs,
                                  &plugin_info)) {
     return;
   }
 
+  VLOG(1) << "Register Widevine CDM with Chrome";
+
   // true = Add to beginning of list to override any existing registrations.
   PluginService::GetInstance()->RegisterInternalPlugin(
       plugin_info.ToWebPluginInfo(), true);
@@ -228,6 +227,16 @@ void RegisterWidevineCdmWithChrome(const base::Version& cdm_version,
   // update their plugin list caches.
   PluginService::GetInstance()->RefreshPlugins();
   PluginService::GetInstance()->PurgePluginListCache(NULL, false);
+
+  // Also register Widevine with the CdmService.
+  const base::FilePath cdm_path =
+      GetPlatformDirectory(cdm_install_dir)
+          .AppendASCII(base::GetNativeLibraryName(kWidevineCdmLibraryName));
+  const std::vector<std::string> supported_codecs = base::SplitString(
+      codecs, std::string(1, kCdmSupportedCodecsValueDelimiter),
+      base::TRIM_WHITESPACE, base::SPLIT_WANT_NONEMPTY);
+  CdmService::GetInstance()->RegisterCdm(content::CdmInfo(
+      kWidevineCdmType, cdm_version, cdm_path, supported_codecs));
 }
 
 }  // namespace
@@ -239,19 +248,22 @@ class WidevineCdmComponentInstallerTraits : public ComponentInstallerTraits {
 
  private:
   // The following methods override ComponentInstallerTraits.
-  bool CanAutoUpdate() const override;
-  bool OnCustomInstall(const base::DictionaryValue& manifest,
-                               const base::FilePath& install_dir) override;
+  bool SupportsGroupPolicyEnabledComponentUpdates() const override;
+  bool RequiresNetworkEncryption() const override;
+  update_client::CrxInstaller::Result OnCustomInstall(
+      const base::DictionaryValue& manifest,
+      const base::FilePath& install_dir) override;
   bool VerifyInstallation(
       const base::DictionaryValue& manifest,
       const base::FilePath& install_dir) const override;
-  void ComponentReady(
-      const base::Version& version,
-      const base::FilePath& path,
-      scoped_ptr<base::DictionaryValue> manifest) override;
-  base::FilePath GetBaseDirectory() const override;
+  void ComponentReady(const base::Version& version,
+                      const base::FilePath& path,
+                      std::unique_ptr<base::DictionaryValue> manifest) override;
+  base::FilePath GetRelativeInstallDir() const override;
   void GetHash(std::vector<uint8_t>* hash) const override;
   std::string GetName() const override;
+  update_client::InstallerAttributes GetInstallerAttributes() const override;
+  std::vector<std::string> GetMimeTypes() const override;
 
   // Checks and updates CDM adapter if necessary to make sure the latest CDM
   // adapter is always used.
@@ -260,7 +272,7 @@ class WidevineCdmComponentInstallerTraits : public ComponentInstallerTraits {
   // VerifyInstallation).
   void UpdateCdmAdapter(const base::Version& cdm_version,
                         const base::FilePath& cdm_install_dir,
-                        scoped_ptr<base::DictionaryValue> manifest);
+                        std::unique_ptr<base::DictionaryValue> manifest);
 
   DISALLOW_COPY_AND_ASSIGN(WidevineCdmComponentInstallerTraits);
 };
@@ -268,23 +280,29 @@ class WidevineCdmComponentInstallerTraits : public ComponentInstallerTraits {
 WidevineCdmComponentInstallerTraits::WidevineCdmComponentInstallerTraits() {
 }
 
-bool WidevineCdmComponentInstallerTraits::CanAutoUpdate() const {
+bool WidevineCdmComponentInstallerTraits::
+    SupportsGroupPolicyEnabledComponentUpdates() const {
   return true;
 }
 
-bool WidevineCdmComponentInstallerTraits::OnCustomInstall(
+bool WidevineCdmComponentInstallerTraits::RequiresNetworkEncryption() const {
+  return false;
+}
+
+update_client::CrxInstaller::Result
+WidevineCdmComponentInstallerTraits::OnCustomInstall(
     const base::DictionaryValue& manifest,
     const base::FilePath& install_dir) {
-  return true;
+  return update_client::CrxInstaller::Result(0);
 }
 
 // Once the CDM is ready, check the CDM adapter.
 void WidevineCdmComponentInstallerTraits::ComponentReady(
     const base::Version& version,
     const base::FilePath& path,
-    scoped_ptr<base::DictionaryValue> manifest) {
+    std::unique_ptr<base::DictionaryValue> manifest) {
   if (!IsCompatibleWithChrome(*manifest)) {
-    DLOG(WARNING) << "Installed Widevine CDM component is incompatible.";
+    VLOG(1) << "Installed Widevine CDM component is incompatible.";
     return;
   }
 
@@ -302,15 +320,15 @@ bool WidevineCdmComponentInstallerTraits::VerifyInstallation(
     const base::FilePath& install_dir) const {
   return IsCompatibleWithChrome(manifest) &&
          base::PathExists(GetPlatformDirectory(install_dir)
-                              .AppendASCII(kWidevineCdmFileName));
+                              .AppendASCII(base::GetNativeLibraryName(
+                                  kWidevineCdmLibraryName)));
 }
 
 // The base directory on Windows looks like:
 // <profile>\AppData\Local\Google\Chrome\User Data\WidevineCdm\.
-base::FilePath WidevineCdmComponentInstallerTraits::GetBaseDirectory() const {
-  base::FilePath result;
-  PathService::Get(chrome::DIR_COMPONENT_WIDEVINE_CDM, &result);
-  return result;
+base::FilePath WidevineCdmComponentInstallerTraits::GetRelativeInstallDir()
+    const {
+  return base::FilePath(FILE_PATH_LITERAL("WidevineCdm"));
 }
 
 void WidevineCdmComponentInstallerTraits::GetHash(
@@ -319,47 +337,72 @@ void WidevineCdmComponentInstallerTraits::GetHash(
 }
 
 std::string WidevineCdmComponentInstallerTraits::GetName() const {
-  return kWidevineCdmManifestName;
+  return kWidevineCdmDisplayName;
+}
+
+update_client::InstallerAttributes
+WidevineCdmComponentInstallerTraits::GetInstallerAttributes() const {
+  return update_client::InstallerAttributes();
+}
+
+std::vector<std::string> WidevineCdmComponentInstallerTraits::GetMimeTypes()
+    const {
+  return std::vector<std::string>();
+}
+
+static bool HasValidAdapter(const base::FilePath& adapter_version_path,
+                            const base::FilePath& adapter_install_path,
+                            const std::string& chrome_version) {
+  std::string adapter_version;
+  return base::ReadFileToString(adapter_version_path, &adapter_version) &&
+         adapter_version == chrome_version &&
+         base::PathExists(adapter_install_path);
 }
 
 void WidevineCdmComponentInstallerTraits::UpdateCdmAdapter(
     const base::Version& cdm_version,
     const base::FilePath& cdm_install_dir,
-    scoped_ptr<base::DictionaryValue> manifest) {
+    std::unique_ptr<base::DictionaryValue> manifest) {
   const base::FilePath adapter_version_path =
       GetPlatformDirectory(cdm_install_dir).AppendASCII(kCdmAdapterVersionName);
   const base::FilePath adapter_install_path =
       GetPlatformDirectory(cdm_install_dir)
           .AppendASCII(kWidevineCdmAdapterFileName);
 
+  VLOG(1) << "UpdateCdmAdapter: version" << cdm_version.GetString()
+          << " adapter_install_path=" << adapter_install_path.AsUTF8Unsafe()
+          << " adapter_version_path=" << adapter_version_path.AsUTF8Unsafe();
+
+  base::FilePath adapter_source_path;
+  PathService::Get(chrome::FILE_WIDEVINE_CDM_ADAPTER, &adapter_source_path);
+
   const std::string chrome_version = version_info::GetVersionNumber();
   DCHECK(!chrome_version.empty());
-  std::string adapter_version;
-  if (!base::ReadFileToString(adapter_version_path, &adapter_version) ||
-      adapter_version != chrome_version ||
-      !base::PathExists(adapter_install_path)) {
+
+  // If we are not using bundled CDM and we don't have a valid adapter, create
+  // the version file and copy the CDM adapter from |adapter_source_path| to
+  // |adapter_install_path|.
+  if (adapter_install_path != adapter_source_path &&
+      !HasValidAdapter(adapter_version_path, adapter_install_path,
+                       chrome_version)) {
     int bytes_written = base::WriteFile(
         adapter_version_path, chrome_version.data(), chrome_version.size());
     if (bytes_written < 0 ||
         static_cast<size_t>(bytes_written) != chrome_version.size()) {
-      DLOG(WARNING) << "Failed to write Widevine CDM adapter version file.";
+      PLOG(WARNING) << "Failed to write Widevine CDM adapter version file.";
       // Ignore version file writing failure and try to copy the CDM adapter.
     }
 
-    base::FilePath adapter_source_path;
-    PathService::Get(chrome::FILE_WIDEVINE_CDM_ADAPTER, &adapter_source_path);
     if (!base::CopyFile(adapter_source_path, adapter_install_path)) {
-      DLOG(WARNING) << "Failed to copy Widevine CDM adapter.";
+      PLOG(WARNING) << "Failed to copy Widevine CDM adapter.";
       return;
     }
   }
 
-  BrowserThread::PostTask(content::BrowserThread::UI,
-                          FROM_HERE,
-                          base::Bind(&RegisterWidevineCdmWithChrome,
-                                     cdm_version,
-                                     adapter_install_path,
-                                     base::Passed(&manifest)));
+  BrowserThread::PostTask(
+      content::BrowserThread::UI, FROM_HERE,
+      base::Bind(&RegisterWidevineCdmWithChrome, cdm_version, cdm_install_dir,
+                 base::Passed(&manifest)));
 }
 
 #endif  // defined(WIDEVINE_CDM_AVAILABLE) && defined(WIDEVINE_CDM_IS_COMPONENT)
@@ -370,7 +413,7 @@ void RegisterWidevineCdmComponent(ComponentUpdateService* cus) {
   PathService::Get(chrome::FILE_WIDEVINE_CDM_ADAPTER, &adapter_source_path);
   if (!base::PathExists(adapter_source_path))
     return;
-  scoped_ptr<ComponentInstallerTraits> traits(
+  std::unique_ptr<ComponentInstallerTraits> traits(
       new WidevineCdmComponentInstallerTraits);
   // |cus| will take ownership of |installer| during installer->Register(cus).
   DefaultComponentInstaller* installer =
