@@ -8,9 +8,9 @@
 
 #include "base/command_line.h"
 #include "base/lazy_instance.h"
-#include "base/prefs/pref_service.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
+#include "build/build_config.h"
 #include "chrome/browser/browsing_data/browsing_data_appcache_helper.h"
 #include "chrome/browser/browsing_data/browsing_data_cookie_helper.h"
 #include "chrome/browser/browsing_data/browsing_data_database_helper.h"
@@ -20,26 +20,31 @@
 #include "chrome/browser/browsing_data/cookies_tree_model.h"
 #include "chrome/browser/chrome_notification_types.h"
 #include "chrome/browser/content_settings/chrome_content_settings_utils.h"
-#include "chrome/browser/media/media_capture_devices_dispatcher.h"
-#include "chrome/browser/media/media_stream_capture_indicator.h"
+#include "chrome/browser/content_settings/host_content_settings_map_factory.h"
+#include "chrome/browser/media/webrtc/media_capture_devices_dispatcher.h"
+#include "chrome/browser/media/webrtc/media_stream_capture_indicator.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/render_messages.h"
-#include "components/content_settings/content/common/content_settings_messages.h"
 #include "components/content_settings/core/browser/content_settings_details.h"
+#include "components/content_settings/core/browser/content_settings_info.h"
+#include "components/content_settings/core/browser/content_settings_registry.h"
 #include "components/content_settings/core/browser/content_settings_utils.h"
 #include "components/content_settings/core/browser/host_content_settings_map.h"
+#include "components/prefs/pref_service.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/navigation_controller.h"
 #include "content/public/browser/navigation_details.h"
 #include "content/public/browser/navigation_entry.h"
+#include "content/public/browser/navigation_handle.h"
 #include "content/public/browser/notification_registrar.h"
 #include "content/public/browser/notification_service.h"
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/render_view_host.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/browser/web_contents_delegate.h"
+#include "content/public/common/content_constants.h"
 #include "net/base/registry_controlled_domains/registry_controlled_domain.h"
 #include "net/cookies/canonical_cookie.h"
 #include "storage/common/fileapi/file_system_types.h"
@@ -49,22 +54,21 @@ using content::NavigationController;
 using content::NavigationEntry;
 using content::WebContents;
 
-DEFINE_WEB_CONTENTS_USER_DATA_KEY(TabSpecificContentSettings);
-
 namespace {
 
-ContentSettingsUsagesState::CommittedDetails GetCommittedDetails(
-    const content::LoadCommittedDetails& details) {
-  ContentSettingsUsagesState::CommittedDetails committed_details;
-  committed_details.current_url_valid = !!details.entry;
-  if (details.entry)
-    committed_details.current_url = details.entry->GetURL();
-  committed_details.previous_url = details.previous_url;
-  return committed_details;
+static TabSpecificContentSettings* GetForWCGetter(
+    const base::Callback<content::WebContents*(void)>& wc_getter) {
+  WebContents* web_contents = wc_getter.Run();
+  if (!web_contents)
+    return nullptr;
+
+  return TabSpecificContentSettings::FromWebContents(web_contents);
 }
 
 }  // namespace
 
+DEFINE_WEB_CONTENTS_USER_DATA_KEY(TabSpecificContentSettings);
+
 TabSpecificContentSettings::SiteDataObserver::SiteDataObserver(
     TabSpecificContentSettings* tab_specific_content_settings)
     : tab_specific_content_settings_(tab_specific_content_settings) {
@@ -87,33 +91,31 @@ TabSpecificContentSettings::TabSpecificContentSettings(WebContents* tab)
       blocked_local_shared_objects_(
           Profile::FromBrowserContext(tab->GetBrowserContext())),
       geolocation_usages_state_(
-          Profile::FromBrowserContext(tab->GetBrowserContext())
-              ->GetHostContentSettingsMap(),
-          CONTENT_SETTINGS_TYPE_GEOLOCATION,
-          prefs::kAcceptLanguages,
-          Profile::FromBrowserContext(tab->GetBrowserContext())->GetPrefs()),
+          HostContentSettingsMapFactory::GetForProfile(
+              Profile::FromBrowserContext(tab->GetBrowserContext())),
+          CONTENT_SETTINGS_TYPE_GEOLOCATION),
       midi_usages_state_(
-          Profile::FromBrowserContext(tab->GetBrowserContext())
-              ->GetHostContentSettingsMap(),
-          CONTENT_SETTINGS_TYPE_MIDI_SYSEX,
-          prefs::kAcceptLanguages,
-          Profile::FromBrowserContext(tab->GetBrowserContext())->GetPrefs()),
+          HostContentSettingsMapFactory::GetForProfile(
+              Profile::FromBrowserContext(tab->GetBrowserContext())),
+          CONTENT_SETTINGS_TYPE_MIDI_SYSEX),
       pending_protocol_handler_(ProtocolHandler::EmptyProtocolHandler()),
       previous_protocol_handler_(ProtocolHandler::EmptyProtocolHandler()),
       pending_protocol_handler_setting_(CONTENT_SETTING_DEFAULT),
       load_plugins_link_enabled_(true),
       microphone_camera_state_(MICROPHONE_CAMERA_NOT_ACCESSED),
+      subresource_filter_enabled_(false),
+      subresource_filter_blockage_indicated_(false),
       observer_(this) {
-  ClearBlockedContentSettingsExceptForCookies();
-  ClearCookieSpecificContentSettings();
+  ClearContentSettingsExceptForNavigationRelatedSettings();
+  ClearNavigationRelatedContentSettings();
 
-  observer_.Add(Profile::FromBrowserContext(tab->GetBrowserContext())
-                    ->GetHostContentSettingsMap());
+  observer_.Add(HostContentSettingsMapFactory::GetForProfile(
+      Profile::FromBrowserContext(tab->GetBrowserContext())));
 }
 
 TabSpecificContentSettings::~TabSpecificContentSettings() {
-  FOR_EACH_OBSERVER(
-      SiteDataObserver, observer_list_, ContentSettingsDestroyed());
+  for (SiteDataObserver& observer : observer_list_)
+    observer.ContentSettingsDestroyed();
 }
 
 TabSpecificContentSettings* TabSpecificContentSettings::GetForFrame(
@@ -131,15 +133,14 @@ TabSpecificContentSettings* TabSpecificContentSettings::GetForFrame(
 }
 
 // static
-void TabSpecificContentSettings::CookiesRead(int render_process_id,
-                                             int render_frame_id,
-                                             const GURL& url,
-                                             const GURL& frame_url,
-                                             const net::CookieList& cookie_list,
-                                             bool blocked_by_policy) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  TabSpecificContentSettings* settings =
-      GetForFrame(render_process_id, render_frame_id);
+void TabSpecificContentSettings::CookiesRead(
+    const base::Callback<content::WebContents*(void)>& wc_getter,
+    const GURL& url,
+    const GURL& frame_url,
+    const net::CookieList& cookie_list,
+    bool blocked_by_policy) {
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  TabSpecificContentSettings* settings = GetForWCGetter(wc_getter);
   if (settings) {
     settings->OnCookiesRead(url, frame_url, cookie_list,
                             blocked_by_policy);
@@ -148,16 +149,14 @@ void TabSpecificContentSettings::CookiesRead(int render_process_id,
 
 // static
 void TabSpecificContentSettings::CookieChanged(
-    int render_process_id,
-    int render_frame_id,
+    const base::Callback<WebContents*(void)>& wc_getter,
     const GURL& url,
     const GURL& frame_url,
     const std::string& cookie_line,
     const net::CookieOptions& options,
     bool blocked_by_policy) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  TabSpecificContentSettings* settings =
-      GetForFrame(render_process_id, render_frame_id);
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  TabSpecificContentSettings* settings = GetForWCGetter(wc_getter);
   if (settings)
     settings->OnCookieChanged(url, frame_url, cookie_line, options,
                               blocked_by_policy);
@@ -171,7 +170,7 @@ void TabSpecificContentSettings::WebDatabaseAccessed(
     const base::string16& name,
     const base::string16& display_name,
     bool blocked_by_policy) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
   TabSpecificContentSettings* settings = GetForFrame(
       render_process_id, render_frame_id);
   if (settings)
@@ -184,7 +183,7 @@ void TabSpecificContentSettings::DOMStorageAccessed(int render_process_id,
                                                     const GURL& url,
                                                     bool local,
                                                     bool blocked_by_policy) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
   TabSpecificContentSettings* settings = GetForFrame(
       render_process_id, render_frame_id);
   if (settings)
@@ -198,7 +197,7 @@ void TabSpecificContentSettings::IndexedDBAccessed(
     const GURL& url,
     const base::string16& description,
     bool blocked_by_policy) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
   TabSpecificContentSettings* settings = GetForFrame(
       render_process_id, render_frame_id);
   if (settings)
@@ -210,7 +209,7 @@ void TabSpecificContentSettings::FileSystemAccessed(int render_process_id,
                                                     int render_frame_id,
                                                     const GURL& url,
                                                     bool blocked_by_policy) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  DCHECK_CURRENTLY_ON(BrowserThread::UI);
   TabSpecificContentSettings* settings = GetForFrame(
       render_process_id, render_frame_id);
   if (settings)
@@ -218,22 +217,25 @@ void TabSpecificContentSettings::FileSystemAccessed(int render_process_id,
 }
 
 // static
-void TabSpecificContentSettings::ServiceWorkerAccessed(int render_process_id,
-                                                       int render_frame_id,
-                                                       const GURL& scope,
-                                                       bool blocked_by_policy) {
+void TabSpecificContentSettings::ServiceWorkerAccessed(
+    int render_process_id,
+    int render_frame_id,
+    const GURL& scope,
+    bool blocked_by_policy_javascript,
+    bool blocked_by_policy_cookie) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   TabSpecificContentSettings* settings =
       GetForFrame(render_process_id, render_frame_id);
   if (settings)
-    settings->OnServiceWorkerAccessed(scope, blocked_by_policy);
+    settings->OnServiceWorkerAccessed(scope, blocked_by_policy_javascript,
+                                      blocked_by_policy_cookie);
 }
 
 bool TabSpecificContentSettings::IsContentBlocked(
     ContentSettingsType content_type) const {
-  DCHECK(content_type != CONTENT_SETTINGS_TYPE_GEOLOCATION)
+  DCHECK_NE(CONTENT_SETTINGS_TYPE_GEOLOCATION, content_type)
       << "Geolocation settings handled by ContentSettingGeolocationImageModel";
-  DCHECK(content_type != CONTENT_SETTINGS_TYPE_NOTIFICATIONS)
+  DCHECK_NE(CONTENT_SETTINGS_TYPE_NOTIFICATIONS, content_type)
       << "Notifications settings handled by "
       << "ContentSettingsNotificationsImageModel";
 
@@ -243,34 +245,50 @@ bool TabSpecificContentSettings::IsContentBlocked(
       content_type == CONTENT_SETTINGS_TYPE_COOKIES ||
       content_type == CONTENT_SETTINGS_TYPE_POPUPS ||
       content_type == CONTENT_SETTINGS_TYPE_MIXEDSCRIPT ||
-      content_type == CONTENT_SETTINGS_TYPE_MEDIASTREAM ||
       content_type == CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC ||
       content_type == CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA ||
       content_type == CONTENT_SETTINGS_TYPE_PPAPI_BROKER ||
       content_type == CONTENT_SETTINGS_TYPE_AUTOMATIC_DOWNLOADS ||
-      content_type == CONTENT_SETTINGS_TYPE_MIDI_SYSEX) {
-    return content_blocked_[content_type];
+      content_type == CONTENT_SETTINGS_TYPE_MIDI_SYSEX ||
+      content_type == CONTENT_SETTINGS_TYPE_KEYGEN) {
+    const auto& it = content_settings_status_.find(content_type);
+    if (it != content_settings_status_.end())
+      return it->second.blocked;
   }
 
   return false;
 }
 
+bool TabSpecificContentSettings::IsSubresourceBlocked() const {
+  return subresource_filter_enabled_;
+}
+
 bool TabSpecificContentSettings::IsBlockageIndicated(
     ContentSettingsType content_type) const {
-  return content_blockage_indicated_to_user_[content_type];
+  const auto& it = content_settings_status_.find(content_type);
+  if (it != content_settings_status_.end())
+    return it->second.blockage_indicated_to_user;
+  return false;
+}
+
+bool TabSpecificContentSettings::IsSubresourceBlockageIndicated() const {
+  return subresource_filter_blockage_indicated_;
 }
 
 void TabSpecificContentSettings::SetBlockageHasBeenIndicated(
     ContentSettingsType content_type) {
-  content_blockage_indicated_to_user_[content_type] = true;
+  content_settings_status_[content_type].blockage_indicated_to_user = true;
+}
+
+void TabSpecificContentSettings::SetSubresourceBlockageIndicated() {
+  subresource_filter_blockage_indicated_ = true;
 }
 
 bool TabSpecificContentSettings::IsContentAllowed(
     ContentSettingsType content_type) const {
   // This method currently only returns meaningful values for the content type
-  // cookies, mediastream, PPAPI broker, and downloads.
+  // cookies, media, PPAPI broker, downloads, and MIDI sysex.
   if (content_type != CONTENT_SETTINGS_TYPE_COOKIES &&
-      content_type != CONTENT_SETTINGS_TYPE_MEDIASTREAM &&
       content_type != CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC &&
       content_type != CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA &&
       content_type != CONTENT_SETTINGS_TYPE_PPAPI_BROKER &&
@@ -279,7 +297,10 @@ bool TabSpecificContentSettings::IsContentAllowed(
     return false;
   }
 
-  return content_allowed_[content_type];
+  const auto& it = content_settings_status_.find(content_type);
+  if (it != content_settings_status_.end())
+    return it->second.allowed;
+  return false;
 }
 
 void TabSpecificContentSettings::OnContentBlocked(ContentSettingsType type) {
@@ -294,17 +315,19 @@ void TabSpecificContentSettings::OnContentBlockedWithDetail(
   DCHECK(type != CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC &&
          type != CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA)
       << "Media stream settings handled by OnMediaStreamPermissionSet";
-  if (type < 0 || type >= CONTENT_SETTINGS_NUM_TYPES)
+  if (!content_settings::ContentSettingsRegistry::GetInstance()->Get(type))
     return;
 
   // TODO(robwu): Should this be restricted to cookies only?
-  // In the past, content_allowed_ was set to false, but this logic was inverted
-  // in https://codereview.chromium.org/13375004 to fix an issue with the cookie
-  // permission UI. This unconditional assignment seems incorrect, because the
-  // flag will now always be true after calling either OnContentBlocked or
-  // OnContentAllowed. Consequently IsContentAllowed will always return true
-  // for every supported setting that is not handled elsewhere.
-  content_allowed_[type] = true;
+  // In the past, content_settings_status_[type].allowed was set to false, but
+  // this logic was inverted in https://codereview.chromium.org/13375004 to
+  // fix an issue with the cookie permission UI. This unconditional assignment
+  // seems incorrect, because the flag will now always be true after calling
+  // either OnContentBlocked or OnContentAllowed. Consequently IsContentAllowed
+  // will always return true for every supported setting that is not handled
+  // elsewhere.
+  ContentSettingsStatus& status = content_settings_status_[type];
+  status.allowed = true;
 
 #if defined(OS_ANDROID)
   if (type == CONTENT_SETTINGS_TYPE_POPUPS) {
@@ -312,8 +335,8 @@ void TabSpecificContentSettings::OnContentBlockedWithDetail(
     // visible for blocked popups.  Instead we have info bars which could be
     // dismissed.  Have to clear the blocked state so we properly notify the
     // relevant pieces again.
-    content_blocked_[type] = false;
-    content_blockage_indicated_to_user_[type] = false;
+    status.blocked = false;
+    status.blockage_indicated_to_user = false;
   }
 #endif
 
@@ -323,8 +346,8 @@ void TabSpecificContentSettings::OnContentBlockedWithDetail(
     blocked_plugin_names_.push_back(details);
   }
 
-  if (!content_blocked_[type]) {
-    content_blocked_[type] = true;
+  if (!status.blocked) {
+    status.blocked = true;
     // TODO: it would be nice to have a way of mocking this in tests.
     content::NotificationService::current()->Notify(
         chrome::NOTIFICATION_WEB_CONTENT_SETTINGS_CHANGED,
@@ -334,9 +357,9 @@ void TabSpecificContentSettings::OnContentBlockedWithDetail(
     if (type == CONTENT_SETTINGS_TYPE_MIXEDSCRIPT) {
       content_settings::RecordMixedScriptAction(
           content_settings::MIXED_SCRIPT_ACTION_DISPLAYED_SHIELD);
-      content_settings::RecordMixedScriptActionWithRAPPOR(
-          content_settings::MIXED_SCRIPT_ACTION_DISPLAYED_SHIELD,
-          GURL(base::UTF16ToUTF8(details)));
+    } else if (type == CONTENT_SETTINGS_TYPE_PLUGINS) {
+      content_settings::RecordPluginsAction(
+          content_settings::PLUGINS_ACTION_DISPLAYED_BLOCKED_ICON_IN_OMNIBOX);
     }
   }
 }
@@ -348,19 +371,21 @@ void TabSpecificContentSettings::OnContentAllowed(ContentSettingsType type) {
          type != CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA)
       << "Media stream settings handled by OnMediaStreamPermissionSet";
   bool access_changed = false;
+  ContentSettingsStatus& status = content_settings_status_[type];
 #if defined(OS_ANDROID)
   if (type == CONTENT_SETTINGS_TYPE_PROTECTED_MEDIA_IDENTIFIER &&
-    content_blocked_[type]) {
-    // content_allowed_[type] is always set to true in OnContentBlocked, so we
-    // have to use content_blocked_ to detect whether the protected media
-    // setting has changed.
-    content_blocked_[type] = false;
+      status.blocked) {
+    // content_settings_status_[type].allowed is always set to true in
+    // OnContentBlocked, so we have to use
+    // content_settings_status_[type].blocked to detect whether the protected
+    // media setting has changed.
+    status.blocked = false;
     access_changed = true;
   }
 #endif
 
-  if (!content_allowed_[type]) {
-    content_allowed_[type] = true;
+  if (!status.allowed) {
+    status.allowed = true;
     access_changed = true;
   }
 
@@ -448,15 +473,25 @@ void TabSpecificContentSettings::OnLocalStorageAccessed(
 
 void TabSpecificContentSettings::OnServiceWorkerAccessed(
     const GURL& scope,
-    bool blocked_by_policy) {
+    bool blocked_by_policy_javascript,
+    bool blocked_by_policy_cookie) {
   DCHECK(scope.is_valid());
-  if (blocked_by_policy) {
+  if (blocked_by_policy_javascript || blocked_by_policy_cookie) {
     blocked_local_shared_objects_.service_workers()->AddServiceWorker(
         scope.GetOrigin(), std::vector<GURL>(1, scope));
-    OnContentBlocked(CONTENT_SETTINGS_TYPE_COOKIES);
   } else {
     allowed_local_shared_objects_.service_workers()->AddServiceWorker(
         scope.GetOrigin(), std::vector<GURL>(1, scope));
+  }
+
+  if (blocked_by_policy_javascript) {
+    OnContentBlocked(CONTENT_SETTINGS_TYPE_JAVASCRIPT);
+  } else {
+    OnContentAllowed(CONTENT_SETTINGS_TYPE_JAVASCRIPT);
+  }
+  if (blocked_by_policy_cookie) {
+    OnContentBlocked(CONTENT_SETTINGS_TYPE_COOKIES);
+  } else {
     OnContentAllowed(CONTENT_SETTINGS_TYPE_COOKIES);
   }
 }
@@ -505,6 +540,16 @@ void TabSpecificContentSettings::OnGeolocationPermissionSet(
       content::NotificationService::NoDetails());
 }
 
+void TabSpecificContentSettings::OnDidUseKeygen(const GURL& origin_url) {
+  HostContentSettingsMap* map = HostContentSettingsMapFactory::GetForProfile(
+      Profile::FromBrowserContext(web_contents()->GetBrowserContext()));
+  GURL url = web_contents()->GetLastCommittedURL();
+  if (map->GetContentSetting(url, url, CONTENT_SETTINGS_TYPE_KEYGEN,
+                             std::string()) != CONTENT_SETTING_ALLOW) {
+    OnContentBlocked(CONTENT_SETTINGS_TYPE_KEYGEN);
+  }
+}
+
 #if defined(OS_ANDROID) || defined(OS_CHROMEOS)
 void TabSpecificContentSettings::OnProtectedMediaIdentifierPermissionSet(
     const GURL& requesting_origin,
@@ -582,16 +627,20 @@ void TabSpecificContentSettings::OnMediaStreamPermissionSet(
     media_stream_requested_audio_device_ = media_stream_requested_audio_device;
     media_stream_selected_audio_device_ = media_stream_selected_audio_device;
     bool mic_blocked = (new_microphone_camera_state & MICROPHONE_BLOCKED) != 0;
-    content_allowed_[CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC] = !mic_blocked;
-    content_blocked_[CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC] = mic_blocked;
+    ContentSettingsStatus& status =
+        content_settings_status_[CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC];
+    status.allowed = !mic_blocked;
+    status.blocked = mic_blocked;
   }
 
   if (new_microphone_camera_state & CAMERA_ACCESSED) {
     media_stream_requested_video_device_ = media_stream_requested_video_device;
     media_stream_selected_video_device_ = media_stream_selected_video_device;
     bool cam_blocked = (new_microphone_camera_state & CAMERA_BLOCKED) != 0;
-    content_allowed_[CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA] = !cam_blocked;
-    content_blocked_[CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA] = cam_blocked;
+    ContentSettingsStatus& status =
+        content_settings_status_[CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA];
+    status.allowed = !cam_blocked;
+    status.blocked = cam_blocked;
   }
 
   if (microphone_camera_state_ != new_microphone_camera_state) {
@@ -615,13 +664,15 @@ void TabSpecificContentSettings::OnMidiSysExAccessBlocked(
   OnContentBlocked(CONTENT_SETTINGS_TYPE_MIDI_SYSEX);
 }
 
-void TabSpecificContentSettings::ClearBlockedContentSettingsExceptForCookies() {
-  for (size_t i = 0; i < arraysize(content_blocked_); ++i) {
-    if (i == CONTENT_SETTINGS_TYPE_COOKIES)
+void TabSpecificContentSettings::
+ClearContentSettingsExceptForNavigationRelatedSettings() {
+  for (auto& status : content_settings_status_) {
+    if (status.first == CONTENT_SETTINGS_TYPE_COOKIES ||
+        status.first == CONTENT_SETTINGS_TYPE_JAVASCRIPT)
       continue;
-    content_blocked_[i] = false;
-    content_allowed_[i] = false;
-    content_blockage_indicated_to_user_[i] = false;
+    status.second.blocked = false;
+    status.second.blockage_indicated_to_user = false;
+    status.second.allowed = false;
   }
   microphone_camera_state_ = MICROPHONE_CAMERA_NOT_ACCESSED;
   load_plugins_link_enabled_ = true;
@@ -631,23 +682,34 @@ void TabSpecificContentSettings::ClearBlockedContentSettingsExceptForCookies() {
       content::NotificationService::NoDetails());
 }
 
-void TabSpecificContentSettings::ClearCookieSpecificContentSettings() {
+void TabSpecificContentSettings::ClearNavigationRelatedContentSettings() {
   blocked_local_shared_objects_.Reset();
   allowed_local_shared_objects_.Reset();
-  content_blocked_[CONTENT_SETTINGS_TYPE_COOKIES] = false;
-  content_allowed_[CONTENT_SETTINGS_TYPE_COOKIES] = false;
-  content_blockage_indicated_to_user_[CONTENT_SETTINGS_TYPE_COOKIES] = false;
+  for (ContentSettingsType type :
+    {CONTENT_SETTINGS_TYPE_COOKIES, CONTENT_SETTINGS_TYPE_JAVASCRIPT}) {
+    ContentSettingsStatus& status =
+        content_settings_status_[type];
+    status.blocked = false;
+    status.blockage_indicated_to_user = false;
+    status.allowed = false;
+  }
   content::NotificationService::current()->Notify(
       chrome::NOTIFICATION_WEB_CONTENT_SETTINGS_CHANGED,
       content::Source<WebContents>(web_contents()),
       content::NotificationService::NoDetails());
 }
 
+void TabSpecificContentSettings::FlashDownloadBlocked() {
+  OnContentBlockedWithDetail(CONTENT_SETTINGS_TYPE_PLUGINS,
+                             base::UTF8ToUTF16(content::kFlashPluginName));
+}
+
 void TabSpecificContentSettings::SetDownloadsBlocked(bool blocked) {
-  content_blocked_[CONTENT_SETTINGS_TYPE_AUTOMATIC_DOWNLOADS] = blocked;
-  content_allowed_[CONTENT_SETTINGS_TYPE_AUTOMATIC_DOWNLOADS] = !blocked;
-  content_blockage_indicated_to_user_[
-    CONTENT_SETTINGS_TYPE_AUTOMATIC_DOWNLOADS] = false;
+  ContentSettingsStatus& status =
+      content_settings_status_[CONTENT_SETTINGS_TYPE_AUTOMATIC_DOWNLOADS];
+  status.blocked = blocked;
+  status.allowed = !blocked;
+  status.blockage_indicated_to_user = false;
   content::NotificationService::current()->Notify(
       chrome::NOTIFICATION_WEB_CONTENT_SETTINGS_CHANGED,
       content::Source<WebContents>(web_contents()),
@@ -655,14 +717,20 @@ void TabSpecificContentSettings::SetDownloadsBlocked(bool blocked) {
 }
 
 void TabSpecificContentSettings::SetPopupsBlocked(bool blocked) {
-  content_blocked_[CONTENT_SETTINGS_TYPE_POPUPS] = blocked;
-  content_blockage_indicated_to_user_[CONTENT_SETTINGS_TYPE_POPUPS] = false;
+  ContentSettingsStatus& status =
+      content_settings_status_[CONTENT_SETTINGS_TYPE_POPUPS];
+  status.blocked = blocked;
+  status.blockage_indicated_to_user = false;
   content::NotificationService::current()->Notify(
       chrome::NOTIFICATION_WEB_CONTENT_SETTINGS_CHANGED,
       content::Source<WebContents>(web_contents()),
       content::NotificationService::NoDetails());
 }
 
+void TabSpecificContentSettings::SetSubresourceBlocked(bool enabled) {
+  subresource_filter_enabled_ = enabled;
+}
+
 void TabSpecificContentSettings::SetPepperBrokerAllowed(bool allowed) {
   if (allowed) {
     OnContentAllowed(CONTENT_SETTINGS_TYPE_PPAPI_BROKER);
@@ -689,7 +757,8 @@ void TabSpecificContentSettings::OnContentSettingChanged(
       details.primary_pattern().Matches(entry_url)) {
     Profile* profile =
         Profile::FromBrowserContext(web_contents()->GetBrowserContext());
-    const HostContentSettingsMap* map = profile->GetHostContentSettingsMap();
+    const HostContentSettingsMap* map =
+        HostContentSettingsMapFactory::GetForProfile(profile);
 
     if (content_type == CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC ||
         content_type == CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA) {
@@ -698,8 +767,9 @@ void TabSpecificContentSettings::OnContentSettingChanged(
                                                       media_origin,
                                                       content_type,
                                                       std::string());
-      content_allowed_[content_type] = setting == CONTENT_SETTING_ALLOW;
-      content_blocked_[content_type] = setting == CONTENT_SETTING_BLOCK;
+      ContentSettingsStatus& status = content_settings_status_[content_type];
+      status.allowed = setting == CONTENT_SETTING_ALLOW;
+      status.blocked = setting == CONTENT_SETTING_BLOCK;
     }
     RendererContentSettingRules rules;
     GetRendererContentSettingRules(map, &rules);
@@ -722,41 +792,54 @@ bool TabSpecificContentSettings::OnMessageReceived(
   IPC_BEGIN_MESSAGE_MAP(TabSpecificContentSettings, message)
     IPC_MESSAGE_HANDLER(ChromeViewHostMsg_ContentBlocked,
                         OnContentBlockedWithDetail)
+    IPC_MESSAGE_HANDLER(ChromeViewHostMsg_DidUseKeygen, OnDidUseKeygen)
     IPC_MESSAGE_UNHANDLED(handled = false)
   IPC_END_MESSAGE_MAP()
   return handled;
 }
 
-void TabSpecificContentSettings::DidNavigateMainFrame(
-    const content::LoadCommittedDetails& details,
-    const content::FrameNavigateParams& params) {
-  if (!details.is_in_page) {
-    // Clear "blocked" flags.
-    ClearBlockedContentSettingsExceptForCookies();
-    blocked_plugin_names_.clear();
-    GeolocationDidNavigate(details);
-    MidiDidNavigate(details);
-  }
-}
-
-void TabSpecificContentSettings::DidStartProvisionalLoadForFrame(
-    content::RenderFrameHost* render_frame_host,
-    const GURL& validated_url,
-    bool is_error_page,
-    bool is_iframe_srcdoc) {
-  if (render_frame_host->GetParent())
+void TabSpecificContentSettings::DidStartNavigation(
+    content::NavigationHandle* navigation_handle) {
+  if (!navigation_handle->IsInMainFrame() || navigation_handle->IsSamePage())
     return;
 
+  const content::NavigationController& controller =
+      web_contents()->GetController();
+  content::NavigationEntry* last_committed_entry =
+      controller.GetLastCommittedEntry();
+  if (last_committed_entry)
+    previous_url_ = last_committed_entry->GetURL();
+
   // If we're displaying a network error page do not reset the content
   // settings delegate's cookies so the user has a chance to modify cookie
   // settings.
-  if (!is_error_page)
-    ClearCookieSpecificContentSettings();
+  if (!navigation_handle->IsErrorPage())
+    ClearNavigationRelatedContentSettings();
   ClearGeolocationContentSettings();
   ClearMidiContentSettings();
   ClearPendingProtocolHandler();
 }
 
+void TabSpecificContentSettings::DidFinishNavigation(
+      content::NavigationHandle* navigation_handle) {
+  if (!navigation_handle->IsInMainFrame() ||
+      !navigation_handle->HasCommitted() ||
+      navigation_handle->IsSamePage()) {
+    return;
+  }
+
+  // Clear "blocked" flags.
+  ClearContentSettingsExceptForNavigationRelatedSettings();
+  blocked_plugin_names_.clear();
+  GeolocationDidNavigate(navigation_handle);
+  MidiDidNavigate(navigation_handle);
+
+  if (web_contents()->GetVisibleURL().SchemeIsHTTPOrHTTPS()) {
+    content_settings::RecordPluginsAction(
+        content_settings::PLUGINS_ACTION_TOTAL_NAVIGATIONS);
+  }
+}
+
 void TabSpecificContentSettings::AppCacheAccessed(const GURL& manifest_url,
                                                   bool blocked_by_policy) {
   if (blocked_by_policy) {
@@ -779,7 +862,8 @@ void TabSpecificContentSettings::RemoveSiteDataObserver(
 }
 
 void TabSpecificContentSettings::NotifySiteDataObservers() {
-  FOR_EACH_OBSERVER(SiteDataObserver, observer_list_, OnSiteDataAccessed());
+  for (SiteDataObserver& observer : observer_list_)
+    observer.OnSiteDataAccessed();
 }
 
 void TabSpecificContentSettings::ClearGeolocationContentSettings() {
@@ -791,11 +875,45 @@ void TabSpecificContentSettings::ClearMidiContentSettings() {
 }
 
 void TabSpecificContentSettings::GeolocationDidNavigate(
-    const content::LoadCommittedDetails& details) {
-  geolocation_usages_state_.DidNavigate(GetCommittedDetails(details));
+    content::NavigationHandle* navigation_handle) {
+  ContentSettingsUsagesState::CommittedDetails committed_details;
+  committed_details.current_url = navigation_handle->GetURL();
+  committed_details.previous_url = previous_url_;
+
+  geolocation_usages_state_.DidNavigate(committed_details);
 }
 
 void TabSpecificContentSettings::MidiDidNavigate(
-    const content::LoadCommittedDetails& details) {
-  midi_usages_state_.DidNavigate(GetCommittedDetails(details));
+    content::NavigationHandle* navigation_handle) {
+  ContentSettingsUsagesState::CommittedDetails committed_details;
+  committed_details.current_url = navigation_handle->GetURL();
+  committed_details.previous_url = previous_url_;
+  midi_usages_state_.DidNavigate(committed_details);
+}
+
+void TabSpecificContentSettings::BlockAllContentForTesting() {
+  content_settings::ContentSettingsRegistry* registry =
+      content_settings::ContentSettingsRegistry::GetInstance();
+  for (const content_settings::ContentSettingsInfo* info : *registry) {
+    ContentSettingsType type = info->website_settings_info()->type();
+    if (type != CONTENT_SETTINGS_TYPE_GEOLOCATION &&
+        type != CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC &&
+        type != CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA) {
+      OnContentBlocked(type);
+    }
+  }
+
+  // Geolocation and media must be blocked separately, as the generic
+  // TabSpecificContentSettings::OnContentBlocked does not apply to them.
+  OnGeolocationPermissionSet(web_contents()->GetLastCommittedURL(), false);
+  MicrophoneCameraStateFlags media_blocked =
+      static_cast<MicrophoneCameraStateFlags>(
+          TabSpecificContentSettings::MICROPHONE_ACCESSED |
+          TabSpecificContentSettings::MICROPHONE_BLOCKED |
+          TabSpecificContentSettings::CAMERA_ACCESSED |
+          TabSpecificContentSettings::CAMERA_BLOCKED);
+  OnMediaStreamPermissionSet(
+      web_contents()->GetLastCommittedURL(),
+      media_blocked,
+      std::string(), std::string(), std::string(), std::string());
 }
