@@ -20,7 +20,7 @@ DevToolsNetworkController::~DevToolsNetworkController() {
 DevToolsNetworkInterceptor* DevToolsNetworkController::GetInterceptor(
     const std::string& client_id) {
   DCHECK(thread_checker_.CalledOnValidThread());
-  if (!interceptors_.size() || client_id.empty())
+  if (interceptors_.empty() || client_id.empty())
     return nullptr;
 
   DevToolsNetworkInterceptor* interceptor = interceptors_.get(client_id);
@@ -32,21 +32,20 @@ DevToolsNetworkInterceptor* DevToolsNetworkController::GetInterceptor(
 
 void DevToolsNetworkController::SetNetworkState(
     const std::string& client_id,
-    scoped_ptr<DevToolsNetworkConditions> conditions) {
+    std::unique_ptr<DevToolsNetworkConditions> conditions) {
   DCHECK(thread_checker_.CalledOnValidThread());
 
   DevToolsNetworkInterceptor* interceptor = interceptors_.get(client_id);
   if (!interceptor) {
-    DCHECK(conditions);
     if (!conditions)
       return;
-    scoped_ptr<DevToolsNetworkInterceptor> new_interceptor(
+    std::unique_ptr<DevToolsNetworkInterceptor> new_interceptor(
         new DevToolsNetworkInterceptor());
     new_interceptor->UpdateConditions(std::move(conditions));
     interceptors_.set(client_id, std::move(new_interceptor));
   } else {
     if (!conditions) {
-      scoped_ptr<DevToolsNetworkConditions> online_conditions(
+      std::unique_ptr<DevToolsNetworkConditions> online_conditions(
           new DevToolsNetworkConditions());
       interceptor->UpdateConditions(std::move(online_conditions));
       interceptors_.erase(client_id);
@@ -66,7 +65,7 @@ void DevToolsNetworkController::SetNetworkState(
 
   bool is_appcache_offline = appcache_interceptor_->IsOffline();
   if (is_appcache_offline != has_offline_interceptors) {
-    scoped_ptr<DevToolsNetworkConditions> appcache_conditions(
+    std::unique_ptr<DevToolsNetworkConditions> appcache_conditions(
         new DevToolsNetworkConditions(has_offline_interceptors));
     appcache_interceptor_->UpdateConditions(std::move(appcache_conditions));
   }
