@@ -9,7 +9,9 @@
 #include <utility>
 #include <vector>
 
+#include "base/guid.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/memory/weak_ptr.h"
 #include "base/rand_util.h"
 #include "base/stl_util.h"
@@ -21,9 +23,10 @@
 #include "content/public/test/mock_download_manager.h"
 #include "content/public/test/test_browser_thread.h"
 #include "content/public/test/test_utils.h"
+#include "extensions/features/features.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
 #include "chrome/browser/extensions/api/downloads/downloads_api.h"
 #endif
 
@@ -37,33 +40,6 @@ using testing::_;
 
 namespace {
 
-void CheckInfoEqual(const history::DownloadRow& left,
-                    const history::DownloadRow& right) {
-  EXPECT_EQ(left.current_path.value(), right.current_path.value());
-  EXPECT_EQ(left.target_path.value(), right.target_path.value());
-  EXPECT_EQ(left.url_chain.size(), right.url_chain.size());
-  for (unsigned int i = 0;
-       i < left.url_chain.size() && i < right.url_chain.size();
-       ++i) {
-    EXPECT_EQ(left.url_chain[i].spec(), right.url_chain[i].spec());
-  }
-  EXPECT_EQ(left.referrer_url.spec(), right.referrer_url.spec());
-  EXPECT_EQ(left.mime_type, right.mime_type);
-  EXPECT_EQ(left.original_mime_type, right.original_mime_type);
-  EXPECT_EQ(left.start_time.ToTimeT(), right.start_time.ToTimeT());
-  EXPECT_EQ(left.end_time.ToTimeT(), right.end_time.ToTimeT());
-  EXPECT_EQ(left.etag, right.etag);
-  EXPECT_EQ(left.last_modified, right.last_modified);
-  EXPECT_EQ(left.received_bytes, right.received_bytes);
-  EXPECT_EQ(left.total_bytes, right.total_bytes);
-  EXPECT_EQ(left.state, right.state);
-  EXPECT_EQ(left.danger_type, right.danger_type);
-  EXPECT_EQ(left.id, right.id);
-  EXPECT_EQ(left.opened, right.opened);
-  EXPECT_EQ(left.by_ext_id, right.by_ext_id);
-  EXPECT_EQ(left.by_ext_name, right.by_ext_name);
-}
-
 typedef DownloadHistory::IdSet IdSet;
 typedef std::vector<history::DownloadRow> InfoVector;
 typedef testing::StrictMock<content::MockDownloadItem> StrictMockDownloadItem;
@@ -127,7 +103,7 @@ class FakeHistoryAdapter : public DownloadHistory::HistoryAdapter {
     }
   }
 
-  void ExpectWillQueryDownloads(scoped_ptr<InfoVector> infos) {
+  void ExpectWillQueryDownloads(std::unique_ptr<InfoVector> infos) {
     DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
     expect_query_downloads_ = std::move(infos);
   }
@@ -146,27 +122,27 @@ class FakeHistoryAdapter : public DownloadHistory::HistoryAdapter {
       const history::DownloadRow& info) {
     DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
     content::RunAllPendingInMessageLoop(content::BrowserThread::UI);
-    CheckInfoEqual(info, create_download_info_);
+    EXPECT_EQ(info, create_download_info_);
     create_download_info_ = history::DownloadRow();
   }
 
   void ExpectNoDownloadCreated() {
     DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
     content::RunAllPendingInMessageLoop(content::BrowserThread::UI);
-    CheckInfoEqual(history::DownloadRow(), create_download_info_);
+    EXPECT_EQ(history::DownloadRow(), create_download_info_);
   }
 
   void ExpectDownloadUpdated(const history::DownloadRow& info) {
     DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
     content::RunAllPendingInMessageLoop(content::BrowserThread::UI);
-    CheckInfoEqual(update_download_, info);
+    EXPECT_EQ(update_download_, info);
     update_download_ = history::DownloadRow();
   }
 
   void ExpectNoDownloadUpdated() {
     DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
     content::RunAllPendingInMessageLoop(content::BrowserThread::UI);
-    CheckInfoEqual(history::DownloadRow(), update_download_);
+    EXPECT_EQ(history::DownloadRow(), update_download_);
   }
 
   void ExpectNoDownloadsRemoved() {
@@ -191,13 +167,21 @@ class FakeHistoryAdapter : public DownloadHistory::HistoryAdapter {
   bool fail_create_download_;
   base::Closure create_download_callback_;
   history::DownloadRow update_download_;
-  scoped_ptr<InfoVector> expect_query_downloads_;
+  std::unique_ptr<InfoVector> expect_query_downloads_;
   IdSet remove_downloads_;
   history::DownloadRow create_download_info_;
 
   DISALLOW_COPY_AND_ASSIGN(FakeHistoryAdapter);
 };
 
+class TestDownloadHistoryObserver : public DownloadHistory::Observer {
+ public:
+  void OnHistoryQueryComplete() override {
+    on_history_query_complete_called_ = true;
+  }
+  bool on_history_query_complete_called_ = false;
+};
+
 class DownloadHistoryTest : public testing::Test {
  public:
   // Generic callback that receives a pointer to a StrictMockDownloadItem.
@@ -209,7 +193,7 @@ class DownloadHistoryTest : public testing::Test {
         history_(NULL),
         manager_observer_(NULL),
         download_created_index_(0) {}
-  ~DownloadHistoryTest() override { STLDeleteElements(&items_); }
+  ~DownloadHistoryTest() override {}
 
  protected:
   void TearDown() override { download_history_.reset(); }
@@ -226,7 +210,7 @@ class DownloadHistoryTest : public testing::Test {
     return manager_observer_;
   }
 
-  void CreateDownloadHistory(scoped_ptr<InfoVector> infos) {
+  void CreateDownloadHistory(std::unique_ptr<InfoVector> infos) {
     DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
     CHECK(infos.get());
     EXPECT_CALL(manager(), AddObserver(_)).WillOnce(WithArg<0>(Invoke(
@@ -236,19 +220,11 @@ class DownloadHistoryTest : public testing::Test {
     for (size_t index = 0; index < infos->size(); ++index) {
       const history::DownloadRow& row = infos->at(index);
       content::MockDownloadManager::CreateDownloadItemAdapter adapter(
-          history::ToContentDownloadId(row.id),
-          row.current_path,
-          row.target_path,
-          row.url_chain,
-          row.referrer_url,
-          row.mime_type,
-          row.original_mime_type,
-          row.start_time,
-          row.end_time,
-          row.etag,
-          row.last_modified,
-          row.received_bytes,
-          row.total_bytes,
+          row.guid, history::ToContentDownloadId(row.id), row.current_path,
+          row.target_path, row.url_chain, row.referrer_url, row.site_url,
+          row.tab_url, row.tab_referrer_url, row.mime_type,
+          row.original_mime_type, row.start_time, row.end_time, row.etag,
+          row.last_modified, row.received_bytes, row.total_bytes, std::string(),
           history::ToContentDownloadState(row.state),
           history::ToContentDownloadDangerType(row.danger_type),
           history::ToContentDownloadInterruptReason(row.interrupt_reason),
@@ -264,7 +240,8 @@ class DownloadHistoryTest : public testing::Test {
     history_->ExpectWillQueryDownloads(std::move(infos));
     EXPECT_CALL(*manager_.get(), GetAllDownloads(_)).WillRepeatedly(Return());
     download_history_.reset(new DownloadHistory(
-        &manager(), scoped_ptr<DownloadHistory::HistoryAdapter>(history_)));
+        &manager(),
+        std::unique_ptr<DownloadHistory::HistoryAdapter>(history_)));
     content::RunAllPendingInMessageLoop(content::BrowserThread::UI);
     history_->ExpectQueryDownloadsDone();
   }
@@ -351,8 +328,10 @@ class DownloadHistoryTest : public testing::Test {
     GURL referrer(referrer_string);
     std::vector<GURL> url_chain;
     url_chain.push_back(url);
-    InitItem(static_cast<uint32_t>(items_.size() + 1), base::FilePath(path),
-             base::FilePath(path), url_chain, referrer,
+    InitItem(base::GenerateGUID(), static_cast<uint32_t>(items_.size() + 1),
+             base::FilePath(path), base::FilePath(path), url_chain, referrer,
+             GURL("http://example.com"), GURL("http://example.com/tab-url"),
+             GURL("http://example.com/tab-referrer-url"),
              "application/octet-stream", "application/octet-stream",
              (base::Time::Now() - base::TimeDelta::FromMinutes(10)),
              (base::Time::Now() - base::TimeDelta::FromMinutes(1)), "Etag",
@@ -362,11 +341,15 @@ class DownloadHistoryTest : public testing::Test {
              std::string(), info);
   }
 
-  void InitItem(uint32_t id,
+  void InitItem(const std::string& guid,
+                uint32_t id,
                 const base::FilePath& current_path,
                 const base::FilePath& target_path,
                 const std::vector<GURL>& url_chain,
                 const GURL& referrer,
+                const GURL& site_url,
+                const GURL& tab_url,
+                const GURL& tab_referrer_url,
                 const std::string& mime_type,
                 const std::string& original_mime_type,
                 const base::Time& start_time,
@@ -385,13 +368,15 @@ class DownloadHistoryTest : public testing::Test {
     DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
 
     size_t index = items_.size();
-    StrictMockDownloadItem* mock_item = new StrictMockDownloadItem();
-    items_.push_back(mock_item);
+    items_.push_back(base::MakeUnique<StrictMockDownloadItem>());
 
     info->current_path = current_path;
     info->target_path = target_path;
     info->url_chain = url_chain;
     info->referrer_url = referrer;
+    info->site_url = site_url;
+    info->tab_url = tab_url;
+    info->tab_referrer_url = tab_referrer_url;
     info->mime_type = mime_type;
     info->original_mime_type = original_mime_type;
     info->start_time = start_time;
@@ -405,11 +390,13 @@ class DownloadHistoryTest : public testing::Test {
     info->interrupt_reason =
         history::ToHistoryDownloadInterruptReason(interrupt_reason);
     info->id = history::ToHistoryDownloadId(id);
+    info->guid = guid;
     info->opened = opened;
     info->by_ext_id = by_extension_id;
     info->by_ext_name = by_extension_name;
 
     EXPECT_CALL(item(index), GetId()).WillRepeatedly(Return(id));
+    EXPECT_CALL(item(index), GetGuid()).WillRepeatedly(ReturnRefOfCopy(guid));
     EXPECT_CALL(item(index), GetFullPath())
         .WillRepeatedly(ReturnRefOfCopy(current_path));
     EXPECT_CALL(item(index), GetTargetFilePath())
@@ -424,6 +411,12 @@ class DownloadHistoryTest : public testing::Test {
         original_mime_type));
     EXPECT_CALL(item(index), GetReferrerUrl())
         .WillRepeatedly(ReturnRefOfCopy(referrer));
+    EXPECT_CALL(item(index), GetSiteUrl())
+        .WillRepeatedly(ReturnRefOfCopy(site_url));
+    EXPECT_CALL(item(index), GetTabUrl())
+        .WillRepeatedly(ReturnRefOfCopy(tab_url));
+    EXPECT_CALL(item(index), GetTabReferrerUrl())
+        .WillRepeatedly(ReturnRefOfCopy(tab_referrer_url));
     EXPECT_CALL(item(index), GetStartTime()).WillRepeatedly(Return(start_time));
     EXPECT_CALL(item(index), GetEndTime()).WillRepeatedly(Return(end_time));
     EXPECT_CALL(item(index), GetETag()).WillRepeatedly(ReturnRefOfCopy(etag));
@@ -445,7 +438,7 @@ class DownloadHistoryTest : public testing::Test {
     EXPECT_CALL(manager(), GetDownload(id))
         .WillRepeatedly(Return(&item(index)));
     EXPECT_CALL(item(index), IsTemporary()).WillRepeatedly(Return(false));
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
     new extensions::DownloadedByExtension(
         &item(index), by_extension_id, by_extension_name);
 #endif
@@ -467,10 +460,10 @@ class DownloadHistoryTest : public testing::Test {
 
   base::MessageLoopForUI loop_;
   content::TestBrowserThread ui_thread_;
-  std::vector<StrictMockDownloadItem*> items_;
-  scoped_ptr<content::MockDownloadManager> manager_;
+  std::vector<std::unique_ptr<StrictMockDownloadItem>> items_;
+  std::unique_ptr<content::MockDownloadManager> manager_;
   FakeHistoryAdapter* history_;
-  scoped_ptr<DownloadHistory> download_history_;
+  std::unique_ptr<DownloadHistory> download_history_;
   content::DownloadManager::Observer* manager_observer_;
   size_t download_created_index_;
   DownloadItemCallback pre_on_create_handler_;
@@ -490,7 +483,7 @@ TEST_F(DownloadHistoryTest, DownloadHistoryTest_Load) {
                 "http://example.com/referrer.html",
                 &info);
   {
-    scoped_ptr<InfoVector> infos(new InfoVector());
+    std::unique_ptr<InfoVector> infos(new InfoVector());
     infos->push_back(info);
     CreateDownloadHistory(std::move(infos));
     ExpectNoDownloadCreated();
@@ -510,6 +503,55 @@ TEST_F(DownloadHistoryTest, DownloadHistoryTest_Load) {
   ExpectDownloadsRemoved(ids);
 }
 
+// Test that the OnHistoryQueryComplete() observer method is invoked for an
+// observer that was added before the initial history query completing.
+TEST_F(DownloadHistoryTest, DownloadHistoryTest_OnHistoryQueryComplete_Pre) {
+  class TestHistoryAdapter : public DownloadHistory::HistoryAdapter {
+   public:
+    TestHistoryAdapter(
+        history::HistoryService::DownloadQueryCallback* callback_storage)
+        : HistoryAdapter(nullptr),
+          query_callback_(callback_storage) {}
+    void QueryDownloads(const history::HistoryService::DownloadQueryCallback&
+                            callback) override {
+      *query_callback_ = callback;
+    }
+
+    history::HistoryService::DownloadQueryCallback* query_callback_;
+  };
+
+  TestDownloadHistoryObserver observer;
+  history::HistoryService::DownloadQueryCallback query_callback;
+  std::unique_ptr<TestHistoryAdapter> test_history_adapter(
+      new TestHistoryAdapter(&query_callback));
+
+  // Create a new DownloadHistory object. This should cause
+  // TestHistoryAdapter::QueryDownloads() to be called. The TestHistoryAdapter
+  // stored the completion callback.
+  std::unique_ptr<DownloadHistory> history(
+      new DownloadHistory(&manager(), std::move(test_history_adapter)));
+  history->AddObserver(&observer);
+  EXPECT_FALSE(observer.on_history_query_complete_called_);
+  ASSERT_FALSE(query_callback.is_null());
+
+  // Now invoke the query completion callback.
+  std::unique_ptr<std::vector<history::DownloadRow>> query_results(
+      new std::vector<history::DownloadRow>());
+  query_callback.Run(std::move(query_results));
+  EXPECT_TRUE(observer.on_history_query_complete_called_);
+  history->RemoveObserver(&observer);
+}
+
+// Test that the OnHistoryQueryComplete() observer method is invoked for an
+// observer that was added after the initial history query completing.
+TEST_F(DownloadHistoryTest, DownloadHistoryTest_OnHistoryQueryComplete_Post) {
+  TestDownloadHistoryObserver observer;
+  CreateDownloadHistory(std::unique_ptr<InfoVector>(new InfoVector()));
+  download_history()->AddObserver(&observer);
+  EXPECT_TRUE(observer.on_history_query_complete_called_);
+  download_history()->RemoveObserver(&observer);
+}
+
 // Test that WasRestoredFromHistory accurately identifies downloads that were
 // created from history, even during an OnDownloadCreated() handler.
 TEST_F(DownloadHistoryTest, DownloadHistoryTest_WasRestoredFromHistory_True) {
@@ -528,7 +570,7 @@ TEST_F(DownloadHistoryTest, DownloadHistoryTest_WasRestoredFromHistory_True) {
                 "http://example.com/bar.pdf",
                 "http://example.com/referrer.html",
                 &info);
-  scoped_ptr<InfoVector> infos(new InfoVector());
+  std::unique_ptr<InfoVector> infos(new InfoVector());
   infos->push_back(info);
   CreateDownloadHistory(std::move(infos));
 
@@ -546,7 +588,7 @@ TEST_F(DownloadHistoryTest, DownloadHistoryTest_WasRestoredFromHistory_False) {
 
   // Create a DownloadHistory with no history downloads. No
   // DownloadManager::CreateDownload() calls are expected.
-  CreateDownloadHistory(scoped_ptr<InfoVector>(new InfoVector()));
+  CreateDownloadHistory(std::unique_ptr<InfoVector>(new InfoVector()));
 
   // Notify DownloadHistory that a new download was created. The above test
   // expecation should verify that WasRestoredFromHistory is correct for this
@@ -565,7 +607,7 @@ TEST_F(DownloadHistoryTest, DownloadHistoryTest_WasRestoredFromHistory_False) {
 TEST_F(DownloadHistoryTest, DownloadHistoryTest_Create) {
   // Create a fresh item not from history, OnDownloadCreated, OnDownloadUpdated,
   // OnDownloadRemoved.
-  CreateDownloadHistory(scoped_ptr<InfoVector>(new InfoVector()));
+  CreateDownloadHistory(std::unique_ptr<InfoVector>(new InfoVector()));
 
   history::DownloadRow info;
   InitBasicItem(FILE_PATH_LITERAL("/foo/bar.pdf"),
@@ -594,7 +636,7 @@ TEST_F(DownloadHistoryTest, DownloadHistoryTest_Create) {
 // Test that changes to persisted fields in a DownloadItem triggers database
 // updates.
 TEST_F(DownloadHistoryTest, DownloadHistoryTest_Update) {
-  CreateDownloadHistory(scoped_ptr<InfoVector>(new InfoVector()));
+  CreateDownloadHistory(std::unique_ptr<InfoVector>(new InfoVector()));
 
   history::DownloadRow info;
   InitBasicItem(FILE_PATH_LITERAL("/foo/bar.pdf"),
@@ -690,7 +732,7 @@ TEST_F(DownloadHistoryTest, DownloadHistoryTest_Update) {
 TEST_F(DownloadHistoryTest, DownloadHistoryTest_Temporary) {
   // Create a fresh item not from history, OnDownloadCreated, OnDownloadUpdated,
   // OnDownloadRemoved.
-  CreateDownloadHistory(scoped_ptr<InfoVector>(new InfoVector()));
+  CreateDownloadHistory(std::unique_ptr<InfoVector>(new InfoVector()));
 
   history::DownloadRow info;
   InitBasicItem(FILE_PATH_LITERAL("/foo/bar.pdf"),
@@ -733,7 +775,7 @@ TEST_F(DownloadHistoryTest, DownloadHistoryTest_Temporary) {
 
 // Test removing downloads while they're still being added.
 TEST_F(DownloadHistoryTest, DownloadHistoryTest_RemoveWhileAdding) {
-  CreateDownloadHistory(scoped_ptr<InfoVector>(new InfoVector()));
+  CreateDownloadHistory(std::unique_ptr<InfoVector>(new InfoVector()));
 
   history::DownloadRow info;
   InitBasicItem(FILE_PATH_LITERAL("/foo/bar.pdf"),
@@ -783,7 +825,7 @@ TEST_F(DownloadHistoryTest, DownloadHistoryTest_Multiple) {
                 "http://example.com/referrer1.html",
                 &info1);
   {
-    scoped_ptr<InfoVector> infos(new InfoVector());
+    std::unique_ptr<InfoVector> infos(new InfoVector());
     infos->push_back(info0);
     infos->push_back(info1);
     CreateDownloadHistory(std::move(infos));
@@ -806,7 +848,7 @@ TEST_F(DownloadHistoryTest, DownloadHistoryTest_Multiple) {
 TEST_F(DownloadHistoryTest, DownloadHistoryTest_CreateFailed) {
   // Create a fresh item not from history, OnDownloadCreated, OnDownloadUpdated,
   // OnDownloadRemoved.
-  CreateDownloadHistory(scoped_ptr<InfoVector>(new InfoVector()));
+  CreateDownloadHistory(std::unique_ptr<InfoVector>(new InfoVector()));
 
   history::DownloadRow info;
   InitBasicItem(FILE_PATH_LITERAL("/foo/bar.pdf"),
@@ -830,7 +872,7 @@ TEST_F(DownloadHistoryTest, DownloadHistoryTest_CreateFailed) {
 TEST_F(DownloadHistoryTest, DownloadHistoryTest_UpdateWhileAdding) {
   // Create a fresh item not from history, OnDownloadCreated, OnDownloadUpdated,
   // OnDownloadRemoved.
-  CreateDownloadHistory(scoped_ptr<InfoVector>(new InfoVector()));
+  CreateDownloadHistory(std::unique_ptr<InfoVector>(new InfoVector()));
 
   history::DownloadRow info;
   InitBasicItem(FILE_PATH_LITERAL("/foo/bar.pdf"),
