@@ -4,10 +4,11 @@
 
 #include <stdint.h>
 
+#include "base/callback.h"
 #include "base/location.h"
+#include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/stringprintf.h"
-#include "base/thread_task_runner_handle.h"
 #include "chrome/browser/extensions/activity_log/activity_log.h"
 #include "chrome/browser/extensions/extension_apitest.h"
 #include "chrome/browser/extensions/extension_browsertest.h"
@@ -17,6 +18,7 @@
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/common/chrome_switches.h"
 #include "content/public/browser/notification_service.h"
+#include "content/public/browser/notification_types.h"
 #include "content/public/test/browser_test_utils.h"
 #include "content/public/test/test_utils.h"
 #include "net/dns/mock_host_resolver.h"
@@ -41,10 +43,9 @@ class ActivityLogPrerenderTest : public ExtensionApiTest {
   static void Prerender_Arguments(
       const std::string& extension_id,
       uint16_t port,
-      scoped_ptr<std::vector<scoped_refptr<Action>>> i) {
-    // This is to exit RunLoop (base::MessageLoop::current()->Run()) below
-    base::ThreadTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE, base::MessageLoop::QuitWhenIdleClosure());
+      const base::Closure& quit_when_idle_closure,
+      std::unique_ptr<std::vector<scoped_refptr<Action>>> i) {
+    quit_when_idle_closure.Run();
 
     ASSERT_TRUE(i->size());
     scoped_refptr<Action> last = i->front();
@@ -54,11 +55,10 @@ class ActivityLogPrerenderTest : public ExtensionApiTest {
     ASSERT_EQ("[\"/google_cs.js\"]",
               ActivityLogPolicy::Util::Serialize(last->args()));
     ASSERT_EQ(
-        base::StringPrintf("http://www.google.com.bo:%u/test.html", port),
+        base::StringPrintf("http://www.google.com.bo:%u/title1.html", port),
         last->SerializePageUrl());
-    ASSERT_EQ(
-        base::StringPrintf("www.google.com.bo:%u/test.html", port),
-        last->page_title());
+    ASSERT_EQ(base::StringPrintf("www.google.com.bo:%u/title1.html", port),
+              last->page_title());
     ASSERT_EQ("{\"prerender\":true}",
               ActivityLogPolicy::Util::Serialize(last->other()));
     ASSERT_EQ("", last->api_name());
@@ -77,11 +77,12 @@ IN_PROC_BROWSER_TEST_F(ActivityLogPrerenderTest, TestScriptInjected) {
   ASSERT_TRUE(ext);
 
   ActivityLog* activity_log = ActivityLog::GetInstance(profile());
+  activity_log->SetWatchdogAppActiveForTesting(true);
   ASSERT_TRUE(activity_log);
 
   // Disable rate limiting in PrerenderManager
   prerender::PrerenderManager* prerender_manager =
-      prerender::PrerenderManagerFactory::GetForProfile(profile());
+      prerender::PrerenderManagerFactory::GetForBrowserContext(profile());
   ASSERT_TRUE(prerender_manager);
   prerender_manager->mutable_config().rate_limit_enabled = false;
   // Increase prerenderer limits, otherwise this test fails
@@ -100,12 +101,10 @@ IN_PROC_BROWSER_TEST_F(ActivityLogPrerenderTest, TestScriptInjected) {
       content::NOTIFICATION_LOAD_STOP,
       content::NotificationService::AllSources());
 
-  GURL url(base::StringPrintf(
-      "http://www.google.com.bo:%u/test.html",
-      port));
+  GURL url(base::StringPrintf("http://www.google.com.bo:%u/title1.html", port));
 
   const gfx::Size kSize(640, 480);
-  scoped_ptr<prerender::PrerenderHandle> prerender_handle(
+  std::unique_ptr<prerender::PrerenderHandle> prerender_handle(
       prerender_manager->AddPrerenderFromOmnibox(
           url,
           web_contents->GetController().GetDefaultSessionStorageNamespace(),
@@ -113,18 +112,14 @@ IN_PROC_BROWSER_TEST_F(ActivityLogPrerenderTest, TestScriptInjected) {
 
   page_observer.Wait();
 
+  base::RunLoop run_loop;
   activity_log->GetFilteredActions(
-      ext->id(),
-      Action::ACTION_ANY,
-      "",
-      "",
-      "",
-      -1,
-      base::Bind(
-          ActivityLogPrerenderTest::Prerender_Arguments, ext->id(), port));
+      ext->id(), Action::ACTION_ANY, "", "", "", -1,
+      base::Bind(ActivityLogPrerenderTest::Prerender_Arguments, ext->id(), port,
+                 run_loop.QuitWhenIdleClosure()));
 
   // Allow invocation of Prerender_Arguments
-  base::MessageLoop::current()->Run();
+  run_loop.Run();
 }
 
 }  // namespace extensions
