@@ -4,12 +4,14 @@
 
 #include "chrome/browser/extensions/api/preference/preference_helpers.h"
 
+#include <utility>
+
 #include "base/json/json_writer.h"
-#include "base/prefs/pref_service.h"
 #include "base/values.h"
 #include "chrome/browser/extensions/api/preference/preference_api.h"
 #include "chrome/browser/extensions/extension_util.h"
 #include "chrome/browser/profiles/profile.h"
+#include "components/prefs/pref_service.h"
 #include "extensions/browser/event_router.h"
 #include "extensions/browser/extension_prefs.h"
 #include "extensions/browser/extension_registry.h"
@@ -82,13 +84,13 @@ const char* GetLevelOfControl(
   return kControlledByOtherExtensions;
 }
 
-void DispatchEventToExtensions(
-    Profile* profile,
-    const std::string& event_name,
-    base::ListValue* args,
-    APIPermission::ID permission,
-    bool incognito,
-    const std::string& browser_pref) {
+void DispatchEventToExtensions(Profile* profile,
+                               events::HistogramValue histogram_value,
+                               const std::string& event_name,
+                               base::ListValue* args,
+                               APIPermission::ID permission,
+                               bool incognito,
+                               const std::string& browser_pref) {
   EventRouter* router = EventRouter::Get(profile);
   if (!router || !router->HasEventListener(event_name))
     return;
@@ -125,11 +127,11 @@ void DispatchEventToExtensions(
         }
       }
 
-      scoped_ptr<base::ListValue> args_copy(args->DeepCopy());
-      scoped_ptr<Event> event(
-          new Event(events::UNKNOWN, event_name, args_copy.Pass()));
+      std::unique_ptr<base::ListValue> args_copy(args->DeepCopy());
+      std::unique_ptr<Event> event(
+          new Event(histogram_value, event_name, std::move(args_copy)));
       event->restrict_to_browser_context = restrict_to_profile;
-      router->DispatchEventToExtension(extension->id(), event.Pass());
+      router->DispatchEventToExtension(extension->id(), std::move(event));
     }
   }
 }
