@@ -4,6 +4,9 @@
 
 #include "chrome/browser/extensions/extension_browsertest.h"
 
+#include <stddef.h>
+
+#include <utility>
 #include <vector>
 
 #include "base/command_line.h"
@@ -14,6 +17,7 @@
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
+#include "build/build_config.h"
 #include "chrome/browser/extensions/browsertest_util.h"
 #include "chrome/browser/extensions/component_loader.h"
 #include "chrome/browser/extensions/crx_installer.h"
@@ -31,10 +35,12 @@
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_window.h"
 #include "chrome/browser/ui/extensions/application_launch.h"
+#include "chrome/browser/ui/extensions/extension_message_bubble_factory.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/chrome_version_info.h"
+#include "components/sync/model/string_ordinal.h"
+#include "components/version_info/version_info.h"
 #include "content/public/browser/navigation_controller.h"
 #include "content/public/browser/navigation_entry.h"
 #include "content/public/browser/notification_registrar.h"
@@ -42,6 +48,7 @@
 #include "content/public/browser/render_view_host.h"
 #include "content/public/test/browser_test_utils.h"
 #include "content/public/test/test_utils.h"
+#include "extensions/browser/extension_dialog_auto_confirm.h"
 #include "extensions/browser/extension_host.h"
 #include "extensions/browser/extension_prefs.h"
 #include "extensions/browser/extension_registry.h"
@@ -50,7 +57,6 @@
 #include "extensions/browser/uninstall_reason.h"
 #include "extensions/common/constants.h"
 #include "extensions/common/extension_set.h"
-#include "sync/api/string_ordinal.h"
 
 #if defined(OS_CHROMEOS)
 #include "chromeos/chromeos_switches.h"
@@ -61,6 +67,7 @@ using extensions::ExtensionCreator;
 using extensions::ExtensionRegistry;
 using extensions::FeatureSwitch;
 using extensions::Manifest;
+using extensions::ScopedTestDialogAutoConfirm;
 
 ExtensionBrowserTest::ExtensionBrowserTest()
     : loaded_(false),
@@ -70,7 +77,7 @@ ExtensionBrowserTest::ExtensionBrowserTest()
 #endif
       // Default channel is STABLE but override with UNKNOWN so that unlaunched
       // or incomplete APIs can write tests.
-      current_channel_(chrome::VersionInfo::CHANNEL_UNKNOWN),
+      current_channel_(version_info::Channel::UNKNOWN),
       override_prompt_for_external_extensions_(
           FeatureSwitch::prompt_for_external_extensions(),
           false),
@@ -120,7 +127,11 @@ void ExtensionBrowserTest::SetUp() {
 void ExtensionBrowserTest::SetUpCommandLine(base::CommandLine* command_line) {
   PathService::Get(chrome::DIR_TEST_DATA, &test_data_dir_);
   test_data_dir_ = test_data_dir_.AppendASCII("extensions");
-  observer_.reset(new ExtensionTestNotificationObserver(browser()));
+  observer_.reset(new ChromeExtensionTestNotificationObserver(browser()));
+
+  // We don't want any warning bubbles for, e.g., unpacked extensions.
+  ExtensionMessageBubbleFactory::set_override_for_tests(
+      ExtensionMessageBubbleFactory::OVERRIDE_DISABLED);
 
 #if defined(OS_CHROMEOS)
   if (set_chromeos_user_) {
@@ -128,7 +139,7 @@ void ExtensionBrowserTest::SetUpCommandLine(base::CommandLine* command_line) {
     // ExtensionService and then the real profile with one, as we do when
     // running on chromeos.
     command_line->AppendSwitchASCII(chromeos::switches::kLoginUser,
-                                    "TestUser@gmail.com");
+                                    "testuser@gmail.com");
     command_line->AppendSwitchASCII(chromeos::switches::kLoginProfile, "user");
   }
 #endif
@@ -136,13 +147,19 @@ void ExtensionBrowserTest::SetUpCommandLine(base::CommandLine* command_line) {
 
 void ExtensionBrowserTest::SetUpOnMainThread() {
   InProcessBrowserTest::SetUpOnMainThread();
-  observer_.reset(new ExtensionTestNotificationObserver(browser()));
+  observer_.reset(new ChromeExtensionTestNotificationObserver(browser()));
   if (extension_service()->updater()) {
     extension_service()->updater()->SetExtensionCacheForTesting(
         test_extension_cache_.get());
   }
 }
 
+void ExtensionBrowserTest::TearDownOnMainThread() {
+  ExtensionMessageBubbleFactory::set_override_for_tests(
+      ExtensionMessageBubbleFactory::NO_OVERRIDE);
+  InProcessBrowserTest::TearDownOnMainThread();
+}
+
 const Extension* ExtensionBrowserTest::LoadExtension(
     const base::FilePath& path) {
   return LoadExtensionWithFlags(path, kFlagEnableFileAccess);
@@ -209,14 +226,19 @@ ExtensionBrowserTest::LoadExtensionWithInstallParam(
 
   const std::string extension_id = extension->id();
 
+  // If this is an incognito test (e.g. where the test fixture appended the
+  // --incognito flag), we need to use the original profile when we wait for
+  // notifications.
+  Profile* original_profile = profile()->GetOriginalProfile();
+
   if (!install_param.empty()) {
-    extensions::ExtensionPrefs::Get(profile())
+    extensions::ExtensionPrefs::Get(original_profile)
         ->SetInstallParam(extension_id, install_param);
     // Re-enable the extension if needed.
     if (registry->enabled_extensions().Contains(extension_id)) {
       content::WindowedNotificationObserver load_signal(
           extensions::NOTIFICATION_EXTENSION_LOADED_DEPRECATED,
-          content::Source<Profile>(profile()));
+          content::Source<Profile>(original_profile));
       // Reload the extension so that the
       // NOTIFICATION_EXTENSION_LOADED_DEPRECATED
       // observers may access |install_param|.
@@ -234,12 +256,13 @@ ExtensionBrowserTest::LoadExtensionWithInstallParam(
   {
     content::WindowedNotificationObserver load_signal(
         extensions::NOTIFICATION_EXTENSION_LOADED_DEPRECATED,
-        content::Source<Profile>(profile()));
-    CHECK(!extensions::util::IsIncognitoEnabled(extension_id, profile()))
+        content::Source<Profile>(original_profile));
+    CHECK(!extensions::util::IsIncognitoEnabled(extension_id, original_profile))
         << extension_id << " is enabled in incognito, but shouldn't be";
 
     if (flags & kFlagEnableIncognito) {
-      extensions::util::SetIsIncognitoEnabled(extension_id, profile(), true);
+      extensions::util::SetIsIncognitoEnabled(extension_id, original_profile,
+                                              true);
       load_signal.Wait();
       extension = service->GetExtensionById(extension_id, false);
       CHECK(extension) << extension_id << " not found after reloading.";
@@ -249,10 +272,11 @@ ExtensionBrowserTest::LoadExtensionWithInstallParam(
   {
     content::WindowedNotificationObserver load_signal(
         extensions::NOTIFICATION_EXTENSION_LOADED_DEPRECATED,
-        content::Source<Profile>(profile()));
-    CHECK(extensions::util::AllowFileAccess(extension_id, profile()));
+        content::Source<Profile>(original_profile));
+    CHECK(extensions::util::AllowFileAccess(extension_id, original_profile));
     if (!(flags & kFlagEnableFileAccess)) {
-      extensions::util::SetAllowFileAccess(extension_id, profile(), false);
+      extensions::util::SetAllowFileAccess(extension_id, original_profile,
+                                           false);
       load_signal.Wait();
       extension = service->GetExtensionById(extension_id, false);
       CHECK(extension) << extension_id << " not found after reloading.";
@@ -301,7 +325,8 @@ const Extension* ExtensionBrowserTest::LoadAndLaunchApp(
       content::NOTIFICATION_LOAD_COMPLETED_MAIN_FRAME,
       content::NotificationService::AllSources());
   AppLaunchParams params(profile(), app, extensions::LAUNCH_CONTAINER_NONE,
-                         NEW_WINDOW, extensions::SOURCE_TEST);
+                         WindowOpenDisposition::NEW_WINDOW,
+                         extensions::SOURCE_TEST);
   params.command_line = *base::CommandLine::ForCurrentProcess();
   OpenApplication(params);
   app_loaded_observer.Wait();
@@ -311,7 +336,7 @@ const Extension* ExtensionBrowserTest::LoadAndLaunchApp(
 
 base::FilePath ExtensionBrowserTest::PackExtension(
     const base::FilePath& dir_path) {
-  base::FilePath crx_path = temp_dir_.path().AppendASCII("temp.crx");
+  base::FilePath crx_path = temp_dir_.GetPath().AppendASCII("temp.crx");
   if (!base::DeleteFile(crx_path, false)) {
     ADD_FAILURE() << "Failed to delete crx: " << crx_path.value();
     return base::FilePath();
@@ -349,7 +374,7 @@ base::FilePath ExtensionBrowserTest::PackExtensionWithOptions(
     return base::FilePath();
   }
 
-  scoped_ptr<ExtensionCreator> creator(new ExtensionCreator());
+  std::unique_ptr<ExtensionCreator> creator(new ExtensionCreator());
   if (!creator->Run(dir_path,
                     crx_path,
                     pem_path,
@@ -367,66 +392,21 @@ base::FilePath ExtensionBrowserTest::PackExtensionWithOptions(
   return crx_path;
 }
 
-// This class is used to simulate an installation abort by the user.
-class MockAbortExtensionInstallPrompt : public ExtensionInstallPrompt {
- public:
-  MockAbortExtensionInstallPrompt() : ExtensionInstallPrompt(NULL) {
-  }
-
-  // Simulate a user abort on an extension installation.
-  void ConfirmInstall(Delegate* delegate,
-                      const Extension* extension,
-                      const ShowDialogCallback& show_dialog_callback) override {
-    delegate->InstallUIAbort(true);
-    base::MessageLoopForUI::current()->Quit();
-  }
-
-  void OnInstallSuccess(const Extension* extension, SkBitmap* icon) override {}
-
-  void OnInstallFailure(const extensions::CrxInstallError& error) override {}
-};
-
-class MockAutoConfirmExtensionInstallPrompt : public ExtensionInstallPrompt {
- public:
-  explicit MockAutoConfirmExtensionInstallPrompt(
-      content::WebContents* web_contents)
-    : ExtensionInstallPrompt(web_contents) {}
-
-  // Proceed without confirmation prompt.
-  void ConfirmInstall(Delegate* delegate,
-                      const Extension* extension,
-                      const ShowDialogCallback& show_dialog_callback) override {
-    delegate->InstallUIProceed();
-  }
-};
-
 const Extension* ExtensionBrowserTest::UpdateExtensionWaitForIdle(
     const std::string& id,
     const base::FilePath& path,
     int expected_change) {
-  return InstallOrUpdateExtension(id,
-                                  path,
-                                  INSTALL_UI_TYPE_NONE,
-                                  expected_change,
-                                  Manifest::INTERNAL,
-                                  browser(),
-                                  Extension::NO_FLAGS,
-                                  false,
-                                  false);
+  return InstallOrUpdateExtension(id, path, INSTALL_UI_TYPE_NONE,
+                                  expected_change, Manifest::INTERNAL,
+                                  browser(), Extension::NO_FLAGS, false, false);
 }
 
 const Extension* ExtensionBrowserTest::InstallExtensionFromWebstore(
     const base::FilePath& path,
     int expected_change) {
-  return InstallOrUpdateExtension(std::string(),
-                                  path,
-                                  INSTALL_UI_TYPE_NONE,
-                                  expected_change,
-                                  Manifest::INTERNAL,
-                                  browser(),
-                                  Extension::FROM_WEBSTORE,
-                                  true,
-                                  false);
+  return InstallOrUpdateExtension(
+      std::string(), path, INSTALL_UI_TYPE_AUTO_CONFIRM, expected_change,
+      Manifest::INTERNAL, browser(), Extension::FROM_WEBSTORE, true, false);
 }
 
 const Extension* ExtensionBrowserTest::InstallOrUpdateExtension(
@@ -434,15 +414,9 @@ const Extension* ExtensionBrowserTest::InstallOrUpdateExtension(
     const base::FilePath& path,
     InstallUIType ui_type,
     int expected_change) {
-  return InstallOrUpdateExtension(id,
-                                  path,
-                                  ui_type,
-                                  expected_change,
-                                  Manifest::INTERNAL,
-                                  browser(),
-                                  Extension::NO_FLAGS,
-                                  true,
-                                  false);
+  return InstallOrUpdateExtension(id, path, ui_type, expected_change,
+                                  Manifest::INTERNAL, browser(),
+                                  Extension::NO_FLAGS, true, false);
 }
 
 const Extension* ExtensionBrowserTest::InstallOrUpdateExtension(
@@ -452,15 +426,9 @@ const Extension* ExtensionBrowserTest::InstallOrUpdateExtension(
     int expected_change,
     Browser* browser,
     Extension::InitFromValueFlags creation_flags) {
-  return InstallOrUpdateExtension(id,
-                                  path,
-                                  ui_type,
-                                  expected_change,
-                                  Manifest::INTERNAL,
-                                  browser,
-                                  creation_flags,
-                                  true,
-                                  false);
+  return InstallOrUpdateExtension(id, path, ui_type, expected_change,
+                                  Manifest::INTERNAL, browser, creation_flags,
+                                  true, false);
 }
 
 const Extension* ExtensionBrowserTest::InstallOrUpdateExtension(
@@ -469,15 +437,9 @@ const Extension* ExtensionBrowserTest::InstallOrUpdateExtension(
     InstallUIType ui_type,
     int expected_change,
     Manifest::Location install_source) {
-  return InstallOrUpdateExtension(id,
-                                  path,
-                                  ui_type,
-                                  expected_change,
-                                  install_source,
-                                  browser(),
-                                  Extension::NO_FLAGS,
-                                  true,
-                                  false);
+  return InstallOrUpdateExtension(id, path, ui_type, expected_change,
+                                  install_source, browser(),
+                                  Extension::NO_FLAGS, true, false);
 }
 
 const Extension* ExtensionBrowserTest::InstallOrUpdateExtension(
@@ -489,7 +451,7 @@ const Extension* ExtensionBrowserTest::InstallOrUpdateExtension(
     Browser* browser,
     Extension::InitFromValueFlags creation_flags,
     bool install_immediately,
-    bool is_ephemeral) {
+    bool grant_permissions) {
   ExtensionService* service =
       extensions::ExtensionSystem::Get(profile())->extension_service();
   ExtensionRegistry* registry = ExtensionRegistry::Get(profile());
@@ -497,15 +459,16 @@ const Extension* ExtensionBrowserTest::InstallOrUpdateExtension(
   size_t num_before = registry->enabled_extensions().size();
 
   {
-    scoped_ptr<ExtensionInstallPrompt> install_ui;
+    std::unique_ptr<ScopedTestDialogAutoConfirm> prompt_auto_confirm;
     if (ui_type == INSTALL_UI_TYPE_CANCEL) {
-      install_ui.reset(new MockAbortExtensionInstallPrompt());
+      prompt_auto_confirm.reset(new ScopedTestDialogAutoConfirm(
+          ScopedTestDialogAutoConfirm::CANCEL));
     } else if (ui_type == INSTALL_UI_TYPE_NORMAL) {
-      install_ui.reset(new ExtensionInstallPrompt(
-          browser->tab_strip_model()->GetActiveWebContents()));
+      prompt_auto_confirm.reset(new ScopedTestDialogAutoConfirm(
+          ScopedTestDialogAutoConfirm::NONE));
     } else if (ui_type == INSTALL_UI_TYPE_AUTO_CONFIRM) {
-      install_ui.reset(new MockAutoConfirmExtensionInstallPrompt(
-          browser->tab_strip_model()->GetActiveWebContents()));
+      prompt_auto_confirm.reset(new ScopedTestDialogAutoConfirm(
+          ScopedTestDialogAutoConfirm::ACCEPT));
     }
 
     // TODO(tessamac): Update callers to always pass an unpacked extension
@@ -517,13 +480,18 @@ const Extension* ExtensionBrowserTest::InstallOrUpdateExtension(
     if (crx_path.empty())
       return NULL;
 
+    std::unique_ptr<ExtensionInstallPrompt> install_ui;
+    if (prompt_auto_confirm) {
+      install_ui.reset(new ExtensionInstallPrompt(
+         browser->tab_strip_model()->GetActiveWebContents()));
+    }
     scoped_refptr<extensions::CrxInstaller> installer(
-        extensions::CrxInstaller::Create(service, install_ui.Pass()));
+        extensions::CrxInstaller::Create(service, std::move(install_ui)));
     installer->set_expected_id(id);
     installer->set_creation_flags(creation_flags);
     installer->set_install_source(install_source);
     installer->set_install_immediately(install_immediately);
-    installer->set_is_ephemeral(is_ephemeral);
+    installer->set_allow_silent_install(grant_permissions);
     if (!installer->is_gallery_install()) {
       installer->set_off_store_install_allow_reason(
           extensions::CrxInstaller::OffStoreInstallAllowedInTest);
@@ -541,8 +509,8 @@ const Extension* ExtensionBrowserTest::InstallOrUpdateExtension(
   size_t num_after = registry->enabled_extensions().size();
   EXPECT_EQ(num_before + expected_change, num_after);
   if (num_before + expected_change != num_after) {
-    VLOG(1) << "Num extensions before: " << base::IntToString(num_before)
-            << " num after: " << base::IntToString(num_after)
+    VLOG(1) << "Num extensions before: " << base::SizeTToString(num_before)
+            << " num after: " << base::SizeTToString(num_after)
             << " Installed extensions follow:";
 
     for (const scoped_refptr<const Extension>& extension :
@@ -564,7 +532,7 @@ const Extension* ExtensionBrowserTest::InstallOrUpdateExtension(
   return service->GetExtensionById(last_loaded_extension_id(), false);
 }
 
-void ExtensionBrowserTest::ReloadExtension(const std::string extension_id) {
+void ExtensionBrowserTest::ReloadExtension(const std::string& extension_id) {
   observer_->Watch(extensions::NOTIFICATION_EXTENSION_LOADED_DEPRECATED,
                    content::NotificationService::AllSources());
 
