@@ -6,6 +6,7 @@
 #define CHROME_BROWSER_EXTENSIONS_EXTENSION_SERVICE_H_
 
 #include <map>
+#include <memory>
 #include <set>
 #include <string>
 #include <vector>
@@ -19,10 +20,13 @@
 #include "base/strings/string16.h"
 #include "chrome/browser/extensions/blacklist.h"
 #include "chrome/browser/extensions/extension_management.h"
+#include "chrome/browser/extensions/install_gate.h"
 #include "chrome/browser/extensions/pending_extension_manager.h"
+#include "components/sync/model/string_ordinal.h"
 #include "content/public/browser/notification_observer.h"
 #include "content/public/browser/notification_registrar.h"
 #include "extensions/browser/crx_file_info.h"
+#include "extensions/browser/extension_prefs.h"
 #include "extensions/browser/external_provider_interface.h"
 #include "extensions/browser/install_flag.h"
 #include "extensions/browser/process_manager.h"
@@ -30,9 +34,9 @@
 #include "extensions/common/extension.h"
 #include "extensions/common/extension_set.h"
 #include "extensions/common/manifest.h"
-#include "sync/api/string_ordinal.h"
+#include "extensions/features/features.h"
 
-#if !defined(ENABLE_EXTENSIONS)
+#if !BUILDFLAG(ENABLE_EXTENSIONS)
 #error "Extensions must be enabled"
 #endif
 
@@ -63,6 +67,7 @@ class ExtensionSystem;
 class ExtensionUpdater;
 class ExternalInstallManager;
 class OneShotEvent;
+class RendererStartupHelper;
 class SharedModuleService;
 class UpdateObserver;
 }  // namespace extensions
@@ -228,24 +233,23 @@ class ExtensionService
   bool is_ready() override;
   base::SequencedTaskRunner* GetFileTaskRunner() override;
 
-  // ExternalProvider::Visitor implementation.
+  // ExternalProvider::VisitorInterface implementation.
   // Exposed for testing.
-  bool OnExternalExtensionFileFound(const std::string& id,
-                                    const base::Version* version,
-                                    const base::FilePath& path,
-                                    extensions::Manifest::Location location,
-                                    int creation_flags,
-                                    bool mark_acknowledged,
-                                    bool install_immediately) override;
+  bool OnExternalExtensionFileFound(
+      const extensions::ExternalInstallInfoFile& info) override;
   bool OnExternalExtensionUpdateUrlFound(
-      const std::string& id,
-      const std::string& install_parameter,
-      const GURL& update_url,
-      extensions::Manifest::Location location,
-      int creation_flags,
-      bool mark_acknowledged) override;
+      const extensions::ExternalInstallInfoUpdateUrl& info,
+      bool is_initial_load) override;
   void OnExternalProviderReady(
       const extensions::ExternalProviderInterface* provider) override;
+  void OnExternalProviderUpdateComplete(
+      const extensions::ExternalProviderInterface* provider,
+      const std::vector<
+          std::unique_ptr<extensions::ExternalInstallInfoUpdateUrl>>&
+          external_update_url_extensions,
+      const std::vector<std::unique_ptr<extensions::ExternalInstallInfoFile>>&
+          external_file_extensions,
+      const std::set<std::string>& removed_extensions) override;
 
   // ExtensionManagement::Observer implementation:
   void OnExtensionManagementSettingsChanged() override;
@@ -348,14 +352,6 @@ class ExtensionService
   void SetFileTaskRunnerForTesting(
       const scoped_refptr<base::SequencedTaskRunner>& task_runner);
 
-  // Postpone installations so that we don't have to worry about race
-  // conditions.
-  void OnGarbageCollectIsolatedStorageStart();
-
-  // Restart any extension installs which were delayed for isolated storage
-  // garbage collection.
-  void OnGarbageCollectIsolatedStorageFinished();
-
   // Record a histogram using the PermissionMessage enum values for each
   // permission in |e|.
   // NOTE: If this is ever called with high frequency, the implementation may
@@ -376,6 +372,11 @@ class ExtensionService
   void AddUpdateObserver(extensions::UpdateObserver* observer);
   void RemoveUpdateObserver(extensions::UpdateObserver* observer);
 
+  // Register/unregister an InstallGate with the service.
+  void RegisterInstallGate(extensions::ExtensionPrefs::DelayReason reason,
+                           extensions::InstallGate* install_delayer);
+  void UnregisterInstallGate(extensions::InstallGate* install_delayer);
+
   //////////////////////////////////////////////////////////////////////////////
   // Simple Accessors
 
@@ -431,9 +432,8 @@ class ExtensionService
   void ClearProvidersForTesting();
 
   // Adds an ExternalProviderInterface for the service to use during testing.
-  // Takes ownership of |test_provider|.
   void AddProviderForTesting(
-      extensions::ExternalProviderInterface* test_provider);
+      std::unique_ptr<extensions::ExternalProviderInterface> test_provider);
 
   // Simulate an extension being blacklisted for tests.
   void BlacklistExtensionForTest(const std::string& extension_id);
@@ -452,13 +452,6 @@ class ExtensionService
     browser_terminating_ = value;
   }
 
-  // By default ExtensionService will wait with installing an updated extension
-  // until the extension is idle. Tests might not like this behavior, so you can
-  // disable it with this method.
-  void set_install_updates_when_idle_for_test(bool value) {
-    install_updates_when_idle_ = value;
-  }
-
   // Set a callback to be called when all external providers are ready and their
   // extensions have been installed.
   void set_external_updates_finished_callback_for_test(
@@ -466,11 +459,9 @@ class ExtensionService
     external_updates_finished_callback_ = callback;
   }
 
-
  private:
-  // Creates an ExtensionDownloader for use by the updater.
-  scoped_ptr<extensions::ExtensionDownloader> CreateExtensionDownloader(
-      extensions::ExtensionDownloaderDelegate* delegate);
+  // Loads extensions specified via a command line flag/switch.
+  void LoadExtensionsFromCommandLineFlag(const char* switch_name);
 
   // Reloads the specified extension, sending the onLaunched() event to it if it
   // currently has any window showing. |be_noisy| determines whether noisy
@@ -552,7 +543,7 @@ class ExtensionService
   // Disables the extension if the privilege level has increased
   // (e.g., due to an upgrade).
   void CheckPermissionsIncrease(const extensions::Extension* extension,
-                                bool is_extension_installed);
+                                bool is_extension_loaded);
 
   // Helper that updates the active extension list used for crash reporting.
   void UpdateActiveExtensionsInCrashReporter();
@@ -565,10 +556,17 @@ class ExtensionService
   // Helper method to determine if an extension can be blocked.
   bool CanBlockExtension(const extensions::Extension* extension) const;
 
-  // Helper to determine if updating an extensions should proceed immediately,
-  // or if we should delay the update until further notice.
-  bool ShouldDelayExtensionUpdate(const std::string& extension_id,
-                                  bool install_immediately) const;
+  // Helper to determine if installing an extensions should proceed immediately,
+  // or if we should delay the install until further notice, or if the install
+  // should be aborted. A pending install is delayed or aborted when any of the
+  // delayers say so and only proceeds when all delayers return INSTALL.
+  // |extension| is the extension to be installed. |install_immediately| is the
+  // install flag set with the install. |reason| is the reason associated with
+  // the install delayer that wants to defer or abort the install.
+  extensions::InstallGate::Action ShouldDelayExtensionInstall(
+      const extensions::Extension* extension,
+      bool install_immediately,
+      extensions::ExtensionPrefs::DelayReason* reason) const;
 
   // Manages the blacklisted extensions, intended as callback from
   // Blacklist::GetBlacklistedIDs.
@@ -601,20 +599,22 @@ class ExtensionService
       const base::FilePath& install_dir,
       const base::FilePath& extension_path);
 
+  const base::CommandLine* command_line_ = nullptr;
+
   // The normal profile associated with this ExtensionService.
-  Profile* profile_;
+  Profile* profile_ = nullptr;
 
   // The ExtensionSystem for the profile above.
-  extensions::ExtensionSystem* system_;
+  extensions::ExtensionSystem* system_ = nullptr;
 
   // Preferences for the owning profile.
-  extensions::ExtensionPrefs* extension_prefs_;
+  extensions::ExtensionPrefs* extension_prefs_ = nullptr;
 
   // Blacklist for the owning profile.
-  extensions::Blacklist* blacklist_;
+  extensions::Blacklist* blacklist_ = nullptr;
 
   // Sets of enabled/disabled/terminated/blacklisted extensions. Not owned.
-  extensions::ExtensionRegistry* registry_;
+  extensions::ExtensionRegistry* registry_ = nullptr;
 
   // Set of greylisted extensions. These extensions are disabled if they are
   // already installed in Chromium at the time when they are added to
@@ -624,6 +624,10 @@ class ExtensionService
   // These extensions should appear in registry_.
   extensions::ExtensionSet greylist_;
 
+  // Set of whitelisted enabled extensions loaded from the
+  // --disable-extensions-except command line flag.
+  std::set<std::string> disable_flag_exempted_extensions_;
+
   // The list of extension installs delayed for various reasons.  The reason
   // for delayed install is stored in ExtensionPrefs. These are not part of
   // ExtensionRegistry because they are not yet installed.
@@ -636,37 +640,33 @@ class ExtensionService
   base::FilePath install_directory_;
 
   // Whether or not extensions are enabled.
-  bool extensions_enabled_;
+  bool extensions_enabled_ = true;
 
   // Whether to notify users when they attempt to install an extension.
-  bool show_extensions_prompts_;
-
-  // Whether to delay installing of extension updates until the extension is
-  // idle.
-  bool install_updates_when_idle_;
+  bool show_extensions_prompts_ = true;
 
   // Signaled when all extensions are loaded.
   extensions::OneShotEvent* const ready_;
 
   // Our extension updater, if updates are turned on.
-  scoped_ptr<extensions::ExtensionUpdater> updater_;
+  std::unique_ptr<extensions::ExtensionUpdater> updater_;
 
   // Map unloaded extensions' ids to their paths. When a temporarily loaded
   // extension is unloaded, we lose the information about it and don't have
   // any in the extension preferences file.
-  typedef std::map<std::string, base::FilePath> UnloadedExtensionPathMap;
+  using UnloadedExtensionPathMap = std::map<std::string, base::FilePath>;
   UnloadedExtensionPathMap unloaded_extension_paths_;
 
   // Map of DevToolsAgentHost instances that are detached,
   // waiting for an extension to be reloaded.
-  typedef std::map<std::string, scoped_refptr<content::DevToolsAgentHost> >
-      OrphanedDevTools;
+  using OrphanedDevTools =
+      std::map<std::string, scoped_refptr<content::DevToolsAgentHost>>;
   OrphanedDevTools orphaned_dev_tools_;
 
   content::NotificationRegistrar registrar_;
 
   // Keeps track of loading and unloading component extensions.
-  scoped_ptr<extensions::ComponentLoader> component_loader_;
+  std::unique_ptr<extensions::ComponentLoader> component_loader_;
 
   // A collection of external extension providers.  Each provider reads
   // a source of external extension information.  Examples include the
@@ -678,7 +678,7 @@ class ExtensionService
   // has to wait for the external providers.  Used in
   // OnAllExternalProvidersReady() to determine if an update check is needed to
   // install pending extensions.
-  bool update_once_all_providers_are_ready_;
+  bool update_once_all_providers_are_ready_ = false;
 
   // A callback to be called when all external providers are ready and their
   // extensions have been installed. Normally this is a null callback, but
@@ -688,21 +688,15 @@ class ExtensionService
   // Set when the browser is terminating. Prevents us from installing or
   // updating additional extensions and allows in-progress installations to
   // decide to abort.
-  bool browser_terminating_;
-
-  // Set to true to delay all new extension installations. Acts as a lock to
-  // allow background processing of garbage collection of on-disk state without
-  // needing to worry about race conditions caused by extension installation and
-  // reinstallation.
-  bool installs_delayed_for_gc_;
+  bool browser_terminating_ = false;
 
   // Set to true if this is the first time this ExtensionService has run.
   // Used for specially handling external extensions that are installed the
   // first time.
-  bool is_first_run_;
+  bool is_first_run_ = false;
 
   // Set to true if extensions are all to be blocked.
-  bool block_extensions_;
+  bool block_extensions_ = false;
 
   // Store the ids of reloading extensions. We use this to re-enable extensions
   // which were disabled for a reload.
@@ -714,25 +708,33 @@ class ExtensionService
 
   // The controller for the UI that alerts the user about any blacklisted
   // extensions.
-  scoped_ptr<extensions::ExtensionErrorController> error_controller_;
+  std::unique_ptr<extensions::ExtensionErrorController> error_controller_;
 
   // The manager for extensions that were externally installed that is
   // responsible for prompting the user about suspicious extensions.
-  scoped_ptr<extensions::ExternalInstallManager> external_install_manager_;
+  std::unique_ptr<extensions::ExternalInstallManager> external_install_manager_;
 
   // Sequenced task runner for extension related file operations.
   scoped_refptr<base::SequencedTaskRunner> file_task_runner_;
 
-  scoped_ptr<extensions::ExtensionActionStorageManager>
+  std::unique_ptr<extensions::ExtensionActionStorageManager>
       extension_action_storage_manager_;
 
   // The SharedModuleService used to check for import dependencies.
-  scoped_ptr<extensions::SharedModuleService> shared_module_service_;
+  std::unique_ptr<extensions::SharedModuleService> shared_module_service_;
+
+  // The associated RendererStartupHelper. Guaranteed to outlive the
+  // ExtensionSystem, and thus us.
+  extensions::RendererStartupHelper* renderer_helper_;
 
   base::ObserverList<extensions::UpdateObserver, true> update_observers_;
 
   // Migrates app data when upgrading a legacy packaged app to a platform app
-  scoped_ptr<extensions::AppDataMigrator> app_data_migrator_;
+  std::unique_ptr<extensions::AppDataMigrator> app_data_migrator_;
+
+  using InstallGateRegistry = std::map<extensions::ExtensionPrefs::DelayReason,
+                                       extensions::InstallGate*>;
+  InstallGateRegistry install_delayer_registry_;
 
   FRIEND_TEST_ALL_PREFIXES(ExtensionServiceTest,
                            DestroyingProfileClearsExtensions);
@@ -744,6 +746,8 @@ class ExtensionService
   FRIEND_TEST_ALL_PREFIXES(ExtensionServiceTest,
                            WillNotLoadBlacklistedExtensionsFromDirectory);
   FRIEND_TEST_ALL_PREFIXES(ExtensionServiceTest, ReloadBlacklistedExtension);
+  FRIEND_TEST_ALL_PREFIXES(ExtensionServiceTest,
+                           RemoveExtensionFromBlacklist);
   FRIEND_TEST_ALL_PREFIXES(ExtensionServiceTest, BlacklistedInPrefsFromStartup);
   FRIEND_TEST_ALL_PREFIXES(ExtensionServiceTest,
                            GreylistedExtensionDisabled);
