@@ -5,9 +5,10 @@
 #ifndef CHROME_BROWSER_EXTENSIONS_EXTENSION_WEB_UI_H_
 #define CHROME_BROWSER_EXTENSIONS_EXTENSION_WEB_UI_H_
 
+#include <memory>
 #include <string>
 
-#include "base/memory/scoped_ptr.h"
+#include "base/macros.h"
 #include "chrome/common/extensions/chrome_manifest_url_handlers.h"
 #include "components/favicon_base/favicon_callback.h"
 #include "content/public/browser/web_ui_controller.h"
@@ -19,30 +20,17 @@ class BrowserContext;
 class WebContents;
 }
 
-namespace extensions {
-class BookmarkManagerPrivateDragEventRouter;
-}
-
 namespace user_prefs {
 class PrefRegistrySyncable;
 }
 
-// This class implements WebUI for extensions and allows extensions to put UI in
-// the main tab contents area. For example, each extension can specify an
-// "options_page", and that page is displayed in the tab contents area and is
-// hosted by this class.
-class ExtensionWebUI : public content::WebUIController {
+// A collection of methods to handle Chrome URL overrides that are managed by
+// extensions (such as overriding the new tab page).
+// TODO(devlin): Rename this class to ExtensionURLOverrides.
+class ExtensionWebUI {
  public:
   static const char kExtensionURLOverrides[];
 
-  ExtensionWebUI(content::WebUI* web_ui, const GURL& url);
-
-  ~ExtensionWebUI() override;
-
-  virtual extensions::BookmarkManagerPrivateDragEventRouter*
-      bookmark_manager_private_drag_event_router();
-
-  // BrowserURLHandler
   static bool HandleChromeURLOverride(GURL* url,
                                       content::BrowserContext* browser_context);
   static bool HandleChromeURLOverrideReverse(
@@ -91,14 +79,6 @@ class ExtensionWebUI : public content::WebUIController {
                                            Profile* profile,
                                            base::ListValue* list,
                                            const base::Value* override);
-
-  // TODO(aa): This seems out of place. Why is it not with the event routers for
-  // the other extension APIs?
-  scoped_ptr<extensions::BookmarkManagerPrivateDragEventRouter>
-      bookmark_manager_private_drag_event_router_;
-
-  // The URL this WebUI was created for.
-  GURL url_;
 };
 
 #endif  // CHROME_BROWSER_EXTENSIONS_EXTENSION_WEB_UI_H_
