@@ -12,9 +12,9 @@
 #include "base/command_line.h"
 #include "base/files/file_path.h"
 #include "base/logging.h"
-#include "base/memory/linked_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/metrics/field_trial.h"
-#include "base/prefs/pref_service.h"
+#include "base/stl_util.h"
 #include "base/strings/string_util.h"
 #include "base/trace_event/trace_event.h"
 #include "base/values.h"
@@ -37,8 +37,10 @@
 #include "chrome/common/extensions/extension_constants.h"
 #include "chrome/common/pref_names.h"
 #include "components/crx_file/id_util.h"
+#include "components/prefs/pref_service.h"
 #include "content/public/browser/browser_thread.h"
 #include "extensions/browser/extension_system.h"
+#include "extensions/browser/external_install_info.h"
 #include "extensions/browser/external_provider_interface.h"
 #include "extensions/common/extension.h"
 #include "extensions/common/manifest.h"
@@ -120,10 +122,69 @@ void ExternalProviderImpl::SetPrefs(base::DictionaryValue* prefs) {
   prefs_.reset(prefs);
   ready_ = true;  // Queries for extensions are allowed from this point.
 
+  std::vector<std::unique_ptr<ExternalInstallInfoUpdateUrl>>
+      external_update_url_extensions;
+  std::vector<std::unique_ptr<ExternalInstallInfoFile>>
+      external_file_extensions;
+
+  RetrieveExtensionsFromPrefs(&external_update_url_extensions,
+                              &external_file_extensions);
+  for (const auto& extension : external_update_url_extensions)
+    service_->OnExternalExtensionUpdateUrlFound(*extension, true);
+
+  for (const auto& extension : external_file_extensions)
+    service_->OnExternalExtensionFileFound(*extension);
+
+  service_->OnExternalProviderReady(this);
+}
+
+void ExternalProviderImpl::UpdatePrefs(base::DictionaryValue* prefs) {
+  CHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  // We only expect updates from windows registry.
+  CHECK(crx_location_ == Manifest::EXTERNAL_REGISTRY);
+
+  // Check if the service is still alive. It is possible that it went
+  // away while |loader_| was working on the FILE thread.
+  if (!service_)
+    return;
+
+  std::set<std::string> removed_extensions;
+  // Find extensions that were removed by this ExternalProvider.
+  for (base::DictionaryValue::Iterator i(*prefs_); !i.IsAtEnd(); i.Advance()) {
+    const std::string& extension_id = i.key();
+    // Don't bother about invalid ids.
+    if (!crx_file::id_util::IdIsValid(extension_id))
+      continue;
+    if (!prefs->HasKey(extension_id))
+      removed_extensions.insert(extension_id);
+  }
+
+  prefs_.reset(prefs);
+
+  std::vector<std::unique_ptr<ExternalInstallInfoUpdateUrl>>
+      external_update_url_extensions;
+  std::vector<std::unique_ptr<ExternalInstallInfoFile>>
+      external_file_extensions;
+  RetrieveExtensionsFromPrefs(&external_update_url_extensions,
+                              &external_file_extensions);
+
+  // Notify ExtensionService about completion of finding incremental updates
+  // from this provider.
+  // Provide the list of added and removed extensions.
+  service_->OnExternalProviderUpdateComplete(
+      this, external_update_url_extensions, external_file_extensions,
+      removed_extensions);
+}
+
+void ExternalProviderImpl::RetrieveExtensionsFromPrefs(
+    std::vector<std::unique_ptr<ExternalInstallInfoUpdateUrl>>*
+        external_update_url_extensions,
+    std::vector<std::unique_ptr<ExternalInstallInfoFile>>*
+        external_file_extensions) {
   // Set of unsupported extensions that need to be deleted from prefs_.
   std::set<std::string> unsupported_extensions;
 
-  // Notify ExtensionService about all the extensions this provider has.
+  // Discover all the extensions this provider has.
   for (base::DictionaryValue::Iterator i(*prefs_); !i.IsAtEnd(); i.Advance()) {
     const std::string& extension_id = i.key();
     const base::DictionaryValue* extension = NULL;
@@ -192,8 +253,7 @@ void ExternalProviderImpl::SetPrefs(base::DictionaryValue* prefs) {
         if (supported_locales->GetString(j, &current_locale) &&
             l10n_util::IsValidLocaleSyntax(current_locale)) {
           current_locale = l10n_util::NormalizeLocale(current_locale);
-          if (std::find(browser_locales.begin(), browser_locales.end(),
-                        current_locale) != browser_locales.end()) {
+          if (base::ContainsValue(browser_locales, current_locale)) {
             locale_supported = true;
             break;
           }
@@ -289,17 +349,18 @@ void ExternalProviderImpl::SetPrefs(base::DictionaryValue* prefs) {
         path = base_path.Append(external_crx);
       }
 
-      Version version(external_version);
-      if (!version.IsValid()) {
+      std::unique_ptr<base::Version> version(
+          new base::Version(external_version));
+      if (!version->IsValid()) {
         LOG(WARNING) << "Malformed extension dictionary for extension: "
                      << extension_id.c_str() << ".  Invalid version string \""
                      << external_version << "\".";
         continue;
       }
-      service_->OnExternalExtensionFileFound(extension_id, &version, path,
-                                             crx_location_, creation_flags,
-                                             auto_acknowledge_,
-                                             install_immediately_);
+      external_file_extensions->push_back(
+          base::MakeUnique<ExternalInstallInfoFile>(
+              extension_id, std::move(version), path, crx_location_,
+              creation_flags, auto_acknowledge_, install_immediately_));
     } else {  // if (has_external_update_url)
       CHECK(has_external_update_url);  // Checking of keys above ensures this.
       if (download_location_ == Manifest::INVALID_LOCATION) {
@@ -307,20 +368,18 @@ void ExternalProviderImpl::SetPrefs(base::DictionaryValue* prefs) {
                      << "extensions from update URLs.";
         continue;
       }
-      GURL update_url(external_update_url);
-      if (!update_url.is_valid()) {
+      std::unique_ptr<GURL> update_url(new GURL(external_update_url));
+      if (!update_url->is_valid()) {
         LOG(WARNING) << "Malformed extension dictionary for extension: "
                      << extension_id.c_str() << ".  Key " << kExternalUpdateUrl
                      << " has value \"" << external_update_url
                      << "\", which is not a valid URL.";
         continue;
       }
-      service_->OnExternalExtensionUpdateUrlFound(extension_id,
-                                                  install_parameter,
-                                                  update_url,
-                                                  download_location_,
-                                                  creation_flags,
-                                                  auto_acknowledge_);
+      external_update_url_extensions->push_back(
+          base::MakeUnique<ExternalInstallInfoUpdateUrl>(
+              extension_id, install_parameter, std::move(update_url),
+              download_location_, creation_flags, auto_acknowledge_));
     }
   }
 
@@ -330,8 +389,6 @@ void ExternalProviderImpl::SetPrefs(base::DictionaryValue* prefs) {
     // will be uninstalled later because provider doesn't provide it anymore.
     prefs_->Remove(*it, NULL);
   }
-
-  service_->OnExternalProviderReady(this);
 }
 
 void ExternalProviderImpl::ServiceShutdown() {
@@ -351,8 +408,9 @@ bool ExternalProviderImpl::HasExtension(
 }
 
 bool ExternalProviderImpl::GetExtensionDetails(
-    const std::string& id, Manifest::Location* location,
-    scoped_ptr<Version>* version) const {
+    const std::string& id,
+    Manifest::Location* location,
+    std::unique_ptr<base::Version>* version) const {
   CHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
   CHECK(prefs_.get());
   CHECK(ready_);
@@ -372,7 +430,7 @@ bool ExternalProviderImpl::GetExtensionDetails(
       return false;
 
     if (version)
-      version->reset(new Version(external_version));
+      version->reset(new base::Version(external_version));
 
   } else {
     NOTREACHED();  // Chrome should not allow prefs to get into this state.
@@ -393,9 +451,9 @@ bool ExternalProviderImpl::HandleMinProfileVersion(
   if (profile_ &&
       extension->GetString(kMinProfileCreatedByVersion,
                            &min_profile_created_by_version)) {
-    Version profile_version(
+    base::Version profile_version(
         profile_->GetPrefs()->GetString(prefs::kProfileCreatedByVersion));
-    Version min_version(min_profile_created_by_version);
+    base::Version min_version(min_profile_created_by_version);
     if (min_version.IsValid() && profile_version.CompareTo(min_version) < 0) {
       unsupported_extensions->insert(extension_id);
       VLOG(1) << "Skip installing (or uninstall) external extension: "
@@ -446,14 +504,14 @@ void ExternalProviderImpl::CreateExternalProviders(
   const user_manager::User* user =
       chromeos::ProfileHelper::Get()->GetUserByProfile(profile);
   policy::DeviceLocalAccount::Type account_type;
-  if (user &&
-      connector->IsEnterpriseManaged() &&
-      policy::IsDeviceLocalAccountUser(user->email(), &account_type)) {
+  if (user && connector->IsEnterpriseManaged() &&
+      policy::IsDeviceLocalAccountUser(user->GetAccountId().GetUserEmail(),
+                                       &account_type)) {
     if (account_type == policy::DeviceLocalAccount::TYPE_PUBLIC_SESSION)
       is_chrome_os_public_session = true;
     policy::DeviceLocalAccountPolicyBroker* broker =
         connector->GetDeviceLocalAccountPolicyService()->GetBrokerForUser(
-            user->email());
+            user->GetAccountId().GetUserEmail());
     if (broker) {
       external_loader = broker->extension_loader();
       crx_location = Manifest::EXTERNAL_POLICY;
@@ -479,15 +537,9 @@ void ExternalProviderImpl::CreateExternalProviders(
 
   // Policies are mandatory so they can't be skipped with command line flag.
   if (external_loader.get()) {
-    provider_list->push_back(
-        linked_ptr<ExternalProviderInterface>(
-            new ExternalProviderImpl(
-                service,
-                external_loader,
-                profile,
-                crx_location,
-                Manifest::EXTERNAL_POLICY_DOWNLOAD,
-                Extension::NO_FLAGS)));
+    provider_list->push_back(base::MakeUnique<ExternalProviderImpl>(
+        service, external_loader, profile, crx_location,
+        Manifest::EXTERNAL_POLICY_DOWNLOAD, Extension::NO_FLAGS));
   }
 
   // Load the KioskAppExternalProvider when running in kiosk mode.
@@ -508,27 +560,25 @@ void ExternalProviderImpl::CreateExternalProviders(
       if (connector && connector->IsEnterpriseManaged())
         location = Manifest::EXTERNAL_POLICY;
 
-      scoped_ptr<ExternalProviderImpl> kiosk_app_provider(
+      std::unique_ptr<ExternalProviderImpl> kiosk_app_provider(
           new ExternalProviderImpl(
               service, kiosk_app_manager->CreateExternalLoader(), profile,
               location, Manifest::INVALID_LOCATION, Extension::NO_FLAGS));
       kiosk_app_provider->set_auto_acknowledge(true);
       kiosk_app_provider->set_install_immediately(true);
-      provider_list->push_back(
-          linked_ptr<ExternalProviderInterface>(kiosk_app_provider.release()));
+      provider_list->push_back(std::move(kiosk_app_provider));
     }
 
     // Kiosk secondary app external provider.
     if (!kiosk_app_manager->secondary_app_external_loader_created()) {
-      scoped_ptr<ExternalProviderImpl> secondary_kiosk_app_provider(
+      std::unique_ptr<ExternalProviderImpl> secondary_kiosk_app_provider(
           new ExternalProviderImpl(
               service, kiosk_app_manager->CreateSecondaryAppExternalLoader(),
               profile, Manifest::EXTERNAL_PREF,
               Manifest::EXTERNAL_PREF_DOWNLOAD, Extension::NO_FLAGS));
       secondary_kiosk_app_provider->set_auto_acknowledge(true);
       secondary_kiosk_app_provider->set_install_immediately(true);
-      provider_list->push_back(linked_ptr<ExternalProviderInterface>(
-          secondary_kiosk_app_provider.release()));
+      provider_list->push_back(std::move(secondary_kiosk_app_provider));
     }
 #endif
     return;
@@ -536,13 +586,9 @@ void ExternalProviderImpl::CreateExternalProviders(
 
   // Extensions provided by recommended policies.
   if (external_recommended_loader.get()) {
-    provider_list->push_back(linked_ptr<ExternalProviderInterface>(
-        new ExternalProviderImpl(service,
-                                 external_recommended_loader,
-                                 profile,
-                                 crx_location,
-                                 Manifest::EXTERNAL_PREF_DOWNLOAD,
-                                 Extension::NO_FLAGS)));
+    provider_list->push_back(base::MakeUnique<ExternalProviderImpl>(
+        service, external_recommended_loader, profile, crx_location,
+        Manifest::EXTERNAL_PREF_DOWNLOAD, Extension::NO_FLAGS));
   }
 
   // In tests don't install extensions from default external sources.
@@ -576,125 +622,83 @@ void ExternalProviderImpl::CreateExternalProviders(
         profile->IsNewProfile()
             ? ExternalPrefLoader::DELAY_LOAD_UNTIL_PRIORITY_SYNC
             : ExternalPrefLoader::NONE;
-    provider_list->push_back(
-        linked_ptr<ExternalProviderInterface>(new ExternalProviderImpl(
-            service, new ExternalPrefLoader(external_apps_path_id,
-                                            pref_load_flags, profile),
-            profile, Manifest::EXTERNAL_PREF, Manifest::EXTERNAL_PREF_DOWNLOAD,
-            bundled_extension_creation_flags)));
+    provider_list->push_back(base::MakeUnique<ExternalProviderImpl>(
+        service,
+        new ExternalPrefLoader(external_apps_path_id, pref_load_flags, profile),
+        profile, Manifest::EXTERNAL_PREF, Manifest::EXTERNAL_PREF_DOWNLOAD,
+        bundled_extension_creation_flags));
 
     // OEM default apps.
     int oem_extension_creation_flags =
         bundled_extension_creation_flags | Extension::WAS_INSTALLED_BY_OEM;
     chromeos::ServicesCustomizationDocument* customization =
         chromeos::ServicesCustomizationDocument::GetInstance();
-    provider_list->push_back(linked_ptr<ExternalProviderInterface>(
-        new ExternalProviderImpl(service,
-                                 customization->CreateExternalLoader(profile),
-                                 profile,
-                                 Manifest::EXTERNAL_PREF,
-                                 Manifest::EXTERNAL_PREF_DOWNLOAD,
-                                 oem_extension_creation_flags)));
+    provider_list->push_back(base::MakeUnique<ExternalProviderImpl>(
+        service, customization->CreateExternalLoader(profile), profile,
+        Manifest::EXTERNAL_PREF, Manifest::EXTERNAL_PREF_DOWNLOAD,
+        oem_extension_creation_flags));
   }
 #elif defined(OS_LINUX)
   if (!profile->IsLegacySupervised()) {
-    provider_list->push_back(
-        linked_ptr<ExternalProviderInterface>(
-            new ExternalProviderImpl(
-                service,
-                new ExternalPrefLoader(
-                    chrome::DIR_STANDALONE_EXTERNAL_EXTENSIONS,
-                    ExternalPrefLoader::NONE,
-                    NULL),
-                profile,
-                Manifest::EXTERNAL_PREF,
-                Manifest::EXTERNAL_PREF_DOWNLOAD,
-                bundled_extension_creation_flags)));
+    provider_list->push_back(base::MakeUnique<ExternalProviderImpl>(
+        service,
+        new ExternalPrefLoader(chrome::DIR_STANDALONE_EXTERNAL_EXTENSIONS,
+                               ExternalPrefLoader::NONE, nullptr),
+        profile, Manifest::EXTERNAL_PREF, Manifest::EXTERNAL_PREF_DOWNLOAD,
+        bundled_extension_creation_flags));
   }
 #endif
 
   if (!profile->IsLegacySupervised()) {
 #if defined(OS_WIN)
-    provider_list->push_back(
-        linked_ptr<ExternalProviderInterface>(
-            new ExternalProviderImpl(
-                service,
-                new ExternalRegistryLoader,
-                profile,
-                Manifest::EXTERNAL_REGISTRY,
-                Manifest::EXTERNAL_PREF_DOWNLOAD,
-                Extension::NO_FLAGS)));
+    provider_list->push_back(base::MakeUnique<ExternalProviderImpl>(
+        service, new ExternalRegistryLoader, profile,
+        Manifest::EXTERNAL_REGISTRY, Manifest::EXTERNAL_PREF_DOWNLOAD,
+        Extension::NO_FLAGS));
 #else
-    provider_list->push_back(
-        linked_ptr<ExternalProviderInterface>(
-            new ExternalProviderImpl(
-                service,
-                new ExternalPrefLoader(chrome::DIR_EXTERNAL_EXTENSIONS,
-                                       check_admin_permissions_on_mac,
-                                       NULL),
-                profile,
-                Manifest::EXTERNAL_PREF,
-                Manifest::EXTERNAL_PREF_DOWNLOAD,
-                bundled_extension_creation_flags)));
+    provider_list->push_back(base::MakeUnique<ExternalProviderImpl>(
+        service,
+        new ExternalPrefLoader(chrome::DIR_EXTERNAL_EXTENSIONS,
+                               check_admin_permissions_on_mac, nullptr),
+        profile, Manifest::EXTERNAL_PREF, Manifest::EXTERNAL_PREF_DOWNLOAD,
+        bundled_extension_creation_flags));
 
     // Define a per-user source of external extensions.
 #if defined(OS_MACOSX) || (defined(OS_LINUX) && defined(CHROMIUM_BUILD))
-    provider_list->push_back(
-        linked_ptr<ExternalProviderInterface>(
-            new ExternalProviderImpl(
-                service,
-                new ExternalPrefLoader(chrome::DIR_USER_EXTERNAL_EXTENSIONS,
-                                       ExternalPrefLoader::NONE,
-                                       NULL),
-                profile,
-                Manifest::EXTERNAL_PREF,
-                Manifest::EXTERNAL_PREF_DOWNLOAD,
-                Extension::NO_FLAGS)));
+    provider_list->push_back(base::MakeUnique<ExternalProviderImpl>(
+        service, new ExternalPrefLoader(chrome::DIR_USER_EXTERNAL_EXTENSIONS,
+                                        ExternalPrefLoader::NONE, nullptr),
+        profile, Manifest::EXTERNAL_PREF, Manifest::EXTERNAL_PREF_DOWNLOAD,
+        Extension::NO_FLAGS));
 #endif
 #endif
 
 #if !defined(OS_CHROMEOS)
     // The default apps are installed as INTERNAL but use the external
     // extension installer codeflow.
-    provider_list->push_back(
-        linked_ptr<ExternalProviderInterface>(
-            new default_apps::Provider(
-                profile,
-                service,
-                new ExternalPrefLoader(chrome::DIR_DEFAULT_APPS,
-                                       ExternalPrefLoader::NONE,
-                                       NULL),
-                Manifest::INTERNAL,
-                Manifest::INTERNAL,
-                Extension::FROM_WEBSTORE |
-                    Extension::WAS_INSTALLED_BY_DEFAULT)));
+    provider_list->push_back(base::MakeUnique<default_apps::Provider>(
+        profile, service,
+        new ExternalPrefLoader(chrome::DIR_DEFAULT_APPS,
+                               ExternalPrefLoader::NONE, nullptr),
+        Manifest::INTERNAL, Manifest::INTERNAL,
+        Extension::FROM_WEBSTORE | Extension::WAS_INSTALLED_BY_DEFAULT));
 #endif
 
-    scoped_ptr<ExternalProviderImpl> drive_migration_provider(
+    std::unique_ptr<ExternalProviderImpl> drive_migration_provider(
         new ExternalProviderImpl(
             service,
-            new ExtensionMigrator(profile,
-                                  extension_misc::kDriveHostedAppId,
+            new ExtensionMigrator(profile, extension_misc::kDriveHostedAppId,
                                   extension_misc::kDriveExtensionId),
-            profile,
-            Manifest::EXTERNAL_PREF,
-            Manifest::EXTERNAL_PREF_DOWNLOAD,
-            Extension::FROM_WEBSTORE |
-                Extension::WAS_INSTALLED_BY_DEFAULT));
+            profile, Manifest::EXTERNAL_PREF, Manifest::EXTERNAL_PREF_DOWNLOAD,
+            Extension::FROM_WEBSTORE | Extension::WAS_INSTALLED_BY_DEFAULT));
     drive_migration_provider->set_auto_acknowledge(true);
-    provider_list->push_back(linked_ptr<ExternalProviderInterface>(
-        drive_migration_provider.release()));
+    provider_list->push_back(std::move(drive_migration_provider));
   }
 
-  provider_list->push_back(
-    linked_ptr<ExternalProviderInterface>(
-      new ExternalProviderImpl(
-          service,
-          new ExternalComponentLoader(profile),
-          profile,
-          Manifest::INVALID_LOCATION,
-          Manifest::EXTERNAL_COMPONENT,
-          Extension::FROM_WEBSTORE | Extension::WAS_INSTALLED_BY_DEFAULT)));
+  provider_list->push_back(base::MakeUnique<ExternalProviderImpl>(
+      service, new ExternalComponentLoader(profile), profile,
+      Manifest::INVALID_LOCATION, Manifest::EXTERNAL_COMPONENT,
+      Extension::FROM_WEBSTORE | Extension::WAS_INSTALLED_BY_DEFAULT));
 }
 
 }  // namespace extensions
