@@ -4,9 +4,10 @@
 
 #include "chrome/browser/extensions/external_provider_impl.h"
 
+#include <memory>
+
 #include "base/command_line.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/test/scoped_path_override.h"
 #include "chrome/browser/chrome_notification_types.h"
 #include "chrome/browser/chromeos/app_mode/kiosk_app_manager.h"
@@ -25,18 +26,18 @@
 #include "chrome/test/base/testing_profile.h"
 #include "chromeos/system/fake_statistics_provider.h"
 #include "chromeos/system/statistics_provider.h"
-#include "components/browser_sync/browser/profile_sync_service.h"
-#include "components/browser_sync/common/browser_sync_switches.h"
+#include "components/browser_sync/browser_sync_switches.h"
+#include "components/browser_sync/profile_sync_service.h"
 #include "components/signin/core/browser/profile_oauth2_token_service.h"
 #include "components/signin/core/browser/signin_manager_base.h"
-#include "components/sync_driver/pref_names.h"
-#include "components/syncable_prefs/pref_service_syncable.h"
+#include "components/sync/base/pref_names.h"
+#include "components/sync/model/fake_sync_change_processor.h"
+#include "components/sync/model/sync_change_processor.h"
+#include "components/sync/model/sync_error_factory_mock.h"
+#include "components/sync_preferences/pref_service_syncable.h"
 #include "components/user_manager/fake_user_manager.h"
 #include "content/public/browser/notification_service.h"
 #include "content/public/test/test_utils.h"
-#include "sync/api/fake_sync_change_processor.h"
-#include "sync/api/sync_change_processor.h"
-#include "sync/api/sync_error_factory_mock.h"
 
 namespace extensions {
 
@@ -70,11 +71,8 @@ class ExternalProviderImplChromeOSTest : public ExtensionServiceTestBase {
     extensions::ExternalProviderImpl::CreateExternalProviders(
         service_, profile_.get(), &providers);
 
-    for (ProviderCollection::iterator i = providers.begin();
-         i != providers.end();
-         ++i) {
-      service_->AddProviderForTesting(i->release());
-    }
+    for (std::unique_ptr<ExternalProviderInterface>& provider : providers)
+      service_->AddProviderForTesting(std::move(provider));
   }
 
   // ExtensionServiceTestBase overrides:
@@ -87,7 +85,7 @@ class ExternalProviderImplChromeOSTest : public ExtensionServiceTestBase {
   }
 
  private:
-  scoped_ptr<base::ScopedPathOverride> external_externsions_overrides_;
+  std::unique_ptr<base::ScopedPathOverride> external_externsions_overrides_;
   chromeos::system::ScopedFakeStatisticsProvider fake_statistics_provider_;
   user_manager::FakeUserManager* fake_user_manager_;
   chromeos::ScopedUserManagerEnabler scoped_user_manager_;
@@ -157,7 +155,7 @@ TEST_F(ExternalProviderImplChromeOSTest, PolicyDisabled) {
 
   // Log user in, start sync.
   TestingBrowserProcess::GetGlobal()->SetProfileManager(
-      new ProfileManagerWithoutInit(temp_dir().path()));
+      new ProfileManagerWithoutInit(temp_dir().GetPath()));
   SigninManagerBase* signin =
       SigninManagerFactory::GetForProfile(profile_.get());
   signin->SetAuthenticatedAccountInfo("gaia-id-test_user@gmail.com",
@@ -169,7 +167,7 @@ TEST_F(ExternalProviderImplChromeOSTest, PolicyDisabled) {
   service_->CheckForExternalUpdates();
 
   // Sync is dsabled by policy.
-  profile_->GetPrefs()->SetBoolean(sync_driver::prefs::kSyncManaged, true);
+  profile_->GetPrefs()->SetBoolean(syncer::prefs::kSyncManaged, true);
 
   content::WindowedNotificationObserver(
       extensions::NOTIFICATION_CRX_INSTALLER_DONE,
@@ -199,9 +197,9 @@ TEST_F(ExternalProviderImplChromeOSTest, PriorityCompleted) {
       ->GetSyncableService(syncer::PRIORITY_PREFERENCES)
       ->MergeDataAndStartSyncing(syncer::PRIORITY_PREFERENCES,
                                  syncer::SyncDataList(),
-                                 scoped_ptr<syncer::SyncChangeProcessor>(
+                                 std::unique_ptr<syncer::SyncChangeProcessor>(
                                      new syncer::FakeSyncChangeProcessor),
-                                 scoped_ptr<syncer::SyncErrorFactory>(
+                                 std::unique_ptr<syncer::SyncErrorFactory>(
                                      new syncer::SyncErrorFactoryMock()));
 
   content::WindowedNotificationObserver(
