@@ -9,6 +9,7 @@
 #include <stdint.h>
 
 #include <map>
+#include <memory>
 #include <set>
 #include <string>
 #include <vector>
@@ -16,7 +17,6 @@
 #include "base/compiler_specific.h"
 #include "base/gtest_prod_util.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/scoped_observer.h"
 #include "base/strings/string16.h"
@@ -32,6 +32,7 @@ class SkBitmap;
 
 namespace content {
 class BrowserContext;
+class RenderFrameHost;
 class WebContents;
 struct ContextMenuParams;
 }
@@ -44,8 +45,8 @@ class StateStore;
 // Represents a menu item added by an extension.
 class MenuItem {
  public:
-  // A list of MenuItems.
-  typedef std::vector<MenuItem*> List;
+  using List = std::vector<MenuItem*>;
+  using OwnedList = std::vector<std::unique_ptr<MenuItem>>;
 
   // Key used to identify which extension a menu item belongs to.  A menu item
   // can also belong to a <webview>, in which case |webview_embedder_process_id|
@@ -143,8 +144,8 @@ class MenuItem {
       value_ |= context;
     }
 
-    scoped_ptr<base::Value> ToValue() const {
-      return scoped_ptr<base::Value>(
+    std::unique_ptr<base::Value> ToValue() const {
+      return std::unique_ptr<base::Value>(
           new base::FundamentalValue(static_cast<int>(value_)));
     }
 
@@ -174,7 +175,7 @@ class MenuItem {
     return id_.extension_key.extension_id;
   }
   const std::string& title() const { return title_; }
-  const List& children() { return children_; }
+  const OwnedList& children() { return children_; }
   const Id& id() const { return id_; }
   Id* parent_id() const { return parent_id_.get(); }
   int child_count() const { return children_.size(); }
@@ -210,13 +211,13 @@ class MenuItem {
   bool SetChecked(bool checked);
 
   // Converts to Value for serialization to preferences.
-  scoped_ptr<base::DictionaryValue> ToValue() const;
+  std::unique_ptr<base::DictionaryValue> ToValue() const;
 
   // Returns a new MenuItem created from |value|, or NULL if there is
-  // an error. The caller takes ownership of the MenuItem.
-  static MenuItem* Populate(const std::string& extension_id,
-                            const base::DictionaryValue& value,
-                            std::string* error);
+  // an error.
+  static std::unique_ptr<MenuItem> Populate(const std::string& extension_id,
+                                            const base::DictionaryValue& value,
+                                            std::string* error);
 
   // Sets any document and target URL patterns from |properties|.
   bool PopulateURLPatterns(std::vector<std::string>* document_url_patterns,
@@ -226,12 +227,11 @@ class MenuItem {
  protected:
   friend class MenuManager;
 
-  // Takes ownership of |item| and sets its parent_id_.
-  void AddChild(MenuItem* item);
+  // Adds |item| and sets its parent_id_.
+  void AddChild(std::unique_ptr<MenuItem> item);
 
-  // Takes the child item from this parent. The item is returned and the caller
-  // then owns the pointer.
-  MenuItem* ReleaseChild(const Id& child_id, bool recursive);
+  // Removes the child item from this parent and returns it.
+  std::unique_ptr<MenuItem> ReleaseChild(const Id& child_id, bool recursive);
 
   // Recursively appends all descendant items (children, grandchildren, etc.)
   // to the output |list|.
@@ -261,7 +261,7 @@ class MenuItem {
 
   // If this item is a child of another item, the unique id of its parent. If
   // this is a top-level item with no parent, this will be NULL.
-  scoped_ptr<Id> parent_id_;
+  std::unique_ptr<Id> parent_id_;
 
   // Patterns for restricting what documents this item will appear for. This
   // applies to the frame where the click took place.
@@ -272,7 +272,7 @@ class MenuItem {
   URLPatternSet target_url_patterns_;
 
   // Any children this item may have.
-  List children_;
+  OwnedList children_;
 
   DISALLOW_COPY_AND_ASSIGN(MenuItem);
 };
@@ -300,17 +300,19 @@ class MenuManager : public content::NotificationObserver,
   // items (added via AddChildItem); although those can be reached via the
   // top-level items' children. A view can then decide how to display these,
   // including whether to put them into a submenu if there are more than 1.
-  const MenuItem::List* MenuItems(const MenuItem::ExtensionKey& extension_key);
+  const MenuItem::OwnedList* MenuItems(
+      const MenuItem::ExtensionKey& extension_key);
 
   // Adds a top-level menu item for an extension, requiring the |extension|
-  // pointer so it can load the icon for the extension. Takes ownership of
-  // |item|. Returns a boolean indicating success or failure.
-  bool AddContextItem(const Extension* extension, MenuItem* item);
+  // pointer so it can load the icon for the extension. Returns a boolean
+  // indicating success or failure.
+  bool AddContextItem(const Extension* extension,
+                      std::unique_ptr<MenuItem> item);
 
-  // Add an item as a child of another item which has been previously added, and
-  // takes ownership of |item|. Returns a boolean indicating success or failure.
+  // Add an item as a child of another item which has been previously added.
+  // Returns a boolean indicating success or failure.
   bool AddChildItem(const MenuItem::Id& parent_id,
-                    MenuItem* child);
+                    std::unique_ptr<MenuItem> child);
 
   // Makes existing item with |child_id| a child of the item with |parent_id|.
   // If the child item was already a child of another parent, this will remove
@@ -340,6 +342,7 @@ class MenuManager : public content::NotificationObserver,
   // Called when a menu item is clicked on by the user.
   void ExecuteCommand(content::BrowserContext* context,
                       content::WebContents* web_contents,
+                      content::RenderFrameHost* render_frame_host,
                       const content::ContextMenuParams& params,
                       const MenuItem::Id& menu_item_id);
 
@@ -367,7 +370,7 @@ class MenuManager : public content::NotificationObserver,
   // Reads menu items for the extension from the state storage. Any invalid
   // items are ignored.
   void ReadFromStorage(const std::string& extension_id,
-                       scoped_ptr<base::Value> value);
+                       std::unique_ptr<base::Value> value);
 
   // Removes all "incognito" "split" mode context items.
   void RemoveAllIncognitoContextItems();
@@ -383,15 +386,14 @@ class MenuManager : public content::NotificationObserver,
   // Make sure that there is only one radio item selected at once in any run.
   // If there are no radio items selected, then the first item in the run
   // will get selected. If there are multiple radio items selected, then only
-  // the last one will get selcted.
-  void SanitizeRadioList(const MenuItem::List& item_list);
+  // the last one will get selected.
+  void SanitizeRadioList(const MenuItem::OwnedList& item_list);
 
   // Returns true if item is a descendant of an item with id |ancestor_id|.
   bool DescendantOf(MenuItem* item, const MenuItem::Id& ancestor_id);
 
   // We keep items organized by mapping ExtensionKey to a list of items.
-  typedef std::map<MenuItem::ExtensionKey, MenuItem::List> MenuItemMap;
-  MenuItemMap context_items_;
+  std::map<MenuItem::ExtensionKey, MenuItem::OwnedList> context_items_;
 
   // This lets us make lookup by id fast. It maps id to MenuItem* for
   // all items the menu manager knows about, including all children of top-level
