@@ -4,6 +4,8 @@
 
 #include "chrome/browser/extensions/webstore_startup_installer.h"
 
+#include "base/memory/ptr_util.h"
+
 namespace extensions {
 
 WebstoreStartupInstaller::WebstoreStartupInstaller(
@@ -19,11 +21,11 @@ WebstoreStartupInstaller::WebstoreStartupInstaller(
 
 WebstoreStartupInstaller::~WebstoreStartupInstaller() {}
 
-scoped_ptr<ExtensionInstallPrompt::Prompt>
+std::unique_ptr<ExtensionInstallPrompt::Prompt>
 WebstoreStartupInstaller::CreateInstallPrompt() const {
   if (show_prompt_) {
-    return make_scoped_ptr(new ExtensionInstallPrompt::Prompt(
-        ExtensionInstallPrompt::INSTALL_PROMPT));
+    return base::MakeUnique<ExtensionInstallPrompt::Prompt>(
+        ExtensionInstallPrompt::INSTALL_PROMPT);
   }
   return NULL;
 }
