@@ -74,9 +74,10 @@ class AndroidProviderBackendDelegate : public HistoryBackend::Delegate {
  public:
   AndroidProviderBackendDelegate() {}
 
-  void NotifyProfileError(sql::InitStatus init_status) override {}
+  void NotifyProfileError(sql::InitStatus init_status,
+                          const std::string& diagnostics) override {}
   void SetInMemoryBackend(
-      scoped_ptr<InMemoryHistoryBackend> backend) override {}
+      std::unique_ptr<InMemoryHistoryBackend> backend) override {}
   void NotifyFaviconsChanged(const std::set<GURL>& page_urls,
                              const GURL& icon_url) override {
     favicon_changed_.reset(
@@ -114,9 +115,9 @@ class AndroidProviderBackendDelegate : public HistoryBackend::Delegate {
   }
 
  private:
-  scoped_ptr<history::URLRows> deleted_details_;
-  scoped_ptr<history::URLRows> modified_details_;
-  scoped_ptr<std::set<GURL>> favicon_changed_;
+  std::unique_ptr<history::URLRows> deleted_details_;
+  std::unique_ptr<history::URLRows> modified_details_;
+  std::unique_ptr<std::set<GURL>> favicon_changed_;
 
   DISALLOW_COPY_AND_ASSIGN(AndroidProviderBackendDelegate);
 };
@@ -158,9 +159,9 @@ class AndroidProviderBackendNotifier : public HistoryBackendNotifier {
   }
 
  private:
-  scoped_ptr<history::URLRows> deleted_details_;
-  scoped_ptr<history::URLRows> modified_details_;
-  scoped_ptr<std::set<GURL>> favicon_changed_;
+  std::unique_ptr<history::URLRows> deleted_details_;
+  std::unique_ptr<history::URLRows> modified_details_;
+  std::unique_ptr<std::set<GURL>> favicon_changed_;
 
   DISALLOW_COPY_AND_ASSIGN(AndroidProviderBackendNotifier);
 };
@@ -185,7 +186,8 @@ class AndroidProviderBackendTest : public testing::Test {
     TestingProfile* testing_profile = profile_manager_.CreateTestingProfile(
         chrome::kInitialProfile);
     testing_profile->CreateBookmarkModel(true);
-    bookmark_model_ = BookmarkModelFactory::GetForProfile(testing_profile);
+    bookmark_model_ =
+        BookmarkModelFactory::GetForBrowserContext(testing_profile);
     history_client_.reset(new ChromeHistoryClient(bookmark_model_));
     history_backend_client_ = history_client_->CreateBackendClient();
     bookmarks::test::WaitForBookmarkModelToLoad(bookmark_model_);
@@ -199,10 +201,10 @@ class AndroidProviderBackendTest : public testing::Test {
     // Setup the database directory and files.
     ASSERT_TRUE(temp_dir_.CreateUniqueTempDir());
 
-    history_db_name_ = temp_dir_.path().AppendASCII(kHistoryFilename);
-    thumbnail_db_name_ = temp_dir_.path().AppendASCII(kFaviconsFilename);
-    android_cache_db_name_ = temp_dir_.path().AppendASCII(
-        "TestAndroidCache.db");
+    history_db_name_ = temp_dir_.GetPath().AppendASCII(kHistoryFilename);
+    thumbnail_db_name_ = temp_dir_.GetPath().AppendASCII(kFaviconsFilename);
+    android_cache_db_name_ =
+        temp_dir_.GetPath().AppendASCII("TestAndroidCache.db");
   }
 
   void AddBookmark(const GURL& url) {
@@ -258,8 +260,8 @@ class AndroidProviderBackendTest : public testing::Test {
   base::MessageLoopForUI message_loop_;
   content::TestBrowserThread ui_thread_;
   content::TestBrowserThread file_thread_;
-  scoped_ptr<history::HistoryClient> history_client_;
-  scoped_ptr<history::HistoryBackendClient> history_backend_client_;
+  std::unique_ptr<history::HistoryClient> history_client_;
+  std::unique_ptr<history::HistoryBackendClient> history_backend_client_;
 
   DISALLOW_COPY_AND_ASSIGN(AndroidProviderBackendTest);
 };
@@ -302,8 +304,8 @@ TEST_F(AndroidProviderBackendTest, UpdateTables) {
   history_backend = new HistoryBackend(new AndroidProviderBackendDelegate(),
                                        history_client_->CreateBackendClient(),
                                        message_loop_.task_runner());
-  history_backend->Init(std::string(), false,
-                        TestHistoryDatabaseParamsForPath(temp_dir_.path()));
+  history_backend->Init(false,
+                        TestHistoryDatabaseParamsForPath(temp_dir_.GetPath()));
   history_backend->AddVisits(url1, visits1, history::SOURCE_SYNCED);
   history_backend->AddVisits(url2, visits2, history::SOURCE_SYNCED);
   URLRow url_row;
@@ -328,12 +330,9 @@ TEST_F(AndroidProviderBackendTest, UpdateTables) {
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
   // Set url1 as bookmark.
   AddBookmark(url1);
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
 
   ASSERT_TRUE(backend->EnsureInitializedAndUpdated());
 
@@ -441,8 +440,8 @@ TEST_F(AndroidProviderBackendTest, QueryHistoryAndBookmarks) {
   history_backend = new HistoryBackend(new AndroidProviderBackendDelegate(),
                                        history_client_->CreateBackendClient(),
                                        message_loop_.task_runner());
-  history_backend->Init(std::string(), false,
-                        TestHistoryDatabaseParamsForPath(temp_dir_.path()));
+  history_backend->Init(false,
+                        TestHistoryDatabaseParamsForPath(temp_dir_.GetPath()));
   history_backend->AddVisits(url1, visits1, history::SOURCE_SYNCED);
   history_backend->AddVisits(url2, visits2, history::SOURCE_SYNCED);
 
@@ -469,12 +468,9 @@ TEST_F(AndroidProviderBackendTest, QueryHistoryAndBookmarks) {
   // Set url1 as bookmark.
   AddBookmark(url1);
 
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
 
   std::vector<HistoryAndBookmarkRow::ColumnID> projections;
 
@@ -487,7 +483,7 @@ TEST_F(AndroidProviderBackendTest, QueryHistoryAndBookmarks) {
   projections.push_back(HistoryAndBookmarkRow::FAVICON);
   projections.push_back(HistoryAndBookmarkRow::BOOKMARK);
 
-  scoped_ptr<AndroidStatement> statement(backend->QueryHistoryAndBookmarks(
+  std::unique_ptr<AndroidStatement> statement(backend->QueryHistoryAndBookmarks(
       projections, std::string(), std::vector<base::string16>(),
       std::string("url ASC")));
   ASSERT_TRUE(statement->statement()->Step());
@@ -558,12 +554,9 @@ TEST_F(AndroidProviderBackendTest, InsertHistoryAndBookmark) {
 
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
 
   ASSERT_TRUE(backend->InsertHistoryAndBookmark(row1));
   EXPECT_FALSE(notifier_.deleted_details());
@@ -609,7 +602,7 @@ TEST_F(AndroidProviderBackendTest, InsertHistoryAndBookmark) {
   projections.push_back(HistoryAndBookmarkRow::FAVICON);
   projections.push_back(HistoryAndBookmarkRow::BOOKMARK);
 
-  scoped_ptr<AndroidStatement> statement(backend->QueryHistoryAndBookmarks(
+  std::unique_ptr<AndroidStatement> statement(backend->QueryHistoryAndBookmarks(
       projections, std::string(), std::vector<base::string16>(),
       std::string("url ASC")));
   ASSERT_TRUE(statement->statement()->Step());
@@ -670,12 +663,9 @@ TEST_F(AndroidProviderBackendTest, DeleteHistoryAndBookmarks) {
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
 
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
 
   ASSERT_TRUE(backend->InsertHistoryAndBookmark(row1));
   ASSERT_TRUE(backend->InsertHistoryAndBookmark(row2));
@@ -719,7 +709,7 @@ TEST_F(AndroidProviderBackendTest, DeleteHistoryAndBookmarks) {
   projections.push_back(HistoryAndBookmarkRow::FAVICON);
   projections.push_back(HistoryAndBookmarkRow::BOOKMARK);
 
-  scoped_ptr<AndroidStatement> statement(backend->QueryHistoryAndBookmarks(
+  std::unique_ptr<AndroidStatement> statement(backend->QueryHistoryAndBookmarks(
       projections, std::string(), std::vector<base::string16>(),
       std::string("url ASC")));
   ASSERT_TRUE(statement->statement()->Step());
@@ -760,21 +750,19 @@ TEST_F(AndroidProviderBackendTest, DeleteHistoryAndBookmarks) {
               notifier_.favicon_changed()->find(row2.url()));
 
   ASSERT_EQ(1, deleted_count);
-  scoped_ptr<AndroidStatement> statement1(backend->QueryHistoryAndBookmarks(
-      projections, std::string(), std::vector<base::string16>(),
-      std::string("url ASC")));
+  std::unique_ptr<AndroidStatement> statement1(
+      backend->QueryHistoryAndBookmarks(projections, std::string(),
+                                        std::vector<base::string16>(),
+                                        std::string("url ASC")));
   ASSERT_FALSE(statement1->statement()->Step());
 }
 
 TEST_F(AndroidProviderBackendTest, IsValidHistoryAndBookmarkRow) {
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
 
   // The created time and last visit time are too close to have required visit
   // count.
@@ -862,12 +850,9 @@ TEST_F(AndroidProviderBackendTest, UpdateURL) {
 
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
 
   AndroidURLID id1 = backend->InsertHistoryAndBookmark(row1);
   ASSERT_TRUE(id1);
@@ -1044,12 +1029,9 @@ TEST_F(AndroidProviderBackendTest, UpdateVisitCount) {
 
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
 
   AndroidURLID id1 = backend->InsertHistoryAndBookmark(row1);
   ASSERT_TRUE(id1);
@@ -1127,12 +1109,9 @@ TEST_F(AndroidProviderBackendTest, UpdateLastVisitTime) {
 
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
 
   AndroidURLID id1 = backend->InsertHistoryAndBookmark(row1);
   ASSERT_TRUE(id1);
@@ -1191,12 +1170,9 @@ TEST_F(AndroidProviderBackendTest, UpdateFavicon) {
 
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
 
   AndroidURLID id1 = backend->InsertHistoryAndBookmark(row1);
   ASSERT_TRUE(id1);
@@ -1259,12 +1235,9 @@ TEST_F(AndroidProviderBackendTest, UpdateFavicon) {
 TEST_F(AndroidProviderBackendTest, UpdateSearchTermTable) {
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
   // Insert a keyword search item to verify if the update succeeds.
   HistoryAndBookmarkRow row1;
   row1.set_raw_url("cnn.com");
@@ -1340,12 +1313,9 @@ TEST_F(AndroidProviderBackendTest, UpdateSearchTermTable) {
 TEST_F(AndroidProviderBackendTest, QuerySearchTerms) {
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
   // Insert a keyword search item to verify if we can find it.
   HistoryAndBookmarkRow row1;
   row1.set_raw_url("cnn.com");
@@ -1362,9 +1332,9 @@ TEST_F(AndroidProviderBackendTest, QuerySearchTerms) {
   projections.push_back(SearchRow::ID);
   projections.push_back(SearchRow::SEARCH_TERM);
   projections.push_back(SearchRow::SEARCH_TIME);
-  scoped_ptr<AndroidStatement> statement(backend->QuerySearchTerms(
-      projections, std::string(), std::vector<base::string16>(),
-      std::string()));
+  std::unique_ptr<AndroidStatement> statement(
+      backend->QuerySearchTerms(projections, std::string(),
+                                std::vector<base::string16>(), std::string()));
   ASSERT_TRUE(statement.get());
   ASSERT_TRUE(statement->statement()->Step());
   EXPECT_TRUE(statement->statement()->ColumnInt64(0));
@@ -1377,12 +1347,9 @@ TEST_F(AndroidProviderBackendTest, QuerySearchTerms) {
 TEST_F(AndroidProviderBackendTest, UpdateSearchTerms) {
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
   // Insert a keyword.
   HistoryAndBookmarkRow row1;
   row1.set_raw_url("cnn.com");
@@ -1402,7 +1369,7 @@ TEST_F(AndroidProviderBackendTest, UpdateSearchTerms) {
   projections.push_back(SearchRow::SEARCH_TERM);
   std::vector<base::string16> args;
   args.push_back(term);
-  scoped_ptr<AndroidStatement> statement(backend->QuerySearchTerms(
+  std::unique_ptr<AndroidStatement> statement(backend->QuerySearchTerms(
       projections, "search = ?", args, std::string()));
   ASSERT_TRUE(statement.get());
   ASSERT_TRUE(statement->statement()->Step());
@@ -1484,12 +1451,9 @@ TEST_F(AndroidProviderBackendTest, UpdateSearchTerms) {
 TEST_F(AndroidProviderBackendTest, DeleteSearchTerms) {
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
   // Insert a keyword.
   HistoryAndBookmarkRow row1;
   row1.set_raw_url("cnn.com");
@@ -1509,7 +1473,7 @@ TEST_F(AndroidProviderBackendTest, DeleteSearchTerms) {
   projections.push_back(SearchRow::SEARCH_TERM);
   std::vector<base::string16> args;
   args.push_back(term);
-  scoped_ptr<AndroidStatement> statement(backend->QuerySearchTerms(
+  std::unique_ptr<AndroidStatement> statement(backend->QuerySearchTerms(
       projections, "search = ?", args, std::string()));
   ASSERT_TRUE(statement.get());
   ASSERT_TRUE(statement->statement()->Step());
@@ -1593,12 +1557,9 @@ TEST_F(AndroidProviderBackendTest, DeleteSearchTerms) {
 TEST_F(AndroidProviderBackendTest, InsertSearchTerm) {
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
   SearchRow search_row;
   search_row.set_search_term(UTF8ToUTF16("google"));
   search_row.set_url(GURL("http://google.com"));
@@ -1616,8 +1577,8 @@ TEST_F(AndroidProviderBackendTest, InsertSearchTerm) {
   std::ostringstream oss;
   oss << id;
   args.push_back(UTF8ToUTF16(oss.str()));
-  scoped_ptr<AndroidStatement> statement(backend->QuerySearchTerms(
-      projections, "_id = ?", args, std::string()));
+  std::unique_ptr<AndroidStatement> statement(
+      backend->QuerySearchTerms(projections, "_id = ?", args, std::string()));
   ASSERT_TRUE(statement.get());
   ASSERT_TRUE(statement->statement()->Step());
   EXPECT_EQ(id, statement->statement()->ColumnInt64(0));
@@ -1650,12 +1611,9 @@ TEST_F(AndroidProviderBackendTest, DeleteHistory) {
 
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
 
   AndroidURLID id1 = backend->InsertHistoryAndBookmark(row1);
   ASSERT_TRUE(id1);
@@ -1706,12 +1664,9 @@ TEST_F(AndroidProviderBackendTest, DeleteHistory) {
 TEST_F(AndroidProviderBackendTest, TestMultipleNestingTransaction) {
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
 
   // Create the nested transactions.
   history_db_.BeginTransaction();
@@ -1759,12 +1714,9 @@ TEST_F(AndroidProviderBackendTest, TestAndroidCTSComplianceForZeroVisitCount) {
   // count is 0.
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
   URLRow url_row(GURL("http://www.google.com"));
   url_row.set_last_visit(Time::Now());
   url_row.set_visit_count(0);
@@ -1781,7 +1733,7 @@ TEST_F(AndroidProviderBackendTest, TestAndroidCTSComplianceForZeroVisitCount) {
   projections.push_back(HistoryAndBookmarkRow::FAVICON);
   projections.push_back(HistoryAndBookmarkRow::BOOKMARK);
 
-  scoped_ptr<AndroidStatement> statement(backend->QueryHistoryAndBookmarks(
+  std::unique_ptr<AndroidStatement> statement(backend->QueryHistoryAndBookmarks(
       projections, std::string(), std::vector<base::string16>(),
       std::string("url ASC")));
 
@@ -1799,12 +1751,9 @@ TEST_F(AndroidProviderBackendTest, AndroidCTSComplianceFolderColumnExists) {
   // is 1.
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
   ASSERT_EQ(sql::INIT_OK, thumbnail_db_.Init(thumbnail_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 &thumbnail_db_,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+      android_cache_db_name_, &history_db_, &thumbnail_db_,
+      history_backend_client_.get(), &notifier_));
   HistoryAndBookmarkRow row1;
   row1.set_raw_url("cnn.com");
   row1.set_url(GURL("http://cnn.com"));
@@ -1835,7 +1784,7 @@ TEST_F(AndroidProviderBackendTest, AndroidCTSComplianceFolderColumnExists) {
 
   projections.push_back(HistoryAndBookmarkRow::URL);
 
-  scoped_ptr<AndroidStatement> statement(backend->QueryHistoryAndBookmarks(
+  std::unique_ptr<AndroidStatement> statement(backend->QueryHistoryAndBookmarks(
       projections, std::string("folder=0"), std::vector<base::string16>(),
       std::string("url ASC")));
   ASSERT_TRUE(statement->statement()->Step());
@@ -1879,8 +1828,8 @@ TEST_F(AndroidProviderBackendTest, QueryWithoutThumbnailDB) {
   history_backend = new HistoryBackend(new AndroidProviderBackendDelegate(),
                                        history_client_->CreateBackendClient(),
                                        message_loop_.task_runner());
-  history_backend->Init(std::string(), false,
-                        TestHistoryDatabaseParamsForPath(temp_dir_.path()));
+  history_backend->Init(false,
+                        TestHistoryDatabaseParamsForPath(temp_dir_.GetPath()));
   history_backend->AddVisits(url1, visits1, history::SOURCE_SYNCED);
   history_backend->AddVisits(url2, visits2, history::SOURCE_SYNCED);
   URLRow url_row;
@@ -1909,12 +1858,9 @@ TEST_F(AndroidProviderBackendTest, QueryWithoutThumbnailDB) {
   // Set url1 as bookmark.
   AddBookmark(url1);
 
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 NULL,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(
+      new AndroidProviderBackend(android_cache_db_name_, &history_db_, NULL,
+                                 history_backend_client_.get(), &notifier_));
 
   std::vector<HistoryAndBookmarkRow::ColumnID> projections;
 
@@ -1927,7 +1873,7 @@ TEST_F(AndroidProviderBackendTest, QueryWithoutThumbnailDB) {
   projections.push_back(HistoryAndBookmarkRow::FAVICON);
   projections.push_back(HistoryAndBookmarkRow::BOOKMARK);
 
-  scoped_ptr<AndroidStatement> statement(backend->QueryHistoryAndBookmarks(
+  std::unique_ptr<AndroidStatement> statement(backend->QueryHistoryAndBookmarks(
       projections, std::string(), std::vector<base::string16>(),
       std::string("url ASC")));
   ASSERT_TRUE(statement->statement()->Step());
@@ -1982,12 +1928,9 @@ TEST_F(AndroidProviderBackendTest, InsertWithoutThumbnailDB) {
   row2.set_favicon(base::RefCountedBytes::TakeVector(&data));
 
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 NULL,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(
+      new AndroidProviderBackend(android_cache_db_name_, &history_db_, NULL,
+                                 history_backend_client_.get(), &notifier_));
 
   ASSERT_TRUE(backend->InsertHistoryAndBookmark(row1));
   EXPECT_FALSE(notifier_.deleted_details());
@@ -2049,12 +1992,9 @@ TEST_F(AndroidProviderBackendTest, DeleteWithoutThumbnailDB) {
     ASSERT_EQ(sql::INIT_OK, history_db.Init(history_db_name_));
     ASSERT_EQ(sql::INIT_OK, thumbnail_db.Init(thumbnail_db_name_));
 
-    scoped_ptr<AndroidProviderBackend> backend(
-        new AndroidProviderBackend(android_cache_db_name_,
-                                   &history_db,
-                                   &thumbnail_db,
-                                   history_backend_client_.get(),
-                                   &notifier_));
+    std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+        android_cache_db_name_, &history_db, &thumbnail_db,
+        history_backend_client_.get(), &notifier_));
 
     ASSERT_TRUE(backend->InsertHistoryAndBookmark(row1));
     ASSERT_TRUE(backend->InsertHistoryAndBookmark(row2));
@@ -2067,12 +2007,9 @@ TEST_F(AndroidProviderBackendTest, DeleteWithoutThumbnailDB) {
     EXPECT_EQ(row1.url(), child->url());
   }
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 NULL,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(
+      new AndroidProviderBackend(android_cache_db_name_, &history_db_, NULL,
+                                 history_backend_client_.get(), &notifier_));
 
   // Delete all rows.
   std::vector<base::string16> args;
@@ -2104,9 +2041,10 @@ TEST_F(AndroidProviderBackendTest, DeleteWithoutThumbnailDB) {
   projections.push_back(HistoryAndBookmarkRow::FAVICON);
   projections.push_back(HistoryAndBookmarkRow::BOOKMARK);
 
-  scoped_ptr<AndroidStatement> statement1(backend->QueryHistoryAndBookmarks(
-      projections, std::string(), std::vector<base::string16>(),
-      std::string("url ASC")));
+  std::unique_ptr<AndroidStatement> statement1(
+      backend->QueryHistoryAndBookmarks(projections, std::string(),
+                                        std::vector<base::string16>(),
+                                        std::string("url ASC")));
   ASSERT_FALSE(statement1->statement()->Step());
 }
 
@@ -2125,24 +2063,18 @@ TEST_F(AndroidProviderBackendTest, UpdateFaviconWithoutThumbnail) {
     ThumbnailDatabase thumbnail_db(NULL);
     ASSERT_EQ(sql::INIT_OK, history_db.Init(history_db_name_));
     ASSERT_EQ(sql::INIT_OK, thumbnail_db.Init(thumbnail_db_name_));
-    scoped_ptr<AndroidProviderBackend> backend(
-        new AndroidProviderBackend(android_cache_db_name_,
-                                   &history_db,
-                                   &thumbnail_db,
-                                   history_backend_client_.get(),
-                                   &notifier_));
+    std::unique_ptr<AndroidProviderBackend> backend(new AndroidProviderBackend(
+        android_cache_db_name_, &history_db, &thumbnail_db,
+        history_backend_client_.get(), &notifier_));
 
     AndroidURLID id1 = backend->InsertHistoryAndBookmark(row1);
     ASSERT_TRUE(id1);
   }
 
   ASSERT_EQ(sql::INIT_OK, history_db_.Init(history_db_name_));
-  scoped_ptr<AndroidProviderBackend> backend(
-      new AndroidProviderBackend(android_cache_db_name_,
-                                 &history_db_,
-                                 NULL,
-                                 history_backend_client_.get(),
-                                 &notifier_));
+  std::unique_ptr<AndroidProviderBackend> backend(
+      new AndroidProviderBackend(android_cache_db_name_, &history_db_, NULL,
+                                 history_backend_client_.get(), &notifier_));
 
   int update_count;
   std::vector<base::string16> update_args;
