@@ -12,25 +12,26 @@
 
 #include "base/hash.h"
 
-namespace media_router {
+class GURL;
 
+// TODO(mfoltz): Right now this is a wrapper for std::string.  Factor methods
+// from media_source_helper here so this object becomes useful; and don't just
+// pass it around by Id.
+namespace media_router {
 
 class MediaSource {
  public:
   using Id = std::string;
 
   explicit MediaSource(const MediaSource::Id& id);
-  MediaSource();
+  explicit MediaSource(const GURL& presentation_url);
   ~MediaSource();
 
   // Gets the ID of the media source.
   MediaSource::Id id() const;
 
   // Returns true if two MediaSource objects use the same media ID.
-  bool Equals(const MediaSource& other) const;
-
-  // Returns true if a MediaSource is empty or uninitialized.
-  bool Empty() const;
+  bool operator==(const MediaSource& other) const;
 
   // Used for logging.
   std::string ToString() const;
