@@ -5,6 +5,7 @@
 #include "chrome/browser/media_galleries/fileapi/iapps_data_provider.h"
 
 #include <map>
+#include <utility>
 
 #include "base/bind.h"
 #include "base/callback.h"
@@ -28,7 +29,7 @@ IAppsDataProvider::IAppsDataProvider(const base::FilePath& library_path)
       needs_refresh_(true),
       is_valid_(false),
       weak_factory_(this) {
-  DCHECK(MediaFileSystemBackend::CurrentlyOnMediaTaskRunnerThread());
+  MediaFileSystemBackend::AssertCurrentlyOnMediaSequence();
   DCHECK(!library_path_.empty());
 
   StartFilePathWatchOnMediaTaskRunner(
@@ -39,7 +40,9 @@ IAppsDataProvider::IAppsDataProvider(const base::FilePath& library_path)
                  weak_factory_.GetWeakPtr()));
 }
 
-IAppsDataProvider::~IAppsDataProvider() {}
+IAppsDataProvider::~IAppsDataProvider() {
+  StopFilePathWatchOnMediaTaskRunner(std::move(library_watcher_));
+}
 
 bool IAppsDataProvider::valid() const {
   return is_valid_;
@@ -50,7 +53,7 @@ void IAppsDataProvider::set_valid(bool valid) {
 }
 
 void IAppsDataProvider::RefreshData(const ReadyCallback& ready_callback) {
-  DCHECK(MediaFileSystemBackend::CurrentlyOnMediaTaskRunnerThread());
+  MediaFileSystemBackend::AssertCurrentlyOnMediaSequence();
   if (!needs_refresh_) {
     ready_callback.Run(valid());
     return;
@@ -66,14 +69,14 @@ const base::FilePath& IAppsDataProvider::library_path() const {
 }
 
 void IAppsDataProvider::OnLibraryWatchStarted(
-    scoped_ptr<base::FilePathWatcher> library_watcher) {
-  DCHECK(MediaFileSystemBackend::CurrentlyOnMediaTaskRunnerThread());
-  library_watcher_.reset(library_watcher.release());
+    std::unique_ptr<base::FilePathWatcher> library_watcher) {
+  MediaFileSystemBackend::AssertCurrentlyOnMediaSequence();
+  library_watcher_ = std::move(library_watcher);
 }
 
 void IAppsDataProvider::OnLibraryChanged(const base::FilePath& path,
                                          bool error) {
-  DCHECK(MediaFileSystemBackend::CurrentlyOnMediaTaskRunnerThread());
+  MediaFileSystemBackend::AssertCurrentlyOnMediaSequence();
   DCHECK_EQ(library_path_.value(), path.value());
   if (error)
     LOG(ERROR) << "Error watching " << library_path_.value();
