@@ -14,10 +14,15 @@
 #include "base/time/time.h"
 #include "build/build_config.h"
 
+namespace content {
+class RenderProcessHost;
+}  // namespace content
+
 namespace memory {
 
 struct TabStats {
   TabStats();
+  TabStats(const TabStats& other);
   ~TabStats();
   bool is_app;            // Browser window is an app.
   bool is_internal_page;  // Internal page, such as NTP or Settings.
@@ -28,6 +33,8 @@ struct TabStats {
   bool has_form_entry;  // User has entered text in a form.
   int discard_count;
   base::TimeTicks last_active;
+  base::TimeTicks last_hidden;
+  content::RenderProcessHost* render_process_host;
   base::ProcessHandle renderer_handle;
   int child_process_host_id;
   base::string16 title;
@@ -35,6 +42,7 @@ struct TabStats {
   int oom_score;
 #endif
   int64_t tab_contents_id;  // Unique ID per WebContents.
+  bool is_auto_discardable;
 };
 
 typedef std::vector<TabStats> TabStatsList;
