@@ -7,19 +7,28 @@
 #include <stdint.h>
 
 #include "base/macros.h"
+#include "extensions/features/features.h"
 #include "net/base/net_errors.h"
 
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
+#include "base/debug/alias.h"
+#include "base/debug/dump_without_crashing.h"
+#include "base/strings/string_util.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/extensions/api/proxy/proxy_api.h"
 #include "chrome/browser/extensions/event_router_forwarder.h"
 #include "chrome/browser/profiles/profile_manager.h"
+#include "chrome/browser/renderer_host/chrome_navigation_ui_data.h"
 #include "content/public/browser/browser_thread.h"
+#include "content/public/browser/child_process_security_policy.h"
 #include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/resource_request_info.h"
+#include "content/public/common/browser_side_navigation_policy.h"
 #include "extensions/browser/api/web_request/web_request_api.h"
+#include "extensions/browser/extension_navigation_ui_data.h"
 #include "extensions/browser/info_map.h"
 #include "extensions/browser/process_manager.h"
+#include "extensions/common/permissions/api_permission.h"
 #include "net/url_request/url_request.h"
 
 using content::BrowserThread;
@@ -90,17 +99,17 @@ class ChromeExtensionsNetworkDelegateImpl
 
  private:
   // ChromeExtensionsNetworkDelegate implementation.
-  void ForwardProxyErrors(net::URLRequest* request) override;
+  void ForwardProxyErrors(net::URLRequest* request, int net_error) override;
   void ForwardStartRequestStatus(net::URLRequest* request) override;
   void ForwardDoneRequestStatus(net::URLRequest* request) override;
   int OnBeforeURLRequest(net::URLRequest* request,
                          const net::CompletionCallback& callback,
                          GURL* new_url) override;
-  int OnBeforeSendHeaders(net::URLRequest* request,
-                          const net::CompletionCallback& callback,
-                          net::HttpRequestHeaders* headers) override;
-  void OnSendHeaders(net::URLRequest* request,
-                     const net::HttpRequestHeaders& headers) override;
+  int OnBeforeStartTransaction(net::URLRequest* request,
+                               const net::CompletionCallback& callback,
+                               net::HttpRequestHeaders* headers) override;
+  void OnStartTransaction(net::URLRequest* request,
+                          const net::HttpRequestHeaders& headers) override;
   int OnHeadersReceived(
       net::URLRequest* request,
       const net::CompletionCallback& callback,
@@ -109,8 +118,10 @@ class ChromeExtensionsNetworkDelegateImpl
       GURL* allowed_unsafe_redirect_url) override;
   void OnBeforeRedirect(net::URLRequest* request,
                         const GURL& new_location) override;
-  void OnResponseStarted(net::URLRequest* request) override;
-  void OnCompleted(net::URLRequest* request, bool started) override;
+  void OnResponseStarted(net::URLRequest* request, int net_error) override;
+  void OnCompleted(net::URLRequest* request,
+                   bool started,
+                   int net_error) override;
   void OnURLRequestDestroyed(net::URLRequest* request) override;
   void OnPACScriptError(int line_number, const base::string16& error) override;
   net::NetworkDelegate::AuthRequiredResponse OnAuthRequired(
@@ -133,14 +144,15 @@ ChromeExtensionsNetworkDelegateImpl::ChromeExtensionsNetworkDelegateImpl(
 ChromeExtensionsNetworkDelegateImpl::~ChromeExtensionsNetworkDelegateImpl() {}
 
 void ChromeExtensionsNetworkDelegateImpl::ForwardProxyErrors(
-    net::URLRequest* request) {
-  if (request->status().status() == net::URLRequestStatus::FAILED) {
-    switch (request->status().error()) {
+    net::URLRequest* request,
+    int net_error) {
+  if (net_error != net::OK) {
+    switch (net_error) {
       case net::ERR_PROXY_AUTH_UNSUPPORTED:
       case net::ERR_PROXY_CONNECTION_FAILED:
       case net::ERR_TUNNEL_CONNECTION_FAILED:
         extensions::ProxyEventRouter::GetInstance()->OnProxyError(
-            event_router_.get(), profile_, request->status().error());
+            event_router_.get(), profile_, net_error);
     }
   }
 }
@@ -159,11 +171,62 @@ int ChromeExtensionsNetworkDelegateImpl::OnBeforeURLRequest(
     net::URLRequest* request,
     const net::CompletionCallback& callback,
     GURL* new_url) {
+  const content::ResourceRequestInfo* info =
+      content::ResourceRequestInfo::ForRequest(request);
+  GURL url(request->url());
+
+  // Block top-level navigations to blob: or filesystem: URLs with extension
+  // origin from non-extension processes.  See https://crbug.com/645028.
+  //
+  // TODO(alexmos): This check is redundant with the one in
+  // ExtensionNavigationThrottle::WillStartRequest, which was introduced in
+  // M56. This check is reintroduced temporarily to tighten this blocking for
+  // apps with a "webview" permission on M55/54 (see https://crbug.com/656752).
+  // It will be removed after it's merged.  Unlike the check in
+  // ExtensionNavigationThrottle, this check is incompatible with PlzNavigate
+  // and is disabled for that mode.
+  bool is_nested_url = url.SchemeIsFileSystem() || url.SchemeIsBlob();
+  bool is_navigation =
+      info && content::IsResourceTypeFrame(info->GetResourceType());
+  url::Origin origin(url);
+  if (is_nested_url && is_navigation && info->IsMainFrame() &&
+      origin.scheme() == extensions::kExtensionScheme &&
+      !extension_info_map_->process_map().Contains(info->GetChildID()) &&
+      !content::IsBrowserSideNavigationEnabled()) {
+    // Relax this restriction for apps that use <webview>.  See
+    // https://crbug.com/652077.
+    const extensions::Extension* extension =
+        extension_info_map_->extensions().GetByID(origin.host());
+    bool has_webview_permission =
+        extension &&
+        extension->permissions_data()->HasAPIPermission(
+            extensions::APIPermission::kWebView);
+    // Check whether the request is coming from a <webview> guest process via
+    // ChildProcessSecurityPolicy.  A guest process should have already been
+    // granted permission to request |origin| when its WebContents was created.
+    // See https://crbug.com/656752.
+    auto* policy = content::ChildProcessSecurityPolicy::GetInstance();
+    bool from_guest =
+        policy->HasSpecificPermissionForOrigin(info->GetChildID(), origin);
+    if (!has_webview_permission || !from_guest) {
+      // TODO(alexmos): Temporary instrumentation to find any regressions for
+      // this blocking.  Remove after verifying that this is not breaking any
+      // legitimate use cases.
+      char origin_copy[256];
+      base::strlcpy(origin_copy, origin.Serialize().c_str(),
+                    arraysize(origin_copy));
+      base::debug::Alias(&origin_copy);
+      base::debug::Alias(&from_guest);
+      base::debug::DumpWithoutCrashing();
+      return net::ERR_ABORTED;
+    }
+  }
+
   return ExtensionWebRequestEventRouter::GetInstance()->OnBeforeRequest(
       profile_, extension_info_map_.get(), request, callback, new_url);
 }
 
-int ChromeExtensionsNetworkDelegateImpl::OnBeforeSendHeaders(
+int ChromeExtensionsNetworkDelegateImpl::OnBeforeStartTransaction(
     net::URLRequest* request,
     const net::CompletionCallback& callback,
     net::HttpRequestHeaders* headers) {
@@ -171,7 +234,7 @@ int ChromeExtensionsNetworkDelegateImpl::OnBeforeSendHeaders(
       profile_, extension_info_map_.get(), request, callback, headers);
 }
 
-void ChromeExtensionsNetworkDelegateImpl::OnSendHeaders(
+void ChromeExtensionsNetworkDelegateImpl::OnStartTransaction(
     net::URLRequest* request,
     const net::HttpRequestHeaders& headers) {
   ExtensionWebRequestEventRouter::GetInstance()->OnSendHeaders(
@@ -185,12 +248,8 @@ int ChromeExtensionsNetworkDelegateImpl::OnHeadersReceived(
     scoped_refptr<net::HttpResponseHeaders>* override_response_headers,
     GURL* allowed_unsafe_redirect_url) {
   return ExtensionWebRequestEventRouter::GetInstance()->OnHeadersReceived(
-      profile_,
-      extension_info_map_.get(),
-      request,
-      callback,
-      original_response_headers,
-      override_response_headers,
+      profile_, extension_info_map_.get(), request, callback,
+      original_response_headers, override_response_headers,
       allowed_unsafe_redirect_url);
 }
 
@@ -201,36 +260,32 @@ void ChromeExtensionsNetworkDelegateImpl::OnBeforeRedirect(
       profile_, extension_info_map_.get(), request, new_location);
 }
 
-
 void ChromeExtensionsNetworkDelegateImpl::OnResponseStarted(
-    net::URLRequest* request) {
+    net::URLRequest* request,
+    int net_error) {
   ExtensionWebRequestEventRouter::GetInstance()->OnResponseStarted(
-      profile_, extension_info_map_.get(), request);
-  ForwardProxyErrors(request);
+      profile_, extension_info_map_.get(), request, net_error);
+  ForwardProxyErrors(request, net_error);
 }
 
-void ChromeExtensionsNetworkDelegateImpl::OnCompleted(
-    net::URLRequest* request,
-    bool started) {
-  if (request->status().status() == net::URLRequestStatus::SUCCESS) {
-    bool is_redirect = request->response_headers() &&
-        net::HttpResponseHeaders::IsRedirectResponseCode(
-            request->response_headers()->response_code());
-    if (!is_redirect) {
-      ExtensionWebRequestEventRouter::GetInstance()->OnCompleted(
-          profile_, extension_info_map_.get(), request);
-    }
-    return;
-  }
+void ChromeExtensionsNetworkDelegateImpl::OnCompleted(net::URLRequest* request,
+                                                      bool started,
+                                                      int net_error) {
+  DCHECK_NE(net::ERR_IO_PENDING, net_error);
 
-  if (request->status().status() == net::URLRequestStatus::FAILED ||
-      request->status().status() == net::URLRequestStatus::CANCELED) {
+  if (net_error != net::OK) {
     ExtensionWebRequestEventRouter::GetInstance()->OnErrorOccurred(
-        profile_, extension_info_map_.get(), request, started);
+        profile_, extension_info_map_.get(), request, started, net_error);
     return;
   }
 
-  NOTREACHED();
+  bool is_redirect = request->response_headers() &&
+                     net::HttpResponseHeaders::IsRedirectResponseCode(
+                         request->response_headers()->response_code());
+  if (!is_redirect) {
+    ExtensionWebRequestEventRouter::GetInstance()->OnCompleted(
+        profile_, extension_info_map_.get(), request, net_error);
+  }
 }
 
 void ChromeExtensionsNetworkDelegateImpl::OnURLRequestDestroyed(
@@ -259,12 +314,12 @@ ChromeExtensionsNetworkDelegateImpl::OnAuthRequired(
 
 }  // namespace
 
-#endif  // defined(ENABLE_EXTENSIONS)
+#endif  // BUILDFLAG(ENABLE_EXTENSIONS)
 
 // static
 ChromeExtensionsNetworkDelegate* ChromeExtensionsNetworkDelegate::Create(
     extensions::EventRouterForwarder* event_router) {
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
   return new ChromeExtensionsNetworkDelegateImpl(event_router);
 #else
   return new ChromeExtensionsNetworkDelegate();
@@ -279,14 +334,14 @@ ChromeExtensionsNetworkDelegate::~ChromeExtensionsNetworkDelegate() {}
 
 void ChromeExtensionsNetworkDelegate::set_extension_info_map(
     extensions::InfoMap* extension_info_map) {
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
   extension_info_map_ = extension_info_map;
 #endif
 }
 
 void ChromeExtensionsNetworkDelegate::ForwardProxyErrors(
-    net::URLRequest* request) {
-}
+    net::URLRequest* request,
+    int net_error) {}
 
 void ChromeExtensionsNetworkDelegate::ForwardStartRequestStatus(
     net::URLRequest* request) {
@@ -303,17 +358,16 @@ int ChromeExtensionsNetworkDelegate::OnBeforeURLRequest(
   return net::OK;
 }
 
-int ChromeExtensionsNetworkDelegate::OnBeforeSendHeaders(
+int ChromeExtensionsNetworkDelegate::OnBeforeStartTransaction(
     net::URLRequest* request,
     const net::CompletionCallback& callback,
     net::HttpRequestHeaders* headers) {
   return net::OK;
 }
 
-void ChromeExtensionsNetworkDelegate::OnSendHeaders(
+void ChromeExtensionsNetworkDelegate::OnStartTransaction(
     net::URLRequest* request,
-    const net::HttpRequestHeaders& headers) {
-}
+    const net::HttpRequestHeaders& headers) {}
 
 int ChromeExtensionsNetworkDelegate::OnHeadersReceived(
     net::URLRequest* request,
@@ -329,15 +383,13 @@ void ChromeExtensionsNetworkDelegate::OnBeforeRedirect(
     const GURL& new_location) {
 }
 
-
 void ChromeExtensionsNetworkDelegate::OnResponseStarted(
-    net::URLRequest* request) {
-}
-
-void ChromeExtensionsNetworkDelegate::OnCompleted(
     net::URLRequest* request,
-    bool started) {
-}
+    int net_error) {}
+
+void ChromeExtensionsNetworkDelegate::OnCompleted(net::URLRequest* request,
+                                                  bool started,
+                                                  int net_error) {}
 
 void ChromeExtensionsNetworkDelegate::OnURLRequestDestroyed(
     net::URLRequest* request) {
