@@ -4,11 +4,11 @@
 
 #include "chrome/browser/notifications/message_center_notification_manager.h"
 
+#include <memory>
 #include <utility>
 
 #include "base/logging.h"
-#include "base/memory/scoped_ptr.h"
-#include "base/stl_util.h"
+#include "base/memory/ptr_util.h"
 #include "base/strings/stringprintf.h"
 #include "build/build_config.h"
 #include "chrome/browser/extensions/api/notification_provider/notification_provider_api.h"
@@ -39,23 +39,15 @@
 #endif
 
 #if defined(USE_ASH)
+#include "ash/common/system/web_notification/web_notification_tray.h"
 #include "ash/shell.h"
-#include "ash/system/web_notification/web_notification_tray.h"
 #endif
 
-// Mac does support native notifications and defines this method
-// in notification_ui_manager_mac.mm
-#if !defined(OS_MACOSX)
-// static
-NotificationUIManager*
-NotificationUIManager::CreateNativeNotificationManager() {
-  return nullptr;
-}
-#endif
+using message_center::NotifierId;
 
 MessageCenterNotificationManager::MessageCenterNotificationManager(
     message_center::MessageCenter* message_center,
-    scoped_ptr<message_center::NotifierSettingsProvider> settings_provider)
+    std::unique_ptr<message_center::NotifierSettingsProvider> settings_provider)
     : message_center_(message_center),
       settings_provider_(std::move(settings_provider)),
       system_observer_(this),
@@ -65,14 +57,14 @@ MessageCenterNotificationManager::MessageCenterNotificationManager(
   message_center_->SetNotifierSettingsProvider(settings_provider_.get());
 
 #if defined(OS_CHROMEOS)
-  blockers_.push_back(make_scoped_ptr(
-      new LoginStateNotificationBlockerChromeOS(message_center)));
+  blockers_.push_back(
+      base::MakeUnique<LoginStateNotificationBlockerChromeOS>(message_center));
 #else
-  blockers_.push_back(make_scoped_ptr(
-      new ScreenLockNotificationBlocker(message_center)));
+  blockers_.push_back(
+      base::MakeUnique<ScreenLockNotificationBlocker>(message_center));
 #endif
-  blockers_.push_back(make_scoped_ptr(
-      new FullscreenNotificationBlocker(message_center)));
+  blockers_.push_back(
+      base::MakeUnique<FullscreenNotificationBlocker>(message_center));
 
 #if defined(OS_WIN) || defined(OS_MACOSX) \
   || (defined(OS_LINUX) && !defined(OS_CHROMEOS))
@@ -83,11 +75,9 @@ MessageCenterNotificationManager::MessageCenterNotificationManager(
 }
 
 MessageCenterNotificationManager::~MessageCenterNotificationManager() {
-  message_center_->SetNotifierSettingsProvider(NULL);
+  message_center_->SetNotifierSettingsProvider(nullptr);
   message_center_->RemoveObserver(this);
 
-  STLDeleteContainerPairSecondPointers(profile_notifications_.begin(),
-                                       profile_notifications_.end());
   profile_notifications_.clear();
 }
 
@@ -99,8 +89,9 @@ void MessageCenterNotificationManager::Add(const Notification& notification,
   if (Update(notification, profile))
     return;
 
-  ProfileNotification* profile_notification =
-      new ProfileNotification(profile, notification);
+  std::unique_ptr<ProfileNotification> profile_notification_ptr =
+      base::MakeUnique<ProfileNotification>(profile, notification);
+  ProfileNotification* profile_notification = profile_notification_ptr.get();
 
   ExtensionWelcomeNotificationFactory::GetForBrowserContext(profile)->
       ShowWelcomeNotificationIfNecessary(profile_notification->notification());
@@ -109,7 +100,7 @@ void MessageCenterNotificationManager::Add(const Notification& notification,
   // via the notification within a ProfileNotification object or the profile ID
   // will not be correctly set for ChromeOS.
   // Takes ownership of profile_notification.
-  AddProfileNotification(profile_notification);
+  AddProfileNotification(std::move(profile_notification_ptr));
 
   // TODO(liyanhou): Change the logic to only send notifications to one party.
   // Currently, if there is an app with notificationProvider permission,
@@ -124,8 +115,9 @@ void MessageCenterNotificationManager::Add(const Notification& notification,
     AddNotificationToAlternateProvider(profile_notification->notification(),
                                        profile, extension_id);
 
-  message_center_->AddNotification(make_scoped_ptr(
-      new message_center::Notification(profile_notification->notification())));
+  message_center_->AddNotification(
+      base::MakeUnique<message_center::Notification>(
+          profile_notification->notification()));
 }
 
 bool MessageCenterNotificationManager::Update(const Notification& notification,
@@ -140,9 +132,9 @@ bool MessageCenterNotificationManager::Update(const Notification& notification,
   // Since tag is provided by arbitrary JS, we need to use origin_url
   // (which is an app url in case of app/extension) to scope the tags
   // in the given profile.
-  for (NotificationMap::iterator iter = profile_notifications_.begin();
+  for (auto iter = profile_notifications_.begin();
        iter != profile_notifications_.end(); ++iter) {
-    ProfileNotification* old_notification = (*iter).second;
+    ProfileNotification* old_notification = (*iter).second.get();
     if (old_notification->notification().tag() == tag &&
         old_notification->notification().origin_url() == origin_url &&
         old_notification->profile_id() ==
@@ -153,23 +145,27 @@ bool MessageCenterNotificationManager::Update(const Notification& notification,
 
       // Add/remove notification in the local list but just update the same
       // one in MessageCenter.
-      delete old_notification;
+      std::unique_ptr<ProfileNotification> new_notification =
+          base::MakeUnique<ProfileNotification>(profile, notification);
+      const Notification& notification = new_notification->notification();
+      // Delete the old one after the new one is created to ensure we don't run
+      // out of KeepAlives.
       profile_notifications_.erase(old_id);
-      ProfileNotification* new_notification =
-          new ProfileNotification(profile, notification);
-      profile_notifications_[new_notification->notification().id()] =
-          new_notification;
+      profile_notifications_[notification.id()] = std::move(new_notification);
 
       // TODO(liyanhou): Add routing updated notifications to alternative
       // providers.
 
+      // Non-persistent Web Notifications rely on receiving the Display() event
+      // to inform the developer, even when replacing a previous notification.
+      if (notification.notifier_id().type == NotifierId::WEB_PAGE)
+        notification.delegate()->Display();
+
       // WARNING: You MUST use AddProfileNotification or update the message
       // center via the notification within a ProfileNotification object or the
       // profile ID will not be correctly set for ChromeOS.
       message_center_->UpdateNotification(
-          old_id,
-          make_scoped_ptr(new message_center::Notification(
-              new_notification->notification())));
+          old_id, base::MakeUnique<message_center::Notification>(notification));
 
       return true;
     }
@@ -185,10 +181,9 @@ const Notification* MessageCenterNotificationManager::FindById(
 
   std::string profile_notification_id =
       ProfileNotification::GetProfileNotificationId(delegate_id, profile_id);
-  NotificationMap::const_iterator iter =
-      profile_notifications_.find(profile_notification_id);
+  auto iter = profile_notifications_.find(profile_notification_id);
   if (iter == profile_notifications_.end())
-    return NULL;
+    return nullptr;
   return &(iter->second->notification());
 }
 
@@ -202,12 +197,11 @@ bool MessageCenterNotificationManager::CancelById(
       ProfileNotification::GetProfileNotificationId(delegate_id, profile_id);
   // See if this ID hasn't been shown yet.
   // If it has been shown, remove it.
-  NotificationMap::iterator iter =
-      profile_notifications_.find(profile_notification_id);
+  auto iter = profile_notifications_.find(profile_notification_id);
   if (iter == profile_notifications_.end())
     return false;
 
-  RemoveProfileNotification(iter->second);
+  RemoveProfileNotification(iter->first);
   message_center_->RemoveNotification(profile_notification_id,
                                       /* by_user */ false);
   return true;
@@ -246,12 +240,12 @@ bool MessageCenterNotificationManager::CancelAllBySourceOrigin(
   // because there may be multiple notifications from the same source.
   bool removed = false;
 
-  for (NotificationMap::iterator loopiter = profile_notifications_.begin();
-       loopiter != profile_notifications_.end(); ) {
-    NotificationMap::iterator curiter = loopiter++;
+  for (auto loopiter = profile_notifications_.begin();
+       loopiter != profile_notifications_.end();) {
+    auto curiter = loopiter++;
     if ((*curiter).second->notification().origin_url() == source) {
       const std::string id = curiter->first;
-      RemoveProfileNotification(curiter->second);
+      RemoveProfileNotification(id);
       message_center_->RemoveNotification(id, /* by_user */ false);
       removed = true;
     }
@@ -264,12 +258,12 @@ bool MessageCenterNotificationManager::CancelAllByProfile(
   // Same pattern as CancelAllBySourceOrigin.
   bool removed = false;
 
-  for (NotificationMap::iterator loopiter = profile_notifications_.begin();
-       loopiter != profile_notifications_.end(); ) {
-    NotificationMap::iterator curiter = loopiter++;
+  for (auto loopiter = profile_notifications_.begin();
+       loopiter != profile_notifications_.end();) {
+    auto curiter = loopiter++;
     if (profile_id == (*curiter).second->profile_id()) {
       const std::string id = curiter->first;
-      RemoveProfileNotification(curiter->second);
+      RemoveProfileNotification(id);
       message_center_->RemoveNotification(id, /* by_user */ false);
       removed = true;
     }
@@ -278,7 +272,8 @@ bool MessageCenterNotificationManager::CancelAllByProfile(
 }
 
 void MessageCenterNotificationManager::CancelAll() {
-  message_center_->RemoveAllNotifications(/* by_user */ false);
+  message_center_->RemoveAllNotifications(
+      false /* by_user */, message_center::MessageCenter::RemoveType::ALL);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -286,9 +281,9 @@ void MessageCenterNotificationManager::CancelAll() {
 void MessageCenterNotificationManager::OnNotificationRemoved(
     const std::string& id,
     bool by_user) {
-  NotificationMap::const_iterator iter = profile_notifications_.find(id);
+  auto iter = profile_notifications_.find(id);
   if (iter != profile_notifications_.end())
-    RemoveProfileNotification(iter->second);
+    RemoveProfileNotification(iter->first);
 }
 
 void MessageCenterNotificationManager::OnCenterVisibilityChanged(
@@ -347,28 +342,41 @@ void MessageCenterNotificationManager::AddNotificationToAlternateProvider(
 }
 
 void MessageCenterNotificationManager::AddProfileNotification(
-    ProfileNotification* profile_notification) {
+    std::unique_ptr<ProfileNotification> profile_notification) {
   const Notification& notification = profile_notification->notification();
   std::string id = notification.id();
   // Notification ids should be unique.
   DCHECK(profile_notifications_.find(id) == profile_notifications_.end());
-  profile_notifications_[id] = profile_notification;
+  profile_notifications_[id] = std::move(profile_notification);
 }
 
 void MessageCenterNotificationManager::RemoveProfileNotification(
-    ProfileNotification* profile_notification) {
-  std::string id = profile_notification->notification().id();
-  profile_notifications_.erase(id);
-  delete profile_notification;
+    const std::string& notification_id) {
+  auto it = profile_notifications_.find(notification_id);
+  if (it == profile_notifications_.end())
+    return;
+
+  // Delay destruction of the ProfileNotification until after all the work
+  // removing it from |profile_notifications_| is complete. This must be done
+  // because this ProfileNotification might have the one ScopedKeepAlive object
+  // that was keeping the browser alive, and destroying it would result in a re-
+  // entrant call to this class. Because every method in this class touches
+  // |profile_notifications_|, |profile_notifications_| must always be in a
+  // self-consistent state in moments where re-entrance might happen.
+  // https://crbug.com/649971
+  std::unique_ptr<ProfileNotification> notification = std::move(it->second);
+  profile_notifications_.erase(it);
+  // Now that the map modifications are complete, going out of scope will
+  // destroy the notification.
 }
 
 ProfileNotification* MessageCenterNotificationManager::FindProfileNotification(
     const std::string& id) const {
-  NotificationMap::const_iterator iter = profile_notifications_.find(id);
+  auto iter = profile_notifications_.find(id);
   if (iter == profile_notifications_.end())
-    return NULL;
+    return nullptr;
 
-  return (*iter).second;
+  return (*iter).second.get();
 }
 
 std::string
