@@ -6,29 +6,68 @@
 
 #include <algorithm>
 #include <map>
+#include <utility>
 #include <vector>
 
 #include "base/bind.h"
 #include "base/logging.h"
-#include "base/prefs/pref_service.h"
 #include "base/stl_util.h"
 #include "chrome/browser/chrome_notification_types.h"
-#include "chrome/common/pref_names.h"
+#include "components/password_manager/core/browser/password_manager_util.h"
 #include "components/password_manager/core/browser/password_store_change.h"
-#include "components/user_prefs/pref_registry_syncable.h"
+#include "components/password_manager/core/common/password_manager_pref_names.h"
+#include "components/pref_registry/pref_registry_syncable.h"
+#include "components/prefs/pref_service.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/notification_service.h"
 
 using autofill::PasswordForm;
 using content::BrowserThread;
-using std::vector;
+using password_manager::PasswordStoreChange;
+using password_manager::PasswordStoreChangeList;
+using password_manager::PasswordStoreDefault;
+
+namespace {
+
+bool AddLoginToBackend(
+    const std::unique_ptr<PasswordStoreX::NativeBackend>& backend,
+    const PasswordForm& form,
+    PasswordStoreChangeList* changes) {
+  *changes = backend->AddLogin(form);
+  return (!changes->empty() &&
+          changes->back().type() == PasswordStoreChange::ADD);
+}
+
+bool RemoveLoginsByURLAndTimeFromBackend(
+    PasswordStoreX::NativeBackend* backend,
+    const base::Callback<bool(const GURL&)>& url_filter,
+    base::Time delete_begin,
+    base::Time delete_end,
+    PasswordStoreChangeList* changes) {
+  ScopedVector<autofill::PasswordForm> forms;
+  if (!backend->GetAllLogins(&forms))
+    return false;
+
+  for (const autofill::PasswordForm* form : forms) {
+    if (url_filter.Run(form->origin) && form->date_created >= delete_begin &&
+        (delete_end.is_null() || form->date_created < delete_end) &&
+        !backend->RemoveLogin(*form, changes))
+      return false;
+  }
+
+  return true;
+}
+
+}  // namespace
 
 PasswordStoreX::PasswordStoreX(
     scoped_refptr<base::SingleThreadTaskRunner> main_thread_runner,
     scoped_refptr<base::SingleThreadTaskRunner> db_thread_runner,
-    LoginDatabase* login_db,
+    std::unique_ptr<password_manager::LoginDatabase> login_db,
     NativeBackend* backend)
-    : PasswordStoreDefault(main_thread_runner, db_thread_runner, login_db),
+    : PasswordStoreDefault(main_thread_runner,
+                           db_thread_runner,
+                           std::move(login_db)),
       backend_(backend),
       migration_checked_(!backend),
       allow_fallback_(false) {}
@@ -38,8 +77,7 @@ PasswordStoreX::~PasswordStoreX() {}
 PasswordStoreChangeList PasswordStoreX::AddLoginImpl(const PasswordForm& form) {
   CheckMigration();
   PasswordStoreChangeList changes;
-  if (use_native_backend() && backend_->AddLogin(form)) {
-    changes.push_back(PasswordStoreChange(PasswordStoreChange::ADD, form));
+  if (use_native_backend() && AddLoginToBackend(backend_, form, &changes)) {
     allow_fallback_ = false;
   } else if (allow_default_store()) {
     changes = PasswordStoreDefault::AddLoginImpl(form);
@@ -51,8 +89,7 @@ PasswordStoreChangeList PasswordStoreX::UpdateLoginImpl(
     const PasswordForm& form) {
   CheckMigration();
   PasswordStoreChangeList changes;
-  if (use_native_backend() && backend_->UpdateLogin(form)) {
-    changes.push_back(PasswordStoreChange(PasswordStoreChange::UPDATE, form));
+  if (use_native_backend() && backend_->UpdateLogin(form, &changes)) {
     allow_fallback_ = false;
   } else if (allow_default_store()) {
     changes = PasswordStoreDefault::UpdateLoginImpl(form);
@@ -64,8 +101,7 @@ PasswordStoreChangeList PasswordStoreX::RemoveLoginImpl(
     const PasswordForm& form) {
   CheckMigration();
   PasswordStoreChangeList changes;
-  if (use_native_backend() && backend_->RemoveLogin(form)) {
-    changes.push_back(PasswordStoreChange(PasswordStoreChange::REMOVE, form));
+  if (use_native_backend() && backend_->RemoveLogin(form, &changes)) {
     allow_fallback_ = false;
   } else if (allow_default_store()) {
     changes = PasswordStoreDefault::RemoveLoginImpl(form);
@@ -73,102 +109,119 @@ PasswordStoreChangeList PasswordStoreX::RemoveLoginImpl(
   return changes;
 }
 
+PasswordStoreChangeList PasswordStoreX::RemoveLoginsByURLAndTimeImpl(
+    const base::Callback<bool(const GURL&)>& url_filter,
+    base::Time delete_begin,
+    base::Time delete_end) {
+  CheckMigration();
+  PasswordStoreChangeList changes;
+
+  if (use_native_backend() &&
+      RemoveLoginsByURLAndTimeFromBackend(backend_.get(), url_filter,
+                                          delete_begin, delete_end, &changes)) {
+    LogStatsForBulkDeletion(changes.size());
+    allow_fallback_ = false;
+  } else if (allow_default_store()) {
+    changes = PasswordStoreDefault::RemoveLoginsByURLAndTimeImpl(
+        url_filter, delete_begin, delete_end);
+  }
+
+  return changes;
+}
+
 PasswordStoreChangeList PasswordStoreX::RemoveLoginsCreatedBetweenImpl(
-    const base::Time& delete_begin,
-    const base::Time& delete_end) {
+    base::Time delete_begin,
+    base::Time delete_end) {
   CheckMigration();
-  vector<PasswordForm*> forms;
   PasswordStoreChangeList changes;
   if (use_native_backend() &&
-      backend_->GetLoginsCreatedBetween(delete_begin, delete_end, &forms) &&
-      backend_->RemoveLoginsCreatedBetween(delete_begin, delete_end)) {
-    for (vector<PasswordForm*>::const_iterator it = forms.begin();
-         it != forms.end(); ++it) {
-      changes.push_back(PasswordStoreChange(PasswordStoreChange::REMOVE,
-                                            **it));
-    }
+      backend_->RemoveLoginsCreatedBetween(
+          delete_begin, delete_end, &changes)) {
     LogStatsForBulkDeletion(changes.size());
     allow_fallback_ = false;
   } else if (allow_default_store()) {
     changes = PasswordStoreDefault::RemoveLoginsCreatedBetweenImpl(delete_begin,
                                                                    delete_end);
   }
-  STLDeleteElements(&forms);
+  return changes;
+}
+
+PasswordStoreChangeList PasswordStoreX::RemoveLoginsSyncedBetweenImpl(
+    base::Time delete_begin,
+    base::Time delete_end) {
+  CheckMigration();
+  PasswordStoreChangeList changes;
+  if (use_native_backend() &&
+      backend_->RemoveLoginsSyncedBetween(delete_begin, delete_end, &changes)) {
+    LogStatsForBulkDeletionDuringRollback(changes.size());
+    allow_fallback_ = false;
+  } else if (allow_default_store()) {
+    changes = PasswordStoreDefault::RemoveLoginsSyncedBetweenImpl(delete_begin,
+                                                                  delete_end);
+  }
+  return changes;
+}
+
+PasswordStoreChangeList PasswordStoreX::DisableAutoSignInForOriginsImpl(
+    const base::Callback<bool(const GURL&)>& origin_filter) {
+  CheckMigration();
+  PasswordStoreChangeList changes;
+  if (use_native_backend() &&
+      backend_->DisableAutoSignInForOrigins(origin_filter, &changes)) {
+    allow_fallback_ = false;
+  } else if (allow_default_store()) {
+    changes =
+        PasswordStoreDefault::DisableAutoSignInForOriginsImpl(origin_filter);
+  }
   return changes;
 }
 
 namespace {
-struct LoginLessThan {
-  bool operator()(const PasswordForm* a, const PasswordForm* b) {
-    return a->origin < b->origin;
-  }
-};
-}  // anonymous namespace
 
-void PasswordStoreX::SortLoginsByOrigin(NativeBackend::PasswordFormList* list) {
-  // In login_database.cc, the query has ORDER BY origin_url. Simulate that.
-  std::sort(list->begin(), list->end(), LoginLessThan());
+// Sorts |list| by origin, like the ORDER BY clause in login_database.cc.
+void SortLoginsByOrigin(std::vector<std::unique_ptr<PasswordForm>>* list) {
+  std::sort(list->begin(), list->end(),
+            [](const std::unique_ptr<PasswordForm>& a,
+               const std::unique_ptr<PasswordForm>& b) {
+              return a->origin < b->origin;
+            });
 }
 
-void PasswordStoreX::GetLoginsImpl(
-    const autofill::PasswordForm& form,
-    AuthorizationPromptPolicy prompt_policy,
-    const ConsumerCallbackRunner& callback_runner) {
+}  // anonymous namespace
+
+std::vector<std::unique_ptr<PasswordForm>> PasswordStoreX::FillMatchingLogins(
+    const FormDigest& form) {
   CheckMigration();
-  std::vector<autofill::PasswordForm*> matched_forms;
-  if (use_native_backend() && backend_->GetLogins(form, &matched_forms)) {
+  ScopedVector<autofill::PasswordForm> matched_forms_scopedvector;
+  if (use_native_backend() &&
+      backend_->GetLogins(form, &matched_forms_scopedvector)) {
+    std::vector<std::unique_ptr<PasswordForm>> matched_forms =
+        password_manager_util::ConvertScopedVector(
+            std::move(matched_forms_scopedvector));
     SortLoginsByOrigin(&matched_forms);
     // The native backend may succeed and return no data even while locked, if
     // the query did not match anything stored. So we continue to allow fallback
     // until we perform a write operation, or until a read returns actual data.
-    if (matched_forms.size() > 0)
+    if (!matched_forms.empty())
       allow_fallback_ = false;
-  } else if (allow_default_store()) {
-    DCHECK(matched_forms.empty());
-    PasswordStoreDefault::GetLoginsImpl(form, prompt_policy, callback_runner);
-    return;
+    return matched_forms;
   }
-  // The consumer will be left hanging unless we reply.
-  callback_runner.Run(matched_forms);
+  if (allow_default_store())
+    return PasswordStoreDefault::FillMatchingLogins(form);
+  return std::vector<std::unique_ptr<PasswordForm>>();
 }
 
-void PasswordStoreX::GetAutofillableLoginsImpl(GetLoginsRequest* request) {
+bool PasswordStoreX::FillAutofillableLogins(
+    std::vector<std::unique_ptr<PasswordForm>>* forms) {
   CheckMigration();
+  ScopedVector<autofill::PasswordForm> forms_scopedvector;
   if (use_native_backend() &&
-      backend_->GetAutofillableLogins(request->result())) {
-    SortLoginsByOrigin(request->result());
+      backend_->GetAutofillableLogins(&forms_scopedvector)) {
+    *forms = password_manager_util::ConvertScopedVector(
+        std::move(forms_scopedvector));
+    SortLoginsByOrigin(forms);
     // See GetLoginsImpl() for why we disallow fallback conditionally here.
-    if (request->result()->size() > 0)
-      allow_fallback_ = false;
-  } else if (allow_default_store()) {
-    PasswordStoreDefault::GetAutofillableLoginsImpl(request);
-    return;
-  }
-  // The consumer will be left hanging unless we reply.
-  ForwardLoginsResult(request);
-}
-
-void PasswordStoreX::GetBlacklistLoginsImpl(GetLoginsRequest* request) {
-  CheckMigration();
-  if (use_native_backend() &&
-      backend_->GetBlacklistLogins(request->result())) {
-    SortLoginsByOrigin(request->result());
-    // See GetLoginsImpl() for why we disallow fallback conditionally here.
-    if (request->result()->size() > 0)
-      allow_fallback_ = false;
-  } else if (allow_default_store()) {
-    PasswordStoreDefault::GetBlacklistLoginsImpl(request);
-    return;
-  }
-  // The consumer will be left hanging unless we reply.
-  ForwardLoginsResult(request);
-}
-
-bool PasswordStoreX::FillAutofillableLogins(vector<PasswordForm*>* forms) {
-  CheckMigration();
-  if (use_native_backend() && backend_->GetAutofillableLogins(forms)) {
-    // See GetLoginsImpl() for why we disallow fallback conditionally here.
-    if (forms->size() > 0)
+    if (!forms->empty())
       allow_fallback_ = false;
     return true;
   }
@@ -177,11 +230,17 @@ bool PasswordStoreX::FillAutofillableLogins(vector<PasswordForm*>* forms) {
   return false;
 }
 
-bool PasswordStoreX::FillBlacklistLogins(vector<PasswordForm*>* forms) {
+bool PasswordStoreX::FillBlacklistLogins(
+    std::vector<std::unique_ptr<PasswordForm>>* forms) {
   CheckMigration();
-  if (use_native_backend() && backend_->GetBlacklistLogins(forms)) {
+  ScopedVector<autofill::PasswordForm> forms_scopedvector;
+  if (use_native_backend() &&
+      backend_->GetBlacklistLogins(&forms_scopedvector)) {
+    *forms = password_manager_util::ConvertScopedVector(
+        std::move(forms_scopedvector));
     // See GetLoginsImpl() for why we disallow fallback conditionally here.
-    if (forms->size() > 0)
+    SortLoginsByOrigin(forms);
+    if (!forms->empty())
       allow_fallback_ = false;
     return true;
   }
@@ -191,7 +250,7 @@ bool PasswordStoreX::FillBlacklistLogins(vector<PasswordForm*>* forms) {
 }
 
 void PasswordStoreX::CheckMigration() {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::DB));
+  DCHECK_CURRENTLY_ON(BrowserThread::DB);
   if (migration_checked_ || !backend_.get())
     return;
   migration_checked_ = true;
@@ -208,7 +267,7 @@ void PasswordStoreX::CheckMigration() {
   } else {
     LOG(WARNING) << "Native password store migration failed! " <<
                  "Falling back on default (unencrypted) store.";
-    backend_.reset(NULL);
+    backend_.reset();
   }
 }
 
@@ -216,7 +275,7 @@ bool PasswordStoreX::allow_default_store() {
   if (allow_fallback_) {
     LOG(WARNING) << "Native password store failed! " <<
                  "Falling back on default (unencrypted) store.";
-    backend_.reset(NULL);
+    backend_.reset();
     // Don't warn again. We'll use the default store because backend_ is NULL.
     allow_fallback_ = false;
   }
@@ -225,16 +284,22 @@ bool PasswordStoreX::allow_default_store() {
 
 ssize_t PasswordStoreX::MigrateLogins() {
   DCHECK(backend_.get());
-  vector<PasswordForm*> forms;
+  std::vector<std::unique_ptr<PasswordForm>> forms;
+  std::vector<std::unique_ptr<PasswordForm>> blacklist_forms;
   bool ok = PasswordStoreDefault::FillAutofillableLogins(&forms) &&
-      PasswordStoreDefault::FillBlacklistLogins(&forms);
+            PasswordStoreDefault::FillBlacklistLogins(&blacklist_forms);
+  const size_t autofillable_forms_count = forms.size();
+  forms.resize(autofillable_forms_count + blacklist_forms.size());
+  std::move(blacklist_forms.begin(), blacklist_forms.end(),
+            forms.begin() + autofillable_forms_count);
   if (ok) {
     // We add all the passwords (and blacklist entries) to the native backend
     // before attempting to remove any from the login database, to make sure we
     // don't somehow end up with some of the passwords in one store and some in
     // another. We'll always have at least one intact store this way.
     for (size_t i = 0; i < forms.size(); ++i) {
-      if (!backend_->AddLogin(*forms[i])) {
+      PasswordStoreChangeList changes;
+      if (!AddLoginToBackend(backend_, *forms[i], &changes)) {
         ok = false;
         break;
       }
@@ -257,45 +322,5 @@ ssize_t PasswordStoreX::MigrateLogins() {
     }
   }
   ssize_t result = ok ? forms.size() : -1;
-  STLDeleteElements(&forms);
   return result;
 }
-
-#if !defined(OS_MACOSX) && !defined(OS_CHROMEOS) && defined(OS_POSIX)
-// static
-void PasswordStoreX::RegisterProfilePrefs(
-    user_prefs::PrefRegistrySyncable* registry) {
-  // Normally we should be on the UI thread here, but in tests we might not.
-  registry->RegisterBooleanPref(
-      prefs::kPasswordsUseLocalProfileId,
-      // default: passwords don't use local ids
-      false,
-      user_prefs::PrefRegistrySyncable::UNSYNCABLE_PREF);
-}
-
-// static
-bool PasswordStoreX::PasswordsUseLocalProfileId(PrefService* prefs) {
-  // Normally we should be on the UI thread here, but in tests we might not.
-  return prefs->GetBoolean(prefs::kPasswordsUseLocalProfileId);
-}
-
-namespace {
-// This function is a hack to do something not entirely thread safe: the pref
-// service comes from the UI thread, but it's not ref counted. We keep a pointer
-// to it on the DB thread, and need to invoke a method on the UI thread. This
-// function does that for us without requiring ref counting the pref service.
-// TODO(mdm): Fix this if it becomes a problem. Given that this function will
-// be called once ever per profile, it probably will not cause a problem...
-void UISetPasswordsUseLocalProfileId(PrefService* prefs) {
-  prefs->SetBoolean(prefs::kPasswordsUseLocalProfileId, true);
-}
-}  // anonymous namespace
-
-// static
-void PasswordStoreX::SetPasswordsUseLocalProfileId(PrefService* prefs) {
-  // This method should work on any thread, but we expect the DB thread.
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::DB));
-  BrowserThread::PostTask(BrowserThread::UI, FROM_HERE,
-                          base::Bind(&UISetPasswordsUseLocalProfileId, prefs));
-}
-#endif  // !defined(OS_MACOSX) && !defined(OS_CHROMEOS) && defined(OS_POSIX)
