@@ -5,6 +5,7 @@
 #ifndef CHROME_BROWSER_PLUGINS_PLUGIN_INFO_MESSAGE_FILTER_H_
 #define CHROME_BROWSER_PLUGINS_PLUGIN_INFO_MESSAGE_FILTER_H_
 
+#include <memory>
 #include <string>
 #include <vector>
 
@@ -12,20 +13,25 @@
 #include "base/macros.h"
 #include "base/memory/ref_counted.h"
 #include "base/memory/weak_ptr.h"
-#include "base/prefs/pref_member.h"
 #include "base/sequenced_task_runner_helpers.h"
+#include "chrome/browser/plugins/plugin_metadata.h"
 #include "chrome/browser/plugins/plugin_prefs.h"
 #include "components/content_settings/core/common/content_settings.h"
+#include "components/prefs/pref_member.h"
 #include "content/public/browser/browser_message_filter.h"
+#include "extensions/features/features.h"
+#include "ppapi/features/features.h"
 
 struct ChromeViewHostMsg_GetPluginInfo_Output;
 enum class ChromeViewHostMsg_GetPluginInfo_Status;
 class GURL;
 class HostContentSettingsMap;
-class PluginFinder;
-class PluginMetadata;
 class Profile;
 
+namespace base {
+class SingleThreadTaskRunner;
+}
+
 namespace content {
 class ResourceContext;
 struct WebPluginInfo;
@@ -35,6 +41,14 @@ namespace extensions {
 class ExtensionRegistry;
 }
 
+namespace component_updater {
+struct ComponentInfo;
+}
+
+namespace url {
+class Origin;
+}
+
 // This class filters out incoming IPC messages requesting plugin information.
 class PluginInfoMessageFilter : public content::BrowserMessageFilter {
  public:
@@ -48,25 +62,21 @@ class PluginInfoMessageFilter : public content::BrowserMessageFilter {
     ~Context();
 
     void DecidePluginStatus(
-        const GetPluginInfo_Params& params,
+        const GURL& url,
+        const url::Origin& main_frame_origin,
         const content::WebPluginInfo& plugin,
-        const PluginMetadata* plugin_metadata,
+        PluginMetadata::SecurityStatus security_status,
+        const std::string& plugin_identifier,
         ChromeViewHostMsg_GetPluginInfo_Status* status) const;
-    bool FindEnabledPlugin(int render_frame_id,
-                           const GURL& url,
-                           const GURL& top_origin_url,
-                           const std::string& mime_type,
-                           ChromeViewHostMsg_GetPluginInfo_Status* status,
-                           content::WebPluginInfo* plugin,
-                           std::string* actual_mime_type,
-                           scoped_ptr<PluginMetadata>* plugin_metadata) const;
-    void GetPluginContentSetting(const content::WebPluginInfo& plugin,
-                                 const GURL& policy_url,
-                                 const GURL& plugin_url,
-                                 const std::string& resource,
-                                 ContentSetting* setting,
-                                 bool* is_default,
-                                 bool* is_managed) const;
+    bool FindEnabledPlugin(
+        int render_frame_id,
+        const GURL& url,
+        const url::Origin& main_frame_origin,
+        const std::string& mime_type,
+        ChromeViewHostMsg_GetPluginInfo_Status* status,
+        content::WebPluginInfo* plugin,
+        std::string* actual_mime_type,
+        std::unique_ptr<PluginMetadata>* plugin_metadata) const;
     void MaybeGrantAccess(ChromeViewHostMsg_GetPluginInfo_Status status,
                           const base::FilePath& path) const;
     bool IsPluginEnabled(const content::WebPluginInfo& plugin) const;
@@ -74,7 +84,7 @@ class PluginInfoMessageFilter : public content::BrowserMessageFilter {
    private:
     int render_process_id_;
     content::ResourceContext* resource_context_;
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
     extensions::ExtensionRegistry* extension_registry_;
 #endif
     const HostContentSettingsMap* host_content_settings_map_;
@@ -99,7 +109,7 @@ class PluginInfoMessageFilter : public content::BrowserMessageFilter {
 
   void OnGetPluginInfo(int render_frame_id,
                        const GURL& url,
-                       const GURL& top_origin_url,
+                       const url::Origin& main_frame_origin,
                        const std::string& mime_type,
                        IPC::Message* reply_msg);
 
@@ -109,7 +119,20 @@ class PluginInfoMessageFilter : public content::BrowserMessageFilter {
                      IPC::Message* reply_msg,
                      const std::vector<content::WebPluginInfo>& plugins);
 
-#if defined(ENABLE_PEPPER_CDMS)
+  void ComponentPluginLookupDone(
+      const GetPluginInfo_Params& params,
+      std::unique_ptr<ChromeViewHostMsg_GetPluginInfo_Output> output,
+      std::unique_ptr<PluginMetadata> plugin_metadata,
+      IPC::Message* reply_msg,
+      std::unique_ptr<component_updater::ComponentInfo> cus_plugin_info);
+
+  void GetPluginInfoReply(
+      const GetPluginInfo_Params& params,
+      std::unique_ptr<ChromeViewHostMsg_GetPluginInfo_Output> output,
+      std::unique_ptr<PluginMetadata> plugin_metadata,
+      IPC::Message* reply_msg);
+
+#if BUILDFLAG(ENABLE_PEPPER_CDMS)
   // Returns whether any internal plugin supporting |mime_type| is registered
   // and enabled. Does not determine whether the plugin can actually be
   // instantiated (e.g. whether it has all its dependencies).
