@@ -6,35 +6,43 @@
 
 #include <stddef.h>
 
+#include <memory>
 #include <string>
+#include <utility>
 
 #include "base/bind.h"
 #include "base/command_line.h"
 #include "base/location.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/path_service.h"
-#include "base/prefs/scoped_user_pref_update.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/pattern.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/values.h"
 #include "build/build_config.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/chrome_notification_types.h"
+#include "chrome/browser/content_settings/host_content_settings_map_factory.h"
 #include "chrome/browser/plugins/plugin_installer.h"
 #include "chrome/browser/plugins/plugin_metadata.h"
 #include "chrome/browser/plugins/plugin_prefs_factory.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/common/chrome_constants.h"
+#include "chrome/common/chrome_content_client.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
+#include "components/content_settings/core/browser/host_content_settings_map.h"
+#include "components/content_settings/core/common/content_settings.h"
+#include "components/content_settings/core/common/pref_names.h"
 #include "components/keyed_service/core/keyed_service.h"
+#include "components/prefs/scoped_user_pref_update.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/notification_service.h"
 #include "content/public/browser/plugin_service.h"
+#include "content/public/common/content_constants.h"
 #include "content/public/common/webplugininfo.h"
 
 #if !defined(DISABLE_NACL)
@@ -59,22 +67,14 @@ bool IsComponentUpdatedPepperFlash(const base::FilePath& plugin) {
   return false;
 }
 
-// Returns true if |path| looks like the path to an NPAPI Flash plugin.
-bool IsNpapiFlashPath(const base::FilePath& path) {
-  base::FilePath npapi_flash;
-  // Check NPAPI Flash is installed.
-  if (!PathService::Get(chrome::FILE_FLASH_SYSTEM_PLUGIN, &npapi_flash))
-    return false;
-  // Check for same architecture NPAPI Flash.
-  if (base::FilePath::CompareEqualIgnoreCase(path.value(), npapi_flash.value()))
-    return true;
-#if defined(OS_WIN)
-  // Fuzzy check for NPAPI Flash on Windows.
-  base::FilePath::StringType kSwfPrefix = FILE_PATH_LITERAL("NPSWF");
-  if (path.BaseName().value().compare(0, kSwfPrefix.size(), kSwfPrefix) == 0)
-    return true;
-#endif
-  return false;
+bool IsPDFViewerPlugin(const base::string16& plugin_name) {
+  return plugin_name == base::ASCIIToUTF16(ChromeContentClient::kPDFPluginName);
+}
+
+bool IsAdobeFlashPlayerPlugin(const base::string16& plugin_name) {
+  return (plugin_name == base::ASCIIToUTF16(content::kFlashPluginName) ||
+          plugin_name == base::ASCIIToUTF16(
+              PluginMetadata::kAdobeFlashPlayerGroupName));
 }
 
 }  // namespace
@@ -146,7 +146,8 @@ void PluginPrefs::EnablePluginGroupInternal(
 
   // Update the state for all plugins in the group.
   for (size_t i = 0; i < plugins.size(); ++i) {
-    scoped_ptr<PluginMetadata> plugin(finder->GetPluginMetadata(plugins[i]));
+    std::unique_ptr<PluginMetadata> plugin(
+        finder->GetPluginMetadata(plugins[i]));
     if (group_name != plugin->name())
       continue;
     plugin_state_.Set(plugins[i].path, enabled);
@@ -165,7 +166,7 @@ void PluginPrefs::EnablePlugin(
   content::WebPluginInfo plugin;
   bool can_enable = true;
   if (PluginService::GetInstance()->GetPluginInfoByPath(path, &plugin)) {
-    scoped_ptr<PluginMetadata> plugin_metadata(
+    std::unique_ptr<PluginMetadata> plugin_metadata(
         finder->GetPluginMetadata(plugin));
     PolicyStatus plugin_status = PolicyStatusForPlugin(plugin.name);
     PolicyStatus group_status = PolicyStatusForPlugin(plugin_metadata->name());
@@ -206,7 +207,7 @@ void PluginPrefs::EnablePluginInternal(
   base::string16 group_name;
   for (size_t i = 0; i < plugins.size(); ++i) {
     if (plugins[i].path == path) {
-      scoped_ptr<PluginMetadata> plugin_metadata(
+      std::unique_ptr<PluginMetadata> plugin_metadata(
           plugin_finder->GetPluginMetadata(plugins[i]));
       // set the group name for this plugin.
       group_name = plugin_metadata->name();
@@ -217,7 +218,7 @@ void PluginPrefs::EnablePluginInternal(
 
   bool all_disabled = true;
   for (size_t i = 0; i < plugins.size(); ++i) {
-    scoped_ptr<PluginMetadata> plugin_metadata(
+    std::unique_ptr<PluginMetadata> plugin_metadata(
         plugin_finder->GetPluginMetadata(plugins[i]));
     DCHECK(!plugin_metadata->name().empty());
     if (group_name == plugin_metadata->name()) {
@@ -241,19 +242,25 @@ void PluginPrefs::EnablePluginInternal(
 PluginPrefs::PolicyStatus PluginPrefs::PolicyStatusForPlugin(
     const base::string16& name) const {
   base::AutoLock auto_lock(lock_);
-  if (IsStringMatchedInSet(name, policy_enabled_plugin_patterns_)) {
+
+  // Special handling for PDF based on its specific policy.
+  if (IsPDFViewerPlugin(name) && always_open_pdf_externally_)
+    return POLICY_DISABLED;
+
+  if (IsStringMatchedInSet(name, policy_enabled_plugin_patterns_))
     return POLICY_ENABLED;
-  } else if (IsStringMatchedInSet(name, policy_disabled_plugin_patterns_) &&
-             !IsStringMatchedInSet(
-                 name, policy_disabled_plugin_exception_patterns_)) {
+
+  if (IsStringMatchedInSet(name, policy_disabled_plugin_patterns_) &&
+      !IsStringMatchedInSet(
+          name, policy_disabled_plugin_exception_patterns_)) {
     return POLICY_DISABLED;
-  } else {
-    return NO_POLICY;
   }
+
+  return NO_POLICY;
 }
 
 bool PluginPrefs::IsPluginEnabled(const content::WebPluginInfo& plugin) const {
-  scoped_ptr<PluginMetadata> plugin_metadata(
+  std::unique_ptr<PluginMetadata> plugin_metadata(
       PluginFinder::GetInstance()->GetPluginMetadata(plugin));
   base::string16 group_name = plugin_metadata->name();
 
@@ -303,6 +310,14 @@ void PluginPrefs::UpdatePatternsAndNotify(std::set<base::string16>* patterns,
   NotifyPluginStatusChanged();
 }
 
+void PluginPrefs::UpdatePdfPolicy(const std::string& pref_name) {
+  base::AutoLock auto_lock(lock_);
+  always_open_pdf_externally_ =
+      prefs_->GetBoolean(prefs::kPluginsAlwaysOpenPdfExternally);
+
+  NotifyPluginStatusChanged();
+}
+
 /*static*/
 bool PluginPrefs::IsStringMatchedInSet(
     const base::string16& name,
@@ -336,6 +351,7 @@ void PluginPrefs::ListValueToStringSet(const base::ListValue* src,
 void PluginPrefs::SetPrefs(PrefService* prefs) {
   prefs_ = prefs;
   bool update_internal_dir = false;
+  bool plugins_migrated = false;
   base::FilePath last_internal_dir =
       prefs_->GetFilePath(prefs::kPluginsLastInternalDirectory);
   base::FilePath cur_internal_dir;
@@ -346,47 +362,39 @@ void PluginPrefs::SetPrefs(PrefService* prefs) {
         prefs::kPluginsLastInternalDirectory, cur_internal_dir);
   }
 
-  bool migrate_to_pepper_flash = false;
-#if defined(OS_WIN) || defined(OS_MACOSX)
-  // If NPAPI Flash is enabled while Pepper Flash is disabled, we would like to
-  // turn Pepper Flash on. And we want to do it once, when NPAPI is disabled in
-  // Chrome 45.
-  // TODO(wfh): Remove this code once it has been run by most users, around
-  // Chrome 49 or Chrome 50. See crbug.com/514250.
-  if (!prefs_->GetBoolean(prefs::kNpapiFlashMigratedToPepperFlash)) {
-    prefs_->SetBoolean(prefs::kNpapiFlashMigratedToPepperFlash, true);
-    migrate_to_pepper_flash = true;
-  }
-#endif
-
   {  // Scoped update of prefs::kPluginsPluginsList.
     ListPrefUpdate update(prefs_, prefs::kPluginsPluginsList);
     base::ListValue* saved_plugins_list = update.Get();
     if (saved_plugins_list && !saved_plugins_list->empty()) {
-      // The following four variables are only valid when
-      // |migrate_to_pepper_flash| is set to true.
-      base::FilePath pepper_flash;
-      base::DictionaryValue* pepper_flash_node = NULL;
-      bool npapi_flash_enabled = false;
-      if (migrate_to_pepper_flash) {
-        PathService::Get(chrome::FILE_PEPPER_FLASH_PLUGIN, &pepper_flash);
-      }
-
-      for (base::ListValue::iterator it = saved_plugins_list->begin();
-           it != saved_plugins_list->end();
-           ++it) {
-        if (!(*it)->IsType(base::Value::TYPE_DICTIONARY)) {
+      for (const auto& plugin_value : *saved_plugins_list) {
+        base::DictionaryValue* plugin;
+        if (!plugin_value->GetAsDictionary(&plugin)) {
           LOG(WARNING) << "Invalid entry in " << prefs::kPluginsPluginsList;
           continue;  // Oops, don't know what to do with this item.
         }
 
-        base::DictionaryValue* plugin =
-            static_cast<base::DictionaryValue*>(*it);
-        base::string16 group_name;
         bool enabled;
         if (!plugin->GetBoolean("enabled", &enabled))
           enabled = true;
 
+        // Migrate disabled plugins and re-enable them all internally.
+        // TODO(http://crbug.com/662006): Remove migration eventually.
+        if (!enabled) {
+          base::string16 name;
+          plugin->GetString("name", &name);
+          if (IsPDFViewerPlugin(name))
+            prefs->SetBoolean(prefs::kPluginsAlwaysOpenPdfExternally, true);
+          if (IsAdobeFlashPlayerPlugin(name)) {
+            HostContentSettingsMapFactory::GetForProfile(profile_)
+                ->SetDefaultContentSetting(CONTENT_SETTINGS_TYPE_PLUGINS,
+                                           CONTENT_SETTING_BLOCK);
+          }
+          // TODO(http://crbug.com/662002): Remove the enabled flag completely.
+          enabled = true;
+          plugin->SetBoolean("enabled", true);
+          plugins_migrated = true;
+        }
+
         base::FilePath::StringType path;
         // The plugin list constains all the plugin files in addition to the
         // plugin groups.
@@ -433,27 +441,9 @@ void PluginPrefs::SetPrefs(PrefService* prefs) {
             }
           }
 
-          if (migrate_to_pepper_flash && IsNpapiFlashPath(plugin_path)) {
-            npapi_flash_enabled = enabled;
-          } else if (migrate_to_pepper_flash &&
-                     base::FilePath::CompareEqualIgnoreCase(
-                         path, pepper_flash.value())) {
-            if (!enabled)
-              pepper_flash_node = plugin;
-          }
-
           plugin_state_.Set(plugin_path, enabled);
-        } else if (!enabled && plugin->GetString("name", &group_name)) {
-          // Otherwise this is a list of groups.
-          plugin_group_state_[group_name] = false;
         }
       }
-
-      if (npapi_flash_enabled && pepper_flash_node) {
-        DCHECK(migrate_to_pepper_flash);
-        pepper_flash_node->SetBoolean("enabled", true);
-        plugin_state_.Set(pepper_flash, true);
-      }
     } else {
       // If the saved plugin list is empty, then the call to UpdatePreferences()
       // below failed in an earlier run, possibly because the user closed the
@@ -466,8 +456,11 @@ void PluginPrefs::SetPrefs(PrefService* prefs) {
     }
   }  // Scoped update of prefs::kPluginsPluginsList.
 
+  UMA_HISTOGRAM_BOOLEAN("Plugin.EnabledStatusMigrationDone", plugins_migrated);
+
   // Build the set of policy enabled/disabled plugin patterns once and cache it.
   // Don't do this in the constructor, there's no profile available there.
+  // TODO(http://crbug.com/662002): : Remove the prefs completely.
   ListValueToStringSet(prefs_->GetList(prefs::kPluginsDisabledPlugins),
                        &policy_disabled_plugin_patterns_);
   ListValueToStringSet(
@@ -475,6 +468,8 @@ void PluginPrefs::SetPrefs(PrefService* prefs) {
       &policy_disabled_plugin_exception_patterns_);
   ListValueToStringSet(prefs_->GetList(prefs::kPluginsEnabledPlugins),
                        &policy_enabled_plugin_patterns_);
+  always_open_pdf_externally_ =
+      prefs_->GetBoolean(prefs::kPluginsAlwaysOpenPdfExternally);
 
   registrar_.Init(prefs_);
 
@@ -494,6 +489,9 @@ void PluginPrefs::SetPrefs(PrefService* prefs) {
                  base::Bind(&PluginPrefs::UpdatePatternsAndNotify,
                             base::Unretained(this),
                             &policy_enabled_plugin_patterns_));
+  registrar_.Add(prefs::kPluginsAlwaysOpenPdfExternally,
+                 base::Bind(&PluginPrefs::UpdatePdfPolicy,
+                 base::Unretained(this)));
 
   NotifyPluginStatusChanged();
 }
@@ -503,14 +501,15 @@ void PluginPrefs::ShutdownOnUIThread() {
   registrar_.RemoveAll();
 }
 
-PluginPrefs::PluginPrefs() : profile_(NULL),
+PluginPrefs::PluginPrefs() : always_open_pdf_externally_(false),
+                             profile_(NULL),
                              prefs_(NULL) {
 }
 
 PluginPrefs::~PluginPrefs() {
 }
 
-void PluginPrefs::SetPolicyEnforcedPluginPatterns(
+void PluginPrefs::SetPolicyEnforcedPluginPatternsForTests(
     const std::set<base::string16>& disabled_patterns,
     const std::set<base::string16>& disabled_exception_patterns,
     const std::set<base::string16>& enabled_patterns) {
@@ -519,6 +518,12 @@ void PluginPrefs::SetPolicyEnforcedPluginPatterns(
   policy_enabled_plugin_patterns_ = enabled_patterns;
 }
 
+void PluginPrefs::SetAlwaysOpenPdfExternallyForTests(
+    bool always_open_pdf_externally) {
+  always_open_pdf_externally_ = always_open_pdf_externally;
+}
+
+
 void PluginPrefs::OnUpdatePreferences(
     const std::vector<content::WebPluginInfo>& plugins) {
   if (!prefs_)
@@ -538,16 +543,16 @@ void PluginPrefs::OnUpdatePreferences(
   // Add the plugin files.
   std::set<base::string16> group_names;
   for (size_t i = 0; i < plugins.size(); ++i) {
-    base::DictionaryValue* summary = new base::DictionaryValue();
+    std::unique_ptr<base::DictionaryValue> summary(new base::DictionaryValue());
     summary->SetString("path", plugins[i].path.value());
     summary->SetString("name", plugins[i].name);
     summary->SetString("version", plugins[i].version);
     bool enabled = true;
     plugin_state_.Get(plugins[i].path, &enabled);
     summary->SetBoolean("enabled", enabled);
-    plugins_list->Append(summary);
+    plugins_list->Append(std::move(summary));
 
-    scoped_ptr<PluginMetadata> plugin_metadata(
+    std::unique_ptr<PluginMetadata> plugin_metadata(
         finder->GetPluginMetadata(plugins[i]));
     // Insert into a set of all group names.
     group_names.insert(plugin_metadata->name());
@@ -556,7 +561,7 @@ void PluginPrefs::OnUpdatePreferences(
   // Add the plugin groups.
   for (std::set<base::string16>::const_iterator it = group_names.begin();
       it != group_names.end(); ++it) {
-    base::DictionaryValue* summary = new base::DictionaryValue();
+    std::unique_ptr<base::DictionaryValue> summary(new base::DictionaryValue());
     summary->SetString("name", *it);
     bool enabled = true;
     std::map<base::string16, bool>::iterator gstate_it =
@@ -564,7 +569,7 @@ void PluginPrefs::OnUpdatePreferences(
     if (gstate_it != plugin_group_state_.end())
       enabled = gstate_it->second;
     summary->SetBoolean("enabled", enabled);
-    plugins_list->Append(summary);
+    plugins_list->Append(std::move(summary));
   }
 }
 
