@@ -9,6 +9,7 @@
 #include "base/path_service.h"
 #include "base/run_loop.h"
 #include "base/strings/utf_string_conversions.h"
+#include "build/build_config.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/chrome_paths.h"
 #include "content/public/browser/plugin_service.h"
@@ -64,10 +65,14 @@ class PluginPrefsTest : public ::testing::Test {
       const std::set<base::string16>& disabled,
       const std::set<base::string16>& disabled_exceptions,
       const std::set<base::string16>& enabled) {
-    plugin_prefs_->SetPolicyEnforcedPluginPatterns(
+    plugin_prefs_->SetPolicyEnforcedPluginPatternsForTests(
         disabled, disabled_exceptions, enabled);
   }
 
+  void SetAlwaysOpenPdfExternally(bool value) {
+    plugin_prefs_->SetAlwaysOpenPdfExternallyForTests(value);
+  }
+
  protected:
   void EnablePluginSynchronously(bool enabled,
                                  const base::FilePath& path,
@@ -207,7 +212,6 @@ TEST_F(PluginPrefsTest, UnifiedPepperFlashState) {
   base::ShadowingAtExitManager at_exit_manager_;  // Destroys the PluginService.
 
   PluginService::GetInstance()->Init();
-  PluginService::GetInstance()->DisablePluginsDiscoveryForTesting();
 
   base::string16 component_updated_plugin_name(
       ASCIIToUTF16("Component-updated Pepper Flash"));
@@ -285,3 +289,15 @@ TEST_F(PluginPrefsTest, UnifiedPepperFlashState) {
 }
 
 #endif
+
+TEST_F(PluginPrefsTest, AlwaysOpenPdfExternally) {
+  EXPECT_EQ(PluginPrefs::NO_POLICY,
+            plugin_prefs_->PolicyStatusForPlugin(
+                base::ASCIIToUTF16(ChromeContentClient::kPDFPluginName)));
+
+  SetAlwaysOpenPdfExternally(true);
+
+  EXPECT_EQ(PluginPrefs::POLICY_DISABLED,
+            plugin_prefs_->PolicyStatusForPlugin(
+                base::ASCIIToUTF16(ChromeContentClient::kPDFPluginName)));
+}
