@@ -2,26 +2,29 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <memory>
+#include <utility>
+
 #include "base/files/file_path.h"
-#include "base/message_loop/message_loop.h"
-#include "base/message_loop/message_loop_proxy.h"
-#include "base/prefs/pref_service.h"
 #include "base/run_loop.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/time/time.h"
+#include "build/build_config.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/chrome_notification_types.h"
 #include "chrome/browser/policy/cloud/user_cloud_policy_manager_factory.h"
 #include "chrome/browser/policy/cloud/user_policy_signin_service_factory.h"
 #include "chrome/browser/prefs/browser_prefs.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/signin/fake_profile_oauth2_token_service.h"
-#include "chrome/browser/signin/fake_profile_oauth2_token_service_wrapper.h"
-#include "chrome/browser/signin/fake_signin_manager.h"
+#include "chrome/browser/signin/account_fetcher_service_factory.h"
+#include "chrome/browser/signin/chrome_signin_client_factory.h"
+#include "chrome/browser/signin/fake_account_fetcher_service_builder.h"
+#include "chrome/browser/signin/fake_profile_oauth2_token_service_builder.h"
+#include "chrome/browser/signin/fake_signin_manager_builder.h"
 #include "chrome/browser/signin/profile_oauth2_token_service_factory.h"
-#include "chrome/browser/signin/signin_manager.h"
 #include "chrome/browser/signin/signin_manager_factory.h"
+#include "chrome/browser/signin/test_signin_client_builder.h"
 #include "chrome/test/base/testing_browser_process.h"
-#include "chrome/test/base/testing_pref_service_syncable.h"
 #include "chrome/test/base/testing_profile.h"
 #include "components/policy/core/browser/browser_policy_connector.h"
 #include "components/policy/core/common/cloud/cloud_external_data_manager.h"
@@ -30,6 +33,12 @@
 #include "components/policy/core/common/cloud/mock_user_cloud_policy_store.h"
 #include "components/policy/core/common/cloud/user_cloud_policy_manager.h"
 #include "components/policy/core/common/schema_registry.h"
+#include "components/prefs/pref_service.h"
+#include "components/signin/core/browser/account_tracker_service.h"
+#include "components/signin/core/browser/fake_account_fetcher_service.h"
+#include "components/signin/core/browser/fake_profile_oauth2_token_service.h"
+#include "components/signin/core/browser/signin_manager.h"
+#include "components/sync_preferences/testing_pref_service_syncable.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/notification_details.h"
 #include "content/public/browser/notification_service.h"
@@ -37,6 +46,7 @@
 #include "content/public/test/test_browser_thread_bundle.h"
 #include "google_apis/gaia/gaia_constants.h"
 #include "google_apis/gaia/google_service_auth_error.h"
+#include "net/base/net_errors.h"
 #include "net/http/http_status_code.h"
 #include "net/url_request/test_url_fetcher_factory.h"
 #include "net/url_request/url_request_context_getter.h"
@@ -46,7 +56,7 @@
 #include "testing/gtest/include/gtest/gtest.h"
 
 #if defined(OS_ANDROID)
-#include "chrome/browser/policy/cloud/user_policy_signin_service_android.h"
+#include "chrome/browser/policy/cloud/user_policy_signin_service_mobile.h"
 #else
 #include "chrome/browser/policy/cloud/user_policy_signin_service.h"
 #endif
@@ -61,6 +71,7 @@ namespace policy {
 
 namespace {
 
+const char kTestGaiaId[] = "gaia-id-testuser@test.com";
 const char kTestUser[] = "testuser@test.com";
 
 #if !defined(OS_ANDROID)
@@ -77,36 +88,16 @@ const char kHostedDomainResponse[] =
     "  \"hd\": \"test.com\""
     "}";
 
-class SigninManagerFake : public FakeSigninManager {
- public:
-  explicit SigninManagerFake(Profile* profile)
-      : FakeSigninManager(profile) {
-    Initialize(profile, NULL);
-  }
-
-  void ForceSignOut() {
-    // Allow signing out now.
-    prohibit_signout_ = false;
-    SignOut();
-  }
-
-  static BrowserContextKeyedService* Build(content::BrowserContext* profile) {
-    return new SigninManagerFake(static_cast<Profile*>(profile));
-  }
-};
-
 UserCloudPolicyManager* BuildCloudPolicyManager(
     content::BrowserContext* context) {
-  MockUserCloudPolicyStore *store = new MockUserCloudPolicyStore();
+  MockUserCloudPolicyStore* store = new MockUserCloudPolicyStore();
   EXPECT_CALL(*store, Load()).Times(AnyNumber());
 
   return new UserCloudPolicyManager(
-      scoped_ptr<UserCloudPolicyStore>(store),
-      base::FilePath(),
-      scoped_ptr<CloudExternalDataManager>(),
-      base::MessageLoopProxy::current(),
-      base::MessageLoopProxy::current(),
-      base::MessageLoopProxy::current());
+      std::unique_ptr<UserCloudPolicyStore>(store), base::FilePath(),
+      std::unique_ptr<CloudExternalDataManager>(),
+      base::ThreadTaskRunnerHandle::Get(), base::ThreadTaskRunnerHandle::Get(),
+      base::ThreadTaskRunnerHandle::Get());
 }
 
 class UserPolicySigninServiceTest : public testing::Test {
@@ -130,12 +121,16 @@ class UserPolicySigninServiceTest : public testing::Test {
     // a valid login token, while on other platforms, the login refresh token
     // is specified directly.
 #if defined(OS_ANDROID)
-    GetTokenService()->IssueRefreshTokenForUser(kTestUser,
-                                                "oauth2_login_refresh_token");
+    GetTokenService()->UpdateCredentials(
+        AccountTrackerService::PickAccountIdForAccount(
+            profile_.get()->GetPrefs(), kTestGaiaId, kTestUser),
+        "oauth2_login_refresh_token");
 #endif
     service->RegisterForPolicy(
         kTestUser,
-#if !defined(OS_ANDROID)
+#if defined(OS_ANDROID)
+        kTestGaiaId,
+#else
         "mock_oauth_token",
 #endif
         base::Bind(&UserPolicySigninServiceTest::OnRegisterCompleted,
@@ -143,14 +138,14 @@ class UserPolicySigninServiceTest : public testing::Test {
     ASSERT_TRUE(IsRequestActive());
   }
 
-  virtual void SetUp() OVERRIDE {
+  void SetUp() override {
     UserPolicySigninServiceFactory::SetDeviceManagementServiceForTesting(
         &device_management_service_);
 
     local_state_.reset(new TestingPrefServiceSimple);
     chrome::RegisterLocalState(local_state_->registry());
     system_request_context_getter_ = new net::TestURLRequestContextGetter(
-        base::MessageLoopProxy::current());
+        base::ThreadTaskRunnerHandle::Get());
     TestingBrowserProcess::GetGlobal()->SetSystemRequestContext(
         system_request_context_getter_.get());
     TestingBrowserProcess::GetGlobal()->SetLocalState(local_state_.get());
@@ -160,8 +155,8 @@ class UserPolicySigninServiceTest : public testing::Test {
 
     // Create a testing profile with cloud-policy-on-signin enabled, and bring
     // up a UserCloudPolicyManager with a MockUserCloudPolicyStore.
-    scoped_ptr<TestingPrefServiceSyncable> prefs(
-        new TestingPrefServiceSyncable());
+    std::unique_ptr<sync_preferences::TestingPrefServiceSyncable> prefs(
+        new sync_preferences::TestingPrefServiceSyncable());
     chrome::RegisterUserProfilePrefs(prefs->registry());
 
     // UserCloudPolicyManagerFactory isn't a real
@@ -172,16 +167,22 @@ class UserPolicySigninServiceTest : public testing::Test {
     UserCloudPolicyManagerFactory::GetInstance()->RegisterTestingFactory(
         BuildCloudPolicyManager);
     TestingProfile::Builder builder;
-    builder.SetPrefService(scoped_ptr<PrefServiceSyncable>(prefs.Pass()));
+    builder.SetPrefService(
+        std::unique_ptr<sync_preferences::PrefServiceSyncable>(
+            std::move(prefs)));
     builder.AddTestingFactory(SigninManagerFactory::GetInstance(),
-                              SigninManagerFake::Build);
+                              BuildFakeSigninManagerBase);
     builder.AddTestingFactory(ProfileOAuth2TokenServiceFactory::GetInstance(),
-                              FakeProfileOAuth2TokenServiceWrapper::Build);
+                              BuildFakeProfileOAuth2TokenService);
+    builder.AddTestingFactory(AccountFetcherServiceFactory::GetInstance(),
+                              FakeAccountFetcherServiceBuilder::BuildForTests);
+    builder.AddTestingFactory(ChromeSigninClientFactory::GetInstance(),
+                              signin::BuildTestSigninClient);
 
-    profile_ = builder.Build().Pass();
+    profile_ = builder.Build();
     url_factory_.set_remove_fetcher_on_delete(true);
 
-    signin_manager_ = static_cast<SigninManagerFake*>(
+    signin_manager_ = static_cast<FakeSigninManager*>(
         SigninManagerFactory::GetForProfile(profile_.get()));
     // Tests are responsible for freeing the UserCloudPolicyManager instances
     // they inject.
@@ -196,7 +197,7 @@ class UserPolicySigninServiceTest : public testing::Test {
     Mock::VerifyAndClearExpectations(mock_store_);
   }
 
-  virtual void TearDown() OVERRIDE {
+  void TearDown() override {
     UserPolicySigninServiceFactory::SetDeviceManagementServiceForTesting(NULL);
     UserCloudPolicyManagerFactory::GetInstance()->ClearTestingFactory();
     // Free the profile before we clear out the browser prefs.
@@ -205,14 +206,14 @@ class UserPolicySigninServiceTest : public testing::Test {
         TestingBrowserProcess::GetGlobal();
     testing_browser_process->SetLocalState(NULL);
     local_state_.reset();
-    testing_browser_process->SetBrowserPolicyConnector(NULL);
+    testing_browser_process->ShutdownBrowserPolicyConnector();
     base::RunLoop run_loop;
     run_loop.RunUntilIdle();
   }
 
   virtual void AddProfile() {
     // For this test, the user should not be signed in yet.
-    DCHECK(signin_manager_->GetAuthenticatedUsername().empty());
+    DCHECK(!signin_manager_->IsAuthenticated());
 
     // Initializing UserPolicySigninService while the user is not signed in
     // should result in the store being cleared to remove any lingering policy.
@@ -274,7 +275,7 @@ class UserPolicySigninServiceTest : public testing::Test {
                 CreateJob(DeviceManagementRequestJob::TYPE_REGISTRATION, _))
         .WillOnce(device_management_service_.CreateAsyncJob(
             &register_request));
-    EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _, _))
+    EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _))
         .Times(1);
 
     // Now mimic the user being a hosted domain - this should cause a Register()
@@ -306,7 +307,7 @@ class UserPolicySigninServiceTest : public testing::Test {
     EXPECT_CALL(device_management_service_,
                 CreateJob(DeviceManagementRequestJob::TYPE_POLICY_FETCH, _))
         .WillOnce(device_management_service_.CreateAsyncJob(&fetch_request));
-    EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _, _))
+    EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _))
         .Times(1);
 
     signin_service->FetchPolicyForSignedInUser(
@@ -345,10 +346,10 @@ class UserPolicySigninServiceTest : public testing::Test {
     Mock::VerifyAndClearExpectations(this);
   }
 
-  scoped_ptr<TestingProfile> profile_;
+  std::unique_ptr<TestingProfile> profile_;
   MockUserCloudPolicyStore* mock_store_;  // Not owned.
   SchemaRegistry schema_registry_;
-  scoped_ptr<UserCloudPolicyManager> manager_;
+  std::unique_ptr<UserCloudPolicyManager> manager_;
 
   // BrowserPolicyConnector and UrlFetcherFactory want to initialize and free
   // various components asynchronously via tasks, so create fake threads here.
@@ -356,7 +357,7 @@ class UserPolicySigninServiceTest : public testing::Test {
 
   net::TestURLFetcherFactory url_factory_;
 
-  SigninManagerFake* signin_manager_;
+  FakeSigninManager* signin_manager_;
 
   // Used in conjunction with OnRegisterCompleted() to test client registration
   // callbacks.
@@ -370,19 +371,19 @@ class UserPolicySigninServiceTest : public testing::Test {
   // BrowserPolicyConnector).
   MockDeviceManagementService device_management_service_;
 
-  scoped_ptr<TestingPrefServiceSimple> local_state_;
+  std::unique_ptr<TestingPrefServiceSimple> local_state_;
   scoped_refptr<net::URLRequestContextGetter> system_request_context_getter_;
 };
 
 class UserPolicySigninServiceSignedInTest : public UserPolicySigninServiceTest {
  public:
-  virtual void AddProfile() OVERRIDE {
+  void AddProfile() override {
     // UserCloudPolicyManager should not be initialized.
     ASSERT_FALSE(manager_->core()->service());
 
     // Set the user as signed in.
     SigninManagerFactory::GetForProfile(profile_.get())->
-        SetAuthenticatedUsername(kTestUser);
+        SetAuthenticatedAccountInfo(kTestGaiaId, kTestUser);
 
     // Let the SigninService know that the profile has been created.
     content::NotificationService::current()->Notify(
@@ -394,8 +395,8 @@ class UserPolicySigninServiceSignedInTest : public UserPolicySigninServiceTest {
 
 TEST_F(UserPolicySigninServiceTest, InitWhileSignedOut) {
   // Make sure user is not signed in.
-  ASSERT_TRUE(SigninManagerFactory::GetForProfile(profile_.get())->
-      GetAuthenticatedUsername().empty());
+  ASSERT_FALSE(SigninManagerFactory::GetForProfile(profile_.get())->
+      IsAuthenticated());
 
   // UserCloudPolicyManager should not be initialized.
   ASSERT_FALSE(manager_->core()->service());
@@ -417,8 +418,10 @@ TEST_F(UserPolicySigninServiceSignedInTest, InitWhileSignedIn) {
   ASSERT_FALSE(IsRequestActive());
 
   // Make oauth token available.
-  GetTokenService()->IssueRefreshTokenForUser(kTestUser,
-                                              "oauth_login_refresh_token");
+  GetTokenService()->UpdateCredentials(
+      SigninManagerFactory::GetForProfile(profile_.get())
+          ->GetAuthenticatedAccountId(),
+      "oauth_login_refresh_token");
 
   // Client registration should be in progress since we now have an oauth token.
   EXPECT_EQ(mock_store_->signin_username_, kTestUser);
@@ -436,8 +439,10 @@ TEST_F(UserPolicySigninServiceSignedInTest, InitWhileSignedInOAuthError) {
   ASSERT_FALSE(IsRequestActive());
 
   // Make oauth token available.
-  GetTokenService()->IssueRefreshTokenForUser(kTestUser,
-                                              "oauth_login_refresh_token");
+  GetTokenService()->UpdateCredentials(
+      SigninManagerFactory::GetForProfile(profile_.get())
+          ->GetAuthenticatedAccountId(),
+      "oauth_login_refresh_token");
 
   // Client registration should be in progress since we now have an oauth token.
   ASSERT_TRUE(IsRequestActive());
@@ -455,15 +460,17 @@ TEST_F(UserPolicySigninServiceTest, SignInAfterInit) {
   ASSERT_FALSE(manager_->core()->service());
 
   // Now sign in the user.
-  SigninManagerFactory::GetForProfile(profile_.get())->SetAuthenticatedUsername(
-      kTestUser);
+  SigninManagerFactory::GetForProfile(profile_.get())
+      ->SetAuthenticatedAccountInfo(kTestGaiaId, kTestUser);
 
   // Complete initialization of the store.
   mock_store_->NotifyStoreLoaded();
 
   // Make oauth token available.
-  GetTokenService()->IssueRefreshTokenForUser(kTestUser,
-                                              "oauth_login_refresh_token");
+  GetTokenService()->UpdateCredentials(
+      SigninManagerFactory::GetForProfile(profile_.get())
+          ->GetAuthenticatedAccountId(),
+      "oauth_login_refresh_token");
 
   // UserCloudPolicyManager should be initialized.
   EXPECT_EQ(mock_store_->signin_username_, kTestUser);
@@ -479,15 +486,18 @@ TEST_F(UserPolicySigninServiceTest, SignInWithNonEnterpriseUser) {
   ASSERT_FALSE(manager_->core()->service());
 
   // Now sign in a non-enterprise user (blacklisted gmail.com domain).
-  SigninManagerFactory::GetForProfile(profile_.get())->SetAuthenticatedUsername(
-      "non_enterprise_user@gmail.com");
+  SigninManagerFactory::GetForProfile(profile_.get())
+      ->SetAuthenticatedAccountInfo("gaia-id-non_enterprise_user@gmail.com",
+                                    "non_enterprise_user@gmail.com");
 
   // Complete initialization of the store.
   mock_store_->NotifyStoreLoaded();
 
   // Make oauth token available.
-  GetTokenService()->IssueRefreshTokenForUser(kTestUser,
-                                              "oauth_login_refresh_token");
+  GetTokenService()->UpdateCredentials(
+      SigninManagerFactory::GetForProfile(profile_.get())
+          ->GetAuthenticatedAccountId(),
+      "oauth_login_refresh_token");
 
   // UserCloudPolicyManager should not be initialized and there should be no
   // DMToken request active.
@@ -501,12 +511,14 @@ TEST_F(UserPolicySigninServiceTest, UnregisteredClient) {
   ASSERT_FALSE(manager_->core()->service());
 
   // Now sign in the user.
-  SigninManagerFactory::GetForProfile(profile_.get())->SetAuthenticatedUsername(
-      kTestUser);
+  SigninManagerFactory::GetForProfile(profile_.get())
+      ->SetAuthenticatedAccountInfo(kTestGaiaId, kTestUser);
 
   // Make oauth token available.
-  GetTokenService()->IssueRefreshTokenForUser(kTestUser,
-                                              "oauth_login_refresh_token");
+  GetTokenService()->UpdateCredentials(
+      SigninManagerFactory::GetForProfile(profile_.get())
+          ->GetAuthenticatedAccountId(),
+      "oauth_login_refresh_token");
 
   // UserCloudPolicyManager should be initialized.
   EXPECT_EQ(mock_store_->signin_username_, kTestUser);
@@ -529,12 +541,14 @@ TEST_F(UserPolicySigninServiceTest, RegisteredClient) {
   ASSERT_FALSE(manager_->core()->service());
 
   // Now sign in the user.
-  SigninManagerFactory::GetForProfile(profile_.get())->SetAuthenticatedUsername(
-      kTestUser);
+  SigninManagerFactory::GetForProfile(profile_.get())
+      ->SetAuthenticatedAccountInfo(kTestGaiaId, kTestUser);
 
   // Make oauth token available.
-  GetTokenService()->IssueRefreshTokenForUser(kTestUser,
-                                              "oauth_login_refresh_token");
+  GetTokenService()->UpdateCredentials(
+      SigninManagerFactory::GetForProfile(profile_.get())
+          ->GetAuthenticatedAccountId(),
+      "oauth_login_refresh_token");
 
   // UserCloudPolicyManager should be initialized.
   EXPECT_EQ(mock_store_->signin_username_, kTestUser);
@@ -569,7 +583,9 @@ TEST_F(UserPolicySigninServiceSignedInTest, SignOutAfterInit) {
   EXPECT_CALL(*mock_store_, Clear());
 
   // Now sign out.
-  SigninManagerFactory::GetForProfile(profile_.get())->SignOut();
+  SigninManagerFactory::GetForProfile(profile_.get())
+      ->SignOut(signin_metrics::SIGNOUT_TEST,
+                signin_metrics::SignoutDelete::IGNORE_METRIC);
 
   // UserCloudPolicyManager should be shut down.
   ASSERT_FALSE(manager_->core()->service());
@@ -593,7 +609,7 @@ TEST_F(UserPolicySigninServiceTest, RegisterPolicyClientOAuthFailure) {
       GoogleServiceAuthError::FromServiceError("fail"));
 #else
   net::TestURLFetcher* fetcher = url_factory_.GetFetcherByID(0);
-  fetcher->set_status(net::URLRequestStatus(net::URLRequestStatus::FAILED, -1));
+  fetcher->set_status(net::URLRequestStatus::FromError(net::ERR_FAILED));
   fetcher->delegate()->OnURLFetchComplete(fetcher);
 #endif
 
@@ -650,7 +666,7 @@ TEST_F(UserPolicySigninServiceTest, RegisterPolicyClientFailedRegistration) {
   EXPECT_CALL(device_management_service_,
               CreateJob(DeviceManagementRequestJob::TYPE_REGISTRATION, _))
       .WillOnce(device_management_service_.CreateAsyncJob(&register_request));
-  EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _, _))
+  EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _))
         .Times(1);
 
   // Now mimic the user being a hosted domain - this should cause a Register()
@@ -684,7 +700,7 @@ TEST_F(UserPolicySigninServiceTest, RegisterPolicyClientSucceeded) {
   EXPECT_CALL(device_management_service_,
               CreateJob(DeviceManagementRequestJob::TYPE_REGISTRATION, _))
       .WillOnce(device_management_service_.CreateAsyncJob(&register_request));
-  EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _, _))
+  EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _))
       .Times(1);
 
   // Now mimic the user being a hosted domain - this should cause a Register()
@@ -717,7 +733,7 @@ TEST_F(UserPolicySigninServiceTest, FetchPolicyFailed) {
   EXPECT_CALL(device_management_service_,
               CreateJob(DeviceManagementRequestJob::TYPE_POLICY_FETCH, _))
       .WillOnce(device_management_service_.CreateAsyncJob(&fetch_request));
-  EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _, _))
+  EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _))
       .Times(1);
   UserPolicySigninService* signin_service =
       UserPolicySigninServiceFactory::GetForProfile(profile_.get());
@@ -765,7 +781,7 @@ TEST_F(UserPolicySigninServiceTest, PolicyFetchFailureTemporary) {
   EXPECT_CALL(device_management_service_,
               CreateJob(DeviceManagementRequestJob::TYPE_POLICY_FETCH, _))
       .WillOnce(device_management_service_.CreateAsyncJob(&fetch_request));
-  EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _, _))
+  EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _))
       .Times(1);
   manager_->RefreshPolicies();
   Mock::VerifyAndClearExpectations(this);
@@ -791,7 +807,7 @@ TEST_F(UserPolicySigninServiceTest, PolicyFetchFailureDisableManagement) {
   EXPECT_CALL(device_management_service_,
               CreateJob(DeviceManagementRequestJob::TYPE_POLICY_FETCH, _))
       .WillOnce(device_management_service_.CreateAsyncJob(&fetch_request));
-  EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _, _))
+  EXPECT_CALL(device_management_service_, StartJob(_, _, _, _, _, _))
       .Times(1);
   manager_->RefreshPolicies();
   Mock::VerifyAndClearExpectations(this);
