@@ -6,22 +6,30 @@
 
 #include <limits.h>
 #include <stddef.h>
+
 #include <utility>
 
 #include "base/bind.h"
 #include "base/macros.h"
-#include "base/memory/scoped_vector.h"
-#include "base/prefs/pref_value_map.h"
+#include "base/memory/ptr_util.h"
 #include "base/values.h"
 #include "build/build_config.h"
+#include "chrome/browser/net/disk_cache_dir_policy_handler.h"
+#include "chrome/browser/net/safe_search_util.h"
+#include "chrome/browser/policy/file_selection_dialogs_policy_handler.h"
+#include "chrome/browser/policy/javascript_policy_handler.h"
 #include "chrome/browser/policy/managed_bookmarks_policy_handler.h"
+#include "chrome/browser/policy/network_prediction_policy_handler.h"
 #include "chrome/browser/profiles/incognito_mode_policy_handler.h"
+#include "chrome/browser/sessions/restore_on_startup_policy_handler.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/features.h"
 #include "chrome/common/pref_names.h"
 #include "components/bookmarks/common/bookmark_pref_names.h"
+#include "components/certificate_transparency/pref_names.h"
 #include "components/content_settings/core/common/pref_names.h"
 #include "components/metrics/metrics_pref_names.h"
+#include "components/ntp_snippets/pref_names.h"
 #include "components/password_manager/core/common/password_manager_pref_names.h"
 #include "components/policy/core/browser/autofill_policy_handler.h"
 #include "components/policy/core/browser/configuration_policy_handler.h"
@@ -33,42 +41,38 @@
 #include "components/policy/core/common/policy_map.h"
 #include "components/policy/core/common/policy_pref_names.h"
 #include "components/policy/core/common/schema.h"
+#include "components/policy/policy_constants.h"
+#include "components/prefs/pref_value_map.h"
 #include "components/search_engines/default_search_policy_handler.h"
 #include "components/signin/core/common/signin_pref_names.h"
+#include "components/spellcheck/spellcheck_build_features.h"
 #include "components/ssl_config/ssl_config_prefs.h"
+#include "components/sync/driver/sync_policy_handler.h"
 #include "components/translate/core/common/translate_pref_names.h"
 #include "components/variations/pref_names.h"
-#include "policy/policy_constants.h"
+#include "extensions/features/features.h"
 
 #if BUILDFLAG(ANDROID_JAVA_UI)
 #include "chrome/browser/search/contextual_search_policy_handler_android.h"
 #endif
 
-#if !defined(OS_IOS)
-#include "chrome/browser/net/disk_cache_dir_policy_handler.h"
-#include "chrome/browser/policy/file_selection_dialogs_policy_handler.h"
-#include "chrome/browser/policy/javascript_policy_handler.h"
-#include "chrome/browser/policy/network_prediction_policy_handler.h"
-#include "chrome/browser/sessions/restore_on_startup_policy_handler.h"
-#include "components/sync_driver/sync_policy_handler.h"
-#endif
-
 #if defined(OS_CHROMEOS)
+#include "ash/common/accessibility_types.h"
 #include "chrome/browser/chromeos/platform_keys/key_permissions_policy_handler.h"
 #include "chrome/browser/chromeos/policy/configuration_policy_handler_chromeos.h"
+#include "chrome/browser/policy/default_geolocation_policy_handler.h"
 #include "chromeos/chromeos_pref_names.h"
 #include "chromeos/dbus/power_policy_controller.h"
 #include "components/drive/drive_pref_names.h"
 #include "components/user_manager/user.h"
 #include "components/user_manager/user_manager.h"
-#include "ui/chromeos/accessibility_types.h"
 #endif
 
-#if !defined(OS_ANDROID) && !defined(OS_IOS)
+#if !defined(OS_ANDROID)
 #include "chrome/browser/download/download_dir_policy_handler.h"
 #endif
 
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
 #include "chrome/browser/extensions/api/messaging/native_messaging_policy_handler.h"
 #include "chrome/browser/extensions/extension_management_constants.h"
 #include "chrome/browser/extensions/policy_handlers.h"
@@ -76,6 +80,14 @@
 #include "extensions/common/manifest.h"
 #endif
 
+#if defined(ENABLE_PLUGINS)
+#include "chrome/browser/plugins/plugin_policy_handler.h"
+#endif
+
+#if BUILDFLAG(ENABLE_SPELLCHECK)
+#include "components/spellcheck/browser/pref_names.h"
+#endif
+
 namespace policy {
 
 namespace {
@@ -104,8 +116,8 @@ const PolicyToPreferenceMapEntry kSimplePolicyMap[] = {
   { key::kWPADQuickCheckEnabled,
     prefs::kQuickCheckEnabled,
     base::Value::TYPE_BOOLEAN },
-  { key::kDisableSpdy,
-    prefs::kDisableSpdy,
+  { key::kPacHttpsUrlStrippingEnabled,
+    prefs::kPacHttpsUrlStrippingEnabled,
     base::Value::TYPE_BOOLEAN },
   { key::kSafeBrowsingEnabled,
     prefs::kSafeBrowsingEnabled,
@@ -113,15 +125,12 @@ const PolicyToPreferenceMapEntry kSimplePolicyMap[] = {
   { key::kForceGoogleSafeSearch,
     prefs::kForceGoogleSafeSearch,
     base::Value::TYPE_BOOLEAN },
-  { key::kForceYouTubeSafetyMode,
-    prefs::kForceYouTubeSafetyMode,
-    base::Value::TYPE_BOOLEAN },
+  { key::kForceYouTubeRestrict,
+    prefs::kForceYouTubeRestrict,
+    base::Value::TYPE_INTEGER},
   { key::kPasswordManagerEnabled,
     password_manager::prefs::kPasswordManagerSavingEnabled,
     base::Value::TYPE_BOOLEAN },
-  { key::kPasswordManagerAllowShowPasswords,
-    password_manager::prefs::kPasswordManagerAllowShowPasswords,
-    base::Value::TYPE_BOOLEAN },
   { key::kPrintingEnabled,
     prefs::kPrintingEnabled,
     base::Value::TYPE_BOOLEAN },
@@ -134,15 +143,9 @@ const PolicyToPreferenceMapEntry kSimplePolicyMap[] = {
   { key::kApplicationLocaleValue,
     prefs::kApplicationLocale,
     base::Value::TYPE_STRING },
-  { key::kDisabledPlugins,
-    prefs::kPluginsDisabledPlugins,
-    base::Value::TYPE_LIST },
-  { key::kDisabledPluginsExceptions,
-    prefs::kPluginsDisabledPluginsExceptions,
-    base::Value::TYPE_LIST },
-  { key::kEnabledPlugins,
-    prefs::kPluginsEnabledPlugins,
-    base::Value::TYPE_LIST },
+  { key::kAlwaysOpenPdfExternally,
+    prefs::kPluginsAlwaysOpenPdfExternally,
+    base::Value::TYPE_BOOLEAN },
   { key::kShowHomeButton,
     prefs::kShowHomeButton,
     base::Value::TYPE_BOOLEAN },
@@ -236,6 +239,9 @@ const PolicyToPreferenceMapEntry kSimplePolicyMap[] = {
   { key::kRequireOnlineRevocationChecksForLocalAnchors,
     ssl_config::prefs::kCertRevocationCheckingRequiredLocalAnchors,
     base::Value::TYPE_BOOLEAN },
+  { key::kEnableSha1ForLocalAnchors,
+    ssl_config::prefs::kCertEnableSha1LocalAnchors,
+    base::Value::TYPE_BOOLEAN },
   { key::kAuthSchemes,
     prefs::kAuthSchemes,
     base::Value::TYPE_STRING },
@@ -332,6 +338,9 @@ const PolicyToPreferenceMapEntry kSimplePolicyMap[] = {
   { key::kRestrictSigninToPattern,
     prefs::kGoogleServicesUsernamePattern,
     base::Value::TYPE_STRING },
+  { key::kDefaultWebBluetoothGuardSetting,
+    prefs::kManagedDefaultWebBluetoothGuardSetting,
+    base::Value::TYPE_INTEGER },
   { key::kDefaultMediaStreamSetting,
     prefs::kManagedDefaultMediaStreamSetting,
     base::Value::TYPE_INTEGER },
@@ -350,12 +359,18 @@ const PolicyToPreferenceMapEntry kSimplePolicyMap[] = {
   { key::kAllowDinosaurEasterEgg,
     prefs::kAllowDinosaurEasterEgg,
     base::Value::TYPE_BOOLEAN },
-
-#if defined(ENABLE_SPELLCHECK)
-  { key::kSpellCheckServiceEnabled,
-    prefs::kSpellCheckUseSpellingService,
+  { key::kAllowedDomainsForApps,
+    prefs::kAllowedDomainsForApps,
+    base::Value::TYPE_STRING },
+  { key::kComponentUpdatesEnabled,
+    prefs::kComponentUpdatesEnabled,
     base::Value::TYPE_BOOLEAN },
-#endif  // defined(ENABLE_SPELLCHECK)
+
+#if BUILDFLAG(ENABLE_SPELLCHECK)
+  { key::kSpellCheckServiceEnabled,
+    spellcheck::prefs::kSpellCheckUseSpellingService,
+    base::Value::TYPE_BOOLEAN },
+#endif  // BUILDFLAG(ENABLE_SPELLCHECK)
 
   { key::kDisableScreenshots,
     prefs::kDisableScreenshots,
@@ -384,23 +399,32 @@ const PolicyToPreferenceMapEntry kSimplePolicyMap[] = {
   { key::kForceEphemeralProfiles,
     prefs::kForceEphemeralProfiles,
     base::Value::TYPE_BOOLEAN },
-  { key::kSSLVersionFallbackMin,
-    ssl_config::prefs::kSSLVersionFallbackMin,
-    base::Value::TYPE_STRING },
-  { key::kRC4Enabled,
-    ssl_config::prefs::kRC4Enabled,
+  { key::kDHEEnabled,
+    ssl_config::prefs::kDHEEnabled,
     base::Value::TYPE_BOOLEAN },
-
-#if !defined(OS_MACOSX) && !defined(OS_IOS)
+  { key::kNTPContentSuggestionsEnabled,
+    ntp_snippets::prefs::kEnableSnippets,
+    base::Value::TYPE_BOOLEAN },
+#if defined(ENABLE_MEDIA_ROUTER)
+  { key::kEnableMediaRouter,
+    prefs::kEnableMediaRouter,
+    base::Value::TYPE_BOOLEAN },
+#endif  // defined(ENABLE_MEDIA_ROUTER)
+#if defined(ENABLE_WEBRTC)
+  { key::kWebRtcUdpPortRange,
+    prefs::kWebRTCUDPPortRange,
+    base::Value::TYPE_STRING },
+#endif  // defined(ENABLE_WEBRTC)
+#if !defined(OS_MACOSX)
   { key::kFullscreenAllowed,
     prefs::kFullscreenAllowed,
     base::Value::TYPE_BOOLEAN },
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
   { key::kFullscreenAllowed,
     extensions::pref_names::kAppFullscreenAllowed,
     base::Value::TYPE_BOOLEAN },
-#endif  // defined(ENABLE_EXTENSIONS)
-#endif  // !defined(OS_MACOSX) && !defined(OS_IOS)
+#endif  // BUILDFLAG(ENABLE_EXTENSIONS)
+#endif  // !defined(OS_MACOSX)
 
 #if defined(OS_CHROMEOS)
   { key::kChromeOsLockOnIdleSuspend,
@@ -418,6 +442,9 @@ const PolicyToPreferenceMapEntry kSimplePolicyMap[] = {
   { key::kExternalStorageDisabled,
     prefs::kExternalStorageDisabled,
     base::Value::TYPE_BOOLEAN },
+  { key::kExternalStorageReadOnly,
+    prefs::kExternalStorageReadOnly,
+    base::Value::TYPE_BOOLEAN },
   { key::kAudioOutputAllowed,
     chromeos::prefs::kAudioOutputAllowed,
     base::Value::TYPE_BOOLEAN },
@@ -502,6 +529,18 @@ const PolicyToPreferenceMapEntry kSimplePolicyMap[] = {
   { key::kUnifiedDesktopEnabledByDefault,
     prefs::kUnifiedDesktopEnabledByDefault,
     base::Value::TYPE_BOOLEAN },
+  { key::kArcEnabled,
+    prefs::kArcEnabled,
+    base::Value::TYPE_BOOLEAN },
+  { key::kArcBackupRestoreEnabled,
+    prefs::kArcBackupRestoreEnabled,
+    base::Value::TYPE_BOOLEAN },
+  { key::kReportArcStatusEnabled,
+    prefs::kReportArcStatusEnabled,
+    base::Value::TYPE_BOOLEAN },
+  { key::kNativePrinters,
+    prefs::kRecommendedNativePrinters,
+    base::Value::TYPE_LIST },
 #endif  // defined(OS_CHROMEOS)
 
 // Metrics reporting is controlled by a platform specific policy for ChromeOS
@@ -530,7 +569,7 @@ const PolicyToPreferenceMapEntry kSimplePolicyMap[] = {
     base::Value::TYPE_STRING },
 #endif  // BUILDFLAG(ANDROID_JAVA_UI)
 
-#if !defined(OS_CHROMEOS) && !defined(OS_ANDROID) && !defined(OS_IOS)
+#if !defined(OS_CHROMEOS) && !defined(OS_ANDROID)
   { key::kNativeMessagingUserLevelHosts,
     extensions::pref_names::kNativeMessagingUserLevelHosts,
     base::Value::TYPE_BOOLEAN },
@@ -540,13 +579,45 @@ const PolicyToPreferenceMapEntry kSimplePolicyMap[] = {
   { key::kBrowserAddPersonEnabled,
     prefs::kBrowserAddPersonEnabled,
     base::Value::TYPE_BOOLEAN },
-#endif  // !defined(OS_CHROMEOS) && !defined(OS_ANDROID) && !defined(OS_IOS)
+  { key::kForceBrowserSignin,
+    prefs::kForceBrowserSignin,
+    base::Value::TYPE_BOOLEAN },
+#endif  // !defined(OS_CHROMEOS) && !defined(OS_ANDROID)
 
 #if defined(OS_WIN)
   { key::kWelcomePageOnOSUpgradeEnabled,
     prefs::kWelcomePageOnOSUpgradeEnabled,
     base::Value::TYPE_BOOLEAN },
 #endif  // OS_WIN
+
+#if !defined(OS_ANDROID)
+  { key::kSuppressUnsupportedOSWarning,
+    prefs::kSuppressUnsupportedOSWarning,
+    base::Value::TYPE_BOOLEAN },
+#endif  // !OS_ANDROID
+
+#if defined(OS_CHROMEOS)
+  { key::kSystemTimezoneAutomaticDetection,
+    prefs::kSystemTimezoneAutomaticDetectionPolicy,
+    base::Value::TYPE_INTEGER },
+#endif
+
+  { key::kTaskManagerEndProcessEnabled,
+    prefs::kTaskManagerEndProcessEnabled,
+    base::Value::TYPE_BOOLEAN },
+
+#if defined(OS_CHROMEOS)
+  { key::kNetworkThrottlingEnabled,
+    prefs::kNetworkThrottlingEnabled,
+    base::Value::TYPE_DICTIONARY },
+
+  { key::kAllowScreenLock, prefs::kAllowScreenLock, base::Value::TYPE_BOOLEAN },
+
+  { key::kQuickUnlockModeWhitelist, prefs::kQuickUnlockModeWhitelist,
+    base::Value::TYPE_LIST },
+  { key::kQuickUnlockTimeout, prefs::kQuickUnlockTimeout,
+    base::Value::TYPE_INTEGER },
+#endif
 };
 
 class ForceSafeSearchPolicyHandler : public TypeCheckingPolicyHandler {
@@ -559,17 +630,30 @@ class ForceSafeSearchPolicyHandler : public TypeCheckingPolicyHandler {
   // ConfigurationPolicyHandler implementation:
   void ApplyPolicySettings(const PolicyMap& policies,
                            PrefValueMap* prefs) override {
-    // If either of the new GoogleSafeSearch or YouTubeSafetyMode policies is
-    // defined, then this one should be ignored. crbug.com/476908
-    // Note: Those policies are declared in kSimplePolicyMap above.
+    // If either of the new ForceGoogleSafeSearch, ForceYouTubeSafetyMode or
+    // ForceYouTubeRestrict policies are defined, then this one should be
+    // ignored. crbug.com/476908, crbug.com/590478
+    // Note: Those policies are declared in kSimplePolicyMap above, except
+    // ForceYouTubeSafetyMode, which has been replaced by ForceYouTubeRestrict.
     if (policies.GetValue(key::kForceGoogleSafeSearch) ||
-        policies.GetValue(key::kForceYouTubeSafetyMode)) {
+        policies.GetValue(key::kForceYouTubeSafetyMode) ||
+        policies.GetValue(key::kForceYouTubeRestrict)) {
       return;
     }
     const base::Value* value = policies.GetValue(policy_name());
     if (value) {
+      bool enabled;
       prefs->SetValue(prefs::kForceGoogleSafeSearch, value->CreateDeepCopy());
-      prefs->SetValue(prefs::kForceYouTubeSafetyMode, value->CreateDeepCopy());
+
+      // Note that ForceYouTubeRestrict is an int policy, we cannot simply deep
+      // copy value, which is a boolean.
+      if (value->GetAsBoolean(&enabled)) {
+        prefs->SetValue(
+            prefs::kForceYouTubeRestrict,
+            base::MakeUnique<base::FundamentalValue>(
+                enabled ? safe_search_util::YOUTUBE_RESTRICT_MODERATE
+                        : safe_search_util::YOUTUBE_RESTRICT_OFF));
+      }
     }
   }
 
@@ -577,30 +661,60 @@ class ForceSafeSearchPolicyHandler : public TypeCheckingPolicyHandler {
   DISALLOW_COPY_AND_ASSIGN(ForceSafeSearchPolicyHandler);
 };
 
-#if defined(ENABLE_EXTENSIONS)
+class ForceYouTubeSafetyModePolicyHandler : public TypeCheckingPolicyHandler {
+ public:
+  ForceYouTubeSafetyModePolicyHandler()
+      : TypeCheckingPolicyHandler(key::kForceYouTubeSafetyMode,
+                                  base::Value::TYPE_BOOLEAN) {}
+  ~ForceYouTubeSafetyModePolicyHandler() override {}
+
+  // ConfigurationPolicyHandler implementation:
+  void ApplyPolicySettings(const PolicyMap& policies,
+                           PrefValueMap* prefs) override {
+    // If only the deprecated ForceYouTubeSafetyMode policy is set,
+    // but not ForceYouTubeRestrict, set ForceYouTubeRestrict to Moderate.
+    if (policies.GetValue(key::kForceYouTubeRestrict))
+      return;
+
+    const base::Value* value = policies.GetValue(policy_name());
+    bool enabled;
+    if (value && value->GetAsBoolean(&enabled)) {
+      prefs->SetValue(
+          prefs::kForceYouTubeRestrict,
+          base::MakeUnique<base::FundamentalValue>(
+              enabled ? safe_search_util::YOUTUBE_RESTRICT_MODERATE
+                      : safe_search_util::YOUTUBE_RESTRICT_OFF));
+    }
+  }
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(ForceYouTubeSafetyModePolicyHandler);
+};
+
+#if BUILDFLAG(ENABLE_EXTENSIONS)
 void GetExtensionAllowedTypesMap(
-    ScopedVector<StringMappingListPolicyHandler::MappingEntry>* result) {
+    std::vector<std::unique_ptr<StringMappingListPolicyHandler::MappingEntry>>*
+        result) {
   // Mapping from extension type names to Manifest::Type.
   for (size_t index = 0;
        index < extensions::schema_constants::kAllowedTypesMapSize;
        ++index) {
     const extensions::schema_constants::AllowedTypesMapEntry& entry =
         extensions::schema_constants::kAllowedTypesMap[index];
-    result->push_back(new StringMappingListPolicyHandler::MappingEntry(
-        entry.name,
-        scoped_ptr<base::Value>(
-            new base::FundamentalValue(entry.manifest_type))));
+    result->push_back(
+        base::MakeUnique<StringMappingListPolicyHandler::MappingEntry>(
+            entry.name, std::unique_ptr<base::Value>(
+                            new base::FundamentalValue(entry.manifest_type))));
   }
 }
 #endif
 
-#if !defined(OS_IOS)
 void GetDeprecatedFeaturesMap(
-    ScopedVector<StringMappingListPolicyHandler::MappingEntry>* result) {
+    std::vector<std::unique_ptr<StringMappingListPolicyHandler::MappingEntry>>*
+        result) {
   // Maps feature tags as specified in policy to the corresponding switch to
   // re-enable them.
 }
-#endif  // !defined(OS_IOS)
 
 }  // namespace
 
@@ -615,222 +729,207 @@ void PopulatePolicyHandlerParameters(PolicyHandlerParameters* parameters) {
 #endif
 }
 
-scoped_ptr<ConfigurationPolicyHandlerList> BuildHandlerList(
+std::unique_ptr<ConfigurationPolicyHandlerList> BuildHandlerList(
     const Schema& chrome_schema) {
-  scoped_ptr<ConfigurationPolicyHandlerList> handlers(
+  std::unique_ptr<ConfigurationPolicyHandlerList> handlers(
       new ConfigurationPolicyHandlerList(
           base::Bind(&PopulatePolicyHandlerParameters),
           base::Bind(&GetChromePolicyDetails)));
   for (size_t i = 0; i < arraysize(kSimplePolicyMap); ++i) {
-    handlers->AddHandler(make_scoped_ptr(new SimplePolicyHandler(
+    handlers->AddHandler(base::MakeUnique<SimplePolicyHandler>(
         kSimplePolicyMap[i].policy_name, kSimplePolicyMap[i].preference_path,
-        kSimplePolicyMap[i].value_type)));
+        kSimplePolicyMap[i].value_type));
   }
 
-  handlers->AddHandler(make_scoped_ptr(new AutofillPolicyHandler()));
-  handlers->AddHandler(make_scoped_ptr(new DefaultSearchPolicyHandler()));
-  handlers->AddHandler(make_scoped_ptr(new ForceSafeSearchPolicyHandler()));
-  handlers->AddHandler(make_scoped_ptr(new IncognitoModePolicyHandler()));
+  handlers->AddHandler(base::MakeUnique<AutofillPolicyHandler>());
+  handlers->AddHandler(base::MakeUnique<DefaultSearchPolicyHandler>());
+  handlers->AddHandler(base::MakeUnique<ForceSafeSearchPolicyHandler>());
+  handlers->AddHandler(base::MakeUnique<ForceYouTubeSafetyModePolicyHandler>());
+  handlers->AddHandler(base::MakeUnique<IncognitoModePolicyHandler>());
   handlers->AddHandler(
-      make_scoped_ptr(new ManagedBookmarksPolicyHandler(chrome_schema)));
-  handlers->AddHandler(make_scoped_ptr(new ProxyPolicyHandler()));
-  handlers->AddHandler(make_scoped_ptr(new URLBlacklistPolicyHandler()));
+      base::MakeUnique<ManagedBookmarksPolicyHandler>(chrome_schema));
+  handlers->AddHandler(base::MakeUnique<ProxyPolicyHandler>());
+  handlers->AddHandler(base::MakeUnique<URLBlacklistPolicyHandler>());
+
+  handlers->AddHandler(base::MakeUnique<SimpleSchemaValidatingPolicyHandler>(
+      key::kCertificateTransparencyEnforcementDisabledForUrls,
+      certificate_transparency::prefs::kCTExcludedHosts, chrome_schema,
+      SCHEMA_STRICT, SimpleSchemaValidatingPolicyHandler::RECOMMENDED_ALLOWED,
+      SimpleSchemaValidatingPolicyHandler::MANDATORY_ALLOWED));
 
 #if BUILDFLAG(ANDROID_JAVA_UI)
   handlers->AddHandler(
-      make_scoped_ptr(new ContextualSearchPolicyHandlerAndroid()));
+      base::MakeUnique<ContextualSearchPolicyHandlerAndroid>());
 #endif
 
-#if !defined(OS_IOS)
-  handlers->AddHandler(
-      make_scoped_ptr(new FileSelectionDialogsPolicyHandler()));
-  handlers->AddHandler(make_scoped_ptr(new JavascriptPolicyHandler()));
-  handlers->AddHandler(make_scoped_ptr(new NetworkPredictionPolicyHandler()));
-  handlers->AddHandler(make_scoped_ptr(new RestoreOnStartupPolicyHandler()));
-  handlers->AddHandler(make_scoped_ptr(new sync_driver::SyncPolicyHandler()));
+  handlers->AddHandler(base::MakeUnique<FileSelectionDialogsPolicyHandler>());
+  handlers->AddHandler(base::MakeUnique<JavascriptPolicyHandler>());
+  handlers->AddHandler(base::MakeUnique<NetworkPredictionPolicyHandler>());
+  handlers->AddHandler(base::MakeUnique<RestoreOnStartupPolicyHandler>());
+  handlers->AddHandler(base::MakeUnique<syncer::SyncPolicyHandler>());
 
-  handlers->AddHandler(make_scoped_ptr(new StringMappingListPolicyHandler(
+  handlers->AddHandler(base::MakeUnique<StringMappingListPolicyHandler>(
       key::kEnableDeprecatedWebPlatformFeatures,
       prefs::kEnableDeprecatedWebPlatformFeatures,
-      base::Bind(GetDeprecatedFeaturesMap))));
-#endif  // !defined(OS_IOS)
+      base::Bind(GetDeprecatedFeaturesMap)));
 
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
+  handlers->AddHandler(base::MakeUnique<extensions::ExtensionListPolicyHandler>(
+      key::kExtensionInstallWhitelist,
+      extensions::pref_names::kInstallAllowList, false));
+  handlers->AddHandler(base::MakeUnique<extensions::ExtensionListPolicyHandler>(
+      key::kExtensionInstallBlacklist, extensions::pref_names::kInstallDenyList,
+      true));
   handlers->AddHandler(
-      make_scoped_ptr(new extensions::ExtensionListPolicyHandler(
-          key::kExtensionInstallWhitelist,
-          extensions::pref_names::kInstallAllowList, false)));
+      base::MakeUnique<extensions::ExtensionInstallForcelistPolicyHandler>());
   handlers->AddHandler(
-      make_scoped_ptr(new extensions::ExtensionListPolicyHandler(
-          key::kExtensionInstallBlacklist,
-          extensions::pref_names::kInstallDenyList, true)));
-  handlers->AddHandler(make_scoped_ptr(
-      new extensions::ExtensionInstallForcelistPolicyHandler()));
-  handlers->AddHandler(
-      make_scoped_ptr(new extensions::ExtensionURLPatternListPolicyHandler(
+      base::MakeUnique<extensions::ExtensionURLPatternListPolicyHandler>(
           key::kExtensionInstallSources,
-          extensions::pref_names::kAllowedInstallSites)));
-  handlers->AddHandler(make_scoped_ptr(new StringMappingListPolicyHandler(
+          extensions::pref_names::kAllowedInstallSites));
+  handlers->AddHandler(base::MakeUnique<StringMappingListPolicyHandler>(
       key::kExtensionAllowedTypes, extensions::pref_names::kAllowedTypes,
-      base::Bind(GetExtensionAllowedTypesMap))));
-  handlers->AddHandler(make_scoped_ptr(
-      new extensions::ExtensionSettingsPolicyHandler(chrome_schema)));
+      base::Bind(GetExtensionAllowedTypesMap)));
+  handlers->AddHandler(
+      base::MakeUnique<extensions::ExtensionSettingsPolicyHandler>(
+          chrome_schema));
 #endif
 
-#if !defined(OS_CHROMEOS) && !defined(OS_ANDROID) && !defined(OS_IOS)
-  handlers->AddHandler(make_scoped_ptr(new DiskCacheDirPolicyHandler()));
+#if !defined(OS_CHROMEOS) && !defined(OS_ANDROID)
+  handlers->AddHandler(base::MakeUnique<DiskCacheDirPolicyHandler>());
 
   handlers->AddHandler(
-      make_scoped_ptr(new extensions::NativeMessagingHostListPolicyHandler(
+      base::MakeUnique<extensions::NativeMessagingHostListPolicyHandler>(
           key::kNativeMessagingWhitelist,
-          extensions::pref_names::kNativeMessagingWhitelist, false)));
+          extensions::pref_names::kNativeMessagingWhitelist, false));
   handlers->AddHandler(
-      make_scoped_ptr(new extensions::NativeMessagingHostListPolicyHandler(
+      base::MakeUnique<extensions::NativeMessagingHostListPolicyHandler>(
           key::kNativeMessagingBlacklist,
-          extensions::pref_names::kNativeMessagingBlacklist, true)));
-#endif  // !defined(OS_CHROMEOS) && !defined(OS_ANDROID) && !defined(OS_IOS)
+          extensions::pref_names::kNativeMessagingBlacklist, true));
+#endif  // !defined(OS_CHROMEOS) && !defined(OS_ANDROID)
 
-#if !defined(OS_ANDROID) && !defined(OS_IOS)
-  handlers->AddHandler(make_scoped_ptr(new DownloadDirPolicyHandler));
+#if !defined(OS_ANDROID)
+  handlers->AddHandler(base::WrapUnique(new DownloadDirPolicyHandler));
 
-  handlers->AddHandler(make_scoped_ptr(new SimpleSchemaValidatingPolicyHandler(
+  handlers->AddHandler(base::MakeUnique<SimpleSchemaValidatingPolicyHandler>(
       key::kRegisteredProtocolHandlers,
       prefs::kPolicyRegisteredProtocolHandlers, chrome_schema, SCHEMA_STRICT,
       SimpleSchemaValidatingPolicyHandler::RECOMMENDED_ALLOWED,
-      SimpleSchemaValidatingPolicyHandler::MANDATORY_PROHIBITED)));
+      SimpleSchemaValidatingPolicyHandler::MANDATORY_PROHIBITED));
 #endif
 
 #if defined(OS_CHROMEOS)
-  handlers->AddHandler(
-      make_scoped_ptr(new extensions::ExtensionListPolicyHandler(
-          key::kAttestationExtensionWhitelist,
-          prefs::kAttestationExtensionWhitelist, false)));
-  handlers->AddHandler(make_scoped_ptr(
+  handlers->AddHandler(base::MakeUnique<extensions::ExtensionListPolicyHandler>(
+      key::kAttestationExtensionWhitelist,
+      prefs::kAttestationExtensionWhitelist, false));
+  handlers->AddHandler(base::WrapUnique(
       NetworkConfigurationPolicyHandler::CreateForDevicePolicy()));
-  handlers->AddHandler(make_scoped_ptr(
+  handlers->AddHandler(base::WrapUnique(
       NetworkConfigurationPolicyHandler::CreateForUserPolicy()));
-  handlers->AddHandler(make_scoped_ptr(new PinnedLauncherAppsPolicyHandler()));
-  handlers->AddHandler(make_scoped_ptr(new ScreenMagnifierPolicyHandler()));
-  handlers->AddHandler(make_scoped_ptr(
-      new LoginScreenPowerManagementPolicyHandler(chrome_schema)));
+  handlers->AddHandler(base::MakeUnique<PinnedLauncherAppsPolicyHandler>());
+  handlers->AddHandler(base::MakeUnique<ScreenMagnifierPolicyHandler>());
+  handlers->AddHandler(
+      base::MakeUnique<LoginScreenPowerManagementPolicyHandler>(chrome_schema));
 
-  ScopedVector<ConfigurationPolicyHandler>
+  std::vector<std::unique_ptr<ConfigurationPolicyHandler>>
       power_management_idle_legacy_policies;
   power_management_idle_legacy_policies.push_back(
-      new IntRangePolicyHandler(key::kScreenDimDelayAC,
-                                prefs::kPowerAcScreenDimDelayMs,
-                                0,
-                                INT_MAX,
-                                true));
+      base::MakeUnique<IntRangePolicyHandler>(key::kScreenDimDelayAC,
+                                              prefs::kPowerAcScreenDimDelayMs,
+                                              0, INT_MAX, true));
   power_management_idle_legacy_policies.push_back(
-      new IntRangePolicyHandler(key::kScreenOffDelayAC,
-                                prefs::kPowerAcScreenOffDelayMs,
-                                0,
-                                INT_MAX,
-                                true));
+      base::MakeUnique<IntRangePolicyHandler>(key::kScreenOffDelayAC,
+                                              prefs::kPowerAcScreenOffDelayMs,
+                                              0, INT_MAX, true));
   power_management_idle_legacy_policies.push_back(
-      new IntRangePolicyHandler(key::kIdleWarningDelayAC,
-                                prefs::kPowerAcIdleWarningDelayMs,
-                                0,
-                                INT_MAX,
-                                true));
-  power_management_idle_legacy_policies.push_back(new IntRangePolicyHandler(
-      key::kIdleDelayAC, prefs::kPowerAcIdleDelayMs, 0, INT_MAX, true));
+      base::MakeUnique<IntRangePolicyHandler>(key::kIdleWarningDelayAC,
+                                              prefs::kPowerAcIdleWarningDelayMs,
+                                              0, INT_MAX, true));
   power_management_idle_legacy_policies.push_back(
-      new IntRangePolicyHandler(key::kScreenDimDelayBattery,
-                                prefs::kPowerBatteryScreenDimDelayMs,
-                                0,
-                                INT_MAX,
-                                true));
+      base::MakeUnique<IntRangePolicyHandler>(
+          key::kIdleDelayAC, prefs::kPowerAcIdleDelayMs, 0, INT_MAX, true));
   power_management_idle_legacy_policies.push_back(
-      new IntRangePolicyHandler(key::kScreenOffDelayBattery,
-                                prefs::kPowerBatteryScreenOffDelayMs,
-                                0,
-                                INT_MAX,
-                                true));
+      base::MakeUnique<IntRangePolicyHandler>(
+          key::kScreenDimDelayBattery, prefs::kPowerBatteryScreenDimDelayMs, 0,
+          INT_MAX, true));
   power_management_idle_legacy_policies.push_back(
-      new IntRangePolicyHandler(key::kIdleWarningDelayBattery,
-                                prefs::kPowerBatteryIdleWarningDelayMs,
-                                0,
-                                INT_MAX,
-                                true));
+      base::MakeUnique<IntRangePolicyHandler>(
+          key::kScreenOffDelayBattery, prefs::kPowerBatteryScreenOffDelayMs, 0,
+          INT_MAX, true));
   power_management_idle_legacy_policies.push_back(
-      new IntRangePolicyHandler(key::kIdleDelayBattery,
-                                prefs::kPowerBatteryIdleDelayMs,
-                                0,
-                                INT_MAX,
-                                true));
-  power_management_idle_legacy_policies.push_back(new IntRangePolicyHandler(
-      key::kIdleActionAC,
-      prefs::kPowerAcIdleAction,
-      chromeos::PowerPolicyController::ACTION_SUSPEND,
-      chromeos::PowerPolicyController::ACTION_DO_NOTHING,
-      false));
-  power_management_idle_legacy_policies.push_back(new IntRangePolicyHandler(
-      key::kIdleActionBattery,
-      prefs::kPowerBatteryIdleAction,
-      chromeos::PowerPolicyController::ACTION_SUSPEND,
-      chromeos::PowerPolicyController::ACTION_DO_NOTHING,
-      false));
+      base::MakeUnique<IntRangePolicyHandler>(
+          key::kIdleWarningDelayBattery, prefs::kPowerBatteryIdleWarningDelayMs,
+          0, INT_MAX, true));
   power_management_idle_legacy_policies.push_back(
-      new DeprecatedIdleActionHandler());
+      base::MakeUnique<IntRangePolicyHandler>(key::kIdleDelayBattery,
+                                              prefs::kPowerBatteryIdleDelayMs,
+                                              0, INT_MAX, true));
+  power_management_idle_legacy_policies.push_back(
+      base::MakeUnique<IntRangePolicyHandler>(
+          key::kIdleActionAC, prefs::kPowerAcIdleAction,
+          chromeos::PowerPolicyController::ACTION_SUSPEND,
+          chromeos::PowerPolicyController::ACTION_DO_NOTHING, false));
+  power_management_idle_legacy_policies.push_back(
+      base::MakeUnique<IntRangePolicyHandler>(
+          key::kIdleActionBattery, prefs::kPowerBatteryIdleAction,
+          chromeos::PowerPolicyController::ACTION_SUSPEND,
+          chromeos::PowerPolicyController::ACTION_DO_NOTHING, false));
+  power_management_idle_legacy_policies.push_back(
+      base::MakeUnique<DeprecatedIdleActionHandler>());
 
-  ScopedVector<ConfigurationPolicyHandler> screen_lock_legacy_policies;
-  screen_lock_legacy_policies.push_back(
-      new IntRangePolicyHandler(key::kScreenLockDelayAC,
-                                prefs::kPowerAcScreenLockDelayMs,
-                                0,
-                                INT_MAX,
-                                true));
-  screen_lock_legacy_policies.push_back(
-      new IntRangePolicyHandler(key::kScreenLockDelayBattery,
-                                prefs::kPowerBatteryScreenLockDelayMs,
-                                0,
-                                INT_MAX,
-                                true));
+  std::vector<std::unique_ptr<ConfigurationPolicyHandler>>
+      screen_lock_legacy_policies;
+  screen_lock_legacy_policies.push_back(base::MakeUnique<IntRangePolicyHandler>(
+      key::kScreenLockDelayAC, prefs::kPowerAcScreenLockDelayMs, 0, INT_MAX,
+      true));
+  screen_lock_legacy_policies.push_back(base::MakeUnique<IntRangePolicyHandler>(
+      key::kScreenLockDelayBattery, prefs::kPowerBatteryScreenLockDelayMs, 0,
+      INT_MAX, true));
 
-  handlers->AddHandler(make_scoped_ptr(new IntRangePolicyHandler(
+  handlers->AddHandler(base::MakeUnique<IntRangePolicyHandler>(
       key::kSAMLOfflineSigninTimeLimit, prefs::kSAMLOfflineSigninTimeLimit, -1,
-      INT_MAX, true)));
-  handlers->AddHandler(make_scoped_ptr(new IntRangePolicyHandler(
+      INT_MAX, true));
+  handlers->AddHandler(base::MakeUnique<IntRangePolicyHandler>(
       key::kLidCloseAction, prefs::kPowerLidClosedAction,
       chromeos::PowerPolicyController::ACTION_SUSPEND,
-      chromeos::PowerPolicyController::ACTION_DO_NOTHING, false)));
-  handlers->AddHandler(make_scoped_ptr(new IntPercentageToDoublePolicyHandler(
+      chromeos::PowerPolicyController::ACTION_DO_NOTHING, false));
+  handlers->AddHandler(base::MakeUnique<IntPercentageToDoublePolicyHandler>(
       key::kPresentationScreenDimDelayScale,
-      prefs::kPowerPresentationScreenDimDelayFactor, 100, INT_MAX, true)));
-  handlers->AddHandler(make_scoped_ptr(new IntPercentageToDoublePolicyHandler(
+      prefs::kPowerPresentationScreenDimDelayFactor, 100, INT_MAX, true));
+  handlers->AddHandler(base::MakeUnique<IntPercentageToDoublePolicyHandler>(
       key::kUserActivityScreenDimDelayScale,
-      prefs::kPowerUserActivityScreenDimDelayFactor, 100, INT_MAX, true)));
-  handlers->AddHandler(make_scoped_ptr(new IntRangePolicyHandler(
-      key::kUptimeLimit, prefs::kUptimeLimit, 3600, INT_MAX, true)));
-  handlers->AddHandler(make_scoped_ptr(new IntRangePolicyHandler(
+      prefs::kPowerUserActivityScreenDimDelayFactor, 100, INT_MAX, true));
+  handlers->AddHandler(base::MakeUnique<IntRangePolicyHandler>(
+      key::kUptimeLimit, prefs::kUptimeLimit, 3600, INT_MAX, true));
+  handlers->AddHandler(base::WrapUnique(new IntRangePolicyHandler(
       key::kDeviceLoginScreenDefaultScreenMagnifierType, NULL, 0,
-      ui::MAGNIFIER_FULL, false)));
+      ash::MAGNIFIER_FULL, false)));
   // TODO(binjin): Remove LegacyPoliciesDeprecatingPolicyHandler for these two
   // policies once deprecation of legacy power management policies is done.
   // http://crbug.com/346229
+  handlers->AddHandler(base::MakeUnique<LegacyPoliciesDeprecatingPolicyHandler>(
+      std::move(power_management_idle_legacy_policies),
+      base::WrapUnique(
+          new PowerManagementIdleSettingsPolicyHandler(chrome_schema))));
+  handlers->AddHandler(base::MakeUnique<LegacyPoliciesDeprecatingPolicyHandler>(
+      std::move(screen_lock_legacy_policies),
+      base::WrapUnique(new ScreenLockDelayPolicyHandler(chrome_schema))));
   handlers->AddHandler(
-      make_scoped_ptr(new LegacyPoliciesDeprecatingPolicyHandler(
-          std::move(power_management_idle_legacy_policies),
-          make_scoped_ptr(
-              new PowerManagementIdleSettingsPolicyHandler(chrome_schema)))));
+      base::MakeUnique<ExternalDataPolicyHandler>(key::kUserAvatarImage));
   handlers->AddHandler(
-      make_scoped_ptr(new LegacyPoliciesDeprecatingPolicyHandler(
-          std::move(screen_lock_legacy_policies),
-          make_scoped_ptr(new ScreenLockDelayPolicyHandler(chrome_schema)))));
-  handlers->AddHandler(
-      make_scoped_ptr(new ExternalDataPolicyHandler(key::kUserAvatarImage)));
-  handlers->AddHandler(
-      make_scoped_ptr(new ExternalDataPolicyHandler(key::kWallpaperImage)));
-  handlers->AddHandler(make_scoped_ptr(new SimpleSchemaValidatingPolicyHandler(
+      base::MakeUnique<ExternalDataPolicyHandler>(key::kWallpaperImage));
+  handlers->AddHandler(base::WrapUnique(new SimpleSchemaValidatingPolicyHandler(
       key::kSessionLocales, NULL, chrome_schema, SCHEMA_STRICT,
       SimpleSchemaValidatingPolicyHandler::RECOMMENDED_ALLOWED,
       SimpleSchemaValidatingPolicyHandler::MANDATORY_PROHIBITED)));
-  handlers->AddHandler(make_scoped_ptr(
-      new chromeos::KeyPermissionsPolicyHandler(chrome_schema)));
+  handlers->AddHandler(
+      base::MakeUnique<chromeos::KeyPermissionsPolicyHandler>(chrome_schema));
+  handlers->AddHandler(base::WrapUnique(new DefaultGeolocationPolicyHandler()));
 #endif  // defined(OS_CHROMEOS)
 
+#if defined(ENABLE_PLUGINS)
+  handlers->AddHandler(base::MakeUnique<PluginPolicyHandler>());
+#endif  // defined(ENABLE_PLUGINS)
+
   return handlers;
 }
 
