@@ -8,9 +8,7 @@
 
 #include "base/bind.h"
 #include "base/command_line.h"
-#include "base/metrics/histogram.h"
-#include "base/prefs/pref_registry_simple.h"
-#include "base/prefs/pref_service.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/strings/string_number_conversions.h"
 #include "build/build_config.h"
 #include "chrome/browser/browser_process.h"
@@ -23,10 +21,12 @@
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
 #include "components/keyed_service/content/browser_context_dependency_manager.h"
+#include "components/prefs/pref_registry_simple.h"
+#include "components/prefs/pref_service.h"
 #include "components/rappor/rappor_utils.h"
 #include "components/search_engines/template_url_prepopulate_data.h"
-#include "components/syncable_prefs/pref_service_syncable.h"
-#include "components/syncable_prefs/synced_pref_change_registrar.h"
+#include "components/sync_preferences/pref_service_syncable.h"
+#include "components/sync_preferences/synced_pref_change_registrar.h"
 #include "content/public/browser/browser_url_handler.h"
 #include "crypto/hmac.h"
 #include "net/base/registry_controlled_domains/registry_controlled_domain.h"
@@ -35,6 +35,7 @@ namespace {
 
 const int kSessionStartupPrefValueMax = SessionStartupPref::kPrefValueMax;
 
+#if !defined(OS_ANDROID)
 // Record a sample for the Settings.NewTabPage rappor metric.
 void SampleNewTabPageURL(Profile* profile) {
   GURL ntp_url(chrome::kChromeUINewTabURL);
@@ -48,6 +49,7 @@ void SampleNewTabPageURL(Profile* profile) {
                                             "Settings.NewTabPage", ntp_url);
   }
 }
+#endif
 
 }  // namespace
 
@@ -58,10 +60,10 @@ PrefMetricsService::PrefMetricsService(Profile* profile)
       weak_factory_(this) {
   RecordLaunchPrefs();
 
-  syncable_prefs::PrefServiceSyncable* prefs =
+  sync_preferences::PrefServiceSyncable* prefs =
       PrefServiceSyncableFromProfile(profile_);
   synced_pref_change_registrar_.reset(
-      new syncable_prefs::SyncedPrefChangeRegistrar(prefs));
+      new sync_preferences::SyncedPrefChangeRegistrar(prefs));
 
   RegisterSyncedPrefObservers();
 }
@@ -78,13 +80,14 @@ PrefMetricsService::PrefMetricsService(Profile* profile,
 PrefMetricsService::~PrefMetricsService() {
 }
 
-void PrefMetricsService::RecordLaunchPrefs() {
-  bool show_home_button = prefs_->GetBoolean(prefs::kShowHomeButton);
-  bool home_page_is_ntp = prefs_->GetBoolean(prefs::kHomePageIsNewTabPage);
+// static
+void PrefMetricsService::RecordHomePageLaunchMetrics(bool show_home_button,
+                                                     bool homepage_is_ntp,
+                                                     const GURL& homepage_url) {
   UMA_HISTOGRAM_BOOLEAN("Settings.ShowHomeButton", show_home_button);
   if (show_home_button) {
     UMA_HISTOGRAM_BOOLEAN("Settings.GivenShowHomeButton_HomePageIsNewTabPage",
-                          home_page_is_ntp);
+                          homepage_is_ntp);
   }
 
   // For non-NTP homepages, see if the URL comes from the same TLD+1 as a known
@@ -94,8 +97,7 @@ void PrefMetricsService::RecordLaunchPrefs() {
   // (pages that share a TLD+1 with a known engine but aren't actually search
   // pages, e.g. plus.google.com).  Additionally, record the TLD+1 of non-NTP
   // homepages through the privacy-preserving Rappor service.
-  if (!home_page_is_ntp) {
-    GURL homepage_url(prefs_->GetString(prefs::kHomePage));
+  if (!homepage_is_ntp) {
     if (homepage_url.is_valid()) {
       UMA_HISTOGRAM_ENUMERATION(
           "Settings.HomePageEngineType",
@@ -106,9 +108,28 @@ void PrefMetricsService::RecordLaunchPrefs() {
           homepage_url);
     }
   }
+}
 
+void PrefMetricsService::RecordLaunchPrefs() {
+  // On Android, determining whether the homepage is enabled requires waiting
+  // for a response from a third party provider installed on the device.  So,
+  // it will be logged later once all the dependent information is available.
+  // See DeferredStartupHandler.java.
+#if !defined(OS_ANDROID)
+  GURL homepage_url(prefs_->GetString(prefs::kHomePage));
+  RecordHomePageLaunchMetrics(prefs_->GetBoolean(prefs::kShowHomeButton),
+                              prefs_->GetBoolean(prefs::kHomePageIsNewTabPage),
+                              homepage_url);
+#endif
+
+  // Android does not support overriding the NTP URL.
+#if !defined(OS_ANDROID)
   SampleNewTabPageURL(profile_);
+#endif
 
+  // Tab restoring is always done on Android, so these metrics are not
+  // applicable.  Also, startup pages are not supported on Android
+#if !defined(OS_ANDROID)
   int restore_on_startup = prefs_->GetInteger(prefs::kRestoreOnStartup);
   UMA_HISTOGRAM_ENUMERATION("Settings.StartupPageLoadSettings",
                             restore_on_startup, kSessionStartupPrefValueMax);
@@ -136,7 +157,9 @@ void PrefMetricsService::RecordLaunchPrefs() {
       }
     }
   }
+#endif
 
+  // Android does not support pinned tabs.
 #if !defined(OS_ANDROID)
   StartupTabs startup_tabs = PinnedTabCodec::ReadPinnedTabs(profile_);
   UMA_HISTOGRAM_CUSTOM_COUNTS("Settings.PinnedTabs",
@@ -182,7 +205,7 @@ void PrefMetricsService::OnPrefChanged(
     const LogHistogramValueCallback& callback,
     const std::string& path,
     bool from_sync) {
-  syncable_prefs::PrefServiceSyncable* prefs =
+  sync_preferences::PrefServiceSyncable* prefs =
       PrefServiceSyncableFromProfile(profile_);
   const PrefService::Preference* pref = prefs->FindPreference(path.c_str());
   DCHECK(pref);
