@@ -6,6 +6,7 @@
 
 #include <stddef.h>
 
+#include <memory>
 #include <vector>
 
 #include "base/compiler_specific.h"
@@ -13,22 +14,21 @@
 #include "base/files/file_util.h"
 #include "base/files/scoped_temp_dir.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
-#include "base/memory/scoped_ptr.h"
-#include "base/prefs/json_pref_store.h"
-#include "base/prefs/persistent_pref_store.h"
-#include "base/prefs/pref_service.h"
-#include "base/prefs/pref_service_factory.h"
-#include "base/prefs/pref_store.h"
-#include "base/prefs/testing_pref_service.h"
 #include "base/run_loop.h"
 #include "base/strings/string_util.h"
 #include "base/values.h"
 #include "components/pref_registry/pref_registry_syncable.h"
+#include "components/prefs/json_pref_store.h"
+#include "components/prefs/persistent_pref_store.h"
+#include "components/prefs/pref_service.h"
+#include "components/prefs/pref_service_factory.h"
+#include "components/prefs/pref_store.h"
+#include "components/prefs/testing_pref_service.h"
 #include "components/user_prefs/tracked/mock_validation_delegate.h"
 #include "components/user_prefs/tracked/pref_hash_filter.h"
 #include "components/user_prefs/tracked/pref_names.h"
-#include "components/user_prefs/tracked/pref_service_hash_store_contents.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace {
@@ -98,7 +98,6 @@ class ProfilePrefStoreManagerTest : public testing::Test {
         reset_recorded_(false) {}
 
   void SetUp() override {
-    ProfilePrefStoreManager::RegisterPrefs(local_state_.registry());
     ProfilePrefStoreManager::RegisterProfilePrefs(profile_pref_registry_.get());
     for (const PrefHashFilter::TrackedPreferenceMetadata* it = kConfiguration;
          it != kConfiguration + arraysize(kConfiguration);
@@ -128,12 +127,9 @@ class ProfilePrefStoreManagerTest : public testing::Test {
   }
 
   void ReloadConfiguration() {
-    manager_.reset(new ProfilePrefStoreManager(profile_dir_.path(),
-                                               configuration_,
-                                               kReportingIdCount,
-                                               seed_,
-                                               "device_id",
-                                               &local_state_));
+    manager_.reset(new ProfilePrefStoreManager(
+        profile_dir_.GetPath(), configuration_, kReportingIdCount, seed_,
+        "device_id", &local_state_));
   }
 
   void TearDown() override { DestroyPrefStore(); }
@@ -144,10 +140,10 @@ class ProfilePrefStoreManagerTest : public testing::Test {
   void VerifyResetRecorded(bool reset_expected) {
     EXPECT_EQ(reset_expected, reset_recorded_);
 
-    base::PrefServiceFactory pref_service_factory;
+    PrefServiceFactory pref_service_factory;
     pref_service_factory.set_user_prefs(pref_store_);
 
-    scoped_ptr<PrefService> pref_service(
+    std::unique_ptr<PrefService> pref_service(
         pref_service_factory.Create(profile_pref_registry_.get()));
 
     EXPECT_EQ(
@@ -158,10 +154,10 @@ class ProfilePrefStoreManagerTest : public testing::Test {
   void ClearResetRecorded() {
     reset_recorded_ = false;
 
-    base::PrefServiceFactory pref_service_factory;
+    PrefServiceFactory pref_service_factory;
     pref_service_factory.set_user_prefs(pref_store_);
 
-    scoped_ptr<PrefService> pref_service(
+    std::unique_ptr<PrefService> pref_service(
         pref_service_factory.Create(profile_pref_registry_.get()));
 
     ProfilePrefStoreManager::ClearResetTime(pref_service.get());
@@ -197,27 +193,18 @@ class ProfilePrefStoreManagerTest : public testing::Test {
     }
   }
 
-  void InitializeDeprecatedCombinedProfilePrefStore() {
-    scoped_refptr<PersistentPrefStore> pref_store =
-        manager_->CreateDeprecatedCombinedProfilePrefStore(
-            main_message_loop_.task_runner());
-    InitializePrefStore(pref_store.get());
-    pref_store = NULL;
-    base::RunLoop().RunUntilIdle();
-  }
-
   void InitializePrefStore(PersistentPrefStore* pref_store) {
     pref_store->AddObserver(&registry_verifier_);
     PersistentPrefStore::PrefReadError error = pref_store->ReadPrefs();
     EXPECT_EQ(PersistentPrefStore::PREF_READ_ERROR_NO_FILE, error);
     pref_store->SetValue(kTrackedAtomic,
-                         make_scoped_ptr(new base::StringValue(kFoobar)),
+                         base::MakeUnique<base::StringValue>(kFoobar),
                          WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
     pref_store->SetValue(kProtectedAtomic,
-                         make_scoped_ptr(new base::StringValue(kHelloWorld)),
+                         base::MakeUnique<base::StringValue>(kHelloWorld),
                          WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
     pref_store->SetValue(kUnprotectedPref,
-                         make_scoped_ptr(new base::StringValue(kFoobar)),
+                         base::MakeUnique<base::StringValue>(kFoobar),
                          WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
     pref_store->RemoveObserver(&registry_verifier_);
     pref_store->CommitPendingWrite();
@@ -237,8 +224,8 @@ class ProfilePrefStoreManagerTest : public testing::Test {
 
   void ReplaceStringInPrefs(const std::string& find,
                             const std::string& replace) {
-    base::FileEnumerator file_enum(
-        profile_dir_.path(), true, base::FileEnumerator::FILES);
+    base::FileEnumerator file_enum(profile_dir_.GetPath(), true,
+                                   base::FileEnumerator::FILES);
 
     for (base::FilePath path = file_enum.Next(); !path.empty();
          path = file_enum.Next()) {
@@ -279,7 +266,7 @@ class ProfilePrefStoreManagerTest : public testing::Test {
   scoped_refptr<user_prefs::PrefRegistrySyncable> profile_pref_registry_;
   RegistryVerifier registry_verifier_;
   MockValidationDelegate mock_validation_delegate_;
-  scoped_ptr<ProfilePrefStoreManager> manager_;
+  std::unique_ptr<ProfilePrefStoreManager> manager_;
   scoped_refptr<PersistentPrefStore> pref_store_;
 
   std::string seed_;
@@ -331,83 +318,6 @@ TEST_F(ProfilePrefStoreManagerTest, ProtectValues) {
   ExpectValidationObserved(kProtectedAtomic);
 }
 
-TEST_F(ProfilePrefStoreManagerTest, MigrateFromOneFile) {
-  InitializeDeprecatedCombinedProfilePrefStore();
-
-  // The deprecated model stores hashes in local state (on supported
-  // platforms)..
-  ASSERT_EQ(
-      ProfilePrefStoreManager::kPlatformSupportsPreferenceTracking,
-      local_state_.GetUserPrefValue(
-          PrefServiceHashStoreContents::kProfilePreferenceHashes) != NULL);
-
-  LoadExistingPrefs();
-
-  // After a first migration, the hashes were copied to the two user preference
-  // files but were not cleaned.
-  ASSERT_EQ(
-      ProfilePrefStoreManager::kPlatformSupportsPreferenceTracking,
-      local_state_.GetUserPrefValue(
-          PrefServiceHashStoreContents::kProfilePreferenceHashes) != NULL);
-
-  ExpectStringValueEquals(kTrackedAtomic, kFoobar);
-  ExpectStringValueEquals(kProtectedAtomic, kHelloWorld);
-  VerifyResetRecorded(false);
-
-  LoadExistingPrefs();
-
-  // In a subsequent launch, the local state hash store should be reset.
-  ASSERT_FALSE(local_state_.GetUserPrefValue(
-      PrefServiceHashStoreContents::kProfilePreferenceHashes));
-
-  ExpectStringValueEquals(kTrackedAtomic, kFoobar);
-  ExpectStringValueEquals(kProtectedAtomic, kHelloWorld);
-  VerifyResetRecorded(false);
-}
-
-TEST_F(ProfilePrefStoreManagerTest, MigrateWithTampering) {
-  InitializeDeprecatedCombinedProfilePrefStore();
-
-  ReplaceStringInPrefs(kFoobar, kBarfoo);
-  ReplaceStringInPrefs(kHelloWorld, kGoodbyeWorld);
-
-  // The deprecated model stores hashes in local state (on supported
-  // platforms)..
-  ASSERT_EQ(
-      ProfilePrefStoreManager::kPlatformSupportsPreferenceTracking,
-      local_state_.GetUserPrefValue(
-          PrefServiceHashStoreContents::kProfilePreferenceHashes) != NULL);
-
-  LoadExistingPrefs();
-
-  // After a first migration, the hashes were copied to the two user preference
-  // files but were not cleaned.
-  ASSERT_EQ(
-      ProfilePrefStoreManager::kPlatformSupportsPreferenceTracking,
-      local_state_.GetUserPrefValue(
-          PrefServiceHashStoreContents::kProfilePreferenceHashes) != NULL);
-
-  // kTrackedAtomic is unprotected and thus will be loaded as it appears on
-  // disk.
-  ExpectStringValueEquals(kTrackedAtomic, kBarfoo);
-
-  // If preference tracking is supported, the tampered value of kProtectedAtomic
-  // will be discarded at load time, leaving this preference undefined.
-  EXPECT_NE(ProfilePrefStoreManager::kPlatformSupportsPreferenceTracking,
-            pref_store_->GetValue(kProtectedAtomic, NULL));
-  VerifyResetRecorded(
-      ProfilePrefStoreManager::kPlatformSupportsPreferenceTracking);
-
-  LoadExistingPrefs();
-
-  // In a subsequent launch, the local state hash store would be reset.
-  ASSERT_FALSE(local_state_.GetUserPrefValue(
-      PrefServiceHashStoreContents::kProfilePreferenceHashes));
-
-  ExpectStringValueEquals(kTrackedAtomic, kBarfoo);
-  VerifyResetRecorded(false);
-}
-
 TEST_F(ProfilePrefStoreManagerTest, InitializePrefsFromMasterPrefs) {
   base::DictionaryValue master_prefs;
   master_prefs.Set(kTrackedAtomic, new base::StringValue(kFoobar));
@@ -571,7 +481,7 @@ TEST_F(ProfilePrefStoreManagerTest, ProtectedToUnprotected) {
   // Trigger the logic that migrates it back to the unprotected preferences
   // file.
   pref_store_->SetValue(kProtectedAtomic,
-                        make_scoped_ptr(new base::StringValue(kGoodbyeWorld)),
+                        base::WrapUnique(new base::StringValue(kGoodbyeWorld)),
                         WriteablePrefStore::DEFAULT_PREF_WRITE_FLAGS);
   LoadExistingPrefs();
   ExpectStringValueEquals(kProtectedAtomic, kGoodbyeWorld);
