@@ -66,6 +66,9 @@ enum FinalStatus {
   FINAL_STATUS_NAVIGATION_INTERCEPTED = 51,
   FINAL_STATUS_PRERENDERING_DISABLED = 52,
   FINAL_STATUS_CELLULAR_NETWORK = 53,
+  FINAL_STATUS_BLOCK_THIRD_PARTY_COOKIES = 54,
+  FINAL_STATUS_CREDENTIAL_MANAGER_API = 55,
+  FINAL_STATUS_NOSTATE_PREFETCH_FINISHED = 56,
   FINAL_STATUS_MAX,
 };
 
