@@ -5,8 +5,9 @@
 #ifndef CHROME_BROWSER_PRERENDER_PRERENDER_TAB_HELPER_H_
 #define CHROME_BROWSER_PRERENDER_PRERENDER_TAB_HELPER_H_
 
+#include <memory>
+
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/time/time.h"
 #include "chrome/browser/prerender/prerender_histograms.h"
@@ -15,14 +16,6 @@
 #include "content/public/browser/web_contents_user_data.h"
 #include "url/gurl.h"
 
-namespace autofill {
-struct PasswordForm;
-}
-
-namespace password_manager {
-class PasswordManager;
-}
-
 namespace prerender {
 
 class PrerenderManager;
@@ -37,7 +30,6 @@ class PrerenderTabHelper
 
   // content::WebContentsObserver implementation.
   void DidGetRedirectForResourceRequest(
-      content::RenderFrameHost* render_frame_host,
       const content::ResourceRedirectDetails& details) override;
   void DidStopLoading() override;
   void DidStartProvisionalLoadForFrame(
@@ -71,6 +63,11 @@ class PrerenderTabHelper
   // Retrieves the PrerenderManager, or NULL, if none was found.
   PrerenderManager* MaybeGetPrerenderManager() const;
 
+  // Returns the current TimeTicks synchronized with PrerenderManager ticks. In
+  // tests the clock can be mocked out in PrerenderManager, but in production
+  // this should be always TimeTicks::Now().
+  base::TimeTicks GetTimeTicksFromPrerenderManager() const;
+
   // Returns whether the WebContents being observed is currently prerendering.
   bool IsPrerendering();
 
