@@ -5,21 +5,24 @@
 #ifndef CHROME_BROWSER_PROFILES_AVATAR_MENU_H_
 #define CHROME_BROWSER_PROFILES_AVATAR_MENU_H_
 
+#include <stddef.h>
+
 #include <string>
 #include <vector>
 
-#include "base/basictypes.h"
 #include "base/compiler_specific.h"
+#include "base/files/file_path.h"
+#include "base/macros.h"
 #include "base/scoped_observer.h"
 #include "base/strings/string16.h"
-#include "chrome/browser/profiles/profile_info_cache_observer.h"
+#include "chrome/browser/profiles/profile_attributes_storage.h"
 #include "chrome/browser/profiles/profile_metrics.h"
-#include "chrome/browser/ui/host_desktop.h"
+#include "chrome/common/features.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/browser/web_contents_observer.h"
 #include "ui/gfx/image/image.h"
 
-#if defined(ENABLE_SUPERVISED_USERS)
+#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
 #include "chrome/browser/supervised_user/supervised_user_service_observer.h"
 #endif
 
@@ -27,7 +30,7 @@ class AvatarMenuActions;
 class AvatarMenuObserver;
 class Browser;
 class Profile;
-class ProfileInfoInterface;
+class ProfileAttributesStorage;
 class ProfileList;
 class SupervisedUserService;
 
@@ -37,14 +40,16 @@ class SupervisedUserService;
 // data changes, and the view for this model should forward actions
 // back to it in response to user events.
 class AvatarMenu :
-#if defined(ENABLE_SUPERVISED_USERS)
+#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
     public SupervisedUserServiceObserver,
 #endif
-    public ProfileInfoCacheObserver {
+    public ProfileAttributesStorage::Observer {
  public:
   // Represents an item in the menu.
   struct Item {
-    Item(size_t menu_index, size_t profile_index, const gfx::Image& icon);
+    Item(size_t menu_index, const base::FilePath& profile_path,
+         const gfx::Image& icon);
+    Item(const Item& other);
     ~Item();
 
     // The icon to be displayed next to the item.
@@ -79,16 +84,13 @@ class AvatarMenu :
     // profiles.
     size_t menu_index;
 
-    // The index in the |profile_cache| for this profile.
-    size_t profile_index;
-
     // The path of this profile.
     base::FilePath profile_path;
   };
 
   // Constructor. |observer| can be NULL. |browser| can be NULL and a new one
   // will be created if an action requires it.
-  AvatarMenu(ProfileInfoInterface* profile_cache,
+  AvatarMenu(ProfileAttributesStorage* profile_storage,
              AvatarMenuObserver* observer,
              Browser* browser);
   ~AvatarMenu() override;
@@ -96,15 +98,10 @@ class AvatarMenu :
   // True if avatar menu should be displayed.
   static bool ShouldShowAvatarMenu();
 
-  // Sets |image| to the avatar corresponding to the profile at |profile_path|
-  // and sets |is_rectangle| to true unless |image| is a built-in profile
-  // avatar. For built-in profile avatars, returns the non-high res version.
+  // Sets |image| to the avatar corresponding to the profile at |profile_path|.
+  // For built-in profile avatars, returns the non-high res version.
   static void GetImageForMenuButton(const base::FilePath& profile_path,
-                                    gfx::Image* image,
-                                    bool* is_rectangle);
-
-  // Compare items by name.
-  static bool CompareItems(const Item* item1, const Item* item2);
+                                    gfx::Image* image);
 
   // Opens a Browser with the specified profile in response to the user
   // selecting an item. If |always_create| is true then a new window is created
@@ -129,6 +126,9 @@ class AvatarMenu :
   // Gets the Item at the specified index.
   const Item& GetItemAt(size_t index) const;
 
+  // Gets the index in this menu for which profile_path is equal to |path|.
+  size_t GetIndexOfItemWithProfilePath(const base::FilePath& path);
+
   // Returns the index of the active profile.
   size_t GetActiveProfileIndex();
 
@@ -137,10 +137,6 @@ class AvatarMenu :
   // string will be returned.
   base::string16 GetSupervisedUserInformation() const;
 
-  // Returns the icon for the supervised user which will be displayed in the
-  // avatar menu.
-  const gfx::Image& GetSupervisedUserIcon() const;
-
   // This menu is also used for the always-present Mac system menubar. If the
   // last active browser changes, the menu will need to reference that browser.
   void ActiveBrowserChanged(Browser* browser);
@@ -152,7 +148,7 @@ class AvatarMenu :
   bool ShouldShowEditProfileLink() const;
 
  private:
-  // ProfileInfoCacheObserver:
+  // ProfileAttributesStorage::Observer:
   void OnProfileAdded(const base::FilePath& profile_path) override;
   void OnProfileWasRemoved(const base::FilePath& profile_path,
       const base::string16& profile_name) override;
@@ -166,7 +162,7 @@ class AvatarMenu :
       const base::FilePath& profile_path) override;
   void OnProfileIsOmittedChanged(const base::FilePath& profile_path) override;
 
-#if defined(ENABLE_SUPERVISED_USERS)
+#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
   // SupervisedUserServiceObserver:
   void OnCustodianInfoChanged() override;
 #endif
@@ -175,19 +171,19 @@ class AvatarMenu :
   void Update();
 
   // The model that provides the list of menu items.
-  scoped_ptr<ProfileList> profile_list_;
+  std::unique_ptr<ProfileList> profile_list_;
 
   // The controller for avatar menu actions.
-  scoped_ptr<AvatarMenuActions> menu_actions_;
+  std::unique_ptr<AvatarMenuActions> menu_actions_;
 
-#if defined(ENABLE_SUPERVISED_USERS)
+#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
   // Observes changes to a supervised user's custodian info.
   ScopedObserver<SupervisedUserService, SupervisedUserServiceObserver>
       supervised_user_observer_;
 #endif
 
-  // The cache that provides the profile information. Weak.
-  ProfileInfoInterface* profile_info_;
+  // The storage that provides the profile attributes. Weak.
+  ProfileAttributesStorage* profile_storage_;
 
   // The observer of this model, which is notified of changes. Weak.
   AvatarMenuObserver* observer_;
