@@ -6,6 +6,7 @@
 #define CHROME_BROWSER_PUSH_MESSAGING_PUSH_MESSAGING_SERVICE_IMPL_H_
 
 #include <stdint.h>
+#include <memory>
 #include <set>
 #include <vector>
 
@@ -13,18 +14,20 @@
 #include "base/compiler_specific.h"
 #include "base/gtest_prod_util.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "chrome/browser/background/background_trigger.h"
+#include "chrome/common/features.h"
 #include "components/content_settings/core/browser/content_settings_observer.h"
 #include "components/content_settings/core/common/content_settings.h"
 #include "components/gcm_driver/common/gcm_messages.h"
 #include "components/gcm_driver/gcm_app_handler.h"
 #include "components/gcm_driver/gcm_client.h"
+#include "components/gcm_driver/instance_id/instance_id.h"
 #include "components/keyed_service/core/keyed_service.h"
 #include "content/public/browser/push_messaging_service.h"
-#include "content/public/common/permission_status.mojom.h"
+#include "content/public/common/push_event_payload.h"
 #include "content/public/common/push_messaging_status.h"
+#include "third_party/WebKit/public/platform/modules/permissions/permission_status.mojom.h"
 #include "third_party/WebKit/public/platform/modules/push_messaging/WebPushPermissionStatus.h"
 
 #if defined(ENABLE_NOTIFICATIONS)
@@ -34,10 +37,15 @@
 class Profile;
 class PushMessagingAppIdentifier;
 class PushMessagingServiceObserver;
+class PushMessagingServiceTest;
+class ScopedKeepAlive;
+struct PushSubscriptionOptions;
 
 namespace gcm {
 class GCMDriver;
-class GCMProfileService;
+}
+namespace instance_id {
+class InstanceIDDriver;
 }
 
 namespace user_prefs {
@@ -58,6 +66,7 @@ class PushMessagingServiceImpl : public content::PushMessagingService,
 
   // gcm::GCMAppHandler implementation.
   void ShutdownHandler() override;
+  void OnStoreReset() override;
   void OnMessage(const std::string& app_id,
                  const gcm::IncomingMessage& message) override;
   void OnMessagesDeleted(const std::string& app_id) override;
@@ -69,34 +78,27 @@ class PushMessagingServiceImpl : public content::PushMessagingService,
   bool CanHandle(const std::string& app_id) const override;
 
   // content::PushMessagingService implementation:
-  GURL GetPushEndpoint() override;
-  void SubscribeFromDocument(
-      const GURL& requesting_origin,
-      int64_t service_worker_registration_id,
-      const std::string& sender_id,
-      int renderer_id,
-      int render_frame_id,
-      bool user_visible,
-      const content::PushMessagingService::RegisterCallback& callback) override;
-  void SubscribeFromWorker(
-      const GURL& requesting_origin,
-      int64_t service_worker_registration_id,
-      const std::string& sender_id,
-      bool user_visible,
-      const content::PushMessagingService::RegisterCallback& callback) override;
-  void GetEncryptionInfo(
-      const GURL& origin,
-      int64_t service_worker_registration_id,
-      const content::PushMessagingService::EncryptionInfoCallback& callback)
-      override;
-  void Unsubscribe(
-      const GURL& requesting_origin,
-      int64_t service_worker_registration_id,
-      const std::string& sender_id,
-      const content::PushMessagingService::UnregisterCallback&) override;
+  GURL GetEndpoint(bool standard_protocol) const override;
+  void SubscribeFromDocument(const GURL& requesting_origin,
+                             int64_t service_worker_registration_id,
+                             int renderer_id,
+                             int render_frame_id,
+                             const content::PushSubscriptionOptions& options,
+                             const RegisterCallback& callback) override;
+  void SubscribeFromWorker(const GURL& requesting_origin,
+                           int64_t service_worker_registration_id,
+                           const content::PushSubscriptionOptions& options,
+                           const RegisterCallback& callback) override;
+  void GetEncryptionInfo(const GURL& origin,
+                         int64_t service_worker_registration_id,
+                         const std::string& sender_id,
+                         const EncryptionInfoCallback& callback) override;
+  void Unsubscribe(const GURL& requesting_origin,
+                   int64_t service_worker_registration_id,
+                   const std::string& sender_id,
+                   const UnregisterCallback&) override;
   blink::WebPushPermissionStatus GetPermissionStatus(
-      const GURL& requesting_origin,
-      const GURL& embedding_origin,
+      const GURL& origin,
       bool user_visible) override;
   bool SupportNonVisibleMessages() override;
 
@@ -119,6 +121,8 @@ class PushMessagingServiceImpl : public content::PushMessagingService,
       const base::Closure& callback);
 
  private:
+  friend class PushMessagingBrowserTest;
+  FRIEND_TEST_ALL_PREFIXES(PushMessagingServiceTest, NormalizeSenderInfo);
   FRIEND_TEST_ALL_PREFIXES(PushMessagingServiceTest, PayloadEncryptionTest);
 
   // A subscription is pending until it has succeeded or failed.
@@ -139,72 +143,96 @@ class PushMessagingServiceImpl : public content::PushMessagingService,
 
   // Subscribe methods ---------------------------------------------------------
 
-  void SubscribeEnd(
-      const content::PushMessagingService::RegisterCallback& callback,
-      const std::string& subscription_id,
-      const std::vector<uint8_t>& p256dh,
-      const std::vector<uint8_t>& auth,
-      content::PushRegistrationStatus status);
+  void DoSubscribe(const PushMessagingAppIdentifier& app_identifier,
+                   const content::PushSubscriptionOptions& options,
+                   const RegisterCallback& callback,
+                   blink::mojom::PermissionStatus permission_status);
 
-  void SubscribeEndWithError(
-      const content::PushMessagingService::RegisterCallback& callback,
-      content::PushRegistrationStatus status);
+  void SubscribeEnd(const RegisterCallback& callback,
+                    const std::string& subscription_id,
+                    const std::vector<uint8_t>& p256dh,
+                    const std::vector<uint8_t>& auth,
+                    content::PushRegistrationStatus status);
 
-  void DidSubscribe(
-      const PushMessagingAppIdentifier& app_identifier,
-      const content::PushMessagingService::RegisterCallback& callback,
-      const std::string& subscription_id,
-      gcm::GCMClient::Result result);
+  void SubscribeEndWithError(const RegisterCallback& callback,
+                             content::PushRegistrationStatus status);
+
+  void DidSubscribe(const PushMessagingAppIdentifier& app_identifier,
+                    const std::string& sender_id,
+                    const RegisterCallback& callback,
+                    const std::string& subscription_id,
+                    instance_id::InstanceID::Result result);
 
   void DidSubscribeWithEncryptionInfo(
       const PushMessagingAppIdentifier& app_identifier,
-      const content::PushMessagingService::RegisterCallback& callback,
+      const RegisterCallback& callback,
       const std::string& subscription_id,
       const std::string& p256dh,
       const std::string& auth_secret);
 
-  void DidRequestPermission(
-      const PushMessagingAppIdentifier& app_identifier,
-      const std::string& sender_id,
-      const content::PushMessagingService::RegisterCallback& callback,
-      content::PermissionStatus permission_status);
-
   // GetEncryptionInfo method --------------------------------------------------
 
-  void DidGetEncryptionInfo(
-      const PushMessagingService::EncryptionInfoCallback& callback,
-      const std::string& p256dh,
-      const std::string& auth_secret) const;
+  void DidGetEncryptionInfo(const EncryptionInfoCallback& callback,
+                            const std::string& p256dh,
+                            const std::string& auth_secret) const;
 
   // Unsubscribe methods -------------------------------------------------------
 
-  void Unsubscribe(const std::string& app_id,
-                   const std::string& sender_id,
-                   const content::PushMessagingService::UnregisterCallback&);
+  // |origin|, |service_worker_registration_id| and |app_id| should be provided
+  // whenever they can be obtained. It's valid for |origin| to be empty and
+  // |service_worker_registration_id| to be kInvalidServiceWorkerRegistrationId,
+  // or for app_id to be empty, but not both at once.
+  void UnsubscribeInternal(content::PushUnregistrationReason reason,
+                           const GURL& origin,
+                           int64_t service_worker_registration_id,
+                           const std::string& app_id,
+                           const std::string& sender_id,
+                           const UnregisterCallback& callback);
 
-  void DidUnsubscribe(bool was_subscribed,
-                      const content::PushMessagingService::UnregisterCallback&,
-                      gcm::GCMClient::Result result);
+  void DidClearPushSubscriptionId(content::PushUnregistrationReason reason,
+                                  const std::string& app_id,
+                                  const std::string& sender_id,
+                                  const UnregisterCallback& callback);
+
+  void DidDeleteID(const std::string& app_id,
+                   bool was_subscribed,
+                   const UnregisterCallback&,
+                   instance_id::InstanceID::Result result);
+
+  void DidUnsubscribe(const std::string& app_id_when_instance_id,
+                      bool was_subscribed,
+                      const UnregisterCallback& callback,
+                      content::PushUnregistrationStatus status);
 
   // OnContentSettingChanged methods -------------------------------------------
 
   void UnsubscribeBecausePermissionRevoked(
       const PushMessagingAppIdentifier& app_identifier,
-      const base::Closure& closure,
+      const UnregisterCallback& callback,
       const std::string& sender_id,
       bool success,
       bool not_found);
 
   // Helper methods ------------------------------------------------------------
 
+  // Normalizes the |sender_info|. In most cases the |sender_info| will be
+  // passed through to the GCM Driver as-is, but NIST P-256 application server
+  // keys have to be encoded using the URL-safe variant of the base64 encoding.
+  std::string NormalizeSenderInfo(const std::string& sender_info) const;
+
   // Checks if a given origin is allowed to use Push.
   bool IsPermissionSet(const GURL& origin);
 
-  // Returns whether incoming messages should support payloads.
-  bool AreMessagePayloadsEnabled() const;
+  // Wrapper around {GCMDriver, InstanceID}::GetEncryptionInfo.
+  void GetEncryptionInfoForAppId(
+      const std::string& app_id,
+      const std::string& sender_id,
+      gcm::GCMEncryptionProvider::EncryptionInfoCallback callback);
 
   gcm::GCMDriver* GetGCMDriver() const;
 
+  instance_id::InstanceIDDriver* GetInstanceIDDriver() const;
+
   // Testing methods -----------------------------------------------------------
 
   // Callback to be invoked when a message has been dispatched. Enables tests to
@@ -213,7 +241,7 @@ class PushMessagingServiceImpl : public content::PushMessagingService,
       base::Callback<void(const std::string& app_id,
                           const GURL& origin,
                           int64_t service_worker_registration_id,
-                          const std::string& message_data)>;
+                          const content::PushEventPayload& payload)>;
 
   void SetMessageDispatchedCallbackForTesting(
       const MessageDispatchedCallback& callback) {
@@ -238,7 +266,14 @@ class PushMessagingServiceImpl : public content::PushMessagingService,
 
   MessageDispatchedCallback message_dispatched_callback_for_testing_;
 
-  scoped_ptr<PushMessagingServiceObserver> push_messaging_service_observer_;
+  std::unique_ptr<PushMessagingServiceObserver>
+      push_messaging_service_observer_;
+
+#if BUILDFLAG(ENABLE_BACKGROUND)
+  // KeepAlive registered while we have in-flight push messages, to make sure
+  // we can finish processing them without being interrupted.
+  std::unique_ptr<ScopedKeepAlive> in_flight_keep_alive_;
+#endif
 
   base::WeakPtrFactory<PushMessagingServiceImpl> weak_factory_;
 
