@@ -2,6 +2,19 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+cr.exportPath('print_preview');
+
+/**
+ * @typedef {{selectSaveAsPdfDestination: boolean,
+ *            layoutSettings.portrait: boolean,
+ *            pageRange: string,
+ *            headersAndFooters: boolean,
+ *            backgroundColorsAndImages: boolean,
+ *            margins: number}}
+ * @see chrome/browser/printing/print_preview_pdf_generated_browsertest.cc
+ */
+print_preview.PreviewSettings;
+
 cr.define('print_preview', function() {
   'use strict';
 
@@ -14,38 +27,50 @@ cr.define('print_preview', function() {
     cr.EventTarget.call(this);
 
     // Bind global handlers
-    global['setInitialSettings'] = this.onSetInitialSettings_.bind(this);
-    global['setUseCloudPrint'] = this.onSetUseCloudPrint_.bind(this);
-    global['setPrinters'] = this.onSetPrinters_.bind(this);
-    global['updateWithPrinterCapabilities'] =
+    global.setInitialSettings = this.onSetInitialSettings_.bind(this);
+    global.setUseCloudPrint = this.onSetUseCloudPrint_.bind(this);
+    global.setPrinters = this.onSetPrinters_.bind(this);
+    global.updateWithPrinterCapabilities =
         this.onUpdateWithPrinterCapabilities_.bind(this);
-    global['failedToGetPrinterCapabilities'] =
+    global.failedToGetPrinterCapabilities =
         this.onFailedToGetPrinterCapabilities_.bind(this);
-    global['failedToGetPrivetPrinterCapabilities'] =
+    global.failedToGetPrivetPrinterCapabilities =
       this.onFailedToGetPrivetPrinterCapabilities_.bind(this);
-    global['reloadPrintersList'] = this.onReloadPrintersList_.bind(this);
-    global['printToCloud'] = this.onPrintToCloud_.bind(this);
-    global['fileSelectionCancelled'] =
+    global.failedToGetExtensionPrinterCapabilities =
+        this.onFailedToGetExtensionPrinterCapabilities_.bind(this);
+    global.reloadPrintersList = this.onReloadPrintersList_.bind(this);
+    global.printToCloud = this.onPrintToCloud_.bind(this);
+    global.fileSelectionCancelled =
         this.onFileSelectionCancelled_.bind(this);
-    global['fileSelectionCompleted'] =
+    global.fileSelectionCompleted =
         this.onFileSelectionCompleted_.bind(this);
-    global['printPreviewFailed'] = this.onPrintPreviewFailed_.bind(this);
-    global['invalidPrinterSettings'] =
+    global.printPreviewFailed = this.onPrintPreviewFailed_.bind(this);
+    global.invalidPrinterSettings =
         this.onInvalidPrinterSettings_.bind(this);
-    global['onDidGetDefaultPageLayout'] =
+    global.onDidGetDefaultPageLayout =
         this.onDidGetDefaultPageLayout_.bind(this);
-    global['onDidGetPreviewPageCount'] =
+    global.onDidGetPreviewPageCount =
         this.onDidGetPreviewPageCount_.bind(this);
-    global['onDidPreviewPage'] = this.onDidPreviewPage_.bind(this);
-    global['updatePrintPreview'] = this.onUpdatePrintPreview_.bind(this);
-    global['printScalingDisabledForSourcePDF'] =
-        this.onPrintScalingDisabledForSourcePDF_.bind(this);
-    global['onDidGetAccessToken'] = this.onDidGetAccessToken_.bind(this);
-    global['autoCancelForTesting'] = this.autoCancelForTesting_.bind(this);
-    global['onPrivetPrinterChanged'] = this.onPrivetPrinterChanged_.bind(this);
-    global['onPrivetCapabilitiesSet'] =
-      this.onPrivetCapabilitiesSet_.bind(this);
-    global['onPrivetPrintFailed'] = this.onPrivetPrintFailed_.bind(this);
+    global.onDidPreviewPage = this.onDidPreviewPage_.bind(this);
+    global.updatePrintPreview = this.onUpdatePrintPreview_.bind(this);
+    global.onDidGetAccessToken = this.onDidGetAccessToken_.bind(this);
+    global.autoCancelForTesting = this.autoCancelForTesting_.bind(this);
+    global.onPrivetPrinterChanged = this.onPrivetPrinterChanged_.bind(this);
+    global.onPrivetCapabilitiesSet =
+        this.onPrivetCapabilitiesSet_.bind(this);
+    global.onPrivetPrintFailed = this.onPrivetPrintFailed_.bind(this);
+    global.onExtensionPrintersAdded =
+        this.onExtensionPrintersAdded_.bind(this);
+    global.onExtensionCapabilitiesSet =
+        this.onExtensionCapabilitiesSet_.bind(this);
+    global.onEnableManipulateSettingsForTest =
+        this.onEnableManipulateSettingsForTest_.bind(this);
+    global.printPresetOptionsFromDocument =
+        this.onPrintPresetOptionsFromDocument_.bind(this);
+    global.onProvisionalPrinterResolved =
+        this.onProvisionalDestinationResolved_.bind(this);
+    global.failedToResolveProvisionalPrinter =
+        this.failedToResolveProvisionalDestination_.bind(this);
   };
 
   /**
@@ -65,6 +90,8 @@ cr.define('print_preview', function() {
     GET_CAPABILITIES_FAIL: 'print_preview.NativeLayer.GET_CAPABILITIES_FAIL',
     INITIAL_SETTINGS_SET: 'print_preview.NativeLayer.INITIAL_SETTINGS_SET',
     LOCAL_DESTINATIONS_SET: 'print_preview.NativeLayer.LOCAL_DESTINATIONS_SET',
+    MANIPULATE_SETTINGS_FOR_TEST:
+        'print_preview.NativeLayer.MANIPULATE_SETTINGS_FOR_TEST',
     PAGE_COUNT_READY: 'print_preview.NativeLayer.PAGE_COUNT_READY',
     PAGE_LAYOUT_READY: 'print_preview.NativeLayer.PAGE_LAYOUT_READY',
     PAGE_PREVIEW_READY: 'print_preview.NativeLayer.PAGE_PREVIEW_READY',
@@ -77,7 +104,14 @@ cr.define('print_preview', function() {
     PRIVET_PRINTER_CHANGED: 'print_preview.NativeLayer.PRIVET_PRINTER_CHANGED',
     PRIVET_CAPABILITIES_SET:
         'print_preview.NativeLayer.PRIVET_CAPABILITIES_SET',
-    PRIVET_PRINT_FAILED: 'print_preview.NativeLayer.PRIVET_PRINT_FAILED'
+    PRIVET_PRINT_FAILED: 'print_preview.NativeLayer.PRIVET_PRINT_FAILED',
+    EXTENSION_PRINTERS_ADDED:
+        'print_preview.NativeLayer.EXTENSION_PRINTERS_ADDED',
+    EXTENSION_CAPABILITIES_SET:
+        'print_preview.NativeLayer.EXTENSION_CAPABILITIES_SET',
+    PRINT_PRESET_OPTIONS: 'print_preview.NativeLayer.PRINT_PRESET_OPTIONS',
+    PROVISIONAL_DESTINATION_RESOLVED:
+        'print_preview.NativeLayer.PROVISIONAL_DESTINATION_RESOLVED'
   };
 
   /**
@@ -158,6 +192,24 @@ cr.define('print_preview', function() {
       chrome.send('getPrivetPrinterCapabilities', [destinationId]);
     },
 
+    /**
+     * Requests that extension system dispatches an event requesting the list of
+     * extension managed printers.
+     */
+    startGetExtensionDestinations: function() {
+      chrome.send('getExtensionPrinters');
+    },
+
+    /**
+     * Requests an extension destination's printing capabilities. A
+     * EXTENSION_CAPABILITIES_SET event will be dispatched in response.
+     * @param {string} destinationId The ID of the destination whose
+     *     capabilities are requested.
+     */
+    startGetExtensionDestinationCapabilities: function(destinationId) {
+      chrome.send('getExtensionPrinterCapabilities', [destinationId]);
+    },
+
     /**
      * Requests the destination's printing capabilities. A CAPABILITIES_SET
      * event will be dispatched in response.
@@ -167,6 +219,35 @@ cr.define('print_preview', function() {
       chrome.send('getPrinterCapabilities', [destinationId]);
     },
 
+    /**
+     * Requests Chrome to resolve provisional extension destination by granting
+     * the provider extension access to the printer. Chrome will respond with
+     * the resolved destination properties by calling
+     * {@code onProvisionalPrinterResolved}, or in case of an error
+     * {@code failedToResolveProvisionalPrinter}
+     * @param {string} provisionalDestinationId
+     */
+    grantExtensionPrinterAccess: function(provisionalDestinationId) {
+      chrome.send('grantExtensionPrinterAccess', [provisionalDestinationId]);
+    },
+
+    /**
+     * @param {!print_preview.Destination} destination Destination to print to.
+     * @param {!print_preview.ticket_items.Color} color Color ticket item.
+     * @return {number} Native layer color model.
+     * @private
+     */
+    getNativeColorModel_: function(destination, color) {
+      // For non-local printers native color model is ignored anyway.
+      var option = destination.isLocal ? color.getSelectedOption() : null;
+      var nativeColorModel = parseInt(option ? option.vendor_id : null, 10);
+      if (isNaN(nativeColorModel)) {
+        return color.getValue() ?
+            NativeLayer.ColorMode_.COLOR : NativeLayer.ColorMode_.GRAY;
+      }
+      return nativeColorModel;
+    },
+
     /**
      * Requests that a preview be generated. The following events may be
      * dispatched in response:
@@ -179,7 +260,7 @@ cr.define('print_preview', function() {
      * @param {!print_preview.PrintTicketStore} printTicketStore Used to get the
      *     state of the print ticket.
      * @param {!print_preview.DocumentInfo} documentInfo Document data model.
-     * @param {number} ID of the preview request.
+     * @param {number} requestId ID of the preview request.
      */
     startGetPreview: function(
         destination, printTicketStore, documentInfo, requestId) {
@@ -188,9 +269,9 @@ cr.define('print_preview', function() {
 
       var ticket = {
         'pageRange': printTicketStore.pageRange.getDocumentPageRanges(),
+        'mediaSize': printTicketStore.mediaSize.getValue(),
         'landscape': printTicketStore.landscape.getValue(),
-        'color': printTicketStore.color.getValue() ?
-            NativeLayer.ColorMode_.COLOR : NativeLayer.ColorMode_.GRAY,
+        'color': this.getNativeColorModel_(destination, printTicketStore.color),
         'headerFooterEnabled': printTicketStore.headerFooter.getValue(),
         'marginsType': printTicketStore.marginsType.getValue(),
         'isFirstRequest': requestId == 0,
@@ -202,16 +283,17 @@ cr.define('print_preview', function() {
                 print_preview.Destination.GooglePromotedId.SAVE_AS_PDF,
         'printWithCloudPrint': destination != null && !destination.isLocal,
         'printWithPrivet': destination != null && destination.isPrivet,
+        'printWithExtension': destination != null && destination.isExtension,
         'deviceName': destination == null ? 'foo' : destination.id,
         'generateDraftData': documentInfo.isModifiable,
         'fitToPageEnabled': printTicketStore.fitToPage.getValue(),
-
+        'scaleFactor': printTicketStore.scaling.getValueAsNumber(),
         // NOTE: Even though the following fields don't directly relate to the
         // preview, they still need to be included.
         'duplex': printTicketStore.duplex.getValue() ?
             NativeLayer.DuplexMode.LONG_EDGE : NativeLayer.DuplexMode.SIMPLEX,
-        'copies': printTicketStore.copies.getValueAsNumber(),
-        'collate': printTicketStore.collate.getValue(),
+        'copies': 1,
+        'collate': true,
         'shouldPrintBackgrounds': printTicketStore.cssBackground.getValue(),
         'shouldPrintSelectionOnly': printTicketStore.selectionOnly.getValue()
       };
@@ -247,23 +329,29 @@ cr.define('print_preview', function() {
      * @param {!print_preview.Destination} destination Destination to print to.
      * @param {!print_preview.PrintTicketStore} printTicketStore Used to get the
      *     state of the print ticket.
-     * @param {print_preview.CloudPrintInterface} cloudPrintInterface Interface
+     * @param {cloudprint.CloudPrintInterface} cloudPrintInterface Interface
      *     to Google Cloud Print.
      * @param {!print_preview.DocumentInfo} documentInfo Document data model.
      * @param {boolean=} opt_isOpenPdfInPreview Whether to open the PDF in the
      *     system's preview application.
+     * @param {boolean=} opt_showSystemDialog Whether to open system dialog for
+     *     advanced settings.
      */
     startPrint: function(destination, printTicketStore, cloudPrintInterface,
-                         documentInfo, opt_isOpenPdfInPreview) {
+                         documentInfo, opt_isOpenPdfInPreview,
+                         opt_showSystemDialog) {
       assert(printTicketStore.isTicketValid(),
              'Trying to print when ticket is not valid');
 
+      assert(!opt_showSystemDialog || (cr.isWindows && destination.isLocal),
+             'Implemented for Windows only');
+
       var ticket = {
         'pageRange': printTicketStore.pageRange.getDocumentPageRanges(),
+        'mediaSize': printTicketStore.mediaSize.getValue(),
         'pageCount': printTicketStore.pageRange.getPageNumberSet().size,
         'landscape': printTicketStore.landscape.getValue(),
-        'color': printTicketStore.color.getValue() ?
-            NativeLayer.ColorMode_.COLOR : NativeLayer.ColorMode_.GRAY,
+        'color': this.getNativeColorModel_(destination, printTicketStore.color),
         'headerFooterEnabled': printTicketStore.headerFooter.getValue(),
         'marginsType': printTicketStore.marginsType.getValue(),
         'generateDraftData': true, // TODO(rltoscano): What should this be?
@@ -278,12 +366,15 @@ cr.define('print_preview', function() {
             print_preview.Destination.GooglePromotedId.SAVE_AS_PDF,
         'printWithCloudPrint': !destination.isLocal,
         'printWithPrivet': destination.isPrivet,
+        'printWithExtension': destination.isExtension,
+        'scaleFactor': printTicketStore.scaling.getValueAsNumber(),
         'deviceName': destination.id,
         'isFirstRequest': false,
         'requestID': -1,
         'fitToPageEnabled': printTicketStore.fitToPage.getValue(),
         'pageWidth': documentInfo.pageSize.width,
         'pageHeight': documentInfo.pageSize.height,
+        'showSystemDialog': opt_showSystemDialog
       };
 
       if (!destination.isLocal) {
@@ -307,8 +398,9 @@ cr.define('print_preview', function() {
         };
       }
 
-      if (destination.isPrivet) {
+      if (destination.isPrivet || destination.isExtension) {
         ticket['ticket'] = printTicketStore.createPrintTicket(destination);
+        ticket['capabilities'] = JSON.stringify(destination.capabilities);
       }
 
       if (opt_isOpenPdfInPreview) {
@@ -325,20 +417,14 @@ cr.define('print_preview', function() {
 
     /** Shows the system's native printing dialog. */
     startShowSystemDialog: function() {
+      assert(!cr.isWindows);
       chrome.send('showSystemDialog');
     },
 
-    /** Shows Google Cloud Print's web-based print dialog.
-     * @param {number} pageCount Number of pages to print.
-     */
-    startShowCloudPrintDialog: function(pageCount) {
-      chrome.send('printWithCloudPrintDialog', [pageCount]);
-    },
-
     /** Closes the print preview dialog. */
     startCloseDialog: function() {
       chrome.send('closePrintPreviewDialog');
-      chrome.send('DialogClose');
+      chrome.send('dialogClose');
     },
 
     /** Hide the print preview dialog and allow the native layer to close it. */
@@ -347,11 +433,13 @@ cr.define('print_preview', function() {
     },
 
     /**
-     * Opens the Google Cloud Print sign-in dialog. The DESTINATIONS_RELOAD
-     * event will be dispatched in response.
+     * Opens the Google Cloud Print sign-in tab. The DESTINATIONS_RELOAD event
+     *     will be dispatched in response.
+     * @param {boolean} addAccount Whether to open an 'add a new account' or
+     *     default sign in page.
      */
-    startCloudPrintSignIn: function() {
-      chrome.send('signIn');
+    startCloudPrintSignIn: function(addAccount) {
+      chrome.send('signIn', [addAccount]);
     },
 
     /** Navigates the user to the system printer settings interface. */
@@ -359,9 +447,14 @@ cr.define('print_preview', function() {
       chrome.send('manageLocalPrinters');
     },
 
-    /** Navigates the user to the Google Cloud Print management page. */
-    startManageCloudDestinations: function() {
-      chrome.send('manageCloudPrinters');
+    /**
+     * Navigates the user to the Google Cloud Print management page.
+     * @param {?string} user Email address of the user to open the management
+     *     page for (user must be currently logged in, indeed) or {@code null}
+     *     to open this page for the primary user.
+     */
+    startManageCloudDestinations: function(user) {
+      chrome.send('manageCloudPrinters', [user || '']);
     },
 
     /** Forces browser to open a new tab with the given URL address. */
@@ -383,7 +476,7 @@ cr.define('print_preview', function() {
 
       var nativeInitialSettings = new print_preview.NativeInitialSettings(
           initialSettings['printAutomaticallyInKioskMode'] || false,
-          initialSettings['hidePrintWithSystemDialogLink'] || false,
+          initialSettings['appKioskMode'] || false,
           numberFormatSymbols[0] || ',',
           numberFormatSymbols[1] || '.',
           unitType,
@@ -392,7 +485,8 @@ cr.define('print_preview', function() {
           initialSettings['documentHasSelection'] || false,
           initialSettings['shouldPrintSelectionOnly'] || false,
           initialSettings['printerName'] || null,
-          initialSettings['appState'] || null);
+          initialSettings['appState'] || null,
+          initialSettings['defaultDestinationSelectionRules'] || null);
 
       var initialSettingsSetEvent = new Event(
           NativeLayer.EventType.INITIAL_SETTINGS_SET);
@@ -402,13 +496,15 @@ cr.define('print_preview', function() {
 
     /**
      * Turn on the integration of Cloud Print.
-     * @param {string} cloudPrintURL The URL to use for cloud print servers.
+     * @param {{cloudPrintURL: string, appKioskMode: string}} settings
+     *     cloudPrintUrl: The URL to use for cloud print servers.
      * @private
      */
-    onSetUseCloudPrint_: function(cloudPrintURL) {
+    onSetUseCloudPrint_: function(settings) {
       var cloudPrintEnableEvent = new Event(
           NativeLayer.EventType.CLOUD_PRINT_ENABLE);
-      cloudPrintEnableEvent.baseCloudPrintUrl = cloudPrintURL;
+      cloudPrintEnableEvent.baseCloudPrintUrl = settings['cloudPrintUrl'] || '';
+      cloudPrintEnableEvent.appKioskMode = settings['appKioskMode'] || false;
       this.dispatchEvent(cloudPrintEnableEvent);
     },
 
@@ -440,7 +536,7 @@ cr.define('print_preview', function() {
     /**
      * Called when native layer gets settings information for a requested local
      * destination.
-     * @param {string} printerId printer affected by error.
+     * @param {string} destinationId Printer affected by error.
      * @private
      */
     onFailedToGetPrinterCapabilities_: function(destinationId) {
@@ -455,7 +551,7 @@ cr.define('print_preview', function() {
     /**
      * Called when native layer gets settings information for a requested privet
      * destination.
-     * @param {string} printerId printer affected by error.
+     * @param {string} destinationId Printer affected by error.
      * @private
      */
     onFailedToGetPrivetPrinterCapabilities_: function(destinationId) {
@@ -467,6 +563,21 @@ cr.define('print_preview', function() {
       this.dispatchEvent(getCapsFailEvent);
     },
 
+    /**
+     * Called when native layer fails to get settings information for a
+     * requested extension destination.
+     * @param {string} destinationId Printer affected by error.
+     * @private
+     */
+    onFailedToGetExtensionPrinterCapabilities_: function(destinationId) {
+      var getCapsFailEvent = new Event(
+          NativeLayer.EventType.GET_CAPABILITIES_FAIL);
+      getCapsFailEvent.destinationId = destinationId;
+      getCapsFailEvent.destinationOrigin =
+          print_preview.Destination.Origin.EXTENSION;
+      this.dispatchEvent(getCapsFailEvent);
+    },
+
     /** Reloads the printer list. */
     onReloadPrintersList_: function() {
       cr.dispatchSimpleEvent(this, NativeLayer.EventType.DESTINATIONS_RELOAD);
@@ -550,13 +661,17 @@ cr.define('print_preview', function() {
      * @param {number} pageCount The number of pages.
      * @param {number} previewResponseId The preview request id that resulted in
      *      this response.
+     * @param {number} fitToPageScaling The scaling percentage required to fit
+     *      the document to page, rounded to the nearest integer.
      * @private
      */
-    onDidGetPreviewPageCount_: function(pageCount, previewResponseId) {
+    onDidGetPreviewPageCount_: function(pageCount, previewResponseId,
+                                        fitToPageScaling) {
       var pageCountChangeEvent = new Event(
           NativeLayer.EventType.PAGE_COUNT_READY);
       pageCountChangeEvent.pageCount = pageCount;
       pageCountChangeEvent.previewResponseId = previewResponseId;
+      pageCountChangeEvent.fitToPageScaling = fitToPageScaling;
       this.dispatchEvent(pageCountChangeEvent);
     },
 
@@ -611,15 +726,18 @@ cr.define('print_preview', function() {
     },
 
     /**
-     * Updates the fit to page option state based on the print scaling option of
-     * source pdf. PDF's have an option to enable/disable print scaling. When we
-     * find out that the print scaling option is disabled for the source pdf, we
-     * uncheck the fitToPage_ to page checkbox. This function is called from C++
-     * code.
+     * Updates print preset options from source PDF document.
+     * Called from PrintPreviewUI::OnSetOptionsFromDocument().
+     * @param {{disableScaling: boolean, copies: number,
+     *          duplex: number}} options Specifies
+     *     printing options according to source document presets.
      * @private
      */
-    onPrintScalingDisabledForSourcePDF_: function() {
-      cr.dispatchSimpleEvent(this, NativeLayer.EventType.DISABLE_SCALING);
+    onPrintPresetOptionsFromDocument_: function(options) {
+      var printPresetOptionsEvent = new Event(
+          NativeLayer.EventType.PRINT_PRESET_OPTIONS);
+      printPresetOptionsEvent.optionsFromDocument = options;
+      this.dispatchEvent(printPresetOptionsEvent);
     },
 
     /**
@@ -668,6 +786,117 @@ cr.define('print_preview', function() {
             new Event(NativeLayer.EventType.PRIVET_PRINT_FAILED);
       privetPrintFailedEvent.httpError = http_error;
       this.dispatchEvent(privetPrintFailedEvent);
+    },
+
+    /**
+     * @param {Array<!{extensionId: string,
+     *                 extensionName: string,
+     *                 id: string,
+     *                 name: string,
+     *                 description: (string|undefined),
+     *                 provisional: (boolean|undefined)}>} printers The list
+     *     containing information about printers added by an extension.
+     * @param {boolean} done Whether this is the final list of extension
+     *     managed printers.
+     */
+    onExtensionPrintersAdded_: function(printers, done) {
+      var event = new Event(NativeLayer.EventType.EXTENSION_PRINTERS_ADDED);
+      event.printers = printers;
+      event.done = done;
+      this.dispatchEvent(event);
+    },
+
+    /**
+     * Called when an extension responds to a request for an extension printer
+     * capabilities.
+     * @param {string} printerId The printer's ID.
+     * @param {!Object} capabilities The reported printer capabilities.
+     */
+    onExtensionCapabilitiesSet_: function(printerId,
+                                          capabilities) {
+      var event = new Event(NativeLayer.EventType.EXTENSION_CAPABILITIES_SET);
+      event.printerId = printerId;
+      event.capabilities = capabilities;
+      this.dispatchEvent(event);
+    },
+
+    /**
+     * Called when Chrome reports that attempt to resolve a provisional
+     * destination failed.
+     * @param {string} destinationId The provisional destination ID.
+     * @private
+     */
+    failedToResolveProvisionalDestination_: function(destinationId) {
+      var evt = new Event(
+          NativeLayer.EventType.PROVISIONAL_DESTINATION_RESOLVED);
+      evt.provisionalId = destinationId;
+      evt.destination = null;
+      this.dispatchEvent(evt);
+    },
+
+    /**
+     * Called when Chrome reports that a provisional destination has been
+     * successfully resolved.
+     * Currently used only for extension provided destinations.
+     * @param {string} provisionalDestinationId The provisional destination id.
+     * @param {!{extensionId: string,
+     *           extensionName: string,
+     *           id: string,
+     *           name: string,
+     *           description: (string|undefined)}} destinationInfo The resolved
+     *     destination info.
+     * @private
+     */
+    onProvisionalDestinationResolved_: function(provisionalDestinationId,
+                                                destinationInfo) {
+      var evt = new Event(
+          NativeLayer.EventType.PROVISIONAL_DESTINATION_RESOLVED);
+      evt.provisionalId = provisionalDestinationId;
+      evt.destination = destinationInfo;
+      this.dispatchEvent(evt);
+    },
+
+   /**
+     * Allows for onManipulateSettings to be called
+     * from the native layer.
+     * @private
+     */
+    onEnableManipulateSettingsForTest_: function() {
+      global.onManipulateSettingsForTest =
+          this.onManipulateSettingsForTest_.bind(this);
+    },
+
+    /**
+     * Dispatches an event to print_preview.js to change
+     * a particular setting for print preview.
+     * @param {!print_preview.PreviewSettings} settings Object containing the
+     *     value to be changed and that value should be set to.
+     * @private
+     */
+    onManipulateSettingsForTest_: function(settings) {
+      var manipulateSettingsEvent =
+          new Event(NativeLayer.EventType.MANIPULATE_SETTINGS_FOR_TEST);
+      manipulateSettingsEvent.settings = settings;
+      this.dispatchEvent(manipulateSettingsEvent);
+    },
+
+    /**
+     * Sends a message to the test, letting it know that an
+     * option has been set to a particular value and that the change has
+     * finished modifying the preview area.
+     */
+    previewReadyForTest: function() {
+      if (global.onManipulateSettingsForTest)
+        chrome.send('UILoadedForTest');
+    },
+
+    /**
+     * Notifies the test that the option it tried to change
+     * had not been changed successfully.
+     */
+    previewFailedForTest: function() {
+      if (global.onManipulateSettingsForTest)
+        chrome.send('UIFailedLoadingForTest');
     }
   };
 
@@ -675,6 +904,8 @@ cr.define('print_preview', function() {
    * Initial settings retrieved from the native layer.
    * @param {boolean} isInKioskAutoPrintMode Whether the print preview should be
    *     in auto-print mode.
+   * @param {boolean} isInAppKioskMode Whether the print preview is in App Kiosk
+   *     mode.
    * @param {string} thousandsDelimeter Character delimeter of thousands digits.
    * @param {string} decimalDelimeter Character delimeter of the decimal point.
    * @param {!print_preview.MeasurementSystem.UnitType} unitType Unit type of
@@ -689,11 +920,13 @@ cr.define('print_preview', function() {
    * @param {?string} systemDefaultDestinationId ID of the system default
    *     destination.
    * @param {?string} serializedAppStateStr Serialized app state.
+   * @param {?string} serializedDefaultDestinationSelectionRulesStr Serialized
+   *     default destination selection rules.
    * @constructor
    */
   function NativeInitialSettings(
       isInKioskAutoPrintMode,
-      hidePrintWithSystemDialogLink,
+      isInAppKioskMode,
       thousandsDelimeter,
       decimalDelimeter,
       unitType,
@@ -702,7 +935,8 @@ cr.define('print_preview', function() {
       documentHasSelection,
       selectionOnly,
       systemDefaultDestinationId,
-      serializedAppStateStr) {
+      serializedAppStateStr,
+      serializedDefaultDestinationSelectionRulesStr) {
 
     /**
      * Whether the print preview should be in auto-print mode.
@@ -712,11 +946,11 @@ cr.define('print_preview', function() {
     this.isInKioskAutoPrintMode_ = isInKioskAutoPrintMode;
 
     /**
-     * Whether we should hide the link which shows the system print dialog.
+     * Whether the print preview should switch to App Kiosk mode.
      * @type {boolean}
      * @private
      */
-    this.hidePrintWithSystemDialogLink_ = hidePrintWithSystemDialogLink;
+    this.isInAppKioskMode_ = isInAppKioskMode;
 
     /**
      * Character delimeter of thousands digits.
@@ -780,6 +1014,14 @@ cr.define('print_preview', function() {
      * @private
      */
     this.serializedAppStateStr_ = serializedAppStateStr;
+
+    /**
+     * Serialized default destination selection rules.
+     * @type {?string}
+     * @private
+     */
+    this.serializedDefaultDestinationSelectionRulesStr_ =
+        serializedDefaultDestinationSelectionRulesStr;
   };
 
   NativeInitialSettings.prototype = {
@@ -791,11 +1033,11 @@ cr.define('print_preview', function() {
     },
 
     /**
-     * @return {boolean} Whether we should hide the link which shows the
-           system print dialog.
+     * @return {boolean} Whether the print preview should switch to App Kiosk
+     *     mode.
      */
-    get hidePrintWithSystemDialogLink() {
-      return this.hidePrintWithSystemDialogLink_;
+    get isInAppKioskMode() {
+      return this.isInAppKioskMode_;
     },
 
     /** @return {string} Character delimeter of thousands digits. */
@@ -826,12 +1068,12 @@ cr.define('print_preview', function() {
       return this.documentTitle_;
     },
 
-    /** @return {bool} Whether the document has selection. */
+    /** @return {boolean} Whether the document has selection. */
     get documentHasSelection() {
       return this.documentHasSelection_;
     },
 
-    /** @return {bool} Whether selection only should be printed. */
+    /** @return {boolean} Whether selection only should be printed. */
     get selectionOnly() {
       return this.selectionOnly_;
     },
@@ -844,6 +1086,11 @@ cr.define('print_preview', function() {
     /** @return {?string} Serialized app state. */
     get serializedAppStateStr() {
       return this.serializedAppStateStr_;
+    },
+
+    /** @return {?string} Serialized default destination selection rules. */
+    get serializedDefaultDestinationSelectionRulesStr() {
+      return this.serializedDefaultDestinationSelectionRulesStr_;
     }
   };
 
