@@ -4,15 +4,16 @@
 
 #include <list>
 
-#include "base/prefs/pref_service.h"
 #include "base/run_loop.h"
+#include "chrome/browser/interstitials/chrome_controller_client.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/safe_browsing/safe_browsing_blocking_page.h"
-#include "chrome/browser/safe_browsing/safe_browsing_service.h"
+#include "chrome/browser/safe_browsing/test_safe_browsing_service.h"
 #include "chrome/browser/safe_browsing/threat_details.h"
 #include "chrome/browser/safe_browsing/ui_manager.h"
-#include "chrome/common/pref_names.h"
 #include "chrome/test/base/chrome_render_view_host_test_harness.h"
+#include "components/prefs/pref_service.h"
+#include "components/safe_browsing_db/safe_browsing_prefs.h"
 #include "content/public/browser/interstitial_page.h"
 #include "content/public/browser/navigation_entry.h"
 #include "content/public/browser/render_process_host.h"
@@ -53,26 +54,6 @@ class TestSafeBrowsingBlockingPage : public SafeBrowsingBlockingPage {
   }
 };
 
-class TestSafeBrowsingUIManager: public SafeBrowsingUIManager {
- public:
-  explicit TestSafeBrowsingUIManager(SafeBrowsingService* service)
-      : SafeBrowsingUIManager(service) {
-  }
-
-  void SendSerializedThreatDetails(const std::string& serialized) override {
-    details_.push_back(serialized);
-  }
-
-  std::list<std::string>* GetDetails() {
-    return &details_;
-  }
-
- private:
-  ~TestSafeBrowsingUIManager() override {}
-
-  std::list<std::string> details_;
-};
-
 class TestSafeBrowsingBlockingPageFactory
     : public SafeBrowsingBlockingPageFactory {
  public:
@@ -111,6 +92,7 @@ class SafeBrowsingBlockingPageTest : public ChromeRenderViewHostTestHarness {
     ChromeRenderViewHostTestHarness::SetUp();
     SafeBrowsingBlockingPage::RegisterFactory(&factory_);
     ResetUserResponse();
+    SafeBrowsingUIManager::CreateWhitelistForTesting(web_contents());
   }
 
   void TearDown() override {
@@ -130,21 +112,18 @@ class SafeBrowsingBlockingPageTest : public ChromeRenderViewHostTestHarness {
   }
 
   void Navigate(const char* url,
-                int page_id,
                 int nav_entry_id,
                 bool did_create_new_entry) {
-    NavigateInternal(url, page_id, nav_entry_id, did_create_new_entry, false);
+    NavigateInternal(url, nav_entry_id, did_create_new_entry, false);
   }
 
   void NavigateCrossSite(const char* url,
-                         int page_id,
                          int nav_entry_id,
                          bool did_create_new_entry) {
-    NavigateInternal(url, page_id, nav_entry_id, did_create_new_entry, true);
+    NavigateInternal(url, nav_entry_id, did_create_new_entry, true);
   }
 
   void NavigateInternal(const char* url,
-                        int page_id,
                         int nav_entry_id,
                         bool did_create_new_entry,
                         bool is_cross_site) {
@@ -156,7 +135,7 @@ class SafeBrowsingBlockingPageTest : public ChromeRenderViewHostTestHarness {
             : web_contents()->GetMainFrame();
 
     content::WebContentsTester::For(web_contents())
-        ->TestDidNavigate(render_frame_host, page_id, nav_entry_id,
+        ->TestDidNavigate(render_frame_host, nav_entry_id,
                           did_create_new_entry, GURL(url),
                           ui::PAGE_TRANSITION_TYPED);
   }
@@ -173,7 +152,6 @@ class SafeBrowsingBlockingPageTest : public ChromeRenderViewHostTestHarness {
         web_contents()->GetMainFrame();
     WebContentsTester::For(web_contents())->TestDidNavigate(
         rfh,
-        entry->GetPageID(),
         entry->GetUniqueID(),
         false,
         entry->GetURL(),
@@ -232,15 +210,15 @@ class SafeBrowsingBlockingPageTest : public ChromeRenderViewHostTestHarness {
     resource->callback =
         base::Bind(&SafeBrowsingBlockingPageTest::OnBlockingPageComplete,
                    base::Unretained(this));
-    resource->callback_thread =
-        content::BrowserThread::GetMessageLoopProxyForThread(
-            content::BrowserThread::IO);
+    resource->callback_thread = content::BrowserThread::GetTaskRunnerForThread(
+        content::BrowserThread::IO);
     resource->url = url;
     resource->is_subresource = is_subresource;
     resource->threat_type = SB_THREAT_TYPE_URL_MALWARE;
-    resource->render_process_host_id =
-        web_contents()->GetRenderProcessHost()->GetID();
-    resource->render_frame_id = web_contents()->GetMainFrame()->GetRoutingID();
+    resource->web_contents_getter =
+        SafeBrowsingUIManager::UnsafeResource::GetWebContentsGetter(
+            web_contents()->GetRenderProcessHost()->GetID(),
+            web_contents()->GetMainFrame()->GetRoutingID());
     resource->threat_source = safe_browsing::ThreatSource::LOCAL_PVER3;
   }
 
@@ -254,8 +232,7 @@ TEST_F(SafeBrowsingBlockingPageTest, MalwarePageDontProceed) {
   // Enable malware details.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Start a load.
   controller().LoadURL(GURL(kBadURL), content::Referrer(),
@@ -280,8 +257,8 @@ TEST_F(SafeBrowsingBlockingPageTest, MalwarePageDontProceed) {
   EXPECT_FALSE(controller().GetPendingEntry());
 
   // A report should have been sent.
-  EXPECT_EQ(1u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(1u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Tests showing a blocking page for a malware page and then proceeding.
@@ -289,8 +266,7 @@ TEST_F(SafeBrowsingBlockingPageTest, MalwarePageProceed) {
   // Enable malware reports.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Start a load.
   controller().LoadURL(GURL(kBadURL), content::Referrer(),
@@ -308,13 +284,13 @@ TEST_F(SafeBrowsingBlockingPageTest, MalwarePageProceed) {
   // The interstitial is shown until the navigation commits.
   ASSERT_TRUE(InterstitialPage::GetInterstitialPage(web_contents()));
   // Commit the navigation.
-  Navigate(kBadURL, 1, pending_id, true);
+  Navigate(kBadURL, pending_id, true);
   // The interstitial should be gone now.
   ASSERT_FALSE(InterstitialPage::GetInterstitialPage(web_contents()));
 
   // A report should have been sent.
-  EXPECT_EQ(1u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(1u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Tests showing a blocking page for a page that contains malware subresources
@@ -323,14 +299,13 @@ TEST_F(SafeBrowsingBlockingPageTest, PageWithMalwareResourceDontProceed) {
   // Enable malware reports.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Navigate somewhere.
-  Navigate(kGoogleURL, 1, 0, true);
+  Navigate(kGoogleURL, 0, true);
 
   // Navigate somewhere else.
-  Navigate(kGoodURL, 2, 0, true);
+  Navigate(kGoodURL, 0, true);
 
   // Simulate that page loading a bad-resource triggering an interstitial.
   ShowInterstitial(true, kBadURL);
@@ -349,8 +324,8 @@ TEST_F(SafeBrowsingBlockingPageTest, PageWithMalwareResourceDontProceed) {
   EXPECT_EQ(kGoogleURL, controller().GetActiveEntry()->GetURL().spec());
 
   // A report should have been sent.
-  EXPECT_EQ(1u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(1u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Tests showing a blocking page for a page that contains malware subresources
@@ -359,11 +334,10 @@ TEST_F(SafeBrowsingBlockingPageTest, PageWithMalwareResourceProceed) {
   // Enable malware reports.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Navigate somewhere.
-  Navigate(kGoodURL, 1, 0, true);
+  Navigate(kGoodURL, 0, true);
 
   // Simulate that page loading a bad-resource triggering an interstitial.
   ShowInterstitial(true, kBadURL);
@@ -381,8 +355,8 @@ TEST_F(SafeBrowsingBlockingPageTest, PageWithMalwareResourceProceed) {
   EXPECT_EQ(kGoodURL, controller().GetActiveEntry()->GetURL().spec());
 
   // A report should have been sent.
-  EXPECT_EQ(1u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(1u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Tests showing a blocking page for a page that contains multiple malware
@@ -393,14 +367,13 @@ TEST_F(SafeBrowsingBlockingPageTest,
   // Enable malware reports.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Navigate somewhere.
-  Navigate(kGoogleURL, 1, 0, true);
+  Navigate(kGoogleURL, 0, true);
 
   // Navigate somewhere else.
-  Navigate(kGoodURL, 2, 0, true);
+  Navigate(kGoodURL, 0, true);
 
   // Simulate that page loading a bad-resource triggering an interstitial.
   ShowInterstitial(true, kBadURL);
@@ -424,8 +397,8 @@ TEST_F(SafeBrowsingBlockingPageTest,
   EXPECT_EQ(kGoogleURL, controller().GetActiveEntry()->GetURL().spec());
 
   // A report should have been sent.
-  EXPECT_EQ(1u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(1u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Tests showing a blocking page for a page that contains multiple malware
@@ -435,14 +408,13 @@ TEST_F(SafeBrowsingBlockingPageTest,
   // Enable malware reports.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Navigate somewhere.
-  Navigate(kGoogleURL, 1, 0, true);
+  Navigate(kGoogleURL, 0, true);
 
   // Navigate somewhere else.
-  Navigate(kGoodURL, 2, 0, true);
+  Navigate(kGoodURL, 0, true);
 
   // Simulate that page loading a bad-resource triggering an interstitial.
   ShowInterstitial(true, kBadURL);
@@ -460,8 +432,8 @@ TEST_F(SafeBrowsingBlockingPageTest,
   EXPECT_EQ(OK, user_response());
 
   // A report should have been sent.
-  EXPECT_EQ(1u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(1u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 
   ResetUserResponse();
 
@@ -482,8 +454,8 @@ TEST_F(SafeBrowsingBlockingPageTest,
 
   // No report should have been sent -- we don't create a report the
   // second time.
-  EXPECT_EQ(0u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(0u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Tests showing a blocking page for a page that contains multiple malware
@@ -492,11 +464,10 @@ TEST_F(SafeBrowsingBlockingPageTest, PageWithMultipleMalwareResourceProceed) {
   // Enable malware reports.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Navigate somewhere else.
-  Navigate(kGoodURL, 1, 0, true);
+  Navigate(kGoodURL, 0, true);
 
   // Simulate that page loading a bad-resource triggering an interstitial.
   ShowInterstitial(true, kBadURL);
@@ -514,8 +485,8 @@ TEST_F(SafeBrowsingBlockingPageTest, PageWithMultipleMalwareResourceProceed) {
   EXPECT_EQ(OK, user_response());
 
   // A report should have been sent.
-  EXPECT_EQ(1u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(1u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 
   ResetUserResponse();
 
@@ -534,8 +505,8 @@ TEST_F(SafeBrowsingBlockingPageTest, PageWithMultipleMalwareResourceProceed) {
 
   // No report should have been sent -- we don't create a report the
   // second time.
-  EXPECT_EQ(0u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(0u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Tests showing a blocking page then navigating back and forth to make sure the
@@ -544,11 +515,10 @@ TEST_F(SafeBrowsingBlockingPageTest, NavigatingBackAndForth) {
   // Enable malware reports.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Navigate somewhere.
-  Navigate(kGoodURL, 1, 0, true);
+  Navigate(kGoodURL, 0, true);
 
   // Now navigate to a bad page triggerring an interstitial.
   controller().LoadURL(GURL(kBadURL), content::Referrer(),
@@ -560,7 +530,7 @@ TEST_F(SafeBrowsingBlockingPageTest, NavigatingBackAndForth) {
 
   // Proceed, then navigate back.
   ProceedThroughInterstitial(sb_interstitial);
-  NavigateCrossSite(kBadURL, 2, pending_id, true);  // Commit the navigation.
+  NavigateCrossSite(kBadURL, pending_id, true);  // Commit the navigation.
   GoBack(true);
 
   // We are back on the good page.
@@ -579,15 +549,15 @@ TEST_F(SafeBrowsingBlockingPageTest, NavigatingBackAndForth) {
   // Let's proceed and make sure everything is OK (bug 17627).
   ProceedThroughInterstitial(sb_interstitial);
   // Commit the navigation.
-  NavigateCrossSite(kBadURL, 2, pending_id, false);
+  NavigateCrossSite(kBadURL, pending_id, false);
   sb_interstitial = GetSafeBrowsingBlockingPage();
   ASSERT_FALSE(sb_interstitial);
   ASSERT_EQ(2, controller().GetEntryCount());
   EXPECT_EQ(kBadURL, controller().GetActiveEntry()->GetURL().spec());
 
   // Two reports should have been sent.
-  EXPECT_EQ(2u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(2u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Tests that calling "don't proceed" after "proceed" has been called doesn't
@@ -596,8 +566,7 @@ TEST_F(SafeBrowsingBlockingPageTest, ProceedThenDontProceed) {
   // Enable malware reports.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Start a load.
   controller().LoadURL(GURL(kBadURL), content::Referrer(),
@@ -623,8 +592,8 @@ TEST_F(SafeBrowsingBlockingPageTest, ProceedThenDontProceed) {
   EXPECT_FALSE(GetSafeBrowsingBlockingPage());
 
   // Only one report should have been sent.
-  EXPECT_EQ(1u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(1u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Tests showing a blocking page for a malware page with reports disabled.
@@ -632,8 +601,7 @@ TEST_F(SafeBrowsingBlockingPageTest, MalwareReportsDisabled) {
   // Disable malware reports.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, false);
+  SetExtendedReportingPref(profile->GetPrefs(), false);
 
   // Start a load.
   controller().LoadURL(GURL(kBadURL), content::Referrer(),
@@ -658,8 +626,8 @@ TEST_F(SafeBrowsingBlockingPageTest, MalwareReportsDisabled) {
   EXPECT_FALSE(controller().GetPendingEntry());
 
   // No report should have been sent.
-  EXPECT_EQ(0u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(0u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Test that toggling the checkbox has the anticipated effects.
@@ -667,8 +635,7 @@ TEST_F(SafeBrowsingBlockingPageTest, MalwareReportsToggling) {
   // Disable malware reports.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, false);
+  SetExtendedReportingPref(profile->GetPrefs(), false);
 
   // Start a load.
   controller().LoadURL(GURL(kBadURL), content::Referrer(),
@@ -682,20 +649,17 @@ TEST_F(SafeBrowsingBlockingPageTest, MalwareReportsToggling) {
 
   base::RunLoop().RunUntilIdle();
 
-  EXPECT_FALSE(profile->GetPrefs()->GetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled));
+  EXPECT_FALSE(IsExtendedReportingEnabled(*profile->GetPrefs()));
 
   // Simulate the user check the report agreement checkbox.
-  sb_interstitial->SetReportingPreference(true);
+  sb_interstitial->controller()->SetReportingPreference(true);
 
-  EXPECT_TRUE(profile->GetPrefs()->GetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled));
+  EXPECT_TRUE(IsExtendedReportingEnabled(*profile->GetPrefs()));
 
   // Simulate the user uncheck the report agreement checkbox.
-  sb_interstitial->SetReportingPreference(false);
+  sb_interstitial->controller()->SetReportingPreference(false);
 
-  EXPECT_FALSE(profile->GetPrefs()->GetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled));
+  EXPECT_FALSE(IsExtendedReportingEnabled(*profile->GetPrefs()));
 }
 
 // Test that extended reporting option is not shown on blocking an HTTPS main
@@ -704,8 +668,7 @@ TEST_F(SafeBrowsingBlockingPageTest, ExtendedReportingNotShownOnSecurePage) {
   // Enable malware details.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Start a load.
   controller().LoadURL(GURL(kBadHTTPSURL), content::Referrer(),
@@ -727,8 +690,8 @@ TEST_F(SafeBrowsingBlockingPageTest, ExtendedReportingNotShownOnSecurePage) {
   EXPECT_FALSE(GetSafeBrowsingBlockingPage());
 
   // No report should have been sent.
-  EXPECT_EQ(0u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(0u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Test that extended reporting option is not shown on blocking an HTTPS
@@ -738,8 +701,7 @@ TEST_F(SafeBrowsingBlockingPageTest,
   // Enable malware details.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Commit a load.
   content::WebContentsTester::For(web_contents())
@@ -762,8 +724,8 @@ TEST_F(SafeBrowsingBlockingPageTest,
   EXPECT_FALSE(GetSafeBrowsingBlockingPage());
 
   // No report should have been sent.
-  EXPECT_EQ(0u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(0u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Test that extended reporting option is not shown on blocking an HTTP
@@ -773,8 +735,7 @@ TEST_F(SafeBrowsingBlockingPageTest,
   // Enable malware details.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Commit a load.
   content::WebContentsTester::For(web_contents())
@@ -797,8 +758,8 @@ TEST_F(SafeBrowsingBlockingPageTest,
   EXPECT_FALSE(GetSafeBrowsingBlockingPage());
 
   // No report should have been sent.
-  EXPECT_EQ(0u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(0u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Test that extended reporting option is shown on blocking an HTTPS
@@ -808,8 +769,7 @@ TEST_F(SafeBrowsingBlockingPageTest,
   // Enable malware details.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Commit a load.
   content::WebContentsTester::For(web_contents())
@@ -832,8 +792,8 @@ TEST_F(SafeBrowsingBlockingPageTest,
   EXPECT_FALSE(GetSafeBrowsingBlockingPage());
 
   // A report should have been sent.
-  EXPECT_EQ(1u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(1u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // Test that extended reporting option is not shown on blocking an HTTPS
@@ -844,8 +804,7 @@ TEST_F(SafeBrowsingBlockingPageTest,
   // Enable malware details.
   Profile* profile = Profile::FromBrowserContext(
       web_contents()->GetBrowserContext());
-  profile->GetPrefs()->SetBoolean(
-      prefs::kSafeBrowsingExtendedReportingEnabled, true);
+  SetExtendedReportingPref(profile->GetPrefs(), true);
 
   // Commit a load.
   content::WebContentsTester::For(web_contents())
@@ -875,8 +834,8 @@ TEST_F(SafeBrowsingBlockingPageTest,
   EXPECT_FALSE(GetSafeBrowsingBlockingPage());
 
   // No report should have been sent.
-  EXPECT_EQ(0u, ui_manager_->GetDetails()->size());
-  ui_manager_->GetDetails()->clear();
+  EXPECT_EQ(0u, ui_manager_->GetThreatDetails()->size());
+  ui_manager_->GetThreatDetails()->clear();
 }
 
 // TODO(mattm): Add test for extended reporting not shown or sent in incognito
