@@ -2,51 +2,73 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "chrome/browser/shell_integration.h"
+#include "chrome/browser/shell_integration_win.h"
 
 #include <windows.h>
 #include <shlwapi.h>
 #include <shobjidl.h>
 #include <propkey.h>  // Needs to come after shobjidl.h.
+#include <stddef.h>
+#include <stdint.h>
+
+#include <memory>
+#include <utility>
+#include <vector>
 
 #include "base/bind.h"
+#include "base/callback.h"
 #include "base/command_line.h"
 #include "base/files/file_enumerator.h"
 #include "base/files/file_util.h"
+#include "base/macros.h"
+#include "base/memory/ptr_util.h"
+#include "base/memory/weak_ptr.h"
 #include "base/message_loop/message_loop.h"
+#include "base/metrics/histogram_macros.h"
+#include "base/metrics/user_metrics.h"
+#include "base/metrics/user_metrics_action.h"
 #include "base/path_service.h"
-#include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
+#include "base/time/time.h"
+#include "base/timer/timer.h"
 #include "base/win/registry.h"
 #include "base/win/scoped_comptr.h"
 #include "base/win/scoped_propvariant.h"
 #include "base/win/shortcut.h"
 #include "base/win/windows_version.h"
 #include "chrome/browser/policy/policy_path_parser.h"
+#include "chrome/browser/shell_integration.h"
 #include "chrome/browser/web_applications/web_app.h"
+#include "chrome/browser/win/settings_app_monitor.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/chrome_paths_internal.h"
 #include "chrome/common/chrome_switches.h"
+#include "chrome/common/shell_handler_win.mojom.h"
+#include "chrome/grit/generated_resources.h"
 #include "chrome/installer/setup/setup_util.h"
 #include "chrome/installer/util/browser_distribution.h"
 #include "chrome/installer/util/create_reg_key_work_item.h"
 #include "chrome/installer/util/install_util.h"
+#include "chrome/installer/util/scoped_user_protocol_entry.h"
 #include "chrome/installer/util/set_reg_value_work_item.h"
 #include "chrome/installer/util/shell_util.h"
 #include "chrome/installer/util/util_constants.h"
 #include "chrome/installer/util/work_item.h"
 #include "chrome/installer/util/work_item_list.h"
+#include "components/variations/variations_associated_data.h"
 #include "content/public/browser/browser_thread.h"
+#include "content/public/browser/utility_process_mojo_client.h"
+#include "ui/base/l10n/l10n_util.h"
 
 using content::BrowserThread;
 
+namespace shell_integration {
+
 namespace {
 
-const wchar_t kAppListAppNameSuffix[] = L"AppList";
-
-// Helper function for ShellIntegration::GetAppId to generates profile id
+// Helper function for GetAppId to generates profile id
 // from profile path. "profile_id" is composed of sanitized basenames of
 // user data dir and profile dir joined by a ".".
 base::string16 GetProfileIdFromPath(const base::FilePath& profile_path) {
@@ -83,6 +105,7 @@ base::string16 GetProfileIdFromPath(const base::FilePath& profile_path) {
 }
 
 base::string16 GetAppListAppName() {
+  static const base::char16 kAppListAppNameSuffix[] = L"AppList";
   BrowserDistribution* dist = BrowserDistribution::GetDistribution();
   base::string16 app_name(dist->GetBaseAppId());
   app_name.append(kAppListAppNameSuffix);
@@ -129,53 +152,25 @@ base::string16 GetExpectedAppId(const base::CommandLine& command_line,
   }
   DCHECK(!app_name.empty());
 
-  return ShellIntegration::GetAppModelIdForProfile(app_name, profile_path);
+  return win::GetAppModelIdForProfile(app_name, profile_path);
 }
 
-void MigrateChromiumShortcutsCallback() {
+void MigrateTaskbarPinsCallback() {
   // This should run on the file thread.
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::FILE));
+  DCHECK_CURRENTLY_ON(BrowserThread::FILE);
 
   // Get full path of chrome.
   base::FilePath chrome_exe;
   if (!PathService::Get(base::FILE_EXE, &chrome_exe))
     return;
 
-  // Locations to check for shortcuts migration.
-  static const struct {
-    int location_id;
-    const wchar_t* sub_dir;
-  } kLocations[] = {
-    {
-      base::DIR_TASKBAR_PINS,
-      NULL
-    }, {
-      base::DIR_USER_DESKTOP,
-      NULL
-    }, {
-      base::DIR_START_MENU,
-      NULL
-    }, {
-      base::DIR_APP_DATA,
-      L"Microsoft\\Internet Explorer\\Quick Launch\\User Pinned\\StartMenu"
-    }
-  };
-
-  for (int i = 0; i < arraysize(kLocations); ++i) {
-    base::FilePath path;
-    if (!PathService::Get(kLocations[i].location_id, &path)) {
-      NOTREACHED();
-      continue;
-    }
-
-    if (kLocations[i].sub_dir)
-      path = path.Append(kLocations[i].sub_dir);
-
-    bool check_dual_mode = kLocations[i].location_id == base::DIR_START_MENU ||
-                           kLocations[i].location_id == base::DIR_TASKBAR_PINS;
-    ShellIntegration::MigrateShortcutsInPathInternal(chrome_exe, path,
-                                                     check_dual_mode);
+  base::FilePath pins_path;
+  if (!PathService::Get(base::DIR_TASKBAR_PINS, &pins_path)) {
+    NOTREACHED();
+    return;
   }
+
+  win::MigrateShortcutsInPathInternal(chrome_exe, pins_path);
 }
 
 // Windows 8 introduced a new protocol->executable binding system which cannot
@@ -207,59 +202,318 @@ base::string16 GetAppForProtocolUsingAssocQuery(const GURL& url) {
 }
 
 base::string16 GetAppForProtocolUsingRegistry(const GURL& url) {
-  base::string16 url_spec = base::ASCIIToUTF16(url.possibly_invalid_spec());
-  const base::string16 cmd_key_path =
-      base::ASCIIToUTF16(url.scheme() + "\\shell\\open\\command");
-  base::win::RegKey cmd_key(HKEY_CLASSES_ROOT,
-                            cmd_key_path.c_str(),
-                            KEY_READ);
-  size_t split_offset = url_spec.find(L':');
-  if (split_offset == base::string16::npos)
-    return base::string16();
-  const base::string16 parameters = url_spec.substr(split_offset + 1,
-                                                    url_spec.length() - 1);
-  base::string16 application_to_launch;
-  if (cmd_key.ReadValue(NULL, &application_to_launch) == ERROR_SUCCESS) {
-    base::ReplaceSubstringsAfterOffset(&application_to_launch,
-                                       0,
-                                       L"%1",
-                                       parameters);
-    return application_to_launch;
+  base::string16 command_to_launch;
+
+  // First, try and extract the application's display name.
+  base::string16 cmd_key_path = base::ASCIIToUTF16(url.scheme());
+  base::win::RegKey cmd_key_name(HKEY_CLASSES_ROOT, cmd_key_path.c_str(),
+                                 KEY_READ);
+  if (cmd_key_name.ReadValue(NULL, &command_to_launch) == ERROR_SUCCESS &&
+      !command_to_launch.empty()) {
+    return command_to_launch;
   }
+
+  // Otherwise, parse the command line in the registry, and return the basename
+  // of the program path if it exists.
+  cmd_key_path = base::ASCIIToUTF16(url.scheme() + "\\shell\\open\\command");
+  base::win::RegKey cmd_key_exe(HKEY_CLASSES_ROOT, cmd_key_path.c_str(),
+                                KEY_READ);
+  if (cmd_key_exe.ReadValue(NULL, &command_to_launch) == ERROR_SUCCESS) {
+    base::CommandLine command_line(
+        base::CommandLine::FromString(command_to_launch));
+    return command_line.GetProgram().BaseName().value();
+  }
+
   return base::string16();
 }
 
-
-ShellIntegration::DefaultWebClientState
-    GetDefaultWebClientStateFromShellUtilDefaultState(
-        ShellUtil::DefaultState default_state) {
+DefaultWebClientState GetDefaultWebClientStateFromShellUtilDefaultState(
+    ShellUtil::DefaultState default_state) {
   switch (default_state) {
     case ShellUtil::NOT_DEFAULT:
-      return ShellIntegration::NOT_DEFAULT;
+      return DefaultWebClientState::NOT_DEFAULT;
     case ShellUtil::IS_DEFAULT:
-      return ShellIntegration::IS_DEFAULT;
+      return DefaultWebClientState::IS_DEFAULT;
     default:
       DCHECK_EQ(ShellUtil::UNKNOWN_DEFAULT, default_state);
-      return ShellIntegration::UNKNOWN_DEFAULT;
+      return DefaultWebClientState::UNKNOWN_DEFAULT;
   }
 }
 
+// A recorder of user actions in the Windows Settings app.
+class DefaultBrowserActionRecorder : public win::SettingsAppMonitor::Delegate {
+ public:
+  // Creates the recorder and the monitor that drives it. |continuation| will be
+  // run once the monitor's initialization completes (regardless of success or
+  // failure).
+  explicit DefaultBrowserActionRecorder(base::Closure continuation)
+      : continuation_(std::move(continuation)), settings_app_monitor_(this) {}
+
+ private:
+  // win::SettingsAppMonitor::Delegate:
+  void OnInitialized(HRESULT result) override {
+    UMA_HISTOGRAM_BOOLEAN("SettingsAppMonitor.InitializationResult",
+                          SUCCEEDED(result));
+    if (SUCCEEDED(result)) {
+      base::RecordAction(
+          base::UserMetricsAction("SettingsAppMonitor.Initialized"));
+    }
+    continuation_.Run();
+    continuation_ = base::Closure();
+  }
+
+  void OnAppFocused() override {
+    base::RecordAction(
+        base::UserMetricsAction("SettingsAppMonitor.AppFocused"));
+  }
+
+  void OnChooserInvoked() override {
+    base::RecordAction(
+        base::UserMetricsAction("SettingsAppMonitor.ChooserInvoked"));
+  }
+
+  void OnBrowserChosen(const base::string16& browser_name) override {
+    if (browser_name ==
+        BrowserDistribution::GetDistribution()->GetDisplayName()) {
+      base::RecordAction(
+          base::UserMetricsAction("SettingsAppMonitor.ChromeBrowserChosen"));
+    } else {
+      base::RecordAction(
+          base::UserMetricsAction("SettingsAppMonitor.OtherBrowserChosen"));
+    }
+  }
+
+  void OnPromoFocused() override {
+    base::RecordAction(
+        base::UserMetricsAction("SettingsAppMonitor.PromoFocused"));
+  }
+
+  void OnPromoChoiceMade(bool accept_promo) override {
+    if (accept_promo) {
+      base::RecordAction(
+          base::UserMetricsAction("SettingsAppMonitor.CheckItOut"));
+    } else {
+      base::RecordAction(
+          base::UserMetricsAction("SettingsAppMonitor.SwitchAnyway"));
+    }
+  }
+
+  // A closure to be run once initialization completes.
+  base::Closure continuation_;
+
+  // Monitors user interaction with the Windows Settings app for the sake of
+  // reporting user actions.
+  win::SettingsAppMonitor settings_app_monitor_;
+
+  DISALLOW_COPY_AND_ASSIGN(DefaultBrowserActionRecorder);
+};
+
+// A function bound up in a callback with a DefaultBrowserActionRecorder and
+// a closure to keep the former alive until the time comes to run the latter.
+void OnSettingsAppFinished(
+    std::unique_ptr<DefaultBrowserActionRecorder> recorder,
+    const base::Closure& on_finished_callback) {
+  recorder.reset();
+  on_finished_callback.Run();
+}
+
+// There is no way to make sure the user is done with the system settings, but a
+// signal that the interaction is finished is needed for UMA. A timer of 2
+// minutes is used as a substitute. The registry keys for the protocol
+// association with an app are also monitored to signal the end of the
+// interaction early when it is clear that the user made a choice (e.g. http
+// and https for default browser).
+//
+// This helper class manages both the timer and the registry watchers and makes
+// sure the callback for the end of the settings interaction is only run once.
+// This class also manages its own lifetime.
+class OpenSystemSettingsHelper {
+ public:
+  // Begin the monitoring and will call |on_finished_callback| when done.
+  // Takes in a null-terminated array of |protocols| whose registry keys must be
+  // watched. The array must contain at least one element.
+  static void Begin(const wchar_t* const protocols[],
+                    const base::Closure& on_finished_callback) {
+    DCHECK_CURRENTLY_ON(BrowserThread::FILE);
+
+    delete instance_;
+    instance_ = new OpenSystemSettingsHelper(protocols, on_finished_callback);
+  }
+
+ private:
+  // The reason the settings interaction concluded. Do not modify the ordering
+  // because it is used for UMA.
+  enum ConcludeReason { REGISTRY_WATCHER, TIMEOUT, NUM_CONCLUDE_REASON_TYPES };
+
+  OpenSystemSettingsHelper(const wchar_t* const protocols[],
+                           const base::Closure& on_finished_callback)
+      : scoped_user_protocol_entry_(protocols[0]),
+        on_finished_callback_(on_finished_callback),
+        weak_ptr_factory_(this) {
+    static const wchar_t kUrlAssociationFormat[] =
+        L"SOFTWARE\\Microsoft\\Windows\\Shell\\Associations\\UrlAssociations\\"
+        L"%ls\\UserChoice";
+
+    // Remember the start time.
+    start_time_ = base::TimeTicks::Now();
+
+    for (const wchar_t* const* scan = &protocols[0]; *scan != nullptr; ++scan) {
+      AddRegistryKeyWatcher(
+          base::StringPrintf(kUrlAssociationFormat, *scan).c_str());
+    }
+    // Only the watchers that were succesfully initialized are counted.
+    registry_watcher_count_ = registry_key_watchers_.size();
+
+    timer_.Start(
+        FROM_HERE, base::TimeDelta::FromMinutes(2),
+        base::Bind(&OpenSystemSettingsHelper::ConcludeInteraction,
+                   weak_ptr_factory_.GetWeakPtr(), ConcludeReason::TIMEOUT));
+  }
+
+  // Called when a change is detected on one of the registry keys being watched.
+  // Note: All types of modification to the registry key will trigger this
+  //       function even if the value change is the only one that matters. This
+  //       is good enough for now.
+  void OnRegistryKeyChanged() {
+    DCHECK_CURRENTLY_ON(BrowserThread::FILE);
+
+    // Make sure all the registry watchers have fired.
+    if (--registry_watcher_count_ == 0) {
+      UMA_HISTOGRAM_MEDIUM_TIMES(
+          "DefaultBrowser.SettingsInteraction.RegistryWatcherDuration",
+          base::TimeTicks::Now() - start_time_);
+
+      ConcludeInteraction(ConcludeReason::REGISTRY_WATCHER);
+    }
+  }
+
+  // Ends the monitoring with the system settings. Will call
+  // |on_finished_callback_| and then dispose of this class instance to make
+  // sure the callback won't get called subsequently.
+  void ConcludeInteraction(ConcludeReason conclude_reason) {
+    DCHECK_CURRENTLY_ON(BrowserThread::FILE);
+
+    UMA_HISTOGRAM_ENUMERATION(
+        "DefaultBrowser.SettingsInteraction.ConcludeReason", conclude_reason,
+        NUM_CONCLUDE_REASON_TYPES);
+    on_finished_callback_.Run();
+    delete instance_;
+    instance_ = nullptr;
+  }
+
+  // Helper function to create a registry watcher for a given |key_path|. Do
+  // nothing on initialization failure.
+  void AddRegistryKeyWatcher(const wchar_t* key_path) {
+    auto reg_key = base::MakeUnique<base::win::RegKey>(HKEY_CURRENT_USER,
+                                                       key_path, KEY_NOTIFY);
+
+    if (reg_key->Valid() &&
+        reg_key->StartWatching(
+            base::Bind(&OpenSystemSettingsHelper::OnRegistryKeyChanged,
+                       weak_ptr_factory_.GetWeakPtr()))) {
+      registry_key_watchers_.push_back(std::move(reg_key));
+    }
+  }
+
+  // Used to make sure only one instance is alive at the same time.
+  static OpenSystemSettingsHelper* instance_;
+
+  // This is needed to make sure that Windows displays an entry for the protocol
+  // inside the "Choose default apps by protocol" settings page.
+  ScopedUserProtocolEntry scoped_user_protocol_entry_;
+
+  // The function to call when the interaction with the system settings is
+  // finished.
+  base::Closure on_finished_callback_;
+
+  // The number of time the registry key watchers must fire.
+  int registry_watcher_count_ = 0;
+
+  // There can be multiple registry key watchers as some settings modify
+  // multiple protocol associations. e.g. Changing the default browser modifies
+  // the http and https associations.
+  std::vector<std::unique_ptr<base::win::RegKey>> registry_key_watchers_;
+
+  base::OneShotTimer timer_;
+
+  // Records the time it takes for the final registry watcher to get signaled.
+  base::TimeTicks start_time_;
+
+  // Weak ptrs are used to bind this class to the callbacks of the timer and the
+  // registry watcher. This makes it possible to self-delete after one of the
+  // callbacks is executed to cancel the remaining ones.
+  base::WeakPtrFactory<OpenSystemSettingsHelper> weak_ptr_factory_;
+
+  DISALLOW_COPY_AND_ASSIGN(OpenSystemSettingsHelper);
+};
+
+OpenSystemSettingsHelper* OpenSystemSettingsHelper::instance_ = nullptr;
+
+// Helper class to determine if Chrome is pinned to the taskbar. Hides the
+// complexity of managing the lifetime of a UtilityProcessMojoClient.
+class IsPinnedToTaskbarHelper {
+ public:
+  using ResultCallback = win::IsPinnedToTaskbarCallback;
+  using ErrorCallback = win::ConnectionErrorCallback;
+  static void GetState(const ErrorCallback& error_callback,
+                       const ResultCallback& result_callback);
+
+ private:
+  IsPinnedToTaskbarHelper(const ErrorCallback& error_callback,
+                          const ResultCallback& result_callback);
+
+  void OnConnectionError();
+  void OnIsPinnedToTaskbarResult(bool succeeded, bool is_pinned_to_taskbar);
+
+  content::UtilityProcessMojoClient<mojom::ShellHandler> shell_handler_;
+
+  ErrorCallback error_callback_;
+  ResultCallback result_callback_;
+
+  DISALLOW_COPY_AND_ASSIGN(IsPinnedToTaskbarHelper);
+};
+
+// static
+void IsPinnedToTaskbarHelper::GetState(const ErrorCallback& error_callback,
+                                       const ResultCallback& result_callback) {
+  // Self-deleting when the ShellHandler completes.
+  new IsPinnedToTaskbarHelper(error_callback, result_callback);
+}
+
+IsPinnedToTaskbarHelper::IsPinnedToTaskbarHelper(
+    const ErrorCallback& error_callback,
+    const ResultCallback& result_callback)
+    : shell_handler_(
+          l10n_util::GetStringUTF16(IDS_UTILITY_PROCESS_SHELL_HANDLER_NAME)),
+      error_callback_(error_callback),
+      result_callback_(result_callback) {
+  // |shell_handler_| owns the callbacks and is guaranteed to be destroyed
+  // before |this|, therefore making base::Unretained() safe to use.
+  shell_handler_.set_error_callback(base::Bind(
+      &IsPinnedToTaskbarHelper::OnConnectionError, base::Unretained(this)));
+  shell_handler_.set_disable_sandbox();
+  shell_handler_.Start();
+
+  shell_handler_.service()->IsPinnedToTaskbar(
+      base::Bind(&IsPinnedToTaskbarHelper::OnIsPinnedToTaskbarResult,
+                 base::Unretained(this)));
+}
+
+void IsPinnedToTaskbarHelper::OnConnectionError() {
+  error_callback_.Run();
+  delete this;
+}
+
+void IsPinnedToTaskbarHelper::OnIsPinnedToTaskbarResult(
+    bool succeeded,
+    bool is_pinned_to_taskbar) {
+  result_callback_.Run(succeeded, is_pinned_to_taskbar);
+  delete this;
+}
+
 }  // namespace
 
-ShellIntegration::DefaultWebClientSetPermission
-    ShellIntegration::CanSetAsDefaultBrowser() {
-  BrowserDistribution* distribution = BrowserDistribution::GetDistribution();
-  if (distribution->GetDefaultBrowserControlPolicy() !=
-          BrowserDistribution::DEFAULT_BROWSER_FULL_CONTROL)
-    return SET_DEFAULT_NOT_ALLOWED;
-
-  if (ShellUtil::CanMakeChromeDefaultUnattended())
-    return SET_DEFAULT_UNATTENDED;
-  else
-    return SET_DEFAULT_INTERACTIVE;
-}
-
-bool ShellIntegration::SetAsDefaultBrowser() {
+bool SetAsDefaultBrowser() {
   base::FilePath chrome_exe;
   if (!PathService::Get(base::FILE_EXE, &chrome_exe)) {
     LOG(ERROR) << "Error getting app exe path";
@@ -278,7 +532,7 @@ bool ShellIntegration::SetAsDefaultBrowser() {
   return true;
 }
 
-bool ShellIntegration::SetAsDefaultProtocolClient(const std::string& protocol) {
+bool SetAsDefaultProtocolClient(const std::string& protocol) {
   if (protocol.empty())
     return false;
 
@@ -301,7 +555,81 @@ bool ShellIntegration::SetAsDefaultProtocolClient(const std::string& protocol) {
   return true;
 }
 
-bool ShellIntegration::SetAsDefaultBrowserInteractive() {
+DefaultWebClientSetPermission GetDefaultWebClientSetPermission() {
+  BrowserDistribution* distribution = BrowserDistribution::GetDistribution();
+  if (distribution->GetDefaultBrowserControlPolicy() !=
+          BrowserDistribution::DEFAULT_BROWSER_FULL_CONTROL)
+    return SET_DEFAULT_NOT_ALLOWED;
+  if (ShellUtil::CanMakeChromeDefaultUnattended())
+    return SET_DEFAULT_UNATTENDED;
+  // Windows 8 and 10 both introduced a new way to set the default web client
+  // which require user interaction.
+  return SET_DEFAULT_INTERACTIVE;
+}
+
+bool IsElevationNeededForSettingDefaultProtocolClient() {
+  return base::win::GetVersion() < base::win::VERSION_WIN8;
+}
+
+base::string16 GetApplicationNameForProtocol(const GURL& url) {
+  base::string16 application_name;
+  // Windows 8 or above has a new protocol association query.
+  if (base::win::GetVersion() >= base::win::VERSION_WIN8) {
+    application_name = GetAppForProtocolUsingAssocQuery(url);
+    if (!application_name.empty())
+      return application_name;
+  }
+
+  return GetAppForProtocolUsingRegistry(url);
+}
+
+DefaultWebClientState GetDefaultBrowser() {
+  return GetDefaultWebClientStateFromShellUtilDefaultState(
+      ShellUtil::GetChromeDefaultState());
+}
+
+// There is no reliable way to say which browser is default on a machine (each
+// browser can have some of the protocols/shortcuts). So we look for only HTTP
+// protocol handler. Even this handler is located at different places in
+// registry on XP and Vista:
+// - HKCR\http\shell\open\command (XP)
+// - HKCU\Software\Microsoft\Windows\Shell\Associations\UrlAssociations\
+//   http\UserChoice (Vista)
+// This method checks if Firefox is default browser by checking these
+// locations and returns true if Firefox traces are found there. In case of
+// error (or if Firefox is not found)it returns the default value which
+// is false.
+bool IsFirefoxDefaultBrowser() {
+  bool ff_default = false;
+  if (base::win::GetVersion() >= base::win::VERSION_VISTA) {
+    base::string16 app_cmd;
+    base::win::RegKey key(HKEY_CURRENT_USER,
+                          ShellUtil::kRegVistaUrlPrefs, KEY_READ);
+    if (key.Valid() && (key.ReadValue(L"Progid", &app_cmd) == ERROR_SUCCESS) &&
+        app_cmd == L"FirefoxURL")
+      ff_default = true;
+  } else {
+    base::string16 key_path(L"http");
+    key_path.append(ShellUtil::kRegShellOpen);
+    base::win::RegKey key(HKEY_CLASSES_ROOT, key_path.c_str(), KEY_READ);
+    base::string16 app_cmd;
+    if (key.Valid() && (key.ReadValue(L"", &app_cmd) == ERROR_SUCCESS) &&
+        base::string16::npos !=
+        base::ToLowerASCII(app_cmd).find(L"firefox"))
+      ff_default = true;
+  }
+  return ff_default;
+}
+
+DefaultWebClientState IsDefaultProtocolClient(const std::string& protocol) {
+  return GetDefaultWebClientStateFromShellUtilDefaultState(
+      ShellUtil::GetChromeDefaultProtocolClientState(
+          base::UTF8ToUTF16(protocol)));
+}
+
+namespace win {
+
+bool SetAsDefaultBrowserUsingIntentPicker() {
   base::FilePath chrome_exe;
   if (!PathService::Get(base::FILE_EXE, &chrome_exe)) {
     NOTREACHED() << "Error getting app exe path";
@@ -318,8 +646,35 @@ bool ShellIntegration::SetAsDefaultBrowserInteractive() {
   return true;
 }
 
-bool ShellIntegration::SetAsDefaultProtocolClientInteractive(
-    const std::string& protocol) {
+void SetAsDefaultBrowserUsingSystemSettings(
+    const base::Closure& on_finished_callback) {
+  DCHECK_CURRENTLY_ON(BrowserThread::FILE);
+
+  base::FilePath chrome_exe;
+  if (!PathService::Get(base::FILE_EXE, &chrome_exe)) {
+    NOTREACHED() << "Error getting app exe path";
+    on_finished_callback.Run();
+    return;
+  }
+
+  // Create an action recorder that will open the settings app once it has
+  // initialized.
+  std::unique_ptr<DefaultBrowserActionRecorder> recorder(
+      new DefaultBrowserActionRecorder(base::Bind(
+          base::IgnoreResult(&ShellUtil::ShowMakeChromeDefaultSystemUI),
+          base::Unretained(BrowserDistribution::GetDistribution()),
+          chrome_exe)));
+
+  // The helper manages its own lifetime. Bind the action recorder
+  // into the finished callback to keep it alive throughout the
+  // interaction.
+  static const wchar_t* const kProtocols[] = {L"http", L"https", nullptr};
+  OpenSystemSettingsHelper::Begin(
+      kProtocols, base::Bind(&OnSettingsAppFinished, base::Passed(&recorder),
+                             on_finished_callback));
+}
+
+bool SetAsDefaultProtocolClientUsingIntentPicker(const std::string& protocol) {
   base::FilePath chrome_exe;
   if (!PathService::Get(base::FILE_EXE, &chrome_exe)) {
     NOTREACHED() << "Error getting app exe path";
@@ -338,63 +693,30 @@ bool ShellIntegration::SetAsDefaultProtocolClientInteractive(
   return true;
 }
 
-ShellIntegration::DefaultWebClientState ShellIntegration::GetDefaultBrowser() {
-  return GetDefaultWebClientStateFromShellUtilDefaultState(
-      ShellUtil::GetChromeDefaultState());
-}
+void SetAsDefaultProtocolClientUsingSystemSettings(
+    const std::string& protocol,
+    const base::Closure& on_finished_callback) {
+  DCHECK_CURRENTLY_ON(BrowserThread::FILE);
 
-ShellIntegration::DefaultWebClientState
-    ShellIntegration::IsDefaultProtocolClient(const std::string& protocol) {
-  return GetDefaultWebClientStateFromShellUtilDefaultState(
-      ShellUtil::GetChromeDefaultProtocolClientState(
-          base::UTF8ToUTF16(protocol)));
-}
-
-base::string16 ShellIntegration::GetApplicationNameForProtocol(
-    const GURL& url) {
-  // Windows 8 or above requires a new protocol association query.
-  if (base::win::GetVersion() >= base::win::VERSION_WIN8)
-    return GetAppForProtocolUsingAssocQuery(url);
-  else
-    return GetAppForProtocolUsingRegistry(url);
-}
-
-// There is no reliable way to say which browser is default on a machine (each
-// browser can have some of the protocols/shortcuts). So we look for only HTTP
-// protocol handler. Even this handler is located at different places in
-// registry on XP and Vista:
-// - HKCR\http\shell\open\command (XP)
-// - HKCU\Software\Microsoft\Windows\Shell\Associations\UrlAssociations\
-//   http\UserChoice (Vista)
-// This method checks if Firefox is defualt browser by checking these
-// locations and returns true if Firefox traces are found there. In case of
-// error (or if Firefox is not found)it returns the default value which
-// is false.
-bool ShellIntegration::IsFirefoxDefaultBrowser() {
-  bool ff_default = false;
-  if (base::win::GetVersion() >= base::win::VERSION_VISTA) {
-    base::string16 app_cmd;
-    base::win::RegKey key(HKEY_CURRENT_USER,
-                          ShellUtil::kRegVistaUrlPrefs, KEY_READ);
-    if (key.Valid() && (key.ReadValue(L"Progid", &app_cmd) == ERROR_SUCCESS) &&
-        app_cmd == L"FirefoxURL")
-      ff_default = true;
-  } else {
-    base::string16 key_path(L"http");
-    key_path.append(ShellUtil::kRegShellOpen);
-    base::win::RegKey key(HKEY_CLASSES_ROOT, key_path.c_str(), KEY_READ);
-    base::string16 app_cmd;
-    if (key.Valid() && (key.ReadValue(L"", &app_cmd) == ERROR_SUCCESS) &&
-        base::string16::npos !=
-        base::StringToLowerASCII(app_cmd).find(L"firefox"))
-      ff_default = true;
+  base::FilePath chrome_exe;
+  if (!PathService::Get(base::FILE_EXE, &chrome_exe)) {
+    NOTREACHED() << "Error getting app exe path";
+    on_finished_callback.Run();
+    return;
   }
-  return ff_default;
+
+  // The helper manages its own lifetime.
+  base::string16 wprotocol(base::UTF8ToUTF16(protocol));
+  const wchar_t* const kProtocols[] = {wprotocol.c_str(), nullptr};
+  OpenSystemSettingsHelper::Begin(kProtocols, on_finished_callback);
+
+  BrowserDistribution* dist = BrowserDistribution::GetDistribution();
+  ShellUtil::ShowMakeChromeDefaultProtocolClientSystemUI(dist, chrome_exe,
+                                                         wprotocol);
 }
 
-base::string16 ShellIntegration::GetAppModelIdForProfile(
-    const base::string16& app_name,
-    const base::FilePath& profile_path) {
+base::string16 GetAppModelIdForProfile(const base::string16& app_name,
+                                       const base::FilePath& profile_path) {
   std::vector<base::string16> components;
   components.push_back(app_name);
   const base::string16 profile_id(GetProfileIdFromPath(profile_path));
@@ -403,7 +725,7 @@ base::string16 ShellIntegration::GetAppModelIdForProfile(
   return ShellUtil::BuildAppModelId(components);
 }
 
-base::string16 ShellIntegration::GetChromiumModelIdForProfile(
+base::string16 GetChromiumModelIdForProfile(
     const base::FilePath& profile_path) {
   BrowserDistribution* dist = BrowserDistribution::GetDistribution();
   base::FilePath chrome_exe;
@@ -417,30 +739,28 @@ base::string16 ShellIntegration::GetChromiumModelIdForProfile(
       profile_path);
 }
 
-base::string16 ShellIntegration::GetAppListAppModelIdForProfile(
-    const base::FilePath& profile_path) {
-  return ShellIntegration::GetAppModelIdForProfile(
-      GetAppListAppName(), profile_path);
-}
-
-void ShellIntegration::MigrateChromiumShortcuts() {
+void MigrateTaskbarPins() {
   if (base::win::GetVersion() < base::win::VERSION_WIN7)
     return;
 
   // This needs to happen eventually (e.g. so that the appid is fixed and the
   // run-time Chrome icon is merged with the taskbar shortcut), but this is not
   // urgent and shouldn't delay Chrome startup.
-  static const int64 kMigrateChromiumShortcutsDelaySeconds = 15;
+  static const int64_t kMigrateTaskbarPinsDelaySeconds = 15;
   BrowserThread::PostDelayedTask(
       BrowserThread::FILE, FROM_HERE,
-      base::Bind(&MigrateChromiumShortcutsCallback),
-      base::TimeDelta::FromSeconds(kMigrateChromiumShortcutsDelaySeconds));
+      base::Bind(&MigrateTaskbarPinsCallback),
+      base::TimeDelta::FromSeconds(kMigrateTaskbarPinsDelaySeconds));
 }
 
-int ShellIntegration::MigrateShortcutsInPathInternal(
-    const base::FilePath& chrome_exe,
-    const base::FilePath& path,
-    bool check_dual_mode) {
+void GetIsPinnedToTaskbarState(
+    const ConnectionErrorCallback& on_error_callback,
+    const IsPinnedToTaskbarCallback& result_callback) {
+  IsPinnedToTaskbarHelper::GetState(on_error_callback, result_callback);
+}
+
+int MigrateShortcutsInPathInternal(const base::FilePath& chrome_exe,
+                                   const base::FilePath& path) {
   DCHECK(base::win::GetVersion() >= base::win::VERSION_WIN7);
 
   // Enumerate all pinned shortcuts in the given path directly.
@@ -513,36 +833,23 @@ int ShellIntegration::MigrateShortcutsInPathInternal(
       }
     }
 
-    // Only set dual mode if the expected app id is the default app id.
+    // Clear dual_mode property from any shortcuts that previously had it (it
+    // was only ever installed on shortcuts with the
+    // |default_chromium_model_id|).
     BrowserDistribution* dist = BrowserDistribution::GetDistribution();
     base::string16 default_chromium_model_id(
         ShellUtil::GetBrowserModelId(dist, is_per_user_install));
-    if (check_dual_mode && expected_app_id == default_chromium_model_id) {
-      const bool dual_mode_desired =
-          InstallUtil::ShouldInstallMetroProperties();
+    if (expected_app_id == default_chromium_model_id) {
       propvariant.Reset();
       if (property_store->GetValue(PKEY_AppUserModel_IsDualMode,
                                    propvariant.Receive()) != S_OK) {
         // When in doubt, prefer to not update the shortcut.
         NOTREACHED();
         continue;
-      } else {
-        switch (propvariant.get().vt) {
-          case VT_EMPTY:
-            // If dual_mode is not set at all, make sure it gets set to true if
-            // desired.
-            if (dual_mode_desired)
-              updated_properties.set_dual_mode(true);
-            break;
-          case VT_BOOL:
-            // Make sure dual_mode is set as desired.
-            if ((!!propvariant.get().boolVal) != dual_mode_desired)
-              updated_properties.set_dual_mode(dual_mode_desired);
-            break;
-          default:
-            NOTREACHED();
-            continue;
-        }
+      }
+      if (propvariant.get().vt == VT_BOOL &&
+                 !!propvariant.get().boolVal) {
+        updated_properties.set_dual_mode(false);
       }
     }
 
@@ -560,31 +867,34 @@ int ShellIntegration::MigrateShortcutsInPathInternal(
   return shortcuts_migrated;
 }
 
-base::FilePath ShellIntegration::GetStartMenuShortcut(
-    const base::FilePath& chrome_exe) {
+base::FilePath GetStartMenuShortcut(const base::FilePath& chrome_exe) {
   static const int kFolderIds[] = {
     base::DIR_COMMON_START_MENU,
     base::DIR_START_MENU,
   };
   BrowserDistribution* dist = BrowserDistribution::GetDistribution();
-  base::string16 shortcut_name(
-      dist->GetShortcutName(BrowserDistribution::SHORTCUT_CHROME));
+  const base::string16 shortcut_name(dist->GetShortcutName() +
+                                     installer::kLnkExt);
+  base::FilePath programs_folder;
   base::FilePath shortcut;
 
   // Check both the common and the per-user Start Menu folders for system-level
   // installs.
   size_t folder = InstallUtil::IsPerUserInstall(chrome_exe) ? 1 : 0;
   for (; folder < arraysize(kFolderIds); ++folder) {
-    if (!PathService::Get(kFolderIds[folder], &shortcut)) {
+    if (!PathService::Get(kFolderIds[folder], &programs_folder)) {
       NOTREACHED();
       continue;
     }
 
-    shortcut = shortcut.Append(shortcut_name).Append(shortcut_name +
-                                                     installer::kLnkExt);
+    shortcut = programs_folder.Append(shortcut_name);
     if (base::PathExists(shortcut))
       return shortcut;
   }
 
   return base::FilePath();
 }
+
+}  // namespace win
+
+}  // namespace shell_integration
