@@ -5,12 +5,13 @@
 #include "chrome/browser/signin/easy_unlock_service.h"
 
 #include <stddef.h>
+
 #include <map>
+#include <memory>
 #include <string>
 #include <utility>
 
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/run_loop.h"
 #include "base/values.h"
 #include "chrome/browser/chromeos/login/users/mock_user_manager.h"
@@ -26,7 +27,7 @@
 #include "chromeos/dbus/fake_power_manager_client.h"
 #include "components/signin/core/account_id/account_id.h"
 #include "components/signin/core/browser/signin_manager_base.h"
-#include "components/syncable_prefs/testing_pref_service_syncable.h"
+#include "components/sync_preferences/testing_pref_service_syncable.h"
 #include "content/public/test/test_browser_thread_bundle.h"
 #include "device/bluetooth/bluetooth_adapter_factory.h"
 #include "device/bluetooth/test/mock_bluetooth_adapter.h"
@@ -143,10 +144,10 @@ class TestAppManagerFactory {
 
   // Creates a TestAppManager for the provided browser context. If a
   // TestAppManager was already created for the context, returns NULL.
-  scoped_ptr<TestAppManager> Create(content::BrowserContext* context) {
+  std::unique_ptr<TestAppManager> Create(content::BrowserContext* context) {
     if (Find(context))
-      return scoped_ptr<TestAppManager>();
-    scoped_ptr<TestAppManager> app_manager(new TestAppManager());
+      return std::unique_ptr<TestAppManager>();
+    std::unique_ptr<TestAppManager> app_manager(new TestAppManager());
     mapping_[context] = app_manager.get();
     return app_manager;
   }
@@ -176,19 +177,19 @@ TestAppManagerFactory* app_manager_factory = NULL;
 
 // EasyUnlockService factory function injected into testing profiles.
 // It creates an EasyUnlockService with test AppManager.
-scoped_ptr<KeyedService> CreateEasyUnlockServiceForTest(
+std::unique_ptr<KeyedService> CreateEasyUnlockServiceForTest(
     content::BrowserContext* context) {
   EXPECT_TRUE(app_manager_factory);
   if (!app_manager_factory)
     return nullptr;
 
-  scoped_ptr<EasyUnlockAppManager> app_manager =
+  std::unique_ptr<EasyUnlockAppManager> app_manager =
       app_manager_factory->Create(context);
   EXPECT_TRUE(app_manager.get());
   if (!app_manager.get())
     return nullptr;
 
-  scoped_ptr<EasyUnlockServiceRegular> service(
+  std::unique_ptr<EasyUnlockServiceRegular> service(
       new EasyUnlockServiceRegular(Profile::FromBrowserContext(context)));
   service->Initialize(std::move(app_manager));
   return std::move(service);
@@ -212,11 +213,11 @@ class EasyUnlockServiceTest : public testing::Test {
         .WillRepeatedly(testing::Invoke(
             this, &EasyUnlockServiceTest::is_bluetooth_adapter_present));
 
-    scoped_ptr<DBusThreadManagerSetter> dbus_setter =
+    std::unique_ptr<DBusThreadManagerSetter> dbus_setter =
         chromeos::DBusThreadManager::GetSetterForTesting();
     power_manager_client_ = new FakePowerManagerClient;
     dbus_setter->SetPowerManagerClient(
-        scoped_ptr<PowerManagerClient>(power_manager_client_));
+        std::unique_ptr<PowerManagerClient>(power_manager_client_));
 
     ON_CALL(*mock_user_manager_, Shutdown()).WillByDefault(Return());
     ON_CALL(*mock_user_manager_, IsLoggedInAsUserWithGaiaAccount())
@@ -276,7 +277,7 @@ class EasyUnlockServiceTest : public testing::Test {
 
  private:
   // Sets up a test profile with a user id.
-  void SetUpProfile(scoped_ptr<TestingProfile>* profile,
+  void SetUpProfile(std::unique_ptr<TestingProfile>* profile,
                     const AccountId& account_id) {
     ASSERT_TRUE(profile);
     ASSERT_FALSE(profile->get());
@@ -296,8 +297,8 @@ class EasyUnlockServiceTest : public testing::Test {
   }
 
  protected:
-  scoped_ptr<TestingProfile> profile_;
-  scoped_ptr<TestingProfile> secondary_profile_;
+  std::unique_ptr<TestingProfile> profile_;
+  std::unique_ptr<TestingProfile> secondary_profile_;
   chromeos::MockUserManager* mock_user_manager_;
 
  private:
