@@ -10,23 +10,27 @@
 #include "base/callback.h"
 #include "base/logging.h"
 #include "base/macros.h"
-#include "base/prefs/pref_service.h"
 #include "base/run_loop.h"
 #include "base/strings/stringprintf.h"
 #include "chrome/browser/interstitials/security_interstitial_page_test_utils.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ssl/cert_report_helper.h"
 #include "chrome/browser/ssl/certificate_reporting_test_utils.h"
+#include "chrome/browser/ssl/security_state_tab_helper.h"
 #include "chrome/browser/ssl/ssl_cert_reporter.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/test/base/in_process_browser_test.h"
 #include "components/captive_portal/captive_portal_detector.h"
+#include "components/prefs/pref_service.h"
+#include "components/security_state/core/security_state.h"
 #include "content/public/browser/interstitial_page.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/test/browser_test_utils.h"
 #include "net/cert/x509_certificate.h"
+#include "net/test/cert_test_util.h"
+#include "net/test/test_data_directory.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "url/gurl.h"
 
@@ -63,9 +67,10 @@ class CaptivePortalBlockingPageForTesting : public CaptivePortalBlockingPage {
       content::WebContents* web_contents,
       const GURL& request_url,
       const GURL& login_url,
-      scoped_ptr<SSLCertReporter> ssl_cert_reporter,
+      std::unique_ptr<SSLCertReporter> ssl_cert_reporter,
       const net::SSLInfo& ssl_info,
-      const base::Callback<void(bool)>& callback,
+      const base::Callback<void(content::CertificateRequestResultType)>&
+          callback,
       bool is_wifi,
       const std::string& wifi_ssid)
       : CaptivePortalBlockingPage(web_contents,
@@ -85,7 +90,8 @@ class CaptivePortalBlockingPageForTesting : public CaptivePortalBlockingPage {
 };
 
 class CaptivePortalBlockingPageTest
-    : public certificate_reporting_test_utils::CertificateReportingTest {
+    : public certificate_reporting_test_utils::CertificateReportingTest,
+      public InProcessBrowserTest {
  public:
   CaptivePortalBlockingPageTest() {}
 
@@ -95,7 +101,7 @@ class CaptivePortalBlockingPageTest
                         ExpectWiFi expect_wifi,
                         ExpectWiFiSSID expect_wifi_ssid,
                         ExpectLoginURL expect_login_url,
-                        scoped_ptr<SSLCertReporter> ssl_cert_reporter,
+                        std::unique_ptr<SSLCertReporter> ssl_cert_reporter,
                         const std::string& expected_login_hostname);
 
   void TestInterstitial(bool is_wifi_connection,
@@ -111,7 +117,7 @@ class CaptivePortalBlockingPageTest
                         ExpectWiFi expect_wifi,
                         ExpectWiFiSSID expect_wifi_ssid,
                         ExpectLoginURL expect_login_url,
-                        scoped_ptr<SSLCertReporter> ssl_cert_reporter);
+                        std::unique_ptr<SSLCertReporter> ssl_cert_reporter);
 
   void TestCertReporting(certificate_reporting_test_utils::OptIn opt_in);
 
@@ -126,20 +132,22 @@ void CaptivePortalBlockingPageTest::TestInterstitial(
     ExpectWiFi expect_wifi,
     ExpectWiFiSSID expect_wifi_ssid,
     ExpectLoginURL expect_login_url,
-    scoped_ptr<SSLCertReporter> ssl_cert_reporter,
+    std::unique_ptr<SSLCertReporter> ssl_cert_reporter,
     const std::string& expected_login_hostname) {
   content::WebContents* contents =
       browser()->tab_strip_model()->GetActiveWebContents();
   DCHECK(contents);
   net::SSLInfo ssl_info;
-  ssl_info.cert = new net::X509Certificate(
-      login_url.host(), "CA", base::Time::Max(), base::Time::Max());
+  ssl_info.cert =
+      net::ImportCertFromFile(net::GetTestCertsDirectory(), "ok_cert.pem");
+  ssl_info.cert_status = net::CERT_STATUS_COMMON_NAME_INVALID;
   // Blocking page is owned by the interstitial.
   CaptivePortalBlockingPage* blocking_page =
       new CaptivePortalBlockingPageForTesting(
           contents, GURL(kBrokenSSL), login_url, std::move(ssl_cert_reporter),
-          ssl_info, base::Callback<void(bool)>(), is_wifi_connection,
-          wifi_ssid);
+          ssl_info,
+          base::Callback<void(content::CertificateRequestResultType)>(),
+          is_wifi_connection, wifi_ssid);
   blocking_page->Show();
 
   WaitForInterstitialAttach(contents);
@@ -159,6 +167,14 @@ void CaptivePortalBlockingPageTest::TestInterstitial(
   EXPECT_EQ(expect_login_url == EXPECT_LOGIN_URL_NO,
             IsInterstitialDisplayingText(contents->GetInterstitialPage(),
                                          kGenericLoginURLText));
+
+  // Check that a red/dangerous lock icon is showing on the interstitial.
+  SecurityStateTabHelper* helper =
+      SecurityStateTabHelper::FromWebContents(contents);
+  ASSERT_TRUE(helper);
+  security_state::SecurityInfo security_info;
+  helper->GetSecurityInfo(&security_info);
+  EXPECT_EQ(security_state::DANGEROUS, security_info.security_level);
 }
 
 void CaptivePortalBlockingPageTest::TestInterstitial(
@@ -180,7 +196,7 @@ void CaptivePortalBlockingPageTest::TestInterstitial(
     ExpectWiFi expect_wifi,
     ExpectWiFiSSID expect_wifi_ssid,
     ExpectLoginURL expect_login_url,
-    scoped_ptr<SSLCertReporter> ssl_cert_reporter) {
+    std::unique_ptr<SSLCertReporter> ssl_cert_reporter) {
   TestInterstitial(is_wifi_connection, wifi_ssid, login_url, expect_wifi,
                    expect_wifi_ssid, expect_login_url,
                    std::move(ssl_cert_reporter), login_url.host());
@@ -192,7 +208,7 @@ void CaptivePortalBlockingPageTest::TestCertReporting(
 
   certificate_reporting_test_utils::SetCertReportingOptIn(browser(), opt_in);
   base::RunLoop run_loop;
-  scoped_ptr<SSLCertReporter> ssl_cert_reporter =
+  std::unique_ptr<SSLCertReporter> ssl_cert_reporter =
       certificate_reporting_test_utils::SetUpMockSSLCertReporter(
           &run_loop,
           opt_in == certificate_reporting_test_utils::EXTENDED_REPORTING_OPT_IN
@@ -316,7 +332,8 @@ class CaptivePortalBlockingPageIDNTest : public SecurityInterstitialIDNTest {
     CaptivePortalBlockingPage* blocking_page =
         new CaptivePortalBlockingPageForTesting(
             contents, GURL(kBrokenSSL), request_url, nullptr, empty_ssl_info,
-            base::Callback<void(bool)>(), false, "");
+            base::Callback<void(content::CertificateRequestResultType)>(),
+            false, "");
     return blocking_page;
   }
 };
