@@ -9,8 +9,8 @@
 #include "base/bind.h"
 #include "base/bind_helpers.h"
 #include "base/callback_helpers.h"
-#include "base/metrics/histogram.h"
-#include "base/prefs/pref_service.h"
+#include "base/memory/ptr_util.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/time/time.h"
 #include "chrome/browser/browser_process.h"
@@ -22,20 +22,18 @@
 #include "chrome/browser/ssl/ssl_cert_reporter.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
+#include "components/prefs/pref_service.h"
 #include "components/security_interstitials/core/controller_client.h"
 #include "components/security_interstitials/core/metrics_helper.h"
 #include "components/security_interstitials/core/ssl_error_ui.h"
-#include "content/public/browser/cert_store.h"
 #include "content/public/browser/interstitial_page.h"
 #include "content/public/browser/interstitial_page_delegate.h"
 #include "content/public/browser/navigation_entry.h"
 #include "content/public/browser/render_process_host.h"
-#include "content/public/browser/signed_certificate_timestamp_store.h"
+#include "content/public/browser/ssl_status.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/common/renderer_preferences.h"
-#include "content/public/common/ssl_status.h"
 #include "net/base/net_errors.h"
-#include "net/base/net_util.h"
 
 using base::TimeTicks;
 using content::InterstitialPage;
@@ -61,7 +59,8 @@ enum SSLExpirationAndDecision {
 
 // Rappor prefix, which is used for both overridable and non-overridable
 // interstitials so we don't leak the "overridable" bit.
-const char kSSLRapporPrefix[] = "ssl2";
+const char kDeprecatedSSLRapporPrefix[] = "ssl2";
+const char kSSLRapporPrefix[] = "ssl3";
 
 std::string GetSamplingEventName(const bool overridable, const int cert_error) {
   std::string event_name(kEventNameBase);
@@ -99,69 +98,64 @@ void RecordSSLExpirationPageEventState(bool expired_but_previously_allowed,
   }
 }
 
+std::unique_ptr<ChromeMetricsHelper> CreateMetricsHelper(
+    content::WebContents* web_contents,
+    int cert_error,
+    const GURL& request_url,
+    bool overridable) {
+  // Set up the metrics helper for the SSLErrorUI.
+  security_interstitials::MetricsHelper::ReportDetails reporting_info;
+  reporting_info.metric_prefix =
+      overridable ? "ssl_overridable" : "ssl_nonoverridable";
+  reporting_info.rappor_prefix = kSSLRapporPrefix;
+  reporting_info.deprecated_rappor_prefix = kDeprecatedSSLRapporPrefix;
+  reporting_info.rappor_report_type = rappor::LOW_FREQUENCY_UMA_RAPPOR_TYPE;
+  reporting_info.deprecated_rappor_report_type = rappor::UMA_RAPPOR_TYPE;
+  return base::MakeUnique<ChromeMetricsHelper>(
+      web_contents, request_url, reporting_info,
+      GetSamplingEventName(overridable, cert_error));
+}
+
 }  // namespace
 
 // static
 InterstitialPageDelegate::TypeID SSLBlockingPage::kTypeForTesting =
     &SSLBlockingPage::kTypeForTesting;
 
-// Note that we always create a navigation entry with SSL errors.
-// No error happening loading a sub-resource triggers an interstitial so far.
-SSLBlockingPage::SSLBlockingPage(content::WebContents* web_contents,
-                                 int cert_error,
-                                 const net::SSLInfo& ssl_info,
-                                 const GURL& request_url,
-                                 int options_mask,
-                                 const base::Time& time_triggered,
-                                 scoped_ptr<SSLCertReporter> ssl_cert_reporter,
-                                 const base::Callback<void(bool)>& callback)
-    : SecurityInterstitialPage(web_contents, request_url),
-      callback_(callback),
-      ssl_info_(ssl_info),
-      overridable_(IsOverridable(
-          options_mask,
-          Profile::FromBrowserContext(web_contents->GetBrowserContext()))),
-      expired_but_previously_allowed_(
-          (options_mask & SSLErrorUI::EXPIRED_BUT_PREVIOUSLY_ALLOWED) != 0),
-      controller_(new ChromeControllerClient(web_contents)) {
-  // Get the language and override prefs for the SSLErrorUI.
-  std::string languages;
+// static
+SSLBlockingPage* SSLBlockingPage::Create(
+    content::WebContents* web_contents,
+    int cert_error,
+    const net::SSLInfo& ssl_info,
+    const GURL& request_url,
+    int options_mask,
+    const base::Time& time_triggered,
+    std::unique_ptr<SSLCertReporter> ssl_cert_reporter,
+    const base::Callback<void(content::CertificateRequestResultType)>&
+        callback) {
+  // Override prefs for the SSLErrorUI.
   Profile* profile =
       Profile::FromBrowserContext(web_contents->GetBrowserContext());
-  if (profile) {
-    languages = profile->GetPrefs()->GetString(prefs::kAcceptLanguages);
-    if (!profile->GetPrefs()->GetBoolean(prefs::kSSLErrorOverrideAllowed)) {
-      options_mask |= SSLErrorUI::HARD_OVERRIDE_DISABLED;
-    }
+  if (profile &&
+      !profile->GetPrefs()->GetBoolean(prefs::kSSLErrorOverrideAllowed)) {
+    options_mask |= SSLErrorUI::HARD_OVERRIDE_DISABLED;
   }
-  if (overridable_)
+  bool overridable = IsOverridable(
+      options_mask,
+      Profile::FromBrowserContext(web_contents->GetBrowserContext()));
+  if (overridable)
     options_mask |= SSLErrorUI::SOFT_OVERRIDE_ENABLED;
   else
     options_mask &= ~SSLErrorUI::SOFT_OVERRIDE_ENABLED;
 
-  // Set up the metrics helper for the SSLErrorUI.
-  security_interstitials::MetricsHelper::ReportDetails reporting_info;
-  reporting_info.metric_prefix =
-      overridable_ ? "ssl_overridable" : "ssl_nonoverridable";
-  reporting_info.rappor_prefix = kSSLRapporPrefix;
-  reporting_info.rappor_report_type = rappor::UMA_RAPPOR_TYPE;
-  ChromeMetricsHelper* chrome_metrics_helper =
-      new ChromeMetricsHelper(web_contents, request_url, reporting_info,
-                              GetSamplingEventName(overridable_, cert_error));
-  chrome_metrics_helper->StartRecordingCaptivePortalMetrics(overridable_);
-  controller_->set_metrics_helper(make_scoped_ptr(chrome_metrics_helper));
+  std::unique_ptr<ChromeMetricsHelper> metrics_helper(
+      CreateMetricsHelper(web_contents, cert_error, request_url, overridable));
+  metrics_helper.get()->StartRecordingCaptivePortalMetrics(overridable);
 
-  cert_report_helper_.reset(new CertReportHelper(
-      std::move(ssl_cert_reporter), web_contents, request_url, ssl_info,
-      certificate_reporting::ErrorReport::INTERSTITIAL_SSL, overridable_,
-      controller_->metrics_helper()));
-
-  ssl_error_ui_.reset(new SSLErrorUI(request_url, cert_error, ssl_info,
-                                     options_mask, time_triggered, languages,
-                                     controller_.get()));
-
-  // Creating an interstitial without showing (e.g. from chrome://interstitials)
-  // it leaks memory, so don't create it here.
+  return new SSLBlockingPage(web_contents, cert_error, ssl_info, request_url,
+                             options_mask, time_triggered,
+                             std::move(ssl_cert_reporter), overridable,
+                             std::move(metrics_helper), callback);
 }
 
 bool SSLBlockingPage::ShouldCreateNewNavigation() const {
@@ -176,45 +170,64 @@ SSLBlockingPage::~SSLBlockingPage() {
   if (!callback_.is_null()) {
     // The page is closed without the user having chosen what to do, default to
     // deny.
-    RecordSSLExpirationPageEventState(
-        expired_but_previously_allowed_, false, overridable_);
+    RecordSSLExpirationPageEventState(expired_but_previously_allowed_, false,
+                                      overridable_);
     NotifyDenyCertificate();
   }
 }
 
-void SSLBlockingPage::AfterShow() {
-  controller_->set_interstitial_page(interstitial_page());
-}
-
 void SSLBlockingPage::PopulateInterstitialStrings(
     base::DictionaryValue* load_time_data) {
   ssl_error_ui_->PopulateStringsForHTML(load_time_data);
   cert_report_helper_->PopulateExtendedReportingOption(load_time_data);
 }
 
+// Note that we always create a navigation entry with SSL errors.
+// No error happening loading a sub-resource triggers an interstitial so far.
+SSLBlockingPage::SSLBlockingPage(
+    content::WebContents* web_contents,
+    int cert_error,
+    const net::SSLInfo& ssl_info,
+    const GURL& request_url,
+    int options_mask,
+    const base::Time& time_triggered,
+    std::unique_ptr<SSLCertReporter> ssl_cert_reporter,
+    bool overridable,
+    std::unique_ptr<ChromeMetricsHelper> metrics_helper,
+    const base::Callback<void(content::CertificateRequestResultType)>& callback)
+    : SecurityInterstitialPage(web_contents,
+                               request_url,
+                               std::move(metrics_helper)),
+      callback_(callback),
+      ssl_info_(ssl_info),
+      overridable_(overridable),
+      expired_but_previously_allowed_(
+          (options_mask & SSLErrorUI::EXPIRED_BUT_PREVIOUSLY_ALLOWED) != 0),
+      cert_report_helper_(new CertReportHelper(
+          std::move(ssl_cert_reporter),
+          web_contents,
+          request_url,
+          ssl_info,
+          certificate_reporting::ErrorReport::INTERSTITIAL_SSL,
+          overridable_,
+          time_triggered,
+          controller()->metrics_helper())),
+      ssl_error_ui_(new SSLErrorUI(request_url,
+                                   cert_error,
+                                   ssl_info,
+                                   options_mask,
+                                   time_triggered,
+                                   controller())) {
+  // Creating an interstitial without showing (e.g. from chrome://interstitials)
+  // it leaks memory, so don't create it here.
+}
+
 void SSLBlockingPage::OverrideEntry(NavigationEntry* entry) {
-  const int process_id = web_contents()->GetRenderProcessHost()->GetID();
-  const int cert_id = content::CertStore::GetInstance()->StoreCert(
-      ssl_info_.cert.get(), process_id);
-  DCHECK(cert_id);
-
-  content::SignedCertificateTimestampStore* sct_store(
-      content::SignedCertificateTimestampStore::GetInstance());
-  content::SignedCertificateTimestampIDStatusList sct_ids;
-  for (const auto& sct_and_status : ssl_info_.signed_certificate_timestamps) {
-    const int sct_id(sct_store->Store(sct_and_status.sct.get(), process_id));
-    DCHECK(sct_id);
-    sct_ids.push_back(content::SignedCertificateTimestampIDAndStatus(
-        sct_id, sct_and_status.status));
-  }
-
-  entry->GetSSL() =
-      content::SSLStatus(content::SECURITY_STYLE_AUTHENTICATION_BROKEN, cert_id,
-                         sct_ids, ssl_info_);
+  entry->GetSSL() = content::SSLStatus(ssl_info_);
 }
 
 void SSLBlockingPage::SetSSLCertReporterForTesting(
-    scoped_ptr<SSLCertReporter> ssl_cert_reporter) {
+    std::unique_ptr<SSLCertReporter> ssl_cert_reporter) {
   cert_report_helper_->SetSSLCertReporterForTesting(
       std::move(ssl_cert_reporter));
 }
@@ -251,7 +264,7 @@ void SSLBlockingPage::OnProceed() {
 
   // Accepting the certificate resumes the loading of the page.
   DCHECK(!callback_.is_null());
-  callback_.Run(true);
+  callback_.Run(content::CERTIFICATE_REQUEST_RESULT_TYPE_CONTINUE);
   callback_.Reset();
 }
 
@@ -272,7 +285,7 @@ void SSLBlockingPage::NotifyDenyCertificate() {
   if (callback_.is_null())
     return;
 
-  callback_.Run(false);
+  callback_.Run(content::CERTIFICATE_REQUEST_RESULT_TYPE_CANCEL);
   callback_.Reset();
 }
 
