@@ -5,13 +5,13 @@
 #include "chrome/browser/sync/glue/extension_setting_data_type_controller.h"
 
 #include "base/bind.h"
-#include "base/metrics/histogram.h"
+#include "base/metrics/histogram_macros.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/profiles/profile.h"
-#include "components/sync_driver/generic_change_processor.h"
+#include "components/sync/driver/generic_change_processor.h"
+#include "components/sync/model/syncable_service.h"
 #include "content/public/browser/browser_thread.h"
 #include "extensions/browser/extension_system.h"
-#include "sync/api/syncable_service.h"
 
 using content::BrowserThread;
 
@@ -19,23 +19,14 @@ namespace browser_sync {
 
 ExtensionSettingDataTypeController::ExtensionSettingDataTypeController(
     syncer::ModelType type,
-    const base::Closure& error_callback,
-    sync_driver::SyncClient* sync_client,
+    const base::Closure& dump_stack,
+    syncer::SyncClient* sync_client,
     Profile* profile)
-    : NonUIDataTypeController(
-          BrowserThread::GetMessageLoopProxyForThread(BrowserThread::UI),
-          error_callback,
-          sync_client),
-      type_(type),
+    : NonUIDataTypeController(type, dump_stack, sync_client),
       profile_(profile) {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
   DCHECK(type == syncer::EXTENSION_SETTINGS || type == syncer::APP_SETTINGS);
 }
 
-syncer::ModelType ExtensionSettingDataTypeController::type() const {
-  return type_;
-}
-
 syncer::ModelSafeGroup
 ExtensionSettingDataTypeController::model_safe_group() const {
   return syncer::GROUP_FILE;
@@ -46,12 +37,12 @@ ExtensionSettingDataTypeController::~ExtensionSettingDataTypeController() {}
 bool ExtensionSettingDataTypeController::PostTaskOnBackendThread(
     const tracked_objects::Location& from_here,
     const base::Closure& task) {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  DCHECK(CalledOnValidThread());
   return BrowserThread::PostTask(BrowserThread::FILE, from_here, task);
 }
 
 bool ExtensionSettingDataTypeController::StartModels() {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
+  DCHECK(CalledOnValidThread());
   extensions::ExtensionSystem::Get(profile_)->InitForRegularProfile(true);
   return true;
 }
