@@ -6,21 +6,21 @@
 #define CHROME_BROWSER_SYNC_GLUE_THEME_DATA_TYPE_CONTROLLER_H_
 
 #include "base/macros.h"
-#include "components/sync_driver/ui_data_type_controller.h"
+#include "components/sync/driver/ui_data_type_controller.h"
 
 class Profile;
 
 namespace browser_sync {
 
-class ThemeDataTypeController : public sync_driver::UIDataTypeController {
+class ThemeDataTypeController : public syncer::UIDataTypeController {
  public:
-  ThemeDataTypeController(const base::Closure& error_callback,
-                          sync_driver::SyncClient* sync_client,
+  // |dump_stack| is called when an unrecoverable error occurs.
+  ThemeDataTypeController(const base::Closure& dump_stack,
+                          syncer::SyncClient* sync_client,
                           Profile* profile);
-
- private:
   ~ThemeDataTypeController() override;
 
+ private:
   // UIDataTypeController implementations.
   bool StartModels() override;
 
