@@ -6,6 +6,7 @@
 
 #include <utility>
 
+#include "base/memory/ptr_util.h"
 #include "base/memory/singleton.h"
 #include "base/time/time.h"
 #include "build/build_config.h"
@@ -22,6 +23,7 @@
 #include "chrome/browser/services/gcm/gcm_profile_service_factory.h"
 #include "chrome/browser/signin/about_signin_internals_factory.h"
 #include "chrome/browser/signin/chrome_signin_client_factory.h"
+#include "chrome/browser/signin/gaia_cookie_manager_service_factory.h"
 #include "chrome/browser/signin/profile_oauth2_token_service_factory.h"
 #include "chrome/browser/signin/signin_manager_factory.h"
 #include "chrome/browser/sync/chrome_sync_client.h"
@@ -29,19 +31,20 @@
 #include "chrome/browser/themes/theme_service_factory.h"
 #include "chrome/browser/web_data_service_factory.h"
 #include "chrome/common/channel_info.h"
-#include "components/browser_sync/browser/profile_sync_components_factory_impl.h"
-#include "components/browser_sync/browser/profile_sync_service.h"
+#include "components/browser_sync/profile_sync_components_factory_impl.h"
+#include "components/browser_sync/profile_sync_service.h"
 #include "components/keyed_service/content/browser_context_dependency_manager.h"
 #include "components/network_time/network_time_tracker.h"
 #include "components/signin/core/browser/profile_oauth2_token_service.h"
 #include "components/signin/core/browser/signin_manager.h"
-#include "components/sync_driver/signin_manager_wrapper.h"
-#include "components/sync_driver/startup_controller.h"
-#include "components/sync_driver/sync_util.h"
+#include "components/sync/driver/signin_manager_wrapper.h"
+#include "components/sync/driver/startup_controller.h"
+#include "components/sync/driver/sync_util.h"
 #include "content/public/browser/browser_thread.h"
+#include "extensions/features/features.h"
 #include "url/gurl.h"
 
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
 #include "extensions/browser/extension_system_provider.h"
 #include "extensions/browser/extensions_browser_client.h"
 #endif
@@ -50,6 +53,8 @@
 #include "chrome/browser/ui/global_error/global_error_service_factory.h"
 #endif
 
+using browser_sync::ProfileSyncService;
+
 namespace {
 
 void UpdateNetworkTimeOnUIThread(base::Time network_time,
@@ -80,15 +85,14 @@ ProfileSyncServiceFactory* ProfileSyncServiceFactory::GetInstance() {
 ProfileSyncService* ProfileSyncServiceFactory::GetForProfile(
     Profile* profile) {
   if (!ProfileSyncService::IsSyncAllowedByFlag())
-    return NULL;
+    return nullptr;
 
   return static_cast<ProfileSyncService*>(
       GetInstance()->GetServiceForBrowserContext(profile, true));
 }
 
 // static
-sync_driver::SyncService*
-ProfileSyncServiceFactory::GetSyncServiceForBrowserContext(
+syncer::SyncService* ProfileSyncServiceFactory::GetSyncServiceForBrowserContext(
     content::BrowserContext* context) {
   return GetForProfile(Profile::FromBrowserContext(context));
 }
@@ -104,6 +108,7 @@ ProfileSyncServiceFactory::ProfileSyncServiceFactory()
   DependsOn(autofill::PersonalDataManagerFactory::GetInstance());
   DependsOn(BookmarkModelFactory::GetInstance());
   DependsOn(ChromeSigninClientFactory::GetInstance());
+  DependsOn(GaiaCookieManagerServiceFactory::GetInstance());
 #if !defined(OS_ANDROID)
   DependsOn(GlobalErrorServiceFactory::GetInstance());
 #endif
@@ -117,7 +122,7 @@ ProfileSyncServiceFactory::ProfileSyncServiceFactory()
   DependsOn(ThemeServiceFactory::GetInstance());
 #endif
   DependsOn(WebDataServiceFactory::GetInstance());
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
   DependsOn(
       extensions::ExtensionsBrowserClient::Get()->GetExtensionSystemFactory());
 #endif
@@ -151,10 +156,11 @@ KeyedService* ProfileSyncServiceFactory::BuildServiceInstanceFor(
   AboutSigninInternalsFactory::GetForProfile(profile);
 
   init_params.signin_wrapper =
-      make_scoped_ptr(new SupervisedUserSigninManagerWrapper(profile, signin));
-
+      base::MakeUnique<SupervisedUserSigninManagerWrapper>(profile, signin);
   init_params.oauth2_token_service =
       ProfileOAuth2TokenServiceFactory::GetForProfile(profile);
+  init_params.gaia_cookie_manager_service =
+      GaiaCookieManagerServiceFactory::GetForProfile(profile);
 
   // TODO(tim): Currently, AUTO/MANUAL settings refer to the *first* time sync
   // is set up and *not* a browser restart for a manual-start platform (where
@@ -163,26 +169,24 @@ KeyedService* ProfileSyncServiceFactory::BuildServiceInstanceFor(
   // need to take care that ProfileSyncService doesn't get tripped up between
   // those two cases. Bug 88109.
   init_params.start_behavior = browser_defaults::kSyncAutoStarts
-                                   ? browser_sync::AUTO_START
-                                   : browser_sync::MANUAL_START;
+                                   ? ProfileSyncService::AUTO_START
+                                   : ProfileSyncService::MANUAL_START;
 
-  init_params.sync_client =
-      make_scoped_ptr(new browser_sync::ChromeSyncClient(profile));
+  if (!client_factory_) {
+    init_params.sync_client =
+        base::MakeUnique<browser_sync::ChromeSyncClient>(profile);
+  } else {
+    init_params.sync_client = client_factory_->Run(profile);
+  }
 
   init_params.network_time_update_callback = base::Bind(&UpdateNetworkTime);
   init_params.base_directory = profile->GetPath();
   init_params.url_request_context = profile->GetRequestContext();
   init_params.debug_identifier = profile->GetDebugName();
   init_params.channel = chrome::GetChannel();
-
-  init_params.db_thread = content::BrowserThread::GetMessageLoopProxyForThread(
-      content::BrowserThread::DB);
-  init_params.file_thread =
-      content::BrowserThread::GetMessageLoopProxyForThread(
-          content::BrowserThread::FILE);
   init_params.blocking_pool = content::BrowserThread::GetBlockingPool();
 
-  auto pss = make_scoped_ptr(new ProfileSyncService(std::move(init_params)));
+  auto pss = base::MakeUnique<ProfileSyncService>(std::move(init_params));
 
   // Will also initialize the sync client.
   pss->Initialize();
@@ -191,5 +195,15 @@ KeyedService* ProfileSyncServiceFactory::BuildServiceInstanceFor(
 
 // static
 bool ProfileSyncServiceFactory::HasProfileSyncService(Profile* profile) {
-  return GetInstance()->GetServiceForBrowserContext(profile, false) != NULL;
+  return GetInstance()->GetServiceForBrowserContext(profile, false) != nullptr;
 }
+
+// static
+void ProfileSyncServiceFactory::SetSyncClientFactoryForTest(
+    SyncClientFactory* client_factory) {
+  client_factory_ = client_factory;
+}
+
+// static
+ProfileSyncServiceFactory::SyncClientFactory*
+    ProfileSyncServiceFactory::client_factory_ = nullptr;
