@@ -4,6 +4,8 @@
 
 #include <stddef.h>
 
+#include <memory>
+
 #include <set>
 #include "base/macros.h"
 #include "base/strings/utf_string_conversions.h"
@@ -12,11 +14,12 @@
 #include "chrome/browser/signin/account_tracker_service_factory.h"
 #include "chrome/browser/signin/chrome_signin_client_factory.h"
 #include "chrome/browser/signin/signin_error_controller_factory.h"
+#include "chrome/browser/signin/signin_manager_factory.h"
 #include "chrome/browser/sync/profile_sync_test_util.h"
 #include "chrome/browser/sync/sync_ui_util.h"
 #include "chrome/grit/generated_resources.h"
 #include "chrome/test/base/testing_profile.h"
-#include "components/browser_sync/browser/profile_sync_service_mock.h"
+#include "components/browser_sync/profile_sync_service_mock.h"
 #include "components/signin/core/browser/fake_auth_status_provider.h"
 #include "components/signin/core/browser/fake_signin_manager.h"
 #include "components/signin/core/browser/signin_manager.h"
@@ -33,6 +36,8 @@ using ::testing::Return;
 using ::testing::ReturnRef;
 using ::testing::SetArgPointee;
 using ::testing::_;
+using browser_sync::ProfileSyncService;
+using browser_sync::ProfileSyncServiceMock;
 using content::BrowserThread;
 
 // A number of distinct states of the ProfileSyncService can be generated for
@@ -61,8 +66,8 @@ void VerifySyncGlobalErrorResult(ProfileSyncServiceMock* service,
                                  GoogleServiceAuthError::State error_state,
                                  bool is_signed_in,
                                  bool is_error) {
-  EXPECT_CALL(*service, HasSyncSetupCompleted())
-              .WillRepeatedly(Return(is_signed_in));
+  EXPECT_CALL(*service, IsFirstSetupComplete())
+      .WillRepeatedly(Return(is_signed_in));
 
   GoogleServiceAuthError auth_error(error_state);
   EXPECT_CALL(*service, GetAuthError()).WillRepeatedly(ReturnRef(auth_error));
@@ -87,10 +92,10 @@ class SyncUIUtilTest : public testing::Test {
 // Test that GetStatusLabelsForSyncGlobalError returns an error if a
 // passphrase is required.
 TEST_F(SyncUIUtilTest, PassphraseGlobalError) {
-  scoped_ptr<Profile> profile = MakeSignedInTestingProfile();
+  std::unique_ptr<Profile> profile = MakeSignedInTestingProfile();
   ProfileSyncServiceMock service(
       CreateProfileSyncServiceParamsForTest(profile.get()));
-  browser_sync::SyncBackendHost::Status status;
+  syncer::SyncBackendHost::Status status;
   EXPECT_CALL(service, QueryDetailedSyncStatus(_))
               .WillRepeatedly(Return(false));
   EXPECT_CALL(service, IsPassphraseRequired())
@@ -107,10 +112,10 @@ TEST_F(SyncUIUtilTest, PassphraseGlobalError) {
 // Test that GetStatusLabelsForSyncGlobalError returns an error if a
 // passphrase is required and not for auth errors.
 TEST_F(SyncUIUtilTest, AuthAndPassphraseGlobalError) {
-  scoped_ptr<Profile> profile(MakeSignedInTestingProfile());
+  std::unique_ptr<Profile> profile(MakeSignedInTestingProfile());
   ProfileSyncServiceMock service(
       CreateProfileSyncServiceParamsForTest(profile.get()));
-  browser_sync::SyncBackendHost::Status status;
+  syncer::SyncBackendHost::Status status;
   EXPECT_CALL(service, QueryDetailedSyncStatus(_))
               .WillRepeatedly(Return(false));
 
@@ -118,8 +123,7 @@ TEST_F(SyncUIUtilTest, AuthAndPassphraseGlobalError) {
               .WillRepeatedly(Return(true));
   EXPECT_CALL(service, IsPassphraseRequiredForDecryption())
               .WillRepeatedly(Return(true));
-  EXPECT_CALL(service, HasSyncSetupCompleted())
-              .WillRepeatedly(Return(true));
+  EXPECT_CALL(service, IsFirstSetupComplete()).WillRepeatedly(Return(true));
 
   GoogleServiceAuthError auth_error(
       GoogleServiceAuthError::INVALID_GAIA_CREDENTIALS);
@@ -136,12 +140,12 @@ TEST_F(SyncUIUtilTest, AuthAndPassphraseGlobalError) {
 // Test that GetStatusLabelsForSyncGlobalError does not indicate errors for
 // auth errors (these are reported through SigninGlobalError).
 TEST_F(SyncUIUtilTest, AuthStateGlobalError) {
-  scoped_ptr<Profile> profile(MakeSignedInTestingProfile());
+  std::unique_ptr<Profile> profile(MakeSignedInTestingProfile());
   ProfileSyncService::InitParams init_params =
       CreateProfileSyncServiceParamsForTest(profile.get());
   NiceMock<ProfileSyncServiceMock> service(&init_params);
 
-  browser_sync::SyncBackendHost::Status status;
+  syncer::SyncBackendHost::Status status;
   EXPECT_CALL(service, QueryDetailedSyncStatus(_))
               .WillRepeatedly(Return(false));
 
@@ -155,13 +159,9 @@ TEST_F(SyncUIUtilTest, AuthStateGlobalError) {
     GoogleServiceAuthError::ACCOUNT_DISABLED,
     GoogleServiceAuthError::SERVICE_UNAVAILABLE,
     GoogleServiceAuthError::TWO_FACTOR,
-    GoogleServiceAuthError::REQUEST_CANCELED,
-    GoogleServiceAuthError::HOSTED_NOT_ALLOWED
+    GoogleServiceAuthError::REQUEST_CANCELED
   };
 
-  FakeSigninManagerBase signin(
-      ChromeSigninClientFactory::GetForProfile(profile.get()),
-      AccountTrackerServiceFactory::GetForProfile(profile.get()));
   for (size_t i = 0; i < arraysize(table); ++i) {
     VerifySyncGlobalErrorResult(&service,
                                 table[i],
@@ -190,7 +190,7 @@ class FakeSigninManagerForSyncUIUtilTest : public FakeSigninManagerBase {
             ChromeSigninClientFactory::GetForProfile(profile),
             AccountTrackerServiceFactory::GetForProfile(profile)),
         auth_in_progress_(false) {
-    Initialize(NULL);
+    Initialize(nullptr);
   }
 
   ~FakeSigninManagerForSyncUIUtilTest() override {}
@@ -216,34 +216,34 @@ void GetDistinctCase(ProfileSyncServiceMock* service,
   // immutable so can only be allocated in this method.
   switch (caseNumber) {
     case STATUS_CASE_SETUP_IN_PROGRESS: {
-      EXPECT_CALL(*service, HasSyncSetupCompleted())
+      EXPECT_CALL(*service, IsFirstSetupComplete())
           .WillRepeatedly(Return(false));
       EXPECT_CALL(*service, IsFirstSetupInProgress())
           .WillRepeatedly(Return(true));
-      browser_sync::SyncBackendHost::Status status;
+      syncer::SyncBackendHost::Status status;
       EXPECT_CALL(*service, QueryDetailedSyncStatus(_))
           .WillRepeatedly(DoAll(SetArgPointee<0>(status), Return(false)));
       return;
     }
     case STATUS_CASE_SETUP_ERROR: {
-      EXPECT_CALL(*service, HasSyncSetupCompleted())
+      EXPECT_CALL(*service, IsFirstSetupComplete())
           .WillRepeatedly(Return(false));
       EXPECT_CALL(*service, IsFirstSetupInProgress())
           .WillRepeatedly(Return(false));
       EXPECT_CALL(*service, HasUnrecoverableError())
           .WillRepeatedly(Return(true));
-      browser_sync::SyncBackendHost::Status status;
+      syncer::SyncBackendHost::Status status;
       EXPECT_CALL(*service, QueryDetailedSyncStatus(_))
           .WillRepeatedly(DoAll(SetArgPointee<0>(status), Return(false)));
       return;
     }
     case STATUS_CASE_AUTHENTICATING: {
-      EXPECT_CALL(*service, HasSyncSetupCompleted())
+      EXPECT_CALL(*service, IsFirstSetupComplete())
           .WillRepeatedly(Return(true));
       EXPECT_CALL(*service, IsSyncActive()).WillRepeatedly(Return(true));
       EXPECT_CALL(*service, IsPassphraseRequired())
           .WillRepeatedly(Return(false));
-      browser_sync::SyncBackendHost::Status status;
+      syncer::SyncBackendHost::Status status;
       EXPECT_CALL(*service, QueryDetailedSyncStatus(_))
           .WillRepeatedly(DoAll(SetArgPointee<0>(status), Return(false)));
       EXPECT_CALL(*service, HasUnrecoverableError())
@@ -252,12 +252,12 @@ void GetDistinctCase(ProfileSyncServiceMock* service,
       return;
     }
     case STATUS_CASE_AUTH_ERROR: {
-      EXPECT_CALL(*service, HasSyncSetupCompleted())
+      EXPECT_CALL(*service, IsFirstSetupComplete())
           .WillRepeatedly(Return(true));
       EXPECT_CALL(*service, IsSyncActive()).WillRepeatedly(Return(true));
       EXPECT_CALL(*service, IsPassphraseRequired())
           .WillRepeatedly(Return(false));
-      browser_sync::SyncBackendHost::Status status;
+      syncer::SyncBackendHost::Status status;
       EXPECT_CALL(*service, QueryDetailedSyncStatus(_))
           .WillRepeatedly(DoAll(SetArgPointee<0>(status), Return(false)));
       provider->SetAuthError(
@@ -268,14 +268,14 @@ void GetDistinctCase(ProfileSyncServiceMock* service,
       return;
     }
     case STATUS_CASE_PROTOCOL_ERROR: {
-      EXPECT_CALL(*service, HasSyncSetupCompleted())
+      EXPECT_CALL(*service, IsFirstSetupComplete())
           .WillRepeatedly(Return(true));
       EXPECT_CALL(*service, IsSyncActive()).WillRepeatedly(Return(true));
       EXPECT_CALL(*service, IsPassphraseRequired())
           .WillRepeatedly(Return(false));
       syncer::SyncProtocolError protocolError;
-      protocolError.action = syncer::STOP_AND_RESTART_SYNC;
-      browser_sync::SyncBackendHost::Status status;
+      protocolError.action = syncer::UPGRADE_CLIENT;
+      syncer::SyncBackendHost::Status status;
       status.sync_protocol_error = protocolError;
       EXPECT_CALL(*service, QueryDetailedSyncStatus(_))
           .WillRepeatedly(DoAll(SetArgPointee<0>(status), Return(false)));
@@ -284,10 +284,10 @@ void GetDistinctCase(ProfileSyncServiceMock* service,
       return;
     }
     case STATUS_CASE_PASSPHRASE_ERROR: {
-      EXPECT_CALL(*service, HasSyncSetupCompleted())
+      EXPECT_CALL(*service, IsFirstSetupComplete())
           .WillRepeatedly(Return(true));
       EXPECT_CALL(*service, IsSyncActive()).WillRepeatedly(Return(true));
-      browser_sync::SyncBackendHost::Status status;
+      syncer::SyncBackendHost::Status status;
       EXPECT_CALL(*service, QueryDetailedSyncStatus(_))
           .WillRepeatedly(DoAll(SetArgPointee<0>(status), Return(false)));
       EXPECT_CALL(*service, HasUnrecoverableError())
@@ -299,12 +299,12 @@ void GetDistinctCase(ProfileSyncServiceMock* service,
       return;
     }
     case STATUS_CASE_SYNCED: {
-      EXPECT_CALL(*service, HasSyncSetupCompleted())
+      EXPECT_CALL(*service, IsFirstSetupComplete())
           .WillRepeatedly(Return(true));
       EXPECT_CALL(*service, IsSyncActive()).WillRepeatedly(Return(true));
       EXPECT_CALL(*service, IsPassphraseRequired())
           .WillRepeatedly(Return(false));
-      browser_sync::SyncBackendHost::Status status;
+      syncer::SyncBackendHost::Status status;
       EXPECT_CALL(*service, QueryDetailedSyncStatus(_))
           .WillRepeatedly(DoAll(SetArgPointee<0>(status), Return(false)));
       EXPECT_CALL(*service, HasUnrecoverableError())
@@ -315,12 +315,12 @@ void GetDistinctCase(ProfileSyncServiceMock* service,
     }
     case STATUS_CASE_SYNC_DISABLED_BY_POLICY: {
       EXPECT_CALL(*service, IsManaged()).WillRepeatedly(Return(true));
-      EXPECT_CALL(*service, HasSyncSetupCompleted())
+      EXPECT_CALL(*service, IsFirstSetupComplete())
           .WillRepeatedly(Return(false));
       EXPECT_CALL(*service, IsSyncActive()).WillRepeatedly(Return(false));
       EXPECT_CALL(*service, IsPassphraseRequired())
           .WillRepeatedly(Return(false));
-      browser_sync::SyncBackendHost::Status status;
+      syncer::SyncBackendHost::Status status;
       EXPECT_CALL(*service, QueryDetailedSyncStatus(_))
           .WillRepeatedly(DoAll(SetArgPointee<0>(status), Return(false)));
       EXPECT_CALL(*service, HasUnrecoverableError())
@@ -332,13 +332,39 @@ void GetDistinctCase(ProfileSyncServiceMock* service,
   }
 }
 
+// Returns the expected value for the output argument |action_type| for each
+// of the distinct cases.
+sync_ui_util::ActionType GetActionTypeforDistinctCase(int case_number) {
+  switch (case_number) {
+    case STATUS_CASE_SETUP_IN_PROGRESS:
+      return sync_ui_util::NO_ACTION;
+    case STATUS_CASE_SETUP_ERROR:
+      return sync_ui_util::REAUTHENTICATE;
+    case STATUS_CASE_AUTHENTICATING:
+      return sync_ui_util::NO_ACTION;
+    case STATUS_CASE_AUTH_ERROR:
+      return sync_ui_util::NO_ACTION;
+    case STATUS_CASE_PROTOCOL_ERROR:
+      return sync_ui_util::UPGRADE_CLIENT;
+    case STATUS_CASE_PASSPHRASE_ERROR:
+      return sync_ui_util::ENTER_PASSPHRASE;
+    case STATUS_CASE_SYNCED:
+      return sync_ui_util::NO_ACTION;
+    case STATUS_CASE_SYNC_DISABLED_BY_POLICY:
+      return sync_ui_util::NO_ACTION;
+    default:
+      NOTREACHED();
+      return sync_ui_util::NO_ACTION;
+  }
+}
+
 // This test ensures that a each distinctive ProfileSyncService statuses
 // will return a unique combination of status and link messages from
 // GetStatusLabels().
 TEST_F(SyncUIUtilTest, DistinctCasesReportUniqueMessageSets) {
   std::set<base::string16> messages;
   for (int idx = 0; idx != NUMBER_OF_STATUS_CASES; idx++) {
-    scoped_ptr<Profile> profile(new TestingProfile());
+    std::unique_ptr<Profile> profile(new TestingProfile());
     ProfileSyncService::InitParams init_params =
         CreateProfileSyncServiceParamsForTest(profile.get());
     NiceMock<ProfileSyncServiceMock> service(&init_params);
@@ -346,14 +372,17 @@ TEST_F(SyncUIUtilTest, DistinctCasesReportUniqueMessageSets) {
     EXPECT_CALL(service, GetAuthError()).WillRepeatedly(ReturnRef(error));
     FakeSigninManagerForSyncUIUtilTest signin(profile.get());
     signin.SetAuthenticatedAccountInfo(kTestGaiaId, kTestUser);
-    scoped_ptr<FakeAuthStatusProvider> provider(new FakeAuthStatusProvider(
+    std::unique_ptr<FakeAuthStatusProvider> provider(new FakeAuthStatusProvider(
         SigninErrorControllerFactory::GetForProfile(profile.get())));
     GetDistinctCase(&service, &signin, provider.get(), idx);
     base::string16 status_label;
     base::string16 link_label;
+    sync_ui_util::ActionType action_type = sync_ui_util::NO_ACTION;
     sync_ui_util::GetStatusLabels(profile.get(), &service, signin,
                                   sync_ui_util::WITH_HTML, &status_label,
-                                  &link_label);
+                                  &link_label, &action_type);
+
+    EXPECT_EQ(GetActionTypeforDistinctCase(idx), action_type);
     // If the status and link message combination is already present in the set
     // of messages already seen, this is a duplicate rather than a unique
     // message, and the test has failed.
@@ -376,7 +405,7 @@ TEST_F(SyncUIUtilTest, DistinctCasesReportUniqueMessageSets) {
 // honored.
 TEST_F(SyncUIUtilTest, HtmlNotIncludedInStatusIfNotRequested) {
   for (int idx = 0; idx != NUMBER_OF_STATUS_CASES; idx++) {
-    scoped_ptr<Profile> profile(MakeSignedInTestingProfile());
+    std::unique_ptr<Profile> profile(MakeSignedInTestingProfile());
     ProfileSyncService::InitParams init_params =
         CreateProfileSyncServiceParamsForTest(profile.get());
     NiceMock<ProfileSyncServiceMock> service(&init_params);
@@ -384,15 +413,17 @@ TEST_F(SyncUIUtilTest, HtmlNotIncludedInStatusIfNotRequested) {
     EXPECT_CALL(service, GetAuthError()).WillRepeatedly(ReturnRef(error));
     FakeSigninManagerForSyncUIUtilTest signin(profile.get());
     signin.SetAuthenticatedAccountInfo(kTestGaiaId, kTestUser);
-    scoped_ptr<FakeAuthStatusProvider> provider(new FakeAuthStatusProvider(
+    std::unique_ptr<FakeAuthStatusProvider> provider(new FakeAuthStatusProvider(
         SigninErrorControllerFactory::GetForProfile(profile.get())));
     GetDistinctCase(&service, &signin, provider.get(), idx);
     base::string16 status_label;
     base::string16 link_label;
+    sync_ui_util::ActionType action_type = sync_ui_util::NO_ACTION;
     sync_ui_util::GetStatusLabels(profile.get(), &service, signin,
                                   sync_ui_util::PLAIN_TEXT, &status_label,
-                                  &link_label);
+                                  &link_label, &action_type);
 
+    EXPECT_EQ(GetActionTypeforDistinctCase(idx), action_type);
     // Ensures a search for string 'href' (found in links, not a string to be
     // found in an English language message) fails when links are excluded from
     // the status label.
@@ -406,3 +437,89 @@ TEST_F(SyncUIUtilTest, HtmlNotIncludedInStatusIfNotRequested) {
     signin.Shutdown();
   }
 }
+
+TEST_F(SyncUIUtilTest, UnrecoverableErrorWithActionableError) {
+  std::unique_ptr<Profile> profile(MakeSignedInTestingProfile());
+  SigninManagerBase* signin =
+      SigninManagerFactory::GetForProfile(profile.get());
+
+  ProfileSyncServiceMock service(
+      CreateProfileSyncServiceParamsForTest(profile.get()));
+  EXPECT_CALL(service, IsFirstSetupComplete()).WillRepeatedly(Return(true));
+  EXPECT_CALL(service, HasUnrecoverableError()).WillRepeatedly(Return(true));
+
+  // First time action is not set. We should get unrecoverable error.
+  syncer::SyncStatus status;
+  EXPECT_CALL(service, QueryDetailedSyncStatus(_))
+      .WillOnce(DoAll(SetArgPointee<0>(status), Return(true)));
+
+  base::string16 link_label;
+  base::string16 unrecoverable_error_status_label;
+  sync_ui_util::ActionType action_type = sync_ui_util::NO_ACTION;
+  sync_ui_util::GetStatusLabels(profile.get(), &service, *signin,
+                                sync_ui_util::PLAIN_TEXT,
+                                &unrecoverable_error_status_label, &link_label,
+                                &action_type);
+
+  // Expect the generic unrecoverable error action which is to reauthenticate.
+  EXPECT_EQ(sync_ui_util::REAUTHENTICATE, action_type);
+
+  // This time set action to UPGRADE_CLIENT. Ensure that status label differs
+  // from previous one.
+  status.sync_protocol_error.action = syncer::UPGRADE_CLIENT;
+  EXPECT_CALL(service, QueryDetailedSyncStatus(_))
+      .WillOnce(DoAll(SetArgPointee<0>(status), Return(true)));
+  base::string16 upgrade_client_status_label;
+  sync_ui_util::GetStatusLabels(profile.get(), &service, *signin,
+                                sync_ui_util::PLAIN_TEXT,
+                                &upgrade_client_status_label, &link_label,
+                                &action_type);
+  // Expect an explicit 'client upgrade' action.
+  EXPECT_EQ(sync_ui_util::UPGRADE_CLIENT, action_type);
+
+  EXPECT_NE(unrecoverable_error_status_label, upgrade_client_status_label);
+}
+
+TEST_F(SyncUIUtilTest, ActionableErrorWithPassiveMessage) {
+  std::unique_ptr<Profile> profile(MakeSignedInTestingProfile());
+  SigninManagerBase* signin =
+      SigninManagerFactory::GetForProfile(profile.get());
+
+  ProfileSyncServiceMock service(
+      CreateProfileSyncServiceParamsForTest(profile.get()));
+  EXPECT_CALL(service, IsFirstSetupComplete()).WillRepeatedly(Return(true));
+  EXPECT_CALL(service, HasUnrecoverableError()).WillRepeatedly(Return(true));
+
+  // Set action to UPGRADE_CLIENT.
+  syncer::SyncStatus status;
+  status.sync_protocol_error.action = syncer::UPGRADE_CLIENT;
+  EXPECT_CALL(service, QueryDetailedSyncStatus(_))
+      .WillOnce(DoAll(SetArgPointee<0>(status), Return(true)));
+
+  base::string16 first_actionable_error_status_label;
+  base::string16 link_label;
+  sync_ui_util::ActionType action_type = sync_ui_util::NO_ACTION;
+  sync_ui_util::GetStatusLabels(profile.get(), &service, *signin,
+                                sync_ui_util::PLAIN_TEXT,
+                                &first_actionable_error_status_label,
+                                &link_label, &action_type);
+  // Expect a 'client upgrade' call to action.
+  EXPECT_EQ(sync_ui_util::UPGRADE_CLIENT, action_type);
+
+  // This time set action to ENABLE_SYNC_ON_ACCOUNT.
+  status.sync_protocol_error.action = syncer::ENABLE_SYNC_ON_ACCOUNT;
+  EXPECT_CALL(service, QueryDetailedSyncStatus(_))
+      .WillOnce(DoAll(SetArgPointee<0>(status), Return(true)));
+
+  base::string16 second_actionable_error_status_label;
+  action_type = sync_ui_util::NO_ACTION;
+  sync_ui_util::GetStatusLabels(profile.get(), &service, *signin,
+                                sync_ui_util::PLAIN_TEXT,
+                                &second_actionable_error_status_label,
+                                &link_label, &action_type);
+  // Expect a passive message instead of a call to action.
+  EXPECT_EQ(sync_ui_util::NO_ACTION, action_type);
+
+  EXPECT_NE(first_actionable_error_status_label,
+            second_actionable_error_status_label);
+}
