@@ -2,12 +2,13 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "chrome/browser/sync/profile_sync_service.h"
+#include "base/macros.h"
 #include "chrome/browser/sync/test/integration/profile_sync_service_harness.h"
 #include "chrome/browser/sync/test/integration/sync_test.h"
-#include "sync/internal_api/public/base/model_type.h"
-#include "sync/internal_api/public/read_node.h"
-#include "sync/internal_api/public/read_transaction.h"
+#include "components/browser_sync/profile_sync_service.h"
+#include "components/sync/base/model_type.h"
+#include "components/sync/syncable/read_node.h"
+#include "components/sync/syncable/read_transaction.h"
 
 // This file contains tests that exercise enabling and disabling data
 // types.
@@ -33,7 +34,7 @@ bool DoesTopLevelNodeExist(syncer::UserShare* user_share,
     return node.InitTypeRoot(type) == syncer::BaseNode::INIT_OK;
 }
 
-bool IsUnready(const sync_driver::DataTypeStatusTable& data_type_status_table,
+bool IsUnready(const syncer::DataTypeStatusTable& data_type_status_table,
                syncer::ModelType type) {
   return data_type_status_table.GetUnreadyErrorTypes().Has(type);
 }
@@ -44,12 +45,15 @@ IN_PROC_BROWSER_TEST_F(EnableDisableSingleClientTest, EnableOneAtATime) {
   // Setup sync with no enabled types.
   ASSERT_TRUE(GetClient(0)->SetupSync(syncer::ModelTypeSet()));
 
+  syncer::UserShare* user_share = GetSyncService(0)->GetUserShare();
+  const syncer::DataTypeStatusTable& data_type_status_table =
+      GetSyncService(0)->data_type_status_table();
+
   const syncer::ModelTypeSet registered_types =
       GetSyncService(0)->GetRegisteredDataTypes();
-  syncer::UserShare* user_share = GetSyncService(0)->GetUserShare();
-  const sync_driver::DataTypeStatusTable& data_type_status_table =
-      GetSyncService(0)->data_type_status_table();
-  for (syncer::ModelTypeSet::Iterator it = registered_types.First();
+  const syncer::ModelTypeSet registered_user_types =
+      Intersection(registered_types, syncer::UserSelectableTypes());
+  for (syncer::ModelTypeSet::Iterator it = registered_user_types.First();
        it.Good(); it.Inc()) {
     ASSERT_TRUE(GetClient(0)->EnableSyncForDatatype(it.Get()));
 
@@ -91,7 +95,7 @@ IN_PROC_BROWSER_TEST_F(EnableDisableSingleClientTest, DisableOneAtATime) {
 
   syncer::UserShare* user_share = GetSyncService(0)->GetUserShare();
 
-  const sync_driver::DataTypeStatusTable& data_type_status_table =
+  const syncer::DataTypeStatusTable& data_type_status_table =
       GetSyncService(0)->data_type_status_table();
 
   // Make sure all top-level nodes exist first.
