@@ -7,7 +7,9 @@
 #include <tuple>
 #include <utility>
 
-#include "base/metrics/histogram.h"
+#include "base/memory/ptr_util.h"
+#include "base/metrics/histogram_macros.h"
+#include "base/stl_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/threading/thread_restrictions.h"
@@ -82,14 +84,14 @@ void ReadDatabaseContents(LevelDBWrapper* db, DatabaseContents* contents) {
   DCHECK(db);
   DCHECK(contents);
 
-  scoped_ptr<LevelDBWrapper::Iterator> itr(db->NewIterator());
+  std::unique_ptr<LevelDBWrapper::Iterator> itr(db->NewIterator());
   for (itr->SeekToFirst(); itr->Valid(); itr->Next()) {
     std::string key = itr->key().ToString();
     std::string value = itr->value().ToString();
 
     std::string file_id;
     if (RemovePrefix(key, kFileMetadataKeyPrefix, &file_id)) {
-      scoped_ptr<FileMetadata> metadata(new FileMetadata);
+      std::unique_ptr<FileMetadata> metadata(new FileMetadata);
       if (!metadata->ParseFromString(itr->value().ToString())) {
         util::Log(logging::LOG_WARNING, FROM_HERE,
                   "Failed to parse a FileMetadata");
@@ -109,7 +111,7 @@ void ReadDatabaseContents(LevelDBWrapper* db, DatabaseContents* contents) {
         continue;
       }
 
-      scoped_ptr<FileTracker> tracker(new FileTracker);
+      std::unique_ptr<FileTracker> tracker(new FileTracker);
       if (!tracker->ParseFromString(itr->value().ToString())) {
         util::Log(logging::LOG_WARNING, FROM_HERE,
                   "Failed to parse a Tracker");
@@ -169,7 +171,7 @@ void RemoveUnreachableItemsFromDB(DatabaseContents* contents,
   ScopedVector<FileTracker> reachable_trackers;
   for (size_t i = 0; i < contents->file_trackers.size(); ++i) {
     FileTracker* tracker = contents->file_trackers[i];
-    if (ContainsKey(visited_trackers, tracker->tracker_id())) {
+    if (base::ContainsKey(visited_trackers, tracker->tracker_id())) {
       reachable_trackers.push_back(tracker);
       contents->file_trackers[i] = nullptr;
     } else {
@@ -187,7 +189,7 @@ void RemoveUnreachableItemsFromDB(DatabaseContents* contents,
   ScopedVector<FileMetadata> referred_file_metadata;
   for (size_t i = 0; i < contents->file_metadata.size(); ++i) {
     FileMetadata* metadata = contents->file_metadata[i];
-    if (ContainsKey(referred_file_ids, metadata->file_id())) {
+    if (base::ContainsKey(referred_file_ids, metadata->file_id())) {
       referred_file_metadata.push_back(metadata);
       contents->file_metadata[i] = nullptr;
     } else {
@@ -200,13 +202,14 @@ void RemoveUnreachableItemsFromDB(DatabaseContents* contents,
 }  // namespace
 
 // static
-scoped_ptr<MetadataDatabaseIndex>
-MetadataDatabaseIndex::Create(LevelDBWrapper* db) {
+std::unique_ptr<MetadataDatabaseIndex> MetadataDatabaseIndex::Create(
+    LevelDBWrapper* db) {
   DCHECK(db);
 
-  scoped_ptr<ServiceMetadata> service_metadata = InitializeServiceMetadata(db);
+  std::unique_ptr<ServiceMetadata> service_metadata =
+      InitializeServiceMetadata(db);
   if (!service_metadata)
-    return scoped_ptr<MetadataDatabaseIndex>();
+    return std::unique_ptr<MetadataDatabaseIndex>();
 
   DatabaseContents contents;
   PutVersionToDB(kCurrentDatabaseVersion, db);
@@ -215,31 +218,31 @@ MetadataDatabaseIndex::Create(LevelDBWrapper* db) {
                                service_metadata->sync_root_tracker_id(),
                                db);
 
-  scoped_ptr<MetadataDatabaseIndex> index(new MetadataDatabaseIndex(db));
+  std::unique_ptr<MetadataDatabaseIndex> index(new MetadataDatabaseIndex(db));
   index->Initialize(std::move(service_metadata), &contents);
   return index;
 }
 
 // static
-scoped_ptr<MetadataDatabaseIndex>
-MetadataDatabaseIndex::CreateForTesting(DatabaseContents* contents,
-                                        LevelDBWrapper* db) {
-  scoped_ptr<MetadataDatabaseIndex> index(new MetadataDatabaseIndex(db));
-  index->Initialize(make_scoped_ptr(new ServiceMetadata), contents);
+std::unique_ptr<MetadataDatabaseIndex> MetadataDatabaseIndex::CreateForTesting(
+    DatabaseContents* contents,
+    LevelDBWrapper* db) {
+  std::unique_ptr<MetadataDatabaseIndex> index(new MetadataDatabaseIndex(db));
+  index->Initialize(base::WrapUnique(new ServiceMetadata), contents);
   return index;
 }
 
 void MetadataDatabaseIndex::Initialize(
-    scoped_ptr<ServiceMetadata> service_metadata,
+    std::unique_ptr<ServiceMetadata> service_metadata,
     DatabaseContents* contents) {
   service_metadata_ = std::move(service_metadata);
 
   for (size_t i = 0; i < contents->file_metadata.size(); ++i)
-    StoreFileMetadata(make_scoped_ptr(contents->file_metadata[i]));
+    StoreFileMetadata(base::WrapUnique(contents->file_metadata[i]));
   contents->file_metadata.weak_clear();
 
   for (size_t i = 0; i < contents->file_trackers.size(); ++i)
-    StoreFileTracker(make_scoped_ptr(contents->file_trackers[i]));
+    StoreFileTracker(base::WrapUnique(contents->file_trackers[i]));
   contents->file_trackers.weak_clear();
 
   UMA_HISTOGRAM_COUNTS("SyncFileSystem.MetadataNumber", metadata_by_id_.size());
@@ -278,7 +281,7 @@ bool MetadataDatabaseIndex::GetFileTracker(int64_t tracker_id,
 }
 
 void MetadataDatabaseIndex::StoreFileMetadata(
-    scoped_ptr<FileMetadata> metadata) {
+    std::unique_ptr<FileMetadata> metadata) {
   PutFileMetadataToDB(*metadata.get(), db_);
   if (!metadata) {
     NOTREACHED();
@@ -290,7 +293,7 @@ void MetadataDatabaseIndex::StoreFileMetadata(
 }
 
 void MetadataDatabaseIndex::StoreFileTracker(
-    scoped_ptr<FileTracker> tracker) {
+    std::unique_ptr<FileTracker> tracker) {
   PutFileTrackerToDB(*tracker.get(), db_);
   if (!tracker) {
     NOTREACHED();
@@ -520,7 +523,7 @@ void MetadataDatabaseIndex::AddToAppIDIndex(
   DVLOG(3) << "  Add to app_root_by_app_id_: " << new_tracker.app_id();
 
   DCHECK(new_tracker.active());
-  DCHECK(!ContainsKey(app_root_by_app_id_, new_tracker.app_id()));
+  DCHECK(!base::ContainsKey(app_root_by_app_id_, new_tracker.app_id()));
   app_root_by_app_id_[new_tracker.app_id()] = new_tracker.tracker_id();
 }
 
@@ -532,7 +535,7 @@ void MetadataDatabaseIndex::UpdateInAppIDIndex(
   if (IsAppRoot(old_tracker) && !IsAppRoot(new_tracker)) {
     DCHECK(old_tracker.active());
     DCHECK(!new_tracker.active());
-    DCHECK(ContainsKey(app_root_by_app_id_, old_tracker.app_id()));
+    DCHECK(base::ContainsKey(app_root_by_app_id_, old_tracker.app_id()));
 
     DVLOG(3) << "  Remove from app_root_by_app_id_: " << old_tracker.app_id();
 
@@ -540,7 +543,7 @@ void MetadataDatabaseIndex::UpdateInAppIDIndex(
   } else if (!IsAppRoot(old_tracker) && IsAppRoot(new_tracker)) {
     DCHECK(!old_tracker.active());
     DCHECK(new_tracker.active());
-    DCHECK(!ContainsKey(app_root_by_app_id_, new_tracker.app_id()));
+    DCHECK(!base::ContainsKey(app_root_by_app_id_, new_tracker.app_id()));
 
     DVLOG(3) << "  Add to app_root_by_app_id_: " << new_tracker.app_id();
 
@@ -552,7 +555,7 @@ void MetadataDatabaseIndex::RemoveFromAppIDIndex(
     const FileTracker& tracker) {
   if (IsAppRoot(tracker)) {
     DCHECK(tracker.active());
-    DCHECK(ContainsKey(app_root_by_app_id_, tracker.app_id()));
+    DCHECK(base::ContainsKey(app_root_by_app_id_, tracker.app_id()));
 
     DVLOG(3) << "  Remove from app_root_by_app_id_: " << tracker.app_id();
 
@@ -567,7 +570,8 @@ void MetadataDatabaseIndex::AddToFileIDIndexes(
   trackers_by_file_id_[new_tracker.file_id()].Insert(new_tracker);
 
   if (trackers_by_file_id_[new_tracker.file_id()].size() > 1) {
-    DVLOG_IF(3, !ContainsKey(multi_tracker_file_ids_, new_tracker.file_id()))
+    DVLOG_IF(3,
+             !base::ContainsKey(multi_tracker_file_ids_, new_tracker.file_id()))
         << "  Add to multi_tracker_file_ids_: " << new_tracker.file_id();
     multi_tracker_file_ids_.insert(new_tracker.file_id());
   }
@@ -580,7 +584,7 @@ void MetadataDatabaseIndex::UpdateInFileIDIndexes(
   DCHECK_EQ(old_tracker.file_id(), new_tracker.file_id());
 
   std::string file_id = new_tracker.file_id();
-  DCHECK(ContainsKey(trackers_by_file_id_, file_id));
+  DCHECK(base::ContainsKey(trackers_by_file_id_, file_id));
 
   if (old_tracker.active() && !new_tracker.active())
     trackers_by_file_id_[file_id].Deactivate(new_tracker.tracker_id());
@@ -602,7 +606,7 @@ void MetadataDatabaseIndex::RemoveFromFileIDIndexes(
   found->second.Erase(tracker.tracker_id());
 
   if (trackers_by_file_id_[tracker.file_id()].size() <= 1) {
-    DVLOG_IF(3, ContainsKey(multi_tracker_file_ids_, tracker.file_id()))
+    DVLOG_IF(3, base::ContainsKey(multi_tracker_file_ids_, tracker.file_id()))
         << "  Remove from multi_tracker_file_ids_: " << tracker.file_id();
     multi_tracker_file_ids_.erase(tracker.file_id());
   }
@@ -623,8 +627,8 @@ void MetadataDatabaseIndex::AddToPathIndexes(
 
   if (trackers_by_parent_and_title_[parent][title].size() > 1 &&
       !title.empty()) {
-    DVLOG_IF(3, !ContainsKey(multi_backing_file_paths_,
-                             ParentIDAndTitle(parent, title)))
+    DVLOG_IF(3, !base::ContainsKey(multi_backing_file_paths_,
+                                   ParentIDAndTitle(parent, title)))
         << "  Add to multi_backing_file_paths_: " << parent << " " << title;
     multi_backing_file_paths_.insert(ParentIDAndTitle(parent, title));
   }
@@ -665,17 +669,17 @@ void MetadataDatabaseIndex::UpdateInPathIndexes(
 
     if (trackers_by_parent_and_title_[parent][old_title].size() <= 1 &&
         !old_title.empty()) {
-      DVLOG_IF(3, ContainsKey(multi_backing_file_paths_,
-                              ParentIDAndTitle(parent, old_title)))
-          << "  Remove from multi_backing_file_paths_: "
-          << parent << " " << old_title;
+      DVLOG_IF(3, base::ContainsKey(multi_backing_file_paths_,
+                                    ParentIDAndTitle(parent, old_title)))
+          << "  Remove from multi_backing_file_paths_: " << parent << " "
+          << old_title;
       multi_backing_file_paths_.erase(ParentIDAndTitle(parent, old_title));
     }
 
     if (trackers_by_parent_and_title_[parent][title].size() > 1 &&
         !title.empty()) {
-      DVLOG_IF(3, !ContainsKey(multi_backing_file_paths_,
-                               ParentIDAndTitle(parent, title)))
+      DVLOG_IF(3, !base::ContainsKey(multi_backing_file_paths_,
+                                     ParentIDAndTitle(parent, title)))
           << "  Add to multi_backing_file_paths_: " << parent << " " << title;
       multi_backing_file_paths_.insert(ParentIDAndTitle(parent, title));
     }
@@ -695,8 +699,8 @@ void MetadataDatabaseIndex::RemoveFromPathIndexes(
   int64_t parent = tracker.parent_tracker_id();
   std::string title = GetTrackerTitle(tracker);
 
-  DCHECK(ContainsKey(trackers_by_parent_and_title_, parent));
-  DCHECK(ContainsKey(trackers_by_parent_and_title_[parent], title));
+  DCHECK(base::ContainsKey(trackers_by_parent_and_title_, parent));
+  DCHECK(base::ContainsKey(trackers_by_parent_and_title_[parent], title));
 
   DVLOG(3) << "  Remove from trackers_by_parent_and_title_: "
            << parent << " " << title;
@@ -705,10 +709,10 @@ void MetadataDatabaseIndex::RemoveFromPathIndexes(
 
   if (trackers_by_parent_and_title_[parent][title].size() <= 1 &&
       !title.empty()) {
-    DVLOG_IF(3, ContainsKey(multi_backing_file_paths_,
-                            ParentIDAndTitle(parent, title)))
-        << "  Remove from multi_backing_file_paths_: "
-        << parent << " " << title;
+    DVLOG_IF(3, base::ContainsKey(multi_backing_file_paths_,
+                                  ParentIDAndTitle(parent, title)))
+        << "  Remove from multi_backing_file_paths_: " << parent << " "
+        << title;
     multi_backing_file_paths_.erase(ParentIDAndTitle(parent, title));
   }
 
@@ -721,8 +725,8 @@ void MetadataDatabaseIndex::RemoveFromPathIndexes(
 
 void MetadataDatabaseIndex::AddToDirtyTrackerIndexes(
     const FileTracker& new_tracker) {
-  DCHECK(!ContainsKey(dirty_trackers_, new_tracker.tracker_id()));
-  DCHECK(!ContainsKey(demoted_dirty_trackers_, new_tracker.tracker_id()));
+  DCHECK(!base::ContainsKey(dirty_trackers_, new_tracker.tracker_id()));
+  DCHECK(!base::ContainsKey(demoted_dirty_trackers_, new_tracker.tracker_id()));
 
   if (new_tracker.dirty()) {
     DVLOG(3) << "  Add to dirty_trackers_: " << new_tracker.tracker_id();
@@ -737,16 +741,16 @@ void MetadataDatabaseIndex::UpdateInDirtyTrackerIndexes(
 
   int64_t tracker_id = new_tracker.tracker_id();
   if (old_tracker.dirty() && !new_tracker.dirty()) {
-    DCHECK(ContainsKey(dirty_trackers_, tracker_id) ||
-           ContainsKey(demoted_dirty_trackers_, tracker_id));
+    DCHECK(base::ContainsKey(dirty_trackers_, tracker_id) ||
+           base::ContainsKey(demoted_dirty_trackers_, tracker_id));
 
     DVLOG(3) << "  Remove from dirty_trackers_: " << tracker_id;
 
     dirty_trackers_.erase(tracker_id);
     demoted_dirty_trackers_.erase(tracker_id);
   } else if (!old_tracker.dirty() && new_tracker.dirty()) {
-    DCHECK(!ContainsKey(dirty_trackers_, tracker_id));
-    DCHECK(!ContainsKey(demoted_dirty_trackers_, tracker_id));
+    DCHECK(!base::ContainsKey(dirty_trackers_, tracker_id));
+    DCHECK(!base::ContainsKey(demoted_dirty_trackers_, tracker_id));
 
     DVLOG(3) << "  Add to dirty_trackers_: " << tracker_id;
 
@@ -758,8 +762,8 @@ void MetadataDatabaseIndex::RemoveFromDirtyTrackerIndexes(
     const FileTracker& tracker) {
   if (tracker.dirty()) {
     int64_t tracker_id = tracker.tracker_id();
-    DCHECK(ContainsKey(dirty_trackers_, tracker_id) ||
-           ContainsKey(demoted_dirty_trackers_, tracker_id));
+    DCHECK(base::ContainsKey(dirty_trackers_, tracker_id) ||
+           base::ContainsKey(demoted_dirty_trackers_, tracker_id));
 
     DVLOG(3) << "  Remove from dirty_trackers_: " << tracker_id;
     dirty_trackers_.erase(tracker_id);
