@@ -17,7 +17,6 @@
 #include "chrome/browser/ui/app_list/test/chrome_app_list_test_support.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_finder.h"
-#include "chrome/browser/ui/host_desktop.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/test/base/in_process_browser_test.h"
@@ -31,7 +30,7 @@
 
 #if defined(OS_CHROMEOS)
 #include "chromeos/chromeos_switches.h"
-#include "chromeos/login/user_names.h"
+#include "components/user_manager/user_names.h"
 #endif  // defined(OS_CHROMEOS)
 
 // Browser Test for AppListController that runs on all platforms supporting
@@ -40,21 +39,20 @@ typedef InProcessBrowserTest AppListControllerBrowserTest;
 
 // Test the CreateNewWindow function of the controller delegate.
 IN_PROC_BROWSER_TEST_F(AppListControllerBrowserTest, CreateNewWindow) {
-  const chrome::HostDesktopType desktop = chrome::GetActiveDesktop();
-  AppListService* service = test::GetAppListService();
+  AppListService* service = AppListService::Get();
   AppListControllerDelegate* controller(service->GetControllerDelegate());
   ASSERT_TRUE(controller);
 
-  EXPECT_EQ(1U, chrome::GetBrowserCount(browser()->profile(), desktop));
+  EXPECT_EQ(1U, chrome::GetBrowserCount(browser()->profile()));
   EXPECT_EQ(0U, chrome::GetBrowserCount(
-      browser()->profile()->GetOffTheRecordProfile(), desktop));
+                    browser()->profile()->GetOffTheRecordProfile()));
 
   controller->CreateNewWindow(browser()->profile(), false);
-  EXPECT_EQ(2U, chrome::GetBrowserCount(browser()->profile(), desktop));
+  EXPECT_EQ(2U, chrome::GetBrowserCount(browser()->profile()));
 
   controller->CreateNewWindow(browser()->profile(), true);
   EXPECT_EQ(1U, chrome::GetBrowserCount(
-      browser()->profile()->GetOffTheRecordProfile(), desktop));
+                    browser()->profile()->GetOffTheRecordProfile()));
 }
 
 // Browser Test for AppListController that observes search result changes.
@@ -127,16 +125,9 @@ class AppListControllerSearchResultsBrowserTest
   DISALLOW_COPY_AND_ASSIGN(AppListControllerSearchResultsBrowserTest);
 };
 
-
-// Flaky on Mac. https://crbug.com/415264
-#if defined(OS_MACOSX)
-#define MAYBE_UninstallSearchResult DISABLED_UninstallSearchResult
-#else
-#define MAYBE_UninstallSearchResult UninstallSearchResult
-#endif
 // Test showing search results, and uninstalling one of them while displayed.
 IN_PROC_BROWSER_TEST_F(AppListControllerSearchResultsBrowserTest,
-                       MAYBE_UninstallSearchResult) {
+                       UninstallSearchResult) {
   base::FilePath test_extension_path;
   ASSERT_TRUE(PathService::Get(chrome::DIR_TEST_DATA, &test_extension_path));
   test_extension_path = test_extension_path.AppendASCII("extensions")
@@ -147,7 +138,7 @@ IN_PROC_BROWSER_TEST_F(AppListControllerSearchResultsBrowserTest,
                        1 /* expected_change: new install */);
   ASSERT_TRUE(extension);
 
-  AppListService* service = test::GetAppListService();
+  AppListService* service = AppListService::Get();
   ASSERT_TRUE(service);
   service->ShowForProfile(browser()->profile());
 
@@ -191,7 +182,7 @@ void AppListControllerGuestModeBrowserTest::SetUpCommandLine(
     base::CommandLine* command_line) {
   command_line->AppendSwitch(chromeos::switches::kGuestSession);
   command_line->AppendSwitchASCII(chromeos::switches::kLoginUser,
-                                  chromeos::login::kGuestUserName);
+                                  user_manager::kGuestUserName);
   command_line->AppendSwitchASCII(chromeos::switches::kLoginProfile,
                                   TestingProfile::kTestUserProfileDir);
   command_line->AppendSwitch(switches::kIncognito);
@@ -199,7 +190,7 @@ void AppListControllerGuestModeBrowserTest::SetUpCommandLine(
 
 // Test creating the initial app list in guest mode.
 IN_PROC_BROWSER_TEST_F(AppListControllerGuestModeBrowserTest, Incognito) {
-  AppListService* service = test::GetAppListService();
+  AppListService* service = AppListService::Get();
   EXPECT_TRUE(service->GetCurrentAppListProfile());
 
   service->ShowForProfile(browser()->profile());
