@@ -8,19 +8,21 @@
 #include <stddef.h>
 
 #include <map>
+#include <memory>
 
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
+#include "base/observer_list.h"
 #include "build/build_config.h"
 #include "chrome/browser/apps/drive/drive_app_uninstall_sync_service.h"
 #include "chrome/browser/sync/glue/sync_start_util.h"
 #include "components/keyed_service/core/keyed_service.h"
-#include "sync/api/string_ordinal.h"
-#include "sync/api/sync_change.h"
-#include "sync/api/sync_change_processor.h"
-#include "sync/api/sync_error_factory.h"
-#include "sync/api/syncable_service.h"
-#include "sync/protocol/app_list_specifics.pb.h"
+#include "components/sync/model/string_ordinal.h"
+#include "components/sync/model/sync_change.h"
+#include "components/sync/model/sync_change_processor.h"
+#include "components/sync/model/sync_error_factory.h"
+#include "components/sync/model/syncable_service.h"
+#include "components/sync/protocol/app_list_specifics.pb.h"
 
 #if defined(OS_CHROMEOS)
 class ArcAppModelBuilder;
@@ -38,12 +40,15 @@ namespace sync_pb {
 class AppListSpecifics;
 }
 
+namespace user_prefs {
+class PrefRegistrySyncable;
+}
+
 namespace app_list {
 
 class AppListFolderItem;
 class AppListItem;
 class AppListModel;
-class ModelPrefUpdater;
 
 // Keyed Service that owns, stores, and syncs an AppListModel for a profile.
 class AppListSyncableService : public syncer::SyncableService,
@@ -59,19 +64,34 @@ class AppListSyncableService : public syncer::SyncableService,
     std::string item_name;
     std::string parent_id;
     syncer::StringOrdinal item_ordinal;
+    syncer::StringOrdinal item_pin_ordinal;
 
     std::string ToString() const;
   };
 
+  class Observer {
+   public:
+    // Notifies that sync model was updated.
+    virtual void OnSyncModelUpdated() = 0;
+
+   protected:
+    virtual ~Observer() = default;
+  };
+
+  using SyncItemMap = std::map<std::string, std::unique_ptr<SyncItem>>;
+
   // Populates the model when |extension_system| is ready.
   AppListSyncableService(Profile* profile,
                          extensions::ExtensionSystem* extension_system);
 
   ~AppListSyncableService() override;
 
+  // Registers prefs to support local storage.
+  static void RegisterProfilePrefs(user_prefs::PrefRegistrySyncable* registry);
+
   // Adds |item| to |sync_items_| and |model_|. If a sync item already exists,
   // updates the existing sync item instead.
-  void AddItem(scoped_ptr<AppListItem> app_item);
+  void AddItem(std::unique_ptr<AppListItem> app_item);
 
   // Removes sync item matching |id|.
   void RemoveItem(const std::string& id);
@@ -88,9 +108,26 @@ class AppListSyncableService : public syncer::SyncableService,
   // Sets the name of the folder for OEM apps.
   void SetOemFolderName(const std::string& name);
 
-  // Gets the app list model, building it if it doesn't yet exist.
+  // Returns optional pin position for the app specified by |app_id|. If app is
+  // not synced or does not have associated pin position then empty ordinal is
+  // returned.
+  syncer::StringOrdinal GetPinPosition(const std::string& app_id);
+
+  // Sets pin position and how it is pinned for the app specified by |app_id|.
+  // Empty |item_pin_ordinal| indicates that the app has no pin.
+  void SetPinPosition(const std::string& app_id,
+                      const syncer::StringOrdinal& item_pin_ordinal);
+
+  // Gets the app list model.
   AppListModel* GetModel();
 
+  // Returns true if this service was initialized.
+  bool IsInitialized() const;
+
+  // Registers new observers and makes sure that service is started.
+  void AddObserverAndStart(Observer* observer);
+  void RemoveObserver(Observer* observer);
+
   Profile* profile() { return profile_; }
   size_t GetNumSyncItemsForTest();
   const std::string& GetOemFolderNameForTest() const {
@@ -98,12 +135,14 @@ class AppListSyncableService : public syncer::SyncableService,
   }
   void ResetDriveAppProviderForTest();
 
+  const SyncItemMap& sync_items() const { return sync_items_; }
+
   // syncer::SyncableService
   syncer::SyncMergeResult MergeDataAndStartSyncing(
       syncer::ModelType type,
       const syncer::SyncDataList& initial_sync_data,
-      scoped_ptr<syncer::SyncChangeProcessor> sync_processor,
-      scoped_ptr<syncer::SyncErrorFactory> error_handler) override;
+      std::unique_ptr<syncer::SyncChangeProcessor> sync_processor,
+      std::unique_ptr<syncer::SyncErrorFactory> error_handler) override;
   void StopSyncing(syncer::ModelType type) override;
   syncer::SyncDataList GetAllSyncData(syncer::ModelType type) const override;
   syncer::SyncError ProcessSyncChanges(
@@ -112,7 +151,6 @@ class AppListSyncableService : public syncer::SyncableService,
 
  private:
   class ModelObserver;
-  typedef std::map<std::string, SyncItem*> SyncItemMap;
 
   // KeyedService
   void Shutdown() override;
@@ -145,7 +183,7 @@ class AppListSyncableService : public syncer::SyncableService,
   bool RemoveDefaultApp(AppListItem* item, SyncItem* sync_item);
 
   // Deletes a sync item from |sync_items_| and sends a DELETE action.
-  void DeleteSyncItem(SyncItem* sync_item);
+  void DeleteSyncItem(const std::string& item_id);
 
   // Updates existing entry in |sync_items_| from |app_item|.
   void UpdateSyncItem(AppListItem* app_item);
@@ -202,25 +240,43 @@ class AppListSyncableService : public syncer::SyncableService,
   // an OEM (extension->was_installed_by_oem() is true).
   bool AppIsOem(const std::string& id);
 
+  // Initializes sync items from the local storage while sync service is not
+  // enabled.
+  void InitFromLocalStorage();
+
+  // Helper that notifies observers that sync model has been updated.
+  void NotifyObserversSyncUpdated();
+
+  // Handles model update start/finish.
+  void HandleUpdateStarted();
+  void HandleUpdateFinished();
+
+  // Returns true if extension service is ready.
+  bool IsExtensionServiceReady() const;
+
   Profile* profile_;
   extensions::ExtensionSystem* extension_system_;
-  scoped_ptr<AppListModel> model_;
-  scoped_ptr<ModelObserver> model_observer_;
-  scoped_ptr<ModelPrefUpdater> model_pref_updater_;
-  scoped_ptr<ExtensionAppModelBuilder> apps_builder_;
+  std::unique_ptr<AppListModel> model_;
+  std::unique_ptr<ModelObserver> model_observer_;
+  std::unique_ptr<ExtensionAppModelBuilder> apps_builder_;
 #if defined(OS_CHROMEOS)
-  scoped_ptr<ArcAppModelBuilder> arc_apps_builder_;
+  std::unique_ptr<ArcAppModelBuilder> arc_apps_builder_;
 #endif
-  scoped_ptr<syncer::SyncChangeProcessor> sync_processor_;
-  scoped_ptr<syncer::SyncErrorFactory> sync_error_handler_;
+  std::unique_ptr<syncer::SyncChangeProcessor> sync_processor_;
+  std::unique_ptr<syncer::SyncErrorFactory> sync_error_handler_;
   SyncItemMap sync_items_;
   syncer::SyncableService::StartSyncFlare flare_;
   bool initial_sync_data_processed_;
   bool first_app_list_sync_;
   std::string oem_folder_name_;
 
+  // List of observers.
+  base::ObserverList<Observer> observer_list_;
+
   // Provides integration with Drive apps.
-  scoped_ptr<DriveAppProvider> drive_app_provider_;
+  std::unique_ptr<DriveAppProvider> drive_app_provider_;
+
+  base::WeakPtrFactory<AppListSyncableService> weak_ptr_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(AppListSyncableService);
 };
