@@ -10,16 +10,16 @@
 
 #include "base/bind.h"
 #include "base/command_line.h"
+#include "base/json/json_reader.h"
 #include "base/json/json_string_value_serializer.h"
 #include "base/macros.h"
 #include "base/memory/singleton.h"
 #include "base/metrics/user_metrics.h"
-#include "base/prefs/pref_service.h"
 #include "base/strings/string_piece.h"
 #include "build/build_config.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/chrome_notification_types.h"
-#include "chrome/browser/media/media_stream_devices_controller.h"
+#include "chrome/browser/media/webrtc/media_stream_devices_controller.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/search/hotword_service.h"
 #include "chrome/browser/search/hotword_service_factory.h"
@@ -36,9 +36,9 @@
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
-#include "components/search_engines/template_url_prepopulate_data.h"
+#include "components/prefs/pref_service.h"
 #include "components/search_engines/template_url_service.h"
-#include "components/ui/zoom/zoom_controller.h"
+#include "components/zoom/zoom_controller.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/notification_details.h"
 #include "content/public/browser/notification_observer.h"
@@ -180,19 +180,14 @@ class StartPageService::StartPageWebContentsDelegate
                       const gfx::Rect& initial_pos,
                       bool user_gesture,
                       bool* was_blocked) override {
-    chrome::ScopedTabbedBrowserDisplayer displayer(
-        profile_, chrome::GetActiveDesktop());
+    chrome::ScopedTabbedBrowserDisplayer displayer(profile_);
     // Force all links to open in a new tab, even if they were trying to open a
     // new window.
-    disposition =
-        disposition == NEW_BACKGROUND_TAB ? disposition : NEW_FOREGROUND_TAB;
-    chrome::AddWebContents(displayer.browser(),
-                           nullptr,
-                           new_contents,
-                           disposition,
-                           initial_pos,
-                           user_gesture,
-                           was_blocked);
+    disposition = disposition == WindowOpenDisposition::NEW_BACKGROUND_TAB
+                      ? disposition
+                      : WindowOpenDisposition::NEW_FOREGROUND_TAB;
+    chrome::AddWebContents(displayer.browser(), nullptr, new_contents,
+                           disposition, initial_pos, user_gesture, was_blocked);
   }
 
   content::WebContents* OpenURLFromTab(
@@ -202,10 +197,10 @@ class StartPageService::StartPageWebContentsDelegate
     // window.
     chrome::NavigateParams new_tab_params(
         static_cast<Browser*>(nullptr), params.url, params.transition);
-    if (params.disposition == NEW_BACKGROUND_TAB) {
-      new_tab_params.disposition = NEW_BACKGROUND_TAB;
+    if (params.disposition == WindowOpenDisposition::NEW_BACKGROUND_TAB) {
+      new_tab_params.disposition = WindowOpenDisposition::NEW_BACKGROUND_TAB;
     } else {
-      new_tab_params.disposition = NEW_FOREGROUND_TAB;
+      new_tab_params.disposition = WindowOpenDisposition::NEW_FOREGROUND_TAB;
       new_tab_params.window_action = chrome::NavigateParams::SHOW_WINDOW;
     }
 
@@ -334,16 +329,13 @@ StartPageService::StartPageService(Profile* profile)
       search_engine_is_google_(false),
       backoff_entry_(&kDoodleBackoffPolicy),
       weak_factory_(this) {
-  if (switches::IsExperimentalAppListEnabled()) {
-    TemplateURLService* template_url_service =
-        TemplateURLServiceFactory::GetForProfile(profile_);
-    const TemplateURL* default_provider =
-        template_url_service->GetDefaultSearchProvider();
-    search_engine_is_google_ =
-        TemplateURLPrepopulateData::GetEngineType(
-            *default_provider, template_url_service->search_terms_data()) ==
-        SEARCH_ENGINE_GOOGLE;
-  }
+  TemplateURLService* template_url_service =
+      TemplateURLServiceFactory::GetForProfile(profile_);
+  const TemplateURL* default_provider =
+      template_url_service->GetDefaultSearchProvider();
+  search_engine_is_google_ =
+      default_provider->GetEngineType(
+          template_url_service->search_terms_data()) == SEARCH_ENGINE_GOOGLE;
 
   network_change_observer_.reset(new NetworkChangeObserver(this));
 }
@@ -383,15 +375,14 @@ void StartPageService::UpdateRecognitionState() {
 void StartPageService::Init() {
   // Do not load the start page web contents in tests because many tests assume
   // no WebContents exist except the ones they make.
-  if (switches::IsExperimentalAppListEnabled() &&
-      !base::CommandLine::ForCurrentProcess()->HasSwitch(
-          ::switches::kTestType)) {
-    content::BrowserThread::PostDelayedTask(
-        content::BrowserThread::UI, FROM_HERE,
-        base::Bind(&StartPageService::LoadContentsIfNeeded,
-                   weak_factory_.GetWeakPtr()),
-        base::TimeDelta::FromSeconds(kLoadContentsDelaySeconds));
-  }
+  if (base::CommandLine::ForCurrentProcess()->HasSwitch(::switches::kTestType))
+    return;
+
+  content::BrowserThread::PostDelayedTask(
+      content::BrowserThread::UI, FROM_HERE,
+      base::Bind(&StartPageService::LoadContentsIfNeeded,
+                 weak_factory_.GetWeakPtr()),
+      base::TimeDelta::FromSeconds(kLoadContentsDelaySeconds));
 }
 
 void StartPageService::LoadContentsIfNeeded() {
@@ -409,7 +400,7 @@ void StartPageService::AppListShown() {
   } else if (contents_->IsCrashed()) {
     LoadStartPageURL();
   } else if (contents_->GetWebUI()) {
-    contents_->GetWebUI()->CallJavascriptFunction(
+    contents_->GetWebUI()->CallJavascriptFunctionUnsafe(
         "appList.startPage.onAppListShown");
   }
 
@@ -419,9 +410,6 @@ void StartPageService::AppListShown() {
 }
 
 void StartPageService::AppListHidden() {
-  if (!app_list::switches::IsExperimentalAppListEnabled())
-    UnloadContents();
-
   if (speech_recognizer_) {
     StopSpeechRecognition();
   }
@@ -480,8 +468,7 @@ bool StartPageService::HotwordEnabled() {
 }
 
 content::WebContents* StartPageService::GetStartPageContents() {
-  return app_list::switches::IsExperimentalAppListEnabled() ? contents_.get()
-                                                            : NULL;
+  return contents_.get();
 }
 
 content::WebContents* StartPageService::GetSpeechRecognitionContents() {
@@ -499,15 +486,13 @@ void StartPageService::OnSpeechResult(
     speech_result_obtained_ = true;
     RecordAction(UserMetricsAction("AppList_SearchedBySpeech"));
   }
-  FOR_EACH_OBSERVER(StartPageObserver,
-                    observers_,
-                    OnSpeechResult(query, is_final));
+  for (auto& observer : observers_)
+    observer.OnSpeechResult(query, is_final);
 }
 
 void StartPageService::OnSpeechSoundLevelChanged(int16_t level) {
-  FOR_EACH_OBSERVER(StartPageObserver,
-                    observers_,
-                    OnSpeechSoundLevelChanged(level));
+  for (auto& observer : observers_)
+    observer.OnSpeechSoundLevelChanged(level);
 }
 
 void StartPageService::OnSpeechRecognitionStateChanged(
@@ -546,9 +531,8 @@ void StartPageService::OnSpeechRecognitionStateChanged(
   speech_button_toggled_manually_ = false;
   speech_result_obtained_ = false;
   state_ = new_state;
-  FOR_EACH_OBSERVER(StartPageObserver,
-                    observers_,
-                    OnSpeechRecognitionStateChanged(new_state));
+  for (auto& observer : observers_)
+    observer.OnSpeechRecognitionStateChanged(new_state);
 }
 
 void StartPageService::GetSpeechAuthParameters(std::string* auth_scope,
@@ -582,11 +566,11 @@ void StartPageService::DidNavigateMainFrame(
   //
   // Use a temporary zoom level for this web contents (aka isolated zoom
   // mode) so changes to its zoom aren't reflected in any preferences.
-  ui_zoom::ZoomController::FromWebContents(contents_.get())
-      ->SetZoomMode(ui_zoom::ZoomController::ZOOM_MODE_ISOLATED);
+  zoom::ZoomController::FromWebContents(contents_.get())
+      ->SetZoomMode(zoom::ZoomController::ZOOM_MODE_ISOLATED);
   // Set to have a zoom level of 0, which corresponds to 100%, so the
   // contents aren't affected by the browser's default zoom level.
-  ui_zoom::ZoomController::FromWebContents(contents_.get())->SetZoomLevel(0);
+  zoom::ZoomController::FromWebContents(contents_.get())->SetZoomLevel(0);
 }
 
 void StartPageService::DidFailProvisionalLoad(
@@ -625,7 +609,7 @@ void StartPageService::LoadContents() {
   // The ZoomController needs to be created before the web contents is observed
   // by this object. Otherwise it will react to DidNavigateMainFrame after this
   // object does, resetting the zoom mode in the process.
-  ui_zoom::ZoomController::CreateForWebContents(contents_.get());
+  zoom::ZoomController::CreateForWebContents(contents_.get());
   Observe(contents_.get());
 
   LoadStartPageURL();
@@ -673,10 +657,10 @@ void StartPageService::OnURLFetchComplete(const net::URLFetcher* source) {
   if (json_start_index != std::string::npos)
     json_data_substr.remove_prefix(json_start_index);
 
-  JSONStringValueDeserializer deserializer(json_data_substr);
-  deserializer.set_allow_trailing_comma(true);
+  JSONStringValueDeserializer deserializer(json_data_substr,
+                                           base::JSON_ALLOW_TRAILING_COMMAS);
   int error_code = 0;
-  scoped_ptr<base::Value> doodle_json =
+  std::unique_ptr<base::Value> doodle_json =
       deserializer.Deserialize(&error_code, nullptr);
 
   base::TimeDelta recheck_delay;
@@ -691,7 +675,7 @@ void StartPageService::OnURLFetchComplete(const net::URLFetcher* source) {
     recheck_delay = base::TimeDelta::FromMilliseconds(kMaximumRecheckDelayMs);
 
     if (contents_ && contents_->GetWebUI()) {
-      contents_->GetWebUI()->CallJavascriptFunction(
+      contents_->GetWebUI()->CallJavascriptFunctionUnsafe(
           "appList.startPage.onAppListDoodleUpdated", *doodle_json,
           base::StringValue(
               UIThreadSearchTermsData(profile_).GoogleBaseURLValue()));
