@@ -7,6 +7,11 @@
 
 #include "base/macros.h"
 #include "chrome/browser/ui/ash/launcher/launcher_item_controller.h"
+#include "chrome/browser/ui/browser_list.h"
+
+namespace ash {
+class ShelfModel;
+}
 
 namespace content {
 class WebContents;
@@ -22,8 +27,8 @@ class ChromeLauncherController;
 // Item controller for an browser shortcut.
 class BrowserShortcutLauncherItemController : public LauncherItemController {
  public:
-  explicit BrowserShortcutLauncherItemController(
-      ChromeLauncherController* controller);
+  BrowserShortcutLauncherItemController(ChromeLauncherController* controller,
+                                        ash::ShelfModel* shelf_model);
 
   ~BrowserShortcutLauncherItemController() override;
 
@@ -34,8 +39,10 @@ class BrowserShortcutLauncherItemController : public LauncherItemController {
   void SetShelfIDForBrowserWindowContents(Browser* browser,
                                           content::WebContents* web_contents);
 
+  // Check if there is any active browsers windows.
+  bool IsListOfActiveBrowserEmpty();
+
   // LauncherItemController overrides:
-  bool IsOpen() const override;
   bool IsVisible() const override;
   void Launch(ash::LaunchSource source, int event_flags) override;
   ShelfItemDelegate::PerformedAction Activate(
@@ -44,7 +51,6 @@ class BrowserShortcutLauncherItemController : public LauncherItemController {
   ash::ShelfItemDelegate::PerformedAction ItemSelected(
       const ui::Event& event) override;
   base::string16 GetTitle() override;
-  ui::MenuModel* CreateContextMenu(aura::Window* root_window) override;
   ash::ShelfMenuModel* CreateApplicationMenu(int event_flags) override;
   bool IsDraggable() override;
   bool CanPin() const override;
@@ -72,6 +78,11 @@ class BrowserShortcutLauncherItemController : public LauncherItemController {
   // list.
   bool IsBrowserRepresentedInBrowserList(Browser* browser);
 
+  // Get a list of active browsers.
+  BrowserList::BrowserVector GetListOfActiveBrowsers();
+
+  ash::ShelfModel* shelf_model_;
+
   DISALLOW_COPY_AND_ASSIGN(BrowserShortcutLauncherItemController);
 };
 
