@@ -16,14 +16,16 @@
 #include "chrome/common/extensions/chrome_extension_messages.h"
 #include "content/public/browser/ax_event_notification_details.h"
 #include "content/public/browser/browser_context.h"
+#include "ui/accessibility/ax_action_data.h"
+#include "ui/accessibility/ax_enums.h"
+#include "ui/aura/env.h"
 #include "ui/aura/window.h"
-#include "ui/views/accessibility/ax_aura_obj_cache.h"
 #include "ui/views/accessibility/ax_aura_obj_wrapper.h"
 #include "ui/views/view.h"
 #include "ui/views/widget/widget.h"
 
 #if defined(OS_CHROMEOS)
-#include "ash/wm/window_util.h"
+#include "ash/wm/window_util.h"  // nogncheck
 #endif
 
 using content::BrowserContext;
@@ -41,6 +43,7 @@ void AutomationManagerAura::Enable(BrowserContext* context) {
   ResetSerializer();
 
   SendEvent(context, current_tree_->GetRoot(), ui::AX_EVENT_LOAD_COMPLETE);
+  views::AXAuraObjCache::GetInstance()->SetDelegate(this);
 
 #if defined(OS_CHROMEOS)
   aura::Window* active_window = ash::wm::GetActiveWindow();
@@ -65,17 +68,9 @@ void AutomationManagerAura::HandleEvent(BrowserContext* context,
   if (!enabled_)
     return;
 
-  if (!context && g_browser_process->profile_manager())
-    context = g_browser_process->profile_manager()->GetLastUsedProfile();
-
-  if (!context) {
-    LOG(WARNING) << "Accessibility notification but no browser context";
-    return;
-  }
-
   views::AXAuraObjWrapper* aura_obj =
       views::AXAuraObjCache::GetInstance()->GetOrCreate(view);
-  SendEvent(context, aura_obj, event_type);
+  SendEvent(nullptr, aura_obj, event_type);
 }
 
 void AutomationManagerAura::HandleAlert(content::BrowserContext* context,
@@ -89,36 +84,62 @@ void AutomationManagerAura::HandleAlert(content::BrowserContext* context,
   SendEvent(context, obj, ui::AX_EVENT_ALERT);
 }
 
-void AutomationManagerAura::DoDefault(int32_t id) {
+void AutomationManagerAura::PerformAction(
+    const ui::AXActionData& data) {
   CHECK(enabled_);
-  current_tree_->DoDefault(id);
-}
 
-void AutomationManagerAura::Focus(int32_t id) {
-  CHECK(enabled_);
-  current_tree_->Focus(id);
-}
-
-void AutomationManagerAura::MakeVisible(int32_t id) {
-  CHECK(enabled_);
-  current_tree_->MakeVisible(id);
-}
-
-void AutomationManagerAura::SetSelection(int32_t anchor_id,
-                                         int32_t anchor_offset,
-                                         int32_t focus_id,
-                                         int32_t focus_offset) {
-  CHECK(enabled_);
-  if (anchor_id != focus_id) {
-    NOTREACHED();
-    return;
+  switch (data.action) {
+    case ui::AX_ACTION_DO_DEFAULT:
+      current_tree_->DoDefault(data.target_node_id);
+      break;
+    case ui::AX_ACTION_SET_FOCUS:
+      current_tree_->Focus(data.target_node_id);
+      break;
+    case ui::AX_ACTION_SCROLL_TO_MAKE_VISIBLE:
+      current_tree_->MakeVisible(data.target_node_id);
+      break;
+    case ui::AX_ACTION_SET_SELECTION:
+      if (data.anchor_node_id != data.focus_node_id) {
+        NOTREACHED();
+        return;
+      }
+      current_tree_->SetSelection(
+          data.anchor_node_id, data.anchor_offset, data.focus_offset);
+      break;
+    case ui::AX_ACTION_SHOW_CONTEXT_MENU:
+      current_tree_->ShowContextMenu(data.target_node_id);
+      break;
+    case ui::AX_ACTION_SET_ACCESSIBILITY_FOCUS:
+      // Sent by ChromeVox but doesn't need to be handled by aura.
+      break;
+    case ui::AX_ACTION_SET_SEQUENTIAL_FOCUS_NAVIGATION_STARTING_POINT:
+      // Sent by ChromeVox but doesn't need to be handled by aura.
+      break;
+    case ui::AX_ACTION_DECREMENT:
+    case ui::AX_ACTION_HIT_TEST:
+    case ui::AX_ACTION_INCREMENT:
+    case ui::AX_ACTION_REPLACE_SELECTED_TEXT:
+    case ui::AX_ACTION_SCROLL_TO_POINT:
+    case ui::AX_ACTION_SET_SCROLL_OFFSET:
+    case ui::AX_ACTION_SET_VALUE:
+      // Not implemented yet.
+      NOTREACHED();
+      break;
+    case ui::AX_ACTION_NONE:
+      NOTREACHED();
+      break;
   }
-  current_tree_->SetSelection(anchor_id, anchor_offset, focus_offset);
 }
 
-void AutomationManagerAura::ShowContextMenu(int32_t id) {
-  CHECK(enabled_);
-  current_tree_->ShowContextMenu(id);
+void AutomationManagerAura::OnChildWindowRemoved(
+    views::AXAuraObjWrapper* parent) {
+  if (!enabled_)
+    return;
+
+  if (!parent)
+    parent = current_tree_->GetRoot();
+
+  SendEvent(nullptr, parent, ui::AX_EVENT_CHILDREN_CHANGED);
 }
 
 AutomationManagerAura::AutomationManagerAura()
@@ -135,6 +156,15 @@ void AutomationManagerAura::ResetSerializer() {
 void AutomationManagerAura::SendEvent(BrowserContext* context,
                                       views::AXAuraObjWrapper* aura_obj,
                                       ui::AXEvent event_type) {
+  if (!context && g_browser_process->profile_manager()) {
+    context = g_browser_process->profile_manager()->GetLastUsedProfile();
+  }
+
+  if (!context) {
+    LOG(WARNING) << "Accessibility notification but no browser context";
+    return;
+  }
+
   if (processing_events_) {
     pending_events_.push_back(std::make_pair(aura_obj, event_type));
     return;
@@ -146,9 +176,17 @@ void AutomationManagerAura::SendEvent(BrowserContext* context,
     LOG(ERROR) << "Unable to serialize one accessibility event.";
     return;
   }
+
+  // Make sure the focused node is serialized.
+  views::AXAuraObjWrapper* focus =
+      views::AXAuraObjCache::GetInstance()->GetFocus();
+  if (focus)
+    current_tree_serializer_->SerializeChanges(focus, &params.update);
+
   params.tree_id = 0;
   params.id = aura_obj->GetID();
   params.event_type = event_type;
+  params.mouse_location = aura::Env::GetInstance()->last_mouse_location();
   AutomationEventRouter* router = AutomationEventRouter::GetInstance();
   router->DispatchAccessibilityEvent(params);
 
