@@ -7,11 +7,13 @@
 
 #include <stdint.h>
 
+#include <memory>
+
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/extensions/api/automation_internal/automation_action_adapter.h"
 #include "chrome/browser/ui/aura/accessibility/ax_tree_source_aura.h"
 #include "ui/accessibility/ax_tree_serializer.h"
+#include "ui/views/accessibility/ax_aura_obj_cache.h"
 
 namespace base {
 template <typename T>
@@ -33,7 +35,8 @@ using AuraAXTreeSerializer =
                          ui::AXTreeData>;
 
 // Manages a tree of automation nodes.
-class AutomationManagerAura : public extensions::AutomationActionAdapter {
+class AutomationManagerAura : public extensions::AutomationActionAdapter,
+                              views::AXAuraObjCache::Delegate {
  public:
   // Get the single instance of this class.
   static AutomationManagerAura* GetInstance();
@@ -52,14 +55,10 @@ class AutomationManagerAura : public extensions::AutomationActionAdapter {
   void HandleAlert(content::BrowserContext* context, const std::string& text);
 
   // AutomationActionAdapter implementation.
-  void DoDefault(int32_t id) override;
-  void Focus(int32_t id) override;
-  void MakeVisible(int32_t id) override;
-  void SetSelection(int32_t anchor_id,
-                    int32_t anchor_offset,
-                    int32_t focus_id,
-                    int32_t focus_offset) override;
-  void ShowContextMenu(int32_t id) override;
+  void PerformAction(const ui::AXActionData& data) override;
+
+  // views::AXAuraObjCache::Delegate implementation.
+  void OnChildWindowRemoved(views::AXAuraObjWrapper* parent) override;
 
  private:
   friend struct base::DefaultSingletonTraits<AutomationManagerAura>;
@@ -80,11 +79,11 @@ class AutomationManagerAura : public extensions::AutomationActionAdapter {
   // Holds the active views-based accessibility tree. A tree currently consists
   // of all views descendant to a |Widget| (see |AXTreeSourceViews|).
   // A tree becomes active when an event is fired on a descendant view.
-  scoped_ptr<AXTreeSourceAura> current_tree_;
+  std::unique_ptr<AXTreeSourceAura> current_tree_;
 
   // Serializes incremental updates on the currently active tree
   // |current_tree_|.
-  scoped_ptr<AuraAXTreeSerializer> current_tree_serializer_;
+  std::unique_ptr<AuraAXTreeSerializer> current_tree_serializer_;
 
   bool processing_events_;
 
