@@ -15,7 +15,7 @@
 #include "chrome/browser/ui/webui/ntp/new_tab_ui.h"
 #include "components/bookmarks/browser/bookmark_model.h"
 #include "components/bookmarks/common/bookmark_pref_names.h"
-#include "components/syncable_prefs/pref_service_syncable.h"
+#include "components/sync_preferences/pref_service_syncable.h"
 #include "content/public/browser/navigation_entry.h"
 #include "content/public/browser/web_contents.h"
 
@@ -24,25 +24,15 @@ using bookmarks::BookmarkNode;
 
 namespace {
 
-bool IsNTPWebUI(content::WebContents* web_contents) {
-  content::WebUI* web_ui = NULL;
-  // Use the committed entry so the bookmarks bar disappears at the same time
-  // the page does.
-  if (web_contents->GetController().GetLastCommittedEntry())
-    web_ui = web_contents->GetCommittedWebUI();
-  else
-    web_ui = web_contents->GetWebUI();
-  return web_ui && NewTabUI::FromWebUIController(web_ui->GetController());
-}
-
-bool IsInstantNTP(content::WebContents* web_contents) {
+bool IsNTP(content::WebContents* web_contents) {
   // Use the committed entry so the bookmarks bar disappears at the same time
   // the page does.
   const content::NavigationEntry* entry =
       web_contents->GetController().GetLastCommittedEntry();
   if (!entry)
     entry = web_contents->GetController().GetVisibleEntry();
-  return search::NavEntryIsInstantNTP(web_contents, entry);
+  return (entry && NewTabUI::IsNewTab(entry->GetURL())) ||
+         search::NavEntryIsInstantNTP(web_contents, entry);
 }
 
 }  // namespace
@@ -77,7 +67,7 @@ bool BookmarkTabHelper::ShouldShowBookmarkBar() const {
       !prefs->GetBoolean(bookmarks::prefs::kShowBookmarkBar))
     return false;
 
-  return IsNTPWebUI(web_contents()) || IsInstantNTP(web_contents());
+  return IsNTP(web_contents());
 }
 
 BookmarkTabHelper::BookmarkTabHelper(content::WebContents* web_contents)
@@ -86,9 +76,8 @@ BookmarkTabHelper::BookmarkTabHelper(content::WebContents* web_contents)
       bookmark_model_(NULL),
       delegate_(NULL),
       bookmark_drag_(NULL) {
-  Profile* profile =
-      Profile::FromBrowserContext(web_contents->GetBrowserContext());
-  bookmark_model_ = BookmarkModelFactory::GetForProfile(profile);
+  bookmark_model_ = BookmarkModelFactory::GetForBrowserContext(
+      web_contents->GetBrowserContext());
   if (bookmark_model_)
     bookmark_model_->AddObserver(this);
 }
@@ -144,7 +133,7 @@ void BookmarkTabHelper::DidNavigateMainFrame(
 
 void BookmarkTabHelper::DidStartNavigationToPendingEntry(
     const GURL& /*url*/,
-    content::NavigationController::ReloadType /*reload_type*/) {
+    content::ReloadType /*reload_type*/) {
   UpdateStarredStateForCurrentURL();
 }
 
