@@ -17,7 +17,7 @@
 #include "chrome/browser/ui/search/search_model.h"
 #include "chrome/browser/ui/search/search_tab_helper.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
-#include "chrome/common/instant_types.h"
+#include "chrome/common/search/instant_types.h"
 #include "chrome/common/url_constants.h"
 #include "components/omnibox/browser/omnibox_popup_model.h"
 #include "components/omnibox/browser/omnibox_view.h"
@@ -34,13 +34,6 @@
 
 namespace {
 
-InstantSearchPrerenderer* GetInstantSearchPrerenderer(Profile* profile) {
-  DCHECK(profile);
-  InstantService* instant_service =
-      InstantServiceFactory::GetForProfile(profile);
-  return instant_service ? instant_service->instant_search_prerenderer() : NULL;
-}
-
 // Helper class for posting a task to reload a tab, to avoid doing a re-entrant
 // navigation, since it can be called when starting a navigation. This class
 // makes sure to only execute the reload if the WebContents still exists.
@@ -103,42 +96,38 @@ BrowserInstantController::~BrowserInstantController() {
   instant_service->RemoveObserver(this);
 }
 
-bool BrowserInstantController::OpenInstant(WindowOpenDisposition disposition,
+void BrowserInstantController::OpenInstant(WindowOpenDisposition disposition,
                                            const GURL& url) {
   // Unsupported dispositions.
-  if (disposition == NEW_BACKGROUND_TAB || disposition == NEW_WINDOW ||
-      disposition == NEW_FOREGROUND_TAB)
-    return false;
+  if (disposition == WindowOpenDisposition::NEW_BACKGROUND_TAB ||
+      disposition == WindowOpenDisposition::NEW_WINDOW ||
+      disposition == WindowOpenDisposition::NEW_FOREGROUND_TAB) {
+    return;
+  }
 
   // The omnibox currently doesn't use other dispositions, so we don't attempt
   // to handle them. If you hit this DCHECK file a bug and I'll (sky) add
   // support for the new disposition.
-  DCHECK(disposition == CURRENT_TAB) << disposition;
+  DCHECK(disposition == WindowOpenDisposition::CURRENT_TAB)
+      << static_cast<int>(disposition);
 
   const base::string16& search_terms =
       search::ExtractSearchTermsFromURL(profile(), url);
-  EmbeddedSearchRequestParams request_params(url);
   if (search_terms.empty())
-    return false;
+    return;
 
   InstantSearchPrerenderer* prerenderer =
-      GetInstantSearchPrerenderer(profile());
-  if (prerenderer) {
-    if (prerenderer->CanCommitQuery(GetActiveWebContents(), search_terms)) {
-      // Submit query to render the prefetched results. Browser will swap the
-      // prerendered contents with the active tab contents.
-      prerenderer->Commit(search_terms, request_params);
-      return false;
-    } else {
-      prerenderer->Cancel();
-    }
-  }
+      InstantSearchPrerenderer::GetForProfile(profile());
+  if (!prerenderer)
+    return;
 
-  // If we will not be replacing search terms from this URL, don't send to
-  // InstantController.
-  if (!search::IsQueryExtractionAllowedForURL(profile(), url))
-    return false;
-  return instant_.SubmitQuery(search_terms, request_params);
+  if (prerenderer->CanCommitQuery(GetActiveWebContents(), search_terms)) {
+    // Submit query to render the prefetched results. Browser will swap the
+    // prerendered contents with the active tab contents.
+    prerenderer->Commit(search_terms, EmbeddedSearchRequestParams(url));
+  } else {
+    prerenderer->Cancel();
+  }
 }
 
 Profile* BrowserInstantController::profile() const {
@@ -154,10 +143,8 @@ void BrowserInstantController::ActiveTabChanged() {
 }
 
 void BrowserInstantController::TabDeactivated(content::WebContents* contents) {
-  instant_.TabDeactivated(contents);
-
   InstantSearchPrerenderer* prerenderer =
-      GetInstantSearchPrerenderer(profile());
+      InstantSearchPrerenderer::GetForProfile(profile());
   if (prerenderer)
     prerenderer->Cancel();
 }
@@ -171,9 +158,7 @@ void BrowserInstantController::ModelChanged(
     // Record some actions corresponding to the mode change. Note that to get
     // the full story, it's necessary to look at other UMA actions as well,
     // such as tab switches.
-    if (new_mode.is_search_results())
-      content::RecordAction(base::UserMetricsAction("InstantExtended.ShowSRP"));
-    else if (new_mode.is_ntp())
+    if (new_mode.is_ntp())
       content::RecordAction(base::UserMetricsAction("InstantExtended.ShowNTP"));
 
     instant_.SearchModeChanged(old_state.mode, new_mode);
@@ -204,11 +189,11 @@ void BrowserInstantController::DefaultSearchProviderChanged(
     if (!instant_service->IsInstantProcess(rph->GetID()))
       continue;
 
-    if (google_base_url_domain_changed &&
-        SearchTabHelper::FromWebContents(contents)->model()->mode().is_ntp()) {
+    SearchModel* model = SearchTabHelper::FromWebContents(contents)->model();
+    if (google_base_url_domain_changed && model->mode().is_origin_ntp()) {
+      GURL local_ntp_url(chrome::kChromeSearchLocalNtpUrl);
       // Replace the server NTP with the local NTP.
-      content::NavigationController::LoadURLParams params(
-          search::GetLocalInstantURL(profile()));
+      content::NavigationController::LoadURLParams params(local_ntp_url);
       params.should_replace_current_entry = true;
       params.referrer = content::Referrer();
       params.transition_type = ui::PAGE_TRANSITION_RELOAD;
