@@ -13,12 +13,11 @@
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_commands.h"
 #include "chrome/browser/ui/browser_tabstrip.h"
-#include "chrome/browser/ui/host_desktop.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/test/base/testing_browser_process.h"
 #include "chrome/test/base/testing_profile.h"
 #include "components/bookmarks/test/bookmark_test_helpers.h"
-#include "components/syncable_prefs/pref_service_syncable.h"
+#include "components/sync_preferences/pref_service_syncable.h"
 #include "content/public/test/test_browser_thread_bundle.h"
 
 CocoaProfileTest::CocoaProfileTest()
@@ -57,9 +56,8 @@ void CocoaProfileTest::SetUp() {
   ASSERT_TRUE(profile_manager_.SetUp());
 
   profile_ = profile_manager_.CreateTestingProfile(
-      "Person 1", scoped_ptr<syncable_prefs::PrefServiceSyncable>(),
-      base::UTF8ToUTF16("Person 1"), 0, std::string(),
-      testing_factories_);
+      "Person 1", std::unique_ptr<sync_preferences::PrefServiceSyncable>(),
+      base::UTF8ToUTF16("Person 1"), 0, std::string(), testing_factories_);
   ASSERT_TRUE(profile_);
 
   // TODO(shess): These are needed in case someone creates a browser
@@ -74,7 +72,7 @@ void CocoaProfileTest::SetUp() {
 
   profile_->CreateBookmarkModel(true);
   bookmarks::test::WaitForBookmarkModelToLoad(
-      BookmarkModelFactory::GetForProfile(profile_));
+      BookmarkModelFactory::GetForBrowserContext(profile_));
 
   browser_.reset(CreateBrowser());
   ASSERT_TRUE(browser_.get());
@@ -97,6 +95,5 @@ void CocoaProfileTest::CloseBrowserWindow() {
 }
 
 Browser* CocoaProfileTest::CreateBrowser() {
-  return new Browser(Browser::CreateParams(profile(),
-                                           chrome::GetActiveDesktop()));
+  return new Browser(Browser::CreateParams(profile()));
 }
