@@ -9,6 +9,7 @@
 
 #include "base/mac/scoped_nsobject.h"
 #import "chrome/browser/ui/cocoa/styled_text_field.h"
+#import "chrome/browser/ui/cocoa/themed_window.h"
 #import "chrome/browser/ui/cocoa/url_drop_target.h"
 
 @class AutocompleteTextFieldCell;
@@ -45,16 +46,13 @@ class AutocompleteTextFieldObserver {
   // Return |true| if there is a selection to copy.
   virtual bool CanCopy() = 0;
 
-  // Clears the |pboard| and adds the field's current selection.
-  // Called when the user does a copy or drag.
+  // Creates a pasteboard item from the field's current selection.
+  virtual base::scoped_nsobject<NSPasteboardItem> CreatePasteboardItem() = 0;
+
+  // Copies the pasteboard item returned from |CreatePasteboardItem()| to
+  // |pboard|.
   virtual void CopyToPasteboard(NSPasteboard* pboard) = 0;
 
-  // Returns true if the Show URL option should be available.
-  virtual bool ShouldEnableShowURL() = 0;
-
-  // Shows the underlying URL.  See OmniboxView::ShowURL().
-  virtual void ShowURL() = 0;
-
   // Returns true if the current clipboard text supports paste and go
   // (or paste and search).
   virtual bool CanPasteAndGo() = 0;
@@ -86,6 +84,9 @@ class AutocompleteTextFieldObserver {
   // Called when -insertText: is invoked on the editor.
   virtual void OnInsertText() = 0;
 
+  // Called before a -drawRect: operation.
+  virtual void OnBeforeDrawRect() = 0;
+
   // Called after the completion of a -drawRect: operation.
   virtual void OnDidDrawRect() = 0;
 
@@ -109,15 +110,13 @@ class AutocompleteTextFieldObserver {
   // Called before the text field handles a mouse down event.
   virtual void OnMouseDown(NSInteger button_number) = 0;
 
-  // Returns true if mouse down should select all.
-  virtual bool ShouldSelectAllOnMouseDown() = 0;
-
  protected:
   virtual ~AutocompleteTextFieldObserver() {}
 };
 
 @interface AutocompleteTextField : StyledTextField<NSTextViewDelegate,
-                                                   URLDropTarget> {
+                                                   URLDropTarget,
+                                                   ThemedWindowDrawing> {
  @private
   // Undo manager for this text field.  We use a specific instance rather than
   // the standard undo manager in order to let us clear the undo stack at will.
