@@ -24,6 +24,9 @@ class LocationBarDecoration;
   std::vector<LocationBarDecoration*> leftDecorations_;
   std::vector<LocationBarDecoration*> rightDecorations_;
 
+  // Decorations with tracking areas attached to the AutocompleteTextField.
+  std::vector<LocationBarDecoration*> mouseTrackingDecorations_;
+
   // If YES then the text field will not draw a focus ring or show the insertion
   // pointer.
   BOOL hideFocusState_;
@@ -33,13 +36,20 @@ class LocationBarDecoration;
 
   // Retains the NSEvent that caused the controlView to become firstResponder.
   base::scoped_nsobject<NSEvent> focusEvent_;
+
+  // The coordinate system line width that draws a single pixel line.
+  CGFloat singlePixelLineWidth_;
 }
 
 @property(assign, nonatomic) BOOL isPopupMode;
+@property(assign, nonatomic) CGFloat singlePixelLineWidth;
 
 // Line height used for text in this cell.
 - (CGFloat)lineHeight;
 
+// Remove all of the tracking areas.
+- (void)clearTrackingArea;
+
 // Clear |leftDecorations_| and |rightDecorations_|.
 - (void)clearDecorations;
 
@@ -60,6 +70,18 @@ class LocationBarDecoration;
 - (NSRect)frameForDecoration:(const LocationBarDecoration*)aDecoration
                      inFrame:(NSRect)cellFrame;
 
+// Returns true if it's okay to drop dragged data into the view at the
+// given location.
+- (BOOL)canDropAtLocationInWindow:(NSPoint)location
+                           ofView:(AutocompleteTextField*)controlView;
+
+// Find the decoration under the location in the window. Return |NULL| if
+// there's nothing in the location.
+- (LocationBarDecoration*)decorationForLocationInWindow:(NSPoint)location
+                                                 inRect:(NSRect)cellFrame
+                                                 ofView:(AutocompleteTextField*)
+                                                            field;
+
 // Find the decoration under the event.  |NULL| if |theEvent| is not
 // over anything.
 - (LocationBarDecoration*)decorationForEvent:(NSEvent*)theEvent
@@ -79,15 +101,22 @@ class LocationBarDecoration;
            inRect:(NSRect)cellFrame
            ofView:(AutocompleteTextField*)controlView;
 
+// Called by |AutocompleteTextField| to pass the mouse up event to the omnibox
+// decorations.
+- (void)mouseUp:(NSEvent*)theEvent
+         inRect:(NSRect)cellFrame
+         ofView:(AutocompleteTextField*)controlView;
+
 // Overridden from StyledTextFieldCell to include decorations adjacent
 // to the text area which don't handle mouse clicks themselves.
 // Keyword-search bubble, for instance.
 - (NSRect)textCursorFrameForFrame:(NSRect)cellFrame;
 
-// Setup decoration tooltips on |controlView| by calling
-// |-addToolTip:forRect:|.
-- (void)updateToolTipsInRect:(NSRect)cellFrame
-                      ofView:(AutocompleteTextField*)controlView;
+// Setup decoration tooltips and mouse tracking on |controlView| by calling
+// |-addToolTip:forRect:| and |SetupTrackingArea()|.
+- (void)updateMouseTrackingAndToolTipsInRect:(NSRect)cellFrame
+                                      ofView:
+                                          (AutocompleteTextField*)controlView;
 
 // Gets and sets |hideFocusState|. This allows the text field to have focus but
 // to appear unfocused.
@@ -99,3 +128,11 @@ class LocationBarDecoration;
 - (void)handleFocusEvent:(NSEvent*)event
                   ofView:(AutocompleteTextField*)controlView;
 @end
+
+// Methods which are either only for testing, or only public for testing.
+@interface AutocompleteTextFieldCell (TestingAPI)
+
+// Returns |mouseTrackingDecorations_|.
+- (const std::vector<LocationBarDecoration*>&)mouseTrackingDecorations;
+
+@end
