@@ -8,17 +8,30 @@
 #import "chrome/browser/ui/cocoa/omnibox/omnibox_view_mac.h"
 #include "chrome/browser/ui/location_bar/location_bar_util.h"
 #include "chrome/grit/generated_resources.h"
-#include "grit/theme_resources.h"
+#include "chrome/grit/theme_resources.h"
+#include "skia/ext/skia_utils_mac.h"
 #include "ui/base/l10n/l10n_util_mac.h"
+#include "ui/base/material_design/material_design_controller.h"
+#include "ui/gfx/color_palette.h"
 
 SelectedKeywordDecoration::SelectedKeywordDecoration() {
-  search_image_.reset([OmniboxViewMac::ImageForResource(
-      IDR_KEYWORD_SEARCH_MAGNIFIER) retain]);
-  SetTextColor([NSColor blackColor]);
+  // Note: the unit test
+  // SelectedKeywordDecorationTest.UsesPartialKeywordIfNarrow expects to work
+  // with a fully-initialized SelectedKeywordDecoration (i.e. one that has a
+  // text color and image). During ordinary operation,
+  // LocationBarViewMac::Layout() sets the image before the decoration is
+  // actually used, which the unit test does as well. If
+  // SelectedKeywordDecoration's initialization process changes, the unit test
+  // should also be updated.
+  SetTextColor(GetBackgroundBorderColor());
 }
 
 SelectedKeywordDecoration::~SelectedKeywordDecoration() {}
 
+NSColor* SelectedKeywordDecoration::GetBackgroundBorderColor() {
+  return skia::SkColorToSRGBNSColor(gfx::kGoogleBlue700);
+}
+
 CGFloat SelectedKeywordDecoration::GetWidthForSpace(CGFloat width) {
   const CGFloat full_width =
       GetWidthForImageAndLabel(search_image_, full_string_);
@@ -47,20 +60,25 @@ void SelectedKeywordDecoration::SetKeyword(const base::string16& short_name,
                                            bool is_extension_keyword) {
   const base::string16 min_name(
       location_bar_util::CalculateMinString(short_name));
-  NSString* full_string = is_extension_keyword ?
-      base::SysUTF16ToNSString(short_name) :
-      l10n_util::GetNSStringF(IDS_OMNIBOX_KEYWORD_TEXT, short_name);
+  const int keyword_text_id = IDS_OMNIBOX_KEYWORD_TEXT_MD;
+
+  NSString* full_string =
+      is_extension_keyword
+          ? base::SysUTF16ToNSString(short_name)
+          : l10n_util::GetNSStringF(keyword_text_id, short_name);
 
   // The text will be like "Search <name>:".  "<name>" is a parameter
-  // derived from |short_name|.
+  // derived from |short_name|. If we're using Material Design, the text will
+  // be like "Search <name>" instead.
   full_string_.reset([full_string copy]);
 
   if (min_name.empty()) {
     partial_string_.reset();
   } else {
-    NSString* partial_string = is_extension_keyword ?
-        base::SysUTF16ToNSString(min_name) :
-        l10n_util::GetNSStringF(IDS_OMNIBOX_KEYWORD_TEXT, min_name);
+    NSString* partial_string =
+        is_extension_keyword
+            ? base::SysUTF16ToNSString(min_name)
+            : l10n_util::GetNSStringF(keyword_text_id, min_name);
     partial_string_.reset([partial_string copy]);
   }
 }
