@@ -27,7 +27,8 @@ class TabStripModelObserverBridge : public TabStripModelObserver {
   ~TabStripModelObserverBridge() override;
 
   // Overridden from TabStripModelObserver
-  void TabInsertedAt(content::WebContents* contents,
+  void TabInsertedAt(TabStripModel* tab_strip_model,
+                     content::WebContents* contents,
                      int index,
                      bool foreground) override;
   void TabClosingAt(TabStripModel* tab_strip_model,
@@ -51,7 +52,8 @@ class TabStripModelObserverBridge : public TabStripModelObserver {
                      content::WebContents* old_contents,
                      content::WebContents* new_contents,
                      int index) override;
-  void TabPinnedStateChanged(content::WebContents* contents,
+  void TabPinnedStateChanged(TabStripModel* tab_strip_model,
+                             content::WebContents* contents,
                              int index) override;
   void TabStripEmpty() override;
 
