@@ -10,53 +10,28 @@
 #include "base/macros.h"
 #include "base/strings/utf_string_conversions.h"
 #include "chrome/browser/ui/cocoa/cocoa_test_helper.h"
+#include "chrome/test/base/testing_profile.h"
+#include "content/public/test/test_browser_thread_bundle.h"
+#include "content/public/test/test_web_contents_factory.h"
+#include "net/test/test_certificate_data.h"
 #include "testing/gtest_mac.h"
 
 @interface WebsiteSettingsBubbleController (ExposedForTesting)
-- (NSSegmentedControl*)segmentedControl;
-- (NSTabView*)tabView;
 - (NSView*)permissionsView;
-- (NSView*)connectionTabContentView;
-- (NSImageView*)identityStatusIcon;
-- (NSTextField*)identityStatusDescriptionField;
-- (NSImageView*)connectionStatusIcon;
-- (NSTextField*)connectionStatusDescriptionField;
-- (NSButton*)helpButton;
+- (NSButton*)resetDecisionsButton;
+- (NSButton*)connectionHelpButton;
 @end
 
 @implementation WebsiteSettingsBubbleController (ExposedForTesting)
-- (NSSegmentedControl*)segmentedControl {
-  return segmentedControl_.get();
-}
-- (NSTabView*)tabView {
-  return tabView_.get();
-}
 - (NSView*)permissionsView {
   return permissionsView_;
 }
-
-- (NSView*)connectionTabContentView {
-  return connectionTabContentView_;
+- (NSButton*)resetDecisionsButton {
+  return resetDecisionsButton_;
 }
+- (NSButton*)connectionHelpButton {
 
-- (NSImageView*)identityStatusIcon {
-  return identityStatusIcon_;
-}
-
-- (NSTextField*)identityStatusDescriptionField {
-  return identityStatusDescriptionField_;
-}
-
-- (NSImageView*)connectionStatusIcon {
-  return connectionStatusIcon_;
-}
-
-- (NSTextField*)connectionStatusDescriptionField {
-  return connectionStatusDescriptionField_;
-}
-
-- (NSButton*)helpButton {
-  return helpButton_;
+  return connectionHelpButton_;
 }
 @end
 
@@ -88,33 +63,26 @@ enum PermissionMenuIndices {
 };
 
 const ContentSettingsType kTestPermissionTypes[] = {
-  // NOTE: FULLSCREEN does not support "Always block", so it must appear as
-  // one of the first three permissions.
-  CONTENT_SETTINGS_TYPE_FULLSCREEN,
   CONTENT_SETTINGS_TYPE_IMAGES,
   CONTENT_SETTINGS_TYPE_JAVASCRIPT,
   CONTENT_SETTINGS_TYPE_PLUGINS,
   CONTENT_SETTINGS_TYPE_POPUPS,
   CONTENT_SETTINGS_TYPE_GEOLOCATION,
   CONTENT_SETTINGS_TYPE_NOTIFICATIONS,
-  CONTENT_SETTINGS_TYPE_MOUSELOCK,
   CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC
 };
 
 const ContentSetting kTestSettings[] = {
-  CONTENT_SETTING_DEFAULT,
   CONTENT_SETTING_DEFAULT,
   CONTENT_SETTING_DEFAULT,
   CONTENT_SETTING_ALLOW,
   CONTENT_SETTING_BLOCK,
   CONTENT_SETTING_ALLOW,
   CONTENT_SETTING_BLOCK,
-  CONTENT_SETTING_ALLOW,
   CONTENT_SETTING_BLOCK
 };
 
 const ContentSetting kTestDefaultSettings[] = {
-  CONTENT_SETTING_ALLOW,
   CONTENT_SETTING_BLOCK,
   CONTENT_SETTING_ASK
 };
@@ -124,10 +92,8 @@ const content_settings::SettingSource kTestSettingSources[] = {
   content_settings::SETTING_SOURCE_USER,
   content_settings::SETTING_SOURCE_USER,
   content_settings::SETTING_SOURCE_USER,
-  content_settings::SETTING_SOURCE_USER,
   content_settings::SETTING_SOURCE_POLICY,
   content_settings::SETTING_SOURCE_POLICY,
-  content_settings::SETTING_SOURCE_EXTENSION,
   content_settings::SETTING_SOURCE_EXTENSION
 };
 
@@ -159,9 +125,10 @@ class WebsiteSettingsBubbleControllerTest : public CocoaTest {
     controller_ = [WebsiteSettingsBubbleControllerForTesting alloc];
     [controller_ setDefaultWindowWidth:default_width];
     [controller_ initWithParentWindow:test_window()
-                 websiteSettingsUIBridge:bridge_
-                 webContents:nil
-                 isInternalPage:NO];
+              websiteSettingsUIBridge:bridge_
+                          webContents:web_contents_factory_.CreateWebContents(
+                                          &profile_)
+                                  url:GURL("https://www.google.com")];
     window_ = [controller_ window];
     [controller_ showWindow:nil];
   }
@@ -177,21 +144,20 @@ class WebsiteSettingsBubbleControllerTest : public CocoaTest {
     // coordinate origin. It is into this that all views get placed.
     NSArray* window_subviews = [[window_ contentView] subviews];
     EXPECT_EQ(1U, [window_subviews count]);
-    NSArray* subviews = [[window_subviews lastObject] subviews];
+    NSArray* bubble_subviews = [[window_subviews lastObject] subviews];
+    NSArray* security_section_subviews =
+        [[bubble_subviews firstObject] subviews];
 
     /**
-     *Expect 4 views:
+     *Expect 3 views:
      * - the identity
      * - identity status
      * - security details link
-     * -  segmented control (which implements the tab strip),
-     * - and the tab view.
      */
-    EXPECT_EQ(5U, [subviews count]);
+    EXPECT_EQ(3U, [security_section_subviews count]);
 
     bool desired_result = match_type == TEXT_EQUAL;
-
-    for (NSView* view in subviews) {
+    for (NSView* view in security_section_subviews) {
       if ([view isKindOfClass:[NSTextField class]]) {
         NSTextField* text_field = static_cast<NSTextField*>(view);
         if ([[text_field stringValue] isEqual:text] == desired_result)
@@ -228,109 +194,67 @@ class WebsiteSettingsBubbleControllerTest : public CocoaTest {
       permission_info_list.push_back(info);
     }
     ChosenObjectInfoList chosen_object_info_list;
-    bridge_->SetPermissionInfo(permission_info_list, chosen_object_info_list);
+    bridge_->SetPermissionInfo(permission_info_list,
+                               std::move(chosen_object_info_list));
   }
 
+  content::TestBrowserThreadBundle thread_bundle_;
+  TestingProfile profile_;
+  content::TestWebContentsFactory web_contents_factory_;
+
   WebsiteSettingsBubbleControllerForTesting* controller_;  // Weak, owns self.
   NSWindow* window_;  // Weak, owned by controller.
 };
 
-TEST_F(WebsiteSettingsBubbleControllerTest, BasicIdentity) {
+TEST_F(WebsiteSettingsBubbleControllerTest, ConnectionHelpButton) {
   WebsiteSettingsUI::IdentityInfo info;
-  info.site_identity = std::string("nhl.com");
+  info.site_identity = std::string("example.com");
   info.identity_status = WebsiteSettings::SITE_IDENTITY_STATUS_UNKNOWN;
 
   CreateBubble();
 
-  // Test setting the site identity.
   bridge_->SetIdentityInfo(const_cast<WebsiteSettingsUI::IdentityInfo&>(info));
-  NSTextField* identity_field = FindTextField(TEXT_EQUAL, @"nhl.com");
-  ASSERT_TRUE(identity_field != nil);
 
-  // Test changing the site identity, and ensure that the UI is updated.
-  info.site_identity = std::string("google.com");
-  bridge_->SetIdentityInfo(const_cast<WebsiteSettingsUI::IdentityInfo&>(info));
-  EXPECT_EQ(identity_field, FindTextField(TEXT_EQUAL, @"google.com"));
-
-  // Find the identity status field.
-  NSTextField* identity_status_field =
-      FindTextField(TEXT_NOT_EQUAL, @"google.com");
-  ASSERT_NE(identity_field, identity_status_field);
-
-  // Ensure the text of the identity status field changes when the status does.
-  NSString* status = [identity_status_field stringValue];
-  info.identity_status = WebsiteSettings::SITE_IDENTITY_STATUS_CERT;
-  bridge_->SetIdentityInfo(const_cast<WebsiteSettingsUI::IdentityInfo&>(info));
-  EXPECT_NSNE(status, [identity_status_field stringValue]);
+  EXPECT_EQ([[controller_ connectionHelpButton] action],
+            @selector(openConnectionHelp:));
 }
 
-TEST_F(WebsiteSettingsBubbleControllerTest, SetIdentityInfo) {
+TEST_F(WebsiteSettingsBubbleControllerTest, ResetDecisionsButton) {
   WebsiteSettingsUI::IdentityInfo info;
-  info.site_identity = std::string("nhl.com");
+  info.site_identity = std::string("example.com");
   info.identity_status = WebsiteSettings::SITE_IDENTITY_STATUS_UNKNOWN;
-  info.identity_status_description = std::string("Identity1");
-  info.connection_status = WebsiteSettings::SITE_CONNECTION_STATUS_UNKNOWN;
-  info.connection_status_description = std::string("Connection1");
-  info.cert_id = 0;
 
   CreateBubble();
 
-  // Set the identity, and test that the description fields on the Connection
-  // tab are set properly.
+  // Set identity info, specifying that the button should not be shown.
+  info.show_ssl_decision_revoke_button = false;
   bridge_->SetIdentityInfo(const_cast<WebsiteSettingsUI::IdentityInfo&>(info));
-  EXPECT_NSEQ(@"Identity1",
-              [[controller_ identityStatusDescriptionField] stringValue]);
-  EXPECT_NSEQ(@"Connection1",
-              [[controller_ connectionStatusDescriptionField] stringValue]);
+  EXPECT_EQ([controller_ resetDecisionsButton], nil);
 
-  // Check the contents of the images, and make sure they change after the
-  // status changes.
-
-  NSImage* identity_icon = [[controller_ identityStatusIcon] image];
-  NSImage* connection_icon = [[controller_ connectionStatusIcon] image];
-  // Icons should be the same when they are both unknown.
-  EXPECT_EQ(identity_icon, connection_icon);
-
-  // Ensure that the link button for certificate info is not there -- the
-  // help link button should be the first one found.
-  NSMutableArray* buttons = FindAllSubviewsOfClass(
-      [controller_ connectionTabContentView], [NSButton class]);
-  ASSERT_EQ(1U, [buttons count]);
-  EXPECT_NSEQ([controller_ helpButton], [buttons objectAtIndex:0]);
-
-  // Check that it has a target and action linked up.
-  NSButton* link_button = static_cast<NSButton*>([buttons objectAtIndex:0]);
-  EXPECT_NSEQ(controller_, [link_button target]);
-  EXPECT_TRUE([link_button action] == @selector(showHelpPage:));
-
-  info.identity_status = WebsiteSettings::SITE_IDENTITY_STATUS_CERT;
-  info.connection_status = WebsiteSettings::SITE_CONNECTION_STATUS_ENCRYPTED;
-  info.cert_id = 1;
+  // Set identity info, specifying that the button should be shown.
+  info.certificate = net::X509Certificate::CreateFromBytes(
+      reinterpret_cast<const char*>(google_der), sizeof(google_der));
+  info.show_ssl_decision_revoke_button = true;
   bridge_->SetIdentityInfo(const_cast<WebsiteSettingsUI::IdentityInfo&>(info));
+  EXPECT_NE([controller_ resetDecisionsButton], nil);
 
-  EXPECT_NE(identity_icon, [[controller_ identityStatusIcon] image]);
-  EXPECT_NE(connection_icon, [[controller_ connectionStatusIcon] image]);
+  // Check that clicking the button calls the right selector.
+  EXPECT_EQ([[controller_ resetDecisionsButton] action],
+            @selector(resetCertificateDecisions:));
 
-  // The certificate info button should be there now.
-  buttons = FindAllSubviewsOfClass(
-      [controller_ connectionTabContentView], [NSButton class]);
-  ASSERT_EQ(2U, [buttons count]);
-  EXPECT_NSNE([controller_ helpButton], [buttons objectAtIndex:1]);
-
-  // Check that it has a target and action linked up.
-  link_button = static_cast<NSButton*>([buttons objectAtIndex:1]);
-  EXPECT_NSEQ(controller_, [link_button target]);
-  EXPECT_TRUE([link_button action] == @selector(showCertificateInfo:));
+  // Since the bubble is only created once per identity, we only need to check
+  // the button is *added* when needed. So we don't check that it's removed
+  // when we set an identity with `show_ssl_decision_revoke_button == false`
+  // again.
 }
 
 TEST_F(WebsiteSettingsBubbleControllerTest, SetPermissionInfo) {
   CreateBubble();
   SetTestPermissions();
 
-  // There should be three subviews per permission (an icon, a label and a
-  // select box), plus a text label for the Permission section.
+  // There should be three subviews per permission.
   NSArray* subviews = [[controller_ permissionsView] subviews];
-  EXPECT_EQ(arraysize(kTestPermissionTypes) * 3 + 1, [subviews count]);
+  EXPECT_EQ(arraysize(kTestPermissionTypes) * 3 , [subviews count]);
 
   // Ensure that there is a distinct label for each permission.
   NSMutableSet* labels = [NSMutableSet set];
@@ -338,8 +262,7 @@ TEST_F(WebsiteSettingsBubbleControllerTest, SetPermissionInfo) {
     if ([view isKindOfClass:[NSTextField class]])
       [labels addObject:[static_cast<NSTextField*>(view) stringValue]];
   }
-  // The section header ("Permissions") will also be found, hence the +1.
-  EXPECT_EQ(arraysize(kTestPermissionTypes) + 1, [labels count]);
+  EXPECT_EQ(arraysize(kTestPermissionTypes), [labels count]);
 
   // Ensure that the button labels are distinct, and look for the correct
   // number of disabled buttons.
@@ -356,25 +279,9 @@ TEST_F(WebsiteSettingsBubbleControllerTest, SetPermissionInfo) {
   }
   EXPECT_EQ(arraysize(kTestPermissionTypes), [labels count]);
 
-  // 4 of the buttons should be disabled -- the ones that have a setting source
+  // 3 of the buttons should be disabled -- the ones that have a setting source
   // of SETTING_SOURCE_POLICY or SETTING_SOURCE_EXTENSION.
-  EXPECT_EQ(4, disabled_count);
-}
-
-TEST_F(WebsiteSettingsBubbleControllerTest, SetSelectedTab) {
-  CreateBubble();
-  NSSegmentedControl* segmentedControl = [controller_ segmentedControl];
-  NSTabView* tabView = [controller_ tabView];
-
-  // Test whether SetSelectedTab properly changes both the segmented control
-  // (which implements the tabs) as well as the visible tab contents.
-  // NOTE: This implicitly (and deliberately) tests that the tabs appear in a
-  // specific order: Permissions, Connection.
-  EXPECT_EQ(0, [segmentedControl selectedSegment]);
-  EXPECT_EQ(0, [tabView indexOfTabViewItem:[tabView selectedTabViewItem]]);
-  bridge_->SetSelectedTab(WebsiteSettingsUI::TAB_ID_CONNECTION);
-  EXPECT_EQ(1, [segmentedControl selectedSegment]);
-  EXPECT_EQ(1, [tabView indexOfTabViewItem:[tabView selectedTabViewItem]]);
+  EXPECT_EQ(3, disabled_count);
 }
 
 TEST_F(WebsiteSettingsBubbleControllerTest, WindowWidth) {
