@@ -7,10 +7,11 @@
 #include <stddef.h>
 
 #include "base/command_line.h"
+#include "base/feature_list.h"
 #include "base/macros.h"
-#include "base/prefs/pref_service.h"
 #include "base/stl_util.h"
 #include "base/strings/utf_string_conversions.h"
+#include "chrome/browser/browser_process.h"
 #include "chrome/browser/chrome_notification_types.h"
 #include "chrome/browser/content_settings/chrome_content_settings_utils.h"
 #include "chrome/browser/content_settings/cookie_settings_factory.h"
@@ -18,20 +19,28 @@
 #include "chrome/browser/custom_handlers/protocol_handler_registry.h"
 #include "chrome/browser/custom_handlers/protocol_handler_registry_factory.h"
 #include "chrome/browser/infobars/infobar_service.h"
-#include "chrome/browser/media/media_capture_devices_dispatcher.h"
+#include "chrome/browser/media/webrtc/media_capture_devices_dispatcher.h"
+#include "chrome/browser/permissions/permission_uma_util.h"
+#include "chrome/browser/permissions/permission_util.h"
 #include "chrome/browser/plugins/chrome_plugin_service_filter.h"
+#include "chrome/browser/plugins/plugin_utils.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/blocked_content/popup_blocker_tab_helper.h"
 #include "chrome/browser/ui/collected_cookies_infobar_delegate.h"
 #include "chrome/browser/ui/content_settings/content_setting_bubble_model_delegate.h"
+#include "chrome/common/chrome_features.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/render_messages.h"
 #include "chrome/grit/generated_resources.h"
-#include "components/content_settings/content/common/content_settings_messages.h"
+#include "chrome/grit/theme_resources.h"
 #include "components/content_settings/core/browser/content_settings_utils.h"
 #include "components/content_settings/core/browser/cookie_settings.h"
 #include "components/content_settings/core/common/content_settings.h"
+#include "components/prefs/pref_service.h"
+#include "components/rappor/rappor_utils.h"
+#include "components/strings/grit/components_strings.h"
+#include "components/subresource_filter/content/browser/content_subresource_filter_driver_factory.h"
 #include "components/url_formatter/elide_url.h"
 #include "content/public/browser/notification_service.h"
 #include "content/public/browser/render_frame_host.h"
@@ -41,8 +50,6 @@
 #include "content/public/browser/web_contents.h"
 #include "content/public/browser/web_contents_delegate.h"
 #include "content/public/common/origin_util.h"
-#include "grit/components_strings.h"
-#include "grit/theme_resources.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 #include "ui/resources/grit/ui_resources.h"
@@ -110,7 +117,7 @@ ContentSettingSimpleBubbleModel::ContentSettingSimpleBubbleModel(
   // Notifications do not have a bubble.
   DCHECK_NE(content_type, CONTENT_SETTINGS_TYPE_NOTIFICATIONS);
   SetTitle();
-  SetManageLink();
+  SetManageText();
   SetCustomLink();
 }
 
@@ -120,11 +127,10 @@ ContentSettingSimpleBubbleModel*
 }
 
 void ContentSettingSimpleBubbleModel::SetTitle() {
-  TabSpecificContentSettings* content_settings = NULL;
-  if (web_contents()) {
-    content_settings =
-        TabSpecificContentSettings::FromWebContents(web_contents());
-  }
+  TabSpecificContentSettings* content_settings =
+      web_contents()
+          ? TabSpecificContentSettings::FromWebContents(web_contents())
+          : nullptr;
 
   static const ContentSettingsTypeIdEntry kBlockedTitleIDs[] = {
     {CONTENT_SETTINGS_TYPE_COOKIES, IDS_BLOCKED_COOKIES_TITLE},
@@ -144,20 +150,19 @@ void ContentSettingSimpleBubbleModel::SetTitle() {
     {CONTENT_SETTINGS_TYPE_PPAPI_BROKER, IDS_ALLOWED_PPAPI_BROKER_TITLE},
     {CONTENT_SETTINGS_TYPE_AUTOMATIC_DOWNLOADS, IDS_ALLOWED_DOWNLOAD_TITLE},
   };
-  const ContentSettingsTypeIdEntry *title_ids = kBlockedTitleIDs;
+  const ContentSettingsTypeIdEntry* title_ids = kBlockedTitleIDs;
   size_t num_title_ids = arraysize(kBlockedTitleIDs);
   if (content_settings && content_settings->IsContentAllowed(content_type()) &&
       !content_settings->IsContentBlocked(content_type())) {
     title_ids = kAccessedTitleIDs;
     num_title_ids = arraysize(kAccessedTitleIDs);
   }
-  int title_id =
-      GetIdForContentType(title_ids, num_title_ids, content_type());
+  int title_id = GetIdForContentType(title_ids, num_title_ids, content_type());
   if (title_id)
-    set_title(l10n_util::GetStringUTF8(title_id));
+    set_title(l10n_util::GetStringUTF16(title_id));
 }
 
-void ContentSettingSimpleBubbleModel::SetManageLink() {
+void ContentSettingSimpleBubbleModel::SetManageText() {
   static const ContentSettingsTypeIdEntry kLinkIDs[] = {
     {CONTENT_SETTINGS_TYPE_COOKIES, IDS_BLOCKED_COOKIES_LINK},
     {CONTENT_SETTINGS_TYPE_IMAGES, IDS_BLOCKED_IMAGES_LINK},
@@ -171,19 +176,23 @@ void ContentSettingSimpleBubbleModel::SetManageLink() {
     {CONTENT_SETTINGS_TYPE_AUTOMATIC_DOWNLOADS, IDS_BLOCKED_DOWNLOADS_LINK},
     {CONTENT_SETTINGS_TYPE_MIDI_SYSEX, IDS_MIDI_SYSEX_BUBBLE_MANAGE_LINK},
   };
-  set_manage_link(l10n_util::GetStringUTF8(
+  set_manage_text(l10n_util::GetStringUTF8(
       GetIdForContentType(kLinkIDs, arraysize(kLinkIDs), content_type())));
 }
 
 void ContentSettingSimpleBubbleModel::OnManageLinkClicked() {
   if (delegate())
     delegate()->ShowContentSettingsPage(content_type());
+
+  if (content_type() == CONTENT_SETTINGS_TYPE_PLUGINS) {
+    content_settings::RecordPluginsAction(
+        content_settings::PLUGINS_ACTION_CLICKED_MANAGE_PLUGIN_BLOCKING);
+  }
 }
 
 void ContentSettingSimpleBubbleModel::SetCustomLink() {
   static const ContentSettingsTypeIdEntry kCustomIDs[] = {
     {CONTENT_SETTINGS_TYPE_COOKIES, IDS_BLOCKED_COOKIES_INFO},
-    {CONTENT_SETTINGS_TYPE_PLUGINS, IDS_BLOCKED_PLUGINS_LOAD_ALL},
     {CONTENT_SETTINGS_TYPE_MIXEDSCRIPT, IDS_ALLOW_INSECURE_CONTENT_BUTTON},
   };
   int custom_link_id =
@@ -216,6 +225,8 @@ class ContentSettingSingleRadioGroup : public ContentSettingSimpleBubbleModel {
 
   ContentSetting block_setting_;
   int selected_item_;
+
+  DISALLOW_COPY_AND_ASSIGN(ContentSettingSingleRadioGroup);
 };
 
 ContentSettingSingleRadioGroup::ContentSettingSingleRadioGroup(
@@ -223,7 +234,9 @@ ContentSettingSingleRadioGroup::ContentSettingSingleRadioGroup(
     WebContents* web_contents,
     Profile* profile,
     ContentSettingsType content_type)
-    : ContentSettingSimpleBubbleModel(delegate, web_contents, profile,
+    : ContentSettingSimpleBubbleModel(delegate,
+                                      web_contents,
+                                      profile,
                                       content_type),
       block_setting_(CONTENT_SETTING_BLOCK),
       selected_item_(0) {
@@ -248,17 +261,15 @@ bool ContentSettingSingleRadioGroup::settings_changed() const {
 // content type and setting the default value based on the content setting.
 void ContentSettingSingleRadioGroup::SetRadioGroup() {
   GURL url = web_contents()->GetURL();
-  base::string16 display_host = url_formatter::FormatUrlForSecurityDisplay(
-      url, profile()->GetPrefs()->GetString(prefs::kAcceptLanguages));
+  base::string16 display_host =
+      url_formatter::FormatUrlForSecurityDisplay(url);
   if (display_host.empty())
     display_host = base::ASCIIToUTF16(url.spec());
 
   TabSpecificContentSettings* content_settings =
       TabSpecificContentSettings::FromWebContents(web_contents());
-  bool allowed =
-      !content_settings->IsContentBlocked(content_type());
-  DCHECK(!allowed ||
-         content_settings->IsContentAllowed(content_type()));
+  bool allowed = !content_settings->IsContentBlocked(content_type());
+  DCHECK(!allowed || content_settings->IsContentAllowed(content_type()));
 
   RadioGroup radio_group;
   radio_group.url = url;
@@ -323,7 +334,6 @@ void ContentSettingSingleRadioGroup::SetRadioGroup() {
   radio_group.radio_items.push_back(radio_block_label);
   ContentSetting setting;
   SettingSource setting_source = SETTING_SOURCE_NONE;
-  bool setting_is_wildcard = false;
 
   if (content_type() == CONTENT_SETTINGS_TYPE_COOKIES) {
     content_settings::CookieSettings* cookie_settings =
@@ -334,23 +344,13 @@ void ContentSettingSingleRadioGroup::SetRadioGroup() {
     SettingInfo info;
     HostContentSettingsMap* map =
         HostContentSettingsMapFactory::GetForProfile(profile());
-    scoped_ptr<base::Value> value =
+    std::unique_ptr<base::Value> value =
         map->GetWebsiteSetting(url, url, content_type(), std::string(), &info);
     setting = content_settings::ValueToContentSetting(value.get());
     setting_source = info.source;
-    setting_is_wildcard =
-        info.primary_pattern == ContentSettingsPattern::Wildcard() &&
-        info.secondary_pattern == ContentSettingsPattern::Wildcard();
   }
 
-  if (content_type() == CONTENT_SETTINGS_TYPE_PLUGINS &&
-      setting == CONTENT_SETTING_ALLOW &&
-      setting_is_wildcard) {
-    // In the corner case of unrecognized plugins (which are now blocked by
-    // default) we indicate the blocked state in the UI and allow the user to
-    // whitelist.
-    radio_group.default_item = 1;
-  } else if (setting == CONTENT_SETTING_ALLOW) {
+  if (setting == CONTENT_SETTING_ALLOW) {
     radio_group.default_item = kAllowButtonIndex;
     // |block_setting_| is already set to |CONTENT_SETTING_BLOCK|.
   } else {
@@ -358,25 +358,21 @@ void ContentSettingSingleRadioGroup::SetRadioGroup() {
     block_setting_ = setting;
   }
 
-  set_setting_is_managed(setting_source != SETTING_SOURCE_USER &&
-                         setting != CONTENT_SETTING_ASK);
-  if (setting_source != SETTING_SOURCE_USER) {
-    set_radio_group_enabled(false);
-  } else {
-    set_radio_group_enabled(true);
-  }
+  set_radio_group_enabled(setting_source == SETTING_SOURCE_USER);
+
   selected_item_ = radio_group.default_item;
   set_radio_group(radio_group);
 }
 
 void ContentSettingSingleRadioGroup::SetNarrowestContentSetting(
     ContentSetting setting) {
-  if (profile()) {
-    HostContentSettingsMapFactory::GetForProfile(profile())
-        ->SetNarrowestContentSetting(bubble_content().radio_group.url,
-                                     bubble_content().radio_group.url,
-                                     content_type(), setting);
-  }
+  if (!profile())
+    return;
+
+  auto* map = HostContentSettingsMapFactory::GetForProfile(profile());
+  map->SetNarrowestContentSetting(bubble_content().radio_group.url,
+                                  bubble_content().radio_group.url,
+                                  content_type(), setting);
 }
 
 void ContentSettingSingleRadioGroup::OnRadioClicked(int radio_index) {
@@ -390,11 +386,12 @@ class ContentSettingCookiesBubbleModel : public ContentSettingSingleRadioGroup {
   ContentSettingCookiesBubbleModel(Delegate* delegate,
                                    WebContents* web_contents,
                                    Profile* profile);
-
   ~ContentSettingCookiesBubbleModel() override;
 
  private:
   void OnCustomLinkClicked() override;
+
+  DISALLOW_COPY_AND_ASSIGN(ContentSettingCookiesBubbleModel);
 };
 
 ContentSettingCookiesBubbleModel::ContentSettingCookiesBubbleModel(
@@ -430,34 +427,59 @@ void ContentSettingCookiesBubbleModel::OnCustomLinkClicked() {
 
 // ContentSettingPluginBubbleModel ---------------------------------------------
 
-class ContentSettingPluginBubbleModel : public ContentSettingSingleRadioGroup {
+class ContentSettingPluginBubbleModel : public ContentSettingSimpleBubbleModel {
  public:
   ContentSettingPluginBubbleModel(Delegate* delegate,
                                   WebContents* web_contents,
                                   Profile* profile);
 
-  ~ContentSettingPluginBubbleModel() override;
-
  private:
   void OnLearnMoreLinkClicked() override;
   void OnCustomLinkClicked() override;
+
+  void RunPluginsOnPage();
+
+  DISALLOW_COPY_AND_ASSIGN(ContentSettingPluginBubbleModel);
 };
 
 ContentSettingPluginBubbleModel::ContentSettingPluginBubbleModel(
     Delegate* delegate,
     WebContents* web_contents,
     Profile* profile)
-    : ContentSettingSingleRadioGroup(delegate,
-                                     web_contents,
-                                     profile,
-                                     CONTENT_SETTINGS_TYPE_PLUGINS) {
-  // Disable the "Run all plugins this time" link if the setting is managed and
-  // can't be controlled by the user or if the user already clicked on the link
-  // and ran all plugins.
-  set_custom_link_enabled(!setting_is_managed() &&
-                          web_contents &&
-                          TabSpecificContentSettings::FromWebContents(
-                              web_contents)->load_plugins_link_enabled());
+    : ContentSettingSimpleBubbleModel(delegate,
+                                      web_contents,
+                                      profile,
+                                      CONTENT_SETTINGS_TYPE_PLUGINS) {
+  SettingInfo info;
+  HostContentSettingsMap* map =
+      HostContentSettingsMapFactory::GetForProfile(profile);
+  GURL url = web_contents->GetURL();
+  std::unique_ptr<base::Value> value =
+      map->GetWebsiteSetting(url, url, content_type(), std::string(), &info);
+  ContentSetting setting = content_settings::ValueToContentSetting(value.get());
+
+  // If the setting is not managed by the user, hide the "Manage..." link.
+  if (info.source != SETTING_SOURCE_USER)
+    set_manage_text(std::string());
+
+  // The user cannot manually run Flash on the BLOCK setting when either holds:
+  //  - The setting is from Policy. User cannot override admin intent.
+  //  - HTML By Default is on - Flash has been hidden from the plugin list, so
+  //    it's impossible to dynamically run the nonexistent plugin.
+  bool run_blocked = setting == CONTENT_SETTING_BLOCK &&
+                     (info.source != SETTING_SOURCE_USER ||
+                      PluginUtils::ShouldPreferHtmlOverPlugins(map));
+
+  if (!run_blocked) {
+    set_custom_link(l10n_util::GetStringUTF8(IDS_BLOCKED_PLUGINS_LOAD_ALL));
+    // Disable the "Run all plugins this time" link if the user already clicked
+    // on the link and ran all plugins.
+    set_custom_link_enabled(
+        web_contents &&
+        TabSpecificContentSettings::FromWebContents(web_contents)
+            ->load_plugins_link_enabled());
+  }
+
   // Build blocked plugin list.
   if (web_contents) {
     TabSpecificContentSettings* content_settings =
@@ -475,21 +497,28 @@ ContentSettingPluginBubbleModel::ContentSettingPluginBubbleModel(
   }
 
   set_learn_more_link(l10n_util::GetStringUTF8(IDS_LEARN_MORE));
-}
 
-ContentSettingPluginBubbleModel::~ContentSettingPluginBubbleModel() {
-  // If the user elected to allow all plugins then run plugins at this time.
-  if (settings_changed() && selected_item() == kAllowButtonIndex)
-    OnCustomLinkClicked();
+  content_settings::RecordPluginsAction(
+      content_settings::PLUGINS_ACTION_DISPLAYED_BUBBLE);
 }
 
 void ContentSettingPluginBubbleModel::OnLearnMoreLinkClicked() {
   if (delegate())
     delegate()->ShowLearnMorePage(CONTENT_SETTINGS_TYPE_PLUGINS);
+
+  content_settings::RecordPluginsAction(
+      content_settings::PLUGINS_ACTION_CLICKED_LEARN_MORE);
 }
 
 void ContentSettingPluginBubbleModel::OnCustomLinkClicked() {
   content::RecordAction(UserMetricsAction("ClickToPlay_LoadAll_Bubble"));
+  content_settings::RecordPluginsAction(
+      content_settings::PLUGINS_ACTION_CLICKED_RUN_ALL_PLUGINS_THIS_TIME);
+
+  RunPluginsOnPage();
+}
+
+void ContentSettingPluginBubbleModel::RunPluginsOnPage() {
   // Web contents can be NULL if the tab was closed while the plugins
   // settings bubble is visible.
   if (!web_contents())
@@ -500,8 +529,8 @@ void ContentSettingPluginBubbleModel::OnCustomLinkClicked() {
       web_contents(), true, std::string());
 #endif
   set_custom_link_enabled(false);
-  TabSpecificContentSettings::FromWebContents(web_contents())->
-      set_load_plugins_link_enabled(false);
+  TabSpecificContentSettings::FromWebContents(web_contents())
+      ->set_load_plugins_link_enabled(false);
 }
 
 // ContentSettingPopupBubbleModel ----------------------------------------------
@@ -519,6 +548,8 @@ class ContentSettingPopupBubbleModel : public ContentSettingSingleRadioGroup {
   int32_t item_id_from_item_index(int index) const {
     return bubble_content().list_items[index].item_id;
   }
+
+  DISALLOW_COPY_AND_ASSIGN(ContentSettingPopupBubbleModel);
 };
 
 ContentSettingPopupBubbleModel::ContentSettingPopupBubbleModel(
@@ -529,28 +560,28 @@ ContentSettingPopupBubbleModel::ContentSettingPopupBubbleModel(
                                      web_contents,
                                      profile,
                                      CONTENT_SETTINGS_TYPE_POPUPS) {
-  if (web_contents) {
-    // Build blocked popup list.
-    std::map<int32_t, GURL> blocked_popups =
-        PopupBlockerTabHelper::FromWebContents(web_contents)
-            ->GetBlockedPopupRequests();
-    for (const std::pair<int32_t, GURL>& blocked_popup : blocked_popups) {
-      std::string title(blocked_popup.second.spec());
-      // The pop-up may not have a valid URL.
-      if (title.empty())
-        title = l10n_util::GetStringUTF8(IDS_TAB_LOADING_TITLE);
-      ListItem popup_item(ui::ResourceBundle::GetSharedInstance().GetImageNamed(
-                              IDR_DEFAULT_FAVICON),
-                          title, true, blocked_popup.first);
-      add_list_item(popup_item);
-    }
+  if (!web_contents)
+    return;
+
+  // Build blocked popup list.
+  auto* helper = PopupBlockerTabHelper::FromWebContents(web_contents);
+  std::map<int32_t, GURL> blocked_popups = helper->GetBlockedPopupRequests();
+  for (const std::pair<int32_t, GURL>& blocked_popup : blocked_popups) {
+    std::string title(blocked_popup.second.spec());
+    // The pop-up may not have a valid URL.
+    if (title.empty())
+      title = l10n_util::GetStringUTF8(IDS_TAB_LOADING_TITLE);
+    ListItem popup_item(ui::ResourceBundle::GetSharedInstance().GetImageNamed(
+                            IDR_DEFAULT_FAVICON),
+                        title, true, blocked_popup.first);
+    add_list_item(popup_item);
   }
 }
 
 void ContentSettingPopupBubbleModel::OnListItemClicked(int index) {
   if (web_contents()) {
-    PopupBlockerTabHelper::FromWebContents(web_contents())
-        ->ShowBlockedPopup(item_id_from_item_index(index));
+    auto* helper = PopupBlockerTabHelper::FromWebContents(web_contents());
+    helper->ShowBlockedPopup(item_id_from_item_index(index));
   }
 }
 
@@ -582,7 +613,7 @@ ContentSettingMediaStreamBubbleModel::ContentSettingMediaStreamBubbleModel(
   SetTitle();
   SetRadioGroup();
   SetMediaMenus();
-  SetManageLink();
+  SetManageText();
   SetCustomLink();
 }
 
@@ -592,19 +623,15 @@ ContentSettingMediaStreamBubbleModel::~ContentSettingMediaStreamBubbleModel() {
   if (!web_contents())
     return;
 
-  for (const std::pair<content::MediaStreamType, MediaMenu>& media_menu :
-       bubble_content().media_menus) {
-    if (media_menu.second.selected_device.id !=
-        media_menu.second.default_device.id) {
-      UpdateDefaultDeviceForType(media_menu.first,
-                                 media_menu.second.selected_device.id);
-    }
+  for (const auto& media_menu : bubble_content().media_menus) {
+    const MediaMenu& menu = media_menu.second;
+    if (menu.selected_device.id != menu.default_device.id)
+      UpdateDefaultDeviceForType(media_menu.first, menu.selected_device.id);
   }
 
   // Update the media settings if the radio button selection was changed.
-  if (selected_item_ != bubble_content().radio_group.default_item) {
+  if (selected_item_ != bubble_content().radio_group.default_item)
     UpdateSettings(radio_item_setting_[selected_item_]);
-  }
 }
 
 ContentSettingMediaStreamBubbleModel*
@@ -647,7 +674,7 @@ void ContentSettingMediaStreamBubbleModel::SetTitle() {
   } else if (CameraAccessed()) {
     title_id = IDS_CAMERA_ACCESSED;
   }
-  set_title(l10n_util::GetStringUTF8(title_id));
+  set_title(l10n_util::GetStringUTF16(title_id));
 }
 
 void ContentSettingMediaStreamBubbleModel::SetRadioGroup() {
@@ -658,8 +685,7 @@ void ContentSettingMediaStreamBubbleModel::SetRadioGroup() {
   radio_group.url = url;
 
   base::string16 display_host_utf16 =
-      url_formatter::FormatUrlForSecurityDisplay(
-          url, profile()->GetPrefs()->GetString(prefs::kAcceptLanguages));
+      url_formatter::FormatUrlForSecurityDisplay(url);
   std::string display_host(base::UTF16ToUTF8(display_host_utf16));
   if (display_host.empty())
     display_host = url.spec();
@@ -721,30 +747,33 @@ void ContentSettingMediaStreamBubbleModel::SetRadioGroup() {
 
 void ContentSettingMediaStreamBubbleModel::UpdateSettings(
     ContentSetting setting) {
-  if (profile()) {
-    HostContentSettingsMap* content_settings =
-        HostContentSettingsMapFactory::GetForProfile(profile());
-    TabSpecificContentSettings* tab_content_settings =
-        TabSpecificContentSettings::FromWebContents(web_contents());
-    // The same patterns must be used as in other places (e.g. the infobar) in
-    // order to override the existing rule. Otherwise a new rule is created.
-    // TODO(markusheintz): Extract to a helper so that there is only a single
-    // place to touch.
-    ContentSettingsPattern primary_pattern =
-        ContentSettingsPattern::FromURLNoWildcard(
-            tab_content_settings->media_stream_access_origin());
-    ContentSettingsPattern secondary_pattern =
-        ContentSettingsPattern::Wildcard();
-    if (MicrophoneAccessed()) {
-      content_settings->SetContentSetting(
-          primary_pattern, secondary_pattern,
-          CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC, std::string(), setting);
-    }
-    if (CameraAccessed()) {
-      content_settings->SetContentSetting(
-          primary_pattern, secondary_pattern,
-          CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA, std::string(), setting);
-    }
+  if (!profile())
+    return;
+
+  TabSpecificContentSettings* tab_content_settings =
+      TabSpecificContentSettings::FromWebContents(web_contents());
+  // The same urls must be used as in other places (e.g. the infobar) in
+  // order to override the existing rule. Otherwise a new rule is created.
+  // TODO(markusheintz): Extract to a helper so that there is only a single
+  // place to touch.
+  HostContentSettingsMap* map =
+      HostContentSettingsMapFactory::GetForProfile(profile());
+  if (MicrophoneAccessed()) {
+    PermissionUtil::ScopedRevocationReporter scoped_revocation_reporter(
+        profile(), tab_content_settings->media_stream_access_origin(), GURL(),
+        CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC, PermissionSourceUI::PAGE_ACTION);
+    map->SetContentSettingDefaultScope(
+        tab_content_settings->media_stream_access_origin(), GURL(),
+        CONTENT_SETTINGS_TYPE_MEDIASTREAM_MIC, std::string(), setting);
+  }
+  if (CameraAccessed()) {
+    PermissionUtil::ScopedRevocationReporter scoped_revocation_reporter(
+        profile(), tab_content_settings->media_stream_access_origin(), GURL(),
+        CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA,
+        PermissionSourceUI::PAGE_ACTION);
+    map->SetContentSettingDefaultScope(
+        tab_content_settings->media_stream_access_origin(), GURL(),
+        CONTENT_SETTINGS_TYPE_MEDIASTREAM_CAMERA, std::string(), setting);
   }
 }
 
@@ -823,7 +852,7 @@ void ContentSettingMediaStreamBubbleModel::SetMediaMenus() {
   }
 }
 
-void ContentSettingMediaStreamBubbleModel::SetManageLink() {
+void ContentSettingMediaStreamBubbleModel::SetManageText() {
   // By default, the manage link refers to both media types. We only need
   // to change the link text if only one media type was accessed.
   int link_id;
@@ -838,7 +867,7 @@ void ContentSettingMediaStreamBubbleModel::SetManageLink() {
     return;
   }
 
-  set_manage_link(l10n_util::GetStringUTF8(link_id));
+  set_manage_text(l10n_util::GetStringUTF8(link_id));
 }
 
 void ContentSettingMediaStreamBubbleModel::SetCustomLink() {
@@ -884,6 +913,8 @@ class ContentSettingDomainListBubbleModel
   void MaybeAddDomainList(const std::set<std::string>& hosts, int title_id);
   void SetDomainsAndCustomLink();
   void OnCustomLinkClicked() override;
+
+  DISALLOW_COPY_AND_ASSIGN(ContentSettingDomainListBubbleModel);
 };
 
 ContentSettingDomainListBubbleModel::ContentSettingDomainListBubbleModel(
@@ -945,15 +976,15 @@ void ContentSettingDomainListBubbleModel::OnCustomLinkClicked() {
       TabSpecificContentSettings::FromWebContents(web_contents());
   const ContentSettingsUsagesState::StateMap& state_map =
       content_settings->geolocation_usages_state().state_map();
-  HostContentSettingsMap* settings_map =
+  HostContentSettingsMap* map =
       HostContentSettingsMapFactory::GetForProfile(profile());
-
   for (const std::pair<GURL, ContentSetting>& map_entry : state_map) {
-    settings_map->SetContentSetting(
-        ContentSettingsPattern::FromURLNoWildcard(map_entry.first),
-        ContentSettingsPattern::FromURLNoWildcard(embedder_url),
-        CONTENT_SETTINGS_TYPE_GEOLOCATION, std::string(),
-        CONTENT_SETTING_DEFAULT);
+    PermissionUtil::ScopedRevocationReporter(
+        profile(), map_entry.first, embedder_url,
+        CONTENT_SETTINGS_TYPE_GEOLOCATION, PermissionSourceUI::PAGE_ACTION);
+    map->SetContentSettingDefaultScope(map_entry.first, embedder_url,
+                                       CONTENT_SETTINGS_TYPE_GEOLOCATION,
+                                       std::string(), CONTENT_SETTING_DEFAULT);
   }
 }
 
@@ -970,6 +1001,8 @@ class ContentSettingMixedScriptBubbleModel
 
  private:
   void OnCustomLinkClicked() override;
+
+  DISALLOW_COPY_AND_ASSIGN(ContentSettingMixedScriptBubbleModel);
 };
 
 ContentSettingMixedScriptBubbleModel::ContentSettingMixedScriptBubbleModel(
@@ -987,7 +1020,10 @@ ContentSettingMixedScriptBubbleModel::ContentSettingMixedScriptBubbleModel(
 }
 
 void ContentSettingMixedScriptBubbleModel::OnCustomLinkClicked() {
-  DCHECK(web_contents());
+  DCHECK(rappor_service());
+  if (!web_contents())
+    return;
+
   web_contents()->SendToAllFrames(
       new ChromeViewMsg_SetAllowRunningInsecureContent(MSG_ROUTING_NONE, true));
   web_contents()->GetMainFrame()->Send(new ChromeViewMsg_ReloadFrame(
@@ -995,8 +1031,9 @@ void ContentSettingMixedScriptBubbleModel::OnCustomLinkClicked() {
 
   content_settings::RecordMixedScriptAction(
       content_settings::MIXED_SCRIPT_ACTION_CLICKED_ALLOW);
-  content_settings::RecordMixedScriptActionWithRAPPOR(
-      content_settings::MIXED_SCRIPT_ACTION_CLICKED_ALLOW,
+
+  rappor::SampleDomainAndRegistryFromGURL(
+      rappor_service(), "ContentSettings.MixedScript.UserClickedAllow",
       web_contents()->GetLastCommittedURL());
 }
 
@@ -1012,6 +1049,7 @@ ContentSettingRPHBubbleModel::ContentSettingRPHBubbleModel(
                                       profile,
                                       CONTENT_SETTINGS_TYPE_PROTOCOL_HANDLERS),
       selected_item_(0),
+      interacted_(false),
       registry_(registry),
       pending_handler_(ProtocolHandler::EmptyProtocolHandler()),
       previous_handler_(ProtocolHandler::EmptyProtocolHandler()) {
@@ -1033,12 +1071,12 @@ ContentSettingRPHBubbleModel::ContentSettingRPHBubbleModel(
 
   // Note that we ignore the |title| parameter.
   if (previous_handler_.IsEmpty()) {
-    set_title(l10n_util::GetStringFUTF8(
+    set_title(l10n_util::GetStringFUTF16(
         IDS_REGISTER_PROTOCOL_HANDLER_CONFIRM,
         base::UTF8ToUTF16(pending_handler_.url().host()),
         protocol));
   } else {
-    set_title(l10n_util::GetStringFUTF8(
+    set_title(l10n_util::GetStringFUTF16(
         IDS_REGISTER_PROTOCOL_HANDLER_CONFIRM_REPLACE,
         base::UTF8ToUTF16(pending_handler_.url().host()),
         protocol,
@@ -1073,10 +1111,25 @@ ContentSettingRPHBubbleModel::ContentSettingRPHBubbleModel(
   set_radio_group(radio_group);
 }
 
+ContentSettingRPHBubbleModel::~ContentSettingRPHBubbleModel() {
+  if (!web_contents() || !interacted_)
+    return;
+
+  // The user has one chance to deal with the RPH content setting UI,
+  // then we remove it.
+  auto* settings = TabSpecificContentSettings::FromWebContents(web_contents());
+  settings->ClearPendingProtocolHandler();
+  content::NotificationService::current()->Notify(
+      chrome::NOTIFICATION_WEB_CONTENT_SETTINGS_CHANGED,
+      content::Source<WebContents>(web_contents()),
+      content::NotificationService::NoDetails());
+}
+
 void ContentSettingRPHBubbleModel::OnRadioClicked(int radio_index) {
   if (selected_item_ == radio_index)
     return;
 
+  interacted_ = true;
   selected_item_ = radio_index;
 
   if (radio_index == RPH_ALLOW)
@@ -1090,17 +1143,13 @@ void ContentSettingRPHBubbleModel::OnRadioClicked(int radio_index) {
 }
 
 void ContentSettingRPHBubbleModel::OnDoneClicked() {
-  // The user has one chance to deal with the RPH content setting UI,
-  // then we remove it.
-  TabSpecificContentSettings::FromWebContents(web_contents())->
-      ClearPendingProtocolHandler();
-  content::NotificationService::current()->Notify(
-      chrome::NOTIFICATION_WEB_CONTENT_SETTINGS_CHANGED,
-      content::Source<WebContents>(web_contents()),
-      content::NotificationService::NoDetails());
+  interacted_ = true;
 }
 
 void ContentSettingRPHBubbleModel::RegisterProtocolHandler() {
+  if (!web_contents())
+    return;
+
   // A no-op if the handler hasn't been ignored, but needed in case the user
   // selects sequences like register/ignore/register.
   registry_->RemoveIgnoredHandler(pending_handler_);
@@ -1111,16 +1160,22 @@ void ContentSettingRPHBubbleModel::RegisterProtocolHandler() {
 }
 
 void ContentSettingRPHBubbleModel::UnregisterProtocolHandler() {
+  if (!web_contents())
+    return;
+
   registry_->OnDenyRegisterProtocolHandler(pending_handler_);
-  TabSpecificContentSettings::FromWebContents(web_contents())->
-      set_pending_protocol_handler_setting(CONTENT_SETTING_BLOCK);
+  auto* settings = TabSpecificContentSettings::FromWebContents(web_contents());
+  settings->set_pending_protocol_handler_setting(CONTENT_SETTING_BLOCK);
   ClearOrSetPreviousHandler();
 }
 
 void ContentSettingRPHBubbleModel::IgnoreProtocolHandler() {
+  if (!web_contents())
+    return;
+
   registry_->OnIgnoreRegisterProtocolHandler(pending_handler_);
-  TabSpecificContentSettings::FromWebContents(web_contents())->
-      set_pending_protocol_handler_setting(CONTENT_SETTING_DEFAULT);
+  auto* settings = TabSpecificContentSettings::FromWebContents(web_contents());
+  settings->set_pending_protocol_handler_setting(CONTENT_SETTING_DEFAULT);
   ClearOrSetPreviousHandler();
 }
 
@@ -1132,6 +1187,49 @@ void ContentSettingRPHBubbleModel::ClearOrSetPreviousHandler() {
   }
 }
 
+// ContentSettingSubresourceFilterBubbleModel ----------------------------------
+
+ContentSettingSubresourceFilterBubbleModel::
+    ContentSettingSubresourceFilterBubbleModel(Delegate* delegate,
+                                               WebContents* web_contents,
+                                               Profile* profile)
+    : ContentSettingBubbleModel(delegate, web_contents, profile) {
+  SetTitle();
+  SetMessage();
+  SetManageText();
+}
+
+ContentSettingSubresourceFilterBubbleModel::
+    ~ContentSettingSubresourceFilterBubbleModel() {}
+
+void ContentSettingSubresourceFilterBubbleModel::SetTitle() {
+  set_title(
+      l10n_util::GetStringUTF16(IDS_FILTERED_DECEPTIVE_CONTENT_PROMPT_TITLE));
+}
+
+void ContentSettingSubresourceFilterBubbleModel::SetManageText() {
+  set_manage_text(
+      l10n_util::GetStringUTF8(IDS_FILTERED_DECEPTIVE_CONTENT_PROMPT_RELOAD));
+  set_show_manage_text_as_button(true);
+}
+
+void ContentSettingSubresourceFilterBubbleModel::SetMessage() {
+  set_message(l10n_util::GetStringUTF16(
+      IDS_FILTERED_DECEPTIVE_CONTENT_PROMPT_EXPLANATION));
+}
+
+void ContentSettingSubresourceFilterBubbleModel::OnManageLinkClicked() {
+  subresource_filter::ContentSubresourceFilterDriverFactory* driver_factory =
+      subresource_filter::ContentSubresourceFilterDriverFactory::
+          FromWebContents(web_contents());
+  driver_factory->OnReloadRequested();
+}
+
+ContentSettingSubresourceFilterBubbleModel*
+ContentSettingSubresourceFilterBubbleModel::AsSubresourceFilterBubbleModel() {
+  return this;
+}
+
 // ContentSettingMidiSysExBubbleModel ------------------------------------------
 
 class ContentSettingMidiSysExBubbleModel
@@ -1146,6 +1244,8 @@ class ContentSettingMidiSysExBubbleModel
   void MaybeAddDomainList(const std::set<std::string>& hosts, int title_id);
   void SetDomainsAndCustomLink();
   void OnCustomLinkClicked() override;
+
+  DISALLOW_COPY_AND_ASSIGN(ContentSettingMidiSysExBubbleModel);
 };
 
 ContentSettingMidiSysExBubbleModel::ContentSettingMidiSysExBubbleModel(
@@ -1206,15 +1306,15 @@ void ContentSettingMidiSysExBubbleModel::OnCustomLinkClicked() {
       TabSpecificContentSettings::FromWebContents(web_contents());
   const ContentSettingsUsagesState::StateMap& state_map =
       content_settings->midi_usages_state().state_map();
-  HostContentSettingsMap* settings_map =
+  HostContentSettingsMap* map =
       HostContentSettingsMapFactory::GetForProfile(profile());
-
   for (const std::pair<GURL, ContentSetting>& map_entry : state_map) {
-    settings_map->SetContentSetting(
-        ContentSettingsPattern::FromURLNoWildcard(map_entry.first),
-        ContentSettingsPattern::FromURLNoWildcard(embedder_url),
-        CONTENT_SETTINGS_TYPE_MIDI_SYSEX, std::string(),
-        CONTENT_SETTING_DEFAULT);
+    PermissionUtil::ScopedRevocationReporter(
+        profile(), map_entry.first, embedder_url,
+        CONTENT_SETTINGS_TYPE_MIDI_SYSEX, PermissionSourceUI::PAGE_ACTION);
+    map->SetContentSettingDefaultScope(map_entry.first, embedder_url,
+                                       CONTENT_SETTINGS_TYPE_MIDI_SYSEX,
+                                       std::string(), CONTENT_SETTING_DEFAULT);
   }
 }
 
@@ -1266,14 +1366,13 @@ ContentSettingBubbleModel*
   return nullptr;
 }
 
-ContentSettingBubbleModel::ContentSettingBubbleModel(
-    Delegate* delegate,
-    WebContents* web_contents,
-    Profile* profile)
+ContentSettingBubbleModel::ContentSettingBubbleModel(Delegate* delegate,
+                                                     WebContents* web_contents,
+                                                     Profile* profile)
     : web_contents_(web_contents),
       profile_(profile),
       delegate_(delegate),
-      setting_is_managed_(false) {
+      rappor_service_(g_browser_process->rappor_service()) {
   registrar_.Add(this, content::NOTIFICATION_WEB_CONTENTS_DESTROYED,
                  content::Source<WebContents>(web_contents));
   registrar_.Add(this, chrome::NOTIFICATION_PROFILE_DESTROYED,
@@ -1289,16 +1388,22 @@ ContentSettingBubbleModel::RadioGroup::~RadioGroup() {}
 
 ContentSettingBubbleModel::DomainList::DomainList() {}
 
+ContentSettingBubbleModel::DomainList::DomainList(const DomainList& other) =
+    default;
+
 ContentSettingBubbleModel::DomainList::~DomainList() {}
 
 ContentSettingBubbleModel::MediaMenu::MediaMenu() : disabled(false) {}
 
+ContentSettingBubbleModel::MediaMenu::MediaMenu(const MediaMenu& other) =
+    default;
+
 ContentSettingBubbleModel::MediaMenu::~MediaMenu() {}
 
 ContentSettingBubbleModel::BubbleContent::BubbleContent()
     : radio_group_enabled(false),
-      custom_link_enabled(false) {
-}
+      custom_link_enabled(false),
+      show_manage_text_as_button(false) {}
 
 ContentSettingBubbleModel::BubbleContent::~BubbleContent() {}
 
@@ -1309,11 +1414,11 @@ void ContentSettingBubbleModel::Observe(
   if (type == content::NOTIFICATION_WEB_CONTENTS_DESTROYED) {
     DCHECK_EQ(web_contents_,
               content::Source<WebContents>(source).ptr());
-    web_contents_ = NULL;
+    web_contents_ = nullptr;
   } else {
     DCHECK_EQ(chrome::NOTIFICATION_PROFILE_DESTROYED, type);
     DCHECK_EQ(profile_, content::Source<Profile>(source).ptr());
-    profile_ = NULL;
+    profile_ = nullptr;
   }
 }
 
@@ -1328,3 +1433,8 @@ ContentSettingMediaStreamBubbleModel*
   // In general, bubble models might not inherit from the media bubble model.
   return nullptr;
 }
+
+ContentSettingSubresourceFilterBubbleModel*
+ContentSettingBubbleModel::AsSubresourceFilterBubbleModel() {
+  return nullptr;
+}
