@@ -4,17 +4,21 @@
 
 #include "chrome/browser/ui/passwords/manage_passwords_state.h"
 
+#include <iterator>
 #include <utility>
 #include <vector>
 
+#include "base/memory/ptr_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "components/password_manager/core/browser/password_form_manager.h"
 #include "components/password_manager/core/browser/password_manager.h"
+#include "components/password_manager/core/browser/stub_form_saver.h"
 #include "components/password_manager/core/browser/stub_password_manager_client.h"
 #include "components/password_manager/core/browser/stub_password_manager_driver.h"
-#include "components/password_manager/core/common/credential_manager_types.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "url/gurl.h"
+#include "url/origin.h"
 
 using ::testing::_;
 using ::testing::Contains;
@@ -26,6 +30,16 @@ using ::testing::UnorderedElementsAre;
 
 namespace {
 
+std::vector<const autofill::PasswordForm*> GetRawPointers(
+    const std::vector<std::unique_ptr<autofill::PasswordForm>>& forms) {
+  std::vector<const autofill::PasswordForm*> result;
+  std::transform(forms.begin(), forms.end(), std::back_inserter(result),
+                 [](const std::unique_ptr<autofill::PasswordForm>& form) {
+                   return form.get();
+                 });
+  return result;
+}
+
 class ManagePasswordsStateTest : public testing::Test {
  public:
   ManagePasswordsStateTest() : password_manager_(&stub_client_) {}
@@ -33,12 +47,20 @@ class ManagePasswordsStateTest : public testing::Test {
   void SetUp() override {
     test_local_form_.origin = GURL("http://example.com");
     test_local_form_.username_value = base::ASCIIToUTF16("username");
+    test_local_form_.username_element = base::ASCIIToUTF16("username_element");
     test_local_form_.password_value = base::ASCIIToUTF16("12345");
 
     test_submitted_form_ = test_local_form_;
     test_submitted_form_.username_value = base::ASCIIToUTF16("new one");
     test_submitted_form_.password_value = base::ASCIIToUTF16("asdfjkl;");
 
+    test_local_federated_form_ = test_local_form_;
+    test_local_federated_form_.federation_origin =
+        url::Origin(GURL("https://idp.com"));
+    test_local_federated_form_.password_value.clear();
+    test_local_federated_form_.signon_realm =
+        "federation://example.com/accounts.com";
+
     test_federated_form_.origin = GURL("https://idp.com");
     test_federated_form_.username_value = base::ASCIIToUTF16("username");
 
@@ -47,11 +69,23 @@ class ManagePasswordsStateTest : public testing::Test {
 
   autofill::PasswordForm& test_local_form() { return test_local_form_; }
   autofill::PasswordForm& test_submitted_form() { return test_submitted_form_; }
+  autofill::PasswordForm& test_local_federated_form() {
+    return test_local_federated_form_;
+  }
   autofill::PasswordForm& test_federated_form() { return test_federated_form_; }
+  std::vector<std::unique_ptr<autofill::PasswordForm>>& test_stored_forms() {
+    return test_stored_forms_;
+  }
   ManagePasswordsState& passwords_data() { return passwords_data_; }
 
-  // Returns a PasswordFormManager containing test_local_form() as a best match.
-  scoped_ptr<password_manager::PasswordFormManager> CreateFormManager();
+  // Returns a PasswordFormManager containing |test_stored_forms_| as the best
+  // matches.
+  std::unique_ptr<password_manager::PasswordFormManager> CreateFormManager();
+
+  // Returns a PasswordFormManager containing test_federated_form() as a stored
+  // federated credential.
+  std::unique_ptr<password_manager::PasswordFormManager>
+  CreateFormManagerWithFederation();
 
   // Pushes irrelevant updates to |passwords_data_| and checks that they don't
   // affect the state.
@@ -63,10 +97,13 @@ class ManagePasswordsStateTest : public testing::Test {
   // Pushes a blacklisted form and checks that it doesn't affect the state.
   void TestBlacklistedUpdates();
 
-  MOCK_METHOD1(CredentialCallback,
-               void(const password_manager::CredentialInfo&));
+  MOCK_METHOD1(CredentialCallback, void(const autofill::PasswordForm*));
 
  private:
+  // Implements both CreateFormManager and CreateFormManagerWithFederation.
+  std::unique_ptr<password_manager::PasswordFormManager>
+  CreateFormManagerInternal(bool include_federated);
+
   password_manager::StubPasswordManagerClient stub_client_;
   password_manager::StubPasswordManagerDriver driver_;
   password_manager::PasswordManager password_manager_;
@@ -74,30 +111,48 @@ class ManagePasswordsStateTest : public testing::Test {
   ManagePasswordsState passwords_data_;
   autofill::PasswordForm test_local_form_;
   autofill::PasswordForm test_submitted_form_;
+  autofill::PasswordForm test_local_federated_form_;
   autofill::PasswordForm test_federated_form_;
+  std::vector<std::unique_ptr<autofill::PasswordForm>> test_stored_forms_;
 };
 
-scoped_ptr<password_manager::PasswordFormManager>
+std::unique_ptr<password_manager::PasswordFormManager>
 ManagePasswordsStateTest::CreateFormManager() {
-  scoped_ptr<password_manager::PasswordFormManager> test_form_manager(
+  return CreateFormManagerInternal(false);
+}
+
+std::unique_ptr<password_manager::PasswordFormManager>
+ManagePasswordsStateTest::CreateFormManagerWithFederation() {
+  return CreateFormManagerInternal(true);
+}
+
+std::unique_ptr<password_manager::PasswordFormManager>
+ManagePasswordsStateTest::CreateFormManagerInternal(bool include_federated) {
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
       new password_manager::PasswordFormManager(
           &password_manager_, &stub_client_, driver_.AsWeakPtr(),
-          test_local_form(), false));
-  test_form_manager->SimulateFetchMatchingLoginsFromPasswordStore();
-  ScopedVector<autofill::PasswordForm> stored_forms;
-  stored_forms.push_back(new autofill::PasswordForm(test_local_form()));
-  test_form_manager->OnGetPasswordStoreResults(std::move(stored_forms));
-  EXPECT_EQ(1u, test_form_manager->best_matches().size());
-  EXPECT_EQ(test_local_form(),
-            *test_form_manager->best_matches().begin()->second);
+          test_local_form(),
+          base::WrapUnique(new password_manager::StubFormSaver)));
+  if (include_federated) {
+    test_stored_forms_.push_back(
+        base::MakeUnique<autofill::PasswordForm>(test_local_federated_form()));
+  }
+  test_form_manager->OnGetPasswordStoreResults(std::move(test_stored_forms_));
+  EXPECT_EQ(include_federated ? 1u : 0u,
+            test_form_manager->form_fetcher()->GetFederatedMatches().size());
+  if (include_federated) {
+    EXPECT_EQ(
+        test_local_federated_form(),
+        *test_form_manager->form_fetcher()->GetFederatedMatches().front());
+  }
   return test_form_manager;
 }
 
 void ManagePasswordsStateTest::TestNoisyUpdates() {
   const std::vector<const autofill::PasswordForm*> forms =
-      passwords_data_.GetCurrentForms();
+      GetRawPointers(passwords_data_.GetCurrentForms());
   const std::vector<const autofill::PasswordForm*> federated_forms =
-      passwords_data_.federated_credentials_forms();
+      GetRawPointers(passwords_data_.federation_providers_forms());
   const password_manager::ui::State state = passwords_data_.state();
   const GURL origin = passwords_data_.origin();
 
@@ -110,8 +165,9 @@ void ManagePasswordsStateTest::TestNoisyUpdates() {
       password_manager::PasswordStoreChange::ADD, form);
   password_manager::PasswordStoreChangeList list(1, change);
   passwords_data().ProcessLoginsChanged(list);
-  EXPECT_EQ(forms, passwords_data().GetCurrentForms());
-  EXPECT_EQ(federated_forms, passwords_data().federated_credentials_forms());
+  EXPECT_EQ(forms, GetRawPointers(passwords_data().GetCurrentForms()));
+  EXPECT_EQ(federated_forms,
+            GetRawPointers(passwords_data().federation_providers_forms()));
   EXPECT_EQ(state, passwords_data().state());
   EXPECT_EQ(origin, passwords_data().origin());
 
@@ -120,8 +176,9 @@ void ManagePasswordsStateTest::TestNoisyUpdates() {
   list[0] = password_manager::PasswordStoreChange(
       password_manager::PasswordStoreChange::UPDATE, form);
   passwords_data().ProcessLoginsChanged(list);
-  EXPECT_EQ(forms, passwords_data().GetCurrentForms());
-  EXPECT_EQ(federated_forms, passwords_data().federated_credentials_forms());
+  EXPECT_EQ(forms, GetRawPointers(passwords_data().GetCurrentForms()));
+  EXPECT_EQ(federated_forms,
+            GetRawPointers(passwords_data().federation_providers_forms()));
   EXPECT_EQ(state, passwords_data().state());
   EXPECT_EQ(origin, passwords_data().origin());
 
@@ -129,17 +186,18 @@ void ManagePasswordsStateTest::TestNoisyUpdates() {
   list[0] = password_manager::PasswordStoreChange(
       password_manager::PasswordStoreChange::REMOVE, form);
   passwords_data().ProcessLoginsChanged(list);
-  EXPECT_EQ(forms, passwords_data().GetCurrentForms());
-  EXPECT_EQ(federated_forms, passwords_data().federated_credentials_forms());
+  EXPECT_EQ(forms, GetRawPointers(passwords_data().GetCurrentForms()));
+  EXPECT_EQ(federated_forms,
+            GetRawPointers(passwords_data().federation_providers_forms()));
   EXPECT_EQ(state, passwords_data().state());
   EXPECT_EQ(origin, passwords_data().origin());
 }
 
 void ManagePasswordsStateTest::TestAllUpdates() {
   const std::vector<const autofill::PasswordForm*> forms =
-      passwords_data_.GetCurrentForms();
+      GetRawPointers(passwords_data_.GetCurrentForms());
   const std::vector<const autofill::PasswordForm*> federated_forms =
-      passwords_data_.federated_credentials_forms();
+      GetRawPointers(passwords_data_.federation_providers_forms());
   const password_manager::ui::State state = passwords_data_.state();
   const GURL origin = passwords_data_.origin();
   EXPECT_NE(GURL::EmptyGURL(), origin);
@@ -154,7 +212,8 @@ void ManagePasswordsStateTest::TestAllUpdates() {
   password_manager::PasswordStoreChangeList list(1, change);
   passwords_data().ProcessLoginsChanged(list);
   EXPECT_THAT(passwords_data().GetCurrentForms(), Contains(Pointee(form)));
-  EXPECT_EQ(federated_forms, passwords_data().federated_credentials_forms());
+  EXPECT_EQ(federated_forms,
+            GetRawPointers(passwords_data().federation_providers_forms()));
   EXPECT_EQ(state, passwords_data().state());
   EXPECT_EQ(origin, passwords_data().origin());
 
@@ -164,7 +223,8 @@ void ManagePasswordsStateTest::TestAllUpdates() {
       password_manager::PasswordStoreChange::UPDATE, form);
   passwords_data().ProcessLoginsChanged(list);
   EXPECT_THAT(passwords_data().GetCurrentForms(), Contains(Pointee(form)));
-  EXPECT_EQ(federated_forms, passwords_data().federated_credentials_forms());
+  EXPECT_EQ(federated_forms,
+            GetRawPointers(passwords_data().federation_providers_forms()));
   EXPECT_EQ(state, passwords_data().state());
   EXPECT_EQ(origin, passwords_data().origin());
 
@@ -172,8 +232,9 @@ void ManagePasswordsStateTest::TestAllUpdates() {
   list[0] = password_manager::PasswordStoreChange(
       password_manager::PasswordStoreChange::REMOVE, form);
   passwords_data().ProcessLoginsChanged(list);
-  EXPECT_EQ(forms, passwords_data().GetCurrentForms());
-  EXPECT_EQ(federated_forms, passwords_data().federated_credentials_forms());
+  EXPECT_EQ(forms, GetRawPointers(passwords_data().GetCurrentForms()));
+  EXPECT_EQ(federated_forms,
+            GetRawPointers(passwords_data().federation_providers_forms()));
   EXPECT_EQ(state, passwords_data().state());
   EXPECT_EQ(origin, passwords_data().origin());
 
@@ -182,9 +243,9 @@ void ManagePasswordsStateTest::TestAllUpdates() {
 
 void ManagePasswordsStateTest::TestBlacklistedUpdates() {
   const std::vector<const autofill::PasswordForm*> forms =
-      passwords_data_.GetCurrentForms();
+      GetRawPointers(passwords_data_.GetCurrentForms());
   const std::vector<const autofill::PasswordForm*> federated_forms =
-      passwords_data_.federated_credentials_forms();
+      GetRawPointers(passwords_data_.federation_providers_forms());
   const password_manager::ui::State state = passwords_data_.state();
   const GURL origin = passwords_data_.origin();
   EXPECT_NE(GURL::EmptyGURL(), origin);
@@ -197,8 +258,9 @@ void ManagePasswordsStateTest::TestBlacklistedUpdates() {
   list.push_back(password_manager::PasswordStoreChange(
       password_manager::PasswordStoreChange::ADD, blacklisted));
   passwords_data().ProcessLoginsChanged(list);
-  EXPECT_EQ(forms, passwords_data().GetCurrentForms());
-  EXPECT_EQ(federated_forms, passwords_data().federated_credentials_forms());
+  EXPECT_EQ(forms, GetRawPointers(passwords_data().GetCurrentForms()));
+  EXPECT_EQ(federated_forms,
+            GetRawPointers(passwords_data().federation_providers_forms()));
   EXPECT_EQ(state, passwords_data().state());
   EXPECT_EQ(origin, passwords_data().origin());
 
@@ -206,15 +268,16 @@ void ManagePasswordsStateTest::TestBlacklistedUpdates() {
   list[0] = password_manager::PasswordStoreChange(
       password_manager::PasswordStoreChange::REMOVE, blacklisted);
   passwords_data().ProcessLoginsChanged(list);
-  EXPECT_EQ(forms, passwords_data().GetCurrentForms());
-  EXPECT_EQ(federated_forms, passwords_data().federated_credentials_forms());
+  EXPECT_EQ(forms, GetRawPointers(passwords_data().GetCurrentForms()));
+  EXPECT_EQ(federated_forms,
+            GetRawPointers(passwords_data().federation_providers_forms()));
   EXPECT_EQ(state, passwords_data().state());
   EXPECT_EQ(origin, passwords_data().origin());
 }
 
 TEST_F(ManagePasswordsStateTest, DefaultState) {
   EXPECT_THAT(passwords_data().GetCurrentForms(), IsEmpty());
-  EXPECT_THAT(passwords_data().federated_credentials_forms(), IsEmpty());
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
   EXPECT_EQ(password_manager::ui::INACTIVE_STATE, passwords_data().state());
   EXPECT_EQ(GURL::EmptyGURL(), passwords_data().origin());
   EXPECT_FALSE(passwords_data().form_manager());
@@ -223,7 +286,9 @@ TEST_F(ManagePasswordsStateTest, DefaultState) {
 }
 
 TEST_F(ManagePasswordsStateTest, PasswordSubmitted) {
-  scoped_ptr<password_manager::PasswordFormManager> test_form_manager(
+  test_stored_forms().push_back(
+      base::MakeUnique<autofill::PasswordForm>(test_local_form()));
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
       CreateFormManager());
   test_form_manager->ProvisionallySave(
       test_submitted_form(),
@@ -232,7 +297,7 @@ TEST_F(ManagePasswordsStateTest, PasswordSubmitted) {
 
   EXPECT_THAT(passwords_data().GetCurrentForms(),
               ElementsAre(Pointee(test_local_form())));
-  EXPECT_THAT(passwords_data().federated_credentials_forms(), IsEmpty());
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
   EXPECT_EQ(password_manager::ui::PENDING_PASSWORD_STATE,
             passwords_data().state());
   EXPECT_EQ(test_submitted_form().origin, passwords_data().origin());
@@ -243,7 +308,9 @@ TEST_F(ManagePasswordsStateTest, PasswordSubmitted) {
 }
 
 TEST_F(ManagePasswordsStateTest, PasswordSaved) {
-  scoped_ptr<password_manager::PasswordFormManager> test_form_manager(
+  test_stored_forms().push_back(
+      base::MakeUnique<autofill::PasswordForm>(test_local_form()));
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
       CreateFormManager());
   test_form_manager->ProvisionallySave(
       test_submitted_form(),
@@ -255,18 +322,31 @@ TEST_F(ManagePasswordsStateTest, PasswordSaved) {
   passwords_data().TransitionToState(password_manager::ui::MANAGE_STATE);
   EXPECT_THAT(passwords_data().GetCurrentForms(),
               ElementsAre(Pointee(test_local_form())));
-  EXPECT_THAT(passwords_data().federated_credentials_forms(), IsEmpty());
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
   EXPECT_EQ(password_manager::ui::MANAGE_STATE,
             passwords_data().state());
   EXPECT_EQ(test_submitted_form().origin, passwords_data().origin());
   TestAllUpdates();
 }
 
+TEST_F(ManagePasswordsStateTest, PasswordSubmittedFederationsPresent) {
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
+      CreateFormManagerWithFederation());
+  test_form_manager->ProvisionallySave(
+      test_submitted_form(),
+      password_manager::PasswordFormManager::IGNORE_OTHER_POSSIBLE_USERNAMES);
+  passwords_data().OnPendingPassword(std::move(test_form_manager));
+
+  EXPECT_THAT(passwords_data().GetCurrentForms(),
+              ElementsAre(Pointee(test_local_federated_form())));
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
+}
+
 TEST_F(ManagePasswordsStateTest, OnRequestCredentials) {
-  ScopedVector<autofill::PasswordForm> local_credentials;
-  local_credentials.push_back(new autofill::PasswordForm(test_local_form()));
-  ScopedVector<autofill::PasswordForm> federated_credentials;
-  federated_credentials.push_back(
+  std::vector<std::unique_ptr<autofill::PasswordForm>> local_credentials;
+  local_credentials.emplace_back(new autofill::PasswordForm(test_local_form()));
+  std::vector<std::unique_ptr<autofill::PasswordForm>> federated_credentials;
+  federated_credentials.emplace_back(
       new autofill::PasswordForm(test_federated_form()));
   const GURL origin = test_local_form().origin;
   passwords_data().OnRequestCredentials(
@@ -275,37 +355,32 @@ TEST_F(ManagePasswordsStateTest, OnRequestCredentials) {
       &ManagePasswordsStateTest::CredentialCallback, base::Unretained(this)));
   EXPECT_THAT(passwords_data().GetCurrentForms(),
               ElementsAre(Pointee(test_local_form())));
-  EXPECT_THAT(passwords_data().federated_credentials_forms(),
+  EXPECT_THAT(passwords_data().federation_providers_forms(),
               ElementsAre(Pointee(test_federated_form())));
   EXPECT_EQ(password_manager::ui::CREDENTIAL_REQUEST_STATE,
             passwords_data().state());
   EXPECT_EQ(origin, passwords_data().origin());
   TestAllUpdates();
 
-  password_manager::CredentialInfo credential_info(
-      test_local_form(),
-      password_manager::CredentialType::CREDENTIAL_TYPE_PASSWORD);
-  EXPECT_CALL(*this, CredentialCallback(_))
-      .WillOnce(testing::SaveArg<0>(&credential_info));
+  EXPECT_CALL(*this, CredentialCallback(nullptr));
   passwords_data().TransitionToState(password_manager::ui::MANAGE_STATE);
-  EXPECT_EQ(password_manager::CredentialType::CREDENTIAL_TYPE_EMPTY,
-            credential_info.type);
   EXPECT_TRUE(passwords_data().credentials_callback().is_null());
   EXPECT_THAT(passwords_data().GetCurrentForms(),
               ElementsAre(Pointee(test_local_form())));
-  EXPECT_THAT(passwords_data().federated_credentials_forms(), IsEmpty());
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
   EXPECT_EQ(password_manager::ui::MANAGE_STATE, passwords_data().state());
   EXPECT_EQ(origin, passwords_data().origin());
   TestAllUpdates();
 }
 
 TEST_F(ManagePasswordsStateTest, AutoSignin) {
-  ScopedVector<autofill::PasswordForm> local_credentials;
-  local_credentials.push_back(new autofill::PasswordForm(test_local_form()));
-  passwords_data().OnAutoSignin(std::move(local_credentials));
+  std::vector<std::unique_ptr<autofill::PasswordForm>> local_credentials;
+  local_credentials.emplace_back(new autofill::PasswordForm(test_local_form()));
+  passwords_data().OnAutoSignin(std::move(local_credentials),
+                                test_local_form().origin);
   EXPECT_THAT(passwords_data().GetCurrentForms(),
               ElementsAre(Pointee(test_local_form())));
-  EXPECT_THAT(passwords_data().federated_credentials_forms(), IsEmpty());
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
   EXPECT_EQ(password_manager::ui::AUTO_SIGNIN_STATE, passwords_data().state());
   EXPECT_EQ(test_local_form().origin, passwords_data().origin());
   TestAllUpdates();
@@ -313,14 +388,14 @@ TEST_F(ManagePasswordsStateTest, AutoSignin) {
   passwords_data().TransitionToState(password_manager::ui::MANAGE_STATE);
   EXPECT_THAT(passwords_data().GetCurrentForms(),
               ElementsAre(Pointee(test_local_form())));
-  EXPECT_THAT(passwords_data().federated_credentials_forms(), IsEmpty());
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
   EXPECT_EQ(password_manager::ui::MANAGE_STATE, passwords_data().state());
   EXPECT_EQ(test_local_form().origin, passwords_data().origin());
   TestAllUpdates();
 }
 
 TEST_F(ManagePasswordsStateTest, AutomaticPasswordSave) {
-  scoped_ptr<password_manager::PasswordFormManager> test_form_manager(
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
       CreateFormManager());
   test_form_manager->ProvisionallySave(
       test_submitted_form(),
@@ -336,53 +411,100 @@ TEST_F(ManagePasswordsStateTest, AutomaticPasswordSave) {
 
   passwords_data().TransitionToState(password_manager::ui::MANAGE_STATE);
   EXPECT_THAT(passwords_data().GetCurrentForms(),
-              UnorderedElementsAre(Pointee(test_local_form()),
-                                   Pointee(test_submitted_form())));
-  EXPECT_THAT(passwords_data().federated_credentials_forms(), IsEmpty());
+              ElementsAre(Pointee(test_submitted_form())));
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
   EXPECT_EQ(password_manager::ui::MANAGE_STATE, passwords_data().state());
   EXPECT_EQ(test_submitted_form().origin, passwords_data().origin());
   TestAllUpdates();
 }
 
+TEST_F(ManagePasswordsStateTest, AutomaticPasswordSaveWithFederations) {
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
+      CreateFormManagerWithFederation());
+  test_form_manager->ProvisionallySave(
+      test_submitted_form(),
+      password_manager::PasswordFormManager::IGNORE_OTHER_POSSIBLE_USERNAMES);
+
+  passwords_data().OnAutomaticPasswordSave(std::move(test_form_manager));
+  EXPECT_THAT(passwords_data().GetCurrentForms(),
+              UnorderedElementsAre(Pointee(test_submitted_form()),
+                                   Pointee(test_local_federated_form())));
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
+}
+
 TEST_F(ManagePasswordsStateTest, PasswordAutofilled) {
-  autofill::PasswordFormMap password_form_map;
-  password_form_map.insert(std::make_pair(
-      test_local_form().username_value,
-      make_scoped_ptr(new autofill::PasswordForm(test_local_form()))));
+  std::map<base::string16, const autofill::PasswordForm*> password_form_map;
+  password_form_map.insert(
+      std::make_pair(test_local_form().username_value, &test_local_form()));
   GURL origin("https://example.com");
-  passwords_data().OnPasswordAutofilled(password_form_map, origin);
+  passwords_data().OnPasswordAutofilled(password_form_map, origin, nullptr);
 
   EXPECT_THAT(passwords_data().GetCurrentForms(),
               ElementsAre(Pointee(test_local_form())));
-  EXPECT_THAT(passwords_data().federated_credentials_forms(), IsEmpty());
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
   EXPECT_EQ(password_manager::ui::MANAGE_STATE, passwords_data().state());
   EXPECT_EQ(origin, passwords_data().origin());
 
-  // |passwords_data| should hold a separate copy of test_local_form().
+  TestAllUpdates();
+}
+
+TEST_F(ManagePasswordsStateTest, PasswordAutofillWithSavedFederations) {
+  std::map<base::string16, const autofill::PasswordForm*> password_form_map;
+  password_form_map.insert(
+      std::make_pair(test_local_form().username_value, &test_local_form()));
+  GURL origin("https://example.com");
+  std::vector<const autofill::PasswordForm*> federated;
+  federated.push_back(&test_local_federated_form());
+  passwords_data().OnPasswordAutofilled(password_form_map, origin, &federated);
+
+  // |federated| represents the locally saved federations. These are bundled in
+  // the "current forms".
   EXPECT_THAT(passwords_data().GetCurrentForms(),
-              Not(Contains(&test_local_form())));
+              UnorderedElementsAre(Pointee(test_local_form()),
+                                   Pointee(test_local_federated_form())));
+  // |federated_credentials_forms()| do not refer to the saved federations.
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
+  EXPECT_EQ(password_manager::ui::MANAGE_STATE, passwords_data().state());
+}
+
+TEST_F(ManagePasswordsStateTest, ActiveOnMixedPSLAndNonPSLMatched) {
+  std::map<base::string16, const autofill::PasswordForm*> password_form_map;
+  password_form_map.insert(
+      std::make_pair(test_local_form().username_value, &test_local_form()));
+  autofill::PasswordForm psl_matched_test_form = test_local_form();
+  psl_matched_test_form.is_public_suffix_match = true;
+  password_form_map.insert(std::make_pair(psl_matched_test_form.username_value,
+                                          &psl_matched_test_form));
+  GURL origin("https://example.com");
+  passwords_data().OnPasswordAutofilled(password_form_map, origin, nullptr);
+
+  EXPECT_THAT(passwords_data().GetCurrentForms(),
+              ElementsAre(Pointee(test_local_form())));
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
+  EXPECT_EQ(password_manager::ui::MANAGE_STATE, passwords_data().state());
+  EXPECT_EQ(origin, passwords_data().origin());
+
   TestAllUpdates();
 }
 
 TEST_F(ManagePasswordsStateTest, InactiveOnPSLMatched) {
   autofill::PasswordForm psl_matched_test_form = test_local_form();
   psl_matched_test_form.is_public_suffix_match = true;
-  autofill::PasswordFormMap password_form_map;
-  password_form_map.insert(std::make_pair(
-      psl_matched_test_form.username_value,
-      make_scoped_ptr(new autofill::PasswordForm(psl_matched_test_form))));
-  passwords_data().OnPasswordAutofilled(password_form_map,
-                                        GURL("https://m.example.com/"));
+  std::map<base::string16, const autofill::PasswordForm*> password_form_map;
+  password_form_map.insert(std::make_pair(psl_matched_test_form.username_value,
+                                          &psl_matched_test_form));
+  passwords_data().OnPasswordAutofilled(
+      password_form_map, GURL("https://m.example.com/"), nullptr);
 
   EXPECT_THAT(passwords_data().GetCurrentForms(), IsEmpty());
-  EXPECT_THAT(passwords_data().federated_credentials_forms(), IsEmpty());
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
   EXPECT_EQ(password_manager::ui::INACTIVE_STATE, passwords_data().state());
   EXPECT_EQ(GURL::EmptyGURL(), passwords_data().origin());
   EXPECT_FALSE(passwords_data().form_manager());
 }
 
 TEST_F(ManagePasswordsStateTest, OnInactive) {
-  scoped_ptr<password_manager::PasswordFormManager> test_form_manager(
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
       CreateFormManager());
   test_form_manager->ProvisionallySave(
       test_submitted_form(),
@@ -392,7 +514,7 @@ TEST_F(ManagePasswordsStateTest, OnInactive) {
             passwords_data().state());
   passwords_data().OnInactive();
   EXPECT_THAT(passwords_data().GetCurrentForms(), IsEmpty());
-  EXPECT_THAT(passwords_data().federated_credentials_forms(), IsEmpty());
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
   EXPECT_EQ(password_manager::ui::INACTIVE_STATE, passwords_data().state());
   EXPECT_EQ(GURL::EmptyGURL(), passwords_data().origin());
   EXPECT_FALSE(passwords_data().form_manager());
@@ -400,7 +522,7 @@ TEST_F(ManagePasswordsStateTest, OnInactive) {
 }
 
 TEST_F(ManagePasswordsStateTest, PendingPasswordAddBlacklisted) {
-  scoped_ptr<password_manager::PasswordFormManager> test_form_manager(
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
       CreateFormManager());
   test_form_manager->ProvisionallySave(
       test_submitted_form(),
@@ -413,10 +535,10 @@ TEST_F(ManagePasswordsStateTest, PendingPasswordAddBlacklisted) {
 }
 
 TEST_F(ManagePasswordsStateTest, RequestCredentialsAddBlacklisted) {
-  ScopedVector<autofill::PasswordForm> local_credentials;
-  local_credentials.push_back(new autofill::PasswordForm(test_local_form()));
-  ScopedVector<autofill::PasswordForm> federated_credentials;
-  federated_credentials.push_back(
+  std::vector<std::unique_ptr<autofill::PasswordForm>> local_credentials;
+  local_credentials.emplace_back(new autofill::PasswordForm(test_local_form()));
+  std::vector<std::unique_ptr<autofill::PasswordForm>> federated_credentials;
+  federated_credentials.emplace_back(
       new autofill::PasswordForm(test_federated_form()));
   const GURL origin = test_local_form().origin;
   passwords_data().OnRequestCredentials(
@@ -430,16 +552,17 @@ TEST_F(ManagePasswordsStateTest, RequestCredentialsAddBlacklisted) {
 }
 
 TEST_F(ManagePasswordsStateTest, AutoSigninAddBlacklisted) {
-  ScopedVector<autofill::PasswordForm> local_credentials;
-  local_credentials.push_back(new autofill::PasswordForm(test_local_form()));
-  passwords_data().OnAutoSignin(std::move(local_credentials));
+  std::vector<std::unique_ptr<autofill::PasswordForm>> local_credentials;
+  local_credentials.emplace_back(new autofill::PasswordForm(test_local_form()));
+  passwords_data().OnAutoSignin(std::move(local_credentials),
+                                test_local_form().origin);
   EXPECT_EQ(password_manager::ui::AUTO_SIGNIN_STATE, passwords_data().state());
 
   TestBlacklistedUpdates();
 }
 
 TEST_F(ManagePasswordsStateTest, AutomaticPasswordSaveAddBlacklisted) {
-  scoped_ptr<password_manager::PasswordFormManager> test_form_manager(
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
       CreateFormManager());
   test_form_manager->ProvisionallySave(
       test_submitted_form(),
@@ -451,19 +574,18 @@ TEST_F(ManagePasswordsStateTest, AutomaticPasswordSaveAddBlacklisted) {
 }
 
 TEST_F(ManagePasswordsStateTest, BackgroundAutofilledAddBlacklisted) {
-  autofill::PasswordFormMap password_form_map;
-  password_form_map.insert(std::make_pair(
-      test_local_form().username_value,
-      make_scoped_ptr(new autofill::PasswordForm(test_local_form()))));
+  std::map<base::string16, const autofill::PasswordForm*> password_form_map;
+  password_form_map.insert(
+      std::make_pair(test_local_form().username_value, &test_local_form()));
   passwords_data().OnPasswordAutofilled(
-      password_form_map, password_form_map.begin()->second->origin);
+      password_form_map, password_form_map.begin()->second->origin, nullptr);
   EXPECT_EQ(password_manager::ui::MANAGE_STATE, passwords_data().state());
 
   TestBlacklistedUpdates();
 }
 
 TEST_F(ManagePasswordsStateTest, PasswordUpdateAddBlacklisted) {
-  scoped_ptr<password_manager::PasswordFormManager> test_form_manager(
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
       CreateFormManager());
   test_form_manager->ProvisionallySave(
       test_submitted_form(),
@@ -476,7 +598,9 @@ TEST_F(ManagePasswordsStateTest, PasswordUpdateAddBlacklisted) {
 }
 
 TEST_F(ManagePasswordsStateTest, PasswordUpdateSubmitted) {
-  scoped_ptr<password_manager::PasswordFormManager> test_form_manager(
+  test_stored_forms().push_back(
+      base::MakeUnique<autofill::PasswordForm>(test_local_form()));
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
       CreateFormManager());
   test_form_manager->ProvisionallySave(
       test_submitted_form(),
@@ -485,7 +609,7 @@ TEST_F(ManagePasswordsStateTest, PasswordUpdateSubmitted) {
 
   EXPECT_THAT(passwords_data().GetCurrentForms(),
               ElementsAre(Pointee(test_local_form())));
-  EXPECT_THAT(passwords_data().federated_credentials_forms(), IsEmpty());
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
   EXPECT_EQ(password_manager::ui::PENDING_PASSWORD_UPDATE_STATE,
             passwords_data().state());
   EXPECT_EQ(test_submitted_form().origin, passwords_data().origin());
@@ -495,65 +619,84 @@ TEST_F(ManagePasswordsStateTest, PasswordUpdateSubmitted) {
   TestAllUpdates();
 }
 
-TEST_F(ManagePasswordsStateTest, ChooseCredentialLocal) {
-  passwords_data().OnRequestCredentials(ScopedVector<autofill::PasswordForm>(),
-                                        ScopedVector<autofill::PasswordForm>(),
-                                        test_local_form().origin);
-  passwords_data().set_credentials_callback(base::Bind(
-      &ManagePasswordsStateTest::CredentialCallback, base::Unretained(this)));
-  password_manager::CredentialInfo credential_info(
-      test_local_form(),
-      password_manager::CredentialType::CREDENTIAL_TYPE_PASSWORD);
-  EXPECT_CALL(*this, CredentialCallback(credential_info));
-  passwords_data().ChooseCredential(
-      test_local_form(),
-      password_manager::CredentialType::CREDENTIAL_TYPE_PASSWORD);
+TEST_F(ManagePasswordsStateTest, AndroidPasswordUpdateSubmitted) {
+  autofill::PasswordForm android_form;
+  android_form.signon_realm = "android://dHJhc2g=@com.example.android/";
+  android_form.origin = GURL(android_form.signon_realm);
+  android_form.username_value = test_submitted_form().username_value;
+  android_form.password_value = base::ASCIIToUTF16("old pass");
+  test_stored_forms().push_back(
+      base::MakeUnique<autofill::PasswordForm>(android_form));
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
+      CreateFormManager());
+  test_form_manager->ProvisionallySave(
+      test_submitted_form(),
+      password_manager::PasswordFormManager::IGNORE_OTHER_POSSIBLE_USERNAMES);
+  passwords_data().OnUpdatePassword(std::move(test_form_manager));
+
+  EXPECT_THAT(passwords_data().GetCurrentForms(),
+              ElementsAre(Pointee(android_form)));
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
+  EXPECT_EQ(password_manager::ui::PENDING_PASSWORD_UPDATE_STATE,
+            passwords_data().state());
+  EXPECT_EQ(test_submitted_form().origin, passwords_data().origin());
+  ASSERT_TRUE(passwords_data().form_manager());
+  android_form.password_value = test_submitted_form().password_value;
+  EXPECT_EQ(android_form,
+            passwords_data().form_manager()->pending_credentials());
+  TestAllUpdates();
 }
 
-TEST_F(ManagePasswordsStateTest, ChooseCredentialFederated) {
-  passwords_data().OnRequestCredentials(ScopedVector<autofill::PasswordForm>(),
-                                        ScopedVector<autofill::PasswordForm>(),
-                                        test_local_form().origin);
+TEST_F(ManagePasswordsStateTest, PasswordUpdateSubmittedWithFederations) {
+  test_stored_forms().push_back(
+      base::MakeUnique<autofill::PasswordForm>(test_local_form()));
+  std::unique_ptr<password_manager::PasswordFormManager> test_form_manager(
+      CreateFormManagerWithFederation());
+  test_form_manager->ProvisionallySave(
+      test_submitted_form(),
+      password_manager::PasswordFormManager::IGNORE_OTHER_POSSIBLE_USERNAMES);
+  passwords_data().OnUpdatePassword(std::move(test_form_manager));
+
+  EXPECT_THAT(passwords_data().GetCurrentForms(),
+              UnorderedElementsAre(Pointee(test_local_form()),
+                                   Pointee(test_local_federated_form())));
+  EXPECT_THAT(passwords_data().federation_providers_forms(), IsEmpty());
+}
+
+TEST_F(ManagePasswordsStateTest, ChooseCredentialLocal) {
+  passwords_data().OnRequestCredentials(
+      std::vector<std::unique_ptr<autofill::PasswordForm>>(),
+      std::vector<std::unique_ptr<autofill::PasswordForm>>(),
+      test_local_form().origin);
   passwords_data().set_credentials_callback(base::Bind(
       &ManagePasswordsStateTest::CredentialCallback, base::Unretained(this)));
-  password_manager::CredentialInfo credential_info(
-      test_federated_form(),
-      password_manager::CredentialType::CREDENTIAL_TYPE_FEDERATED);
-  EXPECT_CALL(*this, CredentialCallback(credential_info));
-  passwords_data().ChooseCredential(
-      test_federated_form(),
-      password_manager::CredentialType::CREDENTIAL_TYPE_FEDERATED);
+  EXPECT_CALL(*this, CredentialCallback(&test_local_form()));
+  passwords_data().ChooseCredential(&test_local_form());
 }
 
 TEST_F(ManagePasswordsStateTest, ChooseCredentialEmpty) {
-  passwords_data().OnRequestCredentials(ScopedVector<autofill::PasswordForm>(),
-                                        ScopedVector<autofill::PasswordForm>(),
-                                        test_local_form().origin);
-  autofill::PasswordForm password_form(test_local_form());
+  passwords_data().OnRequestCredentials(
+      std::vector<std::unique_ptr<autofill::PasswordForm>>(),
+      std::vector<std::unique_ptr<autofill::PasswordForm>>(),
+      test_local_form().origin);
   passwords_data().set_credentials_callback(base::Bind(
       &ManagePasswordsStateTest::CredentialCallback, base::Unretained(this)));
   password_manager::CredentialInfo credential_info(
       test_federated_form(),
       password_manager::CredentialType::CREDENTIAL_TYPE_EMPTY);
-  EXPECT_CALL(*this, CredentialCallback(credential_info));
-  passwords_data().ChooseCredential(
-      test_federated_form(),
-      password_manager::CredentialType::CREDENTIAL_TYPE_EMPTY);
+  EXPECT_CALL(*this, CredentialCallback(nullptr));
+  passwords_data().ChooseCredential(nullptr);
 }
 
 TEST_F(ManagePasswordsStateTest, ChooseCredentialLocalWithNonEmptyFederation) {
-  passwords_data().OnRequestCredentials(ScopedVector<autofill::PasswordForm>(),
-                                        ScopedVector<autofill::PasswordForm>(),
-                                        test_local_form().origin);
-  autofill::PasswordForm form(test_federated_form());
-  form.federation_url = GURL("https://federation.test/");
+  passwords_data().OnRequestCredentials(
+      std::vector<std::unique_ptr<autofill::PasswordForm>>(),
+      std::vector<std::unique_ptr<autofill::PasswordForm>>(),
+      test_local_form().origin);
   passwords_data().set_credentials_callback(base::Bind(
       &ManagePasswordsStateTest::CredentialCallback, base::Unretained(this)));
-  password_manager::CredentialInfo credential_info(
-      form, password_manager::CredentialType::CREDENTIAL_TYPE_FEDERATED);
-  EXPECT_CALL(*this, CredentialCallback(credential_info));
-  passwords_data().ChooseCredential(
-      form, password_manager::CredentialType::CREDENTIAL_TYPE_PASSWORD);
+  EXPECT_CALL(*this, CredentialCallback(&test_local_federated_form()));
+  passwords_data().ChooseCredential(&test_local_federated_form());
 }
 
 }  // namespace
