@@ -6,9 +6,12 @@
 
 #include <stdint.h>
 
+#include <memory>
+#include <tuple>
+
 #include "base/compiler_specific.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/metrics/field_trial.h"
 #include "base/strings/string16.h"
 #include "base/strings/utf_string_conversions.h"
@@ -27,8 +30,8 @@
 #include "chrome/browser/ui/browser_instant_controller.h"
 #include "chrome/browser/ui/search/search_tab_helper.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
-#include "chrome/common/instant_types.h"
 #include "chrome/common/render_messages.h"
+#include "chrome/common/search/instant_types.h"
 #include "components/omnibox/browser/autocomplete_match.h"
 #include "content/public/browser/navigation_controller.h"
 #include "content/public/browser/web_contents.h"
@@ -62,7 +65,7 @@ class DummyPrerenderContents : public PrerenderContents {
       bool call_did_finish_load);
 
   void StartPrerendering(
-      const gfx::Size& size,
+      const gfx::Rect& bounds,
       content::SessionStorageNamespace* session_storage_namespace) override;
   bool GetChildId(int* child_id) const override;
   bool GetRouteId(int* route_id) const override;
@@ -108,7 +111,7 @@ DummyPrerenderContents::DummyPrerenderContents(
 }
 
 void DummyPrerenderContents::StartPrerendering(
-    const gfx::Size& size,
+    const gfx::Rect& bounds,
     content::SessionStorageNamespace* session_storage_namespace) {
   content::SessionStorageNamespaceMap session_storage_namespace_map;
   session_storage_namespace_map[std::string()] = session_storage_namespace;
@@ -166,8 +169,8 @@ class InstantSearchPrerendererTest : public InstantUnitTestBase {
             bool call_did_finish_load) {
     AddTab(browser(), GURL(url::kAboutBlankURL));
 
-    PrerenderManagerFactory::GetForProfile(browser()->profile())->
-        SetPrerenderContentsFactory(
+    PrerenderManagerFactory::GetForBrowserContext(browser()->profile())
+        ->SetPrerenderContentsFactoryForTest(
             new DummyPrerenderContentsFactory(call_did_finish_load));
     if (prerender_search_results_base_page) {
       content::SessionStorageNamespace* session_storage_namespace =
@@ -180,11 +183,11 @@ class InstantSearchPrerendererTest : public InstantUnitTestBase {
   }
 
   InstantSearchPrerenderer* GetInstantSearchPrerenderer() {
-    return instant_service_->instant_search_prerenderer();
+    return instant_service_->GetInstantSearchPrerenderer();
   }
 
   const GURL& GetPrerenderURL() {
-    return GetInstantSearchPrerenderer()->prerender_url_;
+    return GetInstantSearchPrerenderer()->prerender_url();
   }
 
   void SetLastQuery(const base::string16& query) {
@@ -325,10 +328,9 @@ TEST_F(InstantSearchPrerendererTest, PrerenderingAllowed) {
   data.SetURL("https://www.dummyurl.com/search?q=%s&img=1");
   data.SetShortName(ASCIIToUTF16("t"));
   data.SetKeyword(ASCIIToUTF16("k"));
-  TemplateURL* t_url = new TemplateURL(data);
   TemplateURLService* service =
       TemplateURLServiceFactory::GetForProfile(profile());
-  service->Add(t_url);
+  service->Add(base::MakeUnique<TemplateURL>(data));
   service->Load();
   AutocompleteMatch custom_search_type_match(
       NULL, 1100, false, AutocompleteMatchType::SEARCH_SUGGEST);
@@ -345,16 +347,6 @@ TEST_F(InstantSearchPrerendererTest, PrerenderingAllowed) {
                                    AutocompleteMatchType::URL_WHAT_YOU_TYPED);
   EXPECT_FALSE(AutocompleteMatch::IsSearchType(url_type_match.type));
   EXPECT_FALSE(prerenderer->IsAllowed(url_type_match, active_tab));
-
-  // Search results page supports Instant search. InstantSearchPrerenderer is
-  // used only when the underlying page doesn't support Instant.
-  NavigateAndCommitActiveTab(GURL("https://www.google.com/alt#quux=foo&strk"));
-  active_tab = GetActiveWebContents();
-  EXPECT_FALSE(
-      search::ExtractSearchTermsFromURL(profile(), active_tab->GetURL())
-          .empty());
-  EXPECT_FALSE(search::ShouldPrefetchSearchResultsOnSRP());
-  EXPECT_FALSE(prerenderer->IsAllowed(search_type_match, active_tab));
 }
 
 TEST_F(InstantSearchPrerendererTest, UsePrerenderPage) {
@@ -363,9 +355,9 @@ TEST_F(InstantSearchPrerendererTest, UsePrerenderPage) {
   // Open a search results page. A prerendered page exists for |url|. Make sure
   // the browser swaps the current tab contents with the prerendered contents.
   GURL url("https://www.google.com/alt#quux=foo&strk");
-  browser()->OpenURL(content::OpenURLParams(url, Referrer(), CURRENT_TAB,
-                                            ui::PAGE_TRANSITION_TYPED,
-                                            false));
+  browser()->OpenURL(content::OpenURLParams(url, Referrer(),
+                                            WindowOpenDisposition::CURRENT_TAB,
+                                            ui::PAGE_TRANSITION_TYPED, false));
   EXPECT_EQ(GetPrerenderURL(), GetActiveWebContents()->GetURL());
   EXPECT_EQ(static_cast<PrerenderHandle*>(NULL), prerender_handle());
 }
@@ -381,9 +373,9 @@ TEST_F(InstantSearchPrerendererTest, PrerenderRequestCancelled) {
   // Open a search results page. Prerendered page does not exists for |url|.
   // Make sure the browser navigates the current tab to this |url|.
   GURL url("https://www.google.com/alt#quux=foo&strk");
-  browser()->OpenURL(content::OpenURLParams(url, Referrer(), CURRENT_TAB,
-                                            ui::PAGE_TRANSITION_TYPED,
-                                            false));
+  browser()->OpenURL(content::OpenURLParams(url, Referrer(),
+                                            WindowOpenDisposition::CURRENT_TAB,
+                                            ui::PAGE_TRANSITION_TYPED, false));
   EXPECT_NE(GetPrerenderURL(), GetActiveWebContents()->GetURL());
   EXPECT_EQ(url, GetActiveWebContents()->GetURL());
 }
@@ -396,9 +388,9 @@ TEST_F(InstantSearchPrerendererTest,
   // prerendered search query("foo"). Make sure the browser swaps the current
   // tab contents with the prerendered contents.
   GURL url("https://www.google.com/alt#quux=pen&strk");
-  browser()->OpenURL(content::OpenURLParams(url, Referrer(), CURRENT_TAB,
-                                            ui::PAGE_TRANSITION_TYPED,
-                                            false));
+  browser()->OpenURL(content::OpenURLParams(url, Referrer(),
+                                            WindowOpenDisposition::CURRENT_TAB,
+                                            ui::PAGE_TRANSITION_TYPED, false));
   EXPECT_EQ(GetPrerenderURL(), GetActiveWebContents()->GetURL());
   EXPECT_EQ(static_cast<PrerenderHandle*>(NULL), prerender_handle());
 }
@@ -410,9 +402,9 @@ TEST_F(InstantSearchPrerendererTest,
   // Open a search results page. Make sure the InstantSearchPrerenderer cancels
   // the active prerender request upon the receipt of empty search query.
   GURL url("https://www.google.com/alt#quux=&strk");
-  browser()->OpenURL(content::OpenURLParams(url, Referrer(), CURRENT_TAB,
-                                            ui::PAGE_TRANSITION_TYPED,
-                                            false));
+  browser()->OpenURL(content::OpenURLParams(url, Referrer(),
+                                            WindowOpenDisposition::CURRENT_TAB,
+                                            ui::PAGE_TRANSITION_TYPED, false));
   EXPECT_NE(GetPrerenderURL(), GetActiveWebContents()->GetURL());
   EXPECT_EQ(url, GetActiveWebContents()->GetURL());
   EXPECT_EQ(static_cast<PrerenderHandle*>(NULL), prerender_handle());
@@ -425,9 +417,9 @@ TEST_F(InstantSearchPrerendererTest,
   // Open a search results page. Make sure the InstantSearchPrerenderer cancels
   // the active prerender request for unsupported window dispositions.
   GURL url("https://www.google.com/alt#quux=pen&strk");
-  browser()->OpenURL(content::OpenURLParams(url, Referrer(), NEW_FOREGROUND_TAB,
-                                            ui::PAGE_TRANSITION_TYPED,
-                                            false));
+  browser()->OpenURL(content::OpenURLParams(
+      url, Referrer(), WindowOpenDisposition::NEW_FOREGROUND_TAB,
+      ui::PAGE_TRANSITION_TYPED, false));
   content::WebContents* new_tab =
       browser()->tab_strip_model()->GetWebContentsAt(1);
   EXPECT_NE(GetPrerenderURL(), new_tab->GetURL());
@@ -478,15 +470,8 @@ TEST_F(ReuseInstantSearchBasePageTest,
                                            ASCIIToUTF16("joy")));
 }
 
-#if !defined(OS_IOS) && !defined(OS_ANDROID)
+#if !defined(OS_ANDROID)
 class TestUsePrerenderPage : public InstantSearchPrerendererTest {
- protected:
-  void SetUp() override {
-    // Disable query extraction flag in field trials.
-    ASSERT_TRUE(base::FieldTrialList::CreateFieldTrial(
-        "EmbeddedSearch", "Group1 strk:20 query_extraction:0"));
-    InstantUnitTestBase::SetUpWithoutQueryExtraction();
-  }
 };
 
 TEST_F(TestUsePrerenderPage, ExtractSearchTermsAndUsePrerenderPage) {
@@ -497,9 +482,9 @@ TEST_F(TestUsePrerenderPage, ExtractSearchTermsAndUsePrerenderPage) {
   // key. Make sure UsePrerenderedPage() extracts the search terms from the URL
   // and uses the prerendered page contents.
   GURL url("https://www.google.com/alt#quux=foo");
-  browser()->OpenURL(content::OpenURLParams(url, Referrer(), CURRENT_TAB,
-                                            ui::PAGE_TRANSITION_TYPED,
-                                            false));
+  browser()->OpenURL(content::OpenURLParams(url, Referrer(),
+                                            WindowOpenDisposition::CURRENT_TAB,
+                                            ui::PAGE_TRANSITION_TYPED, false));
   EXPECT_EQ(GetPrerenderURL(), GetActiveWebContents()->GetURL());
   EXPECT_EQ(static_cast<PrerenderHandle*>(NULL), prerender_handle());
 }
@@ -509,9 +494,9 @@ TEST_F(TestUsePrerenderPage, DoNotUsePrerenderPage) {
 
   // Do not use prerendered page for renderer initiated search request.
   GURL url("https://www.google.com/alt#quux=foo");
-  browser()->OpenURL(content::OpenURLParams(url, Referrer(), CURRENT_TAB,
-                                            ui::PAGE_TRANSITION_LINK,
-                                            true  /* is_renderer_initiated */));
+  browser()->OpenURL(content::OpenURLParams(
+      url, Referrer(), WindowOpenDisposition::CURRENT_TAB,
+      ui::PAGE_TRANSITION_LINK, true /* is_renderer_initiated */));
   EXPECT_NE(GetPrerenderURL(), GetActiveWebContents()->GetURL());
   EXPECT_EQ(static_cast<PrerenderHandle*>(NULL), prerender_handle());
 }
@@ -524,7 +509,8 @@ TEST_F(TestUsePrerenderPage, SetEmbeddedSearchRequestParams) {
   // trials. Search results page URL does not contain search terms replacement
   // key.
   GURL url("https://www.google.com/url?bar=foo&aqs=chrome...0&ie=utf-8&oq=f");
-  browser()->instant_controller()->OpenInstant(CURRENT_TAB, url);
+  browser()->instant_controller()->OpenInstant(
+      WindowOpenDisposition::CURRENT_TAB, url);
   content::MockRenderProcessHost* process =
       static_cast<content::MockRenderProcessHost*>(
           prerender_contents()->GetRenderViewHost()->GetProcess());
@@ -533,13 +519,13 @@ TEST_F(TestUsePrerenderPage, SetEmbeddedSearchRequestParams) {
   ASSERT_TRUE(message);
 
   // Verify the IPC message params.
-  base::Tuple<base::string16, EmbeddedSearchRequestParams> params;
+  std::tuple<base::string16, EmbeddedSearchRequestParams> params;
   ChromeViewMsg_SearchBoxSubmit::Read(message, &params);
-  EXPECT_EQ("foo", base::UTF16ToASCII(base::get<0>(params)));
-  EXPECT_EQ("f", base::UTF16ToASCII(base::get<1>(params).original_query));
-  EXPECT_EQ("utf-8", base::UTF16ToASCII(base::get<1>(params).input_encoding));
-  EXPECT_EQ("", base::UTF16ToASCII(base::get<1>(params).rlz_parameter_value));
+  EXPECT_EQ("foo", base::UTF16ToASCII(std::get<0>(params)));
+  EXPECT_EQ("f", base::UTF16ToASCII(std::get<1>(params).original_query));
+  EXPECT_EQ("utf-8", base::UTF16ToASCII(std::get<1>(params).input_encoding));
+  EXPECT_EQ("", base::UTF16ToASCII(std::get<1>(params).rlz_parameter_value));
   EXPECT_EQ("chrome...0",
-            base::UTF16ToASCII(base::get<1>(params).assisted_query_stats));
+            base::UTF16ToASCII(std::get<1>(params).assisted_query_stats));
 }
 #endif
