@@ -6,14 +6,15 @@
 
 #include <stdint.h>
 
+#include <memory>
+#include <tuple>
 #include <vector>
 
 #include "base/command_line.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/metrics/field_trial.h"
 #include "base/strings/string16.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/tuple.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/search/search.h"
 #include "chrome/browser/search_engines/template_url_service_factory.h"
@@ -21,9 +22,9 @@
 #include "chrome/browser/ui/search/search_tab_helper.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/instant_types.h"
-#include "chrome/common/ntp_logging_events.h"
 #include "chrome/common/render_messages.h"
+#include "chrome/common/search/instant_types.h"
+#include "chrome/common/search/ntp_logging_events.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/test/base/browser_with_test_window_test.h"
 #include "chrome/test/base/search_test_utils.h"
@@ -49,16 +50,15 @@ class MockSearchIPCRouterDelegate : public SearchIPCRouter::Delegate {
 
   MOCK_METHOD1(OnInstantSupportDetermined, void(bool supports_instant));
   MOCK_METHOD1(FocusOmnibox, void(OmniboxFocusState state));
-  MOCK_METHOD3(NavigateToURL, void(const GURL&, WindowOpenDisposition, bool));
   MOCK_METHOD1(OnDeleteMostVisitedItem, void(const GURL& url));
   MOCK_METHOD1(OnUndoMostVisitedDeletion, void(const GURL& url));
   MOCK_METHOD0(OnUndoAllMostVisitedDeletions, void());
   MOCK_METHOD2(OnLogEvent, void(NTPLoggingEventType event,
                                 base::TimeDelta time));
   MOCK_METHOD2(OnLogMostVisitedImpression,
-               void(int position, const base::string16& provider));
+               void(int position, NTPLoggingTileSource tile_source));
   MOCK_METHOD2(OnLogMostVisitedNavigation,
-               void(int position, const base::string16& provider));
+               void(int position, NTPLoggingTileSource tile_source));
   MOCK_METHOD1(PasteIntoOmnibox, void(const base::string16&));
   MOCK_METHOD1(OnChromeIdentityCheck, void(const base::string16& identity));
   MOCK_METHOD0(OnHistorySyncCheck, void());
@@ -69,7 +69,6 @@ class MockSearchIPCRouterPolicy : public SearchIPCRouter::Policy {
   virtual ~MockSearchIPCRouterPolicy() {}
 
   MOCK_METHOD1(ShouldProcessFocusOmnibox, bool(bool));
-  MOCK_METHOD1(ShouldProcessNavigateToURL, bool(bool));
   MOCK_METHOD0(ShouldProcessDeleteMostVisitedItem, bool());
   MOCK_METHOD0(ShouldProcessUndoMostVisitedDeletion, bool());
   MOCK_METHOD0(ShouldProcessUndoAllMostVisitedDeletions, bool());
@@ -77,8 +76,6 @@ class MockSearchIPCRouterPolicy : public SearchIPCRouter::Policy {
   MOCK_METHOD1(ShouldProcessPasteIntoOmnibox, bool(bool));
   MOCK_METHOD0(ShouldProcessChromeIdentityCheck, bool());
   MOCK_METHOD0(ShouldProcessHistorySyncCheck, bool());
-  MOCK_METHOD0(ShouldSendSetPromoInformation, bool());
-  MOCK_METHOD0(ShouldSendSetDisplayInstantResults, bool());
   MOCK_METHOD0(ShouldSendSetSuggestionToPrefetch, bool());
   MOCK_METHOD1(ShouldSendSetInputInProgress, bool(bool));
   MOCK_METHOD0(ShouldSendOmniboxFocusChanged, bool());
@@ -113,9 +110,8 @@ class SearchIPCRouterTest : public BrowserWithTestWindowTest {
     data.alternate_urls.push_back("http://foo.com/alt#quux={searchTerms}");
     data.search_terms_replacement_key = "espv";
 
-    TemplateURL* template_url = new TemplateURL(data);
-    // Takes ownership of |template_url|.
-    template_url_service->Add(template_url);
+    TemplateURL* template_url =
+        template_url_service->Add(base::MakeUnique<TemplateURL>(data));
     template_url_service->SetUserSelectedDefaultSearchProvider(template_url);
     process()->sink().ClearMessages();
   }
@@ -142,30 +138,13 @@ class SearchIPCRouterTest : public BrowserWithTestWindowTest {
     ASSERT_NE(static_cast<SearchTabHelper*>(NULL), search_tab_helper);
     search_tab_helper->ipc_router().set_delegate_for_testing(mock_delegate());
     search_tab_helper->ipc_router().set_policy_for_testing(
-        make_scoped_ptr(new MockSearchIPCRouterPolicy));
+        base::WrapUnique(new MockSearchIPCRouterPolicy));
   }
 
   bool MessageWasSent(uint32_t id) {
     return process()->sink().GetFirstMessageMatching(id) != NULL;
   }
 
-  void VerifyDisplayInstantResultsMsg(bool expected_param_value) {
-    SetupMockDelegateAndPolicy();
-    MockSearchIPCRouterPolicy* policy = GetSearchIPCRouterPolicy();
-    EXPECT_CALL(*policy, ShouldSendSetDisplayInstantResults()).Times(1)
-        .WillOnce(testing::Return(true));
-
-    GetSearchIPCRouter().SetDisplayInstantResults();
-    const IPC::Message* message = process()->sink().GetFirstMessageMatching(
-        ChromeViewMsg_SearchBoxSetDisplayInstantResults::ID);
-    EXPECT_NE(static_cast<const IPC::Message*>(NULL), message);
-    base::Tuple<bool> display_instant_results_param;
-    ChromeViewMsg_SearchBoxSetDisplayInstantResults::Read(
-        message, &display_instant_results_param);
-    EXPECT_EQ(expected_param_value,
-              base::get<0>(display_instant_results_param));
-  }
-
   MockSearchIPCRouterDelegate* mock_delegate() { return &delegate_; }
 
   MockSearchIPCRouterPolicy* GetSearchIPCRouterPolicy() {
@@ -208,19 +187,17 @@ class SearchIPCRouterTest : public BrowserWithTestWindowTest {
 TEST_F(SearchIPCRouterTest, IgnoreMessagesFromNonInstantRenderers) {
   NavigateAndCommitActiveTab(GURL("file://foo/bar"));
   SetupMockDelegateAndPolicy();
-  GURL destination_url("www.foo.com");
-  EXPECT_CALL(*mock_delegate(), NavigateToURL(destination_url, CURRENT_TAB,
-                                              true)).Times(0);
+  EXPECT_CALL(*mock_delegate(), FocusOmnibox(OMNIBOX_FOCUS_VISIBLE)).Times(0);
   content::WebContents* contents = web_contents();
   bool is_active_tab = IsActiveTab(contents);
   EXPECT_TRUE(is_active_tab);
 
   MockSearchIPCRouterPolicy* policy = GetSearchIPCRouterPolicy();
-  EXPECT_CALL(*policy, ShouldProcessNavigateToURL(is_active_tab)).Times(0);
+  EXPECT_CALL(*policy, ShouldProcessFocusOmnibox(is_active_tab)).Times(0);
 
-  OnMessageReceived(ChromeViewHostMsg_SearchBoxNavigate(
-      contents->GetRoutingID(), GetSearchIPCRouterSeqNo(), destination_url,
-      CURRENT_TAB, true));
+  OnMessageReceived(ChromeViewHostMsg_FocusOmnibox(
+      contents->GetRoutingID(), GetSearchIPCRouterSeqNo(),
+      OMNIBOX_FOCUS_VISIBLE));
 }
 
 TEST_F(SearchIPCRouterTest, ProcessFocusOmniboxMsg) {
@@ -277,57 +254,20 @@ TEST_F(SearchIPCRouterTest, HandleTabChangedEvents) {
   EXPECT_TRUE(IsActiveTab(contents));
 }
 
-TEST_F(SearchIPCRouterTest, ProcessNavigateToURLMsg) {
-  NavigateAndCommitActiveTab(GURL(chrome::kChromeSearchLocalNtpUrl));
-  SetupMockDelegateAndPolicy();
-  GURL destination_url("www.foo.com");
-  EXPECT_CALL(*mock_delegate(), NavigateToURL(destination_url, CURRENT_TAB,
-                                              true)).Times(1);
-  content::WebContents* contents = web_contents();
-  bool is_active_tab = IsActiveTab(contents);
-  EXPECT_TRUE(is_active_tab);
-
-  MockSearchIPCRouterPolicy* policy = GetSearchIPCRouterPolicy();
-  EXPECT_CALL(*policy, ShouldProcessNavigateToURL(is_active_tab)).Times(1)
-      .WillOnce(testing::Return(true));
-
-  OnMessageReceived(ChromeViewHostMsg_SearchBoxNavigate(
-      contents->GetRoutingID(), GetSearchIPCRouterSeqNo(), destination_url,
-      CURRENT_TAB, true));
-}
-
-TEST_F(SearchIPCRouterTest, IgnoreNavigateToURLMsg) {
-  NavigateAndCommitActiveTab(GURL("chrome-search://foo/bar"));
-  SetupMockDelegateAndPolicy();
-  GURL destination_url("www.foo.com");
-  EXPECT_CALL(*mock_delegate(), NavigateToURL(destination_url, CURRENT_TAB,
-                                              true)).Times(0);
-  content::WebContents* contents = web_contents();
-  bool is_active_tab = IsActiveTab(contents);
-  EXPECT_TRUE(is_active_tab);
-
-  MockSearchIPCRouterPolicy* policy = GetSearchIPCRouterPolicy();
-  EXPECT_CALL(*policy, ShouldProcessNavigateToURL(is_active_tab)).Times(1)
-      .WillOnce(testing::Return(false));
-
-  OnMessageReceived(ChromeViewHostMsg_SearchBoxNavigate(
-      contents->GetRoutingID(), GetSearchIPCRouterSeqNo(), destination_url,
-      CURRENT_TAB, true));
-}
-
 TEST_F(SearchIPCRouterTest, ProcessLogEventMsg) {
   base::TimeDelta delta = base::TimeDelta::FromMilliseconds(123);
   NavigateAndCommitActiveTab(GURL(chrome::kChromeSearchLocalNtpUrl));
   SetupMockDelegateAndPolicy();
   MockSearchIPCRouterPolicy* policy = GetSearchIPCRouterPolicy();
-  EXPECT_CALL(*mock_delegate(), OnLogEvent(NTP_MOUSEOVER, delta)).Times(1);
+  EXPECT_CALL(*mock_delegate(), OnLogEvent(NTP_ALL_TILES_LOADED, delta))
+      .Times(1);
   EXPECT_CALL(*policy, ShouldProcessLogEvent()).Times(1)
       .WillOnce(testing::Return(true));
 
   content::WebContents* contents = web_contents();
   OnMessageReceived(ChromeViewHostMsg_LogEvent(
       contents->GetRoutingID(), GetSearchIPCRouterSeqNo(),
-      NTP_MOUSEOVER, delta));
+      NTP_ALL_TILES_LOADED, delta));
 }
 
 TEST_F(SearchIPCRouterTest, IgnoreLogEventMsg) {
@@ -335,14 +275,15 @@ TEST_F(SearchIPCRouterTest, IgnoreLogEventMsg) {
   NavigateAndCommitActiveTab(GURL("chrome-search://foo/bar"));
   SetupMockDelegateAndPolicy();
   MockSearchIPCRouterPolicy* policy = GetSearchIPCRouterPolicy();
-  EXPECT_CALL(*mock_delegate(), OnLogEvent(NTP_MOUSEOVER, delta)).Times(0);
+  EXPECT_CALL(*mock_delegate(), OnLogEvent(NTP_ALL_TILES_LOADED, delta))
+      .Times(0);
   EXPECT_CALL(*policy, ShouldProcessLogEvent()).Times(1)
       .WillOnce(testing::Return(false));
 
   content::WebContents* contents = web_contents();
   OnMessageReceived(ChromeViewHostMsg_LogEvent(
       contents->GetRoutingID(), GetSearchIPCRouterSeqNo(),
-      NTP_MOUSEOVER, delta));
+      NTP_ALL_TILES_LOADED, delta));
 }
 
 TEST_F(SearchIPCRouterTest, ProcessLogMostVisitedImpressionMsg) {
@@ -350,14 +291,14 @@ TEST_F(SearchIPCRouterTest, ProcessLogMostVisitedImpressionMsg) {
   SetupMockDelegateAndPolicy();
   MockSearchIPCRouterPolicy* policy = GetSearchIPCRouterPolicy();
   EXPECT_CALL(*mock_delegate(),
-      OnLogMostVisitedImpression(3, base::ASCIIToUTF16("Server"))).Times(1);
+      OnLogMostVisitedImpression(3, NTPLoggingTileSource::SERVER)).Times(1);
   EXPECT_CALL(*policy, ShouldProcessLogEvent()).Times(1)
       .WillOnce(testing::Return(true));
 
   content::WebContents* contents = web_contents();
   OnMessageReceived(ChromeViewHostMsg_LogMostVisitedImpression(
       contents->GetRoutingID(), GetSearchIPCRouterSeqNo(), 3,
-      base::ASCIIToUTF16("Server")));
+      NTPLoggingTileSource::SERVER));
 }
 
 TEST_F(SearchIPCRouterTest, ProcessLogMostVisitedNavigationMsg) {
@@ -365,14 +306,14 @@ TEST_F(SearchIPCRouterTest, ProcessLogMostVisitedNavigationMsg) {
   SetupMockDelegateAndPolicy();
   MockSearchIPCRouterPolicy* policy = GetSearchIPCRouterPolicy();
   EXPECT_CALL(*mock_delegate(),
-      OnLogMostVisitedNavigation(3, base::ASCIIToUTF16("Server"))).Times(1);
+      OnLogMostVisitedNavigation(3, NTPLoggingTileSource::SERVER)).Times(1);
   EXPECT_CALL(*policy, ShouldProcessLogEvent()).Times(1)
       .WillOnce(testing::Return(true));
 
   content::WebContents* contents = web_contents();
   OnMessageReceived(ChromeViewHostMsg_LogMostVisitedNavigation(
       contents->GetRoutingID(), GetSearchIPCRouterSeqNo(), 3,
-      base::ASCIIToUTF16("Server")));
+      NTPLoggingTileSource::SERVER));
 }
 
 TEST_F(SearchIPCRouterTest, ProcessChromeIdentityCheckMsg) {
@@ -522,15 +463,9 @@ TEST_F(SearchIPCRouterTest, IgnoreMessageIfThePageIsNotActive) {
   content::WebContents* contents = web_contents();
   bool is_active_tab = IsActiveTab(contents);
   GURL item_url("www.foo.com");
-  EXPECT_CALL(*mock_delegate(), NavigateToURL(item_url, CURRENT_TAB,
-                                              true)).Times(0);
-  // At this point, in a real test, the navigation would cause the
-  // SearchIPCRouter's page sequence to advance. In this test it doesn't, so
-  // we'll decrement the sequence number on this side to simulate it.
-  --page_seq_no;
-  EXPECT_CALL(*policy, ShouldProcessNavigateToURL(is_active_tab)).Times(0);
-  OnMessageReceived(ChromeViewHostMsg_SearchBoxNavigate(
-      contents->GetRoutingID(), page_seq_no, item_url, CURRENT_TAB, true));
+
+  // Navigate away from the NTP. Afterwards, all messages should be ignored.
+  NavigateAndCommitActiveTab(item_url);
 
   EXPECT_CALL(*mock_delegate(), OnDeleteMostVisitedItem(item_url)).Times(0);
   EXPECT_CALL(*policy, ShouldProcessDeleteMostVisitedItem()).Times(0);
@@ -553,11 +488,12 @@ TEST_F(SearchIPCRouterTest, IgnoreMessageIfThePageIsNotActive) {
       contents->GetRoutingID(), page_seq_no, OMNIBOX_FOCUS_VISIBLE));
 
   base::TimeDelta delta = base::TimeDelta::FromMilliseconds(123);
-  EXPECT_CALL(*mock_delegate(), OnLogEvent(NTP_MOUSEOVER, delta)).Times(0);
+  EXPECT_CALL(*mock_delegate(), OnLogEvent(NTP_ALL_TILES_LOADED, delta))
+      .Times(0);
   EXPECT_CALL(*policy, ShouldProcessLogEvent()).Times(0);
-  OnMessageReceived(ChromeViewHostMsg_LogEvent(contents->GetRoutingID(),
-                                               page_seq_no,
-                                               NTP_MOUSEOVER, delta));
+  OnMessageReceived(
+      ChromeViewHostMsg_LogEvent(contents->GetRoutingID(), page_seq_no,
+                                 NTP_ALL_TILES_LOADED, delta));
 
   base::string16 text;
   EXPECT_CALL(*mock_delegate(), PasteIntoOmnibox(text)).Times(0);
@@ -601,75 +537,6 @@ TEST_F(SearchIPCRouterTest, IgnorePasteAndOpenDropdownMsg) {
       contents->GetRoutingID(), GetSearchIPCRouterSeqNo(), text));
 }
 
-TEST_F(SearchIPCRouterTest, SendSetPromoInformationMsg) {
-  NavigateAndCommitActiveTab(GURL("chrome-search://foo/bar"));
-  SetupMockDelegateAndPolicy();
-  MockSearchIPCRouterPolicy* policy = GetSearchIPCRouterPolicy();
-  EXPECT_CALL(*policy, ShouldSendSetPromoInformation()).Times(1)
-      .WillOnce(testing::Return(true));
-
-  GetSearchIPCRouter().SetPromoInformation(true);
-  EXPECT_TRUE(MessageWasSent(ChromeViewMsg_SearchBoxPromoInformation::ID));
-}
-
-TEST_F(SearchIPCRouterTest, DoNotSendSetPromoInformationMsg) {
-  NavigateAndCommitActiveTab(GURL("chrome-search://foo/bar"));
-  SetupMockDelegateAndPolicy();
-  MockSearchIPCRouterPolicy* policy = GetSearchIPCRouterPolicy();
-  EXPECT_CALL(*policy, ShouldSendSetPromoInformation()).Times(1)
-      .WillOnce(testing::Return(false));
-
-  GetSearchIPCRouter().SetPromoInformation(false);
-  EXPECT_FALSE(MessageWasSent(ChromeViewMsg_SearchBoxPromoInformation::ID));
-}
-
-TEST_F(SearchIPCRouterTest,
-       SendSetDisplayInstantResultsMsg_EnableInstantOnResultsPage) {
-  ASSERT_TRUE(base::FieldTrialList::CreateFieldTrial(
-      "EmbeddedSearch",
-      "Group1 espv:42 query_extraction:1 prefetch_results_srp:1"));
-  NavigateAndCommitActiveTab(GURL("https://foo.com/url?espv&bar=abc"));
-
-  // Make sure ChromeViewMsg_SearchBoxSetDisplayInstantResults message param is
-  // set to true if the underlying page is a results page and
-  // "prefetch_results_srp" flag is enabled via field trials.
-  VerifyDisplayInstantResultsMsg(true);
-}
-
-TEST_F(SearchIPCRouterTest,
-       SendSetDisplayInstantResultsMsg_DisableInstantOnResultsPage) {
-  // |prefetch_results_srp" flag is disabled via field trials.
-  ASSERT_TRUE(base::FieldTrialList::CreateFieldTrial(
-      "EmbeddedSearch",
-      "Group1 espv:42 query_extraction:1 prefetch_results_srp:0"));
-  NavigateAndCommitActiveTab(GURL("https://foo.com/url?espv&bar=abc"));
-
-  // Make sure ChromeViewMsg_SearchBoxSetDisplayInstantResults message param is
-  // set to false.
-  VerifyDisplayInstantResultsMsg(false);
-}
-
-TEST_F(SearchIPCRouterTest,
-       SendSetDisplayInstantResultsMsg_EnableInstantOutsideSearchResultsPage) {
-  NavigateAndCommitActiveTab(GURL(chrome::kChromeSearchLocalNtpUrl));
-  // Make sure ChromeViewMsg_SearchBoxSetDisplayInstantResults param is set to
-  // true if the underlying page is not a search results page.
-  VerifyDisplayInstantResultsMsg(true);
-}
-
-TEST_F(SearchIPCRouterTest, DoNotSendSetDisplayInstantResultsMsg) {
-  NavigateAndCommitActiveTab(GURL("chrome-search://foo/bar"));
-  SetupMockDelegateAndPolicy();
-  MockSearchIPCRouterPolicy* policy = GetSearchIPCRouterPolicy();
-  EXPECT_CALL(*policy, ShouldSendSetDisplayInstantResults()).Times(1)
-      .WillOnce(testing::Return(false));
-
-  process()->sink().ClearMessages();
-  GetSearchIPCRouter().SetDisplayInstantResults();
-  EXPECT_FALSE(MessageWasSent(
-      ChromeViewMsg_SearchBoxSetDisplayInstantResults::ID));
-}
-
 TEST_F(SearchIPCRouterTest, SendSetSuggestionToPrefetch) {
   NavigateAndCommitActiveTab(GURL("chrome-search://foo/bar"));
   SetupMockDelegateAndPolicy();
