@@ -4,12 +4,11 @@
 
 #include "chrome/browser/ui/search/search_tab_helper.h"
 
+#include <memory>
 #include <set>
 
-#include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
-#include "base/metrics/histogram.h"
-#include "base/strings/string16.h"
+#include "base/memory/ptr_util.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/strings/string_util.h"
 #include "build/build_config.h"
 #include "chrome/browser/chrome_notification_types.h"
@@ -19,11 +18,9 @@
 #include "chrome/browser/search/search.h"
 #include "chrome/browser/signin/signin_manager_factory.h"
 #include "chrome/browser/sync/profile_sync_service_factory.h"
-#include "chrome/browser/ui/app_list/app_list_util.h"
 #include "chrome/browser/ui/browser_window.h"
 #include "chrome/browser/ui/location_bar/location_bar.h"
 #include "chrome/browser/ui/omnibox/clipboard_utils.h"
-#include "chrome/browser/ui/search/instant_search_prerenderer.h"
 #include "chrome/browser/ui/search/instant_tab.h"
 #include "chrome/browser/ui/search/search_ipc_router_policy_impl.h"
 #include "chrome/browser/ui/search/search_tab_helper_delegate.h"
@@ -31,17 +28,16 @@
 #include "chrome/browser/ui/webui/ntp/ntp_user_data_logger.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/grit/generated_resources.h"
-#include "components/browser_sync/browser/profile_sync_service.h"
+#include "components/browser_sync/profile_sync_service.h"
 #include "components/google/core/browser/google_util.h"
 #include "components/omnibox/browser/omnibox_edit_model.h"
 #include "components/omnibox/browser/omnibox_popup_model.h"
 #include "components/omnibox/browser/omnibox_view.h"
 #include "components/search/search.h"
 #include "components/signin/core/browser/signin_manager.h"
-#include "content/public/browser/navigation_controller.h"
+#include "components/strings/grit/components_strings.h"
 #include "content/public/browser/navigation_details.h"
 #include "content/public/browser/navigation_entry.h"
-#include "content/public/browser/navigation_type.h"
 #include "content/public/browser/notification_service.h"
 #include "content/public/browser/notification_source.h"
 #include "content/public/browser/render_frame_host.h"
@@ -63,7 +59,7 @@ bool IsCacheableNTP(const content::WebContents* contents) {
   const content::NavigationEntry* entry =
       contents->GetController().GetLastCommittedEntry();
   return search::NavEntryIsInstantNTP(contents, entry) &&
-         entry->GetURL() != GURL(chrome::kChromeSearchLocalNtpUrl);
+         entry->GetURL() != chrome::kChromeSearchLocalNtpUrl;
 }
 
 bool IsNTP(const content::WebContents* contents) {
@@ -71,22 +67,18 @@ bool IsNTP(const content::WebContents* contents) {
   // whereas we want the visible entry.
   const content::NavigationEntry* entry =
       contents->GetController().GetVisibleEntry();
-  if (entry && entry->GetVirtualURL() == GURL(chrome::kChromeUINewTabURL))
+  if (entry && entry->GetVirtualURL() == chrome::kChromeUINewTabURL)
     return true;
 
   return search::IsInstantNTP(contents);
 }
 
-bool IsSearchResults(const content::WebContents* contents) {
-  return !search::GetSearchTerms(contents).empty();
-}
-
 bool IsLocal(const content::WebContents* contents) {
   if (!contents)
     return false;
   const content::NavigationEntry* entry =
       contents->GetController().GetVisibleEntry();
-  return entry && entry->GetURL() == GURL(chrome::kChromeSearchLocalNtpUrl);
+  return entry && entry->GetURL() == chrome::kChromeSearchLocalNtpUrl;
 }
 
 // Returns true if |contents| are rendered inside an Instant process.
@@ -131,28 +123,24 @@ void RecordNewTabLoadTime(content::WebContents* contents) {
 // disable a feature that should not be shown to users who prefer not to sync
 // their history.
 bool IsHistorySyncEnabled(Profile* profile) {
-  ProfileSyncService* sync =
+  browser_sync::ProfileSyncService* sync =
       ProfileSyncServiceFactory::GetInstance()->GetForProfile(profile);
   return sync &&
       sync->GetPreferredDataTypes().Has(syncer::HISTORY_DELETE_DIRECTIVES);
 }
 
-bool OmniboxHasFocus(OmniboxView* omnibox) {
-  return omnibox && omnibox->model()->has_focus();
-}
-
 }  // namespace
 
 SearchTabHelper::SearchTabHelper(content::WebContents* web_contents)
     : WebContentsObserver(web_contents),
       is_search_enabled_(search::IsInstantExtendedAPIEnabled()),
       web_contents_(web_contents),
-      ipc_router_(web_contents,
-                  this,
-                  make_scoped_ptr(new SearchIPCRouterPolicyImpl(web_contents))),
+      ipc_router_(
+          web_contents,
+          this,
+          base::WrapUnique(new SearchIPCRouterPolicyImpl(web_contents))),
       instant_service_(NULL),
-      delegate_(NULL),
-      omnibox_has_focus_fn_(&OmniboxHasFocus) {
+      delegate_(NULL) {
   if (!is_search_enabled_)
     return;
 
@@ -168,15 +156,11 @@ SearchTabHelper::~SearchTabHelper() {
     instant_service_->RemoveObserver(this);
 }
 
-void SearchTabHelper::InitForPreloadedNTP() {
-  UpdateMode(true, true);
-}
-
 void SearchTabHelper::OmniboxInputStateChanged() {
   if (!is_search_enabled_)
     return;
 
-  UpdateMode(false, false);
+  UpdateMode(false);
 }
 
 void SearchTabHelper::OmniboxFocusChanged(OmniboxFocusState state,
@@ -191,50 +175,15 @@ void SearchTabHelper::OmniboxFocusChanged(OmniboxFocusState state,
   // Don't send oninputstart/oninputend updates in response to focus changes
   // if there's a navigation in progress. This prevents Chrome from sending
   // a spurious oninputend when the user accepts a match in the omnibox.
-  if (web_contents_->GetController().GetPendingEntry() == NULL) {
+  if (web_contents_->GetController().GetPendingEntry() == NULL)
     ipc_router_.SetInputInProgress(IsInputInProgress());
-
-    InstantSearchPrerenderer* prerenderer =
-        InstantSearchPrerenderer::GetForProfile(profile());
-    if (!prerenderer || !search::ShouldPrerenderInstantUrlOnOmniboxFocus())
-      return;
-
-    if (state == OMNIBOX_FOCUS_NONE) {
-      prerenderer->Cancel();
-      return;
-    }
-
-    if (!IsSearchResultsPage()) {
-      prerenderer->Init(
-          web_contents_->GetController().GetDefaultSessionStorageNamespace(),
-          web_contents_->GetContainerBounds().size());
-    }
-  }
 }
 
 void SearchTabHelper::NavigationEntryUpdated() {
   if (!is_search_enabled_)
     return;
 
-  UpdateMode(false, false);
-}
-
-void SearchTabHelper::InstantSupportChanged(bool instant_support) {
-  if (!is_search_enabled_)
-    return;
-
-  InstantSupportState new_state = instant_support ? INSTANT_SUPPORT_YES :
-      INSTANT_SUPPORT_NO;
-
-  model_.SetInstantSupportState(new_state);
-
-  content::NavigationEntry* entry =
-      web_contents_->GetController().GetLastCommittedEntry();
-  if (entry) {
-    search::SetInstantSupportStateInNavigationEntry(new_state, entry);
-    if (delegate_ && !instant_support)
-      delegate_->OnWebContentsInstantSupportDisabled(web_contents_);
-  }
+  UpdateMode(false);
 }
 
 bool SearchTabHelper::SupportsInstant() const {
@@ -254,16 +203,12 @@ void SearchTabHelper::Submit(const base::string16& text,
 void SearchTabHelper::OnTabActivated() {
   ipc_router_.OnTabActivated();
 
-  OmniboxView* omnibox_view = GetOmniboxView();
-  if (search::ShouldPrerenderInstantUrlOnOmniboxFocus() &&
-      omnibox_has_focus_fn_(omnibox_view)) {
-    InstantSearchPrerenderer* prerenderer =
-        InstantSearchPrerenderer::GetForProfile(profile());
-    if (prerenderer && !IsSearchResultsPage()) {
-      prerenderer->Init(
-          web_contents_->GetController().GetDefaultSessionStorageNamespace(),
-          web_contents_->GetContainerBounds().size());
-    }
+  if (search::IsInstantNTP(web_contents_)) {
+    // Force creation of NTPUserDataLogger, if we loaded an NTP. The
+    // NTPUserDataLogger tries to detect whether the NTP is being created at
+    // startup or from the user opening a new tab, and if we wait until later,
+    // it won't correctly detect this case.
+    NTPUserDataLogger::GetOrCreateFromWebContents(web_contents_);
   }
 }
 
@@ -271,25 +216,18 @@ void SearchTabHelper::OnTabDeactivated() {
   ipc_router_.OnTabDeactivated();
 }
 
-bool SearchTabHelper::IsSearchResultsPage() {
-  return model_.mode().is_origin_search();
-}
-
-void SearchTabHelper::RenderViewCreated(
-    content::RenderViewHost* render_view_host) {
-  ipc_router_.SetPromoInformation(IsAppLauncherEnabled());
-}
-
 void SearchTabHelper::DidStartNavigationToPendingEntry(
     const GURL& url,
-    content::NavigationController::ReloadType /* reload_type */) {
+    content::ReloadType /* reload_type */) {
   if (search::IsNTPURL(url, profile())) {
     // Set the title on any pending entry corresponding to the NTP. This
     // prevents any flickering of the tab title.
     content::NavigationEntry* entry =
         web_contents_->GetController().GetPendingEntry();
-    if (entry)
-      entry->SetTitle(l10n_util::GetStringUTF16(IDS_NEW_TAB_TITLE));
+    if (entry) {
+      web_contents_->UpdateTitleForEntry(
+          entry, l10n_util::GetStringUTF16(IDS_NEW_TAB_TITLE));
+    }
   }
 }
 
@@ -315,9 +253,10 @@ void SearchTabHelper::DidNavigateMainFrame(
   content::NavigationEntry* entry =
       web_contents_->GetController().GetLastCommittedEntry();
   if (entry && entry->GetTitle().empty() &&
-      (entry->GetVirtualURL() == GURL(chrome::kChromeUINewTabURL) ||
+      (entry->GetVirtualURL() == chrome::kChromeUINewTabURL ||
        search::NavEntryIsInstantNTP(web_contents_, entry))) {
-    entry->SetTitle(l10n_util::GetStringUTF16(IDS_NEW_TAB_TITLE));
+    web_contents_->UpdateTitleForEntry(
+        entry, l10n_util::GetStringUTF16(IDS_NEW_TAB_TITLE));
   }
 }
 
@@ -339,11 +278,7 @@ void SearchTabHelper::NavigationEntryCommitted(
   if (!load_details.is_main_frame)
     return;
 
-  if (search::ShouldAssignURLToInstantRenderer(web_contents_->GetURL(),
-                                               profile()))
-    ipc_router_.SetDisplayInstantResults();
-
-  UpdateMode(true, false);
+  UpdateMode(true);
 
   content::NavigationEntry* entry =
       web_contents_->GetController().GetVisibleEntry();
@@ -357,16 +292,12 @@ void SearchTabHelper::NavigationEntryCommitted(
     // support for the navigated page. So, copy over the Instant support from
     // the previous entry. If the page does not support Instant, update the
     // location bar from here to turn off search terms replacement.
-    search::SetInstantSupportStateInNavigationEntry(model_.instant_support(),
-                                                    entry);
     if (delegate_ && model_.instant_support() == INSTANT_SUPPORT_NO)
       delegate_->OnWebContentsInstantSupportDisabled(web_contents_);
     return;
   }
 
   model_.SetInstantSupportState(INSTANT_SUPPORT_UNKNOWN);
-  search::SetInstantSupportStateInNavigationEntry(model_.instant_support(),
-                                                  entry);
 
   if (InInstantProcess(profile(), web_contents_))
     ipc_router_.OnNavigationEntryCommitted();
@@ -382,28 +313,7 @@ void SearchTabHelper::ThemeInfoChanged(const ThemeBackgroundInfo& theme_info) {
 
 void SearchTabHelper::MostVisitedItemsChanged(
     const std::vector<InstantMostVisitedItem>& items) {
-  // When most visited change, the NTP usually reloads the tiles. This means
-  // our metrics get inconsistent. So we'd rather emit stats now.
-  InstantTab::EmitNtpStatistics(web_contents_);
   ipc_router_.SendMostVisitedItems(items);
-  LogMostVisitedItemsSource(items);
-}
-
-void SearchTabHelper::LogMostVisitedItemsSource(
-    const std::vector<InstantMostVisitedItem>& items) {
-  for (auto item : items) {
-    NTPLoggingEventType event;
-    if (item.is_server_side_suggestion) {
-      event = NTP_SERVER_SIDE_SUGGESTION;
-    } else {
-      event = NTP_CLIENT_SIDE_SUGGESTION;
-    }
-    // The metrics are emitted for each suggestion as the design requirement
-    // even the ntp_user_data_logger.cc now only supports the scenario:
-    // all suggestions are provided by server OR
-    // all suggestions are provided by client.
-    this->OnLogEvent(event, base::TimeDelta());
-  }
 }
 
 void SearchTabHelper::FocusOmnibox(OmniboxFocusState state) {
@@ -447,27 +357,6 @@ void SearchTabHelper::FocusOmnibox(OmniboxFocusState state) {
 #endif
 }
 
-void SearchTabHelper::NavigateToURL(const GURL& url,
-                                    WindowOpenDisposition disposition,
-                                    bool is_most_visited_item_url) {
-  // Make sure the specified URL is actually on the most visited or suggested
-  // items list.
-  // TODO(treib): The |is_most_visited_item_url| is meaningless: the way it's
-  // currently set by the renderer means it can't be used to decide which list
-  // of items (most visited or suggestions) to use for the validation check. Can
-  // it be removed?
-  if (!instant_service_ || !instant_service_->IsValidURLForNavigation(url))
-    return;
-
-  if (is_most_visited_item_url) {
-    content::RecordAction(
-        base::UserMetricsAction("InstantExtended.MostVisitedClicked"));
-  }
-
-  if (delegate_)
-    delegate_->NavigateOnThumbnailClick(url, disposition, web_contents_);
-}
-
 void SearchTabHelper::OnDeleteMostVisitedItem(const GURL& url) {
   DCHECK(!url.is_empty());
   if (instant_service_)
@@ -495,20 +384,20 @@ void SearchTabHelper::OnLogEvent(NTPLoggingEventType event,
 }
 
 void SearchTabHelper::OnLogMostVisitedImpression(
-    int position, const base::string16& provider) {
+    int position, NTPLoggingTileSource tile_source) {
 // TODO(kmadhusu): Move platform specific code from here and get rid of #ifdef.
 #if !defined(OS_ANDROID)
   NTPUserDataLogger::GetOrCreateFromWebContents(
-      web_contents())->LogMostVisitedImpression(position, provider);
+      web_contents())->LogMostVisitedImpression(position, tile_source);
 #endif
 }
 
 void SearchTabHelper::OnLogMostVisitedNavigation(
-    int position, const base::string16& provider) {
+    int position, NTPLoggingTileSource tile_source) {
 // TODO(kmadhusu): Move platform specific code from here and get rid of #ifdef.
 #if !defined(OS_ANDROID)
   NTPUserDataLogger::GetOrCreateFromWebContents(
-      web_contents())->LogMostVisitedNavigation(position, provider);
+      web_contents())->LogMostVisitedNavigation(position, tile_source);
 #endif
 }
 
@@ -554,15 +443,27 @@ void SearchTabHelper::OnHistorySyncCheck() {
   ipc_router_.SendHistorySyncCheckResult(IsHistorySyncEnabled(profile()));
 }
 
-void SearchTabHelper::UpdateMode(bool update_origin, bool is_preloaded_ntp) {
+void SearchTabHelper::InstantSupportChanged(bool instant_support) {
+  if (!is_search_enabled_)
+    return;
+
+  InstantSupportState new_state = instant_support ? INSTANT_SUPPORT_YES :
+      INSTANT_SUPPORT_NO;
+
+  model_.SetInstantSupportState(new_state);
+
+  if (web_contents_->GetController().GetLastCommittedEntry() && delegate_ &&
+      !instant_support) {
+    delegate_->OnWebContentsInstantSupportDisabled(web_contents_);
+  }
+}
+
+void SearchTabHelper::UpdateMode(bool update_origin) {
   SearchMode::Type type = SearchMode::MODE_DEFAULT;
   SearchMode::Origin origin = SearchMode::ORIGIN_DEFAULT;
-  if (IsNTP(web_contents_) || is_preloaded_ntp) {
+  if (IsNTP(web_contents_)) {
     type = SearchMode::MODE_NTP;
     origin = SearchMode::ORIGIN_NTP;
-  } else if (IsSearchResults(web_contents_)) {
-    type = SearchMode::MODE_SEARCH_RESULTS;
-    origin = SearchMode::ORIGIN_SEARCH;
   }
   if (!update_origin)
     origin = model_.mode().origin;
