@@ -5,7 +5,8 @@
 #ifndef CHROME_BROWSER_UI_STARTUP_SESSION_CRASHED_INFOBAR_DELEGATE_H_
 #define CHROME_BROWSER_UI_STARTUP_SESSION_CRASHED_INFOBAR_DELEGATE_H_
 
-#include "chrome/browser/infobars/confirm_infobar_delegate.h"
+#include "base/macros.h"
+#include "components/infobars/core/confirm_infobar_delegate.h"
 
 class Browser;
 class Profile;
@@ -19,14 +20,15 @@ class SessionCrashedInfoBarDelegate : public ConfirmInfoBarDelegate {
 
  private:
   explicit SessionCrashedInfoBarDelegate(Profile* profile);
-  virtual ~SessionCrashedInfoBarDelegate();
+  ~SessionCrashedInfoBarDelegate() override;
 
   // ConfirmInfoBarDelegate:
-  virtual int GetIconID() const OVERRIDE;
-  virtual base::string16 GetMessageText() const OVERRIDE;
-  virtual int GetButtons() const OVERRIDE;
-  virtual base::string16 GetButtonLabel(InfoBarButton button) const OVERRIDE;
-  virtual bool Accept() OVERRIDE;
+  infobars::InfoBarDelegate::InfoBarIdentifier GetIdentifier() const override;
+  gfx::VectorIconId GetVectorIconId() const override;
+  base::string16 GetMessageText() const override;
+  int GetButtons() const override;
+  base::string16 GetButtonLabel(InfoBarButton button) const override;
+  bool Accept() override;
 
   bool accepted_;
   Profile* profile_;
