@@ -6,7 +6,9 @@
 
 #include <stddef.h>
 
-#include <algorithm>   // For max().
+#include <algorithm>  // For max().
+#include <limits>
+#include <memory>
 #include <set>
 
 #include "apps/app_load_service.h"
@@ -15,23 +17,21 @@
 #include "base/bind_helpers.h"
 #include "base/command_line.h"
 #include "base/compiler_specific.h"
+#include "base/debug/alias.h"
 #include "base/environment.h"
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
 #include "base/lazy_instance.h"
 #include "base/logging.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/metrics/statistics_recorder.h"
 #include "base/metrics/user_metrics.h"
 #include "base/metrics/user_metrics_action.h"
-#include "base/prefs/pref_registry_simple.h"
-#include "base/prefs/pref_service.h"
-#include "base/profiler/scoped_profile.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_tokenizer.h"
+#include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/threading/thread_restrictions.h"
 #include "base/trace_event/trace_event.h"
@@ -47,24 +47,29 @@
 #include "chrome/browser/prefs/incognito_mode_prefs.h"
 #include "chrome/browser/prefs/session_startup_pref.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/profiles/profile_attributes_entry.h"
+#include "chrome/browser/profiles/profile_attributes_storage.h"
 #include "chrome/browser/profiles/profile_manager.h"
 #include "chrome/browser/profiles/profiles_state.h"
 #include "chrome/browser/search_engines/template_url_service_factory.h"
-#include "chrome/browser/ui/app_list/app_list_service.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_dialogs.h"
 #include "chrome/browser/ui/browser_finder.h"
 #include "chrome/browser/ui/browser_window.h"
 #include "chrome/browser/ui/startup/startup_browser_creator_impl.h"
 #include "chrome/browser/ui/user_manager.h"
+#include "chrome/browser/ui/webui/settings/reset_settings_handler.h"
 #include "chrome/common/chrome_constants.h"
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/chrome_result_codes.h"
 #include "chrome/common/chrome_switches.h"
+#include "chrome/common/features.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/installer/util/browser_distribution.h"
 #include "components/google/core/browser/google_util.h"
+#include "components/prefs/pref_registry_simple.h"
+#include "components/prefs/pref_service.h"
 #include "components/search_engines/util.h"
 #include "components/signin/core/common/profile_management_switches.h"
 #include "components/url_formatter/url_fixer.h"
@@ -74,9 +79,10 @@
 #include "content/public/common/content_switches.h"
 #include "extensions/common/switches.h"
 #include "net/base/port_util.h"
+#include "printing/features/features.h"
 
 #if defined(USE_ASH)
-#include "ash/shell.h"
+#include "ash/shell.h"  // nogncheck
 #endif
 
 #if defined(OS_CHROMEOS)
@@ -85,11 +91,12 @@
 #include "chrome/browser/chromeos/profiles/profile_helper.h"
 #include "chrome/browser/lifetime/application_lifetime.h"
 #include "chromeos/chromeos_switches.h"
+#include "chromeos/cryptohome/cryptohome_parameters.h"
 #include "components/user_manager/user_manager.h"
 #endif
 
 #if defined(TOOLKIT_VIEWS) && defined(OS_LINUX)
-#include "ui/events/devices/x11/touch_factory_x11.h"
+#include "ui/events/devices/x11/touch_factory_x11.h"  // nogncheck
 #endif
 
 #if defined(OS_MACOSX)
@@ -98,28 +105,23 @@
 
 #if defined(OS_WIN)
 #include "chrome/browser/metrics/jumplist_metrics_win.h"
-#include "components/search_engines/desktop_search_win.h"
 #endif
 
-#if defined(ENABLE_PRINT_PREVIEW)
+#if BUILDFLAG(ENABLE_PRINT_PREVIEW)
 #include "chrome/browser/printing/cloud_print/cloud_print_proxy_service.h"
 #include "chrome/browser/printing/cloud_print/cloud_print_proxy_service_factory.h"
 #include "chrome/browser/printing/print_dialog_cloud.h"
 #endif
 
+#if BUILDFLAG(ENABLE_APP_LIST)
+#include "chrome/browser/ui/app_list/app_list_service.h"
+#endif
+
 using content::BrowserThread;
 using content::ChildProcessSecurityPolicy;
 
 namespace {
 
-#if defined(OS_WIN)
-const wchar_t kSetDefaultBrowserHelpUrl[] =
-    L"https://support.google.com/chrome?p=default_browser";
-
-// Not thread-safe. Always use or modify this callback on the UI thread.
-base::Closure* g_default_browser_callback = nullptr;
-#endif  // defined(OS_WIN)
-
 // Keeps track on which profiles have been launched.
 class ProfileLaunchObserver : public content::NotificationObserver {
  public:
@@ -166,10 +168,7 @@ class ProfileLaunchObserver : public content::NotificationObserver {
 
   void AddLaunched(Profile* profile) {
     launched_profiles_.insert(profile);
-    // Since the startup code only executes for browsers launched in
-    // desktop mode, i.e., HOST_DESKTOP_TYPE_NATIVE. Ash should never get here.
-    if (chrome::FindBrowserWithProfile(profile,
-                                       chrome::HOST_DESKTOP_TYPE_NATIVE)) {
+    if (chrome::FindBrowserWithProfile(profile)) {
       // A browser may get opened before we get initialized (e.g., in tests),
       // so we never see the NOTIFICATION_BROWSER_WINDOW_READY for it.
       opened_profiles_.insert(profile);
@@ -216,8 +215,7 @@ class ProfileLaunchObserver : public content::NotificationObserver {
   void ActivateProfile() {
     // We need to test again, in case the profile got deleted in the mean time.
     if (profile_to_activate_) {
-      Browser* browser = chrome::FindBrowserWithProfile(
-          profile_to_activate_, chrome::HOST_DESKTOP_TYPE_NATIVE);
+      Browser* browser = chrome::FindBrowserWithProfile(profile_to_activate_);
       // |profile| may never get launched, e.g., if it only had
       // incognito Windows and one of them was used to exit Chrome.
       // So it won't have a browser in that case.
@@ -273,27 +271,27 @@ bool ShowUserManagerOnStartupIfNeeded(
   // ChromeOS never shows the User Manager on startup.
   return false;
 #else
-  const ProfileInfoCache& profile_info =
-      g_browser_process->profile_manager()->GetProfileInfoCache();
-  size_t profile_index = profile_info.GetIndexOfProfileWithPath(
-      last_used_profile->GetPath());
+  ProfileAttributesEntry* entry = nullptr;
+  bool has_entry =
+      g_browser_process->profile_manager()
+          ->GetProfileAttributesStorage()
+          .GetProfileAttributesWithPath(last_used_profile->GetPath(), &entry);
 
-  if (profile_index == std::string::npos ||
-      !profile_info.ProfileIsSigninRequiredAtIndex(profile_index)) {
+  if (!has_entry || !entry->IsSigninRequired()) {
     // Signin is not required. However, guest, system or locked profiles cannot
     // be re-opened on startup. The only exception is if there's already a Guest
     // window open in a separate process (for example, launching a new browser
     // after clicking on a downloaded file in Guest mode).
     if ((!last_used_profile->IsGuestSession() &&
          !last_used_profile->IsSystemProfile()) ||
-        (chrome::GetTotalBrowserCountForProfile(
-           last_used_profile->GetOffTheRecordProfile()) > 0)) {
+        (chrome::GetBrowserCount(last_used_profile->GetOffTheRecordProfile()) >
+         0)) {
       return false;
     }
   }
 
   // Show the User Manager.
-  profiles::UserManagerProfileSelected action =
+  profiles::UserManagerAction action =
       command_line.HasSwitch(switches::kShowAppList) ?
           profiles::USER_MANAGER_SELECT_PROFILE_APP_LAUNCHER :
           profiles::USER_MANAGER_SELECT_PROFILE_NO_ACTION;
@@ -307,8 +305,7 @@ bool ShowUserManagerOnStartupIfNeeded(
 
 StartupBrowserCreator::StartupBrowserCreator()
     : is_default_browser_dialog_suppressed_(false),
-      show_main_browser_window_(true) {
-}
+      show_main_browser_window_(true) {}
 
 StartupBrowserCreator::~StartupBrowserCreator() {}
 
@@ -327,10 +324,9 @@ bool StartupBrowserCreator::Start(const base::CommandLine& cmd_line,
                                   Profile* last_used_profile,
                                   const Profiles& last_opened_profiles) {
   TRACE_EVENT0("startup", "StartupBrowserCreator::Start");
-  TRACK_SCOPED_REGION("Startup", "StartupBrowserCreator::Start");
   SCOPED_UMA_HISTOGRAM_TIMER("Startup.StartupBrowserCreator_Start");
   return ProcessCmdLineImpl(cmd_line, cur_dir, true, last_used_profile,
-                            last_opened_profiles, this);
+                            last_opened_profiles);
 }
 
 // static
@@ -373,21 +369,8 @@ bool StartupBrowserCreator::LaunchBrowser(
     StartupBrowserCreatorImpl lwp(cur_dir, command_line, this, is_first_run);
     const std::vector<GURL> urls_to_launch =
         GetURLsFromCommandLine(command_line, cur_dir, profile);
-    chrome::HostDesktopType host_desktop_type =
-        chrome::HOST_DESKTOP_TYPE_NATIVE;
-
-#if defined(USE_ASH) && !defined(OS_CHROMEOS)
-    // We want to maintain only one type of instance for now, either ASH
-    // or desktop.
-    // TODO(shrikant): Remove this code once we decide on running both desktop
-    // and ASH instances side by side.
-    if (ash::Shell::HasInstance())
-      host_desktop_type = chrome::HOST_DESKTOP_TYPE_ASH;
-#endif
-
-    const bool launched = lwp.Launch(profile, urls_to_launch,
-                               in_synchronous_profile_launch_,
-                               host_desktop_type);
+    const bool launched =
+        lwp.Launch(profile, urls_to_launch, in_synchronous_profile_launch_);
     in_synchronous_profile_launch_ = false;
     if (!launched) {
       LOG(ERROR) << "launch error";
@@ -463,14 +446,14 @@ SessionStartupPref StartupBrowserCreator::GetSessionStartupPref(
 
   // A browser starting for a profile being unlocked should always restore.
   if (!profile->IsGuestSession()) {
-    ProfileInfoCache& info_cache =
-        g_browser_process->profile_manager()->GetProfileInfoCache();
-    size_t index = info_cache.GetIndexOfProfileWithPath(profile->GetPath());
+    ProfileAttributesEntry* entry = nullptr;
+    bool has_entry =
+        g_browser_process->profile_manager()
+            ->GetProfileAttributesStorage()
+            .GetProfileAttributesWithPath(profile->GetPath(), &entry);
 
-    if (index != std::string::npos &&
-        info_cache.ProfileIsSigninRequiredAtIndex(index)) {
+    if (has_entry && entry->IsSigninRequired())
       pref.type = SessionStartupPref::LAST;
-    }
   }
 
   if (pref.type == SessionStartupPref::LAST &&
@@ -495,43 +478,27 @@ void StartupBrowserCreator::RegisterLocalStatePrefs(
 #if defined(OS_WIN)
   registry->RegisterStringPref(prefs::kLastWelcomedOSVersion, std::string());
   registry->RegisterBooleanPref(prefs::kWelcomePageOnOSUpgradeEnabled, true);
+  registry->RegisterBooleanPref(prefs::kHasSeenWin10PromoPage, false);
 #endif
+  registry->RegisterBooleanPref(prefs::kSuppressUnsupportedOSWarning, false);
   registry->RegisterBooleanPref(prefs::kWasRestarted, false);
 }
 
-
-#if defined(OS_WIN)
 // static
-bool StartupBrowserCreator::SetDefaultBrowserCallback(
-    const base::Closure& callback) {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  if (!g_default_browser_callback) {
-    // This won't leak because the worker class invoking this function always
-    // calls ClearDefaultBrowserCallback() in its destructor.
-    g_default_browser_callback = new base::Closure(callback);
-    return true;
-  }
-  return false;
+void StartupBrowserCreator::RegisterProfilePrefs(PrefRegistrySimple* registry) {
+  // Default to true so that existing users are not shown the Welcome page.
+  // ProfileManager handles setting this to false for new profiles upon
+  // creation.
+  registry->RegisterBooleanPref(prefs::kHasSeenWelcomePage, true);
 }
 
-// static
-void StartupBrowserCreator::ClearDefaultBrowserCallback() {
-  DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  delete g_default_browser_callback;
-  g_default_browser_callback = nullptr;
-}
-
-// static
-const wchar_t* StartupBrowserCreator::GetDefaultBrowserUrl() {
-  return kSetDefaultBrowserHelpUrl;
-}
-#endif  // defined(OS_WIN)
-
 // static
 std::vector<GURL> StartupBrowserCreator::GetURLsFromCommandLine(
     const base::CommandLine& command_line,
     const base::FilePath& cur_dir,
     Profile* profile) {
+  DCHECK(profile);
+
   std::vector<GURL> urls;
 
   const base::CommandLine::StringVector& params = command_line.GetArgs();
@@ -557,64 +524,60 @@ std::vector<GURL> StartupBrowserCreator::GetURLsFromCommandLine(
     //  http://code.google.com/p/chromium/issues/detail?id=60641
     GURL url = GURL(param.MaybeAsASCII());
 
-#if defined(OS_WIN)
-    TemplateURLService* template_url_service =
-        TemplateURLServiceFactory::GetForProfile(profile);
-    DCHECK(template_url_service);
-    base::string16 search_terms;
-    if (DetectWindowsDesktopSearch(
-            url, template_url_service->search_terms_data(), &search_terms)) {
-      base::RecordAction(base::UserMetricsAction("DesktopSearch"));
-
-      if (ShouldRedirectWindowsDesktopSearchToDefaultSearchEngine(
-            profile->GetPrefs())) {
-        const GURL search_url(GetDefaultSearchURLForSearchTerms(
-            template_url_service, search_terms));
-        if (search_url.is_valid()) {
-          urls.push_back(search_url);
-          continue;
-        }
-      }
-    }
-#endif  // defined(OS_WIN)
-
     // http://crbug.com/371030: Only use URLFixerUpper if we don't have a valid
     // URL, otherwise we will look in the current directory for a file named
     // 'about' if the browser was started with a about:foo argument.
-    if (!url.is_valid()) {
+    // http://crbug.com/424991: Always use URLFixerUpper on file:// URLs,
+    // otherwise we wouldn't correctly handle '#' in a file name.
+    if (!url.is_valid() || url.SchemeIsFile()) {
       base::ThreadRestrictions::ScopedAllowIO allow_io;
       url = url_formatter::FixupRelativeFile(cur_dir, param);
     }
     // Exclude dangerous schemes.
-    if (url.is_valid()) {
-      ChildProcessSecurityPolicy* policy =
-          ChildProcessSecurityPolicy::GetInstance();
-      if (policy->IsWebSafeScheme(url.scheme()) ||
-          url.SchemeIs(url::kFileScheme) ||
+    if (!url.is_valid())
+      continue;
+
+    const GURL settings_url = GURL(chrome::kChromeUISettingsURL);
+    bool url_points_to_an_approved_settings_page = false;
 #if defined(OS_CHROMEOS)
-          // In ChromeOS, allow any settings page to be specified on the command
-          // line. See ExistingUserController::OnLoginSuccess.
-          (url.spec().find(chrome::kChromeUISettingsURL) == 0) ||
+    // In ChromeOS, allow any settings page to be specified on the command line.
+    url_points_to_an_approved_settings_page =
+        url.GetOrigin() == settings_url.GetOrigin();
 #else
-          ((url.spec().find(std::string(chrome::kChromeUISettingsURL) +
-                            chrome::kResetProfileSettingsSubPage) == 0)) ||
-#endif
-          (url.spec().compare(url::kAboutBlankURL) == 0)) {
-        urls.push_back(url);
-      }
+    // Exposed for external cleaners to offer a settings reset to the
+    // user. The allowed URLs must match exactly.
+    const GURL reset_settings_url =
+        settings_url.Resolve(chrome::kResetProfileSettingsSubPage);
+    url_points_to_an_approved_settings_page = url == reset_settings_url;
+#if defined(OS_WIN)
+    // On Windows, also allow a hash for the Chrome Cleanup Tool.
+    const GURL reset_settings_url_with_cct_hash = reset_settings_url.Resolve(
+        std::string("#") +
+        settings::ResetSettingsHandler::kCctResetSettingsHash);
+    url_points_to_an_approved_settings_page =
+        url_points_to_an_approved_settings_page ||
+        url == reset_settings_url_with_cct_hash;
+#endif  // defined(OS_WIN)
+#endif  // defined(OS_CHROMEOS)
+
+    ChildProcessSecurityPolicy* policy =
+        ChildProcessSecurityPolicy::GetInstance();
+    if (policy->IsWebSafeScheme(url.scheme()) ||
+        url.SchemeIs(url::kFileScheme) ||
+        url_points_to_an_approved_settings_page ||
+        (url.spec().compare(url::kAboutBlankURL) == 0)) {
+      urls.push_back(url);
     }
   }
   return urls;
 }
 
-// static
 bool StartupBrowserCreator::ProcessCmdLineImpl(
     const base::CommandLine& command_line,
     const base::FilePath& cur_dir,
     bool process_startup,
     Profile* last_used_profile,
-    const Profiles& last_opened_profiles,
-    StartupBrowserCreator* browser_creator) {
+    const Profiles& last_opened_profiles) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   TRACE_EVENT0("startup", "StartupBrowserCreator::ProcessCmdLineImpl");
 
@@ -626,7 +589,7 @@ bool StartupBrowserCreator::ProcessCmdLineImpl(
 
   bool silent_launch = false;
 
-#if defined(ENABLE_PRINT_PREVIEW)
+#if BUILDFLAG(ENABLE_PRINT_PREVIEW)
   // If we are just displaying a print dialog we shouldn't open browser
   // windows.
   if (command_line.HasSwitch(switches::kCloudPrintFile) &&
@@ -634,7 +597,7 @@ bool StartupBrowserCreator::ProcessCmdLineImpl(
                                                            command_line)) {
     silent_launch = true;
   }
-#endif  // defined(ENABLE_PRINT_PREVIEW)
+#endif  // BUILDFLAG(ENABLE_PRINT_PREVIEW)
 
   if (command_line.HasSwitch(switches::kExplicitlyAllowedPorts)) {
     std::string allowed_ports =
@@ -680,7 +643,9 @@ bool StartupBrowserCreator::ProcessCmdLineImpl(
   // possible. We should instead cleanly exit and go back to the OOBE screen,
   // where we will launch again after the timeout has expired.
   if (chromeos::DemoAppLauncher::IsDemoAppSession(
-      command_line.GetSwitchValueASCII(chromeos::switches::kLoginUser))) {
+          cryptohome::Identification::FromString(
+              command_line.GetSwitchValueASCII(chromeos::switches::kLoginUser))
+              .GetAccountId())) {
     chrome::AttemptUserExit();
     return false;
   }
@@ -729,7 +694,7 @@ bool StartupBrowserCreator::ProcessCmdLineImpl(
     // chrome to shut down.
     // TODO(jackhou): Do this properly once keep-alive is handled by the
     // background page of apps. Tracked at http://crbug.com/175381
-    if (chrome::GetTotalBrowserCountForProfile(last_used_profile) != 0)
+    if (chrome::GetBrowserCount(last_used_profile) != 0)
       return true;
   }
 
@@ -750,24 +715,11 @@ bool StartupBrowserCreator::ProcessCmdLineImpl(
     // chrome to shut down.
     // TODO(jackhou): Do this properly once keep-alive is handled by the
     // background page of apps. Tracked at http://crbug.com/175381
-    if (chrome::GetTotalBrowserCountForProfile(last_used_profile) != 0)
+    if (chrome::GetBrowserCount(last_used_profile) != 0)
       return true;
   }
 
 #if defined(OS_WIN)
-  // Intercept a specific url when setting the default browser asynchronously.
-  // This only happens on Windows 10+.
-  if (g_default_browser_callback) {
-    base::CommandLine::StringType default_browser_url_(
-        kSetDefaultBrowserHelpUrl);
-    for (const auto& arg : command_line.GetArgs()) {
-      if (arg == default_browser_url_) {
-        g_default_browser_callback->Run();
-        return true;
-      }
-    }
-  }
-
   // Log whether this process was a result of an action in the Windows Jumplist.
   if (command_line.HasSwitch(switches::kWinJumplistAction)) {
     jumplist::LogJumplistActionFromSwitchValue(
@@ -788,6 +740,7 @@ bool StartupBrowserCreator::ProcessCmdLineImpl(
   // - Only incognito windows were open when the browser exited.
   // |last_used_profile| is the last used incognito profile. Restoring it will
   // create a browser window for the corresponding original profile.
+  // - All of the last opened profiles fail to initialize.
   if (last_opened_profiles.empty()) {
     if (ShowUserManagerOnStartupIfNeeded(last_used_profile, command_line))
       return true;
@@ -795,9 +748,8 @@ bool StartupBrowserCreator::ProcessCmdLineImpl(
     Profile* profile_to_open = last_used_profile->IsGuestSession() ?
         last_used_profile->GetOffTheRecordProfile() : last_used_profile;
 
-    if (!browser_creator->LaunchBrowser(command_line, profile_to_open,
-                                        cur_dir, is_process_startup,
-                                        is_first_run)) {
+    if (!LaunchBrowser(command_line, profile_to_open, cur_dir,
+                       is_process_startup, is_first_run)) {
       return false;
     }
   } else {
@@ -812,7 +764,6 @@ bool StartupBrowserCreator::ProcessCmdLineImpl(
       last_used_profile = last_opened_profiles[0];
     }
 #endif
-
     // Launch the last used profile with the full command line, and the other
     // opened profiles without the URLs to launch.
     base::CommandLine command_line_without_urls(command_line.GetProgram());
@@ -823,9 +774,36 @@ bool StartupBrowserCreator::ProcessCmdLineImpl(
       command_line_without_urls.AppendSwitchNative(switch_it->first,
                                                    switch_it->second);
     }
+
+    // TODO(scottmg): DEBUG_ variables added for https://crbug.com/614753.
+    size_t DEBUG_num_profiles_on_entry = last_opened_profiles.size();
+    base::debug::Alias(&DEBUG_num_profiles_on_entry);
+    int DEBUG_loop_counter = 0;
+    base::debug::Alias(&DEBUG_loop_counter);
+
+    base::debug::Alias(&last_opened_profiles);
+    const Profile* DEBUG_profile_0 = nullptr;
+    const Profile* DEBUG_profile_1 = nullptr;
+    if (last_opened_profiles.size() > 0)
+      DEBUG_profile_0 = last_opened_profiles[0];
+    if (last_opened_profiles.size() > 1)
+      DEBUG_profile_1 = last_opened_profiles[1];
+    base::debug::Alias(&DEBUG_profile_0);
+    base::debug::Alias(&DEBUG_profile_1);
+
+    size_t DEBUG_num_profiles_at_loop_start =
+        std::numeric_limits<size_t>::max();
+    base::debug::Alias(&DEBUG_num_profiles_at_loop_start);
+
+    Profiles::const_iterator DEBUG_it_begin = last_opened_profiles.begin();
+    base::debug::Alias(&DEBUG_it_begin);
+    Profiles::const_iterator DEBUG_it_end = last_opened_profiles.end();
+    base::debug::Alias(&DEBUG_it_end);
+
     // Launch the profiles in the order they became active.
     for (Profiles::const_iterator it = last_opened_profiles.begin();
-         it != last_opened_profiles.end(); ++it) {
+         it != last_opened_profiles.end(); ++it, ++DEBUG_loop_counter) {
+      DEBUG_num_profiles_at_loop_start = last_opened_profiles.size();
       DCHECK(!(*it)->IsGuestSession());
       // Don't launch additional profiles which would only open a new tab
       // page. When restarting after an update, all profiles will reopen last
@@ -836,10 +814,9 @@ bool StartupBrowserCreator::ProcessCmdLineImpl(
           startup_pref.type == SessionStartupPref::DEFAULT &&
           !HasPendingUncleanExit(*it))
         continue;
-
-      if (!browser_creator->LaunchBrowser((*it == last_used_profile) ?
-          command_line : command_line_without_urls, *it, cur_dir,
-          is_process_startup, is_first_run))
+      if (!LaunchBrowser((*it == last_used_profile) ? command_line
+                                                    : command_line_without_urls,
+                         *it, cur_dir, is_process_startup, is_first_run))
         return false;
       // We've launched at least one browser.
       is_process_startup = chrome::startup::IS_NOT_PROCESS_STARTUP;
@@ -893,7 +870,9 @@ void StartupBrowserCreator::ProcessCommandLineOnProfileCreated(
     Profile::CreateStatus status) {
   if (status != Profile::CREATE_STATUS_INITIALIZED)
     return;
-  ProcessCmdLineImpl(command_line, cur_dir, false, profile, Profiles(), NULL);
+  StartupBrowserCreator startup_browser_creator;
+  startup_browser_creator.ProcessCmdLineImpl(command_line, cur_dir, false,
+                                             profile, Profiles());
 }
 
 // static
@@ -906,14 +885,15 @@ void StartupBrowserCreator::ProcessCommandLineAlreadyRunning(
 
   // The profile isn't loaded yet and so needs to be loaded asynchronously.
   if (!profile) {
-    profile_manager->CreateProfileAsync(profile_path,
-        base::Bind(&StartupBrowserCreator::ProcessCommandLineOnProfileCreated,
-                   command_line, cur_dir), base::string16(), std::string(),
-                   std::string());
+    profile_manager->CreateProfileAsync(
+        profile_path,
+        base::Bind(&ProcessCommandLineOnProfileCreated, command_line, cur_dir),
+        base::string16(), std::string(), std::string());
     return;
   }
-
-  ProcessCmdLineImpl(command_line, cur_dir, false, profile, Profiles(), NULL);
+  StartupBrowserCreator startup_browser_creator;
+  startup_browser_creator.ProcessCmdLineImpl(command_line, cur_dir, false,
+                                             profile, Profiles());
 }
 
 // static
@@ -933,13 +913,86 @@ base::FilePath GetStartupProfilePath(const base::FilePath& user_data_dir,
         command_line.GetSwitchValuePath(switches::kProfileDirectory));
   }
 
+#if BUILDFLAG(ENABLE_APP_LIST)
   // If we are showing the app list then chrome isn't shown so load the app
   // list's profile rather than chrome's.
-  if (command_line.HasSwitch(switches::kShowAppList)) {
-    return AppListService::Get(chrome::HOST_DESKTOP_TYPE_NATIVE)->
-        GetProfilePath(user_data_dir);
-  }
+  if (command_line.HasSwitch(switches::kShowAppList))
+    return AppListService::Get()->GetProfilePath(user_data_dir);
+#endif
 
   return g_browser_process->profile_manager()->GetLastUsedProfileDir(
       user_data_dir);
 }
+
+#if !defined(OS_CHROMEOS) && !defined(OS_ANDROID)
+Profile* GetStartupProfile(const base::FilePath& user_data_dir,
+                           const base::CommandLine& command_line) {
+  ProfileManager* profile_manager = g_browser_process->profile_manager();
+
+  base::FilePath profile_path =
+      GetStartupProfilePath(user_data_dir, command_line);
+  Profile* profile = profile_manager->GetProfile(profile_path);
+
+  // If there is no entry in profile attributes storage, the profile is deleted,
+  // and we should show the user manager. Also, when using
+  // --new-profile-management, if the profile is locked we should show the user
+  // manager as well. When neither of these is true, we can safely start up with
+  // |profile|.
+  auto* storage = &profile_manager->GetProfileAttributesStorage();
+  ProfileAttributesEntry* entry;
+  bool has_entry = storage->GetProfileAttributesWithPath(profile_path, &entry);
+  if (has_entry && (!switches::IsNewProfileManagement() ||
+                    !entry->IsSigninRequired() || !profile)) {
+    return profile;
+  }
+
+  // We want to show the user manager. To indicate this, return the guest
+  // profile. However, we can only do this if the system profile (where the user
+  // manager lives) also exists (or is creatable).
+  return profile_manager->GetProfile(ProfileManager::GetSystemProfilePath()) ?
+         profile_manager->GetProfile(ProfileManager::GetGuestProfilePath()) :
+         nullptr;
+}
+
+Profile* GetFallbackStartupProfile() {
+  ProfileManager* profile_manager = g_browser_process->profile_manager();
+  // The only known reason for profiles to fail initialization is being unable
+  // to create the profile directory, and this has already happened in
+  // GetStartupProfilePath() before calling this function. In this case,
+  // creation of new profiles is expected to fail. So only existing profiles are
+  // attempted for fallback.
+
+  // If the last used profile could not be initialized, see if any of other last
+  // opened profiles can be initialized successfully.
+  auto* storage = &profile_manager->GetProfileAttributesStorage();
+  for (Profile* profile : ProfileManager::GetLastOpenedProfiles()) {
+    // Return any profile that is not locked.
+    ProfileAttributesEntry* entry;
+    bool has_entry = storage->GetProfileAttributesWithPath(profile->GetPath(),
+                                                           &entry);
+    if (!has_entry || !entry->IsSigninRequired())
+      return profile;
+  }
+
+  // Couldn't initialize any last opened profiles. Try to show the user manager,
+  // which requires successful initialization of the guest and system profiles.
+  Profile* guest_profile =
+      profile_manager->GetProfile(ProfileManager::GetGuestProfilePath());
+  Profile* system_profile =
+      profile_manager->GetProfile(ProfileManager::GetSystemProfilePath());
+  if (guest_profile && system_profile)
+    return guest_profile;
+
+  // Couldn't show the user manager either. Try to open any profile that is not
+  // locked.
+  for (ProfileAttributesEntry* entry : storage->GetAllProfilesAttributes()) {
+    if (!entry->IsSigninRequired()) {
+      Profile* profile = profile_manager->GetProfile(entry->GetPath());
+      if (profile)
+        return profile;
+    }
+  }
+
+  return nullptr;
+}
+#endif  // !defined(OS_CHROMEOS) && !defined(OS_ANDROID)
