@@ -10,7 +10,8 @@
 #include "base/location.h"
 #include "base/profiler/scoped_tracker.h"
 #include "base/single_thread_task_runner.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
+#include "base/time/time.h"
 #include "chrome/browser/extensions/extension_message_bubble_controller.h"
 #include "chrome/browser/extensions/extension_util.h"
 #include "chrome/browser/profiles/profile.h"
@@ -18,7 +19,8 @@
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_window.h"
 #include "chrome/browser/ui/extensions/extension_action_view_controller.h"
-#include "chrome/browser/ui/extensions/extension_message_bubble_factory.h"
+#include "chrome/browser/ui/extensions/extension_message_bubble_bridge.h"
+#include "chrome/browser/ui/extensions/settings_api_bubble_helpers.h"
 #include "chrome/browser/ui/layout_constants.h"
 #include "chrome/browser/ui/tabs/tab_strip_model.h"
 #include "chrome/browser/ui/toolbar/component_toolbar_actions_factory.h"
@@ -26,14 +28,13 @@
 #include "chrome/browser/ui/toolbar/toolbar_actions_bar_delegate.h"
 #include "chrome/browser/ui/toolbar/toolbar_actions_bar_observer.h"
 #include "chrome/common/pref_names.h"
+#include "chrome/grit/theme_resources.h"
 #include "components/crx_file/id_util.h"
 #include "components/pref_registry/pref_registry_syncable.h"
 #include "extensions/browser/extension_system.h"
 #include "extensions/browser/runtime_data.h"
 #include "extensions/common/extension.h"
 #include "extensions/common/feature_switch.h"
-#include "grit/theme_resources.h"
-#include "ui/base/resource/material_design/material_design_controller.h"
 #include "ui/base/resource/resource_bundle.h"
 #include "ui/gfx/image/image_skia.h"
 
@@ -43,25 +44,6 @@ using WeakToolbarActions = std::vector<ToolbarActionViewController*>;
 
 enum DimensionType { WIDTH, HEIGHT };
 
-// Returns the width or height of the toolbar action icon size.
-int GetIconDimension(DimensionType type) {
-  if (ui::MaterialDesignController::IsModeMaterial())
-    return 28;
-
-  static bool initialized = false;
-  static int icon_height = 0;
-  static int icon_width = 0;
-  if (!initialized) {
-    initialized = true;
-    gfx::ImageSkia* skia =
-        ui::ResourceBundle::GetSharedInstance().GetImageSkiaNamed(
-            IDR_BROWSER_ACTION);
-    icon_height = skia->height();
-    icon_width = skia->width();
-  }
-  return type == WIDTH ? icon_width : icon_height;
-}
-
 // Takes a reference vector |reference| of length n, where n is less than or
 // equal to the length of |to_sort|, and rearranges |to_sort| so that
 // |to_sort|'s first n elements match the n elements of |reference| (the order
@@ -97,6 +79,9 @@ void SortContainer(std::vector<Type1>* to_sort,
   }
 }
 
+// How long to wait until showing an extension message bubble.
+int g_extension_bubble_appearance_wait_time_in_seconds = 5;
+
 }  // namespace
 
 // static
@@ -121,12 +106,17 @@ ToolbarActionsBar::ToolbarActionsBar(ToolbarActionsBarDelegate* delegate,
       model_observer_(this),
       suppress_layout_(false),
       suppress_animation_(true),
-      checked_extension_bubble_(false),
+      should_check_extension_bubble_(!main_bar),
       is_drag_in_progress_(false),
       popped_out_action_(nullptr),
+      is_popped_out_sticky_(false),
+      is_showing_bubble_(false),
+      tab_strip_observer_(this),
       weak_ptr_factory_(this) {
   if (model_)  // |model_| can be null in unittests.
     model_observer_.Add(model_);
+
+  tab_strip_observer_.Add(browser_->tab_strip_model());
 }
 
 ToolbarActionsBar::~ToolbarActionsBar() {
@@ -134,19 +124,24 @@ ToolbarActionsBar::~ToolbarActionsBar() {
   // the order of deletion between the views and the ToolbarActionsBar.
   DCHECK(toolbar_actions_.empty()) <<
       "Must call DeleteActions() before destruction.";
-  FOR_EACH_OBSERVER(ToolbarActionsBarObserver, observers_,
-                    OnToolbarActionsBarDestroyed());
+  for (ToolbarActionsBarObserver& observer : observers_)
+    observer.OnToolbarActionsBarDestroyed();
 }
 
 // static
 int ToolbarActionsBar::IconWidth(bool include_padding) {
-  return GetIconDimension(WIDTH) +
+  return IconHeight() +
          (include_padding ? GetLayoutConstant(TOOLBAR_STANDARD_SPACING) : 0);
 }
 
 // static
 int ToolbarActionsBar::IconHeight() {
-  return GetIconDimension(HEIGHT);
+#if defined(OS_MACOSX)
+  // On the Mac, the spec is a 24x24 button in a 28x28 space.
+  return 24;
+#else
+  return 28;
+#endif
 }
 
 // static
@@ -183,9 +178,7 @@ gfx::Size ToolbarActionsBar::GetPreferredSize() const {
 }
 
 int ToolbarActionsBar::GetMinimumWidth() const {
-  if (!platform_settings_.chevron_enabled || toolbar_actions_.empty())
-    return platform_settings_.item_spacing;
-  return 2 * platform_settings_.item_spacing + delegate_->GetChevronWidth();
+  return platform_settings_.item_spacing;
 }
 
 int ToolbarActionsBar::GetMaximumWidth() const {
@@ -195,18 +188,7 @@ int ToolbarActionsBar::GetMaximumWidth() const {
 int ToolbarActionsBar::IconCountToWidth(int icons) const {
   if (icons < 0)
     icons = toolbar_actions_.size();
-  const bool display_chevron =
-      platform_settings_.chevron_enabled &&
-      static_cast<size_t>(icons) < toolbar_actions_.size();
-  if (icons == 0 && !display_chevron)
-    return platform_settings_.item_spacing;
-
-  const int icons_size = (icons == 0) ? 0 :
-      (icons * IconWidth(true)) - platform_settings_.item_spacing;
-  const int chevron_size = display_chevron ? delegate_->GetChevronWidth() : 0;
-  const int side_padding = platform_settings_.item_spacing * 2;
-
-  return icons_size + chevron_size + side_padding;
+  return icons * IconWidth(true) + platform_settings_.item_spacing;
 }
 
 size_t ToolbarActionsBar::WidthToIconCount(int pixels) const {
@@ -214,16 +196,10 @@ size_t ToolbarActionsBar::WidthToIconCount(int pixels) const {
   if (pixels >= IconCountToWidth(-1))
     return toolbar_actions_.size();
 
-  // We reserve space for the padding on either side of the toolbar and,
-  // if enabled, for the chevron.
-  int available_space = pixels - (platform_settings_.item_spacing * 2);
-  if (platform_settings_.chevron_enabled)
-    available_space -= delegate_->GetChevronWidth();
-
   // Now we add an extra between-item padding value so the space can be divided
   // evenly by (size of icon with padding).
   return static_cast<size_t>(std::max(
-      0, available_space + platform_settings_.item_spacing) / IconWidth(true));
+      0, pixels - platform_settings_.item_spacing) / IconWidth(true));
 }
 
 size_t ToolbarActionsBar::GetIconCount() const {
@@ -283,10 +259,13 @@ size_t ToolbarActionsBar::GetEndIndexInBounds() const {
 bool ToolbarActionsBar::NeedsOverflow() const {
   DCHECK(!in_overflow_mode());
   // We need an overflow view if either the end index is less than the number of
-  // icons, or if a drag is in progress with the redesign turned on (since the
-  // user can drag an icon into the app menu).
+  // icons, if a drag is in progress with the redesign turned on (since the
+  // user can drag an icon into the app menu), or if there is a non-sticky
+  // popped out action (because the action will pop back into overflow when the
+  // menu opens).
   return GetEndIndexInBounds() != toolbar_actions_.size() ||
-         (is_drag_in_progress_ && !platform_settings_.chevron_enabled);
+         (is_drag_in_progress_ && !platform_settings_.chevron_enabled) ||
+         (popped_out_action_ && !is_popped_out_sticky_);
 }
 
 gfx::Rect ToolbarActionsBar::GetFrameForIndex(
@@ -337,7 +316,7 @@ ToolbarActionsBar::GetActions() const {
 }
 
 void ToolbarActionsBar::CreateActions() {
-  DCHECK(toolbar_actions_.empty());
+  CHECK(toolbar_actions_.empty());
   // If the model isn't initialized, wait for it.
   if (!model_ || !model_->actions_initialized())
     return;
@@ -383,18 +362,13 @@ void ToolbarActionsBar::CreateActions() {
   // haven't already shown the bubble.
   // Extension bubbles can also highlight a subset of actions, so don't show the
   // bubble if the toolbar is already highlighting a different set.
-  if (!checked_extension_bubble_ && !is_highlighting()) {
-    checked_extension_bubble_ = true;
+  if (should_check_extension_bubble_ && !is_highlighting()) {
+    should_check_extension_bubble_ = false;
     // CreateActions() can be called as part of the browser window set up, which
     // we need to let finish before showing the actions.
-    scoped_ptr<extensions::ExtensionMessageBubbleController> controller =
-        ExtensionMessageBubbleFactory(browser_).GetController();
-    if (controller) {
-      base::ThreadTaskRunnerHandle::Get()->PostTask(
-          FROM_HERE, base::Bind(&ToolbarActionsBar::MaybeShowExtensionBubble,
-                                weak_ptr_factory_.GetWeakPtr(),
-                                base::Passed(std::move(controller))));
-    }
+    base::ThreadTaskRunnerHandle::Get()->PostTask(
+        FROM_HERE, base::Bind(&ToolbarActionsBar::MaybeShowExtensionBubble,
+                              weak_ptr_factory_.GetWeakPtr()));
   }
 }
 
@@ -459,8 +433,8 @@ void ToolbarActionsBar::OnDragEnded() {
 
   DCHECK(is_drag_in_progress_);
   is_drag_in_progress_ = false;
-  FOR_EACH_OBSERVER(ToolbarActionsBarObserver,
-                    observers_, OnToolbarActionDragDone());
+  for (ToolbarActionsBarObserver& observer : observers_)
+    observer.OnToolbarActionDragDone();
 }
 
 void ToolbarActionsBar::OnDragDrop(int dragged_index,
@@ -485,16 +459,25 @@ void ToolbarActionsBar::OnDragDrop(int dragged_index,
 }
 
 void ToolbarActionsBar::OnAnimationEnded() {
+  // Notify the observers now, since showing a bubble or popup could potentially
+  // cause another animation to start.
+  for (ToolbarActionsBarObserver& observer : observers_)
+    observer.OnToolbarActionsBarAnimationEnded();
+
   // Check if we were waiting for animation to complete to either show a
   // message bubble, or to show a popup.
-  if (pending_extension_bubble_controller_) {
-    MaybeShowExtensionBubble(std::move(pending_extension_bubble_controller_));
+  if (pending_bubble_controller_) {
+    ShowToolbarActionBubble(std::move(pending_bubble_controller_));
   } else if (!popped_out_closure_.is_null()) {
     popped_out_closure_.Run();
     popped_out_closure_.Reset();
   }
 }
 
+void ToolbarActionsBar::OnBubbleClosed() {
+  is_showing_bubble_ = false;
+}
+
 bool ToolbarActionsBar::IsActionVisibleOnMainBar(
     const ToolbarActionViewController* action) const {
   if (in_overflow_mode())
@@ -507,11 +490,13 @@ bool ToolbarActionsBar::IsActionVisibleOnMainBar(
 }
 
 void ToolbarActionsBar::PopOutAction(ToolbarActionViewController* controller,
+                                     bool is_sticky,
                                      const base::Closure& closure) {
   DCHECK(!in_overflow_mode()) << "Only the main bar can pop out actions.";
   DCHECK(!popped_out_action_) << "Only one action can be popped out at a time!";
   bool needs_redraw = !IsActionVisibleOnMainBar(controller);
   popped_out_action_ = controller;
+  is_popped_out_sticky_ = is_sticky;
   if (needs_redraw) {
     // We suppress animation for this draw, because we need the action to get
     // into position immediately, since it's about to show its popup.
@@ -533,6 +518,7 @@ void ToolbarActionsBar::UndoPopOut() {
   DCHECK(popped_out_action_);
   ToolbarActionViewController* controller = popped_out_action_;
   popped_out_action_ = nullptr;
+  is_popped_out_sticky_ = false;
   popped_out_closure_.Reset();
   if (!IsActionVisibleOnMainBar(controller))
     delegate_->Redraw(true);
@@ -568,32 +554,67 @@ void ToolbarActionsBar::RemoveObserver(ToolbarActionsBarObserver* observer) {
   observers_.RemoveObserver(observer);
 }
 
-void ToolbarActionsBar::MaybeShowExtensionBubble(
-    scoped_ptr<extensions::ExtensionMessageBubbleController> controller) {
-  controller->HighlightExtensionsIfNecessary();  // Safe to call multiple times.
+void ToolbarActionsBar::ShowToolbarActionBubble(
+    std::unique_ptr<ToolbarActionsBarBubbleDelegate> bubble) {
+  DCHECK(!in_overflow_mode());
   if (delegate_->IsAnimating()) {
     // If the toolbar is animating, we can't effectively anchor the bubble,
     // so wait until animation stops.
-    pending_extension_bubble_controller_ = std::move(controller);
-  } else if (controller->ShouldShow()) {
-    // We check ShouldShow() above because the affected extensions may have been
-    // removed since the controller was initialized.
-    const std::vector<std::string>& affected_extensions =
-        controller->GetExtensionIdList();
-    ToolbarActionViewController* anchor_action = nullptr;
-    for (const std::string& id : affected_extensions) {
-      anchor_action = GetActionForId(id);
-      if (anchor_action)
-        break;
-    }
-    delegate_->ShowExtensionMessageBubble(std::move(controller), anchor_action);
+    pending_bubble_controller_ = std::move(bubble);
+  } else if (bubble->ShouldShow()) {
+    // We check ShouldShow() above since we show the bubble asynchronously, and
+    // it might no longer have been valid.
+    is_showing_bubble_ = true;
+    delegate_->ShowToolbarActionBubble(std::move(bubble));
   }
 }
 
+void ToolbarActionsBar::ShowToolbarActionBubbleAsync(
+    std::unique_ptr<ToolbarActionsBarBubbleDelegate> bubble) {
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE, base::Bind(&ToolbarActionsBar::ShowToolbarActionBubble,
+                            weak_ptr_factory_.GetWeakPtr(),
+                            base::Passed(std::move(bubble))));
+}
+
+void ToolbarActionsBar::MaybeShowExtensionBubble() {
+  std::unique_ptr<extensions::ExtensionMessageBubbleController> controller =
+      model_->GetExtensionMessageBubbleController(browser_);
+  if (!controller)
+    return;
+
+  DCHECK(controller->ShouldShow());
+  controller->HighlightExtensionsIfNecessary();  // Safe to call multiple times.
+
+  // Not showing the bubble right away (during startup) has a few benefits:
+  // We don't have to worry about focus being lost due to the Omnibox (or to
+  // other things that want focus at startup). This allows Esc to work to close
+  // the bubble and also solves the keyboard accessibility problem that comes
+  // with focus being lost (we don't have a good generic mechanism of injecting
+  // bubbles into the focus cycle). Another benefit of delaying the show is
+  // that fade-in works (the fade-in isn't apparent if the the bubble appears at
+  // startup).
+  std::unique_ptr<ToolbarActionsBarBubbleDelegate> delegate(
+      new ExtensionMessageBubbleBridge(std::move(controller)));
+  base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+      FROM_HERE, base::Bind(&ToolbarActionsBar::ShowToolbarActionBubble,
+                            weak_ptr_factory_.GetWeakPtr(),
+                            base::Passed(std::move(delegate))),
+      base::TimeDelta::FromSeconds(
+          g_extension_bubble_appearance_wait_time_in_seconds));
+}
+
+// static
+void ToolbarActionsBar::set_extension_bubble_appearance_wait_time_for_testing(
+    int time_in_seconds) {
+  g_extension_bubble_appearance_wait_time_in_seconds = time_in_seconds;
+}
+
 void ToolbarActionsBar::OnToolbarActionAdded(
     const ToolbarActionsModel::ToolbarItem& item,
     int index) {
-  DCHECK(GetActionForId(item.id) == nullptr)
+  CHECK(model_->actions_initialized());
+  CHECK(GetActionForId(item.id) == nullptr)
       << "Asked to add a toolbar action view for an action that already "
          "exists";
 
@@ -601,10 +622,6 @@ void ToolbarActionsBar::OnToolbarActionAdded(
                           model_->CreateActionForItem(browser_, this, item));
   delegate_->AddViewForAction(toolbar_actions_[index], index);
 
-  // If we are still initializing the container, don't bother animating.
-  if (!model_->actions_initialized())
-    return;
-
   // We may need to resize (e.g. to show the new icon, or the chevron). We don't
   // need to check if an extension is upgrading here, because ResizeDelegate()
   // checks to see if the container is already the proper size, and because
@@ -627,9 +644,11 @@ void ToolbarActionsBar::OnToolbarActionRemoved(const std::string& action_id) {
   // The action should outlive the UI element (which is owned by the delegate),
   // so we can't delete it just yet. But we should remove it from the list of
   // actions so that any width calculations are correct.
-  scoped_ptr<ToolbarActionViewController> removed_action(*iter);
+  std::unique_ptr<ToolbarActionViewController> removed_action(*iter);
   toolbar_actions_.weak_erase(iter);
   delegate_->RemoveViewForAction(removed_action.get());
+  if (popped_out_action_ == removed_action.get())
+    UndoPopOut();
   removed_action.reset();
 
   // If the extension is being upgraded we don't want the bar to shrink
@@ -648,7 +667,6 @@ void ToolbarActionsBar::OnToolbarActionRemoved(const std::string& action_id) {
       // removed an entry directly from the overflow list).
       delegate_->Redraw(false);
     } else {
-      delegate_->SetChevronVisibility(false);
       // Either we went from overflow to no-overflow, or we shrunk the no-
       // overflow container by 1.  Either way the size changed, so animate.
       ResizeDelegate(gfx::Tween::EASE_OUT, false);
@@ -682,7 +700,7 @@ void ToolbarActionsBar::ResizeDelegate(gfx::Tween::Type tween_type,
   int desired_width = GetPreferredSize().width();
   if (desired_width !=
       delegate_->GetWidth(ToolbarActionsBarDelegate::GET_WIDTH_CURRENT)) {
-    delegate_->ResizeAndAnimate(tween_type, desired_width, suppress_chevron);
+    delegate_->ResizeAndAnimate(tween_type, desired_width);
   } else if (delegate_->IsAnimating()) {
     // It's possible that we're right where we're supposed to be in terms of
     // width, but that we're also currently resizing. If this is the case, end
@@ -697,8 +715,8 @@ void ToolbarActionsBar::ResizeDelegate(gfx::Tween::Type tween_type,
     delegate_->Redraw(false);
   }
 
-  FOR_EACH_OBSERVER(ToolbarActionsBarObserver,
-                    observers_, OnToolbarActionsBarDidStartResize());
+  for (ToolbarActionsBarObserver& observer : observers_)
+    observer.OnToolbarActionsBarDidStartResize();
 }
 
 void ToolbarActionsBar::OnToolbarHighlightModeChanged(bool is_highlighting) {
@@ -741,7 +759,7 @@ void ToolbarActionsBar::OnToolbarHighlightModeChanged(bool is_highlighting) {
 
 void ToolbarActionsBar::OnToolbarModelInitialized() {
   // We shouldn't have any actions before the model is initialized.
-  DCHECK(toolbar_actions_.empty());
+  CHECK(toolbar_actions_.empty());
   CreateActions();
 
   // TODO(robliao): Remove ScopedTracker below once https://crbug.com/463337 is
@@ -752,6 +770,14 @@ void ToolbarActionsBar::OnToolbarModelInitialized() {
   ResizeDelegate(gfx::Tween::EASE_OUT, false);
 }
 
+void ToolbarActionsBar::TabInsertedAt(TabStripModel* tab_strip_model,
+                                      content::WebContents* contents,
+                                      int index,
+                                      bool foreground) {
+  if (foreground)
+    extensions::MaybeShowExtensionControlledNewTabPage(browser_, contents);
+}
+
 void ToolbarActionsBar::ReorderActions() {
   if (toolbar_actions_.empty())
     return;
