@@ -5,7 +5,6 @@
 #include "chrome/browser/ui/views/accessibility/invert_bubble_view.h"
 
 #include "base/macros.h"
-#include "base/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/views/frame/browser_view.h"
@@ -13,13 +12,14 @@
 #include "chrome/browser/ui/views/toolbar/toolbar_view.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/grit/generated_resources.h"
+#include "components/prefs/pref_service.h"
+#include "components/strings/grit/components_strings.h"
 #include "content/public/browser/page_navigator.h"
-#include "grit/components_strings.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 #include "ui/base/window_open_disposition.h"
 #include "ui/gfx/color_utils.h"
-#include "ui/views/bubble/bubble_delegate.h"
+#include "ui/views/bubble/bubble_dialog_delegate.h"
 #include "ui/views/controls/label.h"
 #include "ui/views/controls/link.h"
 #include "ui/views/controls/link_listener.h"
@@ -36,14 +36,15 @@ const char kDarkThemeSearchUrl[] =
 const char kLearnMoreUrl[] =
     "https://groups.google.com/a/googleproductforums.com/d/topic/chrome/Xrco2HsXS-8/discussion";
 
-class InvertBubbleView : public views::BubbleDelegateView,
+class InvertBubbleView : public views::BubbleDialogDelegateView,
                          public views::LinkListener {
  public:
   InvertBubbleView(Browser* browser, views::View* anchor_view);
   ~InvertBubbleView() override;
 
  private:
-  // Overridden from views::BubbleDelegateView:
+  // Overridden from views::BubbleDialogDelegateView:
+  int GetDialogButtons() const override;
   void Init() override;
 
   // Overridden from views::LinkListener:
@@ -61,24 +62,29 @@ class InvertBubbleView : public views::BubbleDelegateView,
 };
 
 InvertBubbleView::InvertBubbleView(Browser* browser, views::View* anchor_view)
-    : views::BubbleDelegateView(anchor_view, views::BubbleBorder::TOP_RIGHT),
+    : views::BubbleDialogDelegateView(anchor_view,
+                                      views::BubbleBorder::TOP_RIGHT),
       browser_(browser),
       high_contrast_(NULL),
       dark_theme_(NULL),
       learn_more_(NULL),
-      close_(NULL) {
-}
+      close_(NULL) {}
 
 InvertBubbleView::~InvertBubbleView() {
 }
 
+int InvertBubbleView::GetDialogButtons() const {
+  return ui::DIALOG_BUTTON_NONE;
+}
+
 void InvertBubbleView::Init() {
   ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
   const gfx::FontList& original_font_list =
       rb.GetFontList(ui::ResourceBundle::MediumFont);
 
   views::Label* title = new views::Label(
-      base::string16(), original_font_list.Derive(2, gfx::Font::BOLD));
+      base::string16(),
+      original_font_list.Derive(2, gfx::Font::NORMAL, gfx::Font::Weight::BOLD));
   title->SetMultiLine(true);
 
   learn_more_ = new views::Link(l10n_util::GetStringUTF16(IDS_LEARN_MORE));
@@ -148,7 +154,9 @@ void InvertBubbleView::OpenLink(const std::string& url, int event_flags) {
       ui::DispositionFromEventFlags(event_flags);
   content::OpenURLParams params(
       GURL(url), content::Referrer(),
-      disposition == CURRENT_TAB ? NEW_FOREGROUND_TAB : disposition,
+      disposition == WindowOpenDisposition::CURRENT_TAB
+          ? WindowOpenDisposition::NEW_FOREGROUND_TAB
+          : disposition,
       ui::PAGE_TRANSITION_LINK, false);
   browser_->OpenURL(params);
 }
@@ -165,7 +173,7 @@ void MaybeShowInvertBubbleView(BrowserView* browser_view) {
       !pref_service->GetBoolean(prefs::kInvertNotificationShown)) {
     pref_service->SetBoolean(prefs::kInvertNotificationShown, true);
     InvertBubbleView* delegate = new InvertBubbleView(browser, anchor);
-    views::BubbleDelegateView::CreateBubble(delegate)->Show();
+    views::BubbleDialogDelegateView::CreateBubble(delegate)->Show();
   }
 }
 
