@@ -5,10 +5,16 @@
 #ifndef CHROME_BROWSER_UI_VIEWS_ASH_CHROME_BROWSER_MAIN_EXTRA_PARTS_ASH_H_
 #define CHROME_BROWSER_UI_VIEWS_ASH_CHROME_BROWSER_MAIN_EXTRA_PARTS_ASH_H_
 
-#include "base/compiler_specific.h"
+#include <memory>
+
 #include "base/macros.h"
 #include "chrome/browser/chrome_browser_main_extra_parts.h"
 
+class ChromeLauncherControllerMus;
+class ImmersiveContextMus;
+class ImmersiveHandlerFactoryMus;
+class SystemTrayClient;
+
 class ChromeBrowserMainExtraPartsAsh : public ChromeBrowserMainExtraParts {
  public:
   ChromeBrowserMainExtraPartsAsh();
@@ -20,6 +26,11 @@ class ChromeBrowserMainExtraPartsAsh : public ChromeBrowserMainExtraParts {
   void PostMainMessageLoopRun() override;
 
  private:
+  std::unique_ptr<ChromeLauncherControllerMus> chrome_launcher_controller_mus_;
+  std::unique_ptr<ImmersiveHandlerFactoryMus> immersive_handler_factory_;
+  std::unique_ptr<ImmersiveContextMus> immersive_context_;
+  std::unique_ptr<SystemTrayClient> system_tray_client_;
+
   DISALLOW_COPY_AND_ASSIGN(ChromeBrowserMainExtraPartsAsh);
 };
 
