@@ -7,7 +7,7 @@
 #include <string>
 
 #include "base/logging.h"
-#include "base/prefs/pref_service.h"
+#include "base/memory/ptr_util.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "chrome/browser/bookmarks/bookmark_model_factory.h"
@@ -21,13 +21,15 @@
 #include "components/bookmarks/browser/bookmark_utils.h"
 #include "components/constrained_window/constrained_window_views.h"
 #include "components/history/core/browser/history_service.h"
+#include "components/strings/grit/components_strings.h"
 #include "components/url_formatter/url_fixer.h"
 #include "components/user_prefs/user_prefs.h"
-#include "ui/accessibility/ax_view_state.h"
+#include "ui/accessibility/ax_node_data.h"
 #include "ui/base/l10n/l10n_util.h"
+#include "ui/base/material_design/material_design_controller.h"
 #include "ui/events/event.h"
 #include "ui/views/background.h"
-#include "ui/views/controls/button/label_button.h"
+#include "ui/views/controls/button/md_text_button.h"
 #include "ui/views/controls/label.h"
 #include "ui/views/controls/menu/menu_runner.h"
 #include "ui/views/controls/textfield/textfield.h"
@@ -64,7 +66,7 @@ BookmarkEditorView::BookmarkEditorView(
       title_tf_(NULL),
       parent_(parent),
       details_(details),
-      bb_model_(BookmarkModelFactory::GetForProfile(profile)),
+      bb_model_(BookmarkModelFactory::GetForBrowserContext(profile)),
       running_menu_for_root_(false),
       show_tree_(configuration == SHOW_TREE) {
   DCHECK(profile);
@@ -159,10 +161,9 @@ bool BookmarkEditorView::HandleKeyEvent(views::Textfield* sender,
     return false;
 }
 
-void BookmarkEditorView::GetAccessibleState(ui::AXViewState* state) {
-  views::DialogDelegateView::GetAccessibleState(state);
-  state->name =
-      l10n_util::GetStringUTF16(IDS_BOOKMARK_EDITOR_TITLE);
+void BookmarkEditorView::GetAccessibleNodeData(ui::AXNodeData* node_data) {
+  views::DialogDelegateView::GetAccessibleNodeData(node_data);
+  node_data->SetName(l10n_util::GetStringUTF8(IDS_BOOKMARK_EDITOR_TITLE));
 }
 
 void BookmarkEditorView::ButtonPressed(views::Button* sender,
@@ -190,7 +191,7 @@ bool BookmarkEditorView::IsCommandIdEnabled(int command_id) const {
 
 bool BookmarkEditorView::GetAcceleratorForCommandId(
     int command_id,
-    ui::Accelerator* accelerator) {
+    ui::Accelerator* accelerator) const {
   return GetWidget()->GetAccelerator(command_id, accelerator);
 }
 
@@ -244,8 +245,9 @@ void BookmarkEditorView::ShowContextMenuForView(
        tree_model_->GetRoot());
 
   context_menu_runner_.reset(new views::MenuRunner(
-      GetMenuModel(),
-      views::MenuRunner::HAS_MNEMONICS | views::MenuRunner::CONTEXT_MENU));
+      GetMenuModel(), views::MenuRunner::HAS_MNEMONICS |
+                          views::MenuRunner::CONTEXT_MENU |
+                          views::MenuRunner::ASYNC));
 
   if (context_menu_runner_->RunMenuAt(source->GetWidget()->GetTopLevelWidget(),
                                       NULL,
@@ -331,9 +333,9 @@ void BookmarkEditorView::Init() {
     tree_view_->SetRootShown(false);
     tree_view_->set_context_menu_controller(this);
 
-    new_folder_button_.reset(new views::LabelButton(this,
+    new_folder_button_.reset(views::MdTextButton::CreateSecondaryUiButton(
+        this,
         l10n_util::GetStringUTF16(IDS_BOOKMARK_EDITOR_NEW_FOLDER_BUTTON)));
-    new_folder_button_->SetStyle(views::Button::STYLE_BUTTON);
     new_folder_button_->set_owned_by_client();
     new_folder_button_->SetEnabled(false);
   }
@@ -376,9 +378,7 @@ void BookmarkEditorView::Init() {
         l10n_util::GetStringUTF16(IDS_BOOKMARK_EDITOR_URL_LABEL));
 
     url_tf_ = new views::Textfield;
-    PrefService* prefs =
-        profile_ ? user_prefs::UserPrefs::Get(profile_) : NULL;
-    url_tf_->SetText(chrome::FormatBookmarkURLForDisplay(url, prefs));
+    url_tf_->SetText(chrome::FormatBookmarkURLForDisplay(url));
     url_tf_->set_controller(this);
     url_tf_->SetAccessibleName(
         l10n_util::GetStringUTF16(IDS_BOOKMARK_AX_EDITOR_URL_LABEL));
@@ -415,8 +415,7 @@ void BookmarkEditorView::Reset() {
   // tree_view will try to invoke something on the model we just deleted.
   tree_view_->SetModel(NULL);
 
-  EditorNode* root_node = CreateRootNode();
-  tree_model_.reset(new EditorTreeModel(root_node));
+  tree_model_.reset(new EditorTreeModel(CreateRootNode()));
 
   tree_view_->SetModel(tree_model_.get());
   tree_view_->SetController(this);
@@ -437,7 +436,9 @@ GURL BookmarkEditorView::GetInputURL() const {
 void BookmarkEditorView::UserInputChanged() {
   if (details_.GetNodeType() != BookmarkNode::FOLDER) {
     const GURL url(GetInputURL());
-    if (!url.is_valid())
+    if (ui::MaterialDesignController::IsSecondaryUiMaterial())
+      url_tf_->SetInvalid(!url.is_valid());
+    else if (!url.is_valid())
       url_tf_->SetBackgroundColor(kErrorColor);
     else
       url_tf_->UseDefaultBackgroundColor();
@@ -459,11 +460,11 @@ void BookmarkEditorView::NewFolder() {
 
 BookmarkEditorView::EditorNode* BookmarkEditorView::AddNewFolder(
     EditorNode* parent) {
-  EditorNode* new_node = new EditorNode(
-      l10n_util::GetStringUTF16(IDS_BOOKMARK_EDITOR_NEW_FOLDER_NAME), 0);
-  // |new_node| is now owned by |parent|.
-  tree_model_->Add(parent, new_node, parent->child_count());
-  return new_node;
+  return tree_model_->Add(
+      parent,
+      base::MakeUnique<EditorNode>(
+          l10n_util::GetStringUTF16(IDS_BOOKMARK_EDITOR_NEW_FOLDER_NAME), 0),
+      parent->child_count());
 }
 
 void BookmarkEditorView::ExpandAndSelect() {
@@ -489,10 +490,12 @@ void BookmarkEditorView::ExpandAndSelect() {
   tree_view_->SetSelectedNode(b_node);
 }
 
-BookmarkEditorView::EditorNode* BookmarkEditorView::CreateRootNode() {
-  EditorNode* root_node = new EditorNode(base::string16(), 0);
+std::unique_ptr<BookmarkEditorView::EditorNode>
+BookmarkEditorView::CreateRootNode() {
+  std::unique_ptr<EditorNode> root_node =
+      base::MakeUnique<EditorNode>(base::string16(), 0);
   const BookmarkNode* bb_root_node = bb_model_->root_node();
-  CreateNodes(bb_root_node, root_node);
+  CreateNodes(bb_root_node, root_node.get());
   DCHECK(root_node->child_count() >= 2 && root_node->child_count() <= 4);
   DCHECK_EQ(BookmarkNode::BOOKMARK_BAR, bb_root_node->GetChild(0)->type());
   DCHECK_EQ(BookmarkNode::OTHER_NODE, bb_root_node->GetChild(1)->type());
@@ -507,9 +510,10 @@ void BookmarkEditorView::CreateNodes(const BookmarkNode* bb_node,
     const BookmarkNode* child_bb_node = bb_node->GetChild(i);
     if (child_bb_node->IsVisible() && child_bb_node->is_folder() &&
         bb_model_->client()->CanBeEditedByUser(child_bb_node)) {
-      EditorNode* new_b_node = new EditorNode(child_bb_node->GetTitle(),
-                                              child_bb_node->id());
-      b_node->Add(new_b_node, b_node->child_count());
+      EditorNode* new_b_node =
+          b_node->Add(base::MakeUnique<EditorNode>(child_bb_node->GetTitle(),
+                                                   child_bb_node->id()),
+                      b_node->child_count());
       CreateNodes(child_bb_node, new_b_node);
     }
   }
