@@ -7,6 +7,7 @@
 
 #include <stdint.h>
 
+#include <memory>
 #include <vector>
 
 #include "base/compiler_specific.h"
@@ -62,8 +63,8 @@ class BookmarkEditorView : public BookmarkEditor,
   // empty strings. Public purely for testing.
   class EditorTreeModel : public ui::TreeNodeModel<EditorNode> {
    public:
-    explicit EditorTreeModel(EditorNode* root)
-        : ui::TreeNodeModel<EditorNode>(root) {}
+    explicit EditorTreeModel(std::unique_ptr<EditorNode> root)
+        : ui::TreeNodeModel<EditorNode>(std::move(root)) {}
 
     void SetTitle(ui::TreeModelNode* node,
                   const base::string16& title) override;
@@ -90,7 +91,7 @@ class BookmarkEditorView : public BookmarkEditor,
 
   // views::View:
   gfx::Size GetPreferredSize() const override;
-  void GetAccessibleState(ui::AXViewState* state) override;
+  void GetAccessibleNodeData(ui::AXNodeData* node_data) override;
 
   // views::TreeViewController:
   void OnTreeViewSelectionChanged(views::TreeView* tree_view) override;
@@ -109,7 +110,7 @@ class BookmarkEditorView : public BookmarkEditor,
   bool IsCommandIdChecked(int command_id) const override;
   bool IsCommandIdEnabled(int command_id) const override;
   bool GetAcceleratorForCommandId(int command_id,
-                                  ui::Accelerator* accelerator) override;
+                                  ui::Accelerator* accelerator) const override;
   void ExecuteCommand(int command_id, int event_flags) override;
 
   // Creates a Window and adds the BookmarkEditorView to it. When the window is
@@ -169,7 +170,7 @@ class BookmarkEditorView : public BookmarkEditor,
 
   // Creates a returns the new root node. This invokes CreateNodes to do
   // the real work.
-  EditorNode* CreateRootNode();
+  std::unique_ptr<EditorNode> CreateRootNode();
 
   // Adds and creates a child node in b_node for all children of bb_node that
   // are folders.
@@ -209,7 +210,7 @@ class BookmarkEditorView : public BookmarkEditor,
 
   // Creates a new folder as a child of the selected node. If no node is
   // selected, the new folder is added as a child of the bookmark node. Starts
-  // editing on the new gorup as well.
+  // editing on the new group as well.
   void NewFolder();
 
   // Creates a new EditorNode as the last child of parent. The new node is
@@ -229,13 +230,13 @@ class BookmarkEditorView : public BookmarkEditor,
   Profile* profile_;
 
   // Model driving the TreeView.
-  scoped_ptr<EditorTreeModel> tree_model_;
+  std::unique_ptr<EditorTreeModel> tree_model_;
 
   // Displays star folder.
   views::TreeView* tree_view_;
 
   // Used to create a new folder.
-  scoped_ptr<views::LabelButton> new_folder_button_;
+  std::unique_ptr<views::LabelButton> new_folder_button_;
 
   // The label for the url text field.
   views::Label* url_label_;
@@ -256,8 +257,8 @@ class BookmarkEditorView : public BookmarkEditor,
   const EditDetails details_;
 
   // The context menu.
-  scoped_ptr<ui::SimpleMenuModel> context_menu_model_;
-  scoped_ptr<views::MenuRunner> context_menu_runner_;
+  std::unique_ptr<ui::SimpleMenuModel> context_menu_model_;
+  std::unique_ptr<views::MenuRunner> context_menu_runner_;
 
   // Mode used to create nodes from.
   bookmarks::BookmarkModel* bb_model_;
