@@ -14,7 +14,6 @@
 #include "chrome/browser/browsing_data/cookies_tree_model.h"
 #include "chrome/grit/generated_resources.h"
 #include "net/cookies/canonical_cookie.h"
-#include "net/cookies/parsed_cookie.h"
 #include "third_party/skia/include/core/SkColor.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/gfx/canvas.h"
@@ -69,14 +68,6 @@ void CookieInfoView::SetCookie(const std::string& domain,
   Layout();
 }
 
-void CookieInfoView::SetCookieString(const GURL& url,
-                                     const std::string& cookie_line) {
-  net::ParsedCookie pc(cookie_line);
-  net::CanonicalCookie cookie(url, pc);
-  SetCookie(pc.HasDomain() ? pc.Domain() : url.host(), cookie);
-}
-
-
 void CookieInfoView::ClearCookieDisplay() {
   base::string16 no_cookie_string =
       l10n_util::GetStringUTF16(IDS_COOKIES_COOKIE_NONESELECTED);
@@ -120,7 +111,7 @@ void CookieInfoView::AddLabelRow(int layout_id,
 
   // Now that the Textfield is in the view hierarchy, it can be initialized.
   textfield->SetReadOnly(true);
-  textfield->SetBorder(views::Border::NullBorder());
+  textfield->SetBorder(views::NullBorder());
   // Color these borderless text areas the same as the containing dialog.
   textfield->SetBackgroundColor(GetNativeTheme()->GetSystemColor(
       ui::NativeTheme::kColorId_DialogBackground));
