@@ -4,8 +4,7 @@
 
 #include "chrome/browser/ui/views/download/download_feedback_dialog_view.h"
 
-#include "base/metrics/histogram.h"
-#include "base/prefs/pref_service.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/supports_user_data.h"
 #include "chrome/browser/platform_util.h"
 #include "chrome/browser/profiles/profile.h"
@@ -13,8 +12,10 @@
 #include "chrome/grit/chromium_strings.h"
 #include "chrome/grit/generated_resources.h"
 #include "components/constrained_window/constrained_window_views.h"
+#include "components/prefs/pref_service.h"
+#include "components/safe_browsing_db/safe_browsing_prefs.h"
+#include "components/strings/grit/components_strings.h"
 #include "content/public/browser/page_navigator.h"
-#include "grit/components_strings.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/views/controls/link.h"
 #include "ui/views/controls/message_box_view.h"
@@ -45,8 +46,10 @@ void DownloadFeedbackDialogView::Show(
     content::PageNavigator* navigator,
     const UserDecisionCallback& callback) {
   // This dialog should only be shown if it hasn't been shown before.
-  DCHECK(!profile->GetPrefs()->HasPrefPath(
-      prefs::kSafeBrowsingExtendedReportingEnabled));
+  DCHECK(!safe_browsing::ExtendedReportingPrefExists(*profile->GetPrefs()));
+
+  // Determine if any prefs need to be updated prior to showing the dialog.
+  safe_browsing::UpdatePrefsBeforeSecurityInterstitial(profile->GetPrefs());
 
   // Only one dialog should be shown at a time, so check to see if another one
   // is open. If another one is open, treat this parallel call as if reporting
@@ -75,11 +78,14 @@ DownloadFeedbackDialogView::DownloadFeedbackDialogView(
     : profile_(profile),
       navigator_(navigator),
       callback_(callback),
-      explanation_box_view_(new views::MessageBoxView(
-          views::MessageBoxView::InitParams(l10n_util::GetStringUTF16(
-              IDS_FEEDBACK_SERVICE_DIALOG_EXPLANATION)))),
-      link_view_(new views::Link(l10n_util::GetStringUTF16(
-          IDS_SAFE_BROWSING_PRIVACY_POLICY_PAGE))),
+      explanation_box_view_(
+          new views::MessageBoxView(views::MessageBoxView::InitParams(
+              l10n_util::GetStringUTF16(safe_browsing::ChooseOptInTextResource(
+                  *profile->GetPrefs(),
+                  IDS_FEEDBACK_SERVICE_DIALOG_EXPLANATION,
+                  IDS_FEEDBACK_SERVICE_DIALOG_EXPLANATION_SCOUT))))),
+      link_view_(new views::Link(
+          l10n_util::GetStringUTF16(IDS_SAFE_BROWSING_PRIVACY_POLICY_PAGE))),
       title_text_(l10n_util::GetStringUTF16(IDS_FEEDBACK_SERVICE_DIALOG_TITLE)),
       ok_button_text_(l10n_util::GetStringUTF16(
           IDS_FEEDBACK_SERVICE_DIALOG_OK_BUTTON_LABEL)),
@@ -101,8 +107,7 @@ base::string16 DownloadFeedbackDialogView::GetDialogButtonLabel(
 }
 
 bool DownloadFeedbackDialogView::OnButtonClicked(bool accepted) {
-  profile_->GetPrefs()->SetBoolean(prefs::kSafeBrowsingExtendedReportingEnabled,
-                                   accepted);
+  safe_browsing::SetExtendedReportingPref(profile_->GetPrefs(), accepted);
   DialogStatusData* data =
      static_cast<DialogStatusData*>(profile_->GetUserData(kDialogStatusKey));
   DCHECK(data);
@@ -156,8 +161,9 @@ void DownloadFeedbackDialogView::LinkClicked(
       ui::DispositionFromEventFlags(event_flags);
   content::OpenURLParams params(
       GURL(l10n_util::GetStringUTF8(IDS_SAFE_BROWSING_PRIVACY_POLICY_URL)),
-      content::Referrer(),
-      disposition == CURRENT_TAB ? NEW_FOREGROUND_TAB : disposition,
+      content::Referrer(), disposition == WindowOpenDisposition::CURRENT_TAB
+                               ? WindowOpenDisposition::NEW_FOREGROUND_TAB
+                               : disposition,
       ui::PAGE_TRANSITION_LINK, false);
   navigator_->OpenURL(params);
 }
