@@ -5,25 +5,31 @@
 #ifndef CHROME_BROWSER_UI_VIEWS_FIND_BAR_VIEW_H_
 #define CHROME_BROWSER_UI_VIEWS_FIND_BAR_VIEW_H_
 
+#include <memory>
+
 #include "base/compiler_specific.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/strings/string16.h"
-#include "chrome/browser/ui/views/dropdown_bar_view.h"
-#include "ui/views/controls/button/button.h"
-#include "ui/views/controls/textfield/textfield.h"
+#include "chrome/browser/ui/views/dropdown_bar_host_delegate.h"
+#include "ui/views/controls/button/vector_icon_button_delegate.h"
 #include "ui/views/controls/textfield/textfield_controller.h"
+#include "ui/views/view.h"
 #include "ui/views/view_targeter_delegate.h"
 
 class FindBarHost;
 class FindNotificationDetails;
 
+namespace gfx {
+class Range;
+}
+
 namespace views {
-class ImageButton;
 class Label;
 class MouseEvent;
 class Painter;
 class Separator;
+class Textfield;
+class VectorIconButton;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -33,18 +39,12 @@ class Separator;
 // button. It communicates the user search words to the FindBarHost.
 //
 ////////////////////////////////////////////////////////////////////////////////
-class FindBarView : public DropdownBarView,
-                    public views::ButtonListener,
+class FindBarView : public views::View,
+                    public DropdownBarHostDelegate,
+                    public views::VectorIconButtonDelegate,
                     public views::TextfieldController,
                     public views::ViewTargeterDelegate {
  public:
-  // A tag denoting which button the user pressed.
-  enum ButtonTag {
-    FIND_PREVIOUS_TAG = 0,  // The Find Previous button.
-    FIND_NEXT_TAG,          // The Find Next button.
-    CLOSE_TAG,              // The Close button (the 'X').
-  };
-
   explicit FindBarView(FindBarHost* host);
   ~FindBarView() override;
 
@@ -68,17 +68,16 @@ class FindBarView : public DropdownBarView,
   // Clears the current Match Count value in the Find text box.
   void ClearMatchCount();
 
-  // Claims focus for the text field and selects its contents.
-  void SetFocusAndSelection(bool select_all) override;
-
-  // DropdownBarView:
-  void OnPaint(gfx::Canvas* canvas) override;
-  void OnPaintBackground(gfx::Canvas* canvas) override;
+  // views::View:
   void Layout() override;
   gfx::Size GetPreferredSize() const override;
 
-  // views::ButtonListener:
+  // DropdownBarHostDelegate:
+  void SetFocusAndSelection(bool select_all) override;
+
+  // views::VectorIconButtonDelegate:
   void ButtonPressed(views::Button* sender, const ui::Event& event) override;
+  SkColor GetVectorIconBaseColor() const override;
 
   // views::TextfieldController:
   bool HandleKeyEvent(views::Textfield* sender,
@@ -90,11 +89,6 @@ class FindBarView : public DropdownBarView,
   views::View* TargetForRect(View* root, const gfx::Rect& rect) override;
 
  private:
-  // Does mode-specific init. The NonMaterial version should eventually be
-  // removed in favor of Material.
-  void InitViewsForNonMaterial();
-  void InitViewsForMaterial();
-
   // Starts finding |search_text|.  If the text is empty, stops finding.
   void Find(const base::string16& search_text);
 
@@ -103,35 +97,14 @@ class FindBarView : public DropdownBarView,
 
   // DropdownBarView:
   const char* GetClassName() const override;
-  void OnThemeChanged() override;
   void OnNativeThemeChanged(const ui::NativeTheme* theme) override;
 
   // Returns the color for the icons on the buttons per the current NativeTheme.
   SkColor GetTextColorForIcon();
 
-  // We use a hidden view to grab mouse clicks and bring focus to the find
-  // text box. This is because although the find text box may look like it
-  // extends all the way to the find button, it only goes as far as to the
-  // match_count label. The user, however, expects being able to click anywhere
-  // inside what looks like the find text box (including on or around the
-  // match_count label) and have focus brought to the find box.
-  class FocusForwarderView : public views::View {
-   public:
-    explicit FocusForwarderView(
-        views::Textfield* view_to_focus_on_mousedown)
-      : view_to_focus_on_mousedown_(view_to_focus_on_mousedown) {}
-
-   private:
-    bool OnMousePressed(const ui::MouseEvent& event) override;
-
-    views::Textfield* view_to_focus_on_mousedown_;
-
-    DISALLOW_COPY_AND_ASSIGN(FocusForwarderView);
-  };
-
-  // Returns the OS-specific view for the find bar that acts as an intermediary
+  // The OS-specific view for the find bar that acts as an intermediary
   // between us and the WebContentsView.
-  FindBarHost* find_bar_host() const;
+  FindBarHost* find_bar_host_;
 
   // Used to detect if the input text, not including the IME composition text,
   // has changed or not.
@@ -139,13 +112,13 @@ class FindBarView : public DropdownBarView,
 
   // The controls in the window.
   views::Textfield* find_text_;
-  scoped_ptr<views::Painter> find_text_border_;
+  std::unique_ptr<views::Painter> find_text_border_;
   views::Label* match_count_text_;
   views::View* focus_forwarder_view_;
   views::Separator* separator_;
-  views::ImageButton* find_previous_button_;
-  views::ImageButton* find_next_button_;
-  views::ImageButton* close_button_;
+  views::VectorIconButton* find_previous_button_;
+  views::VectorIconButton* find_next_button_;
+  views::VectorIconButton* close_button_;
 
   // The preferred height of the find bar.
   int preferred_height_;
