@@ -6,6 +6,7 @@
 
 #include <utility>
 
+#include "base/command_line.h"
 #include "base/debug/leak_annotations.h"
 #include "base/i18n/rtl.h"
 #include "build/build_config.h"
@@ -23,26 +24,25 @@
 #include "chrome/browser/ui/views/frame/native_browser_frame_factory.h"
 #include "chrome/browser/ui/views/frame/system_menu_model_builder.h"
 #include "chrome/browser/ui/views/frame/top_container_view.h"
+#include "chrome/common/chrome_switches.h"
 #include "ui/base/hit_test.h"
-#include "ui/base/theme_provider.h"
 #include "ui/events/event_handler.h"
 #include "ui/gfx/font_list.h"
-#include "ui/gfx/screen.h"
+#include "ui/native_theme/native_theme_dark_aura.h"
 #include "ui/views/controls/menu/menu_runner.h"
 #include "ui/views/widget/native_widget.h"
 
 #if defined(OS_CHROMEOS)
-#include "ash/session/session_state_delegate.h"
-#include "ash/shell.h"
-#include "ui/native_theme/native_theme_dark_aura.h"
+#include "ash/common/session/session_state_delegate.h"  // nogncheck
+#include "ash/common/wm_shell.h"  // nogncheck
 #endif
 
 #if defined(OS_LINUX)
 #include "chrome/browser/ui/views/frame/browser_command_handler_linux.h"
 #endif
 
-#if defined(OS_WIN)
-#include "ui/native_theme/native_theme_dark_win.h"
+#if defined(OS_LINUX) && !defined(OS_CHROMEOS)
+#include "ui/views/widget/desktop_aura/x11_desktop_handler.h"
 #endif
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -52,10 +52,7 @@ BrowserFrame::BrowserFrame(BrowserView* browser_view)
     : native_browser_frame_(nullptr),
       root_view_(nullptr),
       browser_frame_view_(nullptr),
-      browser_view_(browser_view),
-      theme_provider_(
-          &ThemeService::GetThemeProviderForProfile(browser_view_->browser()
-                                                        ->profile())) {
+      browser_view_(browser_view) {
   browser_view_->set_frame(this);
   set_is_secondary_widget(false);
   // Don't focus anything on creation, selecting a tab will set the focus.
@@ -83,14 +80,15 @@ void BrowserFrame::InitBrowserFrame() {
     chrome::GetSavedWindowBoundsAndShowState(browser_view_->browser(),
                                              &params.bounds,
                                              &params.show_state);
-  }
 
-  if (browser_view_->browser()->profile()->IsOffTheRecord()) {
-#if defined(OS_WIN)
-    params.native_theme = ui::NativeThemeDarkWin::instance();
-#elif defined(OS_CHROMEOS)
-    params.native_theme = ui::NativeThemeDarkAura::instance();
-#endif
+    params.workspace = browser_view_->browser()->initial_workspace();
+    const base::CommandLine& parsed_command_line =
+        *base::CommandLine::ForCurrentProcess();
+
+    if (parsed_command_line.HasSwitch(switches::kWindowWorkspace)) {
+      params.workspace =
+          parsed_command_line.GetSwitchValueASCII(switches::kWindowWorkspace);
+    }
   }
 
   Init(params);
@@ -105,11 +103,6 @@ void BrowserFrame::InitBrowserFrame() {
 #endif
 }
 
-void BrowserFrame::SetThemeProvider(scoped_ptr<ui::ThemeProvider> provider) {
-  owned_theme_provider_ = std::move(provider);
-  theme_provider_ = owned_theme_provider_.get();
-}
-
 int BrowserFrame::GetMinimizeButtonOffset() const {
   return native_browser_frame_->GetMinimizeButtonOffset();
 }
@@ -140,7 +133,7 @@ views::View* BrowserFrame::GetLocationIconView() const {
   return browser_frame_view_->GetLocationIconView();
 }
 
-views::View* BrowserFrame::GetFrameView() const {
+BrowserNonClientFrameView* BrowserFrame::GetFrameView() const {
   return browser_frame_view_;
 }
 
@@ -181,7 +174,20 @@ bool BrowserFrame::GetAccelerator(int command_id,
 }
 
 const ui::ThemeProvider* BrowserFrame::GetThemeProvider() const {
-  return theme_provider_;
+  return &ThemeService::GetThemeProviderForProfile(
+      browser_view_->browser()->profile());
+}
+
+const ui::NativeTheme* BrowserFrame::GetNativeTheme() const {
+#if defined(OS_WIN) || defined(OS_CHROMEOS)
+  if (browser_view_->browser()->profile()->GetProfileType() ==
+          Profile::INCOGNITO_PROFILE &&
+      ThemeServiceFactory::GetForProfile(browser_view_->browser()->profile())
+          ->UsingDefaultTheme()) {
+    return ui::NativeThemeDarkAura::instance();
+  }
+#endif
+  return views::Widget::GetNativeTheme();
 }
 
 void BrowserFrame::SchedulePaintInRect(const gfx::Rect& rect) {
@@ -207,10 +213,21 @@ void BrowserFrame::OnNativeWidgetActivationChanged(bool active) {
     // ourselves as the last active browser window to ensure that we get treated
     // as such by the rest of Chrome.
     BrowserList::SetLastActive(browser_view_->browser());
+  } else {
+    BrowserList::NotifyBrowserNoLongerActive(browser_view_->browser());
   }
   Widget::OnNativeWidgetActivationChanged(active);
 }
 
+void BrowserFrame::OnNativeWidgetWorkspaceChanged() {
+  chrome::SaveWindowWorkspace(browser_view_->browser(), GetWorkspace());
+#if !defined(OS_CHROMEOS) && defined(USE_X11)
+  BrowserList::MoveBrowsersInWorkspaceToFront(
+      views::X11DesktopHandler::get()->GetWorkspace());
+#endif
+  Widget::OnNativeWidgetWorkspaceChanged();
+}
+
 void BrowserFrame::ShowContextMenuForView(views::View* source,
                                           const gfx::Point& p,
                                           ui::MenuSourceType source_type) {
@@ -227,25 +244,22 @@ void BrowserFrame::ShowContextMenuForView(views::View* source,
   if (hit_test == HTCAPTION || hit_test == HTNOWHERE) {
     menu_runner_.reset(new views::MenuRunner(
         GetSystemMenuModel(),
-        views::MenuRunner::HAS_MNEMONICS | views::MenuRunner::CONTEXT_MENU));
-    if (menu_runner_->RunMenuAt(source->GetWidget(),
-                                nullptr,
-                                gfx::Rect(p, gfx::Size(0, 0)),
-                                views::MENU_ANCHOR_TOPLEFT,
-                                source_type) ==
-        views::MenuRunner::MENU_DELETED) {
-      return;
-    }
+        views::MenuRunner::HAS_MNEMONICS | views::MenuRunner::CONTEXT_MENU |
+            views::MenuRunner::ASYNC,
+        base::Bind(&BrowserFrame::OnMenuClosed, base::Unretained(this))));
+    menu_runner_->RunMenuAt(source->GetWidget(), nullptr,
+                            gfx::Rect(p, gfx::Size(0, 0)),
+                            views::MENU_ANCHOR_TOPLEFT, source_type);
   }
 }
 
 ui::MenuModel* BrowserFrame::GetSystemMenuModel() {
 #if defined(OS_CHROMEOS)
   ash::SessionStateDelegate* delegate =
-      ash::Shell::GetInstance()->session_state_delegate();
+      ash::WmShell::Get()->GetSessionStateDelegate();
   if (delegate && delegate->NumberOfLoggedInUsers() > 1) {
     // In Multi user mode, the number of users as well as the order of users
-    // can change. Coming here we have more then one user and since the menu
+    // can change. Coming here we have more than one user and since the menu
     // model contains the user information, it must get updated to show any
     // changes happened since the last invocation.
     menu_model_builder_.reset();
@@ -259,12 +273,10 @@ ui::MenuModel* BrowserFrame::GetSystemMenuModel() {
   return menu_model_builder_->menu_model();
 }
 
-AvatarMenuButton* BrowserFrame::GetAvatarMenuButton() {
-  return browser_frame_view_->avatar_button();
+views::View* BrowserFrame::GetNewAvatarMenuButton() {
+  return browser_frame_view_->GetProfileSwitcherView();
 }
 
-#if defined(FRAME_AVATAR_BUTTON)
-NewAvatarButton* BrowserFrame::GetNewAvatarMenuButton() {
-  return browser_frame_view_->new_avatar_button();
+void BrowserFrame::OnMenuClosed() {
+  menu_runner_.reset();
 }
-#endif
