@@ -5,19 +5,18 @@
 #ifndef CHROME_BROWSER_UI_VIEWS_FRAME_BROWSER_NON_CLIENT_FRAME_VIEW_ASH_H_
 #define CHROME_BROWSER_UI_VIEWS_FRAME_BROWSER_NON_CLIENT_FRAME_VIEW_ASH_H_
 
-#include "ash/shell_observer.h"
+#include <memory>
+
+#include "ash/common/shell_observer.h"
 #include "base/gtest_prod_util.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "chrome/browser/ui/views/frame/browser_non_client_frame_view.h"
 #include "chrome/browser/ui/views/tab_icon_view_model.h"
-#include "ui/views/controls/button/button.h"
 
 class TabIconView;
 class WebAppLeftHeaderView;
 
 namespace ash {
-class FrameBorderHitTestController;
 class FrameCaptionButton;
 class FrameCaptionButtonContainerView;
 class HeaderPainter;
@@ -30,11 +29,8 @@ class ToggleImageButton;
 
 class BrowserNonClientFrameViewAsh : public BrowserNonClientFrameView,
                                      public ash::ShellObserver,
-                                     public TabIconViewModel,
-                                     public views::ButtonListener {
+                                     public TabIconViewModel {
  public:
-  static const char kViewClassName[];
-
   BrowserNonClientFrameViewAsh(BrowserFrame* frame, BrowserView* browser_view);
   ~BrowserNonClientFrameViewAsh() override;
 
@@ -63,11 +59,13 @@ class BrowserNonClientFrameViewAsh : public BrowserNonClientFrameView,
   void OnPaint(gfx::Canvas* canvas) override;
   void Layout() override;
   const char* GetClassName() const override;
-  void GetAccessibleState(ui::AXViewState* state) override;
+  void GetAccessibleNodeData(ui::AXNodeData* node_data) override;
   gfx::Size GetMinimumSize() const override;
   void ChildPreferredSizeChanged(views::View* child) override;
 
   // ash::ShellObserver:
+  void OnOverviewModeStarting() override;
+  void OnOverviewModeEnded() override;
   void OnMaximizeModeStarted() override;
   void OnMaximizeModeEnded() override;
 
@@ -75,12 +73,9 @@ class BrowserNonClientFrameViewAsh : public BrowserNonClientFrameView,
   bool ShouldTabIconViewAnimate() const override;
   gfx::ImageSkia GetFaviconForTabIconView() override;
 
-  // views::ButtonListener:
-  void ButtonPressed(views::Button* sender, const ui::Event& event) override;
-
  protected:
   // BrowserNonClientFrameView:
-  void UpdateNewAvatarButtonImpl() override;
+  void UpdateProfileIcons() override;
 
  private:
   FRIEND_TEST_ALL_PREFIXES(BrowserNonClientFrameViewAshTest, WindowHeader);
@@ -90,12 +85,11 @@ class BrowserNonClientFrameViewAsh : public BrowserNonClientFrameView,
                            ImmersiveFullscreen);
   FRIEND_TEST_ALL_PREFIXES(BrowserNonClientFrameViewAshTest,
                            ToggleMaximizeModeRelayout);
+  FRIEND_TEST_ALL_PREFIXES(BrowserNonClientFrameViewAshTest,
+                           AvatarDisplayOnTeleportedWindow);
   FRIEND_TEST_ALL_PREFIXES(WebAppLeftHeaderViewTest, BackButton);
   FRIEND_TEST_ALL_PREFIXES(WebAppLeftHeaderViewTest, LocationIcon);
-
-  // views::NonClientFrameView:
-  bool DoesIntersectRect(const views::View* target,
-                         const gfx::Rect& rect) const override;
+  friend class BrowserHeaderPainterAsh;
 
   // Distance between the left edge of the NonClientFrameView and the tab strip.
   int GetTabStripLeftInset() const;
@@ -119,26 +113,14 @@ class BrowserNonClientFrameViewAsh : public BrowserNonClientFrameView,
   // accoutrements.
   bool UseWebAppHeaderStyle() const;
 
-  // Layout the avatar button.
-  void LayoutAvatar();
-#if defined(FRAME_AVATAR_BUTTON)
-  void LayoutNewStyleAvatar();
-#endif
+  void LayoutProfileIndicatorIcon();
 
   // Returns true if there is anything to paint. Some fullscreen windows do not
   // need their frames painted.
   bool ShouldPaint() const;
 
-  // Paints the header background when the frame is in immersive fullscreen and
-  // tab light bar is visible.
-  void PaintImmersiveLightbarStyleHeader(gfx::Canvas* canvas);
-
   void PaintToolbarBackground(gfx::Canvas* canvas);
 
-  // Draws the line under the header for windows without a toolbar and not using
-  // the packaged app header style.
-  void PaintContentEdge(gfx::Canvas* canvas);
-
   // View which contains the window controls.
   ash::FrameCaptionButtonContainerView* caption_button_container_;
 
@@ -150,11 +132,7 @@ class BrowserNonClientFrameViewAsh : public BrowserNonClientFrameView,
   TabIconView* window_icon_;
 
   // Helper class for painting the header.
-  scoped_ptr<ash::HeaderPainter> header_painter_;
-
-  // Updates the hittest bounds overrides based on the window show type.
-  scoped_ptr<ash::FrameBorderHitTestController>
-      frame_border_hit_test_controller_;
+  std::unique_ptr<ash::HeaderPainter> header_painter_;
 
   DISALLOW_COPY_AND_ASSIGN(BrowserNonClientFrameViewAsh);
 };
