@@ -25,7 +25,7 @@
 #include "chrome/browser/ui/views/tabs/tab_strip.h"
 #include "components/web_modal/web_contents_modal_dialog_host.h"
 #include "ui/base/hit_test.h"
-#include "ui/base/resource/material_design/material_design_controller.h"
+#include "ui/base/material_design/material_design_controller.h"
 #include "ui/gfx/geometry/point.h"
 #include "ui/gfx/geometry/size.h"
 #include "ui/gfx/scrollbar_size.h"
@@ -68,15 +68,13 @@ class BrowserViewLayout::WebContentsModalDialogHostViews
   }
 
   ~WebContentsModalDialogHostViews() override {
-    FOR_EACH_OBSERVER(ModalDialogHostObserver,
-                      observer_list_,
-                      OnHostDestroying());
+    for (ModalDialogHostObserver& observer : observer_list_)
+      observer.OnHostDestroying();
   }
 
   void NotifyPositionRequiresUpdate() {
-    FOR_EACH_OBSERVER(ModalDialogHostObserver,
-                      observer_list_,
-                      OnPositionRequiresUpdate());
+    for (ModalDialogHostObserver& observer : observer_list_)
+      observer.OnPositionRequiresUpdate();
   }
 
   gfx::Point GetDialogPosition(const gfx::Size& size) override {
@@ -173,8 +171,6 @@ gfx::Size BrowserViewLayout::GetMinimumSize() {
       (browser()->SupportsWindowFeature(Browser::FEATURE_TOOLBAR) ||
        browser()->SupportsWindowFeature(Browser::FEATURE_LOCATIONBAR)) ?
            toolbar_->GetMinimumSize() : gfx::Size());
-  if (tabstrip_size.height() && toolbar_size.height())
-    toolbar_size.Enlarge(0, -GetLayoutConstant(TABSTRIP_TOOLBAR_OVERLAP));
   gfx::Size bookmark_bar_size;
   if (bookmark_bar_ &&
       bookmark_bar_->visible() &&
@@ -404,14 +400,11 @@ int BrowserViewLayout::LayoutTabStripRegion(int top) {
 int BrowserViewLayout::LayoutToolbar(int top) {
   int browser_view_width = vertical_layout_rect_.width();
   bool toolbar_visible = delegate_->IsToolbarVisible();
-  int y = top;
-  y -= (toolbar_visible && delegate_->IsTabStripVisible()) ?
-      GetLayoutConstant(TABSTRIP_TOOLBAR_OVERLAP) : 0;
   int height = toolbar_visible ? toolbar_->GetPreferredSize().height() : 0;
   toolbar_->SetVisible(toolbar_visible);
-  toolbar_->SetBounds(vertical_layout_rect_.x(), y, browser_view_width, height);
-
-  return y + height;
+  toolbar_->SetBounds(vertical_layout_rect_.x(), top, browser_view_width,
+                      height);
+  return toolbar_->bounds().bottom();
 }
 
 int BrowserViewLayout::LayoutBookmarkAndInfoBars(int top, int browser_view_y) {
@@ -465,15 +458,11 @@ int BrowserViewLayout::LayoutInfoBar(int top) {
     if (!immersive_mode_controller_->ShouldHideTabIndicators())
       top += TabStrip::GetImmersiveHeight();
   }
-  // Raise the |infobar_container_| by its vertical overlap.
   infobar_container_->SetVisible(InfobarVisible());
-  int height;
-  int overlapped_top = top - infobar_container_->GetVerticalOverlap(&height);
-  infobar_container_->SetBounds(vertical_layout_rect_.x(),
-                                overlapped_top,
-                                vertical_layout_rect_.width(),
-                                height);
-  return overlapped_top + height;
+  infobar_container_->SetBounds(
+      vertical_layout_rect_.x(), top, vertical_layout_rect_.width(),
+      infobar_container_->GetPreferredSize().height());
+  return top + infobar_container_->height();
 }
 
 void BrowserViewLayout::LayoutContentsContainerView(int top, int bottom) {
