@@ -4,11 +4,14 @@
 
 #include "chrome/browser/ui/views/frame/immersive_mode_controller_ash.h"
 
+#include "ash/common/material_design/material_design_controller.h"
+#include "ash/common/wm/window_state.h"
+#include "ash/shared/immersive_revealed_lock.h"
 #include "ash/shell.h"
-#include "ash/wm/immersive_revealed_lock.h"
-#include "ash/wm/window_state.h"
+#include "ash/wm/window_state_aura.h"
 #include "base/macros.h"
 #include "chrome/browser/chrome_notification_types.h"
+#include "chrome/browser/ui/ash/ash_util.h"
 #include "chrome/browser/ui/exclusive_access/exclusive_access_manager.h"
 #include "chrome/browser/ui/exclusive_access/fullscreen_controller.h"
 #include "chrome/browser/ui/views/frame/browser_view.h"
@@ -16,7 +19,15 @@
 #include "chrome/browser/ui/views/tabs/tab_strip.h"
 #include "content/public/browser/notification_service.h"
 #include "content/public/browser/web_contents.h"
+#include "services/ui/public/cpp/property_type_converters.h"
+#include "services/ui/public/cpp/window_tree_client.h"
+#include "services/ui/public/interfaces/window_manager.mojom.h"
+#include "ui/aura/mus/mus_util.h"
 #include "ui/aura/window.h"
+#include "ui/compositor/paint_context.h"
+#include "ui/compositor/paint_recorder.h"
+#include "ui/views/mus/native_widget_mus.h"
+#include "ui/views/mus/window_manager_connection.h"
 #include "ui/views/view.h"
 #include "ui/views/widget/widget.h"
 #include "ui/views/window/non_client_view.h"
@@ -49,21 +60,41 @@ class ImmersiveRevealedLockAsh : public ImmersiveRevealedLock {
       : lock_(lock) {}
 
  private:
-  scoped_ptr<ash::ImmersiveRevealedLock> lock_;
+  std::unique_ptr<ash::ImmersiveRevealedLock> lock_;
 
   DISALLOW_COPY_AND_ASSIGN(ImmersiveRevealedLockAsh);
 };
 
+// TODO(sky): remove this, should instead create a layer that is clone of
+// existing layers. http://crbug.com/640378.
+class DelegatingPaintView : public views::View {
+ public:
+  explicit DelegatingPaintView(views::View* view) : view_(view) {}
+
+  ~DelegatingPaintView() override {}
+
+  // views::View:
+  void PaintChildren(const ui::PaintContext& context) override {
+    view_->Paint(ui::PaintContext(
+        context, ui::PaintContext::CLONE_WITHOUT_INVALIDATION));
+  }
+
+ private:
+  views::View* view_;
+
+  DISALLOW_COPY_AND_ASSIGN(DelegatingPaintView);
+};
+
 }  // namespace
 
 ImmersiveModeControllerAsh::ImmersiveModeControllerAsh()
-    : controller_(new ash::ImmersiveFullscreenController),
+    : ImmersiveModeController(Type::ASH),
+      controller_(new ash::ImmersiveFullscreenController),
       browser_view_(nullptr),
       native_window_(nullptr),
       observers_enabled_(false),
       use_tab_indicators_(false),
-      visible_fraction_(1) {
-}
+      visible_fraction_(1) {}
 
 ImmersiveModeControllerAsh::~ImmersiveModeControllerAsh() {
   EnableWindowObservers(false);
@@ -131,10 +162,6 @@ void ImmersiveModeControllerAsh::OnFindBarVisibleBoundsChanged(
   find_bar_visible_bounds_in_screen_ = new_visible_bounds_in_screen;
 }
 
-void ImmersiveModeControllerAsh::SetupForTest() {
-  controller_->SetupForTest();
-}
-
 void ImmersiveModeControllerAsh::EnableWindowObservers(bool enable) {
   if (observers_enabled_ == enable)
     return;
@@ -144,10 +171,20 @@ void ImmersiveModeControllerAsh::EnableWindowObservers(bool enable) {
                                                    ->exclusive_access_manager()
                                                    ->fullscreen_controller());
   if (enable) {
-    ash::wm::GetWindowState(native_window_)->AddObserver(this);
+    if (chrome::IsRunningInMash()) {
+      // TODO: http://crbug.com/640381.
+      NOTIMPLEMENTED();
+    } else {
+      ash::wm::GetWindowState(native_window_)->AddObserver(this);
+    }
     registrar_.Add(this, chrome::NOTIFICATION_FULLSCREEN_CHANGED, source);
   } else {
-    ash::wm::GetWindowState(native_window_)->RemoveObserver(this);
+    if (chrome::IsRunningInMash()) {
+      // TODO: http://crbug.com/640381.
+      NOTIMPLEMENTED();
+    } else {
+      ash::wm::GetWindowState(native_window_)->RemoveObserver(this);
+    }
     registrar_.Remove(this, chrome::NOTIFICATION_FULLSCREEN_CHANGED, source);
   }
 }
@@ -161,7 +198,13 @@ void ImmersiveModeControllerAsh::LayoutBrowserRootView() {
   widget->GetRootView()->Layout();
 }
 
+// TODO(yiyix|tdanderson): Once Chrome OS material design is enabled by default,
+// remove all code related to immersive mode hints (here, in TabStrip and
+// BrowserNonClientFrameViewAsh::OnPaint()). See crbug.com/614453.
 bool ImmersiveModeControllerAsh::UpdateTabIndicators() {
+  if (ash::MaterialDesignController::IsImmersiveModeMaterial())
+    return false;
+
   bool has_tabstrip = browser_view_->IsBrowserTypeNormal();
   if (!IsEnabled() || !has_tabstrip) {
     use_tab_indicators_ = false;
@@ -181,16 +224,60 @@ bool ImmersiveModeControllerAsh::UpdateTabIndicators() {
   return false;
 }
 
+void ImmersiveModeControllerAsh::CreateMashRevealWidget() {
+  if (!chrome::IsRunningInMash())
+    return;
+
+  DCHECK(!mash_reveal_widget_);
+  mash_reveal_widget_.reset(new views::Widget);
+  views::Widget::InitParams init_params(views::Widget::InitParams::TYPE_POPUP);
+  std::map<std::string, std::vector<uint8_t>> window_properties;
+  window_properties
+      [ui::mojom::WindowManager::kRendererParentTitleArea_Property] =
+          mojo::ConvertTo<std::vector<uint8_t>>(true);
+  window_properties[ui::mojom::WindowManager::kName_Property] =
+      mojo::ConvertTo<std::vector<uint8_t>>(
+          std::string("ChromeImmersiveRevealWindow"));
+  init_params.native_widget = new views::NativeWidgetMus(
+      mash_reveal_widget_.get(),
+      views::WindowManagerConnection::Get()->client()->NewWindow(
+          &window_properties),
+      ui::mojom::CompositorFrameSinkType::DEFAULT);
+  init_params.accept_events = false;
+  init_params.ownership = views::Widget::InitParams::WIDGET_OWNS_NATIVE_WIDGET;
+  init_params.activatable = views::Widget::InitParams::ACTIVATABLE_NO;
+  init_params.parent_mus = GetMusWindow(native_window_);
+  const gfx::Rect& top_container_bounds =
+      browser_view_->top_container()->bounds();
+  init_params.bounds =
+      gfx::Rect(0, -top_container_bounds.height(), top_container_bounds.width(),
+                top_container_bounds.height());
+  mash_reveal_widget_->Init(init_params);
+  mash_reveal_widget_->SetContentsView(
+      new DelegatingPaintView(browser_view_->top_container()));
+  mash_reveal_widget_->StackAtTop();
+  mash_reveal_widget_->Show();
+}
+
+void ImmersiveModeControllerAsh::DestroyMashRevealWidget() {
+  mash_reveal_widget_.reset();
+}
+
 void ImmersiveModeControllerAsh::OnImmersiveRevealStarted() {
+  DestroyMashRevealWidget();
+
   visible_fraction_ = 0;
   browser_view_->top_container()->SetPaintToLayer(true);
-  browser_view_->top_container()->SetFillsBoundsOpaquely(false);
+  browser_view_->top_container()->layer()->SetFillsBoundsOpaquely(false);
   UpdateTabIndicators();
   LayoutBrowserRootView();
-  FOR_EACH_OBSERVER(Observer, observers_, OnImmersiveRevealStarted());
+  CreateMashRevealWidget();
+  for (Observer& observer : observers_)
+    observer.OnImmersiveRevealStarted();
 }
 
 void ImmersiveModeControllerAsh::OnImmersiveRevealEnded() {
+  DestroyMashRevealWidget();
   visible_fraction_ = 0;
   browser_view_->top_container()->SetPaintToLayer(false);
   UpdateTabIndicators();
@@ -198,6 +285,7 @@ void ImmersiveModeControllerAsh::OnImmersiveRevealEnded() {
 }
 
 void ImmersiveModeControllerAsh::OnImmersiveFullscreenExited() {
+  DestroyMashRevealWidget();
   browser_view_->top_container()->SetPaintToLayer(false);
   UpdateTabIndicators();
   LayoutBrowserRootView();
@@ -207,6 +295,12 @@ void ImmersiveModeControllerAsh::SetVisibleFraction(double visible_fraction) {
   if (visible_fraction_ != visible_fraction) {
     visible_fraction_ = visible_fraction;
     browser_view_->Layout();
+
+    if (mash_reveal_widget_) {
+      gfx::Rect bounds = mash_reveal_widget_->GetNativeWindow()->bounds();
+      bounds.set_y(visible_fraction * bounds.height() - bounds.height());
+      mash_reveal_widget_->SetBounds(bounds);
+    }
   }
 }
 
@@ -250,16 +344,22 @@ void ImmersiveModeControllerAsh::Observe(
   if (!controller_->IsEnabled())
     return;
 
+  if (chrome::IsRunningInMash()) {
+    // TODO: http://crbug.com/640384.
+    NOTIMPLEMENTED();
+    return;
+  }
+
   bool tab_indicator_visibility_changed = UpdateTabIndicators();
 
-  // Auto hide the shelf in immersive browser fullscreen. When auto hidden, the
-  // shelf displays a 3px 'light bar' when it is closed. When in immersive
-  // browser fullscreen and tab fullscreen, hide the shelf completely and
-  // prevent it from being revealed.
+  // Auto hide the shelf in immersive browser fullscreen. When auto hidden and
+  // Material Design is not enabled, the shelf displays a 3px 'light bar' when
+  // it is closed. When in immersive browser fullscreen and tab fullscreen, hide
+  // the shelf completely and prevent it from being revealed.
   bool in_tab_fullscreen = content::Source<FullscreenController>(source)->
       IsWindowFullscreenForTabOrPending();
-  ash::wm::GetWindowState(native_window_)->set_hide_shelf_when_fullscreen(
-      in_tab_fullscreen);
+  ash::wm::GetWindowState(native_window_)
+      ->set_hide_shelf_when_fullscreen(in_tab_fullscreen);
   ash::Shell::GetInstance()->UpdateShelfVisibility();
 
   if (tab_indicator_visibility_changed)
