@@ -6,6 +6,7 @@
 
 #include <algorithm>
 
+#include "base/lazy_instance.h"
 #include "base/macros.h"
 #include "build/build_config.h"
 #include "chrome/browser/search/search.h"
@@ -13,28 +14,38 @@
 #include "chrome/browser/ui/layout_constants.h"
 #include "chrome/browser/ui/views/location_bar/location_bar_view.h"
 #include "chrome/browser/ui/views/omnibox/omnibox_result_view.h"
+#include "chrome/browser/ui/views/theme_copying_widget.h"
 #include "components/omnibox/browser/omnibox_view.h"
-#include "grit/theme_resources.h"
-#include "ui/base/resource/material_design/material_design_controller.h"
-#include "ui/base/resource/resource_bundle.h"
+#include "third_party/skia/include/core/SkDrawLooper.h"
 #include "ui/base/theme_provider.h"
 #include "ui/compositor/clip_recorder.h"
 #include "ui/compositor/paint_recorder.h"
 #include "ui/gfx/canvas.h"
+#include "ui/gfx/geometry/safe_integer_conversions.h"
 #include "ui/gfx/image/image.h"
+#include "ui/gfx/image/image_skia_operations.h"
 #include "ui/gfx/path.h"
-#include "ui/resources/grit/ui_resources.h"
+#include "ui/gfx/shadow_value.h"
 #include "ui/views/controls/image_view.h"
-#include "ui/views/resources/grit/views_resources.h"
 #include "ui/views/view_targeter.h"
 #include "ui/views/widget/widget.h"
 #include "ui/views/window/non_client_view.h"
 
+namespace {
+
+// Cache the shadow images so that potentially expensive shadow drawing isn't
+// repeated.
+base::LazyInstance<gfx::ImageSkia> g_top_shadow = LAZY_INSTANCE_INITIALIZER;
+base::LazyInstance<gfx::ImageSkia> g_bottom_shadow = LAZY_INSTANCE_INITIALIZER;
+
+}  // namespace
+
 class OmniboxPopupContentsView::AutocompletePopupWidget
-    : public views::Widget,
+    : public ThemeCopyingWidget,
       public base::SupportsWeakPtr<AutocompletePopupWidget> {
  public:
-  AutocompletePopupWidget() {}
+  explicit AutocompletePopupWidget(views::Widget* role_model)
+      : ThemeCopyingWidget(role_model) {}
   ~AutocompletePopupWidget() override {}
 
  private:
@@ -49,7 +60,7 @@ OmniboxPopupView* OmniboxPopupContentsView::Create(
     OmniboxView* omnibox_view,
     OmniboxEditModel* edit_model,
     LocationBarView* location_bar_view) {
-  OmniboxPopupContentsView* view = NULL;
+  OmniboxPopupContentsView* view = nullptr;
   view = new OmniboxPopupContentsView(
       font_list, omnibox_view, edit_model, location_bar_view);
   view->Init();
@@ -72,16 +83,33 @@ OmniboxPopupContentsView::OmniboxPopupContentsView(
   // The contents is owned by the LocationBarView.
   set_owned_by_client();
 
-  ui::ResourceBundle* rb = &ui::ResourceBundle::GetSharedInstance();
-  if (ui::MaterialDesignController::IsModeMaterial()) {
-    top_shadow_ = rb->GetImageSkiaNamed(IDR_OMNIBOX_DROPDOWN_SHADOW_TOP);
-    bottom_shadow_ = rb->GetImageSkiaNamed(IDR_OMNIBOX_DROPDOWN_SHADOW_BOTTOM);
-  } else {
-    bottom_shadow_ = rb->GetImageSkiaNamed(IDR_BUBBLE_B);
+  if (g_top_shadow.Get().isNull()) {
+    std::vector<gfx::ShadowValue> shadows;
+    // Blur by 1dp. See comment below about blur accounting.
+    shadows.emplace_back(gfx::Vector2d(), 2, SK_ColorBLACK);
+    g_top_shadow.Get() =
+        gfx::ImageSkiaOperations::CreateHorizontalShadow(shadows, false);
+  }
+  if (g_bottom_shadow.Get().isNull()) {
+    const int kSmallShadowBlur = 3;
+    const int kLargeShadowBlur = 8;
+    const int kLargeShadowYOffset = 3;
+
+    std::vector<gfx::ShadowValue> shadows;
+    // gfx::ShadowValue counts blur pixels both inside and outside the shape,
+    // whereas these blur values only describe the outside portion, hence they
+    // must be doubled.
+    shadows.emplace_back(gfx::Vector2d(), 2 * kSmallShadowBlur,
+                         SK_ColorBLACK);
+    shadows.emplace_back(gfx::Vector2d(0, kLargeShadowYOffset),
+                         2 * kLargeShadowBlur, SK_ColorBLACK);
+
+    g_bottom_shadow.Get() =
+        gfx::ImageSkiaOperations::CreateHorizontalShadow(shadows, true);
   }
 
   SetEventTargeter(
-      scoped_ptr<views::ViewTargeter>(new views::ViewTargeter(this)));
+      std::unique_ptr<views::ViewTargeter>(new views::ViewTargeter(this)));
 }
 
 void OmniboxPopupContentsView::Init() {
@@ -120,14 +148,8 @@ gfx::Rect OmniboxPopupContentsView::GetPopupBounds() const {
 void OmniboxPopupContentsView::LayoutChildren() {
   gfx::Rect contents_rect = GetContentsBounds();
   contents_rect.Inset(GetLayoutInsets(OMNIBOX_DROPDOWN));
-  contents_rect.Inset(0, views::NonClientFrameView::kClientEdgeThickness, 0, 0);
-
-  // In the non-material dropdown, the colored/clickable regions within the
-  // dropdown are only as wide as the location bar. In the material version,
-  // these are full width, and OmniboxResultView instead insets the icons/text
-  // inside to be aligned with the location bar.
-  if (!ui::MaterialDesignController::IsModeMaterial())
-    contents_rect.Inset(start_margin_, 0, end_margin_, 0);
+  contents_rect.Inset(start_margin_, g_top_shadow.Get().height(), end_margin_,
+                      0);
 
   int top = contents_rect.y();
   for (size_t i = 0; i < AutocompleteResult::kMaxMatches; ++i) {
@@ -144,12 +166,13 @@ void OmniboxPopupContentsView::LayoutChildren() {
 // OmniboxPopupContentsView, OmniboxPopupView overrides:
 
 bool OmniboxPopupContentsView::IsOpen() const {
-  return popup_ != NULL;
+  return popup_ != nullptr;
 }
 
 void OmniboxPopupContentsView::InvalidateLine(size_t line) {
   OmniboxResultView* result = result_view_at(line);
   result->Invalidate();
+  result->SchedulePaint();
 
   if (HasMatchAt(line) && GetMatchAtIndex(line).associated_keyword.get()) {
     result->ShowKeyword(IsSelectedIndex(line) &&
@@ -165,7 +188,7 @@ void OmniboxPopupContentsView::UpdatePopupAppearance() {
   if (model_->result().empty() || omnibox_view_->IsImeShowingPopup()) {
     // No matches or the IME is showing a popup window which may overlap
     // the omnibox popup window.  Close any existing popup.
-    if (popup_ != NULL) {
+    if (popup_) {
       size_animation_.Stop();
 
       // NOTE: Do NOT use CloseNow() here, as we may be deep in a callstack
@@ -200,24 +223,11 @@ void OmniboxPopupContentsView::UpdatePopupAppearance() {
   for (size_t i = result_size; i < AutocompleteResult::kMaxMatches; ++i)
     child_at(i)->SetVisible(false);
 
-  // In non-material mode, we want the popup to appear as if it's overlaying
-  // the top of the page content, i.e., is flush against the client edge at the
-  // bottom of the toolbar. However, if the bookmarks bar is attached, we want
-  // to draw over it (so as not to push the results below it), but that means
-  // the toolbar won't be drawing a client edge separating itself from the
-  // popup. So we unconditionally overlap the toolbar by the thickness of the
-  // client edge and draw our own edge (see OnPaint()), which fixes the
-  // attached bookmark bar case without breaking the other case.
-  int top_edge_overlap = views::NonClientFrameView::kClientEdgeThickness;
-  if (ui::MaterialDesignController::IsModeMaterial()) {
-    // In material mode, we cover the bookmark bar similarly, but instead of
-    // appearing below the client edge, we want the popup to appear to overlay
-    // the bottom of the toolbar. So instead of drawing a client edge atop the
-    // popup, we shift the popup to completely cover the client edge, and then
-    // draw an additional semitransparent shadow above that. So the total
-    // overlap necessary is the client edge thickness plus the shadow height.
-    top_edge_overlap += top_shadow_->height();
-  }
+  // We want the popup to appear to overlay the bottom of the toolbar. So we
+  // shift the popup to completely cover the client edge, and then draw an
+  // additional semitransparent shadow above that.
+  int top_edge_overlap = views::NonClientFrameView::kClientEdgeThickness +
+                         g_top_shadow.Get().height();
 
   gfx::Point top_left_screen_coord;
   int width;
@@ -235,11 +245,11 @@ void OmniboxPopupContentsView::UpdatePopupAppearance() {
     size_animation_.Reset();
   target_bounds_ = new_target_bounds;
 
-  if (popup_ == NULL) {
+  if (!popup_) {
     views::Widget* popup_parent = location_bar_view_->GetWidget();
 
     // If the popup is currently closed, we need to create it.
-    popup_ = (new AutocompletePopupWidget)->AsWeakPtr();
+    popup_ = (new AutocompletePopupWidget(popup_parent))->AsWeakPtr();
 
     views::Widget::InitParams params(views::Widget::InitParams::TYPE_POPUP);
 #if defined(OS_WIN)
@@ -252,19 +262,18 @@ void OmniboxPopupContentsView::UpdatePopupAppearance() {
     params.parent = popup_parent->GetNativeView();
     params.bounds = GetPopupBounds();
     params.context = popup_parent->GetNativeWindow();
-    params.native_theme = popup_parent->GetNativeTheme();
     popup_->Init(params);
     // Third-party software such as DigitalPersona identity verification can
     // hook the underlying window creation methods and use SendMessage to
     // synchronously change focus/activation, resulting in the popup being
     // destroyed by the time control returns here.  Bail out in this case to
     // avoid a NULL dereference.
-    if (!popup_.get())
+    if (!popup_)
       return;
     popup_->SetVisibilityAnimationTransition(views::Widget::ANIMATE_NONE);
     popup_->SetContentsView(this);
     popup_->StackAbove(omnibox_view_->GetRelativeWindowForPopup());
-    if (!popup_.get()) {
+    if (!popup_) {
       // For some IMEs GetRelativeWindowForPopup triggers the omnibox to lose
       // focus, thereby closing (and destroying) the popup.
       // TODO(sky): this won't be needed once we close the omnibox on input
@@ -327,7 +336,7 @@ bool OmniboxPopupContentsView::IsStarredMatch(
 void OmniboxPopupContentsView::AnimationProgressed(
     const gfx::Animation* animation) {
   // We should only be running the animation when the popup is already visible.
-  DCHECK(popup_ != NULL);
+  DCHECK(popup_);
   popup_->SetBounds(GetPopupBounds());
 }
 
@@ -345,7 +354,7 @@ void OmniboxPopupContentsView::Layout() {
 
 views::View* OmniboxPopupContentsView::GetTooltipHandlerForPoint(
     const gfx::Point& point) {
-  return NULL;
+  return nullptr;
 }
 
 bool OmniboxPopupContentsView::OnMousePressed(
@@ -371,8 +380,9 @@ void OmniboxPopupContentsView::OnMouseReleased(
   }
 
   if (event.IsOnlyMiddleMouseButton() || event.IsOnlyLeftMouseButton()) {
-    OpenSelectedLine(event, event.IsOnlyLeftMouseButton() ? CURRENT_TAB :
-                                                            NEW_BACKGROUND_TAB);
+    OpenSelectedLine(event, event.IsOnlyLeftMouseButton()
+                                ? WindowOpenDisposition::CURRENT_TAB
+                                : WindowOpenDisposition::NEW_BACKGROUND_TAB);
   }
 }
 
@@ -404,7 +414,7 @@ void OmniboxPopupContentsView::OnGestureEvent(ui::GestureEvent* event) {
       break;
     case ui::ET_GESTURE_TAP:
     case ui::ET_GESTURE_SCROLL_END:
-      OpenSelectedLine(*event, CURRENT_TAB);
+      OpenSelectedLine(*event, WindowOpenDisposition::CURRENT_TAB);
       break;
     default:
       return;
@@ -424,17 +434,14 @@ int OmniboxPopupContentsView::CalculatePopupHeight() {
   // Add enough space on the top and bottom so it looks like there is the same
   // amount of space between the text and the popup border as there is in the
   // interior between each row of text.
-  return popup_height + views::NonClientFrameView::kClientEdgeThickness +
-      GetLayoutInsets(OMNIBOX_DROPDOWN).height() +
-      bottom_shadow_->height() -
-      GetLayoutConstant(OMNIBOX_DROPDOWN_BORDER_INTERIOR);
+  return popup_height + GetLayoutInsets(OMNIBOX_DROPDOWN).height() +
+         g_top_shadow.Get().height() + g_bottom_shadow.Get().height();
 }
 
 OmniboxResultView* OmniboxPopupContentsView::CreateResultView(
     int model_index,
     const gfx::FontList& font_list) {
-  return new OmniboxResultView(this, model_index, location_bar_view_,
-                               font_list);
+  return new OmniboxResultView(this, model_index, font_list);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -445,26 +452,17 @@ const char* OmniboxPopupContentsView::GetClassName() const {
 }
 
 void OmniboxPopupContentsView::OnPaint(gfx::Canvas* canvas) {
-  // Top border.
-  if (ui::MaterialDesignController::IsModeMaterial()) {
-    canvas->TileImageInt(*top_shadow_, 0, 0, width(), top_shadow_->height());
-  } else {
-    canvas->FillRect(gfx::Rect(0, 0, width(),
-                               views::NonClientFrameView::kClientEdgeThickness),
-                     location_bar_view_->GetThemeProvider()->GetColor(
-                         ThemeProperties::COLOR_TOOLBAR_BOTTOM_SEPARATOR));
-  }
-
-  // Bottom border.
-  canvas->TileImageInt(*bottom_shadow_, 0, height() - bottom_shadow_->height(),
-                       width(), bottom_shadow_->height());
+  canvas->TileImageInt(g_top_shadow.Get(), 0, 0, width(),
+                       g_top_shadow.Get().height());
+  canvas->TileImageInt(g_bottom_shadow.Get(), 0,
+                       height() - g_bottom_shadow.Get().height(), width(),
+                       g_bottom_shadow.Get().height());
 }
 
 void OmniboxPopupContentsView::PaintChildren(const ui::PaintContext& context) {
   gfx::Rect contents_bounds = GetContentsBounds();
-  const int interior = GetLayoutConstant(OMNIBOX_DROPDOWN_BORDER_INTERIOR);
-  contents_bounds.Inset(0, views::NonClientFrameView::kClientEdgeThickness, 0,
-                        bottom_shadow_->height() - interior);
+  contents_bounds.Inset(0, g_top_shadow.Get().height(), 0,
+                        g_bottom_shadow.Get().height());
 
   ui::ClipRecorder clip_recorder(context);
   clip_recorder.ClipRect(contents_bounds);
