@@ -8,6 +8,8 @@
 #include <vector>
 
 #include "base/compiler_specific.h"
+#include "base/gtest_prod_util.h"
+#include "base/macros.h"
 #include "base/memory/ref_counted.h"
 #include "base/timer/timer.h"
 #include "chrome/browser/ui/views/tabs/tab.h"
@@ -56,23 +58,6 @@ class TabStrip : public views::View,
                  public views::ViewTargeterDelegate,
                  public TabController {
  public:
-  static const char kViewClassName[];
-
-  // Horizontal offset for the new tab button to bring it closer to the
-  // rightmost tab.
-  static const int kNewTabButtonHorizontalOffset;
-
-  // The vertical offset of the tab strip button. This offset applies only to
-  // restored windows.
-  static const int kNewTabButtonVerticalOffset;
-
-  // The size of the new tab button must be hardcoded because we need to be
-  // able to lay it out before we are able to get its image from the
-  // ui::ThemeProvider.  It also makes sense to do this, because the size of the
-  // new tab button should not need to be calculated dynamically.
-  static const int kNewTabButtonAssetWidth;
-  static const int kNewTabButtonAssetHeight;
-
   explicit TabStrip(TabStripController* controller);
   ~TabStrip() override;
 
@@ -116,8 +101,9 @@ class TabStrip : public views::View,
                int to_model_index,
                const TabRendererData& data);
 
-  // Removes a tab at the specified index.
-  void RemoveTabAt(int model_index);
+  // Removes a tab at the specified index. If the tab with |contents| is being
+  // dragged then the drag is completed.
+  void RemoveTabAt(content::WebContents* contents, int model_index);
 
   // Sets the tab data at the specified model index.
   void SetTabData(int model_index, const TabRendererData& data);
@@ -148,6 +134,10 @@ class TabStrip : public views::View,
     return tabs_.ideal_bounds(tab_data_index);
   }
 
+  // Max x-coordinate the tabstrip draws at, which is the right edge of the new
+  // tab button.
+  int max_x() const { return newtab_button_bounds_.right(); }
+
   // Returns the Tab at |index|.
   Tab* tab_at(int index) const { return tabs_.view_at(index); }
 
@@ -199,6 +189,12 @@ class TabStrip : public views::View,
   // Sets a painting style with miniature "tab indicator" rectangles at the top.
   void SetImmersiveStyle(bool enable);
 
+  // Returns the alpha that inactive tabs and the new tab button should use to
+  // blend against the frame background.  Inactive tabs and the new tab button
+  // differ in whether they change alpha when tab multiselection is occurring;
+  // |for_new_tab_button| toggles between the two calculations.
+  SkAlpha GetInactiveAlpha(bool for_new_tab_button) const;
+
   // Returns true if Tabs in this TabStrip are currently changing size or
   // position.
   bool IsAnimating() const;
@@ -211,9 +207,10 @@ class TabStrip : public views::View,
   void FileSupported(const GURL& url, bool supported);
 
   // TabController overrides:
-  const ui::ListSelectionModel& GetSelectionModel() override;
+  const ui::ListSelectionModel& GetSelectionModel() const override;
   bool SupportsMultipleSelection() override;
   bool ShouldHideCloseButtonForInactiveTabs() override;
+  bool MaySetClip() override;
   void SelectTab(Tab* tab) override;
   void ExtendSelectionTo(Tab* tab) override;
   void ToggleSelected(Tab* tab) override;
@@ -235,10 +232,16 @@ class TabStrip : public views::View,
   Tab* GetTabAt(Tab* tab, const gfx::Point& tab_in_tab_coordinates) override;
   void OnMouseEventInTab(views::View* source,
                          const ui::MouseEvent& event) override;
-  bool ShouldPaintTab(const Tab* tab, gfx::Rect* clip) override;
+  bool ShouldPaintTab(
+      const Tab* tab,
+      const base::Callback<gfx::Path(const gfx::Size&)>& border_callback,
+      gfx::Path* clip) override;
+  bool CanPaintThrobberToLayer() const override;
   bool IsImmersiveStyle() const override;
+  SkColor GetToolbarTopSeparatorColor() const override;
+  int GetBackgroundResourceId(bool* custom_image) const override;
   void UpdateTabAccessibilityState(const Tab* tab,
-                                   ui::AXViewState* state) override;
+                                   ui::AXNodeData* node_data) override;
 
   // MouseWatcherListener overrides:
   void MouseMovedOutOfHost() override;
@@ -254,7 +257,7 @@ class TabStrip : public views::View,
   int OnDragUpdated(const ui::DropTargetEvent& event) override;
   void OnDragExited() override;
   int OnPerformDrop(const ui::DropTargetEvent& event) override;
-  void GetAccessibleState(ui::AXViewState* state) override;
+  void GetAccessibleNodeData(ui::AXNodeData* node_data) override;
   views::View* GetTooltipHandlerForPoint(const gfx::Point& point) override;
 
   // Returns preferred height in immersive style.
@@ -310,14 +313,8 @@ class TabStrip : public views::View,
     DISALLOW_COPY_AND_ASSIGN(DropInfo);
   };
 
-  // Horizontal gap between pinned and non-pinned tabs.
-  static const int kPinnedToNonPinnedGap;
-
   void Init();
 
-  // Creates and returns a new tab. The caller owners the returned tab.
-  Tab* CreateTab();
-
   // Invoked from |AddTabAt| after the newly created tab has been inserted.
   void StartInsertTabAnimation(int model_index);
 
@@ -382,7 +379,7 @@ class TabStrip : public views::View,
   int GetPinnedTabCount() const;
 
   // Returns the last tab in the strip that's actually visible.  This will be
-  // the actual last tab unless the strip is in the overflow state.
+  // the actual last tab unless the strip is in the overflow node_data.
   const Tab* GetLastVisibleTab() const;
 
   // Adds the tab at |index| to |tabs_closing_map_| and removes the tab from
@@ -404,7 +401,7 @@ class TabStrip : public views::View,
   void DraggedTabsDetached();
 
   // Used by TabDragController when the user stops dragging tabs. |move_only| is
-  // true if the move behavior is TabDragController::MOVE_VISIBILE_TABS.
+  // true if the move behavior is TabDragController::MOVE_VISIBLE_TABS.
   // |completed| is true if the drag operation completed successfully, false if
   // it was reverted.
   void StoppedDraggingTabs(const Tabs& tabs,
@@ -440,20 +437,11 @@ class TabStrip : public views::View,
 
   // -- Tab Resize Layout -----------------------------------------------------
 
-  // Returns the exact (unrounded) current width of each tab.
-  void GetCurrentTabWidths(double* unselected_width,
-                           double* selected_width) const;
-
-  // Returns the exact (unrounded) desired width of each tab, based on the
-  // desired strip width and number of tabs.  If
-  // |width_of_tabs_for_mouse_close_| is nonnegative we use that value in
-  // calculating the desired strip width; otherwise we use the current width.
-  // |pinned_tab_count| gives the number of pinned tabs and |tab_count| the
-  // number of pinned and non-pinned tabs.
-  void GetDesiredTabWidths(int tab_count,
-                           int pinned_tab_count,
-                           double* unselected_width,
-                           double* selected_width) const;
+  // Returns the current width of each tab. If the space for tabs is not evenly
+  // divisible into these widths, the initial tabs in the strip will be 1 px
+  // larger.
+  int current_inactive_width() const { return current_inactive_width_; }
+  int current_active_width() const { return current_active_width_; }
 
   // Perform an animated resize-relayout of the TabStrip immediately.
   void ResizeLayoutTabs();
@@ -508,14 +496,9 @@ class TabStrip : public views::View,
   // index of the first non-pinned tab.
   int GenerateIdealBoundsForPinnedTabs(int* first_non_pinned_index);
 
-  // Returns the width needed for the new tab button (and padding).
-  static int new_tab_button_width() {
-    return kNewTabButtonAssetWidth + kNewTabButtonHorizontalOffset;
-  }
-
   // Returns the width of the area that contains tabs. This does not include
   // the width of the new tab button.
-  int tab_area_width() const { return width() - new_tab_button_width(); }
+  int GetTabAreaWidth() const;
 
   // Starts various types of TabStrip animations.
   void StartResizeLayoutAnimation();
@@ -532,19 +515,21 @@ class TabStrip : public views::View,
   int GetStartXForNormalTabs() const;
 
   // Returns the tab to use for event handling. This uses FindTabForEventFrom()
-  // to do the actual searching.
+  // to do the actual searching.  This method should be called when
+  // |touch_layout_| is set.
   Tab* FindTabForEvent(const gfx::Point& point);
 
-  // Returns the tab to use for event handling starting at index |start| and
-  // iterating by |delta|.
+  // Helper for FindTabForEvent().  Returns the tab to use for event handling
+  // starting at index |start| and iterating by |delta|.
   Tab* FindTabForEventFrom(const gfx::Point& point, int start, int delta);
 
   // For a given point, finds a tab that is hit by the point. If the point hits
   // an area on which two tabs are overlapping, the tab is selected as follows:
   // - If one of the tabs is active, select it.
   // - Select the left one.
-  // If no tabs are hit, returns NULL.
-  views::View* FindTabHitByPoint(const gfx::Point& point);
+  // If no tabs are hit, returns null.  This method should be called when
+  // |touch_layout_| is not set.
+  Tab* FindTabHitByPoint(const gfx::Point& point);
 
   // Returns the x-coordinates of the tabs.
   std::vector<int> GetTabXCoordinates();
@@ -590,7 +575,7 @@ class TabStrip : public views::View,
   views::ViewModelT<Tab> tabs_;
   TabsClosingMap tabs_closing_map_;
 
-  scoped_ptr<TabStripController> controller_;
+  std::unique_ptr<TabStripController> controller_;
 
   // The "New Tab" button.
   NewTabButton* newtab_button_;
@@ -598,20 +583,18 @@ class TabStrip : public views::View,
   // Ideal bounds of the new tab button.
   gfx::Rect newtab_button_bounds_;
 
-  // The current widths of various types of tabs.  We save these so that, as
-  // users close tabs while we're holding them at the same size, we can lay out
-  // tabs exactly and eliminate the "pixel jitter" we'd get from just leaving
-  // them all at their existing, rounded widths.
-  double current_unselected_width_;
-  double current_selected_width_;
+  // Returns the current widths of each type of tab.  If the tabstrip width is
+  // not evenly divisible into these widths, the initial tabs in the strip will
+  // be 1 px larger.
+  int current_inactive_width_;
+  int current_active_width_;
 
-  // If this value is nonnegative, it is used in GetDesiredTabWidths() to
-  // calculate how much space in the tab strip to use for tabs.  Most of the
-  // time this will be -1, but while we're handling closing a tab via the mouse,
-  // we'll set this to the edge of the last tab before closing, so that if we
-  // are closing the last tab and need to resize immediately, we'll resize only
-  // back to this width, thus once again placing the last tab under the mouse
-  // cursor.
+  // If this value is nonnegative, it is used as the width to lay out tabs
+  // (instead of tab_area_width()). Most of the time this will be -1, but while
+  // we're handling closing a tab via the mouse, we'll set this to the edge of
+  // the last tab before closing, so that if we are closing the last tab and
+  // need to resize immediately, we'll resize only back to this width, thus
+  // once again placing the last tab under the mouse cursor.
   int available_width_for_tabs_;
 
   // True if PrepareForCloseAt has been invoked. When true remove animations
@@ -619,7 +602,7 @@ class TabStrip : public views::View,
   bool in_tab_close_;
 
   // Valid for the lifetime of a drag over us.
-  scoped_ptr<DropInfo> drop_info_;
+  std::unique_ptr<DropInfo> drop_info_;
 
   // To ensure all tabs pulse at the same time they share the same animation
   // container. This is that animation container.
@@ -629,11 +612,11 @@ class TabStrip : public views::View,
   // . When a tab is closed to reset the layout.
   // . When a mouse is used and the layout dynamically adjusts and is currently
   //   stacked (|stacked_layout_| is true).
-  scoped_ptr<views::MouseWatcher> mouse_watcher_;
+  std::unique_ptr<views::MouseWatcher> mouse_watcher_;
 
   // The controller for a drag initiated from a Tab. Valid for the lifetime of
   // the drag session.
-  scoped_ptr<TabDragController> drag_controller_;
+  std::unique_ptr<TabDragController> drag_controller_;
 
   views::BoundsAnimator bounds_animator_;
 
@@ -647,7 +630,7 @@ class TabStrip : public views::View,
   bool adjust_layout_;
 
   // Only used while in touch mode.
-  scoped_ptr<StackedTabStripLayout> touch_layout_;
+  std::unique_ptr<StackedTabStripLayout> touch_layout_;
 
   // If true the |stacked_layout_| is set to false when the mouse exits the
   // tabstrip (as determined using MouseWatcher).
@@ -664,7 +647,7 @@ class TabStrip : public views::View,
 
   // Timer used when a tab is closed and we need to relayout. Only used when a
   // tab close comes from a touch device.
-  base::OneShotTimer<TabStrip> resize_layout_timer_;
+  base::OneShotTimer resize_layout_timer_;
 
   // True if tabs are painted as rectangular light-bars.
   bool immersive_style_;
