@@ -6,8 +6,11 @@
 #define CHROME_BROWSER_UI_VIEWS_VALIDATION_MESSAGE_BUBBLE_VIEW_H_
 
 #include "base/macros.h"
+#include "base/memory/weak_ptr.h"
 #include "chrome/browser/ui/validation_message_bubble.h"
-#include "chrome/browser/ui/views/validation_message_bubble_delegate.h"
+#include "content/public/browser/render_widget_host_view.h"
+#include "ui/gfx/geometry/size.h"
+#include "ui/views/bubble/bubble_dialog_delegate.h"
 
 namespace content {
 class WebContents;
@@ -15,8 +18,9 @@ class WebContents;
 
 // A ValidationMessageBubble implementation for Views.
 class ValidationMessageBubbleView
-    : public ValidationMessageBubble,
-      public ValidationMessageBubbleDelegate::Observer {
+    : public views::BubbleDialogDelegateView,
+      public ValidationMessageBubble,
+      public base::SupportsWeakPtr<ValidationMessageBubbleView> {
  public:
   ValidationMessageBubbleView(content::WebContents* web_contents,
                               const gfx::Rect& anchor_in_root_view,
@@ -24,16 +28,22 @@ class ValidationMessageBubbleView
                               const base::string16& sub_text);
   ~ValidationMessageBubbleView() override;
 
+  // BubbleDialogDelegateView overrides:
+  gfx::Size GetPreferredSize() const override;
+  int GetDialogButtons() const override;
+
   // ValidationMessageBubble overrides:
   void SetPositionRelativeToAnchor(
       content::RenderWidgetHost* widget_host,
       const gfx::Rect& anchor_in_root_view) override;
-
-  // ValidationMessageBubbleDelegate::Observer overrides:
-  void WindowClosing() override;
+  void CloseValidationMessage() override;
 
  private:
-  ValidationMessageBubbleDelegate* delegate_;
+  gfx::Rect RootViewToScreenRect(
+      const gfx::Rect& rect_in_root_view,
+      const content::RenderWidgetHostView* render_widget_host_view) const;
+
+  gfx::Size size_;
 
   DISALLOW_COPY_AND_ASSIGN(ValidationMessageBubbleView);
 };
