@@ -7,21 +7,25 @@
 #include <stddef.h>
 
 #include <set>
+#include <utility>
 
 #include "base/bind.h"
 #include "base/bind_helpers.h"
 #include "base/i18n/rtl.h"
 #include "base/i18n/time_formatting.h"
-#include "base/metrics/histogram.h"
-#include "base/prefs/pref_service.h"
+#include "base/logging.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/utf_string_conversions.h"
+#include "base/time/default_clock.h"
 #include "base/time/time.h"
 #include "base/values.h"
 #include "chrome/browser/banners/app_banner_settings_helper.h"
 #include "chrome/browser/bookmarks/bookmark_model_factory.h"
 #include "chrome/browser/engagement/site_engagement_service.h"
+#include "chrome/browser/favicon/fallback_icon_service_factory.h"
+#include "chrome/browser/favicon/large_icon_service_factory.h"
 #include "chrome/browser/history/history_service_factory.h"
 #include "chrome/browser/history/history_utils.h"
 #include "chrome/browser/history/web_history_service_factory.h"
@@ -30,32 +34,40 @@
 #include "chrome/browser/ui/browser_finder.h"
 #include "chrome/browser/ui/chrome_pages.h"
 #include "chrome/browser/ui/webui/favicon_source.h"
+#include "chrome/browser/ui/webui/large_icon_source.h"
 #include "chrome/common/features.h"
 #include "chrome/common/pref_names.h"
 #include "components/bookmarks/browser/bookmark_model.h"
 #include "components/bookmarks/browser/bookmark_utils.h"
-#include "components/browser_sync/browser/profile_sync_service.h"
+#include "components/browser_sync/profile_sync_service.h"
+#include "components/browsing_data/core/history_notice_utils.h"
+#include "components/favicon/core/fallback_icon_service.h"
+#include "components/favicon/core/fallback_url_util.h"
+#include "components/favicon/core/large_icon_service.h"
 #include "components/history/core/browser/history_service.h"
 #include "components/history/core/browser/history_types.h"
 #include "components/history/core/browser/web_history_service.h"
 #include "components/keyed_service/core/service_access_type.h"
+#include "components/prefs/pref_service.h"
 #include "components/query_parser/snippet.h"
-#include "components/sync_driver/device_info.h"
-#include "components/sync_driver/device_info_tracker.h"
+#include "components/strings/grit/components_strings.h"
+#include "components/sync/device_info/device_info.h"
+#include "components/sync/device_info/device_info_tracker.h"
+#include "components/sync/driver/sync_service_observer.h"
+#include "components/sync/protocol/history_delete_directive_specifics.pb.h"
+#include "components/sync/protocol/sync_enums.pb.h"
 #include "components/url_formatter/url_formatter.h"
 #include "content/public/browser/url_data_source.h"
 #include "content/public/browser/web_ui.h"
-#include "grit/components_strings.h"
-#include "sync/protocol/history_delete_directive_specifics.pb.h"
-#include "sync/protocol/sync_enums.pb.h"
+#include "extensions/features/features.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/l10n/time_format.h"
 
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
 #include "chrome/browser/extensions/activity_log/activity_log.h"
 #endif
 
-#if defined(ENABLE_SUPERVISED_USERS)
+#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
 #include "chrome/browser/supervised_user/supervised_user_navigation_observer.h"
 #include "chrome/browser/supervised_user/supervised_user_service.h"
 #include "chrome/browser/supervised_user/supervised_user_service_factory.h"
@@ -66,9 +78,16 @@
 #include "chrome/browser/android/chrome_application.h"
 #endif
 
+#if !defined(OS_ANDROID)
+#include "chrome/browser/ui/webui/md_history_ui.h"
+#endif
+
 // The amount of time to wait for a response from the WebHistoryService.
 static const int kWebHistoryTimeoutSeconds = 3;
 
+// Number of chars to truncate titles when making them "short".
+static const size_t kShortTitleLength = 300;
+
 using bookmarks::BookmarkModel;
 
 namespace {
@@ -88,8 +107,9 @@ static const char kDeviceTypeTablet[] = "tablet";
 
 // Returns a localized version of |visit_time| including a relative
 // indicator (e.g. today, yesterday).
-base::string16 GetRelativeDateLocalized(const base::Time& visit_time) {
-  base::Time midnight = base::Time::Now().LocalMidnight();
+base::string16 GetRelativeDateLocalized(base::Clock* clock,
+                                        const base::Time& visit_time) {
+  base::Time midnight = clock->Now().LocalMidnight();
   base::string16 date_str = ui::TimeFormat::RelativeDate(visit_time, &midnight);
   if (date_str.empty()) {
     date_str = base::TimeFormatFriendlyDate(visit_time);
@@ -119,7 +139,7 @@ bool IsLocalOnlyResult(const BrowsingHistoryHandler::HistoryEntry& entry) {
 
 // Gets the name and type of a device for the given sync client ID.
 // |name| and |type| are out parameters.
-void GetDeviceNameAndType(const ProfileSyncService* sync_service,
+void GetDeviceNameAndType(const browser_sync::ProfileSyncService* sync_service,
                           const std::string& client_id,
                           std::string* name,
                           std::string* type) {
@@ -129,7 +149,7 @@ void GetDeviceNameAndType(const ProfileSyncService* sync_service,
   DCHECK(sync_service->GetDeviceInfoTracker());
   DCHECK(sync_service->GetDeviceInfoTracker()->IsSyncing());
 
-  scoped_ptr<sync_driver::DeviceInfo> device_info =
+  std::unique_ptr<syncer::DeviceInfo> device_info =
       sync_service->GetDeviceInfoTracker()->GetDeviceInfo(client_id);
   if (device_info.get()) {
     *name = device_info->client_name();
@@ -150,14 +170,24 @@ void GetDeviceNameAndType(const ProfileSyncService* sync_service,
   *type = kDeviceTypeLaptop;
 }
 
+void RecordMetricsForNoticeAboutOtherFormsOfBrowsingHistory(bool shown) {
+  UMA_HISTOGRAM_BOOLEAN(
+      "History.ShownHeaderAboutOtherFormsOfBrowsingHistory",
+      shown);
+}
+
 }  // namespace
 
 BrowsingHistoryHandler::HistoryEntry::HistoryEntry(
     BrowsingHistoryHandler::HistoryEntry::EntryType entry_type,
-    const GURL& url, const base::string16& title, base::Time time,
-    const std::string& client_id, bool is_search_result,
-    const base::string16& snippet, bool blocked_visit,
-    const std::string& accept_languages) {
+    const GURL& url,
+    const base::string16& title,
+    base::Time time,
+    const std::string& client_id,
+    bool is_search_result,
+    const base::string16& snippet,
+    bool blocked_visit,
+    base::Clock* clock) {
   this->entry_type = entry_type;
   this->url = url;
   this->title = title;
@@ -167,18 +197,22 @@ BrowsingHistoryHandler::HistoryEntry::HistoryEntry(
   this->is_search_result = is_search_result;
   this->snippet = snippet;
   this->blocked_visit = blocked_visit;
-  this->accept_languages = accept_languages;
+  this->clock = clock;
 }
 
 BrowsingHistoryHandler::HistoryEntry::HistoryEntry()
     : entry_type(EMPTY_ENTRY), is_search_result(false), blocked_visit(false) {
 }
 
+BrowsingHistoryHandler::HistoryEntry::HistoryEntry(const HistoryEntry& other) =
+    default;
+
 BrowsingHistoryHandler::HistoryEntry::~HistoryEntry() {
 }
 
 void BrowsingHistoryHandler::HistoryEntry::SetUrlAndTitle(
-    base::DictionaryValue* result) const {
+    base::DictionaryValue* result,
+    bool limit_title_length) const {
   result->SetString("url", url.spec());
 
   bool using_url_as_the_title = false;
@@ -198,18 +232,22 @@ void BrowsingHistoryHandler::HistoryEntry::SetUrlAndTitle(
     else
       base::i18n::AdjustStringForLocaleDirection(&title_to_set);
   }
-  result->SetString("title", title_to_set);
+
+  result->SetString("title",
+      limit_title_length ? title_to_set.substr(0, kShortTitleLength)
+                         : title_to_set);
 }
 
-scoped_ptr<base::DictionaryValue> BrowsingHistoryHandler::HistoryEntry::ToValue(
+std::unique_ptr<base::DictionaryValue>
+BrowsingHistoryHandler::HistoryEntry::ToValue(
     BookmarkModel* bookmark_model,
     SupervisedUserService* supervised_user_service,
-    const ProfileSyncService* sync_service) const {
-  scoped_ptr<base::DictionaryValue> result(new base::DictionaryValue());
-  SetUrlAndTitle(result.get());
+    const browser_sync::ProfileSyncService* sync_service,
+    bool limit_title_length) const {
+  std::unique_ptr<base::DictionaryValue> result(new base::DictionaryValue());
+  SetUrlAndTitle(result.get(), limit_title_length);
 
-  base::string16 domain =
-      url_formatter::IDNToUnicode(url.host(), accept_languages);
+  base::string16 domain = url_formatter::IDNToUnicode(url.host());
   // When the domain is empty, use the scheme instead. This allows for a
   // sensible treatment of e.g. file: URLs when group by domain is on.
   if (domain.empty())
@@ -220,10 +258,14 @@ scoped_ptr<base::DictionaryValue> BrowsingHistoryHandler::HistoryEntry::ToValue(
   // HistoryEntry. Please update it whenever you add or remove
   // any keys in result.
   result->SetString("domain", domain);
+
+  result->SetString("fallbackFaviconText",
+                    base::UTF16ToASCII(favicon::GetFallbackIconText(url)));
+
   result->SetDouble("time", time.ToJsTime());
 
   // Pass the timestamps in a list.
-  scoped_ptr<base::ListValue> timestamps(new base::ListValue);
+  std::unique_ptr<base::ListValue> timestamps(new base::ListValue);
   for (std::set<int64_t>::const_iterator it = all_timestamps.begin();
        it != all_timestamps.end(); ++it) {
     timestamps->AppendDouble(base::Time::FromInternalValue(*it).ToJsTime());
@@ -234,12 +276,19 @@ scoped_ptr<base::DictionaryValue> BrowsingHistoryHandler::HistoryEntry::ToValue(
   // the monthly view.
   result->SetString("dateShort", base::TimeFormatShortDate(time));
 
+  base::string16 snippet_string;
+  base::string16 date_relative_day;
+  base::string16 date_time_of_day;
+  bool is_blocked_visit = false;
+  int host_filtering_behavior = -1;
+
   // Only pass in the strings we need (search results need a shortdate
-  // and snippet, browse results need day and time information).
+  // and snippet, browse results need day and time information). Makes sure that
+  // values of result are never undefined
   if (is_search_result) {
-    result->SetString("snippet", snippet);
+    snippet_string = snippet;
   } else {
-    base::Time midnight = base::Time::Now().LocalMidnight();
+    base::Time midnight = clock->Now().LocalMidnight();
     base::string16 date_str = ui::TimeFormat::RelativeDate(time, &midnight);
     if (date_str.empty()) {
       date_str = base::TimeFormatFriendlyDate(time);
@@ -249,10 +298,9 @@ scoped_ptr<base::DictionaryValue> BrowsingHistoryHandler::HistoryEntry::ToValue(
           date_str,
           base::TimeFormatFriendlyDate(time));
     }
-    result->SetString("dateRelativeDay", date_str);
-    result->SetString("dateTimeOfDay", base::TimeFormatTimeOfDay(time));
+    date_relative_day = date_str;
+    date_time_of_day = base::TimeFormatTimeOfDay(time);
   }
-  result->SetBoolean("starred", bookmark_model->IsBookmarked(url));
 
   std::string device_name;
   std::string device_type;
@@ -261,18 +309,24 @@ scoped_ptr<base::DictionaryValue> BrowsingHistoryHandler::HistoryEntry::ToValue(
   result->SetString("deviceName", device_name);
   result->SetString("deviceType", device_type);
 
-#if defined(ENABLE_SUPERVISED_USERS)
+#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
   if (supervised_user_service) {
     const SupervisedUserURLFilter* url_filter =
         supervised_user_service->GetURLFilterForUIThread();
     int filtering_behavior =
         url_filter->GetFilteringBehaviorForURL(url.GetWithEmptyPath());
-    result->SetInteger("hostFilteringBehavior", filtering_behavior);
-
-    result->SetBoolean("blockedVisit", blocked_visit);
+    is_blocked_visit = blocked_visit;
+    host_filtering_behavior = filtering_behavior;
   }
 #endif
 
+  result->SetString("dateTimeOfDay", date_time_of_day);
+  result->SetString("dateRelativeDay", date_relative_day);
+  result->SetString("snippet", snippet_string);
+  result->SetBoolean("starred", bookmark_model->IsBookmarked(url));
+  result->SetInteger("hostFilteringBehavior", host_filtering_behavior);
+  result->SetBoolean("blockedVisit", is_blocked_visit);
+
   return result;
 }
 
@@ -285,8 +339,12 @@ bool BrowsingHistoryHandler::HistoryEntry::SortByTimeDescending(
 BrowsingHistoryHandler::BrowsingHistoryHandler()
     : has_pending_delete_request_(false),
       history_service_observer_(this),
-      weak_factory_(this) {
-}
+      web_history_service_observer_(this),
+      sync_service_observer_(this),
+      has_synced_results_(false),
+      has_other_forms_of_browsing_history_(false),
+      clock_(new base::DefaultClock()),
+      weak_factory_(this) {}
 
 BrowsingHistoryHandler::~BrowsingHistoryHandler() {
   query_task_tracker_.TryCancelAll();
@@ -296,14 +354,39 @@ BrowsingHistoryHandler::~BrowsingHistoryHandler() {
 void BrowsingHistoryHandler::RegisterMessages() {
   // Create our favicon data source.
   Profile* profile = Profile::FromWebUI(web_ui());
+
+#if defined(OS_ANDROID)
+  favicon::FallbackIconService* fallback_icon_service =
+      FallbackIconServiceFactory::GetForBrowserContext(profile);
+  favicon::LargeIconService* large_icon_service =
+      LargeIconServiceFactory::GetForBrowserContext(profile);
+  content::URLDataSource::Add(
+      profile, new LargeIconSource(fallback_icon_service, large_icon_service));
+#else
   content::URLDataSource::Add(
       profile, new FaviconSource(profile, FaviconSource::ANY));
+#endif
 
   // Get notifications when history is cleared.
-  history::HistoryService* hs = HistoryServiceFactory::GetForProfile(
+  history::HistoryService* local_history = HistoryServiceFactory::GetForProfile(
       profile, ServiceAccessType::EXPLICIT_ACCESS);
-  if (hs)
-    history_service_observer_.Add(hs);
+  if (local_history)
+    history_service_observer_.Add(local_history);
+
+  // Get notifications when web history is deleted.
+  history::WebHistoryService* web_history =
+      WebHistoryServiceFactory::GetForProfile(profile);
+  if (web_history) {
+    web_history_service_observer_.Add(web_history);
+  } else {
+    // If |web_history| is not available, it means that the history sync is
+    // disabled. Observe |sync_service| so that we can attach the listener
+    // in case it gets enabled later.
+    browser_sync::ProfileSyncService* sync_service =
+        ProfileSyncServiceFactory::GetForProfile(profile);
+    if (sync_service)
+      sync_service_observer_.Add(sync_service);
+  }
 
   web_ui()->RegisterMessageCallback("queryHistory",
       base::Bind(&BrowsingHistoryHandler::HandleQueryHistory,
@@ -332,7 +415,20 @@ bool BrowsingHistoryHandler::ExtractIntegerValueAtIndex(
   return false;
 }
 
+void BrowsingHistoryHandler::OnStateChanged() {
+  // If the history sync was enabled, start observing WebHistoryService.
+  // This method should not be called after we already added the observer.
+  history::WebHistoryService* web_history =
+      WebHistoryServiceFactory::GetForProfile(Profile::FromWebUI(web_ui()));
+  if (web_history) {
+    DCHECK(!web_history_service_observer_.IsObserving(web_history));
+    web_history_service_observer_.Add(web_history);
+    sync_service_observer_.RemoveAll();
+  }
+}
+
 void BrowsingHistoryHandler::WebHistoryTimeout() {
+  has_synced_results_ = false;
   // TODO(dubroy): Communicate the failure to the front end.
   if (!query_task_tracker_.HasTrackedTasks())
     ReturnResultsToFrontEnd();
@@ -366,6 +462,10 @@ void BrowsingHistoryHandler::QueryHistory(
 
   history::WebHistoryService* web_history =
       WebHistoryServiceFactory::GetForProfile(profile);
+
+  // Set this to false until the results actually arrive.
+  results_info_value_.SetBoolean("hasSyncedResults", false);
+
   if (web_history) {
     web_history_query_results_.clear();
     web_history_request_ = web_history->QueryHistory(
@@ -380,8 +480,18 @@ void BrowsingHistoryHandler::QueryHistory(
         FROM_HERE, base::TimeDelta::FromSeconds(kWebHistoryTimeoutSeconds),
         this, &BrowsingHistoryHandler::WebHistoryTimeout);
 
-    // Set this to false until the results actually arrive.
-    results_info_value_.SetBoolean("hasSyncedResults", false);
+    // Test the existence of other forms of browsing history.
+    browsing_data::ShouldShowNoticeAboutOtherFormsOfBrowsingHistory(
+        ProfileSyncServiceFactory::GetInstance()->GetForProfile(profile),
+        web_history,
+        base::Bind(
+            &BrowsingHistoryHandler::OtherFormsOfBrowsingHistoryQueryComplete,
+            weak_factory_.GetWeakPtr()));
+  } else {
+    // The notice could not have been shown, because there is no web history.
+    RecordMetricsForNoticeAboutOtherFormsOfBrowsingHistory(false);
+    has_synced_results_ = false;
+    has_other_forms_of_browsing_history_ = false;
   }
 }
 
@@ -439,7 +549,7 @@ void BrowsingHistoryHandler::HandleRemoveVisits(const base::ListValue* args) {
   if (delete_task_tracker_.HasTrackedTasks() ||
       has_pending_delete_request_ ||
       !profile->GetPrefs()->GetBoolean(prefs::kAllowDeletingBrowserHistory)) {
-    web_ui()->CallJavascriptFunction("deleteFailed");
+    web_ui()->CallJavascriptFunctionUnsafe("deleteFailed");
     return;
   }
 
@@ -449,7 +559,7 @@ void BrowsingHistoryHandler::HandleRemoveVisits(const base::ListValue* args) {
   history::WebHistoryService* web_history =
       WebHistoryServiceFactory::GetForProfile(profile);
 
-  base::Time now = base::Time::Now();
+  base::Time now = clock_->Now();
   std::vector<history::ExpireHistoryArgs> expire_list;
   expire_list.reserve(args->GetSize());
 
@@ -528,7 +638,7 @@ void BrowsingHistoryHandler::HandleRemoveVisits(const base::ListValue* args) {
                    weak_factory_.GetWeakPtr()));
   }
 
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
   // If the profile has activity logging enabled also clean up any URLs from
   // the extension activity log. The extension activity log contains URLS
   // which websites an extension has activity on so it will indirectly
@@ -562,7 +672,7 @@ void BrowsingHistoryHandler::HandleClearBrowsingData(
 void BrowsingHistoryHandler::HandleRemoveBookmark(const base::ListValue* args) {
   base::string16 url = ExtractStringValue(args);
   Profile* profile = Profile::FromWebUI(web_ui());
-  BookmarkModel* model = BookmarkModelFactory::GetForProfile(profile);
+  BookmarkModel* model = BookmarkModelFactory::GetForBrowserContext(profile);
   bookmarks::RemoveAllBookmarks(model, GURL(url));
 }
 
@@ -612,16 +722,58 @@ void BrowsingHistoryHandler::MergeDuplicateResults(
   results->swap(new_results);
 }
 
+void BrowsingHistoryHandler::QueryComplete(
+    const base::string16& search_text,
+    const history::QueryOptions& options,
+    history::QueryResults* results) {
+  DCHECK_EQ(0U, query_results_.size());
+  query_results_.reserve(results->size());
+
+  for (size_t i = 0; i < results->size(); ++i) {
+    history::URLResult const &page = (*results)[i];
+    // TODO(dubroy): Use sane time (crbug.com/146090) here when it's ready.
+    query_results_.push_back(HistoryEntry(
+        HistoryEntry::LOCAL_ENTRY, page.url(), page.title(), page.visit_time(),
+        std::string(), !search_text.empty(), page.snippet().text(),
+        page.blocked_visit(), clock_.get()));
+  }
+
+  // The items which are to be written into results_info_value_ are also
+  // described in chrome/browser/resources/history/history.js in @typedef for
+  // HistoryQuery. Please update it whenever you add or remove any keys in
+  // results_info_value_.
+  results_info_value_.SetString("term", search_text);
+  results_info_value_.SetBoolean("finished", results->reached_beginning());
+
+  // Add the specific dates that were searched to display them.
+  // TODO(sergiu): Put today if the start is in the future.
+  results_info_value_.SetString(
+      "queryStartTime",
+      GetRelativeDateLocalized(clock_.get(), options.begin_time));
+  if (!options.end_time.is_null()) {
+    results_info_value_.SetString(
+        "queryEndTime",
+        GetRelativeDateLocalized(
+            clock_.get(), options.end_time - base::TimeDelta::FromDays(1)));
+  } else {
+    results_info_value_.SetString(
+        "queryEndTime", GetRelativeDateLocalized(clock_.get(), clock_->Now()));
+  }
+  if (!web_history_timer_.IsRunning())
+    ReturnResultsToFrontEnd();
+}
+
 void BrowsingHistoryHandler::ReturnResultsToFrontEnd() {
   Profile* profile = Profile::FromWebUI(web_ui());
-  BookmarkModel* bookmark_model = BookmarkModelFactory::GetForProfile(profile);
+  BookmarkModel* bookmark_model =
+      BookmarkModelFactory::GetForBrowserContext(profile);
   SupervisedUserService* supervised_user_service = NULL;
-#if defined(ENABLE_SUPERVISED_USERS)
+#if BUILDFLAG(ENABLE_SUPERVISED_USERS)
   if (profile->IsSupervised())
     supervised_user_service =
         SupervisedUserServiceFactory::GetForProfile(profile);
 #endif
-  ProfileSyncService* sync_service =
+  browser_sync::ProfileSyncService* sync_service =
       ProfileSyncServiceFactory::GetInstance()->GetForProfile(profile);
 
   // Combine the local and remote results into |query_results_|, and remove
@@ -643,69 +795,30 @@ void BrowsingHistoryHandler::ReturnResultsToFrontEnd() {
     }
   }
 
+  bool is_md = false;
+#if !defined(OS_ANDROID)
+  is_md = MdHistoryUI::IsEnabled(profile);
+#endif
+
   // Convert the result vector into a ListValue.
   base::ListValue results_value;
   for (std::vector<BrowsingHistoryHandler::HistoryEntry>::iterator it =
            query_results_.begin(); it != query_results_.end(); ++it) {
-    scoped_ptr<base::Value> value(
-        it->ToValue(bookmark_model, supervised_user_service, sync_service));
-    results_value.Append(value.release());
+    std::unique_ptr<base::Value> value(it->ToValue(
+        bookmark_model, supervised_user_service, sync_service, is_md));
+    results_value.Append(std::move(value));
   }
 
-  web_ui()->CallJavascriptFunction(
-      "historyResult", results_info_value_, results_value);
+  web_ui()->CallJavascriptFunctionUnsafe("historyResult", results_info_value_,
+                                         results_value);
+  web_ui()->CallJavascriptFunctionUnsafe(
+      "showNotification", base::FundamentalValue(has_synced_results_),
+      base::FundamentalValue(has_other_forms_of_browsing_history_));
   results_info_value_.Clear();
   query_results_.clear();
   web_history_query_results_.clear();
 }
 
-void BrowsingHistoryHandler::QueryComplete(
-    const base::string16& search_text,
-    const history::QueryOptions& options,
-    history::QueryResults* results) {
-  DCHECK_EQ(0U, query_results_.size());
-  query_results_.reserve(results->size());
-  const std::string accept_languages = GetAcceptLanguages();
-
-  for (size_t i = 0; i < results->size(); ++i) {
-    history::URLResult const &page = (*results)[i];
-    // TODO(dubroy): Use sane time (crbug.com/146090) here when it's ready.
-    query_results_.push_back(
-        HistoryEntry(
-            HistoryEntry::LOCAL_ENTRY,
-            page.url(),
-            page.title(),
-            page.visit_time(),
-            std::string(),
-            !search_text.empty(),
-            page.snippet().text(),
-            page.blocked_visit(),
-            accept_languages));
-  }
-
-  // The items which are to be written into results_info_value_ are also
-  // described in chrome/browser/resources/history/history.js in @typedef for
-  // HistoryQuery. Please update it whenever you add or remove any keys in
-  // results_info_value_.
-  results_info_value_.SetString("term", search_text);
-  results_info_value_.SetBoolean("finished", results->reached_beginning());
-
-  // Add the specific dates that were searched to display them.
-  // TODO(sergiu): Put today if the start is in the future.
-  results_info_value_.SetString("queryStartTime",
-                                GetRelativeDateLocalized(options.begin_time));
-  if (!options.end_time.is_null()) {
-    results_info_value_.SetString("queryEndTime",
-        GetRelativeDateLocalized(options.end_time -
-                                 base::TimeDelta::FromDays(1)));
-  } else {
-    results_info_value_.SetString("queryEndTime",
-        GetRelativeDateLocalized(base::Time::Now()));
-  }
-  if (!web_history_timer_.IsRunning())
-    ReturnResultsToFrontEnd();
-}
-
 void BrowsingHistoryHandler::WebHistoryQueryComplete(
     const base::string16& search_text,
     const history::QueryOptions& options,
@@ -714,7 +827,6 @@ void BrowsingHistoryHandler::WebHistoryQueryComplete(
     const base::DictionaryValue* results_value) {
   base::TimeDelta delta = base::TimeTicks::Now() - start_time;
   UMA_HISTOGRAM_TIMES("WebHistory.ResponseTime", delta);
-  const std::string accept_languages = GetAcceptLanguages();
 
   // If the response came in too late, do nothing.
   // TODO(dubroy): Maybe show a banner, and prompt the user to reload?
@@ -780,31 +892,37 @@ void BrowsingHistoryHandler::WebHistoryQueryComplete(
         id->GetString("client_id", &client_id);
 
         web_history_query_results_.push_back(
-            HistoryEntry(
-                HistoryEntry::REMOTE_ENTRY,
-                gurl,
-                title,
-                time,
-                client_id,
-                !search_text.empty(),
-                base::string16(),
-                /* blocked_visit */ false,
-                accept_languages));
+            HistoryEntry(HistoryEntry::REMOTE_ENTRY, gurl, title, time,
+                         client_id, !search_text.empty(), base::string16(),
+                         /* blocked_visit */ false, clock_.get()));
       }
     }
   }
-  results_info_value_.SetBoolean("hasSyncedResults", results_value != NULL);
+  has_synced_results_ = results_value != nullptr;
+  results_info_value_.SetBoolean("hasSyncedResults", has_synced_results_);
   if (!query_task_tracker_.HasTrackedTasks())
     ReturnResultsToFrontEnd();
 }
 
+void BrowsingHistoryHandler::OtherFormsOfBrowsingHistoryQueryComplete(
+    bool found_other_forms_of_browsing_history) {
+  has_other_forms_of_browsing_history_ = found_other_forms_of_browsing_history;
+
+  RecordMetricsForNoticeAboutOtherFormsOfBrowsingHistory(
+      has_other_forms_of_browsing_history_);
+
+  web_ui()->CallJavascriptFunctionUnsafe(
+      "showNotification", base::FundamentalValue(has_synced_results_),
+      base::FundamentalValue(has_other_forms_of_browsing_history_));
+}
+
 void BrowsingHistoryHandler::RemoveComplete() {
   urls_to_be_deleted_.clear();
 
   // Notify the page that the deletion request is complete, but only if a web
   // history delete request is not still pending.
   if (!has_pending_delete_request_)
-    web_ui()->CallJavascriptFunction("deleteComplete");
+    web_ui()->CallJavascriptFunctionUnsafe("deleteComplete");
 }
 
 void BrowsingHistoryHandler::RemoveWebHistoryComplete(bool success) {
@@ -819,8 +937,8 @@ void BrowsingHistoryHandler::SetQueryTimeInWeeks(
     int offset, history::QueryOptions* options) {
   // LocalMidnight returns the beginning of the current day so get the
   // beginning of the next one.
-  base::Time midnight = base::Time::Now().LocalMidnight() +
-                              base::TimeDelta::FromDays(1);
+  base::Time midnight =
+      clock_->Now().LocalMidnight() + base::TimeDelta::FromDays(1);
   options->end_time = midnight -
       base::TimeDelta::FromDays(7 * offset);
   options->begin_time = midnight -
@@ -832,11 +950,14 @@ void BrowsingHistoryHandler::SetQueryTimeInMonths(
   // Configure the begin point of the search to the start of the
   // current month.
   base::Time::Exploded exploded;
-  base::Time::Now().LocalMidnight().LocalExplode(&exploded);
+  clock_->Now().LocalMidnight().LocalExplode(&exploded);
   exploded.day_of_month = 1;
 
   if (offset == 0) {
-    options->begin_time = base::Time::FromLocalExploded(exploded);
+    if (!base::Time::FromLocalExploded(exploded, &options->begin_time)) {
+      // TODO(maksims): implement errors handling here.
+      NOTIMPLEMENTED();
+    }
 
     // Set the end time of this first search to null (which will
     // show results from the future, should the user's clock have
@@ -850,12 +971,18 @@ void BrowsingHistoryHandler::SetQueryTimeInMonths(
     exploded.month -= offset - 1;
     // Set the correct year.
     NormalizeMonths(&exploded);
-    options->end_time = base::Time::FromLocalExploded(exploded);
+    if (!base::Time::FromLocalExploded(exploded, &options->end_time)) {
+      // TODO(maksims): implement errors handling here.
+      NOTIMPLEMENTED();
+    }
 
     exploded.month -= 1;
     // Set the correct year
     NormalizeMonths(&exploded);
-    options->begin_time = base::Time::FromLocalExploded(exploded);
+    if (!base::Time::FromLocalExploded(exploded, &options->begin_time)) {
+      // TODO(maksims): implement errors handling here.
+      NOTIMPLEMENTED();
+    }
   }
 }
 
@@ -872,11 +999,6 @@ static bool DeletionsDiffer(const history::URLRows& deleted_rows,
   return false;
 }
 
-std::string BrowsingHistoryHandler::GetAcceptLanguages() const {
-  Profile* profile = Profile::FromWebUI(web_ui());
-  return profile->GetPrefs()->GetString(prefs::kAcceptLanguages);
-}
-
 void BrowsingHistoryHandler::OnURLsDeleted(
     history::HistoryService* history_service,
     bool all_history,
@@ -884,5 +1006,10 @@ void BrowsingHistoryHandler::OnURLsDeleted(
     const history::URLRows& deleted_rows,
     const std::set<GURL>& favicon_urls) {
   if (all_history || DeletionsDiffer(deleted_rows, urls_to_be_deleted_))
-    web_ui()->CallJavascriptFunction("historyDeleted");
+    web_ui()->CallJavascriptFunctionUnsafe("historyDeleted");
+}
+
+void BrowsingHistoryHandler::OnWebHistoryDeleted() {
+  if (!has_pending_delete_request_)
+    web_ui()->CallJavascriptFunctionUnsafe("historyDeleted");
 }
