@@ -4,35 +4,45 @@
 
 #include "chrome/browser/ui/webui/chromeos/login/core_oobe_handler.h"
 
+#include <type_traits>
+
+#include "ash/common/accessibility_types.h"
 #include "ash/shell.h"
+#include "base/bind.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/chromeos/accessibility/accessibility_manager.h"
 #include "chrome/browser/chromeos/accessibility/magnification_manager.h"
+#include "chrome/browser/chromeos/events/keyboard_driven_event_rewriter.h"
 #include "chrome/browser/chromeos/login/helper.h"
 #include "chrome/browser/chromeos/login/lock/screen_locker.h"
-#include "chrome/browser/chromeos/login/ui/login_display_host_impl.h"
+#include "chrome/browser/chromeos/login/lock/webui_screen_locker.h"
+#include "chrome/browser/chromeos/login/ui/login_display_host.h"
 #include "chrome/browser/chromeos/login/wizard_controller.h"
 #include "chrome/browser/chromeos/policy/browser_policy_connector_chromeos.h"
 #include "chrome/browser/chromeos/policy/device_cloud_policy_manager_chromeos.h"
 #include "chrome/browser/chromeos/system/input_device_settings.h"
 #include "chrome/browser/lifetime/application_lifetime.h"
+#include "chrome/browser/ui/ash/ash_util.h"
 #include "chrome/browser/ui/webui/chromeos/login/oobe_ui.h"
 #include "chrome/browser/ui/webui/chromeos/login/signin_screen_handler.h"
 #include "chrome/common/channel_info.h"
 #include "chrome/common/chrome_constants.h"
+#include "chrome/common/pref_names.h"
 #include "chrome/grit/chromium_strings.h"
 #include "chrome/grit/generated_resources.h"
 #include "chromeos/chromeos_constants.h"
+#include "components/login/base_screen_handler_utils.h"
 #include "components/login/localized_values_builder.h"
+#include "components/prefs/pref_service.h"
+#include "components/strings/grit/components_strings.h"
 #include "components/version_info/version_info.h"
 #include "google_apis/google_api_keys.h"
-#include "grit/components_strings.h"
-#include "ui/chromeos/accessibility_types.h"
-#include "ui/gfx/display.h"
+#include "ui/aura/window_tree_host.h"
+#include "ui/display/display.h"
+#include "ui/display/screen.h"
 #include "ui/gfx/geometry/size.h"
-#include "ui/gfx/screen.h"
 #include "ui/keyboard/keyboard_controller.h"
 
 namespace {
@@ -48,14 +58,16 @@ namespace chromeos {
 CoreOobeHandler::CoreOobeHandler(OobeUI* oobe_ui)
     : BaseScreenHandler(kJsScreenPath),
       oobe_ui_(oobe_ui),
-      show_oobe_ui_(false),
-      version_info_updater_(this),
-      delegate_(NULL) {
-  AccessibilityManager* accessibility_manager = AccessibilityManager::Get();
-  CHECK(accessibility_manager);
-  accessibility_subscription_ = accessibility_manager->RegisterCallback(
-      base::Bind(&CoreOobeHandler::OnAccessibilityStatusChanged,
-                 base::Unretained(this)));
+      version_info_updater_(this) {
+  if (!chrome::IsRunningInMash()) {
+    AccessibilityManager* accessibility_manager = AccessibilityManager::Get();
+    CHECK(accessibility_manager);
+    accessibility_subscription_ = accessibility_manager->RegisterCallback(
+        base::Bind(&CoreOobeHandler::OnAccessibilityStatusChanged,
+                   base::Unretained(this)));
+  } else {
+    NOTIMPLEMENTED();
+  }
 }
 
 CoreOobeHandler::~CoreOobeHandler() {
@@ -129,6 +141,8 @@ void CoreOobeHandler::RegisterMessages() {
               &CoreOobeHandler::HandleEnableLargeCursor);
   AddCallback("enableVirtualKeyboard",
               &CoreOobeHandler::HandleEnableVirtualKeyboard);
+  AddCallback("setForceDisableVirtualKeyboard",
+              &CoreOobeHandler::HandleSetForceDisableVirtualKeyboard);
   AddCallback("enableScreenMagnifier",
               &CoreOobeHandler::HandleEnableScreenMagnifier);
   AddCallback("enableSpokenFeedback",
@@ -146,6 +160,40 @@ void CoreOobeHandler::RegisterMessages() {
               &CoreOobeHandler::HandleEnableDebuggingScreen);
   AddCallback("headerBarVisible",
               &CoreOobeHandler::HandleHeaderBarVisible);
+  AddCallback("raiseTabKeyEvent", &CoreOobeHandler::HandleRaiseTabKeyEvent);
+  AddCallback("setOobeBootstrappingSlave",
+              &CoreOobeHandler::HandleSetOobeBootstrappingSlave);
+}
+
+template <typename... Args>
+void CoreOobeHandler::ExecuteDeferredJSCall(const std::string& function_name,
+                                            std::unique_ptr<Args>... args) {
+  CallJS(function_name, *args...);
+}
+
+template <typename... Args>
+void CoreOobeHandler::CallJSOrDefer(const std::string& function_name,
+                                    const Args&... args) {
+  if (is_initialized_) {
+    CallJS(function_name, args...);
+  } else {
+    // Note that std::conditional is used here in order to obtain a sequence of
+    // base::Value types with the length equal to sizeof...(Args); the C++
+    // template parameter pack expansion rules require that the name of the
+    // parameter pack appears in the pattern, even though the elements of the
+    // Args pack are not actually in this code.
+    deferred_js_calls_.push_back(base::Bind(
+        &CoreOobeHandler::ExecuteDeferredJSCall<
+            typename std::conditional<true, base::Value, Args>::type...>,
+        base::Unretained(this), function_name,
+        base::Passed(::login::MakeValue(args).CreateDeepCopy())...));
+  }
+}
+
+void CoreOobeHandler::ExecuteDeferredJSCalls() {
+  for (const auto& deferred_js_call : deferred_js_calls_)
+    deferred_js_call.Run();
+  deferred_js_calls_.clear();
 }
 
 void CoreOobeHandler::ShowSignInError(
@@ -154,12 +202,12 @@ void CoreOobeHandler::ShowSignInError(
     const std::string& help_link_text,
     HelpAppLauncher::HelpTopic help_topic_id) {
   LOG(ERROR) << "CoreOobeHandler::ShowSignInError: error_text=" << error_text;
-  CallJS("showSignInError", login_attempts, error_text,
+  CallJSOrDefer("showSignInError", login_attempts, error_text,
          help_link_text, static_cast<int>(help_topic_id));
 }
 
 void CoreOobeHandler::ShowTpmError() {
-  CallJS("showTpmError");
+  CallJSOrDefer("showTpmError");
 }
 
 void CoreOobeHandler::ShowDeviceResetScreen() {
@@ -172,11 +220,9 @@ void CoreOobeHandler::ShowDeviceResetScreen() {
     if (wizard_controller && !wizard_controller->login_screen_started()) {
       wizard_controller->AdvanceToScreen(WizardController::kResetScreenName);
     } else {
-      DCHECK(LoginDisplayHostImpl::default_host());
-      if (LoginDisplayHostImpl::default_host()) {
-        LoginDisplayHostImpl::default_host()->StartWizard(
-            WizardController::kResetScreenName);
-      }
+      DCHECK(LoginDisplayHost::default_host());
+      LoginDisplayHost::default_host()->StartWizard(
+          WizardController::kResetScreenName);
     }
   }
 }
@@ -192,55 +238,62 @@ void CoreOobeHandler::ShowEnableDebuggingScreen() {
 }
 
 void CoreOobeHandler::ShowSignInUI(const std::string& email) {
-  CallJS("showSigninUI", email);
+  CallJSOrDefer("showSigninUI", email);
 }
 
 void CoreOobeHandler::ResetSignInUI(bool force_online) {
-  CallJS("resetSigninUI", force_online);
+  CallJSOrDefer("resetSigninUI", force_online);
 }
 
 void CoreOobeHandler::ClearUserPodPassword() {
-  CallJS("clearUserPodPassword");
+  CallJSOrDefer("clearUserPodPassword");
 }
 
 void CoreOobeHandler::RefocusCurrentPod() {
-  CallJS("refocusCurrentPod");
+  CallJSOrDefer("refocusCurrentPod");
 }
 
 void CoreOobeHandler::ShowPasswordChangedScreen(bool show_password_error,
                                                 const std::string& email) {
-  CallJS("showPasswordChangedScreen", show_password_error, email);
+  CallJSOrDefer("showPasswordChangedScreen", show_password_error, email);
 }
 
 void CoreOobeHandler::SetUsageStats(bool checked) {
-  CallJS("setUsageStats", checked);
+  CallJSOrDefer("setUsageStats", checked);
 }
 
 void CoreOobeHandler::SetOemEulaUrl(const std::string& oem_eula_url) {
-  CallJS("setOemEulaUrl", oem_eula_url);
+  CallJSOrDefer("setOemEulaUrl", oem_eula_url);
 }
 
 void CoreOobeHandler::SetTpmPassword(const std::string& tpm_password) {
-  CallJS("setTpmPassword", tpm_password);
+  CallJSOrDefer("setTpmPassword", tpm_password);
 }
 
 void CoreOobeHandler::ClearErrors() {
-  CallJS("clearErrors");
+  CallJSOrDefer("clearErrors");
 }
 
 void CoreOobeHandler::ReloadContent(const base::DictionaryValue& dictionary) {
-  CallJS("reloadContent", dictionary);
+  CallJSOrDefer("reloadContent", dictionary);
 }
 
 void CoreOobeHandler::ShowControlBar(bool show) {
-  CallJS("showControlBar", show);
+  CallJSOrDefer("showControlBar", show);
+}
+
+void CoreOobeHandler::ShowPinKeyboard(bool show) {
+  CallJSOrDefer("showPinKeyboard", show);
 }
 
 void CoreOobeHandler::SetClientAreaSize(int width, int height) {
-  CallJS("setClientAreaSize", width, height);
+  CallJSOrDefer("setClientAreaSize", width, height);
 }
 
 void CoreOobeHandler::HandleInitialized() {
+  DCHECK(!is_initialized_);
+  is_initialized_ = true;
+  ExecuteDeferredJSCalls();
   oobe_ui_->InitializeHandlers();
 }
 
@@ -254,6 +307,11 @@ void CoreOobeHandler::HandleSkipUpdateEnrollAfterEula() {
 void CoreOobeHandler::HandleUpdateCurrentScreen(const std::string& screen) {
   if (delegate_)
     delegate_->OnCurrentScreenChanged(screen);
+  // TODO(mash): Support EventRewriterController; see crbug.com/647781
+  if (!chrome::IsRunningInMash()) {
+    KeyboardDrivenEventRewriter::GetInstance()->SetArrowToTabRewritingEnabled(
+      screen == WizardController::kEulaScreenName);
+  }
 }
 
 void CoreOobeHandler::HandleEnableHighContrast(bool enabled) {
@@ -268,6 +326,10 @@ void CoreOobeHandler::HandleEnableVirtualKeyboard(bool enabled) {
   AccessibilityManager::Get()->EnableVirtualKeyboard(enabled);
 }
 
+void CoreOobeHandler::HandleSetForceDisableVirtualKeyboard(bool disable) {
+  scoped_keyboard_disabler_.SetForceDisableVirtualKeyboard(disable);
+}
+
 void CoreOobeHandler::HandleEnableScreenMagnifier(bool enabled) {
   // TODO(nkostylev): Add support for partial screen magnifier.
   DCHECK(MagnificationManager::Get());
@@ -278,7 +340,7 @@ void CoreOobeHandler::HandleEnableSpokenFeedback(bool /* enabled */) {
   // Checkbox is initialized on page init and updates when spoken feedback
   // setting is changed so just toggle spoken feedback here.
   AccessibilityManager::Get()->ToggleSpokenFeedback(
-      ui::A11Y_NOTIFICATION_NONE);
+      ash::A11Y_NOTIFICATION_NONE);
 }
 
 void CoreOobeHandler::HandleSetDeviceRequisition(
@@ -327,10 +389,14 @@ void CoreOobeHandler::ShowOobeUI(bool show) {
 
 void CoreOobeHandler::UpdateShutdownAndRebootVisibility(
     bool reboot_on_shutdown) {
-  CallJS("showShutdown", !reboot_on_shutdown);
+  CallJSOrDefer("showShutdown", !reboot_on_shutdown);
 }
 
 void CoreOobeHandler::UpdateA11yState() {
+  if (chrome::IsRunningInMash()) {
+    NOTIMPLEMENTED();
+    return;
+  }
   // TODO(dpolukhin): crbug.com/412891
   DCHECK(MagnificationManager::Get());
   base::DictionaryValue a11y_info;
@@ -344,12 +410,12 @@ void CoreOobeHandler::UpdateA11yState() {
                        MagnificationManager::Get()->IsMagnifierEnabled());
   a11y_info.SetBoolean("virtualKeyboardEnabled",
                        AccessibilityManager::Get()->IsVirtualKeyboardEnabled());
-  CallJS("refreshA11yInfo", a11y_info);
+  CallJSOrDefer("refreshA11yInfo", a11y_info);
 }
 
 void CoreOobeHandler::UpdateOobeUIVisibility() {
   const std::string& display = oobe_ui_->display_type();
-  CallJS("showAPIKeysNotice", !google_apis::HasKeysConfigured() &&
+  CallJSOrDefer("showAPIKeysNotice", !google_apis::HasKeysConfigured() &&
                                   (display == OobeUI::kOobeDisplay ||
                                    display == OobeUI::kLoginDisplay));
 
@@ -360,10 +426,10 @@ void CoreOobeHandler::UpdateOobeUIVisibility() {
       channel == version_info::Channel::BETA) {
     should_show_version = false;
   }
-  CallJS("showVersion", should_show_version);
-  CallJS("showOobeUI", show_oobe_ui_);
+  CallJSOrDefer("showVersion", should_show_version);
+  CallJSOrDefer("showOobeUI", show_oobe_ui_);
   if (system::InputDeviceSettings::Get()->ForceKeyboardDrivenUINavigation())
-    CallJS("enableKeyboardFlow", true);
+    CallJSOrDefer("enableKeyboardFlow", true);
 }
 
 void CoreOobeHandler::OnOSVersionLabelTextUpdated(
@@ -373,18 +439,22 @@ void CoreOobeHandler::OnOSVersionLabelTextUpdated(
 
 void CoreOobeHandler::OnEnterpriseInfoUpdated(
     const std::string& message_text, const std::string& asset_id) {
-  CallJS("setEnterpriseInfo", message_text, asset_id);
+  CallJSOrDefer("setEnterpriseInfo", message_text, asset_id);
+}
+
+ui::EventProcessor* CoreOobeHandler::GetEventProcessor() {
+  return ash::Shell::GetPrimaryRootWindow()->GetHost()->event_processor();
 }
 
 void CoreOobeHandler::UpdateLabel(const std::string& id,
                                   const std::string& text) {
-  CallJS("setLabelText", id, text);
+  CallJSOrDefer("setLabelText", id, text);
 }
 
 void CoreOobeHandler::UpdateDeviceRequisition() {
   policy::BrowserPolicyConnectorChromeOS* connector =
       g_browser_process->platform_part()->browser_policy_connector_chromeos();
-  CallJS("updateDeviceRequisition",
+  CallJSOrDefer("updateDeviceRequisition",
          connector->GetDeviceCloudPolicyManager()->GetDeviceRequisition());
 }
 
@@ -394,12 +464,13 @@ void CoreOobeHandler::UpdateKeyboardState() {
   if (keyboard_controller) {
     gfx::Rect bounds = keyboard_controller->current_keyboard_bounds();
     ShowControlBar(bounds.IsEmpty());
+    ShowPinKeyboard(bounds.IsEmpty());
   }
 }
 
 void CoreOobeHandler::UpdateClientAreaSize() {
   const gfx::Size& size =
-      gfx::Screen::GetNativeScreen()->GetPrimaryDisplay().size();
+      display::Screen::GetScreen()->GetPrimaryDisplay().size();
   SetClientAreaSize(size.width(), size.height());
 }
 
@@ -419,11 +490,28 @@ void CoreOobeHandler::HandleLaunchHelpApp(double help_topic_id) {
 }
 
 void CoreOobeHandler::HandleHeaderBarVisible() {
-  LoginDisplayHost* login_display_host = LoginDisplayHostImpl::default_host();
+  LoginDisplayHost* login_display_host = LoginDisplayHost::default_host();
   if (login_display_host)
     login_display_host->SetStatusAreaVisible(true);
   if (ScreenLocker::default_screen_locker())
-    ScreenLocker::default_screen_locker()->delegate()->OnHeaderBarVisible();
+    ScreenLocker::default_screen_locker()->web_ui()->OnHeaderBarVisible();
+}
+
+void CoreOobeHandler::HandleRaiseTabKeyEvent(bool reverse) {
+  ui::KeyEvent event(ui::ET_KEY_PRESSED, ui::VKEY_TAB, ui::EF_NONE);
+  if (reverse)
+    event.set_flags(ui::EF_SHIFT_DOWN);
+  SendEventToProcessor(&event);
+}
+
+void CoreOobeHandler::HandleSetOobeBootstrappingSlave() {
+  const bool is_slave = g_browser_process->local_state()->GetBoolean(
+      prefs::kIsBootstrappingSlave);
+  if (is_slave)
+    return;
+  g_browser_process->local_state()->SetBoolean(prefs::kIsBootstrappingSlave,
+                                               true);
+  chrome::AttemptRestart();
 }
 
 void CoreOobeHandler::InitDemoModeDetection() {
