@@ -4,12 +4,13 @@
 
 #include "chrome/browser/ui/webui/chromeos/login/gaia_screen_handler.h"
 
-#include "ash/system/chromeos/devicetype_utils.h"
+#include "ash/common/system/chromeos/devicetype_utils.h"
 #include "base/bind.h"
+#include "base/callback.h"
 #include "base/guid.h"
 #include "base/logging.h"
-#include "base/metrics/histogram.h"
-#include "base/prefs/pref_service.h"
+#include "base/metrics/histogram_macros.h"
+#include "base/stl_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/browser/browser_process.h"
@@ -21,11 +22,12 @@
 #include "chrome/browser/chromeos/login/users/chrome_user_manager.h"
 #include "chrome/browser/chromeos/net/network_portal_detector_impl.h"
 #include "chrome/browser/chromeos/policy/browser_policy_connector_chromeos.h"
+#include "chrome/browser/chromeos/policy/proto/chrome_device_policy.pb.h"
 #include "chrome/browser/chromeos/profiles/profile_helper.h"
 #include "chrome/browser/chromeos/settings/cros_settings.h"
 #include "chrome/browser/io_thread.h"
 #include "chrome/browser/ui/webui/chromeos/login/signin_screen_handler.h"
-#include "chrome/browser/ui/webui/signin/inline_login_ui.h"
+#include "chrome/browser/ui/webui/signin/signin_utils.h"
 #include "chrome/common/channel_info.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/grit/generated_resources.h"
@@ -35,6 +37,8 @@
 #include "chromeos/system/devicetype.h"
 #include "chromeos/system/version_loader.h"
 #include "components/login/localized_values_builder.h"
+#include "components/prefs/pref_service.h"
+#include "components/strings/grit/components_strings.h"
 #include "components/user_manager/known_user.h"
 #include "components/user_manager/user_manager.h"
 #include "components/version_info/version_info.h"
@@ -42,11 +46,10 @@
 #include "content/public/browser/render_frame_host.h"
 #include "google_apis/gaia/gaia_auth_util.h"
 #include "google_apis/gaia/gaia_urls.h"
-#include "grit/components_strings.h"
 #include "ui/base/ime/chromeos/input_method_manager.h"
-#include "ui/base/l10n/l10n_util.h"
 
 using content::BrowserThread;
+namespace em = enterprise_management;
 
 namespace chromeos {
 
@@ -54,13 +57,54 @@ namespace {
 
 const char kJsScreenPath[] = "login.GaiaSigninScreen";
 const char kAuthIframeParentName[] = "signin-frame";
-const char kAuthIframeParentOrigin[] =
-    "chrome-extension://mfffpogegjflfpflabcdkioaeobkgjik/";
 
 const char kRestrictiveProxyURL[] = "https://www.google.com/generate_204";
 
 const char kEndpointGen[] = "1.0";
 
+// The possible modes that the Gaia signin screen can be in.
+enum GaiaScreenMode {
+  // Default Gaia authentication will be used.
+  GAIA_SCREEN_MODE_DEFAULT = 0,
+
+  // Gaia offline mode will be used.
+  GAIA_SCREEN_MODE_OFFLINE = 1,
+
+  // An interstitial page will be used before SAML redirection.
+  GAIA_SCREEN_MODE_SAML_INTERSTITIAL = 2,
+};
+
+GaiaScreenMode GetGaiaScreenMode(const std::string& email, bool use_offline) {
+  if (use_offline)
+    return GAIA_SCREEN_MODE_OFFLINE;
+
+  int authentication_behavior = 0;
+  CrosSettings::Get()->GetInteger(kLoginAuthenticationBehavior,
+                                  &authentication_behavior);
+  if (authentication_behavior ==
+      em::LoginAuthenticationBehaviorProto::SAML_INTERSTITIAL) {
+    if (email.empty())
+      return GAIA_SCREEN_MODE_SAML_INTERSTITIAL;
+
+    // If there's a populated email, we must check first that this user is using
+    // SAML in order to decide whether to show the interstitial page.
+    const user_manager::User* user =
+        user_manager::UserManager::Get()->FindUser(
+            user_manager::known_user::GetAccountId(email, std::string()));
+
+    if (user && user->using_saml())
+      return GAIA_SCREEN_MODE_SAML_INTERSTITIAL;
+  }
+
+  return GAIA_SCREEN_MODE_DEFAULT;
+}
+
+std::string GetEnterpriseDomain() {
+  policy::BrowserPolicyConnectorChromeOS* connector =
+      g_browser_process->platform_part()->browser_policy_connector_chromeos();
+  return connector->GetEnterpriseDomain();
+}
+
 std::string GetChromeType() {
   switch (chromeos::GetDeviceType()) {
     case chromeos::DeviceType::kChromebox:
@@ -119,7 +163,7 @@ void ClearDnsCache(IOThread* io_thread) {
   if (browser_shutdown::IsTryingToQuit())
     return;
 
-  io_thread->ClearHostCache();
+  io_thread->ClearHostCache(base::Callback<bool(const std::string&)>());
 }
 
 void PushFrontIMIfNotExists(const std::string& input_method,
@@ -127,8 +171,7 @@ void PushFrontIMIfNotExists(const std::string& input_method,
   if (input_method.empty())
     return;
 
-  if (std::find(input_methods->begin(), input_methods->end(), input_method) ==
-      input_methods->end())
+  if (!base::ContainsValue(*input_methods, input_method))
     input_methods->insert(input_methods->begin(), input_method);
 }
 
@@ -189,7 +232,6 @@ void GaiaScreenHandler::LoadGaiaWithVersion(
   base::DictionaryValue params;
 
   params.SetBoolean("forceReload", context.force_reload);
-  params.SetBoolean("useOffline", context.use_offline);
   params.SetString("gaiaId", context.gaia_id);
   params.SetBoolean("readOnlyEmail", true);
   params.SetString("email", context.email);
@@ -197,27 +239,16 @@ void GaiaScreenHandler::LoadGaiaWithVersion(
 
   UpdateAuthParams(&params, IsRestrictiveProxy());
 
-  if (!context.use_offline) {
+  GaiaScreenMode screen_mode = GetGaiaScreenMode(context.email,
+                                                 context.use_offline);
+  params.SetInteger("screenMode", screen_mode);
+  if (screen_mode != GAIA_SCREEN_MODE_OFFLINE) {
     const std::string app_locale = g_browser_process->GetApplicationLocale();
     if (!app_locale.empty())
       params.SetString("hl", app_locale);
-  } else {
-    policy::BrowserPolicyConnectorChromeOS* connector =
-        g_browser_process->platform_part()->browser_policy_connector_chromeos();
-    std::string enterprise_domain(connector->GetEnterpriseDomain());
-    if (!enterprise_domain.empty()) {
-      params.SetString(
-          "enterpriseInfoMessage",
-          l10n_util::GetStringFUTF16(IDS_OFFLINE_LOGIN_DEVICE_MANAGED_BY_NOTICE,
-                                     base::UTF8ToUTF16(enterprise_domain)));
-    }
   }
 
-  base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
-
-  policy::BrowserPolicyConnectorChromeOS* connector =
-      g_browser_process->platform_part()->browser_policy_connector_chromeos();
-  std::string enterprise_domain(connector->GetEnterpriseDomain());
+  std::string enterprise_domain(GetEnterpriseDomain());
   if (!enterprise_domain.empty())
     params.SetString("enterpriseDomain", enterprise_domain);
 
@@ -243,6 +274,7 @@ void GaiaScreenHandler::LoadGaiaWithVersion(
     params.SetBoolean("useEafe", true);
     // Easy login overrides.
     std::string eafe_url = "https://easylogin.corp.google.com/";
+    base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
     if (command_line->HasSwitch(switches::kEafeUrl))
       eafe_url = command_line->GetSwitchValueASCII(switches::kEafeUrl);
     std::string eafe_path = "planters/cbaudioChrome";
@@ -251,8 +283,6 @@ void GaiaScreenHandler::LoadGaiaWithVersion(
 
     params.SetString("gaiaUrl", eafe_url);
     params.SetString("gaiaPath", eafe_path);
-    params.SetString("clientId",
-                     GaiaUrls::GetInstance()->oauth2_chrome_client_id());
   }
 
   frame_state_ = FRAME_STATE_LOADING;
@@ -265,14 +295,17 @@ void GaiaScreenHandler::ReloadGaia(bool force_reload) {
     return;
   }
   NetworkStateInformer::State state = network_state_informer_->state();
-  if (state != NetworkStateInformer::ONLINE) {
+  if (state != NetworkStateInformer::ONLINE &&
+      !signin_screen_handler_->proxy_auth_dialog_need_reload_) {
     VLOG(1) << "Skipping reloading of Gaia since network state="
             << NetworkStateInformer::StatusString(state);
     return;
   }
+
+  signin_screen_handler_->proxy_auth_dialog_need_reload_ = false;
   VLOG(1) << "Reloading Gaia.";
   frame_state_ = FRAME_STATE_LOADING;
-  CallJS("doReload");
+  LoadAuthExtension(force_reload, false /* offline */);
 }
 
 void GaiaScreenHandler::MonitorOfflineIdle(bool is_online) {
@@ -305,7 +338,7 @@ void GaiaScreenHandler::DeclareLocalizedValues(
   builder->Add("fatalErrorTryAgainButton",
                IDS_LOGIN_FATAL_ERROR_TRY_AGAIN_BUTTON);
 
-  builder->AddF("offlineLoginWelcome", IDS_OFFLINE_LOGIN_WELCOME,
+  builder->AddF("loginWelcomeMessage", IDS_LOGIN_WELCOME_MESSAGE,
                 ash::GetChromeOSDeviceTypeResourceId());
   builder->Add("offlineLoginEmail", IDS_OFFLINE_LOGIN_EMAIL);
   builder->Add("offlineLoginPassword", IDS_OFFLINE_LOGIN_PASSWORD);
@@ -318,6 +351,13 @@ void GaiaScreenHandler::DeclareLocalizedValues(
   builder->Add("offlineLoginForgotPasswordDlg",
                IDS_OFFLINE_LOGIN_FORGOT_PASSWORD_DIALOG_TEXT);
   builder->Add("offlineLoginCloseBtn", IDS_OFFLINE_LOGIN_CLOSE_BUTTON_TEXT);
+  builder->Add("enterpriseInfoMessage", IDS_LOGIN_DEVICE_MANAGED_BY_NOTICE);
+  builder->Add("samlInterstitialMessage",
+                IDS_LOGIN_SAML_INTERSTITIAL_MESSAGE);
+  builder->Add("samlInterstitialChangeAccountLink",
+               IDS_LOGIN_SAML_INTERSTITIAL_CHANGE_ACCOUNT_LINK_TEXT);
+  builder->Add("samlInterstitialNextBtn",
+               IDS_LOGIN_SAML_INTERSTITIAL_NEXT_BUTTON_TEXT);
 }
 
 void GaiaScreenHandler::Initialize() {
@@ -342,14 +382,8 @@ void GaiaScreenHandler::RegisterMessages() {
   AddCallback("identifierEntered", &GaiaScreenHandler::HandleIdentifierEntered);
   AddCallback("updateOfflineLogin",
               &GaiaScreenHandler::set_offline_login_is_active);
-}
-
-OobeUI::Screen GaiaScreenHandler::GetCurrentScreen() const {
-  OobeUI::Screen screen = OobeUI::SCREEN_UNKNOWN;
-  OobeUI* oobe_ui = static_cast<OobeUI*>(web_ui()->GetController());
-  if (oobe_ui)
-    screen = oobe_ui->current_screen();
-  return screen;
+  AddCallback("authExtensionLoaded",
+              &GaiaScreenHandler::HandleAuthExtensionLoaded);
 }
 
 void GaiaScreenHandler::OnPortalDetectionCompleted(
@@ -364,7 +398,7 @@ void GaiaScreenHandler::OnPortalDetectionCompleted(
   if (offline_login_is_active() ||
       IsOnline(captive_portal_status_) == IsOnline(previous_status) ||
       disable_restrictive_proxy_check_for_test_ ||
-      GetCurrentScreen() != OobeUI::SCREEN_GAIA_SIGNIN)
+      GetCurrentScreen() != OobeScreen::SCREEN_GAIA_SIGNIN)
     return;
 
   LoadAuthExtension(true /* force */, false /* offline */);
@@ -376,6 +410,11 @@ void GaiaScreenHandler::HandleIdentifierEntered(const std::string& user_email) {
     ShowWhitelistCheckFailedError();
 }
 
+void GaiaScreenHandler::HandleAuthExtensionLoaded() {
+  VLOG(1) << "Auth extension finished loading";
+  auth_extension_being_loaded_ = false;
+}
+
 void GaiaScreenHandler::HandleWebviewLoadAborted(
     const std::string& error_reason_str) {
   // TODO(nkostylev): Switch to int code once webview supports that.
@@ -443,7 +482,6 @@ void GaiaScreenHandler::HandleCompleteAuthentication(
   Delegate()->SetDisplayEmail(sanitized_email);
 
   UserContext user_context(GetAccountId(email, gaia_id));
-  user_context.SetGaiaID(gaia_id);
   user_context.SetKey(Key(password));
   user_context.SetAuthCode(auth_code);
   user_context.SetAuthFlow(using_saml
@@ -510,6 +548,8 @@ void GaiaScreenHandler::HandleGaiaUIReady() {
 
   if (test_expects_complete_login_)
     SubmitLoginFormForTest();
+  if (Delegate())
+    Delegate()->OnGaiaScreenReady();
 }
 
 void GaiaScreenHandler::DoCompleteLogin(const std::string& gaia_id,
@@ -527,7 +567,6 @@ void GaiaScreenHandler::DoCompleteLogin(const std::string& gaia_id,
   const std::string sanitized_email = gaia::SanitizeEmail(typed_email);
   Delegate()->SetDisplayEmail(sanitized_email);
   UserContext user_context(GetAccountId(typed_email, gaia_id));
-  user_context.SetGaiaID(gaia_id);
   user_context.SetKey(Key(password));
   user_context.SetAuthFlow(using_saml
                                ? UserContext::AUTH_FLOW_GAIA_WITH_SAML
@@ -604,10 +643,8 @@ void GaiaScreenHandler::ShowSigninScreenForTest(const std::string& username,
 void GaiaScreenHandler::SubmitLoginFormForTest() {
   VLOG(2) << "Submit login form for test, user=" << test_user_;
 
-  content::RenderFrameHost* frame = InlineLoginUI::GetAuthFrame(
-      web_ui()->GetWebContents(),
-      GURL(kAuthIframeParentOrigin),
-      kAuthIframeParentName);
+  content::RenderFrameHost* frame =
+      signin::GetAuthFrame(web_ui()->GetWebContents(), kAuthIframeParentName);
 
   std::string code =
       "document.getElementById('identifier').value = '" + test_user_ + "';"
@@ -706,6 +743,7 @@ void GaiaScreenHandler::ShowGaiaScreenIfReady() {
   LoadAuthExtension(!gaia_silent_load_ /* force */, false /* offline */);
   signin_screen_handler_->UpdateUIState(
       SigninScreenHandler::UI_STATE_GAIA_SIGNIN, nullptr);
+  core_oobe_actor_->UpdateKeyboardState();
 
   if (gaia_silent_load_) {
     // The variable is assigned to false because silently loaded Gaia page was
@@ -762,6 +800,13 @@ void GaiaScreenHandler::LoadAuthExtension(bool force,
                                           bool offline) {
   VLOG(1) << "LoadAuthExtension, force: " << force
           << ", offline: " << offline;
+
+  if (auth_extension_being_loaded_) {
+    VLOG(1) << "Skip loading the Auth extension as it's already being loaded";
+    return;
+  }
+
+  auth_extension_being_loaded_ = true;
   GaiaContext context;
   context.force_reload = force;
   context.use_offline = offline;
