@@ -6,12 +6,13 @@
 
 #include <stddef.h>
 
-#include "ash/shell_window_ids.h"
-#include "ash/wm/screen_dimmer.h"
+#include <memory>
+
+#include "ash/common/wm/screen_dimmer.h"
 #include "base/command_line.h"
 #include "base/logging.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/values.h"
 #include "chrome/browser/browser_process.h"
@@ -21,7 +22,6 @@
 #include "chrome/browser/chromeos/login/screens/error_screen.h"
 #include "chrome/browser/chromeos/login/startup_utils.h"
 #include "chrome/browser/chromeos/login/ui/login_display_host.h"
-#include "chrome/browser/chromeos/login/ui/login_display_host_impl.h"
 #include "chrome/browser/chromeos/login/wizard_controller.h"
 #include "chrome/browser/chromeos/policy/browser_policy_connector_chromeos.h"
 #include "chrome/browser/chromeos/settings/shutdown_policy_handler.h"
@@ -29,8 +29,10 @@
 #include "chrome/browser/extensions/signin/gaia_auth_extension_loader.h"
 #include "chrome/browser/extensions/tab_helper.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/ui/ash/ash_util.h"
 #include "chrome/browser/ui/webui/about_ui.h"
 #include "chrome/browser/ui/webui/chromeos/login/app_launch_splash_screen_handler.h"
+#include "chrome/browser/ui/webui/chromeos/login/arc_terms_of_service_screen_handler.h"
 #include "chrome/browser/ui/webui/chromeos/login/auto_enrollment_check_screen_handler.h"
 #include "chrome/browser/ui/webui/chromeos/login/base_screen_handler.h"
 #include "chrome/browser/ui/webui/chromeos/login/controller_pairing_screen_handler.h"
@@ -57,15 +59,22 @@
 #include "chrome/browser/ui/webui/chromeos/login/user_image_screen_handler.h"
 #include "chrome/browser/ui/webui/chromeos/login/wrong_hwid_screen_handler.h"
 #include "chrome/browser/ui/webui/options/chromeos/user_image_source.h"
+#include "chrome/browser/ui/webui/settings/md_settings_localized_strings_provider.h"
+#include "chrome/browser/ui/webui/test_files_request_filter.h"
 #include "chrome/browser/ui/webui/theme_source.h"
 #include "chrome/common/chrome_constants.h"
+#include "chrome/common/chrome_switches.h"
+#include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
+#include "chrome/grit/browser_resources.h"
+#include "chrome/grit/chrome_unscaled_resources.h"
+#include "chrome/grit/component_extension_resources.h"
 #include "chromeos/chromeos_switches.h"
 #include "components/policy/core/common/cloud/cloud_policy_constants.h"
+#include "components/prefs/pref_service.h"
 #include "content/public/browser/web_ui.h"
 #include "content/public/browser/web_ui_data_source.h"
-#include "grit/browser_resources.h"
-#include "grit/chrome_unscaled_resources.h"
+#include "content/public/common/content_switches.h"
 #include "ui/base/resource/resource_bundle.h"
 #include "ui/base/webui/web_ui_util.h"
 
@@ -81,17 +90,33 @@ const char* kKnownDisplayTypes[] = {
   OobeUI::kAppLaunchSplashDisplay
 };
 
+OobeScreen kDimOverlayScreenIds[] = {
+  OobeScreen::SCREEN_CONFIRM_PASSWORD,
+  OobeScreen::SCREEN_GAIA_SIGNIN,
+  OobeScreen::SCREEN_OOBE_ENROLLMENT,
+  OobeScreen::SCREEN_PASSWORD_CHANGED,
+  OobeScreen::SCREEN_USER_IMAGE_PICKER
+};
+
 const char kStringsJSPath[] = "strings.js";
+const char kLockJSPath[] = "lock.js";
 const char kLoginJSPath[] = "login.js";
 const char kOobeJSPath[] = "oobe.js";
 const char kKeyboardUtilsJSPath[] = "keyboard_utils.js";
 const char kCustomElementsHTMLPath[] = "custom_elements.html";
 const char kCustomElementsJSPath[] = "custom_elements.js";
+const char kCustomElementsUserPodHTMLPath[] = "custom_elements_user_pod.html";
 
 // Paths for deferred resource loading.
+const char kCustomElementsPinKeyboardHTMLPath[] =
+    "custom_elements/pin_keyboard.html";
+const char kCustomElementsPinKeyboardJSPath[] =
+    "custom_elements/pin_keyboard.js";
 const char kEnrollmentHTMLPath[] = "enrollment.html";
 const char kEnrollmentCSSPath[] = "enrollment.css";
 const char kEnrollmentJSPath[] = "enrollment.js";
+const char kArcPlaystoreCSSPath[] = "playstore.css";
+const char kArcPlaystoreJSPath[] = "playstore.js";
 
 // Creates a WebUIDataSource for chrome://oobe
 content::WebUIDataSource* CreateOobeUIDataSource(
@@ -108,6 +133,18 @@ content::WebUIDataSource* CreateOobeUIDataSource(
     source->AddResourcePath(kCustomElementsHTMLPath,
                             IDR_CUSTOM_ELEMENTS_OOBE_HTML);
     source->AddResourcePath(kCustomElementsJSPath, IDR_CUSTOM_ELEMENTS_OOBE_JS);
+  } else if (display_type == OobeUI::kLockDisplay) {
+    source->SetDefaultResource(IDR_LOCK_HTML);
+    source->AddResourcePath(kLockJSPath, IDR_LOCK_JS);
+    source->AddResourcePath(kCustomElementsHTMLPath,
+                            IDR_CUSTOM_ELEMENTS_LOCK_HTML);
+    source->AddResourcePath(kCustomElementsJSPath, IDR_CUSTOM_ELEMENTS_LOCK_JS);
+    source->AddResourcePath(kCustomElementsPinKeyboardHTMLPath,
+                            IDR_CUSTOM_ELEMENTS_PIN_KEYBOARD_HTML);
+    source->AddResourcePath(kCustomElementsPinKeyboardJSPath,
+                            IDR_CUSTOM_ELEMENTS_PIN_KEYBOARD_JS);
+    source->AddResourcePath(kCustomElementsUserPodHTMLPath,
+                            IDR_CUSTOM_ELEMENTS_USER_POD_HTML);
   } else {
     source->SetDefaultResource(IDR_LOGIN_HTML);
     source->AddResourcePath(kLoginJSPath, IDR_LOGIN_JS);
@@ -115,13 +152,21 @@ content::WebUIDataSource* CreateOobeUIDataSource(
                             IDR_CUSTOM_ELEMENTS_LOGIN_HTML);
     source->AddResourcePath(kCustomElementsJSPath,
                             IDR_CUSTOM_ELEMENTS_LOGIN_JS);
+    source->AddResourcePath(kCustomElementsUserPodHTMLPath,
+                            IDR_CUSTOM_ELEMENTS_USER_POD_HTML);
   }
+
+  // Required for postprocessing of Goolge PlayStore Terms.
+  source->AddResourcePath(kArcPlaystoreCSSPath, IDR_ARC_SUPPORT_PLAYSTORE_CSS);
+  source->AddResourcePath(kArcPlaystoreJSPath, IDR_ARC_SUPPORT_PLAYSTORE_JS);
+
   source->AddResourcePath(kKeyboardUtilsJSPath, IDR_KEYBOARD_UTILS_JS);
-  source->OverrideContentSecurityPolicyFrameSrc(
+  source->OverrideContentSecurityPolicyChildSrc(
       base::StringPrintf(
-          "frame-src chrome://terms/ %s/;",
+          "child-src chrome://terms/ %s/;",
           extensions::kGaiaAuthExtensionOrigin));
-  source->OverrideContentSecurityPolicyObjectSrc("object-src *;");
+  source->OverrideContentSecurityPolicyObjectSrc(
+      "object-src chrome:;");
   source->AddResourcePath("gaia_auth_host.js",
                           StartupUtils::IsWebviewSigninEnabled()
                               ? IDR_GAIA_AUTH_AUTHENTICATOR_JS
@@ -132,13 +177,12 @@ content::WebUIDataSource* CreateOobeUIDataSource(
   source->AddResourcePath(kEnrollmentCSSPath, IDR_OOBE_ENROLLMENT_CSS);
   source->AddResourcePath(kEnrollmentJSPath, IDR_OOBE_ENROLLMENT_JS);
 
-  if (display_type == OobeUI::kOobeDisplay) {
-    source->AddResourcePath("Roboto-Thin.ttf", IDR_FONT_ROBOTO_THIN);
-    source->AddResourcePath("Roboto-Light.ttf", IDR_FONT_ROBOTO_LIGHT);
-    source->AddResourcePath("Roboto-Regular.ttf", IDR_FONT_ROBOTO_REGULAR);
-    source->AddResourcePath("Roboto-Medium.ttf", IDR_FONT_ROBOTO_MEDIUM);
-    source->AddResourcePath("Roboto-Bold.ttf", IDR_FONT_ROBOTO_BOLD);
-  }
+  // Only add a filter when runing as test.
+  base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
+  const bool is_running_test = command_line->HasSwitch(::switches::kTestName) ||
+                               command_line->HasSwitch(::switches::kTestType);
+  if (is_running_test)
+    source->SetRequestFilter(::test::GetTestFilesRequestFilter());
 
   return source;
 }
@@ -163,64 +207,9 @@ const char OobeUI::kLockDisplay[] = "lock";
 const char OobeUI::kUserAddingDisplay[] = "user-adding";
 const char OobeUI::kAppLaunchSplashDisplay[] = "app-launch-splash";
 
-// static
-const char OobeUI::kScreenOobeHIDDetection[] = "hid-detection";
-const char OobeUI::kScreenOobeNetwork[] = "connect";
-const char OobeUI::kScreenOobeEnableDebugging[] = "debugging";
-const char OobeUI::kScreenOobeEula[] = "eula";
-const char OobeUI::kScreenOobeUpdate[] = "update";
-const char OobeUI::kScreenOobeEnrollment[] = "oauth-enrollment";
-const char OobeUI::kScreenOobeReset[] = "reset";
-const char OobeUI::kScreenGaiaSignin[] = "gaia-signin";
-const char OobeUI::kScreenAccountPicker[] = "account-picker";
-const char OobeUI::kScreenKioskAutolaunch[] = "autolaunch";
-const char OobeUI::kScreenKioskEnable[] = "kiosk-enable";
-const char OobeUI::kScreenErrorMessage[] = "error-message";
-const char OobeUI::kScreenUserImagePicker[] = "user-image";
-const char OobeUI::kScreenTpmError[] = "tpm-error-message";
-const char OobeUI::kScreenPasswordChanged[] = "password-changed";
-const char OobeUI::kScreenSupervisedUserCreationFlow[] =
-    "supervised-user-creation";
-const char OobeUI::kScreenTermsOfService[] = "terms-of-service";
-const char OobeUI::kScreenWrongHWID[] = "wrong-hwid";
-const char OobeUI::kScreenAutoEnrollmentCheck[] = "auto-enrollment-check";
-const char OobeUI::kScreenHIDDetection[] = "hid-detection";
-const char OobeUI::kScreenAppLaunchSplash[] = "app-launch-splash";
-const char OobeUI::kScreenConfirmPassword[] = "confirm-password";
-const char OobeUI::kScreenFatalError[] = "fatal-error";
-const char OobeUI::kScreenControllerPairing[] = "controller-pairing";
-const char OobeUI::kScreenHostPairing[] = "host-pairing";
-const char OobeUI::kScreenDeviceDisabled[] = "device-disabled";
-
 OobeUI::OobeUI(content::WebUI* web_ui, const GURL& url)
-    : WebUIController(web_ui),
-      core_handler_(nullptr),
-      network_dropdown_handler_(nullptr),
-      update_view_(nullptr),
-      network_view_(nullptr),
-      debugging_screen_actor_(nullptr),
-      eula_view_(nullptr),
-      reset_view_(nullptr),
-      hid_detection_view_(nullptr),
-      autolaunch_screen_actor_(nullptr),
-      kiosk_enable_screen_actor_(nullptr),
-      wrong_hwid_screen_actor_(nullptr),
-      auto_enrollment_check_screen_actor_(nullptr),
-      supervised_user_creation_screen_actor_(nullptr),
-      app_launch_splash_screen_actor_(nullptr),
-      controller_pairing_screen_actor_(nullptr),
-      host_pairing_screen_actor_(nullptr),
-      device_disabled_screen_actor_(nullptr),
-      error_screen_handler_(nullptr),
-      signin_screen_handler_(nullptr),
-      terms_of_service_screen_actor_(nullptr),
-      user_image_view_(nullptr),
-      kiosk_app_menu_handler_(nullptr),
-      current_screen_(SCREEN_UNKNOWN),
-      previous_screen_(SCREEN_UNKNOWN),
-      ready_(false) {
+    : WebUIController(web_ui) {
   display_type_ = GetDisplayType(url);
-  InitializeScreenMaps();
 
   network_state_informer_ = new NetworkStateInformer();
   network_state_informer_->Init();
@@ -304,6 +293,11 @@ OobeUI::OobeUI(content::WebUI* web_ui, const GURL& url)
   terms_of_service_screen_actor_ = terms_of_service_screen_handler;
   AddScreenHandler(terms_of_service_screen_handler);
 
+  ArcTermsOfServiceScreenHandler* arc_terms_of_service_screen_handler =
+      new ArcTermsOfServiceScreenHandler();
+  arc_terms_of_service_screen_actor_ = arc_terms_of_service_screen_handler;
+  AddScreenHandler(arc_terms_of_service_screen_handler);
+
   UserImageScreenHandler* user_image_screen_handler =
       new UserImageScreenHandler();
   user_image_view_ = user_image_screen_handler;
@@ -363,9 +357,10 @@ OobeUI::OobeUI(content::WebUI* web_ui, const GURL& url)
   content::URLDataSource::Add(profile, about_source);
 
   // Set up the chrome://oobe/ source.
-  content::WebUIDataSource::Add(profile,
-                                CreateOobeUIDataSource(localized_strings,
-                                                       display_type_));
+  content::WebUIDataSource* html_source =
+      CreateOobeUIDataSource(localized_strings, display_type_);
+  content::WebUIDataSource::Add(profile, html_source);
+  settings::AddCrNetworkStrings(html_source);
 
   // Set up the chrome://userimage/ source.
   options::UserImageSource* user_image_source =
@@ -380,9 +375,11 @@ OobeUI::OobeUI(content::WebUI* web_ui, const GURL& url)
 OobeUI::~OobeUI() {
   core_handler_->SetDelegate(nullptr);
   network_dropdown_handler_->RemoveObserver(error_screen_handler_);
-  ash::ScreenDimmer::GetForContainer(
-      ash::kShellWindowId_LockScreenContainersContainer)
-      ->SetDimming(false);
+  if (chrome::IsRunningInMash()) {
+    // TODO: Ash needs to expose screen dimming api. See
+    // http://crbug.com/646034.
+    NOTIMPLEMENTED();
+  }
 }
 
 CoreOobeActor* OobeUI::GetCoreOobeActor() {
@@ -425,6 +422,10 @@ TermsOfServiceScreenActor* OobeUI::GetTermsOfServiceScreenActor() {
   return terms_of_service_screen_actor_;
 }
 
+ArcTermsOfServiceScreenActor* OobeUI::GetArcTermsOfServiceScreenActor() {
+  return arc_terms_of_service_screen_actor_;
+}
+
 WrongHWIDScreenActor* OobeUI::GetWrongHWIDScreenActor() {
   return wrong_hwid_screen_actor_;
 }
@@ -510,46 +511,10 @@ void OobeUI::GetLocalizedStrings(base::DictionaryValue* localized_strings) {
 
   bool new_kiosk_ui = KioskAppMenuHandler::EnableNewKioskUI();
   localized_strings->SetString("newKioskUI", new_kiosk_ui ? "on" : "off");
-}
-
-void OobeUI::InitializeScreenMaps() {
-  screen_names_.resize(SCREEN_UNKNOWN);
-  screen_names_[SCREEN_OOBE_HID_DETECTION] = kScreenOobeHIDDetection;
-  screen_names_[SCREEN_OOBE_NETWORK] = kScreenOobeNetwork;
-  screen_names_[SCREEN_OOBE_EULA] = kScreenOobeEula;
-  screen_names_[SCREEN_OOBE_UPDATE] = kScreenOobeUpdate;
-  screen_names_[SCREEN_OOBE_ENROLLMENT] = kScreenOobeEnrollment;
-  screen_names_[SCREEN_OOBE_ENABLE_DEBUGGING] = kScreenOobeEnableDebugging;
-  screen_names_[SCREEN_OOBE_RESET] = kScreenOobeReset;
-  screen_names_[SCREEN_GAIA_SIGNIN] = kScreenGaiaSignin;
-  screen_names_[SCREEN_ACCOUNT_PICKER] = kScreenAccountPicker;
-  screen_names_[SCREEN_KIOSK_AUTOLAUNCH] = kScreenKioskAutolaunch;
-  screen_names_[SCREEN_KIOSK_ENABLE] = kScreenKioskEnable;
-  screen_names_[SCREEN_ERROR_MESSAGE] = kScreenErrorMessage;
-  screen_names_[SCREEN_USER_IMAGE_PICKER] = kScreenUserImagePicker;
-  screen_names_[SCREEN_TPM_ERROR] = kScreenTpmError;
-  screen_names_[SCREEN_PASSWORD_CHANGED] = kScreenPasswordChanged;
-  screen_names_[SCREEN_CREATE_SUPERVISED_USER_FLOW] =
-      kScreenSupervisedUserCreationFlow;
-  screen_names_[SCREEN_TERMS_OF_SERVICE] = kScreenTermsOfService;
-  screen_names_[SCREEN_WRONG_HWID] = kScreenWrongHWID;
-  screen_names_[SCREEN_AUTO_ENROLLMENT_CHECK] = kScreenAutoEnrollmentCheck;
-  screen_names_[SCREEN_APP_LAUNCH_SPLASH] = kScreenAppLaunchSplash;
-  screen_names_[SCREEN_CONFIRM_PASSWORD] = kScreenConfirmPassword;
-  screen_names_[SCREEN_FATAL_ERROR] = kScreenFatalError;
-  screen_names_[SCREEN_OOBE_CONTROLLER_PAIRING] = kScreenControllerPairing;
-  screen_names_[SCREEN_OOBE_HOST_PAIRING] = kScreenHostPairing;
-  screen_names_[SCREEN_DEVICE_DISABLED] = kScreenDeviceDisabled;
-
-  dim_overlay_screen_ids_.push_back(SCREEN_CONFIRM_PASSWORD);
-  dim_overlay_screen_ids_.push_back(SCREEN_GAIA_SIGNIN);
-  dim_overlay_screen_ids_.push_back(SCREEN_OOBE_ENROLLMENT);
-  dim_overlay_screen_ids_.push_back(SCREEN_PASSWORD_CHANGED);
-  dim_overlay_screen_ids_.push_back(SCREEN_USER_IMAGE_PICKER);
-
-  screen_ids_.clear();
-  for (size_t i = 0; i < screen_names_.size(); ++i)
-    screen_ids_[screen_names_[i]] = static_cast<Screen>(i);
+  localized_strings->SetString(
+      "newOobeUI",
+      g_browser_process->local_state()->GetBoolean(prefs::kOobeMdMode) ? "on"
+                                                                       : "off");
 }
 
 void OobeUI::AddScreenHandler(BaseScreenHandler* handler) {
@@ -574,8 +539,7 @@ void OobeUI::InitializeHandlers() {
       new ShutdownPolicyHandler(CrosSettings::Get(), this));
 
   // Trigger an initial update.
-  shutdown_policy_handler_->CheckIfRebootOnShutdown(
-      base::Bind(&OobeUI::OnShutdownPolicyChanged, base::Unretained(this)));
+  shutdown_policy_handler_->NotifyDelegateWithShutdownPolicy();
 }
 
 void OobeUI::OnScreenAssetsLoaded(const std::string& async_assets_load_id) {
@@ -606,8 +570,7 @@ void OobeUI::ShowSigninScreen(const LoginScreenContext& context,
   if (connector->GetDeviceMode() == policy::DEVICE_MODE_LEGACY_RETAIL_MODE) {
     // If we're in legacy retail mode, the best thing we can do is launch the
     // new offline demo mode.
-    LoginDisplayHost* host = LoginDisplayHostImpl::default_host();
-    host->StartDemoAppLaunch();
+    LoginDisplayHost::default_host()->StartDemoAppLaunch();
     return;
   }
 
@@ -633,29 +596,30 @@ void OobeUI::RemoveObserver(Observer* observer) {
   observer_list_.RemoveObserver(observer);
 }
 
-const std::string& OobeUI::GetScreenName(Screen screen) const {
-  DCHECK(screen >= 0 && screen < SCREEN_UNKNOWN);
-  return screen_names_[static_cast<size_t>(screen)];
-}
-
 void OobeUI::OnCurrentScreenChanged(const std::string& screen) {
   previous_screen_ = current_screen_;
-  DCHECK(screen_ids_.count(screen))
-      << "Screen should be registered in InitializeScreenMaps()";
-  Screen new_screen = screen_ids_[screen];
+  OobeScreen new_screen = GetOobeScreenFromName(screen);
 
-  bool should_dim =
-      std::find(dim_overlay_screen_ids_.begin(), dim_overlay_screen_ids_.end(),
-                new_screen) != dim_overlay_screen_ids_.end();
-  ash::ScreenDimmer* screen_dimmer = ash::ScreenDimmer::GetForContainer(
-      ash::kShellWindowId_LockScreenContainersContainer);
-  screen_dimmer->set_at_bottom(true);
-  screen_dimmer->SetDimming(should_dim);
+  const bool should_dim =
+      std::find(std::begin(kDimOverlayScreenIds),
+                std::end(kDimOverlayScreenIds),
+                new_screen) != std::end(kDimOverlayScreenIds);
+  if (!chrome::IsRunningInMash()) {
+    if (!screen_dimmer_) {
+      screen_dimmer_ = base::MakeUnique<ash::ScreenDimmer>(
+          ash::ScreenDimmer::Container::LOCK_SCREEN);
+    }
+    screen_dimmer_->set_at_bottom(true);
+    screen_dimmer_->SetDimming(should_dim);
+  } else {
+    // TODO: Ash needs to expose screen dimming api. See
+    // http://crbug.com/646034.
+    NOTIMPLEMENTED();
+  }
 
-  FOR_EACH_OBSERVER(Observer,
-                    observer_list_,
-                    OnCurrentScreenChanged(current_screen_, new_screen));
   current_screen_ = new_screen;
+  for (Observer& observer : observer_list_)
+    observer.OnCurrentScreenChanged(current_screen_, new_screen);
 }
 
 }  // namespace chromeos
