@@ -5,12 +5,16 @@
 #include "chrome/browser/ui/webui/chromeos/network_ui.h"
 
 #include <string>
+#include <utility>
 
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
 #include "base/values.h"
+#include "chrome/browser/chromeos/options/network_config_view.h"
 #include "chrome/browser/extensions/tab_helper.h"
+#include "chrome/browser/ui/webui/settings/md_settings_localized_strings_provider.h"
 #include "chrome/common/url_constants.h"
+#include "chrome/grit/browser_resources.h"
 #include "chrome/grit/generated_resources.h"
 #include "chromeos/network/device_state.h"
 #include "chromeos/network/network_configuration_handler.h"
@@ -21,8 +25,8 @@
 #include "content/public/browser/web_ui.h"
 #include "content/public/browser/web_ui_data_source.h"
 #include "content/public/browser/web_ui_message_handler.h"
-#include "grit/browser_resources.h"
 #include "third_party/cros_system_api/dbus/service_constants.h"
+#include "ui/base/l10n/l10n_util.h"
 #include "ui/chromeos/strings/grit/ui_chromeos_strings.h"
 
 namespace chromeos {
@@ -48,15 +52,15 @@ void SetDeviceProperties(base::DictionaryValue* dictionary) {
   if (!device_state)
     return;
 
-  scoped_ptr<base::DictionaryValue> device_dictionary(
+  std::unique_ptr<base::DictionaryValue> device_dictionary(
       device_state->properties().DeepCopy());
 
   if (!device_state->ip_configs().empty()) {
     // Convert IPConfig dictionary to a ListValue.
-    scoped_ptr<base::ListValue> ip_configs(new base::ListValue);
+    std::unique_ptr<base::ListValue> ip_configs(new base::ListValue);
     for (base::DictionaryValue::Iterator iter(device_state->ip_configs());
          !iter.IsAtEnd(); iter.Advance()) {
-      ip_configs->Append(iter.value().DeepCopy());
+      ip_configs->Append(iter.value().CreateDeepCopy());
     }
     device_dictionary->SetWithoutPathExpansion(shill::kIPConfigsProperty,
                                                ip_configs.release());
@@ -76,6 +80,10 @@ class NetworkConfigMessageHandler : public content::WebUIMessageHandler {
         "getShillProperties",
         base::Bind(&NetworkConfigMessageHandler::GetShillProperties,
                    base::Unretained(this)));
+    web_ui()->RegisterMessageCallback(
+        "addNetwork",
+        base::Bind(&NetworkConfigMessageHandler::AddNetwork,
+                   base::Unretained(this)));
   }
 
  private:
@@ -102,7 +110,8 @@ class NetworkConfigMessageHandler : public content::WebUIMessageHandler {
   void GetShillPropertiesSuccess(
       const std::string& service_path,
       const base::DictionaryValue& dictionary) const {
-    scoped_ptr<base::DictionaryValue> dictionary_copy(dictionary.DeepCopy());
+    std::unique_ptr<base::DictionaryValue> dictionary_copy(
+        dictionary.DeepCopy());
 
     // Set the 'ServicePath' property for debugging.
     dictionary_copy->SetStringWithoutPathExpansion("ServicePath", service_path);
@@ -110,23 +119,33 @@ class NetworkConfigMessageHandler : public content::WebUIMessageHandler {
     SetDeviceProperties(dictionary_copy.get());
 
     base::ListValue return_arg_list;
-    return_arg_list.Append(dictionary_copy.release());
-    web_ui()->CallJavascriptFunction("NetworkUI.getShillPropertiesResult",
-                                     return_arg_list);
+    return_arg_list.Append(std::move(dictionary_copy));
+    web_ui()->CallJavascriptFunctionUnsafe("NetworkUI.getShillPropertiesResult",
+                                           return_arg_list);
   }
 
   void ErrorCallback(
       const std::string& guid,
       const std::string& error_name,
-      scoped_ptr<base::DictionaryValue> /* error_data */) const {
+      std::unique_ptr<base::DictionaryValue> /* error_data */) const {
     NET_LOG(ERROR) << "Shill Error: " << error_name << " guid=" << guid;
     base::ListValue return_arg_list;
-    scoped_ptr<base::DictionaryValue> dictionary;
+    std::unique_ptr<base::DictionaryValue> dictionary;
     dictionary->SetStringWithoutPathExpansion(shill::kGuidProperty, guid);
     dictionary->SetStringWithoutPathExpansion("ShillError", error_name);
-    return_arg_list.Append(dictionary.release());
-    web_ui()->CallJavascriptFunction("NetworkUI.getShillPropertiesResult",
-                                     return_arg_list);
+    return_arg_list.Append(std::move(dictionary));
+    web_ui()->CallJavascriptFunctionUnsafe("NetworkUI.getShillPropertiesResult",
+                                           return_arg_list);
+  }
+
+  void AddNetwork(const base::ListValue* args) {
+    std::string onc_type;
+    args->GetString(0, &onc_type);
+    std::string shill_type = (onc_type == ::onc::network_type::kVPN)
+                                 ? shill::kTypeVPN
+                                 : shill::kTypeWifi;
+    NetworkConfigView::ShowForType(
+        shill_type, web_ui()->GetWebContents()->GetTopLevelNativeWindow());
   }
 
   base::WeakPtrFactory<NetworkConfigMessageHandler> weak_ptr_factory_;
@@ -136,6 +155,47 @@ class NetworkConfigMessageHandler : public content::WebUIMessageHandler {
 
 }  // namespace
 
+// static
+void NetworkUI::GetLocalizedStrings(base::DictionaryValue* localized_strings) {
+  localized_strings->SetString("titleText",
+                               l10n_util::GetStringUTF16(IDS_NETWORK_UI_TITLE));
+
+  localized_strings->SetString("titleText",
+                               l10n_util::GetStringUTF16(IDS_NETWORK_UI_TITLE));
+  localized_strings->SetString(
+      "autoRefreshText",
+      l10n_util::GetStringUTF16(IDS_NETWORK_UI_AUTO_REFRESH));
+  localized_strings->SetString(
+      "deviceLogLinkText", l10n_util::GetStringUTF16(IDS_DEVICE_LOG_LINK_TEXT));
+  localized_strings->SetString(
+      "networkRefreshText", l10n_util::GetStringUTF16(IDS_NETWORK_UI_REFRESH));
+  localized_strings->SetString(
+      "clickToExpandText", l10n_util::GetStringUTF16(IDS_NETWORK_UI_EXPAND));
+  localized_strings->SetString(
+      "propertyFormatText",
+      l10n_util::GetStringUTF16(IDS_NETWORK_UI_PROPERTY_FORMAT));
+
+  localized_strings->SetString(
+      "normalFormatOption",
+      l10n_util::GetStringUTF16(IDS_NETWORK_UI_FORMAT_NORMAL));
+  localized_strings->SetString(
+      "managedFormatOption",
+      l10n_util::GetStringUTF16(IDS_NETWORK_UI_FORMAT_MANAGED));
+  localized_strings->SetString(
+      "stateFormatOption",
+      l10n_util::GetStringUTF16(IDS_NETWORK_UI_FORMAT_STATE));
+  localized_strings->SetString(
+      "shillFormatOption",
+      l10n_util::GetStringUTF16(IDS_NETWORK_UI_FORMAT_SHILL));
+
+  localized_strings->SetString(
+      "visibleNetworksLabel",
+      l10n_util::GetStringUTF16(IDS_NETWORK_UI_VISIBLE_NETWORKS));
+  localized_strings->SetString(
+      "favoriteNetworksLabel",
+      l10n_util::GetStringUTF16(IDS_NETWORK_UI_FAVORITE_NETWORKS));
+}
+
 NetworkUI::NetworkUI(content::WebUI* web_ui)
     : content::WebUIController(web_ui) {
   web_ui->AddMessageHandler(new NetworkConfigMessageHandler());
@@ -143,44 +203,14 @@ NetworkUI::NetworkUI(content::WebUI* web_ui)
   // Enable extension API calls in the WebUI.
   extensions::TabHelper::CreateForWebContents(web_ui->GetWebContents());
 
+  base::DictionaryValue localized_strings;
+  GetLocalizedStrings(&localized_strings);
+
   content::WebUIDataSource* html =
       content::WebUIDataSource::Create(chrome::kChromeUINetworkHost);
+  html->AddLocalizedStrings(localized_strings);
 
-  html->AddLocalizedString("titleText", IDS_NETWORK_UI_TITLE);
-  html->AddLocalizedString("autoRefreshText", IDS_NETWORK_UI_AUTO_REFRESH);
-  html->AddLocalizedString("deviceLogLinkText", IDS_DEVICE_LOG_LINK_TEXT);
-  html->AddLocalizedString("networkRefreshText", IDS_NETWORK_UI_REFRESH);
-  html->AddLocalizedString("clickToExpandText", IDS_NETWORK_UI_EXPAND);
-  html->AddLocalizedString("propertyFormatText",
-                           IDS_NETWORK_UI_PROPERTY_FORMAT);
-
-  html->AddLocalizedString("normalFormatOption", IDS_NETWORK_UI_FORMAT_NORMAL);
-  html->AddLocalizedString("managedFormatOption",
-                           IDS_NETWORK_UI_FORMAT_MANAGED);
-  html->AddLocalizedString("stateFormatOption", IDS_NETWORK_UI_FORMAT_STATE);
-  html->AddLocalizedString("shillFormatOption", IDS_NETWORK_UI_FORMAT_SHILL);
-
-  html->AddLocalizedString("visibleNetworksLabel",
-                           IDS_NETWORK_UI_VISIBLE_NETWORKS);
-  html->AddLocalizedString("favoriteNetworksLabel",
-                           IDS_NETWORK_UI_FAVORITE_NETWORKS);
-
-  html->AddLocalizedString("networkConnected",
-                           IDS_ASH_STATUS_TRAY_NETWORK_CONNECTED);
-  html->AddLocalizedString("networkConnecting",
-                           IDS_ASH_STATUS_TRAY_NETWORK_CONNECTING);
-  html->AddLocalizedString("networkDisabled",
-                           IDS_OPTIONS_SETTINGS_NETWORK_DISABLED);
-  html->AddLocalizedString("networkNotConnected",
-                           IDS_ASH_STATUS_TRAY_NETWORK_NOT_CONNECTED);
-  html->AddLocalizedString("OncTypeCellular", IDS_NETWORK_TYPE_CELLULAR);
-  html->AddLocalizedString("OncTypeEthernet", IDS_NETWORK_TYPE_ETHERNET);
-  html->AddLocalizedString("OncTypeVPN", IDS_NETWORK_TYPE_VPN);
-  html->AddLocalizedString("OncTypeWiFi", IDS_NETWORK_TYPE_WIFI);
-  html->AddLocalizedString("OncTypeWimax", IDS_NETWORK_TYPE_WIMAX);
-  html->AddLocalizedString(
-      "vpnNameTemplate",
-      IDS_OPTIONS_SETTINGS_SECTION_THIRD_PARTY_VPN_NAME_TEMPLATE);
+  settings::AddCrNetworkStrings(html);
 
   html->SetJsonPath("strings.js");
   html->AddResourcePath("network_ui.css", IDR_NETWORK_UI_CSS);
