@@ -15,10 +15,10 @@
 #include "content/public/browser/url_data_source.h"
 #include "extensions/common/extension_icon_set.h"
 #include "extensions/common/extension_resource.h"
-#include "third_party/skia/include/core/SkBitmap.h"
 
 class ExtensionIconSet;
 class Profile;
+class SkBitmap;
 
 namespace extensions {
 class Extension;
@@ -74,8 +74,7 @@ class ExtensionIconSource : public content::URLDataSource,
   std::string GetMimeType(const std::string&) const override;
   void StartDataRequest(
       const std::string& path,
-      int render_process_id,
-      int render_frame_id,
+      const content::ResourceRequestInfo::WebContentsGetter& wc_getter,
       const content::URLDataSource::GotDataCallback& callback) override;
 
  private:
@@ -123,7 +122,7 @@ class ExtensionIconSource : public content::URLDataSource,
   //  3) If still no matches, load the default extension / application icon.
   void LoadIconFailed(int request_id);
 
-  // Parses and savse an ExtensionIconRequest for the URL |path| for the
+  // Parses and saves an ExtensionIconRequest for the URL |path| for the
   // specified |request_id|.
   bool ParseData(const std::string& path,
                  int request_id,
@@ -150,11 +149,11 @@ class ExtensionIconSource : public content::URLDataSource,
   std::map<int, int> tracker_map_;
 
   // Maps request_ids to ExtensionIconRequests.
-  std::map<int, ExtensionIconRequest*> request_map_;
+  std::map<int, std::unique_ptr<ExtensionIconRequest>> request_map_;
 
-  scoped_ptr<SkBitmap> default_app_data_;
+  std::unique_ptr<SkBitmap> default_app_data_;
 
-  scoped_ptr<SkBitmap> default_extension_data_;
+  std::unique_ptr<SkBitmap> default_extension_data_;
 
   base::CancelableTaskTracker cancelable_task_tracker_;
 
