@@ -13,10 +13,6 @@ namespace base {
 class RefCountedMemory;
 }
 
-namespace content {
-class RenderViewHost;
-}
-
 class MdDownloadsDOMHandler;
 
 class MdDownloadsUI : public content::WebUIController {
@@ -26,9 +22,6 @@ class MdDownloadsUI : public content::WebUIController {
   static base::RefCountedMemory* GetFaviconResourceBytes(
       ui::ScaleFactor scale_factor);
 
-  // content::WebUIController:
-  void RenderViewReused(content::RenderViewHost* render_view_host) override;
-
  private:
   MdDownloadsDOMHandler* handler_;  // Weak.
 
