@@ -6,14 +6,15 @@
 
 #include <algorithm>
 #include <string>
+#include <unordered_map>
 #include <utility>
 
 #include "base/guid.h"
-#include "base/i18n/string_compare.h"
 #include "base/macros.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/trace_event/trace_event.h"
+#include "build/build_config.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/media/router/create_presentation_connection_request.h"
 #include "chrome/browser/media/router/issue.h"
@@ -22,13 +23,14 @@
 #include "chrome/browser/media/router/media_router.h"
 #include "chrome/browser/media/router/media_router_factory.h"
 #include "chrome/browser/media/router/media_router_metrics.h"
-#include "chrome/browser/media/router/media_router_mojo_impl.h"
 #include "chrome/browser/media/router/media_routes_observer.h"
 #include "chrome/browser/media/router/media_sink.h"
 #include "chrome/browser/media/router/media_sinks_observer.h"
 #include "chrome/browser/media/router/media_source.h"
 #include "chrome/browser/media/router/media_source_helper.h"
+#include "chrome/browser/media/router/mojo/media_router_mojo_impl.h"
 #include "chrome/browser/media/router/presentation_service_delegate_impl.h"
+#include "chrome/browser/media/router/route_request_result.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/sessions/session_tab_helper.h"
 #include "chrome/browser/ui/webui/media_router/media_router_localized_strings_provider.h"
@@ -52,6 +54,8 @@ namespace {
 
 // The amount of time to wait for a response when creating a new route.
 const int kCreateRouteTimeoutSeconds = 20;
+const int kCreateRouteTimeoutSecondsForTab = 60;
+const int kCreateRouteTimeoutSecondsForDesktop = 120;
 
 std::string GetHostFromURL(const GURL& gurl) {
   if (gurl.is_empty()) return std::string();
@@ -70,6 +74,20 @@ std::string TruncateHost(const std::string& host) {
   return truncated.empty() ? host : truncated;
 }
 
+base::TimeDelta GetRouteRequestTimeout(MediaCastMode cast_mode) {
+  switch (cast_mode) {
+    case DEFAULT:
+      return base::TimeDelta::FromSeconds(kCreateRouteTimeoutSeconds);
+    case TAB_MIRROR:
+      return base::TimeDelta::FromSeconds(kCreateRouteTimeoutSecondsForTab);
+    case DESKTOP_MIRROR:
+      return base::TimeDelta::FromSeconds(kCreateRouteTimeoutSecondsForDesktop);
+    default:
+      NOTREACHED();
+      return base::TimeDelta();
+  }
+}
+
 }  // namespace
 
 // static
@@ -105,7 +123,8 @@ class MediaRouterUI::UIIssuesObserver : public IssuesObserver {
 };
 
 MediaRouterUI::UIMediaRoutesObserver::UIMediaRoutesObserver(
-    MediaRouter* router, const MediaSource::Id& source_id,
+    MediaRouter* router,
+    const MediaSource::Id& source_id,
     const RoutesUpdatedCallback& callback)
     : MediaRoutesObserver(router, source_id), callback_(callback) {
   DCHECK(!callback_.is_null());
@@ -116,29 +135,7 @@ MediaRouterUI::UIMediaRoutesObserver::~UIMediaRoutesObserver() {}
 void MediaRouterUI::UIMediaRoutesObserver::OnRoutesUpdated(
     const std::vector<MediaRoute>& routes,
     const std::vector<MediaRoute::Id>& joinable_route_ids) {
-  std::vector<MediaRoute> routes_for_display;
-  std::vector<MediaRoute::Id> joinable_route_ids_for_display;
-  for (const MediaRoute& route : routes) {
-    if (route.for_display()) {
-#ifndef NDEBUG
-      for (const MediaRoute& existing_route : routes_for_display) {
-        if (existing_route.media_sink_id() == route.media_sink_id()) {
-          DVLOG(2) << "Received another route for display with the same sink"
-                   << " id as an existing route. " << route.media_route_id()
-                   << " has the same sink id as "
-                   << existing_route.media_sink_id() << ".";
-        }
-      }
-#endif
-      if (ContainsValue(joinable_route_ids, route.media_route_id())) {
-        joinable_route_ids_for_display.push_back(route.media_route_id());
-      }
-
-      routes_for_display.push_back(route);
-    }
-  }
-
-  callback_.Run(routes_for_display, joinable_route_ids_for_display);
+  callback_.Run(routes, joinable_route_ids);
 }
 
 MediaRouterUI::MediaRouterUI(content::WebUI* web_ui)
@@ -152,19 +149,19 @@ MediaRouterUI::MediaRouterUI(content::WebUI* web_ui)
       weak_factory_(this) {
   // Create a WebUIDataSource containing the chrome://media-router page's
   // content.
-  scoped_ptr<content::WebUIDataSource> html_source(
+  std::unique_ptr<content::WebUIDataSource> html_source(
       content::WebUIDataSource::Create(chrome::kChromeUIMediaRouterHost));
 
   content::WebContents* wc = web_ui->GetWebContents();
   DCHECK(wc);
 
-  router_ = static_cast<MediaRouterMojoImpl*>(
-      MediaRouterFactory::GetApiForBrowserContext(wc->GetBrowserContext()));
+  router_ =
+      MediaRouterFactory::GetApiForBrowserContext(wc->GetBrowserContext());
 
   // Allows UI to load extensionview.
   // TODO(haibinlu): limit object-src to current extension once crbug/514866
   // is fixed.
-  html_source->OverrideContentSecurityPolicyObjectSrc("object-src *;");
+  html_source->OverrideContentSecurityPolicyObjectSrc("object-src chrome:;");
 
   AddLocalizedStrings(html_source.get());
   AddMediaRouterUIResources(html_source.get());
@@ -186,42 +183,58 @@ MediaRouterUI::~MediaRouterUI() {
   // If |create_session_request_| still exists, then it means presentation route
   // request was never attempted.
   if (create_session_request_) {
-    create_session_request_->InvokeErrorCallback(content::PresentationError(
-        content::PRESENTATION_ERROR_SESSION_REQUEST_CANCELLED,
-        "Dialog closed."));
+    bool presentation_sinks_available = std::any_of(
+        sinks_.begin(), sinks_.end(), [](const MediaSinkWithCastModes& sink) {
+          return base::ContainsValue(sink.cast_modes, MediaCastMode::DEFAULT);
+        });
+    if (presentation_sinks_available) {
+      create_session_request_->InvokeErrorCallback(content::PresentationError(
+          content::PRESENTATION_ERROR_SESSION_REQUEST_CANCELLED,
+          "Dialog closed."));
+    } else {
+      create_session_request_->InvokeErrorCallback(content::PresentationError(
+          content::PRESENTATION_ERROR_NO_AVAILABLE_SCREENS,
+          "No screens found."));
+    }
   }
 }
 
 void MediaRouterUI::InitWithDefaultMediaSource(
-    const base::WeakPtr<PresentationServiceDelegateImpl>& delegate) {
-  DCHECK(delegate);
+    content::WebContents* initiator,
+    PresentationServiceDelegateImpl* delegate) {
+  DCHECK(initiator);
   DCHECK(!presentation_service_delegate_);
   DCHECK(!query_result_manager_.get());
 
-  presentation_service_delegate_ = delegate;
-  presentation_service_delegate_->AddDefaultPresentationRequestObserver(this);
-  InitCommon(presentation_service_delegate_->web_contents());
-  if (presentation_service_delegate_->HasDefaultPresentationRequest()) {
-    OnDefaultPresentationChanged(
-        presentation_service_delegate_->GetDefaultPresentationRequest());
+  InitCommon(initiator);
+  if (delegate) {
+    presentation_service_delegate_ = delegate->GetWeakPtr();
+    presentation_service_delegate_->AddDefaultPresentationRequestObserver(this);
+  }
+
+  if (delegate && delegate->HasDefaultPresentationRequest()) {
+    OnDefaultPresentationChanged(delegate->GetDefaultPresentationRequest());
   } else {
     // Register for MediaRoute updates without a media source.
-    routes_observer_.reset(new UIMediaRoutesObserver(router_, MediaSource::Id(),
-          base::Bind(&MediaRouterUI::OnRoutesUpdated, base::Unretained(this))));
+    routes_observer_.reset(new UIMediaRoutesObserver(
+        router_, MediaSource::Id(),
+        base::Bind(&MediaRouterUI::OnRoutesUpdated, base::Unretained(this))));
   }
 }
 
 void MediaRouterUI::InitWithPresentationSessionRequest(
     content::WebContents* initiator,
-    const base::WeakPtr<PresentationServiceDelegateImpl>& delegate,
-    scoped_ptr<CreatePresentationConnectionRequest> create_session_request) {
+    PresentationServiceDelegateImpl* delegate,
+    std::unique_ptr<CreatePresentationConnectionRequest>
+        create_session_request) {
   DCHECK(initiator);
+  DCHECK(delegate);
   DCHECK(create_session_request);
   DCHECK(!create_session_request_);
   DCHECK(!query_result_manager_);
 
   create_session_request_ = std::move(create_session_request);
-  presentation_service_delegate_ = delegate;
+  presentation_service_delegate_ = delegate->GetWeakPtr();
   InitCommon(initiator);
   OnDefaultPresentationChanged(create_session_request_->presentation_request());
 }
@@ -233,6 +246,8 @@ void MediaRouterUI::InitCommon(content::WebContents* initiator) {
   TRACE_EVENT_NESTABLE_ASYNC_INSTANT1("media_router", "UI", initiator,
                                       "MediaRouterUI::InitCommon", this);
 
+  router_->OnUserGesture();
+
   // Create |collator_| before |query_result_manager_| so that |collator_| is
   // already set up when we get a callback from |query_result_manager_|.
   UErrorCode error = U_ZERO_ERROR;
@@ -247,25 +262,54 @@ void MediaRouterUI::InitCommon(content::WebContents* initiator) {
   query_result_manager_.reset(new QueryResultManager(router_));
   query_result_manager_->AddObserver(this);
 
-  // These modes are always available.
-  query_result_manager_->StartSinksQuery(MediaCastMode::DESKTOP_MIRROR,
-                                         MediaSourceForDesktop());
+  // Use a placeholder URL as origin for mirroring.
+  GURL origin(chrome::kChromeUIMediaRouterURL);
+
+  // Desktop mirror mode is always available.
+  query_result_manager_->SetSourcesForCastMode(
+      MediaCastMode::DESKTOP_MIRROR, {MediaSourceForDesktop()}, origin);
   initiator_ = initiator;
-  MediaSource mirroring_source(
-      MediaSourceForTab(SessionTabHelper::IdForTab(initiator)));
-  query_result_manager_->StartSinksQuery(MediaCastMode::TAB_MIRROR,
-                                         mirroring_source);
+  SessionID::id_type tab_id = SessionTabHelper::IdForTab(initiator);
+  if (tab_id != -1) {
+    MediaSource mirroring_source(MediaSourceForTab(tab_id));
+    query_result_manager_->SetSourcesForCastMode(MediaCastMode::TAB_MIRROR,
+                                                 {mirroring_source}, origin);
+  }
+
   UpdateCastModes();
+
+  // Get the current list of media routes, so that the WebUI will have routes
+  // information at initialization.
+  OnRoutesUpdated(router_->GetCurrentRoutes(), std::vector<MediaRoute::Id>());
+}
+
+void MediaRouterUI::InitForTest(
+    MediaRouter* router,
+    content::WebContents* initiator,
+    MediaRouterWebUIMessageHandler* handler,
+    std::unique_ptr<CreatePresentationConnectionRequest>
+        create_session_request) {
+  router_ = router;
+  handler_ = handler;
+  create_session_request_ = std::move(create_session_request);
+  InitCommon(initiator);
+  if (create_session_request_) {
+    OnDefaultPresentationChanged(
+        create_session_request_->presentation_request());
+  }
 }
 
 void MediaRouterUI::OnDefaultPresentationChanged(
     const PresentationRequest& presentation_request) {
-  MediaSource source = presentation_request.GetMediaSource();
+  std::vector<MediaSource> sources = presentation_request.GetMediaSources();
   presentation_request_.reset(new PresentationRequest(presentation_request));
-  query_result_manager_->StartSinksQuery(MediaCastMode::DEFAULT, source);
+  query_result_manager_->SetSourcesForCastMode(
+      MediaCastMode::DEFAULT, sources,
+      presentation_request_->frame_url().GetOrigin());
   // Register for MediaRoute updates.
+  // TODO(crbug.com/627655): Use multiple URLs.
   routes_observer_.reset(new UIMediaRoutesObserver(
-      router_, source.id(),
+      router_, sources[0].id(),
       base::Bind(&MediaRouterUI::OnRoutesUpdated, base::Unretained(this))));
 
   UpdateCastModes();
@@ -273,22 +317,41 @@ void MediaRouterUI::OnDefaultPresentationChanged(
 
 void MediaRouterUI::OnDefaultPresentationRemoved() {
   presentation_request_.reset();
-  query_result_manager_->StopSinksQuery(MediaCastMode::DEFAULT);
+  query_result_manager_->RemoveSourcesForCastMode(MediaCastMode::DEFAULT);
   // Register for MediaRoute updates without a media source.
   routes_observer_.reset(new UIMediaRoutesObserver(
-        router_, MediaSource::Id(),
-        base::Bind(&MediaRouterUI::OnRoutesUpdated, base::Unretained(this))));
+      router_, MediaSource::Id(),
+      base::Bind(&MediaRouterUI::OnRoutesUpdated, base::Unretained(this))));
   UpdateCastModes();
 }
 
 void MediaRouterUI::UpdateCastModes() {
   // Gets updated cast modes from |query_result_manager_| and forwards it to UI.
-  query_result_manager_->GetSupportedCastModes(&cast_modes_);
+  cast_modes_ = query_result_manager_->GetSupportedCastModes();
   if (ui_initialized_) {
     handler_->UpdateCastModes(cast_modes_, GetPresentationRequestSourceName());
   }
 }
 
+void MediaRouterUI::UpdateRoutesToCastModesMapping() {
+  std::unordered_map<MediaSource::Id, MediaCastMode> available_source_map;
+  for (const auto& cast_mode : cast_modes_) {
+    for (const auto& source :
+         query_result_manager_->GetSourcesForCastMode(cast_mode)) {
+      available_source_map.insert(std::make_pair(source.id(), cast_mode));
+    }
+  }
+
+  routes_and_cast_modes_.clear();
+  for (const auto& route : routes_) {
+    auto source_entry = available_source_map.find(route.media_source().id());
+    if (source_entry != available_source_map.end()) {
+      routes_and_cast_modes_.insert(
+          std::make_pair(route.media_route_id(), source_entry->second));
+    }
+  }
+}
+
 void MediaRouterUI::Close() {
   ConstrainedWebDialogDelegate* delegate = GetConstrainedDelegate();
   if (delegate) {
@@ -309,12 +372,29 @@ void MediaRouterUI::UIInitialized() {
 
 bool MediaRouterUI::CreateRoute(const MediaSink::Id& sink_id,
                                 MediaCastMode cast_mode) {
-  return CreateOrConnectRoute(sink_id, cast_mode, MediaRoute::Id());
+  MediaSource::Id source_id;
+  GURL origin;
+  std::vector<MediaRouteResponseCallback> route_response_callbacks;
+  base::TimeDelta timeout;
+  bool incognito;
+  if (!SetRouteParameters(sink_id, cast_mode, &source_id, &origin,
+                          &route_response_callbacks, &timeout, &incognito)) {
+    SendIssueForUnableToCast(cast_mode);
+    return false;
+  }
+  router_->CreateRoute(source_id, sink_id, origin, initiator_,
+                       route_response_callbacks, timeout, incognito);
+  return true;
 }
 
-bool MediaRouterUI::CreateOrConnectRoute(const MediaSink::Id& sink_id,
-                                              MediaCastMode cast_mode,
-                                              const MediaRoute::Id& route_id) {
+bool MediaRouterUI::SetRouteParameters(
+    const MediaSink::Id& sink_id,
+    MediaCastMode cast_mode,
+    MediaSource::Id* source_id,
+    GURL* origin,
+    std::vector<MediaRouteResponseCallback>* route_response_callbacks,
+    base::TimeDelta* timeout,
+    bool* incognito) {
   DCHECK(query_result_manager_.get());
   DCHECK(initiator_);
 
@@ -324,12 +404,14 @@ bool MediaRouterUI::CreateOrConnectRoute(const MediaSink::Id& sink_id,
   // called. However, since the user does not have visibility into the
   // MediaSource, and that it occurs very rarely in practice, we leave it as-is
   // for now.
-  MediaSource source = query_result_manager_->GetSourceForCastMode(cast_mode);
-  if (source.Empty()) {
+  std::unique_ptr<MediaSource> source =
+      query_result_manager_->GetSourceForCastModeAndSink(cast_mode, sink_id);
+  if (!source) {
     LOG(ERROR) << "No corresponding MediaSource for cast mode "
-               << static_cast<int>(cast_mode);
+               << static_cast<int>(cast_mode) << " and sink " << sink_id;
     return false;
   }
+  *source_id = source->id();
 
   bool for_default_source = cast_mode == MediaCastMode::DEFAULT;
   if (for_default_source && !presentation_request_) {
@@ -339,64 +421,65 @@ bool MediaRouterUI::CreateOrConnectRoute(const MediaSink::Id& sink_id,
   }
 
   current_route_request_id_ = ++route_request_counter_;
-  GURL origin;
-  if (for_default_source) {
-    origin = presentation_request_->frame_url().GetOrigin();
-  } else {
-    // Requesting route for mirroring. Use a placeholder URL as origin.
-    origin = GURL(chrome::kChromeUIMediaRouterURL);
-  }
-  DCHECK(origin.is_valid());
+  *origin = for_default_source ? presentation_request_->frame_url().GetOrigin()
+                               : GURL(chrome::kChromeUIMediaRouterURL);
+  DVLOG(1) << "DoCreateRoute: origin: " << *origin;
 
-  DVLOG(1) << "DoCreateRoute: origin: " << origin;
-
-  // There are 3 cases. In all cases the MediaRouterUI will need to be notified.
+  // There are 3 cases. In cases (1) and (3) the MediaRouterUI will need to be
+  // notified. In case (2) the dialog will be closed.
   // (1) Non-presentation route request (e.g., mirroring). No additional
   // notification necessary.
   // (2) Presentation route request for a Presentation API startSession call.
   // The startSession (CreatePresentationConnectionRequest) will need to be
-  // answered with the
-  // route response.
+  // answered with the route response.
   // (3) Browser-initiated presentation route request. If successful,
   // PresentationServiceDelegateImpl will have to be notified. Note that we
   // treat subsequent route requests from a Presentation API-initiated dialogs
   // as browser-initiated.
-  std::vector<MediaRouteResponseCallback> route_response_callbacks;
-  route_response_callbacks.push_back(base::Bind(
-      &MediaRouterUI::OnRouteResponseReceived, weak_factory_.GetWeakPtr(),
-      current_route_request_id_, sink_id));
+  if (!for_default_source || !create_session_request_) {
+    route_response_callbacks->push_back(base::Bind(
+        &MediaRouterUI::OnRouteResponseReceived, weak_factory_.GetWeakPtr(),
+        current_route_request_id_, sink_id, cast_mode,
+        base::UTF8ToUTF16(GetTruncatedPresentationRequestSourceName())));
+  }
   if (for_default_source) {
     if (create_session_request_) {
       // |create_session_request_| will be nullptr after this call, as the
       // object will be transferred to the callback.
-      route_response_callbacks.push_back(
+      route_response_callbacks->push_back(
           base::Bind(&CreatePresentationConnectionRequest::HandleRouteResponse,
                      base::Passed(&create_session_request_)));
+      route_response_callbacks->push_back(
+          base::Bind(&MediaRouterUI::HandleCreateSessionRequestRouteResponse,
+                     weak_factory_.GetWeakPtr()));
     } else if (presentation_service_delegate_) {
-      route_response_callbacks.push_back(
+      route_response_callbacks->push_back(
           base::Bind(&PresentationServiceDelegateImpl::OnRouteResponse,
                      presentation_service_delegate_, *presentation_request_));
     }
   }
 
-  // Start the timer.
-  route_creation_timer_.Start(
-      FROM_HERE, base::TimeDelta::FromSeconds(kCreateRouteTimeoutSeconds), this,
-      &MediaRouterUI::RouteCreationTimeout);
+  *timeout = GetRouteRequestTimeout(cast_mode);
+  *incognito = Profile::FromWebUI(web_ui())->IsOffTheRecord();
 
-  if (route_id.empty()) {
-    router_->CreateRoute(source.id(), sink_id, origin, initiator_,
-                         route_response_callbacks);
-  } else {
-    router_->ConnectRouteByRouteId(source.id(), route_id, origin,
-                                   initiator_, route_response_callbacks);
-  }
   return true;
 }
 
 bool MediaRouterUI::ConnectRoute(const MediaSink::Id& sink_id,
                                  const MediaRoute::Id& route_id) {
-  return CreateOrConnectRoute(sink_id, MediaCastMode::DEFAULT, route_id);
+  MediaSource::Id source_id;
+  GURL origin;
+  std::vector<MediaRouteResponseCallback> route_response_callbacks;
+  base::TimeDelta timeout;
+  bool incognito;
+  if (!SetRouteParameters(sink_id, MediaCastMode::DEFAULT, &source_id, &origin,
+                          &route_response_callbacks, &timeout, &incognito)) {
+    SendIssueForUnableToCast(MediaCastMode::DEFAULT);
+    return false;
+  }
+  router_->ConnectRouteByRouteId(source_id, route_id, origin, initiator_,
+                                 route_response_callbacks, timeout, incognito);
+  return true;
 }
 
 void MediaRouterUI::CloseRoute(const MediaRoute::Id& route_id) {
@@ -409,32 +492,33 @@ void MediaRouterUI::ClearIssue(const std::string& issue_id) {
   router_->ClearIssue(issue_id);
 }
 
+void MediaRouterUI::SearchSinksAndCreateRoute(
+    const MediaSink::Id& sink_id,
+    const std::string& search_criteria,
+    const std::string& domain,
+    MediaCastMode cast_mode) {
+  std::unique_ptr<MediaSource> source =
+      query_result_manager_->GetSourceForCastModeAndSink(cast_mode, sink_id);
+  const std::string source_id = source ? source->id() : "";
+
+  // The CreateRoute() part of the function is accomplished in the callback
+  // OnSearchSinkResponseReceived().
+  router_->SearchSinks(
+      sink_id, source_id, search_criteria, domain,
+      base::Bind(&MediaRouterUI::OnSearchSinkResponseReceived,
+                 weak_factory_.GetWeakPtr(), cast_mode));
+}
+
 void MediaRouterUI::OnResultsUpdated(
     const std::vector<MediaSinkWithCastModes>& sinks) {
   sinks_ = sinks;
 
   const icu::Collator* collator_ptr = collator_.get();
-  std::sort(
-      sinks_.begin(), sinks_.end(),
-      [collator_ptr](const MediaSinkWithCastModes& sink1,
-                     const MediaSinkWithCastModes& sink2) {
-        if (collator_ptr) {
-          base::string16 sink1_name = base::UTF8ToUTF16(sink1.sink.name());
-          base::string16 sink2_name = base::UTF8ToUTF16(sink2.sink.name());
-          UCollationResult result = base::i18n::CompareString16WithCollator(
-              *collator_ptr, sink1_name, sink2_name);
-          if (result != UCOL_EQUAL)
-            return result == UCOL_LESS;
-        } else {
-          // Fall back to simple string comparison if collator is not
-          // available.
-          int val = sink1.sink.name().compare(sink2.sink.name());
-          if (val)
-            return val < 0;
-        }
-
-        return sink1.sink.id() < sink2.sink.id();
-      });
+  std::sort(sinks_.begin(), sinks_.end(),
+            [collator_ptr](const MediaSinkWithCastModes& sink1,
+                           const MediaSinkWithCastModes& sink2) {
+              return sink1.sink.CompareUsingCollator(sink2.sink, collator_ptr);
+            });
 
   if (ui_initialized_) handler_->UpdateSinks(sinks_);
 }
@@ -446,51 +530,127 @@ void MediaRouterUI::SetIssue(const Issue* issue) {
 void MediaRouterUI::OnRoutesUpdated(
     const std::vector<MediaRoute>& routes,
     const std::vector<MediaRoute::Id>& joinable_route_ids) {
-  routes_ = routes;
-  joinable_route_ids_ = joinable_route_ids;
-  if (ui_initialized_) handler_->UpdateRoutes(routes_, joinable_route_ids_);
+  routes_.clear();
+  joinable_route_ids_.clear();
+
+  for (const MediaRoute& route : routes) {
+    if (route.for_display()) {
+#ifndef NDEBUG
+      for (const MediaRoute& existing_route : routes_) {
+        if (existing_route.media_sink_id() == route.media_sink_id()) {
+          DVLOG(2) << "Received another route for display with the same sink"
+                   << " id as an existing route. " << route.media_route_id()
+                   << " has the same sink id as "
+                   << existing_route.media_sink_id() << ".";
+        }
+      }
+#endif
+      if (base::ContainsValue(joinable_route_ids, route.media_route_id())) {
+        joinable_route_ids_.push_back(route.media_route_id());
+      }
+
+      routes_.push_back(route);
+    }
+  }
+  UpdateRoutesToCastModesMapping();
+
+  if (ui_initialized_)
+    handler_->UpdateRoutes(routes_, joinable_route_ids_,
+                           routes_and_cast_modes_);
 }
 
-void MediaRouterUI::OnRouteResponseReceived(const int route_request_id,
-                                            const MediaSink::Id& sink_id,
-                                            const MediaRoute* route,
-                                            const std::string& presentation_id,
-                                            const std::string& error) {
+void MediaRouterUI::OnRouteResponseReceived(
+    int route_request_id,
+    const MediaSink::Id& sink_id,
+    MediaCastMode cast_mode,
+    const base::string16& presentation_request_source_name,
+    const RouteRequestResult& result) {
   DVLOG(1) << "OnRouteResponseReceived";
   // If we receive a new route that we aren't expecting, do nothing.
   if (route_request_id != current_route_request_id_) return;
 
+  const MediaRoute* route = result.route();
   if (!route) {
     // The provider will handle sending an issue for a failed route request.
-    DVLOG(0) << "MediaRouteResponse returned error: " << error;
+    DVLOG(1) << "MediaRouteResponse returned error: " << result.error();
   }
 
-  std::string route_id = route ? route->media_route_id() : std::string();
-  handler_->OnCreateRouteResponseReceived(sink_id, route_id);
+  handler_->OnCreateRouteResponseReceived(sink_id, route);
   current_route_request_id_ = -1;
-  route_creation_timer_.Stop();
+
+  if (result.result_code() == RouteRequestResult::TIMED_OUT)
+    SendIssueForRouteTimeout(cast_mode, presentation_request_source_name);
 }
 
-void MediaRouterUI::RouteCreationTimeout() {
-  current_route_request_id_ = -1;
+void MediaRouterUI::HandleCreateSessionRequestRouteResponse(
+    const RouteRequestResult&) {
+  Close();
+}
 
-  base::string16 host =
-      base::UTF8ToUTF16(GetTruncatedPresentationRequestSourceName());
+void MediaRouterUI::OnSearchSinkResponseReceived(
+    MediaCastMode cast_mode,
+    const MediaSink::Id& found_sink_id) {
+  DVLOG(1) << "OnSearchSinkResponseReceived";
+  handler_->ReturnSearchResult(found_sink_id);
 
-  // TODO(apacible): Update error messages based on current cast mode
-  // (e.g. desktop).
-  std::string issue_title =
-      host.empty() ? l10n_util::GetStringUTF8(
-                         IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_TIMEOUT_FOR_TAB)
-                   : l10n_util::GetStringFUTF8(
-                         IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_TIMEOUT, host);
+  MediaSource::Id source_id;
+  GURL origin;
+  std::vector<MediaRouteResponseCallback> route_response_callbacks;
+  base::TimeDelta timeout;
+  bool incognito;
+  if (!SetRouteParameters(found_sink_id, cast_mode, &source_id, &origin,
+                          &route_response_callbacks, &timeout, &incognito)) {
+    SendIssueForUnableToCast(cast_mode);
+    return;
+  }
+  router_->CreateRoute(source_id, found_sink_id, origin, initiator_,
+                       route_response_callbacks, timeout, incognito);
+}
+
+void MediaRouterUI::SendIssueForRouteTimeout(
+    MediaCastMode cast_mode,
+    const base::string16& presentation_request_source_name) {
+  std::string issue_title;
+  switch (cast_mode) {
+    case DEFAULT:
+      DLOG_IF(ERROR, presentation_request_source_name.empty())
+          << "Empty presentation request source name.";
+      issue_title =
+          l10n_util::GetStringFUTF8(IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_TIMEOUT,
+                                    presentation_request_source_name);
+      break;
+    case TAB_MIRROR:
+      issue_title = l10n_util::GetStringUTF8(
+          IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_TIMEOUT_FOR_TAB);
+      break;
+    case DESKTOP_MIRROR:
+      issue_title = l10n_util::GetStringUTF8(
+          IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_TIMEOUT_FOR_DESKTOP);
+      break;
+    default:
+      NOTREACHED();
+  }
 
   Issue issue(issue_title, std::string(),
               IssueAction(IssueAction::TYPE_DISMISS),
               std::vector<IssueAction>(), std::string(), Issue::NOTIFICATION,
-              false, std::string());
+              false, -1);
   AddIssue(issue);
-  handler_->NotifyRouteCreationTimeout();
+}
+
+void MediaRouterUI::SendIssueForUnableToCast(MediaCastMode cast_mode) {
+  // For a generic error, claim a tab error unless it was specifically desktop
+  // mirroring.
+  std::string issue_title =
+      (cast_mode == MediaCastMode::DESKTOP_MIRROR)
+          ? l10n_util::GetStringUTF8(
+                IDS_MEDIA_ROUTER_ISSUE_UNABLE_TO_CAST_DESKTOP)
+          : l10n_util::GetStringUTF8(
+                IDS_MEDIA_ROUTER_ISSUE_CREATE_ROUTE_TIMEOUT_FOR_TAB);
+  AddIssue(Issue(issue_title, std::string(),
+                 IssueAction(IssueAction::TYPE_DISMISS),
+                 std::vector<IssueAction>(), std::string(), Issue::WARNING,
+                 false, -1));
 }
 
 GURL MediaRouterUI::GetFrameURL() const {
@@ -514,7 +674,9 @@ std::string MediaRouterUI::GetTruncatedPresentationRequestSourceName() const {
 }
 
 const std::string& MediaRouterUI::GetRouteProviderExtensionId() const {
-  return router_->media_route_provider_extension_id();
+  // TODO(crbug.com/597778): remove reference to MediaRouterMojoImpl
+  return static_cast<MediaRouterMojoImpl*>(router_)
+      ->media_route_provider_extension_id();
 }
 
 void MediaRouterUI::SetUIInitializationTimer(const base::Time& start_time) {
