@@ -6,7 +6,7 @@
 
 #include "base/compiler_specific.h"
 #include "base/macros.h"
-#include "base/prefs/pref_service.h"
+#include "base/memory/ptr_util.h"
 #include "base/strings/stringprintf.h"
 #include "chrome/browser/chromeos/login/login_manager_test.h"
 #include "chrome/browser/chromeos/login/startup_utils.h"
@@ -21,6 +21,10 @@
 #include "chrome/common/pref_names.h"
 #include "chrome/test/base/ui_test_utils.h"
 #include "chromeos/settings/cros_settings_names.h"
+#if defined(GOOGLE_CHROME_BUILD)
+#include "components/spellcheck/browser/pref_names.h"
+#endif
+#include "components/prefs/pref_service.h"
 #include "components/signin/core/browser/signin_manager.h"
 #include "components/user_manager/user_manager.h"
 #include "content/public/browser/web_contents.h"
@@ -79,7 +83,7 @@ const PrefTest kPrefTests[] = {
   { kAccountsPrefSupervisedUsersEnabled, true, false },
 #if defined(GOOGLE_CHROME_BUILD)
   { kStatsReportingPref, true, true },
-  { prefs::kSpellCheckUseSpellingService, false, false },
+  { spellcheck::prefs::kSpellCheckUseSpellingService, false, false },
 #endif
 };
 
@@ -89,10 +93,12 @@ class SharedOptionsTest : public LoginManagerTest {
  public:
   SharedOptionsTest()
       : LoginManagerTest(false),
-        device_settings_provider_(NULL),
+        stub_settings_provider_(base::MakeUnique<StubCrosSettingsProvider>()),
+        stub_settings_provider_ptr_(static_cast<StubCrosSettingsProvider*>(
+            stub_settings_provider_.get())),
         test_owner_account_id_(AccountId::FromUserEmail(kTestOwner)),
         test_non_owner_account_id_(AccountId::FromUserEmail(kTestNonOwner)) {
-    stub_settings_provider_.Set(kDeviceOwner, base::StringValue(kTestOwner));
+    stub_settings_provider_->Set(kDeviceOwner, base::StringValue(kTestOwner));
   }
 
   ~SharedOptionsTest() override {}
@@ -104,15 +110,15 @@ class SharedOptionsTest : public LoginManagerTest {
 
     // Add the stub settings provider, moving the device settings provider
     // behind it so our stub takes precedence.
-    device_settings_provider_ = settings->GetProvider(kDeviceOwner);
-    settings->RemoveSettingsProvider(device_settings_provider_);
-    settings->AddSettingsProvider(&stub_settings_provider_);
-    settings->AddSettingsProvider(device_settings_provider_);
+    std::unique_ptr<CrosSettingsProvider> device_settings_provider =
+        settings->RemoveSettingsProvider(settings->GetProvider(kDeviceOwner));
+    settings->AddSettingsProvider(std::move(stub_settings_provider_));
+    settings->AddSettingsProvider(std::move(device_settings_provider));
   }
 
   void TearDownOnMainThread() override {
     CrosSettings* settings = CrosSettings::Get();
-    settings->RemoveSettingsProvider(&stub_settings_provider_);
+    settings->RemoveSettingsProvider(stub_settings_provider_ptr_);
     LoginManagerTest::TearDownOnMainThread();
   }
 
@@ -145,9 +151,9 @@ class SharedOptionsTest : public LoginManagerTest {
   // Creates a browser and navigates to the Settings page.
   Browser* CreateBrowserForUser(const user_manager::User* user) {
     Profile* profile = ProfileHelper::Get()->GetProfileByUserUnsafe(user);
-    SigninManagerFactory::GetForProfile(profile)->
-        SetAuthenticatedAccountInfo(GetGaiaIDForUserID(user->email()),
-                                    user->email());
+    SigninManagerFactory::GetForProfile(profile)->SetAuthenticatedAccountInfo(
+        GetGaiaIDForUserID(user->GetAccountId().GetUserEmail()),
+        user->GetAccountId().GetUserEmail());
 
     ui_test_utils::BrowserAddedObserver observer;
     Browser* browser = CreateBrowser(profile);
@@ -243,8 +249,8 @@ class SharedOptionsTest : public LoginManagerTest {
   void CheckAccountsOverlay(content::WebContents* contents, bool is_owner) {
     // Set cros.accounts.allowGuest to false so we can test the accounts list.
     // This has to be done after the PRE_* test or we can't add the owner.
-    stub_settings_provider_.Set(
-        kAccountsPrefAllowNewUser, base::FundamentalValue(false));
+    stub_settings_provider_ptr_->Set(kAccountsPrefAllowNewUser,
+                                     base::FundamentalValue(false));
 
     bool success;
     std::string js_expression = base::StringPrintf(
@@ -267,8 +273,8 @@ class SharedOptionsTest : public LoginManagerTest {
         (is_owner ? "owner." : "non-owner.");
   }
 
-  StubAccountSettingsProvider stub_settings_provider_;
-  CrosSettingsProvider* device_settings_provider_;
+  std::unique_ptr<CrosSettingsProvider> stub_settings_provider_;
+  StubCrosSettingsProvider* stub_settings_provider_ptr_;
 
   const AccountId test_owner_account_id_;
   const AccountId test_non_owner_account_id_;
@@ -356,7 +362,7 @@ IN_PROC_BROWSER_TEST_F(SharedOptionsTest, ScreenLockPreferencePrimary) {
   // Set the secondary user's preference to true, and reload the primary user's
   // browser to see the updated controlled-by indicator.
   prefs2->SetBoolean(prefs::kEnableAutoScreenLock, true);
-  chrome::Reload(browser, CURRENT_TAB);
+  chrome::Reload(browser, WindowOpenDisposition::CURRENT_TAB);
   content::WaitForLoadStop(contents);
   {
     SCOPED_TRACE("Screen lock false for primary user");
@@ -437,7 +443,7 @@ IN_PROC_BROWSER_TEST_F(SharedOptionsTest, ScreenLockPreferenceSecondary) {
   // Set the preference to false for the primary user and check that the
   // indicator disappears.
   prefs1->SetBoolean(prefs::kEnableAutoScreenLock, false);
-  chrome::Reload(browser, CURRENT_TAB);
+  chrome::Reload(browser, WindowOpenDisposition::CURRENT_TAB);
   content::WaitForLoadStop(contents);
   {
     SCOPED_TRACE("Screen lock false for both users");
