@@ -7,7 +7,6 @@
 #include <limits.h>
 
 #include "base/bind.h"
-#include "base/prefs/pref_service.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/values.h"
@@ -17,8 +16,12 @@
 #include "chrome/browser/signin/signin_promo.h"
 #include "chrome/browser/ui/browser_navigator.h"
 #include "chrome/browser/ui/browser_navigator_params.h"
+#include "chrome/browser/ui/signin_view_controller_delegate.h"
+#include "chrome/browser/ui/user_manager.h"
+#include "chrome/browser/ui/webui/signin/signin_utils.h"
 #include "chrome/common/pref_names.h"
 #include "components/metrics/metrics_pref_names.h"
+#include "components/prefs/pref_service.h"
 #include "components/signin/core/common/signin_pref_names.h"
 #include "content/public/browser/storage_partition.h"
 #include "content/public/browser/user_metrics.h"
@@ -42,6 +45,12 @@ void InlineLoginHandler::RegisterMessages() {
       "switchToFullTab",
       base::Bind(&InlineLoginHandler::HandleSwitchToFullTabMessage,
                  base::Unretained(this)));
+  web_ui()->RegisterMessageCallback("navigationButtonClicked",
+      base::Bind(&InlineLoginHandler::HandleNavigationButtonClicked,
+                 base::Unretained(this)));
+  web_ui()->RegisterMessageCallback("dialogClose",
+      base::Bind(&InlineLoginHandler::HandleDialogClose,
+                 base::Unretained(this)));
 }
 
 void InlineLoginHandler::HandleInitializeMessage(const base::ListValue* args) {
@@ -50,15 +59,28 @@ void InlineLoginHandler::HandleInitializeMessage(const base::ListValue* args) {
       content::BrowserContext::GetStoragePartitionForSite(
           contents->GetBrowserContext(), signin::GetSigninPartitionURL());
   if (partition) {
-    partition->ClearData(
-        content::StoragePartition::REMOVE_DATA_MASK_ALL,
-        content::StoragePartition::QUOTA_MANAGED_STORAGE_MASK_ALL,
-        GURL(),
-        content::StoragePartition::OriginMatcherFunction(),
-        base::Time(),
-        base::Time::Max(),
-        base::Bind(&InlineLoginHandler::ContinueHandleInitializeMessage,
-                   weak_ptr_factory_.GetWeakPtr()));
+    const GURL& current_url = web_ui()->GetWebContents()->GetURL();
+
+    // If the kSignInPromoQueryKeyForceKeepData param is missing, or if it is
+    // present and its value is zero, this means we don't want to keep the
+    // the data.
+    std::string value;
+    if (!net::GetValueForKeyInQuery(current_url,
+                                    signin::kSignInPromoQueryKeyForceKeepData,
+                                    &value) ||
+        value == "0") {
+      partition->ClearData(
+          content::StoragePartition::REMOVE_DATA_MASK_ALL,
+          content::StoragePartition::QUOTA_MANAGED_STORAGE_MASK_ALL,
+          GURL(),
+          content::StoragePartition::OriginMatcherFunction(),
+          base::Time(),
+          base::Time::Max(),
+          base::Bind(&InlineLoginHandler::ContinueHandleInitializeMessage,
+                     weak_ptr_factory_.GetWeakPtr()));
+    } else {
+      ContinueHandleInitializeMessage();
+    }
   }
 }
 
@@ -135,6 +157,26 @@ void InlineLoginHandler::RecordSigninUserActionForAccessPoint(
       content::RecordAction(
           base::UserMetricsAction("Signin_Signin_FromUnknownAccessPoint"));
       break;
+    case signin_metrics::AccessPoint::ACCESS_POINT_PASSWORD_BUBBLE:
+      content::RecordAction(
+          base::UserMetricsAction("Signin_Signin_FromPasswordBubble"));
+      break;
+    case signin_metrics::AccessPoint::ACCESS_POINT_AUTOFILL_DROPDOWN:
+      content::RecordAction(
+          base::UserMetricsAction("Signin_Signin_FromAutofillDropdown"));
+      break;
+    case signin_metrics::AccessPoint::ACCESS_POINT_NTP_CONTENT_SUGGESTIONS:
+      content::RecordAction(
+          base::UserMetricsAction("Signin_Signin_FromNTPContentSuggestions"));
+      break;
+    case signin_metrics::AccessPoint::ACCESS_POINT_RESIGNIN_INFOBAR:
+      content::RecordAction(
+          base::UserMetricsAction("Signin_Signin_FromReSigninInfobar"));
+      break;
+    case signin_metrics::AccessPoint::ACCESS_POINT_TAB_SWITCHER:
+      content::RecordAction(
+          base::UserMetricsAction("Signin_Signin_FromTabSwitcher"));
+      break;
     case signin_metrics::AccessPoint::ACCESS_POINT_MAX:
       NOTREACHED();
       break;
@@ -153,15 +195,21 @@ void InlineLoginHandler::ContinueHandleInitializeMessage() {
   const GURL& current_url = web_ui()->GetWebContents()->GetURL();
   signin_metrics::AccessPoint access_point =
       signin::GetAccessPointForPromoURL(current_url);
-  signin_metrics::LogSigninAccessPointStarted(access_point);
-  RecordSigninUserActionForAccessPoint(access_point);
-  content::RecordAction(base::UserMetricsAction("Signin_SigninPage_Loading"));
+  signin_metrics::Reason reason =
+      signin::GetSigninReasonForPromoURL(current_url);
+
+  if (reason != signin_metrics::Reason::REASON_REAUTHENTICATION &&
+      reason != signin_metrics::Reason::REASON_UNLOCK &&
+      reason != signin_metrics::Reason::REASON_ADD_SECONDARY_ACCOUNT) {
+    signin_metrics::LogSigninAccessPointStarted(access_point);
+    RecordSigninUserActionForAccessPoint(access_point);
+    content::RecordAction(base::UserMetricsAction("Signin_SigninPage_Loading"));
+    params.SetBoolean("isLoginPrimaryAccount", true);
+  }
 
   params.SetString("continueUrl", signin::GetLandingURL(access_point).spec());
 
   Profile* profile = Profile::FromWebUI(web_ui());
-  signin_metrics::Reason reason =
-      signin::GetSigninReasonForPromoURL(current_url);
   std::string default_email;
   if (reason == signin_metrics::Reason::REASON_SIGNIN_PRIMARY_ACCOUNT) {
     default_email =
@@ -173,17 +221,6 @@ void InlineLoginHandler::ContinueHandleInitializeMessage() {
   if (!default_email.empty())
     params.SetString("email", default_email);
 
-  std::string frame_url_id_str;
-  net::GetValueForKeyInQuery(current_url, "frameUrlId", &frame_url_id_str);
-  int frame_url_id;
-  std::string frame_url;
-  if (!frame_url_id_str.empty() &&
-      base::StringToInt(frame_url_id_str, &frame_url_id) &&
-      extensions::GaiaAuthExtensionLoader::Get(profile)
-          ->GetData(frame_url_id, &frame_url)) {
-    params.SetString("frameUrl", frame_url);
-  }
-
   std::string is_constrained;
   net::GetValueForKeyInQuery(
       current_url, signin::kSignInPromoQueryKeyConstrained, &is_constrained);
@@ -197,7 +234,8 @@ void InlineLoginHandler::ContinueHandleInitializeMessage() {
 
   SetExtraInitParams(params);
 
-  web_ui()->CallJavascriptFunction("inline.login.loadAuthExtension", params);
+  web_ui()->CallJavascriptFunctionUnsafe("inline.login.loadAuthExtension",
+                                         params);
 }
 
 void InlineLoginHandler::HandleCompleteLoginMessage(
@@ -211,13 +249,7 @@ void InlineLoginHandler::HandleSwitchToFullTabMessage(
   CHECK(args->GetString(0, &url_str));
 
   Profile* profile = Profile::FromWebUI(web_ui());
-  const int frame_url_id =
-      extensions::GaiaAuthExtensionLoader::Get(profile)->AddData(url_str);
-
-  content::WebContents* web_contents = web_ui()->GetWebContents();
-  GURL main_frame_url(web_contents->GetURL());
-  main_frame_url = net::AppendOrReplaceQueryParameter(
-      main_frame_url, "frameUrlId", base::IntToString(frame_url_id));
+  GURL main_frame_url(web_ui()->GetWebContents()->GetURL());
 
   // Adds extra parameters to the signin URL so that Chrome will close the tab
   // and show the account management view of the avatar menu upon completion.
@@ -225,6 +257,8 @@ void InlineLoginHandler::HandleSwitchToFullTabMessage(
       main_frame_url, signin::kSignInPromoQueryKeyAutoClose, "1");
   main_frame_url = net::AppendOrReplaceQueryParameter(
       main_frame_url, signin::kSignInPromoQueryKeyShowAccountManagement, "1");
+  main_frame_url = net::AppendOrReplaceQueryParameter(
+      main_frame_url, signin::kSignInPromoQueryKeyForceKeepData, "1");
 
   chrome::NavigateParams params(
       profile,
@@ -233,5 +267,23 @@ void InlineLoginHandler::HandleSwitchToFullTabMessage(
       ui::PAGE_TRANSITION_AUTO_TOPLEVEL);
   chrome::Navigate(&params);
 
-  web_ui()->CallJavascriptFunction("inline.login.closeDialog");
+  web_ui()->CallJavascriptFunctionUnsafe("inline.login.closeDialog");
+}
+
+void InlineLoginHandler::HandleNavigationButtonClicked(
+    const base::ListValue* args) {
+  Browser* browser = signin::GetDesktopBrowser(web_ui());
+  DCHECK(browser);
+
+  browser->signin_view_controller()->delegate()->PerformNavigation();
+}
+
+void InlineLoginHandler::HandleDialogClose(const base::ListValue* args) {
+  Browser* browser = signin::GetDesktopBrowser(web_ui());
+  // If the dialog was opened in the User Manager browser will be null here.
+  if (browser)
+    browser->CloseModalSigninWindow();
+
+  // Does nothing if user manager is not showing.
+  UserManager::HideReauthDialog();
 }
