@@ -5,28 +5,31 @@
 #include "chrome/browser/ui/webui/signin/user_manager_screen_handler.h"
 
 #include <stddef.h>
+
 #include <utility>
+#include <vector>
 
 #include "base/bind.h"
+#include "base/feature_list.h"
 #include "base/location.h"
 #include "base/macros.h"
-#include "base/prefs/pref_service.h"
 #include "base/profiler/scoped_tracker.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/value_conversions.h"
 #include "base/values.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/chrome_notification_types.h"
 #include "chrome/browser/extensions/api/screenlock_private/screenlock_private_api.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/profiles/profile_attributes_entry.h"
+#include "chrome/browser/profiles/profile_attributes_storage.h"
 #include "chrome/browser/profiles/profile_avatar_icon_util.h"
-#include "chrome/browser/profiles/profile_info_cache.h"
-#include "chrome/browser/profiles/profile_info_cache_observer.h"
 #include "chrome/browser/profiles/profile_manager.h"
 #include "chrome/browser/profiles/profile_metrics.h"
 #include "chrome/browser/profiles/profile_statistics.h"
+#include "chrome/browser/profiles/profile_statistics_factory.h"
 #include "chrome/browser/profiles/profile_window.h"
 #include "chrome/browser/profiles/profiles_state.h"
 #include "chrome/browser/signin/local_auth.h"
@@ -39,18 +42,24 @@
 #include "chrome/browser/ui/chrome_pages.h"
 #include "chrome/browser/ui/singleton_tabs.h"
 #include "chrome/browser/ui/user_manager.h"
+#include "chrome/browser/ui/webui/profile_helper.h"
+#include "chrome/common/chrome_features.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/grit/chromium_strings.h"
 #include "chrome/grit/generated_resources.h"
+#include "components/prefs/pref_service.h"
 #include "components/proximity_auth/screenlock_bridge.h"
 #include "components/signin/core/account_id/account_id.h"
+#include "components/signin/core/common/profile_management_switches.h"
+#include "components/strings/grit/components_strings.h"
 #include "content/public/browser/notification_service.h"
+#include "content/public/browser/storage_partition.h"
+#include "content/public/browser/user_metrics.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/browser/web_ui.h"
 #include "google_apis/gaia/gaia_auth_fetcher.h"
 #include "google_apis/gaia/gaia_constants.h"
-#include "grit/components_strings.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
@@ -60,7 +69,7 @@
 #include "ui/gfx/image/image_util.h"
 
 #if defined(USE_ASH)
-#include "ash/shell.h"
+#include "ash/shell.h"  // nogncheck
 #endif
 
 namespace {
@@ -83,7 +92,6 @@ const char kKeyIsProfileLoaded[] = "isProfileLoaded";
 
 // JS API callback names.
 const char kJsApiUserManagerInitialize[] = "userManagerInitialize";
-const char kJsApiUserManagerAddUser[] = "addUser";
 const char kJsApiUserManagerAuthLaunchUser[] = "authenticatedLaunchUser";
 const char kJsApiUserManagerLaunchGuest[] = "launchGuest";
 const char kJsApiUserManagerLaunchUser[] = "launchUser";
@@ -95,38 +103,22 @@ const char kJsApiUserManagerRemoveUserWarningLoadStats[] =
     "removeUserWarningLoadStats";
 const char kJsApiUserManagerGetRemoveWarningDialogMessage[] =
     "getRemoveWarningDialogMessage";
+const char kJsApiUserManagerAreAllProfilesLocked[] =
+    "areAllProfilesLocked";
 const size_t kAvatarIconSize = 180;
 const int kMaxOAuthRetries = 3;
 
 void HandleAndDoNothing(const base::ListValue* args) {
 }
 
-// This callback is run if the only profile has been deleted, and a new
-// profile has been created to replace it.
-void OpenNewWindowForProfile(
-    chrome::HostDesktopType desktop_type,
-    Profile* profile,
-    Profile::CreateStatus status) {
-  if (status != Profile::CREATE_STATUS_INITIALIZED)
-    return;
-  profiles::FindOrCreateNewWindowForProfile(
-    profile,
-    chrome::startup::IS_PROCESS_STARTUP,
-    chrome::startup::IS_FIRST_RUN,
-    desktop_type,
-    false);
-}
-
-std::string GetAvatarImageAtIndex(
-    size_t index, ProfileInfoCache* info_cache) {
-  bool is_gaia_picture =
-      info_cache->IsUsingGAIAPictureOfProfileAtIndex(index) &&
-      info_cache->GetGAIAPictureOfProfileAtIndex(index);
+std::string GetAvatarImage(const ProfileAttributesEntry* entry) {
+  bool is_gaia_picture = entry->IsUsingGAIAPicture() &&
+                         entry->GetGAIAPicture() != nullptr;
 
   // If the avatar is too small (i.e. the old-style low resolution avatar),
   // it will be pixelated when displayed in the User Manager, so we should
   // return the placeholder avatar instead.
-  gfx::Image avatar_image = info_cache->GetAvatarIconOfProfileAtIndex(index);
+  gfx::Image avatar_image = entry->GetAvatarIcon();
   if (avatar_image.Width() <= profiles::kAvatarIconWidth ||
       avatar_image.Height() <= profiles::kAvatarIconHeight ) {
     avatar_image = ui::ResourceBundle::GetSharedInstance().GetImageNamed(
@@ -175,7 +167,6 @@ class UrlHashHelper : public chrome::BrowserListObserver {
  private:
   Browser* browser_;
   Profile* profile_;
-  chrome::HostDesktopType desktop_type_;
   std::string hash_;
 
   DISALLOW_COPY_AND_ASSIGN(UrlHashHelper);
@@ -184,7 +175,6 @@ class UrlHashHelper : public chrome::BrowserListObserver {
 UrlHashHelper::UrlHashHelper(Browser* browser, const std::string& hash)
     : browser_(browser),
       profile_(browser->profile()),
-      desktop_type_(browser->host_desktop_type()),
       hash_(hash) {
   BrowserList::AddObserver(this);
 }
@@ -200,14 +190,14 @@ void UrlHashHelper::OnBrowserRemoved(Browser* browser) {
 
 void UrlHashHelper::ExecuteUrlHash() {
   if (hash_ == profiles::kUserManagerSelectProfileAppLauncher) {
-    AppListService* app_list_service = AppListService::Get(desktop_type_);
+    AppListService* app_list_service = AppListService::Get();
     app_list_service->ShowForProfile(profile_);
     return;
   }
 
   Browser* target_browser = browser_;
   if (!target_browser) {
-    target_browser = chrome::FindLastActiveWithProfile(profile_, desktop_type_);
+    target_browser = chrome::FindLastActiveWithProfile(profile_);
     if (!target_browser)
       return;
   }
@@ -218,8 +208,6 @@ void UrlHashHelper::ExecuteUrlHash() {
     chrome::ShowAboutChrome(target_browser);
   else if (hash_ == profiles::kUserManagerSelectProfileChromeSettings)
     chrome::ShowSettings(target_browser);
-  else if (hash_ == profiles::kUserManagerSelectProfileChromeMemory)
-    chrome::ShowMemory(target_browser);
 }
 
 void HandleLogRemoveUserWarningShown(const base::ListValue* args) {
@@ -232,7 +220,7 @@ void HandleLogRemoveUserWarningShown(const base::ListValue* args) {
 // ProfileUpdateObserver ------------------------------------------------------
 
 class UserManagerScreenHandler::ProfileUpdateObserver
-    : public ProfileInfoCacheObserver {
+    : public ProfileAttributesStorage::Observer {
  public:
   ProfileUpdateObserver(
       ProfileManager* profile_manager, UserManagerScreenHandler* handler)
@@ -240,16 +228,16 @@ class UserManagerScreenHandler::ProfileUpdateObserver
         user_manager_handler_(handler) {
     DCHECK(profile_manager_);
     DCHECK(user_manager_handler_);
-    profile_manager_->GetProfileInfoCache().AddObserver(this);
+    profile_manager_->GetProfileAttributesStorage().AddObserver(this);
   }
 
   ~ProfileUpdateObserver() override {
     DCHECK(profile_manager_);
-    profile_manager_->GetProfileInfoCache().RemoveObserver(this);
+    profile_manager_->GetProfileAttributesStorage().RemoveObserver(this);
   }
 
  private:
-  // ProfileInfoCacheObserver implementation:
+  // ProfileAttributesStorage::Observer implementation:
   // If any change has been made to a profile, propagate it to all the
   // visible user manager screens.
   void OnProfileAdded(const base::FilePath& profile_path) override {
@@ -301,12 +289,28 @@ class UserManagerScreenHandler::ProfileUpdateObserver
 
 // UserManagerScreenHandler ---------------------------------------------------
 
-UserManagerScreenHandler::UserManagerScreenHandler()
-    : desktop_type_(chrome::GetActiveDesktop()),
-      weak_ptr_factory_(this) {
-  profileInfoCacheObserver_.reset(
+UserManagerScreenHandler::UserManagerScreenHandler() : weak_ptr_factory_(this) {
+  profile_attributes_storage_observer_.reset(
       new UserManagerScreenHandler::ProfileUpdateObserver(
           g_browser_process->profile_manager(), this));
+
+  // TODO(mahmadi): Remove the following once prefs are cleared for everyone.
+  PrefService* service = g_browser_process->local_state();
+  DCHECK(service);
+
+  const PrefService::Preference* guest_mode_enabled_pref =
+      service->FindPreference(prefs::kBrowserGuestModeEnabled);
+  const PrefService::Preference* add_person_enabled_pref =
+      service->FindPreference(prefs::kBrowserAddPersonEnabled);
+
+  if (base::FeatureList::IsEnabled(features::kMaterialDesignSettings) &&
+      (guest_mode_enabled_pref->HasUserSetting() ||
+       add_person_enabled_pref->HasUserSetting())) {
+    service->ClearPref(guest_mode_enabled_pref->name());
+    service->ClearPref(add_person_enabled_pref->name());
+    content::RecordAction(
+        base::UserMetricsAction("UserManager_Cleared_Legacy_User_Prefs"));
+  }
 }
 
 UserManagerScreenHandler::~UserManagerScreenHandler() {
@@ -315,7 +319,7 @@ UserManagerScreenHandler::~UserManagerScreenHandler() {
 
 void UserManagerScreenHandler::ShowBannerMessage(
     const base::string16& message) {
-  web_ui()->CallJavascriptFunction(
+  web_ui()->CallJavascriptFunctionUnsafe(
       "login.AccountPickerScreen.showBannerMessage",
       base::StringValue(message));
 }
@@ -324,17 +328,18 @@ void UserManagerScreenHandler::ShowUserPodCustomIcon(
     const AccountId& account_id,
     const proximity_auth::ScreenlockBridge::UserPodCustomIconOptions&
         icon_options) {
-  scoped_ptr<base::DictionaryValue> icon = icon_options.ToDictionaryValue();
+  std::unique_ptr<base::DictionaryValue> icon =
+      icon_options.ToDictionaryValue();
   if (!icon || icon->empty())
     return;
-  web_ui()->CallJavascriptFunction(
+  web_ui()->CallJavascriptFunctionUnsafe(
       "login.AccountPickerScreen.showUserPodCustomIcon",
       base::StringValue(account_id.GetUserEmail()), *icon);
 }
 
 void UserManagerScreenHandler::HideUserPodCustomIcon(
     const AccountId& account_id) {
-  web_ui()->CallJavascriptFunction(
+  web_ui()->CallJavascriptFunctionUnsafe(
       "login.AccountPickerScreen.hideUserPodCustomIcon",
       base::StringValue(account_id.GetUserEmail()));
 }
@@ -352,10 +357,10 @@ void UserManagerScreenHandler::SetAuthType(
     return;
 
   user_auth_type_map_[account_id.GetUserEmail()] = auth_type;
-  web_ui()->CallJavascriptFunction("login.AccountPickerScreen.setAuthType",
-                                   base::StringValue(account_id.GetUserEmail()),
-                                   base::FundamentalValue(auth_type),
-                                   base::StringValue(auth_value));
+  web_ui()->CallJavascriptFunctionUnsafe(
+      "login.AccountPickerScreen.setAuthType",
+      base::StringValue(account_id.GetUserEmail()),
+      base::FundamentalValue(auth_type), base::StringValue(auth_value));
 }
 
 proximity_auth::ScreenlockBridge::LockHandler::AuthType
@@ -391,28 +396,14 @@ void UserManagerScreenHandler::HandleInitialize(const base::ListValue* args) {
   args->GetString(0, &url_hash_);
 
   SendUserList();
-  web_ui()->CallJavascriptFunction("cr.ui.Oobe.showUserManagerScreen",
+  web_ui()->CallJavascriptFunctionUnsafe(
+      "cr.ui.UserManager.showUserManagerScreen",
       base::FundamentalValue(IsGuestModeEnabled()),
       base::FundamentalValue(IsAddPersonEnabled()));
-  desktop_type_ = chrome::GetHostDesktopTypeForNativeView(
-      web_ui()->GetWebContents()->GetNativeView());
 
   proximity_auth::ScreenlockBridge::Get()->SetLockHandler(this);
 }
 
-void UserManagerScreenHandler::HandleAddUser(const base::ListValue* args) {
-  if (!IsAddPersonEnabled()) {
-    // The 'Add User' UI should not be showing.
-    NOTREACHED();
-    return;
-  }
-  profiles::CreateAndSwitchToNewProfile(
-      desktop_type_,
-      base::Bind(&UserManagerScreenHandler::OnSwitchToProfileComplete,
-                 weak_ptr_factory_.GetWeakPtr()),
-      ProfileMetrics::ADD_NEW_USER_MANAGER);
-}
-
 void UserManagerScreenHandler::HandleAuthenticatedLaunchUser(
     const base::ListValue* args) {
   const base::Value* profile_path_value;
@@ -423,12 +414,11 @@ void UserManagerScreenHandler::HandleAuthenticatedLaunchUser(
   if (!base::GetValueAsFilePath(*profile_path_value, &profile_path))
     return;
 
-  ProfileInfoCache& info_cache =
-      g_browser_process->profile_manager()->GetProfileInfoCache();
-
   ProfileAttributesEntry* entry;
-  if (!info_cache.GetProfileAttributesWithPath(profile_path, &entry))
+  if (!g_browser_process->profile_manager()->GetProfileAttributesStorage().
+          GetProfileAttributesWithPath(profile_path, &entry)) {
     return;
+  }
 
   base::string16 email_address;
   if (!args->GetString(1, &email_address))
@@ -443,12 +433,8 @@ void UserManagerScreenHandler::HandleAuthenticatedLaunchUser(
 
   // Only try to validate locally or check the password change detection
   // if we actually have a local credential saved.
-  size_t profile_index = info_cache.GetIndexOfProfileWithPath(profile_path);
-  const bool has_local_credential =
-      !info_cache.GetLocalAuthCredentialsOfProfileAtIndex(profile_index)
-          .empty();
-  if (has_local_credential) {
-    if (LocalAuth::ValidateLocalAuthCredentials(profile_index, password)) {
+  if (!entry->GetLocalAuthCredentials().empty()) {
+    if (LocalAuth::ValidateLocalAuthCredentials(entry, password)) {
       ReportAuthenticationResult(true, ProfileMetrics::AUTH_LOCAL);
       return;
     }
@@ -458,8 +444,9 @@ void UserManagerScreenHandler::HandleAuthenticatedLaunchUser(
     // change makes use of a token so we do that... if it's available.
     if (!oauth_client_) {
       oauth_client_.reset(new gaia::GaiaOAuthClient(
-          web_ui()->GetWebContents()->GetBrowserContext()
-              ->GetRequestContext()));
+          content::BrowserContext::GetDefaultStoragePartition(
+              web_ui()->GetWebContents()->GetBrowserContext())->
+                  GetURLRequestContext()));
     }
 
     const std::string token = entry->GetPasswordChangeDetectionToken();
@@ -469,10 +456,18 @@ void UserManagerScreenHandler::HandleAuthenticatedLaunchUser(
     }
   }
 
-  // In order to support the upgrade case where we have a local hash but no
-  // password token, the user perform a full online reauth.
-  UserManager::ShowReauthDialog(web_ui()->GetWebContents()->GetBrowserContext(),
-                                email_address_);
+  content::BrowserContext* browser_context =
+      web_ui()->GetWebContents()->GetBrowserContext();
+
+  if (!email_address_.empty()) {
+    // In order to support the upgrade case where we have a local hash but no
+    // password token, the user must perform a full online reauth.
+    UserManager::ShowReauthDialog(browser_context, email_address_,
+                                  signin_metrics::Reason::REASON_UNLOCK);
+  } else {
+    // Fresh sign in via user manager without existing email address.
+    UserManager::ShowSigninDialog(browser_context, profile_path);
+  }
 }
 
 void UserManagerScreenHandler::HandleRemoveUser(const base::ListValue* args) {
@@ -489,22 +484,27 @@ void UserManagerScreenHandler::HandleRemoveUser(const base::ListValue* args) {
     return;
   }
 
-  if (!profiles::IsMultipleProfilesEnabled()) {
-    NOTREACHED();
+  DCHECK(profiles::IsMultipleProfilesEnabled());
+
+  if (profiles::AreAllNonChildNonSupervisedProfilesLocked()) {
+    web_ui()->CallJavascriptFunctionUnsafe(
+        "cr.webUIListenerCallback",
+        base::StringValue("show-error-dialog"),
+        base::StringValue(l10n_util::GetStringUTF8(
+            IDS_USER_MANAGER_REMOVE_PROFILE_PROFILES_LOCKED_ERROR)));
     return;
   }
 
-  g_browser_process->profile_manager()->ScheduleProfileForDeletion(
-      profile_path,
-      base::Bind(&OpenNewWindowForProfile, desktop_type_));
-  ProfileMetrics::LogProfileDeleteUser(
-      ProfileMetrics::DELETE_PROFILE_USER_MANAGER);
+  // The callback is run if the only profile has been deleted, and a new
+  // profile has been created to replace it.
+  webui::DeleteProfileAtPath(profile_path,
+                             web_ui(),
+                             ProfileMetrics::DELETE_PROFILE_USER_MANAGER);
 }
 
 void UserManagerScreenHandler::HandleLaunchGuest(const base::ListValue* args) {
   if (IsGuestModeEnabled()) {
     profiles::SwitchToGuestProfile(
-        desktop_type_,
         base::Bind(&UserManagerScreenHandler::OnSwitchToProfileComplete,
                    weak_ptr_factory_.GetWeakPtr()));
   } else {
@@ -514,6 +514,20 @@ void UserManagerScreenHandler::HandleLaunchGuest(const base::ListValue* args) {
   }
 }
 
+void UserManagerScreenHandler::HandleAreAllProfilesLocked(
+    const base::ListValue* args) {
+  std::string webui_callback_id;
+  CHECK_EQ(1U, args->GetSize());
+  bool success = args->GetString(0, &webui_callback_id);
+  DCHECK(success);
+
+  AllowJavascript();
+  ResolveJavascriptCallback(
+      base::StringValue(webui_callback_id),
+      base::FundamentalValue(
+          profiles::AreAllNonChildNonSupervisedProfilesLocked()));
+}
+
 void UserManagerScreenHandler::HandleLaunchUser(const base::ListValue* args) {
   const base::Value* profile_path_value = NULL;
   if (!args->Get(0, &profile_path_value))
@@ -523,11 +537,9 @@ void UserManagerScreenHandler::HandleLaunchUser(const base::ListValue* args) {
   if (!base::GetValueAsFilePath(*profile_path_value, &profile_path))
     return;
 
-  const ProfileInfoCache& info_cache =
-      g_browser_process->profile_manager()->GetProfileInfoCache();
-  size_t profile_index = info_cache.GetIndexOfProfileWithPath(profile_path);
-
-  if (profile_index == std::string::npos) {
+  ProfileAttributesEntry* entry;
+  if (!g_browser_process->profile_manager()->GetProfileAttributesStorage().
+          GetProfileAttributesWithPath(profile_path, &entry)) {
     NOTREACHED();
     return;
   }
@@ -537,14 +549,12 @@ void UserManagerScreenHandler::HandleLaunchUser(const base::ListValue* args) {
   // unauthenticated version of "launch" instead of the proper one.  Thus,
   // we have to validate in (secure) C++ code that it really is a profile
   // not needing authentication.  If it is, just ignore the "launch" request.
-  if (info_cache.ProfileIsSigninRequiredAtIndex(profile_index))
+  if (entry->IsSigninRequired())
     return;
   ProfileMetrics::LogProfileAuthResult(ProfileMetrics::AUTH_UNNECESSARY);
 
   profiles::SwitchToProfile(
-      profile_path,
-      desktop_type_,
-      false,  /* reuse any existing windows */
+      profile_path, false, /* reuse any existing windows */
       base::Bind(&UserManagerScreenHandler::OnSwitchToProfileComplete,
                  weak_ptr_factory_.GetWeakPtr()),
       ProfileMetrics::SWITCH_PROFILE_MANAGER);
@@ -589,37 +599,37 @@ void UserManagerScreenHandler::HandleRemoveUserWarningLoadStats(
   if (!profile)
     return;
 
-  if (!chrome::FindAnyBrowser(profile, true, desktop_type_)) {
+  if (!chrome::FindAnyBrowser(profile, true)) {
     // If no windows are open for that profile, the statistics in
-    // ProfileInfoCache are up to date. The statistics in ProfileInfoCache are
-    // returned because the copy in user_pod_row.js may be outdated. However, if
-    // some statistics are missing in ProfileInfoCache (i.e. |item.success| is
-    // false), then the actual statistics are queried instead.
+    // ProfileAttributesStorage are up to date. The statistics in
+    // ProfileAttributesStorage are returned because the copy in user_pod_row.js
+    // may be outdated. However, if some statistics are missing in
+    // ProfileAttributesStorage (i.e. |item.success| is false), then the actual
+    // statistics are queried instead.
     base::DictionaryValue return_value;
     profiles::ProfileCategoryStats stats =
-        profiles::GetProfileStatisticsFromCache(profile_path);
+        ProfileStatistics::GetProfileStatisticsFromAttributesStorage(
+            profile_path);
     bool stats_success = true;
     for (const auto& item : stats) {
-      scoped_ptr<base::DictionaryValue> stat(new base::DictionaryValue);
+      std::unique_ptr<base::DictionaryValue> stat(new base::DictionaryValue);
       stat->SetIntegerWithoutPathExpansion("count", item.count);
       stat->SetBooleanWithoutPathExpansion("success", item.success);
       return_value.SetWithoutPathExpansion(item.category, std::move(stat));
       stats_success &= item.success;
     }
     if (stats_success) {
-      web_ui()->CallJavascriptFunction("updateRemoveWarningDialog",
-                                       base::StringValue(profile_path.value()),
-                                       return_value);
+      web_ui()->CallJavascriptFunctionUnsafe(
+          "updateRemoveWarningDialog", base::StringValue(profile_path.value()),
+          return_value);
       return;
     }
   }
 
-  profiles::GatherProfileStatistics(
-      profile,
+  ProfileStatisticsFactory::GetForProfile(profile)->GatherStatistics(
       base::Bind(
           &UserManagerScreenHandler::RemoveUserDialogLoadStatsCallback,
-          weak_ptr_factory_.GetWeakPtr(), profile_path),
-      &tracker_);
+          weak_ptr_factory_.GetWeakPtr(), profile_path));
 }
 
 void UserManagerScreenHandler::RemoveUserDialogLoadStatsCallback(
@@ -628,14 +638,14 @@ void UserManagerScreenHandler::RemoveUserDialogLoadStatsCallback(
   // Copy result into return_value.
   base::DictionaryValue return_value;
   for (const auto& item : result) {
-    scoped_ptr<base::DictionaryValue> stat(new base::DictionaryValue);
+    std::unique_ptr<base::DictionaryValue> stat(new base::DictionaryValue);
     stat->SetIntegerWithoutPathExpansion("count", item.count);
     stat->SetBooleanWithoutPathExpansion("success", item.success);
     return_value.SetWithoutPathExpansion(item.category, std::move(stat));
   }
-  web_ui()->CallJavascriptFunction("updateRemoveWarningDialog",
-                                   base::StringValue(profile_path.value()),
-                                   return_value);
+  web_ui()->CallJavascriptFunctionUnsafe(
+      "updateRemoveWarningDialog", base::StringValue(profile_path.value()),
+      return_value);
 }
 
 void UserManagerScreenHandler::HandleGetRemoveWarningDialogMessage(
@@ -666,14 +676,13 @@ void UserManagerScreenHandler::HandleGetRemoveWarningDialogMessage(
   base::StringValue message = base::StringValue(
       l10n_util::GetPluralStringFUTF16(message_id, total_count));
 
-  web_ui()->CallJavascriptFunction("updateRemoveWarningDialogSetMessage",
-                                   base::StringValue(profile_path),
-                                   message,
-                                   base::FundamentalValue(total_count));
+  web_ui()->CallJavascriptFunctionUnsafe(
+      "updateRemoveWarningDialogSetMessage", base::StringValue(profile_path),
+      message, base::FundamentalValue(total_count));
 }
 
 void UserManagerScreenHandler::OnGetTokenInfoResponse(
-    scoped_ptr<base::DictionaryValue> token_info) {
+    std::unique_ptr<base::DictionaryValue> token_info) {
   // Password is unchanged so user just mistyped it.  Ask again.
   ReportAuthenticationResult(false, ProfileMetrics::AUTH_FAILED);
 }
@@ -683,7 +692,8 @@ void UserManagerScreenHandler::OnOAuthError() {
   DCHECK(!email_address_.empty());
   oauth_client_.reset();
   UserManager::ShowReauthDialog(web_ui()->GetWebContents()->GetBrowserContext(),
-                                email_address_);
+                                email_address_,
+                                signin_metrics::Reason::REASON_UNLOCK);
 }
 
 void UserManagerScreenHandler::OnNetworkError(int response_code) {
@@ -696,9 +706,6 @@ void UserManagerScreenHandler::RegisterMessages() {
   web_ui()->RegisterMessageCallback(kJsApiUserManagerInitialize,
       base::Bind(&UserManagerScreenHandler::HandleInitialize,
                  base::Unretained(this)));
-  web_ui()->RegisterMessageCallback(kJsApiUserManagerAddUser,
-      base::Bind(&UserManagerScreenHandler::HandleAddUser,
-                 base::Unretained(this)));
   web_ui()->RegisterMessageCallback(kJsApiUserManagerAuthLaunchUser,
       base::Bind(&UserManagerScreenHandler::HandleAuthenticatedLaunchUser,
                  base::Unretained(this)));
@@ -723,6 +730,10 @@ void UserManagerScreenHandler::RegisterMessages() {
       kJsApiUserManagerGetRemoveWarningDialogMessage,
       base::Bind(&UserManagerScreenHandler::HandleGetRemoveWarningDialogMessage,
                  base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      kJsApiUserManagerAreAllProfilesLocked,
+      base::Bind(&UserManagerScreenHandler::HandleAreAllProfilesLocked,
+                 base::Unretained(this)));
 
   const content::WebUI::MessageCallback& kDoNothingCallback =
       base::Bind(&HandleAndDoNothing);
@@ -806,6 +817,9 @@ void UserManagerScreenHandler::GetLocalizedValues(
           IDS_LOGIN_POD_LEGACY_SUPERVISED_USER_REMOVE_WARNING,
           base::UTF8ToUTF16(
               chrome::kLegacySupervisedUserManagementDisplayURL)));
+  localized_strings->SetString(
+      "removeNonOwnerUserWarningText",
+      l10n_util::GetStringUTF16(IDS_LOGIN_POD_NON_OWNER_USER_REMOVE_WARNING));
 
   // Strings needed for the User Manager tutorial slides.
   localized_strings->SetString("tutorialNext",
@@ -848,6 +862,18 @@ void UserManagerScreenHandler::GetLocalizedValues(
   localized_strings->SetString("publicAccountEnter", base::string16());
   localized_strings->SetString("publicAccountEnterAccessibleName",
                                base::string16());
+  localized_strings->SetString("publicAccountMonitoringWarning",
+                               base::string16());
+  localized_strings->SetString("publicAccountLearnMore", base::string16());
+  localized_strings->SetString("publicAccountMonitoringInfo", base::string16());
+  localized_strings->SetString("publicAccountMonitoringInfoItem1",
+                               base::string16());
+  localized_strings->SetString("publicAccountMonitoringInfoItem2",
+                               base::string16());
+  localized_strings->SetString("publicAccountMonitoringInfoItem3",
+                               base::string16());
+  localized_strings->SetString("publicAccountMonitoringInfoItem4",
+                               base::string16());
   localized_strings->SetString("publicSessionSelectLanguage", base::string16());
   localized_strings->SetString("publicSessionSelectKeyboard", base::string16());
   localized_strings->SetString("signinBannerText", base::string16());
@@ -860,12 +886,22 @@ void UserManagerScreenHandler::GetLocalizedValues(
                                 base::string16());
   localized_strings->SetString("multiProfilesOwnerPrimaryOnlyMsg",
                                 base::string16());
+
+  // Error message when trying to add a profile while all profiles are locked.
+  localized_strings->SetString("addProfileAllProfilesLockedError",
+      l10n_util::GetStringUTF16(
+          IDS_USER_MANAGER_ADD_PROFILE_PROFILES_LOCKED_ERROR));
+  // Error message when trying to browse as guest while all profiles are locked.
+  localized_strings->SetString("browseAsGuestAllProfilesLockedError",
+      l10n_util::GetStringUTF16(
+          IDS_USER_MANAGER_GO_GUEST_PROFILES_LOCKED_ERROR));
 }
 
 void UserManagerScreenHandler::SendUserList() {
   base::ListValue users_list;
-  ProfileInfoCache* info_cache =
-      &g_browser_process->profile_manager()->GetProfileInfoCache();
+  std::vector<ProfileAttributesEntry*> entries =
+      g_browser_process->profile_manager()->GetProfileAttributesStorage().
+          GetAllProfilesAttributesSortedByName();
   user_auth_type_map_.clear();
 
   // Profile deletion is not allowed in Metro mode.
@@ -874,45 +910,41 @@ void UserManagerScreenHandler::SendUserList() {
   can_remove = !ash::Shell::HasInstance();
 #endif
 
-  for (size_t i = 0; i < info_cache->GetNumberOfProfiles(); ++i) {
+  for (const ProfileAttributesEntry* entry : entries) {
     // Don't show profiles still in the middle of being set up as new legacy
     // supervised users.
-    if (info_cache->IsOmittedProfileAtIndex(i))
+    if (entry->IsOmitted())
       continue;
 
-    base::DictionaryValue* profile_value = new base::DictionaryValue();
-    base::FilePath profile_path = info_cache->GetPathOfProfileAtIndex(i);
+    std::unique_ptr<base::DictionaryValue> profile_value(
+        new base::DictionaryValue());
+    base::FilePath profile_path = entry->GetPath();
 
-    profile_value->SetString(
-        kKeyUsername, info_cache->GetUserNameOfProfileAtIndex(i));
-    profile_value->SetString(
-        kKeyEmailAddress, info_cache->GetUserNameOfProfileAtIndex(i));
-    profile_value->SetString(
-        kKeyDisplayName,
-        profiles::GetAvatarNameForProfile(profile_path));
-    profile_value->Set(
-        kKeyProfilePath, base::CreateFilePathValue(profile_path));
+    profile_value->SetString(kKeyUsername, entry->GetUserName());
+    profile_value->SetString(kKeyEmailAddress, entry->GetUserName());
+    profile_value->SetString(kKeyDisplayName,
+                             profiles::GetAvatarNameForProfile(profile_path));
+    profile_value->Set(kKeyProfilePath,
+                       base::CreateFilePathValue(profile_path));
     profile_value->SetBoolean(kKeyPublicAccount, false);
     profile_value->SetBoolean(kKeyLegacySupervisedUser,
-                              info_cache->ProfileIsLegacySupervisedAtIndex(i));
-    profile_value->SetBoolean(
-        kKeyChildUser, info_cache->ProfileIsChildAtIndex(i));
-    profile_value->SetBoolean(
-        kKeyNeedsSignin, info_cache->ProfileIsSigninRequiredAtIndex(i));
-    profile_value->SetBoolean(
-        kKeyHasLocalCreds,
-        !info_cache->GetLocalAuthCredentialsOfProfileAtIndex(i).empty());
+                              entry->IsLegacySupervised());
+    profile_value->SetBoolean(kKeyChildUser, entry->IsChild());
+    profile_value->SetBoolean(kKeyNeedsSignin, entry->IsSigninRequired());
+    profile_value->SetBoolean(kKeyHasLocalCreds,
+                              !entry->GetLocalAuthCredentials().empty());
     profile_value->SetBoolean(kKeyIsOwner, false);
     profile_value->SetBoolean(kKeyCanRemove, can_remove);
     profile_value->SetBoolean(kKeyIsDesktop, true);
-    profile_value->SetString(
-        kKeyAvatarUrl, GetAvatarImageAtIndex(i, info_cache));
+    profile_value->SetString(kKeyAvatarUrl, GetAvatarImage(entry));
 
     profiles::ProfileCategoryStats stats =
-        profiles::GetProfileStatisticsFromCache(profile_path);
-    scoped_ptr<base::DictionaryValue> stats_dict(new base::DictionaryValue);
+        ProfileStatistics::GetProfileStatisticsFromAttributesStorage(
+            profile_path);
+    std::unique_ptr<base::DictionaryValue> stats_dict(
+        new base::DictionaryValue);
     for (const auto& item : stats) {
-      scoped_ptr<base::DictionaryValue> stat(new base::DictionaryValue);
+      std::unique_ptr<base::DictionaryValue> stat(new base::DictionaryValue);
       stat->SetIntegerWithoutPathExpansion("count", item.count);
       stat->SetBooleanWithoutPathExpansion("success", item.success);
       stats_dict->SetWithoutPathExpansion(item.category, std::move(stat));
@@ -926,11 +958,12 @@ void UserManagerScreenHandler::SendUserList() {
         g_browser_process->profile_manager()->GetProfileByPath(profile_path);
     profile_value->SetBoolean(kKeyIsProfileLoaded, profile != nullptr);
 
-    users_list.Append(profile_value);
+    users_list.Append(std::move(profile_value));
   }
 
-  web_ui()->CallJavascriptFunction("login.AccountPickerScreen.loadUsers",
-      users_list, base::FundamentalValue(IsGuestModeEnabled()));
+  web_ui()->CallJavascriptFunctionUnsafe(
+      "login.AccountPickerScreen.loadUsers", users_list,
+      base::FundamentalValue(IsGuestModeEnabled()));
 
   // This is the latest C++ code we have in the flow to show the UserManager.
   // This may be invoked more than once per UserManager lifetime; the
@@ -946,22 +979,18 @@ void UserManagerScreenHandler::ReportAuthenticationResult(
 
   if (success) {
     profiles::SwitchToProfile(
-        authenticating_profile_path_,
-        desktop_type_,
-        true,
+        authenticating_profile_path_, true,
         base::Bind(&UserManagerScreenHandler::OnSwitchToProfileComplete,
                    weak_ptr_factory_.GetWeakPtr()),
         ProfileMetrics::SWITCH_PROFILE_UNLOCK);
   } else {
-    web_ui()->CallJavascriptFunction(
-        "cr.ui.Oobe.showSignInError",
-        base::FundamentalValue(0),
+    web_ui()->CallJavascriptFunctionUnsafe(
+        "cr.ui.UserManager.showSignInError", base::FundamentalValue(0),
         base::StringValue(l10n_util::GetStringUTF8(
-            auth == ProfileMetrics::AUTH_FAILED_OFFLINE ?
-                IDS_LOGIN_ERROR_AUTHENTICATING_OFFLINE :
-                IDS_LOGIN_ERROR_AUTHENTICATING)),
-        base::StringValue(""),
-        base::FundamentalValue(0));
+            auth == ProfileMetrics::AUTH_FAILED_OFFLINE
+                ? IDS_LOGIN_ERROR_AUTHENTICATING_OFFLINE
+                : IDS_LOGIN_ERROR_AUTHENTICATING)),
+        base::StringValue(""), base::FundamentalValue(0));
   }
 }
 
@@ -972,11 +1001,12 @@ void UserManagerScreenHandler::OnBrowserWindowReady(Browser* browser) {
   // Unlock the profile after browser opens so startup can read the lock bit.
   // Any necessary authentication must have been successful to reach this point.
   if (!browser->profile()->IsGuestSession()) {
-    ProfileInfoCache& info_cache =
-        g_browser_process->profile_manager()->GetProfileInfoCache();
-    size_t index = info_cache.GetIndexOfProfileWithPath(
-        browser->profile()->GetPath());
-    info_cache.SetProfileSigninRequiredAtIndex(index, false);
+    ProfileAttributesEntry* entry = nullptr;
+    bool has_entry = g_browser_process->profile_manager()->
+        GetProfileAttributesStorage().
+        GetProfileAttributesWithPath(browser->profile()->GetPath(), &entry);
+    DCHECK(has_entry);
+    entry->SetIsSigninRequired(false);
   }
 
   if (!url_hash_.empty()) {
@@ -994,17 +1024,12 @@ void UserManagerScreenHandler::Observe(
     int type,
     const content::NotificationSource& source,
     const content::NotificationDetails& details) {
-  switch (type) {
-    case chrome::NOTIFICATION_BROWSER_WINDOW_READY:
-      // Only respond to one Browser Window Ready event.
-      registrar_.Remove(this,
-                        chrome::NOTIFICATION_BROWSER_WINDOW_READY,
-                        content::NotificationService::AllSources());
-      OnBrowserWindowReady(content::Source<Browser>(source).ptr());
-    break;
-    default:
-      NOTREACHED();
-  }
+  DCHECK_EQ(chrome::NOTIFICATION_BROWSER_WINDOW_READY, type);
+
+  // Only respond to one Browser Window Ready event.
+  registrar_.Remove(this, chrome::NOTIFICATION_BROWSER_WINDOW_READY,
+                    content::NotificationService::AllSources());
+  OnBrowserWindowReady(content::Source<Browser>(source).ptr());
 }
 
 // This callback is run after switching to a new profile has finished. This
@@ -1014,7 +1039,7 @@ void UserManagerScreenHandler::Observe(
 // cause Chrome to close.
 void UserManagerScreenHandler::OnSwitchToProfileComplete(
     Profile* profile, Profile::CreateStatus profile_create_status) {
-  Browser* browser = chrome::FindAnyBrowser(profile, false, desktop_type_);
+  Browser* browser = chrome::FindAnyBrowser(profile, false);
   if (browser && browser->window()) {
     OnBrowserWindowReady(browser);
   } else {
