@@ -8,6 +8,7 @@
 #include <string>
 
 #include "base/compiler_specific.h"
+#include "base/macros.h"
 #include "base/memory/ref_counted.h"
 #include "content/public/browser/url_data_source.h"
 
@@ -26,36 +27,29 @@ class ThemeSource : public content::URLDataSource {
   std::string GetSource() const override;
   void StartDataRequest(
       const std::string& path,
-      int render_process_id,
-      int render_frame_id,
+      const content::ResourceRequestInfo::WebContentsGetter& wc_getter,
       const content::URLDataSource::GotDataCallback& callback) override;
   std::string GetMimeType(const std::string& path) const override;
-  base::MessageLoop* MessageLoopForRequestPath(
+  scoped_refptr<base::SingleThreadTaskRunner> TaskRunnerForRequestPath(
       const std::string& path) const override;
-  bool ShouldReplaceExistingSource() const override;
   bool ShouldServiceRequest(const net::URLRequest* request) const override;
 
  private:
-  // Fetch and send the theme bitmap.
+  // Fetches and sends the theme bitmap.
   void SendThemeBitmap(const content::URLDataSource::GotDataCallback& callback,
                        int resource_id,
-                       float scale_factor);
+                       float scale);
 
-  // Similar to SendThemeBitmap but treat the responded data as image; if the
-  // resource bundle does not contain the data for |scale_factor|, the resource
-  // bundle falls back to the data of a lower scale, which means smaller images
-  // will be served and webui handles the image incorrectly.
-  // See crbug.com/442384.
+  // Used in place of SendThemeBitmap when the desired scale is larger than
+  // what the resource bundle supports.  This can rescale the provided bitmap up
+  // to the desired size.
   void SendThemeImage(const content::URLDataSource::GotDataCallback& callback,
                       int resource_id,
-                      float scale_factor);
+                      float scale);
 
-  // The original profile (never an OTR profile).
+  // The profile this object was initialized with.
   Profile* profile_;
 
-  // We grab the CSS early so we don't have to go back to the UI thread later.
-  scoped_refptr<base::RefCountedMemory> css_bytes_;
-
   DISALLOW_COPY_AND_ASSIGN(ThemeSource);
 };
 
