@@ -4,11 +4,12 @@
 
 #include "chrome/common/chrome_content_client.h"
 
-#include <string.h>
+#include <string>
 
-#include "base/command_line.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/strings/string_split.h"
+#include "base/strings/string_util.h"
+#include "base/test/scoped_command_line.h"
 #include "build/build_config.h"
 #include "content/public/common/content_switches.h"
 #include "extensions/common/constants.h"
@@ -26,41 +27,49 @@ void CheckUserAgentStringOrdering(bool mobile_device) {
   ChromeContentClient content_client;
   std::string buffer = content_client.GetUserAgent();
 
-  base::SplitStringUsingSubstr(buffer, "Mozilla/5.0 (", &pieces);
+  pieces = base::SplitStringUsingSubstr(buffer, "Mozilla/5.0 (",
+                                        base::TRIM_WHITESPACE,
+                                        base::SPLIT_WANT_ALL);
   ASSERT_EQ(2u, pieces.size());
   buffer = pieces[1];
   EXPECT_EQ("", pieces[0]);
 
-  base::SplitStringUsingSubstr(buffer, ") AppleWebKit/", &pieces);
+  pieces = base::SplitStringUsingSubstr(buffer, ") AppleWebKit/",
+                                        base::TRIM_WHITESPACE,
+                                        base::SPLIT_WANT_ALL);
   ASSERT_EQ(2u, pieces.size());
   buffer = pieces[1];
   std::string os_str = pieces[0];
 
-  base::SplitStringUsingSubstr(buffer, " (KHTML, like Gecko) ", &pieces);
+  pieces = base::SplitStringUsingSubstr(buffer, " (KHTML, like Gecko) ",
+                                        base::TRIM_WHITESPACE,
+                                        base::SPLIT_WANT_ALL);
   ASSERT_EQ(2u, pieces.size());
   buffer = pieces[1];
   std::string webkit_version_str = pieces[0];
 
-  base::SplitStringUsingSubstr(buffer, " Safari/", &pieces);
+  pieces = base::SplitStringUsingSubstr(buffer, " Safari/",
+                                        base::TRIM_WHITESPACE,
+                                        base::SPLIT_WANT_ALL);
   ASSERT_EQ(2u, pieces.size());
   std::string product_str = pieces[0];
   std::string safari_version_str = pieces[1];
 
   // Not sure what can be done to better check the OS string, since it's highly
   // platform-dependent.
-  EXPECT_TRUE(os_str.size() > 0);
+  EXPECT_FALSE(os_str.empty());
 
   // Check that the version numbers match.
-  EXPECT_TRUE(webkit_version_str.size() > 0);
-  EXPECT_TRUE(safari_version_str.size() > 0);
+  EXPECT_FALSE(webkit_version_str.empty());
+  EXPECT_FALSE(safari_version_str.empty());
   EXPECT_EQ(webkit_version_str, safari_version_str);
 
-  EXPECT_EQ(0u, product_str.find("Chrome/"));
+  EXPECT_TRUE(
+      base::StartsWith(product_str, "Chrome/", base::CompareCase::SENSITIVE));
   if (mobile_device) {
     // "Mobile" gets tacked on to the end for mobile devices, like phones.
-    const std::string kMobileStr = " Mobile";
-    EXPECT_EQ(kMobileStr,
-              product_str.substr(product_str.size() - kMobileStr.size()));
+    EXPECT_TRUE(
+        base::EndsWith(product_str, " Mobile", base::CompareCase::SENSITIVE));
   }
 }
 
@@ -70,13 +79,11 @@ void CheckUserAgentStringOrdering(bool mobile_device) {
 namespace chrome_common {
 
 TEST(ChromeContentClientTest, Basic) {
-#if !defined(OS_ANDROID)
-  CheckUserAgentStringOrdering(false);
-#else
+#if defined(OS_ANDROID)
   const char* const kArguments[] = {"chrome"};
-  base::CommandLine::Reset();
-  base::CommandLine::Init(1, kArguments);
-  base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
+  base::test::ScopedCommandLine scoped_command_line;
+  base::CommandLine* command_line = scoped_command_line.GetProcessCommandLine();
+  command_line->InitFromArgv(1, kArguments);
 
   // Do it for regular devices.
   ASSERT_FALSE(command_line->HasSwitch(switches::kUseMobileUserAgent));
@@ -86,80 +93,76 @@ TEST(ChromeContentClientTest, Basic) {
   command_line->AppendSwitch(switches::kUseMobileUserAgent);
   ASSERT_TRUE(command_line->HasSwitch(switches::kUseMobileUserAgent));
   CheckUserAgentStringOrdering(true);
+#else
+  CheckUserAgentStringOrdering(false);
 #endif
 }
 
 #if defined(ENABLE_PLUGINS)
 TEST(ChromeContentClientTest, FindMostRecent) {
-  std::vector<content::PepperPluginInfo*> version_vector;
+  std::vector<std::unique_ptr<content::PepperPluginInfo>> version_vector;
   // Test an empty vector.
   EXPECT_EQ(nullptr, ChromeContentClient::FindMostRecentPlugin(version_vector));
 
   // Now test the vector with one element.
-  content::PepperPluginInfo info1;
-  info1.version = "1.0.0.0";
-  version_vector.push_back(&info1);
+  content::PepperPluginInfo info;
+  info.version = "1.0.0.0";
+  version_vector.push_back(base::MakeUnique<content::PepperPluginInfo>(info));
 
   content::PepperPluginInfo* most_recent =
       ChromeContentClient::FindMostRecentPlugin(version_vector);
   EXPECT_EQ("1.0.0.0", most_recent->version);
 
-  // Now do the generic test of a complex vector.
-  content::PepperPluginInfo info2;
-  info2.version = "2.0.0.1";
-  content::PepperPluginInfo info3;
-  info3.version = "3.5.6.7";
-  content::PepperPluginInfo info4;
-  info4.version = "4.0.0.153";
   content::PepperPluginInfo info5;
   info5.version = "5.0.12.1";
   content::PepperPluginInfo info6_12;
   info6_12.version = "6.0.0.12";
   content::PepperPluginInfo info6_13;
   info6_13.version = "6.0.0.13";
-  content::PepperPluginInfo info6_13_d;
-  info6_13_d.version = "6.0.0.13";
-  info6_13_d.is_debug = true;
 
+  // Test highest version is picked.
   version_vector.clear();
-  version_vector.push_back(&info4);
-  version_vector.push_back(&info2);
-  version_vector.push_back(&info6_13);
-  version_vector.push_back(&info3);
-  version_vector.push_back(&info5);
-  version_vector.push_back(&info6_12);
-  version_vector.push_back(&info6_13_d);
+  version_vector.push_back(base::MakeUnique<content::PepperPluginInfo>(info5));
+  version_vector.push_back(
+      base::MakeUnique<content::PepperPluginInfo>(info6_12));
+  version_vector.push_back(
+      base::MakeUnique<content::PepperPluginInfo>(info6_13));
 
   most_recent = ChromeContentClient::FindMostRecentPlugin(version_vector);
   EXPECT_EQ("6.0.0.13", most_recent->version);
-  EXPECT_EQ(true, most_recent->is_debug);
 
-  // Check vector order doesn't matter.
+  // Test that order does not matter, validates tests below.
   version_vector.clear();
-  version_vector.push_back(&info6_13_d);
-  version_vector.push_back(&info6_12);
-  version_vector.push_back(&info5);
-  version_vector.push_back(&info3);
-  version_vector.push_back(&info6_13);
-  version_vector.push_back(&info2);
-  version_vector.push_back(&info4);
+  version_vector.push_back(
+      base::MakeUnique<content::PepperPluginInfo>(info6_13));
+  version_vector.push_back(
+      base::MakeUnique<content::PepperPluginInfo>(info6_12));
+  version_vector.push_back(base::MakeUnique<content::PepperPluginInfo>(info5));
 
   most_recent = ChromeContentClient::FindMostRecentPlugin(version_vector);
   EXPECT_EQ("6.0.0.13", most_recent->version);
-  EXPECT_EQ(true, most_recent->is_debug);
 
-  // Check higher versions still trump debugger.
-  content::PepperPluginInfo info5_d;
-  info5_d.version = "5.0.12.1";
-  info5_d.is_debug = true;
+  // Test real scenarios.
+  content::PepperPluginInfo component_flash;
+  component_flash.version = "4.3.2.1";
+  component_flash.is_external = false;
+  component_flash.name = "component_flash";
 
+  content::PepperPluginInfo system_flash;
+  system_flash.version = "4.3.2.1";
+  system_flash.is_external = true;
+  system_flash.name = "system_flash";
+
+  // The order here should be:
+  // 1. System Flash.
+  // 2. Component update.
   version_vector.clear();
-  version_vector.push_back(&info5_d);
-  version_vector.push_back(&info6_12);
-
+  version_vector.push_back(
+      base::MakeUnique<content::PepperPluginInfo>(system_flash));
+  version_vector.push_back(
+      base::MakeUnique<content::PepperPluginInfo>(component_flash));
   most_recent = ChromeContentClient::FindMostRecentPlugin(version_vector);
-  EXPECT_EQ("6.0.0.12", most_recent->version);
-  EXPECT_EQ(false, most_recent->is_debug);
+  EXPECT_STREQ("system_flash", most_recent->name.c_str());
 }
 #endif  // defined(ENABLE_PLUGINS)
 
