@@ -28,6 +28,7 @@ class Command {
           const base::string16& description,
           const std::string& accelerator,
           bool global);
+  Command(const Command& other);
   ~Command();
 
   // The platform value for the Command.
@@ -60,7 +61,7 @@ class Command {
   bool global() const { return global_; }
 
   // Setter:
-  void set_accelerator(ui::Accelerator accelerator) {
+  void set_accelerator(const ui::Accelerator& accelerator) {
     accelerator_ = accelerator;
   }
   void set_global(bool global) {
