@@ -4,24 +4,72 @@
 
 #include "chrome/installer/mini_installer/configuration.h"
 
+#include <stddef.h>
 #include <stdlib.h>
 
-#include "base/basictypes.h"
+#include <memory>
+
+#include "base/environment.h"
 #include "chrome/installer/mini_installer/appid.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 using mini_installer::Configuration;
 
+namespace {
+
+// A helper class to set the "GoogleUpdateIsMachine" environment variable.
+class ScopedGoogleUpdateIsMachine {
+ public:
+  explicit ScopedGoogleUpdateIsMachine(bool value)
+      : env_(base::Environment::Create()) {
+    env_->SetVar("GoogleUpdateIsMachine", value ? "1" : "0");
+  }
+
+  ~ScopedGoogleUpdateIsMachine() {
+    env_->UnSetVar("GoogleUpdateIsMachine");
+  }
+
+ private:
+  std::unique_ptr<base::Environment> env_;
+};
+
+}  // namespace
+
 class TestConfiguration : public Configuration {
  public:
-  explicit TestConfiguration(const wchar_t* command_line) : Configuration() {
+  explicit TestConfiguration(const wchar_t* command_line)
+      : Configuration(),
+        open_registry_key_result_(false),
+        read_registry_value_result_(0),
+        read_registry_value_(L"") {
     Initialize(command_line);
   }
+  explicit TestConfiguration(const wchar_t* command_line,
+                             LONG ret, const wchar_t* value)
+      : Configuration(),
+        open_registry_key_result_(true),
+        read_registry_value_result_(ret),
+        read_registry_value_(value) {
+    Initialize(command_line);
+  }
+  void SetRegistryResults(bool openkey, LONG ret, const wchar_t* value) {
+  }
  private:
+  bool open_registry_key_result_;
+  LONG read_registry_value_result_;
+  const wchar_t* read_registry_value_ = L"";
+
   void Initialize(const wchar_t* command_line) {
     Clear();
     ASSERT_TRUE(ParseCommandLine(command_line));
   }
+  bool ReadClientStateRegistryValue(
+      const HKEY root_key, const wchar_t* app_guid,
+      LONG* retval, ValueString& value) override {
+    *retval = read_registry_value_result_;
+    value.assign(read_registry_value_);
+    return open_registry_key_result_;
+  }
 };
 
 // Test that the operation type is CLEANUP iff --cleanup is on the cmdline.
@@ -72,13 +120,29 @@ TEST(MiniInstallerConfigurationTest, ChromeAppGuid) {
               TestConfiguration(L"spam.exe").chrome_app_guid());
   EXPECT_TRUE(std::wstring(google_update::kAppGuid) ==
               TestConfiguration(L"spam.exe --chrome").chrome_app_guid());
-  EXPECT_TRUE(std::wstring(google_update::kAppGuid) ==
-              TestConfiguration(L"spam.exe --multi-install --chrome")
-                  .chrome_app_guid());
-  EXPECT_TRUE(std::wstring(google_update::kAppGuid) ==
+  EXPECT_TRUE(std::wstring(google_update::kChromeFrameAppGuid) ==
               TestConfiguration(L"spam.exe --chrome-frame").chrome_app_guid());
   EXPECT_TRUE(std::wstring(google_update::kSxSAppGuid) ==
               TestConfiguration(L"spam.exe --chrome-sxs").chrome_app_guid());
+  EXPECT_TRUE(std::wstring(google_update::kMultiInstallAppGuid) ==
+              TestConfiguration(L"spam.exe --multi-install --chrome")
+                  .chrome_app_guid());
+  EXPECT_TRUE(std::wstring(google_update::kMultiInstallAppGuid) ==
+              TestConfiguration(L"spam.exe --multi-install --chrome",
+                                ERROR_INVALID_FUNCTION, L"")
+                  .chrome_app_guid());
+  EXPECT_TRUE(std::wstring(google_update::kAppGuid) ==
+              TestConfiguration(L"spam.exe --multi-install --chrome",
+                                ERROR_FILE_NOT_FOUND, L"")
+                  .chrome_app_guid());
+  EXPECT_TRUE(std::wstring(google_update::kAppGuid) ==
+              TestConfiguration(L"spam.exe --multi-install --chrome",
+                                ERROR_SUCCESS, L"foo-bar")
+                  .chrome_app_guid());
+  EXPECT_TRUE(std::wstring(google_update::kMultiInstallAppGuid) ==
+              TestConfiguration(L"spam.exe --multi-install --chrome",
+                                ERROR_SUCCESS, L"foo-multi")
+                  .chrome_app_guid());
 }
 
 TEST(MiniInstallerConfigurationTest, HasChrome) {
@@ -119,4 +183,14 @@ TEST(MiniInstallerConfigurationTest, IsSystemLevel) {
   EXPECT_FALSE(TestConfiguration(L"spam.exe").is_system_level());
   EXPECT_FALSE(TestConfiguration(L"spam.exe --chrome").is_system_level());
   EXPECT_TRUE(TestConfiguration(L"spam.exe --system-level").is_system_level());
+
+  {
+    ScopedGoogleUpdateIsMachine env_setter(false);
+    EXPECT_FALSE(TestConfiguration(L"spam.exe").is_system_level());
+  }
+
+  {
+    ScopedGoogleUpdateIsMachine env_setter(true);
+    EXPECT_TRUE(TestConfiguration(L"spam.exe").is_system_level());
+  }
 }
