@@ -5,13 +5,8 @@
 #ifndef CHROME_INSTALLER_UTIL_DELETE_TREE_WORK_ITEM_H_
 #define CHROME_INSTALLER_UTIL_DELETE_TREE_WORK_ITEM_H_
 
-#include <stddef.h>
-
-#include <vector>
-
 #include "base/files/file_path.h"
 #include "base/files/scoped_temp_dir.h"
-#include "base/memory/scoped_ptr.h"
 #include "chrome/installer/util/work_item.h"
 
 // A WorkItem subclass that recursively deletes a file system hierarchy at the
@@ -23,10 +18,6 @@ class DeleteTreeWorkItem : public WorkItem {
  public:
   ~DeleteTreeWorkItem() override;
 
-  bool Do() override;
-
-  void Rollback() override;
-
  private:
   friend class WorkItem;
 
@@ -35,11 +26,14 @@ class DeleteTreeWorkItem : public WorkItem {
   // should be on the same volume; otherwise, the move will be simulated
   // by a copy-and-delete operation.
   DeleteTreeWorkItem(const base::FilePath& root_path,
-                     const base::FilePath& temp_path,
-                     const std::vector<base::FilePath>& key_paths);
+                     const base::FilePath& temp_path);
+
+  // WorkItem:
+  bool DoImpl() override;
+  void RollbackImpl() override;
 
   // Return temporary path for work based on |backup_path_| and |root_path_|.
-  base::FilePath GetBackupPath();
+  const base::FilePath& GetBackupPath();
 
   // Attempts to delete |root_path_|. Returns true on success.
   bool DeleteRoot();
@@ -48,28 +42,20 @@ class DeleteTreeWorkItem : public WorkItem {
   bool MoveRootToBackup();
 
   // Root path to delete.
-  base::FilePath root_path_;
+  const base::FilePath root_path_;
 
   // Temporary directory that can be used.
-  base::FilePath temp_path_;
-
-  // The number of key files.
-  ptrdiff_t num_key_files_;
-
-  // Contains the paths to the key files. If specified, deletion will be
-  // performed only if none of the key files are in use.
-  scoped_ptr<base::FilePath[]> key_paths_;
-
-  // Contains the temp directories for the backed-up key files. The directories
-  // are created and populated in Do() as-needed. We don't use a standard
-  // container for this since base::ScopedTempDir isn't CopyConstructible.
-  scoped_ptr<base::ScopedTempDir[]> key_backup_paths_;
+  const base::FilePath temp_path_;
 
   // The temporary directory into which the original root_path_ has been moved.
-  base::ScopedTempDir backup_path_;
+  base::ScopedTempDir backup_dir_;
+
+  // Caches the return value of GetBackupPath(). This is empty if |backup_dir_|
+  // has not been created.
+  base::FilePath backup_path_;
 
   // Set to true once root_path_ has been moved into backup_path_.
-  bool moved_to_backup_;
+  bool moved_to_backup_ = false;
 };
 
 #endif  // CHROME_INSTALLER_UTIL_DELETE_TREE_WORK_ITEM_H_
