@@ -9,6 +9,7 @@
 
 #include "base/macros.h"
 #include "content/public/test/mock_render_thread.h"
+#include "extensions/features/features.h"
 
 struct ExtensionMsg_ExternalConnectionInfo;
 
@@ -19,27 +20,27 @@ class ChromeMockRenderThread : public content::MockRenderThread {
   ~ChromeMockRenderThread() override;
 
   // content::RenderThread overrides.
-  scoped_refptr<base::SingleThreadTaskRunner> GetIOMessageLoopProxy() override;
+  scoped_refptr<base::SingleThreadTaskRunner> GetIOTaskRunner() override;
 
   //////////////////////////////////////////////////////////////////////////
   // The following functions are called by the test itself.
 
-  // Set IO message loop proxy.
-  void set_io_message_loop_proxy(
+  void set_io_task_runner(
       const scoped_refptr<base::SingleThreadTaskRunner>& task_runner);
 
- private:
+ protected:
   // Overrides base class implementation to add custom handling for
   // print and extensions.
   bool OnMessageReceived(const IPC::Message& msg) override;
 
-#if defined(ENABLE_EXTENSIONS)
+ private:
+#if BUILDFLAG(ENABLE_EXTENSIONS)
   // The callee expects to be returned a valid channel_id.
   void OnOpenChannelToExtension(int routing_id,
                                 const ExtensionMsg_ExternalConnectionInfo& info,
                                 const std::string& channel_name,
                                 bool include_tls_channel_id,
-                                int* port_id);
+                                int request_id);
 #endif
 
   scoped_refptr<base::SingleThreadTaskRunner> io_task_runner_;
