@@ -4,24 +4,26 @@
 
 #include "chrome/renderer/content_settings_observer.h"
 
-#include "base/command_line.h"
-#include "base/metrics/histogram.h"
-#include "components/content_settings/content/common/content_settings_messages.h"
+#include "chrome/common/render_messages.h"
+#include "chrome/common/ssl_insecure_content.h"
 #include "content/public/common/url_constants.h"
 #include "content/public/renderer/document_state.h"
 #include "content/public/renderer/render_frame.h"
 #include "content/public/renderer/render_view.h"
+#include "extensions/features/features.h"
+#include "third_party/WebKit/public/platform/URLConversion.h"
 #include "third_party/WebKit/public/platform/WebContentSettingCallbacks.h"
+#include "third_party/WebKit/public/platform/WebSecurityOrigin.h"
 #include "third_party/WebKit/public/platform/WebURL.h"
 #include "third_party/WebKit/public/web/WebDataSource.h"
 #include "third_party/WebKit/public/web/WebDocument.h"
 #include "third_party/WebKit/public/web/WebFrameClient.h"
 #include "third_party/WebKit/public/web/WebLocalFrame.h"
-#include "third_party/WebKit/public/web/WebSecurityOrigin.h"
 #include "third_party/WebKit/public/web/WebView.h"
+#include "url/origin.h"
 #include "url/url_constants.h"
 
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
 #include "extensions/common/constants.h"
 #include "extensions/common/extension.h"
 #include "extensions/common/permissions/api_permission.h"
@@ -43,63 +45,16 @@ using content::NavigationState;
 
 namespace {
 
-// This enum is histogrammed, so do not add, reorder, or remove values.
-enum {
-  INSECURE_CONTENT_DISPLAY = 0,
-  INSECURE_CONTENT_DISPLAY_HOST_GOOGLE,      // deprecated
-  INSECURE_CONTENT_DISPLAY_HOST_WWW_GOOGLE,  // deprecated
-  INSECURE_CONTENT_DISPLAY_HTML,
-  INSECURE_CONTENT_RUN,
-  INSECURE_CONTENT_RUN_HOST_GOOGLE,      // deprecated
-  INSECURE_CONTENT_RUN_HOST_WWW_GOOGLE,  // deprecated
-  INSECURE_CONTENT_RUN_TARGET_YOUTUBE,   // deprecated
-  INSECURE_CONTENT_RUN_JS,
-  INSECURE_CONTENT_RUN_CSS,
-  INSECURE_CONTENT_RUN_SWF,
-  INSECURE_CONTENT_DISPLAY_HOST_YOUTUBE,           // deprecated
-  INSECURE_CONTENT_RUN_HOST_YOUTUBE,               // deprecated
-  INSECURE_CONTENT_RUN_HOST_GOOGLEUSERCONTENT,     // deprecated
-  INSECURE_CONTENT_DISPLAY_HOST_MAIL_GOOGLE,       // deprecated
-  INSECURE_CONTENT_RUN_HOST_MAIL_GOOGLE,           // deprecated
-  INSECURE_CONTENT_DISPLAY_HOST_PLUS_GOOGLE,       // deprecated
-  INSECURE_CONTENT_RUN_HOST_PLUS_GOOGLE,           // deprecated
-  INSECURE_CONTENT_DISPLAY_HOST_DOCS_GOOGLE,       // deprecated
-  INSECURE_CONTENT_RUN_HOST_DOCS_GOOGLE,           // deprecated
-  INSECURE_CONTENT_DISPLAY_HOST_SITES_GOOGLE,      // deprecated
-  INSECURE_CONTENT_RUN_HOST_SITES_GOOGLE,          // deprecated
-  INSECURE_CONTENT_DISPLAY_HOST_PICASAWEB_GOOGLE,  // deprecated
-  INSECURE_CONTENT_RUN_HOST_PICASAWEB_GOOGLE,      // deprecated
-  INSECURE_CONTENT_DISPLAY_HOST_GOOGLE_READER,     // deprecated
-  INSECURE_CONTENT_RUN_HOST_GOOGLE_READER,         // deprecated
-  INSECURE_CONTENT_DISPLAY_HOST_CODE_GOOGLE,       // deprecated
-  INSECURE_CONTENT_RUN_HOST_CODE_GOOGLE,           // deprecated
-  INSECURE_CONTENT_DISPLAY_HOST_GROUPS_GOOGLE,     // deprecated
-  INSECURE_CONTENT_RUN_HOST_GROUPS_GOOGLE,         // deprecated
-  INSECURE_CONTENT_DISPLAY_HOST_MAPS_GOOGLE,       // deprecated
-  INSECURE_CONTENT_RUN_HOST_MAPS_GOOGLE,           // deprecated
-  INSECURE_CONTENT_DISPLAY_HOST_GOOGLE_SUPPORT,    // deprecated
-  INSECURE_CONTENT_RUN_HOST_GOOGLE_SUPPORT,        // deprecated
-  INSECURE_CONTENT_DISPLAY_HOST_GOOGLE_INTL,       // deprecated
-  INSECURE_CONTENT_RUN_HOST_GOOGLE_INTL,           // deprecated
-  INSECURE_CONTENT_NUM_EVENTS
-};
-
-// Constants for UMA statistic collection.
-static const char kDotJS[] = ".js";
-static const char kDotCSS[] = ".css";
-static const char kDotSWF[] = ".swf";
-static const char kDotHTML[] = ".html";
-
 GURL GetOriginOrURL(const WebFrame* frame) {
-  WebString top_origin = frame->top()->securityOrigin().toString();
+  url::Origin top_origin = url::Origin(frame->top()->getSecurityOrigin());
   // The |top_origin| is unique ("null") e.g., for file:// URLs. Use the
   // document URL as the primary URL in those cases.
   // TODO(alexmos): This is broken for --site-per-process, since top() can be a
   // WebRemoteFrame which does not have a document(), and the WebRemoteFrame's
-  // URL is not replicated.
-  if (top_origin == "null")
+  // URL is not replicated.  See https://crbug.com/628759.
+  if (top_origin.unique() && frame->top()->isWebLocalFrame())
     return frame->top()->document().url();
-  return GURL(top_origin);
+  return top_origin.GetURL();
 }
 
 ContentSetting GetContentSettingFromRules(
@@ -134,14 +89,12 @@ ContentSettingsObserver::ContentSettingsObserver(
     : content::RenderFrameObserver(render_frame),
       content::RenderFrameObserverTracker<ContentSettingsObserver>(
           render_frame),
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
       extension_dispatcher_(extension_dispatcher),
 #endif
-      allow_displaying_insecure_content_(false),
       allow_running_insecure_content_(false),
       content_setting_rules_(NULL),
       is_interstitial_page_(false),
-      npapi_plugins_blocked_(false),
       current_request_id_(0),
       should_whitelist_(should_whitelist) {
   ClearBlockedContentSettings();
@@ -156,12 +109,9 @@ ContentSettingsObserver::ContentSettingsObserver(
     // Copy all the settings from the main render frame to avoid race conditions
     // when initializing this data. See https://crbug.com/333308.
     ContentSettingsObserver* parent = ContentSettingsObserver::Get(main_frame);
-    allow_displaying_insecure_content_ =
-        parent->allow_displaying_insecure_content_;
     allow_running_insecure_content_ = parent->allow_running_insecure_content_;
     temporarily_allowed_plugins_ = parent->temporarily_allowed_plugins_;
     is_interstitial_page_ = parent->is_interstitial_page_;
-    npapi_plugins_blocked_ = parent->npapi_plugins_blocked_;
   }
 }
 
@@ -204,9 +154,6 @@ bool ContentSettingsObserver::OnMessageReceived(const IPC::Message& message) {
   bool handled = true;
   IPC_BEGIN_MESSAGE_MAP(ContentSettingsObserver, message)
     IPC_MESSAGE_HANDLER(ChromeViewMsg_SetAsInterstitial, OnSetAsInterstitial)
-    IPC_MESSAGE_HANDLER(ChromeViewMsg_NPAPINotSupported, OnNPAPINotSupported)
-    IPC_MESSAGE_HANDLER(ChromeViewMsg_SetAllowDisplayingInsecureContent,
-                        OnSetAllowDisplayingInsecureContent)
     IPC_MESSAGE_HANDLER(ChromeViewMsg_SetAllowRunningInsecureContent,
                         OnSetAllowRunningInsecureContent)
     IPC_MESSAGE_HANDLER(ChromeViewMsg_ReloadFrame, OnReloadFrame);
@@ -246,31 +193,35 @@ void ContentSettingsObserver::DidCommitProvisionalLoad(
   GURL url = frame->document().url();
   // If we start failing this DCHECK, please makes sure we don't regress
   // this bug: http://code.google.com/p/chromium/issues/detail?id=79304
-  DCHECK(frame->document().securityOrigin().toString() == "null" ||
+  DCHECK(frame->document().getSecurityOrigin().toString() == "null" ||
          !url.SchemeIs(url::kDataScheme));
 }
 
+void ContentSettingsObserver::OnDestruct() {
+  delete this;
+}
+
 bool ContentSettingsObserver::allowDatabase(const WebString& name,
                                             const WebString& display_name,
-                                            unsigned long estimated_size) {
+                                            unsigned estimated_size) {
   WebFrame* frame = render_frame()->GetWebFrame();
-  if (frame->securityOrigin().isUnique() ||
-      frame->top()->securityOrigin().isUnique())
+  if (frame->getSecurityOrigin().isUnique() ||
+      frame->top()->getSecurityOrigin().isUnique())
     return false;
 
   bool result = false;
   Send(new ChromeViewHostMsg_AllowDatabase(
-      routing_id(), GURL(frame->securityOrigin().toString()),
-      GURL(frame->top()->securityOrigin().toString()), name, display_name,
-      &result));
+      routing_id(), url::Origin(frame->getSecurityOrigin()).GetURL(),
+      url::Origin(frame->top()->getSecurityOrigin()).GetURL(), name,
+      display_name, &result));
   return result;
 }
 
 void ContentSettingsObserver::requestFileSystemAccessAsync(
     const WebContentSettingCallbacks& callbacks) {
   WebFrame* frame = render_frame()->GetWebFrame();
-  if (frame->securityOrigin().isUnique() ||
-      frame->top()->securityOrigin().isUnique()) {
+  if (frame->getSecurityOrigin().isUnique() ||
+      frame->top()->getSecurityOrigin().isUnique()) {
     WebContentSettingCallbacks permissionCallbacks(callbacks);
     permissionCallbacks.doDeny();
     return;
@@ -285,8 +236,8 @@ void ContentSettingsObserver::requestFileSystemAccessAsync(
 
   Send(new ChromeViewHostMsg_RequestFileSystemAccessAsync(
       routing_id(), current_request_id_,
-      GURL(frame->securityOrigin().toString()),
-      GURL(frame->top()->securityOrigin().toString())));
+      url::Origin(frame->getSecurityOrigin()).GetURL(),
+      url::Origin(frame->top()->getSecurityOrigin()).GetURL()));
 }
 
 bool ContentSettingsObserver::allowImage(bool enabled_per_settings,
@@ -315,14 +266,14 @@ bool ContentSettingsObserver::allowImage(bool enabled_per_settings,
 bool ContentSettingsObserver::allowIndexedDB(const WebString& name,
                                              const WebSecurityOrigin& origin) {
   WebFrame* frame = render_frame()->GetWebFrame();
-  if (frame->securityOrigin().isUnique() ||
-      frame->top()->securityOrigin().isUnique())
+  if (frame->getSecurityOrigin().isUnique() ||
+      frame->top()->getSecurityOrigin().isUnique())
     return false;
 
   bool result = false;
   Send(new ChromeViewHostMsg_AllowIndexedDB(
-      routing_id(), GURL(frame->securityOrigin().toString()),
-      GURL(frame->top()->securityOrigin().toString()), name, &result));
+      routing_id(), url::Origin(frame->getSecurityOrigin()).GetURL(),
+      url::Origin(frame->top()->getSecurityOrigin()).GetURL(), name, &result));
   return result;
 }
 
@@ -337,8 +288,7 @@ bool ContentSettingsObserver::allowScript(bool enabled_per_settings) {
     return true;
 
   WebFrame* frame = render_frame()->GetWebFrame();
-  std::map<WebFrame*, bool>::const_iterator it =
-      cached_script_permissions_.find(frame);
+  const auto it = cached_script_permissions_.find(frame);
   if (it != cached_script_permissions_.end())
     return it->second;
 
@@ -348,9 +298,8 @@ bool ContentSettingsObserver::allowScript(bool enabled_per_settings) {
   bool allow = true;
   if (content_setting_rules_) {
     ContentSetting setting = GetContentSettingFromRules(
-        content_setting_rules_->script_rules,
-        frame,
-        GURL(frame->document().securityOrigin().toString()));
+        content_setting_rules_->script_rules, frame,
+        url::Origin(frame->document().getSecurityOrigin()).GetURL());
     allow = setting != CONTENT_SETTING_BLOCK;
   }
   allow = allow || IsWhitelistedForContentSettings();
@@ -380,28 +329,27 @@ bool ContentSettingsObserver::allowScriptFromSource(
 
 bool ContentSettingsObserver::allowStorage(bool local) {
   WebFrame* frame = render_frame()->GetWebFrame();
-  if (frame->securityOrigin().isUnique() ||
-      frame->top()->securityOrigin().isUnique())
+  if (frame->getSecurityOrigin().isUnique() ||
+      frame->top()->getSecurityOrigin().isUnique())
     return false;
-  bool result = false;
 
   StoragePermissionsKey key(
-      GURL(frame->document().securityOrigin().toString()), local);
-  std::map<StoragePermissionsKey, bool>::const_iterator permissions =
-      cached_storage_permissions_.find(key);
+      url::Origin(frame->document().getSecurityOrigin()).GetURL(), local);
+  const auto permissions = cached_storage_permissions_.find(key);
   if (permissions != cached_storage_permissions_.end())
     return permissions->second;
 
+  bool result = false;
   Send(new ChromeViewHostMsg_AllowDOMStorage(
-      routing_id(), GURL(frame->securityOrigin().toString()),
-      GURL(frame->top()->securityOrigin().toString()), local, &result));
+      routing_id(), url::Origin(frame->getSecurityOrigin()).GetURL(),
+      url::Origin(frame->top()->getSecurityOrigin()).GetURL(), local, &result));
   cached_storage_permissions_[key] = result;
   return result;
 }
 
 bool ContentSettingsObserver::allowReadFromClipboard(bool default_value) {
   bool allowed = default_value;
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
   extensions::ScriptContext* current_context =
       extension_dispatcher_->script_context_set().GetCurrent();
   if (current_context) {
@@ -414,7 +362,7 @@ bool ContentSettingsObserver::allowReadFromClipboard(bool default_value) {
 
 bool ContentSettingsObserver::allowWriteToClipboard(bool default_value) {
   bool allowed = default_value;
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
   // All blessed extension pages could historically write to the clipboard, so
   // preserve that for compatibility.
   extensions::ScriptContext* current_context =
@@ -436,57 +384,41 @@ bool ContentSettingsObserver::allowMutationEvents(bool default_value) {
   return IsPlatformApp() ? false : default_value;
 }
 
-static void SendInsecureContentSignal(int signal) {
-  UMA_HISTOGRAM_ENUMERATION("SSL.InsecureContent", signal,
-                            INSECURE_CONTENT_NUM_EVENTS);
-}
-
-bool ContentSettingsObserver::allowDisplayingInsecureContent(
-    bool allowed_per_settings,
-    const blink::WebURL& resource_url) {
-  SendInsecureContentSignal(INSECURE_CONTENT_DISPLAY);
-
-  GURL resource_gurl(resource_url);
-  if (base::EndsWith(resource_gurl.path(), kDotHTML,
-                     base::CompareCase::INSENSITIVE_ASCII))
-    SendInsecureContentSignal(INSECURE_CONTENT_DISPLAY_HTML);
-
-  if (allowed_per_settings || allow_displaying_insecure_content_)
-    return true;
-
-  Send(new ChromeViewHostMsg_DidBlockDisplayingInsecureContent(routing_id()));
-
-  return false;
-}
-
 bool ContentSettingsObserver::allowRunningInsecureContent(
     bool allowed_per_settings,
     const blink::WebSecurityOrigin& origin,
     const blink::WebURL& resource_url) {
-  GURL resource_gurl(resource_url);
-  if (base::EndsWith(resource_gurl.path(), kDotJS,
-                     base::CompareCase::INSENSITIVE_ASCII))
-    SendInsecureContentSignal(INSECURE_CONTENT_RUN_JS);
-  else if (base::EndsWith(resource_gurl.path(), kDotCSS,
-                          base::CompareCase::INSENSITIVE_ASCII))
-    SendInsecureContentSignal(INSECURE_CONTENT_RUN_CSS);
-  else if (base::EndsWith(resource_gurl.path(), kDotSWF,
-                          base::CompareCase::INSENSITIVE_ASCII))
-    SendInsecureContentSignal(INSECURE_CONTENT_RUN_SWF);
+  FilteredReportInsecureContentRan(GURL(resource_url));
 
   if (!allow_running_insecure_content_ && !allowed_per_settings) {
-    DidBlockContentType(CONTENT_SETTINGS_TYPE_MIXEDSCRIPT, origin.host());
+    DidBlockContentType(CONTENT_SETTINGS_TYPE_MIXEDSCRIPT);
     return false;
   }
 
   return true;
 }
 
+bool ContentSettingsObserver::allowAutoplay(bool default_value) {
+  if (!content_setting_rules_)
+    return default_value;
+
+  WebFrame* frame = render_frame()->GetWebFrame();
+  return GetContentSettingFromRules(
+             content_setting_rules_->autoplay_rules, frame,
+             url::Origin(frame->document().getSecurityOrigin()).GetURL()) ==
+         CONTENT_SETTING_ALLOW;
+}
+
+void ContentSettingsObserver::passiveInsecureContentFound(
+    const blink::WebURL& resource_url) {
+  ReportInsecureContent(SslInsecureContentType::DISPLAY);
+  FilteredReportInsecureContentDisplayed(GURL(resource_url));
+}
+
 void ContentSettingsObserver::didUseKeygen() {
   WebFrame* frame = render_frame()->GetWebFrame();
   Send(new ChromeViewHostMsg_DidUseKeygen(
-      routing_id(),
-      GURL(frame->securityOrigin().toString())));
+      routing_id(), url::Origin(frame->getSecurityOrigin()).GetURL()));
 }
 
 void ContentSettingsObserver::didNotAllowPlugins() {
@@ -497,10 +429,6 @@ void ContentSettingsObserver::didNotAllowScript() {
   DidBlockContentType(CONTENT_SETTINGS_TYPE_JAVASCRIPT);
 }
 
-bool ContentSettingsObserver::AreNPAPIPluginsBlocked() const {
-  return npapi_plugins_blocked_;
-}
-
 void ContentSettingsObserver::OnLoadBlockedPlugins(
     const std::string& identifier) {
   temporarily_allowed_plugins_.insert(identifier);
@@ -510,17 +438,8 @@ void ContentSettingsObserver::OnSetAsInterstitial() {
   is_interstitial_page_ = true;
 }
 
-void ContentSettingsObserver::OnNPAPINotSupported() {
-  npapi_plugins_blocked_ = true;
-}
-
-void ContentSettingsObserver::OnSetAllowDisplayingInsecureContent(bool allow) {
-  allow_displaying_insecure_content_ = allow;
-}
-
 void ContentSettingsObserver::OnSetAllowRunningInsecureContent(bool allow) {
   allow_running_insecure_content_ = allow;
-  OnSetAllowDisplayingInsecureContent(allow);
 }
 
 void ContentSettingsObserver::OnReloadFrame() {
@@ -553,9 +472,9 @@ void ContentSettingsObserver::ClearBlockedContentSettings() {
 }
 
 bool ContentSettingsObserver::IsPlatformApp() {
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
   WebFrame* frame = render_frame()->GetWebFrame();
-  WebSecurityOrigin origin = frame->document().securityOrigin();
+  WebSecurityOrigin origin = frame->document().getSecurityOrigin();
   const extensions::Extension* extension = GetExtension(origin);
   return extension && extension->is_platform_app();
 #else
@@ -563,7 +482,7 @@ bool ContentSettingsObserver::IsPlatformApp() {
 #endif
 }
 
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
 const extensions::Extension* ContentSettingsObserver::GetExtension(
     const WebSecurityOrigin& origin) const {
   if (!base::EqualsASCII(base::StringPiece16(origin.protocol()),
@@ -588,14 +507,14 @@ bool ContentSettingsObserver::IsWhitelistedForContentSettings() const {
     return true;
 
   WebFrame* web_frame = render_frame()->GetWebFrame();
-  return IsWhitelistedForContentSettings(web_frame->document().securityOrigin(),
-                                         web_frame->document().url());
+  return IsWhitelistedForContentSettings(
+      web_frame->document().getSecurityOrigin(), web_frame->document().url());
 }
 
 bool ContentSettingsObserver::IsWhitelistedForContentSettings(
     const WebSecurityOrigin& origin,
     const GURL& document_url) {
-  if (document_url == GURL(content::kUnreachableWebDataURL))
+  if (document_url == content::kUnreachableWebDataURL)
     return true;
 
   if (origin.isUnique())
@@ -608,16 +527,11 @@ bool ContentSettingsObserver::IsWhitelistedForContentSettings(
   if (base::EqualsASCII(protocol, content::kChromeDevToolsScheme))
     return true;  // DevTools UI elements should still work.
 
-#if defined(ENABLE_EXTENSIONS)
+#if BUILDFLAG(ENABLE_EXTENSIONS)
   if (base::EqualsASCII(protocol, extensions::kExtensionScheme))
     return true;
 #endif
 
-  // TODO(creis, fsamuel): Remove this once the concept of swapped out
-  // RenderFrames goes away.
-  if (document_url == GURL(content::kSwappedOutURL))
-    return true;
-
   // If the scheme is file:, an empty file name indicates a directory listing,
   // which requires JavaScript to function properly.
   if (base::EqualsASCII(protocol, url::kFileScheme)) {
