@@ -4,6 +4,7 @@
 
 #include "chrome/renderer/net/net_error_helper.h"
 
+#include <memory>
 #include <string>
 #include <utility>
 
@@ -15,11 +16,14 @@
 #include "base/values.h"
 #include "build/build_config.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/localized_error.h"
 #include "chrome/common/render_messages.h"
-#include "chrome/grit/renderer_resources.h"
+#include "chrome/renderer/chrome_render_thread_observer.h"
 #include "components/error_page/common/error_page_params.h"
+#include "components/error_page/common/localized_error.h"
 #include "components/error_page/common/net_error_info.h"
+#include "components/grit/components_resources.h"
+#include "content/public/common/associated_interface_provider.h"
+#include "content/public/common/associated_interface_registry.h"
 #include "content/public/common/content_client.h"
 #include "content/public/common/url_constants.h"
 #include "content/public/renderer/content_renderer_client.h"
@@ -30,6 +34,8 @@
 #include "content/public/renderer/resource_fetcher.h"
 #include "ipc/ipc_message.h"
 #include "ipc/ipc_message_macros.h"
+#include "third_party/WebKit/public/platform/WebCachePolicy.h"
+#include "third_party/WebKit/public/platform/WebSecurityOrigin.h"
 #include "third_party/WebKit/public/platform/WebURL.h"
 #include "third_party/WebKit/public/platform/WebURLError.h"
 #include "third_party/WebKit/public/platform/WebURLRequest.h"
@@ -51,8 +57,8 @@ using content::kUnreachableWebDataURL;
 using error_page::DnsProbeStatus;
 using error_page::DnsProbeStatusToString;
 using error_page::ErrorPageParams;
+using error_page::LocalizedError;
 using error_page::NetErrorHelperCore;
-using error_page::OfflinePageStatus;
 
 namespace {
 
@@ -79,6 +85,7 @@ NetErrorHelperCore::FrameType GetFrameType(RenderFrame* render_frame) {
 NetErrorHelper::NetErrorHelper(RenderFrame* render_frame)
     : RenderFrameObserver(render_frame),
       content::RenderFrameObserverTracker<NetErrorHelper>(render_frame),
+      network_diagnostics_client_binding_(this),
       weak_controller_delegate_factory_(this) {
   RenderThread::Get()->AddObserver(this);
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
@@ -92,6 +99,10 @@ NetErrorHelper::NetErrorHelper(RenderFrame* render_frame)
                                      auto_reload_enabled,
                                      auto_reload_visible_only,
                                      !render_frame->IsHidden()));
+
+  render_frame->GetAssociatedInterfaceRegistry()->AddInterface(
+      base::Bind(&NetErrorHelper::OnNetworkDiagnosticsClientRequest,
+                 base::Unretained(this)));
 }
 
 NetErrorHelper::~NetErrorHelper() {
@@ -149,19 +160,18 @@ bool NetErrorHelper::OnMessageReceived(const IPC::Message& message) {
 
   IPC_BEGIN_MESSAGE_MAP(NetErrorHelper, message)
     IPC_MESSAGE_HANDLER(ChromeViewMsg_NetErrorInfo, OnNetErrorInfo)
-    IPC_MESSAGE_HANDLER(ChromeViewMsg_SetCanShowNetworkDiagnosticsDialog,
-                        OnSetCanShowNetworkDiagnosticsDialog);
     IPC_MESSAGE_HANDLER(ChromeViewMsg_SetNavigationCorrectionInfo,
                         OnSetNavigationCorrectionInfo);
-#if defined(OS_ANDROID)
-    IPC_MESSAGE_HANDLER(ChromeViewMsg_SetOfflinePageInfo, OnSetOfflinePageInfo)
-#endif
     IPC_MESSAGE_UNHANDLED(handled = false)
   IPC_END_MESSAGE_MAP()
 
   return handled;
 }
 
+void NetErrorHelper::OnDestruct() {
+  delete this;
+}
+
 void NetErrorHelper::NetworkStateChanged(bool enabled) {
   core_->NetworkStateChanged(enabled);
 }
@@ -178,17 +188,24 @@ bool NetErrorHelper::ShouldSuppressErrorPage(const GURL& url) {
   return core_->ShouldSuppressErrorPage(GetFrameType(render_frame()), url);
 }
 
+chrome::mojom::NetworkDiagnostics*
+NetErrorHelper::GetRemoteNetworkDiagnostics() {
+  if (!remote_network_diagnostics_) {
+    render_frame()->GetRemoteAssociatedInterfaces()
+        ->GetInterface(&remote_network_diagnostics_);
+  }
+  return remote_network_diagnostics_.get();
+}
+
 void NetErrorHelper::GenerateLocalizedErrorPage(
     const blink::WebURLError& error,
     bool is_failed_post,
     bool can_show_network_diagnostics_dialog,
-    OfflinePageStatus offline_page_status,
-    scoped_ptr<ErrorPageParams> params,
+    std::unique_ptr<ErrorPageParams> params,
     bool* reload_button_shown,
     bool* show_saved_copy_button_shown,
     bool* show_cached_copy_button_shown,
-    bool* show_offline_pages_button_shown,
-    bool* show_offline_copy_button_shown,
+    bool* download_button_shown,
     std::string* error_html) const {
   error_html->clear();
 
@@ -202,18 +219,15 @@ void NetErrorHelper::GenerateLocalizedErrorPage(
     LocalizedError::GetStrings(
         error.reason, error.domain.utf8(), error.unreachableURL, is_failed_post,
         error.staleCopyInCache, can_show_network_diagnostics_dialog,
-        offline_page_status, RenderThread::Get()->GetLocale(),
-        render_frame()->GetRenderView()->GetAcceptLanguages(),
-        std::move(params), &error_strings);
+        ChromeRenderThreadObserver::is_incognito_process(),
+        RenderThread::Get()->GetLocale(), std::move(params), &error_strings);
     *reload_button_shown = error_strings.Get("reloadButton", nullptr);
     *show_saved_copy_button_shown =
         error_strings.Get("showSavedCopyButton", nullptr);
     *show_cached_copy_button_shown =
         error_strings.Get("cacheButton", nullptr);
-    *show_offline_pages_button_shown =
-        error_strings.Get("showOfflinePagesButton", nullptr);
-    *show_offline_copy_button_shown =
-        error_strings.Get("showOfflineCopyButton", nullptr);
+    *download_button_shown =
+        error_strings.Get("downloadButton", nullptr);
     // "t" is the id of the template's root node.
     *error_html = webui::GetTemplatesHtml(template_html, &error_strings, "t");
   }
@@ -232,21 +246,14 @@ void NetErrorHelper::EnablePageHelperFunctions() {
 
 void NetErrorHelper::UpdateErrorPage(const blink::WebURLError& error,
                                      bool is_failed_post,
-                                     bool can_show_network_diagnostics_dialog,
-                                     OfflinePageStatus offline_page_status) {
+                                     bool can_show_network_diagnostics_dialog) {
   base::DictionaryValue error_strings;
   LocalizedError::GetStrings(
-      error.reason,
-      error.domain.utf8(),
-      error.unreachableURL,
-      is_failed_post,
-      error.staleCopyInCache,
-      can_show_network_diagnostics_dialog,
-      offline_page_status,
+      error.reason, error.domain.utf8(), error.unreachableURL, is_failed_post,
+      error.staleCopyInCache, can_show_network_diagnostics_dialog,
+      ChromeRenderThreadObserver::is_incognito_process(),
       RenderThread::Get()->GetLocale(),
-      render_frame()->GetRenderView()->GetAcceptLanguages(),
-      scoped_ptr<ErrorPageParams>(),
-      &error_strings);
+      std::unique_ptr<ErrorPageParams>(), &error_strings);
 
   std::string json;
   JSONWriter::Write(error_strings, &json);
@@ -277,7 +284,6 @@ void NetErrorHelper::FetchNavigationCorrections(
       render_frame()->GetWebFrame(),
       blink::WebURLRequest::RequestContextInternal,
       blink::WebURLRequest::FrameTypeNone,
-      content::ResourceFetcher::PLATFORM_LOADER,
       base::Bind(&NetErrorHelper::OnNavigationCorrectionsFetched,
                  base::Unretained(this)));
 
@@ -301,14 +307,15 @@ void NetErrorHelper::SendTrackingRequest(
   tracking_fetcher_->Start(
       render_frame()->GetWebFrame(),
       blink::WebURLRequest::RequestContextInternal,
-      blink::WebURLRequest::FrameTypeTopLevel,
-      content::ResourceFetcher::PLATFORM_LOADER,
+      blink::WebURLRequest::FrameTypeNone,
       base::Bind(&NetErrorHelper::OnTrackingRequestComplete,
                  base::Unretained(this)));
 }
 
-void NetErrorHelper::ReloadPage(bool ignore_cache) {
-  render_frame()->GetWebFrame()->reload(ignore_cache);
+void NetErrorHelper::ReloadPage(bool bypass_cache) {
+  render_frame()->GetWebFrame()->reload(
+      bypass_cache ? blink::WebFrameLoadType::ReloadBypassingCache
+                   : blink::WebFrameLoadType::Reload);
 }
 
 void NetErrorHelper::LoadPageFromCache(const GURL& page_url) {
@@ -318,30 +325,21 @@ void NetErrorHelper::LoadPageFromCache(const GURL& page_url) {
       "POST"));
 
   blink::WebURLRequest request(page_url);
-  request.setCachePolicy(blink::WebURLRequest::ReturnCacheDataDontLoad);
-
+  request.setCachePolicy(blink::WebCachePolicy::ReturnCacheDataDontLoad);
   web_frame->loadRequest(request);
 }
 
 void NetErrorHelper::DiagnoseError(const GURL& page_url) {
-  render_frame()->Send(new ChromeViewHostMsg_RunNetworkDiagnostics(
-      render_frame()->GetRoutingID(), page_url));
+  GetRemoteNetworkDiagnostics()->RunNetworkDiagnostics(page_url);
 }
 
-void NetErrorHelper::ShowOfflinePages() {
+void NetErrorHelper::DownloadPageLater() {
 #if defined(OS_ANDROID)
-  render_frame()->Send(new ChromeViewHostMsg_ShowOfflinePages(
+  render_frame()->Send(new ChromeViewHostMsg_DownloadPageLater(
       render_frame()->GetRoutingID()));
 #endif  // defined(OS_ANDROID)
 }
 
-void NetErrorHelper::LoadOfflineCopy(const GURL& page_url) {
-#if defined(OS_ANDROID)
-  render_frame()->Send(new ChromeViewHostMsg_LoadOfflineCopy(
-      render_frame()->GetRoutingID(), page_url));
-#endif  // defined(OS_ANDROID)
-}
-
 void NetErrorHelper::OnNetErrorInfo(int status_num) {
   DCHECK(status_num >= 0 && status_num < error_page::DNS_PROBE_MAX);
 
@@ -350,12 +348,6 @@ void NetErrorHelper::OnNetErrorInfo(int status_num) {
   core_->OnNetErrorInfo(static_cast<DnsProbeStatus>(status_num));
 }
 
-void NetErrorHelper::OnSetCanShowNetworkDiagnosticsDialog(
-    bool can_use_local_diagnostics_service) {
-  core_->OnSetCanShowNetworkDiagnosticsDialog(
-      can_use_local_diagnostics_service);
-}
-
 void NetErrorHelper::OnSetNavigationCorrectionInfo(
     const GURL& navigation_correction_url,
     const std::string& language,
@@ -371,13 +363,11 @@ void NetErrorHelper::OnNavigationCorrectionsFetched(
     const std::string& data) {
   // The fetcher may only be deleted after |data| is passed to |core_|.  Move
   // it to a temporary to prevent any potential re-entrancy issues.
-  scoped_ptr<content::ResourceFetcher> fetcher(
+  std::unique_ptr<content::ResourceFetcher> fetcher(
       correction_fetcher_.release());
   bool success = (!response.isNull() && response.httpStatusCode() == 200);
-  core_->OnNavigationCorrectionsFetched(
-      success ? data : "",
-      render_frame()->GetRenderView()->GetAcceptLanguages(),
-      base::i18n::IsRTL());
+  core_->OnNavigationCorrectionsFetched(success ? data : "",
+                                        base::i18n::IsRTL());
 }
 
 void NetErrorHelper::OnTrackingRequestComplete(
@@ -386,9 +376,12 @@ void NetErrorHelper::OnTrackingRequestComplete(
   tracking_fetcher_.reset();
 }
 
-#if defined(OS_ANDROID)
-void NetErrorHelper::OnSetOfflinePageInfo(
-    OfflinePageStatus offline_page_status) {
-  core_->OnSetOfflinePageInfo(offline_page_status);
+void NetErrorHelper::OnNetworkDiagnosticsClientRequest(
+    chrome::mojom::NetworkDiagnosticsClientAssociatedRequest request) {
+  DCHECK(!network_diagnostics_client_binding_.is_bound());
+  network_diagnostics_client_binding_.Bind(std::move(request));
+}
+
+void NetErrorHelper::SetCanShowNetworkDiagnosticsDialog(bool can_show) {
+  core_->OnSetCanShowNetworkDiagnosticsDialog(can_show);
 }
-#endif  // defined(OS_ANDROID)
