@@ -5,19 +5,20 @@
 #ifndef CHROME_RENDERER_NET_NET_ERROR_HELPER_H_
 #define CHROME_RENDERER_NET_NET_ERROR_HELPER_H_
 
+#include <memory>
 #include <string>
 
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "build/build_config.h"
+#include "chrome/common/network_diagnostics.mojom.h"
 #include "chrome/renderer/net/net_error_page_controller.h"
 #include "components/error_page/common/net_error_info.h"
-#include "components/error_page/common/offline_page_types.h"
 #include "components/error_page/renderer/net_error_helper_core.h"
 #include "content/public/renderer/render_frame_observer.h"
 #include "content/public/renderer/render_frame_observer_tracker.h"
-#include "content/public/renderer/render_process_observer.h"
+#include "content/public/renderer/render_thread_observer.h"
+#include "mojo/public/cpp/bindings/associated_binding.h"
 
 class GURL;
 
@@ -38,12 +39,15 @@ struct ErrorPageParams;
 // Listens for NetErrorInfo messages from the NetErrorTabHelper on the
 // browser side and updates the error page with more details (currently, just
 // DNS probe results) if/when available.
+// TODO(crbug.com/578770): Should this class be moved into the error_page
+// component?
 class NetErrorHelper
     : public content::RenderFrameObserver,
       public content::RenderFrameObserverTracker<NetErrorHelper>,
-      public content::RenderProcessObserver,
+      public content::RenderThreadObserver,
       public error_page::NetErrorHelperCore::Delegate,
-      public NetErrorPageController::Delegate {
+      public NetErrorPageController::Delegate,
+      public chrome::mojom::NetworkDiagnosticsClient {
  public:
   explicit NetErrorHelper(content::RenderFrame* render_frame);
   ~NetErrorHelper() override;
@@ -60,11 +64,12 @@ class NetErrorHelper
   void OnStop() override;
   void WasShown() override;
   void WasHidden() override;
+  void OnDestruct() override;
 
   // IPC::Listener implementation.
   bool OnMessageReceived(const IPC::Message& message) override;
 
-  // RenderProcessObserver implementation.
+  // RenderThreadObserver implementation.
   void NetworkStateChanged(bool online) override;
 
   // Initializes |error_html| with the HTML of an error page in response to
@@ -80,41 +85,36 @@ class NetErrorHelper
   bool ShouldSuppressErrorPage(const GURL& url);
 
  private:
+  chrome::mojom::NetworkDiagnostics* GetRemoteNetworkDiagnostics();
+
   // NetErrorHelperCore::Delegate implementation:
   void GenerateLocalizedErrorPage(
       const blink::WebURLError& error,
       bool is_failed_post,
       bool can_use_local_diagnostics_service,
-      error_page::OfflinePageStatus offline_page_status,
-      scoped_ptr<error_page::ErrorPageParams> params,
+      std::unique_ptr<error_page::ErrorPageParams> params,
       bool* reload_button_shown,
       bool* show_saved_copy_button_shown,
       bool* show_cached_copy_button_shown,
-      bool* show_offline_pages_button_shown,
-      bool* show_offline_copy_button_shown,
+      bool* download_button_shown,
       std::string* html) const override;
   void LoadErrorPage(const std::string& html, const GURL& failed_url) override;
   void EnablePageHelperFunctions() override;
-  void UpdateErrorPage(
-      const blink::WebURLError& error,
-      bool is_failed_post,
-      bool can_use_local_diagnostics_service,
-      error_page::OfflinePageStatus offline_page_status) override;
+  void UpdateErrorPage(const blink::WebURLError& error,
+                       bool is_failed_post,
+                       bool can_use_local_diagnostics_service) override;
   void FetchNavigationCorrections(
       const GURL& navigation_correction_url,
       const std::string& navigation_correction_request_body) override;
   void CancelFetchNavigationCorrections() override;
   void SendTrackingRequest(const GURL& tracking_url,
                            const std::string& tracking_request_body) override;
-  void ReloadPage(bool ignore_cache) override;
+  void ReloadPage(bool bypass_cache) override;
   void LoadPageFromCache(const GURL& page_url) override;
   void DiagnoseError(const GURL& page_url) override;
-  void ShowOfflinePages() override;
-  void LoadOfflineCopy(const GURL& page_url) override;
+  void DownloadPageLater() override;
 
   void OnNetErrorInfo(int status);
-  void OnSetCanShowNetworkDiagnosticsDialog(
-      bool can_use_local_diagnostics_service);
   void OnSetNavigationCorrectionInfo(const GURL& navigation_correction_url,
                                      const std::string& language,
                                      const std::string& country_code,
@@ -127,16 +127,20 @@ class NetErrorHelper
   void OnTrackingRequestComplete(const blink::WebURLResponse& response,
                                  const std::string& data);
 
-#if defined(OS_ANDROID)
-  // Called to set the status of the offline pages that will be used to decide
-  // if offline related button will be provided in the error page.
-  void OnSetOfflinePageInfo(error_page::OfflinePageStatus offline_page_status);
-#endif
+  void OnNetworkDiagnosticsClientRequest(
+      chrome::mojom::NetworkDiagnosticsClientAssociatedRequest request);
 
-  scoped_ptr<content::ResourceFetcher> correction_fetcher_;
-  scoped_ptr<content::ResourceFetcher> tracking_fetcher_;
+  // chrome::mojom::NetworkDiagnosticsClient:
+  void SetCanShowNetworkDiagnosticsDialog(bool can_show) override;
 
-  scoped_ptr<error_page::NetErrorHelperCore> core_;
+  std::unique_ptr<content::ResourceFetcher> correction_fetcher_;
+  std::unique_ptr<content::ResourceFetcher> tracking_fetcher_;
+
+  std::unique_ptr<error_page::NetErrorHelperCore> core_;
+
+  mojo::AssociatedBinding<chrome::mojom::NetworkDiagnosticsClient>
+      network_diagnostics_client_binding_;
+  chrome::mojom::NetworkDiagnosticsAssociatedPtr remote_network_diagnostics_;
 
   // Weak factory for vending a weak pointer to a NetErrorPageController. Weak
   // pointers are invalidated on each commit, to prevent getting messages from
