@@ -13,6 +13,7 @@
 #include "content/public/common/referrer.h"
 #include "content/public/renderer/render_thread.h"
 #include "content/public/renderer/render_view.h"
+#include "third_party/WebKit/public/platform/URLConversion.h"
 #include "third_party/WebKit/public/platform/WebPrerenderingSupport.h"
 #include "third_party/WebKit/public/platform/WebString.h"
 #include "third_party/WebKit/public/platform/WebURL.h"
@@ -126,6 +127,18 @@ bool PrerenderDispatcher::OnControlMessageReceived(
   return handled;
 }
 
+void PrerenderDispatcher::OnRenderProcessShutdown() {
+  // Renderer is shutting down, abandon all in-flight prerenders.
+  for (auto it : prerenders_) {
+    const PrerenderExtraData& extra_data =
+        PrerenderExtraData::FromPrerender(it.second);
+    content::RenderThread::Get()->Send(
+        new PrerenderHostMsg_AbandonLinkRelPrerender(
+            extra_data.prerender_id()));
+  }
+  prerenders_.clear();
+}
+
 void PrerenderDispatcher::add(const WebPrerender& prerender) {
   const PrerenderExtraData& extra_data =
       PrerenderExtraData::FromPrerender(prerender);
@@ -143,8 +156,9 @@ void PrerenderDispatcher::add(const WebPrerender& prerender) {
   content::RenderThread::Get()->Send(new PrerenderHostMsg_AddLinkRelPrerender(
       extra_data.prerender_id(), attributes,
       content::Referrer::SanitizeForRequest(
-          GURL(prerender.url()), content::Referrer(GURL(prerender.referrer()),
-                                                   prerender.referrerPolicy())),
+          GURL(prerender.url()),
+          content::Referrer(blink::WebStringToGURL(prerender.referrer()),
+                            prerender.referrerPolicy())),
       extra_data.size(), extra_data.render_view_route_id()));
 }
 
@@ -157,7 +171,7 @@ void PrerenderDispatcher::cancel(const WebPrerender& prerender) {
   // been canceled before it was started), so release it to avoid a
   // leak. Moreover, if it did, the PrerenderClient in Blink will have been
   // detached already.
-   prerenders_.erase(extra_data.prerender_id());
+  prerenders_.erase(extra_data.prerender_id());
 }
 
 void PrerenderDispatcher::abandon(const WebPrerender& prerender) {
@@ -172,4 +186,8 @@ void PrerenderDispatcher::abandon(const WebPrerender& prerender) {
   prerenders_.erase(extra_data.prerender_id());
 }
 
+void PrerenderDispatcher::prefetchFinished() {
+  content::RenderThread::Get()->Send(new PrerenderHostMsg_PrefetchFinished());
+}
+
 }  // namespace prerender
