@@ -4,34 +4,42 @@
 
 #include "chrome/renderer/safe_browsing/phishing_dom_feature_extractor.h"
 
+#include <memory>
+
 #include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/callback.h"
+#include "base/memory/ptr_util.h"
 #include "base/memory/weak_ptr.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/time/time.h"
 #include "chrome/renderer/chrome_content_renderer_client.h"
-#include "chrome/renderer/extensions/chrome_extensions_dispatcher_delegate.h"
-#include "chrome/renderer/extensions/chrome_extensions_renderer_client.h"
 #include "chrome/renderer/safe_browsing/features.h"
 #include "chrome/renderer/safe_browsing/mock_feature_extractor_clock.h"
 #include "chrome/renderer/safe_browsing/test_utils.h"
-#include "chrome/renderer/spellchecker/spellcheck.h"
 #include "chrome/test/base/chrome_render_view_test.h"
+#include "content/common/frame_messages.h"
+#include "content/common/navigation_params.h"
 #include "content/public/common/content_switches.h"
+#include "content/public/common/resource_response.h"
 #include "content/public/renderer/render_frame.h"
 #include "content/public/test/test_utils.h"
-#include "extensions/renderer/dispatcher.h"
+#include "ipc/ipc_message_macros.h"
 #include "net/base/escape.h"
 #include "net/base/registry_controlled_domains/registry_controlled_domain.h"
 #include "testing/gmock/include/gmock/gmock.h"
+#include "third_party/WebKit/public/platform/URLConversion.h"
 #include "third_party/WebKit/public/platform/WebString.h"
 #include "third_party/WebKit/public/web/WebFrame.h"
 #include "third_party/WebKit/public/web/WebLocalFrame.h"
+#include "third_party/WebKit/public/web/WebRuntimeFeatures.h"
 #include "third_party/WebKit/public/web/WebScriptSource.h"
+#include "ui/native_theme/native_theme_switches.h"
 
 using ::testing::DoAll;
 using ::testing::Invoke;
 using ::testing::Return;
+using blink::WebRuntimeFeatures;
 
 namespace safe_browsing {
 
@@ -94,7 +102,7 @@ class TestPhishingDOMFeatureExtractor : public PhishingDOMFeatureExtractor {
     GURL full_url;
     if (parsed_url.has_scheme()) {
       // This is already a complete URL.
-      full_url = GURL(partial_url);
+      full_url = GURL(blink::WebStringToGURL(partial_url));
     } else if (!base_domain_.empty()) {
       // This is a partial URL and only one frame in testing html.
       full_url = GURL("http://" + base_domain_).Resolve(partial_url);
@@ -184,8 +192,52 @@ class PhishingDOMFeatureExtractorTest : public ChromeRenderViewTest {
   }
 
  protected:
+  // Subclasses the ChromeMockRenderThread class for providing functionality
+  // to mock IPCs being sent to the browser from RenderViewTests.
+  class PhishingMockRenderThread : public ChromeMockRenderThread {
+   public:
+    PhishingMockRenderThread()
+        : current_message_routing_id_(-1) {
+    }
+
+   protected:
+    bool OnMessageReceived(const IPC::Message& msg) override {
+      current_message_routing_id_ = msg.routing_id();
+
+      IPC_BEGIN_MESSAGE_MAP(PhishingMockRenderThread, msg)
+        IPC_MESSAGE_HANDLER(FrameHostMsg_BeginNavigation, OnBeginNavigation)
+      IPC_END_MESSAGE_MAP()
+
+      current_message_routing_id_ = -1;
+      return ChromeMockRenderThread::OnMessageReceived(msg);
+    }
+
+    void OnBeginNavigation(
+        const content::CommonNavigationParams& common_params,
+        const content::BeginNavigationParams& begin_params) {
+      std::unique_ptr<IPC::Message> message;
+
+      message.reset(new FrameMsg_CommitNavigation(
+          current_message_routing_id_, content::ResourceResponseHead(),
+          common_params.url, common_params,
+          content::RequestNavigationParams()));
+      content::RenderFrame* frame = content::RenderFrame::FromRoutingID(
+          current_message_routing_id_);
+
+      base::ThreadTaskRunnerHandle::Get()->PostTask(
+          FROM_HERE,
+          base::Bind(
+              base::IgnoreResult(&content::RenderFrame::OnMessageReceived),
+              base::Unretained(frame), *message));
+    }
+
+    int current_message_routing_id_;
+  };
+
   void SetUp() override {
     ChromeRenderViewTest::SetUp();
+    WebRuntimeFeatures::enableOverlayScrollbars(
+        ui::IsOverlayScrollbarEnabled());
     extractor_.reset(new TestPhishingDOMFeatureExtractor(&clock_));
   }
 
@@ -196,17 +248,7 @@ class PhishingDOMFeatureExtractorTest : public ChromeRenderViewTest {
 
   content::ContentRendererClient* CreateContentRendererClient() override {
     ChromeContentRendererClient* client = new TestChromeContentRendererClient();
-#if defined(ENABLE_EXTENSIONS)
-    extension_dispatcher_delegate_.reset(
-        new ChromeExtensionsDispatcherDelegate());
-    ChromeExtensionsRendererClient* ext_client =
-        ChromeExtensionsRendererClient::GetInstance();
-    ext_client->SetExtensionDispatcherForTest(make_scoped_ptr(
-        new extensions::Dispatcher(extension_dispatcher_delegate_.get())));
-#endif
-#if defined(ENABLE_SPELLCHECK)
-    client->SetSpellcheck(new SpellCheck());
-#endif
+    InitChromeContentRendererClient(client);
     return client;
   }
 
@@ -223,9 +265,14 @@ class PhishingDOMFeatureExtractorTest : public ChromeRenderViewTest {
         "document.body.removeChild(document.getElementById('frame1'));"));
   }
 
+  // ChromeMockRenderThread overrides.
+  ChromeMockRenderThread* CreateMockRenderThread() override {
+    return new PhishingMockRenderThread();
+  }
+
   MockFeatureExtractorClock clock_;
   bool success_;
-  scoped_ptr<TestPhishingDOMFeatureExtractor> extractor_;
+  std::unique_ptr<TestPhishingDOMFeatureExtractor> extractor_;
   scoped_refptr<content::MessageLoopRunner> message_loop_;
   base::WeakPtrFactory<PhishingDOMFeatureExtractorTest> weak_factory_;
 };
