@@ -235,11 +235,9 @@ SearchBox::SearchBox(content::RenderView* render_view)
     : content::RenderViewObserver(render_view),
       content::RenderViewObserverTracker<SearchBox>(render_view),
     page_seq_no_(0),
-    app_launcher_enabled_(false),
     is_focused_(false),
     is_input_in_progress_(false),
     is_key_capture_enabled_(false),
-    display_instant_results_(false),
     most_visited_items_cache_(kMaxInstantMostVisitedItemCacheSize),
     query_() {
 }
@@ -269,15 +267,15 @@ void SearchBox::LogEvent(NTPLoggingEventType event) {
 }
 
 void SearchBox::LogMostVisitedImpression(int position,
-                                         const base::string16& provider) {
+                                         NTPLoggingTileSource tile_source) {
   render_view()->Send(new ChromeViewHostMsg_LogMostVisitedImpression(
-      render_view()->GetRoutingID(), page_seq_no_, position, provider));
+      render_view()->GetRoutingID(), page_seq_no_, position, tile_source));
 }
 
 void SearchBox::LogMostVisitedNavigation(int position,
-                                         const base::string16& provider) {
+                                         NTPLoggingTileSource tile_source) {
   render_view()->Send(new ChromeViewHostMsg_LogMostVisitedNavigation(
-      render_view()->GetRoutingID(), page_seq_no_, position, provider));
+      render_view()->GetRoutingID(), page_seq_no_, position, tile_source));
 }
 
 void SearchBox::CheckIsUserSignedInToChromeAs(const base::string16& identity) {
@@ -325,19 +323,6 @@ const EmbeddedSearchRequestParams& SearchBox::GetEmbeddedSearchRequestParams() {
   return embedded_search_request_params_;
 }
 
-void SearchBox::Focus() {
-  render_view()->Send(new ChromeViewHostMsg_FocusOmnibox(
-      render_view()->GetRoutingID(), page_seq_no_, OMNIBOX_FOCUS_VISIBLE));
-}
-
-void SearchBox::NavigateToURL(const GURL& url,
-                              WindowOpenDisposition disposition,
-                              bool is_most_visited_item_url) {
-  render_view()->Send(new ChromeViewHostMsg_SearchBoxNavigate(
-      render_view()->GetRoutingID(), page_seq_no_, url,
-      disposition, is_most_visited_item_url));
-}
-
 void SearchBox::Paste(const base::string16& text) {
   render_view()->Send(new ChromeViewHostMsg_PasteAndOpenDropdown(
       render_view()->GetRoutingID(), page_seq_no_, text));
@@ -380,10 +365,6 @@ bool SearchBox::OnMessageReceived(const IPC::Message& message) {
     IPC_MESSAGE_HANDLER(ChromeViewMsg_SearchBoxFocusChanged, OnFocusChanged)
     IPC_MESSAGE_HANDLER(ChromeViewMsg_SearchBoxMostVisitedItemsChanged,
                         OnMostVisitedChanged)
-    IPC_MESSAGE_HANDLER(ChromeViewMsg_SearchBoxPromoInformation,
-                        OnPromoInformationReceived)
-    IPC_MESSAGE_HANDLER(ChromeViewMsg_SearchBoxSetDisplayInstantResults,
-                        OnSetDisplayInstantResults)
     IPC_MESSAGE_HANDLER(ChromeViewMsg_SearchBoxSetInputInProgress,
                         OnSetInputInProgress)
     IPC_MESSAGE_HANDLER(ChromeViewMsg_SearchBoxSetSuggestionToPrefetch,
@@ -473,14 +454,6 @@ void SearchBox::OnMostVisitedChanged(
   }
 }
 
-void SearchBox::OnPromoInformationReceived(bool is_app_launcher_enabled) {
-  app_launcher_enabled_ = is_app_launcher_enabled;
-}
-
-void SearchBox::OnSetDisplayInstantResults(bool display_instant_results) {
-  display_instant_results_ = display_instant_results;
-}
-
 void SearchBox::OnSetInputInProgress(bool is_input_in_progress) {
   if (is_input_in_progress_ != is_input_in_progress) {
     is_input_in_progress_ = is_input_in_progress;
@@ -545,3 +518,7 @@ void SearchBox::Reset() {
   is_key_capture_enabled_ = false;
   theme_info_ = ThemeBackgroundInfo();
 }
+
+void SearchBox::OnDestruct() {
+  delete this;
+}
