@@ -10,8 +10,8 @@
 
 #include "base/macros.h"
 #include "base/strings/string16.h"
-#include "chrome/common/instant_types.h"
-#include "chrome/common/ntp_logging_events.h"
+#include "chrome/common/search/instant_types.h"
+#include "chrome/common/search/ntp_logging_events.h"
 #include "chrome/renderer/instant_restricted_id_cache.h"
 #include "components/omnibox/common/omnibox_focus_state.h"
 #include "content/public/renderer/render_view_observer.h"
@@ -54,10 +54,10 @@ class SearchBox : public content::RenderViewObserver,
   void LogEvent(NTPLoggingEventType event);
 
   // Sends ChromeViewHostMsg_LogMostVisitedImpression to the browser.
-  void LogMostVisitedImpression(int position, const base::string16& provider);
+  void LogMostVisitedImpression(int position, NTPLoggingTileSource tile_source);
 
   // Sends ChromeViewHostMsg_LogMostVisitedNavigation to the browser.
-  void LogMostVisitedNavigation(int position, const base::string16& provider);
+  void LogMostVisitedNavigation(int position, NTPLoggingTileSource tile_source);
 
   // Sends ChromeViewHostMsg_ChromeIdentityCheck to the browser.
   void CheckIsUserSignedInToChromeAs(const base::string16& identity);
@@ -107,14 +107,6 @@ class SearchBox : public content::RenderViewObserver,
   bool GetMostVisitedItemWithID(InstantRestrictedID most_visited_item_id,
                                 InstantMostVisitedItem* item) const;
 
-  // Sends ChromeViewHostMsg_FocusOmnibox to the browser.
-  void Focus();
-
-  // Sends ChromeViewHostMsg_SearchBoxNavigate to the browser.
-  void NavigateToURL(const GURL& url,
-                     WindowOpenDisposition disposition,
-                     bool is_most_visited_item_url);
-
   // Sends ChromeViewHostMsg_SearchBoxPaste to the browser.
   void Paste(const base::string16& text);
 
@@ -134,17 +126,16 @@ class SearchBox : public content::RenderViewObserver,
   // Sends ChromeViewHostMsg_SearchBoxUndoMostVisitedDeletion to the browser.
   void UndoMostVisitedDeletion(InstantRestrictedID most_visited_item_id);
 
-  bool app_launcher_enabled() const { return app_launcher_enabled_; }
   bool is_focused() const { return is_focused_; }
   bool is_input_in_progress() const { return is_input_in_progress_; }
   bool is_key_capture_enabled() const { return is_key_capture_enabled_; }
-  bool display_instant_results() const { return display_instant_results_; }
   const base::string16& query() const { return query_; }
   const InstantSuggestion& suggestion() const { return suggestion_; }
 
  private:
   // Overridden from content::RenderViewObserver:
   bool OnMessageReceived(const IPC::Message& message) override;
+  void OnDestruct() override;
 
   void OnSetPageSequenceNumber(int page_seq_no);
   void OnChromeIdentityCheckResult(const base::string16& identity,
@@ -155,8 +146,6 @@ class SearchBox : public content::RenderViewObserver,
   void OnHistorySyncCheckResult(bool sync_history);
   void OnMostVisitedChanged(
       const std::vector<InstantMostVisitedItem>& items);
-  void OnPromoInformationReceived(bool is_app_launcher_enabled);
-  void OnSetDisplayInstantResults(bool display_instant_results);
   void OnSetInputInProgress(bool input_in_progress);
   void OnSetSuggestionToPrefetch(const InstantSuggestion& suggestion);
   void OnSubmit(const base::string16& query,
@@ -173,11 +162,9 @@ class SearchBox : public content::RenderViewObserver,
   GURL GetURLForMostVisitedItem(InstantRestrictedID item_id) const;
 
   int page_seq_no_;
-  bool app_launcher_enabled_;
   bool is_focused_;
   bool is_input_in_progress_;
   bool is_key_capture_enabled_;
-  bool display_instant_results_;
   InstantRestrictedIDCache<InstantMostVisitedItem> most_visited_items_cache_;
   ThemeBackgroundInfo theme_info_;
   base::string16 query_;
