@@ -16,8 +16,8 @@
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
-#include "chrome/common/instant_types.h"
-#include "chrome/common/ntp_logging_events.h"
+#include "chrome/common/search/instant_types.h"
+#include "chrome/common/search/ntp_logging_events.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/grit/renderer_resources.h"
 #include "chrome/renderer/searchbox/searchbox.h"
@@ -239,12 +239,6 @@ content::RenderView* GetRenderViewWithCheckedOrigin(const GURL& origin) {
   return render_view;
 }
 
-// Returns the current URL.
-GURL GetCurrentURL(content::RenderView* render_view) {
-  blink::WebView* webview = render_view->GetWebView();
-  return webview ? GURL(webview->mainFrame()->document().url()) : GURL();
-}
-
 }  // namespace
 
 namespace internal {  // for testing.
@@ -416,7 +410,7 @@ class SearchBoxExtensionWrapper : public v8::Extension {
   static void CheckIsUserSignedInToChromeAs(
       const v8::FunctionCallbackInfo<v8::Value>& args);
 
-  // Checks whether the user sync his history.
+  // Checks whether the user syncs their history.
   static void CheckIsUserSyncingHistory(
       const v8::FunctionCallbackInfo<v8::Value>& args);
 
@@ -424,17 +418,6 @@ class SearchBoxExtensionWrapper : public v8::Extension {
   static void DeleteMostVisitedItem(
       const v8::FunctionCallbackInfo<v8::Value>& args);
 
-  // Focuses the omnibox.
-  static void Focus(const v8::FunctionCallbackInfo<v8::Value>& args);
-
-  // Gets whether or not the app launcher is enabled.
-  static void GetAppLauncherEnabled(
-      const v8::FunctionCallbackInfo<v8::Value>& args);
-
-  // Gets the desired navigation behavior from a click event.
-  static void GetDispositionFromClick(
-      const v8::FunctionCallbackInfo<v8::Value>& args);
-
   // Gets Most Visited Items.
   static void GetMostVisitedItems(
       const v8::FunctionCallbackInfo<v8::Value>& args);
@@ -486,11 +469,6 @@ class SearchBoxExtensionWrapper : public v8::Extension {
   static void LogMostVisitedNavigation(
       const v8::FunctionCallbackInfo<v8::Value>& args);
 
-  // Navigates the window to a URL represented by either a URL string or a
-  // restricted ID.
-  static void NavigateContentWindow(
-      const v8::FunctionCallbackInfo<v8::Value>& args);
-
   // Pastes provided value or clipboard's content into the omnibox.
   static void Paste(const v8::FunctionCallbackInfo<v8::Value>& args);
 
@@ -510,10 +488,6 @@ class SearchBoxExtensionWrapper : public v8::Extension {
   static void UndoMostVisitedDeletion(
       const v8::FunctionCallbackInfo<v8::Value>& args);
 
-  // Indicates whether the page supports Instant.
-  static void GetDisplayInstantResults(
-      const v8::FunctionCallbackInfo<v8::Value>& args);
-
  private:
   DISALLOW_COPY_AND_ASSIGN(SearchBoxExtensionWrapper);
 };
@@ -614,12 +588,6 @@ SearchBoxExtensionWrapper::GetNativeFunctionTemplate(
     return v8::FunctionTemplate::New(isolate, CheckIsUserSyncingHistory);
   if (name->Equals(v8::String::NewFromUtf8(isolate, "DeleteMostVisitedItem")))
     return v8::FunctionTemplate::New(isolate, DeleteMostVisitedItem);
-  if (name->Equals(v8::String::NewFromUtf8(isolate, "Focus")))
-    return v8::FunctionTemplate::New(isolate, Focus);
-  if (name->Equals(v8::String::NewFromUtf8(isolate, "GetAppLauncherEnabled")))
-    return v8::FunctionTemplate::New(isolate, GetAppLauncherEnabled);
-  if (name->Equals(v8::String::NewFromUtf8(isolate, "GetDispositionFromClick")))
-    return v8::FunctionTemplate::New(isolate, GetDispositionFromClick);
   if (name->Equals(v8::String::NewFromUtf8(isolate, "GetMostVisitedItems")))
     return v8::FunctionTemplate::New(isolate, GetMostVisitedItems);
   if (name->Equals(v8::String::NewFromUtf8(isolate, "GetMostVisitedItemData")))
@@ -650,8 +618,6 @@ SearchBoxExtensionWrapper::GetNativeFunctionTemplate(
           v8::String::NewFromUtf8(isolate, "LogMostVisitedNavigation"))) {
     return v8::FunctionTemplate::New(isolate, LogMostVisitedNavigation);
   }
-  if (name->Equals(v8::String::NewFromUtf8(isolate, "NavigateContentWindow")))
-    return v8::FunctionTemplate::New(isolate, NavigateContentWindow);
   if (name->Equals(v8::String::NewFromUtf8(isolate, "Paste")))
     return v8::FunctionTemplate::New(isolate, Paste);
   if (name->Equals(
@@ -664,9 +630,6 @@ SearchBoxExtensionWrapper::GetNativeFunctionTemplate(
     return v8::FunctionTemplate::New(isolate, UndoAllMostVisitedDeletions);
   if (name->Equals(v8::String::NewFromUtf8(isolate, "UndoMostVisitedDeletion")))
     return v8::FunctionTemplate::New(isolate, UndoMostVisitedDeletion);
-  if (name->Equals(
-          v8::String::NewFromUtf8(isolate, "GetDisplayInstantResults")))
-    return v8::FunctionTemplate::New(isolate, GetDisplayInstantResults);
   return v8::Local<v8::FunctionTemplate>();
 }
 
@@ -726,52 +689,6 @@ void SearchBoxExtensionWrapper::DeleteMostVisitedItem(
       DeleteMostVisitedItem(args[0]->ToInteger()->Value());
 }
 
-// static
-void SearchBoxExtensionWrapper::Focus(
-    const v8::FunctionCallbackInfo<v8::Value>& args) {
-  content::RenderView* render_view = GetRenderView();
-  if (!render_view) return;
-
-  DVLOG(1) << render_view << " Focus";
-  SearchBox::Get(render_view)->Focus();
-}
-
-// static
-void SearchBoxExtensionWrapper::GetAppLauncherEnabled(
-    const v8::FunctionCallbackInfo<v8::Value>& args) {
-  content::RenderView* render_view = GetRenderView();
-  if (!render_view) return;
-
-  args.GetReturnValue().Set(
-      SearchBox::Get(render_view)->app_launcher_enabled());
-}
-
-// static
-void SearchBoxExtensionWrapper::GetDispositionFromClick(
-    const v8::FunctionCallbackInfo<v8::Value>& args) {
-  content::RenderView* render_view = GetRenderView();
-  if (!render_view) return;
-
-  if (args.Length() != 5) {
-    ThrowInvalidParameters(args);
-    return;
-  }
-
-  bool middle_button = args[0]->BooleanValue();
-  bool alt_key = args[1]->BooleanValue();
-  bool ctrl_key = args[2]->BooleanValue();
-  bool meta_key = args[3]->BooleanValue();
-  bool shift_key = args[4]->BooleanValue();
-
-  WindowOpenDisposition disposition = ui::DispositionFromClick(middle_button,
-                                                               alt_key,
-                                                               ctrl_key,
-                                                               meta_key,
-                                                               shift_key);
-  v8::Isolate* isolate = args.GetIsolate();
-  args.GetReturnValue().Set(v8::Int32::New(isolate, disposition));
-}
-
 // static
 void SearchBoxExtensionWrapper::GetMostVisitedItems(
     const v8::FunctionCallbackInfo<v8::Value>& args) {
@@ -1105,15 +1022,19 @@ void SearchBoxExtensionWrapper::LogMostVisitedImpression(
       GURL(chrome::kChromeSearchMostVisitedUrl));
   if (!render_view) return;
 
-  if (args.Length() < 2 || !args[0]->IsNumber() || args[1]->IsUndefined()) {
+  if (args.Length() < 2 || !args[0]->IsNumber() || !args[1]->IsNumber()) {
     ThrowInvalidParameters(args);
     return;
   }
 
   DVLOG(1) << render_view << " LogMostVisitedImpression";
 
-  SearchBox::Get(render_view)->LogMostVisitedImpression(
-      args[0]->IntegerValue(), V8ValueToUTF16(args[1]));
+  if (args[1]->Uint32Value() <= static_cast<int>(NTPLoggingTileSource::LAST)) {
+    NTPLoggingTileSource tile_source =
+        static_cast<NTPLoggingTileSource>(args[1]->Uint32Value());
+    SearchBox::Get(render_view)->LogMostVisitedImpression(
+        args[0]->IntegerValue(), tile_source);
+  }
 }
 
 // static
@@ -1123,58 +1044,18 @@ void SearchBoxExtensionWrapper::LogMostVisitedNavigation(
       GURL(chrome::kChromeSearchMostVisitedUrl));
   if (!render_view) return;
 
-  if (args.Length() < 2 || !args[0]->IsNumber() || args[1]->IsUndefined()) {
+  if (args.Length() < 2 || !args[0]->IsNumber() || !args[1]->IsNumber()) {
     ThrowInvalidParameters(args);
     return;
   }
 
   DVLOG(1) << render_view << " LogMostVisitedNavigation";
 
-  SearchBox::Get(render_view)->LogMostVisitedNavigation(
-      args[0]->IntegerValue(), V8ValueToUTF16(args[1]));
-}
-
-// static
-void SearchBoxExtensionWrapper::NavigateContentWindow(
-    const v8::FunctionCallbackInfo<v8::Value>& args) {
-  content::RenderView* render_view = GetRenderView();
-  if (!render_view) return;
-
-  if (!args.Length()) {
-    ThrowInvalidParameters(args);
-    return;
-  }
-
-  GURL destination_url;
-  bool is_most_visited_item_url = false;
-  // Check if the url is a rid
-  if (args[0]->IsNumber()) {
-    InstantMostVisitedItem item;
-    if (SearchBox::Get(render_view)->GetMostVisitedItemWithID(
-            args[0]->IntegerValue(), &item)) {
-      destination_url = item.url;
-      is_most_visited_item_url = true;
-    }
-  } else {
-    // Resolve the URL
-    const base::string16& possibly_relative_url = V8ValueToUTF16(args[0]);
-  GURL current_url = GetCurrentURL(render_view);
-    destination_url = internal::ResolveURL(current_url, possibly_relative_url);
-  }
-
-  DVLOG(1) << render_view << " NavigateContentWindow: " << destination_url;
-
-  // Navigate the main frame. Note that the security checks are enforced by the
-  // browser process in InstantService::IsValidURLForNavigation(), but some
-  // simple checks here are useful for avoiding unnecessary IPCs.
-  if (destination_url.is_valid() &&
-      !destination_url.SchemeIs(url::kJavaScriptScheme)) {
-    WindowOpenDisposition disposition = CURRENT_TAB;
-    if (args[1]->IsNumber()) {
-      disposition = (WindowOpenDisposition) args[1]->Uint32Value();
-    }
-    SearchBox::Get(render_view)->NavigateToURL(destination_url, disposition,
-                                               is_most_visited_item_url);
+  if (args[1]->Uint32Value() <= static_cast<int>(NTPLoggingTileSource::LAST)) {
+    NTPLoggingTileSource tile_source =
+        static_cast<NTPLoggingTileSource>(args[1]->Uint32Value());
+    SearchBox::Get(render_view)->LogMostVisitedNavigation(
+        args[0]->IntegerValue(), tile_source);
   }
 }
 
@@ -1239,17 +1120,4 @@ void SearchBoxExtensionWrapper::UndoMostVisitedDeletion(
       ->UndoMostVisitedDeletion(args[0]->ToInteger()->Value());
 }
 
-// static
-void SearchBoxExtensionWrapper::GetDisplayInstantResults(
-    const v8::FunctionCallbackInfo<v8::Value>& args) {
-  content::RenderView* render_view = GetRenderView();
-  if (!render_view) return;
-
-  bool display_instant_results =
-      SearchBox::Get(render_view)->display_instant_results();
-  DVLOG(1) << render_view << " GetDisplayInstantResults" <<
-      display_instant_results;
-  args.GetReturnValue().Set(display_instant_results);
-}
-
 }  // namespace extensions_v8
