@@ -6,12 +6,14 @@
 
 #include "build/build_config.h"
 #include "chrome/test/base/chrome_test_suite.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/interactive_test_utils.h"
 #include "ui/base/test/ui_controls.h"
 
 #if defined(USE_AURA)
 #include "ui/aura/test/ui_controls_factory_aura.h"
 #include "ui/base/test/ui_controls_aura.h"
-#if defined(OS_LINUX) && !defined(OS_CHROMEOS)
+#if defined(USE_X11) && !defined(OS_CHROMEOS)
 #include "ui/views/test/ui_controls_factory_desktop_aurax11.h"
 #endif
 #endif
@@ -47,13 +49,17 @@ class InteractiveUITestSuite : public ChromeTestSuite {
 #endif
 
 #if defined(OS_LINUX)
+#if defined(USE_OZONE)
+    NOTIMPLEMENTED();
+#else
     ui_controls::InstallUIControlsAura(
         views::test::CreateUIControlsDesktopAura());
+#endif  // defined(USE_OZONE)
 #else
     // TODO(win_ash): when running interactive_ui_tests for Win Ash, use above.
     ui_controls::InstallUIControlsAura(aura::test::CreateUIControlsAura(NULL));
-#endif
-#endif
+#endif  // defined(OS_LINUX)
+#endif  // defined(USE_AURA)
   }
 
   void Shutdown() override {
@@ -64,7 +70,7 @@ class InteractiveUITestSuite : public ChromeTestSuite {
 
  private:
 #if defined(OS_WIN)
-  scoped_ptr<base::win::ScopedCOMInitializer> com_initializer_;
+  std::unique_ptr<base::win::ScopedCOMInitializer> com_initializer_;
 #endif
 };
 
@@ -78,11 +84,24 @@ class InteractiveUITestSuiteRunner : public ChromeTestSuiteRunner {
 int main(int argc, char** argv) {
 #if defined(OS_WIN)
   KillAlwaysOnTopWindows(RunType::BEFORE_TEST);
+#endif
+  // TODO(sky): this causes a crash in an autofill test on macosx, figure out
+  // why: http://crbug.com/641969.
+#if !defined(OS_MACOSX)
+  // Without this it's possible for the first browser to start up in the
+  // background, generally because the last test did something that causes the
+  // test to run in the background. Most interactive ui tests assume they are in
+  // the foreground and fail in weird ways if they aren't (for example, clicks
+  // go to the wrong place). This ensures the first browser is always in the
+  // foreground.
+  InProcessBrowserTest::set_global_browser_set_up_function(
+      &ui_test_utils::BringBrowserWindowToFront);
 #endif
   // Run interactive_ui_tests serially, they do not support running in parallel.
   int default_jobs = 1;
   InteractiveUITestSuiteRunner runner;
-  const int result = LaunchChromeTests(default_jobs, &runner, argc, argv);
+  ChromeTestLauncherDelegate delegate(&runner);
+  const int result = LaunchChromeTests(default_jobs, &delegate, argc, argv);
 #if defined(OS_WIN)
   KillAlwaysOnTopWindows(RunType::AFTER_TEST);
 #endif
