@@ -62,9 +62,10 @@
 namespace {
 
 const char* const kCommonSwitches[] = {
+  "disable-infobars",
   "disable-popup-blocking",
   "ignore-certificate-errors",
-  "metrics-recording-only"
+  "metrics-recording-only",
 };
 
 const char* const kDesktopSwitches[] = {
@@ -75,20 +76,18 @@ const char* const kDesktopSwitches[] = {
   "disable-background-networking",
   "disable-web-resources",
   "safebrowsing-disable-auto-update",
-  "safebrowsing-disable-download-protection",
   "disable-client-side-phishing-detection",
-  "disable-component-update",
   "disable-default-apps",
   "enable-logging",
   "log-level=0",
   "password-store=basic",
   "use-mock-keychain",
-  "test-type=webdriver"
+  "test-type=webdriver",
 };
 
 const char* const kAndroidSwitches[] = {
   "disable-fre",
-  "enable-remote-debugging"
+  "enable-remote-debugging",
 };
 
 #if defined(OS_LINUX)
@@ -134,9 +133,9 @@ Status PrepareCommandLine(uint16_t port,
   base::CommandLine command(program);
   Switches switches;
 
-  for (const auto& common_switch : kCommonSwitches)
+  for (auto* common_switch : kCommonSwitches)
     switches.SetUnparsedSwitch(common_switch);
-  for (const auto& desktop_switch : kDesktopSwitches)
+  for (auto* desktop_switch : kDesktopSwitches)
     switches.SetUnparsedSwitch(desktop_switch);
   switches.SetSwitch("remote-debugging-port", base::UintToString(port));
   for (const auto& excluded_switch : capabilities.exclude_switches) {
@@ -152,8 +151,8 @@ Status PrepareCommandLine(uint16_t port,
     command.AppendArg("data:,");
     if (!user_data_dir->CreateUniqueTempDir())
       return Status(kUnknownError, "cannot create temp dir for user data dir");
-    switches.SetSwitch("user-data-dir", user_data_dir->path().value());
-    user_data_dir_path = user_data_dir->path();
+    switches.SetSwitch("user-data-dir", user_data_dir->GetPath().value());
+    user_data_dir_path = user_data_dir->GetPath();
   }
 
   Status status = internal::PrepareUserDataDir(user_data_dir_path,
@@ -167,10 +166,8 @@ Status PrepareCommandLine(uint16_t port,
                   "cannot create temp dir for unpacking extensions");
   }
   status = internal::ProcessExtensions(capabilities.extensions,
-                                       extension_dir->path(),
-                                       true,
-                                       &switches,
-                                       extension_bg_pages);
+                                       extension_dir->GetPath(), true,
+                                       &switches, extension_bg_pages);
   if (status.IsError())
     return status;
   switches.AppendToCommandLine(&command);
@@ -183,12 +180,12 @@ Status WaitForDevToolsAndCheckVersion(
     URLRequestContextGetter* context_getter,
     const SyncWebSocketFactory& socket_factory,
     const Capabilities* capabilities,
-    scoped_ptr<DevToolsHttpClient>* user_client) {
-  scoped_ptr<DeviceMetrics> device_metrics;
+    std::unique_ptr<DevToolsHttpClient>* user_client) {
+  std::unique_ptr<DeviceMetrics> device_metrics;
   if (capabilities && capabilities->device_metrics)
     device_metrics.reset(new DeviceMetrics(*capabilities->device_metrics));
 
-  scoped_ptr<std::set<WebViewInfo::Type>> window_types;
+  std::unique_ptr<std::set<WebViewInfo::Type>> window_types;
   if (capabilities && !capabilities->window_types.empty()) {
     window_types.reset(
         new std::set<WebViewInfo::Type>(capabilities->window_types));
@@ -196,22 +193,38 @@ Status WaitForDevToolsAndCheckVersion(
     window_types.reset(new std::set<WebViewInfo::Type>());
   }
 
-  scoped_ptr<DevToolsHttpClient> client(new DevToolsHttpClient(
+  std::unique_ptr<DevToolsHttpClient> client(new DevToolsHttpClient(
       address, context_getter, socket_factory, std::move(device_metrics),
-      std::move(window_types)));
+      std::move(window_types), capabilities->page_load_strategy));
   base::TimeTicks deadline =
       base::TimeTicks::Now() + base::TimeDelta::FromSeconds(60);
   Status status = client->Init(deadline - base::TimeTicks::Now());
   if (status.IsError())
     return status;
 
+  const BrowserInfo* browser_info = client->browser_info();
+  if (browser_info->is_android &&
+      browser_info->android_package != capabilities->android_package) {
+    // DevTools from Chrome 30 and earlier did not provide an Android-Package
+    // key, so skip the package check for WebView on KitKat and older.
+    // TODO(samuong): Make this unconditional once we stop supporting Android
+    // KitKat WebView apps.
+    if (!(browser_info->browser_name == "webview" &&
+          browser_info->major_version <= 30 &&
+          browser_info->android_package.empty())) {
+      return Status(
+          kSessionNotCreatedException,
+          base::StringPrintf("please close '%s' and try again",
+                             browser_info->android_package.c_str()));
+    }
+  }
+
   base::CommandLine* cmd_line = base::CommandLine::ForCurrentProcess();
   if (cmd_line->HasSwitch("disable-build-check")) {
     LOG(WARNING) << "You are using an unsupported command-line switch: "
                     "--disable-build-check. Please don't report bugs that "
                     "cannot be reproduced with this switch removed.";
-  } else if (client->browser_info()->build_no <
-             kMinimumSupportedChromeBuildNo) {
+  } else if (browser_info->build_no < kMinimumSupportedChromeBuildNo) {
     return Status(kUnknownError, "Chrome version must be >= " +
         GetMinimumSupportedChromeVersion());
   }
@@ -235,8 +248,8 @@ Status CreateBrowserwideDevToolsClientAndConnect(
     const PerfLoggingPrefs& perf_logging_prefs,
     const SyncWebSocketFactory& socket_factory,
     const ScopedVector<DevToolsEventListener>& devtools_event_listeners,
-    scoped_ptr<DevToolsClient>* browser_client) {
-  scoped_ptr<DevToolsClient> client(new DevToolsClientImpl(
+    std::unique_ptr<DevToolsClient>* browser_client) {
+  std::unique_ptr<DevToolsClient> client(new DevToolsClientImpl(
       socket_factory, base::StringPrintf("ws://%s/devtools/browser/",
                                          address.ToString().c_str()),
       DevToolsClientImpl::kBrowserwideDevToolsClientId));
@@ -269,19 +282,19 @@ Status LaunchRemoteChromeSession(
     const SyncWebSocketFactory& socket_factory,
     const Capabilities& capabilities,
     ScopedVector<DevToolsEventListener>* devtools_event_listeners,
-    scoped_ptr<Chrome>* chrome) {
+    std::unique_ptr<Chrome>* chrome) {
   Status status(kOk);
-  scoped_ptr<DevToolsHttpClient> devtools_http_client;
+  std::unique_ptr<DevToolsHttpClient> devtools_http_client;
   status = WaitForDevToolsAndCheckVersion(
       capabilities.debugger_address, context_getter, socket_factory,
-      NULL, &devtools_http_client);
+      &capabilities, &devtools_http_client);
   if (status.IsError()) {
     return Status(kUnknownError, "cannot connect to chrome at " +
                       capabilities.debugger_address.ToString(),
                   status);
   }
 
-  scoped_ptr<DevToolsClient> devtools_websocket_client;
+  std::unique_ptr<DevToolsClient> devtools_websocket_client;
   status = CreateBrowserwideDevToolsClientAndConnect(
       capabilities.debugger_address, capabilities.perf_logging_prefs,
       socket_factory, *devtools_event_listeners, &devtools_websocket_client);
@@ -292,18 +305,19 @@ Status LaunchRemoteChromeSession(
 
   chrome->reset(new ChromeRemoteImpl(std::move(devtools_http_client),
                                      std::move(devtools_websocket_client),
-                                     *devtools_event_listeners));
+                                     *devtools_event_listeners,
+                                     capabilities.page_load_strategy));
   return Status(kOk);
 }
 
 Status LaunchDesktopChrome(
     URLRequestContextGetter* context_getter,
     uint16_t port,
-    scoped_ptr<PortReservation> port_reservation,
+    std::unique_ptr<PortReservation> port_reservation,
     const SyncWebSocketFactory& socket_factory,
     const Capabilities& capabilities,
     ScopedVector<DevToolsEventListener>* devtools_event_listeners,
-    scoped_ptr<Chrome>* chrome) {
+    std::unique_ptr<Chrome>* chrome) {
   base::CommandLine command(base::CommandLine::NO_PROGRAM);
   base::ScopedTempDir user_data_dir;
   base::ScopedTempDir extension_dir;
@@ -371,7 +385,7 @@ Status LaunchDesktopChrome(
   if (!process.IsValid())
     return Status(kUnknownError, "chrome failed to start");
 
-  scoped_ptr<DevToolsHttpClient> devtools_http_client;
+  std::unique_ptr<DevToolsHttpClient> devtools_http_client;
   status = WaitForDevToolsAndCheckVersion(
       NetAddress(port), context_getter, socket_factory, &capabilities,
       &devtools_http_client);
@@ -417,7 +431,7 @@ Status LaunchDesktopChrome(
     return status;
   }
 
-  scoped_ptr<DevToolsClient> devtools_websocket_client;
+  std::unique_ptr<DevToolsClient> devtools_websocket_client;
   status = CreateBrowserwideDevToolsClientAndConnect(
       NetAddress(port), capabilities.perf_logging_prefs, socket_factory,
       *devtools_event_listeners, &devtools_websocket_client);
@@ -426,13 +440,20 @@ Status LaunchDesktopChrome(
                  << status.message();
   }
 
-  scoped_ptr<ChromeDesktopImpl> chrome_desktop(new ChromeDesktopImpl(
-      std::move(devtools_http_client), std::move(devtools_websocket_client),
-      *devtools_event_listeners, std::move(port_reservation),
-      std::move(process), command, &user_data_dir, &extension_dir));
+  std::unique_ptr<ChromeDesktopImpl> chrome_desktop(
+      new ChromeDesktopImpl(std::move(devtools_http_client),
+                            std::move(devtools_websocket_client),
+                            *devtools_event_listeners,
+                            std::move(port_reservation),
+                            capabilities.page_load_strategy,
+                            std::move(process),
+                            command,
+                            &user_data_dir,
+                            &extension_dir,
+                            capabilities.network_emulation_enabled));
   for (size_t i = 0; i < extension_bg_pages.size(); ++i) {
     VLOG(0) << "Waiting for extension bg page load: " << extension_bg_pages[i];
-    scoped_ptr<WebView> web_view;
+    std::unique_ptr<WebView> web_view;
     Status status = chrome_desktop->WaitForPageToLoad(
         extension_bg_pages[i], base::TimeDelta::FromSeconds(10), &web_view);
     if (status.IsError()) {
@@ -449,14 +470,14 @@ Status LaunchDesktopChrome(
 Status LaunchAndroidChrome(
     URLRequestContextGetter* context_getter,
     uint16_t port,
-    scoped_ptr<PortReservation> port_reservation,
+    std::unique_ptr<PortReservation> port_reservation,
     const SyncWebSocketFactory& socket_factory,
     const Capabilities& capabilities,
     ScopedVector<DevToolsEventListener>* devtools_event_listeners,
     DeviceManager* device_manager,
-    scoped_ptr<Chrome>* chrome) {
+    std::unique_ptr<Chrome>* chrome) {
   Status status(kOk);
-  scoped_ptr<Device> device;
+  std::unique_ptr<Device> device;
   if (capabilities.android_device_serial.empty()) {
     status = device_manager->AcquireDevice(&device);
   } else {
@@ -467,10 +488,12 @@ Status LaunchAndroidChrome(
     return status;
 
   Switches switches(capabilities.switches);
-  for (auto common_switch : kCommonSwitches)
+  for (auto* common_switch : kCommonSwitches)
     switches.SetUnparsedSwitch(common_switch);
-  for (auto android_switch : kAndroidSwitches)
+  for (auto* android_switch : kAndroidSwitches)
     switches.SetUnparsedSwitch(android_switch);
+  for (auto excluded_switch : capabilities.exclude_switches)
+    switches.RemoveSwitch(excluded_switch);
   status = device->SetUp(capabilities.android_package,
                          capabilities.android_activity,
                          capabilities.android_process,
@@ -482,7 +505,7 @@ Status LaunchAndroidChrome(
     return status;
   }
 
-  scoped_ptr<DevToolsHttpClient> devtools_http_client;
+  std::unique_ptr<DevToolsHttpClient> devtools_http_client;
   status = WaitForDevToolsAndCheckVersion(NetAddress(port),
                                           context_getter,
                                           socket_factory,
@@ -493,7 +516,7 @@ Status LaunchAndroidChrome(
     return status;
   }
 
-  scoped_ptr<DevToolsClient> devtools_websocket_client;
+  std::unique_ptr<DevToolsClient> devtools_websocket_client;
   status = CreateBrowserwideDevToolsClientAndConnect(
       NetAddress(port), capabilities.perf_logging_prefs, socket_factory,
       *devtools_event_listeners, &devtools_websocket_client);
@@ -502,10 +525,12 @@ Status LaunchAndroidChrome(
                  << status.message();
   }
 
-  chrome->reset(new ChromeAndroidImpl(
-      std::move(devtools_http_client), std::move(devtools_websocket_client),
-      *devtools_event_listeners, std::move(port_reservation),
-      std::move(device)));
+  chrome->reset(new ChromeAndroidImpl(std::move(devtools_http_client),
+                                      std::move(devtools_websocket_client),
+                                      *devtools_event_listeners,
+                                      std::move(port_reservation),
+                                      capabilities.page_load_strategy,
+                                      std::move(device)));
   return Status(kOk);
 }
 
@@ -519,7 +544,7 @@ Status LaunchChrome(
     PortManager* port_manager,
     const Capabilities& capabilities,
     ScopedVector<DevToolsEventListener>* devtools_event_listeners,
-    scoped_ptr<Chrome>* chrome) {
+    std::unique_ptr<Chrome>* chrome) {
   if (capabilities.IsRemoteBrowser()) {
     return LaunchRemoteChromeSession(
         context_getter, socket_factory,
@@ -527,7 +552,7 @@ Status LaunchChrome(
   }
 
   uint16_t port = 0;
-  scoped_ptr<PortReservation> port_reservation;
+  std::unique_ptr<PortReservation> port_reservation;
   Status port_status(kOk);
 
   if (capabilities.IsAndroid()) {
@@ -629,7 +654,7 @@ Status ProcessExtension(const std::string& extension,
       return Status(kUnknownError, "invalid public key length");
   } else {
     // Not a CRX file. Generate RSA keypair to get a valid extension id.
-    scoped_ptr<crypto::RSAPrivateKey> key_pair(
+    std::unique_ptr<crypto::RSAPrivateKey> key_pair(
         crypto::RSAPrivateKey::Create(2048));
     if (!key_pair)
       return Status(kUnknownError, "cannot generate RSA key pair");
@@ -648,7 +673,7 @@ Status ProcessExtension(const std::string& extension,
   base::ScopedTempDir temp_crx_dir;
   if (!temp_crx_dir.CreateUniqueTempDir())
     return Status(kUnknownError, "cannot create temp dir");
-  base::FilePath extension_crx = temp_crx_dir.path().AppendASCII("temp.crx");
+  base::FilePath extension_crx = temp_crx_dir.GetPath().AppendASCII("temp.crx");
   int size = static_cast<int>(decoded_extension.length());
   if (base::WriteFile(extension_crx, decoded_extension.c_str(), size) !=
       size) {
@@ -663,7 +688,7 @@ Status ProcessExtension(const std::string& extension,
   std::string manifest_data;
   if (!base::ReadFileToString(manifest_path, &manifest_data))
     return Status(kUnknownError, "cannot read manifest");
-  scoped_ptr<base::Value> manifest_value =
+  std::unique_ptr<base::Value> manifest_value =
       base::JSONReader::Read(manifest_data);
   base::DictionaryValue* manifest;
   if (!manifest_value || !manifest_value->GetAsDictionary(&manifest))
@@ -749,12 +774,20 @@ Status ProcessExtensions(const std::vector<std::string>& extensions,
     Status status = UnpackAutomationExtension(temp_dir, &automation_extension);
     if (status.IsError())
       return status;
+#if defined(OS_WIN) || defined(OS_MACOSX)
+    // On Chrome for Windows and Mac, a "Disable developer mode extensions"
+    // dialog appears for the automation extension. Suppress this by loading
+    // it as a component extension.
+    UpdateExtensionSwitch(switches, "load-component-extension",
+                          automation_extension.value());
+#else
     if (switches->HasSwitch("disable-extensions")) {
       UpdateExtensionSwitch(switches, "load-component-extension",
                             automation_extension.value());
     } else {
       extension_paths.push_back(automation_extension.value());
     }
+#endif
   }
 
   if (extension_paths.size()) {
@@ -772,8 +805,9 @@ Status WritePrefsFile(
     const base::FilePath& path) {
   int code;
   std::string error_msg;
-  scoped_ptr<base::Value> template_value = base::JSONReader::ReadAndReturnError(
-      template_string, 0, &code, &error_msg);
+  std::unique_ptr<base::Value> template_value =
+      base::JSONReader::ReadAndReturnError(template_string, 0, &code,
+                                           &error_msg);
   base::DictionaryValue* prefs;
   if (!template_value || !template_value->GetAsDictionary(&prefs)) {
     return Status(kUnknownError,
