@@ -13,9 +13,10 @@
 #include "build/build_config.h"
 #include "chrome/common/media_galleries/picasa_types.h"
 #include "chrome/utility/utility_message_handler.h"
+#include "extensions/features/features.h"
 #include "extensions/utility/utility_handler.h"
 
-#if !defined(ENABLE_EXTENSIONS)
+#if !BUILDFLAG(ENABLE_EXTENSIONS)
 #error "Extensions must be enabled"
 #endif
 
@@ -51,11 +52,6 @@ class ExtensionsHandler : public UtilityMessageHandler {
   void OnParseITunesPrefXml(const std::string& itunes_xml_data);
 #endif  // defined(OS_WIN)
 
-#if defined(OS_MACOSX)
-  void OnParseIPhotoLibraryXmlFile(
-      const IPC::PlatformFileForTransit& iphoto_library_file);
-#endif  // defined(OS_MACOSX)
-
 #if defined(OS_WIN) || defined(OS_MACOSX)
   void OnParseITunesLibraryXmlFile(
       const IPC::PlatformFileForTransit& itunes_library_file);
