@@ -4,15 +4,15 @@
 
 #include "chrome/utility/importer/firefox_importer.h"
 
+#include <memory>
 #include <set>
 
 #include "base/files/file_enumerator.h"
 #include "base/files/file_util.h"
 #include "base/json/json_file_value_serializer.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
-#include "base/stl_util.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "build/build_config.h"
@@ -246,9 +246,7 @@ void FirefoxImporter::ImportBookmarks() {
   while (s.Step() && !cancelled())
     post_keyword_ids.insert(s.ColumnInt(0));
 
-  for (size_t i = 0; i < list.size(); ++i) {
-    BookmarkItem* item = list[i];
-
+  for (const auto& item : list) {
     // Folders are added implicitly on adding children, so we only explicitly
     // add empty folders.
     if (item->type != TYPE_BOOKMARK &&
@@ -263,11 +261,11 @@ void FirefoxImporter::ImportBookmarks() {
 
       // Find the bookmark path by tracing their links to parent folders.
       std::vector<base::string16> path;
-      BookmarkItem* child = item;
+      BookmarkItem* child = item.get();
       bool found_path = false;
       bool is_in_toolbar = false;
       while (child->parent >= 0) {
-        BookmarkItem* parent = list[child->parent];
+        BookmarkItem* parent = list[child->parent].get();
         if (livemark_id.find(parent->id) != livemark_id.end()) {
           // Don't import live bookmarks.
           break;
@@ -333,8 +331,6 @@ void FirefoxImporter::ImportBookmarks() {
     }
   }
 
-  STLDeleteElements(&list);
-
   // Write into profile.
   if (!bookmarks.empty() && !cancelled()) {
     const base::string16& first_folder_name =
@@ -542,7 +538,7 @@ void FirefoxImporter::GetSearchEnginesXMLDataFromJSON(
   base::FilePath search_metadata_json_file =
       source_path_.AppendASCII("search-metadata.json");
   JSONFileValueDeserializer metadata_deserializer(search_metadata_json_file);
-  scoped_ptr<base::Value> metadata_root =
+  std::unique_ptr<base::Value> metadata_root =
       metadata_deserializer.Deserialize(NULL, NULL);
   const base::DictionaryValue* search_metadata_root = NULL;
   if (metadata_root)
@@ -554,7 +550,7 @@ void FirefoxImporter::GetSearchEnginesXMLDataFromJSON(
     return;
 
   JSONFileValueDeserializer deserializer(search_json_file);
-  scoped_ptr<base::Value> root = deserializer.Deserialize(NULL, NULL);
+  std::unique_ptr<base::Value> root = deserializer.Deserialize(NULL, NULL);
   const base::DictionaryValue* search_root = NULL;
   if (!root || !root->GetAsDictionary(&search_root))
     return;
@@ -671,7 +667,7 @@ void FirefoxImporter::LoadRootNodeID(sql::Connection* db,
 }
 
 void FirefoxImporter::LoadLivemarkIDs(sql::Connection* db,
-                                       std::set<int>* livemark) {
+                                      std::set<int>* livemark) {
   static const char kFeedAnnotation[] = "livemark/feedURI";
   livemark->clear();
 
@@ -688,8 +684,8 @@ void FirefoxImporter::LoadLivemarkIDs(sql::Connection* db,
 }
 
 void FirefoxImporter::GetTopBookmarkFolder(sql::Connection* db,
-                                            int folder_id,
-                                            BookmarkList* list) {
+                                           int folder_id,
+                                           BookmarkList* list) {
   const char query[] =
       "SELECT b.title "
       "FROM moz_bookmarks b "
@@ -699,21 +695,21 @@ void FirefoxImporter::GetTopBookmarkFolder(sql::Connection* db,
   s.BindInt(0, folder_id);
 
   if (s.Step()) {
-    BookmarkItem* item = new BookmarkItem;
+    std::unique_ptr<BookmarkItem> item = base::MakeUnique<BookmarkItem>();
     item->parent = -1;  // The top level folder has no parent.
     item->id = folder_id;
     item->title = s.ColumnString16(0);
     item->type = TYPE_FOLDER;
     item->favicon = 0;
     item->empty_folder = true;
-    list->push_back(item);
+    list->push_back(std::move(item));
   }
 }
 
 void FirefoxImporter::GetWholeBookmarkFolder(sql::Connection* db,
-                                              BookmarkList* list,
-                                              size_t position,
-                                              bool* empty_folder) {
+                                             BookmarkList* list,
+                                             size_t position,
+                                             bool* empty_folder) {
   if (position >= list->size()) {
     NOTREACHED();
     return;
@@ -732,7 +728,7 @@ void FirefoxImporter::GetWholeBookmarkFolder(sql::Connection* db,
 
   BookmarkList temp_list;
   while (s.Step()) {
-    BookmarkItem* item = new BookmarkItem;
+    std::unique_ptr<BookmarkItem> item = base::MakeUnique<BookmarkItem>();
     item->parent = static_cast<int>(position);
     item->id = s.ColumnInt(0);
     item->url = GURL(s.ColumnString(1));
@@ -743,18 +739,19 @@ void FirefoxImporter::GetWholeBookmarkFolder(sql::Connection* db,
     item->favicon = s.ColumnInt64(6);
     item->empty_folder = true;
 
-    temp_list.push_back(item);
+    temp_list.push_back(std::move(item));
     if (empty_folder != NULL)
       *empty_folder = false;
   }
 
   // Appends all items to the list.
-  for (BookmarkList::iterator i = temp_list.begin();
-       i != temp_list.end(); ++i) {
-    list->push_back(*i);
+  for (auto& bookmark : temp_list) {
+    list->push_back(std::move(bookmark));
     // Recursive add bookmarks in sub-folders.
-    if ((*i)->type == TYPE_FOLDER)
-      GetWholeBookmarkFolder(db, list, list->size() - 1, &(*i)->empty_folder);
+    if (list->back()->type == TYPE_FOLDER) {
+      GetWholeBookmarkFolder(db, list, list->size() - 1,
+                             &list->back()->empty_folder);
+    }
   }
 }
 
