@@ -8,6 +8,7 @@
 #import <IOBluetooth/objc/IOBluetoothHostController.h>
 #include <stddef.h>
 
+#include <memory>
 #include <string>
 
 #include "base/bind.h"
@@ -16,19 +17,19 @@
 #include "base/location.h"
 #include "base/mac/mac_util.h"
 #include "base/mac/sdk_forward_declarations.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/profiler/scoped_tracker.h"
 #include "base/sequenced_task_runner.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/sys_string_conversions.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/time/time.h"
 #include "device/bluetooth/bluetooth_classic_device_mac.h"
+#include "device/bluetooth/bluetooth_common.h"
 #include "device/bluetooth/bluetooth_discovery_session.h"
 #include "device/bluetooth/bluetooth_discovery_session_outcome.h"
 #include "device/bluetooth/bluetooth_low_energy_central_manager_delegate.h"
 #include "device/bluetooth/bluetooth_socket_mac.h"
-#include "device/bluetooth/bluetooth_uuid.h"
 
 namespace {
 
@@ -39,10 +40,6 @@ const int kPollIntervalMs = 500;
 
 namespace device {
 
-// static
-const NSTimeInterval BluetoothAdapterMac::kDiscoveryTimeoutSec =
-    180;  // 3 minutes
-
 // static
 base::WeakPtr<BluetoothAdapter> BluetoothAdapter::CreateAdapter(
     const InitCallback& init_callback) {
@@ -64,14 +61,24 @@ base::WeakPtr<BluetoothAdapterMac> BluetoothAdapterMac::CreateAdapterForTest(
   BluetoothAdapterMac* adapter = new BluetoothAdapterMac();
   adapter->InitForTest(ui_task_runner);
   adapter->name_ = name;
+  adapter->should_update_name_ = false;
   adapter->address_ = address;
   return adapter->weak_ptr_factory_.GetWeakPtr();
 }
 
+// static
+BluetoothUUID BluetoothAdapterMac::BluetoothUUIDWithCBUUID(CBUUID* uuid) {
+  // UUIDString only available OS X >= 10.10.
+  DCHECK(base::mac::IsAtLeastOS10_10());
+  std::string uuid_c_string = base::SysNSStringToUTF8([uuid UUIDString]);
+  return device::BluetoothUUID(uuid_c_string);
+}
+
 BluetoothAdapterMac::BluetoothAdapterMac()
     : BluetoothAdapter(),
       classic_powered_(false),
       num_discovery_sessions_(0),
+      should_update_name_(true),
       classic_discovery_manager_(
           BluetoothDiscoveryManagerMac::CreateClassic(this)),
       weak_ptr_factory_(this) {
@@ -82,8 +89,7 @@ BluetoothAdapterMac::BluetoothAdapterMac()
         [[BluetoothLowEnergyCentralManagerDelegate alloc]
             initWithDiscoveryManager:low_energy_discovery_manager_.get()
                           andAdapter:this]);
-    Class aClass = NSClassFromString(@"CBCentralManager");
-    low_energy_central_manager_.reset([[aClass alloc]
+    low_energy_central_manager_.reset([[CBCentralManager alloc]
         initWithDelegate:low_energy_central_manager_delegate_.get()
                    queue:dispatch_get_main_queue()]);
     low_energy_discovery_manager_->SetCentralManager(
@@ -93,6 +99,14 @@ BluetoothAdapterMac::BluetoothAdapterMac()
 }
 
 BluetoothAdapterMac::~BluetoothAdapterMac() {
+  // When devices will be destroyed, they will need this current instance to
+  // disconnect the gatt connection. To make sure they don't use the mac
+  // adapter, they should be explicitly destroyed here.
+  devices_.clear();
+  // Set low_energy_central_manager_ to nil so no devices will try to use it
+  // while being destroyed after this method. |devices_| is owned by
+  // BluetoothAdapter.
+  low_energy_central_manager_.reset();
 }
 
 std::string BluetoothAdapterMac::GetAddress() const {
@@ -100,6 +114,20 @@ std::string BluetoothAdapterMac::GetAddress() const {
 }
 
 std::string BluetoothAdapterMac::GetName() const {
+  if (!should_update_name_) {
+    return name_;
+  }
+
+  IOBluetoothHostController* controller =
+      [IOBluetoothHostController defaultController];
+  if (controller == nil) {
+    name_ = std::string();
+  } else {
+    name_ = base::SysNSStringToUTF8([controller nameAsString]);
+    if (!name_.empty()) {
+      should_update_name_ = false;
+    }
+  }
   return name_;
 }
 
@@ -116,8 +144,8 @@ bool BluetoothAdapterMac::IsInitialized() const {
 bool BluetoothAdapterMac::IsPresent() const {
   bool is_present = !address_.empty();
   if (IsLowEnergyAvailable()) {
-    is_present = is_present || ([low_energy_central_manager_ state] ==
-                                CBCentralManagerStatePoweredOn);
+    is_present = is_present || ([low_energy_central_manager_ state] !=
+                                CBCentralManagerStateUnsupported);
   }
   return is_present;
 }
@@ -158,6 +186,34 @@ bool BluetoothAdapterMac::IsDiscovering() const {
   return is_discovering;
 }
 
+std::unordered_map<BluetoothDevice*, BluetoothDevice::UUIDSet>
+BluetoothAdapterMac::RetrieveGattConnectedDevicesWithDiscoveryFilter(
+    const BluetoothDiscoveryFilter& discovery_filter) {
+  std::unordered_map<BluetoothDevice*, BluetoothDevice::UUIDSet>
+      connected_devices;
+  std::set<device::BluetoothUUID> uuids;
+  discovery_filter.GetUUIDs(uuids);
+  if (uuids.empty()) {
+    for (BluetoothDevice* device :
+         RetrieveGattConnectedDevicesWithService(nullptr)) {
+      connected_devices[device] = BluetoothDevice::UUIDSet();
+    }
+    return connected_devices;
+  }
+  for (const BluetoothUUID& uuid : uuids) {
+    for (BluetoothDevice* device :
+         RetrieveGattConnectedDevicesWithService(&uuid)) {
+      connected_devices[device].insert(uuid);
+    }
+  }
+  return connected_devices;
+}
+
+BluetoothAdapter::UUIDList BluetoothAdapterMac::GetUUIDs() const {
+  NOTIMPLEMENTED();
+  return UUIDList();
+}
+
 void BluetoothAdapterMac::CreateRfcommService(
     const BluetoothUUID& uuid,
     const ServiceOptions& options,
@@ -178,22 +234,19 @@ void BluetoothAdapterMac::CreateL2capService(
       this, uuid, options, base::Bind(callback, socket), error_callback);
 }
 
-void BluetoothAdapterMac::RegisterAudioSink(
-    const BluetoothAudioSink::Options& options,
-    const AcquiredCallback& callback,
-    const BluetoothAudioSink::ErrorCallback& error_callback) {
-  NOTIMPLEMENTED();
-  error_callback.Run(BluetoothAudioSink::ERROR_UNSUPPORTED_PLATFORM);
-}
-
 void BluetoothAdapterMac::RegisterAdvertisement(
-    scoped_ptr<BluetoothAdvertisement::Data> advertisement_data,
+    std::unique_ptr<BluetoothAdvertisement::Data> advertisement_data,
     const CreateAdvertisementCallback& callback,
-    const CreateAdvertisementErrorCallback& error_callback) {
+    const AdvertisementErrorCallback& error_callback) {
   NOTIMPLEMENTED();
   error_callback.Run(BluetoothAdvertisement::ERROR_UNSUPPORTED_PLATFORM);
 }
 
+BluetoothLocalGattService* BluetoothAdapterMac::GetGattService(
+    const std::string& identifier) const {
+  return nullptr;
+}
+
 void BluetoothAdapterMac::ClassicDeviceFound(IOBluetoothDevice* device) {
   ClassicDeviceAdded(device);
 }
@@ -204,9 +257,8 @@ void BluetoothAdapterMac::ClassicDiscoveryStopped(bool unexpected) {
     num_discovery_sessions_ = 0;
     MarkDiscoverySessionsAsInactive();
   }
-  FOR_EACH_OBSERVER(BluetoothAdapter::Observer,
-                    observers_,
-                    AdapterDiscoveringChanged(this, false));
+  for (auto& observer : observers_)
+    observer.AdapterDiscoveringChanged(this, false);
 }
 
 void BluetoothAdapterMac::DeviceConnected(IOBluetoothDevice* device) {
@@ -219,21 +271,24 @@ void BluetoothAdapterMac::DeviceConnected(IOBluetoothDevice* device) {
 
 // static
 bool BluetoothAdapterMac::IsLowEnergyAvailable() {
-  return base::mac::IsOSYosemiteOrLater();
+  return base::mac::IsAtLeastOS10_10();
 }
 
+void BluetoothAdapterMac::RemovePairingDelegateInternal(
+    BluetoothDevice::PairingDelegate* pairing_delegate) {}
+
 void BluetoothAdapterMac::SetCentralManagerForTesting(
     CBCentralManager* central_manager) {
   CHECK(BluetoothAdapterMac::IsLowEnergyAvailable());
-  [central_manager performSelector:@selector(setDelegate:)
-                        withObject:low_energy_central_manager_delegate_];
-  low_energy_central_manager_.reset(central_manager);
+  central_manager.delegate = low_energy_central_manager_delegate_;
+  low_energy_central_manager_.reset(central_manager,
+                                    base::scoped_policy::RETAIN);
   low_energy_discovery_manager_->SetCentralManager(
       low_energy_central_manager_.get());
 }
 
-void BluetoothAdapterMac::RemovePairingDelegateInternal(
-    BluetoothDevice::PairingDelegate* pairing_delegate) {
+CBCentralManager* BluetoothAdapterMac::GetCentralManager() {
+  return low_energy_central_manager_;
 }
 
 void BluetoothAdapterMac::AddDiscoverySession(
@@ -265,9 +320,8 @@ void BluetoothAdapterMac::AddDiscoverySession(
 
   DVLOG(1) << "Added a discovery session";
   num_discovery_sessions_++;
-  FOR_EACH_OBSERVER(BluetoothAdapter::Observer,
-                    observers_,
-                    AdapterDiscoveringChanged(this, true));
+  for (auto& observer : observers_)
+    observer.AdapterDiscoveringChanged(this, true);
   callback.Run();
 }
 
@@ -292,12 +346,11 @@ void BluetoothAdapterMac::RemoveDiscoverySession(
   }
 
   // Default to dual discovery if |discovery_filter| is NULL.
-  BluetoothDiscoveryFilter::TransportMask transport =
-      BluetoothDiscoveryFilter::Transport::TRANSPORT_DUAL;
+  BluetoothTransport transport = BLUETOOTH_TRANSPORT_DUAL;
   if (discovery_filter)
     transport = discovery_filter->GetTransport();
 
-  if (transport & BluetoothDiscoveryFilter::Transport::TRANSPORT_CLASSIC) {
+  if (transport & BLUETOOTH_TRANSPORT_CLASSIC) {
     if (!classic_discovery_manager_->StopDiscovery()) {
       DVLOG(1) << "Failed to stop classic discovery";
       // TODO: Provide a more precise error here.
@@ -305,9 +358,13 @@ void BluetoothAdapterMac::RemoveDiscoverySession(
       return;
     }
   }
-  if (transport & BluetoothDiscoveryFilter::Transport::TRANSPORT_LE) {
-    if (IsLowEnergyAvailable())
+  if (transport & BLUETOOTH_TRANSPORT_LE) {
+    if (IsLowEnergyAvailable()) {
       low_energy_discovery_manager_->StopDiscovery();
+      for (const auto& device_id_object_pair : devices_) {
+        device_id_object_pair.second->ClearAdvertisementData();
+      }
+    }
   }
 
   DVLOG(1) << "Discovery stopped";
@@ -316,7 +373,7 @@ void BluetoothAdapterMac::RemoveDiscoverySession(
 }
 
 void BluetoothAdapterMac::SetDiscoveryFilter(
-    scoped_ptr<BluetoothDiscoveryFilter> discovery_filter,
+    std::unique_ptr<BluetoothDiscoveryFilter> discovery_filter,
     const base::Closure& callback,
     const DiscoverySessionErrorCallback& error_callback) {
   NOTIMPLEMENTED();
@@ -327,12 +384,11 @@ bool BluetoothAdapterMac::StartDiscovery(
     BluetoothDiscoveryFilter* discovery_filter) {
   // Default to dual discovery if |discovery_filter| is NULL.  IOBluetooth seems
   // allow starting low energy and classic discovery at once.
-  BluetoothDiscoveryFilter::TransportMask transport =
-      BluetoothDiscoveryFilter::Transport::TRANSPORT_DUAL;
+  BluetoothTransport transport = BLUETOOTH_TRANSPORT_DUAL;
   if (discovery_filter)
     transport = discovery_filter->GetTransport();
 
-  if ((transport & BluetoothDiscoveryFilter::Transport::TRANSPORT_CLASSIC) &&
+  if ((transport & BLUETOOTH_TRANSPORT_CLASSIC) &&
       !classic_discovery_manager_->IsDiscovering()) {
     // TODO(krstnmnlsn): If a classic discovery session is already running then
     // we should update its filter. crbug.com/498056
@@ -341,7 +397,7 @@ bool BluetoothAdapterMac::StartDiscovery(
       return false;
     }
   }
-  if (transport & BluetoothDiscoveryFilter::Transport::TRANSPORT_LE) {
+  if (transport & BLUETOOTH_TRANSPORT_LE) {
     // Begin a low energy discovery session or update it if one is already
     // running.
     if (IsLowEnergyAvailable())
@@ -383,14 +439,8 @@ void BluetoothAdapterMac::PollAdapter() {
         base::SysNSStringToUTF8([controller addressAsString]));
     classic_powered = ([controller powerState] == kBluetoothHCIPowerStateON);
 
-    // For performance reasons, cache the adapter's name. It's not uncommon for
-    // a call to [controller nameAsString] to take tens of milliseconds. Note
-    // that this caching strategy might result in clients receiving a stale
-    // name. If this is a significant issue, then some more sophisticated
-    // workaround for the performance bottleneck will be needed. For additional
-    // context, see http://crbug.com/461181 and http://crbug.com/467316
-    if (address != address_ || (!address.empty() && name_.empty()))
-      name_ = base::SysNSStringToUTF8([controller nameAsString]);
+    if (address != address_)
+      should_update_name_ = true;
   }
 
   bool is_present = !address.empty();
@@ -402,8 +452,8 @@ void BluetoothAdapterMac::PollAdapter() {
       FROM_HERE_WITH_EXPLICIT_FUNCTION(
           "461181 BluetoothAdapterMac::PollAdapter::AdapterPresentChanged"));
   if (was_present != is_present) {
-    FOR_EACH_OBSERVER(BluetoothAdapter::Observer, observers_,
-                      AdapterPresentChanged(this, is_present));
+    for (auto& observer : observers_)
+      observer.AdapterPresentChanged(this, is_present);
   }
 
   // TODO(erikchen): Remove ScopedTracker below once http://crbug.com/461181
@@ -413,8 +463,8 @@ void BluetoothAdapterMac::PollAdapter() {
           "461181 BluetoothAdapterMac::PollAdapter::AdapterPowerChanged"));
   if (classic_powered_ != classic_powered) {
     classic_powered_ = classic_powered;
-    FOR_EACH_OBSERVER(BluetoothAdapter::Observer, observers_,
-                      AdapterPoweredChanged(this, classic_powered_));
+    for (auto& observer : observers_)
+      observer.AdapterPoweredChanged(this, classic_powered_);
   }
 
   // TODO(erikchen): Remove ScopedTracker below once http://crbug.com/461181
@@ -442,39 +492,219 @@ void BluetoothAdapterMac::ClassicDeviceAdded(IOBluetoothDevice* device) {
   std::string device_address =
       BluetoothClassicDeviceMac::GetDeviceAddress(device);
 
-  // Only notify observers once per device.
-  if (devices_.count(device_address))
-    return;
+  BluetoothDevice* device_classic = GetDevice(device_address);
 
-  BluetoothDevice* device_classic = new BluetoothClassicDeviceMac(this, device);
-  devices_.set(device_address, make_scoped_ptr(device_classic));
-  FOR_EACH_OBSERVER(BluetoothAdapter::Observer, observers_,
-                    DeviceAdded(this, device_classic));
+  // Only notify observers once per device.
+  if (device_classic != nullptr) {
+    VLOG(3) << "Updating classic device: " << device_classic->GetAddress();
+    device_classic->UpdateTimestamp();
+    return;
+  }
+
+  device_classic = new BluetoothClassicDeviceMac(this, device);
+  devices_.set(device_address, base::WrapUnique(device_classic));
+  VLOG(1) << "Adding new classic device: " << device_classic->GetAddress();
+
+  for (auto& observer : observers_)
+    observer.DeviceAdded(this, device_classic);
 }
 
 void BluetoothAdapterMac::LowEnergyDeviceUpdated(
     CBPeripheral* peripheral,
     NSDictionary* advertisement_data,
     int rssi) {
-  std::string device_address =
-      BluetoothLowEnergyDeviceMac::GetPeripheralHashAddress(peripheral);
-  // Try to find device from |devices_| with key |device_address|,
-  // if has no entry in the map, create new device and insert into |devices_|,
+  BluetoothLowEnergyDeviceMac* device_mac =
+      GetBluetoothLowEnergyDeviceMac(peripheral);
+  // If has no entry in the map, create new device and insert into |devices_|,
   // otherwise update the existing device.
-  DevicesMap::const_iterator iter = devices_.find(device_address);
-  if (iter == devices_.end()) {
+  const bool is_new_device = device_mac == nullptr;
+  if (is_new_device) {
     VLOG(1) << "LowEnergyDeviceUpdated new device";
     // A new device has been found.
-    BluetoothLowEnergyDeviceMac* device_mac = new BluetoothLowEnergyDeviceMac(
-        this, peripheral, advertisement_data, rssi);
-    devices_.add(device_address, scoped_ptr<BluetoothDevice>(device_mac));
-    FOR_EACH_OBSERVER(BluetoothAdapter::Observer, observers_,
-                      DeviceAdded(this, device_mac));
+    device_mac = new BluetoothLowEnergyDeviceMac(this, peripheral);
+  } else if (DoesCollideWithKnownDevice(peripheral, device_mac)) {
     return;
   }
 
+  DCHECK(device_mac);
+
+  // Get Advertised UUIDs
+  BluetoothDevice::UUIDList advertised_uuids;
+  NSArray* service_uuids =
+      [advertisement_data objectForKey:CBAdvertisementDataServiceUUIDsKey];
+  for (CBUUID* uuid in service_uuids) {
+    advertised_uuids.push_back(BluetoothUUID([[uuid UUIDString] UTF8String]));
+  }
+  NSArray* overflow_service_uuids = [advertisement_data
+      objectForKey:CBAdvertisementDataOverflowServiceUUIDsKey];
+  for (CBUUID* uuid in overflow_service_uuids) {
+    advertised_uuids.push_back(BluetoothUUID([[uuid UUIDString] UTF8String]));
+  }
+
+  // Get Service Data.
+  BluetoothDevice::ServiceDataMap service_data_map;
+  NSDictionary* service_data =
+      [advertisement_data objectForKey:CBAdvertisementDataServiceDataKey];
+  for (CBUUID* uuid in service_data) {
+    NSData* data = [service_data objectForKey:uuid];
+    const uint8_t* bytes = static_cast<const uint8_t*>([data bytes]);
+    size_t length = [data length];
+    service_data_map.emplace(BluetoothUUID([[uuid UUIDString] UTF8String]),
+                             std::vector<uint8_t>(bytes, bytes + length));
+  }
+
+  // Get Tx Power.
+  NSNumber* tx_power =
+      [advertisement_data objectForKey:CBAdvertisementDataTxPowerLevelKey];
+  int8_t clamped_tx_power = BluetoothDevice::ClampPower([tx_power intValue]);
+
+  device_mac->UpdateAdvertisementData(
+      BluetoothDevice::ClampPower(rssi), std::move(advertised_uuids),
+      std::move(service_data_map),
+      tx_power == nil ? nullptr : &clamped_tx_power);
+
+  if (is_new_device) {
+    std::string device_address =
+        BluetoothLowEnergyDeviceMac::GetPeripheralHashAddress(peripheral);
+    devices_.add(device_address, std::unique_ptr<BluetoothDevice>(device_mac));
+    for (auto& observer : observers_)
+      observer.DeviceAdded(this, device_mac);
+  } else {
+    for (auto& observer : observers_)
+      observer.DeviceChanged(this, device_mac);
+  }
+}
+
+// TODO(krstnmnlsn): Implement. crbug.com/511025
+void BluetoothAdapterMac::LowEnergyCentralManagerUpdatedState() {}
+
+void BluetoothAdapterMac::AddPairedDevices() {
+  // Add any new paired devices.
+  for (IOBluetoothDevice* device in [IOBluetoothDevice pairedDevices]) {
+    // pairedDevices sometimes includes unknown devices that are not paired.
+    // Radar issue with id 2282763004 has been filed about it.
+    if ([device isPaired]) {
+      ClassicDeviceAdded(device);
+    }
+  }
+}
+
+std::vector<BluetoothDevice*>
+BluetoothAdapterMac::RetrieveGattConnectedDevicesWithService(
+    const BluetoothUUID* uuid) {
+  NSArray* cbUUIDs = nil;
+  if (!uuid) {
+    // It is not possible to ask for all connected peripherals with
+    // -[CBCentralManager retrieveConnectedPeripheralsWithServices:] by passing
+    // nil. To try to get most of the peripherals, the search is done with
+    // Generic Access service.
+    CBUUID* genericAccessServiceUUID = [CBUUID UUIDWithString:@"1800"];
+    cbUUIDs = @[ genericAccessServiceUUID ];
+  } else {
+    NSString* uuidString =
+        base::SysUTF8ToNSString(uuid->canonical_value().c_str());
+    cbUUIDs = @[ [CBUUID UUIDWithString:uuidString] ];
+  }
+  NSArray* peripherals = [low_energy_central_manager_
+      retrieveConnectedPeripheralsWithServices:cbUUIDs];
+  std::vector<BluetoothDevice*> connected_devices;
+  for (CBPeripheral* peripheral in peripherals) {
+    BluetoothLowEnergyDeviceMac* device_mac =
+        GetBluetoothLowEnergyDeviceMac(peripheral);
+    const bool is_new_device = device_mac == nullptr;
+
+    if (!is_new_device && DoesCollideWithKnownDevice(peripheral, device_mac)) {
+      continue;
+    }
+    if (is_new_device) {
+      device_mac = new BluetoothLowEnergyDeviceMac(this, peripheral);
+      std::string device_address =
+          BluetoothLowEnergyDeviceMac::GetPeripheralHashAddress(peripheral);
+      devices_.add(device_address,
+                   std::unique_ptr<BluetoothDevice>(device_mac));
+      for (auto& observer : observers_) {
+        observer.DeviceAdded(this, device_mac);
+      }
+    }
+    connected_devices.push_back(device_mac);
+  }
+  return connected_devices;
+}
+
+void BluetoothAdapterMac::CreateGattConnection(
+    BluetoothLowEnergyDeviceMac* device_mac) {
+  [low_energy_central_manager_ connectPeripheral:device_mac->peripheral_
+                                         options:nil];
+}
+
+void BluetoothAdapterMac::DisconnectGatt(
+    BluetoothLowEnergyDeviceMac* device_mac) {
+  [low_energy_central_manager_
+      cancelPeripheralConnection:device_mac->peripheral_];
+}
+
+void BluetoothAdapterMac::DidConnectPeripheral(CBPeripheral* peripheral) {
   BluetoothLowEnergyDeviceMac* device_mac =
-      static_cast<BluetoothLowEnergyDeviceMac*>(iter->second);
+      GetBluetoothLowEnergyDeviceMac(peripheral);
+  if (!device_mac) {
+    [low_energy_central_manager_ cancelPeripheralConnection:peripheral];
+    return;
+  }
+  device_mac->DidConnectGatt();
+  [device_mac->GetPeripheral() discoverServices:nil];
+}
+
+void BluetoothAdapterMac::DidFailToConnectPeripheral(CBPeripheral* peripheral,
+                                                     NSError* error) {
+  BluetoothLowEnergyDeviceMac* device_mac =
+      GetBluetoothLowEnergyDeviceMac(peripheral);
+  if (!device_mac) {
+    [low_energy_central_manager_ cancelPeripheralConnection:peripheral];
+    return;
+  }
+  VLOG(1) << "Failed to connect to peripheral";
+  BluetoothDevice::ConnectErrorCode error_code =
+      BluetoothDevice::ConnectErrorCode::ERROR_UNKNOWN;
+  if (error) {
+    error_code = BluetoothDeviceMac::GetConnectErrorCodeFromNSError(error);
+    VLOG(1) << "Bluetooth error, domain: " << error.domain.UTF8String
+            << ", error code: " << error.code
+            << ", converted into: " << error_code;
+  }
+  device_mac->DidFailToConnectGatt(error_code);
+}
+
+void BluetoothAdapterMac::DidDisconnectPeripheral(CBPeripheral* peripheral,
+                                                  NSError* error) {
+  BluetoothLowEnergyDeviceMac* device_mac =
+      GetBluetoothLowEnergyDeviceMac(peripheral);
+  if (!device_mac) {
+    [low_energy_central_manager_ cancelPeripheralConnection:peripheral];
+    return;
+  }
+  VLOG(1) << "Disconnected from peripheral.";
+  if (error) {
+    VLOG(1) << "Bluetooth error, domain: " << error.domain.UTF8String
+            << ", error code: " << error.code;
+  }
+  device_mac->DidDisconnectPeripheral(error);
+}
+
+BluetoothLowEnergyDeviceMac*
+BluetoothAdapterMac::GetBluetoothLowEnergyDeviceMac(CBPeripheral* peripheral) {
+  std::string device_address =
+      BluetoothLowEnergyDeviceMac::GetPeripheralHashAddress(peripheral);
+  DevicesMap::const_iterator iter = devices_.find(device_address);
+  if (iter == devices_.end()) {
+    return nil;
+  }
+  return static_cast<BluetoothLowEnergyDeviceMac*>(iter->second);
+}
+
+bool BluetoothAdapterMac::DoesCollideWithKnownDevice(
+    CBPeripheral* peripheral,
+    BluetoothLowEnergyDeviceMac* device_mac) {
+  // Check that there are no collisions.
   std::string stored_device_id = device_mac->GetIdentifier();
   std::string updated_device_id =
       BluetoothLowEnergyDeviceMac::GetPeripheralIdentifier(peripheral);
@@ -488,53 +718,9 @@ void BluetoothAdapterMac::LowEnergyDeviceUpdated(
     // simultaneously present is 1e-6 (see
     // https://en.wikipedia.org/wiki/Birthday_problem#Probability_table).  We
     // ignore the second device by returning.
-    return;
-  }
-
-  // A device has an update.
-  VLOG(2) << "LowEnergyDeviceUpdated";
-  device_mac->Update(peripheral, advertisement_data, rssi);
-  // TODO(scheib): Call DeviceChanged only if UUIDs change. crbug.com/547106
-  FOR_EACH_OBSERVER(BluetoothAdapter::Observer, observers_,
-                    DeviceChanged(this, device_mac));
-}
-
-// TODO(krstnmnlsn): Implement. crbug.com/511025
-void BluetoothAdapterMac::LowEnergyCentralManagerUpdatedState() {}
-
-void BluetoothAdapterMac::RemoveTimedOutDevices() {
-  // Notify observers if any previously seen devices are no longer available,
-  // i.e. if they are no longer paired, connected, nor recently discovered via
-  // an inquiry.
-  std::set<std::string> removed_devices;
-  for (DevicesMap::const_iterator it = devices_.begin(); it != devices_.end();
-       ++it) {
-    BluetoothDevice* device = it->second;
-    if (device->IsPaired() || device->IsConnected())
-      continue;
-
-    NSDate* last_update_time =
-        static_cast<BluetoothDeviceMac*>(device)->GetLastUpdateTime();
-    if (last_update_time &&
-        -[last_update_time timeIntervalSinceNow] < kDiscoveryTimeoutSec)
-      continue;
-
-    FOR_EACH_OBSERVER(
-        BluetoothAdapter::Observer, observers_, DeviceRemoved(this, device));
-    removed_devices.insert(it->first);
-    // The device will be erased from the map in the loop immediately below.
-  }
-  for (const std::string& device_address : removed_devices) {
-    size_t num_removed = devices_.erase(device_address);
-    DCHECK_EQ(num_removed, 1U);
-  }
-}
-
-void BluetoothAdapterMac::AddPairedDevices() {
-  // Add any new paired devices.
-  for (IOBluetoothDevice* device in [IOBluetoothDevice pairedDevices]) {
-    ClassicDeviceAdded(device);
+    return true;
   }
+  return false;
 }
 
 }  // namespace device
