@@ -7,11 +7,13 @@
 
 #include <stdint.h>
 
+#include <list>
+#include <memory>
 #include <string>
 #include <utility>
-#include <vector>
 
 #include "base/callback.h"
+#include "base/files/scoped_file.h"
 #include "base/macros.h"
 #include "base/threading/thread_checker.h"
 #include "build/build_config.h"
@@ -20,8 +22,9 @@
 #include "device/usb/webusb_descriptors.h"
 
 struct libusb_device;
-struct libusb_config_descriptor;
+struct libusb_device_descriptor;
 struct libusb_device_handle;
+struct libusb_config_descriptor;
 
 namespace base {
 class SequencedTaskRunner;
@@ -42,12 +45,8 @@ typedef struct libusb_device_handle* PlatformUsbDeviceHandle;
 
 class UsbDeviceImpl : public UsbDevice {
  public:
-// UsbDevice implementation:
-#if defined(OS_CHROMEOS)
-  void CheckUsbAccess(const ResultCallback& callback) override;
-#endif  // OS_CHROMEOS
+  // UsbDevice implementation:
   void Open(const OpenCallback& callback) override;
-  const UsbConfigDescriptor* GetActiveConfiguration() override;
 
   // These functions are used during enumeration only. The values must not
   // change during the object's lifetime.
@@ -60,9 +59,9 @@ class UsbDeviceImpl : public UsbDevice {
   void set_serial_number(const base::string16& value) {
     serial_number_ = value;
   }
-  void set_device_path(const std::string& value) { device_path_ = value; }
-  void set_webusb_allowed_origins(scoped_ptr<WebUsbDescriptorSet> descriptors) {
-    webusb_allowed_origins_ = std::move(descriptors);
+  void set_webusb_allowed_origins(
+      std::unique_ptr<WebUsbAllowedOrigins> allowed_origins) {
+    webusb_allowed_origins_ = std::move(allowed_origins);
   }
   void set_webusb_landing_page(const GURL& url) { webusb_landing_page_ = url; }
 
@@ -75,30 +74,20 @@ class UsbDeviceImpl : public UsbDevice {
   // Called by UsbServiceImpl only;
   UsbDeviceImpl(scoped_refptr<UsbContext> context,
                 PlatformUsbDevice platform_device,
-                uint16_t vendor_id,
-                uint16_t product_id,
+                const libusb_device_descriptor& descriptor,
                 scoped_refptr<base::SequencedTaskRunner> blocking_task_runner);
 
   ~UsbDeviceImpl() override;
 
+  void ReadAllConfigurations();
+  void RefreshActiveConfiguration();
+
   // Called only by UsbServiceImpl.
   void set_visited(bool visited) { visited_ = visited; }
   bool was_visited() const { return visited_; }
-  void OnDisconnect();
-  void ReadAllConfigurations();
-
-  // Called by UsbDeviceHandleImpl.
-  void Close(scoped_refptr<UsbDeviceHandle> handle);
-  void RefreshActiveConfiguration();
 
  private:
   void GetAllConfigurations();
-#if defined(OS_CHROMEOS)
-  void OnOpenRequestComplete(const OpenCallback& callback,
-                             dbus::FileDescriptor fd);
-  void OpenOnBlockingThreadWithFd(dbus::FileDescriptor fd,
-                                  const OpenCallback& callback);
-#endif
   void OpenOnBlockingThread(const OpenCallback& callback);
   void Opened(PlatformUsbDeviceHandle platform_handle,
               const OpenCallback& callback);
@@ -107,22 +96,9 @@ class UsbDeviceImpl : public UsbDevice {
   PlatformUsbDevice platform_device_;
   bool visited_ = false;
 
-  // On Chrome OS device path is necessary to request access from the permission
-  // broker.
-  std::string device_path_;
-
-  // The current device configuration descriptor. May be null if the device is
-  // in an unconfigured state; if not null, it is a pointer to one of the
-  // items at UsbDevice::configurations_.
-  const UsbConfigDescriptor* active_configuration_ = nullptr;
-
   // Retain the context so that it will not be released before UsbDevice.
   scoped_refptr<UsbContext> context_;
 
-  // Opened handles.
-  typedef std::vector<scoped_refptr<UsbDeviceHandleImpl> > HandlesVector;
-  HandlesVector handles_;
-
   scoped_refptr<base::SequencedTaskRunner> task_runner_;
   scoped_refptr<base::SequencedTaskRunner> blocking_task_runner_;
 
